﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_SharingClone.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  排他的に送信権を制御して双方向の送受信を行う CloneBase 派生クラスです。
 */
class AtomicSharingClone : public SharingClone
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    AtomicSharingClone();


    /*!
    @brief デストラクタです。
     */
    virtual ~AtomicSharingClone();


    /*!
    @brief  送信権の取得を試みます。
    @details    @ref LockStatus_Unlocked の時に呼び出すと、 @ref LockStatus_TryLock に遷移します。
                その後送信権の取得に成功したら、 @ref LockStatus_Locked に遷移します。
                他のステーションが送信権を持っていて送信権の取得に失敗したら、 @ref LockStatus_Unlocked に戻ります。
                @ref LockStatus_TryLock, @ref LockStatus_Locked の時に呼び出しても何も起こりません。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState  動作中ではありません。（IsActive() が false です。）[:progErr]
     */
    Result TryLock();


    /*!
    @brief  送信権を放棄します。
    @details    @ref LockStatus_Locked の時に呼び出すと、 @ref LockStatus_Unlocked に遷移して、他のステーションが送信権を取得できるようになります。
                @ref LockStatus_TryLock の時に呼び出すと、送信権の取得を中断して @ref LockStatus_Unlocked に遷移します。
                @ref LockStatus_Unlocked の時に呼び出しても何も起こりません。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState  動作中ではありません。（IsActive() が false です。）[:progErr]
     */
    Result Unlock();


    /*!
    @brief  送信権の保持状態を表します。
     */
    enum LockStatus
    {
        LockStatus_Unlocked = 0x10,  //!< 送信権を持っていません。
        LockStatus_TryLock = 0x20,   //!< 送信権を取得しようとしています。
        LockStatus_Locked = 0x30     //!< 送信権を持っています。
    };


    /*!
    @brief  送信権の保持状態を取得します。
    @return 送信権の保持状態です。
     */
    LockStatus GetLockStatus() const
    {
        return static_cast<LockStatus>(m_LockStatus & cLockStatusMask);
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


    //! @cond PRIVATE

protected:
    virtual void OnStartup();
    virtual void OnCleanup();
    virtual void OnEnd();

    virtual void OnExit(StationIndex stationIndex);
    virtual void OnDisconnect(StationIndex stationIndex, bool isValid);

    virtual void OnAddParticipant(StationIndex stationIndex);

public:
    virtual void ReceiveReject(StationIndex src, ClockValue clock);
    virtual void ReceiveStop(StationIndex src);

    enum LockResult
    {
        LockResult_Ok,
        LockResult_Ng,
        LockResult_Defer
    };

    LockResult ReceiveTryLock(StationIndex src, ClockValue clock, uint32_t participantBitmap);
    void ReceiveLockOK(StationIndex src, ClockValue clock, uint8_t count, uint32_t participantBitmap);
    void ReceiveLockNG(StationIndex src, ClockValue clock);

public:
    static Type GetTypeStatic()
    {
        return Type_Atomic;
    }
    static Key ToKey(Id id)
    {
        return CloneBase::ToKey(Type_Atomic, StationIndex_Invalid, id);
    }
    virtual Type GetTypeVirtual() const
    {
        return GetTypeStatic();
    }

public:
    class LockCommandToken : public ClockCommandToken
    {
    public:
        explicit LockCommandToken(AtomicSharingClone* pClone)
            : ClockCommandToken(pClone), m_DestBitmap(0)
        {
        }
        virtual CommandType GetType() const
        {
            return CommandType_Lock;
        }
        virtual uint32_t GetDestStationBitmap() const
        {
            return m_DestBitmap;
        }
        void SetDestBitmap(uint32_t destBitmap)
        {
            PIA_TRACE_EX(common::TraceFlagClone, "called LockCommandToken key:0x%016llx state:%d dest:0x%08x -> 0x%08x", GetClone() != NULL ? GetClone()->GetKey() : 0, GetClone() != NULL ? GetClone()->GetState() : 0, m_DestBitmap, destBitmap);
            m_DestBitmap = destBitmap;
        }
        void Exit(StationIndex stationIndex)
        {
            PIA_TRACE_EX(common::TraceFlagClone, "called LockCommandToken key:0x%016llx state:%d dest:0x%08x -> 0x%08x", GetClone() != NULL ? GetClone()->GetKey() : 0, GetClone() != NULL ? GetClone()->GetState() : 0, m_DestBitmap, m_DestBitmap & ~(0x1 << stationIndex));
            m_DestBitmap &= ~(0x1 << stationIndex);
        }
        virtual ClockValue GetClock() const
        {
            return static_cast<const AtomicSharingClone*>(GetClone())->m_LockClock;
        }

    private:
        ClockValue m_DestBitmap;
    };

    virtual bool IsReadyToSetValue() const;

private:
    enum InnerLockStatus
    {
        InnerLockStatus_Unlocked = LockStatus_Unlocked | 0x1,
        InnerLockStatus_KeepLock = LockStatus_Unlocked | 0x2,
        InnerLockStatus_TryLock = LockStatus_TryLock | 0x1,
        InnerLockStatus_Locked = LockStatus_Locked | 0x1
    };

    enum
    {
        cLockStatusMask = 0xf0
    };

    InnerLockStatus m_LockStatus;
    ClockValue m_LockClock;
    ClockValue m_UnlockClock;

    LockCommandToken m_LockCommandToken;

public:
    template <int TEST_ID, typename Arg>
    static void Test(const AtomicSharingClone& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(AtomicSharingClone);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
