﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneElementBase.h>
#include <nn/pia/common/common_TreeMapNode.h>
#include <nn/pia/common/common_OffsetTreeMap.h>
#include <nn/pia/common/common_OffsetList.h>

namespace nn
{
namespace pia
{
namespace clone
{

class CloneProtocol;

/*!
    @brief  送受信を管理する基底クラスです。
 */
class CloneBase
{
public:
protected:
    /*!
    @brief デフォルトコンストラクタです。
     */
    CloneBase();


public:
    /*!
    @brief デストラクタです。
     */
    virtual ~CloneBase();


    /*!
    @brief  このオブジェクトが登録されている CloneProtocol を取得します。
    @return このオブジェクトが登録されている CloneProtocol です。
     */
    const CloneProtocol* GetProtocol() const
    {
        return m_pProtocol;
    }


    /*!
    @brief  クローンを識別するための ID の型です。
     */
    typedef uint32_t Id;


    /*!
    @brief  ID を取得します。
    @return CloneProtocol に登録する際に指定したIDです。 CloneProtocol に登録されていない場合は 0 を返します。
     */
    Id GetId() const
    {
        return static_cast<Id>(m_TreeMapNode.GetKey() & 0xffffffff);
    }


    /*!
    @brief  CloneProtocol に登録されているかどうかを判定します。
    @return CloneProtocol に登録されていれば true を返します。
     */
    bool IsRegisteredWithProtocol() const
    {
        return m_pProtocol != NULL;
    }


    /*!
    @brief  クローンエレメントを登録します。
    @details    このオブジェクトが CloneProtocol に登録されていない状態で呼び出す必要があります。
    @param[in]  pElement    登録するクローンエレメントです。
    @param[in]  elementId   登録するクローンエレメントの ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState      このオブジェクトが CloneProtocol に登録されています。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultAlreadyExists     指定された ID には既にクローンエレメントが登録されています。[:progErr]
     */
    Result RegisterElement(CloneElementBase* pElement, CloneElementBase::Id elementId);


    /*!
    @brief  クローンエレメントの登録を解除します。
    @details    このオブジェクトが CloneProtocol に登録されていない状態で呼び出す必要があります。
    @param[in]  pElement    登録解除するクローンエレメントです。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState      このオブジェクトが CloneProtocol に登録されています。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
     */
    Result UnregisterElement(CloneElementBase* pElement);


    /*!
    @brief  クローンエレメントの登録を解除します。
    @details    このオブジェクトが CloneProtocol に登録されていない状態で呼び出す必要があります。
    @param[in]  elementId   登録解除するクローンエレメントの ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState  このオブジェクトが CloneProtocol に登録されています。 [:progErr]
    @retval ResultNotSet        指定された ID にはクローンエレメントが登録されていません。[:progErr]
     */
    Result UnregisterElement(CloneElementBase::Id elementId);


    /*!
    @brief  指定された ID のクローンエレメントを検索します。
    @param[in]  elementId   検索するクローンエレメントです。
    @return 指定された ID のクローンエレメントを返します。存在しなければ NULL を返します。 
     */
    CloneElementBase* SearchElement(CloneElementBase::Id elementId)
    {
        return m_ElementCollection.Find(elementId);
    }


    /*!
    @brief  指定された ID のクローンエレメントを検索します。
    @param[in]  elementId   検索するクローンエレメントです。
    @return 指定された ID のクローンエレメントを返します。存在しなければ NULL を返します。 
     */
    const CloneElementBase* SearchElement(CloneElementBase::Id elementId) const
    {
        return m_ElementCollection.Find(elementId);
    }


    /*!
    @brief  このオブジェクトが動作中かどうかを取得します。
    @details    CloneProtocol に登録されていて、実際に値の送受信ができる状態であれば動作中と扱います。
                CloneProtocol への登録／登録解除処理中や、 CloneProtocol の開始／終了処理中は動作中ではありません。
    @return 動作中なら true を返します。
     */
    bool IsActive() const
    {
        return (m_State == State_Active);
    }


    //! @name プロファイリング
    //! @{

    /*!
    @brief  送受信のプロファイリング管理オブジェクトを設定します。
    @details    CloneProfiler を設定すると、登録されているクローンエレメントおよびクローン自身が行った送受信をプロファイリングできます。
                ここでの計測は、このクローンが他のクローンと行うすべてのデータのやり取りが計上されるため、
                自ステーション内でのクローン間のデータのやり取りも値に含まれます。
    @details    複数のクローンエレメントが同時に送信する場合、クローン単位で一つにまとめられるため、
                多くの場合、クローンエレメントの送受信回数の合計よりクローンの送受信回数の方が小さくなります。
    @details    圧縮を有効にしていた場合、計測されるサイズは圧縮された状態でのサイズです。
                そのため、登録しているクローンエレメントで計測された送受信サイズの合計より、数値上は小さい値が計測される場合があります。
    @details    引数に NULL を指定すると、既に設定してある CloneProfiler を解除する事ができます。
    @param[in]  pSendProfiler       送信のプロファイリングを管理する CloneProfiler を指定します。
                                    必要ない場合は NULL を指定する必要があります。
    @param[in]  pReceiveProfiler    受信のプロファイリングを管理する CloneProfiler を指定します。
                                    必要ない場合は NULL を指定する必要があります。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
     */
    Result SetProfiler(CloneProfilerBase* pSendProfiler, CloneProfilerBase* pReceiveProfiler);


    /*!
    @brief  設定されている送信プロファイリング管理オブジェクトを取得します。
    @return 設定されている送信プロファイリング管理オブジェクトです。設定されていない場合は NULL を返します。
    @see    SetProfiler
     */
    const CloneProfilerBase* GetSendProfiler() const
    {
        return m_pSendProfiler;
    }


    /*!
    @brief  設定されている受信プロファイリング管理オブジェクトを取得します。
    @return 設定されている受信プロファイリング管理オブジェクトです。設定されていない場合は NULL を返します。
    @see    SetProfiler
     */
    const CloneProfilerBase* GetReceiveProfiler() const
    {
        return m_pReceiveProfiler;
    }


    /*!
    @brief  送受信のプロファイリングされた結果をリセットします。
    @see    SetProfiler
     */
    void ResetProfiler();


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信した クローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンメッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンメッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンメッセージの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンメッセージの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンメッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendUnicastCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンメッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendUnicastSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンメッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンメッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したクローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したクローンメッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したクローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したクローンメッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したクローンメッセージの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したクローンメッセージの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降にユニキャストで送信したクローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降にユニキャストで送信したクローンメッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendUnicastCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降にユニキャストで送信したクローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降にユニキャストで送信したクローンメッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendUnicastSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したシステム用クローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したシステム用クローンメッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したシステム用クローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したシステム用クローンメッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンメッセージの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンメッセージのサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信したクローンメッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信したクローンメッセージの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信したクローンメッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信したクローンメッセージのサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveSize(StationIndex stationIndex) const;

    //! @}

    virtual void Trace(uint64_t flag) const;

    //! @cond PRIVATE

public:
    enum Type
    {
        Type_Send = 0x01,
        Type_Receive = 0x02,
        Type_Atomic = 0x03,
        Type_Sequential = 0x04
    };

    virtual Type GetTypeVirtual() const = 0;


    typedef uint64_t Key;


    static int32_t GetTreeMapNodeOffset()
    {
        NN_PIA_PRAGMA_PUSH_WARNINGS
        NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
        return offsetof(CloneBase, m_TreeMapNode);
        NN_PIA_PRAGMA_POP_WARNINGS
    }

    void UnregisterElementForce(CloneElementBase* pElement);

    bool CheckElementSize(uint32_t sizeMax);
    void AttachProtocol(CloneProtocol* pProtocol);
    void DetachProtocol();

    Type GetType() const
    {
        return static_cast<Type>(m_TreeMapNode.GetKey() >> 48);
    }
    StationIndex GetStationIndex() const
    {
        return static_cast<StationIndex>((m_TreeMapNode.GetKey() >> 32) & 0xff);
    }
    Key GetKey() const
    {
        return m_TreeMapNode.GetKey();
    }
    static Key ToKey(Type type, StationIndex stationIndex, Id id)
    {
        return (static_cast<uint64_t>(type) << 48) | (static_cast<uint64_t>(stationIndex) << 32) | id;
    }


    enum State
    {
        State_Inactive = 0,
        State_Active,
        State_SendingRest,
        State_SendFinished,
        State_Ended,
        State_SendingRestForUnregister,
        State_SendFinishedForUnregister,
        State_EndedForUnregister
    };

    State GetState() const
    {
        return m_State;
    }

    CloneProfilerBase* GetSendProfilerPtr()
    {
        return m_pSendProfiler;
    }
    CloneProfilerBase* GetReceiveProfilerPtr()
    {
        return m_pReceiveProfiler;
    }

public:
    void Startup();
    void Cleanup();
    void End();
    void Unregister();
    void ClearData();

    void Enter(StationIndex stationIndex)
    {
        OnEnter(stationIndex);
    }
    void Exit(StationIndex stationIndex)
    {
        OnExit(stationIndex);
    }
    void Disconnect(StationIndex stationIndex, bool isValid)
    {
        OnDisconnect(stationIndex, isValid);
    }

    void UpdateState();
    bool IsReceiveData() const
    {
        return m_State != State_Inactive;
    }

    bool IsSendingData(StationIndex stationIndex, bool isDebug = false) const;
    void TraceSendDataList() const;

protected:
    virtual void OnStartup()
    {
    }
    virtual void OnCleanup()
    {
    }
    virtual void OnEnd()
    {
    }
    virtual void OnSendFinished()
    {
    }
    virtual bool IsEndProcessFinished()
    {
        return true;
    }

    virtual void OnEnter(StationIndex stationIndex)
    {
        (void)stationIndex;
    }
    virtual void OnExit(StationIndex stationIndex)
    {
        (void)stationIndex;
    }
    virtual void OnDisconnect(StationIndex stationIndex, bool isValid)
    {
        (void)stationIndex;
        (void)isValid;
    }
    virtual void OnRegister()
    {
    }

public:
    virtual bool IsReadyToSetValue() const
    {
        return false;
    }
    virtual uint32_t GetDestBitmap() const = 0;
    virtual bool IsInRequest() const
    {
        return false;
    }

    virtual uint8_t GetLocalRegisterCount() const
    {
        return 0;
    }
    virtual uint64_t GetRegisterCountAll() const
    {
        return 0;
    }
    virtual bool IsMatchLocalRegisterCount(uint64_t registerCountAll) const
    {
        NN_PIA_UNUSED(registerCountAll);
        return true;
    }

    void AddSendData(CloneElementBase::ISendToken* pToken);
    void CancelSendData(CloneElementBase::ISendToken* pToken);
    void ErrorEventDropped();


private:
    typedef common::OffsetList<CloneElementBase::ISendToken> SendDataList;
    typedef common::OffsetList<CloneElementBase::ResendableSendToken> ResendableDataList;

public:
    class DataTokenAccessor
    {
    public:
        DataTokenAccessor(CloneBase* pClone, SystemTime time, SystemTime resendSpan, bool isUnicast, StationIndex stationIndex);
        CloneElementBase::ISendToken* Get() const
        {
            return m_pToken;
        }
        void Reset(bool isUnicast, StationIndex stationIndex, SystemTime resendSpan);
        void Next();
        void Commit(uint32_t dispatchCount, uint32_t destBitmap);

    private:
        void NextToken();
        void NextUnicastToken();

    private:
        CloneBase* m_pClone;
        SystemTime m_Time;
        SystemTime m_ResendSpan;
        CloneElementBase::ISendToken* m_pToken;
        bool m_IsUnicast;
        StationIndex m_TargetStationIndex;
        uint32_t m_TargetBitmap;

        enum ListState
        {
            ListState_First,
            ListState_Resend,
            ListState_End,
            ListState_NoData,
            ListState_Comitted
        };
        ListState m_ListState;
    };

protected:
    void AddDataDest(StationIndex stationIndex);
    void RemoveDataDest(StationIndex stationIndex);
    void AddDataParticipant(StationIndex stationIndex);
    void ComplementData(StationIndex stationIndex);
    void LockData();

public:
    enum CommandType
    {
        CommandType_Announce,
        CommandType_Request,
        CommandType_End,
        CommandType_Lock
    };

    enum
    {
        INVALID_COMMAND_INDEX = 0
    };

    class CommandToken
    {
    protected:
        explicit CommandToken(CloneBase* pClone)
            : m_ListNode(), m_pClone(pClone), m_LastSendTime(InvalidSystemTime)
        {
        }

    public:
        bool IsInList() const
        {
            return m_ListNode.IsFreeListNode() == false;
        }
        virtual CommandType GetType() const = 0;
        virtual uint32_t GetDestStationBitmap() const = 0;
        Id GetCloneId() const
        {
            return m_pClone->GetId();
        }
        SystemTime GetLastSendTime(StationIndex stationIndex) const
        {
            if (stationIndex == StationIndex_Invalid)
            {
                return m_LastSendTime;
            }
            else
            {
                PIA_ASSERT(IsUnicast());
                PIA_ASSERT(stationIndex < MaxStationNum);
                return m_LastSendTimeArray[stationIndex];
            }
        }
        void SetLastSendTime(SystemTime time, StationIndex stationIndex)
        {
            if (stationIndex == StationIndex_Invalid)
            {
                m_LastSendTime = time;
            }
            else
            {
                PIA_ASSERT(IsUnicast());
                PIA_ASSERT(stationIndex < MaxStationNum);
                m_LastSendTimeArray[stationIndex] = time;
            }
        }
        const CloneBase* GetClone() const
        {
            return m_pClone;
        }
        static int32_t GetListNodeOffset()
        {
            NN_PIA_PRAGMA_PUSH_WARNINGS
            NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
            return offsetof(CommandToken, m_ListNode);
            NN_PIA_PRAGMA_POP_WARNINGS
        }
        CloneBase* GetClonePtr()
        {
            return m_pClone;
        }
        // 再送をユニキャストで行うトークンクラスでオーバーライドし、true を返すようにする
        virtual bool IsUnicast() const
        {
            return false;
        }

    private:
        common::ListNode m_ListNode;
        CloneBase* m_pClone;
        SystemTime m_LastSendTime;
        SystemTime m_LastSendTimeArray[MaxStationNum];
    };


    class ClockCommandToken : public CommandToken
    {
    protected:
        ClockCommandToken(CloneBase* pClone)
            : CommandToken(pClone)
        {
        }

    public:
        virtual ClockValue GetClock() const = 0;
    };


    class DataToken
    {
    public:
        explicit DataToken(CloneBase* pClone)
            : m_ListNode(), m_pClone(pClone)
        {
        }
        bool IsInList() const
        {
            return m_ListNode.IsFreeListNode() == false;
        }
        CloneBase* GetClone()
        {
            return m_pClone;
        }
        bool IsReadyToEraseFromList() const
        {
            return m_pClone->IsSendDataEmpty();
        }
        static int32_t GetListNodeOffset()
        {
            NN_PIA_PRAGMA_PUSH_WARNINGS
            NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
            return offsetof(DataToken, m_ListNode);
            NN_PIA_PRAGMA_POP_WARNINGS
        }

    private:
        common::ListNode m_ListNode;
        CloneBase* m_pClone;
    };

protected:
    CloneProtocol* GetProtocolPtr()
    {
        return m_pProtocol;
    }

    void RequestInitialData();


protected:
    bool IsSendDataEmpty() const
    {
        return m_SendDataList.IsEmpty() && m_ResendDataList.IsEmpty();
    }

private:
    common::TreeMapNode<Key> m_TreeMapNode;
    CloneProtocol* m_pProtocol;

    State m_State;

    typedef common::OffsetTreeMap<CloneElementBase::Id, CloneElementBase> ElementCollection;
    ElementCollection m_ElementCollection;


    SendDataList m_SendDataList;
    ResendableDataList m_ResendDataList;

    DataToken m_DataToken;

private:
    CloneProfilerBase* m_pSendProfiler;
    CloneProfilerBase* m_pReceiveProfiler;

public:
    template <int TEST_ID, typename Arg>
    static void Test(const CloneBase& obj, Arg* pArg = NULL);

private:
    // n2920: クライアント監視機能用
    void SetElementNumToMonitoringData();

    NN_PIA_DISALLOW_COPY(CloneBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
