﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/common/common_TreeMapNode.h>
#include <nn/pia/common/common_ListNode.h>

namespace nn
{
namespace pia
{
namespace clone
{

class CloneBase;
class CloneProtocol;
class IDataPacker;
class IDataUnpacker;
class CloneProfilerBase;

/*!
    @brief  送受信されるデータを管理する基底クラスです。
 */
class CloneElementBase
{
protected:
    /*!
    @brief デフォルトコンストラクタです。
     */
    CloneElementBase();


public:
    /*!
    @brief デストラクタです。
     */
    virtual ~CloneElementBase();


    /*!
    @brief  クローンエレメントを識別するための ID の型です。
     */
    typedef uint16_t Id;


    /*!
    @brief  ID を取得します。
    @return CloneBase に登録する際に指定した ID です。 CloneBase に登録されていない場合は 0 を返します。
            （登録する際に ID を 0 と指定していた場合、有効な値として 0 を返す場合もあります。）
     */
    Id GetId() const
    {
        return m_TreeMapNode.GetKey();
    }


    /*!
    @brief  このオブジェクトが登録されている CloneBase を取得します。
    @return このオブジェクトが登録されている CloneBase です。
     */
    const CloneBase* GetCloneBase() const
    {
        return m_pCloneBase;
    }

    /*!
    @brief  このオブジェクトが登録されている非constな CloneBase を取得します。
    @return このオブジェクトが登録されている CloneBase です。
    */
    CloneBase* GetCloneBase()
    {
        return m_pCloneBase;
    }


    /*!
    @brief  このオブジェクトが登録されている CloneBase が登録されている CloneProtocol を取得します。
    @return このオブジェクトが登録されている CloneBase が登録されている CloneProtocol です。
     */
    const CloneProtocol* GetProtocol() const;


    /*!
    @brief  CloneBase に登録されているかどうかを判定します。
    @return CloneBase に登録されていれば true を返します。
     */
    bool IsRegisteredWithCloneBase() const
    {
        return m_pCloneBase != NULL;
    }


    /*!
    @brief  CloneProtocol に登録されているかどうかを判定します。
    @return CloneProtocol に登録されていれば true を返します。
     */
    bool IsRegisteredWithProtocol() const;


    /*!
    @brief  値を設定できる状態かどうかを判定します。
    @return 値を設定できる状態なら true を返します。
     */
    bool IsReadyToSetValue() const;


    //! @name プロファイリング
    //! @{

    /*!
    @brief  送受信のプロファイリング管理オブジェクトを設定します。
    @details    CloneProfiler を設定すると、クローンエレメントが行った送受信をプロファイリングできます。
                ここでの計測は、このクローンエレメントが他のクローンエレメントと行うすべてのデータのやり取りが計上されるため、
                自ステーション内でのクローンエレメント間のデータのやり取りも値に含まれます。
    @details    圧縮を有効にしていた場合でも、計測されるサイズは非圧縮状態でのサイズです。
    @details    引数に NULL を指定すると、既に設定してある CloneProfiler を解除する事ができます。
    @param[in]  pSendProfiler       送信のプロファイリングを管理する CloneProfiler を指定します。
                                    必要ない場合は NULL を指定する必要があります。
    @param[in]  pReceiveProfiler    受信のプロファイリングを管理する CloneProfiler を指定します。
                                    必要ない場合は NULL を指定する必要があります。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
     */
    Result SetProfiler(CloneProfilerBase* pSendProfiler, CloneProfilerBase* pReceiveProfiler);


    /*!
    @brief  設定されている送信プロファイリング管理オブジェクトを取得します。
    @return 設定されている送信プロファイリング管理オブジェクトです。設定されていない場合は NULL を返します。
    @see    SetProfiler
     */
    const CloneProfilerBase* GetSendProfiler() const
    {
        return m_pSendProfiler;
    }


    /*!
    @brief  設定されている受信プロファイリング管理オブジェクトを取得します。
    @return 設定されている受信プロファイリング管理オブジェクトです。設定されていない場合は NULL を返します。
    @see    SetProfiler
     */
    const CloneProfilerBase* GetReceiveProfiler() const
    {
        return m_pReceiveProfiler;
    }


    /*!
    @brief  送受信のプロファイリングされた結果をリセットします。
    @see    SetProfiler
     */
    void ResetProfiler();


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendUnicastCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendUnicastSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に再送したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に再送したクローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestResendCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に再送したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に再送したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestResendSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したクローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降にユニキャストで送信したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降にユニキャストで送信したクローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendUnicastCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降にユニキャストで送信したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降にユニキャストで送信したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendUnicastSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に再送したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に再送したクローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalResendCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に再送したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に再送したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalResendSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したシステム用クローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したシステム用クローンエレメントデータの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンエレメントデータの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したシステム用クローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したシステム用クローンエレメントデータの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信したクローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信したクローンエレメントデータの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信したクローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信したクローンエレメントデータの圧縮前のサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信したシステム用クローンエレメントデータの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信したシステム用クローンエレメントデータの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信したシステム用クローンエレメントデータの圧縮前のサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  クローンエレメントの種類を表す列挙型です。
    */
    enum Type
    {
        Type_Unreliable = 0x1000,    //!< UnreliableCloneElement
        Type_Reliable = 0x2000,      //!< ReliableCloneElement
        Type_Event = 0x3000,         //!< EventCloneElement
        Type_Reckoning = 0x4000,     //!< ReckoningCloneElement
        Type_ReliableLarge = 0x5000  //!< ReliableLargeCloneElement
    };

    /*!
    @brief  クローンエレメントの種類を取得します。
    @return クローンエレメントの種類です。
     */
    virtual Type GetType() const = 0;

    //! @}


    //! @cond PRIVATE

public:
    static int32_t GetTreeMapNodeOffset()
    {
        NN_PIA_PRAGMA_PUSH_WARNINGS
        NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
        return offsetof(CloneElementBase, m_TreeMapNode);
        NN_PIA_PRAGMA_POP_WARNINGS
    }

    virtual bool IsCloneSupported(int cloneType) const
    {
        NN_PIA_UNUSED(cloneType);
        return true;
    }
    void AttachCloneBase(CloneBase* pCloneBase);
    void DetachCloneBase();

    CloneProfilerBase* GetSendProfilerPtr()
    {
        return m_pSendProfiler;
    }
    CloneProfilerBase* GetReceiveProfilerPtr()
    {
        return m_pReceiveProfiler;
    }

    virtual uint32_t GetSize() const = 0;

    virtual bool IsInitialized() const
    {
        return true;
    }

    virtual void ClearData() = 0;
    virtual void AddDest(StationIndex stationIndex)
    {
        NN_PIA_UNUSED(stationIndex);
    }
    virtual void RemoveDest(StationIndex stationIndex)
    {
        NN_PIA_UNUSED(stationIndex);
    }
    virtual void AddParticipant(StationIndex stationIndex)
    {
        NN_PIA_UNUSED(stationIndex);
    }
    virtual void Complement(StationIndex stationIndex)
    {
        NN_PIA_UNUSED(stationIndex);
    }
    virtual void Lock()
    {
    }

    virtual void RequestInitialData()
    {
    }

    uint32_t GetStationNum() const;

public:
    class ISendToken
    {
    public:
        ISendToken()
            : m_ListNode()
        {
        }
        bool IsInList() const
        {
            return m_ListNode.IsFreeListNode() == false;
        }
        virtual Type GetElementType() const = 0;
        virtual uint32_t GetDestBitmap() const = 0;
        virtual bool IsResend() const
        {
            return false;
        }
        virtual uint32_t GetChunkSize() const = 0;
        virtual void WriteChunk(void* pChunkBuffer) const = 0;
        virtual CloneElementBase* GetElement() = 0;
        static int32_t GetListNodeOffset()
        {
            NN_PIA_PRAGMA_PUSH_WARNINGS
            NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
            return offsetof(ISendToken, m_ListNode);
            NN_PIA_PRAGMA_POP_WARNINGS
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: Unknown");
        };

    private:
        common::ListNode m_ListNode;
    };


    class ResendableSendToken : public ISendToken
    {
    public:
        ResendableSendToken()
            : ISendToken(), m_LastSendTime(InvalidSystemTime), m_SendCount(0)
        {
            for (int i = 0; i < MaxStationNum; ++i)
            {
                m_LastSendTimeArray[i] = InvalidSystemTime;
                m_SendCountArray[i] = 0;
            }
        }
        virtual bool IsResend() const
        {
            return true;
        }
        SystemTime GetLastSendTime(StationIndex stationIndex) const
        {
            if (stationIndex == StationIndex_Invalid)
            {
                return m_LastSendTime;
            }
            else
            {
                PIA_ASSERT(IsUnicast());
                PIA_ASSERT(stationIndex < MaxStationNum);
                return m_LastSendTimeArray[stationIndex];
            }
        }
        void SetLastSendTime(SystemTime time, StationIndex stationIndex)
        {
            if (stationIndex == StationIndex_Invalid)
            {
                m_LastSendTime = time;
            }
            else
            {
                PIA_ASSERT(IsUnicast());
                PIA_ASSERT(stationIndex < MaxStationNum);
                m_LastSendTimeArray[stationIndex] = time;
            }
        }
        // 再送をユニキャストで行うトークンクラスでオーバーライドし、true を返すようにする
        virtual bool IsUnicast() const
        {
            return false;
        }
        void SetSendCount(uint32_t sendCount, StationIndex stationIndex)
        {
            if (stationIndex == StationIndex_Invalid)
            {
                m_SendCount = sendCount;
            }
            else
            {
                PIA_ASSERT(IsUnicast());
                PIA_ASSERT(stationIndex < MaxStationNum);
                m_SendCountArray[stationIndex] = sendCount;
            }
        }
        void IncrementSendCount(StationIndex stationIndex)
        {
            if (stationIndex == StationIndex_Invalid)
            {
                ++m_SendCount;
            }
            else
            {
                PIA_ASSERT(IsUnicast());
                PIA_ASSERT(stationIndex < MaxStationNum);
                ++m_SendCountArray[stationIndex];
            }
        }
        uint32_t GetSendCount(StationIndex stationIndex) const
        {
            if (stationIndex == StationIndex_Invalid)
            {
                return m_SendCount;
            }
            else
            {
                PIA_ASSERT(IsUnicast());
                PIA_ASSERT(stationIndex < MaxStationNum);
                return m_SendCountArray[stationIndex];
            }
        }

    private:
        SystemTime m_LastSendTime;
        SystemTime m_LastSendTimeArray[MaxStationNum];
        uint32_t m_SendCount;
        uint32_t m_SendCountArray[MaxStationNum];
    };


    class ConversibleResendableSendToken : public ResendableSendToken
    {
    public:
        ConversibleResendableSendToken()
            : ResendableSendToken(), m_IsResend(false)
        {
        }
        virtual bool IsResend() const
        {
            return m_IsResend;
        }
        void SetResend(bool isResend)
        {
            m_IsResend = isResend;
        }

    private:
        bool m_IsResend;
    };

protected:
    void AddSendData(ISendToken* pToken);
    void CancelSendData(ISendToken* pToken);

protected:
    void ErrorEventDropped();

private:
    common::TreeMapNode<Id> m_TreeMapNode;
    CloneBase* m_pCloneBase;

public:
    template <typename Element, typename CP>
    class DefinitionBase
    {
    protected:
        DefinitionBase()
        {
            CP::GetElementDefinition()->Register(Element::GetTypeStatic(), Element::Receive, Element::GetDataChunkHeaderSize());
        }
    };

    template <typename Element>
    class Definition : public DefinitionBase<Element, CloneProtocol>
    {
    public:
        Definition()
            : DefinitionBase<Element, CloneProtocol>()
        {
        }
    };

private:
    CloneProfilerBase* m_pSendProfiler;
    CloneProfilerBase* m_pReceiveProfiler;

public:
    template <int TEST_ID, typename Arg>
    static void Test(const CloneElementBase& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(CloneElementBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
