﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneProfilerBase.h>
#include <nn/pia/pia_Assert.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  PiaClone のプロファイリング管理クラスです。

    @details    CloneProfiler のインスタンスを CloneProtocol, CloneBase, CloneElementBase 登録することにより、
                それぞれのインスタンスが行った送受信をプロファイリングする機能を追加します。
    @details    同時にプロファイリングする CloneProtocol, CloneBase, CloneElementBase の各インスタンス毎に
                個別の CloneProfiler インスタンスを使用する必要があります。

    @tparam STATION_NUM         計測するステーション数です。通常はセッションに参加する最大ステーション数を指定する必要があります。
    @tparam LATEST_BUFFER_SIZE  直近の LATEST_BUFFER_SIZE 回の common::Scheduler::Dispatch() の間の送受信を計測します。
                                必要のない場合は 0 を指定する必要があります。

    @see    CloneProtocol::SetProfiler, CloneBase::SetProfiler, CloneElementBase::SetProfiler
 */
template <uint32_t STATION_NUM, uint32_t LATEST_BUFFER_SIZE>
class CloneProfiler : public CloneProfilerBase
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    CloneProfiler()
        : CloneProfilerBase(
              (LATEST_BUFFER_SIZE > 0) ? m_DispatchCountBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_CountBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_SizeBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_NoCompressedSizeBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_UnicastCountBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_UnicastSizeBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_ResendCountBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_ResendSizeBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_SystemCountBuffer : NULL,
              (LATEST_BUFFER_SIZE > 0) ? m_SystemSizeBuffer : NULL,
              m_TotalCountBuffer,
              m_TotalSizeBuffer,
              m_TotalNoCompressedSizeBuffer,
              m_TotalUnicastCountBuffer,
              m_TotalUnicastSizeBuffer,
              m_TotalResendCountBuffer,
              m_TotalResendSizeBuffer,
              m_TotalSystemCountBuffer,
              m_TotalSystemSizeBuffer,
              StationNum,
              LATEST_BUFFER_SIZE)
    {
        PIA_COMPILE_ASSERT(STATION_NUM > 0);
    };

    void Print(const CloneProtocol* pCloneProtocol) const
    {
        CloneProfilerBase::Print(pCloneProtocol);
    }

private:
    static const int StationNum = STATION_NUM < 32 ? STATION_NUM : 32;

    // Latest
    static const int DispatchCountBufferLength = LATEST_BUFFER_SIZE;
    static const int CountBufferLength = LATEST_BUFFER_SIZE * StationNum;
    static const int SizeBufferLength = LATEST_BUFFER_SIZE * StationNum;

    uint32_t m_DispatchCountBuffer[DispatchCountBufferLength];

    uint8_t m_CountBuffer[CountBufferLength];
    uint16_t m_SizeBuffer[SizeBufferLength];
    uint16_t m_NoCompressedSizeBuffer[SizeBufferLength];

    uint8_t m_UnicastCountBuffer[CountBufferLength];
    uint16_t m_UnicastSizeBuffer[SizeBufferLength];

    uint8_t m_ResendCountBuffer[CountBufferLength];
    uint16_t m_ResendSizeBuffer[SizeBufferLength];

    uint8_t m_SystemCountBuffer[CountBufferLength];
    uint16_t m_SystemSizeBuffer[SizeBufferLength];

    // Total
    static const int TotalCountBufferLength = StationNum;
    static const int TotalSizeBufferLength = StationNum;

    uint16_t m_TotalCountBuffer[TotalCountBufferLength];
    uint32_t m_TotalSizeBuffer[TotalSizeBufferLength];
    uint32_t m_TotalNoCompressedSizeBuffer[TotalSizeBufferLength];

    uint16_t m_TotalUnicastCountBuffer[TotalCountBufferLength];
    uint32_t m_TotalUnicastSizeBuffer[TotalSizeBufferLength];

    uint16_t m_TotalResendCountBuffer[TotalCountBufferLength];
    uint32_t m_TotalResendSizeBuffer[TotalSizeBufferLength];

    uint16_t m_TotalSystemCountBuffer[TotalCountBufferLength];
    uint32_t m_TotalSystemSizeBuffer[TotalSizeBufferLength];

    NN_PIA_DISALLOW_COPY(CloneProfiler);
};
}
}
}  // end of namespace nn::pia::clone
