﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/pia_Macros.h>


namespace nn
{
namespace pia
{
namespace clone
{
class CloneProtocol;

/*!
    @brief  PiaClone のプロファイリング管理クラスの基底クラスです。
 */
class CloneProfilerBase
{
public:
    enum ProfileTargetType
    {
        ProfileTargetType_None = 0,
        ProfileTargetType_CloneProtocol,
        ProfileTargetType_Clone,
        ProfileTargetType_CloneElement
    };

    enum ProfileTargetDirection
    {
        ProfileTargetDirection_None = 0,
        ProfileTargetDirection_Send,
        ProfileTargetDirection_Receive
    };

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


    //! @cond PRIVATE

public:
    void Setup(ProfileTargetType type, ProfileTargetDirection direction);

    void Reset();

    bool IsProfileLatest() const
    {
        return (m_LatestBufferSize > 0);
    }

    void ProfileByStationIndex(uint32_t dispatchCount, StationIndex stationIndex, uint32_t size, uint32_t noCompressedSize, bool isUnicast, bool isResend, bool isSystem);
    void ProfileByStationBmp(uint32_t dispatchCount, uint32_t stationBmp, uint32_t size, uint32_t noCompressedSize, bool isUnicast, bool isResend, bool isSystem);

    bool IsNocompressedSupported() const;
    bool IsUnicastSupported() const;
    bool IsResendSupported() const;
    bool IsSystemSupported() const;

    uint32_t GetLatestCount(StationIndex stationIndex, uint32_t dispatchCount) const;
    uint32_t GetLatestSize(StationIndex stationIndex, uint32_t dispatchCount) const;
    uint32_t GetLatestNoCompressedSize(StationIndex stationIndex, uint32_t dispatchCount) const;

    uint32_t GetLatestUnicastCount(StationIndex stationIndex, uint32_t dispatchCount) const;
    uint32_t GetLatestUnicastSize(StationIndex stationIndex, uint32_t dispatchCount) const;

    uint32_t GetLatestResendCount(StationIndex stationIndex, uint32_t dispatchCount) const;
    uint32_t GetLatestResendSize(StationIndex stationIndex, uint32_t dispatchCount) const;

    uint32_t GetLatestSystemCount(StationIndex stationIndex, uint32_t dispatchCount) const;
    uint32_t GetLatestSystemSize(StationIndex stationIndex, uint32_t dispatchCount) const;

    uint32_t GetTotalCount(StationIndex stationIndex) const;
    uint32_t GetTotalSize(StationIndex stationIndex) const;
    uint32_t GetTotalNoCompressedSize(StationIndex stationIndex) const;

    uint32_t GetTotalUnicastCount(StationIndex stationIndex) const;
    uint32_t GetTotalUnicastSize(StationIndex stationIndex) const;

    uint32_t GetTotalResendCount(StationIndex stationIndex) const;
    uint32_t GetTotalResendSize(StationIndex stationIndex) const;

    uint32_t GetTotalSystemCount(StationIndex stationIndex) const;
    uint32_t GetTotalSystemSize(StationIndex stationIndex) const;

    void Print(const CloneProtocol* pCloneProtocol) const;

protected:
    CloneProfilerBase(uint32_t* paDispatchCount, uint8_t* paCount, uint16_t* paSize, uint16_t* paNoCompressedSize, uint8_t* paUnicastCount, uint16_t* paUnicastSize, uint8_t* paResendCount, uint16_t* paResendSize, uint8_t* paSystemCount, uint16_t* paSystemSize, uint16_t* paTotalCount, uint32_t* paTotalSize, uint32_t* paTotalNoCompressedSize, uint16_t* paTotalUnicastCount, uint32_t* paTotalUnicastSize, uint16_t* paTotalResendCount, uint32_t* paTotalResendSize, uint16_t* paTotalSystemCount, uint32_t* paTotalSystemSize, uint32_t stationNum, uint32_t latestBufferSize);

private:
    uint32_t GetBufferIdx(uint32_t stationIndex, uint32_t idx) const
    {
        return stationIndex + idx * m_StationNum;
    }

    void AssignBuffer(uint32_t dispatchCount);

    void AddProfileData(StationIndex stationIndex, uint32_t size, uint32_t noCompressedSize, bool isUnicast, bool isResend, bool isSystem);

    template <typename T>
    uint32_t GetLatestData(const T* dataBuffer, StationIndex stationIndex, uint32_t dispatchCount) const;

    template <typename T>
    uint32_t GetTotalData(const T* dataBuffer, StationIndex stationIndex) const;

private:
    ProfileTargetType m_ProfileTargetType;
    ProfileTargetDirection m_ProfileTargetDirection;

    // Latest
    uint32_t* m_paDispatchCount;
    uint8_t* m_paCount;
    uint16_t* m_paSize;
    uint16_t* m_paNoCompressedSize;

    uint8_t* m_paUnicastCount;  // Count 中
    uint16_t* m_paUnicastSize;  // Size 中

    uint8_t* m_paResendCount;  // Count 中
    uint16_t* m_paResendSize;  // Size 中

    uint8_t* m_paSystemCount;  // Count 中
    uint16_t* m_paSystemSize;  // Size 中

    // Total
    uint16_t* m_paTotalCount;
    uint32_t* m_paTotalSize;
    uint32_t* m_paTotalNoCompressedSize;

    uint16_t* m_paTotalUnicastCount;  // TotalCount 中
    uint32_t* m_paTotalUnicastSize;   // TotalSize 中

    uint16_t* m_paTotalResendCount;  // TotalCount 中
    uint32_t* m_paTotalResendSize;   // TotalSize 中

    uint16_t* m_paTotalSystemCount;  // TotalCount 中
    uint32_t* m_paTotalSystemSize;   // TotalSize 中

    //
    uint32_t m_StationNum;
    uint32_t m_LatestBufferSize;

    uint32_t m_CurrentIdx;

    NN_PIA_DISALLOW_COPY(CloneProfilerBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
