﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_PacketDefine.h>
#include <nn/pia/common/common_OffsetTreeMap.h>
#include <nn/pia/common/common_OffsetList.h>
#include <nn/pia/transport/transport_Protocol.h>
#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_SendClone.h>
#include <nn/pia/clone/clone_ReceiveClone.h>
#include <nn/pia/clone/clone_AtomicSharingClone.h>
#include <nn/pia/clone/clone_SequentialSharingClone.h>

namespace nn
{
namespace pia
{
namespace clone
{

class Clock;
class IDataPacker;
class IDataUnpacker;
class CompressDataUnpacker;


/*!
    @brief  ステーション間で値の共有を行うためのプロトコルです。<br/>
 */
class CloneProtocol : public transport::Protocol
{
public:
    PIA_PROTOCOL_TYPE_INFO(nn::pia::transport::ProtocolTypeClone);

    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。
     */
    CloneProtocol();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。
     */
    virtual ~CloneProtocol();
    //! @endcond


    /*!
    @brief  順序比較関数です。
    @param[in]  clock       比較を行う時刻です。
    @param[in]  stationA    比較対象のステーションの @ref StationIndex です。
    @param[in]  stationB    比較対象のステーションの @ref StationIndex です。
    @return     時刻が clock の時に stationA より stationB の方が前に起こったと扱う場合は負の値を、
                後に起こったと扱う場合は正の値を返します。
    @details    stationA と stationB が同じ値で呼び出されることはありません。
    @details    この関数は、ある時刻において順序関係が矛盾しないようになっている必要があります。
                例えば、 ClockPriorityJudgeFunction(clock, stationA, stationB) < 0 であるならば、
                ClockPriorityJudgeFunction(clock, stationB, stationA) > 0 とならなければなりません。
                また、 ClockPriorityJudgeFunction(clock, stationA, stationB) < 0 かつ ClockPriorityJudgeFunction(clock, stationB, stationC) < 0 であるならば
                ClockPriorityJudgeFunction(clock, stationA, statoinC) < 0 とならなければなりません。
    @details    ただし、別の時刻においては順序関係が一致しなくても構いません。
                すなわち、ClockPriorityJudgeFunction(clock1, stationA, stationB) < 0 である場合に、
                ClockPriorityJudgeFunction(clock2, stationA, stationB) > 0 となることは問題ありません。
     */
    typedef int (*ClockPriorityJudgeFunction)(ClockValue clock, StationIndex stationA, StationIndex stationB);


    static const uint32_t CompressionLevelNone = 0;    //!< @ref CloneProtocol::Setting::dataCompressionLevel で指定する送信データの圧縮レベル（圧縮なし）です。
    static const uint32_t CompressionLevelLow = 1;     //!< @ref CloneProtocol::Setting::dataCompressionLevel で指定する送信データの圧縮レベル（速度重視）です。
    static const uint32_t CompressionLevelMiddle = 5;  //!< @ref CloneProtocol::Setting::dataCompressionLevel で指定する送信データの圧縮レベルです。
    static const uint32_t CompressionLevelHigh = 9;    //!< @ref CloneProtocol::Setting::dataCompressionLevel で指定する送信データの圧縮レベル（圧縮率重視）です。

    /*!
    @brief  初期化時に指定する設定構造体です。
    @details    通信するステーション間で同じ値を指定する必要があります。
     */
    struct Setting
    {
    public:
        /*!
        @brief      デフォルトコンストラクタです。
        @details    デフォルト値で初期化されます。
         */
        Setting()
            : clockType(ClockType_Rtc),
              clockPerSec(1000.0f),
              clockPerFrame(1),
              clockRegulationRate(0.1f),
              clockPriorityJudgeMethod(ClockPriorityJudgeMethod_Simple),
              clockPriorityJudgeFunction(NULL),
              dataCompressionLevel(CompressionLevelNone),
              ackBufferNum(32)
        {
        }


        /*!
        @brief  何を基準として時刻を進めるかを表します。
         */
        enum ClockType
        {
            ClockType_Rtc = 0,  //!< RTC を基準に時刻を進めます。
            ClockType_Frame     //!< CloneProtocol::UpdateClock() の呼び出し回数を基準に時刻を進めます。
        };


        /*!
        @brief  時刻を進める基準を指定します。
        @details    @ref CloneProtocol::Setting::ClockType_Rtc を指定した場合は、RTC を基準に時刻を進めます。
                    開始時に時刻調整がされた後はステーション間で時刻がずれることを考慮する必要はありませんが、
                    CloneProtocol::UpdateClock() の呼び出しのタイミングによって、1 フレームあたりに進む時刻が一定ではなくなります。
                    時刻を進める速さを clockPerSec で指定する必要があります。
        @details    @ref CloneProtocol::Setting::ClockType_Frame を指定した場合は、通常は CloneProtocol::UpdateClock() 呼び出しあたりに進む時刻の値が一定になります。
                    ただし、処理落ちなどで CloneProtocol::UpdateClock() の呼び出し回数がステーション間でずれた場合に、
                    ステーション間で時刻が一致しなくなる可能性があります。
                    時刻を進める速さを clockPerFrame で指定する必要があります。
        @details    時刻管理の詳細についてはプログラミングマニュアルを参照してください。
        @see    clockPerSec
        @see    clockPerFrame
         */
        ClockType clockType;


        /*!
        @brief  1 秒あたりに進める時刻の値を指定します。
        @details    @ref CloneProtocol::Setting::clockType に @ref CloneProtocol::Setting::ClockType_Rtc 以外を指定した場合は、この設定は無視されます。
        @see    clockType
         */
        float clockPerSec;


        /*!
        @brief  1 フレームあたりに進める時刻の値を指定します。
        @details    @ref CloneProtocol::Setting::clockType に @ref CloneProtocol::Setting::ClockType_Frame 以外を指定した場合は、この設定は無視されます。
        @see    clockType
         */
        uint32_t clockPerFrame;


        /*!
        @brief  時刻の再調整を行う際に、時刻の進め方を加速／減速する割合を指定します。
        @details    CloneProtocol::RegulateClock() で時刻を再調整する際、ここで指定した割合だけ通常時の時刻の進み方より速く／遅く時刻を進めます。
                    例えば 0.1 を指定した場合、他のステーションより時刻が遅れている場合は、通常時と比べ 110% の速さで時刻が進みます。
                    逆に他のステーションより時刻が進んでいる場合は、通常時と比べ 90% の速さで時刻が進みます。
         */
        float clockRegulationRate;


        /*!
        @brief  別のステーション間で同時刻に起こった事柄を、どちらが先に起こった事として扱うかを判定する方法を表します。
         */
        enum ClockPriorityJudgeMethod
        {
            ClockPriorityJudgeMethod_Simple = 0,  //!< ステーション ID の値の小さいステーションで起こった事柄の方が先として扱います。
            ClockPriorityJudgeMethod_Impartial,   //!< 時刻ごとに優先順を変えることにより、長期的にはステーション間で有利不利が出ないように判定します。
            ClockPriorityJudgeMethod_UserDefine   //!< 判定方法をアプリケーションで指定します。
        };


        /*!
        @brief  別のステーション間で同時刻に起こった事柄を、どちらが先に起こった事として扱うかを判定する方法を指定します。
        @details    @ref CloneProtocol::Setting::ClockPriorityJudgeMethod_UserDefine を指定した場合は、判定関数を @ref CloneProtocol::Setting::clockPriorityJudgeFunction に指定する必要があります。
        @see    clockPriorityJudgeFunction
         */
        ClockPriorityJudgeMethod clockPriorityJudgeMethod;


        /*!
        @brief  別のステーション間で同時刻に起こった事柄を、どちらが先に起こった事として扱うかを判定する関数を指定します。
        @details    @ref CloneProtocol::Setting::clockPriorityJudgeMethod に @ref CloneProtocol::Setting::ClockPriorityJudgeMethod_UserDefine 以外を指定した場合は、この設定は無視されます。
        @see    clockPriorityJudgeMethod;
         */
        ClockPriorityJudgeFunction clockPriorityJudgeFunction;


        /*!
        @brief      送信データの圧縮レベルを指定します。
        @details    @ref CloneProtocol::CompressionLevelNone を指定すると圧縮を行いません。
                    @ref CloneProtocol::CompressionLevelLow が速度重視、@ref CloneProtocol::CompressionLevelHigh が圧縮率重視となります。
                    @ref CloneProtocol::CompressionLevelMiddle はその中間です。
                    デフォルト値は @ref CloneProtocol::CompressionLevelNone です。
                    これらの値以外はサポート対象外です。
                    プラットフォームの CPU 性能によって処理時間が大きく変動します。
         */
        uint32_t dataCompressionLevel;


        /*!
        @brief      Ack 用バッファの数を指定します。
        @details    バッファ 1 個につき、約 1.5 KB のメモリを消費します。1 以上の値を指定する必要があります。実際に通信中に使用されたバッファの数の最大値は、@ref CloneProtocol::GetAckBufferUsedNumMax() で取得できます。
        @see        GetAckBufferUsedNumMax
        */
        int ackBufferNum;
    };


    /*!
    @brief  初期化します。 clone::BeginSetup() ～ clone::EndSetup() 間で呼び出す必要があります。
    @param[in]  setting     各種設定情報です。
                            すべてのステーション間で、同じ値を設定する必要があります。異なる値を設定した場合の動作は不定です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultAlreadyInitialized    既に初期化されています。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @see    Finalize
     */
    Result Initialize(const Setting& setting);


    /*!
    @brief  終了処理を行います。
    @see    Initialize
     */
    void Finalize();


    /*!
    @brief  使用するクローンエレメントの SerializePolicy::GetSerializedSize() で定義できる値の最大値を取得します。
    @details    この値より大きなクローンエレメントを登録してあるクローンを RegisterXXXClone() に渡すと、 @ref ResultInvalidArgument が返ります。
    @details    この値は 1 パケットに収めることのできる最大サイズです。
                使用する通信モジュールやそこで設定する MTU の設定によって値が変わります。
                この値は、CloneProtocol::Initialize() 以降取得することができます。
    @return 使用するクローンエレメントの SerializePolicy::Size で指定できる値の最大値です。
     */
    uint32_t GetElementSizeMax() const
    {
        return m_ElementSizeMax;
    }


    /*!
    @brief  SendClone を指定の ID に登録します。
    @param[in]  pClone  登録する SendClone です。
    @param[in]  cloneId 登録する ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultAlreadyExists     指定された ID には既に登録されています。[:progErr]
     */
    Result RegisterSendClone(SendClone* pClone, CloneBase::Id cloneId)
    {
        return RegisterClone(SendClone::ToKey(cloneId), pClone);
    }


    /*!
    @brief  SendClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても pClone->@ref SendClone::IsRegisteredWithProtocol() "IsRegisteredWithProtocol()" が 
                false になるまで pClone を使いまわしたり同じ ID に再登録したりできません。
    @param[in]  pClone  登録解除する SendClone です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。この CloneProtocol に登録されていない SendClone が指定されています。[:progErr]
     */
    Result UnregisterSendClone(SendClone* pClone)
    {
        return UnregisterClone(pClone);
    }


    /*!
    @brief  SendClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても登録解除した SendClone::IsRegisteredWithProtocol() が false になるまで
                その SendClone を使いまわしたり同じ ID に再登録したりできません。
    @param[in]  cloneId 登録解除する SendClone の ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultNotSet            指定された ID には登録されていません。[:progErr]
     */
    Result UnregisterSendClone(CloneBase::Id cloneId)
    {
        return UnregisterClone(SendClone::ToKey(cloneId));
    }


    /*!
    @brief  指定した ID の SendClone を検索して取得します。
    @param[in]  cloneId 検索する SendClone の ID です。
    @return 指定された ID の SendClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    SendClone* SearchSendClone(CloneBase::Id cloneId)
    {
        return static_cast<SendClone*>(m_CloneCollection.Find(SendClone::ToKey(cloneId)));
    }


    /*!
    @brief  指定した ID の SendClone を検索して取得します。
    @param[in]  cloneId 検索する SendClone の ID です。
    @return 指定された ID の SendClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    const SendClone* SearchSendClone(CloneBase::Id cloneId) const
    {
        return static_cast<const SendClone*>(m_CloneCollection.Find(SendClone::ToKey(cloneId)));
    }


    /*!
    @brief  ReceiveClone を指定の ID に登録します。
    @param[in]  pClone              登録する ReceiveClone です。
    @param[in]  sendStationIndex    送信側の SendClone のある @ref StationIndex です。
    @param[in]  cloneId             登録する ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultAlreadyExists     指定された ID には既に登録されています。[:progErr]
     */
    Result RegisterReceiveClone(ReceiveClone* pClone, StationIndex sendStationIndex, CloneBase::Id cloneId);


    /*!
    @brief  ReceiveClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても pClone->@ref ReceiveClone::IsRegisteredWithProtocol() "IsRegisteredWithProtocol()" が 
                false になるまで pClone を使いまわしたり同じ ID に再登録したりできません。
                （ステーションの不正切断時の代理送信等のため ReceiveClone も送信を行う事があります。）
    @param[in]  pClone  登録解除する ReceiveClone です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。この CloneProtocol に登録されていない ReceiveClone が指定されています。[:progErr]
     */
    Result UnregisterReceiveClone(ReceiveClone* pClone)
    {
        return UnregisterClone(pClone);
    }


    /*!
    @brief  ReceiveClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても登録解除した ReceiveClone::IsRegisteredWithProtocol() が false になるまで
                その ReceiveClone を使いまわしたり同じ ID に再登録したりできません。
                （ステーションの不正切断時の代理送信等のため ReceiveClone も送信を行う事があります。）
    @param[in]  sendStationIndex    送信側の SendClone のある @ref StationIndex です。
    @param[in]  cloneId             登録解除する ReceiveClone の ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultNotSet            指定された ID には登録されていません。[:progErr]
     */
    Result UnregisterReceiveClone(StationIndex sendStationIndex, CloneBase::Id cloneId);


    /*!
    @brief  指定した ID の ReceiveClone を検索して取得します。
    @param[in]  sendStationIndex    送信側の SendClone のある @ref StationIndex です。
    @param[in]  cloneId             検索する ReceiveClone の ID です。
    @return 指定された ID の ReceiveClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    ReceiveClone* SearchReceiveClone(StationIndex sendStationIndex, CloneBase::Id cloneId);


    /*!
    @brief  指定した ID の ReceiveClone を検索して取得します。
    @param[in]  sendStationIndex    送信側の SendClone のある @ref StationIndex です。
    @param[in]  cloneId             検索する ReceiveClone の ID です。
    @return 指定された ID の ReceiveClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    const ReceiveClone* SearchReceiveClone(StationIndex sendStationIndex, CloneBase::Id cloneId) const;


    /*!
    @brief  AtomicSharingClone を指定の ID に登録します。
    @param[in]  pClone  登録する AtomicSharingClone です。
    @param[in]  cloneId 登録する ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultAlreadyExists     指定された ID には既に登録されています。[:progErr]
     */
    Result RegisterAtomicSharingClone(AtomicSharingClone* pClone, CloneBase::Id cloneId)
    {
        return RegisterClone(AtomicSharingClone::ToKey(cloneId), pClone);
    }


    /*!
    @brief  AtomicSharingClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても pClone->@ref AtomicSharingClone::IsRegisteredWithProtocol() "IsRegisteredWithProtocol()" が 
                false になるまで pClone を使いまわしたり同じ ID に再登録したりできません。
                （ステーションの不正切断時の代理送信等のため、送信権を取得していなくても送信を行う事があります。）
    @param[in]  pClone  登録解除する AtomicSharingClone です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。この CloneProtocol に登録されていない AtomicSharingClone が指定されています。[:progErr]
     */
    Result UnregisterAtomicSharingClone(AtomicSharingClone* pClone)
    {
        return UnregisterClone(pClone);
    }


    /*!
    @brief  AtomicSharingClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても登録解除した AtomicSharingClone::IsRegisteredWithProtocol() が false になるまで
                その AtomicSharingClone を使いまわしたり同じ ID に再登録したりできません。
                （ステーションの不正切断時の代理送信等のため、送信権を取得していなくても送信を行う事があります。）
    @param[in]  cloneId 登録解除する AtomicSharingClone の ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultNotSet            指定された ID には登録されていません。[:progErr]
     */
    Result UnregisterAtomicSharingClone(CloneBase::Id cloneId)
    {
        return UnregisterClone(AtomicSharingClone::ToKey(cloneId));
    }


    /*!
    @brief  指定した ID の AtomicSharingClone を検索して取得します。
    @param[in]  cloneId 検索する AtomicSharingClone の ID です。
    @return 指定された ID の AtomicSharingClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    AtomicSharingClone* SearchAtomicSharingClone(CloneBase::Id cloneId)
    {
        return static_cast<AtomicSharingClone*>(m_CloneCollection.Find(AtomicSharingClone::ToKey(cloneId)));
    }


    /*!
    @brief  指定した ID の AtomicSharingClone を検索して取得します。
    @param[in]  cloneId 検索する AtomicSharingClone の ID です。
    @return 指定された ID の AtomicSharingClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    const AtomicSharingClone* SearchAtomicSharingClone(CloneBase::Id cloneId) const
    {
        return static_cast<const AtomicSharingClone*>(m_CloneCollection.Find(AtomicSharingClone::ToKey(cloneId)));
    }


    /*!
    @brief  SequentialSharingClone を指定の ID に登録します。
    @param[in]  pClone  登録する SequentialSharingClone です。
    @param[in]  cloneId 登録する ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultAlreadyExists     指定された ID には既に登録されています。[:progErr]
     */
    Result RegisterSequentialSharingClone(SequentialSharingClone* pClone, CloneBase::Id cloneId)
    {
        return RegisterClone(SequentialSharingClone::ToKey(cloneId), pClone);
    }


    /*!
    @brief  SequentialSharingClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても pClone->@ref SequentialSharingClone::IsRegisteredWithProtocol() "IsRegisteredWithProtocol()" が 
                false になるまで pClone を使いまわしたり同じ ID に再登録したりできません。
    @param[in]  pClone  登録解除する SequentialSharingClone です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。この CloneProtocol に登録されていない AtomicSharingClone が指定されています。[:progErr]
     */
    Result UnregisterSequentialSharingClone(SequentialSharingClone* pClone)
    {
        return UnregisterClone(pClone);
    }


    /*!
    @brief  SequentialSharingClone を登録解除します。
    @details    送信途中のものがある場合は送信しきってから登録解除されます。
                そのため、この関数が成功したとしても登録解除した SequentialSharingClone::IsRegisteredWithProtocol() が false になるまで
                その SequentialSharingClone を使いまわしたり同じ ID に再登録したりできません。
    @param[in]  cloneId 登録解除する SequentialSharingClone の ID です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultNotSet            指定された ID には登録されていません。[:progErr]
     */
    Result UnregisterSequentialSharingClone(CloneBase::Id cloneId)
    {
        return UnregisterClone(SequentialSharingClone::ToKey(cloneId));
    }


    /*!
    @brief  指定した ID の SequentialSharingClone を検索して取得します。
    @param[in]  cloneId 検索する SequentialSharingClone の ID です。
    @return 指定された ID の SequentialSharingClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    SequentialSharingClone* SearchSequentialSharingClone(CloneBase::Id cloneId)
    {
        return static_cast<SequentialSharingClone*>(m_CloneCollection.Find(SequentialSharingClone::ToKey(cloneId)));
    }


    /*!
    @brief  指定した ID の SequentialSharingClone を検索して取得します。
    @param[in]  cloneId 検索する SequentialSharingClone の ID です。
    @return 指定された ID の SequentialSharingClone です。指定した ID のものが無い場合は、NULL を返します。
     */
    const SequentialSharingClone* SearchSequentialSharingClone(CloneBase::Id cloneId) const
    {
        return static_cast<const SequentialSharingClone*>(m_CloneCollection.Find(SequentialSharingClone::ToKey(cloneId)));
    }


    /*!
    @cond PRIVATE
    @brief  通信を開始します。

    @param[in] localStationAddress  自分の StationAddress です。

    @return 成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidState      既に通信中です。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    virtual Result StartupWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を終了します。
     */
    virtual void CleanupWithStationAddress();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  メインの処理です。

    @return ResultSuccess が返されます。
     */
    virtual Result Dispatch();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  接続状態の切り替え。

    @param[in] event    操作内容を表すイベントです。

    @return 成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState      呼び出し状態が不正です。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    virtual Result UpdateProtocolEvent(const transport::ProtocolEvent& event);
    //! @endcond


    /*!
    @brief  CloneProtocol の動作状態を表します。
     */
    enum State
    {
        State_Inactive = 0x10,  //!< 動作中ではありません。
        State_Start = 0x20,     //!< 開始処理中です。
        State_Active = 0x30,    //!< 動作中です。
        State_Stop = 0x40       //!< 終了処理中です。
    };


    /*!
    @brief  現在の動作状態を取得します。
    @return 現在の動作状態です。
     */
    State GetState() const
    {
        return static_cast<State>(m_InnerState & StateMask);
    }


    /*!
    @brief  指定したステーションの CloneProtocol が動作中かどうかを判定します。
    @param[in] stationId    動作中かどうかを調べたいステーションの @ref StationId です。
    @return 指定したステーションが動作中なら true を返します。
     */
    bool IsActiveStation(StationId stationId) const;


    /*!
    @brief  CloneProtocol の動作を開始します。
    @details    通信開始後、@ref CloneProtocol::State_Inactive の時に呼び出すことができます。
                成功すると @ref CloneProtocol::State_Start に遷移して開始処理を行った後、 @ref CloneProtocol::State_Active に遷移します。
                この呼び出し以降、定期的に CloneProtocol::UpdateClock() を呼ぶ必要があります。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState  通信中でないか、@ref nn::pia::clone::CloneProtocol::State_Inactive "State_Inactive" ではありません。[:progErr]
     */
    Result Start();


    /*!
    @brief  CloneProtocol の動作を終了します。
    @details    @ref CloneProtocol::State_Start または @ref CloneProtocol::State_Active の時に呼び出すと @ref CloneProtocol::State_Stop に遷移し、終了処理の完了後に @ref CloneProtocol::State_Inactive に遷移します。
                @ref CloneProtocol::State_Stop の間は、まだ CloneProtocol::UpdateClock() を呼び続ける必要があります。
                @ref CloneProtocol::State_Inactive, @ref CloneProtocol::State_Stop の時に呼び出しても何も起こりません。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState  通信中ではありません。[:progErr]
     */
    Result Stop();


    /*!
    @brief  時刻を 1 フレーム進めます。
    @details    この関数を呼びだした時だけ CloneProtocol の管理する時刻は進みます。
                @ref CloneProtocol::State_Start, @ref CloneProtocol::State_Active, @ref CloneProtocol::State_Stop のいずれかの間は定期的に呼び出す必要があります。
                呼び出しに対してどれだけの時刻が進むかは、CloneProtocol::Initialize() 時に指定する CloneProtocol::Setting で指定する必要があります。
                時刻の値の精度に対して呼び出し間隔が短い場合は時刻が変わらない場合がありますが、遡ることはありません。
    @see    GetClock
     */
    void UpdateClock();


    /*!
    @brief  現在の時刻を取得します。
    @return 現在の時刻です。動作中でない時など有効な時刻が存在しない場合は @ref InvalidClock を返します。
    @see    UpdateClock
     */
    ClockValue GetClock() const;


    /*!
    @brief  時刻の再調整を開始します。
    @details    このステーションの CloneProtocol の管理する時刻を、他のステーションの平均的な時刻にそろえるように再調整します。
                他のステーションの時刻を調べた後、通常より高速／低速に時刻を進めることにより誤差を修正します。
                どの程度高速／低速に進めるかは CloneProtocol::Initialize() 時に指定する CloneProtocol::Setting::clockRegulationRate で指定する必要があります。
                @ref CloneProtocol::Setting::clockType に @ref CloneProtocol::Setting::ClockType_Rtc を指定した場合は、同期通信を長時間継続していると、端末間の時刻のずれが 10 時間で数秒程度大きくなる可能性があります。長時間の同期通信を行う場合は、この関数を定期的に呼び出すことを検討してください。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState  @ref nn::pia::clone::CloneProtocol::State_Active "State_Active" でないか、既に調整を実行中です。[:progErr]
    @see    IsRegulatingClock
     */
    Result RegulateClock();


    /*!
    @brief  時刻の調整を実行中かどうかを取得します。
    @return 時刻の調整を実行中なら true を返します。
    @see    RegulateClock
     */
    bool IsRegulatingClock() const;


    /*!
    @brief  （デバッグ用）時刻の値がオーバーフローするまでの期間を設定します。

    @details    同期通信を開始してから、時刻の値がオーバーフローするまでの期間を設定します。
    @details    本関数はデバッグ用です。製品には組み込まないよう、注意してください。

    @param[in] span @ref CloneProtocol::Setting::ClockType が @ref CloneProtocol::Setting::ClockType_Rtc の場合は、時間（単位は秒）、@ref CloneProtocol::Setting::ClockType_Frame の場合は、フレーム数です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
     */
    Result SetClockOverFlowSpanForDebug(uint32_t span);


    /*!
    @brief      @ref CloneProtocol::SetSendPeriod() で指定した送信間隔の設定値を取得します。

    @return     送信間隔の設定値です。
    */
    uint32_t GetSendPeriod() const
    {
        return m_SendPeriod;
    }


    /*!
    @brief      送信間隔を設定します。
    @details    ここで指定した回数 @ref CloneProtocol::UpdateClock() が呼び出されるたびに、@ref common::Scheduler::Dispatch() 呼び出し時に送信処理が行われます。1 以上の値を指定する必要があります。

    @param[in]  sendPeriod 設定する送信間隔です。
    @return     成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument       引数が不正です。[:progErr]
    */
    Result SetSendPeriod(uint32_t sendPeriod);


    /*!
    @brief  エラーの発生状態を表します。
     */
    enum ErrorType
    {
        ErrorType_Nothing = 0,   //!< エラーはありません。
        ErrorType_EventDropped,  //!< イベントの管理に不整合が生じました。一部のイベントが通知されなかった可能性があります。
                                 //!< EventCloneElement を使用していた場合に発生する可能性があります。
        ErrorType_ClockOverflow  //!< 時刻の値がオーバーフローしました。全体的に値の不整合が起こっている可能性があります。
    };


    /*!
    @brief  エラーの発生状態を取得します。
    @details    エラーが発生した場合は、送受信している値に不整合が起こっている可能性がありますので、
                ゲームを無効として CloneProtocol::Stop() で終了させることを推奨します。
                再び CloneProtocol::Start() を呼び出した際にこの値はリセットされます。
                複数の種類のエラーが発生した場合は、@ref CloneProtocol::ErrorType の値の大きい方が優先されます。
    @return エラーの発生状態です。
     */
    ErrorType GetError() const
    {
        return m_Error;
    }


    /*!
    @brief  計測して求めた基準となる RTT（ミリ秒） に対する再送間隔の比率を指定します。
    @details    RTT に対する再送間隔の比率のデフォルト値は 1.25 です。再送間隔は、(RTT * rttRatio + (送信間隔 + 1) * 1 フレームに相当する時間) となります。1 フレームに相当する時間は、@ref CloneProtocol::UpdateClock() の呼び出し間隔に基づいて内部で計測されます。再送間隔の最大値は、2500 ms です。
    @param[in]  rttRatio 計測して求めた基準となる RTT（ミリ秒） に対する、再送間隔の比率です。0 より大きい値を指定する必要があります。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    Result SetResendSpanRatio(float rttRatio);


    //! @name プロファイリング
    //! @{

    /*!
    @brief  送受信のプロファイリング管理オブジェクトを設定します。
    @details    CloneProfiler を設定すると、 CloneProtocol が行った送受信をプロファイリングできます。
                ここでの計測は、この CloneProtocol が実際に通信路に送信し、通信路から受信したものが計上されます。
                クローンのプロファイリングでは自ステーション内での送受信も計上されますが、
                それらは CloneProtocol 内で解決されるため、 CloneProtocol でのプロファイリングには計上されません。
    @details    圧縮を有効にしていた場合、計測されるサイズは圧縮された状態でのサイズです。
    @details    引数に NULL を指定すると、既に設定してある CloneProfiler を解除する事ができます。
    @param[in]  pSendProfiler       送信のプロファイリングを管理する CloneProfiler を指定します。
                                    必要ない場合は NULL を指定する必要があります。
    @param[in]  pReceiveProfiler    受信のプロファイリングを管理する CloneProfiler を指定します。
                                    必要ない場合は NULL を指定する必要があります。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
     */
    Result SetProfiler(CloneProfilerBase* pSendProfiler, CloneProfilerBase* pReceiveProfiler);


    /*!
    @brief  設定されている送信プロファイリング管理オブジェクトを取得します。
    @return 設定されている送信プロファイリング管理オブジェクトです。設定されていない場合は NULL を返します。
    @see    SetProfiler
     */
    const CloneProfilerBase* GetSendProfiler() const
    {
        return m_pSendProfiler;
    }


    /*!
    @brief  設定されている受信プロファイリング管理オブジェクトを取得します。
    @return 設定されている受信プロファイリング管理オブジェクトです。設定されていない場合は NULL を返します。
    @see    SetProfiler
     */
    const CloneProfilerBase* GetReceiveProfiler() const
    {
        return m_pReceiveProfiler;
    }


    /*!
    @brief  送受信のプロファイリングされた結果をリセットします。
    @see    SetProfiler
     */
    void ResetProfiler();


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信した CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信した CloneProtocol メッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信した CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信した CloneProtocol メッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信した CloneProtocol メッセージの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信した CloneProtocol メッセージの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信した CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信した CloneProtocol メッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendUnicastCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信した CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間にユニキャストで送信した CloneProtocol メッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendUnicastSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用 CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用 CloneProtocol メッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用 CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に送信したシステム用 CloneProtocol メッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestSendSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信した CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信した CloneProtocol メッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信した CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信した CloneProtocol メッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信した CloneProtocol メッセージの圧縮前のサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信した CloneProtocol メッセージの圧縮前のサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendNoCompressedSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降にユニキャストで送信した CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降にユニキャストで送信した CloneProtocol メッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendUnicastCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降にユニキャストで送信した CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降にユニキャストで送信した CloneProtocol メッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendUnicastSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したシステム用 CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したシステム用 CloneProtocol メッセージの個数です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSystemCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に送信したシステム用 CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に送信したシステム用 CloneProtocol メッセージのサイズ（バイト）です。
            送信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalSendSystemSize(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信した CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信した CloneProtocol メッセージの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveCount(StationIndex stationIndex) const;


    /*!
    @brief  直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信した CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return 直近の @ref CloneProfiler "CloneProfiler::LATEST_BUFFER_SIZE" 回の common::Scheduler::Dispatch の間に受信した CloneProtocol メッセージのサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler
     */
    uint32_t GetProfiledLatestReceiveSize(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信した CloneProtocol メッセージの個数を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信した CloneProtocol メッセージの個数です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveCount(StationIndex stationIndex) const;


    /*!
    @brief  ResetProfiler 以降に受信した CloneProtocol メッセージのサイズ（バイト）を取得します。
    @param[in]  stationIndex    プロファイリング対象の @ref StationIndex です。
    @return ResetProfiler() 以降に受信した CloneProtocol メッセージのサイズ（バイト）です。
            受信プロファイリング管理オブジェクトが設定されていない場合は 0xFFFFFFFF を返します。
    @see    SetProfiler, ResetProfiler
     */
    uint32_t GetProfiledTotalReceiveSize(StationIndex stationIndex) const;

    //! @}

    /*!
    @brief      通信中に実際に使用された Ack 用バッファ数の最大値を取得します。
    @details    この値は再初期化時に 0 にリセットされます。
    @return     通信中に実際に使用された Ack 用バッファ数の最大値です。
     */
    int GetAckBufferUsedNumMax() const
    {
        return m_AckMessageBufferUsedNumMax;
    }

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


public:
    /*!
    @cond PRIVATE
    @brief  再送間隔を取得します。（デバッグ用）
     */
    SystemTime GetResendSpan() const
    {
        return m_ResendSpan;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  ユニキャスト用の再送間隔を取得します。（デバッグ用）
     */
    SystemTime GetResendSpanForUnicast(StationIndex stationIndex) const
    {
        return m_ResendSpanForUnicast[stationIndex];
    }
    //! @endcond

    uint32_t GetStationNum() const
    {
        return m_StationNum;
    }

    //! @cond PRIVATE

private:
    static int SimpleClockPriorityJudgeFunction(ClockValue clock, StationIndex stationA, StationIndex stationB);
    static int ImpartialClockPriorityJudgeFunction(ClockValue clock, StationIndex stationA, StationIndex stationB);

public:
    StationIndex GetLocalStationIndex() const
    {
        return m_LocalStationIndex;
    }
    uint32_t GetParticipantBitmap() const
    {
        return m_ParticipantBitmap;
    }
    SystemTime GetSystemTime() const
    {
        return m_SystemTime;
    }
    uint32_t GetDispatchCount() const
    {
        return m_DispatchCount;
    }

    void ErrorEventDropped()
    {
        SetError(ErrorType_EventDropped);
    }

    // n2920 : CloneBaseで参照するために追加
    uint32_t GetValidStationBitmap() const
    {
        return m_ValidStationBitmap;
    }

    uint32_t GetNoticeParticipationBitmap() const
    {
        return m_NoticeParticipationBitmap;
    }

    bool IsBroadcastSupported() const
    {
        return m_IsBroadcastSupported;
    }

private:
    bool IsInitialized() const
    {
        return (m_StationNum > 0);
    }
    bool IsInCommunication() const
    {
        return m_LocalStationIndex != StationIndex_Invalid;
    }

    enum InnerState
    {
        InnerState_Inactive = State_Inactive | 0x1,
        InnerState_SyncClock = State_Start | 0x1,
        InnerState_NoticeStart = State_Start | 0x2,
        InnerState_Active = State_Active | 0x1,
        InnerState_CloneEnding = State_Stop | 0x1,
        InnerState_NoticeStop = State_Stop | 0x2
    };
    InnerState GetInnerState() const
    {
        return m_InnerState;
    }

    void UpdateCloneState();
    void UpdateState();
    void UpdateResendSpan();
    SystemTime CalcResendSpan(int rtt) const;

    void ReceiveMessages();
    void ReceiveMessage(StationIndex src, const void* cpMessage, uint32_t size);
    bool CheckReceivedMessageDestStationBitmap(const void* cpMessage);
    bool UpdateReceivedMessageSequentialId(StationIndex src, const void* cpMessage);

    void SerializeMessage(void* pMessage, uint32_t messageSize, uint32_t noCompressedMessageSize, uint8_t* serializedMessageBuf, uint32_t* serializedMessageSize, uint32_t* noCompressedSerializedMessageSize);
    template <typename MessageStruct>
    void SerializeMessageCore(void* pMessage, uint8_t* serializedMessageBuf, uint32_t* headerSize, uint32_t* serializedHeaderSize);
    void CalcSerializedMessageSize(void* pMessage, uint32_t messageSize, uint32_t noCompressedMessageSize, uint32_t* serializedMessageSize, uint32_t* noCompressedSerializedMessageSize);
    bool SendUnicastMessage(StationIndex dest, void* pMessage, uint32_t messageSize, uint32_t noCompressedMessageSize, bool isSystem);
    bool SendBroadcastMessage(uint32_t destBitmap, void* pMessage, uint32_t messageSize, uint32_t noCompressedMessageSize, bool isSystem);
    void SetSequentialIdToMessage(void* pMessage);
    void SetDestStationBitmapToMessage(void* pMessage, uint16_t destStationBitmap);

    bool SendBroadcastCloneCommand(StationIndex dest, CloneBase::Type cloneType, StationIndex cloneStationIndex, CloneBase::Id cloneId, uint8_t command);
    bool SendBroadcastClockCloneCommand(StationIndex dest, CloneBase::Type cloneType, StationIndex cloneStationIndex, CloneBase::Id cloneId, uint8_t command, ClockValue clock);
    bool SendBroadcastClockAndCountCloneCommand(StationIndex dest, CloneBase::Type cloneType, StationIndex cloneStationIndex, CloneBase::Id cloneId, uint8_t command, ClockValue clock, uint8_t count);
    bool SendBroadcastClockAndCountAndParticipantCloneCommand(StationIndex dest, CloneBase::Type cloneType, StationIndex cloneStationIndex, CloneBase::Id cloneId, uint8_t command, ClockValue clock, uint8_t count, uint32_t participantBitmap);

    void UpdateSystemTime();

    void SendClockRequest();
    void ReceiveClockRequest(StationIndex src, const void* cpMessage, uint32_t size);
    void ReceiveClockReplyMessage(StationIndex src, const void* cpMessage, uint32_t size);

    void NoticeParticipation();
    void ReceiveEnterMessage(StationIndex src, const void* cpMessage, uint32_t size);
    void ReceiveExitMessage(StationIndex src, const void* cpMessage, uint32_t size);
    void ReceiveEnterAckMessage(StationIndex src, const void* cpMessage, uint32_t size);
    void ReceiveExitAckMessage(StationIndex src, const void* cpMessage, uint32_t size);

    struct DispatchCountData
    {
    public:
        void Clear()
        {
            m_MyCount = InvalidDispatchCount;
            m_ReceivedCount = 0;
        }
        uint32_t m_MyCount;
        uint16_t m_ReceivedCount;
    };

private:
    uint32_t m_ElementSizeMax;

    uint32_t m_StationNum;
    StationIndex m_LocalStationIndex;
    uint32_t m_ValidStationBitmap;  // CONNECT,DISCONNECTと直接連動するBitmap
    uint32_t m_ParticipantBitmap;   // CloneProtoclがeMTEnterを受けて参加、eMTExitを受けて不参加とみなすBitmap
    InnerState m_InnerState;

    Clock* m_pClock;

    SystemTime m_SystemTime;
    SystemTime m_ResendSpan;
    SystemTime m_ResendSpanForUnicast[MaxStationNum];
    float m_ResendSpanRatio;
    float m_MillisecondsPerFrame;
    int m_MillisecondsPerFrameSamplingNum;
    static const int MillisecondsPerFrameSamplingNumMax = 10;
    common::Time m_MillisecondsPerFramePreTime;

    SystemTime m_LastUpdateResendSpanTime;
    SystemTime m_LastParticipationCommandTime;
    SystemTime m_LastClockCommandTime;

    uint32_t m_SendPeriod;

    static const SystemTime DefaultResendSpan = 500;

    uint32_t m_DispatchCount;
    DispatchCountData* m_paReceivedDispatchCount;
    static const uint32_t CheckDispatchCount = 0x00003fff;

public:
    static const uint32_t InvalidDispatchCount = 0xffffffff;

private:
    enum
    {
        StateMask = 0xf0
    };

    uint32_t m_NoticeParticipationBitmap;  // 自分がCloneProtocolをStart(),Stop()したときに、相手のAckをチェックするBitmap

private:
    Result RegisterClone(CloneBase::Key key, CloneBase* pClone);
    Result UnregisterClone(CloneBase::Key key);
    Result UnregisterClone(CloneBase* pClone);

    typedef common::OffsetTreeMap<CloneBase::Key, CloneBase> CloneCollection;
    CloneCollection m_CloneCollection;


public:
    void AddSendCommand(CloneBase::CommandToken* pToken);
    void CancelSendCommand(CloneBase::CommandToken* pToken);

private:
    void SendCommand();
    uint32_t SendCommandCore(CloneBase::CommandToken* pToken, bool isUnicast, uint32_t destBitmap, void* pMessage, uint32_t messageSize);

    typedef common::OffsetList<CloneBase::CommandToken> CommandTokenList;
    CommandTokenList m_SendCommandTokenList;
    CommandTokenList m_ResendCommandTokenList;

public:
    void AddSendData(CloneBase::DataToken* pToken);
    void CancelSendData(CloneBase::DataToken* pToken);

private:
    void SendAck();
    void SendData();
    bool SendCloneData(CloneBase::DataToken* pToken);
    void ReceiveData(StationIndex src, CloneBase* pClone, const void* cpMessage, uint32_t messageSize);

    typedef common::OffsetList<CloneBase::DataToken> DataTokenList;
    DataTokenList m_DataTokenList;

    struct DataMessageBuffer
    {
    public:
        void Clear()
        {
            m_DestBitmap = 0;
            m_Size = 0;
        }

        uint32_t m_DestBitmap;
        uint16_t m_Size;
        uint16_t m_NoCompressedSize;
        uint16_t m_Dummy;
        uint8_t m_Message[common::ProtocolMessgaePayloadSizeMax + MessageHeaderSerializeDiffSizeMax];
    };

    DataMessageBuffer m_DataMessageBuffer;
    IDataPacker* m_pDataPacker;
    IDataUnpacker* m_pDataUnpacker;
    CompressDataUnpacker* m_pCompressedDataUnpacker;

    StationIndex m_TryNextUnicastDestStationIndex;

public:
    ClockPriorityJudgeFunction GetClockPriorityJudgeFunction() const
    {
        return m_ClockPriorityJudgeFunction;
    }

private:
    ClockPriorityJudgeFunction m_ClockPriorityJudgeFunction;

private:
    void SetError(ErrorType errorType);

    ErrorType m_Error;

private:
    CloneProfilerBase* m_pSendProfiler;
    CloneProfilerBase* m_pReceiveProfiler;

    uint32_t m_SendMessageSequentialId;
    uint32_t m_ReceivedMessageSequentialId[MaxStationNum];
#if NN_PIA_ENABLE_TRACE
    // 同じ SequentialId を 2 連続で受信した場合の、1 つ目のメッセージ情報トレース用
    struct MessageHeaderForDebug
    {
        uint8_t m_FormatVersion;
        uint8_t m_MessageType;
        uint16_t m_DispatchCount;
    };
    MessageHeaderForDebug m_ReceivedMessageHeaderForDebug[MaxStationNum];
#endif

public:
    typedef bool (*ReceiveElementFunction)(CloneElementBase* pElement, IDataPacker* pAckPacker, const void* cpChunk, uint16_t chunkSize, CloneElementBase::Id elementId, StationIndex src, StationIndex localStationIndex, uint32_t dispatchCount);

    class ElementDefinition
    {
    public:
        ElementDefinition();
        void Register(uint16_t type, ReceiveElementFunction receiveFunction, uint32_t dataChunkHeaderSize);
        ReceiveElementFunction GetReceiveFunction(uint16_t type) const;
        uint32_t GetDataChunkHeaderSizeMax() const
        {
            return m_DataChunkHeaderSizeMax;
        }
        static const uint32_t TypeNum = 16;

    private:
        ReceiveElementFunction m_aReceiveFunction[TypeNum];
        uint32_t m_DataChunkHeaderSizeMax;
    };

    static ElementDefinition* GetElementDefinition();


protected:
    virtual Clock* CreateRtcClock(float clockPerSec, float regulationRate);
    virtual uint16_t TransportGetMaxStationNum();
    virtual uint32_t PayloadSizeManagerGetMtuSize();
    virtual uint32_t SignatureManagerGetSignatureSizeMax();
    virtual const common::Time& TransportGetDispatchedTime();
    virtual int32_t StationGetRtt(StationIndex stationIndex);
    virtual int32_t StationGetRttPulseInterval();
    virtual uint32_t PacketHandlerGetPayloadSizeLimit();
    virtual bool PacketHandlerSendMessageById(StationIndex dest, const void* cpData, uint32_t size);
    virtual bool PacketHandlerSendMessageByBitmap(uint32_t destBitmap, const void* cpData, uint32_t size);
    virtual void PacketHandlerReadIterationBegin(void** ppIterator);
    virtual bool PacketHandlerReadIterationIsEnd(const void* cpIterator);
    virtual void PacketHandlerReadIterationNext(void* pIterator);
    virtual void PacketHandlerReadIteratorGet(StationIndex* pSrc, const void** cppPayloadPtr, uint32_t* pPayloadSize, const void* cpIterator);

public:
    template <int TestId, typename Arg>
    static void Test(const CloneProtocol& obj, Arg* pArg = NULL);

private:
    // n2920: クライアント監視機能用
    uint32_t m_UpdateCount;  // 送信間隔関連処理でも使用
    uint32_t m_ResendCount;
    nn::pia::common::Time m_ActivationStartTime;
    uint32_t m_CompCount;
    float m_CompRateTotal;
    float m_CompAverage;
    uint32_t m_SendCount;
    uint32_t m_SendDataTotal;

    uint32_t m_UpdateCountOffset;
    bool m_IsSendProcessNeeded;

    bool m_IsBroadcastSupported;

    // Ack 用
    static const uint32_t AckMessageBufferSize = common::ProtocolMessgaePayloadSizeMax + MessageHeaderSerializeDiffSizeMax;
    struct AckMessageInfo
    {
        uint8_t messageBuffer[AckMessageBufferSize];
        uint32_t messageSize;
        uint32_t noCompressedMessageSize;
        StationIndex destStationIndex;
    };
    AckMessageInfo* m_pAckMessageInfoArray;
    int m_AckMessageBufferNum;
    int m_AckMessageBufferHeadIndex;
    int m_AckMessageBufferUsedNum;
    int m_AckMessageBufferUsedNumMax;

    NN_PIA_DISALLOW_COPY(CloneProtocol);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
