﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_EventCloneElementBase.h>
#include <nn/pia/clone/clone_SerializePolicyDefinition.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  イベントの送受信を管理します。

    @tparam Value_              イベント発行時に指定する値の型です。値が必要ない場合でも何らかの値を設定する必要があります。
    @tparam BufferSize         イベントバッファのサイズです。正の値を指定する必要があります。
 */
template <typename Value_, uint32_t BufferSize>
class EventCloneElement : public EventCloneElementBase
{
public:
    /*!
    @brief  このイベントで送られる引数の型です。
     */
    typedef Value_ Value;

    /*!
    @brief  Value をシリアライズするためのアルゴリズムです。
     */
    typedef SerializePolicy<Value> ValueSerializePolicy;


    /*!
    @brief デフォルトコンストラクタです。
     */
    EventCloneElement();


    /*!
    @brief デストラクタです。
     */
    virtual ~EventCloneElement()
    {
    }


    /*!
    @brief  イベントを発行します。
    @param[in]  value               イベントの値です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState  登録しているクローンが送信可能状態ではありません。[:progErr]
    @retval ResultBufferIsFull  バッファがいっぱいで発行できません。[:handling]
     */
    Result SetValue(const Value& value);


    /*!
    @brief  次のイベントを参照します。
    @details    参照中のイベントは、もう一度 HandleNext() を呼ぶまで同じものが参照できます。
    @return 新たに参照するイベントの値です。次のイベントが無い場合は NULL を返します。
     */
    const Value* HandleNext()
    {
        HandleNextCore();
        return GetValue();
    }


    /*!
    @brief  現在参照中のイベントの値を指すポインタを取得します。
    @return 現在参照中のイベントの値を指すポインタです。前回呼んだ HandleNext() の返り値と同じです。参照中のイベントが無い場合は NULL を返します。
     */
    const Value* GetValue() const
    {
        return (GetHandlingEventToken() != NULL) ? &(static_cast<const EventToken*>(GetHandlingEventToken())->GetValue()) : NULL;
    }


    /*!
    @brief  現在参照中のイベントの発行時刻を取得します。
    @return 現在参照中のイベントの発行時刻です。参照中のイベントが無い場合は @ref InvalidClock を返します。
     */
    ClockValue GetClock() const
    {
        return (GetHandlingEventToken() != NULL) ? GetHandlingEventToken()->GetClock() : InvalidClock;
    }


    /*!
    @brief  現在参照中のイベントの発行元ステーションインデックスを取得します。
    @return 現在参照中のイベントの発行元ステーションインデックスです。参照中のイベントが無い場合は StationIndex_Invalid を返します。
     */
    StationIndex GetSetterStationIndex() const
    {
        return (GetHandlingEventToken() != NULL) ? GetHandlingEventToken()->GetSetterStationIndex() : StationIndex_Invalid;
    }


    /*!
    @brief  現在参照中のイベントのインデックスを取得します。
    @return 現在参照中のイベントのインデックスです。参照中のイベントが無い場合は 0 を返します。
     */
    Index GetEventIndex() const
    {
        return (GetHandlingEventToken() != NULL) ? GetHandlingEventToken()->GetIndex() : 0;
    }

protected:
    virtual uint32_t GetSize() const
    {
        return ValueSerializePolicy::GetSerializedSize();
    }


private:
    class EventToken : public EventTokenBase
    {
    public:
        EventToken()
            : EventTokenBase(), m_Value()
        {
        }
        void SetValue(const Value& value)
        {
            m_Value = value;
        }
        const Value& GetValue() const
        {
            return m_Value;
        }

        virtual void Serialize(void* pBuffer) const
        {
            ValueSerializePolicy::Serialize(pBuffer, m_Value);
        }

        virtual void Deserialize(const void* cpBuffer)
        {
            ValueSerializePolicy::Deserialize(&m_Value, cpBuffer);
        }

    private:
        Value m_Value;
    };


    class EventTokenBuffer : public EventTokenBufferBase
    {
    public:
        explicit EventTokenBuffer(EventCloneElementBase* pElement);

    protected:
        virtual EventTokenBase* At(int idx)
        {
            return &m_EventTokenArray[idx];
        }
        virtual const EventTokenBase* At(int idx) const
        {
            return &m_EventTokenArray[idx];
        }

    private:
        EventToken m_EventTokenArray[BufferSize];
    };


    EventTokenBuffer m_EventTokenBuffer;
    NN_PIA_DISALLOW_COPY(EventCloneElement);
};


//! @cond
NN_PIA_PRAGMA_PUSH_WARNINGS
NN_PIA_DISABLE_WARNING_WIN_4355 //n2219:memo C4355の warning を抑制開始 コンストラクタでのthisの使用

template <typename Value, uint32_t BufferSize>
EventCloneElement<Value, BufferSize>::EventCloneElement()
    : EventCloneElementBase(&m_EventTokenBuffer),
      m_EventTokenBuffer(this)
{
    PIA_COMPILE_ASSERT(BufferSize >= BufferSizeMin && BufferSize <= BufferSizeMax);
}

NN_PIA_PRAGMA_POP_WARNINGS  //n2219:memo warning を抑制終了


template <typename Value, uint32_t BufferSize>
Result EventCloneElement<Value, BufferSize>::SetValue(const Value& value)
{
    EventTokenBase* pTokenBase;
    Result r = SetValueCore(&pTokenBase);
    if (r.IsSuccess() && pTokenBase != NULL)
    {
        EventToken* pToken = static_cast<EventToken*>(pTokenBase);
        pToken->SetValue(value);
    }
    return r;
}


template <typename Value, uint32_t BufferSize>
EventCloneElement<Value, BufferSize>::EventTokenBuffer::EventTokenBuffer(EventCloneElementBase* pElement)
    : EventTokenBufferBase(BufferSize)
{
    for (uint32_t i = 0; i < BufferSize; ++i)
    {
        m_EventTokenArray[i].Init(pElement);
        m_EventTokenArray[i].SetState(EventTokenBase::State_EventTokenEmpty);
        m_EventTokenArray[i].SetIndex(static_cast<Index>(i));
    }
}

//! @endcond
}
}
}  // end of namespace nn::pia::clone
