﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneElementBase.h>
#include <nn/pia/common/common_OffsetList.h>

namespace nn
{
namespace pia
{
namespace clone
{

class IDataPacker;

/*!
    @brief  イベントの送受信を管理する基底クラスです。
 */
class EventCloneElementBase : public CloneElementBase
{
public:
    /*!
    @brief デストラクタです。
     */
    virtual ~EventCloneElementBase();


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


    /*!
    @brief  イベントの取りこぼしエラーが発生したかどうかを取得します。
    @details    AtomicSharingClone で使用しているときにステーションの異常切断が発生すると、
                受信するはずだったイベントを受信し損ねる場合があります。
                いずれかの EventCloneElement でこのエラーが発生した場合、
                CloneProtocol::GetError() が CloneProtocol::ErrorType_EventDropped を返すようになります。
    @return イベントの取りこぼしエラーが発生していたら true を返します。
    @see    CloneProtocol::GetError
     */
    bool IsEventDropped() const
    {
        return m_IsEventDropped;
    }

    /*!
    @brief 現在のイベントバッファの使用数を取得します。

    @details ReceiveClone、AtomicSharingClone、SequentialSharingClone に登録されている場合、
             返り値が 0 にならない場合があります。
             次のデータを受信時に、消してもよいデータがバッファから削除されます。

    @return イベントバッファの使用数を返します。
     */
    uint32_t GetUsingBufferNum() const;


    /*!
    @cond PRIVATE
    @brief イベントバッファ中の指定したイベントが、どのステーションからの Ack を待っているかを取得します。

    @param[in] index イベントバッファ中の一番古いイベントを 0 番としたときの、取得対象のイベントのインデックスです。
                     0 以上、現在のイベントバッファの使用数未満の値を設定してください。

    @return 指定したイベントが Ack を待っているステーションのステーションインデックスを表すビットマップを返します。
    @see    GetUsingBufferNum
     */
    uint32_t GetBufferEventWaitingStationIndexBitmap(int index) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief イベントバッファ中の指定したイベントの発行時刻を取得します。

    @param[in] index イベントバッファ中の一番古いイベントを 0 番としたときの、取得対象のイベントのインデックスです。
                     0 以上、現在のイベントバッファの使用数未満の値を設定してください。

    @return 指定したイベントの発行時刻を返します。取得対象のイベントが無い場合は 0 を返します。
    @see    GetUsingBufferNum
     */
    ClockValue GetBufferEventClock(int index) const;
    //! @endcond


    //! @cond PRIVATE

public:
    virtual Type GetType() const
    {
        return GetTypeStatic();
    }
    virtual bool IsCloneSupported(int cloneType) const;

    static Type GetTypeStatic()
    {
        return Type_Event;
    }

    static bool Receive(
        CloneElementBase* pElement,
        IDataPacker* pAckPacker,
        const void* cpChunk,
        uint16_t chunkSize,
        Id elementId,
        StationIndex src,
        StationIndex localStationIndex,
        uint32_t dispatchCount);

    static uint32_t GetDataChunkHeaderSize();

public:
    typedef uint16_t Index;

private:
    enum ReceiveDataResponse
    {
        ReceiveDataResponse_Nothing,
        ReceiveDataResponse_Ack,
        ReceiveDataResponse_PreserverMigration
    };
    ReceiveDataResponse ReceiveData(const void* cpData, uint32_t size, StationIndex src, StationIndex setter, ClockValue clock, uint64_t registerCountAll, uint32_t destBitmap, Index index, Index eraseIndex);
    void ReceiveAck(StationIndex src, Index index);
    void ReceivePreserverMigration(StationIndex src, Index index);

    void ReceiveInitial(StationIndex src, Index index);
    void ReceiveInitialAck(StationIndex src);

public:
    virtual void ClearData();

    virtual void RemoveDest(StationIndex stationIndex);
    virtual void AddParticipant(StationIndex stationIndex);
    virtual void Complement(StationIndex stationIndex);
    virtual void Lock();

protected:
    static const uint32_t BufferSizeMin = 2;
    static const uint32_t BufferSizeMax = 0x3fff;

public:
    class EventSendToken : public CloneElementBase::ResendableSendToken
    {
    public:
        enum TokenType
        {
            TokenType_Initial,
            TokenType_Event
        };
        virtual TokenType GetTokenType() const = 0;
    };


    class EventInitialToken : public EventSendToken
    {
    public:
        explicit EventInitialToken(EventCloneElementBase* pElement)
            : EventSendToken(), m_pElement(pElement)
        {
        }

        virtual Type GetElementType() const
        {
            return Type_Event;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_DestBitmap;
        }
        virtual TokenType GetTokenType() const
        {
            return TokenType_Initial;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }

        Id GetId() const
        {
            return m_pElement->GetId();
        }

        Index GetInitialIndex() const
        {
            return m_pElement->m_pEventTokenBuffer->GetNextIndex();
        }
        void SetDestBitmap(uint32_t destBitmap)
        {
            m_DestBitmap = destBitmap;
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: EventInitialToken");
        };

    private:
        EventCloneElementBase* m_pElement;
        uint32_t m_DestBitmap;
    };


    class EventTokenBase : public EventSendToken
    {
    public:
        EventTokenBase()
            : EventSendToken()
        {
        }
        void Init(EventCloneElementBase* pElement)
        {
            m_pElement = pElement;
        }

        virtual Type GetElementType() const
        {
            return Type_Event;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_DestBitmap;
        }
        virtual TokenType GetTokenType() const
        {
            return TokenType_Event;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual bool IsUnicast() const
        {
#if NN_PIA_ENABLE_CLONE_BROADCAST_RESEND
            return false;
#else
            return true;
#endif
        }

        Index GetIndex() const
        {
            return m_Index;
        }
        StationIndex GetSetterStationIndex() const
        {
            return m_SetterStationIndex;
        }
        StationIndex GetPreserverStationIndex() const
        {
            return m_PreserverStationIndex;
        }
        const ClockValue& GetClock() const
        {
            return m_Clock;
        }
        uint64_t GetRegisterCountAll() const
        {
            return m_RegisterCountAll;
        }

        void SetDestBitmap(uint32_t destBitmap)
        {
            m_DestBitmap = destBitmap;
        }
        void RemoveDestBmp(uint32_t bmp)
        {
            m_DestBitmap &= ~bmp;
        }
        void RemoveDestAll()
        {
            m_DestBitmap = 0;
        }
        void SetIndex(Index index)
        {
            m_Index = index;
        }
        void SetSetterStationIndex(StationIndex stationIndex)
        {
            m_SetterStationIndex = stationIndex;
        }
        void SetPreserverStationIndex(StationIndex stationIndex)
        {
            m_PreserverStationIndex = stationIndex;
        }
        void SetClock(ClockValue clock)
        {
            m_Clock = clock;
        }
        void SetRegisterCountAll(uint64_t count)
        {
            m_RegisterCountAll = count;
        }

        enum State
        {
            State_EventTokenEmpty,
            State_EventTokenSet,
            State_EventTokenReceived,
            State_EventTokenHandled
        };

        State GetState() const
        {
            return m_State;
        }
        void SetState(State state)
        {
            m_State = state;
        }

        Id GetId() const
        {
            return m_pElement->GetId();
        }
        virtual uint32_t GetSize() const
        {
            return m_pElement->GetSize();
        }
        virtual void Serialize(void* pBuffer) const = 0;
        virtual void Deserialize(const void* cpBuffer) = 0;
        Index GetEraseIndex() const
        {
            return m_pElement->m_pEventTokenBuffer->GetEraseIndex();
        }

        void CancelSendData()
        {
            m_pElement->CancelSendData(this);
        }
        void AddSendData()
        {
            m_pElement->AddSendData(this);
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: EventTokenBase");
        };

    private:
        EventCloneElementBase* m_pElement;
        uint32_t m_DestBitmap;
        Index m_Index;
        StationIndex m_SetterStationIndex;
        StationIndex m_PreserverStationIndex;
        ClockValue m_Clock;
        uint64_t m_RegisterCountAll;
        State m_State;
    };


    class EventTokenBufferBase
    {
    protected:
        explicit EventTokenBufferBase(uint32_t size);

    public:
        void Clear();
        void InitIndex(Index index);
        EventTokenBase* Get(Index index);
        EventTokenBase* AssignNext();
        EventTokenBase* Assign(Index index);
        void Recycle();
        bool ShiftHead();
        Index GetHeadIndex() const
        {
            return At(m_HeadIdx)->GetIndex();
        }
        bool IsEmpty() const
        {
            return (m_Num == 0);
        }
        Index GetEraseIndex() const
        {
            return m_EraseIndex;
        }
        Index GetNextIndex() const;

        void RemoveDest(StationIndex stationIndex);
        void Complement(StationIndex stationIndex, uint32_t destBitmap);
        void Lock();

        uint32_t GetUsingBufferNum()
        {
            return m_Num;
        }

        uint32_t GetBufferEventWaitingStationIndexBitmap(int index) const;
        ClockValue GetBufferEventClock(int index) const;

    protected:
        virtual EventTokenBase* At(int idx) = 0;
        virtual const EventTokenBase* At(int idx) const = 0;

    private:
        int m_BufferSize;
        int m_HeadIdx;
        int m_Num;

        Index m_EraseIndex;
    };

protected:
    explicit EventCloneElementBase(EventTokenBufferBase* pEventTokenBuffer);

protected:
    Result SetValueCore(EventTokenBase** ppTokenBase);
    void HandleNextCore();
    const EventTokenBase* GetHandlingEventToken() const
    {
        return m_pHandlingEventToken;
    }


private:
    enum IndexState
    {
        IndexState_EventInvalid,
        IndexState_EventInit,
        IndexState_EventAdjust,
        IndexState_EventValid
    };

    IndexState m_IndexState;

    Index m_HandledIndex;
    EventTokenBase* m_pHandlingEventToken;
    Index m_PreIndex;

    EventInitialToken m_InitialToken;
    EventTokenBufferBase* m_pEventTokenBuffer;

    bool m_IsEventDropped;

    static Definition<EventCloneElementBase> s_Definition;

public:
    template <int TEST_ID, typename Arg>
    static void Test(const EventCloneElementBase& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(EventCloneElementBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
