﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneBase.h>


namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  一方向の送受信の受信側を管理する CloneBase 派生クラスです。
 */
class ReceiveClone : public clone::CloneBase
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    ReceiveClone();


    /*!
    @brief デストラクタです。
     */
    virtual ~ReceiveClone();


    /*!
    @brief  CloneProtocol に登録するときに送信側として指定された @ref StationIndex を取得します。
    @return 送信側の @ref StationIndex です。 CloneProtocol に登録されてない場合は @ref StationIndex_Invalid を返します。
     */
    StationIndex GetSendStationIndex() const
    {
        return GetStationIndex();
    }


    /*!
    @brief  この ReceiveClone に送信する SendClone が存在するかどうかを判定します。
    @return この ReceiveClone に送信する SendClone が存在すれば true を返します。
    @see    SendClone::IsConnected, SendClone::IsConnectedWithAll
     */
    bool IsConnected() const
    {
        return IsActive() && m_IsReceiving;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


public:
    // デバッグ用
    /*!
    @cond PRIVATE
    @brief  受信中かどうかを取得します。（デバッグ用）
     */
    bool IsReceiving() const
    {
        return m_IsReceiving;
    }
    //! @endcond

    //! @cond PRIVATE

public:
    static Type GetTypeStatic()
    {
        return Type_Receive;
    }
    static Key ToKey(StationIndex stationIndex, Id id)
    {
        return CloneBase::ToKey(Type_Receive, stationIndex, id);
    }
    virtual Type GetTypeVirtual() const
    {
        return GetTypeStatic();
    }

protected:
    virtual void OnStartup();
    virtual void OnCleanup();
    virtual void OnEnd();
    virtual bool IsEndProcessFinished();

    virtual void OnEnter(StationIndex stationIndex);
    virtual void OnExit(StationIndex stationIndex);
    virtual void OnDisconnect(StationIndex stationIndex, bool isValid);
    virtual void OnRegister();

public:
    virtual uint32_t GetDestBitmap() const;
    virtual bool IsInRequest() const
    {
        return m_RequestCommandToken.IsInList();
    }

    virtual uint8_t GetLocalRegisterCount() const
    {
        return m_LocalRegisterCount;
    }
    virtual bool IsMatchLocalRegisterCount(uint64_t registerCountAll) const;
    void IncrementLocalRegisterCount();

public:
    bool ReceiveAnnounce();
    void ReceiveAccept(ClockValue clock);
    void ReceiveReject(ClockValue clock);
    void ReceiveEnd();
    void ReceiveStop(StationIndex src);

public:
    class RequestCommandToken : public ClockCommandToken
    {
    public:
        explicit RequestCommandToken(ReceiveClone* pClone)
            : ClockCommandToken(pClone), m_Clock(InvalidClock)
        {
        }
        virtual CommandType GetType() const
        {
            return CommandType_Request;
        }
        virtual uint32_t GetDestStationBitmap() const
        {
            return (0x1 << (static_cast<const ReceiveClone*>(GetClone())->GetStationIndex()));
        }
        virtual ClockValue GetClock() const
        {
            return m_Clock;
        }
        void SetClock(ClockValue clock)
        {
            m_Clock = clock;
        }

    private:
        ClockValue m_Clock;
    };

private:
    RequestCommandToken m_RequestCommandToken;
    bool m_IsReceiving;
    bool m_IsSendingStationDisconnected;

    uint8_t m_LocalRegisterCount;  // 自分の Register カウンタ (2 bit)

public:
    template <int TEST_ID, typename Arg>
    static void Test(const ReceiveClone& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(ReceiveClone);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
