﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneElementBase.h>


namespace nn
{
namespace pia
{
namespace clone
{


class IDataPacker;

/*!
    @brief  到達保証された値の送受信を管理する基底クラスです。
 */
class ReliableCloneElementBase : public CloneElementBase
{
protected:
    /*!
    @brief デフォルトコンストラクタです。
     */
    ReliableCloneElementBase();

public:
    /*!
    @brief デストラクタです。
     */
    virtual ~ReliableCloneElementBase();


    /*!
    @brief  @ref ReliableCloneElement::GetValue "GetValue()" で有効な値が取得できるかどうかを取得します。
    @return @ref ReliableCloneElement::GetValue "GetValue()" で有効な値を取得できる場合は true を返します。
     */
    bool IsValidValue() const
    {
        return m_SetterStationIndex != StationIndex_Invalid;
    }


    /*!
    @brief  値が設定された時刻を取得します。
    @return 値が設定された時刻です。 IsValidValue() が false の場合に取得できる値は不定です。
     */
    ClockValue GetClock() const
    {
        return m_Clock;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


public:
    /*!
    @cond PRIVATE
    @brief  現在の値をセットした StationIndex を取得します。（デバッグ用）
     */
    StationIndex GetSetter() const
    {
        return m_SetterStationIndex;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  現在の値の送信責任のある StationIndex を取得します。（デバッグ用）
     */
    StationIndex GetPreserver() const
    {
        return m_PreserverStationIndex;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  現在の値の送信先ビットマップを取得します。（デバッグ用）
     */
    uint32_t GetDestBitmap() const
    {
        return m_DestBitmap;
    }
    //! @endcond

    //! @cond PRIVATE

public:
    virtual Type GetType() const
    {
        return GetTypeStatic();
    }

    static Type GetTypeStatic()
    {
        return Type_Reliable;
    }
    static bool Receive(CloneElementBase* pElement, IDataPacker* pAckPacker, const void* cpChunk, uint16_t chunkSize, Id elementId, StationIndex src, StationIndex localStationIndex, uint32_t dispatchCount);
    static uint32_t GetDataChunkHeaderSize();

private:
    bool ReceiveRequest(StationIndex src);
    bool ReceiveData(const void* cpData, uint32_t size, StationIndex src, StationIndex setter, ClockValue clock, uint32_t destBitmap);
    void ReceiveNoData(StationIndex src);
    void ReceivePreserverMigration(StationIndex src, StationIndex setter, ClockValue clock);
    void ReceiveAck(StationIndex src, StationIndex setter, ClockValue clock);

public:
    virtual void ClearData();

    virtual void AddDest(StationIndex stationIndex);
    virtual void RemoveDest(StationIndex stationIndex);
    virtual void Complement(StationIndex stationIndex);

    virtual void RequestInitialData();

private:
    class ReliableSendToken : public CloneElementBase::ResendableSendToken
    {
    public:
        enum TokenType
        {
            TokenType_Request,
            TokenType_Data
        };
        virtual TokenType GetTokenType() const = 0;
    };


    class ReliableRequestToken : public ReliableSendToken
    {
    public:
        explicit ReliableRequestToken(ReliableCloneElementBase* pElement)
            : ReliableSendToken(), m_pElement(pElement)
        {
        }
        virtual Type GetElementType() const
        {
            return Type_Reliable;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_pElement->m_DestBitmap;
        }
        virtual TokenType GetTokenType() const
        {
            return TokenType_Request;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: ReliableRequestToken");
        };

    private:
        Id GetId() const
        {
            return m_pElement->GetId();
        }

    private:
        ReliableCloneElementBase* m_pElement;
    };


    class ReliableDataToken : public ReliableSendToken
    {
    public:
        explicit ReliableDataToken(ReliableCloneElementBase* pElement)
            : ReliableSendToken(), m_pElement(pElement)
        {
        }
        virtual Type GetElementType() const
        {
            return Type_Reliable;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_pElement->m_DestBitmap;
        }
        virtual TokenType GetTokenType() const
        {
            return TokenType_Data;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual bool IsUnicast() const
        {
#if NN_PIA_ENABLE_CLONE_BROADCAST_RESEND
            return false;
#else
            return true;
#endif
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: ReliableDataToken");
        };

    private:
        uint32_t GetSize() const
        {
            return m_pElement->GetSize();
        }
        Id GetId() const
        {
            return m_pElement->GetId();
        }
        ClockValue GetClock() const
        {
            return m_pElement->m_Clock;
        }
        StationIndex GetSetterStationIndex() const
        {
            return m_pElement->m_SetterStationIndex;
        }
        void Serialize(void* pBuffer) const
        {
            m_pElement->Serialize(pBuffer);
        }

    private:
        ReliableCloneElementBase* m_pElement;
    };


    class ReliableInitialToken : public CloneElementBase::ISendToken
    {
    public:
        explicit ReliableInitialToken(ReliableCloneElementBase* pElement)
            : ISendToken(), m_pElement(pElement)
        {
        }
        virtual Type GetElementType() const
        {
            return Type_Reliable;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_DestBitmap;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: ReliableInitialToken");
        };

    private:
        uint32_t GetSize() const
        {
            return m_pElement->GetSize();
        }
        bool IsValidValue() const
        {
            PIA_ASSERT(m_pElement->IsValidValue());
            return true;
        }
        Id GetId() const
        {
            return m_pElement->GetId();
        }
        ClockValue GetClock() const
        {
            return m_pElement->m_Clock;
        }
        StationIndex GetSetterStationIndex() const
        {
            return m_pElement->m_SetterStationIndex;
        }
        void Serialize(void* pBuffer) const
        {
            m_pElement->Serialize(pBuffer);
        }

    public:
        void AddDest(StationIndex stationIndex)
        {
            m_DestBitmap |= (0x1 << stationIndex);
        }
        void RemoveDest(StationIndex stationIndex)
        {
            m_DestBitmap &= (0x1 << stationIndex);
        }
        void ClearDest()
        {
            m_DestBitmap = 0;
        }

    private:
        ReliableCloneElementBase* m_pElement;
        uint32_t m_DestBitmap;
    };


protected:
    Result SetValueCore();

    virtual void Serialize(void* pBuffer) const = 0;
    virtual void Deserialize(const void* cpData) = 0;
    virtual void ClearValue() = 0;

private:
    StationIndex m_SetterStationIndex;
    StationIndex m_PreserverStationIndex;
    ClockValue m_Clock;
    uint32_t m_DestBitmap;
    ReliableRequestToken m_RequestToken;
    ReliableDataToken m_DataToken;
    ReliableInitialToken m_InitialToken;

    static Definition<ReliableCloneElementBase> s_Definition;

public:
    template <int TEST_ID, typename Arg>
    static void Test(const ReliableCloneElementBase& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(ReliableCloneElementBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
