﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_ReliableLargeCloneElementBase.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  到達保証されたサイズの大きい値の送受信を管理します。

    @tparam Value_              管理する値の型です。値を初期化する関数 void Value_::Clear() が定義されている必要があります。
    @tparam SerializedSize      シリアライズした Value のサイズです。正の値を指定する必要があります。@ref CloneProtocol::GetElementSizeMax() より大きな値は使用できません。
    @tparam DivideSize         分割後のデータ 1 つ当たりのサイズです。正の値を指定する必要があります。@ref CloneProtocol::GetElementSizeMax() より大きな値は使用できません。
 */
template <typename Value_, uint32_t SerializedSize, uint32_t DivideSize = 1094>
class ReliableLargeCloneElement : public ReliableLargeCloneElementBase
{
public:
    /*!
    @brief  このオブジェクトが管理する値の型です。 
     */
    typedef Value_ Value;


    /*!
    @brief  Value をシリアライズするためのアルゴリズムです。
     */
    typedef SerializePolicy<Value> ValueSerializePolicy;


    /*!
    @brief デフォルトコンストラクタです。
     */
    ReliableLargeCloneElement()
        : ReliableLargeCloneElementBase(m_SerializedBuffer, m_aDivideInfo, Size, DivideSize, DivideNum),
          m_Value()
    {
    }


    /*!
    @brief デストラクタです。
     */
    virtual ~ReliableLargeCloneElement()
    {
    }


    /*!
    @brief  値を設定します。
    @param[in]  value   設定する値です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState  登録している CloneBase が送信可能状態ではありません。[:progErr]
    @retval ResultTemporaryUnavailable 値を設定できるタイミングではありません。値を設定するためには時刻を進める必要があります。[:handling]
     */
    Result SetValue(const Value& value);


    /*!
    @brief  値を取得します。
    @return このオブジェクトが管理する値です。 IsValidValue() が false の場合に取得できる値は不定です。
     */
    const Value& GetValue() const
    {
        return m_Value;
    }


    //! @cond PRIVATE

protected:
    virtual void Serialize()
    {
        ValueSerializePolicy::Serialize(m_SerializedBuffer, m_Value);
    }
    virtual void Deserialize()
    {
        ValueSerializePolicy::Deserialize(&m_Value, m_SerializedBuffer);
    }
    virtual void ClearValue()
    {
        m_Value.Clear();
    }


private:
    static const uint32_t Size = SerializedSize;
    static const uint32_t DivideLastSize = (Size - 1) % DivideSize + 1;
    static const uint32_t DivideNum = (Size - DivideLastSize) / DivideSize + 1;

private:
    Value m_Value;
    uint8_t m_SerializedBuffer[Size];
    DivideInfo m_aDivideInfo[DivideNum];

    //! @endcond
};


//! @cond
template <typename Value, uint32_t SerializedSize, uint32_t DivideSize>
Result ReliableLargeCloneElement<Value, SerializedSize, DivideSize>::SetValue(const Value& value)
{
    PIA_COMPILE_ASSERT(DivideNum <= 0xffff);

    Result r = SetValueCore();
    if (r.IsSuccess())
    {
        m_Value = value;
        PostSetValue();
    }
    return r;
}
//! @endcond
}
}
}  // end of namespace nn::pia::clone
