﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneElementBase.h>
#include <nn/pia/clone/clone_ReliableLargeChunkType.h>

namespace nn
{
namespace pia
{
namespace clone
{


class IDataPacker;


/*!
    @brief  到達保証されたサイズの大きい値の送受信を管理する基底クラスです。
 */
class ReliableLargeCloneElementBase : public CloneElementBase
{
public:
    /*!
    @brief デストラクタです。
     */
    virtual ~ReliableLargeCloneElementBase();


    /*!
    @brief  @ref ReliableCloneElement::GetValue "GetValue()" で有効な値が取得できるかどうかを取得します。
    @return @ref ReliableCloneElement::GetValue "GetValue()" で有効な値を取得できる場合は true を返します。
     */
    bool IsValidValue() const
    {
        return m_ValueSetterStationIndex != StationIndex_Invalid;
    }


    /*!
    @brief  値が設定された時刻を取得します。
    @return 値が設定された時刻です。 IsValidValue() が false の場合に取得できる値は不定です。
     */
    ClockValue GetClock() const
    {
        return m_ValueClock;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


public:
    /*!
    @cond PRIVATE
    @brief  現在の値をセットした StationIndex を取得します。（デバッグ用）
     */
    StationIndex GetSetter() const
    {
        return m_ValueSetterStationIndex;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  現在の値の送信責任のある StationIndex を取得します。（デバッグ用）
     */
    StationIndex GetPreserver() const
    {
        return m_ValuePreserverStationIndex;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  現在の値の送信先ビットマップを取得します。（デバッグ用）
     */
    uint32_t GetDestBitmap(uint32_t divideIndex) const
    {
        return (divideIndex < m_DivideNum) ? m_aDivideInfo[divideIndex].m_ValueDestBitmap : 0;
    }
    //! @endcond

    //! @cond PRIVATE

public:
    virtual Type GetType() const
    {
        return GetTypeStatic();
    }

    static Type GetTypeStatic()
    {
        return Type_ReliableLarge;
    }
    static bool Receive(CloneElementBase* pElement, IDataPacker* pAckPacker, const void* cpChunk, uint16_t chunkSize, Id elementId, StationIndex src, StationIndex localStationIndex, uint32_t dispatchCount);
    static uint32_t GetDataChunkHeaderSize();

private:
    bool ReceiveRequest(StationIndex src, uint16_t divideIndexOffset, uint32_t divideIndexFlag);
    ReliableLargeChunkType ReceiveData(const void* cpData, uint32_t size, StationIndex src, StationIndex setter, ClockValue clock, uint16_t divideIndex, uint32_t destBitmap);
    void ReceiveNoData(StationIndex src);
    void ReceivePreserverMigration(StationIndex src, StationIndex setter, ClockValue clock);
    void ReceiveAck(StationIndex src, StationIndex setter, ClockValue clock, uint16_t divideIndex);

public:
    virtual void ClearData();

    virtual void AddDest(StationIndex stationIndex);
    virtual void RemoveDest(StationIndex stationIndex);
    virtual void Complement(StationIndex stationIndex);

    virtual void RequestInitialData();

private:
    class ReliableLargeSendToken : public CloneElementBase::ResendableSendToken
    {
    public:
        enum TokenType
        {
            TokenType_Request,
            TokenType_Data
        };
        virtual TokenType GetTokenType() const = 0;
    };

    class ReliableLargeRequestToken : public ReliableLargeSendToken
    {
    public:
        explicit ReliableLargeRequestToken(ReliableLargeCloneElementBase* pElement)
            : ReliableLargeSendToken(), m_pElement(pElement), m_DestBitmap(0)
        {
        }
        virtual Type GetElementType() const
        {
            return Type_ReliableLarge;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_DestBitmap;
        }
        virtual TokenType GetTokenType() const
        {
            return TokenType_Request;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: ReliableLargeRequestToken");
        };

    public:
        void SetDest(uint32_t destBitmap)
        {
            m_DestBitmap = destBitmap;
        }

    private:
        Id GetId() const
        {
            return m_pElement->GetId();
        }

    private:
        ReliableLargeCloneElementBase* m_pElement;
        uint32_t m_DestBitmap;
    };

protected:
    class ReliableLargeDataToken : public ReliableLargeSendToken
    {
    public:
        explicit ReliableLargeDataToken()
            : ReliableLargeSendToken()
        {
        }
        void Init(ReliableLargeCloneElementBase* pElement, uint32_t divideIndex)
        {
            m_pElement = pElement;
            m_DivideIndex = divideIndex;
        }
        virtual Type GetElementType() const
        {
            return Type_ReliableLarge;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_pElement->m_aDivideInfo[m_DivideIndex].m_SerializedDestBitmap;
        }
        virtual TokenType GetTokenType() const
        {
            return TokenType_Data;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual bool IsUnicast() const
        {
#if NN_PIA_ENABLE_CLONE_BROADCAST_RESEND
            return false;
#else
            return true;
#endif
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: ReliableLargeDataToken");
        };

    private:
        uint32_t GetSize() const
        {
            return (m_DivideIndex < m_pElement->m_DivideNum - 1) ? m_pElement->m_DivideSize : m_pElement->m_DivideLastSize;
        }
        Id GetId() const
        {
            return m_pElement->GetId();
        }
        ClockValue GetClock() const
        {
            return m_pElement->m_SerializedClock;
        }
        StationIndex GetSetterStationIndex() const
        {
            return m_pElement->m_SerializedSetterStationIndex;
        }

    private:
        ReliableLargeCloneElementBase* m_pElement;
        uint32_t m_DivideIndex;
    };


    class ReliableLargeInitialToken : public CloneElementBase::ISendToken
    {
    public:
        explicit ReliableLargeInitialToken()
            : ISendToken()
        {
        }
        void Init(ReliableLargeCloneElementBase* pElement, uint32_t divideIndex)
        {
            m_pElement = pElement;
            m_DivideIndex = divideIndex;
        }
        virtual Type GetElementType() const
        {
            return Type_ReliableLarge;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_DestBitmap;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: ReliableLargeInitialToken");
        };

    private:
        uint32_t GetSize() const
        {
            return (m_DivideIndex < m_pElement->m_DivideNum - 1) ? m_pElement->m_DivideSize : m_pElement->m_DivideLastSize;
        }
        bool IsValidValue() const
        {
            PIA_ASSERT(m_pElement->IsValidValue());
            return true;
        }
        Id GetId() const
        {
            return m_pElement->GetId();
        }
        ClockValue GetClock() const
        {
            return m_pElement->m_SerializedClock;
        }
        StationIndex GetSetterStationIndex() const
        {
            return m_pElement->m_SerializedSetterStationIndex;
        }

    public:
        void AddDest(StationIndex stationIndex)
        {
            m_DestBitmap |= (0x1 << stationIndex);
        }
        void RemoveDest(StationIndex stationIndex)
        {
            m_DestBitmap &= (0x1 << stationIndex);
        }
        void ClearDest()
        {
            m_DestBitmap = 0;
        }

    private:
        ReliableLargeCloneElementBase* m_pElement;
        uint32_t m_DivideIndex;
        uint32_t m_DestBitmap;
    };

protected:
    struct DivideInfo
    {
    public:
        DivideInfo()
            : m_ValueDestBitmap(0), m_SerializedDestBitmap(0), m_IsSerializedValid(false), m_DataToken(), m_InitialToken()
        {
        }

        uint32_t m_ValueDestBitmap;
        uint32_t m_SerializedDestBitmap;
        bool m_IsSerializedValid;
        ReliableLargeDataToken m_DataToken;
        ReliableLargeInitialToken m_InitialToken;
    };


protected:
    ReliableLargeCloneElementBase(uint8_t* pBuffer, DivideInfo* aDivideInfo, uint32_t serializedSize, uint32_t divideSize, uint32_t divideNum);

    virtual uint32_t GetSize() const
    {
        return m_DivideSize;
    }
    bool IsEqualValueAndSerialized() const
    {
        return m_ValueClock == m_SerializedClock && m_ValueSetterStationIndex == m_SerializedSetterStationIndex;
    }
    void GetRequestDivideIndex(uint16_t* pOffset, uint32_t* pFlag);

protected:
    Result SetValueCore();
    void PostSetValue();
    virtual void Serialize() = 0;
    virtual void Deserialize() = 0;
    virtual void ClearValue() = 0;


private:
    StationIndex m_ValueSetterStationIndex;
    StationIndex m_ValuePreserverStationIndex;
    ClockValue m_ValueClock;

    StationIndex m_SerializedSetterStationIndex;
    StationIndex m_SerializedPreserverStationIndex;
    ClockValue m_SerializedClock;

    uint8_t* m_SerializedBuffer;
    DivideInfo* m_aDivideInfo;

    uint32_t m_SerializedSize;
    uint32_t m_DivideSize;
    uint32_t m_DivideLastSize;
    uint32_t m_DivideNum;

    ReliableLargeRequestToken m_RequestToken;

#if 0
    StationIndex m_SetterStationIndex;
    StationIndex m_PreserverStationIndex;
    ClockValue m_Clock;
    uint32_t m_DestBitmap;
    ReliableRequestToken m_RequestToken;
    ReliableDataToken m_DataToken;
    ReliableInitialToken m_InitialToken;
#endif

    static Definition<ReliableLargeCloneElementBase> s_Definition;

public:
    template <int TEST_ID, typename Arg>
    static void Test(const ReliableLargeCloneElementBase& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(ReliableLargeCloneElementBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
