﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#pragma once
#if 0

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/common/common_ByteOrder.h>


namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  元のデータのバイトオーダーを反転させてシリアライズします。

    @tparam Type_   シリアライズ対象のデータ型です。

    @details    このクラスが実装されているのは、 Type が uint8_t, int8_t, uint16_t, int16_t, uint32_t, int32_t, uint64_t, int64_t, float, double のものだけです。
                任意の型に対してこのクラスを使用するためには、その型に対応した Reverse() 関数を実装する必要があります。

                例えば、
                @code
                struct Val
                {
                    uint32_t a;
                    float b;
                };
                @endcode
                という型に対して、
                @code
                template<> void ReverseSerializePolicy<Val>::Reverse(Val* pTo, const Val* cpFrom)
                {
                    ReverseValue(&pTo->a, &cpFrom->a);
                    ReverseValue(&pTo->b, &cpFrom->b);
                }
                @endcode
                と実装する必要があります。
 */
template <typename Type_>
class ReverseSerializePolicy
{
public:
    /*!
    @brief  シリアライズ対象のデータ型です。
     */
    typedef Type_ Type;


    /*!
    @cond PRIVATE
    @brief  シリアライズします。
    @param[out] pBuffer 出力先バッファです。
    @param[in]  value   シリアライズ対象のデータです。
     */
    static void Serialize(void* pBuffer, const Type& value)
    {
        Reverse(reinterpret_cast<Type*>(pBuffer), &value);
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  デシリアライズします。
    @param[out] pValue  デシリアライズされたデータの保存先です。
    @param[in]  cpData  デシリアライズするバイト列です。
     */
    static void Deserialize(Type* pValue, const void* cpData)
    {
        Reverse(pValue, reinterpret_cast<const Type*>(cpData));
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  シリアライズされたサイズを取得します。
    @details    @ref nn::pia::clone::CloneProtocol::GetElementSizeMax() "CloneProtocol::GetElementSizeMax()" より大きな値は使用できません。
    @return シリアライズされたサイズです。
     */
    static uint32_t GetSize()
    {
        return sizeof(Type);
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  シリアライズされたサイズです。
    @details    @ref nn::pia::clone::ReliableLargeCloneElement で使用するときは @ref GetSize() ではなくこちらの値が使われます。
     */
    static const uint32_t Size = sizeof(Type);
    //! @endcond

private:
    /*!
    @brief  バイトオーダーを反転させます。
    
    @details    この関数が実装されているのは、 Type が uint8_t, int8_t, uint16_t, int16_t, uint32_t, int32_t, uint64_t, int64_t, float, double のものだけです。
                任意の型に対してこのクラスを使用するためには、その型に対応した Reverse() 関数を実装する必要があります。
    @attention  pTo や cpFrom のアライメントは不正な場合があります。

    @param[out] pTo     反転させたデータの出力先です。
    @param[in]  cpFrom  元データです。
     */
    static void Reverse(Type* pTo, const Type* cpFrom);


    /*!
    @brief  基本型の値のバイトオーダーを反転させます。

    @tparam T   反転させる値の型です。

    @param[out] pTo     反転させた値の出力先です。
    @param[in]  cpFrom  元データです。
     */
    template <typename T>
    static void ReverseValue(T* pTo, const T* cpFrom)
    {
        common::ByteOrder::Swap<sizeof(T)>(pTo, cpFrom);
    }

    NN_PIA_DISALLOW_COPY(ReverseSerializePolicy);
};

//! @cond

template <>
inline void ReverseSerializePolicy<uint8_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<int8_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<uint16_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<int16_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<uint32_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<int32_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<uint64_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<int64_t>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<float>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}
template <>
inline void ReverseSerializePolicy<double>::Reverse(Type* pTo, const Type* pFrom)
{
    ReverseValue(pTo, pFrom);
}

//! @endcond
}
}
}  // end of namespace nn::pia::clone
#endif
