﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneBase.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  一方向の送受信の送信側を管理する CloneBase 派生クラスです。
 */
class SendClone : public clone::CloneBase
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    SendClone();


    /*!
    @brief デストラクタです。
     */
    virtual ~SendClone();


    /*!
    @brief  この SendClone からデータを受け取る ReceiveClone が存在するかどうかを判定します。
    @details    相手の ReceiveClone が途中で登録解除された場合、アプリケーションや内部によるデータの送受信や、
                CloneProtocol の通信終了、ステーションの離脱等が起こるまでは反映されないので注意してください。
    @param[in]  stationId   送信先の @ref StationId です。
    @return この SendClone からデータを受け取る ReceiveClone が stationId に存在すれば true を返します。
    @see    IsConnectedWithAll, ReceiveClone::IsConnected
     */
    bool IsConnected(StationId stationId) const;


    /*!
    @brief  この SendClone からデータを受け取る ReceiveClone が全てのステーションに存在するかどうかを判定します。
    @details    CloneProtocol::IsActiveStation() が true となるステーションが判定対象です。
    @details    相手の ReceiveClone が途中で登録解除された場合、アプリケーションや内部によるデータの送受信等が
                起こるまでは反映されないので注意してください。
    @return この SendClone からデータを受け取る ReceiveClone が全てのステーションに存在すれば true を返します。
    @see    IsConnected, ReceiveClone::IsConnected
     */
    bool IsConnectedWithAll() const;


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


public:
    /*!
    @cond PRIVATE
    @brief  宛先を取得します。（デバッグ用）
     */
    uint32_t GetReceiverBmp() const
    {
        return m_ReceiverBmp;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  開始通知中かどうかを取得します。（デバッグ用）
     */
    bool IsInAnnounce() const
    {
        return m_AnnounceCommandToken.IsInList();
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  開始通知の宛先を取得します。（デバッグ用）
     */
    uint32_t GetAnnounceBmp() const
    {
        return m_AnnounceCommandToken.GetDestStationBitmap();
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  終了通知中かどうかを取得します。（デバッグ用）
     */
    bool IsEnding() const
    {
        return m_EndCommandToken.IsInList();
    }
    //! @endcond


    //! @cond PRIVATE
public:
    static Type GetTypeStatic()
    {
        return Type_Send;
    }
    static Key ToKey(Id id)
    {
        return CloneBase::ToKey(Type_Send, StationIndex_Invalid, id);
    }
    virtual Type GetTypeVirtual() const
    {
        return GetTypeStatic();
    }


protected:
    virtual void OnStartup();
    virtual void OnCleanup();
    virtual void OnEnd();
    virtual void OnSendFinished();
    virtual bool IsEndProcessFinished();

    //virtual void OnEnter(StationIndex stationIndex) {(void)stationIndex;}
    virtual void OnExit(StationIndex stationIndex);
    virtual void OnDisconnect(StationIndex stationIndex, bool isValid);

public:
    bool ReceiveRequest(StationIndex src, ClockValue clock, uint8_t count);
    void ReceiveAnnounceAck(StationIndex src);
    void ReceiveStop(StationIndex src);

public:
    virtual bool IsReadyToSetValue() const;
    virtual uint32_t GetDestBitmap() const;

    virtual uint64_t GetRegisterCountAll() const
    {
        return m_RegisterCountAll;
    }
    void SetRegisterCountAll(StationIndex stationIndex, uint8_t count);

public:
    class AnnounceCommandToken : public CommandToken
    {
    public:
        explicit AnnounceCommandToken(SendClone* pClone)
            : CommandToken(pClone), m_DestBitmap(0)
        {
        }
        virtual CommandType GetType() const
        {
            return CommandType_Announce;
        }
        virtual uint32_t GetDestStationBitmap() const
        {
            return m_DestBitmap;
        }
        void SetDestBitmap(uint32_t destBitmap)
        {
            m_DestBitmap = destBitmap;
        }
        void Exit(StationIndex stationIndex)
        {
            m_DestBitmap &= ~(0x1 << stationIndex);
        }

    private:
        uint32_t m_DestBitmap;
    };


    class EndCommandToken : public CommandToken
    {
    public:
        explicit EndCommandToken(SendClone* pClone)
            : CommandToken(pClone)
        {
        }
        virtual CommandType GetType() const
        {
            return CommandType_End;
        }
        virtual uint32_t GetDestStationBitmap() const
        {
            return static_cast<const SendClone*>(GetClone())->m_ReceiverBmp;
        }
    };


private:
    uint32_t m_ReceiverBmp;

    AnnounceCommandToken m_AnnounceCommandToken;
    EndCommandToken m_EndCommandToken;

    uint64_t m_RegisterCountAll;  // 全員の Register カウンタ (各ステーション 2 bit)

public:
    template <int TEST_ID, typename Arg>
    static void Test(const SendClone& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(SendClone);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
