﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/common/common_ByteOrder.h>

namespace nn
{
namespace pia
{
namespace clone
{

/*!
@brief  クローンエレメントに設定する値のシリアライズ・デシリアライズの方法を定義します。

@tparam Type_   シリアライズ対象のデータ型です。

@details    このクラスが実装されているのは、 Type が uint8_t, int8_t, uint16_t, int16_t, uint32_t, int32_t, uint64_t, int64_t, float, double のものだけです。
任意の型に対してこのクラスを使用するためには、その型に対応したシリアライズ関数とデシリアライズ関数を実装し、シリアライズ後のサイズを GetSerializedSize() の戻り値として定義する必要があります。

例えば、
@code
struct StructValue
{
    uint32_t v1;
    float v2;
};
@endcode
という型に対して、
@code
template<>
void nn::pia::clone::SerializePolicy<StructValue>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    clone::SerializePolicy<uint32_t>::Serialize(offset, value.v1);
    offset += 4;
    clone::SerializePolicy<float>::Serialize(offset, value.v2);
}

template<>
void nn::pia::clone::SerializePolicy<StructValue>::Deserialize(Type* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    clone::SerializePolicy<uint32_t>::Deserialize(&pValue->v1, offset);
    offset += 4;
    clone::SerializePolicy<float>::Deserialize(&pValue->v2, offset);
}

template<>
uint32_t nn::pia::clone::SerializePolicy<StructValue>::GetSerializedSize()
{
    return 4 + 4;
}
@endcode
と実装する必要があります。

異種プラットフォーム間マッチメイクによる異なるプラットフォーム間での通信を行う際には、パディングによる構造体のサイズ変動やエンディアンの不一致等が起こる可能性があります。
その場合に、Serialize / Deserialize / GetSerializedSize 関数で構造体のメモリコピーや sizeof() によるデータサイズ取得を行うように実装すると、正常に通信を行うことができません。
将来にわたって異種プラットフォーム間マッチメイクを行わない場合は、構造体のメモリコピーや sizeof() によるデータサイズ取得を行うように実装することも可能です。
*/
template <typename Type_>
class SerializePolicy
{
    typedef Type_ Type;
public:
    /*!
    @brief  シリアライズします。
    @param[out] pBuffer 出力先バッファです。
    @param[in]  value   シリアライズ対象のデータです。
    */
    static void Serialize(void* pBuffer, const Type& value);

    /*!
    @brief  デシリアライズします。
    @param[out] pValue  デシリアライズされたデータの保存先です。
    @param[in]  cpData  デシリアライズするバイト列です。
    */
    static void Deserialize(Type* pValue, const void* cpData);

    /*!
    @brief  シリアライズされたサイズです。
    @details    @ref nn::pia::clone::CloneProtocol::GetElementSizeMax() "CloneProtocol::GetElementSizeMax()" より大きな値は使用できません。
    @return シリアライズされたサイズです。
    */
    static uint32_t GetSerializedSize();
};

template<>
inline void SerializePolicy<uint8_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU8(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<uint8_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU8(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<uint8_t>::GetSerializedSize()
{
    return sizeof(uint8_t);
}

template<>
inline void SerializePolicy<int8_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU8(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<int8_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU8(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<int8_t>::GetSerializedSize()
{
    return sizeof(int8_t);
}

template<>
inline void SerializePolicy<uint16_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU16(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<uint16_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU16(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<uint16_t>::GetSerializedSize()
{
    return sizeof(uint16_t);
}

template<>
inline void SerializePolicy<int16_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU16(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<int16_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU16(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<int16_t>::GetSerializedSize()
{
    return sizeof(int16_t);
}

template<>
inline void SerializePolicy<uint32_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU32(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<uint32_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU32(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<uint32_t>::GetSerializedSize()
{
    return sizeof(uint32_t);
}

template<>
inline void SerializePolicy<int32_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU32(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<int32_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU32(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<int32_t>::GetSerializedSize()
{
    return sizeof(int32_t);
}

template<>
inline void SerializePolicy<uint64_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU64(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<uint64_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU64(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<uint64_t>::GetSerializedSize()
{
    return sizeof(uint64_t);
}

template<>
inline void SerializePolicy<int64_t>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU64(reinterpret_cast<uint8_t*>(pBuffer), value);
}

template<>
inline void SerializePolicy<int64_t>::Deserialize(Type* pValue, const void* cpData)
{
    *pValue = common::ByteOrder::deserializeU64(reinterpret_cast<const uint8_t*>(cpData));
}

template<>
inline uint32_t SerializePolicy<int64_t>::GetSerializedSize()
{
    return sizeof(int64_t);
}

template<>
inline void SerializePolicy<float>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU32(reinterpret_cast<uint8_t*>(pBuffer), reinterpret_cast<const uint32_t&>(value));
}

template<>
inline void SerializePolicy<float>::Deserialize(Type* pValue, const void* cpData)
{
    uint32_t valueU32 = common::ByteOrder::deserializeU32(reinterpret_cast<const uint8_t*>(cpData));
    *pValue = *(reinterpret_cast<float*>(&valueU32));
}

template<>
inline uint32_t SerializePolicy<float>::GetSerializedSize()
{
    return sizeof(float);
}

template<>
inline void SerializePolicy<double>::Serialize(void* pBuffer, const Type& value)
{
    common::ByteOrder::serializeU64(reinterpret_cast<uint8_t*>(pBuffer), reinterpret_cast<const uint64_t&>(value));
}

template<>
inline void SerializePolicy<double>::Deserialize(Type* pValue, const void* cpData)
{
    uint64_t valueU64 = common::ByteOrder::deserializeU64(reinterpret_cast<const uint8_t*>(cpData));
    *pValue = *(reinterpret_cast<double*>(&valueU64));
}

template<>
inline uint32_t SerializePolicy<double>::GetSerializedSize()
{
    return sizeof(double);
}

}
}
}  // end of namespace nn::pia::clone
