﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneBase.h>


namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  双方向の送受信を行う CloneBase 派生クラスです。
 */
class SharingClone : public CloneBase
{
protected:
    /*!
    @brief デフォルトコンストラクタです。
     */
    SharingClone();

public:
    /*!
    @brief デストラクタです。
     */
    virtual ~SharingClone();


    /*!
    @brief  この SharingClone とデータをやり取りする他のステーションの SharingClone が存在するかどうかを判定します。
    @details    相手の SharingClone が途中で登録解除された場合、アプリケーションや内部によるデータの送受信や、
                CloneProtocol の通信終了、ステーションの離脱等が起こるまでは反映されないので注意してください。
    @param[in]  stationId   相手の @ref nn::pia::StationId "StationId" です。
    @return この SharingClone とデータをやり取りする SharingClone が存在すれば true を返します。
    @see    IsConnectedWithAll
     */
    bool IsConnected(StationId stationId) const;


    /*!
    @brief  この SharingClone とデータをやり取りする SharingClone が全てのステーションに存在するかどうかを判定します。
    @details    CloneProtocol::IsActiveStation() が true となるステーションが判定対象です。
    @details    相手の SharingClone が途中で登録解除された場合、アプリケーションや内部によるデータの送受信等が
                起こるまでは反映されないので注意してください。
    @return この SharingClone とデータをやり取りする SharingClone が全てのステーションに存在すれば true を返します。
    @see    IsConnected
     */
    bool IsConnectedWithAll() const;


public:
    /*!
    @cond PRIVATE
    @brief  宛先を取得します。（デバッグ用）
     */
    uint32_t GetParticipantBmp() const
    {
        return m_ParticipantBmp;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  接続要求の宛先を取得します。（デバッグ用）
     */
    uint32_t GetRequestBmp() const
    {
        return m_RequestCommandToken.GetDestStationBitmap();
    }
    //! @endcond

    virtual void Trace(uint64_t flag) const;

    //! @cond PRIVATE

protected:
    virtual void OnStartup();
    virtual void OnCleanup();
    virtual void OnEnd();
    virtual void OnSendFinished();
    virtual bool IsEndProcessFinished();

    virtual void OnExit(StationIndex stationIndex);
    virtual void OnDisconnect(StationIndex stationIndex, bool isValid);
    virtual void OnRegister();

    virtual void OnAddParticipant(StationIndex stationIndex)
    {
        NN_PIA_UNUSED(stationIndex);
    }

public:
    enum RequestResponse
    {
        RequestResponse_Accept,
        RequestResponse_Reject,
        RequestResponse_Ignore
    };
    RequestResponse ReceiveRequest(StationIndex src, ClockValue clock, uint8_t count);
    void ReceiveAccept(StationIndex src, ClockValue clock, uint8_t count);
    virtual void ReceiveReject(StationIndex src, ClockValue clock);
    void ReceiveEnd(StationIndex src);
    virtual void ReceiveStop(StationIndex src);

public:
    virtual uint32_t GetDestBitmap() const;
    virtual bool IsInRequest() const
    {
        return m_RequestCommandToken.IsInList();
    }

    virtual uint8_t GetLocalRegisterCount() const
    {
        return m_LocalRegisterCount;
    }
    virtual uint64_t GetRegisterCountAll() const
    {
        return m_RegisterCountAll;
    }
    virtual bool IsMatchLocalRegisterCount(uint64_t registerCountAll) const;
    void IncrementLocalRegisterCount();
    void SetRegisterCountAll(StationIndex stationIndex, uint8_t count);

protected:
    bool IsSendingRequest(StationIndex stationIndex) const;

public:
    class RequestCommandToken : public ClockCommandToken
    {
    public:
        explicit RequestCommandToken(SharingClone* pClone)
            : ClockCommandToken(pClone), m_DestBitmap(0), m_Clock(InvalidClock)
        {
        }
        virtual CommandType GetType() const
        {
            return CommandType_Request;
        }
        virtual uint32_t GetDestStationBitmap() const
        {
            return m_DestBitmap;
        }
        void SetDestBitmap(uint32_t destBitmap)
        {
            m_DestBitmap = destBitmap;
        }
        void Exit(StationIndex stationIndex)
        {
            m_DestBitmap &= ~(0x1 << stationIndex);
        }
        virtual ClockValue GetClock() const
        {
            return m_Clock;
        }
        void SetClock(ClockValue clock)
        {
            m_Clock = clock;
        }

    private:
        uint32_t m_DestBitmap;
        ClockValue m_Clock;
    };

    class EndCommandToken : public CommandToken
    {
    public:
        explicit EndCommandToken(SharingClone* pClone)
            : CommandToken(pClone)
        {
        }
        virtual CommandType GetType() const
        {
            return CommandType_End;
        }
        virtual uint32_t GetDestStationBitmap() const
        {
            return static_cast<const SharingClone*>(GetClone())->m_ParticipantBmp;
        }
    };

private:
    uint32_t m_ParticipantBmp;

    RequestCommandToken m_RequestCommandToken;
    EndCommandToken m_EndCommandToken;

    uint8_t m_LocalRegisterCount;  // 自分の Register カウンタ (2 bit)
    uint64_t m_RegisterCountAll;   // 全員の Register カウンタ (各ステーション 2 bit)

public:
    template <int TEST_ID, typename Arg>
    static void Test(const SharingClone& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(SharingClone);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
