﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_UnreliableCloneElementBase.h>
#include <nn/pia/clone/clone_SerializePolicyDefinition.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  到達保証されない値の送受信を管理します。

    @tparam Value_              管理する値の型です。
 */
template <typename Value_>
class UnreliableCloneElement : public UnreliableCloneElementBase
{
public:
    /*!
    @brief  このオブジェクトが管理する値の型です。 
     */
    typedef Value_ Value;


    /*!
    @brief  Value をシリアライズするためのアルゴリズムです。
     */
    typedef SerializePolicy<Value> ValueSerializePolicy;


    /*!
    @brief デフォルトコンストラクタです。
     */
    UnreliableCloneElement()
        : UnreliableCloneElementBase(), m_Value()
    {
    }


    /*!
    @brief デストラクタです。
     */
    virtual ~UnreliableCloneElement()
    {
    }


    /*!
    @brief  値を設定します。
    @param[in]  value   設定する値です。
    @return 成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState  登録している CloneBase が送信可能状態ではありません。[:progErr]
     */
    Result SetValue(const Value& value);


    /*!
    @brief  値を取得します。
    @return このオブジェクトが管理する値です。IsValidValue() が false の場合に取得できる値は不定です。
     */
    const Value& GetValue() const
    {
        return m_Value;
    }


    //! @cond PRIVATE

protected:
    virtual uint32_t GetSize() const
    {
        return ValueSerializePolicy::GetSerializedSize();
    }
    virtual void Serialize(void* pBuffer) const
    {
        ValueSerializePolicy::Serialize(pBuffer, m_Value);
    }
    virtual void Deserialize(const void* cpData)
    {
        ValueSerializePolicy::Deserialize(&m_Value, cpData);
    }
    virtual void ClearValue()
    {
        m_Value = Value();
    }


private:
    Value m_Value;
    NN_PIA_DISALLOW_COPY(UnreliableCloneElement);
    //! @endcond
};


//! @cond
template <typename Value>
Result UnreliableCloneElement<Value>::SetValue(const Value& value)
{
    bool isCopyValue;
    Result r = SetValueCore(&isCopyValue);
    if (isCopyValue)
    {
        m_Value = value;
    }
    return r;
}
//! @endcond
}
}
}  // end of namespace nn::pia::clone
