﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_CloneElementBase.h>


namespace nn
{
namespace pia
{
namespace clone
{


class IDataPacker;

/*!
    @brief  到達保証されない値の送受信を管理する基底クラスです。
 */
class UnreliableCloneElementBase : public CloneElementBase
{
protected:
    /*!
    @brief デフォルトコンストラクタです。
     */
    UnreliableCloneElementBase();

public:
    /*!
    @brief デストラクタです。
     */
    virtual ~UnreliableCloneElementBase();


    /*!
    @brief  @ref UnreliableCloneElement::GetValue "GetValue()" で有効な値が取得できるかどうかを取得します。
    @return @ref UnreliableCloneElement::GetValue "GetValue()" で有効な値を取得できる場合は true を返します。
     */
    bool IsValidValue() const
    {
        return m_IsValidValue;
    }


    /*!
    @brief  値が設定された時刻を取得します。
    @return 値が設定された時刻です。 @ref IsValidValue() が false の場合に取得できる値は不定です。
     */
    ClockValue GetClock() const
    {
        return m_Clock;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


    //! @cond PRIVATE

public:
    virtual Type GetType() const
    {
        return GetTypeStatic();
    }

    static Type GetTypeStatic()
    {
        return Type_Unreliable;
    }
    static bool Receive(CloneElementBase* pElement, IDataPacker* pAckPacker, const void* cpChunk, uint16_t chunkSize, Id elementId, StationIndex src, StationIndex localStationIndex, uint32_t dispatchCount);
    static uint32_t GetDataChunkHeaderSize();

private:
    void ReceiveData(const void* cpData, uint32_t size, ClockValue clock);

public:
    virtual void ClearData();
    virtual void RemoveDest(StationIndex stationIndex)
    {
        // n3492: UnreliableCloneElementBase では宛先の bitmap をメンバとして持っておらず、
        //        CloneBase の bitmap を参照しているだけなので、ここでフラグを降ろす処理は必要ありません。
        //        宛先のフラグが全て降りているときに Clone の m_SendDataList からトークンを削除する処理が無いため
        //        LAN 切断が発生した場合などに m_SendDataList が残ってしまいますが、
        //        アプリ側で CloneProtocol::UpdateClock() を呼ぶことで残ったデータが処理されるので問題ありません。
        NN_PIA_UNUSED(stationIndex);
    }

private:
    class UnreliableSendToken : public CloneElementBase::ISendToken
    {
    public:
        explicit UnreliableSendToken(UnreliableCloneElementBase* pElement)
            : CloneElementBase::ISendToken(), m_pElement(pElement)
        {
        }
        virtual Type GetElementType() const
        {
            return Type_Unreliable;
        }
        virtual uint32_t GetDestBitmap() const;
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }
        virtual void TraceTokenType()
        {
            PIA_TRACE_EX(common::TraceFlagClone, "TokenType: UnreliableSendToken");
        };

    private:
        Id GetId() const
        {
            return m_pElement->GetId();
        }
        uint32_t GetSize() const
        {
            return m_pElement->GetSize();
        }
        ClockValue GetClock() const
        {
            return m_pElement->m_Clock;
        }
        void Serialize(void* pBuffer) const
        {
            m_pElement->Serialize(pBuffer);
        }

    private:
        UnreliableCloneElementBase* m_pElement;
    };

protected:
    Result SetValueCore(bool* pIsCopyValue);

    virtual void Serialize(void* pBuffer) const = 0;
    virtual void Deserialize(const void* cpData) = 0;
    virtual void ClearValue() = 0;

private:
    bool m_IsValidValue;
    ClockValue m_Clock;
    UnreliableSendToken m_SendDataToken;

    static Definition<UnreliableCloneElementBase> s_Definition;

public:
    template <int TEST_ID, typename Arg>
    static void Test(const UnreliableCloneElementBase& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(UnreliableCloneElementBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
