﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


// NN_PIA_COMMON_SET_PROGRAM_INFORMATION で使用する SDK バージョン番号を参照するためのマクロ。
#if NN_PIA_CTR
#define NN_PIA_SDK_VER NN_CURRENT_VERSION_NUMBER
#elif NN_PIA_CAFE
#define NN_PIA_SDK_VER CAFE_OS_SDK_VERSION
#elif NN_PIA_WIN || NN_PIA_NINTENDOSDK || NN_PIA_A || NN_PIA_B
#define NN_PIA_SDK_VER 0xffffffff
#else
#error "Invalid platform."
#endif

// @addtogroup     pia_common_debug_macro             Pia の提供するデバッグ用マクロ定義
// @brief          Pia の提供するデバッグ用マクロです。
//! @{
//!     @name       その他
//!     @{

/*!
      @def    NN_PIA_COMMON_SET_PROGRAM_INFORMATION
      @brief  アプリケーションが使用する SDK や NEX の情報を Pia ライブラリに登録するマクロです。

      @details common::Initialize() 以降で呼び出してもらうことを想定しています。
              NEX バージョン番号の定数を参照しているため、あらかじめ NEX のヘッダファイルを
              #include しておく必要があります。
     */
#define NN_PIA_COMMON_SET_PROGRAM_INFORMATION() \
    nn::pia::common::SetProgramInformation(     \
        NN_PIA_SDK_VER,                         \
        NEX_VERSION_MAJOR,                      \
        NEX_VERSION_MINOR,                      \
        NEX_VERSION_MICRO)

//!     @}
//! @}


//  common モジュールの非メンバ関数群です。

namespace nn
{
namespace pia
{
namespace common
{


/*!
      @brief  common モジュールが初期化された状態であるかどうかを返します。

      @return 初期化されていれば true 、されていなければ false が返されます。
     */
bool IsInitialized(void);


/*!
      @brief  common モジュールを初期化します。

      @details 初期化により、Piaライブラリが使用するヒープがセットアップされます。
              続いて nn::pia::common::Report インスタンスがセットアップされ、
              PIA_REPORT()などが利用できるようになります。
              @if CTR_DOC
              Piaは内部のスレッド管理を効率的にするため、暗黙的にスレッドローカル
              ストレージを使用しています。詳しくはプログラミングマニュアルを参照してください。
              @endif

      @param[in] pMem    Piaライブラリに渡すメモリブロックの先頭を指すポインタ。
                         32bit 環境であれば 4バイト、64bit 環境であれば 8バイトのアライメントが必要です。
      @param[in] size    pMemで渡されたメモリブロックのサイズ。単位はバイト。
                         ヒープを構築できるだけの十分なサイズがない場合、ライブラリ内部で
                         アサート停止します。

      @return 無事に初期化が完了すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

      @retval ResultInvalidArgument     引数が誤っています。[:progErr]
      @retval ResultAlreadyInitialized  既に初期化は終了しています。[:progErr]
     */
Result Initialize(void* pMem, uint32_t size);


/*!
      @brief  common モジュールを終了します。

      @details @ref Initialize 時にセットアップした nn::pia::common::Report インスタンスが破棄され、
              アプリケーションが @ref Initialize でPiaライブラリに供給したメモリが解放されます。
              common モジュールが初期化されていない状態の時にこの API を呼び出した場合は、何もしません。
     */
void Finalize(void);


/*!
      @brief  セットアップの開始を宣言します。

      @details common モジュールのシングルトンなどを作成する前に呼ぶ必要があります。
              アプリケーションが common モジュールのシングルトンを一つも作成しない場合であっても、
              この API と @ref EndSetup は必ず呼び出す必要があります。

      @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

      @retval ResultNotInitialized  common モジュールの初期化関数が呼ばれていません。[:progErr]
      @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
     */
Result BeginSetup(void);


/*!
      @brief  セットアップの終了を宣言します。

      @details common モジュールのシングルトンなどを作成し終えた後に呼ぶ必要があります。
              このAPI呼び出しにより、 common モジュールのシングルトンなどが消費する
              メモリを確定し、ライブラリ内部でのメモリ管理を最適化します。
              アプリケーションが common モジュールのシングルトンを一つも作成しない場合であっても、
              @ref BeginSetup とこの API は必ず呼び出す必要があります。

      @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

      @retval ResultNotInitialized  common モジュールの初期化関数が呼ばれていません。[:progErr]
      @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
     */
Result EndSetup(void);

/*!
      @brief この API を呼び出した時点における Pia ライブラリのメモリ使用量を得ます。

      @details この API は、各モジュールの BeginSetup() ～ EndSetup() 間では
               呼び出してはいけません。もし呼び出してしまうと、誤った値が返されます。

      @return この API を呼び出した時点での Pia ライブラリのメモリ使用量が返されます。
              common モジュールのセットアップがされていないときや、内部でエラーが
              発生した場合は、0 が返されます。

      @retval 0 Pia ライブラリがセットアップされていないとき
     */
uint32_t GetMemoryUsage(void);


/*!
      @cond PRIVATE
      @brief   プログラム情報を登録します。

      @details アプリケーションが使用する SDK や NEX のバージョン番号を登録します。
      @endcond
     */
void SetProgramInformation(uint32_t sdkVer, uint8_t nexMajorVer, uint8_t nexMinorVer, uint8_t nexMicroVer);


/*!
      @cond PRIVATE
      @brief BeginSetup() ～ EndSetup() 間の状態であるかどうかを返します。
      @endcond
     */
bool IsInSetupMode(void);


/*!
      @cond PRIVATE
      @brief デバッグ情報をプリントします。
      @endcond
     */
void PrintDebugInfo(void);


/*!
      @cond PRIVATE
      @brief 環境情報をプリントします。
      @endcond
     */
void PrintEnvironmentInfo(void);


/*!
      @brief     RegisterInternalPrintCallback に渡すコールバック型です。

      @param[in] pStr 文字列の先頭を指すポインタ。ヌル文字で終端されているとは限りません。
      @param[in] len  文字列長。
     */
typedef void (*InternalPrintCallback)(const char* pStr, int32_t len);


/*!
      @brief     Pia のコンソール出力をフックするコールバックを設定する関数です。

      @param[in] pCallback 登録するコールバック関数のポインタ。
      @return    この関数を呼び出す前に登録されていたコールバック関数のポインタが返されます。
                 何も登録されていなければ、NULL が返ります。
     */
InternalPrintCallback RegisterInternalPrintCallback(InternalPrintCallback pCallback);


/*!
    @cond PRIVATE
    @brief 文字列をプリントします。

    @param[in] cpFormat フォーマット文字列です。
    @param[in] varg     引数リストです。
     */
void VPrintf(const char* cpFormat, va_list varg);
//! @endcond


/*!
    @cond PRIVATE
    @brief 文字列をプリントします。

    @param[in] cpFormat フォーマット文字列です。
    @param[in] ...      可変長引数です。
     */
void Printf(const char* cpFormat, ...);
//! @endcond
}
}
}  // end of namespace nn::pia::common
