﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief  バイトオーダー変換

 */
class ByteOrder
{
public:
    static inline uint16_t Swap16(uint16_t v)
    {
        return (v << 8) | (v >> 8);
    }

    static inline uint32_t Swap32(uint32_t v)
    {
        uint32_t w = ((v & 0xff00ff00) >> 8) | ((v & 0x00ff00ff) << 8);
        return (w >> 16) | (w << 16);
    }

    static inline uint64_t Swap64(uint64_t v)
    {
        uint64_t w = ((v & 0xff00ff00ff00ff00) >> 8) | ((v & 0x00ff00ff00ff00ff) << 8);
        uint64_t x = ((w & 0xffff0000ffff0000) >> 16) | ((w & 0x0000ffff0000ffff) << 16);
        return (x >> 32) | (x << 32);
    }

    /*!
    @brief uint16_t の値をネットワークバイトオーダーからホストバイトオーダーに変換

    @param[in] net  ネットワークバイトオーダーの uint16_t の値です。

    @return ホストバイトオーダーの uint16_t の値です。
     */
    static inline uint16_t NetworkToHost16(uint16_t net)
    {
#if NN_PIA_ENDIAN_BIG
        return net;
#else
        return Swap16(net);
#endif
    }


    /*!
    @brief uint32_t の値をネットワークバイトオーダーからホストバイトオーダーに変換

    @param[in] net  ネットワークバイトオーダーの uint32_t の値です。

    @return ホストバイトオーダーの uint32_t の値です。
     */
    static inline uint32_t NetworkToHost32(uint32_t net)
    {
#if NN_PIA_ENDIAN_BIG
        return net;
#else
        return Swap32(net);
#endif
    }


    /*!
    @brief uint64_t の値をネットワークバイトオーダーからホストバイトオーダーに変換

    @param[in] net  ネットワークバイトオーダーの uint64_t の値です。

    @return ホストバイトオーダーの uint64_t の値です。
     */
    static inline uint64_t NetworkToHost64(uint64_t net)
    {
#if NN_PIA_ENDIAN_BIG
        return net;
#else
        return Swap64(net);
#endif
    }


    /*!
    @brief uint16_t の値をホストバイトオーダーからネットワークバイトオーダーに変換

    @param[in] host ホストバイトオーダーの uint16_t の値です。

    @return ネットワークバイトオーダーの uint16_t の値です。
     */
    static inline uint16_t HostToNetwork16(uint16_t host)
    {
#if NN_PIA_ENDIAN_BIG
        return host;
#else
        return Swap16(host);
#endif
    }


    /*!
    @brief uint32_t の値をホストバイトオーダーからネットワークバイトオーダーに変換

    @param[in] host ホストバイトオーダーの uint32_t の値です。

    @return ネットワークバイトオーダーの uint32_t の値です。
     */
    static inline uint32_t HostToNetwork32(uint32_t host)
    {
#if NN_PIA_ENDIAN_BIG
        return host;
#else
        return Swap32(host);
#endif
    }


    /*!
    @brief uint64_t の値をホストバイトオーダーからネットワークバイトオーダーに変換

    @param[in] host ホストバイトオーダーの uint64_t の値です。

    @return ネットワークバイトオーダーの uint64_t の値です。
     */
    static inline uint64_t HostToNetwork64(uint64_t host)
    {
#if NN_PIA_ENDIAN_BIG
        return host;
#else
        return Swap64(host);
#endif
    }

    /*!
    @brief ネットワークバイトオーダー（ビッグエンディアン）にシリアライズします
    */
    static inline void serializeU64(uint8_t* pData, uint64_t a)
    {
        pData[0] = static_cast<uint8_t>((a >> 56) & 0xff);
        pData[1] = static_cast<uint8_t>((a >> 48) & 0xff);
        pData[2] = static_cast<uint8_t>((a >> 40) & 0xff);
        pData[3] = static_cast<uint8_t>((a >> 32) & 0xff);
        pData[4] = static_cast<uint8_t>((a >> 24) & 0xff);
        pData[5] = static_cast<uint8_t>((a >> 16) & 0xff);
        pData[6] = static_cast<uint8_t>((a >> 8) & 0xff);
        pData[7] = static_cast<uint8_t>((a >> 0) & 0xff);
    }

    static inline uint64_t deserializeU64(const uint8_t* pData)
    {
        uint64_t tmp = 0;
        tmp |= static_cast<uint64_t>(pData[0]) << 56;
        tmp |= static_cast<uint64_t>(pData[1]) << 48;
        tmp |= static_cast<uint64_t>(pData[2]) << 40;
        tmp |= static_cast<uint64_t>(pData[3]) << 32;
        tmp |= static_cast<uint64_t>(pData[4]) << 24;
        tmp |= static_cast<uint64_t>(pData[5]) << 16;
        tmp |= static_cast<uint64_t>(pData[6]) << 8;
        tmp |= static_cast<uint64_t>(pData[7]) << 0;
        return tmp;
    }

    static inline void serializeU32(uint8_t* pData, uint32_t a)
    {
        pData[0] = static_cast<uint8_t>((a & 0xff000000) >> 24);
        pData[1] = static_cast<uint8_t>((a & 0x00ff0000) >> 16);
        pData[2] = static_cast<uint8_t>((a & 0x0000ff00) >> 8);
        pData[3] = static_cast<uint8_t>((a & 0x000000ff) >> 0);
    }

    static inline uint32_t deserializeU32(const uint8_t* pData)
    {
        return (static_cast<uint32_t>(pData[0]) << 24) |
               (static_cast<uint32_t>(pData[1]) << 16) |
               (static_cast<uint32_t>(pData[2]) << 8)  |
               (static_cast<uint32_t>(pData[3]) << 0);
    }

    static inline void serializeU16(uint8_t* pData, uint16_t a)
    {
        pData[0] = static_cast<uint8_t>((a & 0xff00) >> 8);
        pData[1] = static_cast<uint8_t>((a & 0x00ff) >> 0);
    }

    static inline uint16_t deserializeU16(const uint8_t* pData)
    {
        return (static_cast<uint16_t>(pData[0]) << 8) | 
               (static_cast<uint16_t>(pData[1]) << 0);
    }

    //n1589:1 byteはシリアライズ関数なくても問題ないが、コーディングを楽にするため
    static inline void serializeU8(uint8_t* pData, uint8_t a)
    {
        pData[0] = a;
    }

    static inline uint8_t deserializeU8(const uint8_t* pData)
    {
        return pData[0];
    }

    /*!
    @cond PRIVATE
    @brief  指定したバイト列をバイト反転させます。
    @tparam Size    反転させるバイト列のバイト数です。1,2,4,8のいずれかのみ指定できます。
    @param[out] pTo     反転させた結果を書き込むアドレスを指定します。
    @param[in]  cpFrom  反転させるバイト列を指定します。
     */
    template <uint32_t Size>
    static inline void Swap(void* pTo, const void* cpFrom);
    //! @endcond
};


//! @cond
template <>
inline void ByteOrder::Swap<1>(void* pTo, const void* cpFrom)
{
    *reinterpret_cast<uint8_t*>(pTo) = *reinterpret_cast<const uint8_t*>(cpFrom);
}

template <>
inline void ByteOrder::Swap<2>(void* pTo, const void* cpFrom)
{
    *reinterpret_cast<uint16_t*>(pTo) = Swap16(*reinterpret_cast<const uint16_t*>(cpFrom));
}

template <>
inline void ByteOrder::Swap<4>(void* pTo, const void* cpFrom)
{
    *reinterpret_cast<uint32_t*>(pTo) = Swap32(*reinterpret_cast<const uint32_t*>(cpFrom));
}

template <>
inline void ByteOrder::Swap<8>(void* pTo, const void* cpFrom)
{
    reinterpret_cast<uint32_t*>(pTo)[0] = Swap32(reinterpret_cast<const uint32_t*>(cpFrom)[1]);
    reinterpret_cast<uint32_t*>(pTo)[1] = Swap32(reinterpret_cast<const uint32_t*>(cpFrom)[0]);
}
//! @endcond
}
}
}  // end of namespace nn::pia::common
