﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_CriticalSection.h>


// @addtogroup     pia_common_debug_macro             Pia の提供するデバッグ用マクロ定義
// @brief          Pia の提供するデバッグ用マクロです。
//! @{
//!     @name       ログ出力
//!     @{

/*!
@def   PIA_BASIC_PRINT
@brief コンソール出力をキャッシュします。 CachedPrint インスタンスが生成されていない場合は普通にコンソール出力します。

@param[in] ... 標準ライブラリの printf の引数と同様です。
 */
#if NN_PIA_UNDER_DEVELOP
#if NN_PIA_ENABLE_CACHEDPRINT
#define PIA_BASIC_PRINT(...)                                              \
    if (::nn::pia::common::CachedPrint::GetInstance() != NULL)              \
    {                                                                       \
        ::nn::pia::common::CachedPrint::GetInstance()->Printf(__VA_ARGS__); \
    }                                                                       \
    else                                                                    \
    {                                                                       \
        ::nn::pia::common::Printf(__VA_ARGS__);                             \
    }
#else
#define PIA_BASIC_PRINT(...) ::nn::pia::common::Printf(__VA_ARGS__)   
#endif
#else
#define PIA_BASIC_PRINT(...) \
    {                          \
    }
#endif


/*!
@def   PIA_BASIC_VPRINT
@brief コンソール出力をキャッシュします。 CachedPrint インスタンスが生成されていない場合は普通にコンソール出力します。

@param[in] format 出力書式文字列(フォーマット文字列)を指定します。
@param[in] varg   可変引数リストを指定します。
 */
#if NN_PIA_UNDER_DEVELOP
#if NN_PIA_ENABLE_CACHEDPRINT
#define PIA_BASIC_VPRINT(format, varg)                                          \
    if (::nn::pia::common::CachedPrint::GetInstance() != NULL)                    \
    {                                                                             \
        ::nn::pia::common::CachedPrint::GetInstance()->VPrintf((format), (varg)); \
    }                                                                             \
    else                                                                          \
    {                                                                             \
        ::nn::pia::common::VPrintf((format), (varg));                             \
    }
#else
#define PIA_BASIC_VPRINT(...) ::nn::pia::common::VPrintf(__VA_ARGS__)   
#endif
#else
#define PIA_BASIC_VPRINT(format, varg) \
    {                                    \
        (void)(format);                  \
        (void)(varg);                    \
    }
#endif

//!     @}
//! @}


namespace nn
{
namespace pia
{
namespace common
{
#if NN_PIA_ENABLE_CACHEDPRINT
/*!
  @brief  コンソール出力をキャッシュします。
          このクラスはデバッグ用です。製品には組み込んではいけません。

 */
class CachedPrint : public common::RootObject
{
public:
    /*!
    @brief インスタンスを作成します（シングルトンパターン）。

    @details このAPIは、 nn::pia::common::BeginSetup() を呼び出してから、
           nn::pia::common::EndSetup() を呼び出すまでの間に呼び出す必要があります。
    
    @param[in] bufferSize   キャッシュするバッファのサイズ（バイト）を指定します。512以上の値を指定する必要があります。

    @return インスタンスの作成に成功すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    @retval ResultNotInitialized     commonモジュールが初期化されていません。[:progErr]
    @retval ResultInvalidState       BeginSetup() ～ EndSetup() 間で呼び出されていません。[:progErr]
    @retval ResultInvalidArgument    引数の指定が不正です。[:progErr]
     */
    static Result CreateInstance(uint32_t bufferSize);


    /*!
    @brief インスタンスを破棄します（シングルトンパターン）。

    @details インスタンスが作成されていないときにこの関数が呼び出された場合は、
           何も実行せずに関数から返ります。
     */
    static void DestroyInstance();


    /*!
    @brief  CachedPrint インスタンスへのポインタを取得します（シングルトンパターン）。

    @return インスタンスが作成されていないときは、 NULL ポインタが返ります。
     */
    static CachedPrint* GetInstance()
    {
        return s_pInstance;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


    /*!
    @cond PRIVATE
    @brief キャッシュに文字列を書き込みます。

    @param[in] cpFormat フォーマット文字列です。
    @param[in] varg     引数リストです。
     */
    void VPrintf(const char* cpFormat, va_list varg);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief キャッシュに文字列を書き込みます。

    @param[in] cpFormat フォーマット文字列です。
    @param[in] ...      可変長引数です。
     */
    void Printf(const char* cpFormat, ...);
    //! @endcond


    /*!
    @brief キャッシュされている文字列をクリアします。
     */
    void Clear();


    /*!
    @brief キャッシュされている文字列をコンソールに出力してクリアします。
     */
    void Flush();


    /*!
    @cond PRIVATE
    @brief キャッシュされている文字列を取得します。文字列は NULL 終端されます。（未検証なのでまだ使ってはいけません。）

    @param[out] pBuffer     文字列を取得するバッファです。
    @param[out] pLength     取得した文字列の長さです。
    @param[in] bufferSize   バッファのサイズ（バイト）です。

    @return    成功すれば、IsSuccess()がtrueを返すResultが返ります。

    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultNoData            キャッシュされている文字列はありません。この場合も与えられたバッファには NULL 終端が書き込まれます。[:handling]
    */
    Result Read(char* pBuffer, uint32_t* pLength, uint32_t bufferSize);
    //! @endcond


private:
    // 実装しません
    CachedPrint();
    CachedPrint(const CachedPrint& rhs);
    bool operator==(const CachedPrint& rhs) const;
    bool operator!=(const CachedPrint& rhs) const;
    CachedPrint& operator=(const CachedPrint& rhs);

    explicit CachedPrint(uint32_t bufferSize);
    ~CachedPrint();

    static CachedPrint* s_pInstance;

    char* m_pBuffer;
    uint32_t m_Size;

    uint32_t m_Top;
    uint32_t m_Next;
    uint32_t m_Repeat;

    CriticalSection m_CriticalSection;
};
#endif // NN_PIA_ENABLE_CACHEDPRINT
}
}
}  // end of namespace nn::pia::common
