﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_RootObject.h>

#define USE_CANCEL_REQUEST 1  // n1769


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief      呼び出しコンテキストのクラスです。

  @details    呼び出しコンテキストは、主に非同期関数の実行状態を記録するのに用いられます。
  <br>
 */
class CallContext : public RootObject
{
public:
    //n1589:todo2:引数にユーザ任意引数があった方がよさそう
    /*!
    @brief   非同期処理の完了時に実行するコールバック関数の typedef です。

    @details 非同期処理完了時にコールバック関数が呼び出されるようにするには、
             CompletionCallback を RegisterCompletionCallback() で登録します。

    @see     RegisterCompletionCallback
     */
    typedef void (*CompletionCallback)(Result result, void* pUserArg);

    /*!
    @cond PRIVATE
    @brief     @ref CallContext の状態の列挙です。
    */
    enum State
    {
        State_Available,       //!< 実行可能。
        State_CallInProgress,  //!< 処理を実行中。この状態のときは他の非同期処理の Context としては使用できません。
        State_CallSuccess,     //!< 処理が成功して終了しました。
        State_CallFailure,     //!< 処理が失敗して終了しました。
        State_CallCancelled    //!< 処理がキャンセルされました。
    };
    //! @endcond

    /*!
    @brief コンストラクタです。

    @details 状態は State_Available にセットされます。
     */
    CallContext(void);


    /*!
    @brief デストラクタです。

    @details デストラクタ呼び出し時に、CallContext の状態が State_CallInProgress であった場合には、アサート停止します。
     */
    ~CallContext(void);


    /*!
    @brief     非同期処理の完了時に実行するコールバック関数を設定します。

    @param[in] pCallback  非同期処理完了時に呼び出されるコールバック関数へのポインタです。
    @param[in] pUserArg   pCallback を呼び出す際に第2引数に与える値を指定します。
    @see CompletionCallback
     */
    void RegisterCompletionCallback(CompletionCallback pCallback, void* pUserArg = NULL);

    /*!
    @cond PRIVATE
    @brief
     */
    bool InitiateCall();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief
     */
    void SignalSuccess(Result result);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief
     */
    void SignalFailure(Result result);
//! @endcond


#if USE_CANCEL_REQUEST
    /*!
    @cond PRIVATE
    @brief
     */
    void SignalCancel();
//! @endcond
#else
    /*!
    @cond PRIVATE
    @brief
     */
    void ForceCancel();
//! @endcond
#endif


#if USE_CANCEL_REQUEST
    /*!
    @brief   非同期処理のキャンセルをリクエストします。

    @details ただし、この API 呼び出しによって即座に非同期処理がキャンセルされるわけでは
             ないことに注意してください。
             この API 呼び出し後は、定期的に IsCompleted() で終了状態をチェックし、
             true を返すまで待つ必要があります。

    @see IsCompleted, GetResult
     */
    void Cancel(void);
#else
    /*!
    @brief   非同期処理のキャンセルを行います。

    @details ただし、 CallContext の状態が State_CallInProgress 以外であった場合は、
             このAPI呼び出しは何もせずに返ります。
             このAPIを用いて非同期処理を終了させた場合、 GetResult で取得される値は
             ResultCancelled となります。

             この CallContext を使用する非同期処理の説明にキャンセルが可能だと
             明記されている場合のみキャンセルすることができます。
             その他の非同期処理ではこの関数を呼び出してはいけません。

    @see GetResult
     */
    void Cancel(void);
#endif


    /*!
    @cond PRIVATE
    @brief
     */
    void Complete(Result result);
    //! @endcond


    /*!
    @brief      非同期処理が進行中であるかどうかを判定します。

    @return     非同期処理が進行中の状態であれば true 、そうでなければ false を返します。
     */
    bool IsInProgress() const
    {
        return m_State == State_CallInProgress;
    }


    /*!
    @brief      非同期処理が終了したかどうかを判定します。

    @return     非同期処理が終了した状態（失敗して終了した状態も含みます）であれば true 、そうでなければ false を返します。
     */
    bool IsCompleted() const
    {
        return m_State == State_CallSuccess ||
               m_State == State_CallFailure ||
               m_State == State_CallCancelled;
    }


    /*!
    @cond PRIVATE
    @brief      状態を設定します。

    @param[in]  state 設定したい状態を指定します。
    @see        GetState
     */
    void SetState(State state);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      現在の状態を取得します。

    @return     現在の CallContext の状態。
     */
    State GetState() const
    {
        return m_State;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    非同期処理の実行結果の Result 値をセットします。

    @param[in]  result 非同期処理の実行結果の Result 値。
     */
    void SetResult(Result result);
    //! @endcond


    /*!
    @brief    非同期処理の実行結果の Result 値を取得します。

    @return   非同期処理の実行結果の Result 値。
     */
    Result GetResult() const
    {
        return m_Result;
    }


#if USE_CANCEL_REQUEST
    /*!
    @cond PRIVATE
    @brief    Cancel() 呼び出しが行われたかどうかを判定します。
     */
    bool IsCancelRequested() const
    {
        return m_IsCancelRequested;
    }
//! @endcond
#endif

    void Reset();

protected:
    State m_State;
    Result m_Result;
    CompletionCallback m_CompletionCallback;
    void* m_pUserArg;
#if USE_CANCEL_REQUEST
    bool m_IsCancelRequested;
#endif

private:
    NN_PIA_DISALLOW_COPY(CallContext);
};


typedef CallContext AsyncContext;
}
}
}  // end of namespace nn::pia::common
