﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

namespace nn
{
namespace pia
{
namespace common
{
/*!
  @brief  日時を表すクラスです。
 */
class DateTime : public nn::pia::common::RootObject
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    DateTime()
        : m_Year(0), m_Month(0), m_Day(0), m_Hour(0), m_Minute(0), m_Second(0), m_IsRegistered(false)
    {
    }

    /*!
    @brief コピーコンストラクタです。

    @param[in] rhs  コピーする DateTime です。
     */
    DateTime(const DateTime& rhs)
    {
        m_Year = rhs.m_Year;
        m_Month = rhs.m_Month;
        m_Day = rhs.m_Day;
        m_Hour = rhs.m_Hour;
        m_Minute = rhs.m_Minute;
        m_Second = rhs.m_Second;
        m_IsRegistered = rhs.m_IsRegistered;
    }

    /*!
    @brief 指定日時で構築するコンストラクタです。

    @param[in] year     年
    @param[in] month    月
    @param[in] day      日
    @param[in] hour     時
    @param[in] minute   分
    @param[in] second   秒
     */
    DateTime(uint16_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t minute, uint8_t second)
    {
        m_Year = year;
        m_Month = month;
        m_Day = day;
        m_Hour = hour;
        m_Minute = minute;
        m_Second = second;
        m_IsRegistered = true;
    }

    /*!
    @brief 代入演算子です。
    @param[in] rhs  代入する日時です。
     */
    DateTime& operator=(const DateTime& rhs)
    {
        m_Year = rhs.m_Year;
        m_Month = rhs.m_Month;
        m_Day = rhs.m_Day;
        m_Hour = rhs.m_Hour;
        m_Minute = rhs.m_Minute;
        m_Second = rhs.m_Second;
        m_IsRegistered = rhs.m_IsRegistered;
        return *this;
    }

    /*!
    @brief 年を指定します。
    @details 1-9999 までを指定する必要があります。
    @param[in] year     年
     */
    void SetYear(uint16_t year)
    {
        m_Year = year;
        m_IsRegistered = true;
    }

    /*!
    @brief 指定された年を取得します。
    @return 指定された年を返します。
     */
    uint16_t GetYear() const
    {
        return m_Year;
    }

    /*!
    @brief 月を指定します。
    @details 1-12 までを指定する必要があります。
    @param[in] month    月
     */
    void SetMonth(uint8_t month)
    {
        m_Month = month;
        m_IsRegistered = true;
    }

    /*!
    @brief 指定された月を取得します。
    @return 指定された月を返します。
     */
    uint8_t GetMonth() const
    {
        return m_Month;
    }

    /*!
    @brief 日を指定します。
    @details 1-31 までを指定する必要があります。
    @param[in] day     日
     */
    void SetDay(uint8_t day)
    {
        m_Day = day;
        m_IsRegistered = true;
    }

    /*!
    @brief 指定された日を取得します。
    @return 指定された日を返します。
     */
    uint8_t GetDay() const
    {
        return m_Day;
    }

    /*!
    @brief 時を指定します。
    @details 0-23 までを指定する必要があります。
    @param[in] hour     時
     */
    void SetHour(uint8_t hour)
    {
        m_Hour = hour;
        m_IsRegistered = true;
    }

    /*!
    @brief 指定された時を取得します。
    @return 指定された時を返します。
     */
    uint8_t GetHour() const
    {
        return m_Hour;
    }

    /*!
    @brief 分を指定します。
    @details 0-59 までを指定する必要があります。
    @param[in] minute     分
     */
    void SetMinute(uint8_t minute)
    {
        m_Minute = minute;
        m_IsRegistered = true;
    }

    /*!
    @brief 指定された分を取得します。
    @return 指定された分を返します。
     */
    uint8_t GetMinute() const
    {
        return m_Minute;
    }

    /*!
    @brief 秒を指定します。
    @details 0-59 までを指定する必要があります。
    @param[in] second     秒
     */
    void SetSecond(uint8_t second)
    {
        m_Second = second;
        m_IsRegistered = true;
    }

    /*!
    @brief 指定された秒を取得します。
    @return 指定された秒を返します。
     */
    uint8_t GetSecond() const
    {
        return m_Second;
    }

    /*!
    @brief 値が指定されたかどうかを取得します。。
    @return 値が指定されていたら true を返します。
     */
    bool IsRegistered() const
    {
        return m_IsRegistered;
    }

    /*!
    @brief 指定された日時が正しいかどうかを取得します。。
    @return 指定された日時が正しければ true を返します。
     */
    bool IsValid() const
    {
        if (m_Year == 0 || m_Year > 9999)
        {
            return false;
        }
        if (m_Month == 0 || m_Month > 12)
        {
            return false;
        }
        if (m_Day == 0)
        {
            return false;
        }
        else if (m_Month == 2)
        {
            uint8_t days = (m_Year % 400 == 0 || (m_Year % 100 != 0 && m_Year % 4 == 0)) ? 29 : 28;
            if (m_Day > days)
            {
                return false;
            }
        }
        else if (m_Month == 4 || m_Month == 6 || m_Month == 9 || m_Month == 11)
        {
            if (m_Day > 30)
            {
                return false;
            }
        }
        else
        {
            if (m_Day > 31)
            {
                return false;
            }
        }
        if (m_Hour > 23)
        {
            return false;
        }
        if (m_Minute > 59)
        {
            return false;
        }
        if (m_Second > 59)
        {
            return false;
        }
        return true;
    }

    /*!
    @brief 指定した日時をクリアします。
     */
    void Clear()
    {
        m_Year = 0;
        m_Month = 0;
        m_Day = 0;
        m_Hour = 0;
        m_Minute = 0;
        m_Second = 0;
        m_IsRegistered = false;
    }

    /*!
    @brief  指定した日時を値として取得します。
    @return 指定した日時を値として返します。
    */
    uint64_t GetValue() const
    {
        return (static_cast<uint64_t>(m_Year) << 40) |
               (static_cast<uint64_t>(m_Month) << 32) |
               (static_cast<uint64_t>(m_Day) << 24) |
               (static_cast<uint64_t>(m_Hour) << 16) |
               (static_cast<uint64_t>(m_Minute) << 8) |
               (static_cast<uint64_t>(m_Second));
    }

    /*!
    @brief      デストラクタです。
     */
    virtual ~DateTime()
    {
    }

private:
    uint16_t m_Year;   //!< 1-9999
    uint8_t m_Month;   //!< 1-12
    uint8_t m_Day;     //!< 1-31
    uint8_t m_Hour;    //!< 0-23
    uint8_t m_Minute;  //!< 0-59
    uint8_t m_Second;  //!< 0-59
    bool m_IsRegistered;
};
}
}
}  // end of namespace nn::pia::common
