﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#if NN_PIA_CTR
#include <nn/os.h>
#elif NN_PIA_A || NN_PIA_B
#include <pthread.h>
#include <semaphore.h>
#endif

namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief      イベントのクラスです。

 */
class Event
{
public:
    /*!
    @brief      コンストラクタです。

    @details イベントを使用するには、明示的に Initialize を呼ぶ必要があります。
     */
    Event();

    /*!
    @brief      デストラクタです。
     */
    ~Event();

    /*!
    @brief      イベントを初期化します。

    @param[in]  bManualReset 手動リセットイベントならば true を、自動リセットイベントであるならば false を指定します。
    @see        Finalize
     */
    void Initialize(bool bManualReset = true);

    /*!
    @brief      イベントを破棄します。

    @see        Initialize
     */
    void Finalize();


    /*!
    @brief      イベントがシグナル状態になるのを待ちます。

    @see        Signal
     */
    void Wait();


    /*!
    @brief      イベントがシグナル状態かどうかを判定します。

    @details    自動リセットイベントの場合、シグナル状態であればクリアされます。

    @see        Wait
     */
    bool TryWait();


    /*!
    @brief      イベントをシグナル状態にします。

    @see        Wait, Reset
     */
    void Signal();


    /*!
    @brief      イベントを非シグナル状態にします。

    @see        Signal
     */
    void Reset();


#if NN_PIA_CTR
    /*!
    @brief      このインスタンスの実体である nn::os::LightEvent を取得します。
    @return     このインスタンスの実体である nn::os::LightEvent です。
    */
    nn::os::LightEvent* GetRawLightEvent()
    {
        return &m_OsEvent;
    }
#endif


protected:
#if NN_PIA_NINTENDOSDK
    nn::os::EventType m_EventInner;  ///< WindowsでのEvent構造体
#elif NN_PIA_CTR
    nn::os::LightEvent m_OsEvent;
#elif NN_PIA_WIN
    HANDLE m_Handle;
#elif NN_PIA_CAFE
    OSEvent m_Event;
#elif NN_PIA_A || NN_PIA_B
    /// シグナルの状態
    enum SignalState
    {
        cSignalState_Auto,         ///< 自動シグナル
        cSignalState_Manual,       ///< 手動シグナルのシグナルではない状態
        cSignalState_ManualSignal  ///< 手動シグナルのシグナルになっている状態
    };
    SignalState mSignalState;  ///< シグナルの状態
    sem_t mSemaphore;
#else
#error "invalid platform";
#endif

private:
    NN_PIA_DISALLOW_COPY(Event);
};
}
}
}  // end of namespace nn::pia::common
