﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief  固定長のリングバッファのクラスです。

 */
template <typename T, int N>
class FixedRingBuffer : public common::RootObject
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    FixedRingBuffer(void)
    {
        Clear();
    }


    /*!
    @brief デストラクタです。
     */
    ~FixedRingBuffer(void)
    {
    }


    /*!
    @brief リングバッファを空にします。
     */
    void Clear(void)
    {
        m_Size = 0;
        m_BeginIndex = 0;
    }


    /*!
    @brief 最後尾に要素を追加します。満杯の場合に実行すると、アサート停止します。
     */
    void PushBack(T value)
    {
        PIA_ASSERT(!IsFull());

        pushBack(value);
    }


    /*!
    @brief 最後尾に要素を追加します。満杯の場合に実行すると、先頭要素が破棄され、上書きされます。アサート停止はしません。
     */
    void PushBackForce(T value)
    {
        pushBack(value);
    }


    /*!
    @brief 先頭から要素を一つ取り除きます。要素が一つも無いときに呼び出した場合は、アサート停止します。
     */
    void PopFront(void)
    {
        PIA_ASSERT(!IsEmpty());

        incBeginIndex();
        --m_Size;
    }


    /*!
    @brief リングバッファは満杯であるかどうかを判定します。
     */
    bool IsFull(void) const
    {
        return m_Size == N;
    }


    /*!
    @brief リングバッファに一つも要素が無いかどうかを判定します。
     */
    bool IsEmpty(void) const
    {
        return m_Size == 0;
    }


    /*!
    @brief リングバッファの要素数を取得します。
     */
    uint32_t GetSize(void) const
    {
        return m_Size;
    }


    /*!
    @brief [] 演算子によるアクセスを提供します（非 const 版）。
     */
    T& operator[](size_t n)
    {
        return m_Values[getIndex(n)];
    }


    /*!
    @brief [] 演算子によるアクセスを提供します（ const 版）。
     */
    const T& operator[](size_t n) const
    {
        return m_Values[getIndex(n)];
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const
    {
        (void)flag;  // Not implemented...
    }


private:
    // 最後尾に要素を追加。要素が満杯の時は、先頭要素が上書きされる。
    void pushBack(T value)
    {
        m_Values[getEndIndex()] = value;
        if (IsFull())
        {
            incBeginIndex();
        }
        else
        {
            ++m_Size;
        }
    }

    // 先頭のインデックスをインクリメント
    void incBeginIndex(void)
    {
        m_BeginIndex = (m_BeginIndex + 1) % N;
    }

    // 現時点での先頭要素が m_Values[] におけるどの位置にあるかを添え字で返す。
    uint32_t getBeginIndex(void) const
    {
        return m_BeginIndex;
    }

    // 現時点での最後尾要素の次の要素が m_Values[] におけるどの位置にあるかを添え字で返す。
    uint32_t getEndIndex(void) const
    {
        return (getBeginIndex() + GetSize()) % N;
    }

    // 具体的なインデックスの値（m_Values[]における添え字の値）を返す
    uint32_t getIndex(size_t n) const
    {
        PIA_ASSERT(n < N);
        return (m_BeginIndex + n) % N;
    }

    uint32_t m_Size;        // 保持されている要素の個数。
    uint32_t m_BeginIndex;  // [0] で示される値の添え字に対応
    T m_Values[N];          // 要素の実体
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
