﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_String.h>

namespace nn
{
namespace pia
{
namespace common
{
/*!
  @brief  固定長のバッファを持つ文字列を表すクラスです。
 */
template <uint32_t N>
class FixedString : public nn::pia::common::String
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    FixedString()
    {
        m_BufferSize = N * MultiByteCharMaxSize;
        Clear();
    }

    /*!
    @brief      デストラクタです。
     */
    virtual ~FixedString()
    {
    }

    /*!
    @brief UTF-8 の文字列から構築するコンストラクタです。

    @param[in] pStr 文字列のポインタです。
     */
    FixedString(const char* pStr)
    {
        m_BufferSize = N * MultiByteCharMaxSize;
        SetString(pStr, CalcBytes(pStr, EncodingType_Utf8), EncodingType_Utf8);
    }

    /*!
    @brief UTF-16 の文字列から構築するコンストラクタです。

    @param[in] pStr 文字列のポインタです。
     */
    FixedString(const charUtf16_t* pStr)
    {
        m_BufferSize = N * MultiByteCharMaxSize;
        SetString(pStr, CalcBytes(pStr, EncodingType_Utf16), EncodingType_Utf16);
    }

    /*!
    @brief 文字列バッファのサイズを取得します。

    @return 文字列バッファのサイズを返します。
     */
    static uint32_t GetBufferSize()
    {
        return N * MultiByteCharMaxSize;
    }

    /*!
    @brief 文字列をクリアします。
     */
    virtual void Clear()
    {
        for (uint32_t i = 0; i < N * MultiByteCharMaxSize + 1; i++)
        {
            m_Buffer[i] = 0;
        }
        m_pBuffer = m_Buffer;
        m_Bytes = 0;
        m_EncodingType = EncodingType_Unknown;
    }

private:
    virtual Result SetString(const void* str, uint32_t byteSize, EncodingType type)
    {
        if (!PIA_IS_VALID_POINTER(str))
        {
            PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "String is NULL.");
        }
        if (byteSize > m_BufferSize)
        {
            PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "String is too long.");
        }
        if (type == EncodingType_Utf16)
        {
            if (byteSize > m_BufferSize / MultiByteCharMaxSize * 2)
            {
                PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "String is too long.");
            }
        }
        else if (type == EncodingType_Utf8)
        {
            const char* p = static_cast<const char*>(str);
            uint32_t length = 0;
            while (*p != 0)
            {
                uint32_t bytes = 0;
                uint8_t ccc = static_cast<uint8_t>(*p);
                if (ccc > 0x00 && ccc <= 0x7f)
                {
                    bytes = 1;
                }
                else if (ccc >= 0xc2 && ccc <= 0xdf)
                {
                    bytes = 2;
                }
                else if (ccc >= 0xe0 && ccc <= 0xef)
                {
                    bytes = 3;
                }
                else if (ccc >= 0xf0 && ccc <= 0xf7)
                {
                    bytes = 4;
                }
                else if (ccc >= 0xf8 && ccc <= 0xfb)
                {
                    bytes = 5;
                }
                else if (ccc >= 0xfc && ccc <= 0xfd)
                {
                    bytes = 6;
                }
                if (bytes > 0)
                {
                    ++length;
                    p += bytes;
                }
                else
                {
                    // ここにくるのはおかしい
                    ++p;
                    PIA_ASSERT(false);
                }
            }
            if (length > m_BufferSize / MultiByteCharMaxSize)
            {
                PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "String is too long.");
            }
        }
        else
        {
            PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "String encoding type is not supported.");
        }

        for (uint32_t i = 0; i < m_BufferSize; i++)
        {
            m_Buffer[i] = 0;
        }

        uint32_t c = 0;
        if (type == EncodingType_Utf8)
        {
            for (const char* p = static_cast<const char*>(str); *p != 0x00 && c < m_BufferSize; ++p)
            {
                m_Buffer[c] = *p;
                ++c;
            }
            m_Buffer[c] = 0x00;
        }
        else if (type == EncodingType_Utf16)
        {
            for (const charUtf16_t* p = static_cast<const charUtf16_t*>(str); *p != 0x0000 && c < m_BufferSize / MultiByteCharMaxSize * 2; ++p)
            {
                charUtf16_t* d = reinterpret_cast<charUtf16_t*>(&m_Buffer[c]);
                *d = *p;
                c += 2;
            }
            m_Buffer[c] = 0x00;
            m_Buffer[c + 1] = 0x00;
        }
        m_pBuffer = m_Buffer;
        m_Bytes = c;
        m_EncodingType = type;

        return ResultSuccess();
    }

    // NULL文字分 +1 しておく
    char m_Buffer[N * MultiByteCharMaxSize + 1];
};
}
}
}  // end of namespace nn::pia::common
