﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once


#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_RootObject.h>
#include <nn/pia/common/common_ByteOrder.h>

namespace nn
{
namespace pia
{
namespace common
{

// 先行宣言。
class String;

/**
 * @brief IPv4アドレス構造体です。
 */
struct InAddr
{
    uint32_t addr;
};


/*!
  @brief  インターネットアドレスを表現するクラスです。

  @details このクラスには、IPアドレス、ポート番号（共にホストバイトオーダー）などの情報が含まれます。
  <br>
 */
class InetAddress : public RootObject
{
public:
    static const uint32_t SerializeSizeMax = sizeof(uint32_t) + sizeof(uint16_t);

    /*!
    @brief    デフォルトコンストラクタです。
              IPアドレス、ポート番号はゼロクリアされた状態にセットされます。
     */
    InetAddress();

    /*!
    @brief    IPアドレスとポート番号を指定して初期化するコンストラクタです。

    @param[in]  uiAddress  IPアドレス
    @param[in]  uiPort     ポート番号
     */
    InetAddress(uint32_t uiAddress, uint16_t uiPort);

    /*!
    @brief    コピーコンストラクタです。

    @param[in]  oInetAddress コピー対象のオブジェクト。
     */
    InetAddress(const InetAddress& oInetAddress);

    /*!
    @brief      デストラクタです。
     */
    ~InetAddress();

    /*!
    @brief      オブジェクトの内容を初期化します。
     */
    void Init()
    {
        m_Address = 0;
        m_Port = 0;
    }

    /*!
    @brief      ホストバイトオーダーのIPアドレスを設定します。

    @param[in]  uiAddress IPアドレス（ホストバイトオーダー）
    @see GetAddress
     */
    void SetAddress(const uint32_t uiAddress)
    {
        m_Address = uiAddress;
    }

    /*!
    @brief      ホストバイトオーダーでIPアドレスを取得します。

    @return     本オブジェクトに設定されていたIPアドレス（ホストバイトオーダー）
    @see SetAddress
     */
    uint32_t GetAddress() const
    {
        return m_Address;
    }

    /*!
    @brief      ホストバイトオーダーのポート番号を設定します。

    @param[in]  ui16Port ポート番号（ホストバイトオーダー）
    @see        GetPort
     */
    void SetPort(const uint16_t ui16Port)
    {
        m_Port = ui16Port;
    }

    /*!
    @brief    ホストバイトオーダーのポート番号を取得します。

    @return   本オブジェクトに設定されていたポート番号（ホストバイトオーダー）
    @see      SetPort
     */
    uint16_t GetPort() const
    {
        return m_Port;
    }

    /*!
    @brief    比較用のキーを取得します。

    @return   比較用のキー
     */
    int64_t GetKey() const;

    /*!
    @brief      代入演算子です。

    @param[in]  oInetAddress 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    InetAddress& operator=(const InetAddress& oInetAddress);

    /*!
    @brief       アドレスの文字列表記を得ます。
                 アドレス文字列は、 255.255.255.255:80 といった書式で表記されます。

    @param[out]  pString アドレス文字列表記をセットするための String オブジェクトを指すポインタ。
     */
    void GetAddressString(String* pString) const;

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] uiTraceFlag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t uiTraceFlag) const;

    /*!
    @brief      等値演算子です。

    @param[in]  oInetAddress 比較対象のオブジェクトです。
    @return     本オブジェクトと比較対象が等しければ true 、等しくなければ false が返ります。
     */
    bool operator==(const InetAddress& oInetAddress) const
    {
        return (GetKey() == oInetAddress.GetKey());
    }

    /*!
    @brief      等値演算子です。

    @param[in]  oInetAddress 比較対象のオブジェクトです。
    @return     本オブジェクトと比較対象が等しくなければ true 、等しければ false が返ります。
     */
    bool operator!=(const InetAddress& oInetAddress) const
    {
        return !operator==(oInetAddress);
    }

    /*!
    @brief      比較演算子です。

    @param[in]  oInetAddress 比較対象のオブジェクトです。
    @return     
     */
    bool operator<(const InetAddress& oInetAddress) const
    {
        return (GetKey() < oInetAddress.GetKey());
    }

    /*!
    @cond PRIVATE
    @brief  オブジェクトをシリアライズしたデータの長さを得ます。単位はバイトです。

    @return シリアライズされたデータの長さが返ります。単位はバイトです。
    @see    Serialize, Deserialize
     */
    uint32_t GetSerializedSize(void) const
    {
        return sizeof(uint32_t) + sizeof(uint16_t);
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      オブジェクトをシリアライズします。

    @param[out] pBuffer    シリアライズされたデータを書き込むバッファを指すポインタ。
    @param[out] pDataLen   シリアライズされたデータの長さが書き込まれます。単位はバイトです。
    @param[in]  bufferSize pBuffer で指定されたバッファのサイズを指定します。
    @return     成功すれば、IsSuccess() が true を返す Result が返ります。
    @retval     ResultInvalidArgument 引数が誤っています（ NULL ポインタであるなど ）。pBuffer, bufferSize で指定されたバッファのサイズが不足していた場合も、このエラーを返します。
    @see        Deserialize, GetSerializedSize
     */
    Result Serialize(uint8_t* pBuffer, uint32_t* pDataLen, uint32_t bufferSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief     シリアライズされたデータから、オブジェクトを復元します。

    @param[in] pData シリアライズされたデータを指すポインタ。

    @return    成功すれば、IsSuccess() が true を返す Result が返ります。
    @retval    ResultInvalidArgument  引数が誤っています（ NULL ポインタであるなど）。
    @see       Serialize, GetSerializedSize
     */
    Result Deserialize(const uint8_t* pData);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      有効なアドレスとポートがセットされているかどうかを判定します。

    @return     有効であれば true が、無効であれば false が返されます。
     */
    bool IsValid(void) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      有効なアドレスがセットされているかどうかを判定します。

    @return     有効であれば true が、無効であれば false が返されます。
     */
    bool IsValidAddress(void) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      有効なポートがセットされているかどうかを判定します。

    @return     有効であれば true が、無効であれば false が返されます。
     */
    bool IsValidPort(void) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      無効化します。
    */
    void Invalidate()
    {
        m_Address = 0;
        m_Port = 0;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      アドレスがプライベートアドレスかどうか判定します。

    @return     プライベートであれば true が、そうでなければ false が返されます。
     */
    bool IsPrivate(void) const;
    //! @endcond


protected:
    uint32_t m_Address;
    uint16_t m_Port;
};
}
}
}  // end of namespace nn::pia::common
