﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_ListNode.h>
#include <nn/pia/common/common_Time.h>

namespace nn
{
namespace pia
{
namespace common
{

/*!
  @cond PRIVATE
  @brief      ジョブの基底クラスです。

  @details ジョブの機能を利用するには、派生クラスで
              純粋仮想関数のExecuteCore()をオーバーライドする必要があります。
 */
class Job : public RootObject
{
public:
    /*!
    @brief ジョブの状態を表現する列挙型です。
     */
    enum State
    {
        State_Initial,   //! ジョブのインスタンスを作成した直後の状態です。
        State_Waiting,   //! ジョブは待ち状態にあります。指定された時間が経過するまで、ジョブは実行されません。
        State_Suspend,   //! ジョブは一時停止状態にあります。
        State_Ready,     //! ジョブは実行可能状態にあります。
        State_Running,   //! ジョブが実行中であることを示す状態です。
        State_Complete,  //! ジョブの処理が完了したことを示す状態です。
        State_Max        //! enum State型の末端を示すために用意したものです。
    };


    /*!
    @brief コンストラクタです。

    @details 本クラスには純粋仮想関数が含まれているため、ジョブの機能を利用するには、
           このクラスから派生させたクラスを作成する必要があります。
     */
    Job();


    /*!
    @brief デストラクタです。
     */
    virtual ~Job();


    /*!
    @brief ジョブをリセットして、再利用可能にします。

    @details スケジューラによって実行されたジョブを再び利用可能にする時などに呼び出します。
           このメンバ関数呼び出しにより、インスタンスはコンストラクタ実行直後の状態と
           等価になります。
           状態が State_Running の時（ExecuteCore実行中）に呼ぶとアサートにかかります。
           その他の状態の時に呼び出すと成功しますが、State_Initial, State_Complete
           以外の時にこの関数を呼び出すと、警告メッセージが出力されることがあります。
     */
    virtual void Reset(bool isCheck = true);


    /*!
    @brief 初期化済み状態（State_Initial）のジョブを実行可能（State_Ready）にします。
           
    @details その他の状態の時に呼び出すと、アサートにかかります。
    <br>
    @param[in] isBackground BackgroundScheduler を使って別スレッドで処理を行う場合は true を指定します。
     */
    void Ready(bool isBackground = false);


    /*!
    @brief サスペンド状態（State_Suspend）のジョブを実行可能（State_Ready）にします。
           
    @details その他の状態の時に呼び出すと、アサートにかかります。
    <br>
    @param[in] isBackground BackgroundScheduler を使って別スレッドで処理を行う場合は true を指定します。
     */
    void Resume(bool isBackground = false);


    /*!
    @brief  ジョブを中断します。
            
    @details 動作自体は Reset() と同じですが、警告メッセージを出しません。
     */
    void Cancel()
    {
        Reset(false);
    }


    /*!
    @brief ジョブの状態を取得します。

    @return 現在のジョブの状態が返ります。
     */
    State GetState() const;


    /*!
    @brief  ジョブを実行中か
    @return State_Waiting, State_Ready, State_Running, State_Suspend のいずれかの状態ならtrueを返します。
     */
    bool IsRunning() const;


    /*!
    @brief  フォアグラウンド実行中か
    @return フォアグラウンドの State_Waiting, State_Ready, State_Running のいずれかの状態ならtrueを返します。
     */
    bool IsForeground() const;

    /*!
    @brief  バックグラウンド実行中か
    @return バックグラウンドの State_Waiting, State_Ready, State_Running のいずれかの状態ならtrueを返します。
     */
    bool IsBackground() const;


    /*!
    @brief  ジョブが何回実行されたかを得ます。

    @return まだ一度もExecute()が実行されていない状態では、ゼロが返されます。
            カウンタのインクリメントは、ExecuteCore()の実行後に行われます。
     */
    uint32_t GetExecutedCount() const
    {
        return m_ExecutedCount;
    }


    /*!
    @brief    待ち状態にあるジョブが次回に実行される時刻を得ます。

    @return   返り値の単位はTickです。
     */
    const Time& GetExecutionTime() const
    {
        return m_ExecutionTime;
    }


    /*!
    @cond PRIVATE
    @brief ジョブを実行します。

    @details このメンバ関数は、スケジューラによって呼び出されることを想定しています。
           アプリケーションプログラムがこの関数を直接呼び出してはいけません。
     */
    void Execute(bool isBackground);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief リストノードのオフセットです。
     */
    static int32_t GetListNodeOffset()
    {
        NN_PIA_PRAGMA_PUSH_WARNINGS
        NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
        return offsetof(Job, m_ListNode);
        NN_PIA_PRAGMA_POP_WARNINGS
    }
    //! @endcond


    /*!
    @brief  Executeの結果の種類を表します。
     */
    enum ExecuteState
    {
        ExecuteState_Continue,            //! 処理が完了していません。続けて実行します。
        ExecuteState_Complete,            //! 処理が完了しました。
        ExecuteState_Cancel,              //! 処理が中断されました。（現状COMPLETEと区別してない）
        ExecuteState_Suspend,             //! 処理は一時中断状態になりました。
        ExecuteState_Wait,                //! 処理は完了していないので一定時間後に再開します。
        ExecuteState_WaitNext,            //! 処理は完了していません。次の Scheduler::Dispatch() で再開します。バックグラウンドでは使えません。
        ExecuteState_ContinueMain,        //! 処理が完了していません。次はメインの Dispatch() で実行します。
        ExecuteState_ContinueBackground,  //! 処理が完了していません。次はバックグラウンドから実行します。
        ExecuteState_Max                  //! enum ExecuteState型の末端を示すために用意したものです。
    };
    PIA_COMPILE_ASSERT(ExecuteState_Max < 255);  // ExecuteState の enum を uint8_t の変数に格納している都合で、上限チェックのコードを入れておきます。

    /*!
    @brief  Executeの結果を表します。
     */
    struct ExecuteResult
    {
    public:
        /*!
        @brief コンストラクタ

        @details ExecuteState_Wait 以外を指定するときは、このオーバーロードを使ってExecuteStateだけ指定する必要があります。
               暗黙の型変換を可能にしていますのでExecuteCoreの返り値にExecuteStateを直接渡すことができます。
         */
        ExecuteResult(ExecuteState executeState)
            : m_ExecuteState(static_cast<uint8_t>(executeState)), m_WaitTime(0)
        {
            PIA_ASSERT(executeState >= 0 && executeState < ExecuteState_Max);
        }

        /*!
        @brief コンストラクタ
               
        @details ExecuteState_Wait を指定するときは、このオーバーロードを使って待ち時間を含めて設定する必要があります。
         */
        ExecuteResult(ExecuteState executeState, uint16_t waitTime)
            : m_ExecuteState(static_cast<uint8_t>(executeState)), m_WaitTime(waitTime)
        {
            PIA_ASSERT(executeState == ExecuteState_Wait || waitTime == 0);
        }

        uint8_t m_ExecuteState;
        uint8_t m_Reserved;
        uint16_t m_WaitTime;
    };


protected:
    /*!
    @brief ジョブで実行される処理を記述するための純粋仮想関数です。

    @details 実際に実行する処理は、この関数をオーバーライドして記述します。
     */
    virtual ExecuteResult ExecuteCore() = 0;


private:
    enum InnerState
    {
        InnerState_Initial,
        InnerState_FgWaiting,
        InnerState_FgReady,
        InnerState_FgRunning,
        InnerState_BgWaiting,
        InnerState_BgReady,
        InnerState_BgRunning,
        InnerState_Suspend,
        InnerState_Complete,
        InnerState_Num
    };


    /*!
    @brief ジョブの状態の文字列表現を得ます。

    @return ジョブの状態を表す文字列へのポインタが返されます。
     */
    static const char* GetInnerStateString(InnerState innerState);


    // ジョブの状態を表すメンバ変数です。
    volatile InnerState m_InnerState;

    // 何度Execute()が実行されたかを示すメンバ変数です。
    uint32_t m_ExecutedCount;

    // 待ち状態にあるジョブが、実行可能状態に遷移できる時刻を保持します。
    Time m_ExecutionTime;

    // リストノード。
    ListNode m_ListNode;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
