﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#include <nn/pia/common/common_FixedRingBuffer.h>

#include <algorithm>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief  最新 N 個の値から、メジアン（中央値）を求めるためのクラスです。

 */
template <typename T, int N>
class LatestMedian : public common::RootObject
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    LatestMedian(void)
    {
        Clear();
    }


    /*!
    @brief デストラクタです。
     */
    ~LatestMedian(void)
    {
    }


    /*!
    @brief 要素をクリアします。コンストラクタ実行直後の状態に戻ります。
     */
    void Clear(void)
    {
        m_Values.Clear();
    }


    /*!
    @brief 最新の値を追加します。保持できる値は N 個のみなので、最も旧い値は捨てられます。
     */
    void Add(T value)
    {
        m_Values.PushBackForce(value);
    }


    /*!
    @brief メジアンを取得します。内部でソート処理などを実行しているので、頻繁に呼び出すのは推奨しません。
           要素が空の時に呼び出された場合は、 0 を返します。
     */
    T GetMedian(void) const
    {
        return GetMedian(N);
    }


    /*!
    @brief 直近の n 個の値を対象にしたメジアンを取得します。
           直近の値の個数が n 未満の場合は、それらの値から
           メジアンを算出します。
           内部でソート処理などを実行しているので、頻繁に呼び出すのは推奨しません。
           要素が空の時に呼び出された場合は、 0 を返します。
     */
    T GetMedian(uint32_t n) const
    {
        PIA_ASSERT(n <= N);

        // 要素数のチェック。
        uint32_t sz = m_Values.GetSize();
        PIA_ASSERT(sz <= N);
        if (sz == 0)
        {
            return 0;  // 要素が空のときは、 0 を返すと約束。
        }

        // 直近の値の個数が n 未満の場合は、それらの値からメジアンを算出します。
        uint32_t num = (sz < n) ? sz : n;  // num は今回使用するサンプリング値の個数。

        // 配列を別に用意し、ソートする。
        // 直近の値 num 個が欲しいので、配列の後ろから値を読み出していく。
        T buf[N];
        for (uint32_t i = 0; i < num; ++i)
        {
            uint32_t idx = sz - 1 - i;  // sz - 1は配列最後の添え字
            buf[i] = m_Values[idx];
        }
        std::sort(buf, buf + num);

        // 中央値の添え字を取得。要素数の偶数／奇数により、メジアンの算出方法は変わる。
        int midIdx = num / 2;
        if (num % 2 == 0)
        {
            return static_cast<T>(buf[midIdx - 1] + buf[midIdx]) / static_cast<T>(2);
        }
        else
        {
            return buf[midIdx];
        }
    }


    /*!
    @brief 要素が空であるかどうかを判定します。
     */
    bool IsEmpty(void) const
    {
        return m_Values.IsEmpty();
    }


    /*!
    @brief 要素が満杯であるかどうかを判定します。
     */
    bool IsFull(void) const
    {
        return m_Values.IsFull();
    }


    /*!
    @brief 要素数を返します。
     */
    int32_t GetSize(void) const
    {
        int32_t ret = static_cast<int32_t>(m_Values.GetSize());
        PIA_ASSERT(ret >= 0);
        return ret;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const
    {
        (void)flag;  // not implemented...
    }


private:
    FixedRingBuffer<T, N> m_Values;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
