﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief      Pia ライブラリ内部のメモリリークを自動検出するためのクラスです。

 */
class MemoryLeakChecker
{
public:
    /*!
    @brief コンストラクタです。
     */
    MemoryLeakChecker(void);


    /*!
    @brief デストラクタです。
     */
    ~MemoryLeakChecker(void);


    /*!
    @brief メモリリーク調査の起点をセットします。

    @details この API は、メモリリークを疑う Pia モジュールの初期化処理を開始する前に
             呼び出すことを想定しています。Pia は最初に common モジュールを初期化し、
             順に上位モジュールの初期化処理を実行していく設計となっておりますが、
             仮に Pia の xxx モジュールを含む上位モジュールでのメモリリークを疑う場合、
             nn::pia::xxx::Initialize() を呼び出す前にこの API を実行する必要があります。
    @details この API は、各モジュールの BeginSetup() ～ EndSetup() 間では
             呼び出してはいけません。

    @return 呼び出しが無事に完了すれば、成功の Result が返されます。
    @retval ResultNotInitialized   nn::pia::common::Initialize が呼ばれていません。[:progErr]
    @retval ResultInvalidState     このAPIを呼び出すタイミングが誤っています。 nn::pia::common::EndSetup() の後で呼び出す必要があります。[:progErr]

    @see Check
     */
    Result Set(void);


    /*!
    @brief メモリリークが発生したかどうかの判定を行います。

    @details この API は、メモリリークを疑う Pia モジュールの終了処理を終えた後に
             呼び出すことを想定しています。Pia は最上位のモジュールから順に
             終了処理を実行していく設計となっておりますが、
             仮に Pia の xxx モジュールを含む上位モジュールでのメモリリークを疑う場合、
             nn::pia::xxx::Finalize() を呼び出した後にこの API を実行する必要があります。
    @details このAPIは、各モジュールの BeginSetup() ～ EndSetup() 間では
             呼び出してはいけません。

    @return 呼び出しが無事に完了し、メモリリークの疑いが無ければ、成功の Result が返されます。
    @retval ResultMemoryLeak      メモリリークの疑いがあります。[:progErr]
    @retval ResultNotInitialized  nn::pia::common::Initialize() が呼ばれていません。[:progErr]
    @retval ResultInvalidState    このAPIを呼び出すタイミングが誤っています。 nn::pia::common::EndSetup() の後で呼び出す必要があります。[:progErr]

    @see Set
     */
    Result Check(void) const;


private:
    // 空きメモリの容量を保持。
    uint32_t m_FreeSize;

    NN_PIA_DISALLOW_COPY(MemoryLeakChecker);
};
}
}
}  // end of namespace nn::pia::common
