﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_OffsetList.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief  オブジェクトを格納するメモリを内部で管理する双方向リストです。

  @details リストを使用する前にSetBuffer()で利用するメモリ領域を指定する必要があります。
  <br>
  @param T    管理するオブジェクトの型。
 */
template <typename T>
class ObjList : public nn::pia::common::ListBase
{
protected:
    struct Node
    {
    public:
        Node()
            : m_Item()
        {
        }
        Node(const Node& n)
            : m_Item(n.m_Item)
        {
        }

    public:
        ListNode m_Link;
        T m_Item;

    private:
        Node& operator=(const Node&);
    };

    enum
    {
        NN_PIA_PRAGMA_PUSH_WARNINGS
        NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
        c_LinkOffset = offsetof(Node, m_Link),
        c_ItemOffset = offsetof(Node, m_Item)
        NN_PIA_PRAGMA_POP_WARNINGS
    };

public:
    /*!
    @brief  要素オブジェクトの型です。
     */
    typedef T ValueType;

    enum
    {
        /*!
        @brief  要素管理クラスのサイズです。

        @details c_NodeSize * (最大要素数) のサイズのバッファを使用します。
         */
        c_NodeSize = sizeof(Node)
    };


    /*!
    @brief  デフォルトコンストラクタです。
     */
    ObjList()
        : ListBase(), m_FreeList(), m_pBuffer(NULL), m_LimitNum(0)
    {
        NN_PIA_PRAGMA_PUSH_WARNINGS
        NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
        m_FreeList.InitOffset(offsetof(Node, m_Link));
        NN_PIA_PRAGMA_POP_WARNINGS
    }


    /*!
    @brief  必要なバッファのサイズを取得します。

    @return 必要なバッファのサイズ。
     */
    static uint32_t GetNeedBufferSize(uint32_t limitNum)
    {
        return limitNum * c_NodeSize;
    }


    /*!
    @brief  オブジェクトを格納するメモリを設定します。

    @details GetNeedBufferSize(limitNum) ( = limitNum * c_NodeSize ) バイトのメモリが必要です。
    <br>
    @param limitNum  リストに格納する最大要素数。
    @param pBuffer   リストが使用するメモリのアドレス。
     */
    void SetBuffer(uint32_t limitNum, void* pBuffer)
    {
        if (limitNum > 0)
        {
            if (pBuffer != NULL)
            {
                m_pBuffer = reinterpret_cast<Node*>(pBuffer);
                m_LimitNum = limitNum;
                for (uint32_t i = 0; i < limitNum; ++i)
                {
                    new (&m_pBuffer[i].m_Link) ListNode();
                    m_FreeList.PushBack(&m_pBuffer[i]);
                }
            }
            else
            {
                PIA_ASSERTMSG(false, "pBuf is null");
            }
        }
        else
        {
            PIA_ASSERTMSG(false, "limitNum[%d] must be larger than zero", limitNum);
        }
    }


    /*!
    @brief  バッファが設定されているかどうかを取得します。

    @return バッファが設定されていればtrue。
     */
    bool IsBufferReady() const
    {
        return m_pBuffer != NULL;
    }


    /*!
    @brief  リストに格納できる最大の要素数を取得します。

    @return リストに格納できる最大の要素数。
     */
    uint32_t MaxSize() const
    {
        return m_LimitNum;
    }


    /*!
    @brief  リストの要素数が最大値に達しているかを取得します。

    @return リストの要素数が最大値に達していればtrue。
     */
    bool IsFull() const
    {
        return m_FreeList.IsEmpty();
    }


    /*!
    @brief  すべての要素を削除します。

    @details 保持している要素すべてに対してデストラクタを呼びます。
     */
    void Clear()
    {
        ListNode* pNode = Terminator()->Next();
        while (pNode != Terminator())
        {
            ListNode* pN = pNode;
            pNode = pNode->Next();
            InitListNode(pN);
            m_FreeList.PushFront(ListNodeToNode(pN));
        }
        Init();
    }


    /*!
    @brief  要素を一つ作成し、リストの末尾に追加します。

    @details 保持できる要素数が上限に達していた場合は、release版ではNULLが返り、その他ではアサートにかかります。
    <br>
    @return 追加した要素。
     */
    T* BirthBack()
    {
        if (IsFull())
        {
            PIA_ASSERTMSG(false, "buffer full");
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        PushBackNode(&(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  要素を一つ作成し、リストの末尾に追加します。

    @details 保持できる要素数が上限に達していた場合は、NULLが返ります。
    <br>
    @return 追加した要素。
     */
    T* TryBirthBack()
    {
        if (IsFull())
        {
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        PushBackNode(&(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  要素を一つ作成し、リストの先頭に追加します。

    @details 保持できる要素数が上限に達していた場合は、release版ではNULLが返り、その他ではアサートにかかります。
    <br>
    @return 追加した要素。
     */
    T* BirthFront()
    {
        if (IsFull())
        {
            PIA_ASSERTMSG(false, "buffer full");
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        PushFrontNode(&(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  要素を一つ作成し、リストの先頭に追加します。

    @details 保持できる要素数が上限に達していた場合は、NULLが返ります。
    <br>
    @return 追加した要素。
     */
    T* TryBirthFront()
    {
        if (IsFull())
        {
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        PushFrontNode(&(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  要素を一つ作成し、指定した要素の前に追加します。

    @details 保持できる要素数が上限に達していた場合は、release版ではNULLが返り、その他ではアサートにかかります。
    <br>
    @param pBasis  基準となる要素。

    @return 追加した要素。
     */
    T* BirthBefore(const T* pBasis)
    {
        if (IsFull())
        {
            PIA_ASSERTMSG(false, "buffer full");
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        InsertBeforeNode(&(ObjToNode(pBasis)->m_Link), &(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  要素を一つ作成し、指定した要素の前に追加します。

    @details 保持できる要素数が上限に達していた場合は、NULLが返ります。
    <br>
    @param pBasis  基準となる要素。

    @return 追加した要素。
     */
    T* TryBirthBefore(const T* pBasis)
    {
        if (IsFull())
        {
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        InsertBeforeNode(&(ObjToNode(pBasis)->m_Link), &(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  要素を一つ作成し、指定した要素の後ろに追加します。

    @details 保持できる要素数が上限に達していた場合は、release版ではNULLが返り、その他ではアサートにかかります。
    <br>
    @param pBasis  基準となる要素。

    @return 追加した要素。
     */
    T* BirthAfter(const T* pBasis)
    {
        if (IsFull())
        {
            PIA_ASSERTMSG(false, "buffer full");
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        InsertAfterNode(&(ObjToNode(pBasis)->m_Link), &(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  要素を一つ作成し、指定した要素の後ろに追加します。

    @details 保持できる要素数が上限に達していた場合は、NULLが返ります。
    <br>
    @param pBasis  基準となる要素。

    @return 追加した要素。
     */
    T* TryBirthAfter(const T* pBasis)
    {
        if (IsFull())
        {
            return NULL;
        }
        Node* pNode = m_FreeList.PopBack();
        new (&pNode->m_Item) T();
        InsertAfterNode(&(ObjToNode(pBasis)->m_Link), &(pNode->m_Link));
        return &(pNode->m_Item);
    }


    /*!
    @brief  指定した要素を削除します。
            
    @details リストに含まれていない要素を指定した場合の挙動は不定です。
    <br>
    @param pObj    削除する要素。
     */
    void Erase(T* pObj)
    {
        Node* pNode = ObjToNode(pObj);
        pObj->~T();
        EraseNode(&(pNode->m_Link));
        m_FreeList.PushFront(pNode);
    }


    /*!
    @brief  先頭の要素を取得します。

    @return 先頭の要素。
            リストが空の場合はNULLを返します。
     */
    T* Front()
    {
        ListNode* pLink = FrontNode();
        if (pLink == NULL)
        {
            return NULL;
        }
        return &(ListNodeToNode(pLink)->m_Item);
    }


    /*!
    @brief  末尾の要素を取得します。

    @return 末尾の要素。
            リストが空の場合はNULLを返します。
     */
    T* Back()
    {
        ListNode* pLink = BackNode();
        if (pLink == NULL)
        {
            return NULL;
        }
        return &(ListNodeToNode(pLink)->m_Item);
    }


    /*!
    @brief  指定した要素の前の要素を取得します。
            
    @param pObj  基準となる要素。
                 リストに含まれていない要素を指定した場合の挙動は不定です。

    @return 指定した要素の前の要素。
            先頭の要素が指定された場合はNULLを返します。
     */
    T* Prev(const T* pObj)
    {
        ListNode* pPrevLink = ObjToNode(pObj)->m_Link.Prev();
        if (pPrevLink == Terminator())
        {
            return NULL;
        }
        return &(ListNodeToNode(pPrevLink)->m_Item);
    }


    /*!
    @brief  指定した要素の後ろの要素を取得します。
            
    @param pObj  基準となる要素。
                 リストに含まれていない要素を指定した場合の挙動は不定です。

    @return 指定した要素の後ろの要素。
            末尾の要素が指定された場合はNULLを返します。
     */
    T* Next(const T* pObj)
    {
        ListNode* pNextLink = ObjToNode(pObj)->m_Link.Next();
        if (pNextLink == Terminator())
        {
            return NULL;
        }
        return &(ListNodeToNode(pNextLink)->m_Item);
    }


    /*!
    @brief  要素がリストに含まれているか調べます。

    @param pObj 調べたい要素。

    @return 含まれていれば true を返します。
     */
    bool IsInclude(const T* pObj) const
    {
        return IsIncludeNode(&(ObjToNode(pObj)->m_Link));
    }


    /*!
    @brief  ObjListのイテレータです。
     */
    class Iterator
    {
        friend class ObjList<T>;

    private:
        Iterator(const ObjList<T>* pList, typename ObjList<T>::Node* pNode)
            : m_pList(pList), m_pNode(pNode)
        {
        }

    public:
        Iterator& operator++()
        {
            m_pNode = m_pList->ListNodeToNode(m_pNode->m_Link.Next());
            return *this;
        }

        bool operator==(const Iterator& x) const
        {
            return m_pNode == x.m_pNode;
        }
        bool operator!=(const Iterator& x) const
        {
            return m_pNode != x.m_pNode;
        }

        T& operator*()
        {
            return m_pNode->m_Item;
        }
        T* operator->()
        {
            return &(m_pNode->m_Item);
        }

    private:
        const ObjList<T>* m_pList;
        typename ObjList<T>::Node* m_pNode;
    };


    /*!
    @brief  ObjListのconst iteratorです。
     */
    class ConstIterator
    {
        friend class ObjList<T>;

    private:
        ConstIterator(const ObjList<T>* pList, typename ObjList<T>::Node* pNode)
            : m_pList(pList), m_pNode(pNode)
        {
        }

    public:
        ConstIterator& operator++()
        {
            m_pNode = m_pList->ListNodeToNode(m_pNode->m_Link.Next());
            return *this;
        }

        bool operator==(const ConstIterator& x) const
        {
            return m_pNode == x.m_pNode;
        }
        bool operator!=(const ConstIterator& x) const
        {
            return m_pNode != x.m_pNode;
        }

        const T& operator*() const
        {
            return m_pNode->m_Item;
        }
        const T* operator->() const
        {
            return &(m_pNode->m_Item);
        }

    private:
        const ObjList<T>* m_pList;
        typename ObjList<T>::Node* m_pNode;
    };


    /*!
    @brief  ObjListのrobust iteratorです。
            
    @details イテレータが指し示す要素を削除してもイテレータを継続できます。
            要素の追加やイテレータが指し示していない要素の削除をした場合はイテレータを継続できません。
     */
    class RobustIterator
    {
        friend class ObjList<T>;

    private:
        RobustIterator(const ObjList<T>* pList, typename ObjList<T>::Node* pNode)
            : m_pList(pList), m_pNode(pNode)
        {
            m_pNextLink = m_pNode->m_Link.Next();
        }

    public:
        RobustIterator& operator++()
        {
            m_pNode = m_pList->ListNodeToNode(m_pNextLink);
            m_pNextLink = m_pNode->m_Link.Next();
            return *this;
        }

        bool operator==(const RobustIterator& x) const
        {
            return m_pNode == x.m_pNode;
        }
        bool operator!=(const RobustIterator& x) const
        {
            return m_pNode != x.m_pNode;
        }

        T& operator*()
        {
            return m_pNode->m_Item;
        }
        T* operator->()
        {
            return &(m_pNode->m_Item);
        }

    private:
        const ObjList<T>* m_pList;
        typename ObjList<T>::Node* m_pNode;
        ListNode* m_pNextLink;
    };


    /*!
    @brief  先頭の要素を指すIteratorを取得します。

    @return 先頭の要素を指すIterator。
     */
    Iterator Begin() const
    {
        return Iterator(this, ListNodeToNode(Terminator()->Next()));
    }


    /*!
    @brief  末尾の要素の次を指すIteratorを取得します。

    @return 末尾の要素の次を指すIterator。
     */
    Iterator End() const
    {
        return Iterator(this, ListNodeToNode(Terminator()));
    }


    /*!
    @brief  先頭の要素を指すConstIteratorを取得します。

    @return 先頭の要素を指すConstIterator。
     */
    ConstIterator ConstBegin() const
    {
        return ConstIterator(this, ListNodeToNode(Terminator()->Next()));
    }


    /*!
    @brief  末尾の要素の次を指すConstIteratorを取得します。

    @return 末尾の要素の次を指すConstIterator。
     */
    ConstIterator ConstEnd() const
    {
        return ConstIterator(this, ListNodeToNode(Terminator()));
    }


    /*!
    @brief  先頭の要素を指すRobustIteratorを取得します。

    @return 先頭の要素を指すRobustIterator。
     */
    RobustIterator RobustBegin() const
    {
        return RobustIterator(this, ListNodeToNode(Terminator()->Next()));
    }


    /*!
    @brief  末尾の要素の次を指すRobustIteratorを取得します。

    @return 末尾の要素の次を指すRobustIterator。
     */
    RobustIterator RobustEnd() const
    {
        return RobustIterator(this, ListNodeToNode(Terminator()));
    }

private:
    static Node* ObjToNode(const T* pObj)
    {
        return reinterpret_cast<Node*>(PtrUtil::addOffset(pObj, -c_ItemOffset));
    }

    static Node* ListNodeToNode(const ListNode* pListNode)
    {
        return reinterpret_cast<Node*>(PtrUtil::addOffset(pListNode, -c_LinkOffset));
    }


private:
    OffsetList<Node> m_FreeList;
    Node* m_pBuffer;
    uint32_t m_LimitNum;

    ObjList(const ObjList&);
    ObjList& operator=(const ObjList&);
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
