﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_PtrUtil.h>
#include <nn/pia/common/common_ListBase.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief  要素のメンバにListNodeを持たせるタイプの双方向リストのクラスです。
          
  @details リストを利用する前に、ListNodeが要素の中のどこにあるのかを以下のように
          設定する必要があります。
          
          class Node 
          {
          public:
              static int32_t GetListNodeOffset() {return offsetof(Node, m_ListNode);}
          private:
              ListNode m_ListNode;
          };

          OffsetList<Node> list;
          list.InitOffset(Node::GetListNodeOffset());

  @param T    管理するオブジェクトの型。

 */
template <typename T>
class OffsetList : public nn::pia::common::ListBase
{
public:
    typedef T Node;


    /*!
    @brief  デフォルトコンストラクタです。
     */
    OffsetList(void)
        : ListBase(), m_Offset(-1)
    {
    }


    /*!
    @brief  Tの先頭アドレスからListNodeへのオフセット値を設定します。
            
    @details 使用する前に必ず呼び出す必要があります。
    <br>
    @param offset    設定するオフセット。
     */
    void InitOffset(int32_t offset)
    {
        m_Offset = offset;
    }


    /*!
    @brief  リストからすべての要素を削除します。
     */
    void Clear()
    {
        ClearNode();
    }


    /*!
    @brief  リストの末尾に要素を追加します。

    @param pObj  追加する要素。
     */
    void PushBack(T* pObj)
    {
        PushBackNode(ObjToListNode(pObj));
    }


    /*!
    @brief  リストの先頭に要素を追加します。

    @param pObj  追加する要素。
     */
    void PushFront(T* pObj)
    {
        PushFrontNode(ObjToListNode(pObj));
    }


    /*!
    @brief  リストの末尾から要素を取り出して返します。

    @return 取り出した要素。
            リストが空の時はNULLを返します。
     */
    T* PopBack()
    {
        return ListNodeToObjWithNullCheck(PopBackNode());
    }


    /*!
    @brief  リストの先頭から要素を取り出して返します。

    @return 取り出した要素。
            リストが空の時はNULLを返します。
     */
    T* PopFront()
    {
        return ListNodeToObjWithNullCheck(PopFrontNode());
    }


    /*!
    @brief  指定した要素の前に要素を追加します。

    @param pBasis  基準となる要素。
                   リストに含まれていない要素を指定した場合の挙動は不定です。
    @param pObj    追加する要素。
     */
    void InsertBefore(const T* pBasis, T* pObj)
    {
        InsertBeforeNode(ObjToListNode(pBasis), ObjToListNode(pObj));
    }


    /*!
    @brief  指定した要素の後ろに要素を追加します。

    @param pBasis  基準となる要素。
                   リストに含まれていない要素を指定した場合の挙動は不定です。
    @param pObj    追加する要素。
     */
    void InsertAfter(const T* pBasis, T* pObj)
    {
        InsertAfterNode(ObjToListNode(pBasis), ObjToListNode(pObj));
    }


    /*!
    @brief  指定した要素を削除します。
            
    @details リストに含まれていない要素を指定した場合の挙動は不定です。
    <br>
    @param pObj    削除する要素。
     */
    void Erase(T* pObj)
    {
        EraseNode(ObjToListNode(pObj));
    }


    /*!
    @brief  先頭の要素を取得します。

    @return 先頭の要素。
            リストが空の場合はNULLを返します。
     */
    T* Front() const
    {
        return ListNodeToObjWithNullCheck(FrontNode());
    }


    /*!
    @brief  末尾の要素を取得します。

    @return 末尾の要素。
            リストが空の場合はNULLを返します。
     */
    T* Back() const
    {
        return ListNodeToObjWithNullCheck(BackNode());
    }


    /*!
    @brief  指定した要素の前の要素を取得します。
            
    @param pObj  基準となる要素。
                 リストに含まれていない要素を指定した場合の挙動は不定です。

    @return 指定した要素の前の要素。
            先頭の要素が指定された場合はNULLを返します。
     */
    T* Prev(const T* pObj) const
    {
        ListNode* pPrev = ObjToListNode(pObj)->Prev();
        if (pPrev == Terminator())
        {
            return NULL;
        }
        else
        {
            return ListNodeToObj(pPrev);
        }
    }


    /*!
    @brief  指定した要素の後ろの要素を取得します。
            
    @param pObj  基準となる要素。
                 リストに含まれていない要素を指定した場合の挙動は不定です。

    @return 指定した要素の後ろの要素。
            末尾の要素が指定された場合はNULLを返します。
     */
    T* Next(const T* pObj) const
    {
        ListNode* pNext = ObjToListNode(pObj)->Next();
        if (pNext == Terminator())
        {
            return NULL;
        }
        else
        {
            return ListNodeToObj(pNext);
        }
    }


    /*!
    @brief  要素がリストに含まれているか調べます。

    @param pObj 調べたい要素。

    @return 含まれていれば true を返します。
     */
    bool IsInclude(const T* pObj) const
    {
        return IsIncludeNode(ObjToListNode(pObj));
    }


    /*!
    @brief  指定した要素が先頭の要素となるようにリストを回転します。

    @param pObj  先頭にする要素
                 リストに含まれていない要素を指定した場合の挙動は不定です。
     */
    void Rotate(const T* pObj)
    {
        RotateNode(ObjToListNode(pObj));
    }


    /*!
    @brief  OffsetListのイテレータです。
     */
    class Iterator
    {
        friend class OffsetList<T>;

    private:
        Iterator(const OffsetList<T>* pList, T* pObj)
            : m_pList(pList), m_pObj(pObj)
        {
        }

    public:
        Iterator& operator++()
        {
            m_pObj = m_pList->ListNodeToObj(m_pList->ObjToListNode(m_pObj)->Next());
            return *this;
        }

        bool operator==(const Iterator& x) const
        {
            return m_pObj == x.m_pObj;
        }
        bool operator!=(const Iterator& x) const
        {
            return m_pObj != x.m_pObj;
        }

        T& operator*()
        {
            return *m_pObj;
        }
        T* operator->()
        {
            return m_pObj;
        }

        const OffsetList<T>* GetList() const
        {
            return m_pList;
        }

    private:
        const OffsetList<T>* m_pList;
        T* m_pObj;
    };


    /*!
    @brief  OffsetListのconst iteratorです。
     */
    class ConstIterator
    {
        friend class OffsetList<T>;

    private:
        ConstIterator(const OffsetList<T>* pList, const T* pObj)
            : m_pList(pList), m_pObj(pObj)
        {
        }

    public:
        ConstIterator& operator++()
        {
            m_pObj = m_pList->ListNodeToObj(m_pList->ObjToListNode(m_pObj)->Next());
            return *this;
        }

        bool operator==(const ConstIterator& x) const
        {
            return m_pObj == x.m_pObj;
        }
        bool operator!=(const ConstIterator& x) const
        {
            return m_pObj != x.m_pObj;
        }

        const T& operator*()
        {
            return *m_pObj;
        }
        const T* operator->()
        {
            return m_pObj;
        }

        const OffsetList<T>* GetList() const
        {
            return m_pList;
        }

    private:
        const OffsetList<T>* m_pList;
        const T* m_pObj;
    };


    /*!
    @brief  OffsetListのrobust iteratorです。
            
    @details イテレータが指し示す要素を削除してもイテレータを継続できます。
            要素の追加やイテレータが指し示していない要素の削除をした場合はイテレータを継続できません。
     */
    class RobustIterator
    {
        friend class OffsetList<T>;

    private:
        RobustIterator(const OffsetList<T>* pList, T* pObj)
            : m_pList(pList), m_pObj(pObj)
        {
            m_pNextNode = m_pList->ObjToListNode(pObj)->Next();
        }

    public:
        RobustIterator& operator++()
        {
            m_pObj = m_pList->ListNodeToObj(m_pNextNode);
            m_pNextNode = m_pList->ObjToListNode(m_pObj)->Next();
            return *this;
        }

        bool operator==(const RobustIterator& x) const
        {
            return m_pObj == x.m_pObj;
        }
        bool operator!=(const RobustIterator& x) const
        {
            return m_pObj != x.m_pObj;
        }

        T& operator*()
        {
            return *m_pObj;
        }
        T* operator->()
        {
            return m_pObj;
        }

        const OffsetList<T>* GetList() const
        {
            return m_pList;
        }

    private:
        const OffsetList<T>* m_pList;
        T* m_pObj;
        ListNode* m_pNextNode;
    };


    /*!
    @brief  先頭の要素を指すIteratorを取得します。

    @return 先頭の要素を指すIterator。
     */
    Iterator Begin() const
    {
        return Iterator(this, ListNodeToObj(Terminator()->Next()));
    }


    /*!
    @brief  末尾の要素の次を指すIteratorを取得します。

    @return 末尾の要素の次を指すIterator。
     */
    Iterator End() const
    {
        return Iterator(this, ListNodeToObj(Terminator()));
    }


    /*!
    @brief  先頭の要素を指すConstIteratorを取得します。

    @return 先頭の要素を指すConstIterator。
     */
    ConstIterator ConstBegin() const
    {
        return ConstIterator(this, ListNodeToObj(Terminator()->Next()));
    }


    /*!
    @brief  末尾の要素の次を指すConstIteratorを取得します。

    @return 末尾の要素の次を指すConstIterator。
     */
    ConstIterator ConstEnd() const
    {
        return ConstIterator(this, ListNodeToObj(Terminator()));
    }


    /*!
    @brief  先頭の要素を指すRobustIteratorを取得します。

    @return 先頭の要素を指すRobustIterator。
     */
    RobustIterator RobustBegin() const
    {
        return RobustIterator(this, ListNodeToObj(Terminator()->Next()));
    }


    /*!
    @brief  末尾の要素の次を指すRobustIteratorを取得します。

    @return 末尾の要素の次を指すRobustIterator。
     */
    RobustIterator RobustEnd() const
    {
        return RobustIterator(this, ListNodeToObj(Terminator()));
    }


private:
    int32_t m_Offset;

    ListNode* ObjToListNode(const T* pObj) const
    {
        return reinterpret_cast<ListNode*>(PtrUtil::addOffset(pObj, m_Offset));
    }

    T* ListNodeToObj(const ListNode* pNode) const
    {
        return reinterpret_cast<T*>(PtrUtil::addOffset(pNode, -m_Offset));
    }

    T* ListNodeToObjWithNullCheck(const ListNode* pNode) const
    {
        if (pNode)
            return reinterpret_cast<T*>(PtrUtil::addOffset(pNode, -m_Offset));
        else
            return NULL;
    }

    OffsetList(const OffsetList&);
    OffsetList& operator=(const OffsetList&);
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
