﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_TreeMap.h>
#include <nn/pia/common/common_PtrUtil.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
    @cond PRIVATE
    @brief  要素のメンバにTreeMapNodeを持たせるタイプのツリーマップのクラスです。
 */
template <typename Key_, typename Value_>
class OffsetTreeMap : public TreeMap<Key_>
{
public:
    typedef Key_ Key;
    typedef Value_ Value;
    typedef TreeMapNode<Key> Node;


    /*!
    @brief  デフォルトコンストラクタです。
     */
    OffsetTreeMap()
        : TreeMap<Key>(), m_Offset(-1)
    {
    }


    /*!
    @brief  Value の先頭アドレスから TreeMapNode へのオフセット値を設定します。
    @details    使用する前に必ず呼び出す必要があります。
    @param[in]  offset  設定するオフセット。
     */
    void InitOffset(int32_t offset)
    {
        m_Offset = offset;
    }


    /*!
    @brief  すべての要素を削除します。
     */
    void Clear()
    {
        TreeMap<Key>::ClearNode();
    }


    /*!
    @brief  指定したキーに対応する要素を検索します。
    @param[in]  key     検索するキー。
    @return キーに対応する要素です。存在しない場合はNULLを返します。
     */
    Value* Find(const Key& key) const
    {
        return NodeToValueWithNullCheck(TreeMap<Key>::FindNode(key));
    }


    /*!
    @brief  指定したキーに対応する要素を追加します。
    @details    同じキーの要素が既に存在する場合は、要素を置き換えます。
    @param[in]  key     追加するキー。
    @param[in]  pValue  追加する要素。
    @return 要素が置き換えられた場合は、置き換える前の古い要素を返します。その他の場合はNULLを返します。
     */
    Value* Insert(const Key& key, Value* pValue)
    {
        return NodeToValueWithNullCheck(TreeMap<Key>::InsertNode(key, ValueToNode(pValue)));
    }


    /*!
    @brief  指定したキーの要素を削除します。
    @details    指定したキーの要素が存在しなければ、削除は行われません。
    @param[in]  key     削除するキー。
    @return 要素が削除された場合は、削除された要素を返します。削除されなかった場合はNULLを返します。
     */
    Value* Erase(const Key& key)
    {
        return NodeToValueWithNullCheck(TreeMap<Key>::EraseNode(key));
    }


    /*!
    @brief  指定した要素を削除します。
    @details    このTreeMapの要素でない物を指定した場合の挙動は不定です。
    @param[in]  pValue  削除する要素。
     */
    void Erase(Value* pValue)
    {
        TreeMap<Key>::EraseNode(ValueToNode(pValue));
    }


    /*!
    @brief  先頭の要素を取得します。
    @return 先頭の要素です。空の場合はNULLを返します。
     */
    Value* Front() const
    {
        return NodeToValueWithNullCheck(TreeMap<Key>::FrontNode());
    }


    /*!
    @brief  最後の要素を取得します。
    @return 最後の要素です。
     */
    Value* Back() const
    {
        return NodeToValueWithNullCheck(TreeMap<Key>::BackNode());
    }


    /*!
    @brief  指定した要素の１つ前の要素を取得します。
    @details    このTreeMapの要素でない物を指定した場合の挙動は不定です。
    @param[in]  pValue  基準とする要素。
    @return １つ前の要素です。先頭の要素を指定した場合はNULLを返します。
     */
    Value* Prev(const Value* pValue) const
    {
        return NodeToValueWithNullCheck(ValueToNode(pValue)->Prev());
    }


    /*!
    @brief  指定した要素の１つ後の要素を取得します。
    @details    このTreeMapの要素でない物を指定した場合の挙動は不定です。
    @param[in]  pValue  基準とする要素。
    @return １つ後の要素です。最後の要素を指定した場合はNULLを返します。
     */
    Value* Next(const Value* pValue) const
    {
        return NodeToValueWithNullCheck(ValueToNode(pValue)->Next());
    }


    /*!
    @brief  指定した要素がこのTreeMapに含まれているか調べます。
    @param[in]  pValue  調べる要素です。
    @return このTreeMapの要素ならtrueを返します。
     */
    bool IsInclude(const Value* pValue) const
    {
        return TreeMap<Key>::IsIncludeNode(ValueToNode(pValue));
    }


    /*!
    @brief  イテレータです。
     */
    class Iterator
    {
        friend class OffsetTreeMap;

    private:
        Iterator(const OffsetTreeMap* pTreeMap, Node* pNode)
            : m_pTreeMap(pTreeMap), m_pNode(pNode)
        {
            m_pValue = m_pTreeMap->NodeToValueWithNullCheck(pNode);
        }

    public:
        Iterator& operator++()
        {
            m_pNode = m_pNode->Next();
            m_pValue = m_pTreeMap->NodeToValueWithNullCheck(m_pNode);
            return *this;
        }

        bool operator==(const Iterator& x) const
        {
            return m_pValue == x.m_pValue;
        }
        bool operator!=(const Iterator& x) const
        {
            return m_pValue != x.m_pValue;
        }

        Value& operator*()
        {
            return *m_pValue;
        }
        Value* operator->()
        {
            return m_pValue;
        }

    private:
        const OffsetTreeMap* m_pTreeMap;
        Value* m_pValue;
        Node* m_pNode;
    };


    /*!
    @brief  constイテレータです。
     */
    class ConstIterator
    {
        friend class OffsetTreeMap;

    private:
        ConstIterator(const OffsetTreeMap* pTreeMap, const Node* pNode)
            : m_pTreeMap(pTreeMap), m_pNode(pNode)
        {
            m_pValue = m_pTreeMap->NodeToValueWithNullCheck(pNode);
        }

    public:
        ConstIterator& operator++()
        {
            m_pNode = m_pNode->Next();
            m_pValue = m_pTreeMap->NodeToValueWithNullCheck(m_pNode);
            return *this;
        }

        bool operator==(const ConstIterator& x) const
        {
            return m_pValue == x.m_pValue;
        }
        bool operator!=(const ConstIterator& x) const
        {
            return m_pValue != x.m_pValue;
        }

        const Value& operator*()
        {
            return *m_pValue;
        }
        const Value* operator->()
        {
            return m_pValue;
        }

    private:
        const OffsetTreeMap* m_pTreeMap;
        const Value* m_pValue;
        const Node* m_pNode;
    };


    /*!
    @brief  robustイテレータです。
    @details    イテレータが指し示す要素を削除してもイテレータを継続できます。
                要素の追加やイテレータが指し示していない要素の削除をした場合はイテレータを継続できません。
     */
    class RobustIterator
    {
        friend class OffsetTreeMap;

    private:
        RobustIterator(const OffsetTreeMap* pTreeMap, Node* pNode)
            : m_pTreeMap(pTreeMap)
        {
            m_pValue = m_pTreeMap->NodeToValueWithNullCheck(pNode);
            m_pNextNode = (pNode != NULL) ? pNode->Next() : NULL;
        }

    public:
        RobustIterator& operator++()
        {
            if (m_pNextNode != NULL)
            {
                m_pValue = m_pTreeMap->NodeToValue(m_pNextNode);
                m_pNextNode = m_pNextNode->Next();
            }
            else
            {
                m_pValue = NULL;
            }
            return *this;
        }

        bool operator==(const RobustIterator& x) const
        {
            return m_pValue == x.m_pValue;
        }
        bool operator!=(const RobustIterator& x) const
        {
            return m_pValue != x.m_pValue;
        }

        Value& operator*()
        {
            return *m_pValue;
        }
        Value* operator->()
        {
            return m_pValue;
        }

    private:
        const OffsetTreeMap* m_pTreeMap;
        Value* m_pValue;
        Node* m_pNextNode;
    };


    /*!
    @brief  先頭の要素を指すIteratorを取得します。

    @return 先頭の要素を指すIterator。
     */
    Iterator Begin() const
    {
        return Iterator(this, TreeMap<Key>::FrontNode());
    }


    /*!
    @brief  末尾の要素の次を指すIteratorを取得します。

    @return 末尾の要素の次を指すIterator。
     */
    Iterator End() const
    {
        return Iterator(this, NULL);
    }


    /*!
    @brief  指定したキーの値以上で最小のキーを持つ要素を指すIteratorを取得します。

    @return 指定したキーの値以上で最小のキーを持つ要素を指すIterator。
     */
    Iterator LowerBound(const Key& key) const
    {
        return Iterator(this, TreeMap<Key>::LowerBoundNode(key));
    }


    /*!
    @brief  先頭の要素を指すConstIteratorを取得します。

    @return 先頭の要素を指すConstIterator。
     */
    ConstIterator ConstBegin() const
    {
        return ConstIterator(this, TreeMap<Key>::FrontNode());
    }


    /*!
    @brief  末尾の要素の次を指すConstIteratorを取得します。

    @return 末尾の要素の次を指すConstIterator。
     */
    ConstIterator ConstEnd() const
    {
        return ConstIterator(this, NULL);
    }


    /*!
    @brief  指定したキーの値以上で最小のキーを持つ要素を指すConstIteratorを取得します。

    @return 指定したキーの値以上で最小のキーを持つ要素を指すConstIterator。
     */
    ConstIterator ConstLowerBound(const Key& key) const
    {
        return ConstIterator(this, TreeMap<Key>::LowerBoundNode(key));
    }


    /*!
    @brief  先頭の要素を指すRobustIteratorを取得します。

    @return 先頭の要素を指すRobustIterator。
     */
    RobustIterator RobustBegin() const
    {
        return RobustIterator(this, TreeMap<Key>::FrontNode());
    }


    /*!
    @brief  末尾の要素の次を指すRobustIteratorを取得します。

    @return 末尾の要素の次を指すRobustIterator。
     */
    RobustIterator RobustEnd() const
    {
        return RobustIterator(this, NULL);
    }


    /*!
    @brief  指定したキーの値以上で最小のキーを持つ要素を指すRobustIteratorを取得します。

    @return 指定したキーの値以上で最小のキーを持つ要素を指すRobustIterator。
     */
    RobustIterator RobustLowerBound(const Key& key) const
    {
        return RobustIterator(this, TreeMap<Key>::LowerBoundNode(key));
    }

private:
    int32_t m_Offset;

    Node* ValueToNode(const Value* pObj) const
    {
        return reinterpret_cast<Node*>(PtrUtil::addOffset(pObj, m_Offset));
    }

    Value* NodeToValue(const Node* pNode) const
    {
        return reinterpret_cast<Value*>(PtrUtil::addOffset(pNode, -m_Offset));
    }

    Value* NodeToValueWithNullCheck(const Node* pNode) const
    {
        if (pNode)
            return reinterpret_cast<Value*>(PtrUtil::addOffset(pNode, -m_Offset));
        else
            return NULL;
    }

    OffsetTreeMap(const OffsetTreeMap&);
    OffsetTreeMap& operator=(const OffsetTreeMap&);
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
