﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <stdint.h>

namespace nn
{
namespace pia
{
namespace common
{

/*!
  @cond PRIVATE
  @brief  ポインタ演算ユーティリティです。
 */
class PtrUtil
{
public:
    /*!
    @brief  アライメントが合うように繰り上げる（2の累乗限定で高速バージョン）

    @param      ptr         アライメントさせたいポインタ
    @param      alignment   アライメントの指定（2の累乗でなければならない。単位はByte）
    @return                 アライメント後のポインタ
     */
    static void* roundUpPow2(const void* ptr, uint32_t alignment)
    {
        // 処理削減のためにASSERTもしない
        return reinterpret_cast<void*>(
            (reinterpret_cast<uintptr_t>(ptr) + (alignment - 1)) & ~(static_cast<uintptr_t>(alignment - 1)));
    }

    /*! 
    @brief  アライメントが合うように繰り上げる（2の累乗でなくてもよい）

    @param      ptr         アライメントさせたいポインタ
    @param      alignment   アライメントの指定（2の累乗でなくてもよい。単位はByte）
    @return                 アライメント後のポインタ
     */
    static void* roundUpN(const void* ptr, uint32_t alignment)
    {
        return reinterpret_cast<void*>(
            ((reinterpret_cast<uintptr_t>(ptr) + alignment - 1) / alignment) * alignment);
    }

    /*! 
    @brief  アライメントが合うように繰り下げる（2の累乗限定で高速バージョン）

    @param      ptr         アライメントさせたいポインタ
    @param      alignment   アライメントの指定（2の累乗でなければならない。単位はByte）
    @return                 アライメント後のポインタ
     */
    static void* roundDownPow2(const void* ptr, uint32_t alignment)
    {
        // 処理削減のためにASSERTもしない
        return reinterpret_cast<void*>(reinterpret_cast<uintptr_t>(ptr) & ~(static_cast<uintptr_t>(alignment - 1)));
    }

    /*! 
    @brief  アライメントが合うように繰り下げる（2の累乗でなくてもよい）

    @param      ptr         アライメントさせたいポインタ
    @param      alignment   アライメントの指定（2の累乗でなくてもよい。単位はByte）
    @return                 アライメント後のポインタ
     */
    static void* roundDownN(const void* ptr, uint32_t alignment)
    {
        return reinterpret_cast<void*>(
            (reinterpret_cast<uintptr_t>(ptr) / alignment) * alignment);
    }

    /*! 
    @brief  アドレスにオフセットを足す

    @param      ptr         アドレス
    @param      offset      足す値
    @return                 計算後の値
     */
    static void* addOffset(const void* ptr, ptrdiff_t offset)
    {
        return reinterpret_cast<void*>(static_cast<uintptr_t>(reinterpret_cast<uintptr_t>(ptr) + offset));
    }

    /*! 
    @brief  アドレスとアドレスの差を計算する(a - b)

    @param      a           アドレス
    @param      b           アドレス
    @return                 差
     */
    static ptrdiff_t diff(const void* a, const void* b)
    {
        return reinterpret_cast<ptrdiff_t>(a) - reinterpret_cast<ptrdiff_t>(b);
    }

    /*! 
    @brief  範囲内かどうかを調べる

    @param      ptr         調べるアドレス
    @param      begin       開始アドレス
    @param      end         終了アドレス
    @return                 範囲内ならtrue
     */
    static bool isInclude(const void* ptr, const void* begin, const void* end)
    {
        return (reinterpret_cast<uintptr_t>(ptr) >= reinterpret_cast<uintptr_t>(begin) && reinterpret_cast<uintptr_t>(ptr) < reinterpret_cast<uintptr_t>(end));
    }

    /*! 
    @brief  アライメントされているかどうかを調べる（2の累乗でなくてもよい）

    @details isAlignedN()と同等の機能です。
    @param      ptr         調べるアドレス
    @param      alignment   アライメント
    @return                 アライメントされていればtrue
     */
    static bool isAligned(const void* ptr, int alignment)
    {
        PIA_ASSERT(alignment != 0);
        // マイナスのalignmentでも大丈夫なはず。
        // 余りはC99以前は処理系依存だが、割り切れる場合には0が返されるしかないはず
        return (reinterpret_cast<uintptr_t>(ptr) % alignment == 0);
    }

    /*! 
    @brief  アライメントされているかどうかを調べる（2の累乗限定で高速バージョン）

    @param      ptr         調べるアドレス
    @param      alignment   アライメント
    @return                 アライメントされていればtrue
     */
    static bool isAlignedPow2(const void* ptr, uint32_t alignment)
    {
        // 処理削減のためにASSERTもしない
        return ((reinterpret_cast<uintptr_t>(ptr) & (alignment - 1)) == 0);
    }

    /*! 
    @brief  アライメントされているかどうかを調べる（2の累乗でなくてもよい）

    @param      ptr         調べるアドレス
    @param      alignment   アライメント
    @return                 アライメントされていればtrue
     */
    static bool isAlignedN(const void* ptr, int alignment)
    {
        PIA_ASSERT(alignment != 0);
        // マイナスのalignmentでも大丈夫なはず。
        // 余りはC99以前は処理系依存だが、割り切れる場合には0が返されるしかないはず
        return (reinterpret_cast<uintptr_t>(ptr) % alignment == 0);
    }
};
///@endcond
}
}
}  // end of namespace nn::pia::common
