﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_RootObject.h>
#include <nn/pia/common/common_Time.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief      アプリ開発者に伝えるレポートをプリントするクラスです。

  @details    レポートにはレベルを設定することができ、レベル毎にログの
              表示／非表示を切り替えることができます。

 */
class Report : public RootObject
{
public:
    // ログレベルのビットマスク。
    static const uint32_t LevelInfo = 0x00000001 << 0;     //!< Pia 内部の詳細情報レベル。
    static const uint32_t LevelWarning = 0x00000001 << 1;  //!< 警告レベル（修正すべきだが無視可能）。
    static const uint32_t LevelError = 0x00000001 << 2;    //!< エラーレベル（復帰可能）。

    //n1589:IsFlagSetチェックのコードを最短化
    /*!
    @cond PRIVATE
     */
    static bool IsFlagSetStatic(uint64_t flag)
    {
#if NN_PIA_RELEASE
        NN_PIA_UNUSED_IN_RELEASE(flag);
        return false;
#else
        if (GetGlobalObject() == NULL)
        {
            return false;
        }

        return GetGlobalObject()->IsFlagSet(flag);
#endif
    }
    //! @endcond

    /*!
    @brief コンストラクタです。ログレベルのビットマスクは全てのビットが1の状態でオブジェクトが構築されます。
     */
    Report(void);

    /*!
    @brief コンストラクタです。引数でログレベルのマスクを指定できます。

    @param[in]    mask    ログレベルのビットマスク。
     */
    explicit Report(uint32_t mask);

    /*!
    @brief デストラクタです。
     */
    ~Report();

    /*!
    @brief マスクを設定します。特定のレベルのログ表示を抑制したりすることができます。

    @details この API により、出力されるログを絞り込むことができます。

    @param[in]    mask    ログレベルのビットマスクの論理和。
     */
    void SetMask(uint32_t mask)
    {
        m_mask = mask;
    }

    /*!
    @brief    特定のフラグが設定されているかどうかを返します。

    @details  なお、 IsFlasSet ( TraceFlagNever ) の結果は true となります。
    <br>
    @return   特定のフラグが設定されていれば true 、そうでなければ false を返します。

    @param[in]    flag    問い合わせるトレースフラグ。
    */
    bool IsFlagSet(uint64_t flag) const
    {
#if NN_PIA_RELEASE
        NN_PIA_UNUSED_IN_RELEASE(flag);
        return false;
#else
        return (m_mask & flag) == flag;
#endif
    }

    /*!
    @cond PRIVATE
    @brief コンソールに文字列を書き出します。可変引数に対応しています。

    @param[in]    mask    ログレベルのビットマスク。
    @param[in]    pStr    C標準ライブラリの printf 引数と同様の書式を与えます。
     */
    void Write(uint32_t mask, const char* pStr, ...);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief コンソールにキャプション文字列を書き出します。

    @param[in]    mask    ログレベルのビットマスク。
     */
    void WriteCaption(uint32_t mask);
    //! @endcond


    /*!
    @brief 時刻表示の ON / OFF を切り替えます。
    @details ON の時は "[Pia:12345] ..." のように基準時刻からの経過時間をミリ秒単位で表示します。
    @param[in] isPrint  true なら時刻を表示します。
     */
    void SetPrintTime(bool isPrint)
    {
        m_IsPrintTime = isPrint;
    }


    /*!
    @brief 時刻表示の ON / OFF を取得します。
    @return 時刻表示が ON になっていれば true を返します。
     */
    bool IsPrintTime() const
    {
        return m_IsPrintTime;
    }


    /*!
    @brief 時刻表示の基準時刻を現在の時刻にします。
     */
    void ResetBaseTime()
    {
        m_BaseTime.SetNow();
    }


    /*!
    @brief グローバルログオブジェクトを登録します。

    @details 引数で指定した Report オブジェクトが、グローバルログオブジェクトとして登録されます。
           PIA_REPORT() マクロは、ここで登録された Report オブジェクトを利用してログ出力を行います。

    @param[in]    pReport    登録するReportオブジェクトへのポインタ。
     */
    static void SetGlobalObject(Report* pReport)
    {
        s_pReport = pReport;
    }

    /*!
    @brief SetGlobalObject() で指定された Report オブジェクトへのポインタが返されます。

    @details nn::pia::common::Initialize() を呼ぶと、内部で暗黙的に Report インスタンスが作成され、
           そのインスタンスがグローバルログオブジェクトとしてセットされます。

    @return グローバルログオブジェクトとして登録されているインスタンスへのポインタ。
     */
    static Report* GetGlobalObject(void)
    {
        return s_pReport;
    }


private:
    // グローバルオブジェクトを指すポインタ。
    static Report* s_pReport;

    // マスク設定は Report インスタンス毎に保持します。
    uint32_t m_mask;

    Time m_BaseTime;
    bool m_IsPrintTime;

    NN_PIA_DISALLOW_COPY(Report);
};


/*!
  @brief 簡単にログを出力するために用意したマクロです。

  @details このマクロでは、グローバルとしてセットした Report オブジェクトが利用されます。
         また、自動的に接頭辞と改行が付加されます。
  @details リリースビルドにおいては、このマクロは何もしません。
  
    @param[in]    mask    ログのビットマスク。
 */
#if NN_PIA_UNDER_DEVELOP
#define PIA_REPORT(mask, ...)                                                       \
    \
{                                                                         \
        ::nn::pia::common::Report* pPiaReport = ::nn::pia::common::Report::GetGlobalObject(); \
        if (pPiaReport)                                                             \
        {                                                                        \
            pPiaReport->WriteCaption(mask);                                         \
            pPiaReport->Write(mask, __VA_ARGS__);                                   \
            pPiaReport->Write(mask, "\n");                                          \
        }                                                                        \
    \
}
#else
#define PIA_REPORT(mask, ...) \
    {                      \
    }
#endif

/*!
  @brief ファイル名と関数名を含めたログ出力を行うマクロです。

  @details このマクロでは、グローバルとしてセットしたReportオブジェクトが利用されます。
           また、自動的に接頭辞と改行が付加されます。
  @details リリースビルドにおいては、このマクロは何もしません。

  @param[in]    mask    ログのビットマスク。
 */
#if NN_PIA_UNDER_DEVELOP
#define PIA_REPORT_EX(mask, ...)                                                               \
    \
{                                                                                    \
        nn::pia::common::Report* pPiaReport = nn::pia::common::Report::GetGlobalObject();            \
        if (pPiaReport)                                                                        \
        {                                                                                   \
            pPiaReport->WriteCaption(mask);                                                    \
            pPiaReport->Write(mask, "%s %s:", PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_FUNC); \
            pPiaReport->Write(mask, __VA_ARGS__);                                              \
            pPiaReport->Write(mask, "\n");                                                     \
        }                                                                                   \
    \
}
#else
#define PIA_REPORT_EX(mask, ...) \
    {                         \
    }
#endif

// ログ出力と Result の return を統合したマクロです。
// ログのレベルは常に LEVEL_ERROR なので、このマクロに渡す Result は
// プログラミングエラー相当のものに限定します。
// common モジュールの初期化前は PIA_BASIC_PRINT されます。
// リリースビルドにおいては、単に Result を return するだけとなります。
#if NN_PIA_UNDER_DEVELOP
#define PIA_RETURN_RESULT_WITH_REPORT(result, ...)                                                                     \
    \
{                                                                                                            \
        ::nn::pia::common::Report* pPiaReport = ::nn::pia::common::Report::GetGlobalObject();                                    \
        if (pPiaReport)                                                                                                \
        {                                                                                                           \
            uint32_t logLevelError_ = ::nn::pia::common::Report::LevelError;                                             \
            pPiaReport->WriteCaption(logLevelError_);                                                                  \
            pPiaReport->Write(logLevelError_, "[%s] %s %s:", #result, PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_FUNC); \
            pPiaReport->Write(logLevelError_, __VA_ARGS__);                                                            \
            pPiaReport->Write(logLevelError_, "\n");                                                                   \
        }                                                                                                           \
        else                                                                                                        \
        {                                                                                                           \
            PIA_BASIC_PRINT("[%s] %s %s:", #result, PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_FUNC);        \
            PIA_BASIC_PRINT(__VA_ARGS__);                                                                   \
            PIA_BASIC_PRINT("\n");                                                                          \
        }                                                                                                           \
        PIA_RETURN_RESULT(result);                                                                                  \
    \
}
#else
#define PIA_RETURN_RESULT_WITH_REPORT(result, ...) \
    {                                           \
        return result();                        \
    }
#endif
}
}
}  // end of namespace nn::pia::common
