﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_OffsetList.h>
#include <nn/pia/common/common_Time.h>
#include <nn/pia/common/common_CriticalSection.h>


namespace nn
{
namespace pia
{
namespace common
{

class Time;
class Job;
class BackgroundScheduler;

/*!
  @brief      ジョブのスケジューラです。

  @details    スケジューラは、ジョブのクラスで定義されている処理を非同期に
              実行するためのオブジェクトです。
              なお、このクラスはスレッドセーフではありません。

 */
class Scheduler : public nn::pia::common::RootObject
{
public:
/*!
    @brief バックグラウンドスケジューラ用スレッドの優先度のデフォルト値です。
     */
#if NN_PIA_CTR
    static const int32_t DefaultBackgroundThreadPriority = 24;
#elif NN_PIA_WIN
    static const int32_t DefaultBackgroundThreadPriority = THREAD_PRIORITY_BELOW_NORMAL;
#elif NN_PIA_CAFE
    static const int32_t DefaultBackgroundThreadPriority = 24;
#elif NN_PIA_NINTENDOSDK
    static const int32_t DefaultBackgroundThreadPriority = 24;
#elif NN_PIA_A
    //n1849
    //10    THREAD_PRIORITY_BACKGROUND  バックグラウンド処理に最適な優先度です。
    //0     THREAD_PRIORITY_DEFAULT     アプリのスレッドとして標準的な優先度です
    //-8    THREAD_PRIORITY_URGENT_DISPLAY  システムが画面描画やInputDevice情報を処理するための設定値です。非常に優先度が高いため、通常、アプリ開発者は使用しません
    static const int32_t DefaultBackgroundThreadPriority = 10;
#elif NN_PIA_B
    //todo:n3389:とりあえずな値
    static const int32_t DefaultBackgroundThreadPriority = 10;
#else
#error "invalid platform";
#endif


    /*!
    @brief インスタンスを作成します（シングルトンパターン）。

    @details このAPIは、 nn::pia::common::BeginSetup() を呼び出してから、
           nn::pia::common::EndSetup() を呼び出すまでの間に呼び出す必要があります。

    @param[in] backgroundThreadPriority    バックグラウンドスケジューラ用スレッドの優先度です。 Pia の各 API を呼び出すスレッドより低い優先度を指定する必要があります。

    @return インスタンスの作成に成功すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    @retval ResultNotInitialized     commonモジュールが初期化されていません。[:progErr]
    @retval ResultInvalidState       BeginSetup() ～ EndSetup() 間で呼び出されていません。[:progErr]
    @retval ResultInvalidArgument    引数が間違っています。[:progErr]
     */
    static Result CreateInstance(int32_t backgroundThreadPriority = DefaultBackgroundThreadPriority);


    /*!
    @brief インスタンスを破棄します（シングルトンパターン）。

    @details インスタンスが作成されていないときにこの関数が呼び出された場合は、
           何も実行せずに関数から返ります。
     */
    static void DestroyInstance();


    /*!
    @brief  Scheduler インスタンスへのポインタを取得します（シングルトンパターン）。

    @return インスタンスが作成されていないときは、 NULL ポインタが返ります。
     */
    static Scheduler* GetInstance()
    {
        return s_pInstance;
    }


    /*!
    @brief ジョブをディスパッチします。

    @details この関数を呼び出すことでジョブが実行されます。
    <br>
    @param[in] timeout ジョブ実行の上限となる、タイムアウト時間を指定します。単位はミリ秒です。
                       ただし、必ずこのタイムアウト時間以下に収まることを保証するものではありません。
                       ゼロを与えると、無制限の時間を指定したと解釈されます。
     */
    void Dispatch(uint32_t timeout = 0);


    /*!
    @cond PRIVATE
    @brief ジョブを登録します。
           
    @details この関数は Job から呼ばれますので、他から呼び出してはいけません。
     */
    void EntryJob(Job* pJob, bool isBackground);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 次の Dispatch() まで待機させるジョブを登録します。
           
    @details この関数は Job から呼ばれますので、他から呼び出してはいけません。
     */
    void EntryJobNext(Job* pJob);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ジョブを削除します。
           
    @details この関数は Job から呼ばれますので、他から呼び出してはいけません。
     */
    void ResetJob(Job* pJob);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Dispatch() が呼ばれたタイミングの Time です。
     */
    const Time& GetDispatchedTime() const
    {
        return m_DispatchedTime;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ジョブ関連の CriticalSection を取得します。
     */
    CriticalSection* GetCriticalSection()
    {
        return &m_CriticalSection;
    }
    //! @endcond
    /*!
    @cond PRIVATE
    @brief 待機中のジョブ数を取得します。
     */
    uint32_t GetJobNum() const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief モニタリングデータをセットします。
     */
    void SetMonitoringData();
    //! @endcond


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

private:
    /*!
    @brief スケジューラのコンストラクタです。
     */
    explicit Scheduler(int32_t backgroundThreadPriority);

    /*!
    @brief スケジューラのデストラクタです。
     */
    virtual ~Scheduler();


    Scheduler(const Scheduler&);
    Scheduler& operator=(const Scheduler&);

    // シングルトンインスタンスへのポインタです。
    static Scheduler* s_pInstance;


    //! ジョブリスト型
    typedef OffsetList<Job> JobList;

    //! ジョブリスト
    JobList m_JobList;
    JobList m_CountBasedJobList;

    //! Dispatch時刻
    Time m_DispatchedTime;

    //! Dispatch 呼び出し回数
    uint32_t m_DispatchCalls;

    mutable CriticalSection m_CriticalSection;

    BackgroundScheduler* m_pBackgroundScheduler;
};
}
}
}  // end of namespace nn::pia::common
