﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief  署名の設定情報を保持するクラスです。

 */
class SignatureSetting
{
public:
    static const uint32_t SignatureKeyDataSizeMax = 32;  //!< 署名に用いられる鍵データの最大バイト数です。

    /*!
    @brief 署名アルゴリズムの種類を表します。
     */
    enum Mode
    {
        Mode_Nothing = 0,  //!< 署名無し
        Mode_HmacMd5       //!< HMAC-MD5 による署名
    };


    /*!
    @brief  コンストラクタです。署名アルゴリズムと鍵を指定して初期化します。

    @param[in] mode         署名アルゴリズムの種類を指定します。
    @param[in] cpKeyData    鍵データのアドレスを設定します。このインスタンスが無くなるか再設定されるまで、参照先のデータの寿命を維持する必要があります。
    @param[in] keySize      鍵データのサイズ（バイト数）を指定します。
     */
    SignatureSetting(Mode mode, const void* cpKeyData, uint32_t keySize);


    /*!
    @brief  コンストラクタです。鍵を指定して既定のアルゴリズム（HMAC-MD5）で初期化します。

    @param[in] cpKeyData    鍵データのアドレスを設定します。このインスタンスが無くなるか再設定されるまで、参照先のデータの寿命を維持する必要があります。
    @param[in] keySize      鍵データのサイズ（バイト数）を指定します。
     */
    SignatureSetting(const void* cpKeyData, uint32_t keySize);


    /*!
    @brief  コンストラクタです。署名なしとして初期化します。
     */
    SignatureSetting()
        : m_Mode(Mode_Nothing), m_cpKeyData(NULL), m_KeySize(0)
    {
    }

    /*!
    @brief  デストラクタです。
    */
    virtual ~SignatureSetting()
    {
    }

    /*!
    @brief  署名の設定を指定します。

    @param[in] mode         署名アルゴリズムの種類を指定します。
    @param[in] cpKeyData    鍵データのアドレスを設定します。このインスタンスが無くなるか再設定されるまで、参照先のデータの寿命を維持する必要があります。
    @param[in] keySize      鍵データのサイズ（バイト数）を指定します。

    @return 正しく設定された場合、成功を表す Result が返されます。
    @retval ResultInvalidArgument   引数の指定が不正です。[:progErr]
     */
    Result Set(Mode mode, const void* cpKeyData, uint32_t keySize);


    /*!
    @brief  指定されている署名アルゴリズムを取得します。
    @return 指定されている署名アルゴリズムです。
     */
    Mode GetMode() const
    {
        return m_Mode;
    }


    /*!
    @brief  指定されている鍵データのアドレスを取得します。
    @return 指定されている鍵データのアドレスです。
     */
    const void* GetKeyData() const
    {
        return m_cpKeyData;
    }


    /*!
    @brief  指定されている鍵データのサイズを取得します。
    @return 指定されている鍵データのサイズ（バイト数）です。
     */
    uint32_t GetKeySize() const
    {
        return m_KeySize;
    }

    virtual void Trace(uint64_t flag) const;

    /*!
    @brief  署名なしを表す署名設定クラスのインスタンスです。
     */
    static const SignatureSetting NoSignature;

private:
    Mode m_Mode;
    const void* m_cpKeyData;
    uint32_t m_KeySize;
};
}
}
}  // end of namespace nn::pia::common
