﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_SignatureSetting.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief  署名鍵のバッファを内部に持つ署名設定情報のクラスです。

  @param  KeySize   鍵のサイズ（バイト数）です。このサイズのバッファが用意されます。

 */
template <uint32_t KeySize>
class SignatureSettingWithKeyBuffer : public SignatureSetting
{
public:
    /*!
    @brief  コンストラクタです。署名アルゴリズムを指定して初期化します。

    @details 鍵データは GetKeyBufferPtr() で得られるアドレスに書き込んでください。

    @param[in] mode 署名アルゴリズムの種類を指定します。
     */
    explicit SignatureSettingWithKeyBuffer(Mode mode)
        : SignatureSetting(mode, m_KeyBuffer, KeySize)
    {
        ClearKeyBuffer();
    }


    /*!
    @brief  コンストラクタです。既定の署名アルゴリズム（HMAC-MD5）で初期化します。

    @details 鍵データは GetKeyBufferPtr() で得られるアドレスに書き込んでください。
     */
    SignatureSettingWithKeyBuffer()
        : SignatureSetting(Mode_HmacMd5, m_KeyBuffer, KeySize)
    {
        ClearKeyBuffer();
    }


    /*!
    @brief  鍵データバッファへのポインタを取得します。

    @return 鍵データバッファへのポインタです。
     */
    uint8_t* GetKeyBufferPtr()
    {
        return m_KeyBuffer;
    }


    /*!
    @brief  署名アルゴリズムを指定します。

    @param[in] mode 署名アルゴリズムの種類を指定します。

    @return 正しく設定された場合、成功を表す Result が返されます。
    @retval ResultInvalidArgument   引数の指定が不正です。[:progErr]
     */
    Result SetMode(Mode mode)
    {
        return Set(mode, m_KeyBuffer, KeySize);
    }


    // 鍵データバッファの内容をクリアします。
    void ClearKeyBuffer()
    {
        for (uint32_t i = 0; i < KeySize; ++i)
        {
            m_KeyBuffer[i] = 0;
        }
    }


private:
    // 使われると m_cpKeyData が m_KeyBuffer を指さなくなるので、外から呼べなくしておきます。
    // SignatureSetting* としてアクセスしたりすると呼べてしまいますが。
    Result Set(Mode mode, const void* cpKeyData, uint32_t keySize)
    {
        return SignatureSetting::Set(mode, cpKeyData, keySize);
    }

private:
    uint8_t m_KeyBuffer[KeySize];
};
}
}
}  // end of namespace nn::pia::common
