﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief      サイズ固定の単純なコンテナです。
 */
template <typename T, int N>
class SimpleContainer : public nn::pia::common::RootObject
{
public:
    typedef T* Iterator;             //!< 非constのイテレータ型です。
    typedef const T* ConstIterator;  //!<   constのイテレータ型です。

    /*!
    @brief    コンテナ先頭にあるインスタンスに対応するイテレータを得ます（const版）。
              コンテナ内に有効な要素が一つも無ければ、Endが返すものと同じイテレータが返されます。
    @return   先頭にあるインスタンスのイテレータを返します。
    @see      End
     */
    ConstIterator Begin(void) const
    {
        return &(m_Buffer[0]);  // 要素が一つも無いとき、End()は&(m_Buffer[0])を返すので、これで十分。
    }


    /*!
    @brief    コンテナ先頭にあるインスタンスに対応するイテレータを得ます（非const版）。
              有効な要素が一つも無ければ、Endが返すものと同じイテレータが返されます。
    @return   先頭にあるインスタンスのイテレータを返します。
    @see      End
     */
    Iterator Begin(void)
    {
        return &(m_Buffer[0]);  // 要素が一つも無いとき、End()は&(m_Buffer[0])を返すので、これで十分。
    }


    /*!
    @brief    最後尾のインスタンスの次に対応するイテレータを得ます（const版）。
    @return   最後尾のインスタンスの次に対応するイテレータを得ます。
    @see      Begin
     */
    ConstIterator End(void) const
    {
        return &(m_Buffer[m_Elements]);
    }


    /*!
    @brief    最後尾の次のインスタンスに対応するイテレータを得ます（非const版）。
    @return   最後尾の次のインスタンスに対応するイテレータを得ます。
    @see      Begin
     */
    Iterator End(void)
    {
        return &(m_Buffer[m_Elements]);
    }


    /*!
    @brief コンストラクタです。
     */
    SimpleContainer(void)
        : m_Elements(0),
          m_Buffer()
    {
    }


    /*!
    @brief コピーコンストラクタです。
     */
    SimpleContainer(const SimpleContainer& rhs)
    {
        *this = rhs;
    }


    /*!
    @brief デストラクタです。
     */
    virtual ~SimpleContainer(void)
    {
    }


    /*!
    @brief コンテナに要素を追加します。

    @return   追加に成功すれば、IsSuccess()がtrueを返すResultが返されます。
    @retval  ResultBufferIsFull コンテナが満杯のため、これ以上要素を追加することができません。
    @see IsFull, Clear, Erase
     */
    Result Add(const T& t)
    {
        if (IsFull())
        {
            PIA_RETURN_RESULT(ResultBufferIsFull);
        }
        else
        {
            m_Buffer[m_Elements++] = t;
            return ResultSuccess();
        }
    }


    /*!
    @brief イテレータで指定された要素をコンテナから取り除きます。
           取り除く要素の後ろにあった要素は、すべて前へ詰められます。
           そのため、要素のサイズが大きい場合や、先頭付近の要素を
           取り除く場合は、大きめのCPU処理コストが発生します。

    @param[in] it 本コンテナから得られた、有効なイテレータ。
    @return       消去した要素の次の要素を指すイテレータ。削除したのが最後尾の要素であれば、End()と同じイテレータが返されます。
    @see Add
     */
    Iterator Erase(Iterator it)
    {
        if ((Begin() <= it) && (it < End()))
        {
            // 後続の要素を詰める。
            for (Iterator i = it; (i + 1) != End(); ++i)
            {
                *i = *(i + 1);
            }
            PIA_ASSERT(m_Elements > 0);
            --m_Elements;
            // 前に詰めたので、「消した場所＝消した場所の次にあった要素」となる。
            // 最後尾の要素を消した場合は、めでたくEnd()と同じになる。
            return it;
        }
        else
        {
            // 無効なイテレータ（範囲外の可能性）。何もしない…。
            return End();
        }
    }

    /*!
    @brief コンテナ内の要素を全て削除します。

    @see IsFull, Clear
     */
    void Clear(void)
    {
        m_Elements = 0;
    }


    /*!
    @brief コンテナが空であるかどうかを判定します。

    @return   空であればtrue、何か一つでも要素があればfalseが返されます。
    @see IsFull, GetSize
     */
    bool IsEmpty(void) const
    {
        return m_Elements == 0;
    }


    /*!
    @brief コンテナが満杯であるかどうかを判定します。

    @return   これ以上要素を追加することができない状態であればtrue、そうでなければfalseが返されます。
    @see IsEmpty, GetSize
     */
    bool IsFull(void) const
    {
        return m_Elements == N;
    }


    /*!
    @brief コンテナ内の要素数を返します。

    @return   コンテナに含まれている有効な要素の数を返します。
    @see IsEmpty, IsFull
     */
    int32_t GetSize(void) const
    {
        return m_Elements;
    }


    /*!
    @brief 代入演算子です。

    @return 本インスタンスへの参照が返ります。
     */
    SimpleContainer& operator=(const SimpleContainer& rhs)
    {
        m_Elements = rhs.m_Elements;

        for (int i = 0; i < N; ++i)
        {
            m_Buffer[i] = rhs.m_Buffer[i];
        }

        return *this;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const
    {
        PIA_TRACE(flag, "----- SimpleContainer::Trace() called. -----");
        PIA_TRACE(flag, "Number of elements: %d", m_Elements);
        PIA_TRACE(flag, "----- SimpleContainer::Trace() end.    -----");
    }


private:
    // コンテナに含まれる、有効な要素の数。
    int32_t m_Elements;

    // 要素の実体。
    T m_Buffer[N];
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
