﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once


#include <nn/pia/common/common_Definitions.h>

#include <nn/pia/common/common_InetAddress.h>


namespace nn
{
namespace pia
{
namespace common
{

/*!
  @brief      ステーションを一意に識別するためのアドレスです。
 */
class StationAddress : public nn::pia::common::RootObject
{
public:
    static const uint32_t SerializeSizeMax = common::InetAddress::SerializeSizeMax;  // m_Extension

    /*!
    @brief コンストラクタです。
     */
    StationAddress(void);


    /*!
    @brief デストラクタです。
     */
    virtual ~StationAddress(void)
    {
    }


    /*!
    @brief コピーコンストラクタです。
     */
    StationAddress(const StationAddress& rhs);

    /*!
    @brief nn::pia::common::InetAddress の const 参照を得ます。

    @return    nn::pia::common::InetAddress の const 参照が返ります。
     */
    const nn::pia::common::InetAddress& GetInetAddress(void) const
    {
        return m_Address;
    }


    /*!
    @brief nn::pia::common::InetAddress の参照を得ます。

    @return    nn::pia::common::InetAddress の参照が返ります。
     */
    nn::pia::common::InetAddress& GetInetAddress(void)
    {
        return m_Address;
    }


    /*!
    @brief nn::pia::common::InetAddress を設定します。

    @param[in] addr 設定するインターネットアドレス。
    @return    この関数は必ず成功します。
     */
    Result SetInetAddress(const nn::pia::common::InetAddress& addr);


    /*!
    @brief      StationAddress の内容をクリアします。
     */
    void Clear(void);


    /*!
    @brief  オブジェクトをシリアライズしたデータの長さを得ます。単位はバイトです。

    @return シリアライズされたデータの長さが返ります。単位はバイトです。
    @see    Serialize, Deserialize
     */
    uint32_t GetSerializedSize(void) const;


    /*!
    @brief      オブジェクトをシリアライズします。

    @param[out] pBuffer    シリアライズされたデータを書き込むバッファを指すポインタ。
    @param[out] pDataLen   シリアライズされたデータの長さが書き込まれます。単位はバイトです。
    @param[in]  bufferSize pBuffer で指定されたバッファのサイズを指定します。
    @return     成功すれば、IsSuccess() が true を返す Result が返ります。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval     ResultInvalidArgument 引数が誤っています（ NULL ポインタであるなど ）。pBuffer, bufferSize で指定されたバッファのサイズが不足していた場合も、このエラーが返されます。[:progErr]
    @see        Deserialize, GetSerializedSize
     */
    Result Serialize(uint8_t* pBuffer, uint32_t* pDataLen, uint32_t bufferSize) const;


    /*!
    @brief     シリアライズされたデータから、オブジェクトを復元します。

    @param[in] pData シリアライズされたデータを指すポインタ。

    @return    成功すれば、IsSuccess() が true を返す Result が返ります。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval    ResultInvalidArgument  引数が誤っています（ NULL ポインタであるなど）。[:progErr]
    @see       Serialize, GetSerializedSize
     */
    Result Deserialize(const uint8_t* pData);


    /*!
    @cond PRIVATE
    @brief      有効なアドレスがセットされているかどうかを判定します。

    @return     有効なアドレスであれば true が、無効なアドレスであれば false が返されます。
     */
    bool IsValid(void) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      無効化します。
    */
    void Invalidate()
    {
        m_Address.Invalidate();
    }
    //! @endcond

    /*!
    @brief 等値演算子です。二つの StationAddress オブジェクトが同等と見なせるかどうかを判定します。

    @return 両者の内容が同等であれば true を、そうでなければ false を返します。
     */
    bool operator==(const StationAddress& rhs) const;


    /*!
    @brief 等値演算子です。二つの StationAddress オブジェクトが異なるかどうかを判定します。

    @return 両者の内容が異なれば true を、そうでなければ false を返します。
     */
    bool operator!=(const StationAddress& rhs) const
    {
        return !(*this == rhs);
    }


    /*!
    @brief 代入演算子です。

    @return 本オブジェクトの参照が返ります。
     */
    StationAddress& operator=(const StationAddress& rhs);


    /*!
    @brief 比較演算子です。

    @return このオブジェクトの方が小さければ true を返します。
     */
    bool operator<(const StationAddress& rhs) const;


    /*!
    @brief 比較演算子です。

    @return このオブジェクトの方が大きければ true を返します。
     */
    bool operator>(const StationAddress& rhs) const;


    /*!
    @brief 大小比較関数です。
    @param[in] a 比較するオブジェクトです。
    @param[in] b 比較するオブジェクトです。
    @return a より b が小さければ -1 、大きければ 1 、等しければ 0 を返します。
     */
    static int Compare(const StationAddress& a, const StationAddress& b);

    /*!
    @cond PRIVATE
    @brief アドレスの文字列を取得します。

    @param[in] pString 文字列クラス。
     */
    void GetAddressString(String* pString) const;
    //! @endcond

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

private:
    common::InetAddress m_Address;
};
}
}
}  // end of namespace nn::pia::common
