﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#include <nn/pia/common/common_RootObject.h>
#include <nn/pia/common/common_Job.h>
#include <nn/pia/common/common_Trace.h>


namespace nn
{
namespace pia
{
namespace common
{

#define NN_PIA_JOBSTEP(STEP) Step(static_cast<StepMethod>(&STEP), #STEP)


/*!
@cond PRIVATE
  @brief      ステップ実行ジョブの基底クラスです。

  @details このクラスを用いると、ジョブのメンバ関数の単位で処理を
              実行することができます。
 */
class StepSequenceJob : public nn::pia::common::Job
{
public:
    typedef Job::ExecuteResult (StepSequenceJob::*StepMethod)();

    class Step : public nn::pia::common::RootObject
    {
    public:
        Step(void)
            : m_pStepMethod(NULL), m_pName(NULL)
        {
        }
        Step(StepMethod pStepMethod, const char* pName)
            : m_pStepMethod(pStepMethod), m_pName(pName)
        {
        }

        virtual ~Step(void)
        {
        }

        const char* GetName(void) const
        {
            return m_pName ? m_pName : "(no name)";
        }

        Job::ExecuteResult Execute(StepSequenceJob* pJob)
        {
            return (pJob->*m_pStepMethod)();
        }

        bool operator==(const Step& rhs) const;
        bool operator!=(const Step& rhs) const
        {
            return !(*this == rhs);
        }

        virtual void Trace(uint64_t flag) const
        {
            PIA_TRACE(flag, "StepSequenceJob::Step::Trace() called. (%s)", GetName());
        }

    private:
        StepMethod m_pStepMethod;
        const char* m_pName;
    };


    /*!
    @brief コンストラクタです。
     */
    StepSequenceJob(void);


    /*!
    @brief デストラクタです。
     */
    virtual ~StepSequenceJob(void);


    virtual void Reset(bool isCheck = true);


    void CancelStepSequence()
    {
        m_IsCancelStepSequence = true;
    }

    virtual void CancelCleanup()
    {
    }

    /*!
    @brief 逐次実行する関数を登録します。

    @details 関数をバッチ的に実行していくためのセッティングを行う関数です。
    <br>
    @param[in] step 次回に実行する関数が内包されたStepインスタンス。
                    通常、このインスタンスは NN_PIA_JOBSTEPマクロで作成します。
     */
    void SetStep(const Step& step)
    {
        m_NextStep = step;
    }

    void WaitForCompletion(uint32_t sleepTime);

    /*!
    @brief トレースフラグをセットします。

    @param[in] flags トレースフラグ。
     */
    void SetTraceFlags(uint64_t flags)
    {
        m_TraceFlags = flags;
    }


    /*!
    @brief トレースフラグを取得します。
     */
    uint64_t GetTraceFlags(void) const
    {
        return m_TraceFlags;
    }

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

protected:
    // オーバーライド。
    virtual Job::ExecuteResult ExecuteCore();

    // SetStep() で登録されているインスタンスを返します。
    const Step& GetStep(void) const
    {
        return m_NextStep;
    }

private:
    // 次回に実行されるジョブが内包されるインスタンス。
    Step m_NextStep;

    volatile bool m_IsCancelStepSequence;

    // 本インスタンスのトレースフラグ。
    uint64_t m_TraceFlags;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
