﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{
/*!
  @brief  文字列を表すクラスです。

 */
class String : public nn::pia::common::RootObject
{
public:
    /*!
    @brief 文字コードの種類を表します。
     */
    enum EncodingType
    {
        EncodingType_Unknown = 0,  //!< 不明な文字コードです
        EncodingType_Utf8,         //!< UTF-8
        EncodingType_Utf16         //!< UTF-16
    };

    /*!
    @brief デフォルトコンストラクタです。
     */
    String()
        : m_pBuffer(NULL), m_BufferSize(0), m_Bytes(0), m_EncodingType(EncodingType_Unknown)
    {
    }

    /*!
    @brief      デストラクタです。
     */
    virtual ~String()
    {
    }

    /*!
    @brief UTF-8 の文字列から構築するコンストラクタです。

    @param[in] pStr 文字列のポインタです。
     */
    String(const char* pStr);

    /*!
    @brief UTF-16 の文字列から構築するコンストラクタです。

    @param[in] pStr 文字列のポインタです。
     */
    String(const charUtf16_t* pStr);

    /*!
    @brief 文字列バッファのポインタを char 型で取得します。

    @return 文字列バッファのポインタ。
     */
    const char* GetCharArray() const;

    /*!
    @brief 等値演算子です。二つの String が同等と見なせるかどうかを判定します。

    @param[in] rhs  比較する文字列です。

    @return 両者の内容が同等であれば true を、そうでなければ false を返します。
     */
    bool operator==(const String& rhs) const;


    /*!
    @brief 等値演算子です。二つの String が異なるかどうかを判定します。

    @param[in] rhs  比較する文字列です。

    @return 両者の内容が異なれば true を、そうでなければ false を返します。
     */
    bool operator!=(const String& rhs) const
    {
        return !(*this == rhs);
    }

    /*!
    @brief 書式に従って文字列を構築します。

    @param[in] pFormatString    フォーマット文字列です。
    @param[in] varg             可変引数リストです。

    @return 書き込み後の文字列の長さです。
     */
    int32_t FormatV(const char* pFormatString, va_list varg);


    /*!
    @brief 書式に従って文字列を構築します。

    @param[in] pFormatString    フォーマット文字列です。
    @param[in] ...              可変引数です。

    @return 書き込み後の文字列の長さです。
     */
    int32_t Format(const char* pFormatString, ...);

    /*!
    @brief 文字列の長さを取得します。

    @return 文字列の長さを返します。
     */
    uint32_t GetStringLength() const;

    /*!
    @brief 文字列バッファのポインタを取得します。

    @return 文字列バッファのポインタ。
     */
    const void* GetBuffer() const;

    /*!
    @brief 文字列のバイト長を取得します。

    @return 文字列のバイト長を返します。
     */
    uint32_t GetBytes() const;

    /*!
    @brief 文字列の文字コードを取得します。

    @return 文字列の文字コードを返します。
     */
    EncodingType GetEncodingType() const;

    /*!
    @brief 文字列をコピーします。

    @param[in] rhs  コピーする String です。

    @return 成功すれば、IsSuccess() が true を返す Result が返ります。
    @retval ResultInvalidArgument 引数が誤っています。コピー先の文字列バッファが不足している、コピー元の文字列が長すぎる、ヌル文字列をコピーしようとした、文字コードが不正などの場合に、このエラーが返ります。
     */
    virtual Result Copy(const String& rhs);

    /*!
    @brief 文字列をクリアします。
     */
    virtual void Clear();

    /*!
    @brief デバッグに有用な情報をプリントします。
 
    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

protected:
    virtual Result SetString(const void* pStr, uint32_t byteSize, EncodingType type);
    uint32_t CalcBytes(const void* pStr, EncodingType type);
    bool IsSameString(const String& rhs) const;

    const void* m_pBuffer;
    uint32_t m_BufferSize;
    uint32_t m_Bytes;
    EncodingType m_EncodingType;

private:
    NN_PIA_DISALLOW_COPY(String);
};
}
}
}  // end of namespace nn::pia::common
