﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_TimeSpan.h>

namespace nn
{
namespace pia
{
namespace common
{

/*!
  @brief      時刻を表すクラスです。

 */
class Time : public RootObject
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    Time()
        : m_Tick(0)
    {
    }


    explicit Time(uint64_t tick)
        : m_Tick(tick)
    {
    }

    /*!
    @brief コピーコンストラクタです。

    @param[in] rhs 本インスタンスを初期化する値を持つ Time インスタンスです。
     */
    Time(const Time& rhs)
        : m_Tick(rhs.m_Tick)
    {
    }


    /*!
    @brief このオブジェクトが現在の時刻を表すようにします。
     */
    void SetNow();


    /*!
    @brief 現在の時刻を表す Time インスタンスを取得します。

    @return 現在の時刻を表す Time インスタンスです。
     */
    static Time GetTime()
    {
        Time time;
        time.SetNow();
        return time;
    }


    /*!
    @brief 代入演算子です。

    @param[in] rhs 代入する Time です。

    @return 本インスタンスへの参照が返されます。
     */
    Time& operator=(const Time& rhs)
    {
        m_Tick = rhs.m_Tick;
        return *this;
    }


    /*!
    @brief 本インスタンスの時刻に時間を加算します。

    @param[in] span 加算する時間です。

    @return 本インスタンスへの参照が返されます。
     */
    Time& operator+=(const TimeSpan& span)
    {
        m_Tick += span.m_TickSpan;
        return *this;
    }


    /*!
    @brief 本インスタンスの時刻に時間を減算します。

    @param[in] span 減算する時間です。

    @return 本インスタンスへの参照が返されます。
     */
    Time& operator-=(const TimeSpan& span)
    {
        m_Tick -= span.m_TickSpan;
        return *this;
    }


    /*!
    @brief 本インスタンスの時刻に時間を加算した値をもつ Time を計算します。

    @param[in] span 加算する時間です。

    @return 加算された時刻を表す Time です。
     */
    Time operator+(const TimeSpan& span) const
    {
        Time time;
        time.m_Tick = m_Tick + span.m_TickSpan;
        return time;
    }


    /*!
    @brief 本インスタンスの時刻に時間を減算した値をもつ Time を計算します。

    @param[in] span 減算する時間です。

    @return 減算された時刻を表す Time です。
     */
    Time operator-(const TimeSpan& span) const
    {
        Time time;
        time.m_Tick = m_Tick - span.m_TickSpan;
        return time;
    }


    /*!
    @brief 時刻の差分を計算します。

    @param[in] rhs 基準となる時刻です。

    @return 時刻の差分を表す TimeSpan です。
     */
    TimeSpan operator-(const Time& rhs) const
    {
        TimeSpan span;
        span.m_TickSpan = m_Tick - rhs.m_Tick;
        return span;
    }


    /*!
    @brief 等値演算子です。二つの Time オブジェクトが同等と見なせるかどうかを判定します。

    @param[in] rhs 比較する Time です。

    @return 両者の表す時刻が同じであれば true を、そうでなければ false を返します。
     */
    bool operator==(const Time& rhs) const
    {
        return (m_Tick == rhs.m_Tick);
    }


    /*!
    @brief 等値演算子です。二つの Time オブジェクトが異なるかどうかを判定します。

    @param[in] rhs 比較する Time です。

    @return 両者の表す時刻が異なれば true を、そうでなければ false を返します。
     */
    bool operator!=(const Time& rhs) const
    {
        return (m_Tick != rhs.m_Tick);
    }


    /*!
    @brief 比較演算子です。本インスタンスの方が古い時刻を表していれば true を返します。

    @param[in] rhs 比較する Time です。

    @return 本インスタンスの方が古い時刻を表していれば true を、そうでなければ false を返します。
     */
    bool operator<(const Time& rhs) const
    {
        return (m_Tick < rhs.m_Tick);
    }


    /*!
    @brief 比較演算子です。本インスタンスの方が古いか同じ時刻を表していれば true を返します。

    @param[in] rhs 比較する Time です。

    @return 本インスタンスの方が古いか同じ時刻を表していれば true を、そうでなければ false を返します。
     */
    bool operator<=(const Time& rhs) const
    {
        return (m_Tick <= rhs.m_Tick);
    }


    /*!
    @brief 比較演算子です。本インスタンスの方が新しい時刻を表していれば true を返します。

    @param[in] rhs 比較する Time です。

    @return 本インスタンスの方が新しい時刻を表していれば true を、そうでなければ false を返します。
     */
    bool operator>(const Time& rhs) const
    {
        return (m_Tick > rhs.m_Tick);
    }


    /*!
    @brief 比較演算子です。本インスタンスの方が新しいか同じ時刻を表していれば true を返します。

    @param[in] rhs 比較する Time です。

    @return 本インスタンスの方が新しいか同じ時刻を表していれば true を、そうでなければ false を返します。
     */
    bool operator>=(const Time& rhs) const
    {
        return (m_Tick >= rhs.m_Tick);
    }


    /*!
    @cond PRIVATE
    @brief 現在から指定時間先の時刻を取得します。

    @param[in] timeout 時間の差分です。

    @return 現在から timeout 先の時刻です。
     */
    static Time ConvertTimeoutToDeadline(const TimeSpan& timeout)
    {
        return (GetTime() += timeout);
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      インスタンスが保持している生の Tick 値を得ます。

    @return     インスタンスが保持する Tick 値。
    @see        SetTick
     */
    uint64_t GetTick(void) const
    {
        return m_Tick;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      生の Tick 値をインスタンスに設定します。

    @param[in]  tick 生の Tick 値です。
    @see        GetTick
     */
    void SetTick(uint64_t tick)
    {
        m_Tick = tick;
    }
    //! @endcond

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;

    static const Time c_Max;  //< 時刻の最大値
    static const Time c_Min;  //< 時刻の最小値

private:
    static Time Create(uint64_t tick);

    uint64_t m_Tick;
};
}
}
}  // end of namespace nn::pia::common
