﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_RootObject.h>

namespace nn
{
namespace pia
{
namespace common
{

/*!
    @brief 時間間隔を表す型です。単位はミリ秒です。
 */
typedef int32_t TimeInterval;


class Time;

/*!
  @brief  時間の間隔を表すクラスです。ミリ秒単位の数値と型変換ができます。

 */
class TimeSpan : public common::RootObject
{

    friend class Time;

public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    TimeSpan()
        : m_TickSpan(0)
    {
    }


    /*!
    @brief ミリ秒単位の数値で初期化するコンストラクタです。

    @param[in] msec 初期化する値をミリ秒単位の時間で指定します。
     */
    TimeSpan(int32_t msec)
    {
        SetFromMSec(msec);
    }


    /*!
    @brief ミリ秒単位の数値で初期化するコンストラクタです。

    @param[in] msec 初期化する値をミリ秒単位の時間で指定します。
     */
    TimeSpan(int64_t msec)
    {
        SetFromMSec(msec);
    }


    /*!
    @brief コピーコンストラクタです。

    @param[in] rhs 本インスタンスを初期化する値を持つ TimeSpan インスタンスです。
     */
    TimeSpan(const TimeSpan& rhs)
        : m_TickSpan(rhs.m_TickSpan)
    {
    }


    /*!
    @brief 代入演算子です。

    @param[in] rhs 代入する TimeSpan です。

    @return 本インスタンスへの参照が返されます。
     */
    TimeSpan& operator=(const TimeSpan& rhs)
    {
        m_TickSpan = rhs.m_TickSpan;
        return *this;
    }


    /*!
    @brief ミリ秒単位の数値を代入する演算子です。

    @param[in] msec 秒単位の時間です。

    @return 本インスタンスへの参照が返されます。
     */
    TimeSpan& operator=(int32_t msec)
    {
        SetFromMSec(msec);
        return *this;
    }


    /*!
    @brief ミリ秒単位の数値を代入する演算子です。

    @param[in] msec 秒単位の時間です。

    @return 本インスタンスへの参照が返されます。
     */
    TimeSpan& operator=(int64_t msec)
    {
        SetFromMSec(msec);
        return *this;
    }


    /*!
    @brief 時間をミリ秒単位の int32_t 値へ変換する演算子です。

    @return ミリ秒単位の時間です。
     */
    operator int32_t() const
    {
        return GetMSec();
    }


    /*!
    @brief 時間をミリ秒単位の int64_t 値へ変換する演算子です。

    @return ミリ秒単位の時間です。
     */
    operator int64_t() const
    {
        return GetMSecS64();
    }


    /*!
    @brief 時間をミリ秒単位の int32_t 値でセットします。

    @param[in] msec ミリ秒単位の時間です。
     */
    void SetFromMSec(int32_t msec)
    {
        m_TickSpan = GetTicksPerMSec() * msec;
    }


    /*!
    @brief 時間をミリ秒単位の int64_t 値でセットします。

    @param[in] msec ミリ秒単位の時間です。
     */
    void SetFromMSec(int64_t msec)
    {
        m_TickSpan = GetTicksPerMSec() * msec;
    }


    /*!
    @brief 時間を秒単位の int32_t 値でセットします。

    @param[in] sec 秒単位の時間です。
     */
    void SetFromSec(int32_t sec)
    {
        m_TickSpan = GetTicksPerSec() * sec;
    }


    /*!
    @brief 時間を秒単位の int64_t 値でセットします。

    @param[in] sec 秒単位の時間です。
     */
    void SetFromSec(int64_t sec)
    {
        m_TickSpan = GetTicksPerSec() * sec;
    }


    /*!
    @brief 時間を秒単位の float 値でセットします。

    @param[in] sec 秒単位の時間です。
     */
    void SetFromSec(float sec)
    {
        m_TickSpan = static_cast<int64_t>(GetTicksPerSec() * sec);
    }


    /*!
    @brief 時間をミリ秒単位の int32_t 値で取得します。

    @return ミリ秒単位の時間です。
     */
    int32_t GetMSec() const
    {
        return static_cast<int32_t>(m_TickSpan / GetTicksPerMSec());
    }

    /*!
    @cond PRIVATE
    @brief 時間をマイクロ秒単位の int32_t 値で取得します。

    @return マイクロ秒単位の時間です。
     */
    int32_t GetUSec() const
    {
        return static_cast<int32_t>(m_TickSpan / GetTicksPerUSec());
    }
    //! @endcond

    /*!
    @brief 時間をミリ秒単位 int64_t の値で取得します。

    @return ミリ秒単位の時間です。
     */
    int64_t GetMSecS64() const
    {
        return m_TickSpan / GetTicksPerMSec();
    }

    /*!
    @cond PRIVATE
    @brief 時間をマイクロ秒単位 int64_t の値で取得します。

    @return マイクロ秒単位の時間です。
     */
    int64_t GetUSecS64() const
    {
        return m_TickSpan / GetTicksPerUSec();
    }
    //! @endcond

    /*!
    @brief 時間を秒単位の int32_t 値で取得します。

    @return 秒単位の時間です。
     */
    int32_t GetSec() const
    {
        return static_cast<int32_t>(m_TickSpan / GetTicksPerSec());
    }


    /*!
    @brief 時間を秒単位の int64_t 値で取得します。

    @return 秒単位の時間です。
     */
    int64_t GetSecS64() const
    {
        return m_TickSpan / GetTicksPerSec();
    }


    /*!
    @brief 時間を秒単位の float 値で取得します。

    @return 秒単位の時間です。
     */
    float GetSecF32() const
    {
        return static_cast<float>(m_TickSpan) / static_cast<float>(GetTicksPerSec());
    }


    /*!
    @brief 等値演算子です。二つのTimeSpanオブジェクトが同等と見なせるかどうかを判定します。

    @param[in] rhs 比較する TimeSpan です。

    @return 両者の表す時間が同じであれば true を、そうでなければ false を返します。
     */
    bool operator==(const TimeSpan& rhs) const
    {
        return (m_TickSpan == rhs.m_TickSpan);
    }


    /*!
    @brief 等値演算子です。二つの TimeSpan オブジェクトが同等と見なせるかどうかを判定します。

    @param[in] rhs 比較するミリ秒単位の int32_t 値です。

    @return 両者の表す時間が同じであれば true を、そうでなければ false を返します。
     */
    bool operator==(int32_t rhs) const
    {
        return (*this == TimeSpan(rhs));
    }


    /*!
    @brief 等値演算子です。二つの TimeSpan オブジェクトが同等と見なせるかどうかを判定します。

    @param[in] rhs 比較するミリ秒単位の int64_t 値です。

    @return 両者の表す時間が同じであれば true を、そうでなければ false を返します。
     */
    bool operator==(int64_t rhs) const
    {
        return (*this == TimeSpan(rhs));
    }


    /*!
    @brief 等値演算子です。二つの TimeSpan オブジェクトが異なるかどうかを判定します。

    @param[in] rhs 比較する TimeSpan です。

    @return 両者の表す時間が異なれば true を、そうでなければ false を返します。
     */
    bool operator!=(const TimeSpan& rhs) const
    {
        return (m_TickSpan != rhs.m_TickSpan);
    }


    /*!
    @brief 等値演算子です。二つの TimeSpan オブジェクトが異なるかどうかを判定します。

    @param[in] rhs 比較するミリ秒単位の int32_t 値です。

    @return 両者の表す時間が異なれば true を、そうでなければ false を返します。
     */
    bool operator!=(int32_t rhs) const
    {
        return (*this != TimeSpan(rhs));
    }


    /*!
    @brief 等値演算子です。二つの TimeSpan オブジェクトが異なるかどうかを判定します。

    @param[in] rhs 比較するミリ秒単位の int64_t 値です。

    @return 両者の表す時間が異なれば true を、そうでなければ false を返します。
     */
    bool operator!=(int64_t rhs) const
    {
        return (*this != TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が短ければ true を、そうでなければ false を返します。

    @param[in] rhs 比較する TimeSpan です。

    @return 自分の表す時間が短ければ true を、そうでなければ false を返します。
     */
    bool operator<(const TimeSpan& rhs) const
    {
        return (m_TickSpan < rhs.m_TickSpan);
    }


    /*!
    @brief 比較演算子です。自分の表す時間が短ければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int32_t 値です。

    @return 自分の表す時間が短ければ true を、そうでなければ false を返します。
     */
    bool operator<(int32_t rhs) const
    {
        return (*this < TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が短ければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int64_t 値です。

    @return 自分の表す時間が短ければ true を、そうでなければ false を返します。
     */
    bool operator<(int64_t rhs) const
    {
        return (*this < TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が短いか等しければ true を、そうでなければ false を返します。

    @param[in] rhs 比較する TimeSpan です。

    @return 自分の表す時間が短いか等しければ true を、そうでなければ false を返します。
     */
    bool operator<=(const TimeSpan& rhs) const
    {
        return (m_TickSpan <= rhs.m_TickSpan);
    }


    /*!
    @brief 比較演算子です。自分の表す時間が短いか等しければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int32_t 値です。

    @return 自分の表す時間が短いか等しければ true を、そうでなければ false を返します。
     */
    bool operator<=(int32_t rhs) const
    {
        return (*this <= TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が短いか等しければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int64_t 値です。

    @return 自分の表す時間が短いか等しければ true を、そうでなければ false を返します。
     */
    bool operator<=(int64_t rhs) const
    {
        return (*this <= TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が長ければ true を、そうでなければ false を返します。

    @param[in] rhs 比較する TimeSpan です。

    @return 自分の表す時間が長ければ true を、そうでなければ false を返します。
     */
    bool operator>(const TimeSpan& rhs) const
    {
        return (m_TickSpan > rhs.m_TickSpan);
    }


    /*!
    @brief 比較演算子です。自分の表す時間が長ければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int32_t 値です。

    @return 自分の表す時間が長ければ true を、そうでなければ false を返します。
     */
    bool operator>(int32_t rhs) const
    {
        return (*this > TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が長ければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int64_t 値です。

    @return 自分の表す時間が長ければ true を、そうでなければ false を返します。
     */
    bool operator>(int64_t rhs) const
    {
        return (*this > TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が長いか等しければ true を、そうでなければ false を返します。

    @param[in] rhs 比較する TimeSpan です。

    @return 自分の表す時間が長いか等しければ true を、そうでなければ false を返します。
     */
    bool operator>=(const TimeSpan& rhs) const
    {
        return (m_TickSpan >= rhs.m_TickSpan);
    }


    /*!
    @brief 比較演算子です。自分の表す時間が長いか等しければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int32_t 値です。

    @return 自分の表す時間が長いか等しければ true を、そうでなければ false を返します。
     */
    bool operator>=(int32_t rhs) const
    {
        return (*this >= TimeSpan(rhs));
    }


    /*!
    @brief 比較演算子です。自分の表す時間が長いか等しければ true を、そうでなければ false を返します。

    @param[in] rhs 比較するミリ秒単位の int64_t 値です。

    @return 自分の表す時間が長いか等しければ true を、そうでなければ false を返します。
     */
    bool operator>=(int64_t rhs) const
    {
        return (*this >= TimeSpan(rhs));
    }


    /*!
    @brief 本インスタンスに時間を加算します。

    @param[in] rhs 加算する時間です。

    @return 本インスタンスへの参照が返されます。
     */
    TimeSpan& operator+=(const TimeSpan& rhs)
    {
        m_TickSpan += rhs.m_TickSpan;
        return *this;
    }


    /*!
    @brief 本インスタンスに時間を減算します。

    @param[in] rhs 減算する時間です。

    @return 本インスタンスへの参照が返されます。
     */
    TimeSpan& operator-=(const TimeSpan& rhs)
    {
        m_TickSpan -= rhs.m_TickSpan;
        return *this;
    }


    /*!
    @brief 本インスタンスに時間を加算した値をもつ TimeSpan を計算します。

    @param[in] rhs 加算する時間です。

    @return 加算された時間を表す Time です。
     */
    TimeSpan operator+(const TimeSpan& rhs) const
    {
        TimeSpan t;
        t.m_TickSpan = m_TickSpan + rhs.m_TickSpan;
        return t;
    }


    /*!
    @brief 本インスタンスに時間を減算した値をもつ TimeSpan を計算します。

    @param[in] rhs 減算する時間です。

    @return 減算された時間を表す TimeSpan です。
     */
    TimeSpan operator-(const TimeSpan& rhs) const
    {
        TimeSpan t;
        t.m_TickSpan = m_TickSpan - rhs.m_TickSpan;
        return t;
    }


    /*!
    @cond PRIVATE
    @brief 生の Tick 値を取得します。

    @return 生の Tick 値です。
     */
    int64_t GetTick() const
    {
        return m_TickSpan;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 生の Tick 値を設定します。
    
    @param[in] tick 生の Tick 値です。
     */
    void SetTick(int64_t tick)
    {
        m_TickSpan = tick;
    }
    //! @endcond


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;

private:
    int64_t m_TickSpan;

    static const int64_t& GetTicksPerUSec();
    static const int64_t& GetTicksPerMSec();
    static const int64_t& GetTicksPerSec();
};
}
}
}  // end of namespace nn::pia::common
