﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_TreeMapNode.h>
#include <nn/pia/common/common_TreeMapBase.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
    @cond PRIVATE
    @brief  AVL木を用いたツリーマップです。
 */
template <typename Key_>
class TreeMap : public common::TreeMapBase
{
public:
    typedef Key_ Key;
    typedef TreeMapNode<Key> Node;

protected:
    TreeMap()
        : TreeMapBase()
    {
    }

    Node* FindNode(const Key& key) const;

    Node* FrontNode() const
    {
        return static_cast<Node*>(FrontNodeBase());
    }

    Node* BackNode() const
    {
        return static_cast<Node*>(BackNodeBase());
    }

    /*!
    @brief  key 以上のキーを持つ最初の要素
     */
    Node* LowerBoundNode(const Key& key) const;

    void ClearNode();

    bool IsIncludeNode(const Node* cpNode) const
    {
        return IsIncludeNodeBase(cpNode);
    }

    Node* InsertNode(const Key& key, Node* pNode);

    Node* EraseNode(const Key& key)
    {
        Node* pNode = FindNode(key);
        if (pNode != NULL)
        {
            EraseNode(pNode);
        }
        return pNode;
    }

    void EraseNode(Node* pNode);

private:
    // コピー禁止
    TreeMap(const TreeMap& rhs);
    TreeMap& operator=(const TreeMap& rhs);

private:
    Node* RotateRight(Node* pNode)
    {
        return static_cast<Node*>(RotateRightBase(pNode));
    }
    Node* RotateLeft(Node* pNode)
    {
        return static_cast<Node*>(RotateLeftBase(pNode));
    }

    Node* GetRoot() const
    {
        return static_cast<Node*>(GetRootBase());
    }

    void ClearNodeImpl(Node* pNode);
};
//! @endcond


//! @cond
template <typename Key>
void TreeMap<Key>::ClearNode()
{
    ClearNodeImpl(GetRoot());
    ClearNodeBase();
}


template <typename Key>
void TreeMap<Key>::ClearNodeImpl(Node* pNode)
{
    pNode->m_Key = Key();
    if (pNode->GetLeft() != NULL)
    {
        ClearNodeImpl(pNode->GetLeft());
    }
    if (pNode->GetRight() != NULL)
    {
        ClearNodeImpl(pNode->GetRight());
    }
}


template <typename Key>
typename TreeMap<Key>::Node* TreeMap<Key>::FindNode(const Key& key) const
{
    Node* pNode = GetRoot();
    while (pNode != NULL)
    {
        if (key < pNode->m_Key)
        {
            pNode = pNode->GetLeft();
        }
        else if (key > pNode->m_Key)
        {
            pNode = pNode->GetRight();
        }
        else
        {
            return pNode;
        }
    }

    return NULL;
}


template <typename Key>
typename TreeMap<Key>::Node* TreeMap<Key>::LowerBoundNode(const Key& key) const
{
    Node* pNode = GetRoot();
    if (pNode == NULL)
    {
        return NULL;
    }

    while (true)
    {
        if (key < pNode->m_Key)
        {
            if (pNode->GetLeft() == NULL)
            {
                return pNode;
            }
            pNode = pNode->GetLeft();
        }
        else if (key > pNode->m_Key)
        {
            if (pNode->GetRight() == NULL)
            {
                return pNode->Next();
            }
            pNode = pNode->GetRight();
        }
        else
        {
            return pNode;
        }
    }
}


template <typename Key>
typename TreeMap<Key>::Node* TreeMap<Key>::InsertNode(const Key& key, Node* pNode)
{
    pNode->m_Key = key;

    if (GetRoot() == NULL)
    {
        // ルートに追加
        InsertNodeRoot(pNode);
        return NULL;
    }

    // 入れる位置を探す
    Node* pN = GetRoot();
    while (true)
    {
        if (key < pN->m_Key)
        {
            if (pN->GetLeft() == NULL)
            {
                // 左に追加
                InsertNodeLeft(pNode, pN);
                return NULL;
            }
            pN = pN->GetLeft();
        }
        else if (key > pN->m_Key)
        {
            if (pN->GetRight() == NULL)
            {
                // 右に追加
                InsertNodeRight(pNode, pN);
                return NULL;
            }
            pN = pN->GetRight();
        }
        else
        {
            // 同じキーがあるので差し替え

            ReplaceNode(pNode, pN);
            pN->InitNode();
            return pN;
        }
    }
}


template <typename Key>
void TreeMap<Key>::EraseNode(Node* pNode)
{
    EraseNodeBase(pNode);
    pNode->InitNode();
}
//! @endcond
}
}
}  // end of namespace nn::pia::common
