﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#include <nn/pia/common/common_RootObject.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief  Piaライブラリ内部のリソース使用量のピーク値などを計測するために用意されたクラスです（サポート対象外）。

 */
class Watermark : public RootObject
{
public:
    /*!
    @brief  SetName() で設定可能な名前の最大長です。

    @see SetName
     */
    static const uint32_t MaxNameLength = 63;


    /*!
    @brief デフォルトコンストラクタです。
           インスタンスは Update() 機能が無効にされた状態で構築されます。
     */
    Watermark(void);


    /*!
    @brief デストラクタです。
     */
    ~Watermark(void);


    /*!
    @brief    Update() による値の更新を有効にします。

    @details  更新が有効な状態で Update() を呼び出すと、必要に応じて最大値／最小値が更新され、
              GetLatestValue() で取得可能な値が更新され、 Update() の呼び出し回数カウンタが
              インクリメントされます。
    @see      SetUpdateDisabled, IsUpdateEnabled
     */
    void SetUpdateEnabled(void)
    {
        m_IsEnabled = true;
    }


    /*!
    @brief    Update() による値の更新を無効にします。

    @details  更新が無効な状態では、 Update() を呼び出しても、最大値／最小値／最新の値／
              Update() 呼び出し回数などの値は更新されません。
    @see      SetUpdateEnabled, IsUpdateEnabled
     */
    void SetUpdateDisabled(void)
    {
        m_IsEnabled = false;
    }


    /*!
    @brief    Update() 機能が有効な状態であるかどうかを返します。

    @return   Update() による値の更新が有効に設定されていれば true 、無効に設定されていれば false を返します。
    @see      SetUpdateEnabled, SetUpdateDisabled
     */
    bool IsUpdateEnabled(void) const
    {
        return m_IsEnabled;
    }


    /*!
    @brief      インスタンスに名前を設定します。

    @param[in]  pName 設定する名前。 NUL 文字で終端される文字列で、文字列長は MaxNameLength 以下でなくてはなりません。
                      無効なポインタが指定されたり、長すぎる名前が指定された場合は、アサート停止します。
    @see        GetName
     */
    void SetName(const char* pName);


    /*!
    @brief    インスタンスに付けられた名前を得ます。

    @return   名前が格納された文字配列を指すポインタ。
    @see      SetName
     */
    const char* GetName(void) const
    {
        return m_Name;
    }


    /*!
    @brief  インスタンスが保持していた最大値、最小値、最新の値、 Update() 呼び出し回数を初期状態に戻します。

    @details SetName() で設定した名前と、 Update() 機能の有効／無効については何も変更を加えません。
    @see    SetName, SetUpdateEnabled, SetUpdateDisabled, IsUpdateEnabled
     */
    void Reset(void);


    /*!
    @brief      値を更新します。

    @details    Update() 機能が有効に設定された状態でこの関数を呼び出すと、
                これまでの最大値、最小値、最新の値、 Update() 呼び出し回数などの
                値が適切に更新されます。
                Update() 機能が無効に設定された状態でこの関数を呼び出した場合には、
                何もせずに返ります。

    @param[in]  value 監視対象の値。
    @see SetUpdateEnabled, SetUpdateDisabled, IsUpdateEnabled, GetMaxValue, GetMinValue, GetLatestValue, GetUpdateCount
     */
    void Update(int64_t value);


    /*!
    @brief      これまでの Update() における最大値を返します。

    @see        Update, GetMinValue, GetLatestValue, GetUpdateCount
     */
    int64_t GetMaxValue(void) const
    {
        return m_MaxValue;
    }


    /*!
    @brief      これまでの Update() における最小値を返します。

    @see        Update, GetMaxValue, GetLatestValue, GetUpdateCount
     */
    int64_t GetMinValue(void) const
    {
        return m_MinValue;
    }


    /*!
    @brief      最後の Update() で渡された、最新の値を取得します。

    @see        Update, GetMinValue, GetMaxValue, GetUpdateCount
     */
    int64_t GetLatestValue(void) const
    {
        return m_LatestValue;
    }

    /*!
    @brief      これまでに Update() が呼ばれた回数を返します。

    @see        Update, GetMaxValue, GetMinValue, GetLatestValue
     */
    int64_t GetUpdateCount(void) const
    {
        return m_UpdateCount;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


private:
    int64_t m_MinValue;      // 最小値。
    int64_t m_MaxValue;      // 最大値。
    int64_t m_LatestValue;   // 最新の値。
    uint64_t m_UpdateCount;  // Update() が実行された回数。

    bool m_IsEnabled;  // Update() 機能が利用できるかどうか

    char m_Name[MaxNameLength + 1];  // Watermark インスタンスに付ける名前。
};
}
}
}  // end of namespace nn::pia::common
