﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#include <nn/pia/common/common_Singleton.h>
#include <nn/pia/common/common_Watermark.h>


#define NN_PIA_WATERMARK_UPDATE(KEY, VALUE)                                                 \
    \
if(nn::pia::common::WatermarkManager::GetInstance())                                        \
    \
{                                                                                    \
        nn::pia::common::WatermarkManager::GetInstance()->GetWatermark(KEY)->Update(VALUE); \
    \
}


#define NN_PIA_WATERMARK_SETNAME(KEY, NAME)                                                 \
    \
if(nn::pia::common::WatermarkManager::GetInstance())                                        \
    \
{                                                                                    \
        nn::pia::common::WatermarkManager::GetInstance()->GetWatermark(KEY)->SetName(NAME); \
    \
}


#define NN_PIA_WATERMARK_TRACE(KEY, FLAG)                                                 \
    \
if(nn::pia::common::WatermarkManager::GetInstance())                                      \
    \
{                                                                                  \
        nn::pia::common::WatermarkManager::GetInstance()->GetWatermark(KEY)->Trace(FLAG); \
    \
}


namespace nn
{
namespace pia
{
namespace common
{

/*!
  @brief  Watermark インスタンス群を管理するクラスです（サポート対象外）。

  @details WatermarkManager クラスは、 Watermark インスタンス群を管理するために用意されたクラスです。
          Pia 内部バッファ使用量のピーク値などを取得することができます。

 */
class WatermarkManager : public nn::pia::common::RootObject
{
public:
    static const int WatermarkInstances = 10;  //!< WatermarkManager が管理する Watermark インスタンスの最大個数。

    // Watermark インスタンスを区別するためのキー。重複してはいけません。
    static const int KeySendthreadstreamBufferNum = 0;     //!< 送信スレッドにおける内部バッファの使用量に対応するキーです。
    static const int KeyReceivethreadstreamBufferNum = 1;  //!< 受信スレッドにおける内部バッファの使用量に対応するキーです。
    static const int KeyInetReceiveBufferNum = 2;
    static const int KeyReliableProtocolSendBufferNum = 3;            //!< ReliableProtocol の送信バッファ使用量に対応するキー。
    static const int KeyReliableProtocolReceiveBufferNum = 4;         //!< ReliableProtocol の受信バッファ使用量に対応するキー。
    static const int KeyStationProtocolReliableSendBufferNum = 5;     //!< StationProtocolReliable の送信バッファ使用量に対応するキー。
    static const int KeyStationProtocolReliableReceiveBufferNum = 6;  //!< StationProtocolReliable の受信バッファ使用量に対応するキー。
    static const int KeySessionProtocolReliableSendBufferNum = 7;     //!< SessionProtocol の Reliable 送信バッファ使用量に対応するキー。
    static const int KeySessionProtocolReliableReceiveBufferNum = 8;  //!< SessionProtocol の Reliable 受信バッファ使用量に対応するキー。
    static const int KeyUnreliableProtocolReceiveBufferNum = 9;       //!< UnreliableProtocol の受信バッファ使用量に対応するキー。
    static const int KeyMax = WatermarkInstances - 1;


    /*!
    @brief    common モジュールにおけるクラスのインスタンスを作成します（シングルトンパターン）。

    @return   インスタンスの作成に成功すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   ResultNotInitialized     common モジュールが未初期化です。[:progErr]
    @retval   ResultInvalidState       CreateInstance() を呼ぶタイミングが誤っています。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。[:progErr]
    @retval   ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
     */
    static Result CreateInstance(void);


    /*!
    @brief    common モジュールにおけるクラスのインスタンスを破棄します（シングルトンパターン）。
     */
    static void DestroyInstance(void);


    /*!
    @brief    common モジュールにおけるクラスのインスタンスを得ます（シングルトンパターン）

    @details  事前に CreateInstance() によってインスタンスを作成していない場合は、
              NULLポインタが返されます。

    @return   インスタンスへのポインタ。
     */
    static WatermarkManager* GetInstance(void)
    {
        return s_pInstance;
    }


    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。
     */
    WatermarkManager(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。
     */
    virtual ~WatermarkManager(void);
    //! @endcond


    /*!
    @brief Watermark インスタンスを得ます（非 const 版）

    @param[in] key キーを指定します。 0 <= key <= KeyMax でなくてはなりません。
                   範囲外のキーを指定した場合はアサート停止します。
    @return    成功すれば Watermark インスタンスへのポインタが、失敗すれば NULL ポインタが返されます。
     */
    Watermark* GetWatermark(int key);


    /*!
    @brief Watermark インスタンスを得ます（ const 版）

    @param[in] key キーを指定します。 0 <= key <= KeyMax でなくてはなりません。
                   範囲外のキーを指定した場合はアサート停止します。
    @return    成功すれば Watermark インスタンスへのポインタが、失敗すれば NULL ポインタが返されます。
     */
    const Watermark* GetWatermark(int key) const;


    /*!
    @brief  WatermarkManager が管理する、全 Watermark インスタンスの Update() 機能を有効にします。

    @see    DisableAllWatermark
     */
    void EnableAllWatermark(void);


    /*!
    @brief  WatermarkManager が管理する、全 Watermark インスタンスの Update() 機能を無効にします。

    @see    EnableAllWatermark
     */
    void DisableAllWatermark(void);


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

private:
    Watermark m_Watermark[WatermarkInstances];

    static WatermarkManager* s_pInstance;  // このクラスの唯一のインスタンスの実体
};
}
}
}  // end of namespace nn::pia::common
