﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/framework/framework_Definitions.h>
#include <nn/pia/clone/clone_CloneProtocol.h>

namespace nn
{
namespace pia
{
namespace framework
{
class ChangeStateJob;
class LoginJob;

class CloneFacade;
class InetService;
class LanFacade;
class LocalFacade;
class NetworkService;
class ReckoningFacade;
class SessionFacade;
class SyncFacade;
class TransportFacade;
class TuneFacade;

/*!
@brief  framework モジュールを使用するためのシングルトンクラスです。
*/
class Framework : public common::RootObject
{
public:
    /*!
    @brief セッション参加処理の進行状態です。
    */
    enum State
    {
        State_NotInitialized = 0,               //!< NEX やネットワークインターフェース、ソケットを終了した状態です。
        State_NetworkCleanedUp = 0,             //!< NEX やネットワークインターフェース、ソケットを終了した状態です。
        State_NetworkStartedUp = 1,             //!< NEX やネットワークインターフェース、ソケットを初期化し、ネットワーク接続が利用可能になった状態であり、ログアウトした状態です。
        State_LoggedOut = 1,                    //!< NEX やネットワークインターフェース、ソケットを初期化し、ネットワーク接続が利用可能になった状態であり、ログアウトした状態です。
        State_LoggedIn = 2,                     //!< ログインし、Pia を終了した状態です。
        State_SessionFinalized = 2,             //!< ログインし、Pia を終了した状態です。
        State_SessionInitialized = 3,           //!< Pia を初期化し、Pia をクリーンアップした状態です。
        State_SessionCleanedUp = 3,             //!< Pia を初期化し、Pia をクリーンアップした状態です。
        State_SessionStartedUp = 4,             //!< Pia をスタートアップし、セッションから離脱した状態です。
        State_SessionLeft = 4,                  //!< Pia をスタートアップし、セッションから離脱した状態です。
        State_SessionJoined = 5                 //!< セッションに参加した状態です。
#if NN_PIA_EXPERIMENT_FRAMEWORK_JOINT_SESSION
        ,
        State_JointSessionJoined = 6,           //!< ジョイントセッションに参加した状態です。
        State_JointSessionLeft = 4              //!< Pia をスタートアップし、ジョイントセッションから離脱した状態です。
#endif
    };

    /*!
    @brief  PiaClone の更新処理結果です。
    */
    struct CloneUpdateResult
    {
        clone::CloneProtocol::ErrorType errorType;       //!< エラーの発生状態です。
        clone::CloneProtocol::State cloneProtocolState;  //!< CloneProtocol の動作状態です。
        clone::ClockValue clock;                         //!< CloneProtocol が管理する時刻です。
    };

    /*!
    @cond PRIVATE
    */
    struct Setting
    {
        NetworkType networkType;
    };
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief コンストラクタです。アプリケーションはこのコンストラクタを直接呼び出すのではなく、 CreateInstance() を使う必要があります。
    */
    Framework(const Setting& setting);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief デストラクタです。
    */
    ~Framework();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                       Framework クラスのインスタンスを作成します（シングルトンパターン）。
    @return                      インスタンスの作成に成功すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized framework モジュールが未初期化です。[:progErr]
    @retval ResultInvalidState   CreateInstance() を呼ぶタイミングが誤っています。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。[:progErr]
    @retval ResultAlreadyExists  既にインスタンスは作成されています。[:progErr]
    @see    DestroyInstance, GetInstance, Setting
    */
    static Result CreateInstance(const Setting& setting);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief Framework クラスのインスタンスを破棄します（シングルトンパターン）。
           インスタンスが存在していなかった場合は、何もしません。
    @see   CreateInstance, GetInstance
    */
    static void DestroyInstance();
    //! @endcond

    /*!
    @brief              Framework クラスのインスタンスを得ます（シングルトンパターン）。
                        事前に CreateInstance() によってインスタンスを作成していない場合は、NULL ポインタが返されます。
                        この関数はスレッドセーフです。
    @return   Framework インスタンスへのポインタです。
    @see      CreateInstance, DestroyInstance
    */
    static Framework* GetInstance()
    {
        return s_pInstance;
    }

    /*!
    @cond   PRIVATE
    @brief                     スタートアップ処理を実行します。
    @details                   framework::EndSetup() 呼び出しで framework モジュールの初期化処理を終えた後で呼び出す必要があります。
    @return                    成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState 既に Startup() が呼ばれているか、framework::BeginSetup() ～ framework::EndSetup() 区間の中で呼び出されている可能性があります。[:progErr]
    @see    Cleanup, framework::EndSetup
    */
    Result Startup();
    //! @endcond

    /*!
    @cond    PRIVATE
    @brief   クリーンアップ処理を実行します。
    @details framework モジュールの終了処理を実行する前に呼び出す必要があります。
    @see     Startup
    */
    void Cleanup();
    //! @endcond

    /*!
    @brief          デバッグに有用な情報をプリントします。
    @param[in] flag トレースフラグの論理和。詳細は @ref common::TraceFlag 型を参照してください。
    */
    void Trace(uint64_t flag) const;

    /*!
    @brief                                   ディスパッチ処理を行います。
    @details                                 通常、1 ゲームフレームに 1 回または 2 回の頻度で呼び出します。アクションゲームでは通信遅延を低減するために、ゲームの通信処理の前後に 1 回ずつ、計 2 回 Update() を呼び出すことを推奨します。 前後で呼び出すことで直前の受信をゲーム処理に反映させることができ、ゲーム処理の結果を直後に送信することができます。
                                             なお、非同期処理中は Update() 失敗時のエラーハンドリングは不要です。
                                             InetService（NexServiceなど）のディスパッチの後、Piaのディスパッチが呼ばれます。
                                             isAutoInitializeAndLogin を有効にして初期化した場合、Framework::State_LoggedIn 以上の状態の間、InetService の接続確認（nex::NgsFacade::IsConnectedなど）が行われます。
    @return                                  成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultSessionConnectionIsLost    メッシュから切断されました。[:handling-leave]
    @retval ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultInvalidState               CreateInstance() が呼ばれていない、もしくは DestroyInstance() を呼んだ可能性があります。[:progErr]
    @retval ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultNexInternalError           [:nexInternalError][:viewer][:handling]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    */
    Result Update();

    /*!
    @brief   PiaClone の更新処理を行います。
    @details 1 ゲームフレームに 1 回呼び出す必要があります。
    @return  PiaClone の更新処理結果が返されます。
    */
    CloneUpdateResult CloneUpdate();

    /*!
    @brief   InetService インスタンスを取得します。
    @details 
    @return  InetService インスタンスのポインタが返されます。
    */
    InetService* GetInetService();

    /*!
    @cond  PRIVATE
    @brief                                ランダムマッチメイクの非同期処理を開始します。
    @return                               成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState            この関数を呼び出すタイミングが誤っています。[:progErr]
    @retval ResultNetworkConnectionIsLost 接続が既に切断されています。[:handling-shutdownNetwork]
    */
    Result JoinRandomSessionAsyncInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief        JoinRandomSessionAsyncInternal の非同期処理の完了をチェックします。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsJoinRandomSessionCompletedInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                                                              JoinRandomSessionAsyncInternal の非同期処理の結果を取得します。
    @return                                                             成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidArgument                                       非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState                                          非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval ResultNetworkConnectionIsLost                               ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultGameServerMaintenance                                 ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval ResultNatCheckFailed                                        NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval ResultDnsFailed                                             DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval ResultSessionIsNotFound                                     非同期処理途中で対象のセッションが消失もしくは締め切られました。[:handling-cleanup]
    @retval ResultCancelled                                             非同期処理がキャンセルされました。[:handling-cleanup]
    @retval ResultStationConnectionFailed                               接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultStationConnectionNatTraversalFailedUnknown            NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEim                             NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdm                   NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEim                   NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdm                             NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEimSamePublicAddress            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdmSamePublicAddress            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalRequestTimeout                            NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]
    @retval ResultJoinRequestDenied                                     セッションホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval ResultMeshIsFull                                            参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedNoCandidate                                リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRttLimit                                   リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRelayNumLimit                              リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedUnknown                                    リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval ResultInvalidSystemMessage                                  セッションホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval ResultIncompatibleFormat                                    相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted                              ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultSessionWrongState                                     参加したセッションの状態が異常でした。[:handling-cleanup]
    */
    Result GetJoinRandomSessionResultInternal() const;
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                                  ランダムマッチのセッション離脱処理です。
    @return                                 成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval   ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていることを確認してください。[:progErr]
    @retval   ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    */
    Result LeaveSessionAsyncInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief  ランダムマッチのセッション離脱処理の完了を確認します。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    bool IsLeaveSessionCompletedInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                                 ランダムマッチのセッション離脱処理の結果を取得します。
    @return                                成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState             処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultSessionIsNotFound        非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultErrorOccurred            内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultGameServerProcessAborted ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    */
    Result GetLeaveSessionResultInternal() const;
    //! @endcond

    /*!
    @brief                                セッションの参加受付を開始します。
    @details                              セッション参加受付を開始する非同期処理を開始します。
                                          セッションホストであるときに実行できます。
                                          開始した処理は @ref Framework::Update() を定期的に呼び出すことで進行します。
                                          処理の完了は @ref Framework::IsOpenSessionCompleted でチェックし、結果は @ref Framework::GetOpenSessionResult で確認できます。
    @return                               非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、状態が State_SessionStartedUp に達していること、セッション参加状態であること、セッションホストであることを確認してください。[:progErr]
    @see    IsOpenSessionCompleted, GetOpenSessionResult
    */
    Result OpenSessionAsync();

    /*!
    @brief   Framework::OpenSessionAsync() の非同期処理の完了をチェックします。
    @details @ref Framework::OpenSessionAsync() で開始された非同期処理は、本関数で完了をチェックできます。
    @return  非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see     OpenSessionAsync, GetOpenSessionResult
    */
    bool IsOpenSessionCompleted() const;

    /*!
    @brief                                 Framework::OpenSessionAsync() の非同期処理の結果を取得します。
    @details                               @ref Framework::OpenSessionAsync() で開始された非同期処理が完了した後、本関数で結果を取得できます。
    @return                                IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultSessionConnectionIsLost  メッシュから切断されました。[:handling-leave]
    @retval ResultSessionIsNotFound        非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @see    OpenSessionAsync, IsOpenSessionCompleted
    */
    Result GetOpenSessionResult() const;

    /*!
    @brief                                セッションの参加締切を行います。
    @details                              セッション参加を締め切る非同期処理を開始します。
                                          セッションホストであるときに実行できます。
                                          開始した処理は @ref Framework::Update() を定期的に呼び出すことで進行します。
                                          処理の完了は @ref Framework::IsCloseSessionCompleted() でチェックし、結果は @ref Framework::GetCloseSessionResult() で確認できます。
                                          本関数の呼び出し直前にセッションに新規参加しようとしているステーションが非同期処理完了後に参加してくる可能性があるため、
                                          セッション参加締切後、マッチメイクセッションとメッシュの参加者数が一致するまで待ってから非同期処理を完了します。
                                          また、一定期間（15秒）経過しても一致しない場合も非同期処理を完了します。この場合、@ref Framework::GetCloseSessionResult() は
                                          ResultNegligibleFault を返します。非同期処理完了後に新規参加してくる可能性がありますので、アプリケーションで適切にハンドリングしてください。
    @return                               非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、状態が State_SessionStartedUp に達していること、セッション参加状態であること、セッションホストであることを確認してください。[:progErr]
    @see    IsCloseSessionCompleted, GetCloseSessionResult
    */
    Result CloseSessionAsync();

    /*!
    @brief   Framework::CloseSessionAsync() の非同期処理の完了をチェックします。
    @details @ref Framework::CloseSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return  非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see     CloseSessionAsync, GetCloseSessionResult
    */
    bool IsCloseSessionCompleted() const;

    /*!
    @brief                                 Framework::CloseSessionAsync() の非同期処理の結果を取得します。
    @details                               @ref Framework::CloseSessionAsync() で開始された非同期処理が完了した後、本関数で結果を取得できます。
    @return                                IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultSessionConnectionIsLost  メッシュから切断されました。[:handling-leave]
    @retval ResultSessionIsNotFound        非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultNegligibleFault          マッチメイクセッションとメッシュの参加人数が一致しない状態で一定時間経過しました。[:handling]
    @see    CloseSessionAsync, IsCloseSessionCompleted
    */
    Result GetCloseSessionResult() const;

    /*!
    @cond  PRIVATE
    @brief                                                              セッション参加処理の進行状態の状態遷移を開始します。
    @details                                                            指定した進行状態に達するまでに必要な処理を行います。切断等が発生した際のエラーハンドリング時にも呼び出すことができ、必要な終了関連処理を行います。
    @details                                                            @ref State_SessionJoined の状態で @ref State_SessionStartedUp、@ref State_SessionLeft を目標状態に指定すると、@ref State_SessionCleanedUp まで遷移します。
    @param[in] transitionState                                          目標とするセッション参加処理の進行状態です。
    @param[in] result                                                   エラーハンドリングの対象となる Result です。エラーハンドリング時以外は、ResultSuccess を指定する必要があります。
    @return                                                             IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState                                          呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval ResultInvalidArgument                                       transitionState に遷移できません。result のハンドリングを行うために終了関連処理が必要だが、現在の進行状態よりも進んだ transitionState を指定した場合などに返ります。
    @see    IsChangeStateCompleted, GetChangeStateResult
    */
    Result ChangeStateAsync(State transitionState, Result result);
    //! @endcond

    /*!
    @brief                        セッション参加処理の進行状態の状態遷移を開始します。
    @details                      指定した進行状態に達するまでに必要な処理を行います。
    @details                      @ref State_SessionJoined の状態で @ref State_SessionStartedUp 、@ref State_SessionLeft を目標状態に指定すると、@ref State_SessionCleanedUp まで遷移します。
    @if NIN_DOC
    @details                      状態を @ref State_SessionInitialized から @ref State_NetworkStartedUp に進行する際に内部で nn::nifm::HandleNetworkRequestResult() を呼ぶため、本体機能の UI を画面に表示することがあります。この間処理をブロックします。
    @details                      状態を @ref State_NetworkStartedUp から @ref State_LoggedIn に進行する際に内部で nn::account::EnsureNetworkServiceAccountAvailable() を呼ぶため、指定したユーザーのネットワークサービスアカウントが利用可能ではない場合に、本体機能の UI を画面に表示します。この間処理をブロックします。
    @details                      ネットワークタイプがローカルかつ framework::SessionStartupSetting::isAutoInitializeLdn が真のとき、内部で LDN ライブラリの初期化処理を行いますが、その後 LDN ライブラリの終了処理を行うまでの間、他の通信機能を利用できなくなります。システムが通信機能を使用する場合があるため、必要なシーン以外では LDN ライブラリを初期化しないでください。
                                  また、ローカル通信の使用が終わったら速やかに LDN ライブラリの終了処理を行ってください。
                                  LDN ライブラリの初期化は、状態を State_SessionInitialized から State_SessionStartedUp に遷移するタイミングで行われ、
                                  終了処理は State_SessionStartedUp から State_SessionInitialized に遷移するタイミングで行われます。
    @endif
    @param[in] transitionState    目標とするセッション参加処理の進行状態です。
    @return                       IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState    呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval ResultInvalidArgument 引数が誤っています。現在の進行状態よりも進んだ transitionState を指定した場合などに返されます。[:progErr]
    @see    IsChangeStateCompleted, GetChangeStateResult
    */
    Result ChangeStateAsync(State transitionState);

    /*!
    @brief                        切断などが発生した際のエラーハンドリングを行います。
    @details                      指定した進行状態に達するまでに必要な終了関連処理を行います。
    @details                      @ref State_SessionJoined の状態で @ref State_SessionStartedUp 、@ref State_SessionLeft を目標状態に指定すると、@ref State_SessionCleanedUp まで遷移します。
    @param[in] transitionState    目標とするセッション参加処理の進行状態です。
    @param[in] result             エラーハンドリングの対象となる Result です。エラーハンドリング時以外は、ResultSuccess を指定する必要があります。
    @return                       IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState    呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval ResultInvalidArgument transitionState に遷移できません。result のハンドリングを行うために終了関連処理が必要な状況で、現在の進行状態よりも進んだ transitionState を指定した場合に返ります。IsSuccess() が真となる Result を引数として与えた場合もこの値が帰ります。
    @see    IsChangeStateCompleted, GetChangeStateResult
    */
    Result HandleErrorAsync(State transitionState, Result result);

    /*!
    @brief   Framework::ChangeStateAsync() の非同期処理の完了をチェックします。
    @details @ref Framework::ChangeStateAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return  非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see     ChangeStateAsync, GetChangeStateResult
    */
    bool IsChangeStateCompleted() const;

    /*!
    @brief                                                              Framework::ChangeStateAsync() の非同期処理の結果を取得します。
    @details                                                            @ref Framework::ChangeStateAsync() で開始された非同期処理が完了した後、本関数で結果を取得できます。
    @return                                                             IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidArgument                                       非同期処理開始時に渡した引数の設定内容が想定外です。登録した設定を確認してください。[:progErr]
    @retval ResultInvalidState                                          呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval ResultAlreadyInitialized                                    既に初期化が完了しています。[:progErr]
    @retval ResultSdkError                                              SDK の API 呼び出しに失敗しました。
    @if CTR_DOC
    @retval ResultSdkViewerErrorCodeError                               SDK の API 呼び出しに失敗しました。nn::pia::Result::GetErrorCode() で取得できるエラーコードは SDK のものになります。
    @endif
    @retval ResultNetworkConnectionIsLost                               ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultGameServerMaintenance                                 ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval ResultNatCheckFailed                                        NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval ResultDnsFailed                                             DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval ResultSessionIsNotFound                                     非同期処理途中で対象のセッションが消失もしくは締め切られました。[:handling-cleanup]
    @retval ResultCancelled                                             非同期処理がキャンセルされました。[:handling-cleanup]
    @retval ResultStationConnectionFailed                               接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultStationConnectionNatTraversalFailedUnknown            NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEim                             NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdm                   NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEim                   NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdm                             NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEimSamePublicAddress            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdmSamePublicAddress            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalRequestTimeout                            NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]
    @retval ResultJoinRequestDenied                                     セッションホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval ResultMeshIsFull                                            参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedNoCandidate                                リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRttLimit                                   リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRelayNumLimit                              リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedUnknown                                    リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval ResultInvalidSystemMessage                                  セッションホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval ResultIncompatibleFormat                                    相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval ResultNexInternalError                                      [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted                              ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultSessionWrongState                                     参加したセッションの状態が異常でした。[:handling-cleanup]
    @retval ResultTimeOut                                               タイムアウトで終了しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultErrorOccurred                                         内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @if NIN_DOC
    @retval ResultWifiOff                                               無線がオフになっています。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                                                 スリープ状態です。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval ResultCancelledByUser                                       ユーザー操作に従って処理を中断しました。[:handling-logout]
    @retval ResultSdkViewerResultError                                  SDK の API 呼び出しに失敗しました。nn::pia::framework::Framework::GetLastSdkResult() で nn::Result を取得し、エラービューアアプレットを使用して適切にハンドリングして下さい。[:handling-logout]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin      他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    @see     ChangeStateAsync, IsChangeStateCompleted
    */
    Result GetChangeStateResult() const;

    /*!
    @brief   Framework::HandleErrorAsync() の非同期処理の完了をチェックします。
    @details @ref Framework::HandleErrorAsync() で開始された非同期処理は、本関数で完了をチェックできます。
    @return  非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see     HandleErrorAsync, GetHandleErrorResult
    */
    bool IsHandleErrorCompleted() const;

    /*!
    @brief                      Framework::HandleErrorAsync() の非同期処理の結果を取得します。
    @details                    @ref Framework::HandleErrorAsync() で開始された非同期処理が完了した後、本関数で結果を取得できます。
    @return                     IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval                     ResultInvalidState 呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @see                        HandleErrorAsync, IsHandleErrorCompleted
    */
    Result GetHandleErrorResult() const;
    // n3492: todo retval が網羅できていないので、追記する必要があります。

    /*!
    @brief                        セッション作成処理を行う非同期処理を開始します。
    @param[in] pSetting           セッション作成に必要なパラメータです。
    @return                       IsSuccess() が真となる Result を返します。
    @retval ResultInvalidState    呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval ResultInvalidArgument 引数が誤っています。[:progErr]
    */
    Result CreateSessionAsync(const session::CreateSessionSetting* pSetting);

    /*!
    @brief        セッション作成処理が成功したか確認します。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsCreateSessionCompleted() const;

    /*!
    @brief                                 セッション作成処理の結果を取得します。
    @return                                IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument          非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState             非同期処理を進行できない内部状態になりました。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultGameServerMaintenance    ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval ResultDnsFailed                DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval ResultNatCheckFailed           NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval ResultCancelled                非同期処理がキャンセルされました。[:handling-cleanup]
    @retval ResultTimeOut                  タイムアウトで終了しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultErrorOccurred            内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultSessionWrongState        参加したセッションの状態が異常でした。[:handling-cleanup]
    */
    Result GetCreateSessionResult() const;

    /*!
    @cond  PRIVATE
    @brief                                セッション作成処理を行う非同期処理を開始します。
    @param[in] pSetting                   セッション作成に必要なパラメータです。
    @return                               IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument         非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]
    @retval ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    */
    Result CreateSessionAsyncInternal(const session::CreateSessionSetting* pSetting);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief        セッション作成処理が成功したか確認します。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsCreateSessionCompletedInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                                 セッション作成処理の結果を取得します。
    @return                                IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument          非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState             非同期処理を進行できない内部状態になりました。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultGameServerMaintenance    ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval ResultDnsFailed                DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval ResultNatCheckFailed           NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval ResultCancelled                非同期処理がキャンセルされました。[:handling-cleanup]
    @retval ResultTimeOut                  タイムアウトで終了しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultErrorOccurred            内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultSessionWrongState        参加したセッションの状態が異常でした。[:handling-cleanup]
    */
    Result GetCreateSessionResultInternal() const;
    //! @endcond

    /*!
    @brief                                セッション検索を行う非同期処理を開始します。
    @param[in] pCriteria                  セッション検索に必要なパラメータです。
    @return                               IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument         非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]
    @retval ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    */
    Result BrowseSessionAsync(const session::SessionSearchCriteria* pCriteria);

    /*!
    @brief        セッション検索が成功したか確認します。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsBrowseSessionCompleted() const;

    /*!
    @brief                                セッション検索の結果を取得します。
    @return                               IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument         非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState            非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval ResultCancelled               実行中の非同期処理をキャンセルしました。[:handling-cleanup]
    @retval ResultGameServerMaintenance   ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    */
    Result GetBrowseSessionResult() const;

    Result BrowseSessionAsyncInternal(const session::SessionSearchCriteria* pCriteria);
    bool IsBrowseSessionCompletedInternal() const;
    Result GetBrowseSessionResultInternal() const;

    /*!
    @brief   BrowseSessionAsync の非同期処理で検索できたセッションのリストを取得します。
    @details @ref Framework::BrowseSessionAsync で開始された非同期処理が完了し、@ref Framework::GetBrowseSessionResult が成功した後、本関数でセッションリストを取得できます。
    @return  検索できたセッションのリストのポインタを返します。セッションが見つからない場合は NULL を返します。
    */
    session::ISessionInfoList* GetBrowsedSessionInfoList();

    /*!
    @brief                                  セッション参加処理を行う非同期処理を開始します。
    @param[in] pSetting                     セッション参加に必要なパラメータです。
    @return                                 IsSuccess() が真となる Result を返します。
    @retval   ResultInvalidArgument         非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]
    @retval   ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    */
    Result JoinSessionAsync(const session::JoinSessionSetting* pSetting);

    /*!
    @brief        セッション参加が成功したか確認します。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsJoinSessionCompleted() const;

    /*!
    @brief                     セッション参加の結果を取得します。
    @return                    IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument                                      非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState                                         非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval ResultNetworkConnectionIsLost                              ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultSessionIsNotFound                                    非同期処理途中で対象のセッションが消失もしくは締め切られました。@cond NIN_DOC ローカル通信でパスワードが一致しなかった場合にも返ります。 @endcond[:handling-cleanup]
    @retval ResultMatchmakeSessionIsFull                               マッチメイクセッションが満員でした。[:handling-cleanup]
    @retval ResultDeniedByParticipants                                 セッション参加者が自分をブラックリストに登録しています。[:handling-cleanup]
    @retval ResultParticipantInBlockList                               セッションに自分がブロックリストに登録している参加者がいました。[:handling-cleanup]
    @retval ResultGameServerMaintenance                                ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval ResultNatCheckFailed                                       NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval ResultDnsFailed                                            DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval ResultCancelled                                            非同期処理がキャンセルされました。[:handling-cleanup]
    @retval ResultInvalidState                                         接続可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval ResultStationConnectionFailed                              接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultStationConnectionNatTraversalFailedUnknown           NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEim                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdm                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEim                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdm                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEimSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdmSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalRequestTimeout                           NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]
    @retval ResultJoinRequestDenied                                    セッションホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval ResultMeshIsFull                                           参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedNoCandidate                               リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRttLimit                                  リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRelayNumLimit                             リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedUnknown                                   リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval ResultInvalidSystemMessage                                 セッションホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval ResultSessionUserPasswordUnmatch                           参加時に指定したユーザーパスワードがセッションに設定されたユーザーパスワードと一致しませんでした。[:handling-cleanup]
    @retval ResultSessionSystemPasswordUnmatch                         参加時に指定したシステムパスワードがセッションに設定されたシステムパスワードと一致しませんでした。[:handling-cleanup]
    @retval ResultSessionIsClosed                                      参加を行おうとしたセッションが締め切られていました。[:handling-cleanup][:viewer-handling]
    @retval ResultHostIsNotFriend                                      参加を行おうとしたセッションのホストがフレンドではありませんでした。[:handling-cleanup]
    @retval ResultCompanionStationIsOffline                            同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval ResultIncompatibleFormat                                   相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted                             ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultSessionWrongState                                    参加したセッションの状態が異常でした。[:handling-cleanup]
    @retval ResultNetworkIsFull                                        ネットワークが満員でした。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultTimeOut                                              タイムアウトで終了しました。
    @if CTR_DOC
    local::UdsJoinSessionSetting で設定した暗号鍵が誤っている場合にも返ります。
    ローカル通信時にのみ返ります。
    @endif
    [:handling-cleanup]
    @if CTR_DOC
    @retval ResultLocalLowerVersion                                          自分の @ref nn::pia::local::LocalNetworkVersion 、または、アプリケーションバージョンが参加を行おうとしたセッションのものよりも低いです。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultLocalHigherVersion                                         自分の @ref nn::pia::local::LocalNetworkVersion 、または、アプリケーションバージョンが参加を行おうとしたセッションのものよりも高いです。ローカル通信時にのみ返ります。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultLocalLowerVersion                                          自分の @ref nn::pia::local::LocalNetworkVersion 、または、ローカル通信バージョンが参加を行おうとしたセッションのものよりも低いです。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @retval ResultLocalHigherVersion                                         自分の @ref nn::pia::local::LocalNetworkVersion 、または、ローカル通信バージョンが参加を行おうとしたセッションのものよりも高いです。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @endif
    @retval ResultErrorOccurred                                        内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    */
    Result GetJoinSessionResult() const;

    /*!
    @cond  PRIVATE
    @brief                                  セッション参加処理を行う非同期処理を開始します。
    @param[in] pSetting                     セッション参加に必要なパラメータです。
    @return                                 IsSuccess() が真となる Result を返します。
    @retval   ResultInvalidArgument         非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]
    @retval   ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    */
    Result JoinSessionAsyncInternal(const session::JoinSessionSetting* pSetting);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief        セッション参加が成功したか確認します。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsJoinSessionCompletedInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                                                             セッション参加の結果を取得します。
    @return                                                            IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument                                      非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval ResultInvalidState                                         非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval ResultNetworkConnectionIsLost                              ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultSessionIsNotFound                                    非同期処理途中で対象のセッションが消失もしくは締め切られました。@cond NIN_DOC ローカル通信でパスワードが一致しなかった場合にも返ります。 @endcond[:handling-cleanup]
    @retval ResultMatchmakeSessionIsFull                               マッチメイクセッションが満員でした。[:handling-cleanup]
    @retval ResultDeniedByParticipants                                 セッション参加者が自分をブラックリストに登録しています。[:handling-cleanup]
    @retval ResultParticipantInBlockList                               セッションに自分がブロックリストに登録している参加者がいました。[:handling-cleanup]
    @retval ResultGameServerMaintenance                                ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval ResultNatCheckFailed                                       NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval ResultDnsFailed                                            DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval ResultCancelled                                            非同期処理がキャンセルされました。[:handling-cleanup]
    @retval ResultInvalidState                                         接続可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval ResultStationConnectionFailed                              接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultStationConnectionNatTraversalFailedUnknown           NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEim                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdm                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEim                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdm                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEimSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdmSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalRequestTimeout                           NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]
    @retval ResultJoinRequestDenied                                    セッションホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval ResultMeshIsFull                                           参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedNoCandidate                               リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRttLimit                                  リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRelayNumLimit                             リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedUnknown                                   リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval ResultInvalidSystemMessage                                 セッションホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval ResultSessionUserPasswordUnmatch                           参加時に指定したユーザーパスワードがセッションに設定されたユーザーパスワードと一致しませんでした。[:handling-cleanup]
    @retval ResultSessionSystemPasswordUnmatch                         参加時に指定したシステムパスワードがセッションに設定されたシステムパスワードと一致しませんでした。[:handling-cleanup]
    @retval ResultSessionIsClosed                                      参加を行おうとしたセッションが締め切られていました。[:handling-cleanup][:viewer-handling]
    @retval ResultHostIsNotFriend                                      参加を行おうとしたセッションのホストがフレンドではありませんでした。[:handling-cleanup]
    @retval ResultCompanionStationIsOffline                            同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval ResultIncompatibleFormat                                   相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval ResultGameServerProcessAborted                             ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval ResultSessionWrongState                                    参加したセッションの状態が異常でした。[:handling-cleanup]
    @retval ResultNetworkIsFull                                        ネットワークが満員でした。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultTimeOut                                              タイムアウトで終了しました。
    @if CTR_DOC 
    local::UdsJoinSessionSetting で設定した暗号鍵が誤っている場合にも返ります。
    ローカル通信時にのみ返ります。
    @endif
    [:handling-cleanup]
    @if CTR_DOC
    @retval ResultLocalLowerVersion                                          自分の @ref local::LocalNetworkVersion 、または、アプリケーションバージョンが参加を行おうとしたセッションのものよりも低いです。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval ResultLocalHigherVersion                                         自分の @ref local::LocalNetworkVersion 、または、アプリケーションバージョンが参加を行おうとしたセッションのものよりも高いです。ローカル通信時にのみ返ります。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultLocalLowerVersion                                          自分の @ref local::LocalNetworkVersion 、または、ローカル通信バージョンが参加を行おうとしたセッションのものよりも低いです。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @retval ResultLocalHigherVersion                                         自分の @ref local::LocalNetworkVersion 、または、ローカル通信バージョンが参加を行おうとしたセッションのものよりも高いです。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @endif
    @retval ResultErrorOccurred                                        内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    */
    Result GetJoinSessionResultInternal() const;
    //! @endcond

    /*!
    @brief    指定されたセッションの設定を更新します。
    @details  非同期に指定したセッションの設定を更新します。
              セッションホストであるときに実行できます。
              処理の完了は @ref IsUpdateSessionSettingCompleted でチェックし、結果は @ref GetUpdateSessionSettingResult で確認できます。
    @param[in] pUpdateSessionSetting                 変更する設定をまとめる UpdateSessionSetting 派生クラスのポインタ
    @return                                          非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultInvalidState            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、ホストであることを確認してください。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。インターネットマッチメイク時や LAN マッチメイク時には、無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultWifiOff                 無線がオフになっています。ローカル通信時にのみ返ります[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSleep                   スリープ状態です。ローカル通信時にのみ返ります[:viewer][:handling-shutdownNetwork]
    */
    Result UpdateSessionSettingAsync(const session::UpdateSessionSetting* pUpdateSessionSetting);

    /*!
    @brief   UpdateSessionSettingAsync の非同期処理の完了をチェックします。
    @details @ref UpdateSessionSettingAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return  非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    */
    bool IsUpdateSessionSettingCompleted() const;

    /*!
    @brief   UpdateSessionSettingAsync の非同期処理の結果を取得します。
    @details @ref UpdateSessionSettingAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。
    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  nn::pia::ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval  nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval  nn::pia::ResultSessionIsNotFound 非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval  nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval  nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    */
    Result GetUpdateSessionSettingResult() const;

    /*!
    @brief      参加中のセッション情報を取得します。
    @details    セッション情報は自動的には更新されません。セッション情報の更新は @ref RequestSessionInfoAsync で行う必要があります。
    @return     参加中のセッション情報を返します。
    */
    const session::ISessionInfo* GetSessionInfo() const;

    /*!
    @brief      参加中のセッション情報の取得要求を行います。
    @details    参加中のセッション情報の取得要求を行う非同期処理を開始します。
    開始した処理は @ref framework::Framework::Update() を定期的に呼び出すことで進行します。
    処理の完了は @ref IsRequestSessionInfoCompleted でチェックし、結果は @ref GetRequestSessionInfoResult で確認できます。
    <br>
    インターネット通信の場合、ゲームサーバーアクセスを行います。1 分間に 10 回以上の呼び出しを行ってはいけません。
    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。セッションに参加していないか、他の非同期処理が行われていないか確認してください。[:progErr]
    @see    IsRequestSessionInfoCompleted, GetRequestSessionInfoResult
    */
    Result RequestSessionInfoAsync();

    /*!
    @brief      RequestSessionInfoAsync の非同期処理の完了をチェックします。
    @details    @ref RequestSessionInfoAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see        RequestSessionInfoAsync, GetRequestSessionInfoResult
    */
    bool IsRequestSessionInfoCompleted() const;

    /*!
    @brief      RequestSessionInfoAsync の非同期処理の結果を取得します。
    @details    @ref RequestSessionInfoAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。
    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidState              非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval nn::pia::ResultSessionIsNotFound         非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLost   ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    */
    Result GetRequestSessionInfoResult() const;

#if NN_PIA_EXPERIMENT_FRAMEWORK_JOINT_SESSION
    /*!
    @cond  PRIVATE
    @brief        ランダムマッチメイクでジョイントセッションに参加する非同期処理を開始します。
    @return       成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    Result JoinRandomJointSessionAsyncInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief        JoinRandomJointSessionAsyncInternal の非同期処理の完了をチェックします。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsJoinRandomJointSessionCompletedInternal();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief        JoinRandomJointSessionAsyncInternal の非同期処理の結果を取得します。
    @return       成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    Result GetJoinRandomJointSessionResultInternal() const;
    //! @endcond

    /*!
    @brief        ジョイントセッションを解散する非同期処理を開始します。
    @return       成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval       nn::pia::ResultInvalidArgument  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval       ResultInvalidState            API を呼び出すタイミングが誤っています。ジョイントセッションホストでないステーションが呼び出した際にも返ります。[:progErr]
    */
    Result DestroyJointSessionAsync();

    /*!
    @brief        DestroyJointSessionAsync の非同期処理の完了をチェックします。
    @retval true  成功です。
    @retval false 失敗です。
    */
    bool IsDestroyJointSessionCompleted() const;

    /*!
    @brief        DestroyJointSessionAsync の非同期処理の結果を取得します。
    @return       成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval nn::pia::ResultInvalidState                    処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultInvalidArgument                 非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost         ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerMaintenance           ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultCompanionStationIsOffline       同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval nn::pia::ResultCompanionStationIsLeft          同行するクライアントとの接続が切断しました。[:handling-cleanup]
    @retval nn::pia::ResultSessionMigrationFailed          セッション移行処理に失敗しました。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError                [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted        ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                    ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin  他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    */
    Result GetDestroyJointSessionResult() const;

    /*!
    @brief      セッション移行中であるかを判定します。
    @return     セッション移行中であれば true を返します。
    */
    bool IsSessionMigrating();
#endif


    /*!
    @cond  PRIVATE
    @brief クリーンアップ、サーバー情報解除、ログアウト、インターネット切断を行います。
    */
    void LeaveSessionAndFinalize();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                           インターネットサービスの初期化処理です。
    @return                          IsSuccess() が真となる Result を返します。
    @retval ResultAlreadyInitialized 既に初期化が完了しています。[:progErr]
    @retval ResultInvalidArgument    RegisterXxxInitializeSetting() で渡した引数が誤っています。
    */
    Result InitializeInetService();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                           Pia の初期化処理です。
    @return                          IsSuccess() が真となる Result を返します。
    @retval ResultAlreadyInitialized 既に初期化が完了しています。[:progErr]
    */
    Result InitializeSession();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief  インターネットサービス（NEX など）、ネットワークサービス（NIFM など）、ソケットを初期化します。
    @return IsSuccess() が真となる Result を返します。
    */
    Result StartupNetwork();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                        ネットワークインターフェースの初期化処理です。
    @return                       IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument RegisterXxxInitializeSetting() で渡した引数が誤っています。
    @retval ResultSdkError        SDK の API 呼び出しに失敗しました。
    */
    Result InitializeNetworkInterface();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief                        ソケットの初期化処理です。
    @return                       IsSuccess() が真となる Result を返します。
    @retval ResultInvalidArgument RegisterXxxInitializeSetting() で渡した引数が誤っています。
    @retval ResultSdkError        SDK の API 呼び出しに失敗しました。
    */
    Result InitializeSocket();
    //! @endcond

    Result Login(common::AsyncContext* pAsyncContext, bool isInChangeStateAsync);
    Result LoginAsync();
    bool IsLoginCompleted() const;
    Result GetLoginResult() const;

    /*!
    @cond  PRIVATE
    @brief                                Pia のスタートアップ処理です。
    @return                               IsSuccess() が真となる Result を返します。
    @retval ResultInvalidState            API を呼び出すタイミングが誤っています。
    @retval ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval ResultSdkError                SDK の API 呼び出しに失敗しました。
    */
    Result StartupSession();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief Pia のクリーンアップ処理です。
    */
    void CleanupSession();
    //! @endcond

    Result Logout(common::AsyncContext* pAsyncContext, bool isInChangeStateAsync);
    Result LogoutAsync();
    bool IsLogoutCompleted() const;
    Result GetLogoutResult() const;

    void LogoutAndWait();

    /*!
    @cond  PRIVATE
    @brief ネットワークの終了処理です。
    */
    void CleanupNetwork();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief ソケットの終了処理です。
    */
    void FinalizeSocket();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief ソケットの終了処理です。
    */
    void FinalizeNetworkInterface();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief インターネットサービスの終了処理です。
    */
    void FinalizeInetService();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief Pia の終了処理です。
    */
    void FinalizeSession();
    //! @endcond

    /*!
    @brief                  ハンドリングタイプを State に変換します。
    @param[in] handlingType 変換するハンドリングタイプです。
    @return                 変換後の State を返します。
    */
    Framework::State ConvertHandlingTypeToState(Result::HandlingType handlingType);

    /*!
    @brief  CloneProtocol を取得します。
    @return CloneProtocol を返します。
    */
    clone::CloneProtocol* GetCloneProtocol();

    /*!
    @brief                        PiaSync を用いた同期通信を開始します。
    @param[in] usingDataIdBitmap  SyncProtocol::Initialize() に渡した Setting 構造体で正の値のサイズを設定した同期データの内、どの同期データを自分の送信対象にするか設定します。
    @param[in] delay              入力遅延を指定します。
    @param[in] sendPeriod         送信間隔（SyncProtocol::Step() を何回呼ぶごとに送信処理を行うか）を指定します。
    @return                       成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState    この関数を呼び出せる SyncProtocol::State ではありません。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument 引数の指定が不正です。[:progErr]
    */
    Result SyncStart(int32_t usingDataIdBitmap, uint32_t delay, uint32_t sendPeriod);

    /*!
    @brief                     同期通信を終了します。
    @return                    成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState この関数を呼び出せる SyncProtocol::State ではありません。または、通信中ではありません。[:progErr]
    */
    Result SyncEnd();

    /*!
    @brief                     自分だけ単独で同期通信を終了します。
    @return                    成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState この関数を呼び出せる SyncProtocol::State ではありません。または、すでに終了フレームが設定されています。または、通信中ではありません。[:progErr]
    */
    Result SyncEndAlone();

    /*!
    @brief                             同期通信を 1 フレーム進めます。
    @return                            成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState         通信中ではありません。[:progErr]
    @retval ResultDataIsNotArrivedYet  同期データが他のステーションから届いていないためフレームを進めることができませんでした。framework::Framework::Update() を呼んだあと再度この関数を呼ぶ必要があります。[:handling]
    @retval ResultDataIsNotSet         自分が送信する同期データがまだ設定されていません。[:progErr]
    @retval ResultTemporaryUnavailable 単独同期終了処理中です。
    */
    Result SyncStep();

    /*!
    @brief      @ref sync::SyncProtocol::State_Synchronized となってからのフレーム数を取得します。

    @details    @ref sync::SyncProtocol::State_Synchronized の間、@ref sync::SyncProtocol::Step に成功すると 1 進みます。

    @return     @ref sync::SyncProtocol::State_Synchronized となってからのフレーム数です。
    @ref sync::SyncProtocol::State_Synchronized でなければ 0 を返しますが、@ref sync::SyncProtocol::State_Synchronized となった最初のフレームも 0 となりますので注意してください。
    */
    uint32_t GetSyncFrameNo();

    /*!
    @brief  同期通信の進行状態を取得します。
    @return 同期通信の進行状態が返されます。
    */
    sync::SyncProtocol::State GetSyncState();

    /*!
    @brief                        PiaSync を用いて受信する同期データを取得します。
    @param[in]  stationId         取得する同期データを送信するステーションの StationId です。
    @param[in]  dataId            取得する同期データのデータ ID です。
    @param[out] pBuffer           同期データを格納するバッファのポインタです。
    @return                       成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState    この関数を呼び出せる SyncProtocol::State ではありません。または、同期データを取得できないフレームです。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument 引数の指定が不正です。[:progErr]
    */
    Result GetSyncData(StationId stationId, uint32_t dataId, void* pBuffer);

    /*!
    @brief                        PiaSync を用いて送信する同期データを設定します。
    @param[in] dataId             設定する同期データの ID です。
    @param[in] pBuffer            設定する同期データです。
    @return                       成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState    この関数を呼び出せる SyncProtocol::State ではありません。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument 引数の指定が不正です。[:progErr]
    @retval ResultAlreadyExists   同期データは既に設定されています。[:progErr]
    */
    Result SetSyncData(uint32_t dataId, const void* pBuffer);

    /*!
    @brief                        同期データを取得できないフレームかどうかを取得します。
    @details                      同期通信中の入力遅延変更機能を使用している場合は、同期データを取得できないフレームがあるため、この関数で同期データを取得できないフレームかを確認する必要があります。
    同期通信中の入力遅延変更機能を使用していない場合は、この関数を使用する必要はありません。
    @param[out] isNoDataFrame     同期データを取得できないフレームであるかです。
    @return                       成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState    呼び出し状態が不正です。[:progErr]
    @retval ResultInvalidArgument 引数の指定が不正です。[:progErr]
    */
    Result GetIsNoDataFrame(bool* isNoDataFrame);

    /*!
    @brief      実際に設定されている入力遅延の値を取得します。
    @return     入力遅延のフレーム数です。
    */
    uint32_t GetDelay();

    /*!
    @brief      入力遅延の最大値を取得します。
    @return     入力遅延のフレーム数の最大値です。
    */
    uint32_t GetMaxDelay();

    /*!
    @brief                        入力遅延の変更依頼を同期データとして設定します。
    @details                      依頼する入力遅延は、初期化時に @ref sync::SyncProtocol::FrameDelayMax 以下の値である必要があります。
                                  依頼する入力遅延として 0 を指定した場合は、変更依頼を行わないものとして扱われます。
                                  この関数が成功しても、依頼した入力遅延の値に変更されることは保証されません。
                                  同じフレームで複数の依頼を受け取った場合は、依頼された入力遅延の値の最大値が優先されます。
                                  入力遅延の変更依頼を同期データとして受け取った時点で、すでに他の依頼による入力遅延の変更中であった場合は、依頼は無視されます。
    @param[in]  newDelay          依頼する入力遅延の値です。
    @return                       成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState    呼び出し状態が不正です。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument 引数の指定が不正です。[:progErr]
    @retval ResultAlreadyExists   既に設定されています。[:progErr]
    */
    Result RequestToChangeDelay(uint32_t newDelay);

    /*!
    @brief  stationId のリストを取得します。
    @param[out] stationIdListBuf       stationId のリストを格納するためのバッファです。
    @param[in]  stationIdListBufLength stationIdListBuf のサイズ（リストの要素数）を指定してください。 
    @param[out] stationNum             ステーション数が格納されます。
    @return                            成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidArgument      引数の指定が不正です。[:progErr]
    @retval ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    */
    Result GetStationIdList(StationId* stationIdListBuf, uint32_t stationIdListBufLength, uint32_t &stationNum);

    /*!
    @brief      StationId に対応する Station を得ます。
    @param[in]  id StationId です。
    @return     指定した StationId に対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなかったり、StationId が無効の場合は NULL ポインタが返ります。
    */
    transport::Station* GetStation(StationId id);

    /*!
    @brief  セッション参加処理の進行状態を取得します。
    @return 参加処理の進行状態を返します。
    */
    State GetState() const
    {
        return m_State;
    }

    void SetState(State state)
    {
        m_State = state;
    }

    /*!
    @brief    UnreliableProtocol を用いて、データを全ステーションに向けて送信します。

    @details データを送信します。ただし、この関数を呼び出しただけでは実際のデータ送信は行われません。
    この関数は、アプリケーションが渡したデータを Pia 内部のパケット生成処理用バッファにコピーします。
    その後、 PiaTransport のディスパッチ処理が実行されることで、パケット生成処理用バッファにコピー
    されていたデータ群がパケットにまとめられ、 Pia 内部の送信スレッドに渡されます。実際の送信処理は、
    この送信スレッドが定期的にパケットを無線モジュールに渡すことで行われます。

    @param[in]  pData    送信したいデータの先頭を指すポインタ。
    @param[in]  dataSize 送信データサイズ。単位はバイトです。 GetSendDataSizeMax() が返す値を超過してはいけません。

    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval   ResultInvalidArgument 引数に誤りがあります。送信データサイズが過大であった場合にも、このエラーが返されます。[:progErr]
    @retval   ResultInvalidState    UnreliableProtocol::Initialize() が呼ばれていない可能性があります。[:progErr]
    @retval   ResultNotInCommunication 通信が可能な状態ではありません。[:handling]
    @retval   ResultTemporaryUnavailable ジョイントセッション処理が進行中のため、一時的に API を利用できない状態です。[:handling]
    */
    Result SendToAllUnreliable(const void* pData, uint32_t dataSize);

    /*!
    @brief  ReliableProtocol を用いて、データを全ステーションに向けて送信します。

    @details この関数でセットされた送信データは、パケットに対して適切なサイズに分割され内部のバッファに保持されます。
    その後の @ref framework::Framework::Update() の中で順次パケットが作成され送信されていきます。
    送信してからしばらくたっても到達が確認できないデータは再送されます。
    再送までの時間は RTT に応じて自動的に調整されます。
    到達が確認できたデータはバッファから削除され、バッファはその後のデータ送信のために使う事ができるようになります。

    @param[in] pData                    送信するデータのアドレスです。
    @param[in] dataSize                 送信するデータのサイズ（バイト）です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState         ReliableProtocol が初期化されていないか、session のセットアップが完了していない可能性があります。[:progErr]
    @retval ResultInvalidArgument      引数が誤っています。[:progErr]
    @retval ResultNotInCommunication 指定された送信先と通信中ではありません。自分か指定した相手がセッション内に存在しません。[:handling]
    @retval ResultBufferIsFull         送信バッファに空きがありません。頻発する場合は、 Initialize() で指定する送信バッファのデータ数を増やすことを検討してください。[:handling]
    */
    Result SendToAllReliable(const void* pData, uint32_t dataSize);

    /*!
    @brief    UnreliableProtocol を用いて、データを特定のステーションに向けて送信します。

    @details データを送信します。ただし、この関数を呼び出しただけでは実際のデータ送信は行われません。
    この関数は、アプリケーションが渡したデータを Pia 内部のパケット生成処理用バッファにコピーします。
    その後、 PiaTransport のディスパッチ処理が実行されることで、パケット生成処理用バッファにコピー
    されていたデータ群がパケットにまとめられ、 Pia 内部の送信スレッドに渡されます。実際の送信処理は、
    この送信スレッドが定期的にパケットを無線モジュールに渡すことで行われます。

    @param[in]  destId   送信先の StationId 。
    @param[in]  pData    送信したいデータの先頭を指すポインタ。
    @param[in]  dataSize 送信データサイズ。単位はバイトです。 GetSendDataSizeMax() が返す値を超過してはいけません。

    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval   ResultInvalidArgument 引数に誤りがあります。送信データサイズが過大であった場合にも、このエラーが返されます。[:progErr]
    @retval   ResultInvalidState    UnreliableProtocol::Initialize() が呼ばれていなかったり、session のセットアップが完了していない可能性があります。[:progErr]
    @retval   ResultNotInCommunication 通信が可能な状態ではありません。[:handling]
    @retval   ResultTemporaryUnavailable ジョイントセッション処理が進行中のため、一時的に API を利用できない状態です。[:handling]
    @retval   ResultNotFound 指定された送信先が見つかりません。
    */
    Result SendUnreliable(StationId destId, const void* pData, uint32_t dataSize);

    /*!
    @brief  ReliableProtocol を用いて、データを特定のステーションに向けて送信します。

    @details この関数でセットされた送信データは、パケットに対して適切なサイズに分割され内部のバッファに保持されます。
    その後の @ref framework::Framework::Update() の中で順次パケットが作成され送信されていきます。
    送信してからしばらくたっても到達が確認できないデータは再送されます。
    再送までの時間は RTT に応じて自動的に調整されます。
    到達が確認できたデータはバッファから削除され、バッファはその後のデータ送信のために使う事ができるようになります。

    @param[in] destId                   送信先の @ref StationId です。
    @param[in] pData                    送信するデータのアドレスです。
    @param[in] dataSize                 送信するデータのサイズ（バイト）です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState         ReliableProtocol が初期化されていないか、session のセットアップが完了していない可能性があります。[:progErr]
    @retval ResultInvalidArgument      引数が誤っています。[:progErr]
    @retval ResultNotInCommunication   指定された送信先と通信中ではありません。自分か指定した相手がセッション内に存在しません。[:handling]
    @retval ResultBufferIsFull         送信バッファに空きがありません。頻発する場合は、 Initialize() で指定する送信バッファのデータ数を増やすことを検討してください。[:handling]
    @retval ResultTemporaryUnavailable ジョイントセッション処理が進行中のため、一時的に API を利用できない状態です。[:handling]
    @retval ResultNotFound             指定された送信先が見つかりません。[:handling]
    */
    Result SendReliable(StationId destId, const void* pData, uint32_t dataSize);

    /*!
    @brief    UnreliableProtocol を用いて、受信したデータをバッファに読み取ります。

    @details データを受信します。ただし、この関数呼び出しが無線モジュールの受信処理実行のトリガーとなっている
    わけではありません。
    無線モジュールからパケットを受信する処理は、 Pia 内部の受信スレッドによって定期的に実行されています。
    この受信スレッドが受信したパケットは、 PiaTransport のディスパッチ処理が実行されるタイミングで
    パケット解析処理用バッファにコピーされ、データが復元されます。
    この関数は、このようにして復元されたデータをアプリケーション側で用意したバッファにコピーする処理を実行します。

    @param[out]  pSrcId        読み取ったデータの送信元 StationId が書き込まれます。
    @param[out]  pRecvBuf      データの読み取り先バッファです。
    @param[out]  pRecvDataSize 実際に読み取ることのできたデータサイズが書き込まれます。単位はバイトです。
    @param[in]   recvBuffSize  データの読み取り先バッファのサイズを指定します。単位はバイトです。

    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval   ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval   ResultInvalidState    UnreliableProtocol::Initialize() が呼ばれていなかったり、session のセットアップが完了していない可能性があります。[:progErr]
    @retval   ResultBufferShortage  読み取りバッファのサイズが不足しています。 recvBuffSize で指定したサイズを上回る大きさのデータが到着していた場合に返されます。[:progErr]
    @retval   ResultNoData          受信バッファが空っぽで、読み取ることのできるデータがありませんでした。[:handling]
    @retval   ResultNotInCommunication 通信が可能な状態ではありません。[:handling]
    @retval   ResultTemporaryUnavailable ジョイントセッション処理が進行中のため、一時的に API を利用できない状態です。[:handling]
    */
    Result ReceiveUnreliable(StationId* pSrcId, void* pRecvBuf, uint32_t* pRecvDataSize, uint32_t recvBuffSize);

    /*!
    @brief  ReliableProtocol を用いて、データを受信します。

    @details @ref framework::Framework::Update() のタイミングで受信したパケットは解析され、内部のバッファに蓄積されていきます。
    SendReliable() で分割されたデータがすべて届いていることが確認できているデータはこの関数で結合され取得できます。
    内部バッファもこのタイミングで解放され、次の受信のために使われます。

    @param[out] pSrcId               受信データの送信元の @ref StationId を書き込むバッファへのポインタです。
    @param[out] pRecvBuf             受信データを書き込むバッファへのポインタです。
    @param[out] pRecvDataSize        受信データのサイズを書き込むバッファへのポインタです。
    @param[in]  recvBuffSize         受信データを書き込むバッファのサイズ（バイト）です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState      初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
    @retval ResultNotInCommunication   通信中でありません。セッションに参加していない状態と同じです。[:handling]
    @retval ResultNoData            受信したデータはありません。[:handling]
    @retval ResultBrokenData        受信したデータが大きすぎて受信バッファに入りきらないか渡されたバッファに書ききれません。セッションのスタートアップからやり直す必要があります。
    この Result はチートなどによって送信側が不正なデータを送ってきたときに発生する可能性があるため実行時にハンドリングする必要がありますが、
    送信側の挙動が正常な場合はこの Result が返らないようにプログラムを修正する必要があります。[:handling]
    */
    Result ReceiveReliable(StationId* pSrcId, void* pRecvBuf, uint32_t* pRecvDataSize, uint32_t recvBuffSize);

    /*!
    @brief    @ref Framework::SendUnreliable() 、@ref Framework::SendToAllUnreliable() で一度に送信可能なデータサイズの最大値を取得します。単位はバイトです。
    @details  この API 呼び出しで返される値は、アプリケーションによるパケットへの署名の有無に影響されます。
    @details  セッション参加後にこの API を呼び出す必要があります。これよりも早いタイミングで呼び出すと、正しい値が返されないことがあります。
    @return   SendUnreliable(), SendToAllUnreliable() で一度に送信可能なデータサイズの最大値が返されます。単位はバイトです。

    @see      SendUnreliable, SendToAllUnreliable
    */
    uint32_t GetSendDataSizeMax();

    /*!
    @brief     inet の初期化設定項目の登録を行います。
    @param[in] inetSetting  setting 登録する設定項目です。
    */
    void RegisterInitializeInetSetting(const InitializeInetSetting& inetSetting);

    /*!
    @brief             lan の初期化設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializeLanSetting(const InitializeLanSetting& setting);

    /*!
    @brief             local の初期化設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializeLocalSetting(const InitializeLocalSetting& setting);

#if NN_PIA_FRAMEWORK_ENABLE_PLAY_SETTING
    /*!
    @brief             play の初期化設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializePlaySetting(const InitializePlaySetting& setting);
#endif

    /*!
    @brief             transport の初期化設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializeTransportSetting(const InitializeTransportSetting& setting);

    /*!
    @brief             clone の初期化設定項目の登録を行います。
    @details           setting.isAutoInitialize を true としてこのメソッドを呼ぶと、自動で clone の初期化を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializeCloneSetting(const InitializeCloneSetting& setting);

    /*!
    @brief             sync の初期化設定項目の登録を行います。
    @details           setting.isAutoInitialize を true としてこのメソッドを呼ぶと、自動で sync の初期化を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializeSyncSetting(const InitializeSyncSetting& setting);

    /*!
    @brief             session の初期化設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializeSessionSetting(const InitializeSessionSetting& setting);

    /*!
    @brief             NetworkInterface の初期化設定項目の登録を行います。
    @if NIN_DOC
    @details           setting.isAutoInitialize を true としてこのメソッドを呼ぶと、自動的に NIFM の初期化を行います。
    @else
    @details           setting.isAutoInitialize を true としてこのメソッドを呼ぶと、自動的に AC の初期化を行います。
    @endif
    @param[in] setting 登録する設定項目です。
    */
    void RegisterStartupNetworkSetting(const StartupNetworkSetting& setting);

    /*!
    @brief             session のスタートアップ設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterStartupSessionSetting(const StartupSessionSetting& setting);

    /*!
    @brief             セッションを構築する際に必要な設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterJoinRandomSessionSetting(const JoinRandomSessionSetting& setting);

#if NN_PIA_EXPERIMENT_FRAMEWORK_JOINT_SESSION
    /*!
    @brief             ジョイントセッションを構築する際に必要な設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterJoinRandomJointSessionSetting(const JoinRandomSessionSetting& setting);
#endif

    /*!
    @cond  PRIVATE
    @brief             CPUProfilingを行う際に必要な設定項目の登録を行います。
    @param[in] setting 登録する設定項目です。
    */
    void RegisterInitializeTuneSetting(const InitializeTuneSetting& setting);
    //! @endcond

    /*!
    @brief             inet の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializeInetSetting();

    /*!
    @brief             lan の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializeLanSetting();

    /*!
    @brief             local の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializeLocalSetting();

#if NN_PIA_FRAMEWORK_ENABLE_PLAY_SETTING
    /*!
    @brief             play の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializePlaySetting();
#endif

    /*!
    @brief             transport の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializeTransportSetting();

    /*!
    @brief             clone の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializeCloneSetting();

    /*!
    @brief             sync の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializeSyncSetting();

    /*!
    @brief             session の初期化設定項目の登録解除を行います。
    */
    void UnregisterInitializeSessionSetting();

    /*!
    @brief             NetworkInterface の初期化設定項目の登録解除を行います。
    */
    void UnregisterStartupNetworkSetting();

    /*!
    @brief             session のスタートアップ設定項目の登録解除を行います。
    */
    void UnregisterStartupSessionSetting();

    /*!
    @brief             セッションを構築する際に必要な設定項目の登録解除を行います。
    */
    void UnregisterJoinRandomSessionSetting();

#if NN_PIA_EXPERIMENT_FRAMEWORK_JOINT_SESSION
    /*!
    @brief             セッションを構築する際に必要な設定項目の登録解除を行います。
    */
    void UnregisterJoinRandomJointSessionSetting();
#endif

    /*!
    @cond  PRIVATE
    @brief             CPUProfilingを行う際に必要な設定項目の登録解除を行います。
    */
    void UnregisterInitializeTuneSetting();
    //! @endcond

    /*!
    @brief             全ての設定項目の登録解除を行います。
    */
    void UnregisterSettingAll();

    /*!
    @cond  PRIVATE
    @brief                       セッションの状態が変化した場合に呼び出されるコールバックの関数の宣言を行います。
    @param[in] eventType         状態変化の内容を示す列挙型です。
    @param[in] stationId         状態変化が発生した対象の StationId です。
    @param[in] pApplicationParam アプリケーションが登録したパラメータを示すポインタです。
    */
    void SessionEventCallback(session::Session::EventType eventType, StationId stationId, void* pApplicationParam);
    //! @endcond

    /*!
    @brief             レイテンシエミュレーション、およびパケロスエミュレーションに必要な設定をセットします。Release ビルド時には失敗します。
    @param[in] emulationParamArg    レイテンシエミュレーション、およびパケロスエミュレーション用の設定です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState      Pia の初期化が行われていない可能性があります。または Release ビルドです。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
    */
    Result SetEmulationParam(const SetEmulationParamArg& emulationParamArg);

    /*!
    @cond  PRIVATE
    @brief             NetworkInterface 接続処理が正常に行われたか確認します。
    */
    bool IsNetworkInterfaceConnected();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief             NetworkType を取得します。
    */
    framework::NetworkType GetNetworkType()
    {
        return m_Setting.networkType;
    }
    //! @endcond

#if NN_PIA_CTR
    /*!
    @brief ユニーク ID からローカル通信 ID を作成します。

    @details 業務部から割り当てられた 20bit のユニーク ID （タイトル固有の ID）から ローカル通信で使用する 32bit 値 (ローカル通信 ID) を生成します。<br>
    業務部からユニーク ID が割り当てられていない場合は、ゲームソフト試作用のユニーク ID ( 0xFF000 ～ 0xFF3FF ) を使用する必要があります。
    ローカル通信時に @ref State_SessionInitialized まで遷移済みであれば、有効な値が返ります。

    @attention    製品版では必ず業務部から割り当てられた ユニーク ID を使用する必要があります。

    @param[in] uniqueId ユニーク IDです。複数タイトル間で通信したい場合はどちらか片方のユニーク ID を指定する必要があります。
    @param[in] isTrial  ユニーク ID が製品版と体験版で共通だった場合の対処用フラグです。製品版-体験版間での通信を行いたくない場合、 体験版側で true を指定する必要があります。製品版は常に false が指定されるよう注意してください。

    @return 作成されたローカル通信 ID が返ります。
    */
    uint32_t CreateLocalCommunicationId(uint32_t uniqueId, bool isTrial = false) const;
#endif

#if NN_PIA_NINTENDOSDK
    /*!
    @if NIN_DOC
    @brief             ResultSdkViewerResultError 発生時に SDK の API から返った nn::Result を取得します。
    @endif
    */
    nn::Result GetLastSdkResult()
    {
        return m_LastSdkResult;
    }

    /*!
    @cond  PRIVATE
    @if NIN_DOC
    @brief             ResultSdkViewerResultError 発生時に SDK の API から返った nn::Result をセットします。
    @endif
    */
    void SetLastSdkResult(nn::Result result)
    {
        m_LastSdkResult = result;
    }
    //! @endcond
#endif

private:
    enum AsyncProcessingName
    {
        AsyncProcessingName_Nothing = 0,
        AsyncProcessingName_ChangeState,
        AsyncProcessingName_HandleError,
        AsyncProcessingName_CreateSession,
        AsyncProcessingName_BrowseSession,
        AsyncProcessingName_JoinSession,
        AsyncProcessingName_Login,
        AsyncProcessingName_Logout
    };

    Result UpdateCommon();
    Result UpdateInetService();
    Result CheckConnectionSession();
    Result CheckConnectionInet();
    Result CheckConnectionLocal();
    Result CheckConnectionLan();

    // 汎用内部コンテキストチェック
    void ResetInternalAsyncState();

    // 汎用内部コンテキストチェック
    bool IsAsyncProcessCompleted(AsyncProcessingName processingName) const;

    // 汎用内部コンテキストチェック
    Result GetAsyncProcessResult(AsyncProcessingName processingName) const;

    void SetupSessionStartupSettingCommon(session::Session::StartupSetting* pStartupSetting) const;

private:
    CloneFacade* m_pCloneFacade;
    InetService* m_pInetService;
    LanFacade* m_pLanFacade;
    LocalFacade* m_pLocalFacade;
    NetworkService* m_pNetworkService;
    ReckoningFacade* m_pReckoningFacade;
    SessionFacade* m_pSessionFacade;
    SyncFacade* m_pSyncFacade;
    TransportFacade* m_pTransportFacade;
    TuneFacade* m_pTuneFacade;

#if NN_PIA_NINTENDOSDK
    nn::Result m_LastSdkResult;
#endif

    bool m_StartupFlag;
    State m_State;
    bool m_IsLeaving;
    ChangeStateJob* m_pChangeStateJob;
    LoginJob* m_pLoginJob;
    common::AsyncContext m_InternalContext;
    AsyncProcessingName m_CurrentAsyncProcessingName;
    Setting m_Setting;
    InitializeInetSetting m_InitializeInetSetting;
    InitializeLanSetting m_InitializeLanSetting;
    InitializeLocalSetting m_InitializeLocalSetting;
#if NN_PIA_FRAMEWORK_ENABLE_PLAY_SETTING
    InitializePlaySetting m_InitializePlaySetting;
#endif
    InitializeTransportSetting m_InitializeTransportSetting;
    InitializeCloneSetting m_InitializeCloneSetting;
    InitializeSyncSetting m_InitializeSyncSetting;
    InitializeSessionSetting m_InitializeSessionSetting;
    StartupNetworkSetting m_StartupNetworkSetting;
    StartupSessionSetting m_SessionStartupSetting;
    JoinRandomSessionSetting* m_pJoinRandomSessionSetting;
    InitializeTuneSetting m_CpuProfilingSetting;

#if NN_PIA_EXPERIMENT_FRAMEWORK_JOINT_SESSION
    // n2908: ジョイントセッション用 RandomSessionSetting
    JoinRandomSessionSetting* m_pJoinRandomJointSessionSetting;
    // n2908: セッションマイグレーション中チェック用
    bool m_IsSessionMigrating;
#endif

#if NN_PIA_CTR
    static const uint32_t MaxSendThreadBufferNum = 32;
    static const uint32_t MaxReceiveThreadBufferNum = 32;
#else
    static const uint32_t MaxSendThreadBufferNum = 64;
    static const uint32_t MaxReceiveThreadBufferNum = 64;
#endif
    static Framework* s_pInstance;

private:
    Framework();

    NN_PIA_DISALLOW_COPY(Framework);
};
}
}
}  // end of namespace nn::pia::framework
