﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/framework/framework_Definitions.h>

#if NN_PIA_FRAMEWORK_ENABLE_NEX

#include <nn/pia/framework/framework_InetService.h>

#include <nn/pia/inet/inet_NexFacade.h>
#include <nn/pia/inet/inet_Api.h>
#include <nn/pia/inet/inet_NexNetworkFactory.h>
#include <nn/pia/inet/inet_NexFacade.h>

#if NN_PIA_NINTENDOSDK
#include <nn/account.h>
#include <nn/account/account_Result.h>
#endif

namespace nn
{
namespace pia
{
namespace framework
{

/*!
@brief NEX サービスの管理クラスです。
*/
class NexService : public InetService
{
private:
    static const nn::nex::Core::ThreadMode NexThreadMode;

public:

    /*!
    @brief Initialize() に NEX のパラメータを渡すための構造体です。
     */
    struct InitializeNexServiceSetting : public InitializeInetServiceSetting
    {
        uint8_t* nexMemory;             //!< NEX で使用するメモリ。@if NIN_DOC 4 KB アラインメントされている必要があります。また、16 KB 以上である必要があります。 @endif
        uint32_t totalMemorySize;       //!< nexMemory のサイズです。
        uint32_t emergencyMemorySize;   //!< nexMemory の内、NEX のヒープ枯渇時に使用する緊急用メモリとして割り当てるサイズです。
    };

    /*!
    @cond  PRIVATE
    @brief コンストラクタです。
    */
    NexService();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief デストラクタです。
    */
    virtual ~NexService();
    //! @endcond

    /*!
    @brief  NEX の NgsFacade のインスタンスを取得します。
    @return NgsFacade のインスタンスへのポインタが返されます。
    */
    nn::nex::NgsFacade* GetNgsFacade();

    /*!
    @brief  NEX サービスにアクセスするための証明書（Credentials）のインスタンスを取得します。
    @return 証明書のインスタンスへのポインタが返されます。
    */
    nn::nex::Credentials* GetCredential();

    /*!
    @cond  PRIVATE
    @brief             inet 初期化処理です。
    @param[in] setting inet の初期化で必要なパラメータです。
    @return            成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    virtual Result InitializeInet(const InitializeInetSetting& setting);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief Inet 終了処理です。
    */
    virtual void FinalizeInet();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief             初期化処理です。
    @param[in] setting 各種パラメータが書き込まれた Setting 構造体です。
    @return            成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    virtual Result InitializeInetService(const InitializeInetServiceSetting& setting);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief 終了処理です。
     */
    virtual void FinalizeInetService();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief ディスパッチ処理を行います。
    */
    virtual Result Dispatch();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief サービス接続状態をチェックします。
    */
    virtual Result CheckConnection();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief  transport の初期化に必要な NetworkFactory を取得します。
    @return NetworkFactory が返されます。
    */
    virtual transport::NetworkFactory* GetNetworkFactory();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief             フレンドサーバーへのログイン処理です。
    @return            成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    virtual Result LoginFriendServer();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief  ログアウト処理です。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    virtual Result LogoutFriendServer();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief              ゲームサーバーへのログインを行います。
    @param[in] pSetting ログインに必要なパラメータです。
    @param[in] timeout  タイムアウトまでの時間を指定します。
    @return             成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    virtual Result LoginGameServer(const InitializeInetSetting* pSetting, uint32_t timeout = 30 * 1000);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief          ログイン処理中かをチェックします。
    @return         ログイン処理中であれば true が返ります。
    */
    virtual bool IsInLoginProcess()
    {
        return (m_pCallContext->GetState() == nn::nex::CallContext::CallInProgress);
    }
    //! @endcond

    virtual bool IsInLogoutProcess()
    {
        return (m_pCallContext->GetState() == nn::nex::CallContext::CallInProgress);
    }

    /*!
    @cond  PRIVATE
    @brief             ゲームサーバーへのログインの前処理を行います。
    @return            成功すれば、 IsSuccess() が true を返す Result が返されます。
    @endif
    */
    virtual Result PreLoginProcess(const InitializeInetSetting* pSetting);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief          ログイン前処理中かをチェックします。
    @return         ログイン前処理中であれば true が返ります。
    */
    virtual bool IsInPreLoginProcess();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief             ゲームサーバーへのログイン完了後の処理を行います。
    @return            成功すれば、 IsSuccess() が true を返す Result が返されます。
    @endif
    */
    virtual Result PostLoginProcess();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief               ゲームサーバーからログアウトを行います。
    @param[in] isBlock   true なら処理が完了するまでブロックします。
                         false なら非同期で処理を行います。
                         ログアウト処理の完了は IsInLogoutProcess() で確認してください。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    virtual Result LogoutGameServer(bool isBlock, bool isInChangeStateAsync);
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief             ログアウト結果を取得します。
    @return            成功すれば、 IsSuccess() が true を返す Result が返されます。
    @endif
    */
    virtual Result GetLogoutOutcome();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief  ゲームサーバーへの接続状態を取得します。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    Result IsNgsConnected();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief ログイン情報を設定します。
    */
    void SetLoginInfo(const inet::NexFacade::LoginInfo& info)
    {
        m_LoginInfo = info;
    }
    //! @endcond

private:
    bool IsInitialized()
    {
        return m_IsNexInitialized;
    }

    // NEX 用ヒープの初期化（ExpHeap）
    static void InitializeRuntimeMemory(uint8_t* pNexMemory, uint32_t totalMemorySize, uint32_t emergencyMemorySize);

    // アプリケーション定義のメモリ確保関数
    static void* AllocFunction(size_t size);

    // アプリケーション定義のメモリ解放関数
    static void FreeFunction(void* pointer);

    // AllocFunction から呼ばれる内部メモリ関数
    static void* InnerAlloc(uint32_t size);

    // FreeFunction から呼ばれる内部メモリ関数
    static void InnerFree(void* ptr);

    //緊急用のメモリを解放します
    static void ReleaseReservedMemory();

    // 緊急用のメモリを確保します
    static void AcquireReservedMemory();

    // NEX 用ヒープの終了処理（ExpHeap）
    static void FinalizeRuntimeMemory();

private:
    bool m_IsNexInitialized;
    bool m_IsInetInitialized;
    nn::nex::NgsFacade* m_pNgsFacade;
    nn::nex::ProtocolCallContext* m_pCallContext;
    inet::NexFacade::LoginInfo m_LoginInfo;
    inet::NexNetworkFactory m_NexNetworkFactory;

#if NN_PIA_NINTENDOSDK_NX
#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 1, 1))
    char m_NsaIdToken[nn::account::NetworkServiceAccountIdTokenLengthMax];
    size_t m_NsaIdTokenSize;
    nn::account::NetworkServiceAccountId m_NsaId;
    nn::account::AsyncContext m_AccountAsyncContext;
    nn::os::SystemEvent m_AccountSystemEvent;
#endif
#endif

    NN_PIA_DISALLOW_COPY(NexService);
};
}
}
}  // end of namespace nn::pia::framework

#endif // NN_PIA_FRAMEWORK_ENABLE_NEX
