﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>
#include <nn/pia/transport/transport_StationLocation.h>

// inet モジュールの非メンバ関数群です。

namespace nn
{
namespace pia
{
namespace inet
{


/*!
    @brief  inet モジュールが初期化された状態であるかどうかを返します。

    @return 初期化されていれば true、されていなければ false が返されます。
*/
bool IsInitialized(void);


/*!
    @brief  （デバッグ用） @ref inet::SetDebugSetting に渡すデバッグ機能設定用構造体です。
*/
struct DebugSetting
{
    DebugSetting() : 
        natTraversalFailureRatioForHost(0),
        natTraversalFailureRatioForClient(0),
        isNatTypeCheckFailure(false),
        isDnsResolutionFailure(false),
        falsifiedNatTraversalFailureResult(ResultNotSet())
    {
    }
    uint8_t natTraversalFailureRatioForHost;        //!< ホストとの NAT トラバーサル処理を失敗させる確率
    uint8_t natTraversalFailureRatioForClient;      //!< クライアントとの NAT トラバーサル処理を失敗させる確率
    bool isNatTypeCheckFailure;                     //!< NAT タイプの判定処理を失敗させるかどうか
    bool isDnsResolutionFailure;                    //!< NAT タイプの判定処理時に行う DNS 名前解決処理を失敗させるかどうか
    Result falsifiedNatTraversalFailureResult;      //!< NAT トラバーサル処理に失敗した際に返す Result を差し替える Result。ResultSuccess を指定する場合は、NAT トラバーサル失敗時に返る可能性のある Result をランダムに選択して返します。
};

/*!
    @brief  inet モジュールを初期化します。

    @return 無事に初期化が完了すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultAlreadyInitialized  既に初期化は終了しています。[:progErr]

    @see    Setting
*/
Result Initialize();


/*!
    @brief  inetモジュールを終了します。
*/
void Finalize(void);


/*!
    @brief  セットアップの開始を宣言します。
    @details inetモジュールのシングルトンなどを作成する前に呼び出す必要があります。
            アプリケーションが inet モジュールのシングルトンを一つも作成しない場合であっても、
            このAPIと @ref EndSetup は必ず呼び出す必要があります。

    @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized  inet モジュールの初期化関数が呼ばれていません。[:progErr]
    @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
*/
Result BeginSetup(void);


/*!
    @brief  セットアップの終了を宣言します。
    @details inet モジュールのシングルトンなどを作成し終えた後に呼美出す必要があります。
            この API 呼び出しにより、inet モジュールのシングルトンなどが消費する
            メモリを確定し、ライブラリ内部でのメモリ管理を最適化します。
            アプリケーションが inet モジュールのシングルトンを一つも作成しない場合であっても、
            @ref BeginSetup とこの API は必ず呼び出す必要があります。

    @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized  inet モジュールの初期化関数が呼ばれていません。[:progErr]
    @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
*/
Result EndSetup(void);


/*!
    @cond PRIVATE
    @brief BeginSetup() ～ EndSetup()間の状態であるかどうかを返します。
    @endcond
*/
bool IsInSetupMode(void);

/*!
    @brief      （デバッグ用） inet のデバッグ用機能について設定します。
    @details    本関数は inet のインスタンスが存在しない状態で呼出可能です。
                DebugSetting 構造体の isEnableNatErrorEmulation を true に設定した場合、NATトラバーサルがエラーで失敗する設定となります。
                本関数を呼び出さなかった場合、本関数が関連するデバッグ用機能は全て無効（使用しない）設定となります。
    @attention  製品版 ROM では、本関数を呼び出さないように注意してください。

    @return 関数の実行結果を返します。処理が成功した場合、IsSuccess() が真となる Result を返します。この関数は常に成功します。
*/
Result SetDebugSetting(const DebugSetting& debugSetting);

/*!
    @cond PRIVATE
    @brief デバッグ用機能の設定を返します。
    @endcond
*/
DebugSetting* GetDebugSetting();

/*!
@cond PRIVATE
@endcond
*/
bool IsBehindNat(const transport::StationLocation& location);

/*!
@cond PRIVATE
@endcond
*/
bool IsPublic(const transport::StationLocation& location);

/*!
@cond PRIVATE
@endcond
*/
bool IsGlobal(const transport::StationLocation& location);

/*!
@cond PRIVATE
@endcond
*/
bool IsEdmMapping(const transport::StationLocation& location);

/*!
@cond PRIVATE
@endcond
*/
bool IsEimMapping(const transport::StationLocation& location);
}
}
}  // end of namespace nn::pia::inet
