﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>
#include <nn/pia/session/session_CommunitySearchCriteria.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  コミュニティ ID をコミュニティの検索条件に指定するクラスです。
  @details 検索結果として得られるコミュニティの最大数は SetResultRange で指定する値になりますが、
           nn::pia::session::Session::CreateInstance 呼び出し時に指定する
           nn::pia::session::Session::Setting 構造体の browsedCommunityInfoListNum 以下の値を
           指定する必要があります。

  @see nn::pia::session::CommunitySearchCriteria
 */
class NexCommunitySearchCriteriaCommunityId : public nn::pia::session::CommunitySearchCriteria
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexCommunitySearchCriteriaCommunityId();

    /*!
    @brief      コンストラクタ。
    @param[in]  communityIdArray        指定するコミュニティIDのリスト。
    @param[in]  communityIdArraySize    指定するコミュニティIDの個数。
                                        session::Session::Setting 構造体の browsedCommunityInfoListNum 以下、かつCommunityInfoListSizeMax 以下の値である必要があります。
    */
    NexCommunitySearchCriteriaCommunityId(uint32_t* communityIdArray, uint32_t communityIdArraySize);

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexCommunitySearchCriteriaCommunityId();

    /*!
    @brief      コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexCommunitySearchCriteriaCommunityId(const NexCommunitySearchCriteriaCommunityId& rhs);

    /*!
    @brief      検索条件として指定するコミュニティ ID のリストをセットします。
    @param[in]  communityIdArray        指定するコミュニティIDのリスト。
    @param[in]  communityIdArraySize    指定するコミュニティIDの個数。
                                         session::Session::Setting 構造体の browsedCommunityInfoListNum 以下、かつCommunityInfoListSizeMax 以下の値である必要があります。

    @return     コミュニティ ID のリストの指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  コミュニティ ID のリストが NULL、または ID の個数が多すぎます。[:progErr]
    */
    Result SetCommunityIdList(uint32_t* communityIdArray, uint32_t communityIdArraySize);

    /*!
    @brief      検索結果の取得範囲を指定します。
    @details    offset に 0xFFFFFFFF を指定すると、検索結果をランダムに並び替えた上で
    指定されたエントリー数を取得するように振る舞います。
    @param[in]  offset  検索条件にマッチしたエントリーリストから検索結果として取得するエントリー群の開始位置。
    @param[in]  size    検索条件にマッチしたエントリーリストから検索結果として取得するエントリー数。
                        session::Session::Setting 構造体の browsedCommunityInfoListNum 以下、かつSearchCriteriaResultSizeMax 以下の値である必要があります。

    @return     処理に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval     nn::pia::ResultInvalidArgument  引数の値が範囲外です。[:progErr]
    */
    virtual Result SetResultRange(uint32_t offset, uint32_t size);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    NexCommunitySearchCriteriaCommunityId& operator=(const NexCommunitySearchCriteriaCommunityId& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexCommunitySearchCriteriaCommunityId& rhs);

    /*!
    @cond PRIVATE
    @brief      指定したコミュニティ ID のリストを取得します。
    @return     コミュニティ ID のリスト。
     */
    const uint32_t* GetCommunityIdList() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したコミュニティ ID のリストのサイズを取得します。
    @return     コミュニティ ID のリストのサイズ。
     */
    uint32_t GetCommunityIdListSize() const;
    //! @endcond

    /*!
    @brief      検索条件をリセットします。
     */
    virtual void Reset();

private:
    uint32_t m_CommunityIdArray[CommunityInfoListSizeMax];
    uint32_t m_CommunityIdArraySize;
};
}
}
}  // end of namespace nn::pia::inet

#endif
