﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_DateTime.h>
#include <nn/pia/session/session_CreateCommunitySetting.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
  @brief  コミュニティ作成処理に関する設定を管理するクラスです。

  @see session::CreateCommunitySetting
 */
class NexCreateCommunitySetting : public session::CreateCommunitySetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexCreateCommunitySetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexCreateCommunitySetting();

    /*!
    @brief      コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexCreateCommunitySetting(const NexCreateCommunitySetting& rhs);

    /*!
    @cond PRIVATE
    @brief 指定した項目の示すビットマスクの各ビットに対応する項目
    */
    enum NexCreateCommunitySettingMask
    {
        NexCreateCommunitySettingMask_StartDate = 0,                                                                //!< 開始日時
        NexCreateCommunitySettingMask_EndDate,                                                                      //!< 終了日時
        NexCreateCommunitySettingMask_Description,                                                                  //!< 説明文字列
        NexCreateCommunitySettingMask_Attribute,                                                                    //!< 属性
        NexCreateCommunitySettingMask_ApplicationData = NexCreateCommunitySettingMask_Attribute + AttributeSizeMax, //!< アプリケーション定義データ
        NexCreateCommunitySettingMask_CommunityUserPassword                                                         //!< ユーザーパスワード
    };
    //! @endcond

    /*!
    @brief      構築するコミュニティの参加可能開始日時を指定します。
    @param[in]  startDate 構築するコミュニティの参加可能開始日時。
    @return     コミュニティの参加可能開始日時の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  日時が不正です。[:progErr]
    */
    Result SetParticipationStartDate(const common::DateTime& startDate);

    /*!
    @cond PRIVATE
    */
    const common::DateTime& GetParticipaitonStartDate() const;
    bool IsSetParticipationStartDate() const;
    //! @endcond

    /*!
    @brief      構築するコミュニティの参加可能終了日時を指定します。
    @param[in]  endDate 構築するコミュニティの参加可能終了日時。
    @return     コミュニティの参加可能終了日時の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  日時が不正です。[:progErr]
    */
    Result SetParticipationEndDate(const common::DateTime& endDate);

    /*!
    @cond PRIVATE
    */
    const common::DateTime& GetParticipaitonEndDate() const;
    bool IsSetParticipationEndDate() const;
    //! @endcond

    /*!
    @brief      構築するコミュニティに説明文字列を指定します。
    @param[in]  description 構築するコミュニティに指定する説明文字列。文字列の長さは DescriptionLengthMax 以下である必要があります。
    @return     説明文字列のコピーに成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または説明文字列の最大長を超えています。[:progErr]
    */
    Result SetDescription(const common::String& description);

    /*!
    @cond PRIVATE
    */
    const common::String& GetDescription() const;
    bool IsSetDescription() const;
    //! @endcond

    /*!
    @brief      構築するコミュニティに属性を指定します。
    @param[in]  index   構築するコミュニティに指定する属性のインデックス。AttributeSizeMax 以下である必要があります。
    @param[in]  value   構築するコミュニティに指定する属性の値。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外です。[:progErr]
    */
    Result SetAttribute(uint32_t index, uint32_t value);

    /*!
    @cond PRIVATE
    */
    uint32_t GetAttribute(uint32_t index) const;
    bool IsSetAttribute(uint32_t index) const;
    //! @endcond

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @param[in]  pData   構築するコミュニティに指定するアプリケーション定義データのバッファ。
    @param[in]  size    構築するコミュニティに指定するアプリケーション定義データのサイズ。ApplicationDataBufferSizeMax 以下である必要があります。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータサイズを取得します。
    @return     アプリケーション定義データサイズが返されます。
    */
    uint32_t GetApplicationDataSize() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 
    */
    void CopyApplicationDataToVector(void* pVectorTmp) const;
    bool IsSetApplicationData() const;
    //! @endcond

    /*!
    @brief      作成したコミュニティにユーザーパスワードを設定します。
    @param[in]  userPassword   設定するユーザーパスワード。パスワードの文字列は SessionUserPasswordLengthMax 以下である必要があります。
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
     */
    Result SetCommunityUserPassword(const common::String& userPassword);

    /*!
    @cond PRIVATE
    */
    const common::String& GetCommunityUserPassword() const;
    bool IsSetCommunityUserPassword() const;
    //! @endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexCreateCommunitySetting& operator=(const NexCreateCommunitySetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexCreateCommunitySetting& rhs);

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    void Trace(uint64_t flag) const;
    //! @endcond

protected:
    common::DateTime m_StartDateTime;
    common::DateTime m_EndDateTime;
    common::FixedString<DescriptionLengthMax> m_Description;
    uint32_t m_Attribute[AttributeSizeMax];
    uint8_t m_ApplicationDataBuffer[ApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;
    common::FixedString<SessionUserPasswordLengthMax> m_UserPassword;

    uint32_t m_SetCondMask;

private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
