﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_InetAddress.h>
#include <nn/pia/session/session_CreateSessionSetting.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
  @brief  セッション構築処理に関する設定を管理するクラスです。

  @see nn::pia::session::CreateSessionSetting
 */
class NexCreateSessionSetting : public session::CreateSessionSetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexCreateSessionSetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexCreateSessionSetting();

    /*!
    @cond PRIVATE
    @brief 指定した項目の示すビットマスクの各ビットに対応する項目
    */
    enum NexCreateSessionSettingMask
    {
        NexCreateSessionSettingMask_GameMode = 0,                                                                                               //!< ゲームモード
        NexCreateSessionSettingMask_SessionType,                                                                                                //!< セッションタイプ
        NexCreateSessionSettingMask_Attribute,                                                                                                  //!< 属性
        NexCreateSessionSettingMask_ApplicationData = NexCreateSessionSettingMask_Attribute + AttributeSizeMax,                                 //!< アプリケーション定義データ
        NexCreateSessionSettingMask_OpenSession,                                                                                                //!< 参加可能かどうか
        NexCreateSessionSettingMask_SessionUserPassword,                                                                                        //!< ユーザーパスワード
        NexCreateSessionSettingMask_MatchmakeSessionOption,                                                                                     //!< マッチメイクセッションオプション
        NexCreateSessionSettingMask_SelectionPriority,                                                                                          //!< マッチメイクの優先度
        NexCreateSessionSettingMask_ExtraParticipant,                                                                                           //!< (予約)
        NexCreateSessionSettingMask_ScoreBasedSettingIndex,                                                                                     //!< スコア換算方法の設定インデックス（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_ScoreBasedRatingValue,                                                                                      //!< レーティング値（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_ScoreBasedDisconnectionRate,                                                                                //!< 切断率（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_ScoreBasedViolationRate,                                                                                    //!< 違反率（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_ScoreBasedCountryCode,                                                                                      //!< 国コード（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_ScoreBasedGeoIp,                                                                                            //!< 位置情報（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_ScoreBasedOverrideIp,                                                                                       //!< デバッグ用IPアドレス（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_Community,                                                                                                  //!< 関連コミュニティ
        NexCreateSessionSettingMask_MatchmakeKeyword,                                                                                           //!< あいことば
        NexCreateSessionSettingMask_AdditionalParamString,                                                                                      //!< 追加文字列パラメータ（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_AdditionalParamValue = NexCreateSessionSettingMask_AdditionalParamString + AdditionalParamStringSizeMax,    //!< 追加パラメータ（スコアベースマッチメイク使用時）
        NexCreateSessionSettingMask_Max = NexCreateSessionSettingMask_AdditionalParamValue + AdditionalParamKeySizeMax
    };
    //! @endcond

    /*!
    @brief      構築するセッションのゲームモードを指定します。
    @param[in]  gameMode    構築するセッションに指定するゲームモード。
    */
    void SetGameMode(uint32_t gameMode);

    /*!
    @cond PRIVATE
    */
    uint32_t GetGameMode() const;
    bool IsSetGameMode() const;
    //! @endcond

    /*!
    @brief      構築するセッションのセッションタイプとホストマイグレーション時にセッションを自動的に参加締切状態にする設定を指定します。
    @details   デフォルトではセッションタイプは SessionType_Anybody、ホストマイグレーション時にセッションを自動的に参加締切状態に変更しない設定になります。
    @param[in]  sessionType         構築するセッションに指定するセッションタイプ。
    @param[in]  isAutoCloseSession  ホストマイグレーション時にセッションを自動的に参加締切状態にする設定。
    */
    void SetSessionType(SessionType sessionType, bool isAutoCloseSession = false);

    /*!
    @cond PRIVATE
    */
    SessionType GetSessionType() const;
    bool IsAutoCloseSession() const;
    bool IsSetSessionType() const;
    //! @endcond

    /*!
    @brief      構築するセッションに属性を指定します。
    @details    設定されなかったインデックスの属性には 0 が設定されます。
    @param[in]  index   構築するセッションに指定する属性のインデックス。AttributeSizeMax 以下である必要があります。
    @param[in]  value   構築するセッションに指定する属性の値。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外です。[:progErr]
    */
    Result SetAttribute(uint32_t index, uint32_t value);

    /*!
    @cond PRIVATE
    */
    uint32_t GetAttribute(uint32_t index) const;
    bool IsSetAttribute(uint32_t index) const;
    //! @endcond

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @param[in]  pData   構築するセッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    構築するセッションに指定するアプリケーション定義データのサイズ。ApplicationDataBufferSizeMax 以下である必要があります。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @cond PRIVATE
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    virtual uint32_t GetApplicationDataSize() const;
    virtual bool IsSetApplicationData() const;
    //! @endcond

    /*!
    @brief      構築したセッションが参加可能で構築されるか、参加不可で構築されるかを指定します。指定しなかった場合、参加不可で構築されます。
    @param[in]  isOpen  true を設定した場合、セッションは始めから参加可能な状態で構築されます。
     */
    void SetOpenSession(bool isOpen);

    /*!
    @cond PRIVATE
     */
    bool IsOpenSession() const;
    bool IsSetOpenSession() const;
    //! @endcond

    /*!
    @brief      作成したセッションにユーザーパスワードを設定します。
    @param[in]  userPassword   設定するユーザーパスワード。パスワードの文字列は SessionUserPasswordLengthMax 以下である必要があります。
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
     */
    Result SetSessionUserPassword(const common::String& userPassword);

    /*!
    @cond PRIVATE
    */
    const common::String& GetSessionUserPassword() const;
    bool IsSetSessionUserPassword() const;
    //! @endcond

    /*!
    @brief      作成したセッションにあいことばマッチメイク用のあいことばを設定します。
    @param[in]  keyword   設定するあいことばマッチメイク用のあいことば。あいことばの文字列は SessionMatchmakeKeywordLength 以下である必要があります。
    @return     あいことばの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはあいことばの最大長を超えています。[:progErr]
    */
    Result SetSessionMatchmakeKeyword(const common::String& keyword);

    /*!
    @cond PRIVATE
    */
    const common::String& GetSessionMatchmakeKeyword() const;
    bool IsSetSessionMatchmakeKeyword() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      構築するセッションのマッチメイクセッションオプションを指定します。
    @param[in]  option    構築するセッションに指定するマッチメイクセッションオプション。
    */
    void SetMatchmakeSessionOption(uint32_t option);
    //! @endcond

    /*!
    @cond PRIVATE
    */
    uint32_t GetMatchmakeSessionOption() const;
    bool IsSetMatchmakeSessionOption() const;
    //! @endcond

    /*!
    @brief      同じ条件のセッションが見つかった際のマッチメイクの優先度を設定します。
    @details    作成したセッションのマッチメイクの優先度を設定します。
                設定しなかった場合、0 が設定されます。
                nn::pia::inet::SelectionMethod_BroadenRangeWithSelectionPriority,
                SelectionMethod_ScoreBased で検索を行った場合のみ有効です。
                nn::pia::inet::SessionSelectionPriorityMax を超えた値を引数にした場合、nn::pia::inet::SessionSelectionPriorityMax が設定されます。

    @param[in]  priority   設定するマッチメイクの優先度
     */
    void SetSelectionPriority(uint8_t priority);

    /*!
    @cond PRIVATE
     */
    uint8_t GetSelectionPriority() const;
    bool IsSetSelectionPriority() const;
    //! @endcond

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用するレーティング値を設定します。
    @param[in]  ratingValue        レーティング値
    */
    void SetRatingValue(uint32_t ratingValue);

    /*!
    @cond PRIVATE
     */
    uint32_t GetRatingValue() const;
    bool IsSetRatingValue() const;
    //! @endcond

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用する切断率を設定します。
    @param[in]  disconnectionRate        切断率
    */
    void SetDisconnectionRate(uint32_t disconnectionRate);

    /*!
    @cond PRIVATE
     */
    uint32_t GetDisconnectionRate() const;
    bool IsSetDisconnectionRate() const;
    //! @endcond

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用する違反率を設定します。
    @param[in]  violationRate        違反率
    */
    void SetViolationRate(uint32_t violationRate);

    /*!
    @cond PRIVATE
     */
    uint32_t GetViolationRate() const;
    bool IsSetViolationRate() const;
    //! @endcond

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に位置情報 (緯度経度、国コード) を使用するかどうかを設定します。
    @details    位置情報を使うかどうかをセットしない場合は距離の比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  use 位置情報を使用するかどうか
    */
    void SetUseGeoIp(bool use);

    /*!
    @cond PRIVATE
     */
    bool GetUseGeoIp() const;
    bool IsSetUseGeoIp() const;
    //! @endcond

    /*!
    @if NIN_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードは ISO 3166-1 alpha-2 によるアルファベット 2 文字（大文字）の国コードを設定します。
    @details    @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
                位置情報が得られた場合はここで指定した国コードは使用されません。
                国コードをセットしない場合は @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
                国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード。ISO 3166-1 alpha-2 によるアルファベット 2 文字（大文字）の文字列である必要があります。
    @return     国コードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  国コードの文字列長が 2 以外、または国コードに指定した文字種が不正です。[:progErr]
    @endif
    */
    Result SetCountryCode(const common::String& countryCode);

    /*!
    @if CTR_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはプラットフォームで定義された値を設定します。
    @details    @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
                位置情報が得られた場合はここで指定した国コードは使用されません。
                国コードをセットしない場合は @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
                国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード
    @endif
    @if CAFE_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはプラットフォームで定義された値を設定します。
    @details    @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
                位置情報が得られた場合はここで指定した国コードは使用されません。
                国コードをセットしない場合は @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
                国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード
    @endif
    */
    void SetCountryCode(uint32_t countryCode);

    /*!
    @cond PRIVATE
    */
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
    const common::String* GetCountryCode() const;
#else
    uint32_t GetCountryCode() const;
#endif
    //! @endcond

    /*!
    @cond PRIVATE
    */
    bool IsSetCountryCode() const;
    //! @endcond

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用するデバッグ用の IP アドレスを設定します。
    @details    自身の IP アドレスの代わりにこの IP アドレスを使用して GeoIP による位置情報を取得します。 デバッグ用にのみ使用可能です。 
    @param[in]  inetAddress     代わりに使用する IP アドレス
    @return     IP アドレスの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  指定した IP アドレスが不正です。[:progErr]
    */
    Result SetOverrideIpAddress(const common::InetAddress& inetAddress);

    /*!
    @cond PRIVATE
     */
    const common::InetAddress& GetOverrideIpAddress() const;
    bool IsSetOverrideIpAddress() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief スコアベースマッチメイク用パラメータが設定されているかどうか
    */
    bool IsSetScoreBasedParameter() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      スコアベースマッチメイク指定時に、比較に使用するパラメータを設定します。
    @param[in]  index           パラメータのインデックス。AdditionalParamKeySizeMax 以下である必要があります。
    @param[in]  keyString       パラメータの名前。AdditionalParamKeyLength 以下である必要があります。
    @param[in]  value           パラメータの値
    @return     パラメータの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  パラメータのインデックスが範囲外、またはパラメータの名前の文字列が NULL、文字コードが不正、最大文字列長を超えています。[:progErr]
    */
    Result SetAdditionalScoreBasedParameterValue(uint32_t index, const common::String& keyString, const ParamVariant& value);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      スコアベースマッチメイク指定時に、比較に使用するパラメータを設定します。
    @param[in]  index           パラメータのインデックス。AdditionalParamStringSizeMax 以下である必要があります。
    @param[in]  keyString       パラメータの名前。AdditionalParamKeyLength 以下である必要があります。
    @param[in]  value           パラメータの文字列。AdditionalParamStringLength 以下である必要があります。
    @return     パラメータの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  パラメータのインデックスが範囲外、またはパラメータの名前やパラメータの文字列が NULL、文字コードが不正、最大文字列長を超えています。[:progErr]
    */
    Result SetAdditionalScoreBasedParameterString(uint32_t index, const common::String& keyString, const common::String& value);
    //! @endcond

    /*!
    @cond PRIVATE
    */
    const common::String* GetAdditionalScoreBasedParameterValueKey(uint32_t index) const;
    const ParamVariant& GetAdditionalScoreBasedParameterValue(uint32_t index) const;
    bool IsSetAdditionalScoreBasedParameterValue(uint32_t index) const;
    const common::String* GetAdditionalScoreBasedParameterStringKey(uint32_t index) const;
    const common::String* GetAdditionalScoreBasedParameterString(uint32_t index) const;
    bool IsSetAdditionalScoreBasedParameterString(uint32_t index) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief アプリケーション定義のデータをコピーします。
    */
    void CopyApplicationDataToVector(void* pVectorTmp) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    void SetExtraParticipantNum(uint16_t num);
    uint16_t GetExtraParticipantNum() const;
    bool IsSetExtraParticipantNum() const;
    //! @endcond

    /*!
    @brief 関連付けるコミュニティの ID を設定します。
    @details inet::NexSessionSearchCriteria::SetCommunityId で同じコミュニティ ID を指定して検索したプレイヤーのみに検索結果として見つけられるセッションになります。
    また、NEX のランキング機能との連携を行う際に必要となります。ランキング機能との連携については、NEX の永続ギャザリングに関するドキュメントを参照してください。
     */
    void SetCommunityId(uint32_t gid);

    /*!
    @cond PRIVATE
    @brief 関連付けるコミュニティの ID を取得します。
     */
    uint32_t GetCommunityId() const;
    bool IsSetCommunityId() const;
    //! @endcond

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexCreateSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexCreateSessionSetting(const NexCreateSessionSetting& rhs);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexCreateSessionSetting& operator=(const NexCreateSessionSetting& rhs);

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

protected:
    uint32_t m_GameMode;
    SessionType m_SessionType;
    bool m_IsAutoCloseSession;
    uint32_t m_Attribute[AttributeSizeMax];
    uint8_t m_ApplicationDataBuffer[ApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;
    bool m_OpenParticipation;
    uint8_t m_ProgressScore;
    uint32_t m_RatingValue;
    uint32_t m_ViolationRate;
    uint32_t m_DisconnectionRate;
    bool m_UseGeoIp;
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
    common::FixedString<CountryCodeLengthMax> m_CountryCode;
#else
    uint32_t m_CountryCode;
#endif
    common::InetAddress m_OverrideIpAddress;
    uint16_t m_ExtraParticipantNum;
    uint32_t m_CommunityId;
    common::FixedString<SessionUserPasswordLengthMax> m_UserPassword;
    uint32_t m_MatchmakeSessionOption;

    common::FixedString<AdditionalParamKeyLength> m_AdditionalParamValueKey[AdditionalParamKeySizeMax];
    struct ParamVariant m_AdditionalParamValue[AdditionalParamKeySizeMax];

    common::FixedString<AdditionalParamKeyLength> m_AdditionalParamStringKey[AdditionalParamStringSizeMax];
    common::FixedString<AdditionalParamStringLength> m_AdditionalParamString[AdditionalParamStringSizeMax];

    common::FixedString<SessionMatchmakeKeywordLength> m_MatchmakeKeyword;

    uint32_t m_SetCondMask;

private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
