﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX && NN_PIA_UNDER_DEVELOP

#include <nn/pia/inet/inet_NexSessionTypes.h>
#include <nn/pia/inet/inet_NexCreateSessionSetting.h>
#include <nn/pia/inet/inet_NexUpdateSessionSetting.h>
#include <nn/pia/common/common_AsyncContext.h>

#if NN_PIA_CTR
#pragma diag_suppress 1300  //n1769 ARMCC 用プラグマ。
#endif
#include <nex.h>
#if NN_PIA_CTR
#pragma diag_warning 1300  //n1769 ARMCC 用プラグマ。
#endif

namespace nn
{
namespace pia
{
namespace session
{
// 先行宣言
class IMatchmakeSession;
}
}
}
namespace nn
{
namespace pia
{
namespace inet
{
class NexDebugSessionJob;

/*!
@brief  （デバッグ用）デバッグ用ダミーセッションを扱うクラスです。（製品版には含まれません）
*/
class NexDebugSession : public common::RootObject
{
public:
    /*!
    @brief    インスタンスを作成します（シングルトンパターン）。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultNotInitialized     inet モジュールが未初期化です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]
    @retval   nn::pia::ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    */
    static Result CreateInstance();

    /*!
    @brief インスタンスを破棄します（シングルトンパターン）。
    @details インスタンスが作成されていないときにこの関数が呼び出された場合は、何も実行せずに関数から返ります。
    */
    static void DestroyInstance();

    /*!
    @brief    NexDebugSession インスタンスへのポインタを取得します（シングルトンパターン）。

    @return インスタンスが作成されていないときは、 NULL ポインタが返ります。
    */
    static NexDebugSession* GetInstance()
    {
        return s_pInstance;
    }
    /*!
    @brief   NexDebugSession 機能をスタートアップします。
    @details NexFacade::Bind より後に呼び出す必要があります。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。NexFacade のインスタンス作成、NexFacade::Bind が呼び出されているか確認してください。[:progErr]
    */
    Result Startup();

    /*!
    @brief   NexDebugSession 機能をクリーンアップします。
    @details NexFacade::Unbind より前に呼び出す必要があります。
    */
    void Cleanup();

    /*!
    @brief    ダミーセッションの構築を行います。
    @details  ダミーセッションの構築を行う非同期処理を開始します。
              構築したダミーセッションのホストになります。クライアントはダミーセッションに参加できません。
              開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
              処理の完了は @ref IsCreateDummySessionCompleted でチェックし、結果は @ref GetCreateDummySessionResult で確認できます。
              構築されたダミーセッションのセッション ID は @ref GetCreatedDummySessionId で取得できます。

    @param[in]  createSessionSetting   セッションを作成する際の条件

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、Session::Startup を呼び出していないか確認してください。[:progErr]

    @see      IsCreateDummySessionCompleted, GetCreateDummySessionResult, GetCreatedDummySessionId
     */
    Result CreateDummySessionAsync(const NexCreateSessionSetting& createSessionSetting);

    /*!
    @brief    CreateDummySessionAsync の非同期処理の完了をチェックします。
    @details @ref CreateDummySessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    CreateDummySessionAsync, GetCreateDummySessionResult
     */
    bool IsCreateDummySessionCompleted();

    /*!
    @brief    CreateDummySessionAsync の非同期処理の結果を取得します。
    @details @ref CreateDummySessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidArgument   非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState      21個以上のダミーセッションを構築しようとしました。[:progErr]
    @retval nn::pia::ResultGameServerMaintenance     ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    CreateDummySessionAsync, IsCreateDummySessionCompleted
    */
    Result GetCreateDummySessionResult() const;

    /*!
        @brief    構築したダミーセッションのセッション ID を取得します。
        @return   構築したダミーセッションのセッション ID を返します。
    */
    uint32_t GetCreatedDummySessionId() const;

    /*!
    @brief    ダミーセッションの削除を行います。
    @details  ダミーセッションの削除を行う非同期処理を開始します。
              開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
              処理の完了は @ref IsDestroyDummySessionCompleted でチェックし、結果は @ref GetDestroyDummySessionResult で確認できます。

    @param[in] sessionId   削除するダミーセッションのセッション ID

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、Session::Startup を呼び出していないか確認してください。[:progErr]

    @see      IsDestroyDummySessionCompleted, GetDestroyDummySessionResult
     */
    Result DestroyDummySessionAsync(uint32_t sessionId);

    /*!
    @brief    DestroyDummySessionAsync の非同期処理の完了をチェックします。
    @details @ref DestroyDummySessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    DestroyDummySessionAsync, GetDestroyDummySessionResult
     */
    bool IsDestroyDummySessionCompleted();

    /*!
    @brief    DestroyDummySessionAsync の非同期処理の結果を取得します。
    @details @ref DestroyDummySessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidState     ホストではないセッションを削除しようとしました。[:progErr]
    @retval nn::pia::ResultNotFound         削除しようとしたセッションが見つかりませんでした。[:handling]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    DestroyDummySessionAsync, IsDestroyDummySessionCompleted
    */
    Result GetDestroyDummySessionResult() const;

    /*!
    @brief    ダミーセッションの設定更新を行います。
    @details  ダミーセッションの設定更新を行う非同期処理を開始します。
              開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
              処理の完了は @ref IsUpdateDummySessionCompleted でチェックし、結果は @ref GetUpdateDummySessionResult で確認できます。

    @param[in] sessionId            設定更新するダミーセッションのセッション ID
    @param[in] updateSessionSetting セッションの更新設定

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、Session::Startup を呼び出していないか確認してください。[:progErr]

    @see      IsUpdateDummySessionCompleted, GetUpdateDummySessionResult
     */
    Result UpdateDummySessionAsync(uint32_t sessionId, const NexUpdateSessionSetting& updateSessionSetting);

    /*!
    @brief    UpdateDummySessionAsync の非同期処理の完了をチェックします。
    @details @ref UpdateDummySessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    UpdateDummySessionAsync, GetUpdateDummySessionResult
     */
    bool IsUpdateDummySessionCompleted();

    /*!
    @brief    UpdateDummySessionAsync の非同期処理の結果を取得します。
    @details @ref UpdateDummySessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidArgument  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState     ホストではないセッションを更新しようとしました。[:progErr]
    @retval nn::pia::ResultNotFound         更新しようとしたセッションが見つかりませんでした。[:handling]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    UpdateDummySessionAsync, IsUpdateDummySessionCompleted
    */
    Result GetUpdateDummySessionResult() const;

    /*!
    @cond PRIVATE
    */
    /// @endcond

private:
    // シングルトンインスタンスへのポインタです。
    static NexDebugSession* s_pInstance;

    /*!
    @brief シングルトンパターンのため、コンストラクタは封印します。
    */
    NexDebugSession();

    /*!
    @brief シングルトンパターンのため、デストラクタは封印します。
    */
    virtual ~NexDebugSession();

    /*!
    @brief シングルトンパターンのため、コピーコンストラクタは封印します。
    */
    NexDebugSession(const NexDebugSession&);

    /*!
    @brief シングルトンパターンのため、代入演算子は封印します。
    */
    NexDebugSession& operator=(const NexDebugSession&);

    virtual Result initialize();
    virtual void finalize();

    uint8_t* m_pMatchMakingClientBuffer;
    nn::nex::MatchmakeExtensionClient* m_pMatchMakingClient;

    NexDebugSessionJob* m_pDebugSessionJob;
    bool m_IsStartedup;
    common::AsyncContext m_InternalAsyncContext;
    uint32_t m_AsyncProcessType;
};
}
}
}  // end of namespace nn::pia::inet

#endif
