﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>
#include <nn/pia/inet/inet_IFacade.h>

#include <nn/pia/common/common_AsyncContext.h>
#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_InetAddress.h>
#include <nn/pia/common/common_SignatureSetting.h>

#include <nn/pia/nat/nat_Definitions.h>

#include <nn/pia/transport/transport_StationConnectionInfo.h>

#if NN_PIA_CTR
#pragma diag_suppress 1300
#endif

#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <RendezVous/Core/src/Client/Credentials.h>
#include <RendezVous/Core/src/Common/Constants/NotificationEvents.h>

#include <RendezVous/Services/Jugem/Login/src/Client/NgsFacade.h>
#include <RendezVous/Services/Jugem/Login/src/Client/NgsBridgeInterface.h>
#include <RendezVous/Services/MatchMaking.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Client/MatchmakeExtensionClient.h>
#include <RendezVous/Services/NATTraversal/src/Client/NATTraversalClient.h>

#if NN_PIA_CTR
#pragma diag_default 1300
#endif

namespace nn
{
namespace pia
{
namespace common
{
class Socket;
}
}
}

namespace nn
{
namespace pia
{
namespace nat
{
class NatTraverser;
}
}
}

namespace nn
{
namespace pia
{
namespace inet
{
class IFacade;
class NexNatRelay;
class NexNatTraversalClient;
class NexNotificationDataJob;
class NexNatServerAddressResolveJob;
class SocketOutputStream;
class SocketInputStream;

#if NN_PIA_CTR
class NexBlockListJob;
#endif

/*!
  @brief        NEX ライブラリのファサード（facade、インターフェースラッパー）クラスです。
  @details      nn::pia::inet::Nex*クラス群のファサードではなく、NEX ライブラリのクラス群のファサードクラスになります。
 */
class NexFacade : public IFacade
{
public:
    /*!
    @brief    ログインした NEX ゲームサーバーの情報
    */
    struct LoginInfo
    {
        LoginInfo()
            : pNgsFacade(NULL), gameServerId(0), pAccessKey(NULL)
        {
        }

        nn::nex::NgsFacade* pNgsFacade;                                    //!< ゲームサーバーにログインした nn::nex::NgsFacade インスタンス
        uint32_t gameServerId;                                             //!< ゲームサーバーにログインする際に指定したゲームサーバー ID
        common::String* pAccessKey;                                        //!< ゲームサーバーにログインする際に指定したアクセスキー
    };

    /*!
    @brief    インスタンスを作成します（シングルトンパターン）。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultNotInitialized     inet モジュールが未初期化です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]
    @retval   nn::pia::ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
     */
    static Result CreateInstance(void);

    /*!
    @brief インスタンスを破棄します（シングルトンパターン）。

    @details インスタンスが作成されていないときにこの関数が呼び出された場合は、
           何も実行せずに関数から返ります。
     */
    static void DestroyInstance(void);

    /*!
    @brief    NexFacade インスタンスへのポインタを取得します（シングルトンパターン）。

    @return インスタンスが作成されていないときは、 NULL ポインタが返ります。
     */
    static NexFacade* GetInstance(void)
    {
        return s_pInstance;
    }

    /*!
    @cond PRIVATE
    @brief    nn::nex::StationURLを @ref nn::pia::transport::StationConnectionInfo に変換します。

    @details nn::nex::MatchmakeExtensionClient::GetSessionURLs で取得した接続先情報を Pia のステーション接続情報に変換します。
    変換したステーション接続情報は nn::pia::session::Mesh::JoinMeshAsync で使用できます。

    @param[in]  nexStationUrlList      変換元となる接続情報です。要素数は1か2である必要があります。
    @param[in]  pStationConnectionInfo 変換先となるステーション接続情報です。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument     引数に誤りがあります。[:progErr]
    @endcond
     */
    static Result ConvertNexStationUrlToStationConnectionInfo(
        const nn::nex::qList<nn::nex::StationURL>& nexStationUrlList,
        transport::StationConnectionInfo* pStationConnectionInfo);

    /*!
    @cond PRIVATE
    @brief       NEX の StationURL を PiaTransport の StationLocation に変換します。
    @param[in]   url        NEX の StationURL
    @param[out]  pLocation  PiaTransport の StationLocation
    @see         ConvertStationLocationToNexStationUrl
    @endcond
     */
    static void ConvertNexStationUrlToStationLocation(const nn::nex::StationURL& url, transport::StationLocation* pLocation);

    /*!
    @cond PRIVATE
    @brief      NEX の InetAddress を Pia の InetAddress に変換します。

    @details この関数はスレッドセーフです。
    <br>
    @param[in]  nexAddress   NEX の InetAddress
    @param[out] pAddress    Pia の InetAddress
    @endcond
     */
    static void ConvertNexInetAddressToInetAddress(const nn::nex::InetAddress& nexAddress, common::InetAddress* pAddress);

    /*!
    @cond PRIVATE
    @brief      Pia の InetAddress を NEX の InetAddress に変換します。

    @details この関数はスレッドセーフです。
    <br>
    @param[in]  address   Pia の InetAddress
    @param[out] pNexAddress    NEX の InetAddress
    @endcond
     */
    static void ConvertInetAddressToNexInetAddress(const common::InetAddress& address, nn::nex::InetAddress* pNexAddress);

    /*!
    @cond PRIVATE
    @brief       PiaTransport の StationLocation を NEX の StationURL に変換します。
    @param[in]   location PiaTransport の StationLocation
    @param[out]  pUrl     NEX の StationURL
    @see         ConvertNexStationUrlToStationLocation
    @endcond
     */
    static void ConvertStationLocationToNexStationUrl(const transport::StationLocation& location, nn::nex::StationURL* pUrl);

    /*!
    @cond PRIVATE
    @endcond
     */
    static void ConvertStationLocationToStationUrlNexList(
        const nn::nex::qList<transport::StationLocation>& stationLocationList,
        nn::nex::qList<nn::nex::StationURL>* pStationUrlList);

    /*!
    @cond PRIVATE
    @endcond
     */
    static void ConvertStationUrlToStationLocationNexList(
        const nn::nex::qList<nn::nex::StationURL>& stationUrlList,
        nn::nex::qList<transport::StationLocation>* pStationLocationList);

    /*!
    @cond PRIVATE
    @brief nex::String を nn::pia::common::String に変換
    @param[in]  nexString  変換元の nex::String
    @param[out] pString 変換先の nn::pia::common::String
    @endcond
     */
    static void ConvertNexStringToString(const nn::nex::String& nexString, common::String* pString);

    /*!
    @cond PRIVATE
    @brief nn::nex::qVector<nn::nex::qByte> の セッション共有キーを nn::pia::common::SignatureSetting に変換します。
    
    @details マッチメイク時にゲームサーバーから取得するセッション共有キーをパケットの署名機能に関する設定用クラスの鍵に変換します。
    変換先の nn::pia::common::SignatureSetting にはあらかじめキー用のバッファを設定しておくか、@ref nn::pia::common::SignatureSettingWithKeyBuffer を使用してください。
    変換したパケット署名機能用の設定は nn::pia::session::Mesh::Startup で必要になります。
    
    @param[in]  nexSignatureKey                     変換元の nn::nex::qVector<nn::nex::qByte>
    @param[out] pSignatureSetting                   変換先の nn::pia::common::SignatureSetting

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument      引数に誤りがあります。nn::pia::common::SignatureSetting のバッファサイズが変換元のサイズ以上であることを確認してください。[:progErr]
    @endcond
     */

    static Result ConvertNexSignatureKeyToSignatureSetting(
        const nn::nex::qVector<nn::nex::qByte>& nexSignatureKey,
        common::SignatureSetting* pSignatureSetting);

    /*!
    @cond PRIVATE
    @endcond
     */
    Result CreateProtocols();

    /*!
    @brief    ログインしたゲームサーバーの情報を登録します。

    @details  本関数は NEX ゲームサーバーへのログインに成功した際の nn::nex::NgsFacade クラスのインスタンス、ゲームサーバー ID、アクセスキー を Pia ライブラリに登録します。
              NEX ゲームサーバーにログインしてから NexFacade::Startup を呼び出すまでの間に呼び出す必要があります。

    @param[in] pLoginInfo   ログインした NEX ゲームサーバー情報 LoginInfo のポインタを指定します。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @see      Unbind, Startup
    */
    Result Bind(const LoginInfo* pLoginInfo);

    /*!
    @brief    登録したゲームサーバーの情報を削除します。

    @details  本関数は NEX ゲームサーバーへのログインに成功した際の nn::nex::NgsFacade クラスのインスタンス、ゲームサーバー ID の登録情報を Pia ライブラリから削除します。
    NexFacade::Cleanup を呼び出してから NEX ゲームサーバーからログアウトするまでの間に呼び出す必要があります。
    @see      Bind, Startup
    */
    void Unbind();

    /*!
    @if CTR_DOC
    @brief    ブロックリストにユーザーを追加する非同期処理を開始します。
    @details  開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。また、NEX のディスパッチ関数も同様に定期的に呼び出す必要があります。
              処理の完了は @ref IsAddToBlockListCompleted でチェックし、結果は @ref GetAddToBlockListResult で確認できます。
    <br>
              既にフレンド関係が成立している場合は登録できません。また、既にブロックリストに追加されているユーザーは無視されます。
              ブロックリスト登録後にフレンド関係が成立しても、自動的にブロックリストから削除されることはありません。

    @param[in] pPrincipalIdArray ブロックリストに登録するユーザーのプリンシパル ID を格納した配列のポインタ。
    @param[in] principalIdArraySize ブロックリストに登録するユーザーのプリンシパル ID を格納した配列のサイズ。配列の最大長は 100 です。
    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]

    @see    IsAddToBlockListCompleted, GetAddToBlockListResult
    @endif
    */
    Result AddToBlockListAsync(PrincipalId* pPrincipalIdArray, uint32_t principalIdArraySize);

    /*!
    @if CTR_DOC
    @brief    AddToBlockListAsync の非同期処理の完了をチェックします。
    @details @ref AddToBlockListAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    AddToBlockListAsync, GetAddToBlockListResult
    @endif
    */
    bool IsAddToBlockListCompleted() const;


    /*!
    @if CTR_DOC
    @brief    AddToBlockListAsync の非同期処理の結果を取得します。
    @details @ref AddToBlockListAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultNexInternalError           [:nexInternalError][:viewer][:handling]

    @see    AddToBlockListAsync, IsAddToBlockListCompleted
    @endif
    */
    Result GetAddToBlockListResult() const;

    /*!
    @if CTR_DOC
    @brief    ブロックリストから指定したユーザーを削除する非同期処理を開始します。。
    @details  開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。また、NEX のディスパッチ関数も同様に定期的に呼び出す必要があります。
              処理の完了は @ref IsRemoveFromBlockListCompleted でチェックし、結果は @ref GetRemoveFromBlockListResult で確認できます。

    @param[in] pPrincipalIdArray ブロックリストから削除するユーザーのプリンシパル ID を格納した配列のポインタ。
    @param[in] principalIdArraySize ブロックリストから削除するユーザーのプリンシパル ID を格納した配列のサイズ。配列の最大長は 100 です。
    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]

    @see    IsRemoveFromBlockListCompleted, GetRemoveFromBlockListResult
    @endif
    */
    Result RemoveFromBlockListAsync(PrincipalId* pPrincipalIdArray, uint32_t principalIdArraySize);

    /*!
    @if CTR_DOC
    @brief    RemoveFromBlockListAsync の非同期処理の完了をチェックします。
    @details @ref RemoveFromBlockListAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    RemoveFromBlockListAsync, GetRemoveFromBlockListResult
    @endif
    */
    bool IsRemoveFromBlockListCompleted() const;

    /*!
    @if CTR_DOC
    @brief    RemoveFromBlockListAsync の非同期処理の結果を取得します。
    @details @ref RemoveFromBlockListAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultNexInternalError           [:nexInternalError][:viewer][:handling]

    @see    RemoveFromBlockListAsync, IsRemoveFromBlockListCompleted
    @endif
    */
    Result GetRemoveFromBlockListResult() const;

    /*!
    @if CTR_DOC
    @brief    ブロックリストをクリアする非同期処理を開始します。
    @details  開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。また、NEX のディスパッチ関数も同様に定期的に呼び出す必要があります。
              処理の完了は @ref IsClearBlockListCompleted でチェックし、結果は @ref GetClearBlockListResult で確認できます。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]

    @see    IsClearBlockListCompleted, GetClearBlockListResult
    @endif
    */
    Result ClearBlockListAsync();

    /*!
    @if CTR_DOC
    @brief    ClearBlockListAsync の非同期処理の完了をチェックします。
    @details @ref ClearBlockListAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    ClearBlockListAsync, GetClearBlockListResult
    @endif
    */
    bool IsClearBlockListCompleted() const;

    /*!
    @if CTR_DOC
    @brief    ClearBlockListAsync の非同期処理の結果を取得します。
    @details @ref ClearBlockListAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultNexInternalError           [:nexInternalError][:viewer][:handling]

    @see    ClearBlockListAsync, IsClearBlockListCompleted
    @endif
    */
    Result GetClearBlockListResult() const;

    /*!
    @if CTR_DOC
    @brief    ブロックリストに登録しているユーザーを取得する非同期処理を開始します。
    @details  開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。また、NEX のディスパッチ関数も同様に定期的に呼び出す必要があります。
              処理の完了は @ref IsRequestBlockListCompleted でチェックし、結果は @ref GetRequestBlockListResult で確認できます。
              非同期処理が成功していた場合は、@ref GetBlockList でブロックリストの登録ユーザーを取得できます。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]

    @see    IsRequestBlockListCompleted, GetRequestBlockListResult, GetBlockList
    @endif
    */
    Result RequestBlockListAsync();

    /*!
    @if CTR_DOC
    @brief    RequestBlockListAsync の非同期処理の完了をチェックします。
    @details @ref RequestBlockListAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    RequestBlockListAsync, GetRequestBlockListResult
    @endif
    */
    bool IsRequestBlockListCompleted() const;

    /*!
    @if CTR_DOC
    @brief    RequestBlockListAsync の非同期処理の結果を取得します。
    @details @ref RequestBlockListAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultNexInternalError           [:nexInternalError][:viewer][:handling]

    @see    RequestBlockListAsync, IsRequestBlockListCompleted
    @endif
    */
    Result GetRequestBlockListResult() const;

    /*!
    @if CTR_DOC
    @brief      ブロックリストに登録されているユーザーを取得します。
    @details    ブロックリストの情報取得は @ref RequestBlockListAsync で行う必要があります。
    @param[in] pPrincipalIdArray ブロックリストに登録されているユーザーのプリンシパル ID をコピーする配列のポインタ。
    @param[in] principalIdArraySize ブロックリストに登録されているユーザーのプリンシパル ID をコピーする配列のサイズ。配列の最大長は 100 です。
    @param[out] pBlockListSize 引数に指定した配列にプリンシパル ID をコピーした回数を格納する変数のポインタ。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]

    @see    RequestBlockListAsync
    @endif
    */
    Result GetBlockList(PrincipalId* pPrincipalIdArray, uint32_t principalIdArraySize, uint32_t* pBlockListSize);

    /*!
    @brief    ゲーム固有のイベントをゲームサーバーに通知します。
    @details  同じゲームサーバーにログインしているフレンドに対して、ゲームが独自に定義したイベントを通知します。
    ゲームサーバーにログインし、Bind の呼び出しから Unbind の呼び出しまでの間に呼び出し可能です。
    イベント通知は 1 分間に 10 回の頻度を越えて呼び出してはいけません。
    <br>
    非同期処理中に nn::nex::MatchmakeExtensionClient::UpdateNotificationData を呼び出します。
    <br>
    この API は非推奨のため、本関数も非推奨となります。

    @param[in] gameNotificationType ゲーム定義のイベントタイプ（nn::nex::NotificationEvents::GameNotificationEvent1 ～ 8）を指定します。
    @param[in] param1 イベントのパラメータを指定します。
    @param[in] param2 イベントのパラメータを指定します。
    @param[in] strParam イベントのパラメータ（文字列）を指定します。文字列長は 256 以内です。
    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]
    */
    Result UpdateNotificationDataAsync(nn::nex::NotificationEvents::NotificationEvents gameNotificationType, uint32_t param1, uint32_t param2, const common::String& strParam);

    /*!
    @brief    UpdateNotificationDataAsync の非同期処理の完了をチェックします。
    @details  UpdateNotificationDataAsync を呼び出した後、本関数で非同期処理の完了をチェックします。

    @return   非同期処理が完了していた場合は true, 処理中の場合は false が返ります。
    */
    bool IsUpdateNotificationDataCompleted() const;

    /*!
    @brief    UpdateNotificationDataAsync の非同期処理の結果を取得します。
    @details  UpdateNotificationDataAsync を呼び出し、IsUpdateNotificationDataCompleted で処理が完了した後、本関数で結果を取得します。

    @return   UpdateNotificationDataAsync の非同期処理の結果を返します。 処理が完了した場合、以下を返します。
    @retval   ResultSuccess                   非同期処理が成功しました。
    @retval   nn::pia::ResultInvalidState     UpdateNotificationDataAsync を呼び出すタイミングが間違っているか、非同期処理が完了していません。[:progErr]
    @retval   nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    */
    Result GetUpdateNotificationDataResult() const;

    /*!
    @brief    ゲームサーバーに通知されたゲーム固有のイベントを取得します。
    @details  同じゲームサーバーにログインしているフレンドから通知された、ゲームが独自に定義したイベントを取得します。
    ゲームサーバーにログインし、Bind の呼び出しから Unbind の呼び出しまでの間に呼び出し可能です。
    通常、イベントの取得はイベントハンドラで取得し、本関数によるイベント取得はログイン時のみとします。
    <br>
    非同期処理中に nn::nex::MatchmakeExtensionClient::GetFriendNotificationData を呼び出します。
    <br>
    この API は非推奨のため、本関数も非推奨となります。

    @param[in] gameNotificationTypeBitMask 取得対象とするゲーム定義のイベントタイプを示すビットマスクを指定します。N ビット目が nn::nex::NotificationEvents::GameNotificationEventN にあたります。
    @param[out] pRetrievedEventList 取得したイベントのリストを格納先を指定します。指定したリストは非同期処理完了までインスタンスを保持する必要があります。
    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]
    */
    Result RetrieveNotificationDataAsync(uint8_t gameNotificationTypeBitMask, nn::nex::qList<nn::nex::NotificationEvent>* pRetrievedEventList);

    /*!
    @brief    RetrieveNotificationDataAsync の非同期処理の完了をチェックします。
    @details  RetrieveNotificationDataAsync を呼び出した後、本関数で非同期処理の完了をチェックします。

    @return   非同期処理が完了していた場合は true, 処理中の場合は false が返ります。
    */
    bool IsRetrieveNotificationDataCompleted() const;

    /*!
    @brief    RetrieveNotificationDataAsync の非同期処理の結果を取得します。
    @details  RetrieveNotificationDataAsync を呼び出し、IsRetrieveNotificationDataCompleted で処理が完了した後、本関数で結果を取得します。

    @return   RetrieveNotificationDataAsync の非同期処理の結果を返します。 処理が完了した場合、以下を返します。
    @retval   ResultSuccess                   非同期処理が成功しました。
    @retval   nn::pia::ResultInvalidState     RetrieveNotificationDataAsync を呼び出すタイミングが間違っているか、非同期処理が完了していません。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultGameServerProcessAborted   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultNexInternalError           [:nexInternalError][:viewer][:handling]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    */
    Result GetRetrieveNotificationDataResult() const;

    /*!
    @cond PRIVATE
    @brief    NexFacade 機能をスタートアップします。session::Mesh クラスを直接操作する場合のみ呼出し可能です。
    @details  本関数は NEX ゲームサーバーにログインし、Bind 実行後に呼び出す必要があります。
              本関数の内部で内部処理用クラスのインスタンスを NEX のアロケータを使用して生成します。生成したインスタンスは NEX 内で管理され、nn::nex::NgsFacade の
              インスタンスと共に破棄されます。既に NEX 内でインスタンスが管理されている場合は、新たにインスタンスを生成しません。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @if CTR_DOC
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っているか、Pia のサポート対象外の NEX のスレッドモードが設定されています。（ Pia はアンセーフトランスポートバッファスレッドモード、またはアンセーフユーザースレッドモードしかサポートしません。メジャーバージョンが 3 の NEX ではアンセーフトランスポートバッファスレッドモードの使用が推奨されています。）[:progErr]
    @endif
    @if CAFE_DOC
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っているか、Pia のサポート対象外の NEX のスレッドモードが設定されています。（ Pia はアンセーフトランスポートバッファスレッドモードしかサポートしません）。[:progErr]
    @endif
    @see      Bind
    @endcond
    */
    Result Startup();

    /*!
    @cond PRIVATE
    @brief    NexFacade 機能をクリーンアップします。session::Mesh クラスを直接操作する場合のみ呼出し可能です。

    @details NEX ゲームサーバーのログアウト前に呼び出す必要があります。
    @endcond
     */
    void Cleanup();

    /*!
    @cond PRIVATE
    @brief    NAT セッション（P2P 通信）を開始します。session::Mesh クラスを直接操作する場合のみ呼出し可能です。

    @details NAT セッションを開始し、P2P 通信に必要な前処理を行います。
             呼び出す場合は nn::pia::session::Mesh::Startup を呼び出す前に本関数を呼び出す必要があります。
             本関数を実行する度に P2P 通信で使用されるポート番号が変化します。

    開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
    処理の完了は @ref IsStartNatSessionCompleted でチェックし、結果は @ref GetStartNatSessionResult で確認できます。

    アプリケーション起動後の初回呼び出し時にのみ、 NAT 種別判定用サーバーの DNS による名前解決処理を行います。 
    名前解決処理は非同期処理中にバックグラウンドスレッドで実行されます。
    @if CTR_DOC
    名前解決処理にはソケットライブラリの nn::socket::GetAddrInfo() を使用しています。名前解決処理が実行される際に nn::socket::Initialize() で
    指定したワークメモリからメモリ確保が行われ、名前解決処理が完了した際に確保したメモリは解放されます。
    @endif
    @if CAFE_DOC
    名前解決処理にはソケットライブラリの SOGetAddrInfo を使用しています。名前解決処理が実行される際にメモリ確保が行われ、名前解決処理が完了した際に
    確保したメモリは解放されます。アプリケーションで名前解決処理用のメモリアロケータを SOResolverRegisterAllocator() で
    登録している場合、バックグラウンドスレッドからアロケータが呼ばれることになりますので注意してください。アロケータが
    スレッドセーフである必要があります。
    @endif

    名前解決前にルータの WAN 側のケーブルを抜いたときなどには、名前解決処理のタイムアウトを待つため、本非同期処理の
    完了に 30 秒以上かかることがあります。 なお、名前解決処理はキャンセルできません。

    @if CAFE_DOC
    本関数の呼び出しに成功した時点でシステムによるバックグラウンドダウンロード機能は停止されます。 
    そして StopNatSession() 呼び出し時にバックグラウンドダウンロード機能は再び有効化されます。

    BG ストレスモードは DevMenu の設定にのみ依存するため、本関数実行後もバックグラウンドダウンロードが行われる可能性があることに注意してください。 
    @endif

    【非同期処理完了時に @ref GetStartNatSessionResult を通して返される可能性のある Result】<br>
    ・ ResultSuccess                        ：非同期処理が成功しました。<br>
    ・ nn::pia::ResultCancelled                 ：非同期処理がキャンセルされました。[:handling-cleanup]<br>
    ・ nn::pia::ResultInvalidState              ：呼び出すタイミングが間違っています。[:progErr]<br>
    ・ nn::pia::ResultNetworkConnectionIsLost   ：ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害などの原因が考えられます。[:handling-shutdownNetwork]<br>
    ・ nn::pia::ResultNatCheckFailed      ：NAT チェックに失敗しました。[:viewer][:handling-cleanup]<br>
    ・ nn::pia::ResultDnsFailed           ：DNS 解決に失敗しています。[:viewer][:handling-cleanup]

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]
    @endcond
    */
    Result StartNatSessionAsync();

    /*!
    @cond PRIVATE
    @brief    StartNatSessionAsync の非同期処理の完了をチェックします。session::Mesh クラスを直接操作する場合のみ呼出し可能です。

    @details StartNatSessionAsync を呼び出した後、本関数で非同期処理の完了をチェックします。

    @return   非同期処理が完了していた場合は true, 処理中の場合は false が返ります。
    @endcond
    */
    bool IsStartNatSessionCompleted() const;

    /*!
    @cond PRIVATE
    @brief    StartNatSessionAsync の非同期処理の結果を取得します。session::Mesh クラスを直接操作する場合のみ呼出し可能です。

    @details StartNatSessionAsync を呼び出し、IsStartNatSessionCompleted で処理が完了した後、本関数で結果を取得します。

    @return   StartNatSessionAsync の非同期処理の結果を返します。 処理が完了した場合、以下を返します。
    @retval   ResultSuccess                        非同期処理が成功しました。
    @retval   nn::pia::ResultCancelled                 非同期処理がキャンセルされました。[:handling-cleanup]
    @retval   nn::pia::ResultInvalidState              StartNatSessionAsync を呼び出すタイミングが間違っているか、非同期処理が完了していません。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost   ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害などの原因が考えられます。[:handling-shutdownNetwork]
    @retval   nn::pia::ResultNatCheckFailed      NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval   nn::pia::ResultDnsFailed           DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @endcond
    */
    Result GetStartNatSessionResult() const;

    /*!
    @cond PRIVATE
    @brief    StartNatSessionAsync の非同期処理をキャンセルします。session::Mesh クラスを直接操作する場合のみ呼出し可能です。

    @details StartNatSessionAsync を呼び出した後、本関数で非同期処理をキャンセルします。
             キャンセル処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
             本関数を呼び出した後、ライブラリは速やかに非同期処理のキャンセルを行いますが、
             キャンセルの完了は @ref IsStartNatSessionCompleted() でチェックする必要があります。

    @return   StartNatSessionAsync の非同期処理のキャンセル結果を返します。 処理が失敗した場合、以下を返します。
    @retval   nn::pia::ResultInvalidState              StartNatSessionAsync の非同期処理が行われていないタイミングで呼び出しています。[:progErr]
    @endcond
    */
    Result CancelStartNatSessionAsync();

    /*!
    @cond PRIVATE
    @brief    NAT セッション（P2P 通信）を終了します。session::Mesh クラスを直接操作する場合のみ呼出し可能です。

    @details nn::pia::session::Mesh::Cleanup を呼び出した後、本関数を呼び出す必要があります。
    @endcond
    */
    void StopNatSession();

    /*!
    @cond PRIVATE
    @endcond
     */
    void CompleteStartNatSession();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリントします。
    @param[in] flag  トレースフラグです。詳細は @ref TraceFlag 型を参照してください。
    @endcond
     */
    void Trace(uint64_t flag) const;

    /*!
    @cond PRIVATE
    @brief    NexFacade 機能がスタートアップされた状態かチェックします。
    @return   スタートアップされた状態の場合、true を返します。
    @endcond
     */
    bool IsStartedUp() const
    {
        return m_IsStartedUp;
    }

    /*!
    @cond PRIVATE
    @brief    NAT セッション（P2P 通信）が開始された状態かチェックします。
    @return   開始された状態の場合、true を返します。
    @endcond
     */
    bool IsStartedNatSession() const
    {
        return m_IsStartedNatSession;
    }

    /*!
    @cond PRIVATE
    @endcond
     */
    nat::NatTraverser* GetNatTraverser()
    {
        return m_pNatTraverser;
    }

    /*!
    @cond PRIVATE
    @endcond
    */
    NexNatRelay* GetNatRelay()
    {
        return m_pNexNatRelay;
    }

    /*!
    @cond PRIVATE
    @brief  MatchmakeExtensionClient のポインタを取得します。
    @return MatchmakeExtensionClient のポインタ。
    @endcond
     */
    nn::nex::MatchmakeExtensionClient* GetMatchMakingClient()
    {
        return m_pMatchMakingClient;
    }

    /*!
    @cond PRIVATE
    @brief  MatchmakeSession のポインタを取得します。
    @return MatchmakeSession のポインタ。
    @endcond
    */
    nn::nex::MatchmakeSession* GetMatchmakeSessionPtr()
    {
        return m_pMatchmakeSession;
    }

    /*!
    @cond PRIVATE
    @brief Startup で指定した NgsFacade のポインタを取得します。
    @return NgsFacade のポインタ。
    @endcond
     */
    nn::nex::NgsBridgeInterface* GetNgsInterface()
    {
        return m_pNgsInterface;
    }

    /*!
    @cond PRIVATE
    @brief Startup で指定した GameServerId を取得します。
    @return GameServerId。
    @endcond
     */
    uint32_t GetGameServerId() const
    {
        return m_GameServerId;
    }

    /*!
    @cond PRIVATE
    @brief NexFacade が管理するマッチメイクセッションの GatheringID を更新します。session::Mesh クラスを直接操作する場合のみ呼出し可能です。
    @details この API で更新される GatheringID は、ホストマイグレーションなどで使用されます。
    @param[in] gatheringID 参加しているマッチメイクセッションの GatheringID を指定する必要があります。
    @endcond
     */
    void UpdateGatheringID(nn::nex::GatheringID gatheringID);

    /*!
    @cond PRIVATE
    @brief UpdateGatheringID で指定した GatheringID を取得します。
    @return GatheringID。
    @endcond
     */
    nn::nex::GatheringID GetGatheringID() const
    {
        return m_GatheringID;
    }

    /*!
    @brief      自分自身の NAT ポートマッピングタイプを取得します。
    @details    自分自身の NAT ポートマッピングタイプを取得します。
                NexFacade::StartNatSessionAsync の非同期処理が完了した後に正しい値を取得できます。
    @return     NAT ポートマッピングタイプ
     */
    nat::Mapping GetNatPropertyMapping() const;
    /*!
    @brief      自分自身の NAT フィルタリングタイプを取得します。
    @details    自分自身の NAT フィルタリングタイプを取得します。
                NexFacade::StartNatSessionAsync の非同期処理が完了した後に正しい値を取得できます。
    @return     NAT フィルタリング
     */
    nat::Filtering GetNatPropertyFiltering() const;
    /*!
    @brief      自分自身の NAT ポートインクリメントを取得します。
    @details    自分自身の NAT ポートインクリメントを取得します。
                NexFacade::StartNatSessionAsync の非同期処理が完了した後に正しい値を取得できます。
    @return     NAT ポートインクリメント
     */
    uint32_t GetNatPropertyPortIncrement() const;
    /*!
    @brief      自分自身の NAT のインターネット側のポートを取得します。
    @details    自分自身の NAT のインターネット側のポートを取得します。
                NexFacade::StartNatSessionAsync の非同期処理が完了した後に正しい値を取得できます。
    @return     NAT のインターネット側のポート
     */
    uint16_t GetNatPropertyPublicPort() const;
    /*!
    @brief      自分自身の NAT の LAN 側のポートを取得します。
    @details    自分自身の NAT の LAN 側のポートを取得します。
                NexFacade::StartNatSessionAsync の非同期処理が完了した後に正しい値を取得できます。
    @return     NAT の LAN 側のポート
     */
    uint16_t GetNatPropertyInterfacePort() const;

    /*!
    @cond PRIVATE
    @brief      NAT トラバーサル処理のタイムアウト基本時間を設定します。
    @details    タイムアウト時間は接続性に大きく影響を与えるため、通常アプリケーションはタイムアウト時間を変更してはいけません。
    @param[in] timeout  タイムアウト時間。単位はミリ秒です。
    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidState     呼び出すタイミングが間違っています。Startup から Cleanup までの区間に呼び出された場合はエラーとなります。
    @endcond
     */
    Result SetNatTraversalTimeout(int32_t timeout);

    /*!
    @cond PRIVATE
    @endcond
     */
    int32_t GetNatTraversalTimeout() const
    {
        return m_NatTraversalTimeout;
    }

    /*!
    @cond PRIVATE
    @brief      NAT トラバーサル処理の EDM タイムアウト拡張時間を設定します。
    @details    タイムアウト時間は接続性に大きく影響を与えるため、通常アプリケーションはタイムアウト時間を変更してはいけません。
    @param[in] timeout  タイムアウト時間。単位はミリ秒です。
    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidState     呼び出すタイミングが間違っています。Startup から Cleanup までの区間に呼び出された場合はエラーとなります。
    @endcond
     */
    Result SetNatTraversalTimeoutEdmExtension(int32_t timeout);

    /*!
    @cond PRIVATE
    @endcond
     */
    int32_t GetNatTraversalTimeoutEdmExtension() const
    {
        return m_NatTraversalTimeoutEdmExtension;
    }

    /*!
    @brief      自分のプリンシパル ID を取得します。
    @details    プリンシパル ID はゲームサーバーにログインし、@ref Bind() 実行後に取得できるようになります。
    @return     自分のプリンシパル ID を返します。
     */
    PrincipalId GetLocalPrincipalId() const;

#if NN_PIA_CAFE
    /*!
      @cond PRIVATE
      @brief    NAT セッション有効時（ P2P 通信中）にシステムによるバックグラウンドダウンロード機能を停止するかを設定します。
      @details  isSuspendBossDaemon が true に指定されている場合、@ref StartNATSession() 呼び出し時にシステムによるバックグラウンドダウンロード機能を停止し、
                @ref StopNATSession() 呼び出し時に有効化します。false が指定されている場合はバックグラウンドダウンロード機能の挙動には影響ありません。
                また、どちらを指定しても、BOSS ライブラリのフォアグランドダウンロード機能には影響を与えません。
                true が指定されている場合、ライブラリ内で自動的に ndm の Initialize(), Finalize() を呼び出します。
                
                デフォルトは「true（停止する）」です。アプリケーションが直接 ndm の機能を使用する場合は、本関数を用いて
                @ref StartNATSession() を呼び出す前に設定を false に変更する必要があります。
      
      @param[in] isSuspendBossDaemon NAT セッション有効時にシステムによるバックグラウンドダウンロード機能を停止するかどうかを指定します。true の場合、停止します。
      @endcond
     */
    void SetSuspendBossDaemonOnNATSession(bool isSuspendBossDaemon)
    {
        m_IsSuspendBossDaemon = isSuspendBossDaemon;
    }

    /*!
      @cond PRIVATE
      @brief    NAT セッション有効時（ P2P 通信中）に システムによるバックグラウンドダウンロード機能停止の設定を取得します。
      @return   StartNatSession() 時にシステムによるバックグラウンドダウンロード機能を停止するかどうか
      @endcond
     */
    bool IsSuspendBossDaemonOnNATSession() const
    {
        return m_IsSuspendBossDaemon;
    }
#endif

    /*!
      @cond PRIVATE
      @brief    Pia 内部処理用 nn::nex::NotificationEventHandler を NgsFacade へ登録します
      
      @return   登録の成否
      @endcond
     */
    bool RegisterNexNotificationEventHandler4Pia(nn::nex::NotificationEventHandler* pEventHandler);

    /*!
      @cond PRIVATE
      @brief    Pia 内部処理用 nn::nex::NotificationEventHandler を登録解除します
      
      @return   登録解除の成否
      @endcond
     */
    bool UnregisterNexNotificationEventHandler4Pia(nn::nex::NotificationEventHandler* pEventHandler);

    /*!
      @cond PRIVATE
     */
    void SetPersistentGatheringCodeKey();
    void ClearPersistentGatheringCodeKey();
    Result SetRelayServiceEnabled(bool isEnabled);
    bool IsRelayServiceEnabled() const;
    Result UpdateNatServerAddress();
    void SetIgnoreNatProbeRequest(bool bIgnore);
    void NotifyProbeRequest(const transport::StationLocation& requesterLocation);
    void NotifyLocalServerConnectionId(const ServerConnectionId scid);
    bool IsTraversed(const transport::StationLocation& targetLocation) const;
    bool IsTraversed(ServerConnectionId scid) const;
    bool IsProcessingNatDetection() const;
    void NotifyLeave(const transport::StationLocation& targetLocation, bool isProbeCleanup, bool isRelayEntryCleanup);
    bool PrepareNatTraversal(const transport::StationLocation& targetLocation,
                             const transport::StationConnectionInfo& targetConnectionInfo,
                             common::AsyncContext* pContext,
                             bool isInverseConnection,
                             bool isRelayServerUsed,
                             StationIndex targetStationIndex);
    bool CheckLatestStationLocation(transport::StationLocation& stationLocation);
    void ResetProbeStatus(const transport::StationLocation& targetLocation, bool bInverseConnection);
    void ReportNatTraversalResult(const transport::StationLocation& stationLocation, Result result, bool bReport);
    //! @endcond

    /*!
    @cond PRIVATE
    */
    virtual uint16_t GetRandomPortRangeMin() const;
    virtual uint16_t GetRandomPortRangeMax() const;
    virtual uint16_t GetReservedPort() const;
    virtual ServerConnectionId GetLocalServerConnectionId() const;
    virtual void RelayProbeRequest(const transport::StationLocation& targetLocation, const transport::StationLocation& requesterLocation);
    virtual void UpdateRelayTable();
    virtual const common::InetAddress& GetRelayServerAddress() const;
    virtual bool IsBeginMonitoringDataAvailable() const;
    //! @endcond

private:
    // シングルトンインスタンスへのポインタです。
    static NexFacade* s_pInstance;

    static const uint32_t GameServerAccessKeySize = 8;

    /*!
    @brief シングルトンパターンのため、コンストラクタは封印します。
     */
    NexFacade(void);

    /*!
    @brief シングルトンパターンのため、デストラクタは封印します。
     */
    virtual ~NexFacade(void);

    /*!
    @brief シングルトンパターンのため、コピーコンストラクタは封印します。
     */
    NexFacade(const NexFacade&);

    /*!
    @brief シングルトンパターンのため、代入演算子は封印します。
     */
    NexFacade& operator=(const NexFacade&);

    Result Initialize();
    void Finalize();

#if NN_PIA_CAFE
    void suspendBossDaemon();
    void resumeBossDaemon();
#endif

    static void NatTraverserStartupCompletionCallback(Result result, void* pUserContext);
    static void NatServerAddressResolveCompletionCallback(Result result, void* pUserContext);
    void CompletedNatServerAddressResolution();
    void CancelCleanupNatServerAddressResolution();

    uint32_t m_GameServerId;
    common::FixedString<GameServerAccessKeySize> m_GameServerAccessKey;
    nn::nex::NgsBridgeInterface* m_pNgsInterface;
    nn::nex::GatheringID m_GatheringID;

    uint8_t* m_pMatchMakingClientBuffer;
    nn::nex::MatchmakeExtensionClient* m_pMatchMakingClient;
    nn::nex::MatchmakeSession* m_pMatchmakeSession;

    uint8_t* m_pNatTraversalRelayClientBuffer;
    nn::nex::NATTraversalRelayClient* m_pNatTraversalRelayClient;
    NexNatRelay* m_pNexNatRelay;

    nat::NatTraverser* m_pNatTraverser;

    common::Socket* m_pSocket;
    SocketOutputStream* m_pOutputStream;
    SocketInputStream* m_pInputStream;

    bool m_IsStartedUp;
    bool m_IsStartedNatSession;
    bool m_IsRelayEnabled;

    int32_t m_NatTraversalTimeout;
    int32_t m_NatTraversalTimeoutEdmExtension;

#if NN_PIA_CAFE
    bool m_IsSuspendBossDaemon;
    int32_t m_NdmInitializeCount;
#endif

    nn::nex::NotificationEventHandler* m_pNexNotificationEventHandler;

    NexNotificationDataJob* m_pNexNotificationDataJob;

    uint8_t m_InternalAsyncProcessType;
    common::AsyncContext m_InternalAsyncConext;

    common::AsyncContext m_NatTraverserStartupCallContext;
    common::AsyncContext m_NatServerAddressResolveCallContext;
    NexNatServerAddressResolveJob* m_pNexNatServerAddressResolveJob;

    nn::nex::StationURL* m_pLocalUrl;
    nn::nex::StationURL* m_pUpdatedUrl;

    nn::nex::ProtocolCallContext* m_pReplaceURLCallContext;

    bool m_IsIgnoredNatProbeRequest;
    ServerConnectionId m_LocalServerConnectionId;

#if NN_PIA_CTR
    NexBlockListJob* m_pNexBlockListJob;
#endif
};
}
}
}  // end of namespace nn::pia::inet

#endif
