﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/session/session_JoinRandomSessionSetting.h>
#include <nn/pia/inet/inet_NexCreateSessionSetting.h>
#include <nn/pia/inet/inet_NexSessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
  @brief  ランダムマッチメイク処理に関する設定を管理するクラスです。

  @see nn::pia::session::JoinRandomSessionSetting
 */
class NexJoinRandomSessionSetting : public session::JoinRandomSessionSetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexJoinRandomSessionSetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexJoinRandomSessionSetting();

    /*!
    @brief      ランダムマッチメイクのセッション作成時の設定を指定します。
    @param[in]  pSetting セッション作成時の設定。
    @return     セッション作成時の設定の指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  引数が NULL ポインタです。[:progErr]
    */
    virtual Result SetCreateSessionSetting(const session::CreateSessionSetting* pSetting);

    /*!
    @brief      ランダムマッチメイクのセッション検索条件を指定します。
    @param[in]  pSearchCriteria     セッション検索条件のリスト。
    @param[in]  searchCriteriaSize  セッション検索条件のリストサイズ。SearchCriteriaListSizeMax 以下である必要があります。
    @return     セッション検索条件の指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  検索条件のリストが NULL ポインタ、検索条件が NexSessionSearchCriteria ではない、または検索条件のリストサイズが大きすぎます。[:progErr]
    */
    virtual Result SetSessionSearchCriteria(const session::SessionSearchCriteria* pSearchCriteria, uint32_t searchCriteriaSize);

    /*!
    @brief      ブロックリストを設定します。
    @param[in]  isMyBlockListUsed      自分がブロックリストに登録しているユーザーをチェックするかどうか。デフォルトは true です。
    @param[in]  isOthersBlockListUsed  自分をブロックリストに登録しているユーザをチェックするかどうか。デフォルトは true です。
    */
    void SetBlockListOption(bool isMyBlockListUsed, bool isOthersBlockListUsed);

    /*!
    @brief      あいことばマッチメイク用のあいことばが重複するセッションを構築しないようにするかどうかを設定します。（NEX 4.2 以降で利用可能）
    @param[in]  isUniqueKeywordEnabled  あいことばが重複するセッションを構築しないようにするかどうか。デフォルトは false です。
    */
    void SetMatchmakeUniqueKeyword(bool isUniqueKeywordEnabled);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexJoinRandomSessionSetting& operator=(const NexJoinRandomSessionSetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexJoinRandomSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexJoinRandomSessionSetting(const NexJoinRandomSessionSetting& rhs);

    /*!
    @brief      設定をリセットします。
    */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    virtual session::CreateSessionSetting* GetCreateSessionSetting();
    virtual session::SessionSearchCriteria* GetSessionSearchCriteria(uint8_t index);
    virtual const session::CreateSessionSetting* GetCreateSessionSetting() const;
    virtual const session::SessionSearchCriteria* GetSessionSearchCriteria(uint8_t index) const;
    virtual uint32_t GetSessionSearchCriteriaSize() const;
    bool IsMyBlockListUsed() const;
    bool IsOthersBlockListUsed() const;
    bool IsMatchmakeUniqueKeywordEnabled() const;
    //! @endcond

protected:

    NexCreateSessionSetting m_CreateSessionSetting;
    NexSessionSearchCriteria m_SessionSearchCriteria[SearchCriteriaListSizeMax];
    uint32_t m_SessionSearchCriteriaSize;
    bool m_IsMyBlockListUsed;
    bool m_IsOthersBlockListUsed;
    bool m_IsMatchmakeUniqueKeywordEnabled;

private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
