﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/session/session_JoinSessionSetting.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
  @brief  セッション参加処理に関する設定を管理するクラスです。

  @see nn::pia::session::JoinSessionSetting
 */

class NexJoinSessionSetting : public session::JoinSessionSetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexJoinSessionSetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexJoinSessionSetting();

    /*!
    @brief      ブロックリストを設定します。
    @param[in]  isMyBlockListUsed      自分がブロックリストに登録しているユーザーをチェックするかどうか。デフォルトは true です。
    @param[in]  isOthersBlockListUsed  自分をブロックリストに登録しているユーザをチェックするかどうか。デフォルトは true です。
    */
    void SetBlockListOption(bool isMyBlockListUsed, bool isOthersBlockListUsed);

    /*!
    @brief      セッションのユーザーパスワードを設定します。
    @details    セッションにユーザーパスワードが設定されている場合に一致が確認されます。 初期状態は空文字列です。
    @param[in]  userPassword  ユーザーパスワード。パスワードの文字列は SessionUserPasswordLengthMax 以下である必要があります。
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
     */
    Result SetSessionUserPassword(const common::String& userPassword);

    /*!
    @brief      セッションのシステムパスワードを設定します。
    @details    セッションにシステムパスワードが設定されている場合に一致が確認されます。 初期状態は空文字列です。
    @param[in]  systemPassword  システムパスワード。パスワードの文字列は SessionSystemPasswordLengthMax 以下である必要があります。
    @return     システムパスワードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはシステムパスワードの最大長を超えています。[:progErr]
     */
    Result SetSessionSystemPassword(const common::String& systemPassword);

    /*!
    @cond PRIVATE
    @brief 
    */
    void SetExtraParticipantNum(uint16_t num);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      セッションのマッチメイクセッションオプションを指定します。
    @param[in]  option    セッションに指定するマッチメイクセッションオプション。
    */
    void SetMatchmakeSessionOption(uint32_t option);
    //! @endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexJoinSessionSetting& operator=(const NexJoinSessionSetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexJoinSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexJoinSessionSetting(const NexJoinSessionSetting& rhs);

    /*!
    @cond PRIVATE
    */
    const common::String& GetSessionUserPassword() const;
    const common::String& GetSessionSystemPassword() const;
    uint16_t GetExtraParticipantNum() const;
    uint32_t GetMatchmakeSessionOption() const;
    bool IsSetMatchmakeSessionOption() const;
    bool IsMyBlockListUsed() const;
    bool IsOthersBlockListUsed() const;
    //! @endcond

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

protected:
    common::FixedString<nn::pia::inet::SessionUserPasswordLengthMax> m_UserPassword;
    common::FixedString<nn::pia::inet::SessionSystemPasswordLengthMax> m_SystemPassword;

    uint16_t m_ExtraParticipantNum;
    uint32_t m_MatchmakeSessionOption;
    bool m_IsSetMatchmakeSessionoption;
    bool m_IsMyBlockListUsed;
    bool m_IsOthersBlockListUsed;

private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
