﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>
#include <nn/pia/transport/transport_NetworkFactory.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  NEX ライブラリ（インターネット通信）を利用したネットワークに必要なクラスを生成するためのクラスです（ファクトリパターン）。

  @details @ref nn::pia::transport::Transport や @ref nn::pia::session::Session に、
           NexNetworkFactory クラスを設定することで NEX ライブラリに合わせたネットワーク関連処理（送受信、セッション管理など）が
           行われるようになります。

  @see nn::pia::transport::Transport::Setting, nn::pia::session::Session::Setting
 */
class NexNetworkFactory : public nn::pia::transport::NetworkFactory
{
public:
    /*!
    @cond PRIVATE
     */

    /*!
     @name 初期化終了処理
     @{
     */

    /*!
    @brief デフォルトコンストラクタ
     */
    NexNetworkFactory()
    {
    }

    /*!
    @brief デストラクタ
     */
    virtual ~NexNetworkFactory()
    {
    }

    /*!
     end of name 初期化終了処理
     @}
     */

    /*!
     @brief 最大参加可能ステーション数を返します。
     */
    virtual uint16_t GetMaxStationNum() const
    {
        return 16;
    }

    virtual transport::NetworkFactory* CreateSelf();

    /*!
    @brief InputStream を生成します。
     */
    virtual nn::pia::common::IPacketInput* CreateInputStream();

    /*!
    @brief OutputStream を生成します。
     */
    virtual nn::pia::common::IPacketOutput* CreateOutputStream();

    /*!
    @brief ConnectStationJob を生成します。
     */
    virtual nn::pia::transport::ConnectStationJob* CreateConnectStationJob();

    /*!
    @brief DisconnectStationJob を生成します。
     */
    virtual nn::pia::transport::DisconnectStationJob* CreateDisconnectStationJob();

    /*!
    @brief CreateMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateCreateMeshJob();

    /*!
    @brief JoinMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateJoinMeshJob();

    /*!
    @brief LeaveMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateLeaveMeshJob();

    /*!
    @brief GenerateMatchmakeSystemPasswordJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateGenerateMatchmakeSystemPasswordJob();

    /*!
    @brief ClearMatchmakeSystemPasswordJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateClearMatchmakeSystemPasswordJob();

    /*!
    @brief Session 用 MatchmakeSessionSystemPassword の保持領域を生成します。
     */
    virtual common::String* CreateMatchmakeSessionSystemPassword();

    /*!
    @brief 必要なプロトコルを構築します。
     */
    virtual Result CreateProtocols();

    /*!
    @brief ProcessHostMigrationJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateProcessHostMigrationJob();

    /*!
    @brief LeaveWithHostMigrationJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateLeaveWithHostMigrationJob();

    /*!
    @brief KickoutManageJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateKickoutManageJob();

    /*!
    @brief PacketHandler の具象インスタンスを構築します。
     */
    virtual transport::PacketHandler* CreatePacketHandler();

    /*!
    @brief 署名が必要かどうか。
     */
    virtual bool IsSignatureRequired() const;

    /*!
    @brief 署名設定の Storage を生成します。
     */
    virtual nn::pia::session::SignatureSettingStorage* CreateSignatureSettingStorage();

    /*!
    @brief 複数候補親交代の使用を許すかどうか。
     */
    virtual bool IsMultiCandidatePermitted() const;

    /*!
    @brief リレー通信機能の使用を許すかどうか。
     */
    virtual bool IsRelayConnectionPermitted() const;

    /*!
    @brief ジョイントセッション機能の使用を許すかどうか。
     */
    virtual bool IsJointSessionPermitted() const;

    /*!
    @brief MonitoringDataSender 派生クラスのインスタンスを生成します。
     */
    virtual common::MonitoringDataSender* CreateMonitoringDataSender();


    /*!
    @brief ネットワーク層におけるパケットのヘッダ部分の合計サイズを返します。単位はバイトです。
     */
    virtual uint32_t GetHeaderSize() const;

    /*!
    @brief MissingStationHandler 具象インスタンスを構築します。
     */
    virtual transport::MissingStationHandler* CreateMissingStationHandler();

    /*!
    @brief Session 用 CreateSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchCreateSessionJob();


    /*!
    @brief Session 用 AutoMatchmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchAutoMatchmakeSessionJob();


    /*!
    @brief Session 用 BrowseMatchmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchBrowseMatchmakeSessionJob();


    /*!
    @brief Session 用 JoinSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchJoinSessionJob();


    /*!
    @brief Session 用 LeaveSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchLeaveSessionJob();


    /*!
    @brief Session 用 DestroySessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchDestroySessionJob();

    /*!
    @brief ジョイントセッション機能用 JointSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateJointSessionJob();

    /*!
    @brief Session 用 UpdateSessionSetting を生成します。
     */
    virtual common::StepSequenceJob* CreateUpdateSessionSettingJob();

    /*!
    @cond PRIVATE
    @brief Session 用 CommunityManagementJob を生成します。
     */
    virtual common::StepSequenceJob* CreateCommunityManagementJob();
    //! @endcond

    /*!
    @brief Session 用 IMatchmakeSession を生成します。
     */
    virtual session::IMatchmakeSession* CreateMatchmakeSession();

    /*!
    @brief Session 用 ISessionInfoList を生成します。
     */
    virtual session::ISessionInfoList* CreateSessionInfoList(uint16_t browseSessionInfoListNum);

    /*!
    @brief Session 用 ICommunityInfoList を生成します。
     */
    virtual session::ICommunityInfoList* CreateCommunityInfoList(uint8_t browsedCommunityInfoListNum);

    /*!
    @brief Session 用 MeshLayerController を生成します。
     */
    virtual session::MeshLayerController* CreateMatchMeshLayerController();

    /*!
    @brief Bind() が必要かどうか。
     */
    virtual bool IsBindRequired() const;

    /*!
    @brief サーバーが必要かどうか。
     */
    virtual bool IsServerRequired() const;

    /*!
    @brief いっしょに遊んだプレイヤー履歴を記録可能かどうか。
    */
    virtual bool IsRecentPlayRecordPermitted() const;

    /*!
    @brief 同じ端末から複数プレイヤーが参加可能かどうか。
    */
    virtual bool IsMultiplayerPermitted() const;

    /*!
    @brief ホストマイグレーションモードを取得します。
     */
    virtual int32_t GetHostMigrationMode() const;

    /*!
    @brief セッションの検索結果の最大数を取得します。
     */
    virtual uint16_t GetBrowseSessionInfoListSizeMax() const;

    /*!
    @brief コミュニティの検索結果の最大数を取得します。
     */
    virtual uint8_t GetBrowseCommunityInfoListSizeMax() const;

    /*!
    @brief MTU の最大値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMax() const;

    /*!
    @brief MTU の最小値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMin() const;

    /*!
    @brief 初期段階での RTT 計測パルス送信間隔のデフォルト値です。単位はミリ秒です。
     */
    virtual int GetDefaultRttInitialPulseInterval() const;

    /*!
    @endcond
     */

protected:
private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
