﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_DateTime.h>
#include <nn/pia/session/session_ISessionInfo.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
@brief  インターネット通信のセッション情報クラスです。

@see session::ISessionInfo
*/
class NexSessionInfo : public session::ISessionInfo
{
public:
    NexSessionInfo();
    virtual ~NexSessionInfo();

    /*!
    @brief      セッションのゲームモードを取得します。
    @return     セッションのゲームモードを返します。
    */
    virtual uint32_t GetGameMode() const;

    /*!
    @brief      セッションの ID を取得します。
    @return     セッションの ID を返します。
    */
    virtual uint32_t GetSessionId() const;

    /*!
    @brief      セッションに関連付けられているコミュニティの ID を取得します。
    @return     セッションに関連付けられているコミュニティの ID を返します。
    */
    virtual uint32_t GetCommunityId() const;

    /*!
    @brief      セッションに参加している人数を取得します。
    @return     セッションに参加している人数を返します。
    */
    virtual uint16_t GetCurrentParticipantNum() const;

    /*!
    @brief      セッションに参加できる最小人数を取得します。
    @return     セッションに参加できる最小人数を返します。
    */
    virtual uint16_t GetMinParticipantNum() const;

    /*!
    @brief      セッションに参加できる最大人数を取得します。
    @return     セッションに参加できる最大人数を返します。
    */
    virtual uint16_t GetMaxParticipantNum() const;

    /*!
    @brief      セッションが参加募集受付状態かどうかを取得します。
    @return     セッションが参加募集受付状態なら true, 参加募集締切状態なら false を返します。
    */
    virtual bool IsOpened() const;

    /*!
    @brief              セッションのアプリケーション定義データを取得します。
    @details            セッションに設定したアプリケーション定義データを取得します。
    @param[out] pBuffer セッションのアプリケーション定義データのコピー用バッファのポインタ。
    @param[in]  size    セッションのアプリケーション定義データのコピー用バッファのサイズ。アプリケーション定義データの最大長は ApplicationDataBufferSizeMax です。
    @return             正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pBuffer, uint32_t size) const;

    /*!
    @brief      セッションのアプリケーション定義データの長さを取得します。
    @return     セッションのアプリケーション定義データの長さが返ります。
    */
    virtual uint32_t GetApplicationDataSize() const;

    /*!
    @brief                  セッションの属性を取得します。
    @param[in]  idx         取得する属性のインデックス。AttributeSizeMax である必要があります。
    @param[out] pAttribute  指定されたインデックスのセッションの属性格納先のポインタ。
    @return                 正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    */
    virtual Result GetAttribute(uint8_t idx, uint32_t* pAttribute) const;

    /*!
    @brief      セッションにユーザーパスワードが設定されているかどうかを確認します。
    @return     ユーザーパスワードが設定されている場合は true が返ります。
    */
    virtual bool IsRestrictedByUserPassword() const;

    /*!
    @cond PRIVATE
    @brief      セッションにシステムパスワードが設定されているかどうかを確認します。
    @return     システムパスワードが設定されている場合は true が返ります。
    */
    virtual bool IsRestrictedBySystemPassword() const;
    //! @endcond

    /*!
    @brief      セッションタイプを取得します。
    @return     設定されているセッションタイプが返ります。
    */
    virtual SessionType GetSessionType() const;

    /*!
    @brief      ホストマイグレーション時にセッションを自動的に参加締切状態にする設定を取得します。
    @return     ホストマイグレーション時にセッションを自動的に参加締切状態にする設定が返ります。
    */
    virtual bool IsAutoCloseSession() const;

    /*!
    @brief      セッションのホストのプリンシパル ID を取得します。
    @return     セッションのホストのプリンシパル ID が返ります。
    */
    virtual PrincipalId GetHostPrincipalId() const;

    /*!
    @brief      セッションの検索条件に指定したプリンシパル ID を取得します。
    @return     セッションの検索条件に指定したプリンシパル ID が返ります。
    */
    virtual PrincipalId GetTargetPrincipalId() const;

    /*!
    @brief      セッションのマッチメイクの優先度を取得します。
    @return     セッションのマッチメイクの優先度が返ります。
    */
    virtual uint8_t GetSelectionPriority() const;

    /*!
    @brief      セッションが開始された時刻を取得します。
    @return     セッションが開始された時刻が返ります。
    */
    virtual const common::DateTime& GetStartedTime() const;

    /*!
    @brief      セッション情報をクリアします。
    */
    virtual void Clear();

    /*!
    @brief          セッション情報をコピーします。
    @param[in] rhs  コピーするセッション情報
    */
    void Copy(const NexSessionInfo& rhs);

    /*!
    @brief          コピーコンストラクタ。
    @param[in] rhs  コピーするセッション情報
    */
    NexSessionInfo(const NexSessionInfo& rhs);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexSessionInfo& operator=(const NexSessionInfo& rhs);

    /*!
    @brief デバッグに有用な情報をプリントします。
    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
    */
    virtual void Trace(uint64_t flag) const;

    /*!
    @cond PRIVATE
    */
    virtual void SetGameMode(uint32_t gameMode);
    virtual void SetSessionId(uint32_t sessionId);
    virtual void SetCurrentParticipantNum(uint16_t participants);
    virtual void SetMinParticipantNum(uint16_t participants);
    virtual void SetMaxParticipantNum(uint16_t participants);
    virtual void SetOpen(bool isOpened);
    virtual void SetAttribute(uint8_t index, uint32_t attribute);
    virtual void SetApplicationData(const void* pData, uint32_t size);
    virtual void SetUserPasswordEnabled(bool isEnabled);
    virtual void SetSystemPasswordEnabled(bool isEnabled);
    virtual void SetSessionType(SessionType sessionType, bool isAutoCloseSession);
    virtual void SetHostPrincipalId(PrincipalId principalId);
    virtual void SetTargetPrincipalId(PrincipalId principalId);
    virtual void SetSelectionPriority(uint8_t priority);
    virtual void SetStartedTime(const common::DateTime& time);
    virtual void SetCommunityId(uint32_t communityId);
    /// @endcond

protected:
    uint32_t m_GameMode;
    uint32_t m_SessionId;
    uint16_t m_CurrentParticipantNum;
    uint16_t m_MinParticipantNum;
    uint16_t m_MaxParticipantNum;
    bool m_bIsOpened;

    uint32_t m_Attribute[AttributeSizeMax];
    uint8_t m_ApplicationData[ApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;
    bool m_bUserPasswordEnabled;
    bool m_bSystemPasswordEnabled;

    SessionType m_SessionType;
    bool m_IsAutoCloseSession;
    PrincipalId m_HostPrincipalId;
    PrincipalId m_TargetPrincipalId;
    uint8_t m_SelectionPriority;
    common::DateTime m_StartedTime;
    uint32_t m_CommunityId;
};
}
}
}  // end of namespace nn::pia::inet

#endif
