﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_InetAddress.h>
#include <nn/pia/session/session_SessionSearchCriteria.h>

#if NN_PIA_CTR
#pragma diag_suppress 1300  //n1769 ARMCC 用プラグマ。
#endif
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSessionSearchCriteria.h>
#include <RendezVous/Core/src/Common/ResultRange.h>
#if NN_PIA_CTR
#pragma diag_warning 1300  //n1769 ARMCC 用プラグマ。
#endif


namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  セッションを検索する際に使用する検索条件のクラスです。
  @details 検索結果として得られるセッションの最大数は SetResultRange で指定する値になりますが、
           nn::pia::session::Session::CreateInstance 呼び出し時に指定する 
           nn::pia::session::Session::Setting 構造体の browsedSessionInfoListNum 以下の値を
           指定する必要があります。

  @see nn::pia::session::SessionSearchCriteria
 */
class NexSessionSearchCriteria : public session::SessionSearchCriteria
{
public:
    /*!
    @cond PRIVATE
    @brief 検索条件として指定した項目の示すビットマスクの各ビットに対応する項目
    */
    enum NexCondMask
    {
        NexCondMask_MinParticipation = 0,                                                                       //!< 最小参加人数
        NexCondMask_MaxParticipation,                                                                           //!< 最大参加人数
        NexCondMask_OpenedOnly,                                                                                 //!< 参加受付状態のセッションのみかどうか
        NexCondMask_VacantOnly,                                                                                 //!< 空きがあるセッションのみかどうか
        NexCondMask_GameMode,                                                                                   //!< ゲームモード
        NexCondMask_SessionType,                                                                                //!< セッションタイプ
        NexCondMask_Attribute,                                                                                  //!< 属性
        NexCondMask_ExcludeNonHost = NexCondMask_Attribute + AttributeSizeMax,                                  //!< ホストの存在しないセッションを除外するかどうか
        NexCondMask_ExcludeUserPassword,                                                                        //!< ユーザーパスワード設定のセッションを除外するかどうか
        NexCondMask_RandomSessionSelectionMethod,                                                               //!< ランダムマッチメイク実行時に参加するセッションの選択方法
        NexCondMask_ScoreBasedSettingIndex,                                                                     //!< スコア換算方法の設定インデックス（スコアベースマッチメイク使用時）
        NexCondMask_ScoreBasedRatingValue,                                                                      //!< レーティング値（スコアベースマッチメイク使用時）
        NexCondMask_ScoreBasedDisconnectionRate,                                                                //!< 切断率（スコアベースマッチメイク使用時）
        NexCondMask_ScoreBasedViolationRate,                                                                    //!< 違反率（スコアベースマッチメイク使用時）
        NexCondMask_ScoreBasedCountryCode,                                                                      //!< 国コード（スコアベースマッチメイク使用時）
        NexCondMask_ScoreBasedGeoIp,                                                                            //!< 位置情報（スコアベースマッチメイク使用時）
        NexCondMask_ScoreBasedOverrideIp,                                                                       //!< デバッグ用IPアドレス（スコアベースマッチメイク使用時）
        NexCondMask_Community,                                                                                  //!< 関連コミュニティ
        NexCondMask_MatchmakeKeyword,                                                                           //!< あいことば
        NexCondMask_AdditionalParamString,                                                                      //!< 追加文字列パラメータ（スコアベースマッチメイク使用時）
        NexCondMask_AdditionalParamValue = NexCondMask_AdditionalParamString + AdditionalParamStringSizeMax,    //!< 追加パラメータ（スコアベースマッチメイク使用時）
        NexCondMask_Max = NexCondMask_AdditionalParamValue + AdditionalParamKeySizeMax
    };
    //! @endcond

    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexSessionSearchCriteria();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexSessionSearchCriteria();

    /*!
    @brief      検索結果の取得範囲を指定します。
    @details    offset に 0xFFFFFFFF を指定すると、検索結果をランダムに並び替えた上で指定されたエントリー数を取得するように振る舞います。
    @param[in]  offset  検索条件にマッチしたエントリーリストから検索結果として取得するエントリー群の開始位置。
    @param[in]  size    検索条件にマッチしたエントリーリストから検索結果として取得するエントリー数。
                        session::Session::Setting 構造体の browsedSessionInfoListNum 以下、かつSearchCriteriaResultSizeMax 以下の値である必要があります。

    @return     処理に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval     nn::pia::ResultInvalidArgument  引数の値が範囲外です。[:progErr]
    */
    virtual Result SetResultRange(uint32_t offset, uint32_t size);

    /*!
    @brief      検索条件に最小参加人数を指定します。
    @param[in]  min 検索条件に指定する最小参加人数。
    */
    void SetMinParticipantNum(uint16_t min);

    /*!
    @brief      検索条件に最小参加人数を範囲指定します。
    @param[in]  min 検索条件に指定する最小参加人数の範囲の下限。
    @param[in]  max 検索条件に指定する最小参加人数の範囲の上限。
    */
    void SetMinParticipantNumRange(uint16_t min, uint16_t max);

    /*!
    @brief      検索条件に最大参加人数を指定します。
    @param[in]  max 検索条件に指定する最大参加人数。
    */
    void SetMaxParticipantNum(uint16_t max);

    /*!
    @brief      検索条件に最大参加人数を範囲指定します。
    @param[in]  min 検索条件に指定する最大参加人数の範囲の下限。
    @param[in]  max 検索条件に指定する最大参加人数の範囲の上限。
    */
    void SetMaxParticipantNumRange(uint16_t min, uint16_t max);

    /*!
    @brief      検索条件に参加受付状態のセッションのみを検索するか指定します。
    @param[in]  bOpenedOnly 参加受付状態のセッションのみを検索する場合は true を指定します。
    */
    void SetOpenedOnly(bool bOpenedOnly);

    /*!
    @brief      検索条件に空きがあるセッションのみを検索するか指定します。
    @param[in]  bVacantOnly 空きがあるセッションのみを検索する場合は true を指定します。
    */
    void SetVacantOnly(bool bVacantOnly);

    /*!
    @brief      検索条件にゲームモードを指定します。
    @details    ゲームモードの指定は必須となります。指定しないまま、セッションの非同期処理を呼び出すとエラーを返します。
    @param[in]  gameMode    検索条件に指定するゲームモード。
    */
    void SetGameMode(uint32_t gameMode);

    /*!
    @brief      検索条件にセッションタイプを指定します。
    @details    指定しなかった場合、デフォルト値（SessionType_Anybody）が指定されます。
    @param[in]  sessionType 検索条件に指定するセッションタイプ。
    */
    void SetSessionType(SessionType sessionType);

    /*!
    @brief      １つの属性に対して検索時に使用する値を指定します。
    @details    設定されなかったインデックスの属性に対しては任意の値を検索対象とします。
    @param[in]  index   指定する属性のインデックス。AttributeSizeMax 以下である必要があります。
    @param[in]  value   指定する属性の値。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外です。[:progErr]
    */
    Result SetAttribute(uint32_t index, uint32_t value);

    /*!
    @brief      １つの属性に対して検索時に使用する値を複数指定します。
    @details    設定されなかったインデックスの属性に対しては任意の値を検索対象とします。
    @param[in]  index           指定する属性のインデックス。AttributeSizeMax 以下である必要があります。
    @param[in]  value           指定する属性の値のリスト。
    @param[in]  attributeNum    指定する属性の値の個数。AttributeArraySizeMax 以下である必要があります。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外、値のリストが NULL、または値の個数が多すぎます。[:progErr]
    */
    Result SetAttribute(uint32_t index, const uint32_t* value, uint32_t attributeNum);

    /*!
    @brief      １つの属性に対して検索時に使用する値を範囲指定します。
    @details    設定されなかったインデックスの属性に対しては任意の値を検索対象とします。
    @param[in]  index           指定する属性のインデックス。AttributeSizeMax 以下である必要があります。
    @param[in]  min             指定する属性の値の下限。
    @param[in]  max             指定する属性の値の上限。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外、または下限値が上限値よりも大きい値になっています。[:progErr]
    */
    Result SetAttributeRange(uint32_t index, uint32_t min, uint32_t max);

    /*!
    @brief      ランダムマッチメイク実行時に参加するセッションの選択方法を指定します。
    @param[in]  selectionMethod  指定するセッションの選択方法。
    */
    void SetRandomSessionSelectionMethod(SelectionMethod selectionMethod);

    /*!
    @brief      ホストが存在しないセッションを検索対象から外すかどうか指定します。（デフォルトは対象から外します）
    @param[in]  bExclude        ホストが存在しないセッションを検索対象から外す場合は true を指定します。
    */
    void SetExcludeNonHostPrincipalId(bool bExclude);

    /*!
    @brief      ユーザパスワードが設定されたセッションを検索対象から外すかどうか指定します。（デフォルトは対象から外します）
    @param[in]  bExclude        ユーザパスワードが設定されたセッションを検索対象から外す場合は true を指定します。
    */
    void SetExcludeUserPasswordSet(bool bExclude);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用するスコア換算方法の設定インデックスを設定します。
    @details    設定インデックスをセットしていない場合は 0 が使用されます。
    @param[in]  settingIndex        スコア換算設定のインデックス
    */
    void SetScoreSettingIndex(uint32_t settingIndex);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用するレーティング値を設定します。
    @details    セットしない場合はレーティング値の差に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  ratingValue        レーティング値
    */
    void SetRatingValue(uint32_t ratingValue);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用する切断率を設定します。
    @details    セットしない場合は切断率の差に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  disconnectionRate        切断率
    */
    void SetDisconnectionRate(uint32_t disconnectionRate);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用する違反率を設定します。
    @details    セットしない場合は違反率の差に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  violationRate        違反率
    */
    void SetViolationRate(uint32_t violationRate);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に位置情報 (緯度経度、国コード) を使用するかどうかを設定します。
    @details    位置情報を使うかどうかをセットしない場合は距離の比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  use     位置情報を使用するかどうか
    */
    void SetUseGeoIp(bool use);

    /*!
    @if NIN_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはISO 3166-1 alpha-2 によるアルファベット 2 文字（大文字）の国コードを設定します。
    @details    @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
    位置情報が得られた場合はここで指定した国コードは使用されません。
    国コードをセットしない場合は @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
    国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード。ISO 3166-1 alpha-2 によるアルファベット 2 文字（大文字）の文字列である必要があります。
    @return     国コードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  国コードの文字列長が 2 以外、または国コードに指定した文字種が不正です。[:progErr]
    @endif
    */
    Result SetCountryCode(const common::String& countryCode);

    /*!
    @if CTR_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはプラットフォームで定義された値を設定します。
    @details    @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
    位置情報が得られた場合はここで指定した国コードは使用されません。
    国コードをセットしない場合は @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
    国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード
    @endif
    @if CAFE_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはプラットフォームで定義された値を設定します。
    @details    @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
    位置情報が得られた場合はここで指定した国コードは使用されません。
    国コードをセットしない場合は @ref SetUseGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
    国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード
    @endif
    */
    void SetCountryCode(uint32_t countryCode);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用するデバッグ用の IP アドレスを設定します。
    @details    自身の IP アドレスの代わりにこの IP アドレスを使用して GeoIP による位置情報を取得します。 デバッグ用にのみ使用可能です。 
    @param[in]  ipAddress        代わりに使用する IP アドレス
    @return     IP アドレスの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  IP アドレスが不正です。[:progErr]
    */
    Result SetOverrideIpAddress(const common::InetAddress& ipAddress);

    /*!
    @brief 関連付けるコミュニティの ID を設定します。
    @details inet::NexCreateSessionSetting::SetCommunityId で同じコミュニティ ID を指定して作成されたセッションを検索の対象とします。
     */
    void SetCommunityId(uint32_t gid);

    /*!
    @brief      あいことばマッチメイク用のあいことばを設定します。
    @param[in]  keyword   設定するあいことばマッチメイク用のあいことば。あいことばの文字列は SessionMatchmakeKeywordLength 以下である必要があります。
    @return     あいことばの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはあいことばの最大長を超えています。[:progErr]
    */
    Result SetSessionMatchmakeKeyword(const common::String& keyword);

    /*!
    @cond PRIVATE
    @brief      スコアベースマッチメイク指定時に、比較に使用するパラメータを設定します。
    @param[in]  index           パラメータのインデックス。AdditionalParamKeySizeMax 以下である必要があります。
    @param[in]  keyString       パラメータの名前。AdditionalParamKeyLength 以下である必要があります。
    @param[in]  value           パラメータの値
    @return     パラメータの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  パラメータのインデックスが範囲外、またはパラメータの名前の文字列が NULL、文字コードが不正、最大文字列長を超えています。[:progErr]
    */
    Result SetAdditionalScoreBasedParameterValue(uint32_t index, const common::String& keyString, const ParamVariant& value);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      スコアベースマッチメイク指定時に、比較に使用するパラメータを設定します。
    @param[in]  index           パラメータのインデックス。AdditionalParamStringSizeMax 以下である必要があります。
    @param[in]  keyString       パラメータの名前。AdditionalParamKeyLength 以下である必要があります。
    @param[in]  value           パラメータの文字列。AdditionalParamStringLength 以下である必要があります。
    @return     パラメータの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  パラメータのインデックスが範囲外、またはパラメータの名前やパラメータの文字列が NULL、文字コードが不正、最大文字列長を超えています。[:progErr]
    */
    Result SetAdditionalScoreBasedParameterString(uint32_t index, const common::String& keyString, const common::String& value);
    //! @endcond

    /*!
    @cond PRIVATE
    */
    const common::String* GetAdditionalScoreBasedParameterValueKey(uint32_t index) const;
    const ParamVariant& GetAdditionalScoreBasedParameterValue(uint32_t index) const;
    bool IsSetAdditionalScoreBasedParameterValue(uint32_t index) const;
    const common::String* GetAdditionalScoreBasedParameterStringKey(uint32_t index) const;
    const common::String* GetAdditionalScoreBasedParameterString(uint32_t index) const;
    bool IsSetAdditionalScoreBasedParameterString(uint32_t index) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      検索条件として設定した項目のビットマスクを取得します。
    @defails    ビットマスクの各ビットに対応する項目は enum NexCondMask で定義されています。
    @return     検索条件として設定した項目のビットマスクが返されます。
    */
    uint32_t GetCondMask() const;
    //! @endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    NexSessionSearchCriteria& operator=(const NexSessionSearchCriteria& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexSessionSearchCriteria& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexSessionSearchCriteria(const NexSessionSearchCriteria& rhs);

    /*!
    @cond PRIVATE
    @brief 設定されている情報を NEX の検索条件/検索範囲クラスに変換します。
    */
    Result ConvertToNexMatchmakeSessionSearchCriteria(
        nn::nex::MatchmakeSessionSearchCriteria* pCriteria,
        nn::nex::InetAddress* pNexInetAddress,
        nn::nex::MatchmakeParam* pMatchmakeParam,
        nn::nex::ResultRange* pResultRange,
        uint8_t reportNum = 2) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
     */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief 最低限設定が必要な項目が設定されているかチェックします。
    */
    bool IsValid() const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    uint32_t GetGameMode() const;
    bool IsSetGameMode() const;
    SessionType GetSessionType() const;
    bool IsSetSessionType() const;
    const uint32_t* GetAttributeList(uint32_t index) const;
    uint32_t GetAttributeListSize(uint32_t index) const;
    uint32_t GetAttributeRangeMin(uint32_t index) const;
    uint32_t GetAttributeRangeMax(uint32_t index) const;
    bool IsAttributeRangeUsed(uint32_t index) const;
    bool IsSetAttribute(uint32_t index) const;
    bool IsExcludeNonHostPrincipalId() const;
    bool IsSetExcludeNonHostPrincipalId() const;
    bool IsExcludeUserPasswordSet() const;
    bool IsSetExcludeUserPasswordSet() const;
    SelectionMethod GetRandomSessionSelectionMethod() const;
    bool IsSetRandomSessionSelectionMethod() const;
    uint32_t GetScoreBasedSettingIndex() const;
    bool IsSetScoreBasedSettingIndex() const;
    uint32_t GetScoreBasedRatingValue() const;
    bool IsSetScoreBasedRatingValue() const;
    uint32_t GetScoreBasedDisconnectionRate() const;
    bool IsSetScoreBasedDisconnectionRate() const;
    uint32_t GetScoreBasedViolationRate() const;
    bool IsSetScoreBasedViolationRate() const;
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
    const common::String* GetScoreBasedCountryCode() const;
#else
    uint32_t GetScoreBasedCountryCode() const;
#endif
    bool IsSetScoreBasedCountryCode() const;
    bool IsScoreBasedGeoIpUsed() const;
    bool IsSetScoreBasedUseGeoIp() const;
    const common::InetAddress& GetOverrideIpAddress() const;
    uint32_t GetCommunityId() const;
    bool IsSetCommunityId() const;
    bool IsOpenedOnly() const;
    bool IsSetOpenedOnly() const;
    bool IsVacantOnly() const;
    bool IsSetVacantOnly() const;
    uint16_t GetMinParticipantRangeMin() const;
    uint16_t GetMinParticipantRangeMax() const;
    bool IsSetMinParticipantNum() const;
    uint16_t GetMaxParticipantRangeMin() const;
    uint16_t GetMaxParticipantRangeMax() const;
    bool IsSetMaxParticipantNum() const;
    const common::String& GetSessionMatchmakeKeyword() const;
    bool IsSetSessionMatchmakeKeyword() const;
    //! @endcond

private:
    uint32_t m_GameMode;
    SessionType m_SessionType;
    uint32_t m_AttributeList[AttributeSizeMax][AttributeArraySizeMax];
    uint32_t m_AttributeSizeList[AttributeSizeMax];
    uint32_t m_AttributeRangeMin[AttributeSizeMax];
    uint32_t m_AttributeRangeMax[AttributeSizeMax];
    bool m_bAttributeRangeUsed[AttributeSizeMax];
    bool m_bExcludeNonHostPrincipalId;
    bool m_bExcludeUserPasswordSet;
    SelectionMethod m_SelectionMethod;

    uint32_t m_ScoreBasedSettingIndex;
    uint32_t m_ScoreBasedRatingValue;
    uint32_t m_ScoreBasedDisconnectionRate;
    uint32_t m_ScoreBasedViolationRate;
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
    common::FixedString<CountryCodeLengthMax> m_ScoreBasedCountryCode;
#else
    uint32_t m_ScoreBasedCountryCode;
#endif
    bool m_IsScoreBasedGeoIpUsed;

    common::InetAddress m_IpAddress;
    uint32_t m_CommunityId;

    uint16_t m_MinParticipantsRangeMax;
    uint16_t m_MinParticipantsRangeMin;
    uint16_t m_MaxParticipantsRangeMax;
    uint16_t m_MaxParticipantsRangeMin;
    bool m_bOpenedOnly;
    bool m_bVacantOnly;

    common::FixedString<AdditionalParamKeyLength> m_AdditionalParamValueKey[AdditionalParamKeySizeMax];
    struct ParamVariant m_AdditionalParamValue[AdditionalParamKeySizeMax];

    common::FixedString<AdditionalParamKeyLength> m_AdditionalParamStringKey[AdditionalParamStringSizeMax];
    common::FixedString<AdditionalParamStringLength> m_AdditionalParamString[AdditionalParamStringSizeMax];

    common::FixedString<SessionMatchmakeKeywordLength> m_MatchmakeKeyword;

    uint32_t m_SetCondMask;
};
}
}
}  // end of namespace nn::pia::inet

#endif
