﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>
#include <nn/pia/session/session_SessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  マッチメイクセッションのオーナー（通常はセッションホストと同じ）を検索条件に指定するクラスです。
  @details 検索結果として得られるセッションの最大数は SetResultRange で指定する値になりますが、
           nn::pia::session::Session::CreateInstance 呼び出し時に指定する
           nn::pia::session::Session::Setting 構造体の browsedSessionInfoListNum 以下の値を
           指定する必要があります。

  @see nn::pia::session::SessionSearchCriteria
 */
class NexSessionSearchCriteriaOwner : public nn::pia::session::SessionSearchCriteria
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexSessionSearchCriteriaOwner();

    /*!
    @brief      コンストラクタ。
    @param[in]  ownerPrincipalId    検索条件として指定するオーナーのプリンシパル ID。
    @param[in]  resultOffset        検索条件にマッチしたエントリーリストから検索結果として取得するエントリー群の開始位置。
    @param[in]  resultSize          検索条件にマッチしたエントリーリストから検索結果として取得するエントリー数。
                                    session::Session::Setting 構造体の browsedSessionInfoListNum 以下、かつSearchCriteriaResultSizeMax 以下の値である必要があります。
    */
    NexSessionSearchCriteriaOwner(PrincipalId ownerPrincipalId, uint32_t resultOffset, uint32_t resultSize);

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexSessionSearchCriteriaOwner();

    /*!
    @brief      検索条件として指定するオーナーのプリンシパル ID をセットします。
    @param[in]  ownerPrincipalId    検索条件として指定するオーナーのプリンシパル ID。
     */
    void SetOwnerPrincipalId(PrincipalId ownerPrincipalId);

    /*!
    @brief      検索結果の取得範囲を指定します。
    @details    offset に 0xFFFFFFFF を指定すると、検索結果をランダムに並び替えた上で指定されたエントリー数を取得するように振る舞います。
    @param[in]  offset  検索条件にマッチしたエントリーリストから検索結果として取得するエントリー群の開始位置。
    @param[in]  size    検索条件にマッチしたエントリーリストから検索結果として取得するエントリー数。
                        session::Session::Setting 構造体の browsedSessionInfoListNum 以下、かつSearchCriteriaResultSizeMax 以下の値である必要があります。

    @return     処理に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval     nn::pia::ResultInvalidArgument  引数の値が範囲外です。[:progErr]
    */
    virtual Result SetResultRange(uint32_t offset, uint32_t size);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    NexSessionSearchCriteriaOwner& operator=(const NexSessionSearchCriteriaOwner& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexSessionSearchCriteriaOwner& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexSessionSearchCriteriaOwner(const NexSessionSearchCriteriaOwner& rhs);

    /*!
    @cond PRIVATE
     */
    PrincipalId GetOwnerPrincipalId() const;
    //! @endcond

    /*!
    @brief      検索条件をリセットします。
     */
    virtual void Reset();

private:
    PrincipalId m_OwnerPrincipalId;
};
}
}
}  // end of namespace nn::pia::inet

#endif
