﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>
#include <nn/pia/session/session_SessionSearchCriteria.h>

#if NN_PIA_NINTENDOSDK
#include <nn/account.h>
#endif

namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  プリンシパル ID をセッションの検索条件として指定するクラスです。
  @details 指定したプリンシパル ID のステーションが参加しているセッションを検索します。
           検索結果として得られるセッションの最大数は nn::pia::session::Session::CreateInstance 呼び出し時に
           指定する nn::pia::session::Session::Setting 構造体の browsedSessionInfoListNum の値になります。

  @see nn::pia::session::SessionSearchCriteria
 */
class NexSessionSearchCriteriaParticipant : public session::SessionSearchCriteria
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexSessionSearchCriteriaParticipant();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexSessionSearchCriteriaParticipant();

    /*!
    @brief      検索条件として指定する相手のプリンシパル ID をセットします。
    @param[in]  idList      検索条件として指定する相手のプリンシパル ID のリスト。
    @param[in]  idListSize  検索条件として指定する相手のプリンシパル ID のリストサイズ。ParticipantArraySizeMax 以下である必要があります。
    @return     検索条件の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  リストが NULL、または値の個数が範囲外です。[:progErr]
    */
    Result SetParticipantPrincipalId(const PrincipalId* idList, uint32_t idListSize);

#if NN_PIA_NINTENDOSDK
    /*!
    @if NIN_DOC
    @brief      検索条件として指定する相手のネットワークサービスアカウント ID をセットします。
    @param[in]  idList      検索条件として指定する相手のネットワークサービスアカウント ID のリスト。
    @param[in]  idListSize  検索条件として指定する相手のネットワークサービスアカウント ID のリストサイズ。ParticipantArraySizeMax 以下である必要があります。
    @return     検索条件の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  リストが NULL、または値の個数が範囲外です。[:progErr]
    @endif
    */
    Result SetParticipantPrincipalIdByNetworkServiceAccountId(nn::account::NetworkServiceAccountId* idList, uint32_t idListSize);
#endif

    /*!
    @brief      検索条件として指定するブロックリスト設定をセットします。
    @param[in]  isMyBlockListUsed      自分がブロックリストに登録しているユーザーをチェックするかどうか。デフォルトは true です。
    @param[in]  isOthersBlockListUsed  自分をブロックリストに登録しているユーザをチェックするかどうか。デフォルトは true です。
    */
    void SetBlockListOption(bool isMyBlockListUsed, bool isOthersBlockListUsed);

    /*!
    @brief      検索結果にアプリケーション定義データを含めるかどうかをセットします。
    @details    アプリケーション定義データを必要としない場合は、サーバー負荷軽減のため、含めてはいけません。
    @param[in]  isEnabled   検索結果にアプリケーション定義データを含めるかどうか。デフォルトは false です。
    */
    void SetApplicationDataEnabled(bool isEnabled);

    /*!
    @cond PRIVATE
     */
    const PrincipalId* GetParticipantPrincipalId() const;
    uint32_t GetParticipantPrincipalIdSize() const;
    bool IsNetworkServiceAccountIdUsed() const;
    bool IsMyBlockListUsed() const;
    bool IsOthersBlockListUsed() const;
    bool IsApplicationDataEnabled() const;
    //! @endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    NexSessionSearchCriteriaParticipant& operator=(const NexSessionSearchCriteriaParticipant& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexSessionSearchCriteriaParticipant& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexSessionSearchCriteriaParticipant(const NexSessionSearchCriteriaParticipant& rhs);

    /*!
    @brief      検索条件をリセットします。
     */
    virtual void Reset();

protected:
    PrincipalId m_ParticipantPrincipalId[ParticipantArraySizeMax];
    uint32_t m_ParticipantPrincipalIdNum;
    bool m_IsNetworkServiceAccoundIdUsed;
    bool m_IsMyBlockListUsed;
    bool m_IsOthersBlockListUsed;
    bool m_IsApplicationDataEnabled;

private:
    // SetResultRange は封印。
    virtual Result SetResultRange(uint32_t offset, uint32_t size);
};
}
}
}  // end of namespace nn::pia::inet

#endif
