﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_DateTime.h>
#include <nn/pia/session/session_UpdateCommunitySetting.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
  @brief  コミュニティ設定の更新処理に関する情報を管理するクラスです。
  @details 更新する値を指定した設定のみ更新されます。更新しない設定について、指定は不要です。

  @see session::UpdateCommunitySetting
 */
class NexUpdateCommunitySetting : public session::UpdateCommunitySetting
{
public:
    /*!
    @cond PRIVATE
    @brief 指定した項目の示すビットマスクの各ビットに対応する項目
    */
    enum NexUpdateCommunitySettingMask
    {
        NexUpdateCommunitySettingMask_MinParticipant = 0,       //!< 最小参加人数
        NexUpdateCommunitySettingMask_MaxParticipant,           //!< 最大参加人数
        NexUpdateCommunitySettingMask_StartDate,                //!< 開始日時
        NexUpdateCommunitySettingMask_EndDate,                  //!< 終了日時
        NexUpdateCommunitySettingMask_Description,              //!< 説明文字列
        NexUpdateCommunitySettingMask_Attribute,                //!< 属性
        NexUpdateCommunitySettingMask_ApplicationData,          //!< アプリケーション定義データ
        NexUpdateCommunitySettingMask_CommunityUserPassword     //!< ユーザーパスワード
    };
    //! @endcond

    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexUpdateCommunitySetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexUpdateCommunitySetting();

    /*!
    @brief      コミュニティの最小参加人数を指定します。
    @param[in]  min 作成するコミュニティに指定する最小参加人数。
    */
    void SetMinParticipantNum(uint16_t min);

    /*!
    @brief      コミュニティの最大参加人数を指定します。
    @param[in]  max 作成するコミュニティに指定する最大参加人数。
    */
    void SetMaxParticipantNum(uint16_t max);

    /*!
    @brief      コミュニティの属性を一括して指定します。
    @param[in]  attributes   属性の配列。配列のサイズは @ref AttributeSizeMax である必要があります。
    */
    void SetAttributes(const uint32_t attributes[nn::pia::inet::AttributeSizeMax]);

    /*!
    @brief      コミュニティのユーザーパスワードを指定します。
    @param[in]  userPassword   設定するユーザーパスワード。パスワードの文字列は CommunityUserPasswordLengthMax 以下である必要があります。
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
     */
    Result SetCommunityUserPassword(const common::String& userPassword);

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @details    inet::ApplicationDataBufferSizeMax を超えないサイズを指定する必要があります。
    @param[in]  pData   コミュニティに指定するアプリケーション定義データのバッファ。
    @param[in]  size    コミュニティに指定するアプリケーション定義データのサイズ。ApplicationDataBufferSizeMax 以下である必要があります。
    @return     アプリケーション定義データの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @brief      コミュニティの開始日時の更新日時を指定します。
    @param[in]  startDate コミュニティの参加可能開始日時。
    @return     コミュニティの参加可能開始日時の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  日時が不正です。[:progErr]
    */
    Result SetParticipationStartDate(const common::DateTime& startDate);

    /*!
    @brief      コミュニティの参加終了日時の更新日時を指定します。
    @param[in]  endDate コミュニティの参加終了日時。
    @return     コミュニティの参加終了日時の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  日時が不正です。[:progErr]
    */
    Result SetParticipationEndDate(const common::DateTime& endDate);

    /*!
    @brief      コミュニティの説明文字列を更新します。
    @param[in]  description 更新後のコミュニティの説明文字列。文字列の長さは DescriptionLengthMax 以下である必要があります。
    @return     説明文字列のコピーに成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または説明文字列の最大長を超えています。[:progErr]
    */
    Result SetDescription(const common::String& description);

    /*!
    @brief      更新するコミュニティIDを指定します。
    @param[in]  communityId     更新するコミュニティID。
    */
    void SetCommunityId(uint32_t communityId);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexUpdateCommunitySetting& operator=(const NexUpdateCommunitySetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexUpdateCommunitySetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexUpdateCommunitySetting(const NexUpdateCommunitySetting& rhs);

    /*!
    @brief 設定をリセットします。
    */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    uint32_t GetCommunityId() const;
    bool IsSetCommunityId() const;
    uint16_t GetMinParticipantNum() const;
    bool IsSetMinParticipantNum() const;
    uint16_t GetMaxParticipantNum() const;
    bool IsSetMaxParticipantNum() const;
    Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    bool IsSetApplicationData() const;
    uint32_t GetApplicationDataSize() const;

    uint32_t GetAttribute(uint32_t index) const;
    bool IsSetAttribute() const;
    
    const common::String& GetDescription() const;
    bool IsSetDescription() const;

    const common::String& GetCommunityUserPassword() const;
    bool IsSetCommunityUserPassword() const;

    const common::DateTime& GetParticipaitonStartDate() const;
    bool IsSetParticipationStartDate() const;
    const common::DateTime& GetParticipaitonEndDate() const;
    bool IsSetParticipationEndDate() const;

    void CopyApplicationDataToVector(void* pVectorTmp) const;
    bool IsValid() const;
    //! @endcond

protected:
    uint32_t m_CommunityId;
    uint16_t m_MinParticipantNum;
    uint16_t m_MaxParticipantNum;
    uint32_t m_Attribute[AttributeSizeMax];
    uint8_t m_ApplicationDataBuffer[ApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;
    common::FixedString<DescriptionLengthMax> m_Description;
    common::FixedString<SessionUserPasswordLengthMax> m_UserPassword;
    common::DateTime m_StartTime;
    common::DateTime m_EndTime;
    uint32_t m_Mask;
private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
