﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_DateTime.h>
#include <nn/pia/session/session_UpdateSessionSetting.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
  @brief  セッション設定の更新処理に関する情報を管理するクラスです。
  @details 更新する値を指定した設定のみ更新されます。更新しない設定について、指定は不要です。

  @see session::UpdateSessionSetting
 */
class NexUpdateSessionSetting : public session::UpdateSessionSetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexUpdateSessionSetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexUpdateSessionSetting();

    /*!
    @brief      セッションの最小参加人数を指定します。
    @param[in]  min 作成するセッションに指定する最小参加人数。
    */
    void SetMinParticipantNum(uint16_t min);

    /*!
    @brief      セッションの最大参加人数を指定します。
    @param[in]  max 作成するセッションに指定する最大参加人数。
    */
    void SetMaxParticipantNum(uint16_t max);

    /*!
    @brief      セッションのセッションタイプとホストマイグレーション時にセッションを自動的に参加締切状態にする設定を指定します。
    @if CTR_DOC
    @details    セッションタイプの変更は inet::SessionType_Friend から inet::SessionType_Anybody への更新のみ可能です。
                セッションタイプを変更せず、自動的に参加締切状態に設定のみを更新することも可能です。デフォルトでは自動的に参加締切状態に変更しない設定になります。
    @endif
    @if CAFE_DOC
    @details    セッションタイプの変更は inet::SessionType_Friend から inet::SessionType_Anybody への更新のみ可能です。
    セッションタイプを変更せず、自動的に参加締切状態に設定のみを更新することも可能です。デフォルトでは自動的に参加締切状態に変更しない設定になります。
    @endif
    @if NIN_DOC
    @details    自動的に参加締切状態に設定のみを更新可能です。デフォルトでは自動的に参加締切状態に変更しない設定になります。
    @endif
    @param[in]  sessionType セッションタイプ。
    @param[in]  isAutoCloseSession  ホストマイグレーション時にセッションを自動的に参加締切状態にする設定。
    */
    void SetSessionType(SessionType sessionType, bool isAutoCloseSession = false);

    /*!
    @brief      セッションの属性を一括して指定します。
    @param[in]  attributes   属性の配列。配列のサイズは @ref AttributeSizeMax である必要があります。
    */
    void SetAttributes(const uint32_t attributes[nn::pia::inet::AttributeSizeMax]);

    /*!
    @brief      セッションの属性をインデックスを指定して設定します。
    @details    本関数でセッションの属性を設定する場合は、他の項目を併せて設定できません。
    @param[in]  index        指定するセッションの属性のインデックス。AttributeSizeMax 以下である必要があります。
    @param[in]  value        指定するセッションの属性の値。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外です。[:progErr]
    @retval     nn::pia::ResultInvalidState     他の項目が設定されています。[:progErr]
    */
    Result SetAttribute(uint32_t index, uint32_t value);

    /*!
    @brief      セッションのユーザーパスワードを指定します。
    @param[in]  userPassword   設定するユーザーパスワード。パスワードの文字列は SessionUserPasswordLengthMax 以下である必要があります。
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
     */
    Result SetSessionUserPassword(const common::String& userPassword);

    /*!
    @brief      あいことばマッチメイク用のあいことばを指定します。
    @param[in]  keyword   設定するあいことばマッチメイク用のあいことば。あいことばの文字列は SessionMatchmakeKeywordLength 以下である必要があります。
    @return     あいことばの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはあいことばの最大長を超えています。[:progErr]
    */
    Result SetSessionMatchmakeKeyword(const common::String& keyword);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用するレーティング値を更新します。
    @param[in]  ratingValue        レーティング値
    */
    void SetRatingValue(uint32_t ratingValue);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用する切断率を更新します。
    @param[in]  disconnectionRate        切断率
    */
    void SetDisconnectionRate(uint32_t disconnectionRate);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用する違反率を更新します。
    @param[in]  violationRate        違反率
    */
    void SetViolationRate(uint32_t violationRate);

    /*!
    @if NIN_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはISO 3166-1 alpha-2 によるアルファベット 2 文字（大文字）の国コードを設定します。
    @details    @ref SetUpdateGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
    位置情報が得られた場合はここで指定した国コードは使用されません。
    国コードをセットしない場合は @ref SetUpdateGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
    国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード。ISO 3166-1 alpha-2 によるアルファベット 2 文字（大文字）の文字列である必要があります。
    @return     国コードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  国コードの文字列長が 2 以外、または国コードに指定した文字種が不正です。[:progErr]
    @endif
    */
    Result SetCountryCode(const common::String& countryCode);

    /*!
    @if CTR_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはプラットフォームで定義された値を設定します。
    @details    @ref SetUpdateGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
    位置情報が得られた場合はここで指定した国コードは使用されません。
    国コードをセットしない場合は @ref SetUpdateGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
    国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード
    @endif
    @if CAFE_DOC
    @brief      スコアベースマッチメイク指定時に、比較に使用する国コードを設定します。国コードはプラットフォームで定義された値を設定します。
    @details    @ref SetUpdateGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合に使用する国コードをセットします。
    位置情報が得られた場合はここで指定した国コードは使用されません。
    国コードをセットしない場合は @ref SetUpdateGeoIp で位置情報を使用しない設定をした場合、もしくは位置情報を使用する設定をしたが位置情報を得られなかった場合、
    国コードの比較に関するスコアがデフォルトスコアとして評価されます。
    @param[in]  countryCode        国コード
    @endif
    */
    void SetCountryCode(uint32_t countryCode);

    /*!
    @brief      スコアベースマッチメイク指定時に、比較に使用する位置情報 (緯度経度、国コード) を更新するか設定します。
    @param[in]  use 位置情報を更新するかどうか
    */
    void SetUpdateGeoIp(bool use);

    /*!
    @brief      同じ条件のセッションが見つかった際のマッチメイクの優先度を指定します。
    @details    作成したセッションのマッチメイクの優先度を指定します。
                nn::pia::inet::SelectionMethod_BroadenRangeWithSelectionPriority, 
                nn::pia::inet::SelectionMethod_ScoreBased で検索を行った場合のみ有効です。
                nn::pia::inet::SessionSelectionPriorityMax を超えた値を引数にした場合、nn::pia::inet::SessionSelectionPriorityMax が設定されます。
    @param[in]  priority   指定するマッチメイクの優先度
     */
    void SetSelectionPriority(uint8_t priority);

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @details    inet::ApplicationDataBufferSizeMax を超えないサイズを指定する必要があります。
    @param[in]  pData   セッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    セッションに指定するアプリケーション定義データのサイズ。ApplicationDataBufferSizeMax 以下である必要があります。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータサイズを取得します。
    @return     アプリケーション定義データサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSize() const;
    //! @endcond

    /*!
    @brief      セッションの開始日時の更新日時を指定します。
    @details    現在時刻より未来の日時が設定された場合は、現在時刻がサーバーでセットされます。
    @param[in]  dateTime 開始日時
    @return     セッションの開始日時の更新日時の指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   日時が設定されていない、または設定した日時が正しくありません。[:progErr]
    */
    Result SetStartedTime(const common::DateTime& dateTime);


    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexUpdateSessionSetting& operator=(const NexUpdateSessionSetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexUpdateSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexUpdateSessionSetting(const NexUpdateSessionSetting& rhs);

    /*!
    @brief 設定をリセットします。
    */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      スコアベースマッチメイク指定時に、比較に使用するパラメータを設定します。
    @param[in]  index           パラメータのインデックス。AdditionalParamKeySizeMax 以下である必要があります。
    @param[in]  keyString       パラメータの名前。AdditionalParamKeyLength 以下である必要があります。
    @param[in]  value           パラメータの値
    @return     パラメータの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  パラメータのインデックスが範囲外、またはパラメータの名前の文字列が NULL、文字コードが不正、最大文字列長を超えています。[:progErr]
    */
    Result SetAdditionalScoreBasedParameterValue(uint32_t index, const common::String& keyString, const struct ParamVariant& value);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      スコアベースマッチメイク指定時に、比較に使用するパラメータを設定します。
    @param[in]  index           パラメータのインデックス。AdditionalParamStringSizeMax 以下である必要があります。
    @param[in]  keyString       パラメータの名前。AdditionalParamKeyLength 以下である必要があります。
    @param[in]  value           パラメータの文字列。AdditionalParamStringLength 以下である必要があります。
    @return     パラメータの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  パラメータのインデックスが範囲外、またはパラメータの名前やパラメータの文字列が NULL、文字コードが不正、最大文字列長を超えています。[:progErr]
    */
    Result SetAdditionalScoreBasedParameterString(uint32_t index, const common::String& keyString, const common::String& value);
    //! @endcond

    /*!
    @cond PRIVATE
    */
    const common::String* GetAdditionalScoreBasedParameterValueKey(uint32_t index) const;
    const struct ParamVariant& GetAdditionalScoreBasedParameterValue(uint32_t index) const;
    bool IsSetAdditionalScoreBasedParameterValue(uint32_t index) const;
    const common::String* GetAdditionalScoreBasedParameterStringKey(uint32_t index) const;
    const common::String* GetAdditionalScoreBasedParameterString(uint32_t index) const;
    bool IsSetAdditionalScoreBasedParameterString(uint32_t index) const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    uint16_t GetMinParticipantNum() const;
    bool IsSetMinParticipantNum() const;
    uint16_t GetMaxParticipantNum() const;
    bool IsSetMaxParticipantNum() const;
    SessionType GetSessionType() const;
    bool IsAutoCloseSession() const;
    bool IsSetSessionType() const;
    uint32_t GetAttribute(uint32_t index) const;
    bool IsAttributeUpdated(uint32_t index) const;
    const common::String& GetSessionUserPassword() const;
    bool IsSetSessionUserPassword() const;
    const common::String& GetSessionMatchmakeKeyword() const;
    bool IsSetSessionMatchmakeKeyword() const;
    const common::DateTime& GetStartedTime() const;
    bool IsSetStartedTime() const;
    uint32_t GetRatingValue() const;
    bool IsSetRatingValue() const;
    uint32_t GetDisconnectionRate() const;
    bool IsSetDisconnectionRate() const;
    uint32_t GetViolationRate() const;
    bool IsSetViolationRate() const;
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
    const common::String* GetCountryCode() const;
#else
    uint32_t GetCountryCode() const;
#endif
    bool IsSetCountryCode() const;
    bool GetUpdateGeoIp() const;
    bool IsSetUpdateGeoIp() const;
    uint8_t GetSelectionPriority() const;
    bool IsSetSelectionPriority() const;
    void CopyApplicationDataToVector(void* pVectorTmp) const;
    bool IsSetScoreBaseParam() const;
    uint32_t GetNexModificationFlag() const;
    bool IsOnlyApplicationDataUpdated() const;
    bool IsOnlyAttributeUpdated() const;
    bool IsOnlyAllAttributesUpdated() const;
    bool IsValid() const;
    //! @endcond

protected:
    uint16_t m_MinParticipantNum;
    uint16_t m_MaxParticipantNum;
    SessionType m_SessionType;
    bool m_IsAutoCloseSession;
    uint32_t m_Attribute[AttributeSizeMax];
    bool m_IsAttributeUpdated[AttributeSizeMax];
    uint8_t m_ApplicationDataBuffer[ApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;
    uint8_t m_SelectionPriority;
    common::FixedString<SessionUserPasswordLengthMax> m_UserPassword;
    common::FixedString<SessionMatchmakeKeywordLength> m_MatchmakeKeyword;
    common::DateTime m_StartedTime;
    uint32_t m_RatingValue;
    bool m_IsSetRatingValue;
    uint32_t m_DisconnectionRate;
    bool m_IsSetDisconnectionRate;
    uint32_t m_ViolationRate;
    bool m_IsSetViolationRate;
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
    common::FixedString<CountryCodeLengthMax> m_CountryCode;
#else
    uint32_t m_CountryCode;
#endif
    bool m_IsSetCountryCode;
    bool m_IsUpdateGeoIp;
    bool m_IsSetUpdateGeoIp;

    common::FixedString<AdditionalParamKeyLength> m_AdditionalParamValueKey[AdditionalParamKeySizeMax];
    struct ParamVariant m_AdditionalParamValue[AdditionalParamKeySizeMax];

    common::FixedString<AdditionalParamKeyLength> m_AdditionalParamStringKey[AdditionalParamStringSizeMax];
    common::FixedString<AdditionalParamStringLength> m_AdditionalParamString[AdditionalParamStringSizeMax];

    uint32_t m_NexModificationFlag;

    /*!
    @cond PRIVATE
    */
    void AddNexModificationFlag(uint32_t flag);
    bool IsNexModificationFlagSet(uint32_t bit) const;
    //! @endcond

private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
