﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_LanSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/session/session_CreateSessionSetting.h>

namespace nn
{
namespace pia
{
namespace lan
{
/*!
  @brief  LAN マッチメイクのセッション構築処理に関する設定を管理するクラスです。

  @see nn::pia::session::CreateSessionSetting
 */
class LanCreateSessionSetting : public nn::pia::session::CreateSessionSetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    LanCreateSessionSetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~LanCreateSessionSetting();

    /*!
    @cond PRIVATE
    @brief 指定した項目の示すビットマスクの各ビットに対応する項目
    */
    enum LanCreateSessionSettingMask
    {
        LanCreateSessionSettingMask_GameMode = 0,                                                                                               //!< ゲームモード
        LanCreateSessionSettingMask_SessionType,                                                                                                //!< セッションタイプ
        LanCreateSessionSettingMask_Attribute,                                                                                                  //!< 属性
        LanCreateSessionSettingMask_ApplicationData = LanCreateSessionSettingMask_Attribute + LanAttributeSizeMax,                              //!< アプリケーション定義データ
        LanCreateSessionSettingMask_OpenSession                                                                                                 //!< 参加可能かどうか
    };
    //! @endcond

    /*!
    @brief      構築するセッションのゲームモードを指定します。
    @param[in]  gameMode    構築するセッションに指定するゲームモード。
    */
    void SetGameMode(uint32_t gameMode);

    /*!
    @cond PRIVATE
    */
    uint32_t GetGameMode() const;
    bool IsSetGameMode() const;
    /*! @endcond */

    /*!
    @brief      構築するセッションタイプを指定します。
    @param[in]  sessionType 構築するセッションに指定するセッションタイプ。
    */
    void SetSessionType(LanSessionType sessionType);

    /*!
    @cond PRIVATE
    */
    LanSessionType GetSessionType() const;
    bool IsSetSessionType() const;
    /*! @endcond */

    /*!
    @brief      構築するセッションに属性を指定します。
    @param[in]  index   構築するセッションに指定する属性のインデックス。LanAttributeSizeMax 以下である必要があります。
    @param[in]  value   構築するセッションに指定する属性の値。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外です。[:progErr]
    */
    Result SetAttribute(uint32_t index, uint32_t value);

    /*!
    @cond PRIVATE
    */
    uint32_t GetAttribute(uint32_t index) const;
    bool IsSetAttribute(uint32_t index) const;
    /*! @endcond */

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @param[in]  pData   構築するセッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    構築するセッションに指定するアプリケーション定義データのサイズ。LanApplicationDataBufferSizeMax 以下である必要があります。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    virtual uint32_t GetApplicationDataSize() const;
    virtual bool IsSetApplicationData() const;
    //! @endcond

    /*!
    @brief      構築したセッションが参加可能で構築されるか、参加不可で構築されるかを指定します。
    @param[in]  isOpen  true を設定した場合、セッションは参加可能な状態で構築されます。
     */
    void SetOpenSession(bool isOpen);

    /*!
    @cond PRIVATE
     */
    bool IsOpenSession() const;
    bool IsSetOpenSession() const;
    //! @endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    LanCreateSessionSetting& operator=(const LanCreateSessionSetting& rhs);

    /*!
    @brief      設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const LanCreateSessionSetting& rhs);

    /*!
    @brief      コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    LanCreateSessionSetting(const LanCreateSessionSetting& rhs);

    /*!
    @brief      設定をリセットします。
     */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief      デバッグに有用な情報をプリントします。
    @param[in]  flag  トレースフラグです。詳細は @ref TraceFlag 型を参照してください。
    @endcond
     */
    virtual void Trace(uint64_t flag) const;


protected:
    uint32_t m_GameMode;
    LanSessionType m_SessionType;
    uint32_t m_Attribute[LanAttributeSizeMax];
    uint8_t m_ApplicationDataBuffer[LanApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;
    bool m_OpenParticipation;
    uint32_t m_SetCondMask;

private:
};
}
}
}  // end of namespace nn::pia::lan
