﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_Definitions.h>

namespace nn
{
namespace pia
{
namespace common
{
class Socket;
}
}
}

namespace nn
{
namespace pia
{
namespace transport
{
class StationConnectionInfo;
}
}
}

namespace nn
{
namespace pia
{
namespace lan
{
class LanProtocol;
class LanMatchmakeUpdateJob;
class SocketOutputStream;
class SocketInputStream;

/*!
  @brief    LAN マッチメイクのファサード（facade、インターフェースラッパー）クラスです。
  @details  同一サブネットの LAN 上に存在する環境の場合、NEX を使用せず、
            ネットワーク関連処理（送受信、セッション管理など）を行えます。
 */
class LanFacade : public nn::pia::common::RootObject
{
public:

    /*!
    @brief  LAN マッチメイクで使用するネットワーク設定の構造体です。
    */
    struct LanNetworkSetting
    {
        LanNetworkSetting() : localAddress(0), subnetmask(0xffffff00), localCommunicationVersion(0)
        {}

        uint32_t localAddress;          //!< ローカルに割当てられている IP アドレス
        uint32_t subnetmask;            //!< ローカルの IP アドレスのサブネットマスク。指定しない場合は 255.255.255.0 が指定されます。
        int localCommunicationVersion;  //!< 通信バージョンです。アプリケーションへのパッチの適用などで通信の仕様が変わる場合に、通信バージョンをインクリメントしておくことで、通信の互換性がないことを表明できます。このバージョンが異なるセッションは、検索可能ですが接続しようとすると失敗します。
    };

    /*!
    @brief    インスタンスを作成します（シングルトンパターン）。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultNotInitialized     lan モジュールが未初期化です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]
    @retval   nn::pia::ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
     */
    static Result CreateInstance(void);

    /*!
    @brief インスタンスを破棄します（シングルトンパターン）。

    @details インスタンスが作成されていないときにこの関数が呼び出された場合は、
           何も実行せずに関数から返ります。
     */
    static void DestroyInstance(void);

    /*!
    @brief    LanFacade インスタンスへのポインタを取得します（シングルトンパターン）。

    @return インスタンスが作成されていないときは、 NULL ポインタが返ります。
     */
    static LanFacade* GetInstance(void);

    /*!
    @cond PRIVATE
    @brief    LanFacade 機能をスタートアップします。

    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidState       呼び出すタイミングが間違っています。[:progErr]
    */
    Result Startup();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief    LanFacade 機能をクリーンアップします。
     */
    void Cleanup();
    //! @endcond

    void RegisterLanProtocol(LanProtocol* pLanProtocol);
    void UnregisterLanProtocol();
    LanProtocol* GetLanProtocol()
    {
        return m_pLanProtocol;
    }
    const LanProtocol* GetLanProtocol() const
    {
        return m_pLanProtocol;
    }

    /*!
    @brief    LAN マッチメイクで使用するネットワーク設定を登録します。
    @details  ローカルに割当てられている IP アドレス、サブネットマスク、通信バージョンを登録します。@ref session::Session::Startup を呼ぶ前に呼び出す必要があります。
    @param[in] setting LAN マッチメイクで使用するネットワーク設定
    @return   関数の実行結果を返します。 処理が失敗した場合、以下を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   nn::pia::ResultInvalidArgument    引数に誤りがあります。[:progErr]
     */
    Result Bind(const LanNetworkSetting& setting);

    /*!
    @brief    登録したローカルアドレスの情報を削除します。
    @details  @ref session::Session::Cleanup を呼び出した後に呼び出す必要があります。
    */
    void Unbind();

    /*!
    @cond PRIVATE
    @brief    LAN セッションを開始します。
     */
    Result StartLanSession();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief    LAN セッションを停止します。
     */
    void StopLanSession();
    //! @endcond

    bool RequestBrowseMessage(uint8_t criteriaNum);
    bool IsBrowseMessageCompleted() const;

    /*!
    @cond PRIVATE
    @brief    通信バージョンを取得します。
    */
    int GetLocalCommunicationVersion() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief    自分の PrincipalId を取得します。
     */
    PrincipalId GetLocalPrincipalId() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief    自分の StationConnectionInfo を取得します。
     */
    transport::StationConnectionInfo& GetLocalStationConnectionInfo();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリントします。
    
    @param[in] flag  トレースフラグです。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;
    //! @endcond

private:
    // シングルトンインスタンスへのポインタです。
    static LanFacade* s_pInstance;

    /*!
    @brief シングルトンパターンのため、コンストラクタは封印します。
     */
    LanFacade(void);

    /*!
    @brief シングルトンパターンのため、デストラクタは封印します。
     */
    ~LanFacade(void);

    /*!
    @brief シングルトンパターンのため、コピーコンストラクタは封印します。
     */
    LanFacade(const LanFacade&);

    /*!
    @brief シングルトンパターンのため、代入演算子は封印します。
     */
    LanFacade& operator=(const LanFacade&);

    Result Initialize();
    void Finalize();

private:
    LanProtocol* m_pLanProtocol;
    LanNetworkSetting m_LanNetworkSetting;
};
}
}
}  // end of namespace nn::pia::lan
