﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_Definitions.h>
#include <nn/pia/lan/lan_LanSessionTypes.h>
#include <nn/pia/transport/transport_NetworkFactory.h>

namespace nn
{
namespace pia
{
namespace lan
{
const uint16_t LanMaxStationNum = 16;
/*!
  @brief    同一サブネットの LAN ネットワークに必要なクラスを生成するためのクラスです（ファクトリパターン）。

  @details  ライブラリ開発者向けの機能です。アプリ開発者は通常使用できません。
            同一サブネットの LAN 上に存在する環境の場合、@ref nn::pia::transport::Transport や @ref nn::pia::session::Session に、
            LanNetworkFactory クラスを設定することで NEX を使用せず、ネットワーク関連処理（送受信、セッション管理など）が
            行われるようになります。

  @see nn::pia::transport::Transport::Setting, nn::pia::session::Session::Setting
 */
class LanNetworkFactory : public nn::pia::transport::NetworkFactory
{
public:
    /*!
    @cond PRIVATE
     */
    /*!
     @name 初期化終了処理
     @{
     */

    /*!
    @brief デフォルトコンストラクタ
     */
    LanNetworkFactory()
    {
    }

    /*!
    @brief デストラクタ
     */
    virtual ~LanNetworkFactory()
    {
    }

    /*!
     end of name 初期化終了処理
     @}
     */
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief 最大参加可能ステーション数を返します。
    */
    virtual uint16_t GetMaxStationNum() const
    {
        return LanMaxStationNum;
    }
    /*! @endcond */

    virtual transport::NetworkFactory* CreateSelf();

    /*!
    @cond PRIVATE
    @brief InputStream を生成します。
     */
    virtual nn::pia::common::IPacketInput* CreateInputStream();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief OutputStream を生成します。
     */
    virtual nn::pia::common::IPacketOutput* CreateOutputStream();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief ConnectStationJob を生成します。
     */
    virtual nn::pia::transport::ConnectStationJob* CreateConnectStationJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief DisconnectStationJob を生成します。
     */
    virtual nn::pia::transport::DisconnectStationJob* CreateDisconnectStationJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief CreateMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateCreateMeshJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief JoinMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateJoinMeshJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief LeaveMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateLeaveMeshJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief 必要なプロトコルを構築します。
     */
    virtual Result CreateProtocols();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief ProcessHostMigrationJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateProcessHostMigrationJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief LeaveWithHostMigrationJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateLeaveWithHostMigrationJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief KickoutManageJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateKickoutManageJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief PacketHandler の具象インスタンスを構築します。
     */
    virtual transport::PacketHandler* CreatePacketHandler();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief 署名が必要かどうか。
     */
    virtual bool IsSignatureRequired() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief 署名設定の Storage を生成します。
     */
    virtual nn::pia::session::SignatureSettingStorage* CreateSignatureSettingStorage();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief 複数候補ホストマイグレーションの使用を許すかどうか。
     */
    virtual bool IsMultiCandidatePermitted() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief リレー通信機能の使用を許すかどうか。
     */
    virtual bool IsRelayConnectionPermitted() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief ジョイントセッション機能の使用を許すかどうか。
     */
    virtual bool IsJointSessionPermitted() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief ネットワーク層におけるパケットのヘッダ部分の合計サイズを返します。単位はバイトです。
     */
    virtual uint32_t GetHeaderSize() const;
    /*! @endcond */

    /* 自動マッチメイク機能用 */
    /*!
    @cond PRIVATE
    @brief Session 用 CreateSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchCreateSessionJob();
    /*! @endcond */


    /*!
    @cond PRIVATE
    @brief Session 用 AutoMatchmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchAutoMatchmakeSessionJob();
    /*! @endcond */


    /*!
    @cond PRIVATE
    @brief Session 用 BrowseMatchmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchBrowseMatchmakeSessionJob();
    /*! @endcond */


    /*!
    @cond PRIVATE
    @brief Session 用 JoinSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchJoinSessionJob();
    /*! @endcond */


    /*!
    @cond PRIVATE
    @brief Session 用 LeaveSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchLeaveSessionJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief Session 用 DestroySessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchDestroySessionJob();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief Session 用 CreateGenerateMatchmakeSystemPasswordJob を生成します。
     */
    virtual common::StepSequenceJob* CreateGenerateMatchmakeSystemPasswordJob();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 CreateClearMatchmakeSystemPasswordJob を生成します。
     */
    virtual common::StepSequenceJob* CreateClearMatchmakeSystemPasswordJob();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 MatchmakeSessionSystemPassword の保持領域を生成します。
     */
    virtual common::String* CreateMatchmakeSessionSystemPassword();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ジョイントセッション機能用 JointSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateJointSessionJob();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 UpdateSessionSetting を生成します。
     */
    virtual common::StepSequenceJob* CreateUpdateSessionSettingJob();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 CommunityManagementJob を生成します。
     */
    virtual common::StepSequenceJob* CreateCommunityManagementJob();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 IMatchmakeSession を生成します。
     */
    virtual session::IMatchmakeSession* CreateMatchmakeSession();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief Session 用 ISessionInfoList を生成します。
     */
    virtual session::ISessionInfoList* CreateSessionInfoList(uint16_t browseSessionInfoListNum);
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief Session 用 ICommunityInfoList を生成します。
     */
    virtual session::ICommunityInfoList* CreateCommunityInfoList(uint8_t browsedCommunityInfoListNum);
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief Session 用 MeshLayerController を生成します。
     */
    virtual session::MeshLayerController* CreateMatchMeshLayerController();
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief Bind() が必要かどうか。
     */
    virtual bool IsBindRequired() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief サーバーが必要かどうか。
     */
    virtual bool IsServerRequired() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief いっしょに遊んだプレイヤー履歴を記録可能かどうか。
    */
    virtual bool IsRecentPlayRecordPermitted() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 同じ端末から複数プレイヤーが参加可能かどうか。
    */
    virtual bool IsMultiplayerPermitted() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief ホストマイグレーションモードを取得します。
     */
    virtual int32_t GetHostMigrationMode() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief セッションの検索結果の最大数を取得します。
     */
    virtual uint16_t GetBrowseSessionInfoListSizeMax() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief コミュニティの検索結果の最大数を取得します。
     */
    virtual uint8_t GetBrowseCommunityInfoListSizeMax() const;
    /*! @endcond */

    /*!
    @cond PRIVATE
    @brief MTU の最大値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMax() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief MTU の最小値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMin() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 初期段階での RTT 計測パルス送信間隔のデフォルト値です。単位はミリ秒です。
     */
    virtual int GetDefaultRttInitialPulseInterval() const;
    //! @endcond

    /* 自動マッチメイク機能用 */

protected:
private:
};
}
}
}  // end of namespace nn::pia::lan
