﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_LanSessionTypes.h>
#include <nn/pia/lan/lan_LanStationInfo.h>
#include <nn/pia/lan/lan_LanNetworkFactory.h>
#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/transport/transport_StationConnectionInfo.h>
#include <nn/pia/session/session_ISessionInfo.h>

namespace nn
{
namespace pia
{
namespace lan
{

/*!
    @brief  LAN マッチメイクのセッション情報クラスです。
    
    @see session::ISessionInfo
*/
class LanSessionInfo : public session::ISessionInfo
{
public:
    LanSessionInfo();
    virtual ~LanSessionInfo();

    /*!
    @brief      セッションのゲームモードを取得します。
    @return     セッションのゲームモードを返します。
    */
    virtual uint32_t GetGameMode() const;

    /*!
    @brief      セッションの ID を取得します。
    @return     セッションの ID を返します。
    */
    virtual uint32_t GetSessionId() const;

    /*!
    @brief      セッションに参加している人数を取得します。
    @return     セッションに参加している人数を返します。
    */
    virtual uint16_t GetCurrentParticipantNum() const;

    /*!
    @brief      セッションに参加できる最小人数を取得します。
    @return     セッションに参加できる最小人数を返します。
    */
    virtual uint16_t GetMinParticipantNum() const;

    /*!
    @brief      セッションに参加できる最大人数を取得します。
    @return     セッションに参加できる最大人数を返します。
    */
    virtual uint16_t GetMaxParticipantNum() const;

    /*!
    @brief      セッションが参加募集受付状態かどうかを取得します。
    @return     セッションが参加募集受付状態なら true, 参加募集締切状態なら false を返します。
    */
    virtual bool IsOpened() const;

    /*!
    @brief              セッションのアプリケーション定義データを取得します。
    @details            セッションに設定したアプリケーション定義データを取得します。
    @param[out] pBuffer セッションのアプリケーション定義データのコピー用バッファのポインタ。
    @param[in]  size    セッションのアプリケーション定義データのコピー用バッファのサイズ。アプリケーション定義データの最大長は LanApplicationDataBufferSizeMax です。
    @return             正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pBuffer, uint32_t size) const;

    /*!
    @brief      セッションのアプリケーション定義データの長さを取得します。
    @return     セッションのアプリケーション定義データの長さが返ります。
    */
    virtual uint32_t GetApplicationDataSize() const;

    /*!
    @brief                  セッションの属性を取得します。
    @param[in]  idx         取得する属性のインデックス。LanAttributeSizeMax 以下である必要があります。
    @param[out] pAttribute  指定されたインデックスのセッションの属性格納先のポインタ。
    @return                 正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    */
    virtual Result GetAttribute(uint8_t idx, uint32_t* pAttribute) const;

    /*!
    @brief      セッションタイプを取得します。
    @return     設定されているセッションタイプが返ります。
    */
    virtual LanSessionType GetSessionType() const;

    /*!
    @brief      セッションのホストのプリンシパル ID を取得します。
    @return     セッションのホストのプリンシパル ID が返ります。
    */
    virtual PrincipalId GetHostPrincipalId() const;

    /*!
    @brief                                  セッションに参加中のステーション情報を取得します。
    @param[out] pSessoinStationInfoList     ステーション情報リスト格納先のポインタ。
    @param[in]  sessionStationInfoListSize  ステーション情報リスト格納先のサイズ。参加中のステーション数以上である必要があります。
    @return                                 正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument           引数が不正です。[:progErr]
    @retval ResultInvalidState              データを取得できる状態ではありません。[:progErr]
    @retval ResultBufferShortage            ステーション情報のサイズがステーション情報格納先のサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetSessionStationInfo(LanStationInfo* pSessoinStationInfoList, uint32_t sessionStationInfoListSize);

    /*!
    @brief      セッションに参加中のステーション情報のサイズを取得します。
    @return     セッションに参加中のステーション情報のサイズを返します。
    */
    virtual uint32_t GetSessionStationInfoSize() const;

    /*!
    @brief      セッションの通信バージョンを取得します。
    @return     セッションの通信バージョンを返します。
    */
    virtual uint8_t GetLocalCommunicationVersion() const;

    /*!
    @brief      セッション情報をクリアします。
    */
    virtual void Clear();

    /*!
    @brief          セッション情報をコピーします。
    @param[in] rhs  コピーするセッション情報
    */
    virtual void Copy(const LanSessionInfo& rhs);

    /*!
    @brief      コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    LanSessionInfo(const LanSessionInfo& rhs);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    LanSessionInfo& operator=(const LanSessionInfo& rhs);


    /*!
    @brief デバッグに有用な情報をプリントします。
    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
    */
    virtual void Trace(uint64_t flag) const;

    /*!
    @cond PRIVATE
    */
    virtual void SetGameMode(uint32_t gameMode);
    virtual void SetSessionId(uint32_t sessionId);
    virtual void SetCurrentParticipantNum(uint16_t participants);
    virtual void SetMinParticipantNum(uint16_t participants);
    virtual void SetMaxParticipantNum(uint16_t participants);
    virtual void SetOpen(bool isOpened);
    virtual void SetSessionType(LanSessionType type);
    virtual void SetAttribute(uint8_t index, uint32_t attribute);
    virtual void SetApplicationData(const void* pData, uint32_t size);
    virtual transport::StationConnectionInfo& GetStationConnectionInfo();
    virtual void SetStationConnectionInfo(const transport::StationConnectionInfo& info);
    virtual void AddSessionStationInfo(const LanStationInfo& stationInfo);
    virtual void ClearSessionStationInfo();
    virtual void SetSystemVersion(uint8_t version);
    virtual uint8_t GetSystemVersion() const;
    virtual void SetLocalCommunicationVersion(uint8_t version);
    Result Serialize(uint8_t* pBuffer, uint32_t* pDataSize, uint32_t bufferSize) const;
    Result Deserialize(const uint8_t* pData, uint32_t dataSize);
    uint32_t GetSerializedSize() const;
    //! @endcond

protected:
    uint32_t m_GameMode;
    uint32_t m_SessionId;
    uint16_t m_CurrentParticipantNum;
    uint16_t m_MinParticipantNum;
    uint16_t m_MaxParticipantNum;
    bool m_bIsOpened;

    uint32_t m_Attribute[LanAttributeSizeMax];
    uint8_t m_SystemVersion;
    uint8_t m_LocalCommunicationVersion;
    uint16_t m_SessionType;
    uint8_t m_ApplicationData[LanApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;
    transport::StationConnectionInfo m_StationConnectionInfo;
    LanStationInfo m_StationInfoList[LanMaxStationNum];
    uint32_t m_StationInfoSize;
};
}
}
}  // end of namespace nn::pia::inet
