﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_LanSessionTypes.h>
#include <nn/pia/session/session_SessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace lan
{
class LanSessionInfo;
/*!
  @brief  LAN マッチメイクのセッションを検索する際に使用する検索条件のクラスです。
  @details 検索結果として得られるセッションの最大数は nn::pia::session::Session::CreateInstance 呼び出し時に
           指定する nn::pia::session::Session::Setting 構造体の browsedSessionInfoListNum の値になります。

  @see nn::pia::session::SessionSearchCriteria
 */
class LanSessionSearchCriteria : public nn::pia::session::SessionSearchCriteria
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    LanSessionSearchCriteria();

    /*!
    @brief      デストラクタ。
     */
    virtual ~LanSessionSearchCriteria();

    /*!
    @cond PRIVATE
    @brief 検索条件として指定した項目の示すビットマスクの各ビットに対応する項目
    */
    enum LanCondMask
    {
        LanCondMask_MinParticipation = 0,                                                                       //!< 最小参加人数
        LanCondMask_MaxParticipation,                                                                           //!< 最大参加人数
        LanCondMask_OpenedOnly,                                                                                 //!< 参加受付状態のセッションのみかどうか
        LanCondMask_VacantOnly,                                                                                 //!< 空きがあるセッションのみかどうか
        LanCondMask_GameMode,                                                                                   //!< ゲームモード
        LanCondMask_SessionType,                                                                                //!< セッションタイプ
        LanCondMask_Attribute                                                                                   //!< 属性
    };
    //! @endcond

    /*!
    @brief      検索条件に最小参加人数を指定します。
    @param[in]  min 検索条件に指定する最小参加人数。
    */
    void SetMinParticipantNum(uint16_t min);

    /*!
    @brief      検索条件に最小参加人数を範囲指定します。
    @param[in]  min 検索条件に指定する最小参加人数の範囲の下限。
    @param[in]  max 検索条件に指定する最小参加人数の範囲の上限。
    */
    void SetMinParticipantNumRange(uint16_t min, uint16_t max);

    /*!
    @brief      検索条件に最大参加人数を指定します。
    @param[in]  max 検索条件に指定する最大参加人数。
    */
    void SetMaxParticipantNum(uint16_t max);

    /*!
    @brief      検索条件に最大参加人数を範囲指定します。
    @param[in]  min 検索条件に指定する最大参加人数の範囲の下限。
    @param[in]  max 検索条件に指定する最大参加人数の範囲の上限。
    */
    void SetMaxParticipantNumRange(uint16_t min, uint16_t max);

    /*!
    @brief      検索条件に参加受付状態のセッションのみを検索するか指定します。
    @param[in]  bOpenedOnly 参加受付状態のセッションのみを検索する場合は true を指定します。
    */
    void SetOpenedOnly(bool bOpenedOnly);

    /*!
    @brief      検索条件に空きがあるセッションのみを検索するか指定します。
    @param[in]  bVacantOnly 空きがあるセッションのみを検索する場合は true を指定します。
    */
    void SetVacantOnly(bool bVacantOnly);

    /*!
    @brief      検索条件にゲームモードを指定します。
    @param[in]  gameMode    検索条件に指定するゲームモード。
    */
    void SetGameMode(uint32_t gameMode);

    /*!
    @brief      検索条件にセッションタイプを指定します。
    @param[in]  sessionType 検索条件に指定するセッションタイプ。
    */
    void SetSessionType(LanSessionType sessionType);

    /*!
    @brief      １つの属性に対して検索時に使用する値を指定します。
    @param[in]  index   指定する属性のインデックス。LanAttributeSizeMax 以下である必要があります。
    @param[in]  value   指定する属性の値。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外です。[:progErr]
    */
    Result SetAttribute(uint32_t index, uint32_t value);

    /*!
    @brief      １つの属性に対して検索時に使用する値を複数指定します。
    @param[in]  index           指定する属性のインデックス。LanAttributeSizeMax 以下である必要があります。
    @param[in]  value           指定する属性の値のリスト。
    @param[in]  attributeNum    指定する属性の値の個数。LanAttributeArraySizeMax 以下である必要があります。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外、値のリストが NULL、または値の個数が多すぎます。[:progErr]
    */
    Result SetAttribute(uint32_t index, const uint32_t* value, uint32_t attributeNum);

    /*!
    @brief      １つの属性に対して検索時に使用する値を範囲指定します。
    @param[in]  index           指定する属性のインデックス。LanAttributeSizeMax 以下である必要があります。
    @param[in]  min             指定する属性の値の下限。
    @param[in]  max             指定する属性の値の上限。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外、または下限値が上限値よりも大きい値になっています。[:progErr]
    */
    Result SetAttributeRange(uint32_t index, uint32_t min, uint32_t max);

    /*!
    @cond PRIVATE
    @brief      指定したマッチメイクセッションが検索条件に合致するか判定します。
    @param[in]  pInfo   検索条件に合致するか判定するマッチメイクセッション
    */
    bool IsHit(const LanSessionInfo* pInfo);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      LanSessionSearchCriteria をシリアライズします。
    */
    Result Serialize(uint8_t* pBuffer, uint32_t* pDataSize, uint32_t bufferSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      LanSessionSearchCriteria をデシリアライズします。
    */
    Result Deserialize(const uint8_t* pData, uint32_t dataSize);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      LanSessionSearchCriteria のシリアライズ後のサイズを取得します。
    */
    uint32_t GetSerializedSize() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリントします。
    @param[in] flag  トレースフラグです。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const LanSessionSearchCriteria& rhs);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    LanSessionSearchCriteria& operator=(const LanSessionSearchCriteria& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    LanSessionSearchCriteria(const LanSessionSearchCriteria& rhs);

    /*!
    @cond PRIVATE
    */
    uint32_t GetGameMode() const;
    bool IsSetGameMode() const;
    LanSessionType GetSessionType() const;
    bool IsSetSessionType() const;
    const uint32_t* GetAttributeList(uint32_t index) const;
    uint32_t GetAttributeListSize(uint32_t index) const;
    uint32_t GetAttributeRangeMin(uint32_t index) const;
    uint32_t GetAttributeRangeMax(uint32_t index) const;
    bool IsAttributeRangeUsed(uint32_t index) const;
    bool IsSetAttribute(uint32_t index) const;

    bool IsOpenedOnly() const;
    bool IsSetOpenedOnly() const;
    bool IsVacantOnly() const;
    bool IsSetVacantOnly() const;

    uint16_t GetMinParticipantRangeMin() const;
    uint16_t GetMinParticipantRangeMax() const;
    bool IsSetMinParticipantNum() const;
    uint16_t GetMaxParticipantRangeMin() const;
    uint16_t GetMaxParticipantRangeMax() const;
    bool IsSetMaxParticipantNum() const;
    //! @endcond

private:
    uint32_t m_GameMode;
    LanSessionType m_SessionType;
    uint32_t m_AttributeList[LanAttributeSizeMax][LanAttributeArraySizeMax];
    uint32_t m_AttributeSizeList[LanAttributeSizeMax];
    uint32_t m_AttributeRangeMin[LanAttributeSizeMax];
    uint32_t m_AttributeRangeMax[LanAttributeSizeMax];
    bool m_bAttributeRangeUsed[LanAttributeSizeMax];

    uint16_t m_MinParticipantsRangeMax;
    uint16_t m_MinParticipantsRangeMin;
    uint16_t m_MaxParticipantsRangeMax;
    uint16_t m_MaxParticipantsRangeMin;
    bool m_bOpenedOnly;
    bool m_bVacantOnly;
    uint32_t m_SetCondMask;
};
}
}
}  // end of namespace nn::pia::lan
