﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_Definitions.h>

#include <nn/pia/lan/lan_LanSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/session/session_UpdateSessionSetting.h>

namespace nn
{
namespace pia
{
namespace lan
{
/*!
  @brief  セッションの条件更新処理に関する情報を管理するクラスです。
  @details 更新する値を指定した設定のみ更新されます。更新しない設定について、指定は不要です。

  @see session::UpdateSessionSetting
 */
class LanUpdateSessionSetting : public nn::pia::session::UpdateSessionSetting
{
public:
    enum ModificationFlag
    {
        ModificationFlag_MinParticipant = ModificationFlag_ApplicationData + 1,
        ModificationFlag_MaxParticipant,
        ModificationFlag_SessionType,
        ModificationFlag_Attribute0,
        ModificationFlag_Attribute1,
        ModificationFlag_Attribute2,
        ModificationFlag_Attribute3,
        ModificationFlag_Attribute4,
        ModificationFlag_Attribute5
    };

    /*!
    @brief      デフォルトコンストラクタ。
     */
    LanUpdateSessionSetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~LanUpdateSessionSetting();

    /*!
    @brief      セッションの最小参加人数を指定します。
    @param[in]  min 作成するセッションに指定する最小参加人数。
    */
    void SetMinParticipantNum(uint16_t min);

    /*!
    @cond PRIVATE
    */
    uint16_t GetMinParticipantNum() const;
    bool IsSetMinParticipantNum() const;
    //! @endcond

    /*!
    @brief      セッションの最大参加人数を指定します。
    @param[in]  max 作成するセッションに指定する最大参加人数。
    */
    void SetMaxParticipantNum(uint16_t max);

    /*!
    @cond PRIVATE
    */
    uint16_t GetMaxParticipantNum() const;
    bool IsSetMaxParticipantNum() const;
    //! @endcond

    /*!
    @brief      セッションのセッションタイプを指定します。
    @param[in]  sessionType セッションタイプ。
    */
    void SetSessionType(LanSessionType sessionType);

    /*!
    @cond PRIVATE
    */
    LanSessionType GetSessionType() const;
    bool IsSetSessionType() const;
    //! @endcond

    /*!
    @brief      セッションの属性を指定します。
    @param[in]  index   セッションに指定する属性のインデックス。LanAttributeSizeMax 以下である必要があります。
    @param[in]  value   セッションに指定する属性の値。
    @return     セッションの属性の指定に成功すれば、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidArgument  属性のインデックスが範囲外です。[:progErr]
    */
    Result SetAttribute(uint32_t index, uint32_t value);

    /*!
    @cond PRIVATE
    */
    uint32_t GetAttribute(uint32_t index) const;
    bool IsSetAttribute(uint32_t index) const;
    //! @endcond

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @details    inet::ApplicationDataBufferSizeMax を超えないサイズを指定する必要があります。
    @param[in]  pData   セッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    セッションに指定するアプリケーション定義データのサイズ。LanApplicationDataBufferSizeMax 以下である必要があります。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータサイズを取得します。
    @return     アプリケーション定義データサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSize() const;
    //! @endcond

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const LanUpdateSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    LanUpdateSessionSetting(const LanUpdateSessionSetting& rhs);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    LanUpdateSessionSetting& operator=(const LanUpdateSessionSetting& rhs);

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond


protected:
    uint16_t m_MinParticipantNum;
    uint16_t m_MaxParticipantNum;
    LanSessionType m_SessionType;
    uint32_t m_Attribute[LanAttributeSizeMax];
    uint8_t m_ApplicationDataBuffer[LanApplicationDataBufferSizeMax];
    uint32_t m_ApplicationDataSize;

private:
};
}
}
}  // end of namespace nn::pia::lan
