﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>


// local モジュールの非メンバ関数群です。

namespace nn
{
namespace pia
{
namespace local
{
/*! @name 初期化終了処理
    @{
 */

/*!
              @brief  local モジュールを初期化します。
              
              @details 事前に nn::pia::common::Initialize を実行する必要があります。

              @return 無事に初期化が完了すれば、成功のResultが返されます。

              @retval ResultInvalidState        nn::pia::commonが初期化されていません。
              @retval ResultAlreadyInitialized  既に初期化は終了しています。
            */
Result Initialize();

/*!
              @brief  local モジュールが初期化された状態であるかどうかを返します。

              @return 初期化されていれば true 、されていなければ false が返されます。
             */
bool IsInitialized(void);

/*!
              @brief  セットアップの開始を宣言します。

              @details local モジュール以下の初期化処理の前、すなわち @ref LocalFacade::CreateInstance および @ref LocalFacade::Initialize を実行する前に呼ぶ必要があります。

              @attention   別のモジュールのセットアップ処理とバッティングすると正しく動作しません。<br>
                      @ref BeginSetup 呼出後、local モジュール以下の初期化処理を行い、速やかに @ref EndSetup を呼出してセットアップを完了させる必要があります。

              @return このAPIを適切なタイミングで呼び出していれば、成功のResultが返されます。

              @retval ResultNotInitialized  local モジュールが初期化されていません。
              @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。
             */
Result BeginSetup(void);

/*!
              @brief  セットアップの終了を宣言します。

              @details local モジュール以下の初期化処理の後、すなわち @ref LocalFacade::CreateInstance および @ref LocalFacade::Initialize を実行した後に呼ぶ必要があります。 <br>
                       このAPI呼び出しにより、local モジュールのシングルトンなどが消費するメモリを確定し、ライブラリ内部でのメモリ管理を最適化します。

              @attention   別のモジュールのセットアップ処理とバッティングすると正しく動作しません。<br>
                      @ref BeginSetup 呼出後、local モジュール以下の初期化処理を行い、速やかに @ref EndSetup を呼出してセットアップを完了させる必要があります。

              @return このAPIを適切なタイミングで呼び出していれば、成功のResultが返されます。

              @retval ResultNotInitialized  local モジュールが初期化されていません。
              @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。
             */
Result EndSetup(void);

/// @cond PRIVATE
bool IsDuringSetup(void);
/// @endcond

/*!
              @brief  local モジュールを終了します。

              @details 本関数を実行する前に PiaLocal モジュールのすべての処理を終了しておく必要があります。
                      なお、@ref nn::pia::local::Initialize を実行していない状態では何もしません。
             */
void Finalize(void);
/*! 
    end of name 初期化終了処理
    @}
 */
}
}
}  // end of namespace nn::pia::local
