﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#include <nn/pia/local/local_LocalNetworkSetting.h>

#include <nn/pia/common/common_CallContext.h>
#include <nn/pia/common/common_CriticalSection.h>

namespace nn
{
namespace pia
{
namespace common
{
class InetAddress;
class StationAddress;
}
}
}  // end of namespace nn::pia::local

namespace nn
{
namespace pia
{
namespace transport
{

class StationConnectionInfo;
}
}
}  // end of namespace nn::pia::transport

namespace nn
{
namespace pia
{
namespace local
{

// 先行宣言
class LocalProtocol;
class LocalNetworkSetting;
class LocalNetworkDescription;
class LocalCreateNetworkSetting;
class LocalConnectNetworkSetting;
class LocalScanNetworkSetting;

#if NN_PIA_CTR
class UdsNetworkSetting;
#endif

#if NN_PIA_NINTENDOSDK
class LdnNetworkSetting;
#endif

/*!
    @brief ローカルネットワークを扱うためのクラスです。
*/
class LocalFacade : public nn::pia::common::RootObject
{
public:
    /*!
    @brief    LocalFacade クラスのインスタンスを作成します（シングルトンパターン）。

    @return   インスタンスの作成に成功すれば、成功の Result が返されます。

    @retval   ResultNotInitialized     local モジュールが未初期化です。
    @retval   ResultInvalidState       インスタンスを作成するタイミングが誤っています。
    @retval   ResultAlreadyExists      既にインスタンスは作成されています。
     */
    static Result CreateInstance();


    /*!
    @brief LocalFacade クラスのインスタンスを破棄します（シングルトンパターン）。
     */
    static void DestroyInstance();

    /*!
    @brief    LocalFacade クラスのインスタンスを得ます（シングルトンパターン）。
              事前に @ref Initialize による初期化を済ませていない場合は、
              NULLポインタが返されます。
    @return   LocalFacade インスタンスへのポインタ。
     */
    static LocalFacade* GetInstance()
    {
        return s_pInstance;
    }

    /*!
    @cond PRIVATE
    @brief LocalFacade クラスのインスタンスが初期化されたかどうかを取得します。
    */
    static bool IsInitialized()
    {
        return s_IsInitialized;
    }
    //! @endcond

private:
    LocalFacade();
    virtual ~LocalFacade();
    /*!
    @brief コピーコンストラクタは封印します。
     */
    LocalFacade(const LocalFacade&);

    /*!
    @brief 代入演算子は封印します。
     */
    LocalFacade& operator=(const LocalFacade&);

public:
    /*!
    @brief      インスタンスを初期化します。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。

    @attention  LocalFacade の受信バッファとしてデバイスメモリを使用することはできないため、
                common::Initialize() にデバイスメモリが渡されていた場合、アサート停止します。

    @param[in]  setting     各種設定情報です。
    
    @return     成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultAlreadyInitialized    既に初期化されています。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]

    @see        Finalize
     */
    Result Initialize(const LocalNetworkSetting& setting);


    /*!
    @brief      終了処理です。

    @see        Initialize
     */
    void Finalize();


    /*!
    @cond PRIVATE
    @brief      スタートアップ処理です。

    @attention  本関数呼び出し中は、スリープモードに遷移してはいけません。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidConnection    無線オフモードに遷移しました。[:handling-cleanup]
    @retval ResultInvalidState         transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。もしくは既に Starup() が呼び出されています。[:progErr]

    @see        Cleanup
     */
    Result Startup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      クリーンアップ処理です。

    @see        Startup
     */
    void Cleanup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ローカルネットワークを作成します。

    @details    ネットワークの設定を受け取って、ローカルネットワーク作成の非同期処理を開始します。
                ネットワークを作成した端末はローカルネットワークホストになります。
                開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                処理の完了は @ref IsCreateNetworkCompleted でチェックし、結果は @ref GetCreateNetworkResult で確認できます。
             
    @param[in] pCreateNetworkSetting    作成するネットワークの設定です。

    @return 正常に処理を開始できた場合、IsSuccess() が真となる Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、既に処理を実行中です。または、他の非同期処理を実行中です。または、既に接続中です。[:progErr]
    @retval ResultErrorOccurred         内部でエラーが発生しました。[:handling-cleanup]

    @see    IsCreateNetworkCompleted, GetCreateNetworkResult
    */
    Result CreateNetworkAsync(const LocalCreateNetworkSetting* pCreateNetworkSetting);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief CreateNetworkAsync の非同期処理の完了をチェックします。

    @details @ref CreateNetworkAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true、処理中の場合は false が返ります。非同期処理が開始されていない場合や他の非同期処理が開始されている場合も false が返ります。

    @see    CreateNetworkAsync, GetCreateNetworkResult
     */
    bool IsCreateNetworkCompleted();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief CreateNetworkAsync の非同期処理の結果を取得します。

    @details    @ref CreateNetworkAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>

    @return CreateNetworkAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultNotInitialized        通信ライブラリが初期化されていません。[:progErr]
    @retval ResultInvalidState          実行可能な状態ではありません。また、結果を取得できる状態ではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     通信できません。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval ResultTimeOut               タイムアウトで終了しました。[:handling-cleanup]
    @retval ResultCancelled             キャンセルされました。[:handling-cleanup]
    @retval ResultErrorOccurred         内部でエラーが発生しました。[:handling-cleanup]
    @if NIN_DOC
    @retval ResultWirelessControllerCountLimitation 無線コントローラの接続台数が制限を超えています。[:viewer][:handling-cleanup]
    @endif

    @see    CreateNetworkAsync, IsCreateNetworkCompleted
     */
    Result GetCreateNetworkResult();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ローカルネットワークを破棄します。

    @details ローカルネットワークを破棄する非同期処理を開始します。
             開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
             処理の完了は @ref IsDestroyNetworkCompleted でチェックし、結果は @ref GetDestroyNetworkResult で確認できます。
             処理完了後は、LocalFacade のクリーンアップを行う必要があります。

    @attention    本関数はローカルネットワークホストでのみ実行可能です。

    @return 正常に処理を開始できた場合、IsSuccess() が真となる Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、既に処理を実行中です。または、他の非同期処理を実行中です。または、自分はローカルネットワークホストではありません。[:progErr]

    @see    IsDestroyNetworkCompleted, GetDestroyNetworkResult
    */
    Result DestroyNetworkAsync();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief DestroyNetworkAsync の非同期処理の完了をチェックします。

    @details @ref DestroyNetworkAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true、処理中の場合は false が返ります。非同期処理が開始されていない場合や他の非同期処理が開始されている場合も false が返ります。

    @see    DestroyNetworkAsync, GetDestroyNetworkResult
     */
    bool IsDestroyNetworkCompleted();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief DestroyNetworkAsync の非同期処理の結果を取得します。

    @details    @ref DestroyNetworkAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>

    @return DestroyNetworkAsync の非同期処理の結果を返します。処理が成功していた場合、IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultNotInitialized        通信ライブラリが初期化されていません。[:progErr]
    @retval ResultInvalidState          実行可能な状態ではありません。また、結果を取得できる状態ではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     通信できません。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval ResultCancelled             キャンセルされました。[:handling-cleanup]
    @retval ResultErrorOccurred         内部でエラーが発生しました。[:handling-cleanup]

    @see    DestroyNetworkAsync, IsDestroyNetworkCompleted
     */
    Result GetDestroyNetworkResult();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ローカルネットワークに接続します。

    @details ローカルネットワークに接続する非同期処理を開始します。接続した端末はローカルネットワーククライアントになります。
             開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
             処理の完了は @ref IsConnectNetworkCompleted でチェックし、結果は @ref GetConnectNetworkResult で確認できます。<br>
             また、処理中は @ref CancelConnectNetworkAsync を呼び出すことで、処理を中止することができます。
             中止した場合も、@ref IsConnectNetworkCompleted で処理の完了が確認できるまで待つ必要があります。<br>
             取得した処理結果が失敗していた場合（中止した場合も含む）は、LocalFacade をクリーンアップし、再度スタートアップを行う必要があります。
             
    @param[in] pConnectNetworkSetting   接続するネットワークの設定です。

    @return 正常に処理を開始できた場合、IsSuccess() が真となる Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、既に処理を実行中です。または、他の非同期処理を実行中です。または、既に接続中です。[:progErr]
    @if CTR_DOC
    @retval ResultLocalLowerVersion          自分の @ref local::LocalNetworkVersion 、または、アプリケーションバージョンが接続先ネットワークのものよりも低いです。[:handling-cleanup]
    @retval ResultLocalHigherVersion         自分の @ref local::LocalNetworkVersion 、または、アプリケーションバージョンが接続先ネットワークのものよりも高いです。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultLocalLowerVersion          自分の @ref local::LocalNetworkVersion が接続先ネットワークのものよりも低いです。[:viewer][:handling-cleanup]
    @retval ResultLocalHigherVersion         自分の @ref local::LocalNetworkVersion が接続先ネットワークのものよりも高いです。[:viewer][:handling-cleanup]
    @endif

    @see    IsConnectNetworkCompleted, GetConnectNetworkResult, CancelConnectNetworkAsync
    */
    Result ConnectNetworkAsync(const LocalConnectNetworkSetting* pConnectNetworkSetting);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ConnectNetworkAsync の非同期処理の完了をチェックします。

    @details @ref ConnectNetworkAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true、処理中の場合は false が返ります。非同期処理が開始されていない場合や他の非同期処理が開始されている場合も false が返ります。

    @see    ConnectNetworkAsync, GetConnectNetworkResult, CancelConnectNetworkAsync
     */
    bool IsConnectNetworkCompleted();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ConnectNetworkAsync の非同期処理の結果を取得します。

    @details    @ref ConnectNetworkAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>

    @return ConnectNetworkAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultNotInitialized        通信ライブラリが初期化されていません。[:progErr]
    @retval ResultInvalidState          実行可能な状態ではありません。また、結果を取得できる状態ではありません。[:progErr]
    @retval ResultNetworkIsNotFound     接続対象のネットワークが見つかりませんでした。[:handling-cleanup]
    @if CTR_DOC
    @retval ResultNetworkIsFull         既に最大接続数に達しています。[:handling-cleanup]
    @retval ResultJoinRequestDenied     ホストに参加を拒否されました。[:handling-cleanup][:viewer-handling]
    @elseif NIN_DOC
    @retval ResultJoinRequestDenied     ホストに参加を拒否されました。既に最大接続数に達している場合にも返ります。[:handling-cleanup][:viewer-handling]
    @endif
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     通信できません。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    @if CTR_DOC
    @retval ResultTimeOut               タイムアウトで終了しました。
                                        @ref nn::pia::local::UdsConnectNetworkSetting::wirelessCryptoKey が誤っている場合にも返ります。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultTimeOut               タイムアウトで終了しました。
                                        @ref nn::pia::local::LdnConnectNetworkSetting::wirelessCryptoKey が誤っている場合にも返ります。[:handling-cleanup]
    @endif
    @retval ResultCancelled             キャンセルされました。[:handling-cleanup]
    @retval ResultErrorOccurred         内部でエラーが発生しました。[:handling-cleanup]
    @if NIN_DOC
    @retval ResultLocalLowerVersion          自分のローカル通信バージョンが接続先ネットワークのものよりも低いです。[:viewer][:handling-cleanup]
    @retval ResultLocalHigherVersion         自分のローカル通信バージョンが接続先ネットワークのものよりも高いです。[:viewer][:handling-cleanup]
    @endif
    @if NIN_DOC
    @retval ResultWirelessControllerCountLimitation 無線コントローラの接続台数が制限を超えています。[:viewer][:handling-cleanup]
    @endif

    @see    ConnectNetworkAsync, IsConnectNetworkCompleted, CancelConnectNetworkAsync
     */
    Result GetConnectNetworkResult();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief    ConnectNetworkAsync の非同期処理をキャンセルします。

    @details    ConnectNetworkAsync 開始された非同期処理は、本関数でキャンセルできます。
                キャンセル処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                本関数を呼び出した後、ライブラリは速やかに非同期処理のキャンセルを行いますが、
                キャンセルの完了は @ref IsConnectNetworkCompleted() でチェックする必要があります。

    @return   ConnectNetworkAsync の非同期処理のキャンセル結果を返します。処理が成功していた場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   ResultInvalidState    ConnectNetworkAsync の非同期処理が行われていないタイミングで呼び出しています。[:progErr]

    @see    ConnectNetworkAsync, IsConnectNetworkCompleted, GetConnectNetworkResult
    */
    Result CancelConnectNetworkAsync();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ローカルネットワークから切断します。

    @details    ローカルネットワークから切断する非同期処理を開始します。
                自分がローカルネットワークホストのときは、ホストマイグレーションが有効のときのみ成功し、手動ホストマイグレーション処理を開始します。
                開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                処理の完了は @ref IsDisconnectNetworkCompleted でチェックし、結果は @ref GetDisconnectNetworkResult で確認できます。
                処理完了後は、LocalFacade のクリーンアップを行う必要があります。
             
    @return 正常に処理を開始できた場合、IsSuccess() が真となる Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、既に処理を実行中です。または、他の非同期処理を実行中です。
                                        または、自分はローカルネットワークホストですがホストマイグレーションが無効です。[:progErr]

    @see    IsDisconnectNetworkCompleted, GetDisconnectNetworkResult
    */
    Result DisconnectNetworkAsync();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief DisconnectNetworkAsync の非同期処理の完了をチェックします。

    @details @ref DisconnectNetworkAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true、処理中の場合は false が返ります。非同期処理が開始されていない場合や他の非同期処理が開始されている場合も false が返ります。

    @see    DisconnectNetworkAsync, GetDisconnectNetworkResult
     */
    bool IsDisconnectNetworkCompleted();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief DisconnectNetworkAsync の非同期処理の結果を取得します。

    @details    @ref DisconnectNetworkAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>

    @return DisconnectNetworkAsync の非同期処理の結果を返します。 処理が成功していた場合、IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultNotInitialized        通信ライブラリが初期化されていません。[:progErr]
    @retval ResultInvalidState          実行可能な状態ではありません。また、結果を取得できる状態ではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     通信できません。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval ResultCancelled             キャンセルされました。[:handling-cleanup]
    @retval ResultErrorOccurred         内部でエラーが発生しました。[:handling-cleanup]

    @see    DisconnectNetworkAsync, IsDisconnectNetworkCompleted
     */
    Result GetDisconnectNetworkResult();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 周囲のローカルネットワークを探索します。

    @details    周囲のローカルネットワークを探索する非同期処理を開始します。ネットワークに接続中は失敗します。
                開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                処理の完了は @ref IsScanNetworkCompleted でチェックし、結果は @ref GetScanNetworkResult で確認できます。<br>
                また、処理中は @ref CancelScanNetworkAsync を呼び出すことで、処理を中止することができます。
                中止した場合も、@ref IsScanNetworkCompleted で処理の完了が確認できるまで待つ必要があります。<br>
                探索して見つけたローカルネットワーク情報は、 @ref GetNetworkDescriptionList 、
                @ref GetNetworkDescription で取得できます。
                探索して見つけたネットワークに接続中のステーション情報は、 @ref GetStationInfoList で取得できます。
                ネットワークから送信された電波の強度は、@ref GetRadioStrength で取得できます。
                これらの情報は、ネットワーク探索処理が完了し、成功してから取得可能となり、
                再び探索を開始すると取得できなくなります。
                また、ネットワークの構築/接続処理を開始すると取得できなくなります。
             
    @param[in] pScanNetworkSetting      探索するネットワークの設定です。
             
    @return 正常に処理を開始できた場合、IsSuccess() が真となる Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、既に処理を実行中です。または、他の非同期処理を実行中です。または、ネットワークに接続中です。[:progErr]

    @see    IsScanNetworkCompleted, GetScanNetworkResult, CancelScanNetworkAsync
    */
    Result ScanNetworkAsync(const LocalScanNetworkSetting* pScanNetworkSetting);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ScanNetworkAsync の非同期処理の完了をチェックします。

    @details @ref ScanNetworkAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true、処理中の場合は false が返ります。非同期処理が開始されていない場合や他の非同期処理が開始されている場合も false が返ります。

    @see    ScanNetworkAsync, GetScanNetworkResult, CancelScanNetworkAsync
     */
    bool IsScanNetworkCompleted();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ScanNetworkAsync の非同期処理の結果を取得します。

    @details    @ref ScanNetworkAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>

    @return ScanNetworkAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultNotInitialized        通信ライブラリが初期化されていません。[:progErr]
    @retval ResultInvalidState          実行可能な状態ではありません。また、結果を取得できる状態ではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     通信できません。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval ResultNegligibleFault       キャンセルされました。
    @retval ResultErrorOccurred         内部でエラーが発生しました。[:handling-cleanup]

    @see    ScanNetworkAsync, IsScanNetworkCompleted, CancelScanNetworkAsync
     */
    Result GetScanNetworkResult();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief    ScanNetworkAsync の非同期処理をキャンセルします。

    @details    ScanNetworkAsync 開始された非同期処理は、本関数でキャンセルできます。
                キャンセル処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                本関数を呼び出した後、ライブラリは速やかに非同期処理のキャンセルを行いますが、
                キャンセルの完了は @ref IsScanNetworkCompleted() でチェックする必要があります。

    @return   ScanNetworkAsync の非同期処理のキャンセル結果を返します。処理が成功していた場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   ResultInvalidState    ConnectNetworkAsync の非同期処理が行われていないタイミングで呼び出しています。[:progErr]

    @see    ScanNetworkAsync, IsScanNetworkCompleted, GetScanNetworkResult
    */
    Result CancelScanNetworkAsync();
//! @endcond


#if NN_PIA_CTR
    /*!
    @brief ユニーク ID からローカル通信 ID を作成します。

    @details 業務部から割り当てられた 20bit のユニーク ID （タイトル固有の ID）から ローカル通信で使用する 32bit 値 (ローカル通信 ID) を生成します。<br>
             業務部からユニーク ID が割り当てられていない場合は、ゲームソフト試作用のユニーク ID ( 0xFF000 ～ 0xFF3FF ) を使用する必要があります。
             transport::Transport::Initialize() を呼び出してから transport::Transport::Finalize() を呼び出すまでの間、有効な値が返ります。
             
    @attention    製品版では必ず業務部から割り当てられた ユニーク ID を使用する必要があります。

    @param[in] uniqueId ユニーク IDです。複数タイトル間で通信したい場合はどちらか片方のユニーク ID を指定する必要があります。
    @param[in] isTrial  ユニーク ID が製品版と体験版で共通だった場合の対処用フラグです。製品版-体験版間での通信を行いたくない場合、 体験版側で true を指定する必要があります。製品版は常に false が指定されるよう注意してください。

    @return 作成されたローカル通信 ID が返ります。
    */
    uint32_t CreateLocalCommunicationId(uint32_t uniqueId, bool isTrial = false) const;
#elif NN_PIA_NINTENDOSDK
    /*!
    @cond PRIVATE
    */
    uint32_t CreateLocalCommunicationId(uint32_t uniqueId, bool isTrial = false) const;
//! @endcond
#endif


    /*!
    @cond PRIVATE
    @brief アプリケーションが設定したビーコンデータを取得します。

    @param[out] pBuffer             取得したデータを格納するバッファです。
    @param[out] pDataSize           取得したデータのサイズです。
    @param[in]  bufferSize          取得したデータを格納するバッファのサイズです。
    @param[in]  pNetworkDescription データを取得したいネットワークの情報です。
             
    @return 正常にデータを取得できた場合、成功の Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidStatet         transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。[:progErr]
    @retval ResultNoData                取得できるデータがありませんでした。
    */
    Result GetApplicationData(void* pBuffer, uint32_t* pDataSize, uint32_t bufferSize, const LocalNetworkDescription* pNetworkDescription) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief アプリケーションが設定したビーコンデータのサイズを取得します。

    @param[out] pDataSize           アプリケーションが設定したビーコンデータのサイズです。
    @param[in]  pNetworkDescription ビーコンデータのサイズを取得したいネットワークの情報です。
             
    @return 正常にデータを取得できた場合、成功の Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidStatet         transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。[:progErr]
    @retval ResultNoData                取得できるデータがありませんでした。
    */
    Result GetApplicationDataSize(uint32_t* pDataSize, const LocalNetworkDescription* pNetworkDescription) const;
    //! @endcond

    /*!
    @brief ビーコンに設定されているデータを取得します。

    @attention この API は、ローカルネットワークに接続中の端末が呼び出すことを想定しています。

    @attention  この API はローカルホストマイグレーション処理が行われている最中には正しい結果が得られない場合があります。
                ホストマイグレーション処理中（@ref LocalFacade::IsDuringHostMigration が true を返す状況）は実行しないよう注意してください。

    @param[out]  pBuffer    データを書き込むためのバッファ。
    @param[out]  pDataSize  取得したデータの長さ。
    @param[in]   bufferSize データを書き込むためのバッファのサイズ。
      
    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultNotInitialized        初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、接続していないか、ホストマイグレーション処理中です[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    @retval ResultNotInCommunication    通信できない状態です。その内、切断検知されるか、またはホストマイグレーションが開始されます。ホストマイグレーション完了後に呼び出すことで成功する可能性があります。
    */
    Result GetApplicationDataFromBeacon(void* pBuffer, uint32_t* pDataSize, uint32_t bufferSize) const;

    /*!
    @brief ビーコンに設定されているデータのサイズを取得します。

    @attention この API は、ローカルネットワークに接続中の端末が呼び出すことを想定しています。

    @attention  この API はローカルホストマイグレーション処理が行われている最中には正しい結果が得られない場合があります。
                ホストマイグレーション処理中（@ref LocalFacade::IsDuringHostMigration が true を返す状況）は実行しないよう注意してください。

    @param[out]  pDataSize  ビーコンに設定されているデータのサイズ。
      
    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultNotInitialized        初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、接続していないか、ホストマイグレーション処理中です[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    @retval ResultNotInCommunication    通信できない状態です。その内、切断検知されるか、またはホストマイグレーションが開始されます。ホストマイグレーション完了後に呼び出すことで成功する可能性があります。
    */
    Result GetApplicationDataSizeFromBeacon(uint32_t* pDataSize) const;

    /*!
    @cond PRIVATE
    @brief ビーコンにデータを設定します。

    @details ローカルネットワークホストのみ実行可能です。

    @attention  この API はローカルホストマイグレーション処理が行われている最中には正しい結果が得られない場合があります。
                ホストマイグレーション処理中（@ref LocalFacade::IsDuringHostMigration が true を返す状況）は実行しないよう注意してください。

    @param[in] pData    設定するデータへのポインタ。
    @param[in] dataSize 設定するデータのサイズ。
    @if CTR_DOC
               @ref UdsBeaconApplicationDataSizeMax 以下である必要があります。
    @elseif NIN_DOC
               @ref LdnBeaconApplicationDataSizeMax 以下である必要があります。
    @endif

    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultNotInitialized        初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、ホストではありません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。または、ローカルネットワークから切断されました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     ローカルネットワークから切断されました。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    */
    Result SetApplicationDataToBeacon(const void* pData, uint32_t dataSize);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ローカルネットワーク情報を取得します。

    @details    @ref ScanNetworkAsync で発見したローカルネットワーク情報のリストを取得します。
             
    @param[out] paLocalNetworkDescriptionList   取得したネットワーク情報を格納するバッファです。
    @param[out] pValidNetworkNum                ネットワーク情報が取得できたネットワーク数です。
    @param[in]  networkDescriptionListSize      取得したネットワーク情報を格納するバッファのサイズです。
             
    @return 正常に情報を取得できた場合、成功の Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、データを取得できる状態ではありません。[:progErr]
    */
    Result GetNetworkDescriptionList(LocalNetworkDescription* paLocalNetworkDescriptionList, uint32_t* pValidNetworkNum, uint32_t networkDescriptionListSize);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ローカルネットワーク情報を取得します。

    @details    @ref ScanNetworkAsync で発見したローカルネットワーク情報を取得します。
             
    @param[out] pLocalNetworkDescription         取得したネットワーク情報を格納するバッファです。
    @param[in]  networkDescriptionIndex         取得したいネットワーク情報のインデックスです。
             
    @return 正常に情報を取得できた場合、成功の Result が返ります。

    @retval ResultNotInitialized        LocalFacade が初期化されていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、データを取得できる状態ではありません。[:progErr]
    */
    Result GetNetworkDescription(LocalNetworkDescription* pLocalNetworkDescription, uint32_t networkDescriptionIndex);
    //! @endcond


    /*!
    @brief 指定したステーションの情報を取得します。

    @param[out] pLocalStationInfo           取得したステーション情報を格納するバッファです。
    @param[in]  address                     情報を取得したいステーションのアドレスです。
             
    @return 正常に情報を取得できた場合、成功の Result が返ります。

    @retval ResultNotInitialized        LocalFacade がスタートアップされていません。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultNotFound              対象のステーションが見つかりませんでした。すでに切断されている可能性があります。
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    */
    Result GetStationInfo(LocalStationInfo* pLocalStationInfo, const common::StationAddress& address) const;

    /*!
    @cond PRIVATE
    @brief 指定したネットワークに参加しているステーションの情報のリストを取得します。

    @param[out] paLocalStationInfoList      取得したステーション情報のリストを格納するバッファです。配列長が、指定したネットワークに接続できるステーションの最大数以上である必要があります。
    @param[in]  listLength                  paLocalStationInfoList の配列長です。指定したネットワークに接続できるステーションの最大数以上である必要があります。
    @param[in]  networkListIndex            情報を取得したいネットワークのインデックスです。
             
    @return 正常に情報を取得できた場合、成功の Result が返ります。

    @retval ResultInvalidArgument       引数が不正です。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、ネットワークが発見されていません。または、データを取得できる状態ではありません。[:progErr]
    @retval ResultNoData                情報が取得できませんでした。
    @retval ResultInvalidConnection     無線オフモードに遷移しました。[:handling-cleanup]
    */
    Result GetStationInfoList(LocalStationInfo* paLocalStationInfoList, uint8_t listLength, uint32_t networkListIndex) const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      発見したネットワーク数を取得します。

    @return     発見したネットワーク数が返ります。
    */
    uint32_t GetFoundNetworkNum() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      自分がローカルネットワークホストであるかを取得します。

    @return     自分がローカルネットワークホストであるかが返ります。
    */
    bool IsHost() const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      自分がローカルネットワーククライアントであるかを取得します。

    @return     自分がローカルネットワーククライアントであるかが返ります。
    */
    bool IsClient() const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      ネットワークに接続されているかを取得します。

    @attention  ホストマイグレーション処理中（@ref LocalFacade::IsDuringHostMigration が true を返す状況）は、
                false を返す場合がありますので、注意してください。
                
    @return     ネットワークに接続されているかが返ります。
    */
    bool IsConnected() const
    {
        return IsHost() || IsClient();
    }
    //! @endcond


    /*!
    @brief      ローカルホストマイグレーション処理中であるかを取得します。

    @return     ローカルホストマイグレーション処理中であるかが返ります。
    */
    bool IsDuringHostMigration() const;


    /*!
    @cond PRIVATE
    @brief      ローカルホストマイグレーションが有効であるかを取得します。

    @return     ローカルホストマイグレーションが有効であるかが返ります。
    */
    bool IsHostMigrationEnabled() const;
    //! @endcond


    /*!
    @brief リンクレベルを取得します。

    @details    リンクレベルは、0～3 の 4 段階あり、0:非常に通信品質が悪い、もしくは通信が成立していない、
                1:通信品質が悪い、2:通信品質があまり良くない、3:通信品質がよい という状態を表します。

    @attention  この API はローカルホストマイグレーション処理が行われている最中には正しい結果が得られない場合があります。
                ホストマイグレーション処理中（@ref LocalFacade::IsDuringHostMigration が true を返す状況）は実行しないよう注意してください。

    @return 現在接続中のネットワークの通信品質を返します。接続していない場合は 0 が返ります。
    */
    uint8_t GetLinkLevel() const;


    /*!
    @cond PRIVATE
    @brief ネットワークを解析し、そのネットワークから送信された電波の強度を取得します。

    @details    電波の強度は、0～3 の 4 段階あり、0:非常に通信品質が悪い、もしくは通信が成立していない、
                1:通信品質が悪い、2:通信品質があまり良くない、3:通信品質がよい という状態を表します。

    @attention  取得するリンクレベルは、@ref ScanNetworkAsync 実行時に保存されるローカルネットワーク情報と対応しています。<br>
                保存されているローカルネットワーク情報リストは@ref GetNetworkDescriptionList で取得できます。

    @param[out] pRadioStrength          電波の強度を記録する変数へのポインタ。データは正しくネットワークを解析できた場合のみ格納されます。
    @param[in]  descriptionIndex    解析対象のネットワーク指定用変数。ローカルネットワーク情報リストの先頭を0として、解析したいネットワークの番号を指定する必要があります。

    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultInvalidState      ローカルネットワーク情報リストに情報が保存されていません。または、transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。[:progErr]
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    */
    Result GetRadioStrength(uint8_t* pRadioStrength, uint32_t descriptionIndex) const;
    //! @endcond


    /*!
    @brief  使用中の IEEE 802.11b/g 無線チャンネルを取得します。

    @attention この API はローカルホストマイグレーション処理が行われている最中には正しい結果が得られない場合があります。
               ホストマイグレーション処理中（@ref LocalFacade::IsDuringHostMigration が true を返す状況）は実行しないよう注意してください。

    @return 現在のチャンネルが返ります。接続していない場合や取得に失敗した場合は 0 が返ります。
    */
    uint8_t GetNetworkChannel() const;

    /*!
    @cond PRIVATE
    @brief  新規クライアントのネットワークへの参加を不許可にします。現在接続中のクライアントには影響しません。

    @details 通信対戦中は新規にクライアントが接続することを拒否したい、と言うような用途を想定した接続拒否機能です。
             再び接続を許可したい場合は @ref AllowParticipating を実行する必要があります。
 
    @note    この API で設定した情報はホストの異常切断によるホストマイグレーション後の新しいネットワークには引き継がれない場合があります。

    @attention    本関数はローカルネットワークホストでのみ実行可能です。

    @param[in] isDisallowToReconnect  false の場合、本 API を呼び出した後に切断されたクライアントだけが再接続できます。

    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultNotInitialized        初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、ホストではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。または、ローカルネットワークから切断されました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     ローカルネットワークから切断されました。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    */
    Result DisallowParticipating(bool isDisallowToReconnect = true);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  新規クライアントのネットワークへの参加を許可します。

    @details @ref DisallowParticipating で拒否した、新規クライアントの接続を再開するための関数です。

    @note    この API で設定した情報はホストの異常切断によるホストマイグレーション後の新しいネットワークには引き継がれない場合があります。

    @attention    本関数はローカルネットワークホストでのみ実行可能です。

    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultNotInitialized        初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、ホストではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。または、ローカルネットワークから切断されました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     ローカルネットワークから切断されました。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    */
    Result AllowParticipating();
    //! @endcond

    /*!
    @cond PRIVATE
    */
    Result ChangeAllowParticipatingState(nn::pia::common::CallContext* pCallContext, AllowParticipatingState state);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  指定したクライアントをネットワークから追放します。

    @attention    本関数はローカルネットワークホストでのみ実行可能です。

    @param[in] address  追放するクライアントの @ref common::StationAddress です。

    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultNotInitialized        初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、ホストではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。または、ローカルネットワークから切断されました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     ローカルネットワークから切断されました。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    */
    Result EjectClient(const common::StationAddress& address);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  すべてのクライアントをネットワークから追放します。

    @attention    本関数はローカルネットワークホストでのみ実行可能です。

    @return 正常に処理が完了した場合、成功の Result が返ります。

    @retval ResultNotInitialized        初期化されていません。[:progErr]
    @retval ResultInvalidState          transport::Transport::Initialize() の呼び出しから transport::Transport::Finalize() の呼び出しまでの間に呼び出す必要があります。または、ホストではありません。[:progErr]
    @if CTR_DOC
    @retval ResultInvalidConnection     無線オフモードに遷移しました。または、ローカルネットワークから切断されました。[:handling-cleanup]
    @elseif NIN_DOC
    @retval ResultInvalidConnection     ローカルネットワークから切断されました。[:handling-cleanup]
    @retval ResultWifiOff               無線がオフになっています。[:viewer][:handling-shutdownNetwork]
    @retval ResultSleep                 スリープ状態です。[:viewer][:handling-shutdownNetwork]
    @endif
    */
    Result EjectClientAll();
    //! @endcond

    /*!
    @brief  ネットワークから切断された理由を取得します。

    @details Startup 時、Cleanup 時に @ref DisconnectReason_None にリセットされます。
    @details 通信エラーの検知には、本関数ではなく session::Session::CheckConnectionError() を使用する必要があります。

    @return ネットワークから切断された理由です。
    */
    DisconnectReason GetDisconnectReason();

    /*!
    @cond PRIVATE
    @brief  新規参加許可/拒否の状態を取得します。

    @return 新規参加許可/拒否の状態です。
    */
    AllowParticipatingState GetAllowParticipatingState() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  送信前のスリープ時間を設定します。

    @param[in] sleepTime    送信前のスリープ時間。単位はミリ秒です。
    */
    void SetSleepBeforeSendTime(int64_t sleepTime);
    //! @endcond

    /*!
    @cond PRIVATE
     */
    void SetLocalCommunicationIdZeroEnabled(bool b);
    bool IsLocalCommunicationIdZeroEnabled() const;
    //! @endcond

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    /*!
    @cond PRIVATE
    */

    void RegisterLocalProtocol(LocalProtocol* pLocalProtocol);
    void UnregisterLocalProtocol();
    LocalProtocol* GetLocalProtocol()
    {
        return m_pLocalProtocol;
    }
    const LocalProtocol* GetLocalProtocol() const
    {
        return m_pLocalProtocol;
    }

    uint32_t GetSessionId() const;
    uint32_t GetSessionIdFromNetworkDescription(const LocalNetworkDescription* pNetworkDescription) const;

    uint8_t GetNetworkParticipantNum() const;

    Result GetNetworkErrorResult() const;
    bool IsNeverConnected() const;

    // Mesh 用
    Result StartupForMesh();
    void CleanupForMesh();
    Result GetHostStationConnectionInfo(nn::pia::transport::StationConnectionInfo* pStationConnectionInfo) const;

    bool IsUpdateConnectionStatusRequested();
    void SetUpdateConnectionStatusRequested(bool b);
    //! @endcond

private:
#if NN_PIA_CTR
    Result InitializeCore(const UdsNetworkSetting& setting);
#endif
#if NN_PIA_NINTENDOSDK
    Result InitializeCore(const LdnNetworkSetting& setting);
#endif

    Result ConvertAddressToStationConnectionInfo(const common::InetAddress& address, nn::pia::transport::StationConnectionInfo* pStationConnectionInfo, uint32_t connectionId) const;

    static void UpdateEventCallbackForMesh(LocalUpdateEvent updateEvent, common::StationAddress address, void* pParam);

private:
    static LocalFacade* s_pInstance;
    static bool s_IsInitialized;

    bool m_IsStarted;

    LocalNetworkType m_LocalNetworkType;
    LocalNetworkSetting* m_pLocalNetworkSetting;

    LocalProtocol* m_pLocalProtocol;

    bool m_IsLocalCommunicationIdZeroEnabled;
};
}
}
}  // end of namespace nn::pia::local
