﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#include <nn/pia/transport/transport_NetworkFactory.h>


namespace nn
{
namespace pia
{
namespace local
{


/*!
  @class  nn::pia::local::LocalNetworkFactory
  @brief  LocalFacade を利用したネットワークに必要なクラスを生成するための基底クラスです（ファクトリパターン）。

  @details アプリケーションが本クラスを使用することはありません。本クラスを継承したクラスを使用してください。

  @if CTR_DOC
  @see nn::pia::local::UdsNetworkFactory
  @elseif NIN_DOC
  @see nn::pia::local::LdnNetworkFactory
  @endif
 */
class LocalNetworkFactory : public nn::pia::transport::NetworkFactory
{
public:
    /*!
    @cond PRIVATE
     */


    /*!
    @brief デストラクタです。
     */
    virtual ~LocalNetworkFactory(void);

    /*!
    @brief InputStream を生成します。
     */
    virtual nn::pia::common::IPacketInput* CreateInputStream() = 0;

    /*!
    @brief OutputStream を生成します。
     */
    virtual nn::pia::common::IPacketOutput* CreateOutputStream() = 0;

    /*!
    @brief ConnectStationJob を生成します。
     */
    virtual nn::pia::transport::ConnectStationJob* CreateConnectStationJob();


    /*!
    @brief DisconnectStationJob を生成します。
     */
    virtual nn::pia::transport::DisconnectStationJob* CreateDisconnectStationJob();


    /*!
    @brief CreateMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateCreateMeshJob();


    /*!
    @brief JoinMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateJoinMeshJob();


    /*!
    @brief LeaveMeshJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateLeaveMeshJob();


    /*!
    @brief ProcessHostMigrationJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateProcessHostMigrationJob();


    /*!
    @brief LeaveWithHostMigrationJob を生成します。
     */
    virtual common::StepSequenceJob* CreateLeaveWithHostMigrationJob();


    /*!
    @brief KickoutManageJob を生成します。
     */
    virtual nn::pia::common::StepSequenceJob* CreateKickoutManageJob();


    /*!
    @brief PacketHandler の具象インスタンスを構築します。
     */
    virtual transport::PacketHandler* CreatePacketHandler();


    /*!
    @brief 署名が必要かどうか。
     */
    virtual bool IsSignatureRequired() const;


    /*!
    @brief SignatureSettingStorage を生成します。
     */
    virtual session::SignatureSettingStorage* CreateSignatureSettingStorage();


    /*!
    @brief ネットワーク層におけるパケットのヘッダ部分の合計サイズを返します。単位はバイトです。
     */
    virtual uint32_t GetHeaderSize() const = 0;


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    /* 自動マッチメイク機能用 */
    /*!
    @brief Session 用 CreateSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchCreateSessionJob();


    /*!
    @brief Session 用 AutoMatchmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchAutoMatchmakeSessionJob();


    /*!
    @brief Session 用 BrowseMatchmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchBrowseMatchmakeSessionJob();


    /*!
    @brief Session 用 JoinSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchJoinSessionJob();


    /*!
    @brief Session 用 LeaveSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchLeaveSessionJob();


    /*!
    @brief Session 用 DestroySessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchDestroySessionJob();

    /*!
    @brief ジョイントセッション機能用 JointSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateJointSessionJob();

    /*!
    @brief Session 用 UpdateSessionSetting を生成します。
     */
    virtual common::StepSequenceJob* CreateUpdateSessionSettingJob();

    /*!
    @cond PRIVATE
    @brief Session 用 CommunityManagementJob を生成します。
     */
    virtual common::StepSequenceJob* CreateCommunityManagementJob();
    //! @endcond

    /*!
    @brief Session 用 GenerateMatchmakeSystemPasswordJob を生成します。
           ローカルには不要なので NULL を返します。
     */
    virtual common::StepSequenceJob* CreateGenerateMatchmakeSystemPasswordJob();

    /*!
    @brief Session 用 ClearMatchmakeSystemPasswordJob を生成します。
           ローカルには不要なので NULL を返します。
     */
    virtual common::StepSequenceJob* CreateClearMatchmakeSystemPasswordJob();

    virtual common::String* CreateMatchmakeSessionSystemPassword();

    /*!
    @brief Session 用 IMatchmakeSession を生成します。
     */
    virtual session::IMatchmakeSession* CreateMatchmakeSession() = 0;

    /*!
    @brief Session 用 ISessionInfoList を生成します。
     */
    virtual session::ISessionInfoList* CreateSessionInfoList(uint16_t browseSessionInfoListNum) = 0;

    /*!
    @brief Session 用 ICommunityInfoList を生成します。
     */
    virtual session::ICommunityInfoList* CreateCommunityInfoList(uint8_t browsedCommunityInfoListNum);

    /*!
    @brief Session 用 MeshLayerController を生成します。
     */
    virtual session::MeshLayerController* CreateMatchMeshLayerController();

    virtual bool IsBindRequired() const;

    virtual bool IsServerRequired() const;

    virtual bool IsRecentPlayRecordPermitted() const = 0;

    virtual bool IsMultiplayerPermitted() const = 0;

    virtual int32_t GetHostMigrationMode() const;

    virtual uint16_t GetBrowseSessionInfoListSizeMax() const = 0;

    virtual uint8_t GetBrowseCommunityInfoListSizeMax() const = 0;

    /*!
    @brief MTU の最大値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMax() const = 0;

    /*!
    @brief MTU の最小値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMin() const = 0;

    /*!
    @brief 初期段階での RTT 計測パルス送信間隔のデフォルト値です。単位はミリ秒です。
     */
    virtual int GetDefaultRttInitialPulseInterval() const;

    /* 自動マッチメイク機能用 */


    /*!
    @endcond
     */

protected:
    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。
     */
    LocalNetworkFactory(void);
    //! @endcond

private:
};
}
}
}  // end of namespace nn::pia::local
