﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#include <nn/pia/session/session_SessionStationInfo.h>

namespace nn
{
namespace pia
{
namespace common
{
class StationAddress;
}
}
}  // end of namespace nn::pia::local

namespace nn
{
namespace pia
{
namespace local
{

static const uint8_t LocalNetworkVersion = 1;  //!< PiaLocal で扱うネットワークのバージョンです。

#if NN_PIA_CTR
static const uint8_t StationMaxEntry = 12;     //!< 最大接続台数です。ローカルネットワークホストも接続台数に含まれる点に注意してください。
static const uint32_t ScanNetworkNumMax = 16;  //!< ネットワーク探索をした際に、1度に保持できる探索結果の最大数です。
#elif NN_PIA_NINTENDOSDK
static const uint8_t StationMaxEntry = 8;  //!< 最大接続台数です。ローカルネットワークホストも接続台数に含まれる点に注意してください。
static const uint32_t ScanNetworkNumMax = 24;  //!< ネットワーク探索をした際に、1度に保持できる探索結果の最大数です。
#else
static const uint8_t StationMaxEntry = 8;  //!< 最大接続台数です。ローカルネットワークホストも接続台数に含まれる点に注意してください。
static const uint32_t ScanNetworkNumMax = 24;  //!< ネットワーク探索をした際に、1度に保持できる探索結果の最大数です。
#endif

/*!
@cond PRIVATE
@brief ローカルネットワークの状態更新イベント。
 */
enum LocalUpdateEvent
{
    LocalUpdateEvent_Unknown = 0,      //!< 下記に定義している以外の未知の状態です。正常に動作している限りこの状態になることはありません。
    LocalUpdateEvent_Disconnected,     //!< ローカルネットワークからの切断イベントです。
    LocalUpdateEvent_MigrationStarted  //!< ホストマイグレーション処理を開始した状態です。
};

typedef void (*LocalUpdateEventCallback)(LocalUpdateEvent e, common::StationAddress address, void* pParam);
//! @endcond

/*!
@cond PRIVATE
 */
enum LocalNetworkType
{
    LocalNetworkType_Uds = 0,
    LocalNetworkType_Ldn
};
//! @endcond

/*!
  @brief  ローカルネットワークの設定用クラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。

  @if CTR_DOC
  @see nn::pia::local::UdsNetworkSetting
  @elseif NIN_DOC
  @see nn::pia::local::LdnNetworkSetting
  @endif
 */
class LocalNetworkSetting : public nn::pia::common::RootObject
{
protected:
    LocalNetworkSetting()
        : isAroundNetworkSearchEnabled(false)
    {
    }

    virtual ~LocalNetworkSetting()
    {
    }

public:
    virtual LocalNetworkType GetLocalNetworkType() const = 0;

    //n1589:todo1:delete isAroundNetworkSearchEnabled
    bool isAroundNetworkSearchEnabled;  //!< @cond CTR_DOC アラウンドネットワークサーチ機能の有効／無効切替。trueならば有効、falseならば無効になります。 @endcond
};


/*!
  @cond PRIVATE
  @brief  ローカルネットワーク作成時の設定用クラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。

  @if CTR_DOC
  @see nn::pia::local::UdsCreateNetworkSetting
  @elseif NIN_DOC
  @see nn::pia::local::LdnCreateNetworkSetting
  @endif
 */
class LocalCreateNetworkSetting : public nn::pia::common::RootObject
{
protected:
    LocalCreateNetworkSetting()
    {
    }

    virtual ~LocalCreateNetworkSetting()
    {
    }

public:
    virtual uint8_t* GetApplicationDataPtr() = 0;
    virtual uint32_t GetApplicationDataSize() const = 0;
    virtual void AddApplicationDataSize(uint32_t size) = 0;
};
//! @endcond


/*!
  @cond PRIVATE
  @brief  ローカルネットワークについての情報を保持するクラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。

  @if CTR_DOC
  @see nn::pia::local::UdsNetworkDescription
  @elseif NIN_DOC
  @see nn::pia::local::LdnNetworkDescription
  @endif
 */
class LocalNetworkDescription : public nn::pia::common::RootObject
{
public:
    virtual uint16_t GetCurrentParticipantNum() const = 0;
    virtual uint16_t GetMaxParticipantNum() const = 0;
    virtual bool IsOpened() const = 0;
    virtual uint16_t GetChannel() const = 0;
    virtual void GetBssid(uint8_t* pBssidBuffer) const = 0;
    virtual uint32_t GetGameMode() const = 0;

protected:
    LocalNetworkDescription()
    {
    }

    virtual ~LocalNetworkDescription()
    {
    }

public:
    virtual void Copy(const LocalNetworkDescription* description) = 0;
};
//! @endcond


/*!
  @cond PRIVATE
  @brief  ローカルネットワークへ接続時の設定用クラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。
  @if CTR_DOC
  @see nn::pia::local::UdsConnectNetworkSetting
  @elseif NIN_DOC
  @see nn::pia::local::LdnConnectNetworkSetting
  @endif
 */
class LocalConnectNetworkSetting : public nn::pia::common::RootObject
{
protected:
    LocalConnectNetworkSetting()
        : pLocalNetworkDescription(NULL),
        hashedSessionUserPassword(0)
    {
    }

public:
    const LocalNetworkDescription* pLocalNetworkDescription;  //!< 接続先ネットワークの情報です。
    uint32_t hashedSessionUserPassword;
};
//! @endcond

/*!
  @cond PRIVATE
  @brief  ローカルネットワーク探索時の設定用クラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。
  @if CTR_DOC
  @see nn::pia::local::UdsScanNetworkSetting
  @elseif NIN_DOC
  @see nn::pia::local::LdnScanNetworkSetting
  @endif
 */
class LocalScanNetworkSetting : public nn::pia::common::RootObject
{
protected:
    LocalScanNetworkSetting()
    {
    }
public:
};
//! @endcond

/*!
@cond PRIVATE
以下はライブラリ内部向けの宣言です。
*/

class LocalBeaconSystemData : public nn::pia::common::RootObject
{
public:
    uint32_t sessionId;      //!< ホストマイグレーション前後で変わらない、セッション ID
    uint32_t hashedUserPassword;    //!< ハッシュ化したユーザーパスワード
    uint8_t networkVersion;  //!< ローカルネットワークバージョン
    uint8_t reserved[3];

protected:
    LocalBeaconSystemData()
        : sessionId(0),
          hashedUserPassword(0),
          networkVersion(LocalNetworkVersion)
    {
        memset(reserved, 0, sizeof(reserved));
    }
};


/*!
ここまでライブラリ内部向けの宣言です。
@endcond
*/


/*!
  @brief  ローカルネットワークに接続しているステーションについての情報を保持するクラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。
  @if CTR_DOC
  @see nn::pia::local::UdsStationInfo
  @elseif NIN_DOC
  @see nn::pia::local::LdnStationInfo
  @endif
 */
//n1589:memo:StationInfoRoleのみ持つ
class LocalStationInfo : public session::SessionStationInfo
{
protected:
    LocalStationInfo()
        : session::SessionStationInfo()
    {
    }
    virtual ~LocalStationInfo()
    {
    }
};
}
}
}  // end of namespace nn::pia::local
