﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_LocalSessionTypes.h>
#include <nn/pia/local/local_LocalNetworkSetting.h>
#include <nn/pia/session/session_ISessionInfo.h>


namespace nn
{
namespace pia
{
namespace local
{

class LocalNetworkDescription;

/*!
    @brief  ローカル通信のセッション情報クラスです。
*/
class LocalSessionInfo : public session::ISessionInfo
{
public:
    /*!
    @brief      デストラクタ。
    */
    virtual ~LocalSessionInfo();

    /*!
    @brief      セッションのゲームモードを取得します。
    @details    通信モード識別用 ID を uint32_t にキャストして返します。
    @return     セッションのゲームモードを返します。
    */
    virtual uint32_t GetGameMode() const = 0;

    /*!
    @brief      セッションの ID を取得します。
    @details    同じセッション ID を持つ異なるセッションが存在する場合があります。
    @return     セッションの ID を返します。
    */
    virtual uint32_t GetSessionId() const = 0;

    /*!
    @brief      セッションに参加している人数を取得します。
    @return     セッションに参加している人数を返します。
    */
    virtual uint16_t GetCurrentParticipantNum() const = 0;

    /*!
    @brief      セッションに参加できる最小人数を取得します。
    @return     セッションに参加できる最小人数を返します。
    */
    virtual uint16_t GetMinParticipantNum() const;

    /*!
    @brief      セッションに参加できる最大人数を取得します。
    @return     セッションに参加できる最大人数を返します。
    */
    virtual uint16_t GetMaxParticipantNum() const = 0;

    /*!
    @brief      セッションが参加募集受付状態かどうかを取得します。
    @return     セッションが参加募集受付状態なら true, 参加募集締切状態なら false を返します。
    */
    virtual bool IsOpened() const = 0;

    /*!
    @brief              セッションのアプリケーション定義データを取得します。
    @details            セッションに設定したアプリケーション定義データを取得します。
    @param[out] pBuffer セッションのアプリケーション定義データのコピー用バッファのポインタ。
    @param[in]  size    セッションのアプリケーション定義データのコピー用バッファのサイズ。
    @return             正常に情報を取得できた場合、成功の Result が返ります。
    */
    virtual Result GetApplicationData(void* pBuffer, uint32_t size) const = 0;

    /*!
    @brief      セッションのアプリケーション定義データの長さを取得します。
    @return     セッションのアプリケーション定義データの長さが返ります。
    */
    virtual uint32_t GetApplicationDataSize() const = 0;

    /*!
    @brief      セッションの電波強度を取得します。
    @return     セッションの電波強度が返ります。
    */
    virtual uint8_t GetRadioStrength() const;

    /*!
    @brief                                  セッションに参加中のステーション情報を取得します。
    @param[out] pSessionStationInfoList     ステーション情報リスト格納先のポインタ。
    @param[in]  sessionStationInfoListSize  ステーション情報リスト格納先のサイズ。
    @return                                 正常に情報を取得できた場合、成功の Result が返ります。
    */
    virtual Result GetSessionStationInfo(LocalStationInfo* pSessionStationInfoList, uint32_t sessionStationInfoListSize) const = 0;

    /*!
    @brief              BSSID（ホストの MAC アドレス）を取得します。
    @details            BSSID のサイズは @ref BssIdSize バイトです。
    @param[out] pBssid  BSSID の格納先バッファのポインタ。
    @param[in]  size    BSSID の格納先バッファのサイズ。
    @return             正常に情報を取得できた場合、成功の Result が返ります。
    */
    virtual Result GetBssid(uint8_t* pBssid, uint32_t size) const = 0;

    /*!
    @brief      セッションにユーザーパスワードが設定されているかどうかを確認します。
    @return     ユーザーパスワードが設定されている場合は true が返ります。
    */
    virtual bool IsRestrictedByUserPassword() const;

    /*!
    @brief      セッション情報をクリアします。
    */
    virtual void Clear();

    /*!
    @brief デバッグに有用な情報をプリントします。
    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    //! @cond PRIVATE
    virtual void SetLocalNetworkDescription(uint32_t networkIndex, const LocalNetworkDescription* pDescription) = 0;
    virtual const LocalNetworkDescription* GetLocalNetworkDescription() const = 0;
    void SetRadioStrength(uint32_t index);                   // index はネットワーク検索時のインデックス
    void SetLinkLevel(uint8_t linkLevel);                    // セッション参加中に電波強度を直接設定
    virtual Result SetSessionStationInfo(uint32_t index) = 0;  // index はネットワーク検索時のインデックス
    virtual void SetRestrictedByUserPassword(bool b);
    //! @endcond

protected:
    void Copy(const LocalSessionInfo& rhs);

protected:
    bool m_IsNetworkDescriptionSet;

    uint8_t m_RadioStrength;

    bool m_IsRestrictedByUserPassword;

    LocalSessionInfo();

private:
    NN_PIA_DISALLOW_COPY(LocalSessionInfo);
};
}
}
}  // end of namespace nn::pia::local2
