﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#include <nn/pia/local/local_LocalSessionTypes.h>
#include <nn/pia/session/session_SessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace local
{
// 先行宣言
class LocalScanNetworkSetting;
class LocalNetworkDescription;

/*!
  @brief  LocalFacade を利用したネットワークのセッションを検索する際に使用する検索条件の基底クラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。

  @if CTR_DOC
  @see nn::pia::local::UdsSessionSearchCriteria
  @elseif NIN_DOC
  @see nn::pia::local::LdnSessionSearchCriteria
  @endif
 */
class LocalSessionSearchCriteria : public nn::pia::session::SessionSearchCriteria
{
public:
    // デストラクタ。
    virtual ~LocalSessionSearchCriteria()
    {
    }

    virtual const LocalScanNetworkSetting* GetLocalScanNetworkSetting() const = 0;

    /*!
    @brief      検索条件に最大参加人数を指定します。
    @param[in]  max 検索条件に指定する最大参加人数。
    */
    void SetMaxParticipantNum(uint16_t max)
    {
        m_MaxParticipantsRangeMax = max;
        m_MaxParticipantsRangeMin = max;
    }

    /*!
    @brief      検索条件に最大参加人数を範囲指定します。
    @param[in]  min 検索条件に指定する最大参加人数の範囲の下限。
    @param[in]  max 検索条件に指定する最大参加人数の範囲の上限。
    */
    void SetMaxParticipantNumRange(uint16_t min, uint16_t max)
    {
        m_MaxParticipantsRangeMax = max;
        m_MaxParticipantsRangeMin = min;
    };

    /*!
    @brief      検索条件に参加受付状態のセッションのみを検索するか指定します。
    @param[in]  bOpenedOnly 参加受付状態のセッションのみを検索する場合は true を指定します。
    */
    void SetOpenedOnly(bool bOpenedOnly)
    {
        m_bOpenedOnly = bOpenedOnly;
    }

    /*!
    @brief      検索条件に空きがあるセッションのみを検索するか指定します。
    @param[in]  bVacantOnly 空きがあるセッションのみを検索する場合は true を指定します。
    */
    void SetVacantOnly(bool bVacantOnly)
    {
        m_bVacantOnly = bVacantOnly;
    }

    /*!
    @cond PRIVATE
     */
    virtual bool IsHit(const LocalNetworkDescription* pDescription) const = 0;
    //!@endcond

    /*!
    @brief  設定をリセットします。
     */
    virtual void Reset()
    {
        session::SessionSearchCriteria::Reset();
        m_MaxParticipantsRangeMax = InvalidParticipantsNum;
        m_MaxParticipantsRangeMin = InvalidParticipantsNum;
        m_bOpenedOnly = true;
        m_bVacantOnly = false;
    }

    // デバッグに有用な情報をプリント。
    virtual void Trace(uint64_t flag) const
    {
        NN_PIA_UNUSED(flag);
    }

protected:
    // コンストラクタ。
    LocalSessionSearchCriteria()
        : session::SessionSearchCriteria(),
          m_MaxParticipantsRangeMax(InvalidParticipantsNum),
          m_MaxParticipantsRangeMin(InvalidParticipantsNum),
          m_bOpenedOnly(true),
          m_bVacantOnly(false)
    {
    }

    void Copy(const LocalSessionSearchCriteria& rhs)
    {
        session::SessionSearchCriteria::Copy(rhs);
        m_MaxParticipantsRangeMax = rhs.m_MaxParticipantsRangeMax;
        m_MaxParticipantsRangeMin = rhs.m_MaxParticipantsRangeMin;
        m_bOpenedOnly = rhs.m_bOpenedOnly;
        m_bVacantOnly = rhs.m_bVacantOnly;
    }

    bool IsHitCommon(const LocalNetworkDescription* pDescription) const;

    uint16_t m_MaxParticipantsRangeMax;
    uint16_t m_MaxParticipantsRangeMin;
    bool m_bOpenedOnly;
    bool m_bVacantOnly;

private:
    NN_PIA_DISALLOW_COPY(LocalSessionSearchCriteria);
};
}
}
}  // end of namespace nn::pia::local
