﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#if NN_PIA_LOCAL_ENABLE_UDS
#include <nn/pia/local/local_LocalCreateSessionSetting.h>
#include <nn/pia/local/local_UdsNetworkSetting.h>

namespace nn
{
namespace pia
{
namespace local
{
/*!
  @brief  UDS ライブラリ（ローカル通信）を利用したネットワークの CreateSession に関する設定を管理するクラスです。
 */
class UdsCreateSessionSetting : public LocalCreateSessionSetting
{
public:
    /*!
    @brief  デフォルトコンストラクタ。
     */
    UdsCreateSessionSetting();

    /*!
    @brief  デストラクタ。
     */
    virtual ~UdsCreateSessionSetting();

    /*!
    @cond PRIVATE
     */
    virtual const LocalCreateNetworkSetting* GetLocalCreateNetworkSetting() const
    {
        return &m_UdsCreateNetworkSetting;
    }
    //! @endcond

    /*!
    @brief      作成するセッションの最大参加人数を指定します。
    @param[in]  max 作成するセッションに指定する最大参加人数。@ref StationMaxEntry 以下である必要があります。
    */
    virtual void SetMaxParticipantNum(uint16_t max)
    {
        LocalCreateSessionSetting::SetMaxParticipantNum(max);
        m_UdsCreateNetworkSetting.maxEntry = max;
    }

    /*!
    @brief      作成するセッションのローカル通信 ID を指定します。
    @param[in]  localCommunicationId ローカル通信 ID です。@ref LocalFacade::CreateLocalCommunicationId で生成した値を指定する必要があります。
    */
    void SetLocalCommunicationId(uint32_t localCommunicationId)
    {
        m_UdsCreateNetworkSetting.localCommunicationId = localCommunicationId;
    }

    /*!
    @brief      作成するセッションの通信モード識別用 ID を指定します。
    @param[in]  subId   通信モード識別用 ID です。0x00 から 0xFE の間で指定する必要があります。
    */
    void SetSubId(uint8_t subId)
    {
        m_UdsCreateNetworkSetting.subId = subId;
    }

    /*!
    @brief      作成するセッションの通信に使用するチャンネルを指定します。
    @param[in]  channel 通信に使用するチャンネルです。0(自動), 1,6,11ch のいずれかを指定する必要があります。製品実機で実行した場合は常にチャンネルを自動で選択します。
    */
    void SetChannel(uint8_t channel)
    {
        m_UdsCreateNetworkSetting.channel = channel;
    }

    /*!
    @brief  無線レイヤの暗号化に使用する暗号鍵とそのサイズを設定します。

    @details 暗号鍵のサイズは、@ref UdsWirelessCryptoKeyLengthMin 以上、@ref UdsWirelessCryptoKeyLengthMax 以下である必要があります。

    @param[in]  pWirelessCryptoKey 無線レイヤの暗号化に使用する暗号鍵です。
    @param[in]  wirelessCryptoKeySize  無線レイヤの暗号化に使用する暗号鍵のサイズです。

    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidArgument   引数の指定が不正です。[:progErr]
    */
    virtual Result SetWirelessCryptoKey(const char* pWirelessCryptoKey, uint8_t wirelessCryptoKeySize);

    /*!
    @brief      アプリケーション定義のデータを指定します。

    @details データのサイズは、@ref UdsBeaconApplicationDataSizeMax 以下である必要があります。

    @param[in]  pData   構築するセッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    構築するセッションに指定するアプリケーション定義データのサイズ。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータサイズを取得します。
    @return     アプリケーション定義データサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSize() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      アプリケーション定義を指定したかどうかを取得します。
    @return     アプリケーション定義を指定したかどうかが返されます。
    */
    virtual bool IsSetApplicationData() const;
    //! @endcond

    /*!
    @brief      設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const UdsCreateSessionSetting& rhs);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    UdsCreateSessionSetting& operator=(const UdsCreateSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    UdsCreateSessionSetting(const UdsCreateSessionSetting& rhs);

    /*!
    @brief  設定をリセットします。
     */
    virtual void Reset();

    // デバッグに有用な情報をプリント。
    virtual void Trace(uint64_t flag) const;

protected:
    UdsCreateNetworkSetting m_UdsCreateNetworkSetting;
    bool m_IsApplicationDataSet;

private:
};
}
}
}  // end of namespace nn::pia::local
#endif
