﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#if NN_PIA_LOCAL_ENABLE_UDS
#include <nn/pia/local/local_LocalNetworkFactory.h>

namespace nn
{
namespace pia
{
namespace local
{


/*!
  @class  nn::pia::local::UdsNetworkFactory
  @brief  UDS ライブラリ（ローカル通信）を利用したネットワークに必要なクラスを生成するためのクラスです（ファクトリパターン）。

  @details @ref nn::pia::transport::Transport や @ref nn::pia::session::Session に、UdsNetworkFactory クラスを設定することで
           LocalFacade に合わせたネットワーク関連処理（送受信、セッション管理など）が行われるようになります。

  @see nn::pia::transport::Transport::Setting, nn::pia::session::Session::Setting
 */
class UdsNetworkFactory : public nn::pia::local::LocalNetworkFactory
{
public:
    /*!
     @name 初期化終了処理
     @{
     */

    /*!
    @brief デフォルトコンストラクタです。
     */
    UdsNetworkFactory(void);


    /*!
    @brief デストラクタです。
     */
    virtual ~UdsNetworkFactory(void);

    /*!
     end of name 初期化終了処理
     @}
     */

    /*!
    @cond PRIVATE
     */

    virtual transport::NetworkFactory* CreateSelf();

    /*!
     @brief 最大参加可能ステーション数を返します。
     */
    virtual uint16_t GetMaxStationNum() const;

    /*!
    @brief InputStream を生成します。
    */
    virtual nn::pia::common::IPacketInput* CreateInputStream();

    /*!
    @brief OutputStream を生成します。
    */
    virtual nn::pia::common::IPacketOutput* CreateOutputStream();

    virtual Result CreateProtocols();

    /*!
    @brief ネットワーク層におけるパケットのヘッダ部分の合計サイズを返します。単位はバイトです。
    */
    virtual uint32_t GetHeaderSize() const;

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    /*!
    @brief Session 用 IMatchmakeSession を生成します。
     */
    virtual session::IMatchmakeSession* CreateMatchmakeSession();

    virtual uint16_t GetBrowseSessionInfoListSizeMax() const;

    virtual session::ISessionInfoList* CreateSessionInfoList(uint16_t browseSessionInfoListNum);

    virtual uint8_t GetBrowseCommunityInfoListSizeMax() const;

    virtual bool IsRecentPlayRecordPermitted() const;
    virtual bool IsMultiplayerPermitted() const;

    /*!
    @brief MTU の最大値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMax() const;

    /*!
    @brief MTU の最小値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMin() const;

    /* 自動マッチメイク機能用 */

    virtual bool IsConnectionRequestPrincipalIdEnabled() const
    {
        return false;
    }

    /*!
    @endcond
     */

protected:
private:
};
}
}
}  // end of namespace nn::pia::local
#endif
