﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#if NN_PIA_LOCAL_ENABLE_UDS
#include <nn/pia/local/local_LocalNetworkSetting.h>

namespace nn
{
namespace pia
{
namespace local
{

static const uint32_t UdsReceiveBufferSizeMin = 4 * 4096;  //!< 受信バッファの最小サイズです。
static const uint32_t UdsScanBufferSizeMin = 1 * 1024;     //!< スキャンバッファの最小サイズです。

static const uint8_t UdsWirelessCryptoKeyLengthMin = nn::uds::UDS_PASSPHRASE_LENGTH_MIN;  //!< 通信に使用する暗号鍵生成のパスフレーズの最小サイズです。
static const uint8_t UdsWirelessCryptoKeyLengthMax = nn::uds::UDS_PASSPHRASE_LENGTH_MAX;  //!< 通信に使用する暗号鍵生成のパスフレーズの最大サイズです。

static const uint32_t UdsBeaconSystemDataSize = 28;
static const uint32_t UdsBeaconApplicationDataSizeMax = nn::uds::NET_DESC_APPDATA_SIZE_MAX - UdsBeaconSystemDataSize;  //!< アプリケーションがビーコンにセットできるデータの最大サイズです。

/*!
  @brief  UDS ライブラリを利用したネットワークの設定用クラスです。
 */
class UdsNetworkSetting : public LocalNetworkSetting
{
public:
    UdsNetworkSetting()
        : LocalNetworkSetting(),
          receiveBufferSize(0),
          pReceiveBuffer(NULL),
          scanBufferSize(0),
          sendOption(nn::uds::NO_WAIT),
          receiveOption(nn::uds::NO_WAIT),
          applicationVersion(0),
          pUserName(NULL)
    {
    }

    virtual ~UdsNetworkSetting()
    {
    }

    /*!
    @cond PRIVATE
     */
    virtual LocalNetworkType GetLocalNetworkType() const
    {
        return LocalNetworkType_Uds;
    }
    //! @endcond

    uint32_t receiveBufferSize;         //!< 受信バッファサイズです。@ref local::UdsReceiveBufferSizeMin 以上、かつ、4096 の倍数である必要があります。
    void* pReceiveBuffer;               //!< 受信バッファです。アプリケーション側で確保したバッファを使用したい場合に設定します。4096 バイト整合されている必要があります。設定されなかった場合は、内部で確保します。
    uint32_t scanBufferSize;            //!< スキャン用バッファサイズです。@ref local::UdsScanBufferSizeMin 以上である必要があります。
    uint8_t sendOption;                 //!< UDS 送信オプションです。nn::uds::NO_WAIT、nn::uds::FORCE_DIRECT_BC などが設定できます。複数のオプションを設定する場合は、OR 演算子を使用します。
    uint8_t receiveOption;              //!< UDS 受信オプションです。常に nn::uds::NO_WAIT をオンにしてください。複数のオプションを設定する場合は、OR 演算子を使用します。
    uint8_t applicationVersion;         //!< アプリケーションのバージョンです。構築されたネットワークは、この値が異なるバージョンのアプリケーションから発見はされますが、接続には失敗します。
    nn::cfg::CTR::UserName* pUserName;  //!< ユーザー名。
};

/*!
  @cond PRIVATE
  @brief  UDS ライブラリを利用したネットワーク作成時の設定用クラスです。
 */
class UdsCreateNetworkSetting : public LocalCreateNetworkSetting
{
public:
    /*!
    @brief  デフォルトコンストラクタ。
     */
    UdsCreateNetworkSetting()
        : LocalCreateNetworkSetting(),
          subId(0),
          maxEntry(0),
          localCommunicationId(0),
          wirelessCryptoKeyLength(0),
          channel(0),
          applicationDataSize(0)
    {
        memset(wirelessCryptoKey, 0, sizeof(wirelessCryptoKey));
        memset(applicationData, 0, sizeof(applicationData));
    }

    virtual ~UdsCreateNetworkSetting()
    {
    }

    /*!
    @cond PRIVATE
     */
    virtual uint8_t* GetApplicationDataPtr()
    {
        return applicationData;
    }
    //! @endcond

    /*!
    @cond PRIVATE
     */
    virtual uint32_t GetApplicationDataSize() const
    {
        return applicationDataSize;
    }
    //! @endcond

    /*!
    @cond PRIVATE
     */
    virtual void AddApplicationDataSize(uint32_t size)
    {
        applicationDataSize += size;
    }
    //! @endcond

    uint8_t subId;                                                                       //!< 通信モード識別用 ID です。0x00 から 0xFE の間で設定する必要があります。
    uint16_t maxEntry;                                                                   //!< ネットワークに接続できるステーションの最大数です。@ref StationMaxEntry 以下である必要があります。
    uint32_t localCommunicationId;                                                       //!< ローカル通信 ID です。 @ref LocalFacade::CreateLocalCommunicationId で生成した値を指定する必要があります。
    char wirelessCryptoKey[UdsWirelessCryptoKeyLengthMax];                                             //!< 無線レイヤの暗号化に使用する暗号鍵です。サイズが @ref UdsWirelessCryptoKeyLengthMin 以上、@ref UdsWirelessCryptoKeyLengthMax 以下である必要があります。
    uint8_t wirelessCryptoKeyLength;                                                            //!< 無線レイヤの暗号化に使用する暗号鍵のサイズです。@ref UdsWirelessCryptoKeyLengthMin 以上、@ref UdsWirelessCryptoKeyLengthMax 以下である必要があります。
    uint8_t channel;                                                                     //!< 通信に使用するチャンネルです。0(自動), 1,6,11ch のいずれかを指定する必要があります。製品実機で実行した場合は常にチャンネルを自動で選択します。
    uint8_t applicationData[UdsBeaconApplicationDataSizeMax + UdsBeaconSystemDataSize];  //!< ビーコンにセットしたい任意データです。サイズが @ref UdsBeaconApplicationDataSizeMax 以下である必要があります。
    uint32_t applicationDataSize;                                                        //!< ビーコンにセットしたい任意データのサイズです。@ref UdsBeaconApplicationDataSizeMax 以下である必要があります。
};
//! @endcond

/*!
  @cond PRIVATE
  @brief  UDS ライブラリを利用したネットワークについての情報を保持するクラスです。
 */
class UdsNetworkDescription : public LocalNetworkDescription
{
public:
    /*!
    @brief  デフォルトコンストラクタ。
     */
    UdsNetworkDescription()
        : LocalNetworkDescription()
    {
    }

    virtual ~UdsNetworkDescription()
    {
    }

    /*!
    @brief      ネットワークに参加中のステーション数を取得します。

    @details    取得できるステーション数はネットワークスキャン時のものです。

    @return     ネットワークに参加中のステーション数です。
     */
    virtual uint16_t GetCurrentParticipantNum() const
    {
        return m_NetworkDescription.GetNowEntry();
    }

    /*!
    @brief      ネットワークに参加可能な最大ステーション数を取得します。

    @return     ネットワークに参加可能な最大ステーション数です。
     */
    virtual uint16_t GetMaxParticipantNum() const
    {
        return m_NetworkDescription.GetMaxEntry();
    }

    /*!
    @brief      参加が許可されているネットワークであるかを取得します。

    @details    取得できる状態はネットワークスキャン時のものです。

    @return     参加が許可されているネットワークであるかを返します。

    @see        LocalFacade::DisallowParticipating, LocalFacade::AllowParticipating
     */
    virtual bool IsOpened() const
    {
        return m_NetworkDescription.CanConnectAsClient();
    }

    /*!
    @cond PRIVATE
     */
    uint32_t GetLocalCommunicationId() const
    {
        return m_NetworkDescription.GetLocalCommunicationId();
    }
    //! @endcond

    /*!
    @brief      通信モード識別用 ID を取得します。

    @return     通信モード識別用 ID を返します。
     */
    uint8_t GetSubId() const
    {
        return m_NetworkDescription.GetSubId();
    }

    /*!
    @brief      通信に使用しているチャンネルを取得します。

    @details    通常参照する必要はありません。

    @return     通信に使用しているチャンネルを返します。
     */
    virtual uint16_t GetChannel() const
    {
        return m_NetworkDescription.GetChannel();
    }

    /*!
    @brief      BSSID（ホストの MAC アドレス）を取得します。

    @details    BSSID のサイズは nn::uds::MAC_ADDRESS_SIZE バイトです。引数には、このサイズのバッファを設定する必要があります。

    @param[out] pBssidBuffer  BSSID を格納するためのバッファです。サイズが nn::uds::MAC_ADDRESS_SIZE バイトである必要があります。
     */
    virtual void GetBssid(uint8_t* pBssidBuffer) const
    {
        if (!PIA_IS_VALID_POINTER(pBssidBuffer))
        {
            PIA_TRACE_EX(common::TraceFlagLocal, "Invalid Argument");
            return;
        }
        memcpy(pBssidBuffer, m_NetworkDescription.GetBssid(), nn::uds::MAC_ADDRESS_SIZE);
    }

    /*!
    @cond PRIVATE
     */
    virtual uint32_t GetGameMode() const
    {
        return static_cast<uint32_t>(GetSubId());
    }
    //! @endcond

    /*!
    @cond PRIVATE
     */
    const nn::uds::NetworkDescription& GetNetworkDescription() const
    {
        return m_NetworkDescription;
    }
    //! @endcond

    /*!
    @cond PRIVATE
     */
    nn::uds::NetworkDescription& GetNetworkDescription()
    {
        return m_NetworkDescription;
    }
    //! @endcond

    /*!
    @cond PRIVATE
     */
    void SetNetworkDescription(const nn::uds::NetworkDescription& networkDescription)
    {
        m_NetworkDescription = networkDescription;
    }
    //! @endcond

    // n2908: NetworkDescription 実体コピー
    virtual void Copy(const LocalNetworkDescription* description)
    {
        const UdsNetworkDescription* pUdsDesc = static_cast<const UdsNetworkDescription*>(description);
        m_NetworkDescription = pUdsDesc->m_NetworkDescription;
    }

private:
    nn::uds::NetworkDescription m_NetworkDescription;  //!< ネットワーク情報です。
};
//! @endcond

/*!
  @cond PRIVATE
  @brief  UDS ライブラリを利用したネットワークへ接続時の設定用クラスです。
 */
class UdsConnectNetworkSetting : public LocalConnectNetworkSetting
{
public:
    UdsConnectNetworkSetting()
        : LocalConnectNetworkSetting(),
          wirelessCryptoKeyLength(0)
    {
        memset(wirelessCryptoKey, 0, sizeof(wirelessCryptoKey));
    }

    char wirelessCryptoKey[UdsWirelessCryptoKeyLengthMax];  //!< 無線レイヤの暗号化に使用する暗号鍵です。サイズが @ref UdsWirelessCryptoKeyLengthMin 以上、@ref UdsWirelessCryptoKeyLengthMax 以下である必要があります。
    uint8_t wirelessCryptoKeyLength;                 //!< 無線レイヤの暗号化に使用する暗号鍵のサイズです。@ref UdsWirelessCryptoKeyLengthMin 以上、@ref UdsWirelessCryptoKeyLengthMax 以下である必要があります。
};
//! @endcond


/*!
  @cond PRIVATE
  @brief  UDS ライブラリを利用したネットワーク探索時の設定用クラスです。
 */
class UdsScanNetworkSetting : public LocalScanNetworkSetting
{
public:
    UdsScanNetworkSetting()
        : LocalScanNetworkSetting(),
          subId(0),
          localCommunicationId(0)
    {
    }

    uint8_t subId;                  //!< 通信モード識別用 ID です。アプリが自由に設定可能です。0xff を指定するとあらゆる ID を探索します。
    uint32_t localCommunicationId;  //!< ローカル通信 ID です。 @ref LocalFacade::CreateLocalCommunicationId で生成した値を指定する必要があります。
};
//! @endcond

/*!
@cond PRIVATE
以下はライブラリ内部向けの宣言です。
*/

class UdsBeaconSystemData : public LocalBeaconSystemData
{
public:
    UdsBeaconSystemData()
        : LocalBeaconSystemData()
    {
    }
};

class UdsBeaconSystemDataHostMigration : public LocalBeaconSystemData
{
public:
    UdsBeaconSystemDataHostMigration()
        : networkId(0),
          applicationVersion(0)
    {
        memset(macAddress, 0, sizeof(macAddress));
        memset(reservedUdsHostMigration, 0, sizeof(reservedUdsHostMigration));
    }

    uint32_t networkId;                             //!< ホストが決めたネットワーク ID。ホストマイグレーション時に接続すべきネットワークかを判断するために使用する。
    uint8_t applicationVersion;                     //!< アプリが設定したアプリケーションバージョン
    uint8_t macAddress[nn::uds::MAC_ADDRESS_SIZE];  //!< ホストマイグレーション後に接続するネットワーク判別用
    uint8_t reservedUdsHostMigration[5];
};

/*!
ここまでライブラリ内部向けの宣言です。
@endcond
*/


/*!
  @brief  UDS ライブラリを利用したネットワークに接続しているステーションについての情報を保持するクラスです。
 */
class UdsStationInfo : public LocalStationInfo
{
public:
    UdsStationInfo()
    {
        Clear();
    }
    virtual ~UdsStationInfo()
    {
    }
    virtual void Clear()
    {
        memset(scrambledLocalFriendCode.value, 0, sizeof(scrambledLocalFriendCode.value));
        memset(userName.userName, 0, sizeof(userName.userName));
        stationId = StationIdInvalid;
        SessionStationInfo::Clear();
    }

    nn::uds::ScrambledLocalFriendCode scrambledLocalFriendCode;  //!< ローカルフレンドコードに変換可能な情報です。
    nn::cfg::UserName userName;                                  //!< ユーザー名です。
    StationId stationId;                                         //!< 対応する StationId です。自身がセッション参加中には、有効なステーションに対して設定されます。ブラウズ時にはそのステーションが有効、無効に関わらず @ref nn::pia::StationIdInvalid が設定されます。
};
}
}
}  // end of namespace nn::pia::local
#endif
