﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#if NN_PIA_LOCAL_ENABLE_UDS

#include <nn/pia/local/local_LocalSessionInfo.h>
#include <nn/pia/local/local_UdsNetworkSetting.h>

namespace nn
{
namespace pia
{
namespace local
{

/*!
    @brief  ローカル通信のセッション情報クラスです。
*/
class UdsSessionInfo : public local::LocalSessionInfo
{
public:
    UdsSessionInfo();
    virtual ~UdsSessionInfo();

    /*!
    @brief      セッション情報をコピーします。
    @details    アプリケーションでセッション情報を保持したい際に使用します。
    @param[in]  rhs コピーするセッション情報
    */
    void Copy(const UdsSessionInfo& rhs);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    UdsSessionInfo& operator=(const UdsSessionInfo& rhs);

    /*!
    @brief コピーコンストラクタ。。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    UdsSessionInfo(const UdsSessionInfo& rhs);

    /*!
    @brief      セッション情報をクリアします。
    */
    virtual void Clear();

    /*!
    @brief      セッションのゲームモードを取得します。
    @details    通信モード識別用 ID を uint32_t にキャストして返します。
    @return     ゲームモードを返します。
    */
    virtual uint32_t GetGameMode() const;

    /*!
    @brief      セッションの ID を取得します。
    @details    同じセッション ID を持つ異なるセッションが存在する場合があります。
    @return     セッションの ID を返します。
    */
    virtual uint32_t GetSessionId() const;

    /*!
    @brief      セッションに参加している人数を取得します。
    @return     セッションに参加している人数を返します。
    */
    virtual uint16_t GetCurrentParticipantNum() const;

    /*!
    @brief      セッションに参加できる最大人数を取得します。
    @return     セッションに参加できる最大人数を返します。
    */
    virtual uint16_t GetMaxParticipantNum() const;

    /*!
    @brief              セッションのアプリケーション定義データを取得します。
    @details            アプリケーションが設定したビーコンデータを取得します。
    @param[out] pBuffer セッションのアプリケーション定義データのコピー用バッファのポインタ。
    @param[in]  size    セッションのアプリケーション定義データのコピー用バッファのサイズ。
    @return             正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultInvalidState      データを取得できる状態ではありません。[:progErr]
    */
    virtual Result GetApplicationData(void* pBuffer, uint32_t size) const;

    /*!
    @brief      セッションのアプリケーション定義データの長さを取得します。
    @return     セッションのアプリケーション定義データの長さを返します。
    */
    virtual uint32_t GetApplicationDataSize() const;


    /*!
    @brief      セッションが参加募集受付状態かどうかを取得します。
    @return     セッションが参加募集受付状態なら true, 参加募集締切状態なら false を返します。
    */
    virtual bool IsOpened() const;

    /*!
    @if CTR_DOC
    @brief      セッションに参加中のステーション情報を取得します。
    @param[out] pSessionStationInfo     ステーション情報格納先のポインタ。
    @param[in]  sessionStationInfoListSize  ステーション情報リスト格納先のサイズ。参加中のステーション数以上である必要があります。
    @return     正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument           引数が不正です。[:progErr]
    @retval ResultBufferShortage            ステーション情報のサイズがステーション情報格納先のサイズよりも大きいため、コピーできません。[:progErr]
    @endif
    */
    virtual Result GetSessionStationInfo(LocalStationInfo* pSessionStationInfo, uint32_t sessionStationInfoListSize) const;

    /*!
    @if CTR_DOC
    @brief      BSSID（ホストの MAC アドレス）を取得します。
    @details    BSSID のサイズは nn::uds::MAC_ADDRESS_SIZE バイトです。引数には、このサイズのバッファを設定する必要があります。
    @param[out] pBssidBuffer                BSSID を格納するためのバッファ。サイズが nn::uds::MAC_ADDRESS_SIZE バイトである必要があります。
    @param[in]  size                        BSSID を格納するためのバッファサイズ
    @return     正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument           引数が不正です。[:progErr]
    @endif
    */
    virtual Result GetBssid(uint8_t* pBssidBuffer, uint32_t size) const;

    /*!
    @brief デバッグに有用な情報をプリントします。
    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    //! @cond PRIVATE
    virtual const LocalNetworkDescription* GetLocalNetworkDescription() const;
    virtual void SetLocalNetworkDescription(uint32_t networkIndex, const LocalNetworkDescription* pDescription);

    virtual Result SetSessionStationInfo(uint32_t index);

    void SetCurrentPaticipantNum(uint16_t participantNum)
    {
        m_CurrentParticipantNum = participantNum;
    }
    void SetMaxParticipantNum(uint16_t maxParticipantNum)
    {
        m_MaxParticipantNum = maxParticipantNum;
    }
    void SetGameMode(uint32_t gameMode)
    {
        m_GameMode = gameMode;
    }
    void SetSessionId(uint32_t sessionId)
    {
        m_SessionId = sessionId;
    }
    void SetOpened(bool b)
    {
        m_IsOpened = b;
    }
    const uint8_t* GetApplicationDataBuffer() const
    {
        return m_ApplicationData;
    }
    uint8_t* GetApplicationDataBuffer()
    {
        return m_ApplicationData;
    }
    uint32_t GetApplicationDataBufferSize() const
    {
        return sizeof(m_ApplicationData);
    }
    uint8_t* GetBssIdBuffer()
    {
        return m_BssId;
    }
    void SetApplicationDataSize(uint32_t dataSize)
    {
        m_ApplicationDataSize = dataSize;
    }
    UdsStationInfo* GetUdsStationInfoArray()
    {
        return m_UdsStationInfoArray;
    }
    //! @endcond

private:
    UdsNetworkDescription m_UdsNetworkDescription;
    UdsStationInfo m_UdsStationInfoArray[StationMaxEntry];

    uint8_t m_BssId[nn::uds::MAC_ADDRESS_SIZE];
    uint8_t m_ApplicationData[local::UdsBeaconApplicationDataSizeMax];
    uint32_t m_ApplicationDataSize;
    uint16_t m_CurrentParticipantNum;
    uint16_t m_MaxParticipantNum;
    uint32_t m_GameMode;
    uint32_t m_SessionId;
    bool m_IsOpened;
};
}
}
}  // end of namespace nn::pia::local
#endif
