﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>

#if NN_PIA_LOCAL_ENABLE_UDS

#include <nn/pia/local/local_LocalUpdateSessionSetting.h>
#include <nn/pia/local/local_UdsNetworkSetting.h>

namespace nn
{
namespace pia
{
namespace local
{

/*!
  @brief   セッションの条件更新処理に関する設定を管理するクラスです。
 */

class UdsUpdateSessionSetting : public LocalUpdateSessionSetting
{
public:
    /*!
    @brief  デフォルトコンストラクタ。
     */
    UdsUpdateSessionSetting();

    /*!
    @brief    デストラクタ
     */
    virtual ~UdsUpdateSessionSetting();

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @details    inet::ApplicationDataBufferSizeMax を超えないサイズを指定する必要があります。
                更新したアプリケーションデータはホストマイグレーション後には引き継がれず、セッション参加時の値になります。
    @param[in]  pData   セッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    セッションに指定するアプリケーション定義データのサイズ。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size);

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      アプリケーション定義データのバッファへのポインタを取得します。
    @return     アプリケーション定義データのバッファへのポインタが返されます。
    */
    virtual const uint8_t* GetApplicationDataPtr() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      アプリケーション定義データのバッファサイズを取得します。
    @return     アプリケーション定義データのバッファサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSizeMax() const;
    //! @endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    UdsUpdateSessionSetting& operator=(const UdsUpdateSessionSetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const UdsUpdateSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    UdsUpdateSessionSetting(const UdsUpdateSessionSetting& rhs);

    /*!
    @brief 設定をリセットします。
    */
    virtual void Reset();

protected:
    uint8_t m_ApplicationDataBuffer[UdsBeaconApplicationDataSizeMax];
};
}
}
}  // end of namespace local

#endif
