﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>


// @addtogroup     pia_common_debug_macro             Pia の提供するデバッグ用マクロ定義
// @brief          Pia の提供するデバッグ用マクロです。
//! @{
//!     @name       アサート
//!     @{

/*!
    @brief  引数 EXP が false もしくは NULL の場合プログラムが強制的に停止します。

    @details このマクロを使った場合、直前まで @ref PIA_BASIC_PRINT で出力した文字列がキャッシュから書き出されます。
    <br>
    @details release ビルド時にはこのマクロは無視されます。

    @param[in] EXP この値が false もしくは NULL の場合プログラムが強制的に停止します。

    @see    PIA_BASIC_PRINT
 */
#if NN_PIA_ENABLE_ASSERT
#define PIA_ASSERT(EXP) \
    (static_cast<void>((EXP) || (::nn::pia::Assert(#EXP, PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE, NULL), true)))

#else
#define PIA_ASSERT(EXP) ((void)0)
#endif


/*!
    @brief  引数 EXP が false もしくは NULL の場合プログラムが強制的に停止します。

    @details このマクロを使った場合、直前まで @ref PIA_BASIC_PRINT で出力した文字列がキャッシュから書き出されます。
    <br>
    @details release ビルド時にはこのマクロは無視されます。

    @param[in] EXP この値が false もしくは NULL の場合プログラムが強制的に停止します。
    @param[in] ... 標準ライブラリの printf の引数と同様です。

    @see    PIA_BASIC_PRINT
 */
#if NN_PIA_ENABLE_ASSERT
#define PIA_ASSERTMSG(EXP, ...) \
    (static_cast<void>((EXP) || (::nn::pia::Assert(#EXP, PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE, __VA_ARGS__), true)))
#else
#define PIA_ASSERTMSG(EXP, ...) ((void)0)
#endif


//! @cond PRIVATE
#define PIA_MACRO_CAT(l, r) l##r
#define PIA_MACRO_EXPANDING_CAT(l, r) PIA_MACRO_CAT(l, r)
//! @endcond

/*!
    @brief  引数 EXP が false もしくは NULL の場合コンパイルエラーとなります。

    @details コンパイル時の（静的）アサートマクロです。
             引数 EXP には定数式を指定する必要があります。

    @param[in] EXP この値が false もしくは NULL の場合コンパイルエラーとなります。
 */
#if NN_PIA_NINTENDOSDK
#define PIA_COMPILE_ASSERT(EXP) static_assert((EXP), "Pia compile assert.")
#else
#define PIA_COMPILE_ASSERT(EXP)                  \
    typedef ::nn::pia::PIA_COMPILE_ASSERT_INNER< \
        sizeof(::nn::pia::PIA_COMPILE_ASSERT_FAILURE<static_cast<bool>(EXP)>)> \
PIA_MACRO_EXPANDING_CAT(pia_compile_assert_at_, __LINE__)
#endif

//!     @}


//!     @name       パニック
//!     @{

/*!
    @brief  プログラムを強制的に停止させます。

    @details このマクロを使った場合、直前まで @ref PIA_BASIC_PRINT で出力した文字列がキャッシュから書き出されます。
    <br>
    @param[in] ... 標準ライブラリの printf の引数と同様です。

    @see    PIA_BASIC_PRINT
 */
#if NN_PIA_ENABLE_ASSERT
#define PIA_PANIC(...) ::nn::pia::Panic(PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE, __VA_ARGS__)
#else
#define PIA_PANIC(...) PIA_BASIC_HALT(__VA_ARGS__)
#endif

//!     @}
//! @}


namespace nn
{
namespace pia
{

/*!
@cond PRIVATE
  @brief           アサート関数です。

  @details         この関数は、 PIA_ASSERT や PIA_ASSERTMSG などのマクロから
                   利用されることを想定しています。
                   この関数を直接呼び出すことは推奨しません。

  @param[in] pExp  条件式の文字列表現。
  @param[in] pPos  コード内の位置を示す文字列表現。( __FILE__ や __FUNCTION__ 、 __MODULE__ など）
  @param[in] line  行番号。
  @param[in] pMsg  アサート時に出力する追加メッセージ。
 */
void Assert(const char* pExp, const char* pPos, uint32_t line, const char* pMsg, ...);
//! @endcond


/*!
@cond PRIVATE
  @brief           パニック関数です。

  @details         この関数は、 PIA_PANIC などのマクロから
                   利用されることを想定しています。
                   この関数を直接呼び出すことは推奨しません。

  @param[in] pPos  コード内の位置を示す文字列表現。( __FILE__ や __FUNCTION__ 、 __MODULE__ など）
  @param[in] line  行番号。
  @param[in] pMsg  アサート時に出力する追加メッセージ。
 */
void Panic(const char* pPos, uint32_t line, const char* pMsg, ...);
//! @endcond


/*!
@cond PRIVATE
  @brief           コンパイルアサート用のテンプレートクラスです。

  @details         これらのテンプレートクラスは PIA_COMPILE_ASSERT などのマクロから
                   利用されることを想定しています。
                   これらのテンプレートクラスをその他の用途で利用することは推奨しません。
 */
template <bool b>
struct PIA_COMPILE_ASSERT_FAILURE;
template <>
struct PIA_COMPILE_ASSERT_FAILURE<true>
{
};
template <int x>
struct PIA_COMPILE_ASSERT_INNER
{
};
//! @endcond
}
}  // end of namespace nn::pia
