﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


//Piaにおいて、nn::Result型を利用するためのヘッダです。

#pragma once

#include <nn/pia/pia_Platform.h>
#include <nn/pia/pia_Types.h>


//! @cond PRIVATE
#define PIA_DEFINE_RESULT(name, handling, viewer, description) \
    typedef ::nn::pia::ResultConst<(handling), (viewer), (description)> name
//! @endcond


#if NN_PIA_NINTENDOSDK
namespace nn
{
namespace err
{
    struct ErrorCode;  // 先行宣言。
}
}
#endif


/*!
  @namespace  nn::pia
  @brief      pia の名前空間です。

  @details PiaのAPIの多くはスレッドセーフではありません。
  個々の API リファレンスにおいて、特にスレッドセーフ／アンセーフに
  ついて言及していないAPIは、基本的にスレッドセーフを保証していません。（スレッドセーフなAPIについての説明は随時追加していく予定です）

 */
namespace nn
{
namespace pia
{


/*!
@brief        処理の結果を示すクラスです。

@ref nn::pia 名前空間に所属する API は、@ref nn::pia::Result クラスの実体を返します。

@section section_valueMeanings 値の意味

Result は 32ビットの値を持ち、内部にエラーハンドリング方法、ビューア使用の可否、種類の3つの情報を含んでいます。
ただしこれらの情報は、将来において変更される可能性があります。

@section section_errorHandling エラーハンドリングの例

以下の点に注意してください。

@subsection section_errorHandling_processingFailures 処理の失敗を判定する

処理の結果はシステムのアップデートによって、アプリケーション開発の時点では定義されていなかった値を返す可能性があります。 このようなケースをカバーするためにも、処理の成否は必ず、 IsSuccess() もしくは IsFailure() で判定する必要があります。
成否を確認せずに一致判定のみを行うことは、想定外のエラーが発生した際に成功と判定してしまう危険性があるため推奨しません。


@attention リファレンスに記載されている Result 以外のものを直接ハンドリングしてはいけません。

@section section_unacceptableErrors 製品では発生しないようにするべきエラー

例えば ResultInvalidArgument, ResultInvalidState 等はアプリケーション側の不具合などにより発生する Result です。
これらの Result は、製品では発生しないように修正する必要があります。

上記以外の Result は、ユーザ操作によって発生するものや、開発中に完全に取り除くことが不可能なものになります。 それらのユーザの手元で発生し得るエラーは、必ずハンドリングを行う必要があります。
詳細は、各 API のページを参照してください。

@section section_handlingUnexpectedErrors 想定外のエラーが発生する場合の対応

処理の結果はシステムのアップデートによって、アプリケーション開発の時点では定義されていなかった値を返す可能性があります。
このため、想定外のエラーが発生する場合の対応コードはほとんどの場合において必要となります。


@section section_unrecoverableFatalErrors 継続不可能な致命的エラー

「致命的なエラー」とは、ハードウェア不具合や本来製品で発生し得ないエラーなどシステムが何をしても復帰できず、かつハンドリング不可能なエラーのことを指します。

このようなエラーが発生した場合は、原則アプリケーションにエラーが渡されることはなく、システムが停止してFATALエラー画面が表示されます。 このエラー表示に遭遇したユーザーはメッセージに従ってマニュアルを確認し、解決しない場合は最終的にユーザサポートに連絡することになります。

システム上エラー表示も不可能な場合はその場で停止します。

*/
class Result
{

protected:
    static const uint32_t ValueSuccess = 0;  // 0 のみが Success、それ以外は Failure であると規定します。

    static const int32_t SizeDescription = 10;
    static const int32_t SizeViewer = 3;
    static const int32_t SizeHandling = 5;

    static const int32_t ShiftsDescription = 0;
    static const int32_t ShiftsViewer = ShiftsDescription + SizeDescription;
    static const int32_t ShiftsHandling = ShiftsViewer + SizeViewer;

#define NN_RESULT_H_MAKE_MASK(size, shift) (((~0u) >> (32 - (size))) << (shift))
#define NN_RESULT_H_MAKE_MASK_HELPER(c) \
    static const uint32_t Mask##c = NN_RESULT_H_MAKE_MASK(Size##c, Shifts##c)
    NN_RESULT_H_MAKE_MASK_HELPER(Description);
    NN_RESULT_H_MAKE_MASK_HELPER(Viewer);
    NN_RESULT_H_MAKE_MASK_HELPER(Handling);
#undef NN_RESULT_H_MAKE_MASK_HELPER
#undef NN_RESULT_H_MAKE_MASK

#define NN_RESULT_H_MAKE_MAX(size) ((~0u) >> (32 - (size)))
#define NN_RESULT_H_MAKE_MAX_HELPER(c) \
    static const int32_t Max##c = NN_RESULT_H_MAKE_MAX(Size##c)
    NN_RESULT_H_MAKE_MAX_HELPER(Description);
    NN_RESULT_H_MAKE_MAX_HELPER(Viewer);
    NN_RESULT_H_MAKE_MAX_HELPER(Handling);
#undef NN_RESULT_H_MAKE_MAX_HELPER
#undef NN_RESULT_H_MAKE_MAX

public:
    /*!
      @brief 期待されるハンドリングの種別を表現するための列挙型です。
     */
    enum HandlingType
    {
        HandlingType_Void = 0,         //!< 無効な値です。
        HandlingType_Ignorable,        //!< 無視することが可能です。
        HandlingType_Retry,            //!< 少し時間をおいてから再実行してください。
        HandlingType_Cleanup,          //!< クリーンアップ処理を実行してください。@if NIN_DOC ローカル通信では Session::Cleanup() 呼び出しの後 nn::ldn::Finalize() も呼び出す必要があります。 @endif
        HandlingType_CleanupWithLeave, //!< セッション離脱処理、クリーンアップ処理を順に実行してください。@if NIN_DOC ローカル通信では Session::Cleanup() 呼び出しの後 nn::ldn::Finalize() も呼び出す必要があります。 @endif
        HandlingType_LogoutWithLeave,  //!< セッション離脱処理、クリーンアップ処理、ログアウト処理を順に実行してください。@if NIN_DOC ローカル通信では Session::Cleanup() 呼び出しの後 nn::ldn::Finalize() も呼び出す必要があります。 @endif
        HandlingType_ShutdownNetwork,  //!< クリーンアップ処理、ログアウト処理（インターネット通信時限定）、ネットワークのシャットダウン処理を順に実行してください。@if NIN_DOC ローカル通信では Session::Cleanup() 呼び出しの後 nn::ldn::Finalize() も呼び出す必要があります。 @endif
        HandlingType_Finalize,         //!< クリーンアップ処理、ログアウト処理（インターネット通信時限定）、ネットワークのシャットダウン処理、Pia の終了処理を順に実行してください。@if NIN_DOC ローカル通信では Session::Cleanup() 呼び出しの後 nn::ldn::Finalize() も呼び出す必要があります。 @endif
        HandlingType_ProgrammingError  //!< プログラミングエラーです。ソースコードを修正してください。
    };

    /*!
    @if CTR_DOC
      @brief     エラー・EULA アプレット使用の指針を表現するための列挙型です。
    @elseif CAFE_DOC
      @brief     エラービューア使用の指針を表現するための列挙型です。
    @elseif NIN_DOC
      @brief     エラービューアアプレット使用の指針を表現するための列挙型です。
    @endif
     */
    enum ViewerType
    {
        ViewerType_Void = 0,     //!< 無効な値です。
        ViewerType_ShouldUse,    //!< 使用すべきです。
        ViewerType_MayUse,       //!< 使用しても構いません。
        ViewerType_ShouldNotUse  //!< 使用すべきではありません。
    };

    /*!
     @brief Pia の汎用 Result を示す列挙型です。
     */
    enum Description
    {
        Description_Success = 0,                                                                        //!< 成功しました
        Description_AllocationFailed,                                                                   //!< メモリなどのリソースの確保に失敗したことを示します。
        Description_AlreadyInitialized,                                                                 //!< 既に初期化は完了しています。
        Description_BrokenData,                                                                         //!< 通信回線を経由してきたデータが壊れている、あるいは改ざんの疑いがあることを示すエラーです。
        Description_BufferShortage,                                                                     //!< 引数で渡されたバッファのサイズが不足しています。
        Description_Cancelled,                                                                          //!< 非同期処理のキャンセルが要求されました。
        Description_NetworkConnectionIsLost,                                                            //!< 接続が切れたことを表します。
        Description_InvalidArgument,                                                                    //!< APIに渡した引数が誤っています。
        Description_InvalidState,                                                                       //!< APIを呼び出すタイミングが誤っています。ライブラリの状態が、このResultを返したAPIを呼び出せる状態ではなかったことを示します。
        Description_NoData,                                                                             //!< データが存在しません。
        Description_NotFound,                                                                           //!< 要求されたものが見つかりませんでした。
        Description_NotImplemented,                                                                     //!< 未実装であることを示します。
        Description_NotInitialized,                                                                     //!< 初期化されていません。
        Description_BufferIsFull,                                                                       //!< 処理を実行するのに必要なバッファが一時的に不足していたために失敗しました。ネットワークトラフィックの混雑が原因の場合もあります。時間をあけてから再度実行すれば、成功する可能性があります。
        Description_TimeOut,                                                                            //!< 非同期処理がタイムアウトしました。
        Description_AlreadyExists,                                                                      //!< 対象は既に存在しています。
        Description_ContainerIsFull,                                                                    //!< コンテナは既に満杯です。
        Description_TemporaryUnavailable,                                                               //!< 一時的に利用できない状態です。
        Description_NotSet = Description_TemporaryUnavailable + 2,                                      //!< 事前に設定されているべき値が設定されていません。
        Description_MemoryLeak,                                                                         //!< PIAライブラリ内部でのメモリリーク疑いです。
        Description_InvalidNode = Description_MemoryLeak + 2,                                           //!< 対象のノードがネットワークに存在しません。
        Description_NegligibleFault = Description_InvalidNode + 2,                                      //!< 無視可能な失敗です。
        Description_InvalidConnection,                                                                  //!< ネットワークに接続されていません。
        Description_ErrorOccurred = Description_InvalidConnection + 2,                                  //!< エラーが発生しました。
        Description_NetworkIsNotFound,                                                                  //!< 接続できないネットワークです。
        Description_NetworkIsFull,                                                                      //!< ネットワークが満員でした。
        Description_LocalLowerVersion,                                                                  //!< 自分のバージョンが通信相手のバージョンよりも低いです。
        Description_LocalHigherVersion,                                                                 //!< 自分のバージョンが通信相手のバージョンよりも高いです。
        Description_WifiOff,                                                                            //!< 無線がオフになっています。
        Description_Sleep,                                                                              //!< スリープ状態です。
        Description_WirelessControllerCountLimitation,                                                  //!< 無線コントローラの接続台数が制限を超えています。
        Description_ConnectionFailed,                                                                   //!< @cond PRIVATE 接続に失敗しました。@endcond
        Description_CreateStationFailed,                                                                //!< @cond PRIVATE Station 作成に失敗しました。@endcond
        Description_VersionMismatched,                                                                  //!< 通信方式バージョン番号が異なります。
        Description_IsNotInCommunication,                                                               //!< 通信中ではありません。
        Description_TableIsFull,                                                                        //!< @cond PRIVATE テーブルは既に満杯のため、これ以上エントリを追加することはできません。@endcond
        Description_SyncDataIsNotArrivedYet = 44,                                                       //!< 全 Station のデータが揃っていません。
        Description_SyncDataIsNotSet = 50,                                                              //!< 送信データが設定されていません。
        Description_NatCheckFailed = 52,                                                                //!< NAT チェックに失敗しました。
        Description_InUsed,                                                                             //!< @cond PRIVATE すでに使用中のため使用できません。@endcond
        Description_DnsFailed,                                                                          //!< NAT チェック前に行う DNS 名前解決に失敗しました。
        Description_NexInternalError,                                                                   //!< ゲームサーバが想定外のリザルトコードを返したことを示します。
        Description_RequestDenied,                                                                      //!< 要求が拒否されました。
        Description_StationConnectionFailed,                                                            //!< Station 間の接続に失敗しました。
        Description_MeshIsFull = Description_StationConnectionFailed + 4,                               //!< メッシュが満員でした。
        Description_InvalidMessage,                                                                     //!< 不正なシステムメッセージを処理しました。
        Description_NatTraversalFailedUnknown,                                                          //!< Station 間の接続に失敗しました。NAT タイプは不明でした。
        Description_NatTraversalFailedLocalEimRemoteEim = Description_NatTraversalFailedUnknown + 3,    //!< Station 間の接続に失敗しました。Local, Remote 共に NAT タイプは EIM でした。
        Description_NatTraversalFailedLocalEimRemoteEdm,                                                //!< Station 間の接続に失敗しました。Local の NAT タイプは EIM, Remote の NAT タイプは EDM でした。
        Description_NatTraversalFailedLocalEdmRemoteEim,                                                //!< Station 間の接続に失敗しました。Local の NAT タイプは EDM, Remote の NAT タイプは EIM でした。
        Description_NatTraversalFailedLocalEdmRemoteEdm,                                                //!< Station 間の接続に失敗しました。Local, Remote 共に NAT タイプは EDM でした。
        Description_RelayFailedNoCandidate,                                                             //!< リレー接続に失敗しました。（リレー候補がいなかったため）
        Description_RelayFailedRttLimit,                                                                //!< リレー接続に失敗しました。（ RTT 制限を超えたため）
        Description_RelayFailedRelayNumLimit,                                                           //!< リレー接続に失敗しました。（リレー数制限を超えたため）
        Description_RelayFailedUnknown,                                                                 //!< リレー接続に失敗しました。（詳細不明）
        Description_NatTraversalRequestTimeout,                                                         //!< Station 間の接続に失敗しました。NAT トラバーサル要求がタイムアウトしました。
        Description_SessionIsNotFound,                                                                  //!< 存在しないセッションでした。
        Description_MatchmakeSessionIsFull,                                                             //!< マッチメイクセッションが満員でした。
        Description_DeniedByParticipants,                                                               //!< 自分をブラックリストに登録しているユーザーが参加していました。
        Description_ParticipantInBlocklist,                                                             //!< ブロックリストに登録されているユーザーが参加していました。
        Description_GameServerMaintenance,                                                              //!< サーバーメンテナンス中です。
        Description_MeshConnectionIsLost,                                                               //!< @cond PRIVATE メッシュが切断状態です。@endcond
        Description_SessionUserPasswordUnmatch,                                                         //!< ユーザーパスワードがセットされているセッションに参加しようとしましたがユーザーパスワードが一致しませんでした。
        Description_SessionSystemPasswordUnmatch,                                                       //!< システムパスワードがセットされているセッションに参加しようとしましたがシステムパスワードが一致しませんでした。
        Description_SessionIsClosed,                                                                    //!< 参加しようとしたセッションが締め切られていました。
        Description_CompanionStationIsOffline,                                                          //!< ジョイントセッション時に同行者として参加させようとしたステーションがサーバーにログインしていませんでした。
        Description_HostIsNotFriend,                                                                    //!< 参加しようとしたセッションのホストとフレンド関係がありませんでした。
        Description_SessionConnectionIsLost,                                                            //!< セッションが切断状態です。
        Description_CompanionStationIsLeft,                                                             //!< ジョイントセッション時に同行者として指定したステーションが切断していました。
        Description_SessionMigrationFailed = Description_CompanionStationIsLeft + 2,                    //!< ジョイントセッション時にセッション移行に必要な処理に失敗しました。
        Description_GameServerProcessAborted,                                                           //!< ゲームサーバーへの処理が異常終了しました。
        Description_SessionWrongState,                                                                  //!< 参加したセッションの状態が異常でした。
        Description_CreateCommunityFailedUpperLimit = Description_SessionWrongState + 4,                //!< コミュニティの作成上限を超えました。
        Description_JoinCommunityFailedUpperLimit,                                                      //!< コミュニティの参加上限を超えました。
        Description_CommunityIsFull,                                                                    //!< 参加しようとしたコミュニティが満員でした。
        Description_CommunityIsNotFound,                                                                //!< 存在しないコミュニティでした。
        Description_CommunityIsClosed,                                                                  //!< 参加しようとしたコミュニティが参加可能期間外でした。
        Description_AlreadyJoinedCommunity,                                                             //!< 参加しようとしたコミュニティに既に参加していた。
        Description_UserAccountNotExisted,                                                              //!< ユーザーアカウントが存在しませんでした。
        Description_CommunityUserPasswordUnmatch,                                                       //!< 参加時に指定したユーザーパスワードがコミュニティに設定されたユーザーパスワードと一致しませんでした。
        Description_NatTraversalFailedLocalEimRemoteEimSamePublicAddress,                               //!< Station 間の接続に失敗しました。Local, Remote 共に NAT タイプは EIM でした。また、Local と Remote は同じグローバルIPアドレスでした。
        Description_NatTraversalFailedLocalEimRemoteEdmSamePublicAddress,                               //!< Station 間の接続に失敗しました。Local の NAT タイプは EIM, Remote の NAT タイプは EDM でした。また、Local と Remote は同じグローバルIPアドレスでした。
        Description_NatTraversalFailedLocalEdmRemoteEimSamePublicAddress,                               //!< Station 間の接続に失敗しました。Local の NAT タイプは EDM, Remote の NAT タイプは EIM でした。また、Local と Remote は同じグローバルIPアドレスでした。
        Description_NatTraversalFailedLocalEdmRemoteEdmSamePublicAddress,                               //!< Station 間の接続に失敗しました。Local, Remote 共に NAT タイプは EDM でした。また、Local と Remote は同じグローバルIPアドレスでした。
        Description_SdkError,                                                                           //!< SDK の API 呼び出しに失敗しました。
        Description_SdkViewerErrorCodeError,                                                            //!< SDK の API 呼び出しに失敗しました。nn::pia::Result::GetErrorCode() で取得できるエラーコードは SDK のものになります。
        Description_CancelledByUser,                                                                    //!< ユーザー操作に従って処理を中断しました。
        Description_SdkViewerResultError,                                                               //!< SDK の API 呼び出しに失敗しました。nn::pia::framework::Framework::GetLastSdkResult() で nn::Result を取得し、エラービューアアプレットを使用して適切にハンドリングして下さい。
        Description_LanLowerVersion,                                                                    //!< 自分のバージョンが通信相手のバージョンよりも低いです。
        Description_LanHigherVersion,                                                                   //!< 自分のバージョンが通信相手のバージョンよりも高いです。
        Description_DuplicateLogin,                                                                     //!< @if NIN_DOC 他の本体から同じアカウントを使ってログインされたため、サーバーから切断されました。@endif
        Description_OnlinePlayLicenseNotAvailable,                                                      //!< @if NIN_DOC ネットワークサービスの利用権限がありません。@endif
        Description_OnlinePlayLicenseError                                                              //!< @if NIN_DOC ネットワークサービスが利用できません。@endif
    };

#if NN_PIA_CTR
    static const uint32_t PiaErrorCodeBase = 180000;  // CTR-Piaには 018-xxxx が割り当てられている。
#elif NN_PIA_WIN
    static const uint32_t PiaErrorCodeBase = 180000;  // 数値はCTRに合わせているが、Windows環境でエラー表示アプレットが用いられることはあるだろうか？
#elif NN_PIA_CAFE
    static const uint32_t PiaErrorCodeBase = 1180000;  // Cafe-Piaには 118-xxxx が割り当てられている。
#elif NN_PIA_NINTENDOSDK
    static const uint32_t PiaErrorCodeBase = 26180000;  // NX-Pia には 2618-xxxx が割り当てられている。
#elif NN_PIA_A
    static const uint32_t PiaErrorCodeBase = 26180000;
#elif NN_PIA_B
    static const uint32_t PiaErrorCodeBase = 26180000;
#else
#error "invalid platform"
#endif

    /*!
    @cond PRIVATE
    @brief       ゲームサーバとの通信で発生したエラーのネットワークエラーコードをセットします。

    @param[in] code NEX の qResult をネットワークエラーコードに変換した値をセットします。
    @endcond
    */
#if NN_PIA_NINTENDOSDK
    static void SetNexNetworkErrorCode(const nn::err::ErrorCode& code);
#else
    static void SetNexNetworkErrorCode(uint32_t code);
#endif

    /*!
    @cond PRIVATE
    @brief       SetNexNetworkErrorCode() でセットされたネットワークエラーコードをクリアします。
    @endcond
     */
    static void ClearNexNetworkErrorCode(void);


    /*!
    @cond PRIVATE
    @brief       SDK の API 呼び出し失敗によるエラーコードをセットします。

    @param[in] code SDK のエラーコードです。
    @endcond
    */
    static void SetSdkErrorCode(uint32_t code)
    {
        s_SdkErrorCode = code;
    }


    /*!
    @cond PRIVATE
    @brief       SetSdkErrorCode() でセットされたエラーコードをクリアします。
    @endcond
    */
    static void ClearSdkErrorCode(void)
    {
        s_SdkErrorCode = 0;
    }

private:
    uint32_t m_Code;
#if NN_PIA_NINTENDOSDK
    static nn::err::ErrorCode s_NexNetworkErrorCode;
#else
    static uint32_t s_NexNetworkErrorCode;
#endif
    static uint32_t s_SdkErrorCode;

protected:
    explicit Result(uint32_t code)
        : m_Code(code)
    {
    }

public:
    Result()
        : m_Code(ValueSuccess)
    {
    }

    Result(const Result& r)
        : m_Code(r.m_Code)
    {
    }

    Result(HandlingType handling, ViewerType viewer, Description description)
        : m_Code(static_cast<uint32_t>(
              ((static_cast<uint32_t>(handling) << ShiftsHandling) & MaskHandling) |
              ((static_cast<uint32_t>(viewer) << ShiftsViewer) & MaskViewer) |
              ((static_cast<uint32_t>(description) << ShiftsDescription) & MaskDescription)))
    {
    }

    /*!
        @name     判定
        @{
    */
    /*!
     * @brief ! IsSuccess() を返します。
     */
    bool IsFailure() const
    {
        return !IsSuccess();
    }

    /*!
     * @brief 処理の結果が成功なら true を、失敗なら false を返します。
     */
    bool IsSuccess() const
    {
        return m_Code == ValueSuccess;
    }
    /*!
        @}
    */

    /*!
     * @brief エラーの詳細を示す Description 列挙体を返します。
     *
     * 列挙体の定義は将来変更されます。開発時のエラー内容確認の用途以外では利用してはいけません。
     */
    enum Description GetDescription() const
    {
        return static_cast<enum Description>(GetCodeBits(MaskDescription, ShiftsDescription));
    }

    uint32_t GetValue() const
    {
        return m_Code;
    }

    /*!
        @name     判定
        @{
    */
    /*!
     * @brief 一致判定を行います。
     *
     * @param [in] rhs  演算子の右オペランドです。
     * @return  一致した場合に true を、一致しない場合に false を返します。
     */
    bool operator==(const Result& rhs) const
    {
        return this->m_Code == rhs.m_Code;
    }

    /*!
     * @brief 不一致判定を行います。
     *
     * @param [in] rhs  演算子の右オペランドです。
     * @return  一致しない場合に true を、一致する場合に false を返します。
     */
    bool operator!=(const Result& rhs) const
    {
        return this->m_Code != rhs.m_Code;
    }
    /*!
        @}
    */


    /*!
      @brief  Pia の Result に対して期待されるハンドリングの種別を返します。

      @return Result に応じたハンドリングの列挙値が返されます。
              Pia 以外の Result や、Pia 内部でのみ使用される Result を
              渡した場合は HandlingType_Void が返されます。
     */
    enum HandlingType GetHandlingType() const
    {
        return static_cast<enum HandlingType>(GetCodeBits(MaskHandling, ShiftsHandling));
    }


    /*!
    @if CTR_DOC
      @brief     エラー・EULA アプレットを使用しても良い Pia の Result であるかどうかを判定します。
      @details   Pia API 呼び出しで失敗の Result が返された場合に、その Result から
                 GetErrorCode で取得したエラーコードを
                 エラー・EULA アプレット
                 に渡すのが適切かどうかを判定する API です。
      @details   ただし、ローカル通信時に返された Result については、この API の返り値にかかわらず、
                 エラー・EULA アプレットを使用してはいけません。
    @endif
    @if CAFE_DOC
      @brief     エラービューアを使用しても良い Pia の Result であるかどうかを判定します。
      @details   Pia API 呼び出しで失敗の Result が返された場合に、その Result から
                 GetErrorCode で取得したエラーコードを
                 エラービューア
                 に渡すのが適切かどうかを判定する API です。
    @endif
    @if NIN_DOC
      @brief     エラービューアアプレットを使用しても良い Pia の Result であるかどうかを判定します。
      @details   Pia API 呼び出しで失敗の Result が返された場合に、その Result から
                 GetErrorCode で取得したエラーコードを
                 エラービューアアプレット
                 に渡すのが適切かどうかを判定する API です。
    @endif

      @return    渡された Result に応じた ViewerType が返されます。
                 Pia 以外の Result が渡された場合は ViewerType_Void が返されます。
     */
    enum ViewerType GetViewerType() const
    {
        return static_cast<enum ViewerType>(GetCodeBits(MaskViewer, ShiftsViewer));
    }

#if !NN_PIA_NINTENDOSDK
    /*!
    @brief       Result に対応するエラーコードを得ます。

    @details     Pia ライブラリが返した Result に対応するエラーコードを得ます。
    @details     例外的に、 ResultNexInternalError に対して本 API を呼び出すと、この Result の
                 原因となった NEX の内部エラーに対応するエラーコードが返されます。
                 この内部エラーには複数の種類があるため、 ResultNexInternalError に対応する
                 返り値が一定しないことがあります。
    @if CTR_DOC
                 得られた値は、エラー・EULAアプレットに渡すことができます。
    @endif
    @if CAFE_DOC
                 得られた値は、エラービューアに渡すことができます。
    @endif

                 IsSuccess() が true となる Result に対しては 0 を、Pia ライブラリで定義されていない
                 Result に本 API を適用した場合はアサート停止します（リリースビルドでは PiaErrorCodeBase を返します）。
                 ResultNexInternalError を除いて、本関数は Pia ライブラリで定義されている Result を一意に識別するコードに変換しますが、
                 全てのコードをユーザーにエラーとして表示する必要はありません。
                 エラーコード表示については、
    @if CTR_DOC
                 CTR ガイドライン インターネット通信編 や、
                 ネットワークエラーコードリスト などのドキュメントも参照してください。
    @endif
    @if CAFE_DOC
                 Wii U ガイドラインにおけるエラー表示の章なども参照してください。
    @endif

    @return      エラーコードが返されます。
     */
#endif
    uint32_t GetErrorCode() const;


#if NN_PIA_NINTENDOSDK
    /*!
    @brief       Result に対応するエラーコードを得ます。

    @details     Pia ライブラリが返した Result に対応するエラーコードを得ます。
    @details     例外的に、 ResultNexInternalError に対して本 API を呼び出すと、この Result の
                 原因となった NEX の内部エラーに対応するエラーコードが返されます。
                 この内部エラーには複数の種類があるため、 ResultNexInternalError に対応する
                 返り値が一定しないことがあります。

                 得られた値は、エラービューアに渡すことができます。

                 IsSuccess() が true となる Result に対しては、pOut が指すインスタンスにゼロクリアされた値がセットされます。
                 Pia ライブラリで定義されていない Result に本 API を適用した場合はアサート停止します。
                 ResultNexInternalError を除いて、本関数は Pia ライブラリで定義されている Result を一意に識別するコードに変換しますが、
                 全てのコードをユーザーにエラーとして表示する必要はありません。

    @param[out]  pOut nn::err::ErrorCode 型のインスタンスを指すポインタを渡します。

    @return      成功すれば、IsSuccess() が true となる Result が返されます。

    @retval      ResultInvalidArgument 引数に誤りがあります（ヌルポインタであるなど）[:progErr]
     */
    Result GetErrorCode(nn::err::ErrorCode* pOut) const;
#endif


    /*!
      @brief  Pia の Result の文字列表現を返します。

      @return Pia の Result の文字列表現が返されます。
     */
    const char* ToString() const;


private:
    uint32_t GetCodeBits(uint32_t mask, int32_t shift) const
    {
        return ((m_Code & mask) >> shift);
    }
};


/*!
@cond PRIVATE
@brief  全ての Result のアサート停止の有効/無効を管理します。
 */
struct AssertableResultAll
{
public:
    /*!
    @brief アサート停止を有効にします。
     */
    static void EnableAssert()
    {
        s_IsAssertEnabled = true;
    }

    /*!
    @brief アサート停止を無効にします。
     */
    static void DisableAssert()
    {
        s_IsAssertEnabled = false;
    }

    /*!
    @brief アサート停止が有効がどうかを取得します。
    @return アサート停止が有効なら true 無効なら false を返します。
     */
    static bool IsAssertEnabled()
    {
        return s_IsAssertEnabled;
    }

private:
    static bool s_IsAssertEnabled;
};
//! @endcond


/*!
@cond PRIVATE
@brief 定数型の Result 。ResultSuccess や、いわゆるエラーの Result はこちらのテンプレート型から生成される。
       アサート停止機能もこのクラスに引っ越しています。
 */
template <Result::HandlingType THandling, Result::ViewerType TViewer, Result::Description TDescription>
struct ResultConst : public Result
{
    static const uint32_t Value =
        static_cast<uint32_t>(
            ((static_cast<uint32_t>(THandling) << ShiftsHandling) & MaskHandling) |
            ((static_cast<uint32_t>(TViewer) << ShiftsViewer) & MaskViewer) |
            ((static_cast<uint32_t>(TDescription) << ShiftsDescription) & MaskDescription));

    ResultConst()
        : Result(Value)
    {
    }

    /*!
    @brief アサート停止を有効にします。
     */
    static void EnableAssert()
    {
        s_IsAssertEnabled = true;
    }

    /*!
    @brief アサート停止を無効にします。
     */
    static void DisableAssert()
    {
        s_IsAssertEnabled = false;
    }

    /*!
    @brief アサート停止が有効がどうかを取得します。
    @return アサート停止が有効なら true 無効なら false を返します。
     */
    static bool IsAssertEnabled()
    {
        return (s_IsAssertEnabled || AssertableResultAll::IsAssertEnabled());
    }

private:
    static bool s_IsAssertEnabled;
};
//! @endcond

//! @cond PRIVATE
template <Result::HandlingType THandling, Result::ViewerType TViewer, Result::Description TDescription>
bool ResultConst<THandling, TViewer, TDescription>::s_IsAssertEnabled = false;
//! @endcond


/*!
  @class nn::pia::ResultAllocationFailed
  @brief メモリなどのリソースの確保に失敗したことを示すリザルトコードです。この Result が返らないようにアプリケーションを実装する必要があります。
*/
PIA_DEFINE_RESULT(
    ResultAllocationFailed,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_AllocationFailed);

/*!
  @class nn::pia::ResultAlreadyInitialized
  @brief 既に初期化が完了していたことを示すリザルトコードです。この Result が返らないようにアプリケーションを実装する必要があります。
*/
PIA_DEFINE_RESULT(
    ResultAlreadyInitialized,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_AlreadyInitialized);

/*!
  @class nn::pia::ResultBufferShortage
  @brief 引数で渡されたバッファのサイズが不足していることを示すリザルトコードです。この Result が返らないようにアプリケーションを実装する必要があります。
*/
PIA_DEFINE_RESULT(
    ResultBufferShortage,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_BufferShortage);

/*!
  @class nn::pia::ResultBrokenData
  @brief 通信回線を経由してきたデータが壊れている、あるいは改ざんの疑いがあることを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultBrokenData,
    Result::HandlingType_Ignorable,
    Result::ViewerType_ShouldNotUse,
    Result::Description_BrokenData);

/*!
  @class nn::pia::ResultCancelled
  @brief 非同期処理のキャンセルが要求されたことを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultCancelled,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_Cancelled);

/*!
  @class nn::pia::ResultNetworkConnectionIsLost
  @brief 接続が既に切断されたことを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultNetworkConnectionIsLost,
    Result::HandlingType_ShutdownNetwork,
    Result::ViewerType_ShouldUse,
    Result::Description_NetworkConnectionIsLost);

/*!
  @class nn::pia::ResultInvalidArgument
  @brief API に渡した引数が誤っていることを示すリザルトコードです。例えば NULL ポインタを渡された等のケースが該当します。この Result が返らないようにアプリケーションを実装する必要があります。
*/
PIA_DEFINE_RESULT(
    ResultInvalidArgument,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_InvalidArgument);


/*!
  @class nn::pia::ResultInvalidState
  @brief API を呼び出すタイミングが誤っていることを示すリザルトコードです。 Pia ライブラリの状態が、この Result を返したAPIを呼び出せる状態ではなかったことを示します。
*/
PIA_DEFINE_RESULT(
    ResultInvalidState,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_InvalidState);

/*!
  @class nn::pia::ResultNoData
  @brief データが存在しない事を示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultNoData,
    Result::HandlingType_Ignorable,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NoData);

/*!
  @class nn::pia::ResultNotFound
  @brief 要求されたものが見つからなかった事を示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultNotFound,
    Result::HandlingType_Ignorable,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NotFound);

/*!
  @class nn::pia::ResultNotImplemented
  @brief 機能が未実装であることを示すリザルトコードです。この Result が返らないようにアプリケーションを実装する必要があります。
*/
PIA_DEFINE_RESULT(
    ResultNotImplemented,
    Result::HandlingType_Void,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NotImplemented);


/*!
  @class nn::pia::ResultNotInitialized
  @brief 関数を呼びだしたオブジェクトやモジュールが未初期化であった事を示すリザルトコードです。この Result が返らないようにアプリケーションを実装する必要があります。
*/
PIA_DEFINE_RESULT(
    ResultNotInitialized,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NotInitialized);

/*!
  @class nn::pia::ResultBufferIsFull
  @brief 処理を実行するのに必要なバッファが一時的に不足していたために失敗したことを示すリザルトコードです。ネットワークトラフィックの混雑によって引き起こされることもあります。時間をあけてから再度実行すれば、成功する可能性があります。
*/
PIA_DEFINE_RESULT(
    ResultBufferIsFull,
    Result::HandlingType_Retry,
    Result::ViewerType_ShouldNotUse,
    Result::Description_BufferIsFull);

/*!
  @class nn::pia::ResultTimeOut
  @brief 非同期処理がタイムアウトしたことを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultTimeOut,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_TimeOut);

/*!
  @class nn::pia::ResultAlreadyExists
  @brief 対象は既に存在していたことを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultAlreadyExists,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_AlreadyExists);

/*!
  @class nn::pia::ResultContainerIsFull
  @brief コンテナは満杯であることを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultContainerIsFull,
    Result::HandlingType_Void,
    Result::ViewerType_ShouldNotUse,
    Result::Description_ContainerIsFull);

/*!
  @class nn::pia::ResultTemporaryUnavailable
  @brief API が一時的に利用できない状態であることを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultTemporaryUnavailable,
    Result::HandlingType_Retry,
    Result::ViewerType_ShouldNotUse,
    Result::Description_TemporaryUnavailable);

/*!
  @class nn::pia::ResultNotSet
  @brief 事前に設定されているべき値が設定されていなかったことを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultNotSet,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NotSet);


/*!
  @class nn::pia::ResultMemoryLeak
  @brief Pia ライブラリ内部のメモリリーク疑いを表現するリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultMemoryLeak,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_MemoryLeak);

/*!
  @cond PRIVATE
  @class nn::pia::ResultInvalidNode
  @brief 【システム用エラー】送信先のノードがネットワーク上に見つからなかったことを示します。
*/
PIA_DEFINE_RESULT(
    ResultInvalidNode,
    Result::HandlingType_Void,
    Result::ViewerType_ShouldNotUse,
    Result::Description_InvalidNode);
//! @endcond

/*!
  @class nn::pia::ResultNegligibleFault
  @brief 無視可能なエラーを表します。
*/
PIA_DEFINE_RESULT(
    ResultNegligibleFault,
    Result::HandlingType_Ignorable,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NegligibleFault);

/*!
  @class nn::pia::ResultInvalidConnection
  @if NIN_DOC
  @brief 接続状態が不正であることを表します。
  @else
  @brief 接続状態が不正であることや、無線オフモードになっていることを表します。
  @endif
*/
PIA_DEFINE_RESULT(
    ResultInvalidConnection,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_InvalidConnection);

/*!
  @class nn::pia::ResultErrorOccurred
  @brief 内部でエラーが発生したことを表します。
*/
PIA_DEFINE_RESULT(
    ResultErrorOccurred,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_ErrorOccurred);

/*!
  @cond PRIVATE
  @class nn::pia::ResultNetworkIsNotFound
  @brief 【システム用エラー】接続できないネットワークであることを表します。
*/
PIA_DEFINE_RESULT(
    ResultNetworkIsNotFound,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NetworkIsNotFound);
//! @endcond

/*!
  @class nn::pia::ResultNetworkIsFull
  @brief ネットワークが満員でした。
*/
PIA_DEFINE_RESULT(
    ResultNetworkIsFull,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_NetworkIsFull);

/*!
  @class nn::pia::ResultLocalLowerVersion
  @brief 自分のバージョンが通信相手のバージョンよりも低いことを表します。
*/
PIA_DEFINE_RESULT(
    ResultLocalLowerVersion,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_LocalLowerVersion);

/*!
  @class nn::pia::ResultLocalHigherVersion
  @brief 自分のバージョンが通信相手のバージョンよりも高いことを表します。
*/
PIA_DEFINE_RESULT(
    ResultLocalHigherVersion,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_LocalHigherVersion);

/*!
  @class nn::pia::ResultWifiOff
  @brief 無線がオフになっていることを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultWifiOff,
    Result::HandlingType_ShutdownNetwork,
    Result::ViewerType_ShouldUse,
    Result::Description_WifiOff);

/*!
  @class nn::pia::ResultSleep
  @brief スリープ状態であることを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultSleep,
    Result::HandlingType_ShutdownNetwork,
    Result::ViewerType_ShouldUse,
    Result::Description_Sleep);

/*!
@class nn::pia::ResultWirelessControllerCountLimitation
@brief 無線コントローラの接続台数が制限を超えていることを示すリザルトコードです。
*/
PIA_DEFINE_RESULT(
    ResultWirelessControllerCountLimitation,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_WirelessControllerCountLimitation);

/*!
  @cond PRIVATE
  @class nn::pia::ResultConnectionFailed
  @brief 【システム用エラー】何らかの理由で接続処理に失敗しました。
        この Result がアプリに伝達されることはありません。
*/
PIA_DEFINE_RESULT(
    ResultConnectionFailed,
    Result::HandlingType_Void,
    Result::ViewerType_ShouldNotUse,
    Result::Description_ConnectionFailed);
//! @endcond

/*!
  @cond PRIVATE
  @class nn::pia::ResultCreateStationFailed
  @brief 【システム用エラー】新しい Station を作成できませんでした。
        この Result がアプリに伝達されることはありません。
*/
PIA_DEFINE_RESULT(
    ResultCreateStationFailed,
    Result::HandlingType_Void,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CreateStationFailed);
//! @endcond

/*!
  @class nn::pia::ResultIncompatibleFormat
  @brief 非互換の通信形式です。
*/
PIA_DEFINE_RESULT(
    ResultIncompatibleFormat,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_VersionMismatched);

/*!
  @class nn::pia::ResultNotInCommunication
  @brief 通信中ではありません。
*/
PIA_DEFINE_RESULT(
    ResultNotInCommunication,
    Result::HandlingType_Ignorable,
    Result::ViewerType_ShouldNotUse,
    Result::Description_IsNotInCommunication);

/*!
  @cond PRIVATE
  @class nn::pia::ResultTableIsFull
  @brief 【システム用エラー】テーブルは既に満杯のため、これ以上エントリを追加することはできません。
        この Result がアプリに伝達されることはありません。
*/
PIA_DEFINE_RESULT(
    ResultTableIsFull,
    Result::HandlingType_Void,
    Result::ViewerType_ShouldNotUse,
    Result::Description_TableIsFull);
//! @endcond

/*!
  @class nn::pia::ResultDataIsNotArrivedYet
  @brief 全 Station のデータが揃っていません
           
  @details このエラーは通信状態の悪化などの理由で、データの同期に問題が生じた際に発生します。<br>
           このエラーを返した場合、アプリケーションはゲームの進行を止めて次のピクチャフレームを待つ必要があります。<br>
           全ての Station が同じタイミングで、同じ回数このエラーに遭遇する訳ではないことに注意してください。
*/
PIA_DEFINE_RESULT(
    ResultDataIsNotArrivedYet,
    Result::HandlingType_Retry,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SyncDataIsNotArrivedYet);


/*!
  @class nn::pia::ResultDataIsNotSet
  @brief 送信データが設定されていません。
*/
PIA_DEFINE_RESULT(
    ResultDataIsNotSet,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SyncDataIsNotSet);


/*!
  @class nn::pia::ResultNatCheckFailed
  @brief NAT チェックに失敗しました。
*/
PIA_DEFINE_RESULT(
    ResultNatCheckFailed,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatCheckFailed);

/*!
  @cond PRIVATE
  @class nn::pia::ResultInUse
  @brief すでに使用中のため使用できません。
*/
PIA_DEFINE_RESULT(
    ResultInUse,
    Result::HandlingType_Void,
    Result::ViewerType_ShouldNotUse,
    Result::Description_InUsed);
//!@endcond


/*!
  @class nn::pia::ResultDnsFailed
  @brief DNS 解決に失敗しました。
*/
PIA_DEFINE_RESULT(
    ResultDnsFailed,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_DnsFailed);


/*!
  @class nn::pia::ResultNexInternalError
  @brief NEX 内部エラーが発生しました。
*/
PIA_DEFINE_RESULT(
    ResultNexInternalError,
    Result::HandlingType_Finalize,
    Result::ViewerType_ShouldUse,
    Result::Description_NexInternalError);


/*!
  @class nn::pia::ResultJoinRequestDenied
  @brief セッションホストによって Join 要求が拒否されました。
*/
PIA_DEFINE_RESULT(
    ResultJoinRequestDenied,
    Result::HandlingType_Cleanup,
    Result::ViewerType_MayUse,
    Result::Description_RequestDenied);


/*!
  @class nn::pia::ResultStationConnectionFailed
  @brief Station 間の接続処理に失敗しました。
*/
PIA_DEFINE_RESULT(
    ResultStationConnectionFailed,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_StationConnectionFailed);


/*!
  @class nn::pia::ResultMeshIsFull
  @brief Join 要求を出したメッシュが満員で、参加できませんでした。
*/
PIA_DEFINE_RESULT(
    ResultMeshIsFull,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_MeshIsFull);


/*!
  @class nn::pia::ResultInvalidSystemMessage
  @brief Join 要求への応答に不正なメッセージが返されました。
*/
PIA_DEFINE_RESULT(
    ResultInvalidSystemMessage,
    Result::HandlingType_ProgrammingError,
    Result::ViewerType_ShouldNotUse,
    Result::Description_InvalidMessage);

/*!
  @class nn::pia::ResultStationConnectionNatTraversalFailedUnknown
  @brief Station 間の NAT トラバーサル処理に失敗しました。NAT タイプは不明でした。
*/
PIA_DEFINE_RESULT(
    ResultStationConnectionNatTraversalFailedUnknown,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedUnknown);

/*!
  @class nn::pia::ResultNatTraversalFailedBothEim
  @brief Station 間の NAT トラバーサル処理に失敗しました。Local, Remote 共に NAT タイプは EIM でした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedBothEim,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEimRemoteEim);

/*!
  @class nn::pia::ResultNatTraversalFailedLocalEimRemoteEdm
  @brief Station 間の NAT トラバーサル処理に失敗しました。Local の NAT タイプは EIM, Remote の NAT タイプは EDM でした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedLocalEimRemoteEdm,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEimRemoteEdm);

/*!
  @class nn::pia::ResultNatTraversalFailedLocalEdmRemoteEim
  @brief Station 間の NAT トラバーサル処理に失敗しました。Local の NAT タイプは EDM, Remote の NAT タイプは EIM でした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedLocalEdmRemoteEim,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEdmRemoteEim);

/*!
  @class nn::pia::ResultNatTraversalFailedBothEdm
  @brief Station 間の NAT トラバーサル処理に失敗しました。Local, Remote 共に NAT タイプは EDM でした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedBothEdm,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEdmRemoteEdm);


/*!
  @class nn::pia::ResultRelayFailedNoCandidate
  @brief リレー接続に失敗しました。（リレー候補がいなかったため）
*/
PIA_DEFINE_RESULT(
    ResultRelayFailedNoCandidate,
    Result::HandlingType_Cleanup,
    Result::ViewerType_MayUse,
    Result::Description_RelayFailedNoCandidate);

/*!
  @class nn::pia::ResultRelayFailedRttLimit
  @brief リレー接続に失敗しました。（ RTT 制限を超えたため）
*/
PIA_DEFINE_RESULT(
    ResultRelayFailedRttLimit,
    Result::HandlingType_Cleanup,
    Result::ViewerType_MayUse,
    Result::Description_RelayFailedRttLimit);
/*!
  @class nn::pia::ResultRelayFailedRelayNumLimit
  @brief リレー接続に失敗しました。（リレー数制限を超えたため）
*/
PIA_DEFINE_RESULT(
    ResultRelayFailedRelayNumLimit,
    Result::HandlingType_Cleanup,
    Result::ViewerType_MayUse,
    Result::Description_RelayFailedRelayNumLimit);
/*!
  @class nn::pia::ResultRelayFailedUnknown
  @brief リレー接続に失敗しました。（詳細不明）
*/
PIA_DEFINE_RESULT(
    ResultRelayFailedUnknown,
    Result::HandlingType_Cleanup,
    Result::ViewerType_MayUse,
    Result::Description_RelayFailedUnknown);

/*!
  @class nn::pia::ResultNatTraversalRequestTimeout
  @brief Station 間の NAT トラバーサル処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalRequestTimeout,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalRequestTimeout);

/*!
  @class nn::pia::ResultSessionIsNotFound
  @brief 既に存在していないセッションになっていました。
*/
PIA_DEFINE_RESULT(
    ResultSessionIsNotFound,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SessionIsNotFound);

/*!
  @class nn::pia::ResultMatchmakeSessionIsFull
  @brief 満員のマッチメイクセッションに参加しようとしました。
*/
PIA_DEFINE_RESULT(
    ResultMatchmakeSessionIsFull,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_MatchmakeSessionIsFull);

/*!
  @class nn::pia::ResultDeniedByParticipants
  @brief 自分をブラックリストに登録しているユーザーが参加していました。
*/
PIA_DEFINE_RESULT(
    ResultDeniedByParticipants,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_DeniedByParticipants);

/*!
  @class nn::pia::ResultParticipantInBlockList
  @brief ブロックリストに登録しているユーザーが参加していました。
*/
PIA_DEFINE_RESULT(
    ResultParticipantInBlockList,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_ParticipantInBlocklist);

/*!
  @class nn::pia::ResultGameServerMaintenance
  @brief ゲームサーバーがメンテナンス中です。
*/
PIA_DEFINE_RESULT(
    ResultGameServerMaintenance,
    Result::HandlingType_LogoutWithLeave,
    Result::ViewerType_ShouldUse,
    Result::Description_GameServerMaintenance);

/*!
  @class nn::pia::ResultSessionUserPasswordUnmatch
  @brief ユーザーパスワードがセットされているセッションに参加しようとしましたがユーザーパスワードが一致しませんでした。
*/
PIA_DEFINE_RESULT(
    ResultSessionUserPasswordUnmatch,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SessionUserPasswordUnmatch);

/*!
  @class nn::pia::ResultSessionSystemPasswordUnmatch
  @brief システムパスワードがセットされているセッションに参加しようとしましたがシステムパスワードが一致しませんでした。
*/
PIA_DEFINE_RESULT(
    ResultSessionSystemPasswordUnmatch,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SessionSystemPasswordUnmatch);

/*!
  @cond PRIVATE
  @class nn::pia::ResultMeshConnectionIsLost
  @brief 【システム用エラー】メッシュが切断状態になりました。
*/
PIA_DEFINE_RESULT(
    ResultMeshConnectionIsLost,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_MeshConnectionIsLost);
//! @endcond

/*!
  @class nn::pia::ResultSessionIsClosed
  @brief 参加しようとしたセッションが締め切られていました。
*/
PIA_DEFINE_RESULT(
    ResultSessionIsClosed,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SessionIsClosed);

/*!
  @class nn::pia::ResultCompanionStationIsOffline
  @brief （ジョイントセッション用）追加で参加させようとしたステーションがサーバーにログインしていませんでした。
*/
PIA_DEFINE_RESULT(
    ResultCompanionStationIsOffline,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CompanionStationIsOffline);

/*!
  @class nn::pia::ResultHostIsNotFriend
  @brief 参加しようとしたセッションのホストとフレンド関係がありませんでした。 
*/
PIA_DEFINE_RESULT(
    ResultHostIsNotFriend,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_HostIsNotFriend);


/*!
  @class nn::pia::ResultSessionConnectionIsLost
  @brief セッションが切断されました。
*/
PIA_DEFINE_RESULT(
    ResultSessionConnectionIsLost,
    Result::HandlingType_CleanupWithLeave,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SessionConnectionIsLost);

/*!
  @class nn::pia::ResultCompanionStationIsLeft
  @brief ジョイントセッション時に同行者として指定したステーションが切断していました。
*/
PIA_DEFINE_RESULT(
    ResultCompanionStationIsLeft,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CompanionStationIsLeft);

/*!
  @class nn::pia::ResultSessionMigrationFailed
  @brief ジョイントセッション時にセッション移行に必要な処理に失敗しました。
*/
PIA_DEFINE_RESULT(
    ResultSessionMigrationFailed,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SessionMigrationFailed);

/*!
  @class nn::pia::ResultGameServerProcessAborted
  @brief ゲームサーバーへの処理が異常終了しました。
*/
PIA_DEFINE_RESULT(
    ResultGameServerProcessAborted,
    Result::HandlingType_LogoutWithLeave,
    Result::ViewerType_ShouldNotUse,
    Result::Description_GameServerProcessAborted);

/*!
  @class nn::pia::ResultSessionWrongState
  @brief 参加したセッションの状態が異常でした。
*/
PIA_DEFINE_RESULT(
    ResultSessionWrongState,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SessionWrongState);

/*!
  @class nn::pia::ResultCreateCommunityFailedUpperLimit
  @brief コミュニティの作成上限を超えました。
*/
PIA_DEFINE_RESULT(
    ResultCreateCommunityFailedUpperLimit,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CreateCommunityFailedUpperLimit);

/*!
  @class nn::pia::ResultJoinCommunityFailedUpperLimit
  @brief コミュニティの参加上限を超えました。
*/
PIA_DEFINE_RESULT(
    ResultJoinCommunityFailedUpperLimit,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_JoinCommunityFailedUpperLimit);

/*!
  @class nn::pia::ResultCommunityIsFull
  @brief 参加しようとしたコミュニティが満員でした。
*/
PIA_DEFINE_RESULT(
    ResultCommunityIsFull,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CommunityIsFull);

/*!
  @class nn::pia::ResultCommunityIsNotFound
  @brief 存在しないコミュニティでした。
*/
PIA_DEFINE_RESULT(
    ResultCommunityIsNotFound,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CommunityIsNotFound);

/*!
  @class nn::pia::ResultCommunityIsClosed
  @brief 参加しようとしたコミュニティが参加可能期間外でした。
*/
PIA_DEFINE_RESULT(
    ResultCommunityIsClosed,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CommunityIsClosed);

/*!
@class nn::pia::ResultCommunityUserPasswordUnmatch
@brief 参加時に指定したユーザーパスワードがコミュニティに設定されたユーザーパスワードと一致しませんでした。
*/
PIA_DEFINE_RESULT(
    ResultCommunityUserPasswordUnmatch,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CommunityUserPasswordUnmatch);

/*!
  @class nn::pia::ResultAlreadyJoinedCommunity
  @brief 参加しようとしたコミュニティに既に参加していた。
*/
PIA_DEFINE_RESULT(
    ResultAlreadyJoinedCommunity,
    Result::HandlingType_Ignorable,
    Result::ViewerType_ShouldNotUse,
    Result::Description_AlreadyJoinedCommunity);

/*!
@class nn::pia::ResultUserAccountNotExisted
@brief ユーザーアカウントが存在しませんでした。
*/
PIA_DEFINE_RESULT(
    ResultUserAccountNotExisted,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldNotUse,
    Result::Description_UserAccountNotExisted);

/*!
  @class nn::pia::ResultNatTraversalFailedBothEimSamePublicAddress
  @brief Station 間の NAT トラバーサル処理に失敗しました。Local, Remote 共に NAT タイプは EIM でした。また、Local と Remote は同じグローバルIPアドレスでした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedBothEimSamePublicAddress,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEimRemoteEimSamePublicAddress);

/*!
@class nn::pia::ResultNatTraversalFailedBothEdmSamePublicAddress
@brief Station 間の NAT トラバーサル処理に失敗しました。Local, Remote 共に NAT タイプは EDM でした。また、Local と Remote は同じグローバルIPアドレスでした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedBothEdmSamePublicAddress,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEdmRemoteEdmSamePublicAddress);

/*!
  @class nn::pia::ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress
  @brief Station 間の NAT トラバーサル処理に失敗しました。Local の NAT タイプは EIM, Remote の NAT タイプは EDM でした。また、Local と Remote は同じグローバルIPアドレスでした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEimRemoteEdmSamePublicAddress);

/*!
  @class nn::pia::ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress
  @brief Station 間の NAT トラバーサル処理に失敗しました。Local の NAT タイプは EDM, Remote の NAT タイプは EIM でした。また、Local と Remote は同じグローバルIPアドレスでした。
*/
PIA_DEFINE_RESULT(
    ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_NatTraversalFailedLocalEdmRemoteEimSamePublicAddress);

/*!
  @class nn::pia::ResultSdkError
  @brief SDK の API 呼び出しに失敗しました。
*/
PIA_DEFINE_RESULT(
    ResultSdkError,
    Result::HandlingType_Finalize,
    Result::ViewerType_ShouldNotUse,
    Result::Description_SdkError);

/*!
  @class nn::pia::ResultSdkViewerErrorCodeError
  @brief SDK の API 呼び出しに失敗しました。nn::pia::Result::GetErrorCode() で取得できるエラーコードは SDK のものになります。
*/
PIA_DEFINE_RESULT(
    ResultSdkViewerErrorCodeError,
    Result::HandlingType_Finalize,
    Result::ViewerType_ShouldUse,
    Result::Description_SdkViewerErrorCodeError);

/*!
@if NIN_DOC
@class nn::pia::ResultSdkViewerResultError
@brief SDK の API 呼び出しに失敗しました。nn::pia::framework::Framework::GetLastSdkResult() で SDK の Result を取得し、エラービューアアプレットを使用して適切にハンドリングして下さい。
@endif
*/
PIA_DEFINE_RESULT(
    ResultSdkViewerResultError,
    Result::HandlingType_LogoutWithLeave,
    Result::ViewerType_ShouldUse,
    Result::Description_SdkViewerResultError);

/*!
@if NIN_DOC
@class nn::pia::ResultCancelledByUser
@brief ユーザー操作に従って処理を中断しました。
@endif
*/
PIA_DEFINE_RESULT(
    ResultCancelledByUser,
    Result::HandlingType_LogoutWithLeave,
    Result::ViewerType_ShouldNotUse,
    Result::Description_CancelledByUser);

/*!
@class nn::pia::ResultLanLowerVersion
@brief 自分のバージョンが通信相手のバージョンよりも低いことを表します。
*/
PIA_DEFINE_RESULT(
    ResultLanLowerVersion,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_LanLowerVersion);

/*!
@class nn::pia::ResultLanHigherVersion
@brief 自分のバージョンが通信相手のバージョンよりも高いことを表します。
*/
PIA_DEFINE_RESULT(
    ResultLanHigherVersion,
    Result::HandlingType_Cleanup,
    Result::ViewerType_ShouldUse,
    Result::Description_LanHigherVersion);

/*!
@if NIN_DOC
@class nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin
@brief 他の本体から同じアカウントを使ってログインされたため、サーバーから切断されました。切断状態ですが、ログアウトを呼び出しても問題ありません。
@endif
*/
PIA_DEFINE_RESULT(
    ResultNetworkConnectionIsLostByDuplicateLogin,
    Result::HandlingType_LogoutWithLeave,
    Result::ViewerType_ShouldUse,
    Result::Description_DuplicateLogin);

/*!
@if NIN_DOC
@class nn::pia::ResultOnlinePlayLicenseNotAvailable
@brief ネットワークサービスの利用権限がありません。
@endif
*/
PIA_DEFINE_RESULT(
    ResultOnlinePlayLicenseNotAvailable,
    Result::HandlingType_LogoutWithLeave,
    Result::ViewerType_ShouldUse,
    Result::Description_OnlinePlayLicenseNotAvailable);

/*!
@if NIN_DOC
@class nn::pia::ResultOnlinePlayLicenseError
@brief ネットワークサービスを利用できません。
@endif
*/
PIA_DEFINE_RESULT(
    ResultOnlinePlayLicenseError,
    Result::HandlingType_LogoutWithLeave,
    Result::ViewerType_ShouldUse,
    Result::Description_OnlinePlayLicenseError);


/*!
  @class nn::pia::ResultSuccess
  @brief 成功の Result です。
*/
PIA_DEFINE_RESULT(
    ResultSuccess,
    Result::HandlingType_Void,
    Result::ViewerType_Void,
    Result::Description_Success);

#define PIA_RETURN_RESULT(result)                                             \
    {                                                                         \
        PIA_ASSERTMSG(result::IsAssertEnabled() == false, "Result:" #result); \
        return result();                                                      \
    }
}
}  // end of namespace nn::pia
