﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/reckoning/reckoning_Definitions.h>
#include <nn/pia/reckoning/reckoning_ReckoningCloneElementBase.h>
#include <nn/pia/reckoning/reckoning_VoidMemberConcealer.h>
#include <nn/pia/clone/clone_SerializePolicyDefinition.h>
#include <nn/pia/clone/clone_CloneProtocol.h>

#include <limits.h>

namespace nn
{
namespace pia
{
namespace reckoning
{

/*!
    @brief  推測航法を用いながらの値の送受信を管理します。

    @tparam ReckoningStrategy_ 推測アルゴリズムを定義するクラスの型です。

    @details    設定された値の一部を標本値として一定数保持・送信し、その標本値から現在の値を推測することにより、
                設定された値をすべて送信することなく値を伝えます。
    
                ReckoningStrategy に指定するクラスには、以下の定義がされている必要があります。

                型

                @li @code Value @endcode
                    SetValue(), GetValue() で設定／取得する値の型です。
                @li @code Sample @endcode
                    標本値の型です。この型で実際に送受信されます。
                @li @code SetValueArg @endcode
                    SetValue() 時に指定して ReckoningStrategy::CheckSample(), ReckoningStrategy::MakeSample() に渡される値の型です。
                    必要が無ければ void を指定する必要があります。
                @li @code Work @endcode
                    推測の計算のための事前計算結果を保持しておく型です。必要が無ければ void を指定する必要があります。

                定数

                @li @code static const uint32_t BufferSize @endcode
                    標本値を保持しておく個数です。

                関数

                @li @code bool Estimate(
                        Value* pValue, 
                        const nn::pia::reckoning::ReckoningCloneElementBase::SampleAccessor<Sample>& accessor, 
                        nn::pia::clone::ClockValue clock, 
                        Work* pWork); @endcode
                    推測値を計算します。
                    推測値が計算できた場合は、その値を *pValue に設定して true を返す必要があります。
                    計算できなかった場合は、 false を返す必要があります。
                @li @code bool CheckSample(
                        bool* pIsReliable, 
                        const Value& value, 
                        const SetValueArg* cpSetValueArg, 
                        const nn::pia::reckoning::ReckoningCloneElementBase::SampleAccessor<Sample>& accessor, 
                        nn::pia::clone::ClockValue clock, 
                        Work* pWork); @endcode
                    SetValue() で設定した値 value を標本値として採用するかどうかを判定します。
                    標本値として採用する場合は true を返す必要があります。
                    標本値の送信にあたり到達保証をする必要がある場合は *pIsReliable に true を設する必要があります。
                @li @code void MakeSample(
                        Sample* pSample, 
                        const Value& value, 
                        const SetValueArg* cpSetValueArg, 
                        const nn::pia::reckoning::ReckoningCloneElementBase::SampleAccessor<Sample>& accessor, 
                        nn::pia::clone::ClockValue clock, 
                        Work* pWork); @endcode
                    標本値として保存する値を計算します。
                    CheckSample() で標本値として採用するよう判定され、実際にバッファが確保できた場合に呼び出されます。
                    *pSample に標本値を設定する必要があります。
                @li @code void OnUpdateSample(
                        const nn::pia::reckoning::ReckoningCloneElementBase::SampleAccessor<Sample>& accessor, 
                        int index, 
                        Work* pWork); @endcode
                    標本値が追加された場合に呼び出されます。
                    必要に応じて pWork の値を更新する必要があります。

                詳細はプログラミングマニュアルを参照してください。
 */
template <
    typename ReckoningStrategy_>
class ReckoningCloneElement : public ReckoningCloneElementBase
{
public:
    /*!
    @brief  推測アルゴリズムの型です。
     */
    typedef ReckoningStrategy_ ReckoningStrategy;


    /*!
    @brief  このオブジェクトが管理する値の型です。
     */
    typedef typename ReckoningStrategy::Value Value;


    /*!
    @brief  標本値の型です。
     */
    typedef typename ReckoningStrategy::Sample Sample;


    /*!
    @brief  SetValue() から、ReckoningStrategy::CheckSample()、ReckoningStrategy::MakeSample() に伝える値の型です。
     */
    typedef typename ReckoningStrategy::SetValueArg SetValueArg;


    /*!
    @brief  ワークバッファの型です。
     */
    typedef typename ReckoningStrategy::Work Work;


    /*!
    @brief  Sample をシリアライズするためのアルゴリズムです。
     */
    typedef clone::SerializePolicy<typename ReckoningStrategy_::Sample> ValueSerializePolicy;


    /*!
    @brief  標本値を保持しておくバッファのサイズです。
     */
    static const uint32_t BufferSize = ReckoningStrategy::BufferSize;


    /*!
    @brief コンストラクタです。
    @param[in] pReckoningStrategy   推測アルゴリズムのインスタンスを指定します。
                                    ここでの指定は省略できますが、 CloneBase::RegisterElement() に渡すまでには 
                                    SetReckoningStrategy() で有効なインスタンスを設定しておく必要があります。
     */
    explicit ReckoningCloneElement(ReckoningStrategy* pReckoningStrategy = NULL);


    /*!
    @brief デストラクタです。
     */
    virtual ~ReckoningCloneElement()
    {
    }


    /*!
    @brief  推測アルゴリズムのインスタンスを指定します。
    @details    @ref clone::CloneBase "CloneBase" に登録されている状態では設定できません。
    @param[in]  pReckoningStrategy 指定する推測アルゴリズムです。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState  CloneBase に登録されています。[:progErr]
     */
    Result SetReckoningStrategy(ReckoningStrategy* pReckoningStrategy);


    /*!
    @brief  設定されている推測アルゴリズムのインスタンスをリセットします。
    @details    @ref clone::CloneBase "CloneBase" に登録されている状態ではリセットできません。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState  CloneBase に登録されています。[:progErr]
     */
    Result ResetReckoningStrategy()
    {
        return SetReckoningStrategy(NULL);
    }


    /*!
    @brief  値を設定します。
    @param[in]  value   設定する値です。
    @param[in]  cpSetValueArg   ReckoningStrategy::CheckSample()、ReckoningStrategy::MakeSample() に伝える値です。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval ResultInvalidState  登録している CloneBase が送信可能状態ではありません。[:progErr]
     */
    Result SetValue(const Value& value, const SetValueArg* cpSetValueArg = NULL);


    /*!
    @brief  値を取得します。
    @return このオブジェクトが管理する値です。 @ref IsValidValue() が false の場合に取得できる値は不定です。
     */
    const Value& GetValue()
    {
        UpdateValue();
        return *m_ValueWork.GetT1Ptr();
    }


    //! @cond PRIVATE

public:
    virtual bool IsInitialized() const
    {
        return PIA_IS_VALID_POINTER(m_pReckoningStrategy);
    }

protected:
    virtual uint32_t GetSize() const
    {
        return ValueSerializePolicy::GetSerializedSize();
    }
    virtual void Serialize(void* pBuffer, const ReckoningSendToken* cpToken);
    virtual void ClearValue()
    {
        m_ValueWork = VoidMemberConcealer<Value, Work>();
    }
    virtual bool EstimateValue(clone::ClockValue clock);

    virtual void Deserialize(ReckoningSendToken* pToken, const void* cpData);
    virtual void OnUpdateSample(int idx);

private:
    ReckoningSendToken* m_apToken[BufferSize];
    SampleReckoningSendToken<Sample> m_aSampleBuffer[BufferSize];

    ReckoningStrategy* m_pReckoningStrategy;

    VoidMemberConcealer<Value, Work> m_ValueWork;

    NN_PIA_DISALLOW_COPY(ReckoningCloneElement);
    //! @endcond
};


//! @cond

template <typename ReckoningStrategy>
ReckoningCloneElement<ReckoningStrategy>::ReckoningCloneElement(ReckoningStrategy* pReckoningStrategy)
    : ReckoningCloneElementBase(m_apToken, BufferSize),
      m_pReckoningStrategy(pReckoningStrategy),
      m_ValueWork()
{
    PIA_COMPILE_ASSERT(BufferSize > 0 && BufferSize <= ULONG_MAX);

    for (uint32_t i = 0; i < static_cast<uint32_t>(BufferSize); ++i)
    {
        m_aSampleBuffer[i].Init(this);
        m_apToken[i] = &m_aSampleBuffer[i];
    }
}


template <typename ReckoningStrategy>
Result ReckoningCloneElement<ReckoningStrategy>::SetReckoningStrategy(ReckoningStrategy* pReckoningStrategy)
{
    if (IsRegisteredWithCloneBase())
    {

        PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidState, "State must not be (IsRegisteredWithCloneBase() == true).");
    }
    m_pReckoningStrategy = pReckoningStrategy;
    return ResultSuccess();
}


template <typename ReckoningStrategy>
Result ReckoningCloneElement<ReckoningStrategy>::SetValue(const Value& value, const SetValueArg* cpSetValueArg)
{
    bool isSend;
    Result r = SetValueCore(&isSend);

    if (isSend)
    {
        const clone::ClockValue clock = GetProtocol()->GetClock();
        const SampleAccessor<Sample> accessor(GetSampleBuffer());

        bool isReliable = false;
        bool isSample = m_pReckoningStrategy->CheckSample(&isReliable, value, cpSetValueArg, accessor, clock, m_ValueWork.GetT2Ptr());
        if (isSample)
        {
            int idx;
            ReckoningSendToken* pToken = AssignTokenForSend(&idx, isReliable);
            PIA_ASSERT(pToken != NULL && idx == 0);

            SampleReckoningSendToken<Sample>* pSampleToken = static_cast<SampleReckoningSendToken<Sample>*>(pToken);
            m_pReckoningStrategy->MakeSample(pSampleToken->GetSamplePtr(), value, cpSetValueArg, accessor, clock, m_ValueWork.GetT2Ptr());
            m_pReckoningStrategy->OnUpdateSample(accessor, idx, m_ValueWork.GetT2Ptr());
        }
    }

    return r;
}


template <typename ReckoningStrategy>
void ReckoningCloneElement<ReckoningStrategy>::Serialize(void* pBuffer, const ReckoningSendToken* cpToken)
{
    const SampleReckoningSendToken<Sample>* cpSampleToken = static_cast<const SampleReckoningSendToken<Sample>*>(cpToken);
    ValueSerializePolicy::Serialize(pBuffer, cpSampleToken->GetSample());
}


template <typename ReckoningStrategy>
bool ReckoningCloneElement<ReckoningStrategy>::EstimateValue(clone::ClockValue clock)
{
    const SampleAccessor<Sample> accessor(GetSampleBuffer());
    return m_pReckoningStrategy->Estimate(m_ValueWork.GetT1Ptr(), accessor, clock, m_ValueWork.GetT2Ptr());
}


template <typename ReckoningStrategy>
void ReckoningCloneElement<ReckoningStrategy>::Deserialize(ReckoningSendToken* pToken, const void* cpData)
{
    SampleReckoningSendToken<Sample>* pSampleToken = static_cast<SampleReckoningSendToken<Sample>*>(pToken);
    ValueSerializePolicy::Deserialize(pSampleToken->GetSamplePtr(), cpData);
}


template <typename ReckoningStrategy>
void ReckoningCloneElement<ReckoningStrategy>::OnUpdateSample(int idx)
{
    const SampleAccessor<Sample> accessor(GetSampleBuffer());
    m_pReckoningStrategy->OnUpdateSample(accessor, idx, m_ValueWork.GetT2Ptr());
}


//! @endcond
}
}
}  // end of namespace nn::pia::reckoning
