﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/reckoning/reckoning_Definitions.h>
#include <nn/pia/clone/clone_CloneElementBase.h>
#include <nn/pia/common/common_OffsetList.h>


namespace nn
{
namespace pia
{
namespace clone
{
class CloneProtocol;
class IDataPacker;
}
}
}


namespace nn
{
namespace pia
{
namespace reckoning
{

/*!
    @brief  推測航法を用いながらの値の送受信を管理する基底クラスです。
 */
class ReckoningCloneElementBase : public clone::CloneElementBase
{
public:
    /*!
    @brief デストラクタです。
     */
    virtual ~ReckoningCloneElementBase();


    /*!
    @brief  @ref ReckoningCloneElement::GetValue "GetValue()" で有効な値が取得できるかどうかを取得します。
    @return @ref ReckoningCloneElement::GetValue "GetValue()" で有効な値を取得できる場合は true を返します。
     */
    bool IsValidValue();


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


    //! @cond PRIVATE

public:
    virtual Type GetType() const
    {
        return GetTypeStatic();
    }
    virtual bool IsCloneSupported(int cloneType) const;

    static Type GetTypeStatic()
    {
        return Type_Reckoning;
    }
    static bool Receive(CloneElementBase* pElement, clone::IDataPacker* pAckPacker, const void* cpChunk, uint16_t chunkSize, Id elementId, StationIndex src, StationIndex myStationIndex, uint32_t dispatchCount);
    static uint32_t GetDataChunkHeaderSize();

private:
    void ReceiveUnreliable(const void* cpData, uint32_t size, StationIndex src, StationIndex setter, clone::ClockValue clock);
    void ReceiveReliable(const void* cpData, uint32_t size, StationIndex src, StationIndex setter, clone::ClockValue clock, uint32_t destBmp);
    void ReceiveAck(StationIndex src, StationIndex setter, clone::ClockValue clock);

public:
    virtual void ClearData();

    virtual void AddDest(StationIndex stationId);
    virtual void RemoveDest(StationIndex stationId);
    virtual void Adjust(StationIndex stationId);

    virtual void RequestInitialData();

public:
    class ReckoningSendToken : public ConversibleResendableSendToken
    {
    public:
        ReckoningSendToken()
            : ConversibleResendableSendToken()
        {
        }
        void Init(ReckoningCloneElementBase* pElement)
        {
            m_pElement = pElement;
        }

        virtual Type GetElementType() const
        {
            return Type_Reckoning;
        }
        virtual uint32_t GetDestBitmap() const
        {
            return m_DestBmp;
        }
        virtual uint32_t GetChunkSize() const;
        virtual void WriteChunk(void* pChunkBuffer) const;
        virtual CloneElementBase* GetElement()
        {
            return m_pElement;
        }

        Id GetId() const
        {
            return m_pElement->GetId();
        }
        uint32_t GetSize() const
        {
            return m_pElement->GetSize();
        }
        clone::ClockValue GetClock() const
        {
            return m_Clock;
        }
        void Serialize(void* pBuffer) const
        {
            m_pElement->Serialize(pBuffer, this);
        }

        StationIndex GetSetterStationIndex() const
        {
            return m_SetterStationIndex;
        }

        void SetDestBitmap(uint32_t destBmp)
        {
            m_DestBmp = destBmp;
        }
        void SetClock(clone::ClockValue clock)
        {
            m_Clock = clock;
        }
        void SetSetterStationIndex(StationIndex stationId)
        {
            m_SetterStationIndex = stationId;
        }

        void AddSendData()
        {
            m_pElement->AddSendData(this);
        }
        void CancelSendData()
        {
            m_pElement->CancelSendData(this);
        }

    private:
        ReckoningCloneElementBase* m_pElement;
        uint32_t m_DestBmp;
        clone::ClockValue m_Clock;
        StationIndex m_SetterStationIndex;
    };

protected:
    template <typename Sample>
    class SampleReckoningSendToken : public ReckoningSendToken
    {
    public:
        SampleReckoningSendToken()
            : ReckoningSendToken()
        {
        }
        Sample* GetSamplePtr()
        {
            return &m_Sample;
        }
        const Sample& GetSample() const
        {
            return m_Sample;
        }

    private:
        Sample m_Sample;
    };

protected:
    ReckoningCloneElementBase(ReckoningSendToken** papBuffer, uint32_t bufferSize);

protected:
    void UpdateValue();
    Result SetValueCore(bool* pIsSend);
    ReckoningSendToken* AssignTokenForSend(int* pIdx, bool isResendable);

    virtual void Serialize(void* pBuffer, const ReckoningSendToken* cpToken) = 0;
    virtual void ClearValue() = 0;
    virtual bool EstimateValue(clone::ClockValue clock) = 0;

    virtual void Deserialize(ReckoningSendToken* pToken, const void* cpData) = 0;
    virtual void OnUpdateSample(int idx) = 0;

protected:
    class SampleBuffer
    {
    public:
        SampleBuffer(ReckoningSendToken** apBuffer, int size);

        void Clear();
        ReckoningSendToken* Get(int idx);
        const ReckoningSendToken* Get(int idx) const;
        bool IsHead(ReckoningSendToken* pToken) const
        {
            return (pToken == m_apBuffer[m_HeadIdx]);
        }
        ReckoningSendToken* Search(clone::ClockValue clock);
        ReckoningSendToken* Assign(int* pIdx, clone::ClockValue clock);
        int GetNum() const
        {
            return m_Num;
        }

        void AddDest(StationIndex stationId, bool IsResponsible);
        void RemoveDest(StationIndex stationId);
        void Adjust();

    private:
        ReckoningSendToken** m_apBuffer;
        int m_Size;
        int m_HeadIdx;
        int m_Num;
    };

    const SampleBuffer& GetSampleBuffer() const
    {
        return m_SampleBuffer;
    }

    //! @endcond

public:
    /*!
    @brief  標本値バッファの値を参照するためのクラスです。
    @tparam Sample  標本値の型です。
     */
    template <typename Sample>
    class SampleAccessor
    {
    public:
        //! @cond PRIVATE
        explicit SampleAccessor(const SampleBuffer& cBuffer)
            : m_cBuffer(cBuffer)
        {
        }
        //! @endcond


        /*!
        @brief  保存されている標本値の数を取得します。
         */
        int GetNum() const
        {
            return m_cBuffer.GetNum();
        }


        /*!
        @brief  標本値が設定された時刻を取得します。
        @param[in] index    標本値のインデックスです。値が小さい方が新しい標本値を表します。
        @return 標本値が設定された時刻です。
         */
        clone::ClockValue GetClock(int index) const
        {
            PIA_ASSERT(index >= 0 && index < GetNum());
            return m_cBuffer.Get(index)->GetClock();
        }


        /*!
        @brief  標本値を取得します。
        @param[in] index    標本値のインデックスです。値が小さい方が新しい標本値を表します。
        @return 標本値です。
         */
        const Sample& GetSample(int index) const
        {
            PIA_ASSERT(index >= 0 && index < GetNum());
            return static_cast<const SampleReckoningSendToken<Sample>*>(m_cBuffer.Get(index))->GetSample();
        }

    private:
        const SampleBuffer& m_cBuffer;
    };


    //! @cond PRIVATE

private:
    SampleBuffer m_SampleBuffer;
    clone::ClockValue m_Clock;
    bool m_IsValidValue;
    StationIndex m_ResponsibleStationIndex;

    static Definition<ReckoningCloneElementBase> s_Definition;

public:
#if 0
    void Test(const CloneProtocol* cpProtocol);
#endif

public:
    template <int TEST_ID, typename Arg>
    static void Test(const ReckoningCloneElementBase& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(ReckoningCloneElementBase);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::reckoning
