﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/reckoning/reckoning_Definitions.h>
#include <nn/pia/reckoning/reckoning_Vector3f.h>
#include <nn/pia/reckoning/reckoning_ReckoningCloneElementBase.h>
#include <nn/pia/clone/clone_CloneBase.h>
#include <nn/pia/clone/clone_SerializePolicyDefinition.h>

namespace nn
{
namespace pia
{
namespace reckoning
{

/*!
  @brief  線形の評価関数を用いて3次元ベクトルを補間するReckoningStrategyクラスです。
 */
class Simple3dReckoningStrategy
{
public:
    typedef Vector3f Value;  //!< SetValue(), GetValue() で設定／取得する値の型です。

    /*!
    @brief  値と停止フラグをセットにした構造体です。
     */
    struct SampleValue
    {
        Value m_Value;
        bool m_IsStop;
    };
    typedef SampleValue Sample;  //!< 実際に送受信される標本値の型です。

    typedef bool SetValueArg;  //!< SetValue() 呼び出し時に、 CheckSample(), MakeSample() に渡すデータの型です。

    /*!
    @brief  推測の計算のための事前計算結果を保持しておく構造体です。
    */
    struct Work
    {
        Value m_Param;
        Value m_ParamBase;
        nn::pia::clone::ClockValue m_BaseClock;
    };

    static const uint32_t BufferSize = 10;  //!< 標本値を保持しておくバッファのサイズです。

    typedef ReckoningCloneElementBase::SampleAccessor<Sample> Accessor;  //!< 標本値バッファへのアクセサです。

    /*!
    @brief デフォルトコンストラクタです。
    @param[in] threshold 判定の際に用いる閾値の値
     */
    Simple3dReckoningStrategy(float threshold);

    /*!
    @brief     SetValue() で設定した値 value を標本値として採用するかどうかを判定する際に使用する閾値の値を設定します。
    @param[in] threshold 判定の際に用いる閾値の値
     */
    void SetThreshold(float threshold);

    /*!
    @brief  SetValue() で設定した値 value を標本値として採用するかどうかを判定する際に使用する閾値の値を返します。
    @return 判定の際に用いる閾値の値
     */
    float GetThreshold()
    {
        return m_Threshold;
    }

    /*!
    @brief      推測値を計算します。
    @param[out] pValue      推測値を出力します。
    @param[in]  accessor    標本値バッファへのアクセサです。
    @param[in]  clock       現在の時刻を与えます。
    @param[in]  cpWork      推測の計算のための事前計算結果を与えます。
    @return     推測値が計算できた場合、pValueに値を格納し、trueを返します。
     */
    bool Estimate(Value* pValue, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const;

    /*!
    @brief      SetValue() で設定した値 value を標本値として採用するかどうかを判定します。
    @param[out] pIsReliable     標本値の送信にあたり到達保証をする必要があるかどうかを出力します。
    @param[in]  value           SetValue() で設定した値を与えます。
    @param[in]  cpSetValueArg   SetValue() の第2引数に設定した値を与えます。valueが停止しているかどうかのフラグを与えます。
    @param[in]  accessor        標本値バッファへのアクセサです。
    @param[in]  clock           現在の時刻を渡します。
    @param[in]  cpWork          推測の計算のための事前計算結果を渡します。
    @return     標本値として採用する場合、trueを返します。
     */
    bool CheckSample(bool* pIsReliable, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const;

    /*!
    @brief      標本値として保存する値を計算します。
    @details    CheckSample() で標本値として採用するよう判定され、実際にバッファが確保できた場合に呼び出されます。
    @param[out] pSample         標本値を出力します。
    @param[in]  value           SetValue() で設定した値を与えます。
    @param[in]  cpSetValueArg   SetValue() の第2引数に設定した値を与えます。valueが停止しているかどうかのフラグを与えます。
    @param[in]  accessor        標本値バッファへのアクセサです。
    @param[in]  clock           現在の時刻を渡します。
    @param[in]  cpWork          推測の計算のための事前計算結果を渡します。
     */
    void MakeSample(Sample* pSample, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const;

    /*!
    @brief      標本値が追加された場合に呼び出されます。
    @details    推測の計算のために使用するパラメータを更新します。
    @param[in]  accessor        標本値バッファへのアクセサです。
    @param[in]  index           標本値のインデックスです。
    @param[in]  pWork           推測の計算のための事前計算結果を渡します。
     */
    void OnUpdateSample(const Accessor& accessor, int index, Work* pWork) const;

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
    */
    void Trace(uint64_t flag) const;

protected:
private:
    float m_Threshold;
    int32_t m_DelayClock;

    NN_PIA_DISALLOW_COPY(Simple3dReckoningStrategy);
};
} // end of namespace reckoning

// シリアル化のために SerializePolicy の各関数を定義します。
namespace clone
{
template<>
inline void SerializePolicy<reckoning::Simple3dReckoningStrategy::Sample>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    SerializePolicy<float>::Serialize(offset, value.m_Value.x);
    offset += sizeof(float);
    SerializePolicy<float>::Serialize(offset, value.m_Value.y);
    offset += sizeof(float);
    SerializePolicy<float>::Serialize(offset, value.m_Value.z);
    offset += sizeof(float);
    SerializePolicy<uint8_t>::Serialize(offset, value.m_IsStop);
    offset += 1;
}

template<>
inline void SerializePolicy<reckoning::Simple3dReckoningStrategy::Sample>::Deserialize(Type* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    SerializePolicy<float>::Deserialize(&pValue->m_Value.x, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->m_Value.y, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->m_Value.z, offset);
    offset += 4;
    SerializePolicy<uint8_t>::Deserialize(reinterpret_cast<uint8_t*>(&pValue->m_IsStop), offset);
    offset += 1;
}

template<>
inline uint32_t SerializePolicy<reckoning::Simple3dReckoningStrategy::Sample>::GetSerializedSize()
{
    return 4 * 3 + 1;
}
} // end of namespace clone

}
}  // end of namespace nn::pia
