﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/reckoning/reckoning_Definitions.h>


namespace nn
{
namespace pia
{
namespace reckoning
{

/*!
  @brief  Vector3fクラスは3次元ベクトルを扱うStrategyのために用意されたクラスです。
 */
class Vector3f
{
public:
    float x;
    float y;
    float z;

    /*!
    @brief デフォルトコンストラクタです。
     */
    Vector3f()
        : x(0.f), y(0.f), z(0.f) {};

    /*!
    @brief 引数で要素の初期化を行うコンストラクタです。
    @param[in] vx, vy, vz  初期化する値を入れる必要があります。
     */
    Vector3f(const float vx, const float vy, const float vz)
    {
        this->x = vx;
        this->y = vy;
        this->z = vz;
    }

    /*!
    @brief 0ベクトルをセットします。
     */
    void MakeZero()
    {
        x = y = z = 0.f;
    }

    /*!
    @brief 2点間の距離の2乗を返します。
    @param[in] v    距離を求めたい対象のベクトル
    @return         距離の2乗を返します。
     */
    float SquaredDistance(const Vector3f& v) const
    {
        float dx = this->x - v.x;
        float dy = this->y - v.y;
        float dz = this->z - v.z;
        return dx * dx + dy * dy + dz * dz;
    }

    /*!
    @brief 対象のベクトルに対して指定したベクトルを加算します。
    @param[out] out 対象ベクトル
    @param[in]  v   加算ベクトル  
     */
    static void Add(Vector3f* out, const Vector3f& v)
    {
        out->x += v.x;
        out->y += v.y;
        out->z += v.z;
        return;
    }

    /*!
    @brief 対象のベクトルに対して指定した値をスカラー乗算します。
    @param[out] out 対象ベクトル
    @param[in]  t   乗算値 
     */
    static void Mul(Vector3f* out, const float& t)
    {
        out->x *= t;
        out->y *= t;
        out->z *= t;
    }

    /*!
    @brief 指定したベクトルを加算して代入します。
    @param[in]  v   加算ベクトル
    @return     加算後のベクトルを返します。
     */
    Vector3f& operator+=(const Vector3f& v)
    {
        x += v.x;
        y += v.y;
        z += v.z;
        return *this;
    }

    /*!
    @brief 指定した値でスカラー除算して代入します。
    @param[in]  t   除算値
    @return     除算後のベクトルを返します。
     */
    Vector3f& operator/=(const float& t)
    {
        x /= t;
        y /= t;
        z /= t;
        return *this;
    }

protected:
private:
};
}
}
}  // end of namespace nn::pia::reckoning
