﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>


namespace nn
{
namespace pia
{
namespace session
{
/*!
@name モジュールの初期化、終了処理
@{
*/

/*!
    @brief   PiaSession モジュールを初期化します。
             事前に nn::pia::common::Initialize を実行しておく必要があります。
    @details session モジュールが動的に確保し使用するメモリは @ref nn::pia::common::Initialize で渡したメモリから確保されます。

    @return このAPIを適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultInvalidState        common モジュールが初期化されていない状態です。[:progErr]
    @retval ResultAlreadyInitialized  既にこの関数を実行し、初期化が完了している状態です。[:progErr]
*/
Result Initialize();

/*!
    @brief   セットアップの開始を宣言します。

    @details session モジュール以下の初期化処理、すなわち @ref Session::CreateInstance を実行する前に呼び出す必要があります。
             別のモジュールのセットアップ処理とバッティングすると正しく動作しませんので
             セットアップ処理は @ref session::BeginSetup 実行後、session モジュール以下の初期化処理を行い、
             速やかに @ref session::EndSetup を実行してセットアップを完了させる必要があります。

    @return このAPIを適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized  session::Initialize が呼ばれていません。[:progErr]
    @retval ResultInvalidState    セットアップ期間中に再度呼び出しています。[:progErr]
*/
Result BeginSetup(void);

/*!
    @cond PRIVATE
    @brief  現在セットアップ期間中かを確認します。
    @details 本関数は内部処理のために用意したものですので、アプリケーションが呼び出す必要はありません。
      
    @return @ref session::BeginSetup 実行から @ref session::EndSetup までの期間であれば true を返します。
*/
bool IsDuringSetup(void);
//! @endcond

/*!
    @brief   セットアップの終了を宣言します。
    @details session モジュール以下の初期化処理、すなわち @ref Session::CreateInstance を実行した後に呼び出す必要があります。
             このAPI呼び出しにより、session モジュールのシングルトンなどが消費するメモリを確定し、ライブラリ内部でのメモリ管理を最適化します。
             別のモジュールのセットアップ処理とバッティングすると正しく動作しませんので
             セットアップ処理は @ref session::BeginSetup 実行後、session モジュール以下の初期化処理を行い、
             速やかに @ref session::EndSetup を実行してセットアップを完了させる必要があります。

    @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized  session::Initialize が呼ばれていません。[:progErr]
    @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
*/
Result EndSetup(void);

/*!
    @brief  PiaSession モジュールが初期化された状態であるかどうかを返します。

    @return 初期化されていれば true 、されていなければ false が返されます。
*/
bool IsInitialized(void);

/*!
    @brief   PiaSession モジュールを終了します。
             @ref nn::pia::common::Finalize を実行する前に実行する必要があります。
    @details 本関数を実行する前に PiaSession モジュールのすべての処理を終了しておく必要があります。
             なお、@ref nn::pia::session::Initialize を実行していない状態では何もしません。
*/
void Finalize(void);

/*!
@}
*/
}
}
}  // end of namespace nn::pia::session
