﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
    @brief   コミュニティを検索する際に使用する検索条件の基底クラスです。
    @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。
    @see nn::pia::inet::NexCommunitySearchCriteriaCommunityId
    @see nn::pia::inet::NexCommunitySearchCriteriaParticipant
    @see nn::pia::inet::NexCommunitySearchCriteriaOwner
    @see nn::pia::inet::NexCommunitySearchCriteriaOfficial
 */
class CommunitySearchCriteria : public nn::pia::common::RootObject
{
public:
    /*!
    @cond PRIVATE
    @brief      検索条件の種類。
     */
    enum SearchCriteriaKey
    {
        SearchCriteriaKey_None = 0,
        SearchCriteriaKey_CommunityId,             //!< コミュニティ ID を検索条件に指定します。
        SearchCriteriaKey_ParticipantPrincipalId,  //!< コミュニティ参加者のプリンシパル ID を検索条件に指定します。
        SearchCriteriaKey_OwnerPrincipalId,        //!< コミュニティのオーナーの プリンシパル ID を検索条件に指定します。
        SearchCriteriaKey_Official                 //!< 公式コミュニティを検索条件に指定します。
    };
    //! @endcond

    /*!
    @brief      デストラクタ
     */
    virtual ~CommunitySearchCriteria();

    /*!
    @brief      検索結果の取得範囲を指定します。
    @details    offset に 0xFFFFFFFF を指定すると、検索結果をランダムに並び替えた上で
                指定されたエントリー数を取得するように振る舞います。
    @param[in]  offset  検索条件にマッチしたエントリーリストから検索結果として取得するエントリー群の開始位置。
    @param[in]  size    検索条件にマッチしたエントリーリストから検索結果として取得するエントリー数。

    @return     処理に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval     nn::pia::ResultInvalidArgument  引数の値が範囲外です。[:progErr]
    */
    virtual Result SetResultRange(uint32_t offset, uint32_t size);

    /*!
    @cond PRIVATE
    @brief      検索条件の種類を取得します。
     */
    SearchCriteriaKey GetKeyType() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      検索条件にマッチしたエントリーリストから検索結果として取得するエントリー群の開始位置を取得します。
     */
    uint32_t GetResultRangeOffset() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      検索条件にマッチしたエントリーリストから検索結果として取得するエントリー数を取得します。
     */
    uint32_t GetResultRangeSize() const;
    //! @endcond

    /*!
    @brief      検索条件をリセットします。
     */
    virtual void Reset();

protected:
    // デフォルトコンストラクタ。
    CommunitySearchCriteria();

    // コンストラクタ
    CommunitySearchCriteria(SearchCriteriaKey key);

    void Copy(const CommunitySearchCriteria& rhs);

    // 検索キーのタイプ
    SearchCriteriaKey m_KeyType;
    // 検索範囲
    uint32_t m_ResultOffset;
    uint32_t m_ResultSize;

private:
    NN_PIA_DISALLOW_COPY(CommunitySearchCriteria);
};
}
}
}  // end of namespace nn::pia::session
