﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

#include <nn/pia/session/session_Types.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
  @brief   セッションの作成処理に関する設定を管理する基底クラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。

@if CTR_DOC
  @see local::UdsCreateSessionSetting
@endif
@if NIN_DOC
  @see local::LdnCreateSessionSetting
@endif
  @see inet::NexCreateSessionSetting
  @see lan::LanCreateSessionSetting
 */
class CreateSessionSetting : public common::RootObject
{
public:
    /*!
    @brief      デストラクタ。
     */
    virtual ~CreateSessionSetting();

    /*!
    @brief      作成するセッションの最小参加人数を指定します。
    @param[in]  min 作成するセッションに指定する最小参加人数。
    */
    void SetMinParticipantNum(uint16_t min);

    /*!
    @cond PRIVATE
    @brief      指定した最小参加人数を取得します。
    @return     指定した最小参加人数を返します。
    */
    uint16_t GetMinParticipantNum() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      最小参加人数を指定したかどうかを取得します。
    @return     最小参加人数を指定したかどうかを返します。
    */
    bool IsSetMinParticipantNum() const;
    //! @endcond

    /*!
    @brief      作成するセッションの最大参加人数を指定します。
    @param[in]  max 作成するセッションに指定する最大参加人数。
    */
    virtual void SetMaxParticipantNum(uint16_t max);

    /*!
    @cond PRIVATE
    @brief      指定した最大参加人数を取得します。
    @return     指定した最大参加人数を返します。
    */
    uint16_t GetMaxParticipantNum() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      最大参加人数を指定したかどうかを取得します。
    @return     最大参加人数を指定したかどうかを返します。
    */
    bool IsSetMaxParticipantNum() const;
    //! @endcond

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @param[in]  pData   構築するセッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    構築するセッションに指定するアプリケーション定義データのサイズ。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size) = 0;

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータサイズを取得します。
    @return     アプリケーション定義データサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSize() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      アプリケーション定義を指定したかどうかを取得します。
    @return     アプリケーション定義を指定したかどうかが返されます。
    */
    virtual bool IsSetApplicationData() const = 0;
    //! @endcond

    /*!
    @brief      設定をリセットします。
     */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

protected:
    // デフォルトコンストラクタ
    CreateSessionSetting();

    void Copy(const CreateSessionSetting& rhs);

    uint16_t m_MinParticipantNum;
    bool m_IsMinParticipantNumSet;
    uint16_t m_MaxParticipantNum;
    bool m_IsMaxParticipantNumSet;

private:
    NN_PIA_DISALLOW_COPY(CreateSessionSetting);
};
}
}
}  // end of namespace session
