﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

namespace nn
{
namespace pia
{
namespace common
{
class String;
class DateTime;
}
}
}

namespace nn
{
namespace pia
{
namespace session
{

/*!
    @brief  コミュニティの情報へのインターフェースです。
 */
class ICommunityInfo : public common::RootObject
{
public:
    ICommunityInfo()
    {
    }
    virtual ~ICommunityInfo()
    {
    }

    /*!
    @brief      コミュニティの ID を取得します。
    @return     コミュニティの ID を返します。
    */
    virtual uint32_t GetCommunityId() const = 0;

    /*!
    @brief      公式コミュニティであるかどうかを取得します。
    @return     
    */
    virtual bool IsOfficialCommunity() const = 0;

    /*!
    @brief      パスワードで参加制限されているかどうかを確認します。
    @return     パスワードで参加制限されている場合は true を返します。
    */
    virtual bool IsRestrictedByPassword() const = 0;

    /*!
    @brief      参加開始日時を取得します。
    @return     参加開始日時を返します。
    */
    virtual const common::DateTime& GetParticipationStartDate() = 0;

    /*!
    @brief      参加終了日時を取得します。
    @return     参加終了日時を返します。
    */
    virtual const common::DateTime& GetParticipationEndDate() = 0;

    /*!
    @brief      コミュニティに関連づけられたセッションの数を取得します。
    @return     コミュニティに関連づけられたセッションの数を返します。
    */
    virtual uint32_t GetRelatedSessionCount() const = 0;

    /*!
    @brief      コミュニティの属性を取得します。
    @param[in]  idx         取得する属性のインデックス。
    @param[out] pAttribute  指定されたインデックスのコミュニティの属性格納先のポインタ。
    @return     正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultInvalidState      データを取得できる状態ではありません。[:progErr]
    */
    virtual Result GetAttribute(uint8_t idx, uint32_t* pAttribute) const = 0;

    /*!
    @brief      コミュニティーのオーナーのプリンシパル ID を取得します。
    */
    virtual PrincipalId GetOwnerPrincipalId() const = 0;

    /*!
    @brief      コミュニティに参加している人数を取得します。
    @return     コミュニティに参加している人数を返します。
    */
    virtual uint16_t GetCurrentParticipantNum() const = 0;

    /*!
    @brief      コミュニティに参加できる最大人数を取得します。
    @return     コミュニティに参加できる最大人数を返します。
    */
    virtual uint16_t GetMaxParticipantNum() const = 0;

    /*!
    @brief      コミュニティに参加できる最少人数を取得します。
    @return     コミュニティに参加できる最少人数を返します。
    */
    virtual uint16_t GetMinParticipantNum() const = 0;

    /*!
    @brief      コミュニティの説明文字列を取得します。
    @return     コミュニティの説明文字列が返ります。
    */
    virtual const common::String& GetDescription() const = 0;

    /*!
    @brief      コミュニティのアプリケーション定義データを取得します。
    @param[out] pBuffer コミュニティのアプリケーション定義データのコピー用バッファのポインタ。
    @param[in]  size    コミュニティのアプリケーション定義データのコピー用バッファのサイズ。
    @return     正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultInvalidState      データを取得できる状態ではありません。[:progErr]
    @if CTR_DOC
    @retval ResultNoData            取得できるデータがありませんでした。ローカル通信時のみ返ります。[:handling]
    @endif
    */
    virtual Result GetApplicationData(void* pBuffer, uint32_t size) = 0;

    /*!
    @brief      コミュニティのアプリケーション定義データの長さを取得します。
    @param[out] pSize   コミュニティのアプリケーション定義データの長さ格納先のポインタ。
    @return     正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   引数が不正です。[:progErr]
    @retval ResultInvalidState      データを取得できる状態ではありません。[:progErr]
    */
    virtual Result GetApplicationDataSize(uint32_t* pSize) = 0;
};
}
}
}  // end of namespace nn::pia::session
