﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
    @brief  セッションの情報へのインターフェースです。
 */
class ISessionInfo : public common::RootObject
{
public:
    /*!
    @brief    デストラクタ
    */
    virtual ~ISessionInfo()
    {
    }

    /*!
    @brief      セッションのゲームモードを取得します。
    @details    インターネット通信時はゲームモードの値を返します。<br />
                ローカル通信時は通信モード識別用 ID を uint32_t にキャストして返します。
    @return     セッションのゲームモードを返します。
    */
    virtual uint32_t GetGameMode() const = 0;

    /*!
    @brief      セッションの ID を取得します。
    @return     セッションの ID を返します。
    */
    virtual uint32_t GetSessionId() const = 0;

    /*!
    @brief      セッションに参加している人数を取得します。
    @return     セッションに参加している人数を返します。
    */
    virtual uint16_t GetCurrentParticipantNum() const = 0;

    /*!
    @brief      セッションに参加できる最小人数を取得します。
    @return     セッションに参加できる最小人数を返します。
    */
    virtual uint16_t GetMinParticipantNum() const = 0;

    /*!
    @brief      セッションに参加できる最大人数を取得します。
    @return     セッションに参加できる最大人数を返します。
    */
    virtual uint16_t GetMaxParticipantNum() const = 0;

    /*!
    @brief      セッションが参加募集受付状態かどうかを取得します。
    @return     セッションが参加募集受付状態なら true, 参加募集締切状態なら false を返します。
    */
    virtual bool IsOpened() const = 0;

    /*!
    @brief              セッションのアプリケーション定義データを取得します。
    @details            セッションに設定したアプリケーション定義データを取得します。
    @param[out] pBuffer セッションのアプリケーション定義データのコピー用バッファのポインタ。
    @param[in]  size    セッションのアプリケーション定義データのコピー用バッファのサイズ。
    @return             正常に情報を取得できた場合、成功の Result が返ります。
    @retval ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pBuffer, uint32_t size) const = 0;

    /*!
    @brief      セッションのアプリケーション定義データの長さを取得します。
    @return     セッションのアプリケーション定義データの長さが返ります。
    */
    virtual uint32_t GetApplicationDataSize() const = 0;

    /*!
    @brief      セッション情報をクリアします。
    */
    virtual void Clear() = 0;

    /*!
    @brief デバッグに有用な情報をプリントします。
    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
    */
    virtual void Trace(uint64_t flag) const = 0;

protected:
    /*!
    @brief    デフォルトコンストラクタ
    */
    ISessionInfo()
    {
    }
};
}
}
}  // end of namespace session
