﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>
#include <nn/pia/session/session_Types.h>

#include <nn/pia/common/common_AsyncContext.h>
#include <nn/pia/common/common_SignatureSetting.h>

#include <nn/pia/transport/transport_Station.h>

namespace nn
{
namespace pia
{
namespace common
{
struct CryptoSetting;
class MonitoringDataSender;
class StationAddress;
}
}
}

namespace nn
{
namespace pia
{
namespace transport
{
class StationConnectionInfo;
class StationProtocol;
class ConnectStationJob;
class DisconnectStationJob;
class ProcessConnectionRequestJob;
class MissingStationHandler;
class NetworkFactory;
class BandwidthCheckProtocol;
}
}
}

///@cond PRIVATE

namespace nn
{
namespace pia
{
namespace session
{

class CreateMeshJob;
class JoinMeshJob;
class LeaveMeshJob;
class ProcessJoinRequestJob;
class ProcessUpdateMeshJob;
class DestroyMeshJob;
class ProcessDestroyMeshJob;
class ProcessHostMigrationJob;
class LeaveWithHostMigrationJob;
class SignatureSettingStorage;
class RelayRouteManageJob;
class KickoutManageJob;
class MeshProtocol;
class SyncClockProtocol;
class MeshEventListener;

/*!
  @class    nn::pia::session::Mesh
  @brief    他の端末との接続を管理するクラスです。

  @details このクラスはシングルトンパターンです。単一のインスタンスしか生成できません。

  @attention    このクラスはメッシュの直接操作用クラスです。セッションを使用するときは Mesh クラスを使用してはいけません。メッシュを直接操作する場合は Session クラスを使用してはいけません。

  @details  Mesh 内部の処理の一部は @ref nn::pia::common::Scheduler にジョブとして登録されますので、 Mesh の機能を使う場合
            @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出す必要があります。
 */
class Mesh : public common::RootObject
{
    friend class CreateMeshJob;
    friend class JoinMeshJob;
    friend class LeaveMeshJob;
    friend class ProcessJoinRequestJob;
    friend class ProcessUpdateMeshJob;
    friend class DestroyMeshJob;
    friend class ProcessDestroyMeshJob;
    friend class ProcessHostMigrationJob;
    friend class LeaveWithHostMigrationJob;
    friend class KickoutManageJob;
    friend class MeshProtocol;
    friend class MeshEventListener;

public:
    /*!
    @brief  @ref nn::pia::session::Mesh::CreateInstance に渡す設定用構造体です。
    */
    struct Setting
    {
        Setting()
            : bitRateCheckMode(session::BitRateCheckMode_Disable)
        {
        }
        transport::NetworkFactory* pNetworkFactory;  //!< @ref nn::pia::transport::NetworkFactory から派生させた具象クラスのインスタンスを指すポインタ（ファクトリパターン）。Mesh インスタンス作成時にのみ使用されます。
        session::NetworkTopology networkTopology;    //!< メッシュのネットワークトポロジー設定です。
        session::BitRateCheckMode bitRateCheckMode;  //!< ビットレート判定機能の設定です。
    };

    /*!
    @brief  @ref nn::pia::session::Mesh::Startup に渡す設定用構造体です。
     */
    struct StartupSetting
    {
        StartupSetting()
            : maxSilenceTime(SessionMaxSilenceTimeDefault),
              keepAliveSendingInterval(SessionKeepAliveIntervalDefault),
              isHostMigrationEnabled(true),
              pPlayerInfo(NULL),
              playerInfoSize(1),
              isPlayersCountedAsParticipants(true),
              pCryptoSetting(NULL),
              signatureSetting(common::SignatureSetting::NoSignature),
              uplinkBitRateLowerLimit(SessionInvalidUplinkBitRateLowerLimit),
              bitRateCheckPacketSize(0),
              bitRateMeasuringSpan(1000),
              isBitRateCheckSkipped(false),
              pToken(NULL),
              isAddRecentPlayRecordEnabled(true)
        {
        }

        uint32_t maxSilenceTime;                                         //!< 許容される最大無通信時間 [msec] を指定します。デフォルトは session::SessionMaxSilenceTimeDefault です。
        uint32_t keepAliveSendingInterval;                               //!< キープアライブの送信間隔 [msec] を指定します。デフォルトは session::SessionKeepAliveIntervalDefault です。
        bool isHostMigrationEnabled;                                     //!< ホストマイグレーション機能を設定します。デフォルトは true (有効) になっています。
        const transport::Station::PlayerInfo* pPlayerInfo;      //!< 自分自身のステーションと関連付けるプレイヤー情報を設定します。他のメッシュ参加者から参照されます。
        uint8_t playerInfoSize;                                          //!< ステーションに関連つけるプレイヤー情報の数です。デフォルトは 1 です。
        bool isPlayersCountedAsParticipants;                             //!< 自分自身のステーションに複数のプレイヤーを関連付ける際、プレイヤー数をセッション参加者数としてカウントするか指定します。デフォルトは true です。
        const common::CryptoSetting* pCryptoSetting;            //!< 暗号化設定の構造体を示すポインタを指定します。NULL を指定した場合は「暗号化しない」設定となります。デフォルトは NULL です。
        common::SignatureSetting signatureSetting;              //!< パケットの署名機能に関する設定用クラスを適切に設定した状態で指定します。デフォルト引数の場合は "署名無し" に設定されます。
        int32_t uplinkBitRateLowerLimit;                                 //!< ビットレート判定機能を使用する際に、アプリケーションが要求する水準をこのメンバで指定します。単位は bps です。デフォルトは session::SessionInvalidUplinkBitRateLowerLimit です。
        uint32_t bitRateCheckPacketSize;                                 //!< ビットレート判定機能を利用する際に、判定に用いる IP パケットサイズを指定します。単位はバイトです。アプリケーションが送受信する IP パケットの平均的なサイズを指定することを想定しています。4の倍数である必要があります。デフォルトは 0 です。
        int32_t bitRateMeasuringSpan;                                    //!< ビットレート判定機能を利用する際に、計測にかける時間を設定します。単位は msec です。デフォルトは 1000 msec です。デフォルト値からの変更はお勧めしません。
        bool isBitRateCheckSkipped;                                      //!< ビットレート判定機能を利用する際に、判定に関わる処理をスキップするかどうかを設定します。スキップする場合には true を設定する必要があります。デフォルトは false です。
        const transport::Station::IdentificationToken* pToken;  //!< 自分自身を示す識別トークンのポインタを指定します。NULL を指定した場合、データが全て NULL の識別トークンが使用されます。デフォルトは NULL です。
        bool isAddRecentPlayRecordEnabled;                               //!< セッション参加者を「いっしょにあそんだ人の記録」に追加するかどうかを指定します。デフォルトは true（有効）になっています。本体機能が「いっしょにあそんだ人の記録」に対応していない場合は無視されます。
    };

    /*!
    @brief  （デバッグ用） @ref nn::pia::session::Mesh::SetDebugSetting に渡すデバッグ機能設定用構造体です。
    */
    struct DebugSetting
    {
        DebugSetting(bool argIsRelayEmulationEnabled = false, bool argIsCheckMaxSilenceTimeEnabled = true)
            : isRelayEmulationEnabled(argIsRelayEmulationEnabled),
              isCheckMaxSilenceTimeEnabled(argIsCheckMaxSilenceTimeEnabled)
        {
        }
        bool isRelayEmulationEnabled;       //!< デバッグ用接続経路エミュレーション機能設定です。
        bool isCheckMaxSilenceTimeEnabled;  //!< 最大無通信時間の設定値の範囲チェックを行います。
    };

    /*!
    @cond PRIVATE
    @brief  Mesh の状態を表す列挙型です。
     */
    enum State
    {
        State_Disconnected,  //!< 他の端末と接続されていない、通信不可能な状態です。
        State_Connected      //!< 他の端末と接続している、通信可能な状態です。
    };
    //! @endcond

    /*!
    @brief  Mesh が通信不可能な状態の場合に、通信不可能になった理由を表す列挙型です。

    @details  Mesh::GetDisconnectReason で取得可能です。
     */
    enum DisconnectReason
    {
        DisconnectReason_UnknownReason = 0,   //!< 通信不可能になった理由が不明です。メッシュ通信中に（メッシュ破棄、離脱などを行わずに） Mesh::Cleanup を行った場合などにこの値となります。
        DisconnectReason_NotYetCommunicated,  //!< まだ通信したことがない場合です。 Mesh::Startup 後に一度もメッシュ構築、参加をしていない場合はこの値となります。
        DisconnectReason_OperationOfOwn,      //!< 自分自身の操作により通信不可能になった場合です。 Mesh::DestroyMesh や Mesh::LeaveMesh などを呼び出してメッシュ離脱した場合はこの値となります。
        DisconnectReason_OperationOfOther,    //!< メッシュ内の自分以外のステーションの操作により通信不可能になった場合です。自身がクライアントでホストがメッシュ破棄した場合などはこの値となります。
        DisconnectReason_KickoutByUser,       //!< Mesh::KickoutStation 呼出によりメッシュから追放された場合です。
        DisconnectReason_KickoutBySystem,     //!< ライブラリ内部処理によってメッシュから追放された場合です。リレー通信機能有効時のリレー依頼数制限などのライブラリ内部処理によってメッシュから追放された場合はこの値となります。
        DisconnectReason_InconsistentInfo,    //!< メッシュ情報が不整合な状態になったので通信不可能になった場合です。ステーション数不一致などが原因の場合はこの値となります。
        DisconnectReason_MigrationFail,       //!< ホストマイグレーション処理に失敗したことにより通信不可能になった場合です。
        DisconnectReason_ExternalFactor,      //!< Mesh 以外の要因で通信不可能になった場合です。通信障害などが原因の場合はこの値となります。
        DisconnectReason_MigrationFatalError  //!< ホストマイグレーション関連で致命的なエラーが発生した場合です。メッシュが分裂状態になったことをライブラリ内部処理によって検知した場合などに発生します。
    };


    /*!
    @brief      （デバッグ用）接続経路エミュレーションの設定を表す列挙型です。

    @details    Mesh::RelayEmulation の第3引数で使用します。
     */
    enum EmulationType
    {
        EmulationType_None = 0,  //!< エミュレーション設定なし
        EmulationType_Relay      //!< リレー接続指定（直通接続不可）
    };


    /*!
    @brief  メッシュ状態変化通知コールバックで引数として渡される列挙型です。

    @details メッシュに発生した状態変化の内容を表現します。
     */
    enum EventType
    {
        EventType_EventJoin,                //!< ステーションが参加したことを示します。
        EventType_EventLeave,               //!< ステーションが離脱したことを示します。
        EventType_EventHostChanged,         //!< メッシュホスト権限が移譲されたことを示します。【ホストマイグレーション有効時のみ発生】
        EventType_EventMigrationFail,       //!< ホストマイグレーションに失敗したことを示します。【ホストマイグレーション有効時のみ発生】
        EventType_EventStartHostMigration   //!< ホストマイグレーション処理が開始したことを示します。【ホストマイグレーション有効時のみ発生】
    };

    /*!
    @brief  メッシュの状態が変化した場合に呼び出されるコールバックの関数型です。

    @details 【引数仕様】
            <ul>
            <li> eventType ： 状態変化の内容を示す列挙型（@ref EventType 列挙型参照）</li>
            <li> id ： 状態変化が発生した対象の StationId</li>
            </ul>
     */
    typedef void (*MeshEventCallback)(EventType eventType, StationId id);


    /*!
    @brief  メッシュホストでジョイン要求が来た際に呼び出される承認、拒否判定コールバックの関数型です。

    @details 返り値として true を返せばジョイン要求の承認、false を返せばジョイン要求の拒否となります。
    <br>
            【引数仕様】
            <ul>
            <li> pRequesterToken ： ジョイン要求の送信元ステーションの識別トークンを示すポインタです。</li>
            </ul>
     */
    typedef bool (*JoiningMeshApprovalCallback)(const transport::Station::IdentificationToken* pRequesterToken);

    /// @cond PRIVATE
    /*!
    @brief  複数候補版ホストマイグレーションで、次期親候補順位を任意に調整するためのコールバック関数。

    @details 引数 targetId について、任意の親候補順位（昇順）を返せば、次期親候補順位を調整可能。
     */
    typedef uint8_t (*CheckHostCandidateRankingCallback)(StationIndex targetId, bool isHost);
    /// @endcond

    /// @cond PRIVATE
    typedef uint32_t (*GetSessionIdCallback)();
    /// @endcond

    /*!
     @name シングルトンインスタンスの操作
     @{
     */

    /*!
    @brief      Mesh のインスタンスを作成します。

    @details    この関数は @ref nn::pia::session::BeginSetup を実行してから @ref nn::pia::session::EndSetup を実行するまでの期間に実行する必要があります。

    @details    デバッグ機能設定用関数 @ref SetDebugSetting でデバッグ用接続経路エミュレーション機能を使用する設定とした場合、
                ネットワークトポロジー設定が NetworkTopology_RelayMesh でなければこの関数呼出が失敗します。

    @attention  製品版 ROM では、@ref SetDebugSetting を呼び出さないように注意してください。

    @param[in]  setting     各種設定情報を指定します。

    @return 関数の実行結果を返します。処理が成功した場合、IsSuccess() が真となる Result を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。<br>
            処理が失敗した場合、以下の Result を返します。
    @retval ResultNotInitialized   session モジュールが未初期化です。[:progErr]
    @retval ResultInvalidState     関数が実行可能な状態ではありません。 @ref nn::pia::session::BeginSetup を実行してから @ref nn::pia::session::EndSetup を実行するまでの期間外に実行した場合にこのエラーが返ります。[:progErr]
    @retval ResultInvalidArgument  引数で指定した設定内容に誤りがあります。[:progErr]
    @retval ResultAlreadyExists    既にインスタンスは作成されています。[:progErr]
     */
    static Result CreateInstance(const session::Mesh::Setting& setting);


    /*!
    @brief Mesh のインスタンスを破棄します。
     */
    static void DestroyInstance();


    /*!
    @brief  Mesh オブジェクトのインスタンスを取得します。

    @details メッシュクラスはシングルトンであり、ひとつのオブジェクトのみが自動的に生成されます。
            この関数を呼び出すことで、唯一のインスタンスへのポインタを取得することが出来ます。

    @return Meshオブジェクトインスタンスへのポインタが返ります。
     */
    static Mesh* GetInstance()
    {
        return s_pMesh;
    }


    /*!
    @brief      （デバッグ用） Mesh のデバッグ用機能について設定します。

    @details    本関数は Mesh のインスタンスが存在しない状態で呼出可能です。 @ref CreateInstance で Mesh のインスタンスを作成する前に呼び出す必要があります。<br>
                <br>
                DebugSetting 構造体の isRelayEmulationEnabled を true に設定した場合、デバッグ用接続経路エミュレーション機能を使用する設定となります。
                接続経路エミュレーション機能を使用する設定の場合、ネットワークトポロジーを NetworkTopology_RelayMesh に設定しない限り @ref CreateInstance が成功しません。<br>
                <br>
                本関数を呼び出さなかった場合、本関数が関連するデバッグ用機能は全て無効（使用しない）設定となります。

    @attention  製品版 ROM では、本関数を呼び出さないように注意してください。

    @return 関数の実行結果を返します。処理が成功した場合、IsSuccess() が真となる Result を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。<br>
            処理が失敗した場合、以下の Result を返します。
    @retval ResultInvalidState     関数が実行可能な状態ではありません。既に Mesh のインスタンスが作成済みの場合に返ります。[:progErr]
     */
    static Result SetDebugSetting(const DebugSetting& debugSetting);

    /*!
     end of name シングルトンインスタンスの操作
     @}
     */

    /*!
     @name 初期化終了処理
     @{
     */

    /*!
    @brief  Mesh をスタートアップします。

    @details StartupSetting 構造体の isHostMigrationEnabled でホストマイグレーション機能の設定を行います。true にすると有効になります。デフォルト値は true です。<br>
             StartupSetting 構造体の pPlayerInfo で自分自身のステーションと関連付けるプレイヤー情報を設定します。<br>
    @if CAFE_DOC
             プレイヤーの名前の長さは16文字（NULL終端を含めない）以内である必要があります。文字コードは UTF-16BE である必要があります。
             言語コードはアプリケーションが使用している言語を、 nn::fp::Language で定義される値で指定して下さい。<br>
             設定した名前と言語コードはメッシュ参加者間で共有されます。自分がメッシュに参加したときにはメッシュの参加者全員を、他のクライアントが
             メッシュに参加したときにはその新規参加者の名前と言語コードを参照し、ライブラリ内で自動的に nn::fp::AddRecentPlayRecord() を使用して
             「いっしょに遊んだ記録」に追加します。 <br>
             「いっしょに遊んだ記録」についての詳細はフレンドプレゼンスライブラリのプログラミングマニュアルおよび nn::fp::AddRecentPlayRecord() の
             リファレンスを参照してください。 
    @endif
    @if CTR_DOC
             プレイヤーの名前の長さは16文字（NULL終端を含めない）以内である必要があります。文字コードは UTF-16LE である必要があります。
             言語コードはアプリケーションで別リージョンの言語を判定する場合に指定する必要があります。本体の言語設定を使用したい場合には、nn::cfg::CTR::GetLanguage() の値を使用可能です。
    @endif

            StartupSetting 構造体の pCryptoSetting で、通信の暗号化に関する設定をすることができます。<br>
            NULL を指定した場合は暗号化されません。デフォルト値は NULL です。

            StartupSetting 構造体の signatureSetting で、パケットの署名機能に関する設定をすることができます。<br>
            インターネット通信で "署名無し" とすることは許可していません。 "署名無し" に設定しようとした場合、@ref Startup は失敗します。

            StartupSetting 構造体の pToken で、自分自身を示す識別トークンを設定することができます。<br>
            この識別トークンは接続処理時に接続相手と共有され、@ref nn::pia::transport::Station と結び付けられて保管されます。<br>
            識別トークンを使用する必要が無い場合は、特に設定を行う必要はありません。デフォルト値は NULL です。

            @ref Startup 以外で、自分自身を示す識別トークンを指定・変更する方法はありません。<br>
            メッシュ使用中に識別トークンを変更するような使用方法には対応していませんので、注意してください。
    @if CTR_DOC

            ローカル通信かつホストマイグレーション有効の場合、ローカルネットワークでホストとの通信異常を検知してホストマイグレーションを開始する場合があります。
            ローカルネットワークで通信異常と判断する期間は Session の最大無通信時間とは無関係となりますので、注意してください。
    @endif

            ビットレート判定機能を有効にしている場合には、@ref StartupSetting 構造体の uplinkBitRateLowerLimit, bitRateCheckPacketSize を設定する必要があります。
            デフォルト値はいずれも無効な値になっているため、uplinkBitRateLowerLimit にはアプリケーションが要求するビットレートの水準 (単位は bps) を、bitRateCheckPacketSize には測定に用いる IP パケットサイズ (単位は byte) を設定する必要があります。

    @param[in]  startupSetting     スタートアップ時の各種設定項目

    @retval ResultInvalidState     関数が実行可能な状態ではありません。既にスタートアップ済みの場合などに返ります。[:progErr]
    @retval ResultInvalidArgument  ステーションと関連付ける名前設定かパケットの署名機能設定に問題がある、もしくは、引数で指定したポインタが無効なポインタです。[:progErr]

    @see    Cleanup, transport::Station::GetIdentificationToken, RegisterJoiningMeshApprovalCallback
     */
    Result Startup(Mesh::StartupSetting& startupSetting);

    /*!
    @brief  Mesh をクリーンアップします。

    @details @ref Cleanup 後に再度 Mesh を使用する場合には、再度 @ref Startup を呼び出す必要があります。<br>
            インターネット通信の場合は NAT セッションの開始（ @ref nn::pia::inet::NexFacade::StartNatSessionAsync ）からやり直す必要があります。

    @see    Startup
     */
    void Cleanup();

    /*!
     end of name 初期化終了処理
     @}
     */

    /*!
     @name メッシュの構築、破棄 及び 参加、離脱
     @{
     */

    /*!
    @cond PRIVATE
    @brief メッシュを構築します。（旧仕様）

    @details 自分がメッシュホストになります。
    <br>
           非同期のメッシュ構築処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           引数で指定する AsyncContext をチェックすることで、処理の完了を監視することができます。

           事前に @ref nn::pia::transport::Transport の準備が完了している必要があります。

            【非同期処理失敗時に AsyncContext を通して返される可能性のある Result】
            <ul>
            <li> ResultInvalidState             ：この API を使用できる状態ではありませんでした。[:progErr]</li>
            </ul>

    @param[in]  pAsyncContext            処理の進行状況をチェックするための AsyncContext のポインタを指定します。

    @return  正常にメッシュ構築処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState    この API を使用できる状態ではありません。[:progErr]
    @retval  ResultInvalidArgument 引数で指定された AsyncContext が、使用できる状態ではありませんでした。[:progErr]

    @see    DestroyMesh, LeaveMeshWithHostMigration
     */
    Result CreateMesh(common::AsyncContext* pAsyncContext);
    //!@endcond

    /*!
    @brief メッシュを構築します。

    @details 自分がメッシュホストになります。
    <br>
           非同期のメッシュ構築処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsCreateMeshCompleted でチェックし、結果は @ref GetCreateMeshResult で確認できます。

           事前に @ref nn::pia::transport::Transport の準備が完了している必要があります。

    @return  正常にメッシュ構築処理を開始できた場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState    この API を使用できる状態ではありません。[:progErr]

    @see    IsCreateMeshCompleted, GetCreateMeshResult
    @see    DestroyMeshAsync, LeaveMeshWithHostMigrationAsync
     */
    Result CreateMeshAsync();

    /*!
    @brief CreateMeshAsync の非同期処理の完了をチェックします。

    @details @ref CreateMeshAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    CreateMeshAsync, GetCreateMeshResult
     */
    bool IsCreateMeshCompleted();

    /*!
    @brief CreateMeshAsync の非同期処理の結果を取得します。

    @details @ref CreateMeshAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return CreateMeshAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultInvalidState          CreateMeshAsync を呼び出すタイミングが間違っている場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @if NIN_DOC
    @retval ResultUserAccountNotExisted    ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @endif

    @see    CreateMeshAsync, IsCreateMeshCompleted
     */
    Result GetCreateMeshResult();


    /*!
    @cond PRIVATE
    @brief メッシュを破棄します。（旧仕様）

    @details メッシュホストのみ使用可能です。
    <br>
           非同期のメッシュ破棄処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           引数で指定する AsyncContext をチェックすることで、処理の完了を監視することができます。<br>
           ※ 正常にメッシュ破棄処理が開始できた場合、非同期処理に失敗することはありません。

           メッシュ破棄処理内で、参加者全員にメッシュを終了させることを通知します。<br>
           通知を受けたステーションは、Mesh の内部状態が初期化され State_Disconnected の状態になります。

    @param[in]  pAsyncContext            処理の進行状況をチェックするための AsyncContext のポインタを指定します。

    @return  正常にメッシュ破棄処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState    メッシュホスト以外が呼び出した場合に返されます。[:progErr]
    @retval  ResultInvalidArgument 引数で指定された AsyncContext が、使用できる状態ではありませんでした。[:progErr]

    @see    CreateMesh, LeaveMeshWithHostMigration
     */
    Result DestroyMesh(common::AsyncContext* pAsyncContext);
    //!@endcond

    /*!
    @brief メッシュを破棄します。

    @details メッシュホストのみ使用可能です。
    <br>
           非同期のメッシュ破棄処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsDestroyMeshCompleted でチェックし、結果は @ref GetDestroyMeshResult で確認できます。<br>
           ※ 正常にメッシュ破棄処理が開始できた場合、非同期処理に失敗することはありません。

           メッシュ破棄処理内で、参加者全員にメッシュを終了させることを通知します。<br>
           通知を受けたステーションは、Mesh の内部状態が初期化され State_Disconnected の状態になります。

    @return  正常にメッシュ破棄処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState    メッシュホスト以外が呼び出した場合、もしくは既に他の非同期処理が開始されている場合に返ります。[:progErr]

    @see    IsDestroyMeshCompleted, GetDestroyMeshResult
    @see    CreateMeshAsync, LeaveMeshWithHostMigrationAsync
     */
    Result DestroyMeshAsync();

    /*!
    @brief DestroyMeshAsync の非同期処理の完了をチェックします。

    @details @ref DestroyMeshAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    DestroyMeshAsync, GetDestroyMeshResult
     */
    bool IsDestroyMeshCompleted();

    /*!
    @brief DestroyMeshAsync の非同期処理の結果を取得します。

    @details    @ref DestroyMeshAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>
                ※ 正常にメッシュ破棄処理が開始できた場合、非同期処理に失敗することはありません。

    @return DestroyMeshAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultInvalidState          非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]

    @see    DestroyMeshAsync, IsDestroyMeshCompleted
     */
    Result GetDestroyMeshResult();


    /*!
    @cond PRIVATE
    @brief メッシュから離脱します。同時にメッシュ参加者にホストマイグレーション処理の開始を指示します。（旧仕様）

    @details メッシュホストのみ使用可能です。また、ホストマイグレーションが有効な場合のみ使用可能です。
    <br>
           非同期のメッシュ離脱処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           引数で指定する AsyncContext をチェックすることで、処理の完了を監視することができます。<br>
           ※ 正常にメッシュ離脱処理が開始できた場合、非同期処理に失敗することはありません。

           非同期処理内で、参加者全員にホストマイグレーション処理の開始指示を通知します。<br>
           通知を受けたステーションではホストマイグレーション処理が行われ、この API を呼び出したメッシュホストとの通信が切断した後も、
           新たなメッシュホストの下でメッシュが継続されます。

    @return  正常にメッシュ離脱処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState    メッシュホスト以外が呼び出した、もしくはホストマイグレーションが無効な場合に返されます。[:progErr]
    @retval  ResultInvalidArgument 引数で指定された AsyncContext が、使用できる状態ではありませんでした。[:progErr]

    @see    CreateMesh, DestroyMesh
     */
    Result LeaveMeshWithHostMigration(common::AsyncContext* pAsyncContext);
    //!@endcond

    /*!
    @brief メッシュから離脱します。同時にメッシュ参加者にホストマイグレーション処理の開始を指示します。

    @details メッシュホストのみ使用可能です。また、ホストマイグレーションが有効な場合のみ使用可能です。
    <br>
           非同期のメッシュ離脱処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsLeaveMeshWithHostMigrationCompleted でチェックし、結果は @ref GetLeaveMeshWithHostMigrationResult で確認できます。<br>
           ※ 正常にメッシュ離脱処理が開始できた場合、非同期処理に失敗することはありません。

           非同期処理内で、参加者全員にホストマイグレーション処理の開始指示を通知します。<br>
           通知を受けたステーションではホストマイグレーション処理が行われ、この API を呼び出したメッシュホストとの通信が切断した後も、
           新たなメッシュホストの下でメッシュが継続されます。

    @return  正常にメッシュ離脱処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState    メッシュホスト以外が呼び出した場合、もしくはホストマイグレーションが無効な場合、あるいは既に他の非同期処理が開始されている場合に返ります。[:progErr]

    @see    IsLeaveMeshWithHostMigrationCompleted, GetLeaveMeshWithHostMigrationResult
    @see    CreateMeshAsync, DestroyMeshAsync
     */
    Result LeaveMeshWithHostMigrationAsync();

    /*!
    @brief LeaveMeshWithHostMigrationAsync の非同期処理の完了をチェックします。

    @details @ref LeaveMeshWithHostMigrationAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    LeaveMeshWithHostMigrationAsync, GetLeaveMeshWithHostMigrationResult
     */
    bool IsLeaveMeshWithHostMigrationCompleted();

    /*!
    @brief LeaveMeshWithHostMigrationAsync の非同期処理の結果を取得します。

    @details    @ref LeaveMeshWithHostMigrationAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>
                ※ 正常にメッシュ破棄処理が開始できた場合、非同期処理に失敗することはありません。

    @return LeaveMeshWithHostMigrationAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultInvalidState          非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]

    @see    LeaveMeshWithHostMigrationAsync, IsLeaveMeshWithHostMigrationCompleted
     */
    Result GetLeaveMeshWithHostMigrationResult();


    /// @cond PRIVATE
    /*!
    @brief Join要求を許可するかどうかを返します。

    @return  Join許可ならば0xFF、拒否ならばそれ以外の値が返されます。
     */
    uint8_t CheckApprovalJoin(transport::Station* pTargetStation);
    /// @endcond


    /*!
    @cond PRIVATE
    @brief メッシュに参加します。（旧仕様）

    @details メッシュへの非同期参加処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            第2引数で指定する AsyncContext をチェックすることで、処理の完了と成否を監視することができます。
            また、処理中は @ref nn::pia::common::AsyncContext::Cancel を呼び出すことで、処理を中止することができます。

            【非同期処理完了時に AsyncContext を通して返される可能性のある Result】
            <ul>
            <li> ResultSuccess                            ：非同期処理に成功しました。</li>
            <li> ResultCancelled                          ：非同期処理がキャンセルされました。[:handling-cleanup]</li>
            <li> ResultInvalidState                       ：接続可能な状態ではありませんでした。[:progErr]</li>
            <li> ResultStationConnectionFailed            ：接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]</li>
            <li> ResultStationConnectionNatTraversalFailedUnknown ：NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]</li>
            <li> ResultNatTraversalFailedBothEim            ：NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]</li>
            <li> ResultNatTraversalFailedLocalEimRemoteEdm  ：NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]</li>
            <li> ResultNatTraversalFailedLocalEdmRemoteEim  ：NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]</li>
            <li> ResultNatTraversalFailedBothEdm            ：NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]</li>
            <li> ResultNatTraversalRequestTimeout           ：NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]</li>
            <li> ResultJoinRequestDenied                  ：メッシュホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]</li>
            <li> ResultMeshIsFull                      ：参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]</li>
            <li> ResultRelayFailedNoCandidate       ：リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]</li>
            <li> ResultRelayFailedRttLimit          ：リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]</li>
            <li> ResultRelayFailedRelayNumLimit     ：リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]</li>
            <li> ResultRelayFailedUnknown           ：リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]</li>
            <li> ResultInvalidSystemMessage               ：メッシュホストから不正な応答が届いたため、処理を中断しました。[:progErr]</li>
            <li> ResultIncompatibleFormat      ：相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]</li>
            </ul>

            処理に失敗（もしくは @ref nn::pia::common::AsyncContext::Cancel で処理を中止）した後に Mesh インスタンスの破棄などをせずに別の処理を行う場合、 @ref Cleanup を呼び出して
            Mesh をクリーンアップしてから行う必要があります。<br>
            インターネット通信の場合は NAT セッションの開始（ @ref nn::pia::inet::NexFacade::StartNatSession ）からやり直す必要があります。

    @param[in]  stationConnectionInfo   参加したいメッシュのホストの接続用情報を指定します。
    @param[in]  pAsyncContext            処理の進行状況をチェックするための AsyncContext のポインタを指定します。

    @return  正常にメッシュへの参加処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidArgument          引数で指定された AsyncContext が使用できる状態ではありませんでした。[:progErr]
    @retval  ResultInvalidState             既にメッシュ参加済み、もしくは JoinMesh 非同期処理開始済みでした。[:progErr]

    @see    LeaveMesh
     */
    Result JoinMesh(const transport::StationConnectionInfo& stationConnectionInfo, common::AsyncContext* pAsyncContext);
    //!@endcond

    /*!
    @brief メッシュに参加します。

    @details メッシュへの非同期参加処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            処理の完了は @ref IsJoinMeshCompleted でチェックし、結果は @ref GetJoinMeshResult で確認できます。<br>
            また、処理中は @ref CancelJoinMeshAsync を呼び出すことで、処理を中止することができます。

            処理に失敗（もしくは @ref CancelJoinMeshAsync で処理を中止）した後に Mesh インスタンスの破棄などをせずに別の処理を行う場合、 @ref Cleanup を呼び出して
            Mesh をクリーンアップしてから行う必要があります。<br>
            インターネット通信の場合は NAT セッションの開始（ @ref nn::pia::inet::NexFacade::StartNatSessionAsync ）からやり直す必要があります。

    @param[in]  stationConnectionInfo   参加したいメッシュのホストの接続用情報を指定します。

    @return  正常にメッシュへの参加処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState     既にメッシュ参加済み、もしくは既に他の非同期処理が開始されている場合に返ります。[:progErr]

    @see    IsJoinMeshCompleted, GetJoinMeshResult, CancelJoinMeshAsync
    @see    LeaveMeshAsync
     */
    Result JoinMeshAsync(const transport::StationConnectionInfo& stationConnectionInfo);

    /*!
    @brief JoinMeshAsync の非同期処理の完了をチェックします。

    @details @ref JoinMeshAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    JoinMeshAsync, GetJoinMeshResult, CancelJoinMeshAsync
     */
    bool IsJoinMeshCompleted();

    /*!
    @brief JoinMeshAsync の非同期処理の結果を取得します。

    @details    @ref JoinMeshAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return JoinMeshAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。それ以外では、以下の Result を返します。
    @retval ResultCancelled                     非同期処理がキャンセルされました。[:handling-cleanup]
    @retval ResultInvalidState                  接続可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval ResultStationConnectionFailed       接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultStationConnectionNatTraversalFailedUnknown    NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEim             NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEimRemoteEdm   NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedLocalEdmRemoteEim   NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalFailedBothEdm             NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval ResultNatTraversalRequestTimeout            NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]
    @retval ResultJoinRequestDenied             メッシュホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval ResultMeshIsFull                 参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedNoCandidate        リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRttLimit           リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedRelayNumLimit      リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval ResultRelayFailedUnknown            リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval ResultInvalidSystemMessage          メッシュホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval ResultIncompatibleFormat 相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @if NIN_DOC
    @retval ResultUserAccountNotExisted    ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @endif

    @see    JoinMeshAsync, IsJoinMeshCompleted, CancelJoinMeshAsync
     */
    Result GetJoinMeshResult();

    /*!
    @brief    JoinMeshAsync の非同期処理をキャンセルします。

    @details    JoinMeshAsync 開始された非同期処理は、本関数でキャンセルできます。
                キャンセル処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                本関数を呼び出した後、ライブラリは速やかに非同期処理のキャンセルを行いますが、
                キャンセルの完了は @ref IsJoinMeshCompleted() でチェックする必要があります。

    @return   JoinMeshAsync の非同期処理のキャンセル結果を返します。処理が成功していた場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   ResultInvalidState    JoinMeshAsync の非同期処理が行われていないタイミングで呼び出しています。[:progErr]

    @see    JoinMeshAsync, IsJoinMeshCompleted, GetJoinMeshResult
    */
    Result CancelJoinMeshAsync();


    /*!
    @cond PRIVATE
    @brief 参加しているメッシュから離脱します。（旧仕様）

    @details メッシュクライアントのみ使用可能です。
    <br>
            メッシュからの非同期離脱処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            引数で指定する AsyncContext をチェックすることで、処理の完了と成否を監視することができます。

    @param[in]  pAsyncContext    処理の進行状況をチェックするための AsyncContext のポインタを指定します。

    @return  正常にメッシュからの離脱処理を開始できた場合、成功の Result が返されます。
    @retval  ResultInvalidState    メッシュホストが呼び出した場合に返されます。[:progErr]
    @retval  ResultInvalidArgument 引数で指定された AsyncContext が、使用できる状態ではありませんでした。[:progErr]

    @see    JoinMesh
     */
    Result LeaveMesh(common::AsyncContext* pAsyncContext);
    //!@endcond

    /*!
    @brief 参加しているメッシュから離脱します。

    @details メッシュクライアントのみ使用可能です。
    <br>
            メッシュからの非同期離脱処理を開始します。開始した処理は @ref nn::pia::common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            処理の完了は @ref IsJoinMeshCompleted でチェックし、結果は @ref GetJoinMeshResult で確認できます。<br>
            ※ 正常にメッシュ離脱処理が開始できた場合、非同期処理に失敗することはありません。

    @return  正常にメッシュからの離脱処理を開始できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState    メッシュホストが呼び出した場合、もしくは既に他の非同期処理が開始されている場合に返ります。[:progErr]

    @see    IsLeaveMeshCompleted, GetLeaveMeshResult
    @see    JoinMeshAsync
     */
    Result LeaveMeshAsync();

    /*!
    @brief LeaveMeshAsync の非同期処理の完了をチェックします。

    @details @ref LeaveMeshAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    LeaveMeshAsync, GetLeaveMeshResult
     */
    bool IsLeaveMeshCompleted();

    /*!
    @brief LeaveMeshAsync の非同期処理の結果を取得します。

    @details    @ref LeaveMeshAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。<br>
                ※ 正常にメッシュ破棄処理が開始できた場合、非同期処理に失敗することはありません。

    @return LeaveMeshAsync の非同期処理の結果を返します。 処理が成功していた場合、 IsSuccess() が真となる Result を返します。処理が失敗していた場合、以下の Result を返します。
    @retval ResultInvalidState          非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]

    @see    LeaveMeshAsync, IsLeaveMeshCompleted
     */
    Result GetLeaveMeshResult();


    /*!
    @brief メッシュから任意のステーションを追放します。

    @details メッシュホストのみ使用可能です。
    <br>
            引数で指定した StationId が割り当てられたステーションへ追放通知を伝達し、
            追放通知を受けた対象のステーションではメッシュ離脱処理が行われます。

            追放通知の通信遅延などが原因で、この関数呼出に成功した後実際にメッシュから離脱するまでにある程度の時間がかかる場合があります。

    @param[in]  id  追放したいステーションに割り当てられている StationId を指定します。

    @return  正常にメッシュからのステーション追放処理を実行できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  ResultInvalidState     メッシュホスト以外が呼び出した場合に返されます。[:progErr]
    @retval  ResultInvalidArgument  引数で指定された StationId が、ホスト自身を示す Id やメッシュに参加していない Id だった場合に返されます。[:progErr]
    @retval  ResultBufferIsFull     追放処理管理用バッファもしくは追放通知送信用バッファに空きが無かった場合に返されます。時間をあけてから再度実行すれば、成功する可能性があります。[:handling]
     */
    Result KickoutStation(StationId id);

    /// @cond PRIVATE
    Result KickoutStationByIndex(StationIndex index);
    /// @endcond

    /*!
     end of name メッシュの構築、破棄 及び 参加、離脱
     @}
     */

    /*!
     @name メッシュ解析
     @{
     */

    /*!
    @brief 現在メッシュに接続している有効なステーション数を取得します。

    @return 接続中のステーション数。
     */
    uint16_t GetStationNum() const;


    /*!
    @brief メッシュホストの StationId を取得します。

    @details メッシュホストが確定していない場合は StationIdInvalid が返ります。

    @return  メッシュホストを示す StationId。
     */
    StationId GetHostStationId() const;


    /*!
    @brief 自分自身を示す StationId を取得します。

    @details 自分自身の StationId が確定していない場合は StationIdInvalid が返ります。

    @return  自分自身を示す StationId。
     */
    StationId GetLocalStationId() const;


    /// @cond PRIVATE
    StationIndex GetHostStationIndex() const;
    StationIndex GetLocalStationIndex() const;
    /// @endcond


    /*!
    @brief 自分がメッシュホストであるかどうかを取得します。

    @details メッシュに参加していない状態であれば、常に false が返されます。

    @return  自分がメッシュホストであれば true、そうでなければ false が返されます。
     */
    bool IsHost() const;


    /*!
    @brief 任意の StationId がメッシュ内で現在有効かどうかを確認します。

    @param[in] id   確認したい StationId を指定します。

    @return 指定された StationId が有効な場合は true、そうでなければ false が返ります。
     */
    bool IsValidStationId(StationId id) const;


    /// @cond PRIVATE
    bool IsValidStationIndex(StationIndex index) const;

    /*!
    @brief メッシュ内で現在未使用の StationIndex を取得します。

    @return  未使用の有効な StationIndex が発見できなければ StationIndex_Invalid が返ります。
     */
    StationIndex GetUsableStationIndex();


    /*!
    @brief メッシュ内で現在有効な StationIndex を示す Bitmap を取得します。

    @details StationIndex との対応ビットは、StationIndex_1 が 0x00000001, StationIndex_2 が 0x00000002 … という順序でシフトします。

    @return  メッシュ内で現在有効な StationIndex に対応するビットが立った状態のビットマップが返ります。
     */
    uint32_t GetValidStationIndexBitmap() const;
    /// @endcond

    /*!
    @brief 現在の Mesh の状態が通信可能かどうか確認します。

    @details 現在の Mesh が通信可能な状態かどうかを Result で示します。通信不可能な状態だった場合、 @ref GetDisconnectReason を使用して通信不可能になった詳細な理由を確認可能です。

    @return  通信可能な状態の場合は IsSuccess() が真となる Result を返します。それ以外では以下の Result を返します。
    @retval  ResultMeshConnectionIsLost  Mesh が通信不可能な状態です。 [:handling-cleanup]

    @see    GetDisconnectReason
     */
    Result CheckConnectionError() const;


    /*!
    @brief Mesh が通信不可能となった理由を取得します。

    @details Mesh が通信可能な状態だった場合は、常に UNKNOWN_REASON が返されます。

    @return  Mesh が通信不可能となった理由を表す DisconnectReason

    @see    CheckConnectionError
     */
    DisconnectReason GetDisconnectReason() const;

    /*!
    @brief  ビットレート判定機能でのビットレート値が取得できるかどうかを取得します。

    @return ビットレート値が取得できる状態になっていれば true を返します。タイムアウトやキャンセル等の理由で測定が失敗に終わった場合も true が返されます。
     */
    bool IsBitRateCheckCompleted() const;

    /*!
    @brief   ビットレート判定機能によって計測された、アップリンク方向の回線のビットレートを取得します。

    @details この API 呼び出しで得られる値は、測定処理において単位時間あたりに受信できたデータ量にすぎず、
             必ずしも回線の真の実力を表すものではないことに注意してください。
             実際には高ビットレートが期待できる回線であったとしても、アプリケーション側の要求水準が
             控えめなものであれば、測定処理におけるデータ送受信量も控えめなものとなるので、
             取得できる bps 値は小さめなものとなります。

    @param[out] pBps 測定されたビットレート値がセットされます。単位は bps です。測定値が得られなかった場合は負の値がセットされます。

    @return  呼び出しが成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval  ResultNotInitialized     インスタンスが初期化されていません。[:progErr]
    @retval  ResultInvalidArgument    引数が誤っています。[:progErr]
    @retval  ResultInvalidState       この関数を呼び出すタイミングが誤っています。ビットレート判定機能を利用しない設定であるにもかかわらず、本 API を呼び出している可能性があります。[:progErr]
     */
    Result GetUplinkBitRate(int32_t* pBps);

    /*!
     end of name メッシュ解析
     @}
     */

    /*!
     @name コールバック関数の登録、登録解除
     @{
     */

    /*!
    @brief  メッシュの状態が変化した場合に呼び出されるコールバック関数を登録します。

    @details メッシュの状態変化（ステーション参加、離脱）発生時に呼び出されるコールバック関数を登録します。
    <br>
            メッシュ状態変化コールバック関数は、基本的に @ref nn::pia::common::Scheduler::Dispatch の内部で呼び出されます。<br>
            例外として、@ref CreateMeshAsync 、 @ref DestroyMeshAsync 呼出時にも発生します。 @ref Cleanup 呼出時にも発生する可能性があります。

            登録する関数内には長時間ブロックするような処理を記述してはいけません。<br>
            また、登録する関数内で session モジュールの関数は呼び出してはいけません。

    @param[in]  callback    登録するコールバック関数へのポインタを指定します。

    @see    UnregisterMeshEventCallback
     */
    void RegisterMeshEventCallback(MeshEventCallback callback);


    /*!
    @brief  登録されているメッシュ状態変化コールバック関数を登録解除します。

    @see    RegisterMeshEventCallback
     */
    void UnregisterMeshEventCallback();


    /*!
    @brief  メッシュホストでジョイン要求が来た際に呼び出される承認、拒否判定コールバックを登録します。

    @details ジョイン要求承認、拒否判定コールバック関数は、@ref nn::pia::common::Scheduler::Dispatch の内部で呼び出されます。
    <br>
            登録する関数内には長時間ブロックするような処理を記述してはいけません。

            コールバック関数の引数で、ジョイン要求送信元ステーションの識別トークンが渡されます。

            コールバック関数を登録しなかった場合、ジョイン要求はメッシュ満員時以外承認されます。

    @param[in]  callback    登録するコールバック関数へのポインタを指定します。

    @see    Startup, UnregisterJoiningMeshApprovalCallback
     */
    void RegisterJoiningMeshApprovalCallback(JoiningMeshApprovalCallback callback);


    /*!
    @brief  登録されているジョイン要求承認、拒否判定コールバック関数を登録解除します。

    @see    RegisterJoiningMeshApprovalCallback
     */
    void UnregisterJoiningMeshApprovalCallback();


    /// @cond PRIVATE
    /*!
    @brief  複数候補版ホストマイグレーションで、次期親候補順位を任意に調整するためのコールバック関数を登録します。

    @details 次期親候補順位を任意に調整するためのコールバック関数は、@ref nn::pia::common::Scheduler::Dispatch の内部で呼び出されます。
    <br>
            登録する関数内には長時間ブロックするような処理を記述してはいけません。

    @param[in]  callback    登録するコールバック関数へのポインタを指定します。
    @return 複数候補版ホストマイグレーションが有効な状態で呼び出せば IsSuccess() が真となる Result が返ります。

    @see    UnregisterCheckHostCandidateRankingCallback
     */
    Result RegisterCheckHostCandidateRankingCallback(CheckHostCandidateRankingCallback callback);


    /*!
    @brief  登録されている次期親候補順位を任意に調整するためのコールバック関数を登録解除します。

    @see    RegisterCheckHostCandidateRankingCallback
     */
    void UnregisterCheckHostCandidateRankingCallback();
    /// @endcond


    /// @cond PRIVATE
    void RegisterEventListener(MeshEventListener* pEventListener);
    void UnregisterEventListener();

    void RegisterGetSessionIdCallback(GetSessionIdCallback callback);
    void UnregisterGetSessionIdCallback();
    /// @endcond


    /*!
     end of name コールバック関数の登録、登録解除
     @}
     */


    /// @cond PRIVATE
    /*!
    @brief システムが自動生成した MeshProtocol を取得します。

    @return  MeshProtocol のポインタ。
     */
    MeshProtocol* GetMeshProtocol();
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief システムが自動生成した StationProtocol を取得します。

    @return  StationProtocol のポインタ。
     */
    transport::StationProtocol* GetStationProtocol();
    /// @endcond


    /*!
     @name 同期クロック
     @{
     */

    /*!
    @brief      メッシュ作成時刻からの経過時間（同期クロック）をミリ秒で返します。

    @details 同期クロックは、メッシュに参加している Station 間で同期される時計です。
                メッシュに参加したクライアントは、定期的に時刻同期パルスをメッシュホストに送信します。
                ホストがこのパルスに応答することで、ホストと各クライアントの時刻同期が実現されます。
                この同期パルス送信間隔はデフォルトで 2 秒ですが、アプリケーションがこの値を
                変更することもできます。詳しくは SetSyncClockPulseInterval(), GetSyncClockPulseInterval()
                を参照してください。

    @return     メッシュ作成時刻からの経過時間をミリ秒で返します。 メッシュホストとの同期が取れていない
                メッシュクライアントがこの API を呼び出したときは、負の値が返されます。
    @see        SetSyncClockPulseInterval, GetSyncClockPulseInterval
     */
    int64_t GetTime() const;


    /*!
    @brief      同期クロックの同期パルス送信間隔を指定します。

    @param[in]  interval 同期パルス送信の時間間隔です。単位はミリ秒です。
    @see        GetTime, GetSyncClockPulseInterval
     */
    void SetSyncClockPulseInterval(int32_t interval);


    /*!
    @brief      同期クロックの同期パルス送信間隔を取得します。

    @return     同期パルス送信間隔を取得します。単位はミリ秒です。
    @see        GetTime, SetSyncClockPulseInterval
     */
    int32_t GetSyncClockPulseInterval() const;

    /*!
     end of name 同期クロック
     @}
     */


    /*!
     @name リレー通信機能
     @{
     */

    /*!
    @brief      リレー接続有効時に、ステーション1台に割り当てられる転送依頼数の最大制限値を設定します。

    @details    リレー接続経路探索＆決定時の指標として本関数で設定された制限値を利用します。<br>
                設定変更しない場合のデフォルト値は "[メッシュ最大参加数] * [メッシュ最大参加数]" です。<br>

                同じメッシュへの参加者は全員同じ値を設定することを推奨します。（異なる値を設定した場合は動作保証外とします）

                リレー接続が有効なネットワークトポロジーの場合かつ @ref Startup 前にのみ使用可能です。

    @param[in]  num  設定したい最大制限値を指定します。

    @return     正常に設定できた場合、成功の Result が返されます。
    @retval     ResultInvalidState    リレー接続が有効ではない、もしくは既に Startup 呼出後の場合に返されます。[:progErr]
     */
    Result SetMaxNumOfRelayRoutesAssignedToStation(uint16_t num);


    /*!
    @brief      リレー接続有効時に、1つのリレー接続経路の合計 RTT の最大制限値を設定します。

    @details    リレー接続経路探索＆決定時の指標として本関数で設定された制限値を利用します。<br>
                設定変更しない場合のデフォルト値は nn::pia::session::SessionMaxRttDefault です。<br>

                同じメッシュへの参加者は全員同じ値を設定することを推奨します。（異なる値を設定した場合は動作保証外とします）

                リレー接続が有効なネットワークトポロジーの場合かつ @ref Startup 前にのみ使用可能です。

    @param[in]  rtt  設定したい最大制限値を指定します。単位は msec です。

    @return     正常に設定できた場合、成功の Result が返されます。
    @retval     ResultInvalidState    リレー接続が有効ではない、もしくは既に Startup 呼出後の場合に返されます。[:progErr]
     */
    Result SetMaxRtt(uint16_t rtt);

    /*!
    @brief      （デバッグ用）リレー接続有効時に、任意の StationIndex と自分自身との接続経路情報を取得します。

    @details    本関数はデバッグ用です。製品には組み込まないようお願いします。<br>
                自分自身と targetIndex の接続経路情報について、以下の規則で pRelayIndex へ格納します。
                （接続経路情報が取得できなかった場合は pRelayIndex の値を変更することはありません）
                <ul>
                <li>直通接続の場合 : targetIndex</li>
                <li>リレー接続の場合 : リレー依頼先の StationIndex</li>
                <li>自分自身の場合 : StationIndex_Invalid</li>
                <li>接続していない場合 : StationIndex_Invalid</li>
                </ul>

    @param[in]  targetIndex 接続経路を取得したい対象の StationIndex を指定します。
    @param[out] pRelayIndex 接続経路情報を示す StationIndex が格納されます。

    @return     正常に取得できた場合、成功の Result が返されます。
    @retval     ResultInvalidState      リレー接続が有効ではない場合に返されます。[:progErr]
    @retval     ResultInvalidArgument   対象の StationIndex が有効な値ではない場合、もしくは引数で指定されたポインタが有効ではない場合に返ります。[:progErr]
     */
    Result GetConnectionRoute(StationIndex targetIndex, StationIndex* pRelayIndex);

    /*!
    @brief      （デバッグ用）リレー接続有効時に、任意の StationIndex 間の接続経路エミュレーション設定を指定します。

    @details    本関数はデバッグ用です。製品には組み込まないようお願いします。

    @details    本関数を使用する場合は @ref Mesh::SetDebugSetting で接続経路エミュレーション機能が有効となるように設定する必要があります。

    @details    EmulationType_Relay を設定した場合、対象の StationIndex が接続した際の接続方法は
                （たとえ実際には直通接続ができる場合でも）必ずリレー接続となります。

    @details    本関数はメッシュに接続していないタイミングで呼び出す必要があります。
                メッシュ参加済みの状態で接続経路エミュレーション設定を変更することは動作保証外とします。

    @param[in]  index1  対象の StationIndex を指定します。
    @param[in]  index2  対象の StationIndex を指定します。
    @param[in]  type    接続方法エミュレーション設定を指定します。

    @return     正常に設定できた場合、成功の Result が返されます。
    @retval     ResultInvalidState      リレー接続が有効ではない場合に返されます。[:progErr]
    @retval     ResultInvalidArgument   対象の StationIndex が有効な値ではない場合、もしくは EmulationType に無効な値を指定した場合に返ります。[:progErr]

    @see    ResetRelayEmulation
     */
    Result RelayEmulation(StationIndex index1, StationIndex index2, EmulationType type);

    /*!
    @brief      （デバッグ用）リレー接続有効時に、接続経路エミュレーション設定を初期化します。

    @details    本関数はデバッグ用です。製品には組み込まないようお願いします。

    @details    Mesh::RelayEmulation を使用して設定された接続経路エミュレーションについて全て初期化します。
                （全ての StationIndex の組み合わせに対して EmulationType_None を設定した状態と同等です）

    @details    本関数はメッシュに接続していないタイミングで呼び出す必要があります。
                メッシュ参加済みの状態で接続経路エミュレーション設定を変更することは動作保証外とします。

    @see    RelayEmulation
    */
    void ResetRelayEmulation();

    /*!
     end of name リレー通信機能
     @}
     */


    /// @cond PRIVATE
    /*!
    @brief      ホストマイグレーション処理の開始判断のフラグを設定します。
    @param[in]  val 設定後のフラグの状態を指定します。
     */
    void SetHostMigrationStarted(bool val);
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief      ホストマイグレーション処理の開始判断のフラグを取得します。
    @return     フラグの状態が返ります。
     */
    bool IsHostMigrationStarted();
    /// @endcond

    /// @cond PRIVATE
    ProcessHostMigrationJob* GetProcessHostMigrationJobPtr();
    KickoutManageJob* GetKickoutManageJobPtr();
    ProcessDestroyMeshJob* GetProcessDestroyMeshJobPtr();
    void SetFatalError(DisconnectReason reason);
    ProcessUpdateMeshJob* GetProcessUpdateMeshJobPtr();
    NetworkTopology GetNetworkTopologyType();
    const SignatureSettingStorage* GetSignatureSettingStorage() const;
    static Result SetDebugSettingForInternal(const DebugSetting& debugSetting);
    void NotifyLeaveStationAddress(const common::StationAddress& leftStationAddress);
    /// @endcond

    /*!
     @name その他
     @{
     */

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void TraceStationConnectionInfoList(uint64_t flag);

    /// @cond PRIVATE
    bool IsHostMigrationEnabled() const;
    HostMigrationMode GetHostMigrationMode() const;
    int GetJoinMeshSequence() const;
    int GetHostMigrationSequence() const;
    void SetSkipCleanupReport(bool isSkip);
    void SetSkipReport(bool isSkip);
    void SendReport(uint8_t type);
    void SetupSendReport();
    bool IsProcessedAtMeshBegin() const;
    bool IsProcessingLeaveMesh() const;
    bool IsProcessingHostMigration() const;
    State GetMeshState() const;
    int32_t GetProcessUpdateMeshJobStepSequence() const;
    uint8_t GetLocalPlayerCount() const;
    bool IsPlayersCountedAsParticipants() const;
    bool IsRecentPlayRecordEnabled() const;
    bool IsMultiPlayerEnabled() const;
    Result CheckPlayerInfo(const transport::Station::PlayerInfo& playerInfo) const;
    Result CheckPlayerInfoSetting(const transport::Station::PlayerInfo* pPlayerInfo, uint8_t playerInfoSize) const;
    void CleanupMonitoringBeginData();
    common::MonitoringDataSender* GetMonitoringDataSender();
    BitRateCheckMode GetBitRateCheckMode();
    transport::BandwidthCheckProtocol* GetBandwidthCheckProtocol();
    bool IsBandwidthCheckProtocolSetupParamValid(int32_t uplinkBitRateLowerLimit, uint32_t bitRateCheckPacketSize, int32_t bitRateMeasuringSpan) const;
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief      CheckApprovalJoin で参加要求をエラーとして拒否するフラグをセットする（要求側では ResultJoinRequestDenied で参加失敗）
    */
    void SetDenyJoinRequestByInvalidState(bool isDeny);   
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief      CheckApprovalJoin で参加要求を満員のため拒否するフラグをセットする（要求側では ResultSessionIsFull で参加失敗）
    */
    void SetDenyJoinRequestBySessionIsFull(bool isFull);
    /// @endcond

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    /*!
     end of name その他
     @}
     */

protected:
    static Mesh* s_pMesh;
    static DebugSetting s_DebugSetting;
    static Result setDebugSettingImpl(const DebugSetting& debugSetting);

private:
    /// @cond PRIVATE
    /*!
    @brief コンストラクタ（使用禁止）
     */
    Mesh(const session::Mesh::Setting& setting);
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief デストラクタ（使用禁止）
     */
    virtual ~Mesh();
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief コピーコンストラクタは封印します。
     */
    Mesh(const Mesh&);
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief 代入演算子は封印します。
     */
    Mesh& operator=(const Mesh&);
    /// @endcond

    Result Initialize(const session::Mesh::Setting& setting, bool isRelayEmulationEnabled);

    Result SetupLocalPlayerInfo();
    Result SetupProtocols();  // システムプロトコルの準備

    void CleanupStatus();  // 内部状態を初期化する Mesh終了時に呼ぶ
    void CleanupJobs();    // Meshが管理するJobを初期化する Mesh終了時に呼ぶ

    void CleanupStationsJobs();

    void FixConnectedId(StationIndex idx);
    void UnfixDisconnectedId(StationIndex idx);

    void StartUse(StationIndex idx);  // 使用中に変える
    void StopUse(StationIndex idx);   // 未使用に変える
    void JoinResponseComplete(StationIndex idx);
    void ResetStationIndexUsage();  // StationIndex の使用状況をリセット

    // メッシュ構築、参加に成功した時刻を記録します。
    void SetMeshStartTime(common::Time startTime);

    // メッシュ構築、参加に成功した時刻として有効な値が記録されているか否かを確認します。
    bool IsMeshStartTimeValid() const;

    // 任意の時刻とメッシュ構築、参加に成功した時刻の差を計算します。
    common::TimeSpan CalcDiffWithMeshStartTime(common::Time target) const;

    void ProcessingAtMeshBegin();
    void ProcessingAtMeshEnd(DisconnectReason reason);
    void ProcessingAtEndHM();
    void MonitoringProcess(DisconnectReason reason, uint8_t type);

    void AddPlayHistory(transport::Station* pTargetStation, transport::Station::PlayerInfo* pLocalPlayerInfo, uint32_t localPlayerInfoSize);

    enum AsyncProcessingName
    {
        AsyncProcessingName_Nothing = 0,
        AsyncProcessingName_CreateSession,
        AsyncProcessingName_DestroySession,
        AsyncProcessingName_LeaveSessionWithHm,
        AsyncProcessingName_JoinSession,
        AsyncProcessingName_LeaveSession
    };

    void ResetInternalAsyncState();
    bool IsAsyncProcessCompleted(AsyncProcessingName processingName);
    Result GetAsyncProcessResult(AsyncProcessingName processingName);
    Result CancelAsyncProcess(AsyncProcessingName processingName);

    Result JoinMeshCore(const transport::StationConnectionInfo& stationConnectionInfo, common::AsyncContext* pAsyncContext);

    void SetMeshState(State meshState);
    void NoticeMeshEvent(EventType eventType, StationIndex idx);

    CreateMeshJob* m_pCreateMeshJob;
    JoinMeshJob* m_pJoinMeshJob;
    LeaveMeshJob* m_pLeaveMeshJob;
    ProcessJoinRequestJob* m_pProcessJoinRequestJob;
    ProcessUpdateMeshJob* m_pProcessUpdateMeshJob;
    DestroyMeshJob* m_pDestroyMeshJob;
    ProcessDestroyMeshJob* m_pProcessDestroyMeshJob;
    ProcessHostMigrationJob* m_pProcessHostMigrationJob;
    LeaveWithHostMigrationJob* m_pLeaveWithHostMigrationJob;
    SignatureSettingStorage* m_pSignatureSettingStorage;
    RelayRouteManageJob* m_pRelayRouteManageJob;
    KickoutManageJob* m_pKickoutManageJob;
    transport::MissingStationHandler* m_pMissingStationHandler;

    MeshEventCallback m_MeshEventCallback;
    JoiningMeshApprovalCallback m_JoiningMeshApprovalCallback;
    CheckHostCandidateRankingCallback m_CheckHostCandidateRankingCallback;
    GetSessionIdCallback m_GetSessionIdCallback;

    uint32_t m_MeshProtocolHandle;
    MeshProtocol* m_pMeshProtocol;
    transport::StationProtocol* m_pStationProtocol;

    uint32_t m_BandwidthCheckProtocolHandle;
    transport::BandwidthCheckProtocol* m_pBandwidthCheckProtocol;

    uint16_t m_StationMax;
    bool m_IsApprovalJoin;

    StationIndex m_HostStationIndex;
    StationIndex m_LocalStationIndex;

    State m_State;
    DisconnectReason m_DisconnectReason;
    HostMigrationMode m_HostMigrationMode;
    NetworkTopology m_NetworkTopologyType;
    BitRateCheckMode m_BitRateCheckMode;

    bool m_IsHostMigrationEnabled;
    bool m_IsServerRequired;
    bool m_IsRecentPlayRecordEnabled;
    bool m_IsMultiplayerEnabled;

    uint32_t m_StationIndexFlagsForJoinResponseComplete;
    uint32_t m_StationIndexUsedFlags;
    uint32_t m_UpdateCount;
    StationIndex m_UpdateCountSourceStationIndex;

    //n1769
    uint32_t m_SyncClockProtocolHandle;

    common::Time m_MeshStartTime;
    bool m_MeshStartTimeValidFlag;

    bool m_IsStarted;

    bool m_HostMigrationStartFlag;

    bool m_AutoAddRecentPlayRecord;

    common::MonitoringDataSender* m_pMonitoringDataSender;

    common::AsyncContext m_InternalContext;
    AsyncProcessingName m_CurrentAsyncProcessingName;

    MeshEventListener* m_pMeshEventListener;

    bool m_IsSkipSendingReport;
    bool m_IsSkipCleanupReport;

    bool m_IsPlayersCountedAsParticipants;
    uint8_t m_LocalPlayerCount;

    uint32_t m_ExpectedFirstUpdateCount;
    bool m_IsReceivedUpdateCount;

    bool m_SetDenyJoinRequestByInvalidState;
    bool m_SetDenyJoinRequestBySessionIsFull;

    transport::Station::PlayerInfo m_PlayerInfoList[MaxPlayerInfoSize];
    uint8_t m_ValidPlayerInfoNum;
};
}
}
}  // end of namespace nn::pia::session

//!@endcond
