﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>
#include <nn/pia/session/session_Types.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
  @brief   セッションを検索する際に使用する検索条件の基底クラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。

@if CTR_DOC
  @see local::UdsSessionSearchCriteria
@endif
@if NIN_DOC
  @see local::LdnSessionSearchCriteria
@endif
  @see inet::NexSessionSearchCriteria
  @see inet::NexSessionSearchCriteriaOwner
  @see inet::NexSessionSearchCriteriaParticipant
  @see lan::LanSessionSearchCriteria
 */
class SessionSearchCriteria : public common::RootObject
{
public:
    /*!
    @cond PRIVATE
    @brief      検索条件の種類。
     */
    enum SearchCriteriaKey
    {
        SearchCriteriaKey_Standard = 0,           //!< 参加人数など標準の検索条件を指定します。
        SearchCriteriaKey_OwnerPrincipalId,       //!< オーナーのプリンシパル ID を検索条件に指定します。
        SearchCriteriaKey_ParticipantPrincipalId, //!< ユーザーのプリンシパル ID を検索条件に指定します。
        SearchCriteriaKey_SessionId               //!< セッション ID を検索条件に指定します。
    };
    //! @endcond

    // デストラクタ。
    virtual ~SessionSearchCriteria();

    /*!
    @brief      検索結果の取得範囲を指定します。
    @details    インターネット通信時のみ offset に 0xFFFFFFFF を指定すると、検索結果をランダムに並び替えた上で
                指定されたエントリー数を取得するように振る舞います。
    @param[in]  offset  検索条件にマッチしたエントリーリストから検索結果として取得するエントリー群の開始位置。
    @param[in]  size    検索条件にマッチしたエントリーリストから検索結果として取得するエントリー数。

    @return     処理に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval     nn::pia::ResultInvalidArgument  引数の値が範囲外です。[:progErr]
    */
    virtual Result SetResultRange(uint32_t offset, uint32_t size);

    /*!
    @cond PRIVATE
    */
    uint32_t GetResultRangeOffset() const;
    //!@endcond

    /*!
    @cond PRIVATE
    */
    uint32_t GetResultRangeSize() const;
    //!@endcond

    /*!
    @cond PRIVATE
    @brief      検索条件の種類を取得します。
     */
    SearchCriteriaKey GetKeyType() const;
    //! @endcond

    /*!
    @brief      設定をリセットします。
     */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const
    {
        NN_PIA_UNUSED(flag);
    }
    //! @endcond

protected:
    // コンストラクタ。
    SessionSearchCriteria();

    void Copy(const SessionSearchCriteria& rhs);

    // 検索範囲
    uint32_t m_ResultOffset;
    uint32_t m_ResultSize;

    // 検索キーのタイプ
    SearchCriteriaKey m_KeyType;

private:
    NN_PIA_DISALLOW_COPY(SessionSearchCriteria);
};
}
}
}  // end of namespace session
