﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>
#include <nn/pia/session/session_Types.h>

namespace nn
{
namespace pia
{
namespace session
{

/*!
  @brief   セッション設定の更新処理に関する設定を管理する基底クラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。

  @see inet::NexUpdateSessionSetting
  @see lan::LanUpdateSessionSetting
  @if CTR_DOC
  @see nn::pia::local::LocalUpdateSessionSetting
  @endif
  @if NIN_DOC
  @see nn::pia::local::LocalUpdateSessionSetting
  @endif

 */

class UpdateSessionSetting : public common::RootObject
{
public:
    enum ModificationFlag
    {
        ModificationFlag_ApplicationData = 1
    };
    /*!
    @brief    デストラクタ
     */
    virtual ~UpdateSessionSetting()
    {
    }

    /*!
    @brief      アプリケーション定義のデータを指定します。
    @details    inet::ApplicationDataBufferSizeMax を超えないサイズを指定する必要があります。
    @param[in]  pData   セッションに指定するアプリケーション定義データのバッファ。
    @param[in]  size    セッションに指定するアプリケーション定義データのサイズ。
    @return     アプリケーション定義データの設定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタ、または指定したデータサイズが大きすぎます。[:progErr]
    */
    virtual Result SetApplicationData(const void* pData, uint32_t size) = 0;

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータを取得します。
    @param[in]  pDataBuf    指定したアプリケーション定義データのコピー用バッファ。
    @param[in]  dataBufSize コピー用バッファのサイズ。
    @return     アプリケーション定義データの取得に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument   バッファが NULL ポインタです。[:progErr]
    @retval     nn::pia::ResultInvalidState      アプリケーション定義データが設定されていません。[:progErr]
    @retval     nn::pia::ResultBufferShortage    設定されているアプリケーション定義データのサイズがバッファのサイズよりも大きいため、コピーできません。[:progErr]
    */
    virtual Result GetApplicationData(void* pDataBuf, uint32_t dataBufSize) const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータサイズを取得します。
    @return     アプリケーション定義データサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSize() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    virtual uint32_t GetModificationFlag() const;
    virtual bool IsModificationFlagSet(ModificationFlag flag) const;
    virtual bool IsSetApplicationData() const;
    //! @endcond

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

protected:
    // コンストラクタ。
    UpdateSessionSetting();

    void Copy(const UpdateSessionSetting& rhs)
    {
        m_ModificationFlag = rhs.m_ModificationFlag;
    }

    uint32_t m_ModificationFlag;

private:
    NN_PIA_DISALLOW_COPY(UpdateSessionSetting);
};
}
}
}  // end of namespace session
