﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/sync/sync_Definitions.h>


// sync モジュールの非メンバ関数群です。

namespace nn
{
namespace pia
{
namespace sync
{
/*! @name 初期化終了処理
    @{
 */

/*!
              @brief  PiaSync モジュールを初期化します。
                      事前に @ref nn::pia::common::Initialize を実行する必要があります。

              @details sync モジュールが動的に確保し使用するメモリは @ref nn::pia::common::Initialize で渡したメモリから
                      確保されます。

              @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。
              @retval nn::pia::ResultInvalidState    既にこの関数を実行し、初期化が完了している状態です。
             */
Result Initialize();

/*!
              @brief  セットアップの開始を宣言します。

              @details sync モジュール以下の初期化処理を実行する前に呼び出す必要があります。
                      
              @attention   別のモジュールのセットアップ処理とバッティングすると正しく動作しません。<br>
                      @ref sync::BeginSetup 呼び出し後、sync モジュール以下の初期化処理を行い、速やかに @ref sync::EndSetup を呼び出してセットアップを完了する必要があります。
                      
              @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。
              @retval nn::pia::ResultNotInitialized  sync::Initialize が呼ばれていません。[:progErr]
              @retval nn::pia::ResultInvalidState    セットアップ期間中に再度呼び出しています。[:progErr]
             */
Result BeginSetup(void);

/*!
              @cond PRIVATE
              @brief  現在セットアップ期間中かを確認します。

              @details 本関数は内部処理のために用意したものですので、アプリケーションが呼び出す必要はありません。
              <br>
              @return @ref sync::BeginSetup 実行から @ref sync::EndSetup までの期間であれば true を返します。
             */
bool IsSetupNow(void);
/** @endcond */

/*!
              @brief  セットアップの終了を宣言します。

              @details sync モジュール以下の初期化処理を実行する後に呼び出す必要があります。<br>
                       この API 呼び出しにより、sync モジュールのシングルトンなどが消費するメモリを確定し、ライブラリ内部でのメモリ管理を最適化します。
                      
              @attention   別のモジュールのセットアップ処理とバッティングすると正しく動作しません。<br>
                      @ref sync::BeginSetup 呼び出し後、sync モジュール以下の初期化処理を行い、速やかに @ref sync::EndSetup を呼び出してセットアップを完了する必要があります。

              @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。

              @retval nn::pia::ResultNotInitialized  sync::Initialize が呼ばれていません。[:progErr]
              @retval nn::pia::ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
             */
Result EndSetup(void);

/*!
              @brief  PiaSync モジュールが初期化された状態であるかどうかを返します。

              @return 初期化されていれば true、されていなければ false が返されます。
             */
bool IsInitialized(void);

/*!
              @brief  PiaSync モジュールを終了します。
                      @ref nn::pia::common::Finalize を実行する前に実行する必要があります。
              
              @details 本関数を実行する前に PiaSync モジュールのすべての処理を終了しておく必要があります。
                      なお、@ref sync::Initialize を実行していない状態では何もしません。
             */
void Finalize(void);
/*! 
    end of name 初期化終了処理
    @}
 */
}
}
}
