﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/sync/sync_Definitions.h>

#include <nn/pia/transport/transport_Protocol.h>

#include <nn/pia/transport/transport_ProtocolEvent.h>
#include <nn/pia/common/common_PacketDefine.h>
#include <nn/pia/transport/transport_Definitions.h>


// 先行宣言
namespace nn
{
namespace pia
{
namespace common
{
class ZlibCompressor;
class ZlibUncompressor;
}
}
}

// 先行宣言
namespace
{
class DataMessageHeader;
}


namespace nn
{
namespace pia
{
namespace sync
{

/*!
    @brief  データ同期通信を行うためのプロトコルです。<br/>
            サポート対象のネットワークトポロジーはフルメッシュ型のみです。スター型、リレーメッシュ型はサポート対象外ですので注意してください。
 */
class SyncProtocol : public transport::Protocol
{
public:
    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。

    @details アプリケーションプログラムは、このコンストラクタを直接呼び出してはいけません。
             インスタンスの作成は、@ref transport::Transport::CreateProtocol を利用してください。
     */
    SyncProtocol(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。

    @details アプリケーションプログラムは、このデストラクタを直接呼び出してはいけません。
             インスタンスの破棄は、@ref transport::Transport::DestroyProtocol を利用してください。
     */
    virtual ~SyncProtocol(void);
    //! @endcond


    PIA_PROTOCOL_TYPE_INFO(nn::pia::transport::ProtocolTypeSync);


    //! @ref SyncProtocol::Setting::maxDelay で指定する入力遅延の最大値で、単位はフレームです。
    static const uint8_t FrameDelayMax = 32;

    //! データ ID の設定可能個数です。
    static const uint32_t DataIdNum = 16;

    // 予約データ ID です。
    static const uint32_t ReservedDataIdNum = 1;
    static const uint32_t ReservedDataIdChangeDelay = DataIdNum + 0;

    // 前回と同じフレームのデータを送る度にカウントされ、どのカウントから送信をスキップするか示します。
    static const uint32_t SendSkipMax = 2;

    // 同期データの最初の送信回数設定のデフォルト値です。
    static const uint32_t DataFirstSendCount = 3;

    static const uint32_t CompressionLevelNone = 0;    //!< @ref SyncProtocol::Setting::dataCompressionLevel で指定する同期データの圧縮レベル（圧縮なし）です。
    static const uint32_t CompressionLevelLow = 1;     //!< @ref SyncProtocol::Setting::dataCompressionLevel で指定する同期データの圧縮レベル（速度重視）です。
    static const uint32_t CompressionLevelMiddle = 5;  //!< @ref SyncProtocol::Setting::dataCompressionLevel で指定する同期データの圧縮レベルです。
    static const uint32_t CompressionLevelHigh = 9;    //!< @ref SyncProtocol::Setting::dataCompressionLevel で指定する同期データの圧縮レベル（圧縮率重視）です。

    /*!
    @brief      初期化時に指定する設定構造体です。
    @details    同期通信するステーション間で同じ値を指定する必要があります。
     */
    struct Setting
    {
    public:
        /*!
        @brief      デフォルトコンストラクタです。
        @details    デフォルト値で初期化されます。
         */
        Setting();

        /*!
        @brief      各データ ID に対応させる同期データのサイズを指定します。
        @details    使用しないデータ ID には 0 を指定する必要があります。
                    設定できる最大値は、@ref SyncProtocol::GetDataUnitSizeMax で取得できます。
        @see        SyncProtocol::GetDataUnitSizeMax
         */
        uint32_t dataUnitSize[DataIdNum];

        /*!
        @brief      入力遅延の最大フレーム数です。
        @details    1 ～ @ref SyncProtocol::FrameDelayMax の値を指定する必要があります。
         */
        uint32_t maxDelay;

        /*!
        @brief      パケットを受信できない状態がこのフレーム数（@ref SyncProtocol::Step の呼び出し回数）連続したら、タイムアウトとして同期通信を終了します。
        @details    0 を指定すると、タイムアウトによる同期通信の終了は行わず、パケットを受信するまで待ち続けます。
         */
        uint32_t timeoutFrame;

        /*!
        @brief      同期通信中の入力遅延変更機能を有効にするかを指定します。デフォルトは false（無効）です。
         */
        bool isChangeDelayEnabled;

        /*!
        @brief      同期データの圧縮レベルを指定します。
        @details    @ref SyncProtocol::CompressionLevelNone を指定すると圧縮を行いません。
                    @ref SyncProtocol::CompressionLevelLow が速度重視、@ref SyncProtocol::CompressionLevelHigh が圧縮率重視となります。
                    @ref SyncProtocol::CompressionLevelMiddle はその中間です。
                    デフォルト値は @ref SyncProtocol::CompressionLevelNone です。
                    これらの値以外はサポート対象外です。
                    プラットフォームの CPU 性能によって処理時間が大きく変動します。
         */
        uint32_t dataCompressionLevel;
    };


    /*!
    @brief      同期データ 1 単位のサイズとして設定できる最大値（バイト数）を取得します。

    @details    @ref transport::Transport の初期化時に設定するパラメータによって、取得される値は異なります。

    @return     同期データ 1 単位のサイズとして設定できる最大値（バイト数）です。

    @see        SyncProtocol::Setting::dataUnitSize
     */
    uint32_t GetDataUnitSizeMax() const;


    /*!
    @brief      インスタンスを初期化します。@ref sync::BeginSetup() ～ @ref sync::EndSetup() 間で呼び出す必要があります。
            
    @param[in]  setting     各種設定情報です。
                            同期通信をするすべてのステーション間で、同じ値を設定する必要があります。異なる値を設定した場合の動作は不定です。
    
    @return     成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultAlreadyInitialized    既に初期化されています。[:progErr]
    @retval ResultInvalidArgument       引数が不正です。[:progErr]

    @see        SyncProtocol::Finalize
     */
    Result Initialize(const Setting& setting);


    /*!
    @brief      終了処理です。

    @see        SyncProtocol::Initialize
     */
    void Finalize();


    /*!
    @brief      @ref SyncProtocol::Initialize で指定したデータ ID の同期データのサイズを取得します。

    @param[in]  dataId  調べたい同期データのデータ ID です。

    @return     指定された同期データのサイズです。
     */
    uint32_t GetDataSize(uint32_t dataId) const;


    /*!
    @brief      @ref SyncProtocol::Initialize で指定した入力遅延の最大値を取得します。

    @return     入力遅延のフレーム数の最大値です。
     */
    uint32_t GetMaxDelay() const
    {
        return static_cast<uint32_t>(m_MaxDelay);
    }

    /*!
    @brief      実際に設定されている入力遅延の値を取得します。

    @return     入力遅延のフレーム数です。
     */
    uint32_t GetDelay() const
    {
        return static_cast<uint32_t>(m_Delay);
    }


    /*!
    @brief      @ref SyncProtocol::Initialize で指定したタイムアウトの設定値を取得します。

    @return     タイムアウトの設定値です。
     */
    uint32_t GetTimeoutFrame() const
    {
        return static_cast<uint32_t>(m_TimeoutFrame);
    }


    /*!
    @brief      @ref SyncProtocol::Start や @ref SyncProtocol::SetSendPeriod で指定した送信間隔の設定値を取得します。

    @return     送信間隔の設定値です。
     */
    uint32_t GetSendPeriod() const
    {
        return m_SendPeriod;
    }

    /*!
    @brief      送信間隔を設定します。

    @param[in]  sendPeriod          設定した同期データの送信、再送信処理の間隔を指定します。
                                    ここで指定した回数 @ref SyncProtocol::Step() が呼び出されるたびに、@ref common::Scheduler::Dispatch() 呼び出し時に送信、再送信処理が行われます。
                                    1 以上の値を指定する必要があります。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval     ResultInvalidArgument       引数が不正です。[:progErr]
     */
    Result SetSendPeriod(uint32_t sendPeriod);


    /*!
    @brief      同期通信を 1 フレーム進めます。

    @details    @ref SyncProtocol::State_NotSynchronized 以外の間は毎フレーム呼び出す必要があります。フレーム毎の処理の先頭で実行することを推奨します。
                @ref SyncProtocol::NeedsSetData が true の時は、この関数を呼ぶ前に @ref SyncProtocol::SetData で送信する同期データを設定してください。
                @ref SyncProtocol::State_NotSynchronized の時にこの関数を呼んでも問題ありません。

                @ref SyncProtocol::EndAlone 呼び出し成功後の単独同期終了処理中は本関数が失敗しますが、
                内部処理を進行させるために毎フレーム呼び出す必要がある点に注意してください。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState                通信中ではありません。[:progErr]
    @retval ResultDataIsNotArrivedYet         同期データが他のステーションから届いていないためフレームを進めることができませんでした。
                                              common::Scheduler::Dispatch() を呼んだあと再度この関数を呼ぶ必要があります。[:handling]
    @retval ResultDataIsNotSet                自分が送信する同期データがまだ設定されていません。[:progErr]
    @retval ResultTemporaryUnavailable        単独同期終了処理中です。

    @see        SyncProtocol::SetData
    */
    Result Step();


    /*!
    @brief      同期通信を開始します。

    @details    @ref SyncProtocol::State_NotSynchronized の間に呼ぶことができ、成功すると @ref SyncProtocol::State_Waiting に遷移します。
                その後、通信相手でも同期通信が開始されると、@ref SyncProtocol::State_Starting, @ref SyncProtocol::State_Synchronized と遷移していきます。

    @param[in]  usingDataIdBitmap   @ref SyncProtocol::Initialize() で指定したデータ ID のうち、今回の同期通信でこのステーションから送信するデータ ID のリストをビットフラグで指定します。
                                    ビットの下位から順に、データ ID の 0 から対応します。
                                    ステーション毎に別々の値を指定できます。
    
    @param[in]  delay               このステーションが設定を希望する入力遅延を指定できます。
                                    実際に設定される入力遅延は、各ステーションが希望した入力遅延の値の中の最大値となります。
                                    初期化時に @ref SyncProtocol::Setting::maxDelay で指定した最大入力遅延以下の値である必要があります。

    @param[in]  sendPeriod          設定した同期データの送信、再送信処理の間隔を指定します。
                                    ここで指定した回数 @ref SyncProtocol::Step() が呼び出されるたびに、@ref common::Scheduler::Dispatch() 呼び出し時に送信、再送信処理が行われます。
                                    1 以上の値を指定する必要があります。
    
    @return     成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    
    @retval ResultInvalidState                      この関数を呼び出せる SyncProtocol::State ではありません。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument                   引数の指定が不正です。[:progErr]

    @see        SyncProtocol::End
     */
    Result Start(uint32_t usingDataIdBitmap, uint32_t delay, uint32_t sendPeriod);


    /*!
    @brief      同期通信を終了します。

    @details    @ref SyncProtocol::State_Waiting, @ref SyncProtocol::State_Starting, @ref SyncProtocol::State_Synchronized の間に呼ぶことができ、成功すると @ref SyncProtocol::State_Ending に遷移します。
                その後、通信相手に終了することを通知したのち、@ref SyncProtocol::State_NotSynchronized に遷移します。
                この関数で同期通信を終了させた場合、通常は終了の理由が、このステーションでは @ref SyncProtocol::EndReason_End に、
                他のステーションでは、@ref SyncProtocol::EndReason_OtherStation になります。
                ただし、同時に他の要因で終了する事になった場合はこの限りではありません。

                この関数を呼んだ直後の段階では @ref SyncProtocol::State_Ending ですので、再び @ref SyncProtocol::Start を呼ぶには、@ref SyncProtocol::State_NotSynchronized になるまで
                数フレームの間 @ref SyncProtocol::Step を呼ぶ必要があります。
    
    @return     成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    
    @retval ResultInvalidState                      この関数を呼び出せる SyncProtocol::State ではありません。または、通信中ではありません。[:progErr]

    @see        SyncProtocol::Start
     */
    Result End();

    /*!
    @brief      自分だけ単独で同期通信を終了します。

    @details    @ref SyncProtocol::State_Synchronized の間に呼ぶことができます。
                <br/>

                この関数を呼び出したフレームの同期データは取得できますが、
                呼び出し後に他のステーションが単独同期終了を行ったことが通知された場合、
                そのステーションが送信した同期データは取得できなくなり、同期ずれの原因となりますので、
                この関数の呼び出しは、受信した同期データを取得後に行う必要があります。

                終了フレームは、既に同期データを設定済みのフレームの内、最後のフレームとなります。
                通信相手に終了フレームを通知し、全ての通信相手が終了フレームに達したことを確認した後、
                @ref SyncProtocol::State_EndedAlone に遷移します。
                この関数を呼び出し成功後、@ref SyncProtocol::State_Synchronized の間は @ref SyncProtocol::Step に失敗し、
                フレームを進めることができなくなるため、自分は終了フレームに達しない点に注意してください。
                しかし、この場合でも @ref SyncProtocol::Step は内部処理進行のために毎フレーム呼ぶ必要があります。

                この関数で同期通信を終了した場合、通常はこのステーションの終了の理由は、@ref SyncProtocol::EndReason_EndAlone になります。
                ただし、同時に他の要因で終了する事になった場合はこの限りではありません。

                @ref SyncProtocol::State_EndedAlone の状態で、全ての通信相手が同期通信を終了すると、
                @ref SyncProtocol::State_NotSynchronized に遷移します。
                また、@ref SyncProtocol::State_EndedAlone の状態、または、その後 @ref SyncProtocol::State_NotSynchronized に遷移した状態で
                自分がセッションから離脱しても、他のステーションは同期通信を維持することができます。
                ただし、@ref SyncProtocol::State_EndedAlone に遷移後に新規ステーションが接続し、
                自分以外のステーションが再度同期通信を開始しようとして @ref SyncProtocol::State_Waiting になっているときに自分が切断すると、
                自分以外のステーションは同期通信を終了し、@ref SyncProtocol::State_NotSynchronized に遷移します。
                また、@ref SyncProtocol::State_EndedAlone で終了したステーションは @ref SyncProtocol::State_NotSynchronized に遷移後、
                再度同期通信を開始することができますが、再度同期通信を開始、または、セッションからの離脱を行わなければ、
                自分以外のステーションは同期通信中の状態になることができません。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    
    @retval ResultInvalidState                      この関数を呼び出せる SyncProtocol::State ではありません。または、すでに終了フレームが設定されています。または、通信中ではありません。[:progErr]
     */
    Result EndAlone();

    /*!
    @brief      指定したステーションが終了するフレームを取得します。

    @details    @ref SyncProtocol::State_NotSynchronized, @ref SyncProtocol::State_Waiting 以外の状態のときに呼ぶことができます。
                @ref SyncProtocol::State_Synchronized になる前に終了する場合、endFrameNo は 0 になりますが、
                実際にはその前に終了しますので注意してください。
                また、終了するステーション自身は、このフレームに達する前に終了することもある点に注意してください。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。
    
    @retval ResultInvalidArgument      引数の指定が不正です。[:progErr]
    @retval ResultInvalidState         この関数を呼び出せる SyncProtocol::State ではありません。[:progErr]
    @retval ResultNoData               終了するフレームが設定されていません。[:handling]
     */
    Result GetEndFrameNo(StationId stationId, uint32_t* endFrameNo) const;

    /*!
    @brief      同期通信の進行状態を表します。
     */
    enum State
    {
        State_NotSynchronized = 0,  //!< 同期通信中ではありません。
        State_Waiting,              //!< 同期通信を開始し、他のステーションが同期通信を開始するのを待っています。
        State_Starting,             //!< 同期通信を開始中です。送信する同期データを設定する必要がありますが、取得はまだできません。
        State_Synchronized,         //!< 同期通信を行っています。送信する同期データを設定する必要があり、取得もできます。
        State_Ending,               //!< 同期通信を終了中です。
        State_EndedAlone            //!< 自分だけ単独で同期通信を終了しました。
    };


    /*!
    @brief      同期通信の進行状態を取得します。

    @return     同期通信の進行状態です。
    */
    State GetState() const
    {
        return m_State;
    }


    /*!
    @brief      送信する同期データを設定する必要があるかどうかを取得します。

    @details    @ref SyncProtocol::State_Starting, @ref SyncProtocol::State_Synchronized の間は送信する同期データを設定する必要があります。
                ただし、@ref SyncProtocol::RequestToChangeDelay による同期通信中の入力遅延変更機能を使用している場合や、
                @ref SyncProtocol::EndAlone による同期通信の単独終了を行った場合には、
                これらの @ref SyncProtocol::State であっても同期データを設定する必要がない場合があります。
                @ref SyncProtocol::Step を呼び出した時の返り値が失敗だった場合、フレームは進んでいないので、
                次の @ref SyncProtocol::Step 呼び出し前に同期データを設定しなおす必要はありません。

    @return     設定する必要があるデータ ID のうちいずれかが設定されていなければ true を返します。
                同期データを設定する必要が無い場合や同期データが既に設定されている場合は、false を返します。

    @see        SyncProtocol::SetData
    */
    bool NeedsSetData() const;


    /*!
    @brief      特定のデータ ID の同期データを設定する必要があるかどうかを取得します。

    @details    @ref SyncProtocol::State_Starting, @ref SyncProtocol::State_Synchronized の間は同期データを設定する必要があります。
                ただし、@ref SyncProtocol::RequestToChangeDelay による同期通信中の入力遅延変更機能を使用している場合や、
                @ref SyncProtocol::EndAlone による同期通信の単独終了を行った場合には、
                これらの @ref SyncProtocol::State であっても同期データを設定する必要がない場合があります。
                @ref SyncProtocol::Step を呼び出した時の返り値が失敗だった場合はフレームは進んでいないので、
                次の @ref SyncProtocol::Step 呼び出し前に同期データを設定しなおす必要はありません。

    @param[in]  dataId  同期データを設定する必要があるかどうかを調べたいデータ ID です。

    @return     指定したデータ ID の同期データを設定する必要がある場合、true を返します。
                同期データを設定する必要が無い場合や同期データが既に設定されている場合は、false を返します。

    @see        SyncProtocol::SetData
    */
    bool NeedsSetData(uint32_t dataId) const;


    /*!
    @brief      受信した同期データを取得できるかどうかを取得します。

    @details    @ref SyncProtocol::State_Synchronized の間は同期データを取得できます。
                ただし、同期通信中の入力遅延変更機能を使用している場合は、この関数が true を返しても
                同期データを取得できないフレームがあるので、@ref SyncProtocol::GetIsNoDataFrame でデータを取得できないフレームかを確認する必要があります。

    @return     同期データを取得できる場合、true を返します。

    @see        SyncProtocol::GetData
    */
    bool CanGetData() const;


    /*!
    @brief      @ref SyncProtocol::State_Synchronized となってからのフレーム数を取得します。

    @details    @ref SyncProtocol::State_Synchronized の間、@ref SyncProtocol::Step に成功すると 1 進みます。

    @return     @ref SyncProtocol::State_Synchronized となってからのフレーム数です。
                @ref SyncProtocol::State_Synchronized でなければ 0 を返しますが、@ref SyncProtocol::State_Synchronized となった最初のフレームも 0 となりますので注意してください。
    */
    uint32_t GetFrameNo() const;


    /*!
    @brief      指定したステーションが同期通信に参加しているかどうかを取得します。

    @param[in]  stationId   調べたいステーションの @ref StationId です。

    @return     指定したステーションが同期通信に参加していれば true を返します。
    */
    bool CheckEntry(StationId stationId) const;


    /*!
    @brief      指定したステーションが送信しているデータ ID のリストをビットフラグで取得します。

    @param[in]  stationId   調べたいステーションの @ref StationId です。

    @return     指定したステーションが送信しているデータ ID のリストです。
                @ref SyncProtocol::CheckEntry で false となるステーションは同期通信を行っていないので、0 を返します。
    */
    uint32_t GetUsingDataIdBitmap(StationId stationId) const;


    /*!
    @brief      同期通信が終了した際の終了理由を表します。
     */
    enum EndReason
    {
        EndReason_Nothing = 0,       //!< 同期通信は終了していません。
        EndReason_End,               //!< このステーションで @ref SyncProtocol::End が呼ばれたために終了しました。
        EndReason_OtherStation,      //!< 他のステーションが同期通信を終了したために終了しました。
        EndReason_ChangeConnection,  //!< セッションに参加しているステーションに変更があったため終了しました。
        EndReason_Timeout,           //!< @ref SyncProtocol::Setting::timeoutFrame フレーム間同期データが届かなかったため終了しました。
        EndReason_EndAlone           //!< このステーションで @ref SyncProtocol::EndAlone が呼ばれたために終了しました。
    };


    /*!
    @brief      同期通信が終了した際の終了理由を取得します。

    @details    @ref SyncProtocol::Start を呼んで次の同期通信を開始した際に値はリセットされます。

    @return     同期通信が終了した際の終了理由です。
     */
    EndReason GetLastEndReason() const;


    /*!
    @brief      同期通信で送信する同期データを設定します。

    @details    @ref SyncProtocol::NeedsSetData が true の時は、@ref SyncProtocol::Step を呼ぶ前に、同期データを設定する必要があります。
                @ref SyncProtocol::State_Synchronized のとき、設定する同期データは、(@ref SyncProtocol::GetFrameNo + @ref SyncProtocol::GetDelay) フレーム用のデータとなります。
    
    @param[in]  dataId      設定する同期データのデータ ID です。
    @param[in]  cpData      設定する同期データです。サイズは @ref SyncProtocol::Initialize で指定したものになります。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    
    @retval ResultInvalidState                      この関数を呼び出せる SyncProtocol::State ではありません。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument                   引数の指定が不正です。[:progErr]
    @retval ResultAlreadyExists                     同期データは既に設定されています。[:progErr]

    @see        SyncProtocol::NeedsSetData、SyncProtocol::Step
     */
    Result SetData(uint32_t dataId, const void* cpData);


    /*!
    @brief      同期通信で受信した同期データを取得します。

    @details    @ref SyncProtocol::CanGetData が true の時は、@ref SyncProtocol::CheckEntry が true となるステーションから、@ref SyncProtocol::GetUsingDataIdBitmap でビットが立っているデータ ID の同期データを取得できます。
                同期通信中の入力遅延変更機能を使用している場合は、同期データを取得できないフレームがあり、そのフレームでは @ref ResultInvalidState を返します。
                同期データを取得できないフレームかどうかは、@ref SyncProtocol::GetIsNoDataFrame で確認する必要があります。
    
    @param[in]  stationId   取得する同期データを送信するステーションの @ref StationId です。
    @param[in]  dataId      取得する同期データのデータ ID です。
    @param[out] pBuffer     同期データを格納するバッファのポインタです。必要なサイズは @ref SyncProtocol::Initialize で指定したサイズです。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    
    @retval ResultInvalidState                      この関数を呼び出せる SyncProtocol::State ではありません。または、同期データを取得できないフレームです。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument                   引数の指定が不正です。[:progErr]
     */
    Result GetData(StationId stationId, uint32_t dataId, void* pBuffer) const
    {
        return this->GetData(stationId, dataId, pBuffer, 0);
    }


    /*!
    @cond PRIVATE
    @brief      同期通信で受信した未来のフレーム用の同期データを取得します。

    @details    @ref SyncProtocol::CanGetData が true の時は、@ref SyncProtocol::CheckEntry が true となるステーションから、@ref SyncProtocol::GetUsingDataIdBitmap でビットが立っているデータ ID の同期データを取得できます。
                取得したい同期データのフレームの、現在のフレームからのオフセットを指定できます。
                同期通信中の入力遅延変更機能を使用している場合は、同期データを取得できないフレームがあり、そのフレームでは @ref ResultInvalidState を返します。
                同期データを取得できないフレームかどうかは、@ref SyncProtocol::GetIsNoDataFrame で確認する必要があります。
                また、指定した未来のフレームの同期データをまだ受信していない場合は、@ref ResultNoData を返します。
    
    @param[in]  stationId   取得する同期データを送信するステーションの @ref StationId です。
    @param[in]  dataId      取得する同期データのデータ ID です。
    @param[out] pBuffer     同期データを格納するバッファのポインタです。必要なサイズは @ref SyncProtocol::Initialize で指定したサイズです。
    @param[in]  frameOffset 取得したい同期データのフレームの、現在のフレームからのオフセットです。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。
    
    @retval ResultInvalidState                      この関数を呼び出せる SyncProtocol::State ではありません。または、同期データを取得できないフレームです。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument                   引数の指定が不正です。[:progErr]
    @retval ResultNoData                            同期データがありません。[:handling]
     */
    Result GetData(StationId stationId, uint32_t dataId, void* pBuffer, uint32_t frameOffset) const;
    //! @endcond


    /*!
    @brief      同期通信で受信した同期データへのポインタを取得します。

    @details    @ref SyncProtocol::CanGetData が true の時は、@ref SyncProtocol::CheckEntry が true となるステーションから、@ref SyncProtocol::GetUsingDataIdBitmap でビットが立っているデータ ID の同期データを取得できます。
                取得したポインタは、次に @ref SyncProtocol::Step を呼ぶまで有効です。
                同期データのサイズは @ref SyncProtocol::Initialize で指定したサイズです。
    
    @param[in]  stationId   取得する同期データを送信するステーションの @ref StationId です。
    @param[in]  dataId      取得する同期データのデータ ID です。

    @return     同期通信で受信した同期データへのポインタです。
                参照できる同期データが存在しない場合は NULL を返します。
     */
    const void* GetDataPtr(StationId stationId, uint32_t dataId) const;


    /*!
    @cond PRIVATE
    @brief      指定したステーションから、現在のフレームから何フレーム後の同期データまでをすでに受信しているかを取得します。

    @details    取得できる同期データ数には、現在のフレーム用の同期データは含みません。
                stationId に自分を指定した場合は、(設定されている入力遅延 - 1) の値を返します。
                stationId が不正な場合や、@ref SyncProtocol::GetUsingDataIdBitmap で 0 が返るステーションの場合は、0 を返します。
    
    @param[in]  stationId   受信した同期データの送信元のステーションの @ref StationId です。

    @return     指定したステーションから受信した、未来のフレーム用の同期データ数です。
     */
    uint32_t GetReceivedDataNum(StationId stationId) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したステーションから見た、自分の先行フレーム数を取得します。

    @details    受信した同期データ数や RTT に基づき、指定したステーションから見て自分のフレームがどれだけ先行しているかを推定します。
                自分が先行している場合は正の値を、自分が遅れている場合は負の値を返します。
                パケロスにより同期データが受信できなかった場合等、実際の先行フレーム数とは異なる値になる点に注意してください。
                また、各ステーションで推定した先行フレーム数は、互いに整合性が取れない点にも注意してください。

    @param[in]  stationId   フレーム数の比較対象ステーションの @ref StationId です。
    @param[in]  frameRate   アプリのフレームレートです。

    @return     指定したステーションから見た、自分の先行フレーム数です。
     */
    int32_t EstimateLeadingFrameNum(StationId stationId, uint32_t frameRate) const;
    //! @endcond

    /*!
    @brief      入力遅延の変更依頼を同期データとして設定します。

    @details    依頼する入力遅延は、初期化時に @ref SyncProtocol::Setting::maxDelay で指定した最大入力遅延以下の値である必要があります。
                依頼する入力遅延として 0 を指定した場合は、変更依頼を行わないものとして扱われます。
                この関数が成功しても、依頼した入力遅延の値に変更されることは保証されません。
                同じフレームで複数の依頼を受け取った場合は、依頼された入力遅延の値の最大値が優先されます。
                入力遅延の変更依頼を同期データとして受け取った時点で、すでに他の依頼による入力遅延の変更中であった場合は、依頼は無視されます。

    @param[in]  newDelay    依頼する入力遅延の値です。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState                      呼び出し状態が不正です。または、通信中ではありません。[:progErr]
    @retval ResultInvalidArgument                   引数の指定が不正です。[:progErr]
    @retval ResultAlreadyExists                     既に設定されています。[:progErr]
     */
    Result RequestToChangeDelay(uint32_t newDelay);

    /*!
    @brief      同期データを取得できないフレームかどうかを取得します。

    @details    同期通信中の入力遅延変更機能を使用している場合は、同期データを取得できないフレームがあるため、この関数で同期データを取得できないフレームかを確認する必要があります。
                同期通信中の入力遅延変更機能を使用していない場合は、この関数を使用する必要はありません。

    @param[out] isNoDataFrame   同期データを取得できないフレームであるかです。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState                      呼び出し状態が不正です。[:progErr]
    @retval ResultInvalidArgument                   引数の指定が不正です。[:progErr]
     */
    Result GetIsNoDataFrame(bool* isNoDataFrame) const
    {
        return this->GetIsNoDataFrame(isNoDataFrame, 0);
    }

    /*!
    @cond PRIVATE
    @brief      同期データを取得できないフレームかどうかを取得します。

    @details    同期通信中の入力遅延変更機能を使用している場合は、同期データを取得できないフレームがあるため、この関数で同期データを取得できないフレームかを確認する必要があります。
                同期通信中の入力遅延変更機能を使用していない場合は、この関数を使用する必要はありません。

    @param[out] isNoDataFrame   同期データを取得できないフレームであるかです。
    @param[in]  frameOffset     現在のフレームからのオフセットです。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState                      呼び出し状態が不正です。[:progErr]
    @retval ResultInvalidArgument                   引数の指定が不正です。[:progErr]
     */
    Result GetIsNoDataFrame(bool* isNoDataFrame, uint32_t frameOffset) const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      同期データを取得できないフレームかどうかが決定しているフレームのオフセットの最大値を取得します。

    @details    現在のフレームからこの関数が返すオフセット後のフレームまでは、同期データを取得できないフレームかどうかが決定しています。

    @return     同期データを取得できないフレームかどうかが決定しているフレームのオフセットの最大値です。
     */
    uint32_t GetMaxFrameOffsetDecidedIfNoDataFrameOrNot() const;
    //! @endcond

    /*!
    @brief      送信する同期データを 1 パケットに収めるための、各データ ID のデータサイズの合計の最大値を求めます。

    @details    この関数で求められる値は、送信間隔 1 のときに、送信や再送により送信する同期データサイズが最大となった場合を想定した値です。

    @param[in]  delay       入力遅延の値です。
    @param[in]  dataIdNum   使用するデータ ID の数です。

    @return     送信する同期データを 1 パケットに収めるための、各データ ID のデータサイズの合計の最大値が返されます。
     */
    uint32_t CalcMaxSyncDataSizeSentInOnePacket(uint32_t delay, uint32_t dataIdNum);


    /*!
    @cond PRIVATE
    @brief      コマ落ち時、同期データの設定を行わずにフレームを進めます。

    @details    コマ落ち時に @ref SyncProtocol::Step の代わりに呼び出します。成功すると、同期データの設定を行わずに 1 フレーム進めることができ、同期データを設定するフレームのオフセット を -1 します。@ref SyncProtocol::State_Synchronized 以外のときに呼び出すと失敗します。同期通信中の入力遅延変更機能とは併用できません。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。
                失敗したときは、@ref SyncProtocol::Step() と同様の Result、または、@ref ResultInvalidState が返されます。

    @retval     ResultInvalidState                  (入力遅延のフレーム数 + 同期データを設定するフレームのオフセット) が 1 以下、または、この関数を呼び出せる @ref SyncProtocol::State ではありません。[:progErr]
     */
    Result StepWithoutSetData();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      処理落ち時に、同期データの設定を終わった後、呼び出します。

    @details    処理落ち時に、同期データの設定を終わった後、呼び出す必要があります。成功すると、同期データを設定するフレームのオフセット を +1 します。@ref SyncProtocol::State_Synchronized 以外のときに呼び出すと失敗します。同期通信中の入力遅延変更機能とは併用できません。

    @return     成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval     ResultInvalidState          (入力遅延のフレーム数 + 同期データを設定するフレームのオフセット) が 入力遅延のフレーム数の最大値以上、または、この関数を呼び出せる @ref SyncProtocol::State ではありません。[:progErr]
    @retval     ResultDataIsNotSet    自分が送信する同期データがまだ設定されていません。[:progErr]
     */
    Result EndSetDataInNoProcessFrame();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  同期データを設定するフレームのオフセットを返します。
     */
    int32_t GetSetDataOffset()
    {
        return m_SetDataOffset;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      受信済みの同期データをチェックし、再送処理を抑える機能を使用するかを設定します。
     */
    void SetUsingReceivedDataCheck(bool isUsingReceivedDataCheck);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を開始します。

    @param[in] localStationAddress  自分の @ref StationIndex です。

    @return 成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultNotInitialized    初期化されていません。
    @retval ResultInvalidState      既に通信中です。
    @retval ResultInvalidArgument   引数が誤っています。
     */
    virtual Result StartupWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を終了します。
     */
    virtual void CleanupWithStationAddress();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  メインの処理です。

    @return ResultSuccess が返されます。
     */
    virtual Result Dispatch();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  接続状態の切り替え。

    @param[in] event    操作内容を表すイベントです。

    @return 成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState      呼び出し状態が不正です。
    @retval ResultInvalidArgument   引数が誤っています。
     */
    virtual Result UpdateProtocolEvent(const transport::ProtocolEvent& event);
    //! @endcond


    /*!
    @brief  デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


private:
    enum MessageType
    {
        MessageType_Start = 0x01,
        MessageType_End = 0x02,
        MessageType_StartEnd = 0x03,
        MessageType_EndAlone = 0x04,
        MessageType_StartAck = 0x11,
        MessageType_EndAck = 0x12,
        MessageType_StartEndAck = 0x13,
        MessageType_EndAloneAck = 0x14,
        MessageType_Data = 0x21,
        MessageType_CompressedData = 0x22
    };

    struct StationInfo
    {
    public:
        StationInfo()
        {
            Clear();
        }

        void Clear();

    public:
        bool isConnected;
        bool isSyncStarted;

        uint16_t syncId;
        uint32_t usingDataIdBitmap;

        uint32_t preparedFrameNo;
        uint32_t receivedDataBitmap;

        uint32_t lastReceiveStepCount;

        bool isEndAloneCalled;
        uint32_t endFrameNo;
        bool isEndedAloneInCurrentSession;
    };

    bool IsInitialized() const
    {
        return (m_StationNum > 0);
    }
    bool IsInCommunication() const
    {
        return (m_LocalStationIndex != StationIndex_Invalid);
    }

    void CheckTimeout();

    void DispatchReceive();
    void ReceiveStartMessage(StationIndex stationIndex, uint16_t syncId, uint32_t usingDataIdBitmap, int32_t delay, uint16_t connectedStationBitmap);
    void ReceiveEndMessage(StationIndex stationIndex, uint16_t syncId);
    void ReceiveStartEndMessage(StationIndex stationIndex, uint16_t syncId);
    void ReceiveEndAloneMessage(StationIndex stationIndex, uint16_t syncId, uint32_t endFrameNo);
    void ReceiveStartAckMessage(StationIndex stationIndex, uint16_t syncId);
    void ReceiveEndAckMessage(StationIndex stationIndex, uint16_t syncId);
    void ReceiveEndAloneAckMessage(StationIndex stationIndex, uint16_t syncId);
    void ReceiveDataMessage(StationIndex stationIndex, uint16_t syncId, const void* payloadPtr, uint32_t size, bool isCompressedData);

    void DispatchSend();
    bool SendCommand(StationIndex stationIndex, int type, uint16_t syncId = 0);
    bool SendCommandMulti(uint32_t destBmp, int type);
    void SendData();
    void ResetSendDataFrameNo();
    bool CreateDataChunk(uint32_t* pSize, bool isDisableDataCompressionForced = false);

    void UpdatePreparedFrameNo();
    void UpdateReceivedDataBitmap();
    bool IsAllReceivedData(uint32_t sendDataFrameNo);

    uint64_t MakeSendFrameBitmap(uint32_t dataFirstSendCount) const;
    bool IsFrameNoSendNeeded(uint32_t sendDataFrameNo);
    uint32_t* GetSendProcessCountPtr(uint32_t frameNo)
    {
        return &m_paSendProcessCnt[GetDataBufferIdx(m_LocalStationIndex, frameNo)];
    }

    StationInfo* GetStationInfo(StationIndex stationIndex);
    const StationInfo* GetStationInfo(StationIndex stationIndex) const
    {
        return const_cast<SyncProtocol*>(this)->GetStationInfo(stationIndex);
    }

    int GetDataBufferIdx(StationIndex stationIndex, uint32_t frameNo) const;

    uint32_t* GetValidDataFlag(StationIndex stationIndex, uint32_t frameNo)
    {
        return &m_paValidDataFlag[stationIndex * m_BufferFrameNum + GetDataBufferIdx(stationIndex, frameNo)];
    }
    const uint32_t* GetValidDataFlag(StationIndex stationIndex, uint32_t frameNo) const
    {
        return const_cast<SyncProtocol*>(this)->GetValidDataFlag(stationIndex, frameNo);
    }

    uint32_t* GetNoDataFrameFlag(uint32_t frameNo)
    {
        return &m_paNoDataFrameFlag[GetDataBufferIdx(m_LocalStationIndex, frameNo)];
    }
    const uint32_t* GetNoDataFrameFlag(uint32_t frameNo) const
    {
        return const_cast<SyncProtocol*>(this)->GetNoDataFrameFlag(frameNo);
    }

    void* GetDataBuffer(StationIndex stationIndex, uint32_t frameNo, uint32_t dataId)
    {
        PIA_ASSERT(dataId < DataIdNum + ReservedDataIdNum);
        return &m_pDataBuffer[(stationIndex * m_BufferFrameNum + GetDataBufferIdx(stationIndex, frameNo)) * m_TotalDataSize + m_DataInfo[dataId].offset];
    }
    const void* GetDataBuffer(StationIndex stationIndex, uint32_t frameNo, uint32_t dataId) const
    {
        return const_cast<SyncProtocol*>(this)->GetDataBuffer(stationIndex, frameNo, dataId);
    }

    // 入力遅延の動的変更
    Result ChangeDelay(uint32_t newDelay);
    bool IsChangeDelayEnabled() const
    {
        return m_DataInfo[ReservedDataIdChangeDelay].size != 0;
    }

    bool EndImpl(EndReason reason);

    void CheckEndedAlone();

    uint16_t CalcConnectedStationBitmap() const;

    // StationIndex 用
    bool CheckEntry(StationIndex stationIndex) const;
    Result GetData(StationIndex stationIndex, uint32_t dataId, void* pBuffer) const
    {
        return this->GetData(stationIndex, dataId, pBuffer, 0);
    }
    Result GetData(StationIndex stationIndex, uint32_t dataId, void* pBuffer, uint32_t frameOffset) const;
    uint32_t GetUsingDataIdBitmap(StationIndex stationIndex) const;

    void RegisterDataMessageHeader(DataMessageHeader* pDMH, MessageType messageType);

private:
    uint32_t m_StationNum;
    StationIndex m_LocalStationIndex;

    State m_State;

    static const uint32_t MessageBufferSize = common::ProtocolMessgaePayloadSizeMax / sizeof(uint32_t) * sizeof(uint32_t);

    uint32_t m_MessageBuffer[MessageBufferSize / sizeof(uint32_t)];
    uint32_t* m_pCompressedMessageBuffer;

    StationInfo* m_paStationInfo;
    uint32_t* m_paValidDataFlag;
    uint32_t* m_paNoDataFrameFlag;
    uint8_t* m_pDataBuffer;
    int32_t* m_pDelayBuffer;

    struct
    {
        uint32_t offset;
        uint32_t size;
    } m_DataInfo[DataIdNum + ReservedDataIdNum];

    uint32_t m_TotalDataSize;

    int32_t m_Delay;
    int32_t m_MaxDelay;
    int32_t m_BufferFrameNum;
    uint32_t m_SendPeriod;
    int32_t m_TimeoutFrame;

    uint32_t m_CurrentFrameNo;
    uint32_t m_PreparedFrameNo;
    uint32_t m_ReceivedDataBitmap;  // m_PreparedFrameNo + 1 以降のフレームの内、全ステーションから同期データを受信済みのフレーム
    bool m_IsUsingReceivedDataCheck;

    int32_t m_CurrentDataBufferIdx;

    EndReason m_LastEndReason;

    uint32_t m_StepCount;
    uint32_t m_StepCountOffset;
    bool m_IsSendStartNeeded;
    bool m_IsSendEndNeeded;
    bool m_IsSendEndAloneNeeded;
    uint32_t m_ParticipantBmp;
    uint32_t m_SendStartDestBmp;
    uint32_t m_SendEndDestBmp;
    uint32_t m_SendEndAloneDestBmp;
    uint32_t m_SendDataFrameNo;
    uint32_t m_SendDataId;
    bool m_IsResetSendDataFrameNoNeeded;

    // 入力遅延の動的変更用
    int32_t m_NewDelay;
    uint32_t m_ChangeDelayRequestFrameNo;

    // 送信する同期データを設定するフレームのオフセット
    int32_t m_SetDataOffset;

    // 送信する同期データが設定されているかのチェックをしないで Step するフラグ
    bool m_IsStepWithoutSetData;

    // データ圧縮・解凍用
    bool m_IsDataCompressionEnabled;
    uint32_t m_DataCompressionLevel;
    static const uint32_t ZlibCompressorWindowBits = 12;
    static const uint32_t ZlibCompressorMemLevel = 5;
#if defined(NN_BUILD_TARGET_PLATFORM_ADDRESS_64) && NN_BUILD_TARGET_PLATFORM_ADDRESS_64 || NN_PIA_B
    // NX-x64 : 38688
    // NX64   : 38704
    // NN_PIA_B: 32bitは38596で十分だが64bitでは足りない。32bitが今後廃れる事を見越して常に多めに確保する
    static const uint32_t ZlibCompressorWorkMemorySize = 38704;
#else
    static const uint32_t ZlibCompressorWorkMemorySize = 38596;
#endif
    common::ZlibCompressor* m_pZlibCompressor;
    uint32_t* m_paZlibCompressorMemory;
    common::ZlibUncompressor* m_pZlibUncompressor;
    uint32_t* m_paZlibUncompressorMemory;

    // 同じ同期データの送信スキップ用
    uint32_t m_SendSkipMax;
    uint32_t m_SendSkip;
    uint32_t m_SameDataCount;
    uint32_t m_PreSendFrameNoMin;
    uint32_t m_PreSendFrameNoMax;

    // 同期データを送信するかの判定用
    uint32_t* m_paSendProcessCnt;      // 各フレームを送信しようとした回数カウント用バッファ
    uint32_t m_SendProcessFrameNoMin;  // 送信しようとしたフレーム番号の最小値
    uint32_t m_SendProcessFrameNoMax;  // 送信しようとしたフレーム番号の最大値
    uint32_t m_DataFirstSendCount;     // 同期データの最初の送信回数設定
    uint64_t m_SendFrameBitmap;        // 送信判定用ビットマップ

    // MonitoringData 用
    uint8_t m_MonitoringMinDelay;                  // ゲーム通信中の入力遅延の最小値
    uint8_t m_MonitoringMaxDelay;                  // ゲーム通信中の入力遅延の最大値
    uint8_t m_MonitoringMinSendPeriod;             // 送信頻度の最小値
    uint8_t m_MonitoringMaxSendPeriod;             // 送信頻度の最大値
    uint32_t m_MonitoringStepCount;                // Step 呼び出し回数
    uint32_t m_MonitoringPreSendFrameNoMaxBuf;     // 前回送信した最大フレーム番号保存用
    uint32_t m_MonitoringResendCountedFrameNoMax;  // 再送フレームとしてカウント済みの最大フレーム番号
    uint8_t m_MonitoringResendFrameCnt;            // 再送フレーム数カウント用
    uint8_t m_MonitoringMaxResendFrameNum;         // 再送されたフレーム数の最大値
};
}
}
}  // end of namespace nn::pia::sync
