﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  transport モジュールが初期化された状態であるかどうかを返します。

  @details この関数はスレッドセーフです。
  <br>
  @return transport モジュールが初期化されていれば true 、されていなければ false が返されます。
  @see    Initialize, Finalize
 */
bool IsInitialized(void);


/*!
  @brief  transport モジュールを初期化します。

  @return 無事に初期化が完了すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
  @retval ResultAlreadyInitialized  既に初期化は終了しています。[:progErr]
  @see    IsInitialized, Finalize
 */
Result Initialize(void);


/*!
  @brief  transport モジュールを終了します。
          Initialize() を呼び出していない状態でこの関数を呼び出した場合は、
          何もせずに返ります。

  @see    IsInitialized, Initialize
 */
void Finalize(void);


/*!
  @brief  セットアップの開始を宣言します。

  @details transport モジュールのシングルトンなどを作成する前に呼ぶ必要があります。
          アプリケーションが transport モジュールのシングルトンを一つも作成しない場合であっても、
          この関数と @ref EndSetup は必ず呼び出す必要があります。

  @return この関数を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
  @retval ResultNotInitialized  transport::Initialize() が呼ばれていません。[:progErr]
  @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
  @see    EndSetup
 */
Result BeginSetup(void);


/*!
  @brief  セットアップの終了を宣言します。

  @details transport モジュールのシングルトンなどを作成し終えた後に呼ぶ必要があります。
          この関数呼び出しにより、 transport モジュールのシングルトンなどが消費する
          メモリを確定し、ライブラリ内部でのメモリ管理を最適化します。
          アプリケーションが transport モジュールのシングルトンを一つも作成しない場合であっても、
          @ref BeginSetup とこの関数は必ず呼び出す必要があります。

  @return この関数を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
  @retval ResultNotInitialized  transport::Initialize() が呼ばれていません。[:progErr]
  @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
  @see    BeginSetup
 */
Result EndSetup(void);


/*!
  @cond PRIVATE
  @brief BeginSetup() ～ EndSetup()間の状態であるかどうかを返します。

  @details この関数はスレッドセーフです。
 */
bool IsInSetupMode(void);
//! @endcond
}
}
}  // end of namespace nn::pia::transport
