﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

namespace nn
{
namespace pia
{
namespace common
{
class String;
class StepSequenceJob;
class IPacketInput;
class IPacketOutput;
class SignatureSetting;
class MonitoringDataSender;
}
}
}  // end of namespace nn::pia::common

namespace nn
{
namespace pia
{
namespace session
{
class SignatureSettingStorage;
class IMatchmakeSession;
class ISessionInfoList;
class MeshLayerController;
class ICommunityInfoList;
}
}
}  // end of namespace nn::pia::session

namespace nn
{
namespace pia
{
namespace transport
{

class ConnectStationJob;
class DisconnectStationJob;
class PacketHandler;
class MissingStationHandler;

/*!
  @brief   ネットワークに必要なクラスを生成するためのインタフェースです（ファクトリパターン）。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用してください。

@if CTR_DOC
  @see nn::pia::local::UdsNetworkFactory
@endif
@if NIN_DOC
  @see nn::pia::local::LdnNetworkFactory
@endif
  @see nn::pia::inet::NexNetworkFactory
  @see nn::pia::lan::LanNetworkFactory
 */
class NetworkFactory : public common::RootObject
{
public:
    /*!
    @cond PRIVATE
    @brief 仮想デストラクタです。
     */
    virtual ~NetworkFactory(void)
    {
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 最大参加可能ステーション数を返します。
    */
    virtual uint16_t GetMaxStationNum() const = 0;
    //! @endcond

    virtual NetworkFactory* CreateSelf() = 0;

    /*!
    @cond PRIVATE
    @brief ConnectStationJob を生成します。
     */
    virtual ConnectStationJob* CreateConnectStationJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief DisconnectStationJob を生成します。
     */
    virtual DisconnectStationJob* CreateDisconnectStationJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief InputStream を生成します。
     */
    virtual common::IPacketInput* CreateInputStream() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief CreateMeshJob を生成します。
     */
    virtual common::StepSequenceJob* CreateCreateMeshJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief JoinMeshJob を生成します。
     */
    virtual common::StepSequenceJob* CreateJoinMeshJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief LeaveMeshJob を生成します。
     */
    virtual common::StepSequenceJob* CreateLeaveMeshJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief OutputStream を生成します。
     */
    virtual common::IPacketOutput* CreateOutputStream() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ProcessHostMigrationJob を生成します。
     */
    virtual common::StepSequenceJob* CreateProcessHostMigrationJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief LeaveWithHostMigrationJob を生成します。
     */
    virtual common::StepSequenceJob* CreateLeaveWithHostMigrationJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief KickoutManageJob を生成します。
     */
    virtual common::StepSequenceJob* CreateKickoutManageJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 必要なプロトコルを構築します。
     */
    virtual Result CreateProtocols()
    {
        return ResultSuccess();
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief PacketHandler の具象インスタンスを構築します。
     */
    virtual PacketHandler* CreatePacketHandler() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 署名が必要かどうか。
     */
    virtual bool IsSignatureRequired() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 署名鍵を保持するストレージを作成します。
     */
    virtual session::SignatureSettingStorage* CreateSignatureSettingStorage() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 複数候補親交代の使用を許すかどうか。
     */
    virtual bool IsMultiCandidatePermitted() const
    {
        return false;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief リレー通信機能の使用を許すかどうか。
     */
    virtual bool IsRelayConnectionPermitted() const
    {
        return false;
    }
    //! @endcond

    virtual bool IsStarConnection() const
    {
        return false;
    }

    virtual bool IsSupportedProtocol(uint16_t protocolType) const;

    /*!
    @cond PRIVATE
    @brief ジョイントセッション機能の使用を許すかどうか。
     */
    virtual bool IsJointSessionPermitted() const
    {
        return false;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief MonitoringDataSender 派生クラスのインスタンスを生成します。
     */
    virtual common::MonitoringDataSender* CreateMonitoringDataSender()
    {
        return NULL;
    };
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ネットワーク層におけるパケットのヘッダ部分の合計サイズを返します。単位はバイトです。
           例えば IPv4 環境であれば、 IP ヘッダ(20byte) + UDP ヘッダ(8byte) = 28 となります。
     */
    virtual uint32_t GetHeaderSize() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief MissingStationHandler 具象インスタンスを構築します。
     */
    virtual MissingStationHandler* CreateMissingStationHandler()
    {
        return NULL;
    }
    //! @endcond

    /* 自動マッチメイク機能用 */
    /*!
    @cond PRIVATE
    @brief Session 用 CreateSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchCreateSessionJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 AutoSessionmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchAutoMatchmakeSessionJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 BrowseSessionmakeSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchBrowseMatchmakeSessionJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 JoinSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchJoinSessionJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 LeaveSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchLeaveSessionJob() = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 DestroySessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateMatchDestroySessionJob() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 CreateGenerateMatchmakeSystemPasswordJob を生成します。
     */
    virtual common::StepSequenceJob* CreateGenerateMatchmakeSystemPasswordJob() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 CreateClearMatchmakeSystemPasswordJob を生成します。
     */
    virtual common::StepSequenceJob* CreateClearMatchmakeSystemPasswordJob() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 MatchmakeSessionSystemPassword の保持領域を生成します。
     */
    virtual common::String* CreateMatchmakeSessionSystemPassword() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ジョイントセッション機能用 JointSessionJob を生成します。
     */
    virtual common::StepSequenceJob* CreateJointSessionJob() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 UpdateSessionSetting を生成します。
     */
    virtual common::StepSequenceJob* CreateUpdateSessionSettingJob() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 CommunityManagementJob を生成します。
     */
    virtual common::StepSequenceJob* CreateCommunityManagementJob() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 ISessionInfoList を生成します。
     */
    virtual session::ISessionInfoList* CreateSessionInfoList(uint16_t browseSessionInfoListNum) = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 ISessionInfoList を生成します。
     */
    virtual session::ICommunityInfoList* CreateCommunityInfoList(uint8_t browsedCommunityInfoListNum) = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief Session 用 IMatchmakeSession を生成します。
     */
    virtual session::IMatchmakeSession* CreateMatchmakeSession() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Session 用 MeshLayerController を生成します。
     */
    virtual session::MeshLayerController* CreateMatchMeshLayerController() = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief Bind() が必要かどうか。
     */
    virtual bool IsBindRequired() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief サーバーが必要かどうか。
     */
    virtual bool IsServerRequired() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief いっしょに遊んだプレイヤー履歴を記録可能かどうか。
    */
    virtual bool IsRecentPlayRecordPermitted() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 同じ端末から複数プレイヤーが参加可能かどうか。
    */
    virtual bool IsMultiplayerPermitted() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief ホストマイグレーションのモードを取得します。
     */
    virtual int32_t GetHostMigrationMode() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief セッションの検索結果の最大数を取得します。
     */
    virtual uint16_t GetBrowseSessionInfoListSizeMax() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief コミュニティの検索結果の最大数を取得します。
     */
    virtual uint8_t GetBrowseCommunityInfoListSizeMax() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief MTU の最大値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMax() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief MTU の最小値（UDP、IPv4 のヘッダのようなネットワーク層のヘッダサイズを含む）です。
     */
    virtual uint32_t GetMtuSizeMin() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 初期段階での RTT 計測パルス送信間隔のデフォルト値です。単位はミリ秒です。
     */
    virtual int GetDefaultRttInitialPulseInterval() const = 0;
    //! @endcond

    virtual bool IsConnectionRequestPrincipalIdEnabled() const
    {
        return true;
    }

    /* 自動マッチメイク機能用 */

protected:
    /*!
    @cond PRIVATE
     */
    NetworkFactory()
    {
    }
    //! @endcond
};
}
}
}  // end of namespace nn::pia::transport
