﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/common/common_ListNode.h>
#include <nn/pia/transport/transport_ProtocolId.h>
#include <nn/pia/transport/transport_ProtocolEvent.h>

#define PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE 0
#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
#include <nn/pia/common/private/common_Statistic.h>
#include <nn/pia/common/common_TimeSpan.h>
#endif

namespace nn
{
namespace pia
{
namespace common
{
class StationAddress;
}

namespace transport
{

class PacketHandler;
class StationPacketHandler;

/*!
  @brief  プロトコルの基底クラスです。

 */
class Protocol : public common::RootObject
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    Protocol(void);


    /*!
    @brief デストラクタです。
     */
    virtual ~Protocol(void);


    /*!
    @cond PRIVATE
    @brief 使用するパケットハンドラを設定します。

    @param[in] pPacketHandler   パケットハンドラです。
     */
    void SetPacketHandler(PacketHandler* pPacketHandler)
    {
        m_pPacketHandler = pPacketHandler;
    }
    //! @endcond

    /*!
    @brief デバッグに有用な情報をプリントします。
 
    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。

     */
    virtual void Trace(uint64_t flag) const;


    /*!
    @cond PRIVATE
    @brief リストノードへのオフセットを取得します。

    @return リストノードへのオフセットです。
     */
    static uint32_t GetListNodeOffset()
    {
        NN_PIA_PRAGMA_PUSH_WARNINGS
        NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
        return offsetof(Protocol, m_ListNode);
        NN_PIA_PRAGMA_POP_WARNINGS
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ポートを設定。

    @param[in] port     ポート番号。
     */
    void SetPort(uint16_t port);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief プロトコルIDを取得

    @return プロトコルID。
     */
    ProtocolId GetProtocolId() const
    {
        return m_ProtocolId;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief プロトコルのタイプを取得

    @return プロトコルのタイプ。
     */
    virtual uint16_t GetProtocolType() const = 0;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を開始

    @param[in] localStationAddress  自分の StationAddress です。

    @return 成功すれば、IsSuccess()がtrueを返すResultが返されます。
     */
    virtual Result StartupWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を開始

    @return 成功すれば、IsSuccess()がtrueを返すResultが返されます。
    */
    virtual Result Startup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を終了
     */
    virtual void CleanupWithStationAddress();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を終了
    */
    virtual void Cleanup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  メインの処理

    @return 成功すれば、IsSuccess()がtrueを返すResultが返されます。
     */
    virtual Result Dispatch();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  接続状態の切り替え

    @param[in] event    操作内容を表すイベントです。

    @return 成功すれば、IsSuccess()がtrueを返すResultが返されます。
     */
    virtual Result UpdateProtocolEvent(const ProtocolEvent& event);
    //! @endcond


#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
    Statistic<TimeSpan>* GetBlockTimeStatistic()
    {
        return &m_BlockTimeStatistic;
    }
#endif

    /*!
    @cond PRIVATE
    @brief  旧いステーションなどから届いてしまったメッセージを
            フィルタリングするかどうかを制御するための関数です。

    @return デフォルトの実装では true を返し、フィルタリングを許可します。
            フィルタリングを許可すべきでない Protocol 派生クラスでは、
            false を返すようにオーバーライドしてください。
     */
    virtual bool IsProtocolFilteringEnabled() const;
    //! @endcond

protected:
    /*!
    @cond PRIVATE
    @brief  パケットハンドラを取得します。

    @return パケットハンドラのポインタです。
     */
    PacketHandler* GetPacketHandler()
    {
        return m_pPacketHandler;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  パケットハンドラを取得します（const 版）。

    @return パケットハンドラの const ポインタです。
     */
    const PacketHandler* GetPacketHandler() const
    {
        return m_pPacketHandler;
    }
    //! @endcond

    StationPacketHandler* GetStationPacketHandler();
    const StationPacketHandler* GetStationPacketHandler() const;

private:
    common::ListNode m_ListNode;
    ProtocolId m_ProtocolId;
    PacketHandler* m_pPacketHandler;

#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
    Statistic<TimeSpan> m_BlockTimeStatistic;
#endif
};
}
}
}  // end of namespace nn::pia::transport


#define PIA_PROTOCOL_TYPE_INFO(protocolType) \
    static uint16_t GetProtocolTypeStatic()  \
    {                                        \
        return protocolType;                 \
    }                                        \
    virtual uint16_t GetProtocolType() const \
    {                                        \
        return GetProtocolTypeStatic();      \
    }
