﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/common/common_OffsetList.h>
#include <nn/pia/transport/transport_ProtocolId.h>
#include <nn/pia/transport/transport_ProtocolEvent.h>
#include <nn/pia/transport/transport_Protocol.h>

namespace nn
{
namespace pia
{
namespace transport
{

class NetworkFactory;
class PacketHandler;

/*!
  @cond PRIVATE
  @brief  プロトコルを管理します。

 */
class ProtocolManager : public common::RootObject
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    ProtocolManager(void);


    /*!
    @brief デストラクタです。
     */
    virtual ~ProtocolManager(void);


    /*!
    @brief 初期化します。
    @return ResultSuccess を返します。
     */
    Result Initialize(NetworkFactory* pNetworkFactory);


    /*!
    @brief 終了します。
     */
    void Finalize();


    /*!
    @brief T型のプロトコルのインスタンスを生成します。

    @param[in] port プロトコルのポートの値です。

    @return 生成したプロトコルのハンドルです。同じプロトコル ID のものが既にある場合は 0 を返します。
            Startup() 後は生成できないので、その場合も 0 を返します。
     */
    template <typename T>
    uint32_t CreateProtocol(uint16_t port = 0)
    {
        if (m_IsRunning)
        {
            return 0;
        }

        transport::Protocol* pProtocol = new (AllocProtocol(sizeof(T))) T();
        transport::ProtocolId id(T::GetProtocolTypeStatic(), port);
        return CreateProtocolImpl(pProtocol, id);
    }


    /*!
    @brief プロトコルのインスタンスを破棄します。

    @param[in] protocolHandle 削除したいプロトコルのハンドルです。
     */
    void DestroyProtocol(uint32_t protocolHandle);


    /*!
    @brief T型のプロトコルのインスタンスを取得します。

    @param[in] protocolHandle 取得したいプロトコルのハンドルです。

    @return プロトコルのインスタンスのアドレスです。指定されたハンドルのプロトコルが存在しなかったり、T型でなかったりした場合は NULL を返します。
     */
    template <typename T>
    T* GetProtocol(uint32_t protocolHandle)
    {
        return static_cast<T*>(SearchProtocol(ProtocolId(protocolHandle), T::GetProtocolTypeStatic()));
    }


    /*!
    @cond PRIVATE
    @brief 通信を開始します。

    @param[in] pPacketHandler   使用する PacketHandler です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
     */
    Result Startup(PacketHandler* pPacketHandler);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を終了します。
     */
    void Cleanup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  各プロトコルの Protocol::StartupWithStationAddress() を呼び出します。

    @param[in] localStationAddress  自分の StationAddress です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
     */
    Result StartupProtocolsWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  各プロトコルの Protocol::Startup() を呼び出します。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    Result StartupProtocols();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  各プロトコルの Protocol::CleanupWithStationAddress() を呼び出します。
     */
    void CleanupProtocolsWithStationAddress();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  各プロトコルの Protocol::Cleanup() を呼び出します。
    */
    void CleanupProtocols();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  メインの処理

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
     */
    Result Dispatch();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  接続状態の切り替え

    @param[in] event    操作内容を表すイベントです。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
     */
    Result UpdateProtocolEvent(const ProtocolEvent& event);
    //! @endcond


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
    void ResetProtocolStatistic();
    void PrintProtocolStatistic(uint32_t printMaxDataNum);
#endif

private:
    void* AllocProtocol(uint32_t size);

    uint32_t CreateProtocolImpl(Protocol* pProtocol, ProtocolId id);

    Protocol* SearchProtocol(ProtocolId protocolId, uint16_t protocolType);

private:
    typedef common::OffsetList<Protocol> ProtocolList;

    NetworkFactory* m_pNetworkFactory;

    ProtocolList m_ProtocolList;

    bool m_IsRunning;

    ProtocolManager(const ProtocolManager&);
    ProtocolManager& operator=(const ProtocolManager&);
};
//! @endcond
}
}
}  // end of namespace nn::pia::transport
