﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/transport/transport_Protocol.h>
#include <nn/pia/common/common_Time.h>


namespace nn
{
namespace pia
{
namespace transport
{
class ProtocolMessageWriter;
class ProtocolMessageReader;
}
}
}


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief    大きなデータを全員に一斉送信するプロトコルです。

 */
class ReliableBroadcastProtocol : public transport::Protocol
{
public:
    PIA_PROTOCOL_TYPE_INFO(ProtocolTypeReliableBroadcast);

    static const uint32_t ThroughputLimitMax = 0x0FFFFFFF; //!< SetThroughputLimit() で設定できる値の最大値です。

    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。

    @details アプリケーションプログラムは、このコンストラクタを直接呼び出してはいけません。
           インスタンスの作成は、 @ref Transport::CreateProtocol を利用してください。
     */
    ReliableBroadcastProtocol();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。

    @details アプリケーションプログラムは、このデストラクタを直接呼び出してはいけません。
           インスタンスの破棄は、 @ref Transport::DestroyProtocol を利用してください。
     */
    virtual ~ReliableBroadcastProtocol();
    //! @endcond


    /*!
    @brief  インスタンスを初期化します。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。
    
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultAlreadyInitialized    既に初期化されています。[:progErr]

    @see Finalize
     */
    Result Initialize();


    /*!
    @brief  終了処理を行います。
    @see Initialize
     */
    void Finalize();


    /*!
    @cond PRIVATE
    @brief  通信を開始します。

    @param[in] localStationAddress  自分の StationAddress です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidState      既に通信中です。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    virtual Result StartupWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を終了します。
     */
    virtual void CleanupWithStationAddress();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  メインの処理です。

    @return ResultSuccess が返されます。
     */
    virtual Result Dispatch();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  接続状態の切り替え。

    @param[in] event    操作内容を表すイベントです。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState      通信中ではありません。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    virtual Result UpdateProtocolEvent(const transport::ProtocolEvent& event);
    //! @endcond

private:
    /*!
    @cond PRIVATE
    @brief  プロトコルの動作モードを表します。
     */
    enum Mode
    {
        Mode_Invalid = 0x00000000,
        Mode_Wait = 0x01000000,
        Mode_Send = 0x02000000,
        Mode_Receive = 0x03000000
    };
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  プロトコルの動作の進行状況を表します。
     */
    enum Condition
    {
        Condition_Wait = 0x00010000,
        Condition_InProgress = 0x00020000,
        Condition_Success = 0x00030000,
        Condition_Failure = 0x00040000
    };
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  現在の動作モードを取得します。
    @return 現在の動作モードです。
     */
    Mode GetMode() const
    {
        return static_cast<Mode>(m_State & 0xFF000000);
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  現在の進行状況を取得します。
    @return 現在の進行状況です。
     */
    Condition GetCondition() const
    {
        return static_cast<Condition>(m_State & 0x00FF0000);
    }
    //! @endcond


public:
    /*!
    @brief  動作状態を表します。
     */
    enum State
    {
        State_Invalid = Mode_Invalid | Condition_Wait,            //!< セッションに参加していません。
        State_Wait = Mode_Wait | Condition_Wait,                  //!< 何も行っていません。
        State_Sending = Mode_Send | Condition_InProgress,         //!< 送信中です。
        State_SendSuccess = Mode_Send | Condition_Success,        //!< 送信に成功しました。
        State_SendFailure = Mode_Send | Condition_Failure,        //!< 送信に失敗しました。
        State_Receiving = Mode_Receive | Condition_InProgress,    //!< 受信中です。
        State_ReceiveSuccess = Mode_Receive | Condition_Success,  //!< 受信に成功しました。
        State_ReceiveFailure = Mode_Receive | Condition_Failure,  //!< 受信に失敗しました。

        // 以下内部管理用
        //! @cond PRIVATE
        State_SendCanceling = State_Sending | 0x1,                      // 送信を自分でキャンセル
        State_SendRejecting = State_Sending | 0x2,                      // 受信者が拒否して送信をキャンセル
        State_SendAbandoning = State_Sending | 0x3,                     // 他人の送信を優先してキャンセル
        State_SendCompleting = State_Sending | 0x4,                     // データが全員に届いて完了通知中
        State_SendCanceled = State_SendFailure | 0x1,                   // 自分で送信を中断
        State_SendRejected = State_SendFailure | 0x2,                   // 送信先に受信拒否された
        State_SendAbandoned = State_SendFailure | 0x3,                  // 送信優先度の調停により中断
        State_ReceiveReady = State_Receiving | 0x1,                     // 受信待ち状態
        State_ReceiveCanceling = State_Receiving | 0x2,                 // 受信キャンセル中
        State_ReceiveCanceledBeforeStart = State_ReceiveFailure | 0x1,  // 自分で受信を中断（受信開始前）
        State_ReceiveCanceled = State_ReceiveFailure | 0x2,             // 自分で受信を中断
        State_ReceiveInterrupted = State_ReceiveFailure | 0x3,          // 送信側が送信を中断して受信失敗
        State_ReceiveMismatch = State_ReceiveFailure | 0x4              // リクエストと受信設定が合致しない
                                                                        //! @endcond
    };


    /*!
    @brief  現在の動作状態を取得します。
    @return 現在の動作状態です。
     */
    State GetState() const
    {
        return static_cast<State>(m_State & 0xFFFF0000);
    }


    /*!
    @brief  送受信の非同期処理実行中かどうかを取得します。

    @details GetState() が State_Sending または State_Receiving を返す時に true になります。
    <br>
    @return 送受信の非同期処理実行中なら true を返します。
    @see GetState
     */
    bool IsRunning() const
    {
        return (GetCondition() == Condition_InProgress);
    }


    /*!
    @brief  データ転送の動作方式を表します。
            
    @details 設定の組み合わせによっては意味のないものであったりする場合もありますので、
            典型的なケースではあらかじめ用意された定数をそのまま使う事をお勧めします。
     */
    class Configuration
    {
    public:
        /*!
        @brief  デフォルトコンストラクタです。
                CurrentConnectionSync と同じ値で初期化されます。
         */
        Configuration()
            : m_Value(CurrentConnectionSync.m_Value)
        {
        }

        /*!
        @cond PRIVATE
        @brief  uint16_t からの変換
         */
        Configuration(uint16_t config)
            : m_Value(config)
        {
        }
        //! @endcond


        /*!
        @cond PRIVATE
        @brief  uint16_t への変換
         */
        operator uint16_t() const
        {
            return m_Value;
        }
        //! @endcond


        /*!
        @brief  途中合流したステーションに対しても送信するかどうかの設定を取得します。
        @return true の場合、送信中に新たにセッションに参加したステーションに対しても送信します。
                false の場合は送信開始時にセッション内に存在するステーションのみに対して送信します。
        @see    SetSendToNewcomer
         */
        bool IsSendToNewcomer() const
        {
            return IsFlagOn(Flag_SendToNewcomer);
        }

        /*!
        @brief  受信完了を同期するかどうかの設定を取得します。
        @return true の場合、受信している全てのステーションが受信完了してから State_ReceiveSuccess となります。
                false の場合、全てのデータを受信したステーションごとに随時 State_ReceiveSuccess となります。
        @see    SetSynchronizeCompletion
         */
        bool IsSynchronizeCompletion() const
        {
            return IsFlagOn(Flag_SyncronizeCompletion);
        }

        /*!
        @brief  全ての送信先に送信が完了しても State_Sending を継続するかどうかの設定を取得します。

        @details true とした場合そのままでは State_SendSuccess になることはありません。
                ReliableBroadcastProtocol::FinishSendEndless() を呼ぶと、それ以降 false を設定した場合と同様の動作をするようになり、
                全ての送信先に送信を終えた後 State_SendSuccess に遷移します。

        @return true の場合、すべての送信先に送信が完了しても State_Sending 状態を維持し、送信先が追加された場合はそちらにも送信します。
                false の場合、全ての送信先に送信が完了すると State_SendSuccess に遷移して送信を終了します。

        @see    SetSendEndless
         */
        bool IsSendEndless() const
        {
            return IsFlagOn(Flag_SendEndless);
        }

        /*!
        @brief  送信先に受信拒否された場合、送信失敗として送信を中断するかどうかの設定を取得します。
        @return true の場合、受信拒否されると State_SendFailure となり、送信全体が失敗したことになります。
                false の場合、受信拒否したステーションだけを送信対象から除外し、他の送信先への送信は継続します。
        @see    SetFailIfRejected
         */
        bool IsFailIfRejected() const
        {
            return IsFlagOn(Flag_FailIfRejected);
        }

        /*!
        @brief  2つのステーションが同時にお互いに送信しようとした場合に、自動的に一方を送信失敗にするかどうかの設定を取得します。

        @details この設定が true の送信同士が衝突した場合は、送信元の StationIndex の値の大きい方の送信が失敗になります。
                一方が true 、もう一方が false の送信の場合は、 true の方の送信が失敗になります。
                両方とも false の場合はどちらも State_Sending を維持してお互いに相手が受信してくれるのを待ち続けるため、
                アプリケーションで適切に処理しないとどちらの送信も完了しなくなってしまいますので注意してください。

        @return true の場合、IsMediateRequest() が false の受信要求や自分より StationIndex の小さい送信先からの受信要求が届いた場合は、
                送信を中断し State_SendFailure に遷移します。
                false の場合、送信先からの受信要求によって自動的に State_SendFailure になることはありません。

        @see    SetMediateRequest
         */
        bool IsMediateRequest() const
        {
            return IsFlagOn(Flag_MediateRequest);
        }


        /*!
        @brief  途中合流したステーションに対しても送信するかどうかの設定を指定します。

        @details 設定の意味の詳細は IsSendToNewcomer() を参照してください。
        <br>
        @param[in] isOn true なら設定を有効にします。
        @see    IsSendToNewcomer
         */
        void SetSendToNewcomer(bool isOn)
        {
            SetFlag(Flag_SendToNewcomer, isOn);
        }

        /*!
        @brief  受信完了を同期するかどうかの設定を指定します。

        @details 設定の意味の詳細は IsSynchronizeCompletion() を参照してください。
        <br>
        @param[in] isOn true なら設定を有効にします。
        @see    IsSynchronizeCompletion
         */
        void SetSynchronizeCompletion(bool isOn)
        {
            SetFlag(Flag_SyncronizeCompletion, isOn);
        }

        /*!
        @brief  全ての送信先に送信が完了しても State_Sending を継続するかどうかの設定を指定します。

        @details 設定の意味の詳細は IsSendEndless() を参照してください。
        <br>
        @param[in] isOn true なら設定を有効にします。
        @see    IsSendEndless
         */
        void SetSendEndless(bool isOn)
        {
            SetFlag(Flag_SendEndless, isOn);
        }

        /*!
        @brief  送信先に受信拒否された場合、送信失敗として送信を中断するかどうかの設定を指定します。

        @details 設定の意味の詳細は IsFailIfRejected() を参照してください。
        <br>
        @param[in] isOn true なら設定を有効にします。
        @see    IsFailIfRejected
         */
        void SetFailIfRejected(bool isOn)
        {
            SetFlag(Flag_FailIfRejected, isOn);
        }

        /*!
        @brief  2つのステーションが同時にお互いに送信しようとした場合に、自動的に一方を送信失敗にするかどうかの設定を指定します。

        @details 設定の意味の詳細は IsMediateRequest() を参照してください。
        <br>
        @param[in] isOn true なら設定を有効にします。
        @see    IsMediateRequest
         */
        void SetMediateRequest(bool isOn)
        {
            SetFlag(Flag_MediateRequest, isOn);
        }


        /*!
        @brief  現在接続しているステーションに同期送信する設定です。

        @details セッション参加を締め切った後、誰かがデータを全員に送信して、全員に届いたら次のシーケンスに進むという使い方を想定した設定です。
        <br>
        @li     IsSendToNewcomer()          : false
        @li     IsSynchronizeCompletion()   : true
        @li     IsSendEndless()             : false
        @li     IsFailIfRejected()          : true
        @li     IsMediateRequest()          : true
         */
        static const Configuration CurrentConnectionSync;

        /*!
        @brief  現在接続しているステーションに非同期送信する設定です。

        @details CurrentConnectionSync に対して、受信側は他の受信者にも届いたことを待たなくていい場合を想定した設定です。
        <br>
        @li     IsSendToNewcomer()          : false
        @li     IsSynchronizeCompletion()   : false
        @li     IsSendEndless()             : false
        @li     IsFailIfRejected()          : false
        @li     IsMediateRequest()          : true
         */
        static const Configuration CurrentConnectionAsync;

        /*!
        @brief  現在接続しているステーションだけでなく合流してきたステーションに対しても送信し、全員が受信し終わるのを待つ設定です。

        @details マッチング中の待ち時間のうちにあらかじめ共有しておくデータを送っておき、
                マッチング終了後に FinishSendEndless で送信を終了させて
                そろって次のシーケンスに進むという使い方を想定した設定です。

         @li    IsSendToNewcomer()          : true
         @li    IsSynchronizeCompletion()   : true
         @li    IsSendEndless()             : true
         @li    IsFailIfRejected()          : true
         @li    IsMediateRequest()          : false
         */
        static const Configuration ShareDataSync;

        /*!
        @brief  現在接続しているステーションだけでなく合流してきたステーションに対してもそれぞれ送信し続ける設定です。

        @details セッションに参加してきたステーションにまず一定のデータを送信して、
                そのデータを受け取ったステーションから順に次のシーケンスに進むという使い方を想定した設定です。

         @li    IsSendToNewcomer()         : true
         @li    IsSynchronizeCompletion()   : false
         @li    IsSendEndless()             : true
         @li    IsFailIfRejected()          : false
         @li    IsMediateRequest()          : false
         */
        static const Configuration ShareDataAsync;


    private:
        enum Flag
        {
            Flag_SendToNewcomer = 1 << 0,
            Flag_SyncronizeCompletion = 1 << 1,
            Flag_SendEndless = 1 << 2,
            Flag_FailIfRejected = 1 << 3,
            Flag_MediateRequest = 1 << 4
        };

        bool IsFlagOn(Flag flag) const
        {
            return ((m_Value & flag) == flag);
        }
        void SetFlag(Flag flag, bool isOn)
        {
            if (isOn)
            {
                m_Value |= flag;
            }
            else
            {
                m_Value &= ~flag;
            }
        }

        uint16_t m_Value;
    };


    /*!
    @brief  データ転送の設定情報を表します。
     */
    struct TransferSetting
    {
        StationIndex m_SourceStationIndex;     //!< 送信元の StationIndex です。
        uint32_t m_DataSize;                   //!< 転送データのサイズ（バイト）です。
        uint32_t m_UserData;                   //!< 送信者が設定したユーザー情報です。
        Configuration m_Configuration;         //!< データ転送の動作方式です。
        StationId GetSourceStationId() const;  //!< 送信元の StationId を取得します。
    };


    /*!
    @brief  現在のデータ転送の設定情報を取得します。

    @return 送信中および送信後ならその送信の設定が、受信中および受信後ならその受信の設定が取得できます。
            受信要求が来る前に受信開始した時や、その状態のまま Cancel() した時など、データを全く受け取っていない場合は NULL を返します。
     */
    const TransferSetting* GetCurrentTransferSetting() const;


    /*!
    @brief  指定したステーションが現在の送信の送信対象とされているかどうかを取得します。

    @details 通常はセッションに参加している自分以外の全てのステーションが送信対象になります。
            @ref Configuration::IsSendToNewcomer() が false の場合は、送信開始以降にセッションに参加したステーションは送信対象になりません。
            @ref Configuration::IsFailIfRejected() が false の場合は、その送信に対して受信拒否したり受信を中断したステーションは送信対象から外されます。

    @param[in] stationId    送信対象かどうか調べたいステーションの StationId です。

    @return 指定したステーションが送信対象なら true を、そうでなければ false を返します。
            送信中、送信後でないときは false を返します。
     */
    bool IsSendTarget(StationId stationId) const;

    /*!
    @cond PRIVATE
    @brief  現在のデータ転送の設定情報を取得します。
     */
    bool IsSendTarget(StationIndex stationIndex) const;
    //! @endcond


    /*!
    @brief  送受信の進捗状況を取得します。

    @details 送信中の進捗は、送信対象となるステーションの増減によって途中で値が小さくなる場合がありますので注意してください。
    <br>
    @return 送受信が完了したデータサイズ（バイト）です。
            送信中、送信後は送信対象としているステーションに対して送信したデータサイズの平均値が取得できます。
            受信中、受信後は自分が受け取ったデータサイズが取得できます。
     */
    uint32_t GetProgress() const;


    /*!
    @brief  特定の送信先への送信の進捗状況を取得します。

    @param[in] stationId    進捗状況を調べたいステーションの StationId です。

    @return 指定したステーションに送信が完了したデータサイズ（バイト）です。
            送信中、送信後でないときや無効な StationId が指定された場合は 0 を返します。
     */
    uint32_t GetSendProgress(StationId stationId) const;


    /*!
    @cond PRIVATE
    @brief  特定の送信先への送信の進捗状況を取得します。
     */
    uint32_t GetSendProgress(StationIndex stationIndex) const;
    //! @endcond


    /*!
    @brief  送信を開始します。

    @details この関数に成功すれば State_Sending に遷移します。
            ResultInvalidArgument で失敗した場合は Reset() されます。

    @details ジョイントセッション処理をまたいでデータを送信することはできません。

    @param[in]  cpData          送信するデータへのポインタです。
                                State_Sending の間（ IsRunning() が true の間）は指定したバッファを保持しておく必要があります。
                                データの先頭から順に送信される保証はありませんので、先頭から GetProgress() バイトは書き換えて大丈夫という事もありません。
    @param[in]  dataSize        送信するデータのサイズです。
    @param[in]  configuration   このデータ転送における動作方式を指定します。
    @param[in]  userData        送信データに付ける付加情報です。
                                受信側のアプリケーションが受信開始前に参照することができ、データを識別するためなどに使用できます。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidState      セッションに参加していないか、他の送受信を実行中です。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
    @retval ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
     */
    Result StartSend(const void* cpData, uint32_t dataSize, Configuration configuration, uint32_t userData = 0);


    /*!
    @brief  Configuration::IsSendEndless() が true となる設定で実行している送信を終了させます。
            動作の詳細は Configuration::IsSendEndless() を参照してください。
    
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidState      送信実行中ではないか、@ref nn::pia::transport::ReliableBroadcastProtocol::Configuration::IsSendEndless() "Configuration::IsSendEndless()" が true となる設定ではありません。[:progErr]

    @see    Configuration::IsSendEndless
     */
    Result FinishSendEndless();


    /*!
    @brief  受信を開始します。

    @details GetRequest() を受けてそのデータを受信する場合はその設定に合わせた引数を指定して呼び出す必要があります。
            アプリケーションのシーケンスなどによりどういう受信をするかあらかじめ分かっている場合は、
            受信要求が来る前にこの関数を呼んでおくと、受信要求を待つ場合に比べて無駄なく受信が開始できます。
            この関数が成功すると State_Receiving に遷移します。
            ResultInvalidArgument で失敗した場合は Reset() されます。

    @details ジョイントセッション処理をまたいでデータを受信することはできません。

    @param[out] pBuffer         受信データを受け取るバッファです。
                                State_Receiving の間（ IsRunning() が true の間）はまだデータを参照できません。
                                データの先頭から順に受信される保証はありませんので、先頭から GetProgress() バイトは参照できるという事もありません。
    @param[in]  bufferSize      受信バッファのサイズです。
                                受信要求のデータサイズがこの値より大きい場合は、この関数は成功しますが State_ReceiveFailure に遷移します。
    @param[in]  sourceStationId この送信者からのデータを受信します。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidState      セッションに参加していないか、他の送受信を実行中です。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
    @retval ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]

    @see    GetRequest
     */
    Result StartReceive(void* pBuffer, uint32_t bufferSize, StationId sourceStationId);


    /*!
    @cond PRIVATE
    @brief  受信を開始します。
     */
    Result StartReceive(void* pBuffer, uint32_t bufferSize, StationIndex sourceStationIndex);
    //! @endcond


    /*!
    @brief  送受信を中断します。

    @details State_Sending 中にこの関数を呼ぶと、その後の common::Scheduler::Dispatch() でキャンセル処理を行った後、通常は State_SendFailure に遷移します。
            ただし、タイミングによっては State_SendSuccess に遷移する場合もあります。
            受信側のステーションも State_ReceiveFailure （または State_ReceiveSuccess ）に遷移します。

            State_Receiving 中にこの関数を呼ぶと、その後の common::Scheduler::Dispatch() でキャンセル処理を行った後、通常は State_ReceiveFailure に遷移します。
            ただし、タイミングによっては State_ReceiveSuccess に遷移する場合もあり、その場合は受信したデータを参照できます。
            Configuration::IsFailIfRejected() が true となる設定で動作させている場合は、その送信自体が中断されます。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidState      送受信中ではありません。[:progErr]

    @see    StartSend, StartReceive
     */
    Result Cancel();


    /*!
    @brief  送受信結果の情報を破棄して State_Wait に戻ります。

    @details 送受信実行中は失敗しますので、その状態で呼び出したい場合は先に Cancel() して送受信を中断させてからこの関数を呼ぶ必要があります。
            この関数は必ずしも使用する必要は無く、 State_SendSuccess, State_SendFailure, State_ReceiveSuccess, State_ReceiveFailure の状態から
            直接次の送受信を開始することも可能です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidState  セッションに参加していないか、送受信を実行中です。[:progErr]
     */
    Result Reset();


    /*!
    @brief  届いている受信要求を取得します。

    @details このデータを受け取る場合は必要なサイズのバッファを用意して StartReceive() を行う必要があります。
            このデータを受け取らない場合は RejectRequest() を呼ぶ必要があります。
            複数の受信要求が届いている場合は、最も優先度の高い（ Configuration::IsMediateRequest() の値に関係なく、送信者の StationIndex が小さい）受信要求が取得できます。

    @return 受信要求のポインタです。
            受信要求が届いてない場合は NULL が返されます。

    @see    StartReceive, RejectRequest
     */
    const TransferSetting* GetRequest() const;


    /*!
    @brief  特定のステーションからの受信要求を取得します。
            
    @details このデータを受け取る場合は必要なサイズのバッファを用意して StartReceive() を行う必要があります。
    <br>
    @param[in] stationId    取得したい受信要求の StationId です。

    @return 受信要求のポインタです。
            受信要求が届いてない場合は NULL が返されます。

    @see    StartReceive, RejectRequest
     */
    const TransferSetting* GetRequest(StationId stationId) const;


    /*!
    @cond PRIVATE
    @brief  特定のステーションからの受信要求を取得します。
     */
    const TransferSetting* GetRequest(StationIndex stationIndex) const;
    //! @endcond


    /*!
    @brief  GetRequest() で取得される受信要求を拒否します。

    @details この受信要求の動作方式が @ref Configuration::IsFailIfRejected() が true となる設定の場合は、その送信自体が中断されます。
            現在受信中の受信要求に対してこの関数を呼んだ場合は Cancel() と同等の処理を行います。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidState  GetRequest() が NULL を返しています。[:progErr]

    @see    GetRequest
     */
    Result RejectRequest();


    /*!
    @brief  @ref GetRequest ( nn::pia::StationId ) で取得される受信要求を拒否します。

    @details この受信要求の動作方式が @ref Configuration::IsFailIfRejected() が true となる設定の場合は、その送信自体が中断されます。
            現在受信中の受信要求に対してこの関数を呼んだ場合は Cancel() と同等の処理を行います。

    @param[in] stationId    拒否したい受信要求の StationId です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidState  GetRequest ( nn::pia::StationId ) が NULL を返しています。[:progErr]

    @see    GetRequest
     */
    Result RejectRequest(StationId stationId);


    /*!
    @cond PRIVATE
    @brief  @ref GetRequest ( StationIndex ) で取得される受信要求を拒否します。
     */
    Result RejectRequest(StationIndex stationIndex);
    //! @endcond


    /*!
    @brief  common::Scheduler::Dispatch の 1 回の呼び出しあたりにこのプロトコルが送信するデータ量の上限を指定します。
            
    @details このプロトコルが common::Scheduler::Dispatch の 1 回あたりに送信するデータ量の平均値が
            この関数で設定した値以下になるように制限されます。
            瞬間的にはここで設定した値以上のサイズの送信が行われることもあります。

            この関数に 0 を指定することでこのプロトコルはデータ送信を行わなくなりますので、
            送信を一時停止させたい場合は、送信側で 0 を指定する必要があります。
            その後送信を再開させる場合は、改めて正の値を設定する必要があります。
            一時停止したまま Cancel() で中断したい場合でも、0 を指定したままだと中断することの通知も行えず、
            中断処理が完了しなくなってしまいますので、注意してください。
            一方、受信側も応答パケットなど若干のデータの送信が行われますので、
            受信側で 0 を指定しても処理が進まなくなってしまう点に注意してください。
            
            設定できる値の最大値は @ref ThroughputLimitMax です。これより大きな値を設定した場合は、@ref ThroughputLimitMax を設定したものと扱います。

            デフォルト値は @ref ThroughputLimitMax です。
            セッション作成時や参加時にデフォルト値にリセットされますので、その後に設定する必要があります。

            通信環境によっては、上限を適切に設定した方が、@ref ThroughputLimitMax を設定した場合よりもスループットが高くなる可能性があります。
            @if CTR_DOC 特に CTR でのインターネット通信時には、アプリケーションで想定するステーション数や通信環境等に合わせて上限値を調整することを推奨します。 @endif

    @param[in] throughputLimit  設定する送信データ量のサイズ（バイト）です。

    @see    GetThroughputLimit
     */
    void SetThroughputLimit(uint32_t throughputLimit);


    /*!
    @brief  common::Scheduler::Dispatch の 1 回の呼び出しあたりにこのプロトコルが送信するデータ量の上限を取得します。

    @return common::Scheduler::Dispatch の 1 回の呼び出しあたりにこのプロトコルが送信するデータ量です。

    @see    SetThroughputLimit
     */
    uint32_t GetThroughputLimit() const
    {
        return m_ThroughputLimit;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


private:
    static const uint16_t InvalidMessageIdx = 0xffff;

    // 進捗を管理
    class Progress
    {
    public:
        uint32_t GetCount() const;
        bool IsOn(uint32_t idx) const;
        bool IsAllOff() const;
        bool IsComplete() const
        {
            return m_FirstOffIdx >= m_UnitNum;
        }

    protected:
        static const uint32_t FlagBlockNum = 4;
        uint32_t m_UnitNum;
        uint32_t m_FirstOffIdx;
        uint32_t m_Flag[FlagBlockNum];
    };

    // 受信時の進捗を管理
    class ReceiverProgress : public Progress
    {
    public:
        void Clear(uint32_t unitNum);
        bool Raise(uint32_t idx);
        void Get(uint32_t* pOffset, uint64_t* pFlag) const;
    };

    // 送信時の進捗を管理
    class SenderProgress : public Progress
    {
    public:
        void Clear(uint32_t unitNum);
        bool Update(uint32_t offset, uint64_t flag);
        bool IsReceivable(uint32_t offset) const;
        void UpdateMessageIdx(uint32_t unitIdx, uint16_t messageIdx);
        uint32_t GetFirstOffIdx() const
        {
            return m_FirstOffIdx;
        }
        uint32_t GetUnitNum() const
        {
            return m_UnitNum;
        }
        uint16_t GetSendMessageIdx(uint32_t unitIdx) const;

    private:
        static const uint32_t DataSize = FlagBlockNum * 32;

        uint16_t m_SendMessageIdx[DataSize];
    };


    // メッセージの種類
    enum MessageType
    {
        MessageType_Invalid = 0x00,
        MessageType_Request = 0x11,
        MessageType_Data = 0x12,
        MessageType_Cancel = 0x18,
        MessageType_Complete = 0x19,
        MessageType_DataAck = 0x21,
        MessageType_CommandAck = 0x28,
        MessageType_Reject = 0x29
    };


    // メッセージへのアクセス
    class MessageAccessor
    {
    public:
        MessageAccessor()
            : m_Type(MessageType_Invalid)
        {
        }
        bool IsSenderMessage() const
        {
            return (m_Type & 0xf0) == 0x10;
        }
        bool IsReceiverMessage() const
        {
            return (m_Type & 0xf0) == 0x20;
        }
        bool IsDataMessage() const
        {
            return m_Type == MessageType_Data;
        }

        uint32_t GetMessageSize() const;
        static bool Unpack(MessageAccessor* pAccessor, const ProtocolMessageReader& reader, uint32_t* pMessageIdx);
        static bool Pack(ProtocolMessageWriter* pWriter, const MessageAccessor& accessor, uint32_t messageIdx);

    public:
        MessageType m_Type;
        uint16_t m_Id;

        uint32_t m_TotalSize;
        uint16_t m_UnitSize;
        uint32_t m_UserData;
        Configuration m_Configuration;

        uint32_t m_Index;
        const void* m_cpPayload;
        uint16_t m_PayloadSize;

        uint32_t m_AckOffset;
        uint64_t m_AckFlag;
    };


    // 転送設定の内部管理
    struct InnerTransferSetting : public TransferSetting
    {
    public:
        bool IsValid() const
        {
            return (m_DataSize > 0);
        }
        bool IsPriorTo(const TransferSetting& setting) const;

    public:
        uint16_t m_Id;
        uint16_t m_UnitSize;
        uint32_t m_UnitNum;
    };


    // 送信者としてのステーションの状態
    enum SenderState
    {
        SenderState_Nothing,
        SenderState_Sending,
        SenderState_Rejected,
        SenderState_Finished
    };

    // 受信者としてのステーションの状態
    enum ReceiverState
    {
        ReceiverState_Nothing,
        ReceiverState_Receiving,
        ReceiverState_Rejecting,
        ReceiverState_Finished,
        ReceiverState_Rejected
    };

    // 各通信相手の状態を管理
    class Station
    {
    public:
        Station();
        void Initialize(ReliableBroadcastProtocol* pProtocol);

        void Startup(StationIndex stationId);
        void Cleanup();

        void HandleMessage(const MessageAccessor& accessor);
        bool SendAck();

        void ForceReserveReject()
        {
            m_ReservedAckFlag |= ReservedAckFlag_Reject;
        }

        void RejectSend();

        void ClearReceiverState()
        {
            m_ReceiverState = ReceiverState_Nothing;
        }
        void StartReceive(uint16_t receiveId, uint32_t unitNum);

        bool IsConnected() const
        {
            return MyStationIndex() != StationIndex_Invalid;
        }
        StationIndex GetStationIndex() const
        {
            return MyStationIndex();
        }
        SenderState GetSenderState() const
        {
            return m_SenderState;
        }
        ReceiverState GetReceiverState() const
        {
            return m_ReceiverState;
        }
        const InnerTransferSetting& GetRequest() const
        {
            return m_Request;
        }
        const SenderProgress* GetProgress() const
        {
            return &m_Progress;
        }

        uint16_t GetReturnedMessageIdx() const
        {
            return m_ReturnedMessageIdx;
        }
        uint16_t GetReceivedMessageIdx() const
        {
            return m_ReceivedMessageIdx;
        }

        void HandleMessageIdx(uint32_t messageIdx);

        uint32_t GetNextSendUnitIndex(bool isOnlyPrior, uint32_t dispatchCount) const;

        bool IsNeedSendRequest() const
        {
            return (GetReceiverState() == ReceiverState_Receiving) && m_IsNeedSendRequest;
        }

        void RecordSendData(uint32_t dispatchCount, uint16_t unitIdx, uint16_t messageIdx);

    private:
        void UpdateRequest(const MessageAccessor& accessor);
        bool CheckRequest(const MessageAccessor& accessor) const;

        StationIndex& MyStationIndex()
        {
            return m_Request.m_SourceStationIndex;
        }
        StationIndex MyStationIndex() const
        {
            return m_Request.m_SourceStationIndex;
        }
        uint16_t& SenderSequenceId()
        {
            return m_Request.m_Id;
        }
        uint16_t SenderSequenceId() const
        {
            return m_Request.m_Id;
        }

    private:
        ReliableBroadcastProtocol* m_pProtocol;

        InnerTransferSetting m_Request;
        SenderState m_SenderState;

        enum ReservedAckFlag
        {
            ReservedAckFlag_None = 0x0,
            ReservedAckFlag_Request = 0x1,
            ReservedAckFlag_Data = 0x2,
            ReservedAckFlag_Reject = 0x4,
            ReservedAckFlag_Command = 0x8
        };
        uint32_t m_ReservedAckFlag;

        SenderProgress m_Progress;
        uint16_t m_ReceivingId;
        ReceiverState m_ReceiverState;

        uint16_t m_ReturnedMessageIdx;
        uint16_t m_ReceivedMessageIdx;

        bool m_IsNeedSendRequest;

        uint32_t m_LastSentDataDispatchCount;
    };

    // 有効な相手か？
    bool IsValidStation(StationIndex stationIndex) const
    {
        return (stationIndex >= StationIndex_1) && (stationIndex < static_cast<StationIndex>(m_MaxConnections)) && (stationIndex != m_LocalStationIndex);
    }

    // 相手の情報の取得
    Station* GetStation(StationIndex stationIndex)
    {
        PIA_ASSERT(IsValidStation(stationIndex));
        return &m_paStation[stationIndex < m_LocalStationIndex ? stationIndex : stationIndex - 1];
    }

    // 相手の情報の取得
    const Station* GetStation(StationIndex stationIndex) const
    {
        PIA_ASSERT(IsValidStation(stationIndex));
        return &m_paStation[stationIndex < m_LocalStationIndex ? stationIndex : stationIndex - 1];
    }

    static uint32_t CalcProgress(const InnerTransferSetting& transferSetting, const Progress& progress);

    void UpdateState();
    bool DispatchSendAck();
    bool DispatchSendCommand();
    bool DispatchSendData();
    void DispatchOnReceiveCanceling();

    void PushRequest(const MessageAccessor& accessor, const Station& sender);
    bool PushData(const MessageAccessor& accessor, const Station& sender);
    void PushComplete(const MessageAccessor& accessor, const Station& sender);
    void PushCancel(const MessageAccessor& accessor, const Station& sender);

    bool CreateMessage(const MessageAccessor& accessor, Station* pDestinationStation);
    uint32_t IssueMessageIdx(const Station* cpDestStation);

    static bool IsSameSetting(const InnerTransferSetting& setting, const MessageAccessor& accessor);


    uint32_t m_MaxConnections;

    State m_State;
    StationIndex m_LocalStationIndex;

    InnerTransferSetting m_TransferSetting;

    void* m_pReceiveBuffer;
    ReceiverProgress m_ReceiveProgress;

    const void* m_cpSendBuffer;
    uint16_t m_NextSendId;
    uint32_t m_NextSendCommandStationIndex;
    uint32_t m_NextSendDataStationIndex;
    bool m_IsFinishSendEndless;

    uint32_t m_NextSendAckStationIndex;

    uint16_t m_SentMessageIdx;

    uint32_t m_ThroughputLimit;
    uint32_t m_RestThroughputLimit;

    uint32_t m_DispatchCount;

    Station* m_paStation;

    common::Time m_StartSendTime;

    static const uint32_t cDefaultPacketNumPerDispatch;
};
}
}
}  // end of namespace nn::pia::transport
