﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/common/common_Types.h>
#include <nn/pia/transport/transport_Protocol.h>

#include <nn/pia/transport/transport_ProtocolEvent.h>

namespace nn
{
namespace pia
{
namespace transport
{

class ReliableSlidingWindow;

/*!
  @brief  到達保証された通信を行うプロトコルです。

 */
class ReliableProtocol : public transport::Protocol
{
public:
    static const uint32_t ThroughputLimitMax = 0x0FFFFFFF; //!< SetThroughputLimit() で設定できる値の最大値です。

    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。

    @details アプリケーションプログラムは、このコンストラクタを直接呼び出してはいけません。
           インスタンスの作成は、 @ref Transport::CreateProtocol を利用してください。
     */
    ReliableProtocol(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。

    @details アプリケーションプログラムは、このデストラクタを直接呼び出してはいけません。
           インスタンスの破棄は、 @ref Transport::DestroyProtocol を利用してください。
     */
    virtual ~ReliableProtocol(void);
    //! @endcond


    PIA_PROTOCOL_TYPE_INFO(ProtocolTypeReliable);


    /*!
    @brief  インスタンスを初期化します。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。

    @details 送信バッファ、受信バッファで実際に利用された数は @ref common::WatermarkManager で
            @ref common::WatermarkManager::KeyReliableProtocolSendBufferNum および
            @ref common::WatermarkManager::KeyReliableProtocolReceiveBufferNum を
            指定することにより計測することができます。
    @details ReliableProtocol が消費するおおよそのメモリ量は、以下の計算式で求められます（単位はバイト）。<br>
             (sendBufferSize * 1500 + receiveBufferSize * 1500) * (Transport::Setting::maxStationNum - 1)

    @param[in] sendBufferSize           送信バッファのデータ数です。 1 以上の値を指定する必要があります。
                                        送信バッファは Send() したデータを送信相手に届いたことが確認されるまで保持するために使われます。
    @param[in] receiveBufferSize        受信バッファのデータ数です。 1 以上の値を指定する必要があります。
                                        受信バッファは、受信したデータを Receive() されるまで保持するために使われます。
    
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval ResultInvalidArgument       引数が誤っています。[:progErr]
    @retval ResultAlreadyInitialized    既に初期化されています。[:progErr]

    @see Finalize
     */
    Result Initialize(uint32_t sendBufferSize, uint32_t receiveBufferSize);


    /*!
    @brief  終了処理を行います。

    @see Initialize
     */
    void Finalize();


    /*!
    @cond PRIVATE
    @brief  通信を開始します。

    @param[in] localStationAddress  自分の StationAddress です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultNotInitialized    初期化されていません。[:progErr]
    @retval ResultInvalidState      既に通信中です。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    virtual Result StartupWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信を終了します。
     */
    virtual void CleanupWithStationAddress();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  メインの処理です。

    @return ResultSuccess が返されます。
     */
    virtual Result Dispatch();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  接続状態の切り替え。

    @param[in] event    操作内容を表すイベントです。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState      呼び出し状態が不正です。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    virtual Result UpdateProtocolEvent(const transport::ProtocolEvent& event);
    //! @endcond


    /*!
    @brief  データを送信します。

    @details この関数でセットされた送信データは、パケットに対して適切なサイズに分割され内部のバッファに保持されます。
            その後の @ref common::Scheduler::Dispatch() の中で順次パケットが作成され送信されていきます。
            送信してからしばらくたっても到達が確認できないデータは再送されます。
            再送までの時間は RTT に応じて自動的に調整されます。
            到達が確認できたデータはバッファから削除され、バッファはその後のデータ送信のために使う事ができるようになります。

    @param[in] destinationStationId     送信先の @ref StationId です。
    @param[in] pData                    送信するデータのアドレスです。
    @param[in] dataSize                 送信するデータのサイズ（バイト）です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState         ReliableProtocol が初期化されていないか、session のセットアップが完了していない可能性があります。[:progErr]
    @retval ResultInvalidArgument      引数が誤っています。[:progErr]
    @retval ResultNotInCommunication 指定された送信先と通信中ではありません。自分か指定した相手がセッション内に存在しません。[:handling]
    @retval ResultBufferIsFull         送信バッファに空きがありません。頻発する場合は、 Initialize() で指定する送信バッファのデータ数を増やすことを検討してください。[:handling]
    @retval ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    @retval ResultNotFound             指定された送信先が見つかりません。[:handling]

    @see Receive, SendToAll
     */
    Result Send(StationId destinationStationId, const void* pData, uint32_t dataSize);


    /*!
    @brief  データを全ステーションに向けて送信します。

    @details この関数でセットされた送信データは、パケットに対して適切なサイズに分割され内部のバッファに保持されます。
            その後の @ref common::Scheduler::Dispatch() の中で順次パケットが作成され送信されていきます。
            送信してからしばらくたっても到達が確認できないデータは再送されます。
            再送までの時間は RTT に応じて自動的に調整されます。
            到達が確認できたデータはバッファから削除され、バッファはその後のデータ送信のために使う事ができるようになります。

    @param[in] pData                    送信するデータのアドレスです。
    @param[in] dataSize                 送信するデータのサイズ（バイト）です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState         ReliableProtocol が初期化されていないか、session のセットアップが完了していない可能性があります。[:progErr]
    @retval ResultInvalidArgument      引数が誤っています。[:progErr]
    @retval ResultNotInCommunication   通信中ではありません。[:handling]
    @retval ResultBufferIsFull         送信バッファに空きがありません。頻発する場合は、 Initialize() で指定する送信バッファのデータ数を増やすことを検討してください。[:handling]

    @see Receive, Send
     */
    Result SendToAll(const void* pData, uint32_t dataSize);


    /*!
    @cond PRIVATE
    @brief  データを送信します。
     */
    Result Send(StationIndex destinationStationIndex, const void* pData, uint32_t dataSize);
    //! @endcond


    /*!
    @brief  データを受信します。

    @details @ref common::Scheduler::Dispatch() のタイミングで受信したパケットは解析され、内部のバッファに蓄積されていきます。
            Send() で分割されたデータがすべて届いていることが確認できているデータはこの関数で結合され取得できます。
            内部バッファもこのタイミングで解放され、次の受信のために使われます。

    @param[out] pSourceStationId    受信データの送信元の @ref StationId を書き込むバッファへのポインタです。
    @param[out] pBuffer             受信データを書き込むバッファへのポインタです。
    @param[out] pDataSize           受信データのサイズを書き込むバッファへのポインタです。
    @param[in]  bufferSize          受信データを書き込むバッファのサイズ（バイト）です。

    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。

    @retval ResultInvalidState      初期化されていません。[:progErr]
    @retval ResultInvalidArgument   引数が誤っています。[:progErr]
    @retval ResultNotInCommunication   通信中でありません。セッションに参加していない状態と同じです。[:handling]
    @retval ResultNoData            受信したデータはありません。[:handling]
    @retval ResultBrokenData        受信したデータが大きすぎて受信バッファに入りきらないか渡されたバッファに書ききれません。セッションのスタートアップからやり直す必要があります。
                                    この Result はチートなどによって送信側が不正なデータを送ってきたときに発生する可能性があるため実行時にハンドリングする必要がありますが、
                                    送信側の挙動が正常な場合はこの Result が返らないようにプログラムを修正する必要があります。[:handling]

    @see Send, SendToAll
     */
    Result Receive(StationId* pSourceStationId, void* pBuffer, uint32_t* pDataSize, uint32_t bufferSize);


    /*!
    @cond PRIVATE
    @brief  データを受信します。
     */
    Result Receive(StationIndex* pSourceStationIndex, void* pBuffer, uint32_t* pDataSize, uint32_t bufferSize);
    //! @endcond


    /*!
    @brief  指定した相手と通信中かどうかを取得します。

    @details 自分および指定した相手がセッション内に存在すれば true を返します。
    <br>
    @param[in] stationId    通信相手の @ref StationId です。

    @return 通信中なら true が返されます。
     */
    bool IsInCommunication(StationId stationId) const;


    /*!
    @cond PRIVATE
    @brief  指定した相手と通信中かどうかを取得します。
     */
    bool IsInCommunication(StationIndex stationId) const;
    //! @endcond


    /*!
    @brief  common::Scheduler::Dispatch の 1 回の呼び出しあたりにこのプロトコルが送信するデータ量の上限を指定します。

    @details このプロトコルが common::Scheduler::Dispatch の 1 回あたりに送信するデータ量の平均値が
    この関数で設定した値以下になるように制限されます。
    瞬間的にはここで設定した値以上のサイズの送信が行われることもあります。

    設定できる値の最大値は @ref ThroughputLimitMax です。これより大きな値を設定した場合は、@ref ThroughputLimitMax を設定したものと扱います。
    この API を呼び出さなかった場合のデフォルト値は @ref ThroughputLimitMax です。

    @param[in] throughputLimit  設定する送信データ量の上限サイズ（バイト）です。
    */
    void SetThroughputLimit(uint32_t throughputLimit);


    // 再送時間算出コールバック型。
    // rtt にはミリ秒単位での RTT が、resendCount にはデータの再送回数が渡されます。
    // コールバックの返り値が再送時間となります。単位はミリ秒です。
    typedef int32_t (*ResendTimeCallback)(int32_t rtt, int32_t resendCount);


    /*!
    @cond PRIVATE
    @brief  再送時間を算出するコールバック関数を登録します。
            デフォルトではコールバック関数は登録されていません。

    @return 直前まで保持されていたコールバック関数のポインタが返されます。
     */
    static ResendTimeCallback RegisterResendTimeCallback(ResendTimeCallback cb);
    //! @endcond


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

#if NN_PIA_EXPERIMENT_RELIABLE_DETAIL_INFO
    void PrintDebugInfo(void) const;
    void ClearAllResendCount(void);
    uint32_t GetAllResendCount(void) const;
#endif

private:
    ReliableSlidingWindow* GetSlidingWindow(StationIndex stationId);
    const ReliableSlidingWindow* GetSlidingWindow(StationIndex stationId) const;

    Result ReceiveImpl(StationIndex* pSourceStationIndex, void* pBuffer, uint32_t* pDataSize, uint32_t bufferSize, bool bCheckStationIdTable);

private:
    StationIndex m_LocalStationIndex;
    uint32_t m_MaxConnections;
    ReliableSlidingWindow* m_paSlidingWindow;
    uint32_t m_DispatchSlidingWindowIdx;

    uint32_t m_ThroughputLimit;
    uint32_t m_RestThroughputLimit;

    static ResendTimeCallback s_pfResendTimeCallback;  // 再送時間算出コールバック。
};
}
}
}  // end of namespace nn::pia::transport
