﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>
#include <nn/pia/transport/transport_UnreliableProtocol.h>

namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  UnreliableProtocol を継承し、負荷を分散しながら送信する機能を追加したクラスです。


 */
class RoundRobinUnreliableProtocol : public UnreliableProtocol
{
public:
    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。

    @details アプリケーションプログラムは、このコンストラクタを直接呼び出してはいけません。
           インスタンスの作成は、 @ref Transport::CreateProtocol を利用してください。
     */
    RoundRobinUnreliableProtocol(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。

    @details アプリケーションプログラムは、このデストラクタを直接呼び出してはいけません。
           インスタンスの破棄は、 @ref Transport::DestroyProtocol を利用してください。
     */
    virtual ~RoundRobinUnreliableProtocol(void);
    //! @endcond


    PIA_PROTOCOL_TYPE_INFO(ProtocolTypeRoundrobinUnreliable);


    /*!
    @cond PRIVATE
    @brief      通信を開始します。

    @param[in]  localStationAddress 自分の（ローカルの） StationAddress 。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    既に Startup() は呼ばれています。[:progErr]

     */
    virtual Result StartupWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      通信を終了します。
     */
    virtual void CleanupWithStationAddress(void);
    //! @endcond


    /*!
    @brief    データを複数のステーションに向けて送信します。

    @details  自分の StationIndex を始点とし、自分の StationIndex より大きい StationIndex が割り当てられた
              Station から順に stationNum で指定した数のステーションに向けてデータを送信します。
              最後にデータを送信した StationIndex を次回呼出し時の始点とします。

              例えば、Station A, B, C, D, E, F のセッションで自分の Station は F, 宛先数 2 で SendToRoundRobin() を
              呼び出した場合は以下のように送信されます。

              1回目の呼出し: SendToRoundRobin(Data1)   Data1 を A と B<br />
              2回目の呼出し: SendToRoundRobin(Data2)   Data2 を C と D<br />
              3回目の呼出し: SendToRoundRobin(Data3)   Data3 を E と A<br />
              4回目の呼出し: SendToRoundRobin(Data4)   Data4 を B と C<br />
              5回目の呼出し: SendToRoundRobin(Data5)   Data5 を D と E<br />
              
              この関数を呼び出しただけでは実際のデータ送信は行われません。
              この関数は、アプリケーションが渡したデータを Pia 内部のパケットバッファにコピーします。
              パケットは Pia 内部の送信スレッドにより定期的に送信されます。

    @param[in]  pData      送信したいデータの先頭を指すポインタ。
    @param[in]  dataSize   送信データサイズ。単位はバイトです。 GetDataSizeLimit() が返す値を超過してはいけません。
    @param[in]  stationNum データを送信したいステーションの数です。
    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval   ResultInvalidArgument 引数に誤りがあります。送信データサイズが過大であった場合にも、このエラーが返されます。[:progErr]
    @retval   ResultInvalidState    Initialize() が呼ばれていない可能性があります。[:progErr]
    @retval   ResultNotInCommunication 通信が可能な状態ではありません。[:handling]
    @see      Send, Receive, MaxDataSize, GetDataSizeLimit, IsInCommunication
     */
    Result SendToRoundRobin(const void* pData, uint32_t dataSize, uint32_t stationNum);


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


private:
    uint32_t m_LatestPariticipatingBitmap;
    StationIndex m_LastSentStationIndex;
};
}
}
}  // end of namespace nn::pia::transport
