﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  パケットのシーケンス ID を管理します。
          それに伴い、パケットロスをプロファイリングする機能も持ちます。

 */
class SequenceIdController : public nn::pia::common::RootObject
{
public:
    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。
     */
    SequenceIdController(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。
     */
    virtual ~SequenceIdController(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信開始
    @return ResultSuccess を返します。
     */
    Result Startup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  通信終了
     */
    void Cleanup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  次の送信シーケンス ID を取得
    @return 次の送信シーケンス ID です。
     */
    uint16_t GetNextSendSequenceId();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  受信シーケンス ID をチェック
    @return 有効なシーケンス ID なら true 、無効なシーケンス ID なら false を返します。
     */
    bool CheckReceivedSequenceId(uint16_t sequenceId);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  StationIndex_All 宛パケットを計測
     */
    void CountReceivedToAllPacket();
    //! @endcond


    /*!
    @brief  プロファイリングを開始します。
     */
    void StartProfile();


    /*!
    @brief  プロファイリングを終了します。
     */
    void StopProfile();


    /*!
    @brief  プロファイルした総パケット数を取得します。
            この関数はスレッドセーフです。
    @return プロファイルした総パケット数です。
     */
    uint16_t GetProfiledTotalPacketNum() const
    {
        return m_ProfileTotalPacketCount;
    }


    /*!
    @brief  プロファイルしたパケットロス数を取得します。
            この関数はスレッドセーフです。
    @return プロファイルしたパケットロス数です。
     */
    uint16_t GetProfiledPacketLossNum() const
    {
        return m_ProfilePacketLossCount;
    }


    /*!
    @brief  プロファイルしたマルチキャストのパケット数を取得します。
            この関数はスレッドセーフです。
    @return プロファイルしたマルチキャストのパケット数です。
     */
    uint16_t GetProfiledMulticastPacketNum() const
    {
        return m_ProfileMulticastPacketCount;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


    /*!
    @cond PRIVATE
    @brief  シーケンスIDの無効値
    @return シーケンスIDの無効値
     */
    static uint16_t GetInvalidSequenceId()
    {
        return InvalidSequenceId;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  プロファイルした総パケット数を取得します。（TransportAnalyzer用）
    @return プロファイルした総パケット数です。
     */
    uint32_t GetProfiledTotalPacketNumForTransportAnalyzer() const
    {
        return m_ProfileTotalPacketCountForTransportAnalyzer;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  プロファイルしたパケットロス数を取得します。（TransportAnalyzer用）
    @return プロファイルしたパケットロス数です。
     */
    uint32_t GetProfiledPacketLossNumForTransportAnalyzer() const
    {
        return m_ProfilePacketLossCountForTransportAnalyzer;
    }
    //! @endcond

private:
    // 実装しない
    SequenceIdController(const SequenceIdController& rhs);
    bool operator==(const SequenceIdController& rhs) const;
    bool operator!=(const SequenceIdController& rhs) const;
    SequenceIdController& operator=(const SequenceIdController& rhs);


    uint16_t m_SendSequenceId;
    uint16_t m_LastReceiveSequenceId;

    uint16_t m_ProfileTotalPacketCount;
    uint16_t m_ProfilePacketLossCount;
    uint16_t m_ProfileMulticastPacketCount;
    bool m_IsProfiling;

    uint32_t m_ProfileTotalPacketCountForTransportAnalyzer;
    uint32_t m_ProfilePacketLossCountForTransportAnalyzer;

    static const uint16_t InvalidSequenceId = 0;
};
}
}
}  // end of namespace nn::pia::transport
