﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/common/common_StationAddress.h>
#include <nn/pia/common/common_PacketDefine.h>
#include <nn/pia/common/common_Time.h>

#include <nn/pia/transport/transport_SequenceIdController.h>


// 先行宣言
namespace nn
{
namespace pia
{
namespace common
{
class StepSequenceJob;
}
}
}


namespace nn
{
namespace pia
{
namespace transport
{


// 先行宣言
class StationProtocol;
class StationProtocolReliable;
class RttProtocol;
class ConnectStationJob;
class DisconnectStationJob;
class ProcessConnectionRequestJob;
class ReliableSlidingWindow;
class NetworkFactory;

/*!
  @brief  ステーションを表現するクラスです。ステーションとは、 Pia のセッションに参加しているマシンを指す言葉です。

 */
class Station : public common::RootObject
{
    friend class StationProtocolReliable;

public:
    static const uint32_t MaxDataSize = common::ProtocolMessgaePayloadSizeMax;  // Send() / Receive() で送受信可能なデータサイズの最大値。単位はバイト。
    static const int32_t InvalidRtt = -1;                                       //!< GetRtt() が返しうる、無効な値です。

    /*!
    @brief Station の状態を示す列挙型です。
     */
    enum StationState
    {
        StationState_Invalid,                    //!< 無効な StationState です。
        StationState_Initial,                    //!< このステーションが初期化済みである状態を示します。
        StationState_CreatingSession,            //!< このステーションへ接続要求を行い、接続処理を開始した状態を示します。
        StationState_JoiningSession,             //!< このステーションから接続要求を受け、接続処理を開始した状態を示します。
        StationState_WaitingConnection,          //!< このステーションとの通信の確立を待っている状態を示します。
        StationState_Participating,              //!< このステーションと通信可能な状態を示します。
        StationState_Leaving,                    //!< このステーションとの通信を切断中である状態を示します。
        StationState_Max = StationState_Leaving  //!< StationState の最大値です。
    };


    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。

    @details インスタンスの StationIndex は、 StationIndex_Invalid に設定されます。
     */
    Station(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。
     */
    virtual ~Station(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Station インスタンスを初期化します。

    @param[in]  pFactory NetworkFactory から派生した具象クラスを指定します。このファクトリを用いて、ジョブのインスタンスが生成されます。
    @param[in]  pStationProtocol StationProtocol ポインタです。
    @param[in]  pRttProtocol RttProtocol ポインタです。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @see        Finalize
     */
    Result Initialize(NetworkFactory* pFactory, StationProtocol* pStationProtocol, RttProtocol* pRttProtocol);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Station インスタンスの解放処理を実行します。

    @see        Initialize
     */
    void Finalize(void);
    //! @endcond

    bool IsStartedUp() const
    {
        return m_IsStartedUp;
    }

    /*!
    @brief    StationIndex を取得します。

    @details この関数はスレッドセーフです。
    @details アプリケーションがジョイントセッション機能を使用する場合は、この関数を呼び出してはいけません。
             動作保証外となります。
    <br>
    @return   インスタンスに設定されていた StationIndex が返されます。
     */
    StationIndex GetStationIndex(void) const
    {
        return m_StationIndex;
    }


    /*!
    @cond PRIVATE
    @brief      StationIndex を設定します。
    @details    この関数は SetStationId と連携して使用する必要があります。
                可能であれば、この関数と SetStationId をほぼ同じタイミングで呼び、
                Station インスタンスにおける StationIndex と StationId の一貫性を保つ必要があります。
                StationIndex は決定できたものの、まだ StationId を決定することができない状況も
                あるかと思いますが、そのときは後で必ず SetStationId を呼び出す必要があります。

    @param[in]  idx このインスタンスに設定する StationIndex 。
    @see        GetStationIndex, SetStationId
     */
    void SetStationIndex(StationIndex idx);
    //! @endcond


    /*!
    @brief    StationId を取得します。

    @return   Station に対応づけられている StationId が返されます。
     */
    StationId GetStationId(void) const
    {
        return m_StationId;
    }


    /*!
    @cond PRIVATE
    @brief      StationId を設定します。
    @details    この関数は SetStationIndex と連携して使用する必要があります。
                可能であれば、この関数と SetStationIndex をほぼ同じタイミングで呼び、
                Station インスタンスにおける StationIndex と StationId の一貫性を保つ必要があります。
                StationIndex は決定できたものの、まだ StationId を決定することができない状況も
                あるかと思いますが、そのときは後で必ず SetStationId を呼び出す必要があります。

    @param[in]  id このインスタンスに設定する StationId 。
    @see        GetStationId, SetStationIndex
     */
    void SetStationId(StationId id);
    //! @endcond


    /*!
    @brief    ステーションアドレスを取得します。

    @return   設定されていた StationAddress への const 参照が返されます。
     */
    const common::StationAddress& GetStationAddress(void) const
    {
        return m_StationAddress;
    }


    /*!
    @cond PRIVATE
    @brief      ステーションアドレスを設定します。

    @param[in]  addr このインスタンスに設定する StationAddress 。
    @see        GetStationAddress
     */
    void SetStationAddress(const common::StationAddress& addr)
    {
        m_StationAddress = addr;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    ConnectStationJob のポインタを得ます（非 const 版）。
    @return   ConnectStationJob インスタンスを指すポインタ。
     */
    ConnectStationJob* GetConnectStationJob(void)
    {
        return m_pConnectStationJob;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    ConnectStationJob のポインタを得ます（ const 版）。
    @return   ConnectStationJob インスタンスを指す const ポインタ。
     */
    const ConnectStationJob* GetConnectStationJob(void) const
    {
        return m_pConnectStationJob;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    DisconnectStationJob のポインタを得ます（非 const 版）。
    @return   DisconnectStationJob インスタンスを指すポインタ。
     */
    DisconnectStationJob* GetDisconnectStationJob(void)
    {
        return m_pDisconnectStationJob;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    DisconnectStationJob のポインタを得ます（ const 版）。
    @return   DisconnectStationJob インスタンスを指す const ポインタ。
     */
    const DisconnectStationJob* GetDisconnectStationJob(void) const
    {
        return m_pDisconnectStationJob;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    ProcessConnectionRequestJob のポインタを得ます（非 const 版）。
    @return   ProcessConnectionRequestJob インスタンスを指すポインタ。
     */
    ProcessConnectionRequestJob* GetProcessConnectionRequestJob(void)
    {
        return m_pProcessConnectionRequestJob;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief    ProcessConnectionRequestJob のポインタを得ます（ const 版）。
    @return   ProcessConnectionRequestJob インスタンスを指す const ポインタ。
     */
    const ProcessConnectionRequestJob* GetProcessConnectionRequestJob(void) const
    {
        return m_pProcessConnectionRequestJob;
    }
    //! @endcond

    /*!
    @brief    SequenceIdController のポインタを得ます（非 const 版）。
              この関数はスレッドセーフです。
    @return   SequenceIdController インスタンスを指すポインタ。
     */
    SequenceIdController* GetSequenceIdController(void)
    {
        return &m_SequenceIdController;
    }


    /*!
    @brief    SequenceIdController のポインタを得ます（ const 版）。
              この関数はスレッドセーフです。
    @return   SequenceIdController インスタンスを指す const ポインタ。
     */
    const SequenceIdController* GetSequenceIdController(void) const
    {
        return &m_SequenceIdController;
    }


    /*!
    @cond PRIVATE
    @brief    Station の受信用の ConnectionId を得ます。
    @return   Station の受信用 ConnectionId。
     */
    uint8_t GetReceivingConnectionId(void) const
    {
        return m_RecvConnectionId;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Station の受信用の ConnectionId を設定します。
    @param[in]  id  この Station インスタンスに設定する受信用 ConnectionId。
     */
    void SetReceivingConnectionId(uint8_t id)
    {
        m_RecvConnectionId = id;
    }
    //! @endcond


    /*!
    @brief    Station の状態を得ます。

    @details この関数はスレッドセーフです。
    <br>
    @return   Station の状態を返します。
     */
    StationState GetState(void) const
    {
        return m_StationState;
    }


    /*!
    @cond PRIVATE
    @brief      Station の状態を設定します。

    @param[in]  state 設定する状態。
    @see        GetState
     */
    void SetState(StationState state)
    {
        m_StationState = state;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Stationのスタートアップ処理を行います。
    @details    この関数を呼び出した後に、SetStationId() で StationId を設定し、
                StationIndex と StationId の一貫性を保つ必要があります。
    @details    この関数を呼び出すと、本インスタンスの StationId は StationIdInvalid に設定されます。

    @param[in]  idx               このインスタンスに設定する StationIndex を与えます。
    @param[in]  addr              このインスタンスに設定する StationAddress を与えます。
    @return     成功すれば true 、失敗すれば false が返されます。
    @see        Cleanup, SetStationId
     */
    bool Startup(StationIndex idx, const common::StationAddress& addr);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Stationのスタートアップ処理を行います。
                このとき、本インスタンスの StationIndex は StationIndex_Invalid に設定されます。
    @details    この関数を呼び出すと、本インスタンスの StationId は StationIdInvalid に設定されます。

    @param[in]  addr              このインスタンスに設定する StationAddress を与えます。
    @return     成功すれば true 、失敗すれば false が返されます。
    @see        Cleanup
     */
    bool Startup(const common::StationAddress& addr);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Stationのスタートアップ処理を行います。
                このとき、本インスタンスの StationIndex は StationIndex_Invalid に設定され、
                StationAddress は StationAddress::Clear() が実行されたのと等しい状態に設定されます。
    @details    この関数を呼び出すと、本インスタンスの StationId は StationIdInvalid に設定されます。

    @return     成功すれば true 、失敗すれば false が返されます。
    @see Cleanup
     */
    bool Startup();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Station のクリーンアップ処理を行います。
                Startup() に対応する API です。 Startup() によって設定されていた内容がクリアされます。

    @see        Startup
     */
    void Cleanup(void);
    //! @endcond


    /*!
    @brief      ローカル Station と、この Station 間の RTT (Round Trip Time) を取得します。

    @details システムは RTT を取得する目的で、定期的に他 Station に向けてパルスを発信しています。
                しかし Station がセッションに接続した直後などでは、パルス発信／応答の処理がまだ
                一度も行われていないタイミングも存在します。
                このとき GetRtt() は InvalidRtt を返しますが、しばらく時間が経過してから GetRtt() を
                呼び出せば、正しい値が返されるようになります。
                本 API は、過去に計測した複数のサンプリング値から得られる中央値を返します。
                そのため、本 API で得られる値はネットワーク環境の微細な変動による影響を受けにくく、
                ある程度安定した値が返されます。

    @return     RTT 値が返されます。単位はミリ秒です。
    @retval     InvalidRtt  RTT 値を取得できる状態ではなかったときに返される、負の値です。
    @see        SetRttPulseInterval, GetRttPulseInterval, SetInitialRttPulseInterval, GetInitialRttPulseInterval, GetRttSamplingNum
     */
    int32_t GetRtt(void) const;


    /*!
    @brief      ローカル Station と、この Station 間の RTT を取得します。 RTT 算出に使用するサンプリング数を指定できます。

    @details    システムは RTT パルス発信／応答によって得られたサンプリング値を複数個保持しています。
                引数指定なしの GetRtt() では、保持されている全てのサンプリング値を用いて中央値を返しますが、
                この引数指定ありの GetRtt() は、指定された個数だけのサンプリング値を用いて中央値を返します。
                このとき、若い側のサンプリング値が使用されるので、小さな値を指定すると、最近のネットワーク環境を
                強く反映した RTT 値が得られます。
                その一方で、あまりに小さな値を設定すると、 RTT 値は変動しやすくなります。
    @param[in]  samplingNum RTT の算出に際して、直近のサンプリング値をいくつ使用するかを指定します。
    @return     RTT 値が返されます。単位はミリ秒です。
    @retval     InvalidRtt  RTT 値を取得できる状態ではなかったときに返される、負の値です。
    @see        SetRttPulseInterval, GetRttPulseInterval, SetInitialRttPulseInterval, GetInitialRttPulseInterval, GetRttSamplingNum
     */
    int32_t GetRtt(uint32_t samplingNum) const;


    /*!
    @cond PRIVATE
    @brief      ローカル Station と、この Station 間で計測された直近の RTT (Round Trip Time) を取得します。

    @return     システムによって直近に計測された RTT 値が返されます。単位はミリ秒です。
    @retval     InvalidRtt  RTT 値を取得できる状態ではなかったときに返される、負の値です。

    @see        GetRtt, SetRttPulseInterval, GetRttPulseInterval, SetInitialRttPulseInterval, GetInitialRttPulseInterval, GetRttSamplingNum
     */
    int32_t GetLatestRtt(void) const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      ローカル Station と、この Station 間で計測された RTT (Round Trip Time) の最小値と最大値を初期化します。

    @see        GetRttMin, GetRttMax
    */
    void ResetRttMinMax();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      ローカル Station と、この Station 間で計測された RTT (Round Trip Time) の最小値を取得します。

    @return     システムによって計測された RTT の最小値が返されます。単位はミリ秒です。
    @retval     InvalidRtt  RTT 値を取得できる状態ではなかったときに返される、負の値です。

    @see        GetRtt, SetRttPulseInterval, GetRttPulseInterval, SetInitialRttPulseInterval, GetInitialRttPulseInterval, GetRttSamplingNum, ResetRttMinMax
    */
    int GetRttMin() const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      ローカル Station と、この Station 間で計測された RTT (Round Trip Time) の最大値を取得します。

    @return     システムによって計測された RTT の最大値が返されます。単位はミリ秒です。
    @retval     InvalidRtt  RTT 値を取得できる状態ではなかったときに返される、負の値です。

    @see        GetRtt, SetRttPulseInterval, GetRttPulseInterval, SetInitialRttPulseInterval, GetInitialRttPulseInterval, GetRttSamplingNum, ResetRttMinMax
    */
    int GetRttMax() const;
    //! @endcond


    /*!
    @brief      システムが保持する RTT (Round Trip Time) サンプリング値の個数を取得します。

    @details    システムは RTT を取得する目的で、定期的に他 Station に向けてパルスを発信し、
                その応答に要した時間を RTT サンプリング値として保存します。
                しかし Station がセッションに接続した直後など、パルス発信／応答の処理が
                開始されたばかりのために、十分な数のサンプリング値が得られていない期間も存在します。
                本 API は、GetRtt() などが返す RTT 値の算出に用いられているサンプリング値の
                個数を返しますので、その RTT 値の信頼性／精度を知ることができます。

    @return     システムが保持しているサンプリング値の個数が返されます。
    @retval     0 RTT 値を取得できる状態ではなかった場合、ゼロが返されます。
    @see        GetRtt, SetRttPulseInterval, GetRttPulseInterval, SetInitialRttPulseInterval, GetInitialRttPulseInterval
     */
    int32_t GetRttSamplingNum(void) const;


    /*!
    @brief      RTT 算出のためのパルス送信間隔を設定します。

    @param[in]  msec  パルス送信間隔。単位はミリ秒です。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。非正数を指定した場合に返されます。[:progErr]
    @see        GetRttPulseInterval, GetRtt, SetInitialRttPulseInterval, GetInitialRttPulseInterval, GetRttSamplingNum
     */
    static Result SetRttPulseInterval(int32_t msec);


    /*!
    @brief      RTT 算出のためのパルス送信間隔を取得します。

    @return     設定されているパルス送信間隔が返ります。単位はミリ秒です。
    @see        SetRttPulseInterval, GetRtt, SetInitialRttPulseInterval, GetInitialRttPulseInterval, GetRttSamplingNum
     */
    static int32_t GetRttPulseInterval(void);


    /*!
    @brief      RTT 算出のための初期パルス送信間隔を設定します。

    @details    RTT 算出のパルス送出は、Station がセッションに参加してから開始されます。
                そのため、セッション参加直後の段階では、RTT 算出のためのサンプリング値の個数が
                十分ではありません。
                そこでシステムは、サンプリング値が十分な個数になるまでは、通常よりも短い間隔で
                パルス送出を試み、早期にサンプリング値の個数を満足させるように動作します。
                本 API は、この初期期間におけるパルス送出間隔を設定します。

    @param[in]  msec  初期パルス送信間隔。単位はミリ秒です。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。非正数を指定した場合に返されます。[:progErr]
    @see        GetInitialRttPulseInterval, GetRtt, SetRttPulseInterval, GetRttPulseInterval, GetRttSamplingNum
     */
    static Result SetInitialRttPulseInterval(int32_t msec);


    /*!
    @brief      RTT 算出のための初期パルス送信間隔を取得します。

    @details    RTT 算出のパルス送出は、Station がセッションに参加してから開始されます。
                そのため、セッション参加直後の段階では、RTT 算出のためのサンプリング値の個数が
                十分ではありません。
                そこでシステムは、サンプリング値が十分な個数になるまでは、通常よりも短い間隔で
                パルス送出を試み、早期にサンプリング値の個数を満足させるように動作します。
                本 API は、この初期期間におけるパルス送出間隔を取得します。

    @return     設定されている初期パルス送信間隔が返ります。単位はミリ秒です。
    @see        SetInitialRttPulseInterval, GetRtt, SetRttPulseInterval, GetRttPulseInterval, GetRttSamplingNum
     */
    static int32_t GetInitialRttPulseInterval(void);


    /*!
    @cond PRIVATE
    @brief    この Station に設定されている StationProtocol へのポインタを返します（非 const 版）。

    @return   StationProtocolへのポインタ。 StationProtocol が設定されていなければ、 NULL ポインタが返ります。
    @see      Startup
     */
    StationProtocol* GetStationProtocol(void)
    {
        return m_pStationProtocol;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    この Station に設定されている StationProtocol へのポインタを返します（ const 版）。

    @return   StationProtocolへのポインタ。 StationProtocol が設定されていなければ、 NULL ポインタが返ります。
    @see      Startup
     */
    const StationProtocol* GetStationProtocol(void) const
    {
        return m_pStationProtocol;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      最後に送信したパケットの送信時刻を記録します。

    @details この関数は、主に KeepAliveSender から呼び出されます。
    @param[in]  t パケット送信時刻。
    @see        GetLatestPacketSentTime
     */
    void SetLatestPacketSentTime(const common::Time& t)
    {
        m_LatestPacketSentTime = t;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      最後に送信したパケットの送信時刻を取得します。

    @details この関数は、主に KeepAliveSender から呼び出されます。
    @return     最後に送信したパケットの送信時刻。
    @see        SetLatestPacketSentTime
     */
    const common::Time& GetLatestPacketSentTime(void) const
    {
        return m_LatestPacketSentTime;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      最後に受信したパケットの受信時刻を記録します。

    @details この関数は、主に KeepAliveReceiver から呼び出されます。
    @param[in]  t パケット受信時刻。
    @see        GetLatestPacketReceivedTime
     */
    void SetLatestPacketReceivedTime(const common::Time& t)
    {
        m_LatestPacketReceivedTime = t;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      最後に受信したパケットの受信時刻を取得します。

    @details この関数は、主に session 層から呼び出され、キープアライブ判定に
                用いられます。
    @return     最後に受信したパケットの受信時刻。
    @see        SetLatestPacketReceivedTime
     */
    const common::Time& GetLatestPacketReceivedTime(void) const
    {
        return m_LatestPacketReceivedTime;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Station の接続/切断処理関連ジョブを強制終了、初期化します。
     */
    void CleanupJobs();
    //! @endcond


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


    static const uint32_t IdentificationTokenMaxDataSize = 32;  //!< 識別トークンの最大データサイズです。

    /*!
    @brief Station と結び付けて保存される識別トークン構造体です。

    @details 識別トークンは、接続した相手と共有されます。
     */
    struct IdentificationToken
    {
        uint8_t data[IdentificationTokenMaxDataSize];  //!< 識別トークンのデータ列です。
    };

    /*!
    @brief Station に結び付けて保存されるプレイヤー ID の構造体です。
    @details
    @if CAFE_DOC
    Cafe では使用されません。
    @endif
    @if CTR_DOC
    CTR では使用されません。
    @endif
    */
    struct PlayerId
    {
        uint64_t id[2];
    };

    typedef common::FixedString<MaxPlayerNameLength> PlayerNameString;
    typedef common::FixedString<MaxUserNameLength> UserNameString;

    /*!
     @brief Station に結び付けて保存されるプレイヤー情報の構造体です。
     @details プレイヤー名、プレイヤー名の言語タイプは接続した相手と共有されます。プレイヤー ID は共有されません。
    */
    struct PlayerInfo
    {
        PlayerInfo()
        {
            Reset();
        }

        /*!
        @brief 設定をコピーします。
        */
        void Copy(const PlayerInfo& rhs)
        {
            screenName.Copy(rhs.screenName);
            nameStringLanguage = rhs.nameStringLanguage;
            playerId = rhs.playerId;
        }

        /*!
        @brief 設定をリセットします。
        */
        void Reset()
        {
            screenName.Clear();
            nameStringLanguage = 0xff;
            playerId.id[0] = 0;
            playerId.id[1] = 0;
        }

        /*!
        @brief プレイヤー名です。名前の長さは MaxPlayerNameLength（NULL終端を含めない）以内である必要があります。
        @if CAFE_DOC 文字コードは UTF-16BE である必要があります。 @endif
        @if CTR_DOC 文字コードは UTF-16 である必要があります。 @endif
        @if NIN_DOC 文字コードは UTF-8 である必要があります。 「いっしょにあそんだ人の記録」に「ゲーム中での名前」として表示されます。@endif
        */
        PlayerNameString screenName;
        /*!
        @brief プレイヤー名の言語タイプです。
        @if CAFE_DOC nn::fp::Language で定義される値で指定して下さい。 @endif
        @if CTR_DOC 本体の言語設定を使用したい場合には、nn::cfg::CTR::GetLanguage() の値を使用可能です。 @endif
        @if NIN_DOC nn::settings::Language の enum 値を uint8_t にキャストして指定してください。@endif
        */
        uint8_t nameStringLanguage;
        /*!
        @brief プレイヤー ID です。
        @if CAFE_DOC 使用されません。@endif
        @if CTR_DOC 使用されません。@endif
        @if NIN_DOC アカウントシステムに登録されたユーザーの識別子（nn::account::Uid）を指定します。指定しない場合はゲストプレイヤーとして扱い、「いっしょにあそんだ人の記録」には記録しません。@endif
        */
        PlayerId playerId;
    };

    /*!
    @cond PRIVATE
    @brief Station に結び付けて保存されるプレイヤーの登録キー構造体です。
    */
    static const uint32_t PlayerRegistrationKeySize = 64;
    struct PlayerRegistrationKey
    {
        uint8_t data[PlayerRegistrationKeySize];
    };
    //! @endcond

    /*!
    @brief Station の識別トークンを取得します。

    @details ローカル Station の場合は @ref session::Session::Startup で設定した識別トークンを取得します。
             ローカル Station 以外の Station の場合は接続時に共有している識別トークンを取得します。
             設定していない場合、この API の処理には成功しますが、取得した識別トークンのデータは全て NULL となります。

             識別トークンの取得に失敗した場合、格納先として指定された識別トークンの内部値を変更することはありません。

    @param[out] pToken          Station の識別トークンが取得できた場合、pToken に格納されます。
    @return 正常に識別トークンを取得できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState     関数が実行可能な状態ではありません。[:progErr]
    @retval ResultInvalidArgument  引数で指定されたポインタが有効ではない場合に返ります。[:progErr]
    @retval ResultNoData   指定された Station の識別トークンが取得できなかった場合に返ります。識別トークンの共有が完了していない可能性があります。[:handling]
     */
    Result GetIdentificationToken(IdentificationToken* pToken);

    /*!
    @brief      Station と結びつけられているプレイヤー情報を取得します。
    @details    ローカル Station の場合は @ref session::Session::Startup で設定したプレイヤー情報を取得します。
                ローカル Station 以外の Station の場合は接続時に共有しているプレイヤー情報を取得します。
                ただし、取得したプレイヤー情報のプレイヤー ID の値は不定のため、使用してはいけません。<br >
                <br >
                プレイヤー情報の取得に失敗した場合、格納先として指定された変数の内部値を変更することはありません。

    @param[in]  index               Station と結び付けられているプレイヤー情報リストから情報を取得したいプレイヤーのインデックス 
    @param[out] pInfo               取得するプレイヤー情報のコピー先
    @return                         正常に情報を取得できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState      関数が実行可能な状態ではありません。[:progErr]
    @retval ResultInvalidArgument   引数で指定されたインデックスやポインタが有効ではない場合に返ります。[:progErr]
    @retval ResultNoData            指定された Station の情報が取得できなかった場合に返ります。情報の共有が完了していない可能性があります。[:handling]
     */
    Result GetPlayerInfo(uint8_t index, PlayerInfo* pInfo);

    /*!
    @brief      Station のプリンシパル ID を取得します。
    @details    プリンシパル ID はセッション参加後に取得できます。
                プリンシパル ID の取得に失敗した場合、格納先として指定されたプリンシパル ID の内部値を変更することはありません。<br />
                本 API で取得するプリンシパル ID はネットワーク種別によって以下のように設定されます。
                <ul>
                @if CTR_DOC
                <li>インターネットマッチメイクの場合、ゲームサーバーから設定されるプリンシパル ID</li>
                @endif
                @if CAFE_DOC
                <li>インターネットマッチメイクの場合、ゲームサーバーから設定されるプリンシパル ID</li>
                @endif
                @if NIN_DOC
                <li>インターネットマッチメイクの場合、ゲームサーバーにログインしたユーザーアカウントに連携されているネットワークサービスアカウント ID</li>
                @endif
                <li>ローカルマッチメイクの場合、ステーションアドレスと Tick 値を元に生成する ID（重複する可能性はほぼゼロです）</li>
                <li>LAN マッチメイクの場合、ステーションアドレスと Tick 値を元に生成する ID（重複する可能性はほぼゼロです）</li>
                </ul>
                
    @param[out] pPrincipalId        Station のプリンシパル ID が取得できた場合、pPrincipalId に格納されます。
    @return                         正常にプリンシパル ID を取得できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval ResultInvalidState      関数が実行可能な状態ではありません。[:progErr]
    @retval ResultInvalidArgument   引数で指定されたポインタが有効ではない場合に返ります。[:progErr]
    @retval ResultNoData            指定された Station のプリンシパル ID が取得できなかった場合に返ります。ステーション情報の共有が完了していない可能性があります。[:handling]
     */
    Result GetPrincipalId(PrincipalId* pPrincipalId);

    /*!
    @brief Station のプレイヤー数を取得します。
     */
    Result GetPlayerCount(uint8_t* pPlayerCount);

    /*!
    @brief Station のマッチメイク参加者数を取得します。
     */
    Result GetParticipantCount(uint8_t* pParticipantCount);

    /*!
    @cond PRIVATE
    @brief      対象の Station との通信経路が直通接続か否かを設定します。
     */
    void SetConnectionRoute(bool isDirect)
    {
        m_IsConnectionRouteDirect = isDirect;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      対象の Station との通信経路がリレー接続か否かを取得します。
    @return     リレー接続の場合は true, 接続できていない場合や通信経路がそれ以外の場合は false を返します。
     */
    bool IsConnectionRouteRelay() const;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      対象の Station との接続経路が直通接続か否かを取得します。
    @return     直通接続の場合は true, 接続できていない場合や通信経路がそれ以外の場合は false を返します。
     */
    bool IsConnectionRouteDirect() const;
    //! @endcond

    /*!
    @cond PRIVATE
     */
    bool GetDisconnectImmidiatelyFlag() const
    {
        return m_IsDisconnectImmidiately;
    }
    //! @endcond

    /*!
    @cond PRIVATE
     */
    void SetDisconnectImmidiatelyFlag(bool flag)
    {
        m_IsDisconnectImmidiately = flag;
    }
    //! @endcond
private:
    // Startup()とCleanup()における共通処理をまとめるために用意したヘルパ関数です。
    // 第一引数で、スタートアップ処理か、クリーンアップ処理かを切り替えます。
    // 第二引数以降で与えられた引数は、メンバ変数に無条件にセットします（すなわち、引数チェック無し）。
    void helperStartupCleanup(bool bStartup, StationIndex idx, const common::StationAddress& addr, StationState state);

    bool m_IsStartedUp;

    common::StationAddress m_StationAddress;
    StationIndex m_StationIndex;
    StationId m_StationId;
    StationState m_StationState;
    ReliableSlidingWindow* m_pReliableSlidingWindow;
    StationProtocol* m_pStationProtocol;
    RttProtocol* m_pRttProtocol;
    ConnectStationJob* m_pConnectStationJob;
    DisconnectStationJob* m_pDisconnectStationJob;
    ProcessConnectionRequestJob* m_pProcessConnectionRequestJob;
    SequenceIdController m_SequenceIdController;
    uint8_t m_RecvConnectionId;

    bool m_IsConnectionRouteDirect;

    common::Time m_LatestPacketSentTime;
    common::Time m_LatestPacketReceivedTime;

    bool m_IsDisconnectImmidiately;

    uint8_t m_PlayerCount;

    Station(const Station&);
    Station& operator=(const Station&);
};
}
}
}  // end of namespace nn::pia::transport
