﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>
#include <nn/pia/transport/transport_StationLocation.h>

namespace nn
{
namespace pia
{
namespace transport
{

/*!
  @brief      ステーションを一意に識別するためのアドレスです。

  @details メインアドレスとサブアドレスの２つの StationLocation を持ちます。
  <br>
 */
class StationConnectionInfo : public nn::pia::common::RootObject
{
public:
    /*!
    @brief コンストラクタです。
     */
    StationConnectionInfo(void);

    /*!
    @brief コピーコンストラクタです。
    @param[in] o コピーコンストラクタの引数です。
     */
    StationConnectionInfo(const StationConnectionInfo& o);

    /*!
    @brief デストラクタです。
     */
    virtual ~StationConnectionInfo(void);

    void SetStationConnectionInfo(const StationConnectionInfo& o);

    /*!
    @brief 代入演算子です。

    @param[in] o 代入する StationConnectionInfo オブジェクトを指定します。
    @return 本オブジェクトの参照が返ります。
     */
    StationConnectionInfo& operator=(const StationConnectionInfo& o);

    bool operator==(const StationConnectionInfo& o) const;

    bool operator!=(const StationConnectionInfo& o) const
    {
        return !(*this == o);
    }

    bool HasPrivateAddress() const
    {
        return m_SubLocation.GetStationAddress().GetInetAddress().IsValid();
    }

    bool IsSamePublicAddress(const StationConnectionInfo& o) const
    {
        return m_MainLocation.IsSameAddress(o.m_MainLocation);
    }

    /*!
    @brief StationLocation （メインアドレス）の const 参照を得ます。

    @return    StationLocation （メインアドレス）の const 参照が返ります。
     */
    const StationLocation& GetMainLocation(void) const
    {
        return m_MainLocation;
    }

    /*!
    @brief StationLocation （メインアドレス）の参照を得ます。

    @return    StationLocation （メインアドレス）の参照が返ります。
     */
    StationLocation& GetMainLocation(void)
    {
        return m_MainLocation;
    }

    /*!
    @brief StationLocation （メインアドレス）を設定します。

    @param[in] o 設定する StationLocation 。
     */
    void SetMainLocation(const StationLocation& o)
    {
        m_MainLocation.SetStationLocation(o);
    }

    /*!
    @brief StationLocation （サブアドレス）の const 参照を得ます。

    @return    StationLocation （サブアドレス）の const 参照が返ります。
     */
    const StationLocation& GetSubLocation(void) const
    {
        return m_SubLocation;
    }

    /*!
    @brief StationLocation （サブアドレス）の参照を得ます。

    @return  StationLocation （サブアドレス）の参照が返ります。
     */
    StationLocation& GetSubLocation(void)
    {
        return m_SubLocation;
    }

    /*!
    @brief StationLocation （サブアドレス）を設定します。

    @param[in] o 設定する StationLocation 。
     */
    void SetSubLocation(const StationLocation& o)
    {
        m_SubLocation.SetStationLocation(o);
    }

    /*!
    @brief StationLocation （メインアドレス）の const 参照を得ます。
    メインアドレスがパブリックアドレスのときに使用されることを想定しています。

    @return    StationLocation （メインアドレス）の const 参照が返ります。
     */
    const StationLocation& GetPublicLocation(void) const
    {
        return m_MainLocation;
    }

    /*!
    @brief StationLocation （メインアドレス）の参照を得ます。
    メインアドレスがパブリックアドレスのときに使用されることを想定しています。

    @return    StationLocation （メインアドレス）の参照が返ります。
     */
    StationLocation& GetPublicLocation(void)
    {
        return m_MainLocation;
    }

    /*!
    @brief StationLocation （サブアドレス）の const 参照を得ます。
    サブアドレスがプライベートアドレスのときに使用されることを想定しています。

    @return    StationLocation （サブアドレス）の const 参照が返ります。
     */
    const StationLocation& GetPrivateLocation(void) const
    {
        return m_SubLocation;
    }

    /*!
    @brief StationLocation （サブアドレス）の参照を得ます。
    サブアドレスがプライベートアドレスのときに使用されることを想定しています。

    @return  StationLocation （サブアドレス）の参照が返ります。
     */
    StationLocation& GetPrivateLocation(void)
    {
        return m_SubLocation;
    }

    virtual uint32_t GetSerializedSize(void) const;
    virtual Result Serialize(uint8_t* pBuffer, uint32_t* pSerializedSize, uint32_t bufferSize) const;
    virtual Result Deserialize(const uint8_t* pData);

    virtual uint32_t GetStreamlinedSerializedSize(void) const;
    virtual Result StreamlinedSerialize(uint8_t* pBuffer, uint32_t* pSerializedSize, uint32_t bufferSize) const;
    virtual Result StreamlinedDeserialize(const uint8_t* pData);

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

private:
    uint32_t LittleMembersSerialize(uint8_t* pBuffer, const StationLocation& src) const;
    uint32_t LittleMembersDeserialize(StationLocation* pDest, const uint8_t* pData);

    StationLocation m_MainLocation;
    StationLocation m_SubLocation;
};
}
}
}
