﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>
#include <nn/pia/common/common_StationAddress.h>

namespace nn
{
namespace pia
{
namespace inet
{
class NexFacade;
}
}
}

namespace nn
{
namespace pia
{
namespace transport
{

// 先行宣言
class StationConnectionInfo;

/*!
  @brief      ステーションを一意に識別するためのアドレスです。

 */
class StationLocation : public nn::pia::common::RootObject
{
    friend class StationConnectionInfo;

public:
    /*!
    @brief コンストラクタです。
     */
    StationLocation(void);


    /*!
    @brief コピーコンストラクタです。
     */
    StationLocation(const StationLocation& o);


    /*!
    @brief デストラクタです。
     */
    virtual ~StationLocation(void);

    void SetStationLocation(const StationLocation& o);

    /*!
    @brief 代入演算子です。

    @return 本オブジェクトの参照が返ります。
     */
    StationLocation& operator=(const StationLocation& o);

    bool operator==(const StationLocation& o) const;

    bool operator!=(const StationLocation& o) const
    {
        return !(*this == o);
    }

    bool IsSameAddress(const StationLocation& o) const
    {
        return m_Address.GetInetAddress().GetAddress() == o.m_Address.GetInetAddress().GetAddress();
    }

    bool IsSameServerCId(const StationLocation& o) const
    {
        return (m_ServiceConnectionId == o.m_ServiceConnectionId);
    }

    bool IsSameConnectionId(const StationLocation& o) const
    {
        return (m_NexConnectionId == o.m_NexConnectionId);
    }

    /*!

    @brief nn::pia::common::StationAddress の const 参照を得ます。

    @return    nn::pia::common::StationAddress の const 参照が返ります。
     */
    const nn::pia::common::StationAddress& GetStationAddress(void) const
    {
        return m_Address;
    }

    /*!
    @brief nn::pia::common::StationAddress の参照を得ます。

    @return    nn::pia::common::StationAddress の参照が返ります。
     */
    nn::pia::common::StationAddress& GetStationAddress(void)
    {
        return m_Address;
    }

    /*!
    @brief nn::pia::common::StationAddress を設定します。

    @param[in] addr 設定するインターネットアドレス。
     */
    void SetStationAddress(const nn::pia::common::StationAddress& addr);

    virtual uint32_t GetSerializedSize(void) const;
    virtual Result Serialize(uint8_t* pBuffer, uint32_t* pSerializedSize, uint32_t bufferSize) const;
    virtual Result Deserialize(const uint8_t* pData);

    void SetNexServerConnectionId(uint32_t o)
    {
        m_ServiceConnectionId = o;
    }

    uint32_t GetNexServerConnectionId() const
    {
        return m_ServiceConnectionId;
    }

    void SetNexUrlType(uint8_t o)
    {
        m_NexUrlType = o;
    }

    uint8_t GetNexUrlType() const
    {
        return m_NexUrlType;
    }

    void SetProbeRequestInitiation(uint8_t o)
    {
        m_IsProbeRequestInitiation = o;
    }

    uint8_t IsProbeRequestInitiation() const
    {
        return m_IsProbeRequestInitiation;
    }

    void SetNatTypeFlag(uint8_t flags)
    {
        m_NatTypeFlags = flags;
    }

    uint8_t GetNatTypeFlag() const
    {
        return m_NatTypeFlags;
    }

    bool IsNatTypeFlagSet(uint8_t flag) const
    {
        return (m_NatTypeFlags & flag) == flag;
    }

    void GetLocationString(common::String* pString) const;

    void SetPrincipalId(PrincipalId principalId)
    {
        m_NexPrincipalId = principalId;
    }

    PrincipalId GetPrincipalId() const
    {
        return m_NexPrincipalId;
    }

    void SetNatMapping(uint8_t o)
    {
        m_NatMapping = o;
    }

    uint8_t GetNatMapping() const
    {
        return m_NatMapping;
    }

    void SetNatFiltering(uint8_t o)
    {
        m_NatFiltering = o;
    }

    uint8_t GetNatFiltering() const
    {
        return m_NatFiltering;
    }

    void SetNexConnectionId(uint32_t o)
    {
        m_NexConnectionId = o;
    }

    uint32_t GetNexConnectionId() const
    {
        return m_NexConnectionId;
    }

    void SetNexStreamId(uint8_t o)
    {
        m_NexStreamId = o;
    }

    uint8_t GetNexStreamId() const
    {
        return m_NexStreamId;
    }

    void SetNexStreamType(uint8_t o)
    {
        m_NexStreamType = o;
    }

    uint8_t GetNexStreamType() const
    {
        return m_NexStreamType;
    }

    void SetRelayServerAddress(const common::InetAddress& inetAddress)
    {
        m_RelayServerAddress = inetAddress;
    }

    const common::InetAddress& GetRelayServerAddress() const
    {
        return m_RelayServerAddress;
    }
    common::InetAddress& GetRelayServerAddress()
    {
        return m_RelayServerAddress;
    }

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

private:
    common::StationAddress m_Address;

    //n1589:これらを別の型にすることでNEX依存を回避している
    // nn::nex::PrincipalID
    PrincipalId m_NexPrincipalId;
    // nn::nex::ConnectionID
    uint32_t m_NexConnectionId;
    // nn::nex::ConnectionID
    uint32_t m_ServiceConnectionId;

    // nn::nex::StationURL::URLType
    uint8_t m_NexUrlType;
    // nn::nex::Stream::ID
    uint8_t m_NexStreamId;
    // nn::nex::Stream::Type
    uint8_t m_NexStreamType;

    // nn::nex::NATProperties::MappingProperties
    uint8_t m_NatMapping;
    // nn::nex::NATProperties::FilteringProperties
    uint8_t m_NatFiltering;

    uint8_t m_NatTypeFlags;

    uint8_t m_IsProbeRequestInitiation;

    common::InetAddress m_RelayServerAddress;
};
}
}
}
