﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/transport/transport_Singleton.h>
#include <nn/pia/transport/transport_Station.h>

#include <nn/pia/common/common_ObjList.h>
#include <nn/pia/common/common_SimpleContainer.h>


// 先行宣言(PiaCommon)
namespace nn
{
namespace pia
{
namespace common
{
class InetAddress;
}
}
}


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  複数の Station インスタンスを統括するクラスです。

  @details StationManager クラスのインスタンスは、 Transport クラスによって構築／破棄されます。
          ですので、アプリケーションプログラムが StationManager クラスの CreateInstance() や
          DestroyInstance() を呼び出す必要はありません。

 */
class StationManager : public common::RootObject
{
public:
    typedef common::ObjList<Station> TStationList;
    typedef common::SimpleContainer<Station*, static_cast<int>(MaxStationNum)> TStationPtrArray;

    /*!
     @brief セッションに参加している Station を参照するためのイテレータです（非 const 版）。
     */
    class Iterator
    {
    public:
        Iterator(TStationPtrArray::Iterator it)
            : m_Iterator(it)
        {
        }
        Station& operator*(void)
        {
            return **m_Iterator;
        }
        Station* operator->(void)
        {
            return *m_Iterator;
        }
        bool operator==(const Iterator& rhs) const
        {
            return m_Iterator == rhs.m_Iterator;
        }
        bool operator!=(const Iterator& rhs) const
        {
            return !(*this == rhs);
        }
        Iterator& operator++(void)
        {
            ++m_Iterator;
            return *this;
        }

    private:
        TStationPtrArray::Iterator m_Iterator;
    };

    /*!
     @brief セッションに参加している Station を参照するためのイテレータです（ const 版）。
     */
    class ConstIterator
    {
    public:
        ConstIterator(TStationPtrArray::ConstIterator it)
            : m_ConstIterator(it)
        {
        }
        const Station& operator*(void)
        {
            return **m_ConstIterator;
        }
        const Station* operator->(void)
        {
            return *m_ConstIterator;
        }
        bool operator==(const ConstIterator& rhs) const
        {
            return m_ConstIterator == rhs.m_ConstIterator;
        }
        bool operator!=(const ConstIterator& rhs) const
        {
            return !(*this == rhs);
        }
        ConstIterator& operator++(void)
        {
            ++m_ConstIterator;
            return *this;
        }

    private:
        TStationPtrArray::ConstIterator m_ConstIterator;
    };


    /*!
    @cond PRIVATE
    @brief    StationManager クラスのインスタンスを作成します（シングルトンパターン）。

    @return   インスタンスの作成に成功すれば、成功の Result が返されます。
    @retval   ResultNotInitialized     transport モジュールが未初期化です。[:progErr]
    @retval   ResultInvalidState       CreateInstance() を呼ぶタイミングが誤っています。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。[:progErr]
    @retval   ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    @see      DestroyInstance, GetInstance
     */
    static Result CreateInstance(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    StationManager クラスのインスタンスを破棄します（シングルトンパターン）。
    @see      CreateInstance, GetInstance
     */
    static void DestroyInstance(void);
    //! @endcond


    /*!
    @brief    StationManager クラスのインスタンスを得ます（シングルトンパターン）
              事前に CreateInstance() によってインスタンスを作成していない場合は、
              NULL ポインタが返されます。
              この関数はスレッドセーフです。

    @return   インスタンスへのポインタ。
    @see      CreateInstance, DestroyInstance
     */
    static StationManager* GetInstance(void)
    {
        return s_pInstance;
    }


    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。
     */
    StationManager(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。
     */
    virtual ~StationManager(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      StationManager を初期化します。

    @param[in]  pFactory    NetworkFactory から派生させた具象クラスのインスタンスを指すポインタを渡します（ファクトリパターン）。
    @param[in]  pStationProtocol StationProtocol ポインタです。
    @param[in]  pRttProtocol RttProtocol ポインタです。
    @param[in]  maxStations 管理する Station 数の最大値を指定します。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultAllocationFailed メモリ確保に失敗しました。 common::Initialize() 呼び出し時により多くのメモリを供給するか、 maxStations の値を小さくする必要があります。[:progErr]
    @see        Finalize
     */
    Result Initialize(NetworkFactory* pFactory, uint32_t maxStations, StationProtocol* pStationProtocol, RttProtocol* pRttProtocol);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      StationManager を解放します。

    @see        Initialize
     */
    void Finalize(void);
    //! @endcond


    /*!
    @brief      Station インスタンスのイテレータを返します（非 const 版）。

    @return     StationManager が管理している Station インスタンス群の
                先頭にある要素のイテレータを返します。
    @see        End
     */
    Iterator Begin(void)
    {
        return Iterator(m_ValidStationPtr.Begin());
    }


    /*!
    @brief      Station インスタンスのイテレータを返します（非 const 版）。

    @return     StationManager が管理している Station インスタンス群の
                末尾の要素の次に対応するイテレータを返します。
    @see        Begin
     */
    Iterator End(void)
    {
        return Iterator(m_ValidStationPtr.End());
    }


    /*!
    @brief      Station インスタンスのイテレータを返します（ const 版）。

    @return     StationManager が管理している Station インスタンス群の
                先頭にある要素のイテレータを返します。
    @see        End
     */
    ConstIterator Begin(void) const
    {
        return ConstIterator(m_ValidStationPtr.Begin());
    }


    /*!
    @brief      Station インスタンスのイテレータを返します（ const 版）。

    @return     StationManager が管理している Station インスタンス群の
                末尾の要素の次に対応するイテレータを返します。
    @see        Begin
     */
    ConstIterator End(void) const
    {
        return ConstIterator(m_ValidStationPtr.End());
    }


    /*!
    @cond PRIVATE
    @brief    ローカルステーションの Station インスタンスを作成します。

    @return   成功すれば、作成した Station インスタンスを指すポインタが返ります。
              既にローカルステーションが作成されているなどの理由で失敗したときは、 NULL が返されます。
    @see      CreateStation
     */
    Station* CreateLocalStation(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    Station インスタンスを作成します。

    @return   成功すれば、作成した Station インスタンスを指すポインタが返ります。
              既に Station 数が Initialize() 時に指定した最大 Station 数に達している
              ような状況では、 NULL が返されます。
    @see      CreateLocalStation, Initialize
     */
    Station* CreateStation(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      指定された Station を破棄します。

    @param[in]  pStation 破棄対象の Station を指すポインタ。このポインタは、 StationManager から取得したものでなければなりません。
    @see        CreateStation, CreateLocalStation
     */
    void DestroyStation(Station* pStation);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      StationIndex で指定された Station を破棄します。

    @param[in]  idx 破棄対象 Station の StationIndex 。
    @see        CreateStation, CreateLocalStation
     */
    void DestroyStation(StationIndex idx);
    //! @endcond


    /*!
    @brief    ローカルステーションを取得します（非 const 版）。

    @details この関数はスレッドセーフです。
    <br>
    @return   ローカルステーションが存在すれば、有効なポインタが返ります。存在していなければ、 NULL ポインタが返されます。
     */
    Station* GetLocalStation(void)
    {
        return m_pLocalStation;
    }


    /*!
    @brief    ローカルステーションを取得します（ const 版）。

    @details この関数はスレッドセーフです。
    <br>
    @return   ローカルステーションが存在すれば、有効なポインタが返ります。存在していなければ、 NULL ポインタが返されます。
     */
    const Station* GetLocalStation(void) const
    {
        return m_pLocalStation;
    }


    /*!
    @brief      StationId に対応する Station を得ます（非 const 版）。

    @param[in]  id StationId 。
    @return     指定した StationId に対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなかったり、StationId が無効の場合は NULL ポインタが返ります。
    @see        GetStationByAddress
     */
    Station* GetStation(StationId id);


    /*!
    @brief      StationId に対応する Station を得ます（ const 版）。

    @param[in]  id StationId 。
    @return     指定した StationId に対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなかったり、StationId が無効の場合は NULL ポインタが返ります。
    @see        GetStationByAddress
     */
    const Station* GetStation(StationId id) const;


    /*!
    @cond PRIVATE
    @brief      StationIndex に対応する Station を得ます（非 const 版）。

    @param[in]  idx StationIndex 。
    @return     指定した StationIndex に対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなかったり、StationIndex が無効の場合は NULL ポインタが返ります。
    @see        GetStationByAddress
     */
    Station* GetStation(StationIndex idx);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      StationIndex に対応する Station を得ます（ const 版）。

    @param[in]  idx StationIndex 。
    @return     指定した StationIndex に対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなかったり、StationIndex が無効の場合は NULL ポインタが返ります。
    @see        GetStationByAddress
     */
    const Station* GetStation(StationIndex idx) const;
    //! @endcond


    /*!
    @brief      StationAddress に対応する Station を得ます（非 const 版）。

    @param[in]  addr ステーションアドレス。
    @return     指定したステーションアドレスに対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなければ、 NULL ポインタが返ります。
    @see        GetStationByAddress
     */
    Station* GetStation(const common::StationAddress& addr);


    /*!
    @brief      StationAddress に対応する Station を得ます（ const 版）。

    @param[in]  addr ステーションアドレス。
    @return     指定したステーションアドレスに対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなければ、 NULL ポインタが返ります。
    @see        GetStationByAddress
     */
    const Station* GetStation(const common::StationAddress& addr) const;


    /*!
    @brief      インターネットアドレスに対応する Station を得ます（非 const 版）。

    @param[in]  addr インターネットアドレス。
    @return     指定したインターネットアドレスに対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなければ、 NULL ポインタが返ります。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @see        GetStation
     */
    Station* GetStationByAddress(const common::InetAddress& addr);


    /*!
    @brief      インターネットアドレスに対応する Station を得ます（ const 版）。

    @param[in]  addr インターネットアドレス。
    @return     指定したインターネットアドレスに対応する Station が存在すれば、その Station を指すポインタが返ります。存在しなければ、 NULL ポインタが返ります。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @see        GetStation
     */
    const Station* GetStationByAddress(const common::InetAddress& addr) const;


    /*!
    @brief      StationId に対応する Station の StationAddress を得ます。

    @param[out] pAddr 書き込み先の StationAddress インスタンスを指すポインタ。
    @param[in]  id    StationId 。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。StationId が無効だった場合もこの Result が返されます。[:progErr]
    @retval     ResultNotFound        引数で指定した Station が見つからなかったことを示します。[:handling]
     */
    Result GetStationAddress(common::StationAddress* pAddr, StationId id) const;


    /*!
    @cond PRIVATE
    @brief      StationIndex に対応する Station の StationAddress を得ます。

    @param[out] pAddr 書き込み先の StationAddress インスタンスを指すポインタ。
    @param[in]  idx    StationIndex 。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。StationIndex が無効だった場合もこの Result が返されます。[:progErr]
    @retval     ResultNotFound        引数で指定した Station が見つからなかったことを示します。[:handling]
     */
    Result GetStationAddress(common::StationAddress* pAddr, StationIndex idx) const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      現時点で有効な Station インスタンスの個数を返します。

    @return     現時点で有効な Station インスタンスの個数。
     */
    uint16_t GetValidStationNum(void) const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      セッションに参加している Station の StationIndex に対応するビットを立てたビットマップを返します。

    @param[in]  bIncludeMyself  自分自身に対応する StationIndex のビットを返り値に含めたい場合は true を指定します。
      
    @return     ビットマップ値。最下位ビット (LSB) が StationIndex_1 に対応します。
      */
    uint32_t GetParticipatingStationBitmap(bool bIncludeMyself) const;
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      session から呼ばれます。Host の StationIndex をセットします。
     */
    void SetHostStationIndex(StationIndex idx)
    {
        m_HostStationIndex = idx;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Host の StationIndex を取得します。
     */
    StationIndex GetHostStationIndex(void) const
    {
        return m_HostStationIndex;
    }
    //! @endcond


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


private:
    TStationList m_StationList;  // Station の実体を管理する。
    uint8_t* m_pWorkBuf;         // m_StationList 用のバッファ

    TStationPtrArray m_ValidStationPtr;    // 現時点で有効な Station のポインタを管理する。
    TStationPtrArray m_InvalidStationPtr;  // 現時点で無効な Station のポインタを管理する。

    bool isInStationList(const Station* pStation) const;
    bool isInPtrArray(const TStationPtrArray& ptrArray, const Station* pStation) const;

    Station* m_pLocalStation;

    StationIndex m_HostStationIndex;

    static StationManager* s_pInstance;  // このクラスの唯一のインスタンスの実体

    StationManager(const StationManager&);
    StationManager& operator=(const StationManager&);
};
}
}
}  // end of namespace nn::pia::transport
