﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

// 先行宣言(common)
namespace nn
{
namespace pia
{
namespace common
{
class IPacketInput;
class IPacketOutput;
}
}
}


namespace nn
{
namespace pia
{
namespace transport
{

// 先行宣言。
class TransportThreadStreamOld;
class NetworkFactory;
class SendThreadStream;
class ReceiveThreadStream;

/*!
  @brief      送受信スレッドを統括するクラスです。

  @details このクラスは送受信スレッドの生成／破棄などを行います。
              また、送受信スレッドの優先度やスリープ間隔を設定する API や、
              レイテンシエミュレーション機能のための API を用意しています。
              このクラスのインスタンス自体は Transport クラスの初期化時に作成されます。

 */
class ThreadStreamManager : public common::RootObject
{
public:
#if NN_PIA_CTR
    static const int32_t SendThreadDefaultPriority = 10;     //!< 送信スレッドのデフォルトの優先度です。
    static const int32_t ReceiveThreadDefaultPriority = 10;  //!< 受信スレッドのデフォルトの優先度です。
#elif NN_PIA_WIN
    static const int32_t SendThreadDefaultPriority = THREAD_PRIORITY_ABOVE_NORMAL;  //!< 送信スレッドのデフォルトの優先度です。
    static const int32_t ReceiveThreadDefaultPriority = THREAD_PRIORITY_ABOVE_NORMAL;  //!< 受信スレッドのデフォルトの優先度です。
#elif NN_PIA_NINTENDOSDK
    static const int32_t SendThreadDefaultPriority = 10;  //!< 送信スレッドのデフォルトの優先度です。
    static const int32_t ReceiveThreadDefaultPriority = 10;  //!< 受信スレッドのデフォルトの優先度です。
#elif NN_PIA_CAFE
    static const int32_t SendThreadDefaultPriority = 10;  //!< 送信スレッドのデフォルトの優先度です。
    static const int32_t ReceiveThreadDefaultPriority = 10;  //!< 受信スレッドのデフォルトの優先度です。
#elif NN_PIA_A
    //n1849
    //UIスレッド優先度を超えてはいけない
    //0 THREAD_PRIORITY_DEFAULT アプリのスレッドとして標準的な優先度です
    //-1 THREAD_PRIORITY_MORE_FAVORABLE 標準的な優先度より、ほんの少し優先を上げた設定値です
    //-2 THREAD_PRIORITY_FOREGROUND UIスレッドの設定値です。システムがUIスレッドを作る際に利用します。通常、アプリ開発者は利用しません
    static const int32_t SendThreadDefaultPriority = -1;  //!< 送信スレッドのデフォルトの優先度です。
    static const int32_t ReceiveThreadDefaultPriority = -1;  //!< 受信スレッドのデフォルトの優先度です。
#elif NN_PIA_B
    //todo:n3389:とりあえずな値
    static const int32_t SendThreadDefaultPriority = -1;  //!< 送信スレッドのデフォルトの優先度です。
    static const int32_t ReceiveThreadDefaultPriority = -1;  //!< 受信スレッドのデフォルトの優先度です。
#else
#error "invalid platform"
#endif

    static const int32_t SendThreadDefaultSleepSpan = 5;     //!< 送信スレッドのデフォルトのスリープ間隔です。単位はミリ秒です。
    static const int32_t ReceiveThreadDefaultSleepSpan = 5;  //!< 受信スレッドのデフォルトのスリープ間隔です。単位はミリ秒です。

    /*!
    @cond PRIVATE
    @brief    ThreadStreamManager クラスのインスタンスを作成します（シングルトンパターン）。

    @details この関数は Pia ライブラリ内部から呼び出されますので、アプリケーションプログラムが
              この関数を直接呼び出す必要はありません。

    @param[in]  pFactory      NetworkFactory から派生した具象クラスを指すポインタ。
    @param[in]  sendThreadMaxPacketNum    送信スレッドの内部バッファ数
    @param[in]  receiveThreadMaxPacketNum 受信スレッドの内部バッファ数
    @param[in]  sendThreadLatencyEmulationPacketNum    送信レイテンシエミュレーション用バッファ数（ゼロだとエミュレーション機能は無効になります）。
    @param[in]  receiveThreadLatencyEmulationPacketNum 受信レイテンシエミュレーション用バッファ数（ゼロだとエミュレーション機能は無効になります）。
    @param[in]  bPacketLossEmulation パケロスエミュレーションを有効にするか否か

    @return   インスタンスの作成に成功すれば、成功の Result が返されます。
    @retval   ResultInvalidArgument    引数が誤っています（ NULL ポインタであるなど）。[:progErr]
    @retval   ResultNotInitialized     transport モジュールが未初期化です。[:progErr]
    @retval   ResultInvalidState       インスタンスを作成するタイミングが誤っています。[:progErr]
    @retval   ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
     */
    static Result CreateInstance(NetworkFactory* pFactory,
                                 uint32_t sendThreadMaxPacketNum,
                                 uint32_t receiveThreadMaxPacketNum,
                                 uint32_t sendThreadLatencyEmulationPacketNum,
                                 uint32_t receiveThreadLatencyEmulationPacketNum,
                                 bool bPacketLossEmulation);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    ThreadStreamManager クラスのインスタンスを破棄します（シングルトンパターン）。

    @details この関数は Pia ライブラリ内部から呼び出されますので、アプリケーションプログラムが
              この関数を直接呼び出す必要はありません。
     */
    static void DestroyInstance(void);
    //! @endcond


    /*!
    @brief    ThreadStreamManager クラスのインスタンスを得ます（シングルトンパターン）。

    @details 事前にインスタンスが作成されていなかった場合には、
              NULL ポインタが返されます。
    @return   ThreadStreamManager インスタンスへのポインタ。
     */
    static ThreadStreamManager* GetInstance(void)
    {
        return s_pInstance;
    }


    /*!
    @cond PRIVATE
    @brief      送受信スレッドのスタートアップ処理を実行します。

    @details この関数は Pia ライブラリ内部から呼び出されますので、アプリケーションプログラムが
                この関数を直接呼び出す必要はありません。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @see        Cleanup
     */
    Result Startup(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      送受信スレッドのクリーンアップ処理を実行します。

    @details この関数は Pia ライブラリ内部から呼び出されますので、アプリケーションプログラムが
                この関数を直接呼び出す必要はありません。
    @see        Startup
     */
    void Cleanup(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。
     */
    virtual ~ThreadStreamManager(void);
    //! @endcond


    /*!
    @brief      送信スレッドの優先度を設定します。

    @details 通常、送信スレッドの優先度は、アプリケーションが common::Scheduler::Dispatch()
                を呼び出すスレッドの優先度よりも高くする必要があります。

    @param[in]  prio 送信スレッドに指定する優先度。[:tuning]
    @see        GetSendThreadPriority, common::Scheduler::Dispatch
     */
    void SetSendThreadPriority(int32_t prio);


    /*!
    @brief      送信スレッドの優先度を取得します。

    @details この関数はスレッドセーフです。
    <br>
    @return     送信スレッドの優先度。
    @see        SetSendThreadPriority
     */
    int32_t GetSendThreadPriority(void) const;


    /*!
    @brief      受信スレッドの優先度を設定します。

    @details 通常、受信スレッドの優先度は、アプリケーションが common::Scheduler::Dispatch()
                を呼び出すスレッドの優先度よりも高くする必要があります。

    @param[in]  prio 受信スレッドに指定する優先度。[:tuning]
    @see        GetReceiveThreadPriority, common::Scheduler::Dispatch
     */
    void SetReceiveThreadPriority(int32_t prio);


    /*!
    @brief      受信スレッドの優先度を取得します。

    @details この関数はスレッドセーフです。
    <br>
    @return     受信スレッドの優先度。
    @see        SetReceiveThreadPriority
     */
    int32_t GetReceiveThreadPriority(void) const;


    /*!
    @brief      送信スレッドのスリープ期間を設定します。

    @details 送信スレッドは定期的にスリープします。このスリープ期間を設定する関数です。
    <br>
    @param[in]  span 送信スレッドがスリープする期間。単位はミリ秒。[:tuning]
    @see        GetSendThreadSleepTimeSpan
     */
    void SetSendThreadSleepTimeSpan(uint32_t span);


    /*!
    @brief      送信スレッドのスリープ期間を取得します。

    @details この関数はスレッドセーフです。
    <br>
    @return     送信スレッドのスリープ期間。単位はミリ秒。
    @see        SetSendThreadSleepTimeSpan
     */
    uint32_t GetSendThreadSleepTimeSpan(void) const;


    /*!
    @brief      受信スレッドのスリープ期間を設定します。

    @details 受信スレッドは定期的にスリープします。このスリープ期間を設定する関数です。
    <br>
    @param[in]  span 受信スレッドがスリープする期間。単位はミリ秒。[:tuning]
    @see        GetReceiveThreadSleepTimeSpan
     */
    void SetReceiveThreadSleepTimeSpan(uint32_t span);


    /*!
    @brief      受信スレッドのスリープ期間を取得します。

    @details この関数はスレッドセーフです。
    <br>
    @return     受信スレッドのスリープ期間。単位はミリ秒。
    @see        SetReceiveThreadSleepTimeSpan
     */
    uint32_t GetReceiveThreadSleepTimeSpan(void) const;


    /*!
    @brief      送信レイテンシエミュレーション時間を設定します。

    @details 送信レイテンシエミュレーション機能は、作成されたパケットを
                すぐに送信するのではなく、一時的にバッファに保存し、規定の時間が
                経過してから保存していたパケットを送信することで実現しています。
                そのため、最大遅延時間を大きくする場合は、それに見合ったサイズの
                バッファを用意しないと、バッファがあふれてしまい、あたかも
                パケロスが頻発しているかのような症状を示すことがあります。
                バッファのサイズ設定については、 Transport::DebugSetting 構造体を参照してください。

    @param[in]  minLatency 最小遅延時間。単位はミリ秒です。
    @param[in]  maxLatency 最大遅延時間。単位はミリ秒です。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    レイテンシエミュレーション機能が無効の可能性があります。[:progErr]
    @see        SetReceiveThreadLatencyEmulation, Transport::DebugSetting
     */
    Result SetSendThreadLatencyEmulation(int32_t minLatency, int32_t maxLatency);


    /*!
    @brief      受信レイテンシエミュレーション時間を設定します。

    @details 受信レイテンシエミュレーション機能は、受信したパケットを
                すぐに上位層に渡すのではなく、一時的にバッファに保存し、規定の時間が
                経過してから保存していたパケットを上位層に渡すことで実現しています。
                そのため、最大遅延時間を大きくする場合は、それに見合ったサイズの
                バッファを用意しないと、バッファがあふれてしまい、あたかも
                パケロスが頻発しているかのような症状を示すことがあります。
                バッファのサイズ設定については、 Transport::DebugSetting 構造体を参照してください。

    @param[in]  minLatency 最小遅延時間。単位はミリ秒です。
    @param[in]  maxLatency 最大遅延時間。単位はミリ秒です。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    レイテンシエミュレーション機能が無効の可能性があります。[:progErr]
    @see        SetSendThreadLatencyEmulation, Transport::DebugSetting
     */
    Result SetReceiveThreadLatencyEmulation(int32_t minLatency, int32_t maxLatency);


    /*!
    @brief      送信スレッドがパケットの送信を試みた回数を得ます。

    @details 送信スレッドは、送信を試みたパケットの数をカウントしています。
                そのカウンタの値が返されます。

    @return     これまでに送信を試みたパケットの総数。
    @see        ClearSendThreadSendPacketNum
     */
    uint32_t GetSendThreadSendPacketNum(void) const;


    /*!
    @brief      送信パケット数カウンタをクリアします。

    @details GetSendThreadSendPacketNum() で取得できる、
                送信を試みたパケット数のカウンタをゼロにクリアします。

    @see        GetSendThreadSendPacketNum
     */
    void ClearSendThreadSendPacketNum(void);


    /*!
    @brief      受信スレッドがパケットの受信に成功した回数を得ます。

    @details 受信スレッドは、受信に成功したパケットの数をカウントしています。
                そのカウンタの値が返されます。

    @return     これまでに受信に成功したパケットの総数。
    @see        ClearReceiveThreadReceivePacketNum
     */
    uint32_t GetReceiveThreadReceivePacketNum(void) const;


    /*!
    @brief      受信パケット数カウンタをクリアします。

    @details GetReceiveThreadReceivePacketNum() で取得できる、
                受信に成功したパケット数のカウンタをゼロにクリアします。

    @see        GetReceiveThreadReceivePacketNum
     */
    void ClearReceiveThreadReceivePacketNum(void);


    /*!
    @brief      送信スレッドが送信を試みたデータの総量を得ます。

    @details 送信スレッドは、送信を試みたデータ量をカウントアップしています。
                そのカウンタの値が返されます。

    @return     これまでに送信を試みたデータの総量。単位はバイトです。
    @see        ClearSendThreadSendDataSize
     */
    uint64_t GetSendThreadSendDataSize(void) const;


    /*!
    @brief      送信データ量カウンタをクリアします。

    @details GetSendThreadSendDataSize() で取得できる、
                送信を試みたデータ量のカウンタをゼロにクリアします。

    @see        GetSendThreadSendDataSize
     */
    void ClearSendThreadSendDataSize(void);


    /*!
    @brief      受信スレッドが受信に成功したデータの総量を得ます。

    @details 受信スレッドは、受信に成功したデータ量をカウントアップしています。
                そのカウンタの値が返されます。

    @return     これまでに受信に成功したデータの総量。単位はバイトです。
    @see        ClearReceiveThreadReceiveDataSize
     */
    uint64_t GetReceiveThreadReceiveDataSize(void) const;


    /*!
    @brief      受信データ量カウンタをクリアします。

    @details GetReceiveThreadReceiveDataSize() で取得できる、
                受信に成功したデータ量のカウンタをゼロにクリアします。

    @see        GetReceiveThreadReceiveDataSize
     */
    void ClearReceiveThreadReceiveDataSize(void);


    /*!
    @brief      送信スレッドがエミュレートするパケロスの確率を設定します。

    @details このパケロスエミュレーション機能を用いると、パケットの送出には
                成功したものの、そのパケットが何らかの理由で消失したという状況を
                エミュレートできます。
                引数に 0% 以外の値をを設定すると、送信スレッドはその確率に応じて
                送信パケットをドロップし、パケロスをエミュレートします。
                デフォルトでは、送信スレッドのパケロス確率は 0% に設定されています。

    @param[in]  lossPercentage 0 ～ 100 の値を設定します。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    パケロスエミュレーション機能が有効に設定されていません。[:progErr]
    @see        GetSendThreadPacketLossRatio, Transport::SetDebugSetting
     */
    Result SetSendThreadPacketLossRatio(int32_t lossPercentage);


    /*!
    @brief      送信スレッドがエミュレートするパケロスの確率を取得します。

    @return     送信スレッドに設定されていたパケロス確率が返されます。値の範囲は 0 ～ 100 です。
    @see        SetSendThreadPacketLossRatio
     */
    int32_t GetSendThreadPacketLossRatio(void) const;


    /*!
    @brief      受信スレッドがエミュレートするパケロスの確率を設定します。

    @details このパケロスエミュレーション機能を用いると、こちらに向けて送出された
                パケットが何らかの理由で消失したという状況をエミュレートできます。
                引数に 0% 以外の値をを設定すると、受信スレッドはその確率に応じて
                受信パケットをドロップし、パケロスをエミュレートします。
                デフォルトでは、受信スレッドのパケロス確率は 0% に設定されています。

    @param[in]  lossPercentage 0 ～ 100 の値を設定します。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    パケロスエミュレーション機能が有効に設定されていません。[:progErr]
    @see        GetReceiveThreadPacketLossRatio, Transport::SetDebugSetting
     */
    Result SetReceiveThreadPacketLossRatio(int32_t lossPercentage);


    /*!
    @brief      受信スレッドがエミュレートするパケロスの確率を取得します。

    @return     受信スレッドに設定されていたパケロス確率が返されます。値の範囲は 0 ～ 100 です。
    @see        SetReceiveThreadPacketLossRatio
     */
    int32_t GetReceiveThreadPacketLossRatio(void) const;


    /*!
    @cond PRIVATE
    @brief      ReceiveThreadStreamOld インスタンスへのポインタを得ます（非 const 版）。
     */
    ReceiveThreadStream* GetReceiveStream(void)
    {
        return m_pReceiveStream;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      ReceiveThreadStreamOld インスタンスへのポインタを得ます（ const 版）。
     */
    const ReceiveThreadStream* GetReceiveStream(void) const
    {
        return m_pReceiveStream;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      SendThreadStreamOld インスタンスへのポインタを得ます（非 const 版）。
     */
    SendThreadStream* GetSendStream(void)
    {
        return m_pSendStream;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      SendThreadStreamOld インスタンスへのポインタを得ます（ const 版）。
     */
    const SendThreadStream* GetSendStream(void) const
    {
        return m_pSendStream;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      NexFacade用に用意。
     */
    common::IPacketInput* GetInputStream(void)
    {
        return m_pInputStream;
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      NexFacade用に用意。
     */
    common::IPacketOutput* GetOutputStream(void)
    {
        return m_pOutputStream;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      モニタリングデータをセットします。
     */
    void SetMonitoringData(void);
    //! @endcond


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


#if NN_PIA_ENABLE_STATISTIC_THREAD_STREAM_BLOCK_TIME
    void ResetThreadStreamStatistic();
    void PrintThreadStreamStatistic(uint32_t printMaxDataNum);
#endif

private:
    /*!
    @brief ファクトリのポインタを与えてもらうことを大前提としているので、デフォルトコンストラクタは使いません。
     */
    ThreadStreamManager(void);

    /*!
    @brief 引数ありコンストラクタです。
     */
    ThreadStreamManager(NetworkFactory* pFactory,
                        uint32_t sendThreadMaxPacketNum,
                        uint32_t receiveThreadMaxPacketNum,
                        uint32_t sendThreadLatencyEmulationPacketNum,
                        uint32_t receiveThreadLatencyEmulationPacketNum,
                        bool bPacketLossEmulation);

    /*!
    @brief シングルトンパターンのため、コピーコンストラクタは封印します。
     */
    ThreadStreamManager(const ThreadStreamManager&);

    /*!
    @brief シングルトンパターンのため、代入演算子は封印します。
     */
    ThreadStreamManager& operator=(const ThreadStreamManager&);


    // ファクトリによって Create されるポインタを受ける変数。
    common::IPacketInput* m_pInputStream;
    common::IPacketOutput* m_pOutputStream;

    ReceiveThreadStream* m_pReceiveStream;
    SendThreadStream* m_pSendStream;

    static ThreadStreamManager* s_pInstance;
};
}
}
}  // end of namespace nn::pia::transport
