﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/transport/transport_Singleton.h>
#include <nn/pia/transport/transport_ProtocolManager.h>

#include <nn/pia/common/common_Time.h>
#include <nn/pia/common/common_Job.h>


// 先行宣言(common)
namespace nn
{
namespace pia
{
namespace common
{
class IPacketInput;
class IPacketOutput;
class StationAddress;
struct CryptoSetting;
}
}
}


namespace nn
{
namespace pia
{
namespace transport
{


// 先行宣言(transport)
class NetworkFactory;
class StationConnectionInfo;
class KeepAliveSender;
class KeepAliveReceiver;
class ThreadStreamManager;
class IRelayAddressTable;
class RelayRouteManager;
class PacketHandler;
class Station;
class StationIdTable;


/*!
  @brief  transport モジュールの中核的な存在のクラスです。

 */
class Transport : public common::RootObject
{
public:
    /*!
    @brief  CreateInstance() に各種パラメータを渡すための構造体です。

    @details この構造体に設定する内部バッファのパラメータは、 PiaTransport のメモリ使用量に
            直結します。大きな値を設定すれば、送受信処理において内部バッファ枯渇による
            ResultBufferIsFull エラーの発生頻度を低減させることができ、スムーズな通信処理が
            期待できますが、もちろんメモリ使用量は増大します。
            PiaTransport にとって必要十分なメモリ量は、アプリケーションの通信仕様に依存します。
            これらのパラメータをチューニングするのを支援する手段として、
            @ref common::WatermarkManager を用意しています。
            このウォーターマーク機能を用いると、実際にプログラムを動作させた際に、
            内部バッファ使用量のピーク値などを取得できるので、アプリケーションプログラマが
            適切な Pia の内部バッファサイズを決定するのに役立ちます。
    @see    CreateInstance, common::WatermarkManager, ThreadStreamManager
    @if CTR_DOC
      @see nn::pia::local::UdsNetworkFactory
    @endif
    @if NIN_DOC
      @see nn::pia::local::LdnNetworkFactory
    @endif
    @see nn::pia::inet::NexNetworkFactory
    @see nn::pia::lan::LanNetworkFactory
     */
    struct Setting
    {

        /*!
        @brief  デフォルトコンストラクタです。
         */
        Setting()
            : pFactory(NULL),
              maxStationNum(0),
              maxSendThreadBufferNum(0),
              maxReceiveThreadBufferNum(0),
              analysisInterval(0),
              isAnalysisAutoPrintDetail(true),
              isAnalysisAutoPrintTotalOnly(false)
        {
        }

        /*!
        @brief  引数つきコンストラクタです。引数で指定した値が各メンバ変数にセットされます。

        @param[in] pFactory                                 NetworkFactory から派生させた具象クラスのインスタンスを指すポインタ（ファクトリパターン）。
        @param[in] maxStationNum                            一つのネットワークに参加可能な最大 Station 数を指定します。
        @param[in] maxSendThreadBufferNum                   Pia 内部の送信スレッドにおける送信パケット用バッファの個数です（1個につき約1.5KB）。 2 以上の値を設定する必要があります。
        @param[in] maxReceiveThreadBufferNum                Pia 内部の受信スレッドにおける受信パケット用バッファの個数です（1個につき約1.5KB）。 2 以上の値を設定する必要があります。
        @param[in] analysisInterval                         TransportAnalyzer による分析データを自動的にコンソールに出力する間隔です。単位は秒です。0 を指定した場合、出力は自動では行われなくなり、データの取得はアプリケーションに委ねられます。
        @param[in] isAnalysisAutoPrintDetail                TransportAnalyzer による分析データを自動的にコンソールに出力する際に、Pia 内部で使用されているプロトコルも出力する場合は true を設定します。
        @param[in] isAnalysisAutoPrintTotalOnly             TransportAnalyzer による分析データを自動的にコンソールに出力する際に、ユニキャスト送信とブロードキャスト送信を分けた分析データも出力する場合は false を設定します。インターネット通信時にはこの設定は無視され、これらを分けた分析データは出力されません。

        @if CTR_DOC
          @see nn::pia::local::UdsNetworkFactory
        @endif
        @if NIN_DOC
          @see nn::pia::local::LdnNetworkFactory
        @endif
        @see nn::pia::inet::NexNetworkFactory
        @see nn::pia::lan::LanNetworkFactory
         */
        Setting(
            NetworkFactory* pFactory,
            uint16_t maxStationNum = 0,
            uint32_t maxSendThreadBufferNum = 0,
            uint32_t maxReceiveThreadBufferNum = 0,
            int32_t analysisInterval = 0,
            bool isAnalysisAutoPrintDetail = true,
            bool isAnalysisAutoPrintTotalOnly = false);

        NetworkFactory* pFactory;            //!< NetworkFactory から派生させた具象クラスのインスタンスを指すポインタ（ファクトリパターン）。
        uint16_t maxStationNum;              //!< 一つのネットワークに参加可能な最大 Station 数を指定します。
        uint32_t maxSendThreadBufferNum;     //!< Pia 内部の送信スレッドにおける送信パケット用バッファの個数です（1個につき約1.5KB）。 2 以上の値を設定する必要があります。
        uint32_t maxReceiveThreadBufferNum;  //!< Pia 内部の受信スレッドにおける受信パケット用バッファの個数です（1個につき約1.5KB）。 2 以上の値を設定する必要があります。
        int32_t analysisInterval;            //!< TransportAnalyzer 機能による測定結果を自動的にコンソールに出力する間隔です。単位は秒です。0 を指定した場合、出力は自動では行われなくなり、データの取得はアプリケーションに委ねられます。
        bool isAnalysisAutoPrintDetail;      //!< TransportAnalyzer 機能による測定結果を自動的にコンソールに出力する際に、Pia 内部で使用されているプロトコルも出力する場合は true を設定します。
        bool isAnalysisAutoPrintTotalOnly;   //!< TransportAnalyzer 機能による測定結果を自動的にコンソールに出力する際に、ユニキャスト送信とブロードキャスト送信の解析結果も出力する場合は false を設定します。
    };


    /*!
    @cond PRIVATE
    @brief コンストラクタです。アプリケーションはこのコンストラクタを直接呼び出すのではなく、 CreateInstance() を使う必要があります。
     */
    Transport(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。
     */
    virtual ~Transport(void);
    //! @endcond


    /*!
    @brief     Transport クラスのインスタンスを作成します（シングルトンパターン）。

    @param[in] setting 各種パラメータが書き込まれた Setting 構造体。

    @return    インスタンスの作成に成功すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval    ResultNotInitialized     transport モジュールが未初期化です。[:progErr]
    @retval    ResultInvalidState       CreateInstance() を呼ぶタイミングが誤っています。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。[:progErr]
    @retval    ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    @retval    ResultInvalidArgument    引数が誤っています。[:progErr]
    @retval    ResultAllocationFailed   内部でメモリ確保に失敗しました。 common::Initialize() で Pia に供給するメモリを増やすか、よりメモリ消費量の少ないパラメータ設定を試す必要があります。[:progErr]
    @see       DestroyInstance, GetInstance, Setting
     */
    static Result CreateInstance(const Transport::Setting& setting);


    /*!
    @brief    Transport クラスのインスタンスを破棄します（シングルトンパターン）。
              インスタンスが存在していなかった場合は、何もしません。

    @see      CreateInstance, GetInstance
     */
    static void DestroyInstance(void);


    /*!
    @brief    Transport クラスのインスタンスを得ます（シングルトンパターン）。
              事前に CreateInstance() によってインスタンスを作成していない場合は、
              NULL ポインタが返されます。
              この関数はスレッドセーフです。

    @return   Transport インスタンスへのポインタ。
    @see      CreateInstance, DestroyInstance
     */
    static Transport* GetInstance(void)
    {
        return s_pInstance;
    }


    /*!
    @cond PRIVATE
    @brief      スタートアップ処理を実行します。

    @details transport::EndSetup() 呼び出しで transport モジュールの
                初期化処理を終えた後で呼び出す必要があります。
    @param[in]  mtuSize             パケットの MTU を指定します。単位はバイトです。
    @param[in]  cpRelayNodeAddress  リレーノードのアドレスへのポインタです。
                                    一時的なインスタンスへのポインタで構いません。
                                    リレーノードを使用しない場合はNULLを指定する必要があります。
    @param[in]  cpCryptoSetting     暗号化設定へのポインタです。
                                    NULL を指定した場合は暗号化されません。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidState 既に Startup() が呼ばれているか、
                                   transport::BeginSetup() ～ transport::EndSetup() 区間の中で
                                   呼び出されている可能性があります。[:progErr]
    @see        Cleanup, transport::EndSetup
     */
    Result Startup(uint32_t mtuSize, const common::StationAddress* cpRelayNodeAddress = NULL, const common::CryptoSetting* cpCryptoSetting = NULL);
//! @endcond


    Result UpdateLocalStationKey();

    /*!
    @cond PRIVATE
    @brief      PacketHandler の暗号化設定を更新します。
    @param[in]  cpCryptoSetting         暗号化設定へのポインタです。NULL を指定した場合は暗号化されません。
    @retval     ResultInvalidArgument   引数が誤っています。[:progErr]
     */
    Result UpdateSetting(const common::CryptoSetting* cpCryptoSetting);
//! @endcond


    /*!
    @cond PRIVATE
    @brief      クリーンアップ処理を実行します。

    @details transport モジュールの終了処理を実行する前に呼び出す必要があります。
    <br>
    @see        Startup
     */
    void Cleanup(void);
    //! @endcond

    void ReserveRestartup()
    {
        m_IsRestartupReserved = true;
    }

    bool IsRestartupReserved() const
    {
        return m_IsRestartupReserved;
    }

    Result Restartup(const common::StationAddress* cpRelayNodeAddress = NULL, const common::CryptoSetting* cpCryptoSetting = NULL);

    /*!
    @brief T型のプロトコルのインスタンスを生成します。

    @details @tparam    T    プロトコルの型を指定します。
    @param[in] port プロトコルのポートの値です。

    @return 生成したプロトコルのハンドルです。同じプロトコル ID のものが既にある場合は 0 を返します。
    @see DestroyProtocol, GetProtocol
     */
    template <typename T>
    uint32_t CreateProtocol(uint16_t port = 0)
    {
        return m_ProtocolManager.CreateProtocol<T>(port);
    }


    /*!
    @brief プロトコルのインスタンスを破棄します。

    @param[in] protocolHandle 削除したいプロトコルのハンドルです。
    @see CreateProtocol, GetProtocol
     */
    void DestroyProtocol(uint32_t protocolHandle)
    {
        m_ProtocolManager.DestroyProtocol(protocolHandle);
    }


    /*!
    @brief T型のプロトコルのインスタンスを取得します。

    @details @tparam    T              プロトコルの型を指定します。
    @param[in] protocolHandle 取得したいプロトコルのハンドルです。

    @return プロトコルのインスタンスのアドレスです。指定されたハンドルのプロトコルが存在しなかったり、T型でなかったりした場合は NULL を返します。
    @see CreateProtocol, DestroyProtocol
     */
    template <typename T>
    T* GetProtocol(uint32_t protocolHandle)
    {
        return m_ProtocolManager.GetProtocol<T>(protocolHandle);
    }


    /*!
    @cond PRIVATE
    @brief      transport モジュールにおける接続状態を返します。

    @details この関数はスレッドセーフです。
    <br>
    @return     ネットワーク接続に問題がなければ、 IsSuccess() が true となる Result が返されます。
    @retval     ResultInvalidState     アプリケーションが適切な初期化関数／セットアップ関数を呼び出していない
                                       疑いがあります。例えば、インターネット接続を試みようとしていながら、
                                       ソケットライブラリが未初期化だった場合、この Result が返されます。[:progErr]
    @cond CTR_DOC
    @retval     ResultNetworkConnectionIsLost ネットワーク接続が断絶したことを示します。無線スイッチ Off や、
                                       アクセスポイントの障害などの原因が考えられます。<br>
                                       ローカル接続環境でこのエラーが返された場合は、PiaLocal の上位
                                       モジュールと PiaLocal の初期化からやり直す必要があります。<br>
                                       インターネット接続環境でこのエラーが返された場合は、クリーンアップ処理と
                                       NEX サーバーのログアウト処理を行った後、ネットワークのシャットダウン処理
                                       （AC のクローズ）を実行する必要があります（ローカル接続環境と同様に初期化から
                                       やり直しても問題ありません）。[:handling]
    @endcond
    @cond CAFE_DOC
    @retval     ResultNetworkConnectionIsLost ネットワーク接続が断絶したことを示します。
                                       アクセスポイントの障害などの原因が考えられます。[:handling-shutdownNetwork]
    @endcond
     */
    Result CheckConnectionError(void) const
    {
        return m_ConnectionError;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief    dispatch() を呼び出した直後の時刻を得ます。

    @return   dispatch() 呼び出し直後の時刻が返ります。
    @see      dispatch
     */
    const common::Time& GetDispatchedTime(void) const
    {
        return m_DispatchedTime;
    }
    //! @endcond


    /*!
    @brief      CreateInstance() 時に設定された、接続可能な最大 Station 数を返します。

    @details この関数はスレッドセーフです。
    <br>
    @return     接続可能な最大 Station 数。
    @see        CreateInstance
     */
    uint16_t GetMaxStationNum(void) const
    {
        return m_MaxStationNum;
    }


    /*!
    @cond PRIVATE
    @brief プロトコルマネージャを取得します。

    @return プロトコルマネージャのアドレスです。
     */
    ProtocolManager* GetProtocolManager()
    {
        return &m_ProtocolManager;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      キープアライブパケット送信間隔を指定します。

    @param[in]  msec キープアライブパケット送信間隔。単位はミリ秒。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval     ResultInvalidArgument 引数に誤りがあります（負数であるなど）[:progErr]
    @see        GetKeepAliveInterval
     */
    Result SetKeepAliveInterval(int32_t msec);
    //! @endcond


    /*!
    @brief      キープアライブパケット送信間隔を取得します。

    @details この関数はスレッドセーフです。
    <br>
    @return     キープアライブパケット送信間隔。単位はミリ秒。
    @see        SetKeepAliveInterval
     */
    int32_t GetKeepAliveInterval(void) const;


    /*!
    @brief      キープアライブ処理を有効にします。

    @see        DisableKeepAlive, IsKeepAliveEnabled
     */
    void EnableKeepAlive(void);


    /*!
    @brief      キープアライブ処理を無効にします。

    @see        EnableKeepAlive, IsKeepAliveEnabled
     */
    void DisableKeepAlive(void);


    /*!
    @brief      キープアライブ処理が有効か無効であるかを返します。

    @details この関数はスレッドセーフです。
    <br>
    @return     有効であれば true 、無効であれば false が返ります。
    @see        EnableKeepAlive, DisableKeepAlive
     */
    bool IsKeepAliveEnabled(void) const;


    /*!
    @brief      自分自身を示すアドレス（@ref StationConnectionInfo ）を取得します。

    @details ネットワークに接続できていない状態では、取得できるアドレスは不定となります。
    <br>
    @param[out] pInfoOut 取得した自分自身を示すアドレスの格納先を示すポインタ
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @see        GetHostStationConnectionInfo
     */
    Result GetLocalStationConnectionInfo(StationConnectionInfo* pInfoOut);


    /*!
    @brief      セッションホストを示すアドレス（@ref StationConnectionInfo ）を取得します。

    @details ネットワークに接続できていない状態では、取得できるアドレスは不定となります。
    <br>
    @param[out] pInfoOut 取得したセッションホストを示すアドレスの格納先を示すポインタ
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @see        GetLocalStationConnectionInfo
     */
    Result GetHostStationConnectionInfo(StationConnectionInfo* pInfoOut);


    /*!
    @cond PRIVATE
    @brief      経路表のインスタンスへのポインタを得ます。
     */
    IRelayAddressTable* GetRelayAddressTable(void)
    {
        return m_pRelayAddressTable;
    }
    //! @endcond


    /// @cond PRIVATE
    /*!
    @brief      リレー依頼先管理クラスのインスタンスへのポインタを得ます。
     */
    RelayRouteManager* GetRelayRouteManagerPtr(void)
    {
        return m_pRelayRouteManager;
    }

    /*!
    @brief      リレー依頼先管理クラスのインスタンスへのポインタを設定します。
     */
    void SetRelayRouteManagerPtr(RelayRouteManager* ptr)
    {
        m_pRelayRouteManager = ptr;
    }
    //! @endcond


    /// @cond PRIVATE
    void SetMonitoringNetworkRtt(bool isBegin);
    //! @endcond

    /// @cond PRIVATE
    PacketHandler* GetPacketHandler(void)
    {
        return m_pPacketHandler;
    }
    //! @endcond


    /*!
    @brief  StationId を @ref StationIndex に変換します。

    @details 自ステーションがセッションに参加していない状態であったり、ジョイントセッション処理中に
             この API を呼び出した場合は失敗の Result が返されます。
    @details EVENT_JOIN, EVENT_LEAVE イベントが通知される、セッションの状態変化イベントコールバック内で
             この関数を呼び出すことは可能です。
    @details この関数はセッション API 使用時とメッシュ API 使用時とで挙動が異なります。
             メッシュ API 使用時は、メッシュ未参加の StationId を指定した場合でも変換に成功します。

    @param[out] pIdx 変換した @ref StationIndex の格納先を示すポインタ
    @param[in]  id   変換する StationId
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    変換処理を実行できる状態ではありません。session のセットアップが完了していない可能性があります。[:progErr]
    @retval     ResultNotFound        指定した @ref nn::pia::StationId "StationId" のステーションはセッションに参加していません。[:handling]
    @retval     ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    @see        ConvertToStationId, session::Session::SessionEventCallback
     */
    Result ConvertToStationIndex(StationIndex* pIdx, StationId id) const;


    /*!
    @cond PRIVATE
    @brief      StationId を @ref StationIndex に変換しますが、引数チェックや状態チェックなどを行いません。

    @details    この関数は各種チェックを省いているため、不用意に呼び出すと、
                メモリアクセス違反などでプログラムがクラッシュする危険があります。
    @param[in]  id   変換する StationId
    @return     変換に失敗した場合は StationIndex_Invalid が返ります。
      */
    StationIndex ConvertToStationIndexUnsafe(StationId id) const;
    //! @endcond


    /*!
    @brief  @ref StationIndex を StationId に変換します。

    @details 自ステーションがセッションに参加していない状態であったり、ジョイントセッション処理中に
             この API を呼び出した場合は失敗の Result が返されます。
    @details EVENT_JOIN, EVENT_LEAVE イベントが通知される、セッションの状態変化イベントコールバック内で
             この関数を呼び出すことは可能です。
    @details この関数はセッション API 使用時とメッシュ API 使用時とで挙動が異なります。
             メッシュ API 使用時は、メッシュ未参加の StationIndex を指定した場合でも変換に成功します。

    @param[out] pId 変換した StationId の格納先を示すポインタ
    @param[in]  idx 変換する @ref StationIndex
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    変換処理を実行できる状態ではありません。session のセットアップが完了していない可能性があります。[:progErr]
    @retval     ResultNotFound        指定した @ref nn::pia::StationIndex "StationIndex" のステーションはセッションに参加していません。[:handling]
    @retval     ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    @see        ConvertToStationIndex, session::Session::SessionEventCallback
     */
    Result ConvertToStationId(StationId* pId, StationIndex idx) const;


    /*!
    @cond PRIVATE
    @brief      @ref StationIndex を StationId に変換しますが、引数チェックや状態チェックなどを行いません。

    @details    この関数は各種チェックを省いているため、不用意に呼び出すと、
                メモリアクセス違反などでプログラムがクラッシュする危険があります。
    @param[in]  idx   変換する StationIndex
    @return     変換に失敗した場合は StationIdInvalid が返ります。
      */
    StationId ConvertToStationIdUnsafe(StationIndex idx) const;
    //! @endcond


    /*!
    @brief  Transport の状態をチェックし、送受信処理などが実行できる状態であるかどうかを判定します。

    @return   session モジュールのセットアップが完了しており、各種の API を問題なく呼び出せる状態であれば、
              IsSuccess() が true を返す Result が返されます。
    @retval   ResultInvalidState          session モジュールのセットアップが完了していません。[:progErr]
    @retval   ResultTemporaryUnavailable  セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    @see      ConvertToStationIndex, ConvertToStationId
     */
    Result CheckTransportStatus() const
    {
        PIA_ASSERT((m_TransportStatus.IsSuccess()) || (m_TransportStatus == ResultInvalidState()) || (m_TransportStatus == ResultTemporaryUnavailable()));
        return m_TransportStatus;
    }


    /*!
    @cond PRIVATE
    @brief    Transport の状態を設定します。session から呼び出されることを想定しています。

    @param[in] state Transport の状態を Result 値で設定します。設定可能な Result 値は
                     ResultInvalidState、ResultTemporaryUnavailable、ResultSuccess() の3種です。
                     session 側の準備が整ったときは ResultSuccess、
                     セッション移行処理を開始するときは ResultTemporaryUnavailable、
                     session を破棄するときは ResultInvalidState を引数に渡して呼び出す必要があります。
    @return    呼び出しが成功すれば、IsSuccess() が true を返す Result が返されます。
    @retval    ResultInvalidArgument 引数が誤っています。state として指定可能な Result 値は上記の 3 種類のみです。[:progErr]
      */
    Result SetTransportStatus(Result state);
    //! @endcond


    // StationId 変換を StationIdTable クラスを使って行うか、ダイレクトに変換するかの制御
    void SetStationIdTableEnabled()
    {
        m_IsStationIdTableEnabled = true;
    }
    void SetStationIdTableDisabled()
    {
        m_IsStationIdTableEnabled = false;
    }
    bool IsStationIdTableEnabled() const
    {
        return m_IsStationIdTableEnabled;
    }

    // session はこの API で変換テーブルクラスのインスタンスを取得し、
    // 更新処理などを実行してもらうことを想定しています。
    const StationIdTable* GetStationIdTable() const
    {
        return m_pStationIdTable;
    }
    StationIdTable* GetStationIdTable()
    {
        return m_pStationIdTable;
    }

    /*!
    @brief  SetDebugSetting() に渡すパラメータをまとめた構造体です。
     */
    struct DebugSetting
    {
        DebugSetting()
            : sendThreadLatencyEmulationPacketNum(0),
              receiveThreadLatencyEmulationPacketNum(0),
              isPacketLossEmulationEnabled(false)
        {
        }  //!< デフォルトコンストラクタです。各メンバ変数をクリアします。

        uint32_t sendThreadLatencyEmulationPacketNum;     //!< 送信レイテンシエミュレーション機能のためのバッファの個数です（1個につき約1.5KB）。
        uint32_t receiveThreadLatencyEmulationPacketNum;  //!< 受信レイテンシエミュレーション機能のためのバッファの個数です（1個につき約1.5KB）。
        bool isPacketLossEmulationEnabled;                //!< パケロスエミュレーション機能を用いる場合は true をセットする必要があります。
    };


    /*!
    @brief   デバッグ用の機能を有効にします。

    @details 引数で設定した内容で、デバッグ用機能を有効にします。
             この API を製品版 ROM で呼び出してはなりません。
    @details この API を使用する際は、Transport::CreateInstance() 呼び出しよりも
             前に呼び出す必要があります。

    @return  成功すれば、 IsSuccess()が true を返す Result が返されます。
    @retval  ResultInvalidState   この API を呼び出すタイミングが誤っています。[:progErr]
     */
    static Result SetDebugSetting(const struct DebugSetting& setting);


    /*!
    @brief デバッグに有用な情報をプリントします。
 
    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    //n1589:NexFacadeで必要
    common::IPacketInput* GetInputStream(void);
    common::IPacketOutput* GetOutputStream(void);

    uint8_t GetSendingConnectionId() const
    {
        return m_SendingConnectionId;
    }

    void OutputStreamUpdateEvent(void);

    // 切断イベント
    enum DisconnectionEventType
    {
        DisconnectionEventType_EventLeave,  // 通常離脱
        DisconnectionEventType_EventRejoin  // 別セッションへ合流するために一時的な離脱
    };

    // イベントリスナー
    typedef DisconnectionEventType (*DisconnectionEventListener)(void* pStationIdTableEntry);

    void RegisterEventListener(DisconnectionEventListener func)
    {
        m_EventListener = func;
    }
    DisconnectionEventListener GetEventListener()
    {
        return m_EventListener;
    }

    bool IsConnectionRequestPrincipalIdEnabled() const
    {
        return m_IsConnectionRequestPrincipalIdEnabled;
    }

    uint32_t GetDispatchCountForTransportAnalyzer() const
    {
        return m_DispatchCountForTransportAnalyzer;
    }

    void ClearDispatchCountForTransportAnalyzer()
    {
        m_DispatchCountForTransportAnalyzer = 0;
    }

private:
    /*!
    @cond PRIVATE
    @brief    ディスパッチ関数です。
    
    @details この関数を実行すると、 PiaTransport の送受信スレッドとの間でデータが受け渡されたり、
              各プロトコル固有の送受信処理などが実行されます。
              この関数は、無線モジュールに対する送受信関数を呼び出してはいませんので、
              送受信処理待ちによるブロックは発生しません
              （無線モジュールに対する送受信関数呼び出しは、 PiaTransport の送受信スレッドの中で
              実行されます）。
              この関数は、少なくとも毎ゲームフレームに 1 回は呼び出してもらうことを想定しています。

    @return   成功すれば、 IsSuccess()が true を返す Result が返されます。
    @retval   ResultInvalidState    プロトコルの初期化処理などが済んでいない可能性があります。[:progErr]
     */
    Result dispatch(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      Transportインスタンスを初期化します。

    @details 使用するファクトリと、最大 Station 数を設定して初期化します。
    <br>
    @param[in]  setting 各種パラメータが書き込まれた Setting 構造体。
    @return     成功すれば、  IsSuccess()が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @see        finalize
     */
    Result initialize(const Transport::Setting& setting);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 解放処理です。
    @see initialize
     */
    void finalize(void);
    //! @endcond


    /*!
    @brief コピーコンストラクタは封印します。
     */
    Transport(const Transport&);

    /*!
    @brief 代入演算子は封印します。
     */
    Transport& operator=(const Transport&);


    /*!
    @brief    dispatch() 直後の時刻を更新します。

    @details CPU負荷低減のため、1回の Dispatch()につき、1回だけ呼び出すようにします。
              この時刻はキープアライブ処理などに使われます。さほど精度が必要とされる処理では無いので、
              dispatch() 一回につき一度の更新で十分だと考えます。
    @see      GetDispatchedTime
     */
    void updateDispatchedTime(void)
    {
        m_DispatchedTime = common::Time::GetTime();
    }


    void updateTransportAnalyzer(void);


    // dispatch() を呼ぶためのジョブ
    class DispatchJob : public common::Job
    {
    protected:
        virtual ExecuteResult ExecuteCore()
        {
            Transport::GetInstance()->dispatch();
            return Job::ExecuteState_WaitNext;
        }
    };

    common::Time m_DispatchedTime;

    NetworkFactory* m_pNetworkFactory;

    ProtocolManager m_ProtocolManager;
    PacketHandler* m_pPacketHandler;

    uint16_t m_MaxStationNum;

    uint8_t m_SendingConnectionId;

    bool m_IsStartup;
    bool m_IsRestartupReserved;

    Result m_ConnectionError;
    DispatchJob m_DispatchJob;

    IRelayAddressTable* m_pRelayAddressTable;
    RelayRouteManager* m_pRelayRouteManager;

    int32_t m_AnalysisInterval;          // TransportAnalyzer::Update() を自動的に呼び出す時間間隔。単位は秒。
    common::Time m_AnalysisUpdatedTime;  // TransportAnalyzer::Update() が最後に呼び出されたときの時間。
    bool m_IsAnalysisAutoPrintDetail;
    bool m_IsAnalysisAutoPrintTotalOnly;
    uint32_t m_DispatchCountForTransportAnalyzer;

    DisconnectionEventListener m_EventListener;

    StationIdTable* m_pStationIdTable;
    bool m_IsStationIdTableEnabled;

    Result m_TransportStatus;  // session によって制御される Transport 側の状態。
    
    bool m_IsConnectionRequestPrincipalIdEnabled;

    static struct DebugSetting s_DebugSetting;

    static Transport* s_pInstance;
};
}
}
}  // end of namespace nn::pia::transport
