﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/transport/transport_TransportAnalysisData.h>
#include <nn/pia/common/common_Time.h>


namespace nn
{
namespace pia
{
namespace transport
{


// 先行宣言
class ConnectionAnalyzer;

/*!
  @brief  TransportAnalyzer クラスは、パケット解析結果や RTT 、パケロス率などを
          報告するために用意されたクラスです。

 */
class TransportAnalyzer : public common::RootObject
{
public:
    /*!
    @brief   トランスポート分析データのプリントコールバック型です。
    @details トランスポート分析データの出力文字列を独自定義の関数に渡したい場合に使用します。

    @param[in] pStr トランスポート分析データのテキスト文字列が渡されます。改行文字は含まれません。
     */
    typedef void (*AnalysisPrintCallback)(const char* pStr);


    /*!
    @cond PRIVATE
    @brief     TransportAnalyzer クラスのインスタンスを作成します（シングルトンパターン）。

    @return    インスタンスの作成に成功すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval    ResultNotInitialized     transport モジュールが未初期化です。[:progErr]
    @retval    ResultInvalidState       CreateInstance() を呼ぶタイミングが誤っています。 BeginSetup() ～ EndSetup() 間で呼び出す必要があります。[:progErr]
    @retval    ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    @retval    ResultAllocationFailed   内部でメモリ確保に失敗しました。 common::Initialize() で Pia に供給するメモリを増やす必要があります。[:progErr]
    @see       DestroyInstance, GetInstance, Setting
    @endcond
     */
    static Result CreateInstance();


    /*!
    @cond PRIVATE
    @brief    TransportAnalyzer クラスのインスタンスを破棄します（シングルトンパターン）。
              インスタンスが存在していなかった場合は、何もしません。

    @see      CreateInstance, GetInstance
    @endcond
     */
    static void DestroyInstance();


    /*!
    @brief    TransportAnalyzer クラスのインスタンスを得ます（シングルトンパターン）。
              事前に CreateInstance() によってインスタンスを作成していない場合は、
              NULL ポインタが返されます。
              この関数はスレッドセーフです。

    @return   TransportAnalyzer インスタンスへのポインタ。
    @see      CreateInstance, DestroyInstance
     */
    static TransportAnalyzer* GetInstance()
    {
        return s_pInstance;
    }

    /*!
    @cond PRIVATE
    @brief    初期化処理を実行します。

    @return    初期化処理が成功すれば、IsSuccess() が true となる Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval    ResultAlreadyInitialized 既に Initialize() は実行されています。[:progErr]
    @retval    ResultAllocationFailed   内部でメモリ確保に失敗しました。 common::Initialize() で Pia に供給するメモリを増やす必要があります。[:progErr]
    @endcond
     */
    Result Initialize();


    /*!
    @cond PRIVATE
    @brief    終了処理を実行します。
    @endcond
     */
    void Finalize();


    /*!
    @cond PRIVATE
    @brief    解析を開始します。

    @return    処理が成功すれば、IsSuccess() が true となる Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval    ResultNotInitialized     TransportAnalyzer インスタンスが未初期化です。[:progErr]
    @retval    ResultInvalidState       この関数を呼び出すタイミングが誤っています。[:progErr]
    @endcond
     */
    Result Startup();


    /*!
    @cond PRIVATE
    @brief    解析を終了します。
    @endcond
     */
    void Cleanup();


    /*!
    @brief     データを更新します。

    @details   Transport::Setting 構造体の analysisInterval メンバ変数に正の値をセットして、
               分析データの自動プリントを有効にした状態であれば、アプリケーションはこの API を
               呼び出す必要はありません。
               analysisInterval メンバ変数に 0 の値をセットし、自動プリント機能を無効にした
               状態でアプリケーションが分析データを取得する際は、明示的にこの API を呼びだし、
               データの更新を行う必要があります。

    @return    処理が成功すれば、IsSuccess() が true となる Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval    ResultNotInitialized     TransportAnalyzer インスタンスが未初期化です。[:progErr]
    @retval    ResultInvalidState       この関数を呼び出すタイミングが誤っています。[:progErr]
     */
    Result Update();


    /*!
    @brief データを取得します。
     */
    const struct TransportAnalysisData& GetTransportAnalysisData() const
    {
        return m_Data;
    }


    /*!
    @brief グラフに区切りをいれるための文字列をコンソールに出力します。

    @param[in] pComment  区切り線に付ける名前の文字列を指すポインタ。文字列に空白類文字を含めることはできません。また、NULL を指定することもできません。

    @retval    ResultInvalidArgument    引数が誤っています。NULL ポインタや、空白類文字を含む文字列を与えることはできません。[:progErr]
    @retval    ResultInvalidState       この関数を呼び出すタイミングが誤っています。[:progErr]
     */
    Result SetMarker(const char* pComment);

    /*!
    @brief プリントコールバックを登録します。

    @param[in] pCallback 登録するコールバック関数ポインタ。

    @return この関数を呼び出す前に登録されていたプリントコールバックが返されます。
     */
    AnalysisPrintCallback RegisterPrintCallback(AnalysisPrintCallback pCallback);


    // コンストラクタです。
    TransportAnalyzer();


    // デストラクタです。
    ~TransportAnalyzer();


    /*!
    @cond PRIVATE
    @brief    モニタリングデータを書き込みます。
    @endcond
     */
    void SetMonitoringData();


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;

private:
    ConnectionAnalyzer* m_pConnectionAnalyzer;
    bool m_IsStartup;
    struct TransportAnalysisData m_Data;

    common::Time m_StartupTime;  // Startup() が呼ばれた時刻。

    static TransportAnalyzer* s_pInstance;
};
}
}
}  // end of namespace nn::pia::transport
