﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/transport/transport_Protocol.h>
#include <nn/pia/common/common_PacketDefine.h>


// 先行宣言。
namespace nn
{
namespace pia
{
namespace transport
{
class ReceivedDataContainer;
}
}
}


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  信頼性の無い（データ欠落の可能性がある）プロトコルのクラスです。
          UnreliableProtocol は、データ到着の順序を保証しません。
          送信側がデータを Send() した順で受信側がデータを Receive() できるとは限りません。
 */
class UnreliableProtocol : public transport::Protocol
{
public:
    /*!
    @brief UnreliableProtocol で一度に送信／受信が可能なデータサイズの最大値です。単位はバイトです。

    @details この定数値は 1440 ですが、アプリケーションが小さめの MTU 値を設定した場合には、実際に
           送受信可能なデータサイズはこの値よりも小さくなります。
     */
    static const uint32_t MaxDataSize = common::ProtocolMessgaePayloadSizeMax;


    /*!
    @brief    一度に送信可能なデータサイズの上限を取得します。単位はバイトです。

    @details  この API 呼び出しで返される値は、アプリケーションによる MTU 値の設定や、
              パケットへの署名の有無に影響されます。
              この関数はスレッドセーフです。
    @details  アプリケーションがメッシュを使用する場合は、メッシュのスタートアップ処理が
              完了した後でこの API を呼び出す必要があります。これよりも早いタイミングで
              呼び出すと、正しい値が返されません。
    @details  アプリケーションがセッションを使用する場合は、セッション参加後にこの API を
              呼び出す必要があります。これよりも早いタイミングで呼び出すと、正しい値が
              返されないことがあります。
    @return   Send(), SendToAll() で一度に送信可能なデータサイズの上限が返されます。単位はバイトです。

    @see      Send, SendToAll
     */
    uint32_t GetDataSizeLimit(void);


    /*!
    @cond PRIVATE
    @brief デフォルトコンストラクタです。

    @details アプリケーションプログラムは、このコンストラクタを直接呼び出してはいけません。
           インスタンスの作成は、 @ref Transport::CreateProtocol を利用してください。
     */
    UnreliableProtocol(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief デストラクタです。

    @details アプリケーションプログラムは、このデストラクタを直接呼び出してはいけません。
           インスタンスの破棄は、 @ref Transport::DestroyProtocol を利用してください。
     */
    virtual ~UnreliableProtocol(void);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief ディスパッチ関数です。

    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval   ResultInvalidState    UnreliableProtocol::Initialize() が実行されていない可能性があります。[:progErr]
     */
    virtual Result Dispatch(void);
    //! @endcond


    virtual Result UpdateProtocolEvent(const ProtocolEvent& event);


    PIA_PROTOCOL_TYPE_INFO(ProtocolTypeUnreliable);


    /*!
    @brief    インスタンスを初期化します。

    @details BeginSetup() ～ EndSetup() 間で呼び出す必要があります。
    <br>

    @param[in]  recvBufNum  受信バッファの個数。受信バッファ1個につき、約 1500 バイトのメモリが使用されます。
    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval   ResultNotInitialized    transport モジュールが未初期化です。[:progErr]
    @retval   ResultAllocationFailed  メモリ確保に失敗しました。 common::Initialize() 呼び出し時に、より多くのメモリを供給することを検討してください。[:progErr]
    @retval   ResultAlreadyInitialized UnreliableProtocol は既に初期化されています。[:progErr]
    @retval   ResultInvalidArgument   引数が誤っています。 recvBufNum に 0 を指定するなどした場合に返されます。[:progErr]
    @retval   ResultInvalidState      BeginSetup() ～ EndSetup() 間で呼び出されていません。[:progErr]
    @see      Finalize
     */
    Result Initialize(uint32_t recvBufNum = 32);


    /*!
    @brief    インスタンスの終了処理を実行します。

    @details Initialize() が呼び出されていない状態でこの関数を呼び出した場合は、何もせずに返ります。
    <br>
    @see      Initialize
     */
    void Finalize(void);


    /*!
    @cond PRIVATE
    @brief      通信を開始します。

    @param[in]  localStationAddress 自分の（ローカルの） StationAddress 。
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval     ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState    既に Startup() は呼ばれています。[:progErr]

     */
    virtual Result StartupWithStationAddress(const common::StationAddress& localStationAddress);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief      通信を終了します。
     */
    virtual void CleanupWithStationAddress(void);
    //! @endcond


    /*!
    @brief    受信したデータをバッファに読み取ります。

    @details データを受信します。ただし、この関数呼び出しが無線モジュールの受信処理実行のトリガーとなっている
              わけではありません。
              無線モジュールからパケットを受信する処理は、 Pia 内部の受信スレッドによって定期的に実行されています。
              この受信スレッドが受信したパケットは、 PiaTransport のディスパッチ処理が実行されるタイミングで
              パケット解析処理用バッファにコピーされ、データが復元されます。
              この関数は、このようにして復元されたデータをアプリケーション側で用意したバッファにコピーする処理を実行します。

    @param[out]  pSrcId        読み取ったデータの送信元 StationId が書き込まれます。
    @param[out]  pRecvBuf      データの読み取り先バッファです。
    @param[out]  pRecvDataSize 実際に読み取ることのできたデータサイズが書き込まれます。単位はバイトです。
    @param[in]   recvBuffSize  データの読み取り先バッファのサイズを指定します。単位はバイトです。

    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval   ResultInvalidArgument 引数に誤りがあります。[:progErr]
    @retval   ResultInvalidState    UnreliableProtocol::Initialize() が呼ばれていなかったり、session のセットアップが完了していない可能性があります。[:progErr]
    @retval   ResultBufferShortage  読み取りバッファのサイズが不足しています。 recvBuffSize で指定したサイズを上回る大きさのデータが到着していた場合に返されます。[:progErr]
    @retval   ResultNoData          受信バッファが空っぽで、読み取ることのできるデータがありませんでした。[:handling]
    @retval   ResultNotInCommunication 通信が可能な状態ではありません。[:handling]
    @retval   ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    @see      Send, SendToAll, MaxDataSize
     */
    Result Receive(StationId* pSrcId, void* pRecvBuf, uint32_t* pRecvDataSize, uint32_t recvBuffSize);


    /*!
    @cond PRIVATE
    @brief    受信したデータをバッファに読み取ります。
     */
    Result Receive(StationIndex* pSrcId, void* pRecvBuf, uint32_t* pRecvDataSize, uint32_t recvBuffSize);
    //! @endcond


    /*!
    @brief    データを特定のステーションに向けて送信します。

    @details データを送信します。ただし、この関数を呼び出しただけでは実際のデータ送信は行われません。
              この関数は、アプリケーションが渡したデータを Pia 内部のパケット生成処理用バッファにコピーします。
              その後、 PiaTransport のディスパッチ処理が実行されることで、パケット生成処理用バッファにコピー
              されていたデータ群がパケットにまとめられ、 Pia 内部の送信スレッドに渡されます。実際の送信処理は、
              この送信スレッドが定期的にパケットを無線モジュールに渡すことで行われます。

    @param[in]  destId   送信先の StationId 。
    @param[in]  pData    送信したいデータの先頭を指すポインタ。
    @param[in]  dataSize 送信データサイズ。単位はバイトです。 GetDataSizeLimit() が返す値を超過してはいけません。
    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval   ResultInvalidArgument 引数に誤りがあります。送信データサイズが過大であった場合にも、このエラーが返されます。[:progErr]
    @retval   ResultInvalidState    UnreliableProtocol::Initialize() が呼ばれていなかったり、session のセットアップが完了していない可能性があります。[:progErr]
    @retval   ResultNotInCommunication 通信が可能な状態ではありません。[:handling]
    @retval   ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    @retval   ResultNotFound 指定された送信先が見つかりません。
    @see      SendToAll, Receive, MaxDataSize, GetDataSizeLimit, IsInCommunication
     */
    Result Send(StationId destId, const void* pData, uint32_t dataSize);


    /*!
    @cond PRIVATE
    @brief    データを特定のステーションに向けて送信します。
     */
    Result Send(StationIndex destId, const void* pData, uint32_t dataSize);
    //! @endcond


    /*!
    @brief    データを全ステーションに向けて送信します。

    @details データを送信します。ただし、この関数を呼び出しただけでは実際のデータ送信は行われません。
              この関数は、アプリケーションが渡したデータを Pia 内部のパケット生成処理用バッファにコピーします。
              その後、 PiaTransport のディスパッチ処理が実行されることで、パケット生成処理用バッファにコピー
              されていたデータ群がパケットにまとめられ、 Pia 内部の送信スレッドに渡されます。実際の送信処理は、
              この送信スレッドが定期的にパケットを無線モジュールに渡すことで行われます。

    @param[in]  pData    送信したいデータの先頭を指すポインタ。
    @param[in]  dataSize 送信データサイズ。単位はバイトです。 GetDataSizeLimit() が返す値を超過してはいけません。
    @return   成功すれば、 IsSuccess() が true を返す Result が返されます。
    @retval   ResultInvalidArgument 引数に誤りがあります。送信データサイズが過大であった場合にも、このエラーが返されます。[:progErr]
    @retval   ResultInvalidState    UnreliableProtocol::Initialize() が呼ばれていない可能性があります。[:progErr]
    @retval   ResultNotInCommunication 通信が可能な状態ではありません。[:handling]
    @retval   ResultTemporaryUnavailable セッション移行処理中のため、一時的に API を利用できない状態です。ジョイントセッション機能使用時にのみ返ります。[:handling]
    @see      Send, Receive, MaxDataSize, GetDataSizeLimit, IsInCommunication
     */
    Result SendToAll(const void* pData, uint32_t dataSize);


    /*!
    @cond PRIVATE
    @brief     データを複数ステーションに向けて送信します。
     */
    Result SendToMulti(uint32_t stationBitmap, const void* pData, uint32_t dataSize);
    //! @endcond


    /*!
    @brief  通信可能な状態であるかどうかを判定します。

    @return 通信が可能な状態であれば true 、そうでなければ false が返されます。
     */
    bool IsInCommunication(void) const;


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


private:
    Result sendImpl(StationIndex destId, const void* pData, uint32_t dataSize);

protected:
    void* m_pRawBuffer;  // m_pReceiveBuffer に渡される生メモリ。
    ReceivedDataContainer* m_pReceiveBuffer;

    StationIndex m_LocalStationIndex;
};
}
}
}  // end of namespace nn::pia::transport
