﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/tune/tune_CpuProfilerImplBase.h>

namespace nn
{
namespace pia
{
namespace tune
{

class CpuProfiler
{
public:
    explicit CpuProfiler();
    ~CpuProfiler();

    /*!
    @brief シングルトンインスタンスを生成します。

           この関数より先にnn::os::InitializeAlarmSystem()を呼んでおく必要があります。(CTR)
     */
    static void CreateInstance();


    /*!
    @brief シングルトンインスタンスを破棄します。
     */
    static void DestroyInstance();

    void ConstructorImpl();

    /*!
    @brief シングルトンインスタンスを取得します。
     */
    static CpuProfiler* GetInstance()
    {
        return s_pInstance;
    }

#if NN_PIA_CTR || NN_PIA_A
    /*!
    @brief 何もしない場合の負荷を計測します。指定時間の計測が終了してからこの関数を抜けます。
    */
    virtual void StartProfilingIdle(uint32_t milliSec);


    /*!
    @brief 通信を行っていない状態の負荷の計測を開始します。この関数を呼んでから指定時間の間を計測します。
    */
    virtual void StartProfilingBase(uint32_t milliSec);


    /*!
    @brief 通信を行っている状態の負荷の計測を開始します。この関数を呼んでから指定時間の間を計測します。
    */
    virtual void StartProfilingTarget(uint32_t milliSec);
#endif

#if NN_PIA_NINTENDOSDK
    /*!
    @brief 負荷計測を開始します。
           FinishProfiling() が呼ばれるまで計測を行います。
     */
    void StartProfiling();

    /*!
    @brief 指定時間(ミリ秒)の間、負荷を計測します。
    */
    void StartAutoProfilingWithMilliSeconds(uint32_t milliSec);
#endif

    /*!
    @brief 計測を終了します。
    */
    void FinishProfiling();

    

    /*!
    @brief 計測結果をコンソールに出力します。
     */
    void Report();

    /*!
    @brief 指定のコアの負荷の計測が終了しているかどうかを取得します。

    @return 計測が終了していればtrueを返します。
    */
    bool IsProfiling(uint32_t coreIndex = 0) const;

    /*!
    @brief アプリに表示できるように総パケット数、パケットロス数を取得できるようにします。
    @param[in]  stationId 送信元ステーション
    @param[out] totalPacketNum 受信できたパケット数
    @param[out] packetLossNum パケットロス数
     */
    void GetPacketLoss(nn::pia::StationId stationId, uint16_t& totalPacketNum, uint16_t& packetLossNum, uint16_t& multicastPacketNum);

    /*!
    @brief アプリに表示できるように計測中のチック数を取得できるようにします。
    @param[out] systemTick システムのチック数
    @param[out] idleTick アイドルのチック数
    @param[in]  coreIndex コア番号
    */
    void GetTickCount(int64_t& systemTick, int64_t& idleTick, uint32_t coreIndex = 0);

    /*!
    @brief アプリに表示できるように計測にかかった時間(ミリ秒)を取得できるようにします。
    @param[out] profilingTime 計測にかかった時間(ミリ秒)
    @param[in]  coreIndex コア番号
    */
    void GetProfileTimeMilliSeconds(int64_t& profileTime, uint32_t coreIndex = 0);

    /*!
    @brief アプリに表示できるように計測中のアイドル時間(ミリ秒)を取得できるようにします。
    @param[out] idleTime 計測中のアイドル時間(ミリ秒)
    @param[in]  coreIndex コア番号
    */
    void GetIdleTimeMilliSeconds(int64_t& idleTime, uint32_t coreIndex = 0);

private:

    PIA_ATTRIBUTE_ALIGN(8) static CpuProfiler* s_pInstance;
    CpuProfilerImplBase* m_pImplBase;
};
}
}
}  // end of namespace nn::pia::tune
