﻿/*--------------------------------------------------------------------------------*
Copyright (C)Nintendo All rights reserved.

These coded instructions, statements, and computer programs contain proprietary
information of Nintendo and/or its licensed developers and are protected by
national and international copyright laws. They may not be disclosed to third
parties or copied or duplicated in any form, in whole or in part, without the
prior written consent of Nintendo.

The content herein is highly confidential and should be handled accordingly.
*--------------------------------------------------------------------------------*/
#pragma once

#include <nn/pia/tune/tune_Definitions.h>

#if NN_PIA_CTR
#include <nn/pia/tune/tune_CpuProfiler.h>
#include <nn/pia/tune/tune_Thread.h>
#include <nn/pia/common/common_Time.h>

namespace nn
{
namespace pia
{
namespace tune
{
class ProfileResult
{
public:
    int32_t count;
    int32_t predictedCount;
    nn::pia::common::TimeSpan period;

    ProfileResult()
    {
        count = 0;
        predictedCount = 0;
        period = 0;
    }

    void CalcPredictedCount(nn::pia::common::TimeSpan timeSpan)
    {
        if (period == 0)
        {
            predictedCount = 0;
        }
        else
        {
            predictedCount = (int32_t)(count * timeSpan.GetMSecS64() / period.GetMSecS64());
        }
    }
};

class CpuProfilerImplCtr : public CpuProfilerImplBase
{
public:
    CpuProfilerImplCtr();

    virtual ~CpuProfilerImplCtr();

    virtual void Finalize()
    {

    }

    /*!
    @brief 何もしない場合の負荷を計測します。指定時間の計測が終了してからこの関数を抜けます。
    */
    virtual void StartProfilingIdle(uint32_t milliSec);


    /*!
    @brief 通信を行っていない状態の負荷の計測を開始します。この関数を呼んでから指定時間の間を計測します。
    */
    virtual void StartProfilingBase(uint32_t milliSec);


    /*!
    @brief 通信を行っている状態の負荷の計測を開始します。この関数を呼んでから指定時間の間を計測します。
    */
    virtual void StartProfilingTarget(uint32_t milliSec);

    /*!
    @brief 計測を終了します。
    */
    virtual void FinishProfiling();

    /*!
    @brief 計測結果をコンソールに出力します。
    */
    virtual void Report();


    /*!
    @brief 計測中かどうかを取得します。

    @return 計測中ならtrueを返します。
    */
    virtual bool IsProfiling(uint32_t coreIndex = 0) const
    {
        NN_PIA_UNUSED(coreIndex);
        return m_IsCounting;
    };

    /*!
    @brief アプリに表示できるように総パケット数、パケットロス数を取得できるようにします。
    @param[in]  stationId 送信元ステーション
    @param[out] totalPacketNum 受信できたパケット数
    @param[out] packetLossNum パケットロス数
    */
    virtual void GetPacketLoss(nn::pia::StationId stationId, uint16_t& totalPacketNum, uint16_t& packetLossNum, uint16_t& multicastPacketNum);

    /*!
    @brief アプリに表示できるように計測中のチック数を取得できるようにします。
    @param[out] systemTick システムのチック数
    @param[out] idleTick アイドルのチック数
    @param[in]  coreIndex コア番号
    */
    virtual void GetTickCount(int64_t& systemTick, int64_t& idleTick, uint32_t coreIndex = 0);

    /*!
    @brief アプリに表示できるように計測にかかった時間(ミリ秒)を取得できるようにします。
    @param[out] profilingTime 計測にかかった時間(ミリ秒)
    @param[in]  coreIndex コア番号
    */
    virtual void GetProfileTimeMilliSeconds(int64_t& profilingTime, uint32_t coreIndex = 0);

    /*!
    @brief アプリに表示できるように計測中のアイドル時間(ミリ秒)を取得できるようにします。
    @param[out] idleTime 計測中のアイドル時間(ミリ秒)
    @param[in]  coreIndex コア番号
    */
    virtual void GetIdleTimeMilliSeconds(int64_t& idleTime, uint32_t coreIndex = 0);
private:
    void StartCount(uint32_t second, ProfileResult* pProfileResult);

    void StopCount();

    void Count();

    void StartProfilePacketLoss();

    void StopProfilePacketLoss();

    void ReportPacketLoss();

    void StartPacketProfile();  //n1769

    void StopPacketProfile();  //n1769

    void ReportPacketProfile();  //n1769

    void SleepProfileSpan();  //n1769

    static void SpinWait(int32_t msec);  //n1769

    static void AlarmHandler(void* param);  //n1769

    PIA_ATTRIBUTE_ALIGN(8) Thread m_CountThread;

    static void RunIdleThread(void* pProfiler);
    nn::fnd::TimeSpan m_ProfileSpan;

    volatile int32_t m_CurrentProfileCount;  // n3492: 一定時間に一度インクリメントされるカウンタ
    volatile bool m_IsCounting;              // n3492: カウント中かどうか
    volatile bool m_IsExitIdleThread;        // n3492: アイドル時間のカウントが終了したかどうか

    nn::pia::common::Time m_CurrentProfileStartTime;

    ProfileResult m_IdleProfileResult;
    ProfileResult m_BaseProfileResult;
    ProfileResult m_TargetProfileResult;
    ProfileResult* m_pCurrentProfileResult;

    CpuProfilerImplBase* m_pImplBase;
};
}
}
}
#endif
