﻿/*--------------------------------------------------------------------------------*
Copyright (C)Nintendo All rights reserved.

These coded instructions, statements, and computer programs contain proprietary
information of Nintendo and/or its licensed developers and are protected by
national and international copyright laws. They may not be disclosed to third
parties or copied or duplicated in any form, in whole or in part, without the
prior written consent of Nintendo.

The content herein is highly confidential and should be handled accordingly.
*--------------------------------------------------------------------------------*/
#pragma once

#include <nn/pia/tune/tune_CpuProfiler.h>
#include <nn/pia/tune/tune_NonStaticAlarm.h>
#include <nn/pia/tune/tune_Thread.h>
#include <nn/pia/common/common_Time.h>

#if NN_PIA_NINTENDOSDK
#include <nn/os.h>

namespace nn
{
namespace pia
{
namespace tune
{

enum ProfilingState
{
    None = 0,
    StartAutoProfile,
    StartProfile,
    Profiling,
    Reporting,
    ProfilingFinished
};

struct TickData
{
    nn::os::Tick systemTick;
    nn::os::Tick idleTick;
};

struct ReportData
{
    ReportData()
        : systemTick(0)
        , idleTick(0)
        , coreIndex(0)
        , profileTime(0)
    {

    }

    nn::os::Tick systemTick;
    nn::os::Tick idleTick;
    uint32_t coreIndex;
    nn::pia::common::TimeSpan profileTime;
};

class CpuProfilerThread
{

public:
    CpuProfilerThread(CpuProfilerImplBase* profiler, uint32_t coreId);
    ~CpuProfilerThread();

    /*!
    @brief 負荷の計測を開始します。
    FinishProfiling() を呼ぶと計測を終了します。
    */
    void StartProfiling();
    void StartProfilingCore();

    /*!
    @brief 負荷の計測を終了します。
    */
    void FinishProfiling();
    void FinishProfilingCore();

    /*!
    @brief 負荷の測定を開始します。
    指定時間経つと計測は自動的に終了します。
    */
    void StartAutoProfilingWithMilliSeconds(uint32_t milliSec);

    /*!
    @brief 計測結果をコンソールに出力します。
    */
    void Report();

    // この関数を通じてCpuProfilerImplNinからReportの終了が通知されます
    void NotifyEndReport();

    /*!
    @brief 計測中かどうかを取得します。
    @return 計測中ならtrueを返します。
    */
    bool IsProfiling() const
    {
        return (
            m_State == StartAutoProfile ||
            m_State == StartProfile ||
            m_State == Profiling ||
            m_State == Reporting
            );
    }

    /*!
    @brief アプリに表示できるように計測時のチック数を取得できるようにします。
    @param[out] systemTick システムのチック数
    @param[out] idleTick アイドルのチック数
    */
    void GetTickCount(int64_t& systemTick, int64_t& idleTick);

    /*!
    @brief アプリに表示できるように計測終了までにかかった時間(ミリ秒)を取得できるようにします。
    @param[out] profilingTime 計測にかかった時間(ミリ秒)
    */
    void GetProfileTimeMilliSeconds(int64_t& profilingTime);

    /*!
    @brief アプリに表示できるように計測中のアイドル時間(ミリ秒)を取得できるようにします。
    @param[out] idleTime 計測中のアイドル時間(ミリ秒)
    */
    void GetIdleTimeMilliSeconds(int64_t& idleTime);

private:
    static void AlarmHandler(void* param);

    struct TickData GetTickData();

    static void ThreadFunc(void* param);
    void Run();

    TickData m_BeforeData;
    TickData m_AfterData;

    uint32_t m_GivenProfileTime;
    struct ReportData m_ReportData;

    nn::pia::common::Time m_CurrentProfileStartTime;
    nn::pia::common::TimeSpan m_ProfiledTimeSpan;    // 計測した時間の長さ

    ProfilingState m_State;

    Thread m_Thread;
    NonStaticAlarm m_Alarm;

    static const size_t s_ThreadStackSize = 1024 * 8;  // スレッド操作スレッドのスタックサイズ
    uint8_t* m_pThreadStack;

    CpuProfilerImplBase* m_pProfilerImpl;

    bool m_IsCalledFinishProfiling;
};
}
}
}
#endif
