﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#include "SampleFramework_Color.h"
#include "SampleFramework_ButtonDefines.h"

#if defined(NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST)
#include "PiaTestLib/PiaTest.h"
#endif  // end of NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST


namespace nns
{
namespace pia
{
namespace sampleframework
{

// 先行宣言。
class SampleFrameworkImplBase;


/*!
  @brief      Piaサンプルデモプログラムで利用されることを想定した、簡単なフレームワークのクラスです。
 */
class SampleFramework
{
public:
    // SampleFramework インスタンス作成における設定パラメータをまとめた構造体です。
    struct Setting
    {
        Setting(void)
            : isUpperScreenOnly(false), isRawTouchpad(false)
        {
        }
        bool isUpperScreenOnly;  // Cafe環境でのみ意味のあるパラメータ。trueにすると、上画面のみをDRCに拡大表示します。
        bool isRawTouchpad;      // Cafe環境でのみ意味のあるパラメータ。trueにすると、生のタッチパネル座標を取得します。
    };

#if NN_PIA_A
    static void SetApp(void* app);
#endif

    /*!
    @brief     インスタンスを作成します（Singletonパターン）。

    @return    インスタンスの作成に成功すればtrue、失敗すればfalseが返ります。

     */
    static bool CreateInstance(void);


    /*!
    @brief     引数による設定を与えたインスタンスを作成します（Singletonパターン）。
    @param[in] setting フレームワークの設定を記述した Setting 構造体。

    @return    インスタンスの作成に成功すればtrue、失敗すればfalseが返ります。
     */
    static bool CreateInstance(const SampleFramework::Setting& setting);


    /*!
    @brief インスタンスを破棄します（Singletonパターン）。
     */
    static void DestroyInstance(void);


    /*!
    @brief インスタンスへのポインタを取得します（Singletonパターン）。
    @return SampleFrameworkインスタンスへのポインタ。
     */
    static SampleFramework* GetInstance(void);


    /*!
    @brief インスタンスが作成されているかどうかを判定します（Singletonパターン）。
    @return 判定結果が返されます。
     */
    static bool IsCreated(void);


    ~SampleFramework(void);


    /*!
    @brief 画面を消去します。
     */
    void ClearScreen(void);


    /*!
    @brief クリアカラーを指定します。
    @param[in] clr  @ref EndFrame() 呼び出し時に塗りつぶされる色を指定します。
     */
    void SetClearColor(const struct Color& clr);


    /*!
    @brief フォントカラーを指定します。

           初期設定は White (1.0f, 1.0f, 1.0f, 1.0f) です。

    @param[in] clr  @ref DrawString で描画されるフォントの色を指定します。
     */
    void SetFontColor(const struct Color& clr);

    /*!
    @brief フォントサイズを設定します。幅、高さを pixel で指定した値に設定します。
           フォントサイズは 8 が初期値になっています。
     */
    void SetFontSize(int pixel);

    /*!
    @brief フォントの幅を取得します。
     */
    int GetFontWidth(void) const;

    /*!
    @brief フォントの高さを取得します。
     */
    int GetFontHeight(void) const;

    /*!
    @brief 指定した位置にテキストを描画します。画面左上座標を(0, 0)とします。

    @param[in] x   X座標。
    @param[in] y   Y座標。
    @param[in] fmt printf書式。
     */
    void DrawString(int32_t x, int32_t y, const char* fmt, ...);


    /*!
    @brief 長方形を描画します。

    @param[in] x   長方形の左上のX座標。
    @param[in] y   長方形の左上のY座標。
    @param[in] w   長方形の幅。
    @param[in] h   長方形の高さ。
    @param[in] clr 長方形の色。
     */
    void DrawRect(int32_t x, int32_t y, int32_t w, int32_t h, const struct Color& clr);


    /*!
    @brief 点を描画するときのサイズを設定します。

    @param[in] pointSize 点のピクセル単位の大きさ
     */
    void SetPointSize(float pointSize);


    /*!
    @brief 点を描画します。

    @param[in] x   点の中心のX座標。
    @param[in] y   点の中心のY座標。
    @param[in] clr 点の色。
     */
    void DrawPoint(int32_t x, int32_t y, const struct Color& clr);


    /*!
    @brief 線を描画するときの太さを設定します。

    @param[in] lineWidth 線のピクセル単位の太さ
     */
    void SetLineWidth(float lineWidth);


    /*!
    @brief 線分を描画します。

    @param[in] x0  線分の始点のX座標。
    @param[in] y0  線分の始点のY座標。
    @param[in] x1  線分の終点のX座標。
    @param[in] y1  線分の終点のY座標。
    @param[in] clr 線分の色。
     */
    void DrawLine(int32_t x0, int32_t y0, int32_t x1, int32_t y1, const struct Color& clr);


    /*!
    @brief 描画する対象画面を変更し、バッファをクリアします。

           このAPIを呼び出した後の @ref DrawString や @ref DrawRect は、変更した対象画面に描画されます。
           @ref BeginFrame の処理内で描画対象は上画面に戻ります。

    @param[in] target 上画面:true  下画面:false
     */
    void ChangeRenderTarget(bool target);


    void SwapBuffers(void);

    /*!
    @brief メインループの先頭で呼ばれることを想定した関数です。

           @ref BeginFrame を実行する以前に描画を止めたい場合に使用します。
           これを実行すると、いわゆる処理落ちの状態と同様に画面が更新されません。
     */
    void SkipFrame(void);

    /*!
    @brief メインループの先頭で呼ばれることを想定した関数です。

    アプリケーションのメインループ先頭で呼ばれることを期待しています。
    キー入力情報の取得などは、この関数の中で実行しています。
     */
    void BeginFrame(void);


    /*!
    @brief メインループの先頭で呼ばれることを想定した関数です。

    BeginFrame() とほぼ同等ですが、こちらは画面のクリアを行いません。
     */
    void BeginFrameNoRender(void);

    /*!
    @brief メインループの終端で呼ばれることを想定した関数です。

    アプリケーションのメインループ終端で呼ばれることを期待しています。
    描画結果が表示されます。
     */
    void EndFrame(void);


    /*!
    @brief メインループの終端で呼ばれることを想定した関数です。

    アプリケーションのメインループ終端で呼ばれることを期待しています。
    EndFrame() とほぼ同等ですが、こちらはフレームバッファのスワップを実行しません。
     */
    void EndFrameNoRender(void);


    /*!
    @brief フレームカウントを取得します。

    フレームカウントは自動的にインクリメントされます。
     */
    uint32_t GetFrameCount(void) const;


    /*!
    @brief FPS(Frames Per Second)を設定します。

    @param[in] fps FPSを設定します。設定可能な値は、60の約数に限定されます。

    @return 成功すればtrueが返されます。設定できない値が与えられた場合は、falseが返ります。
     */
    bool SetFramePerSecond(int32_t fps);


    /*!
    @brief FPS値を取得します。

    @return 現在のFPS値を取得します。
     */
    int32_t GetFramePerSecond(void) const;


    /*!
    @brief ボタンが押されているかどうかを判定します。
    @param[in] button nn::hid::BUTTON_XXXの論理和
    @return 判定結果が返ります。
     */
    bool IsHold(uint32_t button) const;


    /*!
    @brief ボタンを押した瞬間であったかどうかを判定します。
    @param[in] button nn::hid::BUTTON_XXXの論理和
    @return 判定結果が返ります。
     */
    bool IsTrigger(uint32_t button) const;

    /*!
    @brief ボタンを離した瞬間であったかどうかを判定します。
    @param[in] button nn::hid::BUTTON_XXXの論理和
    @return 判定結果が返ります。
     */
    bool IsRelease(uint32_t button) const;


    /*!
    @brief ペンでタッチパネルに触った瞬間であるかどうかを判定します。
    @return 判定結果が返ります。
     */
    bool IsPenTriggered(void) const;


    /*!
    @brief ペンがタッチパネルに触れているかどうかを判定します。
    @return 判定結果が返ります。
     */
    bool IsPenTouched(void) const;


    /*!
    @brief ペンがタッチパネルから離された瞬間であるかどうかを判定します。
    @return 判定結果が返ります。
     */
    bool IsPenReleased(void) const;


    /*!
    @brief ペンが最後に触れていた点の座標を取得します。
    @param[out] x X座標が格納されます。
    @param[out] y Y座標が格納されます。
    @return 座標の取得に成功すれば、trueが返ります。まだ一度もタッチパネルに触れたことがないようであれば、falseが返されます。
     */
    bool GetLatestPenPosition(uint16_t* x, uint16_t* y) const;


    /*!
    @brief スライドパッドのX方向の入力値を取得します。
    @return X方向の入力値が-1.0～1.0に正規化された値が返されます。
     */
    float GetAnalogX(void);


    /*!
    @brief スライドパッドのY方向の入力値を取得します。
    @return Y方向の入力値が-1.0～1.0に正規化された値が返されます。
     */
    float GetAnalogY(void);


    /*!
    @brief スライドパッドの入力値を取得します。
    @param[out] x X方向の入力値が-1.0～1.0に正規化された値が格納されます。
    @param[out] y Y方向の入力値が-1.0～1.0に正規化された値が格納されます。
     */
    void GetAnalog(float* x, float* y);


    /*!
    @brief 加速度センサの値を取得します。
    @param[out]  x  十字ボタンの左方向。NULLポインタを指定してはいけません。
    @param[out]  y  タッチパネルの実装面方向。NULLポインタを指定してはいけません。
    @param[out]  z  十字ボタンの上方向。NULLポインタを指定してはいけません。
    @return 正常に値を取得できればtrue、さもなければfalseが返されます。
     */
    bool GetAccelerationData(int16_t* x, int16_t* y, int16_t* z) const;

    /*!
    @brief カレントスレッドの情報をプリントします。
     */
    void PrintCurrentThreadInfo(void) const;


    /*!
    @brief カレントスレッドを休眠させます。
    @param[in] msec スレッドを休眠させる時間です。単位はミリ秒です。
     */
    void SleepCurrentThreadMilliseconds(int32_t msec);


    // {Secret}
    // デバイスメモリを取得。
    void* AllocateDeviceMemory(size_t size);


    // {Secret}
    // デバイスメモリを返還。
    void FreeDeviceMemory(void* ptr);


    /*!
    @brief この関数を呼び出したスレッドがメインスレッドであるかどうかを判定します。
    @return 判定結果が返ります。
     */
    bool IsInMainThread(void) const;


    /*!
    @brief  オーディオ入出力をオープンします。
     */
    void OpenAudio(void* pAudioIn, void* pAudioInSystemEvent, void* pAudioOut, void* pAudioOutSystemEvent);


    /*!
    @brief  オーディオ入出力をクローズします。
     */
    void CloseAudio(void* pAudioIn, void* pAudioOut);


    // {Secret}
    // パディングの警告を突き止めるためのコード。
    void Checker(void);


    /*!
    @brief プログラムが起動してからの経過時刻の文字列表現を得ます。
    @param[out] str 結果は00:00:00 (HH::MM::SS)形式で記録されます。十分な長さのバッファを与えて下さい。
     */
    void GetPassedTimeString(char str[]);


    /*!
    @brief CachedPrint の Flush 間隔を指定します。
    @param[in] span 指定したフレーム数おきに Flush します。    
     */
    void SetCachedPrintFlushSpan(uint32_t span);


    /*!
    @brief システム終了要求が届いた状態かどうかを判定します。
     */
    bool IsSystemTerminationRequested(void) const;


    /*!
    @brief プログラムの現在の状態をセットします。[ライブラリ検証用]
    @param[in]  sceneNo     サンプルデモのシーン番号をセットします。
    @param[in]  isConnected 通信中であれば true, それ以外は false をセットします。
    @param[in]  isHost    マスターであれば true, それ以外は false をセットします。
    @param[in]  stationNum  通信しているステーション数をセットします。
    @param[in]  linkLevel   無線通信の電波強度（0-3）をセットします。
     */
    void SetProgramStatus(
        uint32_t sceneNo,
        bool isConnected,
        bool isHost = false,
        uint32_t stationNum = 0,
        uint32_t linkLevel = 0);

    /*!
    @brief プログラムの現在のパラメータひとつをセットします。[ライブラリ検証用]
    @param[in]  paramIndex  セットするパラメータのインデックス値を指定します。
    @param[in]  paramVar    任意のパラメータをセットします。
     */
    void SetProgramParam(
        size_t paramIndex,
        uint32_t paramVar);

#if defined(NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST)
    PiaTest::PiaTestCommand GetLastCommandFromHost();
    static void SetPiaTestCommandCustomCallback(PiaTest::PiaTestCommandCustomCallback cb);
#endif  // end of NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST

private:
    SampleFramework(void);
    SampleFramework(const SampleFramework::Setting& setting);
    void constructorImpl(const SampleFramework::Setting& setting);

    SampleFrameworkImplBase* m_pImpl;

    static SampleFramework* s_pInstance;
    bool m_IsBeginFrame;
};
}
}
}  // end of namespace nns::pia::sampleframework
