﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common.h>


#if NN_PIA_UNDER_DEVELOP
#define PIA_SAMPLE_TRACE(FLAG, ...)                                                                                                    \
    \
{                                                                                                                               \
        nns::pia::AppTrace* pNnPiaCommonAppTrace = nns::pia::AppTrace::GetInstance();                                                  \
        if (pNnPiaCommonAppTrace)                                                                                                      \
        {                                                                                                                              \
            if (pNnPiaCommonAppTrace->GetFlag() & (FLAG))                                                                              \
            {                                                                                                                          \
                pNnPiaCommonAppTrace->Write(__VA_ARGS__);                                                                              \
                pNnPiaCommonAppTrace->Write("\n");                                                                                     \
            }                                                                                                                          \
        }                                                                                                                              \
        else                                                                                                                           \
        {                                                                                                                              \
            PIA_SAMPLE_ASSERTMSG(false, "AppTrace instance is not ready.  Did you call nn::pia::common::AppTrace::CreateInstance()?"); \
        }                                                                                                                              \
    \
}

#else
#define PIA_SAMPLE_TRACE(FLAG, ...) (void)(FLAG)
#endif

namespace nns
{
namespace pia
{

/*!
  @brief      アプリケーションが利用することを想定したトレースのクラスです。

              Releaseビルドでは、トレースのコードは無効化されます。
 */
class AppTrace
{
public:
    /*!
    @brief    AppTraceクラスのインスタンスを得ます（シングルトンパターン）

              事前に @ref CreateInstance でインスタンスを作成していなかった場合は、
              NULLポインタが返されます。

    @return   AppTraceインスタンスへのポインタ。
     */
    static AppTrace* GetInstance(void)
    {
        return &s_Instance;
    }

    /*!
    @brief コンストラクタです。

           AppTrace はシングルトンパターンなので、通常ならコンストラクタは
           privateとすべきですが、訳あって、あえてpublicとしています。
     */
    AppTrace(void)
        : m_padding(0), m_flags(0)
    {
    }

    /*!
      :private
    @brief デストラクタです。
     */
    virtual ~AppTrace(void)
    {
    }

    // {secret}
    // マクロから呼び出されることを想定しているので、アプリケーションコードが
    // このメンバを呼び出すのは推奨しません。
    void Write(const char* pFmt, ...);

    /*!
    @brief    指定したトレースフラグをアクティブに設定します。

              このメソッドを使用して、アクティブに設定したいトレースフラグを追加できます。

    @param[in]    flags    アクティブにするトレースフラグ群の論理和。
    */
    void SetFlag(uint64_t flags)
    {
        m_flags |= flags;
    }

    /*!
    @brief    指定したトレースフラグを非アクティブに設定します。

    @param[in]    flags    非アクティブにするトレースフラグ群の論理和。
    */
    void ClearFlag(uint64_t flags)
    {
        m_flags = m_flags & (~flags);
    }

    /*!
    @brief    アクティブなトレースフラグを返します。

    @return   アクティブ状態にあるトレースフラグ群が返されます。
    */
    uint64_t GetFlag(void) const
    {
        return m_flags;
    }

    /*!
    @brief    特定のフラグが設定されているかどうかを返します。

    @param[in]    flag    問い合わせるトレースフラグ。

    @return   特定のフラグが設定されていればtrue、そうでなければfalse。
    */
    bool IsFlagSet(uint64_t flag) const
    {
        return (m_flags & flag) == flag;
    }

    /*!
      @brief  トレースフラグを全てクリアします。
     */
    void ClearFlag(void)
    {
        m_flags = 0;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref common::TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

protected:
private:
    /*!
    @brief コピーコンストラクタは封印します。
     */
    AppTrace(const AppTrace&);

    /*!
    @brief 代入演算子は封印します。
     */
    AppTrace& operator=(const AppTrace&);

    // パディング。
    uint32_t m_padding;

    // このメンバ変数が64ビットであるせいで、オブジェクト構築時には
    // 8バイトアライメントが必要。
    uint64_t m_flags;

    static AppTrace s_Instance;
};
}
}  // end of namespace nns::pia
