﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Common_Setup.h"

#include <nn/pia/common.h>

namespace
{
#if NN_PIA_ENABLE_CACHEDPRINT
// ログ出力をキャッシュしておくバッファのサイズ
#ifdef PIA_PROFILING_TEST_BUILD
const uint32_t CachedPrintBufferSize = 64 * 1024;
#else
#if NN_PIA_CTR
const uint32_t CachedPrintBufferSize = 64 * 1024;
#else
const uint32_t CachedPrintBufferSize = 512 * 1024;
#endif
#endif
#endif

// 例外をフックするためのハンドラ
#if NN_PIA_CTR || NN_PIA_WIN || NN_PIA_NINTENDOSDK || NN_PIA_A || NN_PIA_B
void setExceptionHandlers()
{
    // Not implemented
}
#elif NN_PIA_CAFE
OSExceptionCallback s_PrevDsiExceptionCallback = NULL;
OSExceptionCallback s_PrevIsiExceptionCallback = NULL;
OSExceptionCallback s_PrevProgramExceptionCallback = NULL;


void execCachedPrintFlush()
{
    // CachedPrint のインスタンスが存在するようなら、フラッシュ処理を実行する。
    nn::pia::common::CachedPrint* p = nn::pia::common::CachedPrint::GetInstance();
    if (PIA_IS_VALID_POINTER(p))
    {
        p->Flush();
    }
}

// 例外を処理しなかった場合は FALSE を返す約束。
#define CALL_PREV_CALLBACK(cb)         \
    if (PIA_IS_VALID_POINTER(cb))      \
    {                                  \
        return cb(interruptedContext); \
    }                                  \
    return FALSE

BOOL dsiExceptionHandler(OSContext* interruptedContext)
{
    execCachedPrintFlush();
    CALL_PREV_CALLBACK(s_PrevDsiExceptionCallback);
}

BOOL isiExceptionHandler(OSContext* interruptedContext)
{
    execCachedPrintFlush();
    CALL_PREV_CALLBACK(s_PrevIsiExceptionCallback);
}

BOOL programExceptionHandler(OSContext* interruptedContext)
{
    execCachedPrintFlush();
    CALL_PREV_CALLBACK(s_PrevProgramExceptionCallback);
}

void setExceptionHandlers()
{
    // 例外ハンドラのフック(Cafe)。
    s_PrevDsiExceptionCallback = OSSetExceptionCallback(OS_EXCEPTION_DSI, dsiExceptionHandler);
    s_PrevIsiExceptionCallback = OSSetExceptionCallback(OS_EXCEPTION_ISI, isiExceptionHandler);
    s_PrevProgramExceptionCallback = OSSetExceptionCallback(OS_EXCEPTION_PROGRAM, programExceptionHandler);
}
#endif  // end of NN_PIA_CAFE


}  // end of anonymous namespace

namespace nns
{
namespace pia
{

nn::pia::Result Common_InitializePiaCommon(
    uint8_t* pPiaHeapBuffer,
    uint32_t szPiaHeapBufferSize,
    uint64_t traceFlags,
    bool enableCachedPrint,
    bool enableExceptionHook)
{
    NN_PIA_UNUSED_IN_RELEASE(enableCachedPrint);
    PIA_SAMPLE_PRINT("Common_InitializePiaCommon:traceFlags:0x%016llx, enableCachedPrint:%d, enableExceptionHook:%d\n", traceFlags, enableCachedPrint, enableExceptionHook);

    if (enableExceptionHook)
    {
        setExceptionHandlers();
    }

    PIA_SAMPLE_PRINT("common::Initialize() is called.\n");
    nn::pia::Result result;
    result = nn::pia::common::Initialize(pPiaHeapBuffer, szPiaHeapBufferSize);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    PIA_SAMPLE_PRINT("common::BeginSetup() is called.\n");
    result = nn::pia::common::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

#if NN_PIA_ENABLE_CACHEDPRINT
    if (enableCachedPrint)
    {
        PIA_SAMPLE_PRINT("common::CachedPrint::CreateInstance() is called.\n");
        result = nn::pia::common::CachedPrint::CreateInstance(CachedPrintBufferSize);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }
#endif // NN_PIA_ENABLE_CACHEDPRINT

    PIA_SAMPLE_PRINT("common::Trace::CreateInstance() is called.\n");
    result = nn::pia::common::Trace::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);


    PIA_SAMPLE_PRINT("common::Trace::SetFlag() is called.\n");
#if PIA_TEST_CODE
    uint64_t traceFlagsForPiaTest =
        traceFlags |
        nn::pia::common::TraceFlagCommon |
        nn::pia::common::TraceFlagCommonSocket |
        nn::pia::common::TraceFlagLocal |
        nn::pia::common::TraceFlagTransport |
        //        nn::pia::common::TRACE_FLAG_TRANSPORT_PACKET_SEND |
        //        nn::pia::common::TRACE_FLAG_TRANSPORT_PACKET_RECEIVE |
        nn::pia::common::TraceFlagSession |
        nn::pia::common::TraceFlagSessionConnect |
        nn::pia::common::TraceFlagSessionLeave |
        nn::pia::common::TraceFlagSessionUpdate |
        nn::pia::common::TraceFlagSessionMigration |
        nn::pia::common::TraceFlagSessionJoint |
        nn::pia::common::TraceFlagInet |
        nn::pia::common::TraceFlagInetNat |
        nn::pia::common::TraceFlagInetNatDetail |
        nn::pia::common::TraceFlagInetNatProbe |
        nn::pia::common::TraceFlagAlways;
#if 1
    nn::pia::common::Trace::GetInstance()->SetFlag(traceFlagsForPiaTest);
#else
    // n1769 ログ／トレース出力による PC 側の CPU 負荷の増大を調査するため、試験的にトレースを Off にします。
    nn::pia::common::Trace::GetInstance()->SetFlag(0);
#endif
#else
    nn::pia::common::Trace::GetInstance()->SetFlag(traceFlags);
#endif
    PIA_SAMPLE_PRINT("common::Trace::SetPrintTime() is called.\n");
    nn::pia::common::Trace::GetInstance()->SetPrintTime(true);

    PIA_SAMPLE_PRINT("common::Scheduler::CreateInstance() is called.\n");
    result = nn::pia::common::Scheduler::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    PIA_SAMPLE_PRINT("common::WatermarkManager::CreateInstance() is called.\n");
    result = nn::pia::common::WatermarkManager::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    PIA_SAMPLE_PRINT("common::WatermarkManager::EnableAllWatermark() is called.\n");
    nn::pia::common::WatermarkManager::GetInstance()->EnableAllWatermark();

    PIA_SAMPLE_PRINT("common::EndSetup() is called.\n");
    result = nn::pia::common::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // メモリ消費量をプリント
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Common_InitializePiaCommon:Pia current memory usage: %d\n", nn::pia::common::GetMemoryUsage());
    return result;
}

void Common_FinalizePiaCommon()
{
    PIA_SAMPLE_PRINT("Common_FinalizePiaCommon\n");

    // PiaではFinalize、DestroyInstanceは常に成功します
    nn::pia::common::WatermarkManager::DestroyInstance();
    nn::pia::common::Scheduler::DestroyInstance();
    nn::pia::common::Trace::DestroyInstance();
#if NN_PIA_ENABLE_CACHEDPRINT
    if (nn::pia::common::CachedPrint::GetInstance() != NULL)
    {
        nn::pia::common::CachedPrint::GetInstance()->Flush();
        nn::pia::common::CachedPrint::DestroyInstance();
    }
#endif // NN_PIA_ENABLE_CACHEDPRINT
    nn::pia::common::Finalize();
}
}
}  // end of namespace nns::pia
