﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "ErrorHandling.h"
#include "SampleLib.h"

#include <nn/pia/common.h>

#include <cstdlib>
#include <string.h>

namespace
{
const char* StringPiaModuleType[] =
    {
        "Reserved",
        "Pia",
        "PiaCommon",
        "PiaLocal",
        "PiaTransport",
        "PiaSync",
        "PiaInet",
        "PiaSession",
        "NotDefined"
    };

const char* StringPiaErrorSeverity[] =
    {
        "NO_PROBLEM",
        "PROGRAMMING_ERROR",
        "IGNORABLE",
        "SHOULD_BE_HANDLED",
        "MUST_BE_REINITIALIZED",
        "KickoutReason_Unknown"
    };

const char* StringPiaErrorDescription[] =
    {
        "AllocationFailed",
        "AlreadyInitialized",
        "BrokenData",
        "BufferShortage",
        "Cancelled",
        "ConnectionIsLost",
        "InvalidArgument",
        "InvalidState",
        "NoData",
        "NotFound",
        "NotInpremented",
        "NotInitialized",
        "BufferIsFull",
        "TimeOut"
    };

const char* StringPiaCommonErrorDescription[] =
    {
        "NotInitialized",
        "MemoryLeak",
        "InvalidArgument",
        "AlreadyInitialized",
        "BufferIsFull",
        "InternalError",
        "InvalidState"
    };

const char* StringPiaLocalErrorDescription[] =
    {
        "NotInitialized",
        "InvalidArgument",
        "AlreadyInitialized",
        "InvalidState",
        "NotFoundNetwork",
        "InvalidNode",
        "NothingPacket",
        "NegligibleFault",
        "InvalidConnection",
        "HostMigrationFailure"
    };

const char* StringPiaTransportErrorDescription[] =
    {
        "Description_Success",
        "ConnectionFailed",
        "CreateStationFailed",
        "IncompatibleFormat"
    };

const char* StringPiaSyncErrorDescription[] =
    {
        "Base",
        "NotInitialized",
        "InvalidState",
        "InvalidArgument",
        "DataIsNotArrivedYet",
        "OutOfResource",
        "SynchronizationIsLost",
        "FailedToSynchronize",
        "InvalidDevice",
        "ToBeImplemented"
    };
}  // end of anonymous namespace

namespace nns
{
namespace pia
{

nn::pia::Result ErrorHandling::s_result = nn::pia::ResultSuccess();

char ErrorHandling::s_filename[MaxFilenameLength] = {};
uint32_t ErrorHandling::s_lineNo = 0;

bool ErrorHandling::s_bInitialized = false;

// 初期化処理。
void ErrorHandling::Initialize(void)
{
    if (!s_bInitialized)
    {
        s_bInitialized = true;
    }
}


// 終了処理。
void ErrorHandling::Finalize(void)
{
    if (s_bInitialized)
    {
        s_bInitialized = false;
    }
}


// エラーが検出されたか？
bool ErrorHandling::IsErrorDetected(void)
{
    PIA_SAMPLE_ASSERT(s_bInitialized);

    return s_result.IsFailure();
}


nn::pia::Result ErrorHandling::GetResultCode(void)
{
    return s_result;
}


const char* ErrorHandling::GetFilename(void)
{
    return s_filename;
}


uint32_t ErrorHandling::GetLineNo(void)
{
    return s_lineNo;
}


#if NN_PIA_CTR || NN_PIA_CAFE || NN_PIA_NINTENDOSDK
// SDK 側で用意している nn::Result 型をプリント
void ErrorHandling::PrintSdkResult(nn::Result result)
{
#if !NN_PIA_NINTENDOSDK
    PIA_REPORT(nn::pia::common::Report::LevelError, " Value:    0x%08x", result.GetValue());
    PIA_REPORT(nn::pia::common::Report::LevelError, " Level:       %4d", result.GetLevel());
    PIA_REPORT(nn::pia::common::Report::LevelError, " Summary:     %4d", result.GetSummary());
#else
    PIA_REPORT(nn::pia::common::Report::LevelError, " Value:    0x%08x", result.GetInnerValueForDebug());
#endif
    PIA_REPORT(nn::pia::common::Report::LevelError, " Module:      %4d", result.GetModule());
    PIA_REPORT(nn::pia::common::Report::LevelError, " Description: %4d", result.GetDescription());
    NN_PIA_UNUSED_IN_RELEASE(result);
}
#endif


// Pia 側で用意している nn::pia::Result 型をプリント
void ErrorHandling::PrintResult(nn::pia::Result result)
{
    NN_PIA_UNUSED(StringPiaModuleType);
    NN_PIA_UNUSED(StringPiaErrorSeverity);
    NN_PIA_UNUSED(StringPiaErrorDescription);
    NN_PIA_UNUSED(StringPiaCommonErrorDescription);
    NN_PIA_UNUSED(StringPiaLocalErrorDescription);
    NN_PIA_UNUSED(StringPiaTransportErrorDescription);
    NN_PIA_UNUSED(StringPiaSyncErrorDescription);

    NN_PIA_UNUSED(result);
// n1769 nn::pia::Result 側に ToString() 的なメンバ関数を用意しておき、
// ここでプリントするのがよさそう。まだ用意していないので、無効化しておきます。
#if 0
    PIA_SAMPLE_PRINT(result.ToString());
#endif
}


// デバッグ出力に本クラスで保持しているリザルトコードの文字列表現をプリント。
void ErrorHandling::Print(void)
{
    PIA_REPORT(nn::pia::common::Report::LevelError, "\n----- ErrorHandling Start -----");
    PIA_REPORT(nn::pia::common::Report::LevelError, " Filename:    %s", GetFilename());
    PIA_REPORT(nn::pia::common::Report::LevelError, " Line:        %d", GetLineNo());

    PrintResult(GetResultCode());

    PIA_REPORT(nn::pia::common::Report::LevelError, "----- ErrorHandling End -----\n");
}


// 本クラスで保持しているnn::pia::Result型の値をクリアする。
void ErrorHandling::Clear(void)
{
    s_result = nn::pia::ResultSuccess();
    s_filename[0] = '\0';
    s_lineNo = 0;
}


// n1769
// リザルトコードを強引にセットする。わざとエラーを発生させ、
// エラー処理シーケンスが正常に機能するかどうかをチェックするために用意した。
void ErrorHandling::SetResultCode(nn::pia::Result result)
{
    s_result = result;
}
}
}  // end of namespace nns::pia
