﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Types.h>
#include "Assert.h"

namespace nns
{
namespace pia
{

// StationId を連番のインデックス値に変換するクラスです。
template <size_t N>
class Indexer
{
public:
    static const size_t InvalidIndex = 0xffffffff;
    Indexer()
    {
        for (size_t i = 0; i < N; ++i)
        {
            m_aStationId[i] = nn::pia::StationIdInvalid;
        }
    }

    // 新しい StationId を登録します。
    // 登録すると、その StationId に対応するインデックス値を GetIndex() で取得できるようになります。
    bool Add(nn::pia::StationId id)
    {
        PIA_SAMPLE_ASSERT(id != nn::pia::StationIdInvalid);

        for (size_t i = 0; i < N; ++i)
        {
            if (m_aStationId[i] == nn::pia::StationIdInvalid)
            {
                m_aStationId[i] = id;
                return true;
            }
        }

        PIA_SAMPLE_ASSERTMSG(false, "No more space!");
        return false;
    }

    // 登録してあった StationId を削除します。
    bool Remove(nn::pia::StationId id)
    {
        PIA_SAMPLE_ASSERT(id != nn::pia::StationIdInvalid);

        for (size_t i = 0; i < N; ++i)
        {
            if (m_aStationId[i] == id)
            {
                m_aStationId[i] = nn::pia::StationIdInvalid;
                return true;
            }
        }

        PIA_SAMPLE_ASSERTMSG(false, "StationId is not found!");
        return false;
    }

    // StationId からインデックス値を取得します。
    // 正しく登録されていれば、0 以上、N 未満の値が返されます。
    // Add で登録されていない StationId が指定された場合は InvalidIndex が返されます。
    size_t GetIndex(nn::pia::StationId id) const
    {
        for (size_t i = 0; i < N; ++i)
        {
            if (m_aStationId[i] == id)
            {
                return i;
            }
        }

        // Not found!
        return InvalidIndex;
    }


    // インデックス値から StationId を取得します。
    // 登録されていなければ、StationIdInvalid が返されます。
    nn::pia::StationId GetStationId(size_t idx)
    {
        PIA_SAMPLE_ASSERTMSG(idx < N, "Indexer::GetStationId() called, but idx is out of range.");

        return m_aStationId[idx];
    }


    // 有効なインデックスが何個あるかを返します。
    size_t GetValidIndexNum() const
    {
        size_t sum = 0;
        for (size_t i = 0; i < N; ++i)
        {
            if (m_aStationId[i] != nn::pia::StationIdInvalid)
            {
                ++sum;
            }
        }
        return sum;
    }

private:
    nn::pia::StationId m_aStationId[N];
};
}
}  // end of namespace nns::pia
