﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include "SampleLib/Local_Setup.h"

#if NNS_PIA_ENABLE_LOCAL

#include "SampleLib/Basic_Setup.h"

#define ENABLE_WAIT_STARTUP 0

namespace
{
#if NN_PIA_CTR
// UDS 受信用バッファ
// 4096 バイト整合されたバッファで 4096 の倍数である必要があります。
const uint32_t UdsReceiveBufferSize = 4096 * 20;

// UDS スキャン用バッファ
// ネットワーク1つあたり 1KB 程度必要
const uint32_t UdsScanBufferSize = 5 * 1024;
nn::pia::local::UdsNetworkFactory s_NetworkFactory;

const char TestWirelessCryptoKey[] = "aaabbccc0123456789!";

uint32_t s_UdsTitleId = 0;
uint8_t s_UdsSubId = 0;

nn::pia::local::UdsNetworkDescription s_Description;

#elif NN_PIA_NINTENDOSDK

nn::pia::local::LdnNetworkFactory s_NetworkFactory;

const char TestWirelessCryptoKey[] = "aaabbccc0123456789!";

uint64_t s_LocalCommunicationId = 0x123456789ABCDEF0ULL;
uint16_t s_SceneId = 0x0001;
int16_t s_LocalChannel = 11;

nn::pia::local::LdnNetworkDescription s_Description;

#endif

bool s_IsLocalCommunicationInitialized = false;
}

namespace nns
{
namespace pia
{

#if NN_PIA_CTR
uint32_t Local_GetTitleId()
{
    return s_UdsTitleId;
}

uint8_t Local_GetSubId()
{
    return s_UdsSubId;
}
#elif NN_PIA_NINTENDOSDK
uint64_t Local_GetLocalCommunicationId()
{
    return s_LocalCommunicationId;
}

uint16_t Local_GetSceneId()
{
    return s_SceneId;
}
#endif

nn::pia::Result Local_InitializePiaLocalCore(const LocalSetting& setting, const nn::pia::local::LocalNetworkSetting& networkSetting)
{
    PIA_SAMPLE_PRINT("Local_InitializePiaLocal\n");

#if NN_PIA_CTR
    s_UdsTitleId = static_cast<uint32_t>(setting.idKey1);
    s_UdsSubId = static_cast<uint8_t>(setting.idKey2);
#elif NN_PIA_NINTENDOSDK
    s_LocalCommunicationId = setting.idKey1;
    s_SceneId = setting.idKey2;
#endif

    nn::pia::Result result;

    result = nn::pia::local::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::local::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::local::LocalFacade::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::local::LocalFacade::GetInstance()->Initialize(networkSetting);
    if (result.IsFailure())
    {
        // 失敗しても、nn::pia::local::EndSetup は呼ぶ
        nn::pia::Result resultBuf = nn::pia::local::EndSetup();
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(resultBuf);

        return result;
    }
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::local::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // メモリ消費量をプリント
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Local_InitializePiaLocal:Pia current memory usage: %d\n", nn::pia::common::GetMemoryUsage());

    return result;
}

nn::pia::Result Local_InitializePiaLocal(const LocalSetting& setting)
{
#if NN_PIA_CTR
    // LocalFacade の設定用構造体
    nn::pia::local::UdsNetworkSetting networkSetting;

    networkSetting.isAroundNetworkSearchEnabled = setting.isAroundNetworkSearchEnabled;  // アラウンドネットワークサーチ機能の有効／無効切替
    networkSetting.pUserName = NULL;                                                     // ユーザー名
    networkSetting.receiveBufferSize = UdsReceiveBufferSize;                             // 受信用バッファサイズ
    networkSetting.pReceiveBuffer = NULL;                                                // 受信用バッファ。NULL の場合はライブラリ内で確保します。
    networkSetting.receiveOption = nn::uds::NO_WAIT;                                     // 受信時オプション。PiaSession を使用する場合は、基本的に NO_WAIT を指定する必要があります。
    networkSetting.scanBufferSize = UdsScanBufferSize;                                   // スキャン用バッファサイズ
    networkSetting.sendOption = setting.sendOption;                                      // 送信時オプション
    networkSetting.applicationVersion = 1;                                               // アプリケーションバージョン

    return Local_InitializePiaLocalCore(setting, networkSetting);
#elif NN_PIA_NINTENDOSDK
    // LocalFacade の設定用構造体
    nn::pia::local::LdnNetworkSetting networkSetting;
    networkSetting.isAroundNetworkSearchEnabled = setting.isAroundNetworkSearchEnabled;  // アラウンドネットワークサーチ機能の有効／無効切替
    networkSetting.localCommunicationVersion = 0;                                        // ローカル通信バージョン
    networkSetting.isEncryptionDisabledForDebug = false;                                 // デバッグ用の暗号化無効設定

    return Local_InitializePiaLocalCore(setting, networkSetting);
#else
    return nn::pia::ResultInvalidState();
#endif
}

void Local_FinalizePiaLocal()
{
    PIA_SAMPLE_PRINT("Local_FinalizePiaLocal\n");

    // PiaではFinalize、DestroyInstanceは常に成功します
    if (PIA_IS_VALID_POINTER(nn::pia::local::LocalFacade::GetInstance()))
    {
        nn::pia::local::LocalFacade::GetInstance()->Finalize();
    }
    nn::pia::local::LocalFacade::DestroyInstance();
    nn::pia::local::Finalize();
}

bool Local_InitializeLocalCommunication()
{
    PIA_SAMPLE_PRINT("Local_InitializeLocalCommunication called\n");

    PIA_SAMPLE_ASSERT(s_IsLocalCommunicationInitialized == false);

#if NN_PIA_NINTENDOSDK
    nn::ldn::State ldnState = nn::ldn::GetState();
    PIA_SAMPLE_ASSERTMSG(ldnState == nn::ldn::State_None, "ldnState:%d", ldnState);
    PIA_SAMPLE_PRINT("nn::ldn::Initialize begin\n");
    NN_PIA_UNUSED_IN_RELEASE(ldnState);

    nn::pia::common::Time beginTime = nn::pia::common::Time::GetTime();
    nn::Result sdkResult = nn::ldn::Initialize();
    nn::pia::common::Time endTime = nn::pia::common::Time::GetTime();
    PIA_SAMPLE_PRINT("nn::ldn::Initialize end (%d ms)\n", (endTime - beginTime).GetMSec());
    NN_PIA_UNUSED_IN_RELEASE(beginTime);
    NN_PIA_UNUSED_IN_RELEASE(endTime);
    if (sdkResult.IsFailure())
    {
        PIA_SAMPLE_PRINT("Local_InitializeLocalCommunication failed (result:0x%08x)\n", sdkResult.GetInnerValueForDebug());
        return false;
    }
#else
#endif

#if ENABLE_WAIT_STARTUP
    // Startup() 前無線 off チェック要待機シーン
    for (int i = 0; i < 180; ++i)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "Wait Session::Startup(). %d", i);

        sampleframework::SampleFramework::GetInstance()->EndFrame();
    }
#endif

    s_IsLocalCommunicationInitialized = true;
    return true;
}

void Local_FinalizeLocalCommunication()
{
    PIA_SAMPLE_PRINT("Local_FinalizeLocalCommunication called\n");

    if (s_IsLocalCommunicationInitialized)
    {
#if NN_PIA_NINTENDOSDK
        nn::ldn::State ldnState = nn::ldn::GetState();
        PIA_SAMPLE_ASSERTMSG(ldnState != nn::ldn::State_None, "ldnState:%d", ldnState);
        PIA_SAMPLE_PRINT("nn::ldn::Finalize begin\n");
        NN_PIA_UNUSED_IN_RELEASE(ldnState);

        nn::pia::common::Time beginTime = nn::pia::common::Time::GetTime();
        nn::ldn::Finalize();
        nn::pia::common::Time endTime = nn::pia::common::Time::GetTime();
        PIA_SAMPLE_PRINT("nn::ldn::Finalize end (%d ms)\n", (endTime - beginTime).GetMSec());
        NN_PIA_UNUSED_IN_RELEASE(beginTime);
        NN_PIA_UNUSED_IN_RELEASE(endTime);
#else
#endif
        s_IsLocalCommunicationInitialized = false;
    }
}

bool Local_CreateNetwork(uint16_t maxStations)
{
#if NN_PIA_CTR

    uint32_t communicationId = nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(Local_GetTitleId());
    nn::pia::local::UdsCreateNetworkSetting createNetworkSetting;
    createNetworkSetting.subId = Local_GetSubId();
    createNetworkSetting.localCommunicationId = communicationId;
    createNetworkSetting.maxEntry = maxStations;
    createNetworkSetting.channel = 0;
    createNetworkSetting.applicationDataSize = 0;
    memcpy(createNetworkSetting.wirelessCryptoKey, const_cast<char*>(TestWirelessCryptoKey), sizeof(TestWirelessCryptoKey));
    createNetworkSetting.wirelessCryptoKeyLength = sizeof(TestWirelessCryptoKey);
#elif NN_PIA_NINTENDOSDK

    nn::pia::local::LdnCreateNetworkSetting createNetworkSetting;
    createNetworkSetting.localCommunicationId = Local_GetLocalCommunicationId();
    createNetworkSetting.sceneId = Local_GetSceneId();
    createNetworkSetting.maxEntry = maxStations;
    createNetworkSetting.applicationDataSize = 0;
    memcpy(createNetworkSetting.wirelessCryptoKey, const_cast<char*>(TestWirelessCryptoKey), sizeof(TestWirelessCryptoKey));
    createNetworkSetting.wirelessCryptoKeySize = sizeof(TestWirelessCryptoKey);
    createNetworkSetting.channel = s_LocalChannel;
#endif
    return Local_CreateNetwork(&createNetworkSetting);
}

bool Local_CreateNetwork(nn::pia::local::LocalCreateNetworkSetting* pCreateNetworkSetting)
{
    nn::pia::Result r;

    sampleframework::SampleFramework::GetInstance()->BeginFrame();
    nn::pia::common::Scheduler::GetInstance()->Dispatch();
    r = nn::pia::local::LocalFacade::GetInstance()->CreateNetworkAsync(pCreateNetworkSetting);

    sampleframework::SampleFramework::GetInstance()->EndFrame();

    if (r.IsFailure())
    {
        PIA_SAMPLE_PRINT("LocalFacade::CreateNetworkAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(r);
        return false;
    }

    // CreateNetworkAsync の完了待ち
    for (;;)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // ディスパッチ処理
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "creating network...");

        sampleframework::SampleFramework::GetInstance()->EndFrame();

        if (nn::pia::local::LocalFacade::GetInstance()->IsCreateNetworkCompleted())
        {
            r = nn::pia::local::LocalFacade::GetInstance()->GetCreateNetworkResult();
            if (r.IsFailure())
            {
                PIA_SAMPLE_PRINT("LocalFacade::CreateNetworkAsync failed\n");
                PIA_SAMPLE_PRINT_RESULT(r);
                return false;
            }
            else
            {
                PIA_SAMPLE_PRINT("LocalFacade::CreateNetworkAsync success\n");
                break;
            }
        }
    }

    return true;
}

bool Local_SearchAndConnectToNetwork(nn::pia::transport::StationConnectionInfo* pStationConnectionInfo)
{
    nn::pia::Result r;
#if NN_PIA_CTR
    uint32_t gatheringNum = 0;

    nn::pia::local::UdsScanNetworkSetting scanNetworkSetting;
    scanNetworkSetting.localCommunicationId = nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(Local_GetTitleId());
    scanNetworkSetting.subId = Local_GetSubId();
#elif NN_PIA_NINTENDOSDK
    uint32_t gatheringNum = 0;

    nn::pia::local::LdnScanNetworkSetting scanNetworkSetting;
    scanNetworkSetting.localCommunicationId = Local_GetLocalCommunicationId();
    scanNetworkSetting.sceneId = Local_GetSceneId();
    scanNetworkSetting.channel = s_LocalChannel;
#endif

    uint32_t SearchRetryCount = 0;
    const uint32_t SearchRetryCountMax = 10;

    // ネットワーク探索
    bool isAsyncStarted = false;
    for (;;)
    {
        bool isExit = false;

        sampleframework::SampleFramework::GetInstance()->BeginFrame();
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        if (sampleframework::SampleFramework::GetInstance()->IsTrigger(sampleframework::ButtonStart))
        {
            // START ボタンが押されたら探索を中断します。
            isExit = true;
            if (isAsyncStarted)
            {
                r = nn::pia::local::LocalFacade::GetInstance()->CancelScanNetworkAsync();
                PIA_SAMPLE_PRINT("Local_SearchAndConnectToNetwork canceled\n");
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
            }
        }

        if (isExit == false)
        {
            if (isAsyncStarted == false)
            {
                r = nn::pia::local::LocalFacade::GetInstance()->ScanNetworkAsync(&scanNetworkSetting);
                if (r.IsSuccess())
                {
                    isAsyncStarted = true;
                }
            }
            else if (nn::pia::local::LocalFacade::GetInstance()->IsScanNetworkCompleted())
            {
                r = nn::pia::local::LocalFacade::GetInstance()->GetScanNetworkResult();
                if (r.IsSuccess())
                {
                    r = nn::pia::local::LocalFacade::GetInstance()->GetNetworkDescriptionList(&s_Description, &gatheringNum, 1);
                    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
                    if (gatheringNum > 0)
                    {
                        // ネットワークが見つかりました。
                        isExit = true;
                    }
                    else
                    {
                        ++SearchRetryCount;
                        if (SearchRetryCount > SearchRetryCountMax)
                        {
                            PIA_SAMPLE_PRINT("Local_SearchAndConnectToNetwork timeout\n");
                            isExit = true;
                        }
                    }
                }
                else
                {
                    gatheringNum = 0;

                    if (r == nn::pia::ResultInvalidConnection())
                    {
                        // LocalFacade の再初期化が必要
                        isExit = true;
                    }
                }

                isAsyncStarted = false;
            }
        }

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "searching Local network...");
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 30, "START: cancel");
        sampleframework::SampleFramework::GetInstance()->EndFrame();

        if (isExit)
        {
            break;
        }
    }

    // 非同期処理が完了するのを待つ
    while (nn::pia::local::LocalFacade::GetInstance()->IsScanNetworkCompleted() == false)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "waiting ScanNetwork completed...");
        sampleframework::SampleFramework::GetInstance()->EndFrame();
    }

    if (gatheringNum <= 0)
    {
        // ネットワークが見つかっていないのに抜けるのは失敗
        return false;
    }

// ネットワーク参加
#if NN_PIA_CTR

    nn::pia::local::UdsConnectNetworkSetting connectNetworkSetting;
    memcpy(connectNetworkSetting.wirelessCryptoKey, const_cast<char*>(TestWirelessCryptoKey), sizeof(TestWirelessCryptoKey));
    connectNetworkSetting.wirelessCryptoKeyLength = sizeof(TestWirelessCryptoKey);
    connectNetworkSetting.pLocalNetworkDescription = &s_Description;
#elif NN_PIA_NINTENDOSDK

    nn::pia::local::LdnConnectNetworkSetting connectNetworkSetting;
    memcpy(connectNetworkSetting.wirelessCryptoKey, const_cast<char*>(TestWirelessCryptoKey), sizeof(TestWirelessCryptoKey));
    connectNetworkSetting.wirelessCryptoKeySize = sizeof(TestWirelessCryptoKey);
    connectNetworkSetting.pLocalNetworkDescription = &s_Description;
#endif
    bool isSuccess = Local_ConnectNetwork(&connectNetworkSetting);
    if (isSuccess == false)
    {
        return false;
    }

    // LocalFacade の StartupForMesh（このAPIを呼び出す時点で、ローカルネットワークが構築されている必要があります）
    if (nn::pia::local::LocalFacade::GetInstance()->IsConnected() == false)
    {
        PIA_SAMPLE_PRINT("LocalFacade::IsConnected() == false\n");
        return false;
    }
    r = nn::pia::local::LocalFacade::GetInstance()->StartupForMesh();
    if (r.IsFailure())
    {
        PIA_SAMPLE_PRINT("LocalFacade::StartupForMesh failed\n");
        PIA_SAMPLE_PRINT_RESULT(r);
        PIA_SAMPLE_ASSERT(false);
        return false;
    }

    //セッションホストの StationConnectionInfo を取得
    r = nn::pia::local::LocalFacade::GetInstance()->GetHostStationConnectionInfo(pStationConnectionInfo);
    if (r.IsFailure())
    {
        PIA_SAMPLE_PRINT("LocalFacade::GetHostStationConnectionInfo failed\n");
        PIA_SAMPLE_PRINT_RESULT(r);
        PIA_SAMPLE_ASSERT(false);
        return false;
    }
    return true;
}

bool Local_ScanNetwork(nn::pia::local::LocalScanNetworkSetting* pScanNetworkSetting)
{
    nn::pia::Result r;
    r = nn::pia::local::LocalFacade::GetInstance()->ScanNetworkAsync(pScanNetworkSetting);
    if (r.IsFailure())
    {
        PIA_SAMPLE_PRINT("ScanNetworkAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(r);
        return false;
    }

    while (nn::pia::local::LocalFacade::GetInstance()->IsScanNetworkCompleted() == false)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        if (sampleframework::SampleFramework::GetInstance()->IsTrigger(sampleframework::ButtonB))
        {
            // ネットワーク探索をキャンセル
            r = nn::pia::local::LocalFacade::GetInstance()->CancelScanNetworkAsync();
            if (r.IsSuccess())
            {
                PIA_SAMPLE_PRINT("CancelScanNetworkAsync Success\n");
            }
            else
            {
                PIA_SAMPLE_PRINT("CancelScanNetworkAsync Failure\n");
            }
        }
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "searching...");
        sampleframework::SampleFramework::GetInstance()->EndFrame();
    }

    r = nn::pia::local::LocalFacade::GetInstance()->GetScanNetworkResult();
    if (r.IsFailure())
    {
        PIA_SAMPLE_PRINT("ScanNetworkAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(r);
        return false;
    }

    return true;
}

bool Local_ConnectNetwork(nn::pia::local::LocalConnectNetworkSetting* pConnectNetworkSetting)
{
    nn::pia::Result r;

    sampleframework::SampleFramework::GetInstance()->BeginFrame();
    nn::pia::common::Scheduler::GetInstance()->Dispatch();
    r = nn::pia::local::LocalFacade::GetInstance()->ConnectNetworkAsync(pConnectNetworkSetting);
    sampleframework::SampleFramework::GetInstance()->EndFrame();

    if (r.IsFailure())
    {
        PIA_SAMPLE_PRINT("ConnectNetworkAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(r);
        return false;
    }

    while (nn::pia::local::LocalFacade::GetInstance()->IsConnectNetworkCompleted() == false)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        if (sampleframework::SampleFramework::GetInstance()->IsTrigger(sampleframework::ButtonB))
        {
            // ネットワーク参加をキャンセル
            r = nn::pia::local::LocalFacade::GetInstance()->CancelConnectNetworkAsync();
            if (r.IsSuccess())
            {
                PIA_SAMPLE_PRINT("CancelConnectNetworkAsync Success\n");
            }
            else
            {
                PIA_SAMPLE_PRINT("CancelConnectNetworkAsync Failure\n");
            }
        }
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "connecting...");
        sampleframework::SampleFramework::GetInstance()->EndFrame();
    }

    r = nn::pia::local::LocalFacade::GetInstance()->GetConnectNetworkResult();
    if (r.IsFailure())
    {
        PIA_SAMPLE_PRINT("ConnectNetworkAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(r);
        return false;
    }

    return true;
}

bool Local_DisconnectNetwork(bool isMigration)
{
    if (!PIA_IS_VALID_POINTER(nn::pia::local::LocalFacade::GetInstance()))
    {
        return false;
    }

    if (!nn::pia::local::LocalFacade::GetInstance()->IsConnected())
    {
        return false;
    }

    if (nn::pia::local::LocalFacade::GetInstance()->IsHost())  //ホストの場合
    {
        PIA_SAMPLE_PRINT("IsHost\n");
        if (isMigration)
        {
            // 手動ホストマイグレーションを行う
            nn::pia::Result result = nn::pia::local::LocalFacade::GetInstance()->DisconnectNetworkAsync();
            if (result.IsSuccess())
            {
                while (nn::pia::local::LocalFacade::GetInstance()->IsDisconnectNetworkCompleted() == false)
                {
                    sampleframework::SampleFramework::GetInstance()->BeginFrame();
                    nn::pia::common::Scheduler::GetInstance()->Dispatch();
                    sampleframework::SampleFramework::GetInstance()->EndFrame();
                }
                result = nn::pia::local::LocalFacade::GetInstance()->GetDisconnectNetworkResult();
                return result.IsSuccess();
            }
            else
            {
                return false;
            }
        }
        else
        {
            // ネットワークを破棄する
            nn::pia::Result result = nn::pia::local::LocalFacade::GetInstance()->DestroyNetworkAsync();
            if (result.IsSuccess())
            {
                while (nn::pia::local::LocalFacade::GetInstance()->IsDestroyNetworkCompleted() == false)
                {
                    sampleframework::SampleFramework::GetInstance()->BeginFrame();
                    nn::pia::common::Scheduler::GetInstance()->Dispatch();
                    sampleframework::SampleFramework::GetInstance()->EndFrame();
                }
                result = nn::pia::local::LocalFacade::GetInstance()->GetDestroyNetworkResult();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("DestroyNetwork failure\n");
                }
                return result.IsSuccess();
            }
            else
            {
                return false;
            }
        }
    }
    else if (nn::pia::local::LocalFacade::GetInstance()->IsClient())  //クライアントの場合
    {
        PIA_SAMPLE_PRINT("IsClient\n");
        nn::pia::Result result = nn::pia::local::LocalFacade::GetInstance()->DisconnectNetworkAsync();
        if (result.IsSuccess())
        {
            while (nn::pia::local::LocalFacade::GetInstance()->IsDisconnectNetworkCompleted() == false)
            {
                sampleframework::SampleFramework::GetInstance()->BeginFrame();
                nn::pia::common::Scheduler::GetInstance()->Dispatch();
                sampleframework::SampleFramework::GetInstance()->EndFrame();
            }
            result = nn::pia::local::LocalFacade::GetInstance()->GetDisconnectNetworkResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("DisconnectNetwork failure\n");
            }
            return result.IsSuccess();
        }
        else
        {
            return false;
        }
    }

    return true;
}

nn::pia::Result Local_StartupPiaLocal()
{
    if (!PIA_IS_VALID_POINTER(nn::pia::local::LocalFacade::GetInstance()))
    {
        PIA_RETURN_RESULT(nn::pia::ResultInvalidState);
    }

    return nn::pia::local::LocalFacade::GetInstance()->Startup();
}

void Local_CleanupPiaLocal()
{
    if (PIA_IS_VALID_POINTER(nn::pia::local::LocalFacade::GetInstance()))
    {
        nn::pia::local::LocalFacade::GetInstance()->Cleanup();
    }
}

void Local_CleanupNetwork()
{
    // LocalFacade の CleanupForMesh
    if (PIA_IS_VALID_POINTER(nn::pia::local::LocalFacade::GetInstance()))
    {
        nn::pia::local::LocalFacade::GetInstance()->CleanupForMesh();
    }
}

nn::pia::transport::NetworkFactory* Local_GetNetworkFactory()
{
    return &s_NetworkFactory;
}

bool Local_IsConnected()
{
    if (PIA_IS_VALID_POINTER(nn::pia::local::LocalFacade::GetInstance()))
    {
        // ホストマイグレーション中は true を返す。
        return nn::pia::local::LocalFacade::GetInstance()->IsConnected() || nn::pia::local::LocalFacade::GetInstance()->IsDuringHostMigration();
    }
    else
    {
        return false;
    }
}
}
}  // end of namespace nns::pia

#endif  // NN_PIA_CTR || NN_PIA_NINTENDOSDK
