﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Net_Setup.h"

#if NN_PIA_CTR
#include <nn/version.h>
#include <nn/init.h>
#include <nn/os.h>
#include <nn/ac.h>
#include <nn/friends.h>
#if (NN_VERSION_MAJOR == 2 && NN_VERSION_MINOR == 4 && NN_VERSION_MICRO >= 1) || NN_VERSION_MAJOR >= 3
#include <nn/socket_Initialize.h>
#else
#include <nn/socket.h>
#endif
#elif NN_PIA_CAFE
#include <cafe/net/so.h>
#include <nn/ac.h>
#include <nn/act.h>
#include <nn/fp.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/socket/socket_Api.h>
#include <nn/nifm/nifm_Api.h>
#include <nn/nifm/nifm_ApiNetworkConnection.h>
#include <nn/nifm/nifm_ApiIpAddress.h>
#include <nn/nifm/nifm_ApiCommunicationControlForTest.h>
#include <nn/friends.h>
#include <nn/account.h>
#include <nn/account/account_Selector.h>
#include <nn/err.h>
#elif NN_PIA_A
#include <sys/socket.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <net/if.h>
#elif NN_PIA_B
#include <arpa/inet.h>
#include <ifaddrs.h>
#endif

namespace
{

// ネットワークエラーコード
uint32_t s_NetworkErrorCode = 0;

bool s_IsNetworkIfInitialized = false;
bool s_IsNetworkIfUp = false;
bool s_IsSocketInitialized = false;


#if NN_PIA_CTR

/*
    ソケット API を扱うスレッドの数
    nn::nex::Core::ThreadModeUnsafeTransportBuffer使用時は NEX RVで2つ、Piaで3つ、合計5つ必要です。
    nn::nex::Core::ThreadModeUnsafeUser使用時は NEX RVで1つ、Piaで3つ、合計4つ必要です。
*/
const int32_t SocketSessionMax = 5;
const int32_t SocketBufferSize = 64 * 1024;                                  // １つのソケットの送受信バッファの割り当てに使用するサイズ
const uint32_t SocketBufferTotalSize = SocketBufferSize * SocketSessionMax;  // ソケットライブラリ用ワークメモリのサイズ

// ソケットライブラリに渡すワークメモリ（4096Byteアライメントされたバッファが必要）
PIA_ATTRIBUTE_ALIGN(4096) uint8_t s_SocketBuffer[SocketBufferTotalSize];

// 切断通知を受ける Event
nn::os::Event s_ACDisconnectedEvent;

#elif NN_PIA_CAFE

// 接続状態フラグ
bool s_ACConnected = false;
bool s_UpdateACStatusRun = false;
// ac の接続状態を定期的にチェックするスレッドとスレッドのスタック
PIA_ATTRIBUTE_ALIGN(8) nns::pia::Thread s_UpdateACStatusThread;
PIA_ATTRIBUTE_ALIGN(4096) uint8_t s_UpdateACStatusThreadStack[4 * 1024];

int UpdateACStatus(int argc, void* argv)
{
    NN_PIA_UNUSED(argc);
    NN_PIA_UNUSED(argv);

    while (s_UpdateACStatusRun)
    {
        bool IsConnected;
        nn::Result result = nn::ac::IsApplicationConnected(&IsConnected);
        if (result.IsSuccess())
        {
            // メインスレッドで参照できるように、結果を保存する
            s_ACConnected = IsConnected;
        }
        else
        {
            // nn::ac::Initialize が呼ばれていない場合は切断状態とする
            s_ACConnected = false;
        }

        // 100msec程度の周期でループ
        OSSleepMilliseconds(100);
    }
    return 0;
}

//-----------------------------------------------------------------------------
// フレンドサーバへのログイン非同期処理の完了コールバック
//-----------------------------------------------------------------------------
OSEvent s_FriendPresenceLoginEvent;
void FriendPresenceLoginCompletionCallback(nn::Result result, void* pContext)
{
    nn::Result* pResult = static_cast<nn::Result*>(pContext);
    *pResult = result;

    OSSignalEvent(&s_FriendPresenceLoginEvent);
}

#elif NN_PIA_NINTENDOSDK

// ソケットライブラリに渡す設定
nn::socket::ConfigDefaultWithMemory s_SocketConfig;

// アカウント情報
nns::pia::UserAccountInfo s_UserAccountInfo[nn::account::UserCountMax];
int s_UserAccountNum = 0;
int s_MainUserAccountIndex = -1;

#endif

void PrintIfConfig()
{
    nn::pia::common::InetAddress host, netmask, dns1, dns2, gateway;

#if NN_PIA_CTR
    int32_t ret;
    nn::socket::InAddr addr1, addr2;

    NN_PIA_UNUSED(ret);

    ret = nn::socket::GetPrimaryAddress(reinterpret_cast<uint8_t*>(&addr1), reinterpret_cast<uint8_t*>(&addr2));
    PIA_SAMPLE_ASSERT(ret == 0);
    host.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr1.addr));
    netmask.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr2.addr));
    ret = nn::socket::GetResolver(reinterpret_cast<uint8_t*>(&addr1), reinterpret_cast<uint8_t*>(&addr2));
    PIA_SAMPLE_ASSERT(ret == 0);
    dns1.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr1.addr));
    dns2.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr2.addr));

    ret = nn::socket::GetDefaultGateway(reinterpret_cast<uint8_t*>(&addr1));
    PIA_SAMPLE_ASSERT(ret == 0);
    gateway.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr1.addr));

#elif NN_PIA_CAFE
    nn::Result result;
    nn::ac::IpAddress addr;

    result = nn::ac::GetAssignedAddress(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    host.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr));

    result = nn::ac::GetAssignedSubnet(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    netmask.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr));

    result = nn::ac::GetAssignedPreferedDns(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    dns1.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr));

    result = nn::ac::GetAssignedAlternativeDns(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    dns2.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr));

    result = nn::ac::GetAssignedGateway(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    gateway.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(addr));
#elif NN_PIA_WIN
#elif NN_PIA_NINTENDOSDK
    nn::Result result;
    in_addr inAddrIpAddress;
    in_addr inAddrSubnetMask;
    in_addr inAddrDefaultGateway;
    in_addr inAddrPreferredDns;
    in_addr inAddrAlternateDns;
    PIA_SAMPLE_PRINT("nn::nifm::GetCurrentIpConfigInfo() is called\n");
    result = nn::nifm::GetCurrentIpConfigInfo(
        &inAddrIpAddress,
        &inAddrSubnetMask,
        &inAddrDefaultGateway,
        &inAddrPreferredDns,
        &inAddrAlternateDns);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::nifm::GetCurrentIpConfigInfo() is failed. Result:%08x\n", result.GetInnerValueForDebug());
    }
    host.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(inAddrIpAddress.s_addr));
    netmask.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(inAddrSubnetMask.s_addr));
    gateway.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(inAddrDefaultGateway.s_addr));
    dns1.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(inAddrPreferredDns.s_addr));
    dns2.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(inAddrAlternateDns.s_addr));
#elif NN_PIA_A
    int dummySock = socket(AF_INET, SOCK_STREAM, 0);
    if (dummySock < 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "socket() is failed.[errno:%d]", errno);
    }

    struct ifreq ifr;
    memset(&ifr, 0, sizeof(ifreq));
    ifr.ifr_addr.sa_family = AF_INET;
    strncpy(ifr.ifr_name, "wlan0", IFNAMSIZ - 1);

    if (ioctl(dummySock, SIOCGIFADDR, &ifr) < 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "ioctl(dummySock, SIOCGIFADDR, &ifr) is failed.[errno:%d]", errno);
    }
    host.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr));

    if (ioctl(dummySock, SIOCGIFNETMASK, &ifr) < 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "ioctl(dummySock, SIOCGIFNETMASK, &ifr) is failed.[errno:%d]", errno);
    }
    netmask.SetAddress(nn::pia::common::ByteOrder::NetworkToHost32(((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr));

    close(dummySock);
#elif NN_PIA_B
    struct ifaddrs *ifa_list;
    struct ifaddrs *ifa;
    
    int result = getifaddrs(&ifa_list);
    if (result == 0) {
        for(ifa = ifa_list; ifa != NULL; ifa=ifa->ifa_next) {
            if (!strcmp(ifa->ifa_name, "en1") && ifa->ifa_addr->sa_family == AF_INET)
            {
                host.SetAddress(ntohl( ((struct sockaddr_in *)ifa->ifa_addr)->sin_addr.s_addr ));
                netmask.SetAddress(ntohl( ((struct sockaddr_in *)ifa->ifa_netmask)->sin_addr.s_addr ));
            }
        }
        freeifaddrs(ifa_list);
    }
#endif
    nn::pia::common::FixedString<32> string;

    host.GetAddressString(&string);
    PIA_SAMPLE_PRINT("Net_InitializeNetwork:host   : %s\n", string.GetCharArray());

    netmask.GetAddressString(&string);
    PIA_SAMPLE_PRINT("Net_InitializeNetwork:netmask: %s\n", string.GetCharArray());

    dns1.GetAddressString(&string);
    PIA_SAMPLE_PRINT("Net_InitializeNetwork:dns1   : %s\n", string.GetCharArray());

    dns2.GetAddressString(&string);
    PIA_SAMPLE_PRINT("Net_InitializeNetwork:dns2   : %s\n", string.GetCharArray());

    gateway.GetAddressString(&string);
    PIA_SAMPLE_PRINT("Net_InitializeNetwork:gateway: %s\n", string.GetCharArray());

#if NN_PIA_CTR
#ifndef NN_SWITCH_DISABLE_DEBUG_PRINT
    PIA_SAMPLE_PRINT("\n");
    nn::socket::DumpRoutingTable();
#endif
#endif
}
}  // end of namespace

namespace nns
{
namespace pia
{

bool Net_GetLocalAddress(uint32_t* pIpAddr, uint32_t* pSubnetMask)
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pIpAddr));
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSubnetMask));

#if NN_PIA_CTR
    int32_t ret;
    nn::socket::InAddr addr1, addr2;

    NN_UNUSED_VAR(ret);

    ret = nn::socket::GetPrimaryAddress(reinterpret_cast<uint8_t*>(&addr1), reinterpret_cast<uint8_t*>(&addr2));
    PIA_SAMPLE_ASSERT(ret == 0);
    *pIpAddr = nn::pia::common::ByteOrder::NetworkToHost32(addr1.addr);
    *pSubnetMask = nn::pia::common::ByteOrder::NetworkToHost32(addr2.addr);
#elif NN_PIA_CAFE
    nn::Result result;
    nn::ac::IpAddress addr;

    result = nn::ac::GetAssignedAddress(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    *pIpAddr = nn::pia::common::ByteOrder::NetworkToHost32(addr);

    result = nn::ac::GetAssignedSubnet(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    *pSubnetMask = nn::pia::common::ByteOrder::NetworkToHost32(addr);
#elif NN_PIA_WIN
    //ホスト名を取得する
    char hostname[256];
    if (gethostname(hostname, sizeof(hostname)) != 0)
    {
        return false;
    }

    //ホスト名からIPアドレスを取得する
    HOSTENT* hostent = gethostbyname(hostname);
    if (hostent == NULL)
    {
        return false;
    }
    IN_ADDR inaddr;
    memcpy(&inaddr, hostent->h_addr_list[0], sizeof(struct in_addr));
    *pIpAddr = ntohl(inaddr.S_un.S_addr);
    *pSubnetMask = 0xffffff00;
#elif NN_PIA_NINTENDOSDK
    nn::Result result;
    in_addr inAddrIpAddress;
    in_addr inAddrSubnetMask;
    in_addr inAddrDefaultGateway;
    in_addr inAddrPreferredDns;
    in_addr inAddrAlternateDns;

    PIA_SAMPLE_PRINT("nn::nifm::GetCurrentIpConfigInfo() is called\n");
    result = nn::nifm::GetCurrentIpConfigInfo(
        &inAddrIpAddress,
        &inAddrSubnetMask,
        &inAddrDefaultGateway,
        &inAddrPreferredDns,
        &inAddrAlternateDns);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::nifm::GetCurrentIpConfigInfo() is failed. Result:%08x\n", result.GetInnerValueForDebug());
        return false;
    }

    *pIpAddr = nn::pia::common::ByteOrder::NetworkToHost32(inAddrIpAddress.s_addr);
    *pSubnetMask = nn::pia::common::ByteOrder::NetworkToHost32(inAddrSubnetMask.s_addr);

    NN_PIA_UNUSED(inAddrDefaultGateway);
    NN_PIA_UNUSED(inAddrPreferredDns);
    NN_PIA_UNUSED(inAddrAlternateDns);
#elif NN_PIA_A
    int dummySock = socket(AF_INET, SOCK_STREAM, 0);
    if (dummySock < 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "socket() is failed.[errno:%d]", errno);
    }

    struct ifreq ifr;
    memset(&ifr, 0, sizeof(ifreq));
    ifr.ifr_addr.sa_family = AF_INET;
    strncpy(ifr.ifr_name, "wlan0", IFNAMSIZ - 1);

    if (ioctl(dummySock, SIOCGIFADDR, &ifr) < 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "ioctl(dummySock, SIOCGIFADDR, &ifr) is failed.[errno:%d]", errno);
    }
    *pIpAddr = nn::pia::common::ByteOrder::NetworkToHost32(((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr);

    if (ioctl(dummySock, SIOCGIFNETMASK, &ifr) < 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "ioctl(dummySock, SIOCGIFNETMASK, &ifr) is failed.[errno:%d]", errno);
    }
    *pSubnetMask = nn::pia::common::ByteOrder::NetworkToHost32(((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr);

    close(dummySock);
#elif NN_PIA_B
    struct ifaddrs *ifa_list;
    struct ifaddrs *ifa;
    
    int result = getifaddrs(&ifa_list);
    if (result != 0) {
        return false; // 失敗
    }
    
    for (ifa = ifa_list; ifa != NULL; ifa = ifa->ifa_next) {
        if ((!strcmp(ifa->ifa_name, "en0") || !strcmp(ifa->ifa_name, "en1")) && ifa->ifa_addr->sa_family == AF_INET)
        {
            *pIpAddr = ntohl(((struct sockaddr_in *)ifa->ifa_addr)->sin_addr.s_addr);
            *pSubnetMask = ntohl(((struct sockaddr_in *)ifa->ifa_netmask)->sin_addr.s_addr);
        }
    }
    freeifaddrs(ifa_list);
#else
    return false;
#endif

    return true;
}


NetResult Net_PrepareNetworkInterface(NetworkType networkType)
{
#if NN_PIA_CTR
    nn::Result result;
    nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMicroSeconds(2000));

    // fs::Initializeはありますが、fs::Finalizeは呼ぶ必要はありません
    nn::fs::Initialize();

    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Initialize\n");
    result = nn::ac::Initialize();
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Initialize:failed\n");
        PIA_SAMPLE_PRINT("Result(%08x)\n", result.GetValue());
        s_IsNetworkIfInitialized = false;
        return result == nn::ac::ResultWifiOff() ? NetResult_NicDown : NetResult_AcError;
    }
    s_IsNetworkIfInitialized = true;

    nn::ac::Config config;
    nn::ac::CreateDefaultConfig(&config);

    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Connect:\n");
    result = nn::ac::Connect(config);
    if (result.IsFailure())
    {
        uint32_t errorCode;
        nn::ac::GetLastErrorCode(&errorCode);
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Connect() failed, errcode=%u\n", errorCode);
        s_NetworkErrorCode = errorCode;
        s_IsNetworkIfUp = false;
        return result == nn::ac::ResultWifiOff() ? NetResult_NicDown : NetResult_AcError;
    }
    // 切断通知を受ける Event を登録します
    s_ACDisconnectedEvent.Initialize(true);
    nn::ac::CTR::RegisterDisconnectEvent(&s_ACDisconnectedEvent);
    s_IsNetworkIfUp = true;
    NN_PIA_UNUSED(networkType);
#elif NN_PIA_CAFE
    nn::Result result;

    // ACライブラリを初期化します
    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Initialize\n");
    result = nn::ac::Initialize();
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Initialize:failed\n");
        PIA_SAMPLE_PRINT("Result(%08x)\n", result.GetValue());
        s_IsNetworkIfInitialized = false;
        return NetResult_AcError;
    }
    s_IsNetworkIfInitialized = true;

    // 本体設定のデフォルトプロファイルでネットワークに接続します
    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Connect:\n");
    result = nn::ac::Connect();
    if (result.IsFailure())
    {
        nn::ac::ErrorCode errorCode;
        nn::ac::GetLastErrorCode(&errorCode);
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::ac::Connect() failed, errcode=%u\n", errorCode);
        s_NetworkErrorCode = errorCode;
        s_IsNetworkIfUp = false;
        return NetResult_AcError;
    }

    // フラグを接続状態にする
    s_ACConnected = false;
    // AC の状態をチェックするスレッドを起動します
    s_UpdateACStatusRun = true;
    s_UpdateACStatusThread.Start(UpdateACStatus, NULL, reinterpret_cast<void*>(s_UpdateACStatusThreadStack), sizeof(s_UpdateACStatusThreadStack), 0, OS_PRIORITY_APP_DEFAULT);
    s_IsNetworkIfUp = true;
    NN_PIA_UNUSED(networkType);
#elif NN_PIA_WIN
    // Windows ではネットワークインターフェースは OS 側で自動的に立ち上げてくれているはずなので何もしません
    NN_PIA_UNUSED(networkType);
#elif NN_PIA_NINTENDOSDK
    nn::Result result;

    // NIFM ライブラリを初期化
    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::nifm::Initialize()\n");
    result = nn::nifm::Initialize();
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::nifm::Initialize:failed. [%08x]\n", result.GetInnerValueForDebug());
        s_IsNetworkIfInitialized = false;
        return NetResult_AcError;
    }
    s_IsNetworkIfInitialized = true;

    if (networkType == NetworkType_Lan)
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::nifm::SetLocalNetworkMode(true)\n");
        nn::nifm::SetLocalNetworkMode(true);
    }
    else
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::nifm::SetLocalNetworkMode(false)\n");
        nn::nifm::SetLocalNetworkMode(false);
    }

#if NNS_PIA_USE_HANDLENETWORKREQUESTRESULT
    bool isExit = false;
    while (!isExit)
    {
        // ネットワークインターフェースの利用要求（ブロッキング API）
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::nifm::SubmitNetworkRequestAndWait()\n");
        nn::nifm::SubmitNetworkRequestAndWait();

        // 利用要求提出の結果をハンドリングします
        result = nn::nifm::HandleNetworkRequestResult();
        if (result.IsSuccess())
        {
            PIA_SAMPLE_PRINT("Net_InitializeNetwork: Network request is submitted.\n");
            isExit = true;
        }
        else if (result <= nn::nifm::ResultErrorHandlingCompleted())
        {
            // 利用要求を再提出することで受理される可能性があります
            PIA_SAMPLE_PRINT("Network request is not submitted. Try SubmitNetworkRequest again.\n");
        }
        else
        {
            isExit = true;
        }
    }
#else
    // ネットワークインターフェースの利用要求（ブロッキング API）
    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::nifm::SubmitNetworkRequestAndWait()\n");
    nn::nifm::SubmitNetworkRequestAndWait();
#endif
    // 起動処理の成否をチェック
    if (!nn::nifm::IsNetworkAvailable())
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: Network is not available\n");
        s_IsNetworkIfUp = false;
        return NetResult_AcError;
    }
    s_IsNetworkIfUp = true;
#elif NN_PIA_A || NN_PIA_B
    // 何もしない
    s_IsNetworkIfInitialized = true;
    NN_PIA_UNUSED(networkType);
#endif
    return NetResult_Success;
}

NetResult Net_InitializeSocket()
{
#if NN_PIA_CTR
    nn::Result result;

    size_t requiredMemorySize = nn::socket::GetRequiredMemorySize(SocketBufferSize, SocketSessionMax);
    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::socket::Initialize:requiredMemorySize:%d, SOCKET_BUFFER:%d, SocketSessionMax:%d\n",
                   requiredMemorySize,
                   SocketBufferSize,
                   SocketSessionMax);
    result = nn::socket::Initialize(reinterpret_cast<uptr>(s_SocketBuffer), requiredMemorySize, SocketBufferSize, SocketSessionMax);
    PIA_SAMPLE_ASSERT(requiredMemorySize < sizeof(s_SocketBuffer));

    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::socket::Initialize:failed\n");
        PIA_SAMPLE_PRINT("Result(%08x)\n", result.GetValue());
        s_IsSocketInitialized = false;
        return NetResult_SocketError;
    }
    s_IsSocketInitialized = true;
#elif NN_PIA_CAFE
    // ソケットライブラリを初期化します
    PIA_SAMPLE_PRINT("Net_InitializeNetwork: SOInit\n");
    if (SOInit() != 0)
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: SOInit failed, err=%d\n", SOLastError());
        s_IsSocketInitialized = false;
        return NetResult_SocketError;
    }
    s_IsSocketInitialized = true;
#elif NN_PIA_WIN
    // Winsock を初期化します
    WORD wVersionRequested = MAKEWORD(2, 2);  // 使用するWinSockのバージョン要求
    WSADATA wsaData;                          // WinSockデータ構造体
    int nErrorStatus;

    nErrorStatus = WSAStartup(wVersionRequested, &wsaData);
    if (nErrorStatus != 0)
    {
        PIA_SAMPLE_PRINT("Winsock initialization failed\n");
        s_IsNetworkIfInitialized = false;
        return NetResult_SocketError;
    }
    else
    {
        // 要求したバージョンと同一か確認
        if (LOBYTE(wsaData.wVersion) == 2 && HIBYTE(wsaData.wVersion) == 2)
        {
            // バージョン情報の表示
            PIA_SAMPLE_PRINT("version %d.%d is required\n",
                             LOBYTE(wsaData.wVersion),
                             HIBYTE(wsaData.wVersion));
            PIA_SAMPLE_PRINT("latest version is %d.%d\n",
                             LOBYTE(wsaData.wHighVersion),
                             HIBYTE(wsaData.wHighVersion));
        }
    }
    s_IsNetworkIfInitialized = true;
#elif NN_PIA_NINTENDOSDK
    nn::Result result;

    // Socket ライブラリを初期化
    PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::socket::Initialize()\n");
    result = nn::socket::Initialize(s_SocketConfig);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork: nn::socket::Initialize is failed. [%08x] [err=%d]\n", result.GetInnerValueForDebug(), nn::socket::GetLastErrno());
        s_IsSocketInitialized = false;
        return NetResult_SocketError;
    }
    s_IsSocketInitialized = true;
#elif NN_PIA_A || NN_PIA_B
    // 何もしない
    s_IsSocketInitialized = true;
#endif
    return NetResult_Success;
}

NetResult Net_InitializeAccount(NetworkType networkType)
{
#if NN_PIA_CAFE
    nn::Result sdkResult;

    // アカウントライブラリを初期化します
    PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::act::Initialize\n");
    sdkResult = nn::act::Initialize();
    PIA_SAMPLE_ASSERT(sdkResult.IsSuccess());
    if (sdkResult.IsFailure())
    {
        // ここには来ないはず
        return NetResult_AccountError;
    }
#elif NN_PIA_NINTENDOSDK && NN_PIA_NINTENDOSDK_NX
    nn::Result sdkResult;

    for (int i = 0; i < nn::account::UserCountMax; ++i)
    {
        s_UserAccountInfo[i].userUid = nn::account::InvalidUid;
    }
    s_UserAccountNum = 0;
    s_MainUserAccountIndex = -1;

    // アカウントライブラリの初期化
    PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::Initialize\n");
    nn::account::Initialize();

#if NNS_PIA_USE_SHOWUSERSELECTOR
    // 本関数は UI 操作によりブロックします
    PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::ShowUserSelector\n");
    sdkResult = nn::account::ShowUserSelector(&s_UserAccountInfo[0].userUid, nn::account::DefaultUserSelectionSettings);
    if (sdkResult.IsFailure())
    {
        // UI 操作でアカウント選択をキャンセルした
        s_NetworkErrorCode = sdkResult.GetInnerValueForDebug();
        PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::ShowUserSelector is failed. [%08x]\n", s_NetworkErrorCode);
        return NetResult_AccountError;
    }
    s_UserAccountNum = 1;
#else
    // nn::account::ShowUserSelector() でユーザー選択しない実装（製品コードでは非推奨）
    // アカウントを取得する
    PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::ListAllUsers\n");
    nn::account::Uid userUid[nn::account::UserCountMax];
    sdkResult = nn::account::ListAllUsers(&s_UserAccountNum, userUid, nn::account::UserCountMax);
    PIA_SAMPLE_ASSERT(sdkResult.IsSuccess());

    // アカウントが存在しない
    if (s_UserAccountNum == 0)
    {
        PIA_SAMPLE_PRINT("Net_InitializeAccount: User account is zero\n");
        return NetResult_AccountError;
    }
    for (int i = 0; i < s_UserAccountNum; ++i)
    {
        s_UserAccountInfo[i].userUid = userUid[i];
    }
    PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::ListAllUsers is success. UserAccountNum=%d\n", s_UserAccountNum);
#endif

    // アカウントをオープン状態に変更します
    for (int i = 0; i < s_UserAccountNum; ++i)
    {
        PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::OpenUser\n");
        sdkResult = nn::account::OpenUser(&s_UserAccountInfo[i].userHandle, s_UserAccountInfo[i].userUid);
        if (sdkResult.IsFailure())
        {
            PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::OpenUser is failed. [%08x]\n", sdkResult.GetInnerValueForDebug());
            continue;
        }
        PIA_SAMPLE_PRINT("Net_InitializeAccount: nn::account::OpenUser is success\n");

        // 一番最初に確認できたアカウントをメインとする
        if (s_MainUserAccountIndex < 0)
        {
            s_MainUserAccountIndex = i;
        }
    }
    // 利用可能なアカウントが存在しない
    if (s_MainUserAccountIndex < 0)
    {
        PIA_SAMPLE_PRINT("Net_InitializeAccount: No user account is opened.\n");
        return NetResult_AccountError;
    }
#endif
    NN_PIA_UNUSED(networkType);
    return NetResult_Success;
}

void Net_FinalizeAccount()
{
#if NN_PIA_CAFE
    // アカウントライブラリを終了させる
    PIA_SAMPLE_PRINT(" Net_FinalizeAccount: nn::act::Finalize\n");
    nn::act::Finalize();
#elif NN_PIA_NINTENDOSDK && NN_PIA_NINTENDOSDK_NX
    // アカウントをクローズ状態に変更する
    for (int i = 0; i < s_UserAccountNum; ++i)
    {
        PIA_SAMPLE_PRINT(" Net_FinalizeAccount: nn::account::CloseUser\n");
        nn::account::CloseUser(s_UserAccountInfo[i].userHandle);
        s_UserAccountInfo[i].userUid = nn::account::InvalidUid;
        PIA_SAMPLE_PRINT(" Net_FinalizeAccount: nn::account::CloseUser is success\n");
    }
    s_UserAccountNum = 0;
    s_MainUserAccountIndex = -1;
// アカウントライブラリは Finalize() が無いので、これで終了
#endif
}

#if NN_PIA_NINTENDOSDK
int Net_GetUserAccountInfoSize()
{
    return s_UserAccountNum;
}

UserAccountInfo* Net_GetMainUserAccountInfo()
{
    if (s_MainUserAccountIndex < 0)
    {
        return NULL;
    }
    return &s_UserAccountInfo[s_MainUserAccountIndex];
}

UserAccountInfo* Net_GetUserAccountInfo(int index)
{
    if (index < s_UserAccountNum)
    {
        return &s_UserAccountInfo[index];
    }
    return NULL;
}
#endif

NetResult Net_InitializeFriends()
{
#if NN_PIA_CTR
    nn::os::Event event(true);

    // フレンドライブラリを初期化する
    nn::Result sdkResult = nn::friends::Initialize();
    if (sdkResult.IsFailure())
    {
        // デーモンの内部状態などにより、失敗することがある
        PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::friends::Initialize failed\n");
        s_NetworkErrorCode = nn::friends::ResultToErrorCode(sdkResult);
        PIA_SAMPLE_PRINT("Result(%08x)\n", sdkResult.GetValue());
        return NetResult_FriendsError;
    }

    // フレンドサーバにログインする
    sdkResult = nn::friends::Login(&event);
    if (sdkResult.IsFailure())
    {
        // APと接続できていない場合などに、失敗することがある
        PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::friends::Login failed\n");
        s_NetworkErrorCode = nn::friends::ResultToErrorCode(sdkResult);
        PIA_SAMPLE_PRINT("Result(%08x)\n", sdkResult.GetValue());
        return NetResult_FriendsError;
    }

    // 非同期ログイン処理の終了待ち
    // 処理の可読性を重視してここで行います。
    event.Wait();

    // 非同期ログイン処理の結果取得
    sdkResult = nn::friends::GetLastResponseResult();
    if (sdkResult.IsFailure())
    {
        // デーモンの内部状態などにより、失敗することがある
        PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::friends::GetLastResponseResult failed:ErrorCode:%d\n", nn::friends::ResultToErrorCode(sdkResult));
        s_NetworkErrorCode = nn::friends::ResultToErrorCode(sdkResult);
        PIA_SAMPLE_PRINT("Result(%08x)\n", sdkResult.GetValue());
        return NetResult_FriendsError;
    }
    PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::friends::Login succeeded:MyPrincipalId:%d\n", nn::friends::GetMyPrincipalId());

    // 自アプリがフレンドサーバにログイン出来ていることを確認
    if (!nn::friends::HasLoggedIn())
    {
        PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::friends::HasLoggedIn failed\n");
        return NetResult_FriendsError;
    }
#elif NN_PIA_CAFE
    nn::Result sdkResult;

    // フレンドプレゼンスライブラリを初期化します
    PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::fp::Initialize\n");
    sdkResult = nn::fp::Initialize();
    PIA_SAMPLE_ASSERT(sdkResult.IsSuccess());

    // フレンドサーバへのログインは必須ではないが、フレンドプレゼンスを使用する場合はログインする
    nn::Result asyncResult;

    // 自動モードでイベント初期化
    OSInitEvent(&s_FriendPresenceLoginEvent, FALSE, OS_EVENT_AUTO);
    PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::fp::LoginAsync called\n");
    sdkResult = nn::fp::LoginAsync(FriendPresenceLoginCompletionCallback, &asyncResult);
    if (sdkResult.IsFailure())
    {
        PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::fp::LoginAsync start failed:ErrorCode:%d\n", nn::fp::ResultToErrorCode(sdkResult));
        s_NetworkErrorCode = nn::fp::ResultToErrorCode(sdkResult);
        PIA_SAMPLE_PRINT("Result(%08x)\n", sdkResult.GetValue());
        return NetResult_FriendsError;
    }
    // nn::fp::LoginAsync() 完了のイベントシグナルを待ちます
    OSWaitEvent(&s_FriendPresenceLoginEvent);
    if (asyncResult.IsFailure())
    {
        PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::fp::LoginAsync async failed:ErrorCode:%d\n", nn::fp::ResultToErrorCode(asyncResult));
        s_NetworkErrorCode = nn::fp::ResultToErrorCode(asyncResult);
        PIA_SAMPLE_PRINT("Result(%08x)\n", asyncResult.GetValue());
        return NetResult_FriendsError;
    }
    PIA_SAMPLE_PRINT("Net_InitializeFriends: nn::fp::LoginAsync succeeded:MyPrincipalId:%d\n", nn::fp::GetMyPrincipalId());
#elif NN_PIA_NINTENDOSDK && NN_PIA_NINTENDOSDK_NX
    // フレンドライブラリを初期化します
    PIA_SAMPLE_PRINT("nn::friends::Initialize is called\n");
    nn::friends::Initialize();
#endif


    return NetResult_Success;
}

void Net_FinalizeFriends()
{
#if NN_PIA_CTR
    if (nn::friends::IsInitialized())  // n1769 未初期化状態で Finalize() を呼び出すと、SDK 内部のアサートに引っかかるようなので…。
    {
        // フレンドサーバからログアウト（ログインしていない状態でも呼び出し可能なので、常に呼び出します）
        PIA_SAMPLE_PRINT(" Net_FinalizeFriends: nn::friends::Logout\n");
        nn::friends::Logout();

        // フレンドライブラリを終了させる
        PIA_SAMPLE_PRINT(" Net_FinalizeFriends: nn::friends::Finalize\n");
        nn::friends::Finalize();
    }
#elif NN_PIA_CAFE
    // フレンドサーバからログアウト（ログインしていない状態でも呼び出し可能なので、常に呼び出します）
    PIA_SAMPLE_PRINT(" Net_FinalizeFriends: nn::fp::Logout called\n");
    nn::fp::Logout();

    // フレンドプレゼンスライブラリを終了させる
    PIA_SAMPLE_PRINT(" Net_FinalizeFriends: nn::fp::Finalize\n");
    nn::fp::Finalize();
#elif NN_PIA_NINTENDOSDK
#endif
}

NetResult Net_InitializeNetwork(NetworkType networkType)
{
    s_NetworkErrorCode = 0;

    bool isNeedInitializeNetworkInterface = false;
    bool isNeedInitializeSocket = false;
    bool isNeedInitializeAccount = false;
    bool isNeedInitializeFrineds = false;

    switch (networkType)
    {
    case NetworkType_Local:
    {
        isNeedInitializeNetworkInterface = false;
#if NN_PIA_NINTENDOSDK
        isNeedInitializeSocket = true;
        isNeedInitializeAccount = true;
        isNeedInitializeFrineds = false;
#else
        isNeedInitializeSocket = false;
        isNeedInitializeAccount = false;
        isNeedInitializeFrineds = false;
#endif
    }
    break;
    case NetworkType_Internet:
    {
        isNeedInitializeNetworkInterface = true;
        isNeedInitializeSocket = true;
        isNeedInitializeAccount = true;
        isNeedInitializeFrineds = true;
    }
    break;
    case NetworkType_Lan:
    {
        isNeedInitializeNetworkInterface = true;
        isNeedInitializeSocket = true;
        isNeedInitializeAccount = true;
        isNeedInitializeFrineds = false;
    }
    break;
    }

    if (isNeedInitializeNetworkInterface)
    {
        NetResult netResult = Net_PrepareNetworkInterface(networkType);
        if (netResult != NetResult_Success)
        {
            return netResult;
        }
    }

    if (isNeedInitializeSocket)
    {
        NetResult netResult = Net_InitializeSocket();
        if (netResult != NetResult_Success)
        {
            return netResult;
        }
    }

    if (isNeedInitializeAccount)
    {
        NetResult netResult = Net_InitializeAccount(networkType);
        if (netResult != NetResult_Success)
        {
            return netResult;
        }
    }

    if (isNeedInitializeFrineds)
    {
        NetResult netResult = Net_InitializeFriends();
        if (netResult != NetResult_Success)
        {
            return netResult;
        }
    }

    if (networkType == NetworkType_Internet || networkType == NetworkType_Lan)
    {
        // 有効になったネットワークインターフェースの情報をプリントします
        PrintIfConfig();
    }

    return NetResult_Success;
}

void Net_FinalizeNetwork()
{
    Net_FinalizeFriends();
    Net_FinalizeAccount();

#if NN_PIA_CTR
    nn::Result result;

    if (s_IsSocketInitialized)
    {
        // Finilizing socket library
        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:socket::Finalize\n");
        result = nn::socket::Finalize();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("Net_FinalizeNetwork:socket::Finalize:failed\n");
            PIA_SAMPLE_PRINT("Result(%08x)\n", result.GetValue());
        }
        s_IsSocketInitialized = false;
    }

    if (s_IsNetworkIfUp)
    {
        // 切断通知を受ける Event を破棄します
        s_ACDisconnectedEvent.Finalize();

        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:ac::Close\n");
        result = nn::ac::Close();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("Net_FinalizeNetwork:ac::Close:failed\n");
            PIA_SAMPLE_PRINT("Result(%08x)\n", result.GetValue());
            //n1589:ここでリターンすると次のAC初期化に失敗する
        }
        s_IsNetworkIfUp = false;
    }

    if (s_IsNetworkIfInitialized)
    {
        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:ac::Finalize\n");
        result = nn::ac::Finalize();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("Net_FinalizeNetwork:ac::Finalize:failed\n");
            PIA_SAMPLE_PRINT("Result(%08x)\n", result.GetValue());
        }
        s_IsNetworkIfInitialized = false;
    }

// fs::Initializeはありますが、fs::Finalizeは呼ぶ必要はありません
#elif NN_PIA_CAFE
    nn::Result result;

    if (s_IsSocketInitialized)
    {
        // Finilizing socket library
        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:SOFinish\n");
        int ret = SOFinish();
        if (ret != 0)
        {
            PIA_SAMPLE_PRINT("Net_FinalizeNetwork:SOFinish:failed, err=%d\n", ret);
        }
        s_IsSocketInitialized = false;
    }
    if (s_UpdateACStatusRun)
    {
        // AC の状態をチェックするスレッドを停止し、回収します
        s_UpdateACStatusRun = false;
        s_UpdateACStatusThread.Join();
        s_UpdateACStatusThread.Finalize();
    }

    if (s_IsNetworkIfUp)
    {
        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:ac::Close\n");
        result = nn::ac::Close();
        if (result.IsFailure())
        {
            nn::ac::ErrorCode errorCode;
            nn::ac::GetLastErrorCode(&errorCode);
            PIA_SAMPLE_PRINT("Net_FinalizeNetwork:ac::Close:failed, errcode=%u\n", errorCode);
            PIA_SAMPLE_PRINT("Result(%08x)\n", result.GetValue());
        }
        s_IsNetworkIfUp = false;
    }

    if (s_IsNetworkIfInitialized)
    {
        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:ac::Finalize\n");
        nn::ac::Finalize();
        s_IsNetworkIfInitialized = false;
    }
#elif NN_PIA_WIN
    // Winsock を停止します
    WSACleanup();
    // Windows ではネットワークインターフェースは OS 側が制御するので何もしない
    s_IsNetworkIfInitialized = false;
#elif NN_PIA_NINTENDOSDK
    if (s_IsSocketInitialized)
    {
        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:nn::socket::Finalize\n");
        nn::Result result = nn::socket::Finalize();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("Net_FinalizeNetwork:nn::socket::Finalize is failed. [%08x] [err=%d]\n", result.GetInnerValueForDebug(), nn::socket::GetLastErrno());
        }
        s_IsSocketInitialized = false;
    }
    if (!s_IsNetworkIfInitialized)
    {
        nn::nifm::Initialize();
    }
    // テスト目的でネットワークインターフェースを強制停止させたままになっている場合があるので、リセットしておく
    nn::nifm::SetEthernetCommunicationEnabledForTest(true);
    nn::nifm::SetWirelessCommunicationEnabledForTest(true);
    
    if (s_IsNetworkIfUp)
    {
        // ネットワーク接続が不要になったら要求を取り下げます
        PIA_SAMPLE_PRINT("Net_FinalizeNetwork:nn::nifm::CancelNetworkRequest\n");
        nn::nifm::CancelNetworkRequest();
        s_IsNetworkIfUp = false;
    }
    s_IsNetworkIfInitialized = false;
#elif NN_PIA_A || NN_PIA_B
    s_IsNetworkIfInitialized = false;
#endif
}

bool Net_IsInitialized()
{
    return s_IsNetworkIfInitialized;
}

bool Net_IsConnected()
{
#if NN_PIA_CTR
    if (s_ACDisconnectedEvent.Wait(0))
    {
        return false;
    }
    else
    {
        return true;
    }
#elif NN_PIA_CAFE
    return s_ACConnected;
#elif NN_PIA_WIN
    return true;
#elif NN_PIA_NINTENDOSDK
    if (s_IsNetworkIfInitialized)
    {
        return nn::nifm::IsNetworkAvailable();
    }
    else
    {
        return false;
    }
#elif NN_PIA_A || NN_PIA_B
    return true;
#endif
}

uint32_t Net_GetNetworkErrorCode()
{
    return s_NetworkErrorCode;
}
}
}  // end of namespace nns::pia
