﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include "SampleLib/SampleLib.h"
#include "SampleLib/Nex_Matchmake.h"
#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Inet_Setup.h"

#if NNS_PIA_ENABLE_INET

#include <nn/pia/common.h>
#include <nn/pia/inet/inet_NexFacade.h>

// 同じマッチメイクセッションに何度も参加できる AutoMatchmake を使用します。(NEX 3系のみ)
#define ENABLE_JOIN_SAME_MATCHMAKE 0

namespace
{
const uint32_t MatchmakeAttributeIndex = 0;     // 使用する検索属性のINDEX
const uint32_t MatchmakeAttributeMaxValue = 2;  // 使用する検索属性のMAX値

// マッチメイクに必要なインスタンス
nn::nex::MatchmakeExtensionClient* s_pMatchMakingClient = NULL;
bool s_IsBind = false;


// マッチメイクネットワークエラーコード
uint32_t s_MatchmakeNetworkErrorCode = 0;

// マッチメイクセッション確立時にサーバから取得するセッション共有キーのポインタ
nn::nex::qVector<nn::nex::qByte>* s_pMatchmakeSessionKey;

// SignatureSetting に設定する署名鍵のサイズ
const uint32_t SignatureKeyDataSize = 32;

// セッション共有キーを PiaSession で使用するための署名機能用設定
nn::pia::common::SignatureSettingWithKeyBuffer<SignatureKeyDataSize> s_SignatureSetting;

// ホストのプリンシパルID
nn::nex::PrincipalID s_OwnerPID = INVALID_PRINCIPALID;

//-----------------------------------------------------------------------------
// NGSから通知を処理するイベントハンドラ
//-----------------------------------------------------------------------------
class SampleNotificationEventHandler : public nn::nex::NotificationEventHandler
{
public:
    virtual void ProcessNotificationEvent(const nn::nex::NotificationEvent& oEvent)
    {
        nn::nex::StringStream ss;
        ss << oEvent;
        PIA_SAMPLE_PRINT("ProcessNotificationEvent:%ls\n", ss.CStr());

        // メッセージ毎の処理
        switch (oEvent.GetType())
        {
        case nn::nex::NotificationEvents::ParticipationEvent:
        {
            switch (oEvent.GetSubType())
            {
            case nn::nex::ParticipationEvents::Participate:
            {
                PIA_SAMPLE_PRINT("ParticipationEvents::Participate: [PID %u] join [session %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
            }
            break;
            case nn::nex::ParticipationEvents::EndParticipation:
            {
                PIA_SAMPLE_PRINT("ParticipationEvents::EndParticipation: [PID %u] end [session %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
            }
            break;
            case nn::nex::ParticipationEvents::Disconnect:
            {
                PIA_SAMPLE_PRINT("ParticipationEvents::Disconnect: [PID %u] disconnect [session %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
            }
            break;
            default:
            {
                PIA_SAMPLE_PRINT("ParticipationEvents: unsupported participation event:%d\n", oEvent.GetSubType());
            }
            break;
            }
        }
        break;
        case nn::nex::NotificationEvents::OwnershipChangeEvent:
        {
            PIA_SAMPLE_PRINT(
                "NotificationEvents::OwnershipChangeEvent: [session %u] changed owner to [PID %u]\n",
                oEvent.GetParam1(),
                oEvent.GetParam2());

            // 新しいオーナーの PID を記録
            s_OwnerPID = oEvent.GetParam2();
        }
        break;
        case nn::nex::NotificationEvents::GatheringUnregistered:
        {
            // 削除されたギャザリングのGIDを記録
            nn::nex::GatheringID unregisteredGatheringId = static_cast<nn::nex::GatheringID>(oEvent.GetParam1());
            PIA_SAMPLE_PRINT("NotificationEvents::GatheringUnregistered: [session %u] unregistered\n", unregisteredGatheringId);
            NN_PIA_UNUSED(unregisteredGatheringId);
        }
        break;
        case nn::nex::NotificationEvents::HostChangeEvent:
        {
            PIA_SAMPLE_PRINT("NotificationEvents::HostChangeEvent: [session %u] changed host\n", oEvent.GetParam1());
        }
        break;
        default:
        {
            PIA_SAMPLE_PRINT("NotificationEvents: unsupported notification event:%d\n", oEvent.GetType());
        }
        break;
        }
    }
};

// 通知を処理するイベントハンドラ
SampleNotificationEventHandler s_NotificationEventHandler;
};

namespace nns
{
namespace pia
{

bool Nex_InitializeMatchmake()
{
    // マッチメイクに必要なインスタンスの生成
    PIA_SAMPLE_PRINT("Nex_InitializeMatchmake:qNew nn::nex::MatchmakeExtensionClient.\n");
    s_pMatchMakingClient = qNew nn::nex::MatchmakeExtensionClient();
    PIA_SAMPLE_ASSERT(s_pMatchMakingClient != NULL);

    // マッチメイクサービスを利用するための証明書をセット
    if (!s_pMatchMakingClient->Bind(reinterpret_cast<nn::nex::NgsFacade*>(Nex_GetNgsFacadeInstance())->GetCredentials()))
    {
        PIA_SAMPLE_PRINT("Nex_InitializeMatchmake:Bind failed.\n");
        qDelete s_pMatchMakingClient;
        s_pMatchMakingClient = NULL;
        return false;
    }

    s_pMatchmakeSessionKey = qNew nn::nex::qVector<nn::nex::qByte>();

    // NGSからの通知を処理するイベントハンドラを登録
    reinterpret_cast<nn::nex::NgsFacade*>(Nex_GetNgsFacadeInstance())->RegisterNotificationEventHandler(&s_NotificationEventHandler);

    s_IsBind = true;
    return true;
}

bool Nex_FinalizeMatchmake()
{
    if (s_pMatchMakingClient)
    {
        if (s_IsBind)
        {
            s_pMatchMakingClient->Unbind();
            s_IsBind = false;
            PIA_SAMPLE_PRINT("Nex_FinalizeMatchmake:Unbind.\n");
        }

        // イベントハンドラを削除
        if (Nex_GetNgsFacadeInstance())
        {
            reinterpret_cast<nn::nex::NgsFacade*>(Nex_GetNgsFacadeInstance())->UnregisterNotificationEventHandler(&s_NotificationEventHandler);
        }

        // マッチメイクに必要なインスタンスの破棄
        PIA_SAMPLE_PRINT("Nex_FinalizeMatchmake:qDelete nn::nex::MatchmakeExtensionClient.\n");
        qDelete s_pMatchMakingClient;
        s_pMatchMakingClient = NULL;
    }
    if (s_pMatchmakeSessionKey)
    {
        s_pMatchmakeSessionKey->clear();
        qDelete s_pMatchmakeSessionKey;
        s_pMatchmakeSessionKey = NULL;
    }

    return true;
}

nn::nex::MatchmakeExtensionClient* Nex_GetMatchmakeClient()
{
    return s_pMatchMakingClient;
}

//-----------------------------------------------------------------------------
// オートマッチメイクを行う
// ここで必要なギャザリングを取得し、セッションに参加または作成します
//-----------------------------------------------------------------------------
bool Nex_AutoMatchmake(
    nn::nex::GatheringID* pJoinedGatheringID,
    nn::nex::PrincipalID* pOwnerPID,
    uint32_t matchmakeMode,
    uint16_t matchmakeMaxParticipants,
    const char* matchmakeDescription)
{
    PIA_SAMPLE_PRINT("Nex_AutoMatchmake:matchmakeMode:%d, matchmakeMaxParticipants:%d\n", matchmakeMode, matchmakeMaxParticipants);

    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Nex_AutoMatchmake:MatchmakeExtensionClient is not created.\n");
    }

    // 条件に合致するセッションがなかったときに作成する部屋
    nn::nex::MatchmakeSession* const pMatchmakeSession = qNew nn::nex::MatchmakeSession();
    pMatchmakeSession->SetOpenParticipation(false);

    // MatchmakeSessionの説明文字列 設定は任意です。
    if (matchmakeDescription == NULL)
    {
        pMatchmakeSession->SetDescription(SAMPLE_STRING("PiaInet Automatch"));
    }
    else
    {
        pMatchmakeSession->SetDescription(matchmakeDescription);
    }

    // 最小・最大参加人数
    pMatchmakeSession->SetMinParticipants(1);
    pMatchmakeSession->SetMaxParticipants(matchmakeMaxParticipants);

    // ゲームモード
    pMatchmakeSession->SetGameMode(matchmakeMode);

    // アプリケーションで自由に定義できる検索属性値
    // サンプルのため、自分で作成するセッションの属性値をランダムにする
    pMatchmakeSession->SetAttribute(MatchmakeAttributeIndex, nn::nex::Platform::GetRandomNumber(MatchmakeAttributeMaxValue + 1));

    // マッチメイクシステムタイプ
    pMatchmakeSession->SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // アプリケーションで自由にセットできるバイナリデータ
    nn::nex::qVector<nn::nex::qUnsignedInt8> applicationBuffer;
    applicationBuffer.push_back(1);
    applicationBuffer.push_back(2);
    applicationBuffer.push_back(3);
    pMatchmakeSession->SetApplicationBuffer(applicationBuffer);

    // オーナーがログアウトしたときにOwnerの委譲を行うか
    pMatchmakeSession->SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    // MatchmakeSessionインスタンスはoInGatheringHolderのデストラクタで破壊されます
    nn::nex::GatheringHolder oInGatheringHolder = pMatchmakeSession;
    nn::nex::GatheringHolder oOutGatheringHolder;  // 参加しているギャザリング

    // マッチメイクセッションを検索または作成する
    nn::nex::ProtocolCallContext oContext;

#if !ENABLE_JOIN_SAME_MATCHMAKE
    // 検索条件を複数指定する nn::nex::MatchmakeExtensionClient::AutoMatchmake を使用して AutoMatchmake を行います。

    // 検索条件の作成
    nn::nex::qList<nn::nex::MatchmakeSessionSearchCriteria> criteriaList;

    // 一つ目の条件は完全一致で行う
    {
        nn::nex::MatchmakeSessionSearchCriteria criteria;

        // 最小・最大参加人数
        criteria.SetMinParticipants(1);
        criteria.SetMaxParticipants(matchmakeMaxParticipants);

        // ゲームモード
        criteria.SetGameMode(matchmakeMode);

        // アプリケーションで自由に定義できる検索属性値 最初は0で固定
        criteria.SetAttribute(MatchmakeAttributeIndex, 0);

        // マッチメイクシステムタイプ
        criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

        // リストに登録
        criteriaList.push_back(criteria);
    }

    // 二つ目の条件は範囲指定で行う
    {
        nn::nex::MatchmakeSessionSearchCriteria criteria;

        // 最小・最大参加人数
        criteria.SetMinParticipants(1);
        criteria.SetMaxParticipants(matchmakeMaxParticipants);

        // ゲームモード
        criteria.SetGameMode(matchmakeMode);

        // アプリケーションで自由に定義できる検索属性値 属性値の取りうる範囲全て
        criteria.SetAttributeWithRange(MatchmakeAttributeIndex, 0, MatchmakeAttributeMaxValue);
        criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

        // リストに登録（２回目の検索条件なのでリストの末尾に追加）
        criteriaList.push_back(criteria);
    }

    // AutoMatchmakeの仕様により、マッチメイクセッションから離脱するとそのセッションは以降見つからなくなります。
    bool bSuccess = s_pMatchMakingClient->AutoMatchmake(
        &oContext,
        criteriaList,
        oInGatheringHolder,
        &oOutGatheringHolder,
        SAMPLE_STRING("AutoMatchmake message"));
#else
    // ギャザリングIDを指定する nn::nex::MatchmakeExtensionClient::AutoMatchmake を使用して AutoMatchmake を行います。(Nex 3系のみ)
    PIA_SAMPLE_PRINT("Nex_AutoMatchmake: ENABLE_JOIN_SAME_MATCHMAKE on\n");
    // マッチメイクセッションを検索します
    // 内部で実行される検索処理が完了されるまでブロックされます
    nn::nex::qList<nn::nex::GatheringHolder> listGatheringHolder;
    nn::pia::Result result = Nex_SearchMatchmakeSession(
        s_pMatchMakingClient,
        matchmakeMode,
        matchmakeMaxParticipants,
        2,
        &listGatheringHolder);
    if (result.IsFailure())
    {
        // 検索処理に失敗
        PIA_SAMPLE_PRINT("Nex_AutoMatchmake:Nex_SearchMatchmakeSession failed.\n");
    }

    // 検索結果で取得した nn::nex::GatheringHolder のリストから nn::nex::GatheringID の qList を作成します。
    nn::nex::qList<nn::nex::GatheringID> listGatheringId;
    listGatheringId.clear();
    if (listGatheringHolder.size() < 1)
    {
        PIA_SAMPLE_PRINT("Nex_AutoMatchmake:no matchmake session is found.\n");
        listGatheringId.clear();
    }
    else
    {
        nn::nex::qList<nn::nex::GatheringHolder>::const_iterator it = listGatheringHolder.begin();
        while (it != listGatheringHolder.end())
        {
            nn::nex::GatheringID gid = it->Get()->GetID();
            listGatheringId.push_back(gid);
            ++it;
        }
    }

    // 通常の AutoMatch() と異なり、過去3回にわたって参加したマッチメイクセッションを参加の対象から取り除く処理は含まれていません
    // こちらの AutoMatch() を使用する前に、検索条件を複数指定する AutoMatch() の使用を検討下さい。
    bool bSuccess = s_pMatchMakingClient->AutoMatchmake(
        &oContext,
        listGatheringId,
        oInGatheringHolder,
        &oOutGatheringHolder,
        SAMPLE_STRING("AutoMatchmake message"));
#endif

    if (!bSuccess)
    {
        PIA_SAMPLE_PRINT("Failed to AutoMatchmake\n");
        return false;
    }

    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        Nex_ReportProtocolCallContext(oContext, "Failed to Nex_AutoMatchmake\n");
        s_MatchmakeNetworkErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(oContext.GetOutcome());
        return false;
    }

    *pJoinedGatheringID = oOutGatheringHolder->GetID();
    *pOwnerPID = oOutGatheringHolder->GetOwnerPID();

    //P2Pセッションの署名キーとして、ゲームサーバから通知されたキーを取得する
    s_pMatchmakeSessionKey->clear();
    *s_pMatchmakeSessionKey = reinterpret_cast<nn::nex::MatchmakeSession*>(oOutGatheringHolder.Get())->GetSessionKey();

    PIA_SAMPLE_PRINT(
        "[PID Info >> JoinedGID:%d OwnerPID:%llu HostPID:%llu MyPID:%llu]\n",
        oOutGatheringHolder->GetID(),
        static_cast<uint64_t>(oOutGatheringHolder->GetOwnerPID()),
        static_cast<uint64_t>(oOutGatheringHolder->GetHostPID()),
        static_cast<uint64_t>(s_pMatchMakingClient->GetCredentials()->GetPrincipalID()));

    return true;
}


//-----------------------------------------------------------------------------
// マッチメイクセッションをオープンする
//-----------------------------------------------------------------------------
bool Nex_OpenParticipation(
    nn::nex::GatheringID gatheringID)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Nex_OpenParticiaption:MatchmakeExtensionClient is not created.\n");
    }

    nn::nex::ProtocolCallContext oContext;
    if (s_pMatchMakingClient->OpenParticipation(&oContext, gatheringID))
    {
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            Nex_ReportProtocolCallContext(oContext, "Nex_OpenParticipation:Failed to open participation\n");
            return false;
        }
    }
    else
    {
        PIA_SAMPLE_PRINT("Nex_OpenParticipation:OpenParticipation failed (Startup)\n");
        return false;
    }
    return true;
}

//-----------------------------------------------------------------------------
// マッチメイクセッションから抜ける
//-----------------------------------------------------------------------------
bool Nex_LeaveMatchmakeSession(
    nn::nex::GatheringID gatheringID,
    nn::nex::PrincipalID ownerPID,
    bool hostMigration)
{
    PIA_SAMPLE_PRINT("Nex_LeaveMatchmakeSession\n");

    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Nex_AutoMatchmake:MatchmakeExtensionClient is not created.\n");
    }

    if (gatheringID == INVALID_GATHERINGID)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Argument gatheringId is invalid.\n");
    }

    // サーバとの接続状態を確認します
    if (!s_pMatchMakingClient->IsConnected())
    {
        PIA_SAMPLE_PRINT("Already disconnected to server.\n");
        return true;
    }

    nn::nex::ProtocolCallContext oContext;

    PIA_SAMPLE_PRINT(
        "[PID Info >> OwnerPID:%llu MyPID:%llu]\n",
        static_cast<uint64_t>(ownerPID),
        static_cast<uint64_t>(s_pMatchMakingClient->GetCredentials()->GetPrincipalID()));

    // マッチメイクセッションオーナーの場合、かつ親交代しない場合はマッチメイクセッションを破棄します
    if (s_pMatchMakingClient->GetCredentials()->GetPrincipalID() == ownerPID && !hostMigration)
    {
        PIA_SAMPLE_PRINT("Unregister gathering ... \n");
        if (!s_pMatchMakingClient->UnregisterGathering(&oContext, gatheringID))
        {
            PIA_SAMPLE_PRINT("UnregisterGathering call failed ... \n");
            return false;
        }

        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            Nex_ReportProtocolCallContext(oContext, "failed to Unregister gathering!!\n");
            return false;
        }
        PIA_SAMPLE_PRINT("Unregister gathering successful.\n");
    }
    // オーナーで親交代する場合、またはオーナー以外の場合はマッチメイクセッションから抜けます
    else
    {
        PIA_SAMPLE_PRINT("End participation ... \n");
        if (!s_pMatchMakingClient->EndParticipation(&oContext, gatheringID))
        {
            PIA_SAMPLE_PRINT("Failed to end participation\n");
            return false;
        }

        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            // すでに削除されている場合は正常終了とみなす
            if (oContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionVoid))
            {
                PIA_SAMPLE_PRINT("matchmake session is already unregistered. don't care\n");
                return true;
            }
            s_MatchmakeNetworkErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(oContext.GetOutcome());
            Nex_ReportProtocolCallContext(oContext, "Failed to end participation\n");
            return false;
        }
    }

    return true;
}

bool Nex_GetHostStationURLs(
    nn::nex::GatheringID joinedGatheringID,
    nn::nex::qList<nn::nex::StationURL>* pStationURLs)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Nex_GetHostStationURLs:MatchmakeExtensionClient is not created.\n");
    }

    // セッションの URL をリストアップし、それらに参加を試みます
    nn::nex::ProtocolCallContext oContext;
    if (!s_pMatchMakingClient->GetSessionURLs(&oContext, joinedGatheringID, pStationURLs))
    {
        PIA_SAMPLE_PRINT("Failed Request URLs...\n");
        return false;
    }

    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        // Join したマッチメイクセッションが削除されていることもありえます。
        if (oContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionVoid))
        {
            PIA_SAMPLE_PRINT("matchmake session is already unregistered.\n");
        }
        s_MatchmakeNetworkErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(oContext.GetOutcome());
        Nex_ReportProtocolCallContext(oContext, "Failed to obtain a list of URLs to connect to the game host\n");
        return false;
    }

    return true;
}


//-----------------------------------------------------------------------------
// マッチメイクセッションを検索する
//-----------------------------------------------------------------------------
bool Nex_SearchMatchmakeSession(
    uint32_t matchmakeMode,
    uint16_t matchmakeMaxParticipants,
    uint32_t rangeSize,
    nn::nex::qList<nn::nex::GatheringHolder>* plistGatheringHolder)
{
    PIA_SAMPLE_PRINT("Nex_SearchMatchmakeSession:start.\n");

    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Nex_SearchMatchmakeSession:MatchmakeExtensionClient is not created.\n");
    }

    /* 検索条件設定 開始 */
    // 各設定の詳細は CTR-NEX API リファレンスなどを参照してください。
    nn::nex::MatchmakeSessionSearchCriteria criteria;

    // 最少・最大参加人数
    criteria.SetMinParticipants(1);
    criteria.SetMaxParticipants(matchmakeMaxParticipants);

    // アプリケーション固有のゲームモード
    criteria.SetGameMode(matchmakeMode);

    // アプリケーションで自由に定義できる検索属性値（範囲指定版）
    criteria.SetAttributeWithRange(MatchmakeAttributeIndex, 0, MatchmakeAttributeMaxValue);

    // マッチメイクシステムタイプ
    criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // 参加可能なセッションのみ検索
    criteria.SetExcludeLocked(true);

    // 満員のものを除外する
    criteria.SetVacantOnly(true);
    /* 検索条件設定 終了 */

    // 検索範囲
    nn::nex::ResultRange range;
    range.SetSize(rangeSize);  // 最大取得数
    range.SetOffset(0);        // オフセット値

    // 保存先リストを空にしておく
    plistGatheringHolder->clear();

    nn::nex::ProtocolCallContext searchContext;
    nn::pia::Result result;

    // 非同期検索処理の開始
    if (!s_pMatchMakingClient->BrowseMatchmakeSession(&searchContext, criteria, range, plistGatheringHolder))
    {
        // 非同期検索処理の開始に失敗
        PIA_SAMPLE_PRINT("Nex_SearchMatchmakeSession:BrowseMatchmakeSession failed.(Startup)\n");
        return false;
    }
    else
    {
        // 非同期検索処理の終了待ち
        searchContext.Wait();
        if (!searchContext.GetOutcome())
        {
            // 非同期検索処理に失敗
            PIA_SAMPLE_PRINT("Nex_SearchMatchmakeSession:BrowseMatchmakeSession failed.(Search)\n");

            // 保存先リストを空にしておく
            plistGatheringHolder->clear();
            s_MatchmakeNetworkErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(searchContext.GetOutcome());
            return false;
        }
    }

    return true;
}

//-----------------------------------------------------------------------------
// マッチメイクセッションを作成する
//-----------------------------------------------------------------------------
bool Nex_CreateMatchmakeSession(
    nn::nex::GatheringID* pJoinedGatheringID,
    uint32_t matchmakeMode,
    uint16_t matchmakeMaxParticipants,
    const char* matchmakeDescription)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Nex_CreateMatchmakeSession:MatchmakeExtensionClient is not created.\n");
    }

    // セッションを新規に作成
    nn::nex::MatchmakeSession* pMatchmakeSession = qNew nn::nex::MatchmakeSession();

    // マッチメイクセッションの設定

    // MatchmakeSessionの説明文字列 設定は任意です。
    pMatchmakeSession->SetDescription(matchmakeDescription);

    // 最小・最大参加人数
    pMatchmakeSession->SetMinParticipants(1);
    pMatchmakeSession->SetMaxParticipants(matchmakeMaxParticipants);

    // アプリケーション固有のゲームモード
    pMatchmakeSession->SetGameMode(matchmakeMode);

    // アプリケーションで自由に定義できる検索属性値
    // サンプルとして、属性値を指定範囲内でランダムにします。
    pMatchmakeSession->SetAttribute(MatchmakeAttributeIndex, nn::nex::Platform::GetRandomNumber(MatchmakeAttributeMaxValue + 1));

    // マッチメイクシステムタイプ
    pMatchmakeSession->SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // アプリケーションで自由にセットできるバイナリデータ
    // サンプルとして、適当な値をセットします。
    nn::nex::qVector<nn::nex::qUnsignedInt8> applicationBuffer;
    applicationBuffer.push_back(1);
    applicationBuffer.push_back(2);
    applicationBuffer.push_back(3);
    pMatchmakeSession->SetApplicationBuffer(applicationBuffer);

    // オーナーがログアウトしたときにOwnerの委譲を行うか
    pMatchmakeSession->SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    // セッションを作成した瞬間から参加可能にするか
    // ここでは false にしておき、 Session （P2P通信）の準備ができてから OpenParticipation します。
    pMatchmakeSession->SetOpenParticipation(false);

    // MatchmakeSessionインスタンスは gatheringHolder のデストラクタで破壊されます
    nn::nex::GatheringHolder gatheringHolder(pMatchmakeSession);
    nn::nex::ProtocolCallContext createContext;

    s_pMatchmakeSessionKey->clear();
    // 非同期マッチメイクセッション作成処理の開始
    if (s_pMatchMakingClient->CreateMatchmakeSession(&createContext, gatheringHolder, pJoinedGatheringID, s_pMatchmakeSessionKey))
    {
        // 非同期マッチメイクセッション作成処理の終了待ち
        createContext.Wait();
        if (!createContext.GetOutcome())
        {
            // 非同期マッチメイクセッション作成処理に失敗
            s_MatchmakeNetworkErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(createContext.GetOutcome());
            return false;
        }
    }
    else
    {
        // 非同期マッチメイクセッション作成処理の開始に失敗
        return false;
    }

    PIA_SAMPLE_PRINT(
        "[PID Info >> JoinedGID:%d OwnerPID:%llu HostPID:%llu MyPID:%llu]\n",
        gatheringHolder->GetID(),
        static_cast<uint64_t>(gatheringHolder->GetOwnerPID()),
        static_cast<uint64_t>(gatheringHolder->GetHostPID()),
        static_cast<uint64_t>(s_pMatchMakingClient->GetCredentials()->GetPrincipalID()));

    return true;
}

//-----------------------------------------------------------------------------
// マッチメイクセッションを参加する
//-----------------------------------------------------------------------------
bool Nex_JoinMatchmakeSession(nn::nex::GatheringID gatheringID)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_ASSERTMSG(false, "Nex_JoinMatchmakeSession:MatchmakeExtensionClient is not created.\n");
    }

    nn::nex::ProtocolCallContext joinContext;

    s_pMatchmakeSessionKey->clear();
    // 非同期マッチメイクセッション参加処理の開始
    if (s_pMatchMakingClient->JoinMatchmakeSession(&joinContext, gatheringID, s_pMatchmakeSessionKey))
    {
        // 非同期マッチメイクセッション参加処理の終了待ち
        joinContext.Wait();
        if (!joinContext.GetOutcome())
        {
            // 非同期マッチメイクセッション参加処理に失敗
            // 失敗理由のハンドリング
            if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionFull))
            {
                PIA_SAMPLE_PRINT("JoinMatchmakeSession Error (SessionFull)\n");
            }
            else if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionVoid))
            {
                PIA_SAMPLE_PRINT("JoinMatchmakeSession Error (SessionVoid)\n");
            }
            else
            {
                if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, DeniedByParticipants))
                {
                    PIA_SAMPLE_PRINT("JoinMatchmakeSession Error (DeniedByParticipants)\n");
                }
#if NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 0, 0)
                else if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, ParticipantInBlockList))
#else
                else if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, ParticipantInBlackList))
#endif
                {
                    PIA_SAMPLE_PRINT("JoinMatchmakeSession Error (ParticipantInBlackList)\n");
                }
                else
                {
                    PIA_SAMPLE_PRINT("JoinMatchmakeSession Error (Unknown)\n");
                }
                s_MatchmakeNetworkErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(joinContext.GetOutcome());
            }
            return false;
        }
    }
    else
    {
        PIA_SAMPLE_PRINT("JoinMatchmakeSession Error (Startup)\n");
        // 非同期マッチメイクセッション参加処理の開始に失敗
        return false;
    }

    return true;
}

uint32_t Nex_GetMatchmakeNetworkErrorCode()
{
    return s_MatchmakeNetworkErrorCode;
}

void Nex_ResetMatchmakeNetworkErrorCode()
{
    s_MatchmakeNetworkErrorCode = 0;
}

nn::nex::qVector<nn::nex::qByte>* Nex_GetMatchmakeSessionSignatureKey()
{
    return s_pMatchmakeSessionKey;
}

nn::pia::common::SignatureSetting* Nex_GetMatchmakeSessionSignatureSetting()
{
    nn::pia::Result result = nn::pia::inet::NexFacade::ConvertNexSignatureKeyToSignatureSetting(
        *s_pMatchmakeSessionKey,
        &s_SignatureSetting);
    if (result.IsFailure())
    {
        return NULL;
    }
    return &s_SignatureSetting;
}

nn::nex::PrincipalID Nex_GetOwnerPID()
{
    return s_OwnerPID;
}

void Nex_SetOwnerPID(nn::nex::PrincipalID id)
{
    s_OwnerPID = id;
}

//-----------------------------------------------------------------------------------------
// プロトコルコールコンテキストのエラーをログに出力します
//-----------------------------------------------------------------------------------------
void Nex_ReportProtocolCallContext(nn::nex::ProtocolCallContext& context, const char* pHeaderText)
{
    // 未使用パラメータの警告を抑える
    (void)context;

    if (pHeaderText)
    {
        PIA_SAMPLE_PRINT(pHeaderText);
    }

    PIA_SAMPLE_PRINT("Nex_ReportProtocolCallContext:(Facility:%d Code:0x%X NErr:%d)\n",
                   context.GetOutcome().GetFacility(),
                   context.GetOutcome().GetReturnCode(),
                   nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(context.GetOutcome()));
}
}
}  // end of namespace nns::pia

#endif
