﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Net_Setup.h"

#if NNS_PIA_ENABLE_INET

#if NN_PIA_CTR
#pragma diag_suppress 1300
#endif

#include <nex.h>

#include <OnlineCore/src/version/nex_version.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/SystemErrorResult.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/Type.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/UpdatableTimeProvider.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Transport/UDP/Packet.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportError.h>
#include <OnlineCore/src/Transport/Interface/TransportAdapter.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/Session.h>
#include <NetZ/src/Core/DOCore/SessionDescription.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DirectStream.h>
#include <NetZ/src/Core/DOCore/DirectTransport.h>
#include <NetZ/src/Extensions/SessionSelection/ConsoleSessionEnumerator.h>
#include <NetZ/HostMigration.h>
#include <RendezVous/Services/NATTraversal/src/Client/NATTraversalClient.h>

#if NN_PIA_CTR
#pragma diag_warning 1300
#endif

#if NN_PIA_CAFE
#include <cafe/mem.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/mem.h>
#include <nn/account.h>
#include <nn/account/account_Result.h>
#include <nn/err.h>
#endif

#define NNS_PIA_COPY_LOGIN_ERROR(p, r)  if (p != NULL)      \
                                        {                   \
                                            *p = r;         \
                                        }

namespace
{
// Finalizeの多重呼び出しもOKとする
bool s_Initialized = false;
uint32_t s_InitializedCount = 0;

bool s_Logined = false;

// メインメモリのサイズ
size_t s_MemorySize = 0;

// 緊急用メモリのサイズ
size_t s_EmergencyMemorySize = 0;

// メインメモリ
void* s_pMemory = NULL;

// ヒープ枯渇時に使用する緊急用メモリ
void* s_pEmergencyMemory = NULL;

// メモリ使用量
size_t s_MemUsage = 0;
size_t s_MemMaxUsage = 0;

#if NN_PIA_CTR
// NEX用ヒープ
nn::fnd::ThreadSafeExpHeap s_NexHeap;
#elif NN_PIA_CAFE
MEMHeapHandle s_NexHeap;
#elif NN_PIA_NINTENDOSDK
nn::mem::StandardAllocator s_NexHeap;
#endif

// NgsFacade インスタンス
nn::nex::NgsFacade* s_pNgsFacade = NULL;

//-----------------------------------------------------------------------------------------
// 内部関数の実装
//-----------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------
// AllocFunction から呼ばれる内部メモリ関数
//-----------------------------------------------------------------------------------------
void* InnerAlloc(size_t size)
{
#if NN_PIA_CTR
    void* pBuffer = s_NexHeap.Allocate(size, 32);
#elif NN_PIA_NINTENDOSDK
    void* pBuffer = s_NexHeap.Allocate(size);
#elif NN_PIA_CAFE
    void* pBuffer = MEMAllocFromExpHeapEx(s_NexHeap, size, 8);
#endif
    if (pBuffer == NULL)
    {
        return NULL;
    }

#if NN_PIA_CTR
    s_MemUsage += s_NexHeap.GetSizeOf(pBuffer);
#elif NN_PIA_CAFE
    s_MemUsage += MEMGetSizeForMBlockExpHeap(pBuffer);
#elif NN_PIA_NINTENDOSDK
    s_MemUsage += s_NexHeap.GetSizeOf(pBuffer);
#endif
    if (s_MemUsage > s_MemMaxUsage)
    {
        s_MemMaxUsage = s_MemUsage;
    }

    return pBuffer;
}

//-----------------------------------------------------------------------------------------
// FreeFunction から呼ばれる内部メモリ関数
//-----------------------------------------------------------------------------------------
void InnerFree(void* ptr)
{
#if NN_PIA_CTR
    s_MemUsage -= s_NexHeap.GetSizeOf(ptr);
    s_NexHeap.Free(ptr);
#elif NN_PIA_CAFE
    s_MemUsage -= MEMGetSizeForMBlockExpHeap(ptr);
    MEMFreeToExpHeap(s_NexHeap, ptr);
#elif NN_PIA_NINTENDOSDK
    s_MemUsage -= s_NexHeap.GetSizeOf(ptr);
    s_NexHeap.Free(ptr);
#endif
}

//-----------------------------------------------------------------------------------------
// アプリケーション定義のメモリ解放関数
//-----------------------------------------------------------------------------------------
void FreeFunction(void* pointer)
{
    InnerFree(pointer);
}

//-----------------------------------------------------------------------------------------
//緊急用のメモリを解放します
//-----------------------------------------------------------------------------------------
void ReleaseReservedMemory()
{
    if (s_pEmergencyMemory)
    {
        PIA_SAMPLE_PRINT("ReleaseReservedMemory\n");

        size_t nowUsage = s_MemUsage;
        size_t nowMaxUsage = s_MemMaxUsage;

        FreeFunction(s_pEmergencyMemory);

        s_pEmergencyMemory = NULL;

        s_MemUsage = nowUsage;
        s_MemMaxUsage = nowMaxUsage;
    }
}

//-----------------------------------------------------------------------------------------
// アプリケーション定義のメモリ確保関数
//-----------------------------------------------------------------------------------------
void* AllocFunction(size_t size)
{
    void* p = InnerAlloc(size);

    if (p == NULL)
    {
        PIA_SAMPLE_PRINT("AllocFunction: No enough memory. Reserved memory will be used. Please shutdown NEX library.\n");
        ReleaseReservedMemory();

        p = InnerAlloc(size);
        PIA_SAMPLE_ASSERT(p != NULL);
        if (p == NULL)
        {
            PIA_SAMPLE_PANIC("Reserved memory is not enough");
        }
    }

    return p;
}

//-----------------------------------------------------------------------------------------
// 緊急用のメモリを確保します
//-----------------------------------------------------------------------------------------
void AcquireReservedMemory()
{
    if (s_pEmergencyMemory == NULL)
    {
        PIA_SAMPLE_PRINT("AcquireReservedMemory\n");

        size_t nowUsage = s_MemUsage;
        size_t nowMaxUsage = s_MemMaxUsage;

        s_pEmergencyMemory = AllocFunction(s_EmergencyMemorySize);

        s_MemUsage = nowUsage;
        s_MemMaxUsage = nowMaxUsage;
    }
}

//-----------------------------------------------------------------------------------------
// NEX 用ヒープの初期化（ExpHeap）
//-----------------------------------------------------------------------------------------
void InitializeRuntimeMemory(size_t memorySize, size_t emergencyMemorySize)
{
    s_MemorySize = memorySize;
    s_EmergencyMemorySize = emergencyMemorySize;

    size_t total = memorySize + emergencyMemorySize;
    PIA_SAMPLE_PRINT("InitializeRuntimeMemory:MemorySize:%d, ReservedMemorySize:%d, Total:%d\n", s_MemorySize, s_EmergencyMemorySize, total);

#if NN_PIA_CTR
    // ワークメモリを確保する
    s_pMemory = new uint8_t[total];
    // ヒープを初期化する
    s_NexHeap.Initialize(NN_ANY_TO_UPTR(s_pMemory), total);
#elif NN_PIA_NINTENDOSDK
    // ワークメモリを確保する(4KBの倍数、16KB以上であること、アライメントはアロケータ側で適切に処理してくれるのでサイズだけ合わせる）
    total = ((total / 4096) + 1) * 4096;
    if (total < 16 * 1024)
    {
        total = 16 * 1024;
    }
    s_pMemory = new uint8_t[total];
    s_NexHeap.Initialize(s_pMemory, total);
#elif NN_PIA_CAFE
    // デフォルトヒープからワークメモリを確保する（アライメントは 8byte）
    s_pMemory = MEMAllocFromDefaultHeapEx(total, 8);
    // ヒープを初期化する
    s_NexHeap = MEMCreateExpHeapEx(s_pMemory, total, MEM_HEAP_OPT_0_CLEAR | MEM_HEAP_OPT_THREAD_SAFE);
#endif

    // メモリ関数を登録する
    nn::nex::MemoryManager::SetBasicMemoryFunctions(AllocFunction, FreeFunction);

    if (s_EmergencyMemorySize != 0)
    {
        // 緊急用メモリを確保する
        AcquireReservedMemory();
    }
}

//-----------------------------------------------------------------------------------------
// NEX 用ヒープの終了処理（ExpHeap）
//-----------------------------------------------------------------------------------------
void FinalizeRuntimeMemory()
{
    if (s_EmergencyMemorySize != 0)
    {
        // 緊急用メモリが残っていれば解放する
        ReleaseReservedMemory();
    }

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(
        "FinalizeRuntimeMemory:Nex memory usage: Now memory usage:%d, Max memory usage:%d / %d\n",
        s_MemUsage,
        s_MemMaxUsage,
        s_MemorySize + s_EmergencyMemorySize);

#if NN_PIA_CTR
    // メモリリークしている
    if (s_MemUsage > 0)
    {
        // リークしているメモリを表示する
        s_NexHeap.Dump();
        // リークしているメモリを無効化する
        s_NexHeap.Invalidate();
    }
    // ヒープを破棄する
    s_NexHeap.Finalize();
    // ワークメモリを破棄する
    delete[] s_pMemory;
#elif NN_PIA_NINTENDOSDK
    s_NexHeap.Finalize();
    // ワークメモリを破棄する
    delete[] static_cast<uint8_t*>(s_pMemory);
#elif NN_PIA_CAFE
    // メモリリークしている
    if (s_MemUsage > 0)
    {
        // ヒープの情報をダンプします
        MEMDumpHeap(s_NexHeap);
    }
    // ヒープを破棄します
    MEMDestroyExpHeap(s_NexHeap);
    // ワークメモリを解放する
    MEMFreeToDefaultHeap(s_pMemory);
#endif

    s_MemUsage = 0;
    s_MemMaxUsage = 0;
}
}

namespace nns
{
namespace pia
{
//-----------------------------------------------------------------------------------------
// 外部関数の実装
//-----------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------
// NEX の初期化処理
//-----------------------------------------------------------------------------------------
nn::pia::Result Nex_InitializeNex(size_t memorySize, size_t emergencyMemorySize)
{
    if (!s_Initialized)
    {
        ++s_InitializedCount;
        PIA_SAMPLE_PRINT("Nex_InitializeNex [%d]\n", s_InitializedCount);

        // NEX 用ヒープの初期化処理
        PIA_SAMPLE_PRINT("Nex_InitializeNex:InitializeRuntimeMemory\n");
        InitializeRuntimeMemory(memorySize, emergencyMemorySize);

        // グローバルインスタンスなどの初期化処理
        nn::nex::GlobalVariables::AcquireInstance();

        // スレッドモード設定
        nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeUnsafeTransportBuffer);

        // NEX トレース設定
        nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
        nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

        s_Initialized = true;
    }
    return nn::pia::ResultSuccess();
}

//-----------------------------------------------------------------------------------------
// NEX の終了処理
//-----------------------------------------------------------------------------------------
void Nex_FinalizeNex()
{
    if (!s_Initialized)
    {
        PIA_SAMPLE_PRINT("Nex_FinalizeNex:already finalized\n");
        return;
    }

    if (s_Logined)
    {
        PIA_SAMPLE_PRINT("Nex_FinalizeNex:Logout is not completed.\n");
        return;
    }

    PIA_SAMPLE_PRINT("Nex_FinalizeNex [%d]\n", s_InitializedCount);

    // グローバルインスタンスなどの終了処理
    nn::nex::GlobalVariables::ReleaseInstance();

    // NEX 用ヒープの終了処理
    FinalizeRuntimeMemory();

    s_Initialized = false;
}


//-----------------------------------------------------------------------------------------
// ゲームサーバーへのログイン
//-----------------------------------------------------------------------------------------
#if NN_PIA_NINTENDOSDK
bool Nex_Login(nn::Result* pLoginErrorResult, uint32_t timeout)
{
    // ゲームサーバにログインする
    nn::nex::ProtocolCallContext oContext;

    PIA_SAMPLE_PRINT("Nex_Login:called.\n");

    if (s_Logined)
    {
        PIA_SAMPLE_PRINT("Nex_Login:Alreayd logged-in.\n");
        return true;
    }

    // NgsFacade インスタンスの生成
    PIA_SAMPLE_ASSERT(s_pNgsFacade == NULL);
    s_pNgsFacade = qNew nn::nex::NgsFacade();
    PIA_SAMPLE_ASSERT(s_pNgsFacade != NULL);
    if (s_pNgsFacade == NULL)
    {
        PIA_SAMPLE_PRINT("Nex_Login:NgsFacade instance is not created.\n");
        return false;
    }

#if NN_PIA_NINTENDOSDK_NX
    UserAccountInfo* pInfo = Net_GetMainUserAccountInfo();
    if (pInfo == NULL)
    {
        PIA_SAMPLE_PRINT("Nex_Login:No user account is found to have NetworkServiceAccountId.\n");
        return false;
    }

#if NNS_PIA_USE_ENSURENETWORKSERVICEACCOUTAVAILABLE
#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 1, 1))
    while (true)
    {
        // アカウントがネットワークサービスを利用可能な状態かチェック（本関数は UI 操作によりブロックする可能性があります）
        PIA_SAMPLE_PRINT("Nex_Login: nn::account::EnsureNetworkServiceAccountAvailable\n");
        nn::Result sdkResult = nn::account::EnsureNetworkServiceAccountAvailable(pInfo->userHandle);
        if (nn::account::ResultCancelledByUser::Includes(sdkResult))
        {
            // ユーザーによるキャンセル処理
            PIA_SAMPLE_PRINT("Nex_Login: nn::account::EnsureNetworkServiceAccountAvailable is cancelled.\n");
            return false;
        }
        PIA_SAMPLE_PRINT("Nex_Login: nn::account::EnsureNetworkServiceAccountAvailable is success.\n");

        // ネットワークサービスアカウント ID を取得
        // ユーザーはネットワークサービスアカウントを付け替えることができるため、アプリケーション起動ごとに変わっている可能性がある。
        nn::account::NetworkServiceAccountId userNsaId;
        sdkResult = nn::account::GetNetworkServiceAccountId(&userNsaId, pInfo->userHandle);
        if (!sdkResult.IsSuccess())
        {
            PIA_SAMPLE_PRINT("Nex_Login: nn::account::GetNetworkServiceAccountId is failed. [%08x]\n", sdkResult.GetInnerValueForDebug());
            return false;
        }

        // ネットワークサービスアカウントの ID トークンキャッシュを取得
        char userNsaIdToken[nn::account::NetworkServiceAccountIdTokenLengthMax];
        nn::account::AsyncContext accountAsyncContext;
        nn::os::SystemEvent accountEvent;

        sdkResult = nn::account::EnsureNetworkServiceAccountIdTokenCacheAsync(&accountAsyncContext, pInfo->userHandle);
        if (!sdkResult.IsSuccess())
        {
            PIA_SAMPLE_PRINT("Calling nn::account::EnsureNetworkServiceAccountIdTokenCacheAsync is failed. [%08x]\n", sdkResult.GetInnerValueForDebug());
            return false;
        }

        // 処理の可読性を重視してここでイベントを待ちます。（キャッシュが効いていない場合は長時間ブロックする可能性があります）
        accountAsyncContext.GetSystemEvent(&accountEvent);
        accountEvent.Wait();

        sdkResult = accountAsyncContext.GetResult();
        if (sdkResult.IsSuccess())
        {
            size_t idTokenLength;
            sdkResult = nn::account::LoadNetworkServiceAccountIdTokenCache(&idTokenLength, userNsaIdToken, nn::account::NetworkServiceAccountIdTokenLengthMax, pInfo->userHandle);
            PIA_SAMPLE_ASSERT(sdkResult.IsSuccess());

            PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login\n");
            nn::nex::qResult nexResult = s_pNgsFacade->Login(&oContext, NNS_PIA_LOGIN_GAME_ID, NNS_PIA_LOGIN_ACCESS_KEY, userNsaId, userNsaIdToken, nn::nex::TimeInterval(timeout));
            if (!nexResult)
            {
                // ログイン処理の起動に失敗
                PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login is failed.\n");
                nn::err::ShowError(nexResult.GetNnResult());
                NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, nexResult.GetNnResult());
                return false;
            }
            // 非同期ログイン処理の終了待ち
            // ProtocolCallContext::Wait の内部で nn::nex::Scheduler::Dispatch が呼ばれ、非同期処理が進行します。
            // 処理の可読性を重視してここで行います。（長時間ブロックする可能性があります）
            oContext.Wait();
            if (!oContext.GetOutcome())
            {
                // Loginメソッド呼び出し時の実際のエラー値は、nn::nex::NgsFacade::GetLastLoginNnResult() から取得します。
                nn::Result loginResult = s_pNgsFacade->GetLastLoginNnResult();
                PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login is failed. LoginErrorCode: %08x\n", loginResult.GetInnerValueForDebug());
                // ログインエラーはエラービューワーアプレットに渡します
                nn::err::ShowError(loginResult);
                NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, loginResult);
                return false;
            }
            break;
        }

        if (nn::account::ResultNetworkServiceAccountUnavailable::Includes(sdkResult))
        {
            // ログイン時にネットワークサービスアカウントが利用できないことが発覚
            // nn::account::EnsureNetworkServiceAccountAvailable() でエラーを解消できるため、再試行する。
            // 再試行や本体システム UI の表示が困難な場合は nn::error::ShowError() に渡してもよい。
            continue;
        }
        else if (nn::account::ResultNetworkCommunicationError::Includes(sdkResult))
        {
            // ログイン時に通信関係のエラーが発生
            // これにはアプリケーションの更新が必要な場合や、本体更新が必要な場合も含まれる。
            // nn::error::ShowError() に渡し、詳細なエラー内容をユーザーに提示する必要がある。
            nn::err::ShowError(sdkResult);
            return false;
        }
    }
#else
    while (true)
    {
        // アカウントがネットワークサービスを利用可能な状態かチェック（本関数は UI 操作によりブロックする可能性があります）
        PIA_SAMPLE_PRINT("Nex_Login: nn::account::EnsureNetworkServiceAccountAvailable\n");
        nn::Result sdkResult = nn::account::EnsureNetworkServiceAccountAvailable(pInfo->userHandle);
        if (nn::account::ResultCancelledByUser::Includes(sdkResult))
        {
            // ユーザーによるキャンセル処理
            PIA_SAMPLE_PRINT("Nex_Login: nn::account::EnsureNetworkServiceAccountAvailable is cancelled.\n");
            return false;
        }
        PIA_SAMPLE_PRINT("Nex_Login: nn::account::EnsureNetworkServiceAccountAvailable is success.\n");

        PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login\n");
        nn::nex::qResult nexResult = s_pNgsFacade->Login(&oContext, NNS_PIA_LOGIN_GAME_ID, NNS_PIA_LOGIN_ACCESS_KEY, pInfo->userHandle, nn::nex::TimeInterval(timeout));
        if (!nexResult)
        {
            // ログイン処理の起動に失敗
            PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login is failed.\n");
            nn::err::ShowError(nexResult.GetNnResult());
            NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, nexResult.GetNnResult());
            return false;
        }
        // 非同期ログイン処理の終了待ち
        // ProtocolCallContext::Wait の内部で nn::nex::Scheduler::Dispatch が呼ばれ、非同期処理が進行します。
        // 処理の可読性を重視してここで行います。
        oContext.Wait();
        if (!oContext.GetOutcome())
        {
            // Loginメソッド呼び出し時の実際のエラー値は、nn::nex::NgsFacade::GetLastLoginNnResult() から取得します。
            nn::Result loginResult = s_pNgsFacade->GetLastLoginNnResult();
            PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login is failed. LoginErrorCode: %08x\n", loginResult.GetInnerValueForDebug());
            // Loginメソッド呼び出し後のエラー発生時には、 nn::account::ResultNetworkServiceAccountUnavailable 起因のものであるか判断します。
            if (nn::account::ResultNetworkServiceAccountUnavailable::Includes(loginResult))
            {
                // ログイン時にネットワークサービスアカウントが利用できなかった場合の処理。
                // この場合は、nn::account::EnsureNetworkServiceAccountAvailable() でエラーを解消できる可能性があるため、再試行する。
                continue;
            }
            else
            {
                // それ以外は、エラービューワーアプレットに渡します。
                nn::err::ShowError(loginResult);
                NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, loginResult);
                return false;
            }
        }
        break;
    }
#endif
    if (!s_pNgsFacade->IsConnected())
    {
        PIA_SAMPLE_PRINT("Nex_Login:NgsFacade::IsConnected failed.\n");
        // ログイン成功直後に切断検知した場合もエラービューワーアプレットに渡します。
        nn::Result errorResult = s_pNgsFacade->IsConnected().GetNnResult();
        nn::err::ShowError(errorResult);
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, errorResult);
        return false;
    }
#else
    // nn::account::EnsureNetworkServiceAccountAvailable() でアカウントの利用可否チェックを行わない実装（製品コードでは非推奨）
    PIA_SAMPLE_PRINT("Nex_Login: nn::account::IsNetworkServiceAccountAvailable\n");
    bool isAvailable;
    nn::Result sdkResult = nn::account::IsNetworkServiceAccountAvailable(&isAvailable, pInfo->userHandle);
    if (sdkResult.IsFailure())
    {
        PIA_SAMPLE_PRINT("Nex_Login: nn::account::IsNetworkServiceAccountAvailable is failed. [%08x]\n", sdkResult.GetInnerValueForDebug());
        return false;
    }
    else if (!isAvailable)
    {
        PIA_SAMPLE_PRINT("Nex_Login: nn::account::IsNetworkServiceAccountAvailable is success, but it is not available.\n");
        return false;
    }
    PIA_SAMPLE_PRINT("Nex_Login: nn::account::IsNetworkServiceAccountAvailable is success\n");

    PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login\n");
    nn::nex::qResult nexResult = s_pNgsFacade->Login(&oContext, NNS_PIA_LOGIN_GAME_ID, NNS_PIA_LOGIN_ACCESS_KEY, pInfo->userHandle, nn::nex::TimeInterval(timeout));
    if (!nexResult)
    {
        // ログイン処理の起動に失敗
        PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login is failed.\n");
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, nexResult.GetNnResult());
        return false;
    }

    // 非同期ログイン処理の終了待ち
    // ProtocolCallContext::Wait の内部で nn::nex::Scheduler::Dispatch が呼ばれ、非同期処理が進行します。
    // 処理の可読性を重視してここで行います。
    oContext.Wait();
    if (!oContext.GetOutcome())
    {
        PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login is failed. LoginErrorCode: %08x\n", s_pNgsFacade->GetLastLoginNnResult().GetInnerValueForDebug());
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, s_pNgsFacade->GetLastLoginNnResult());
        return false;
    }
    if (!s_pNgsFacade->IsConnected())
    {
        PIA_SAMPLE_PRINT("Nex_Login:NgsFacade::IsConnected failed.\n");
        nn::Result errorResult = s_pNgsFacade->IsConnected().GetNnResult();
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, errorResult);
        return false;
    }
#endif
    // NN_PIA_NINTENDOSDK_WIN
#else
    // デバッグ機能でログインする
    // ユーザ名
    nn::nex::StringStream name;
    uint32_t debugUserId = 1001 + nn::nex::Platform::GetRandomNumber(4000);
    name << NNS_PIA_DEBUGLOGIN_USER_NAME << "_" << debugUserId;

    nn::nex::qResult nexResult;
    if (nn::pia::common::String(NNS_PIA_DEBUGLOGIN_HOST_URL).GetStringLength() == 0)
    {
        // デバッグログイン
        PIA_SAMPLE_PRINT("DebugLogin: Host=%s Port=%d User=%s Pass=%s Key=%s\n", NNS_PIA_DEBUGLOGIN_HOST_ADDRESS, NNS_PIA_DEBUGLOGIN_HOST_PORT, name.CStr(), NNS_PIA_DEBUGLOGIN_USER_PASSWORD, NNS_PIA_DEBUGLOGIN_ACCESS_KEY);
        nexResult = s_pNgsFacade->DebugLoginDirect(&oContext, NNS_PIA_DEBUGLOGIN_HOST_ADDRESS, NNS_PIA_DEBUGLOGIN_HOST_PORT, name.CStr(), NNS_PIA_DEBUGLOGIN_USER_PASSWORD, NNS_PIA_DEBUGLOGIN_ACCESS_KEY, nn::nex::TimeInterval(timeout));
    }
    else
    {
        // デバッグログイン
        PIA_SAMPLE_PRINT("DebugLogin: Uri=%s User=%s Pass=%s Key=%s\n", NNS_PIA_DEBUGLOGIN_HOST_URL, name.CStr(), NNS_PIA_DEBUGLOGIN_USER_PASSWORD, NNS_PIA_DEBUGLOGIN_ACCESS_KEY);
        nexResult = s_pNgsFacade->DebugLoginDirect(&oContext, NNS_PIA_DEBUGLOGIN_HOST_URL, name.CStr(), NNS_PIA_DEBUGLOGIN_USER_PASSWORD, NNS_PIA_DEBUGLOGIN_ACCESS_KEY, nn::nex::TimeInterval(timeout));
    }
    if (!nexResult)
    {
        // ログイン処理の起動に失敗
        PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::DebugLoginDirect is failed.\n");
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, nexResult.GetNnResult());
        return false;
    }

    // 非同期ログイン処理の終了待ち
    // ProtocolCallContext::Wait の内部で nn::nex::Scheduler::Dispatch が呼ばれ、非同期処理が進行します。
    // 処理の可読性を重視してここで行います。
    oContext.Wait();
    if (!oContext.GetOutcome())
    {
        PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::DebugLoginDirect is failed. LoginErrorCode: %08x\n", s_pNgsFacade->GetLastLoginNnResult().GetInnerValueForDebug());
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, s_pNgsFacade->GetLastLoginNnResult());
        return false;
    }
    if (!s_pNgsFacade->IsConnected())
    {
        PIA_SAMPLE_PRINT("Nex_Login:NgsFacade::IsConnected failed.\n");
        nn::Result errorResult = s_pNgsFacade->IsConnected().GetNnResult();
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorResult, errorResult);
        return false;
    }
#endif

    // ログインが成功していれば NULL にはならない
    PIA_SAMPLE_ASSERT(s_pNgsFacade->GetCredentials() != NULL);

    PIA_SAMPLE_PRINT("Nex_Login:Login success.\n");
    s_Logined = true;
    return true;
}
#else
bool Nex_Login(int32_t* pLoginErrorCode, uint32_t timeout)
{
    // ゲームサーバにログインする
    nn::nex::ProtocolCallContext oContext;

    PIA_SAMPLE_PRINT("Nex_Login:called.\n");

    if (s_Logined)
    {
        PIA_SAMPLE_PRINT("Nex_Login:Alreayd logged-in.\n");
        return true;
    }

    // NgsFacade インスタンスの生成
    PIA_SAMPLE_ASSERT(s_pNgsFacade == NULL);
    s_pNgsFacade = qNew nn::nex::NgsFacade();
    PIA_SAMPLE_ASSERT(s_pNgsFacade != NULL);
    if (s_pNgsFacade == NULL)
    {
        PIA_SAMPLE_PRINT("Nex_Login:NgsFacade instance is not created.\n");
        return false;
    }

#if NN_PIA_CTR || NN_PIA_CAFE
    if (!s_pNgsFacade->Login(&oContext, NNS_PIA_LOGIN_GAME_ID, NNS_PIA_LOGIN_ACCESS_KEY, nn::nex::TimeInterval(timeout)))
    {
        // ログイン処理の起動に失敗
        PIA_SAMPLE_PRINT("Nex_Login:Login failed.\n");
        return false;
    }
    // 非同期ログイン処理の終了待ち
    // ProtocolCallContext::Wait の内部で nn::nex::Scheduler::Dispatch が呼ばれ、非同期処理が進行します。
    // 処理の可読性を重視してここで行います。
    oContext.Wait();
    if (!oContext.GetOutcome())
    {
        PIA_SAMPLE_PRINT("Nex_Login: NgsFacade::Login is failed. LoginErrorCode: %08x\n", s_pNgsFacade->GetLastLoginErrorCode());
        NNS_PIA_COPY_LOGIN_ERROR(pLoginErrorCode, s_pNgsFacade->GetLastLoginErrorCode());
        return false;
    }
#endif

    if (!s_pNgsFacade->IsConnected())
    {
        PIA_SAMPLE_PRINT("Nex_Login:NgsFacade::IsConnected failed.\n");
        return false;
    }
    // ログインが成功していれば NULL にはならない
    PIA_SAMPLE_ASSERT(s_pNgsFacade->GetCredentials() != NULL);

    PIA_SAMPLE_PRINT("Nex_Login:Login success.\n");
    s_Logined = true;
    return true;
}
#endif

//-----------------------------------------------------------------------------------------
// ゲームサーバーからログアウト
//-----------------------------------------------------------------------------------------
bool Nex_Logout()
{
    PIA_SAMPLE_PRINT("Nex_Logout:called.\n");

    if (s_Logined)
    {
        // ゲームサーバからログアウト
        nn::nex::ProtocolCallContext ccLogout;
        PIA_SAMPLE_PRINT("Nex_Logout:Logout\n");
        nn::nex::qResult qResult = s_pNgsFacade->Logout(&ccLogout);
        if (!qResult)
        {
            // ログアウト処理の起動に失敗
            PIA_SAMPLE_PRINT("NgsFacade::Logout Begin failed (%s:%d)", qResult.GetReturnCodeString(), qResult.GetReturnCode());

            // 処理中のCallContextを渡した
            if (ccLogout.GetState() == nn::nex::CallContext::CallInProgress)
            {
                PIA_SAMPLE_PRINT( "ccLogout.GetState() == nn::nex::CallContext::CallInProgress");
                PIA_ASSERT(false);
                return false; // ResultInvalidArgument
            }

            // Bind が実行されていない
            if (ccLogout.GetCredentials() == NULL)
            {
                PIA_SAMPLE_PRINT("ccLogout.GetCredentials() == NULL");
                PIA_ASSERT(false);
                return false; // ResultInvalidState
            }

            // その他、ネットワークに接続されていない場合など
            // 「通信エラーが発生しました」メッセージを表示（エラーコードは不要）
            // ログアウト・終了処理に移行
            PIA_SAMPLE_PRINT("NEX communication error occurred.");
            return false; // ResultGameServerProcessAborted
        }
        ccLogout.Wait();
    }

    if (s_pNgsFacade != NULL)
    {
        // NgsFacade インスタンスの破棄
        qDelete s_pNgsFacade;
        s_pNgsFacade = NULL;
    }
    s_Logined = false;
    return true;
}

//-----------------------------------------------------------------------------------------
// ゲームサーバーとの接続状態を取得
//-----------------------------------------------------------------------------------------
nn::pia::Result Nex_IsNgsConnected()
{
    nn::pia::Result retResult = nn::pia::ResultSuccess();

    if (s_pNgsFacade)
    {
        if (!s_pNgsFacade->IsConnected())
        {
            if (s_pNgsFacade->IsConnected().GetReturnCode() == QERROR(RendezVous, ConnectionDisconnected))
            {
                retResult = nn::pia::ResultNetworkConnectionIsLost();
            }
#if NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 2, 0)
            if (s_pNgsFacade->IsConnected().GetReturnCode() == QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin))
            {
                retResult = nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin();
            }
            else if (s_pNgsFacade->IsConnected().GetReturnCode() == QERROR(Transport, Unknown))
            {
                retResult = nn::pia::ResultNetworkConnectionIsLost();
            }
#endif
        }
    }
    return retResult;
}

nn::pia::PrincipalId Nex_GetLocalPrincipalId()
{
    if (s_pNgsFacade)
    {
        if (s_pNgsFacade->GetCredentials())
        {
            return s_pNgsFacade->GetCredentials()->GetPrincipalID();
        }
    }
    return 0;
}

void Nex_SchedulerDispatch()
{
    if (nn::nex::Scheduler::GetInstance() != NULL)
    {
        nn::nex::Scheduler::GetInstance()->DispatchAll();
    }
}

nn::nex::NgsFacade* Nex_GetNgsFacadeInstance()
{
    return s_pNgsFacade;
}
}
}  // end of namespace nns::pia
#endif  // end of NNS_PIA_ENABLE_INET
