﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "Profiler.h"
#include "Print.h"
#include "Defines.h"
#include "Alarm.h"

#include <nn/pia/pia_Assert.h>
#include <nn/pia/transport.h>
#include <nn/pia/session.h>


namespace
{
#if NN_PIA_CAFE
PIA_ATTRIBUTE_ALIGN(4) uint8_t sStack[256];
#elif NN_PIA_NINTENDOSDK
PIA_ATTRIBUTE_ALIGN(4096) uint8_t sStack[4096];  // nn::os::ThreadStackAlignment = 4096
#endif
}  // end of anonymous namespace

namespace nns
{
namespace pia
{

#if NN_PIA_NINTENDOSDK
uintptr_t Profiler::s_BusyLoopTimes = 0;  // キャリブレーションによって適切な値に調整されます。
#endif

Profiler* Profiler::s_pInstance = NULL;

void Profiler::CreateInstance()
{
#if NN_PIA_NINTENDOSDK
    CalibrateSpinWait();
#endif

    if (s_pInstance == NULL)
    {
        s_pInstance = new Profiler();
    }
}

void Profiler::DestroyInstance()
{
    if (s_pInstance != NULL)
    {
        delete s_pInstance;
        s_pInstance = NULL;
    }
}


Profiler::Profiler()
    : m_CountThread(),
#if NN_PIA_CTR || NN_PIA_CAFE || NN_PIA_NINTENDOSDK
      m_ProfileSpan(0),
#endif
      m_CurrentProfileCount(0),
      m_IsCounting(false),
      m_IsExitIdleThread(false),
      m_CurrentProfileStartTime(),
      m_pCurrentProfileResult(NULL)
{
    Alarm::Initialize();

#if NN_PIA_CTR
    m_CountThread.Start(RunIdleThread, this, 256, nn::os::LOWEST_THREAD_PRIORITY);
#elif NN_PIA_CAFE
    m_CountThread.Start(RunIdleThread, this, sStack, 256, 0, OS_PRIORITY_MAX);
#elif NN_PIA_NINTENDOSDK
    m_CountThread.Start(RunIdleThread, this, sStack, sizeof(sStack), nn::os::LowestThreadPriority);
#endif
}

Profiler::~Profiler()
{
    while (m_IsCounting)
    {
        SleepProfileSpan();
    }

    m_IsExitIdleThread = true;
    while (m_CountThread.IsAlive())
    {
        Thread::Sleep(1);
    }

    m_CountThread.Detach();
    m_CountThread.Finalize();

    Alarm::Finalize();
}


void Profiler::StartProfilingIdle(uint32_t second)
{
    if (m_IsCounting)
    {
        return;
    }

    StartCount(second, &m_IdleProfileResult);

    SleepProfileSpan();

    while (m_IsCounting)
    {
        // 念のため待つ
    }
}

void Profiler::StartProfilingBase(uint32_t second)
{
    if (m_IsCounting)
    {
        return;
    }

    StartCount(second, &m_BaseProfileResult);
}

void Profiler::StartProfilingTarget(uint32_t second)
{
    if (m_IsCounting)
    {
        return;
    }

    StartProfilePacketLoss();
    StartCount(second, &m_TargetProfileResult);
    StartPacketProfile();  //n1769
}

void Profiler::FinishProfiling()
{
    StopCount();
    StopProfilePacketLoss();
    StopPacketProfile();  //n1769
    Report();
    ReportPacketLoss();
    ReportPacketProfile();  //n1769
}

void Profiler::Report()
{
    if (m_IsCounting)
    {
        return;
    }

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("-- Profiler --\n");

    uint16_t stationNum = 0;
    if (nn::pia::session::Mesh::GetInstance() != NULL)
    {
        stationNum = nn::pia::session::Mesh::GetInstance()->GetStationNum();
    }
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("PiaSession StationNum: %d\n", stationNum);

    //    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("ProfileSpan : %d ms\n", static_cast<int32_t>(m_ProfileSpan.GetMilliSeconds()));

    if (m_IdleProfileResult.predictedCount > 0)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Idle Profile Result: %d (%d [ms], %d)\n", m_IdleProfileResult.predictedCount, m_IdleProfileResult.period.GetMSec(), m_IdleProfileResult.count);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Idle Profile Result: N/A\n");
    }

    if (m_BaseProfileResult.predictedCount > 0)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Base Profile Result: %d (%d [ms], %d)\n", m_BaseProfileResult.predictedCount, m_BaseProfileResult.period.GetMSec(), m_BaseProfileResult.count);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Base Profile Result: N/A\n");
    }

    if (m_TargetProfileResult.predictedCount > 0)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Target Profile Result: %d (%d [ms], %d)\n", m_TargetProfileResult.predictedCount, m_TargetProfileResult.period.GetMSec(), m_TargetProfileResult.count);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Target Profile Result: N/A\n");
    }

    if (
            m_IdleProfileResult.predictedCount > 0 && m_BaseProfileResult.predictedCount > 0 && m_TargetProfileResult.predictedCount > 0 && m_BaseProfileResult.predictedCount >= m_TargetProfileResult.predictedCount)
    {
        float rate = static_cast<float>(m_BaseProfileResult.predictedCount - m_TargetProfileResult.predictedCount) / m_IdleProfileResult.predictedCount * 100;
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Profile Result: %.2f %%\n", rate);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Profile Result : N/A\n");
    }
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("(m_IdleProfileResult.predictedCount, m_BaseProfileResult.predictedCount, m_TargetProfileResult.predictedCount) = (%d, %d, %d)\n",
        m_IdleProfileResult.predictedCount,
        m_BaseProfileResult.predictedCount,
        m_TargetProfileResult.predictedCount);
#if NN_PIA_ENABLE_CACHEDPRINT
    if (nn::pia::common::CachedPrint::GetInstance() != NULL)
    {
        nn::pia::common::CachedPrint::GetInstance()->Flush();
    }
#endif // NN_PIA_ENABLE_CACHEDPRINT
}

void Profiler::StartCount(uint32_t second, ProfileResult* pProfileResult)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Profiler::StartCount:%d\n", second);
    m_pCurrentProfileResult = pProfileResult;
    m_CurrentProfileCount = 0;
    m_CurrentProfileStartTime = nn::pia::common::Time::GetTime();
    m_IsCounting = true;

    if (second == 0)
    {
#if NN_PIA_CTR || NN_PIA_CAFE || NN_PIA_NINTENDOSDK
        m_ProfileSpan = 0;
#endif
        return;
    }

#if NN_PIA_CTR
    m_ProfileSpan = nn::fnd::TimeSpan::FromSeconds(second);
#elif NN_PIA_CAFE
    m_ProfileSpan = OSSecondsToTicks(second);
#elif NN_PIA_NINTENDOSDK
    m_ProfileSpan = nn::TimeSpan::FromSeconds(second);
#endif
    Alarm::Set(second * 1000, AlarmHandler, this);
}

void Profiler::StopCount()
{
    m_IsCounting = false;
    m_pCurrentProfileResult->count = m_CurrentProfileCount;
    nn::pia::common::Time currentTime = nn::pia::common::Time::GetTime();
    m_pCurrentProfileResult->period = currentTime - m_CurrentProfileStartTime;
    m_pCurrentProfileResult->CalcPredictedCount(m_IdleProfileResult.period);
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Profiler::StopCount:%d\n", m_pCurrentProfileResult->period.GetMSec());

    //    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Profiler::StopCount:%llu, %llu\n", (uint64_t)currentTime, (uint64_t)m_CurrentProfileStartTime);

    m_pCurrentProfileResult = NULL;
}

void Profiler::Count()
{
    while (m_IsExitIdleThread == false)
    {
        SpinWait(1);  // 1ミリ秒

        if (m_IsCounting)
        {
            ++m_CurrentProfileCount;
        }
    }
}


void Profiler::AlarmHandler(void* param)
{
    Profiler* pProfiler = reinterpret_cast<Profiler*>(param);
    pProfiler->FinishProfiling();
}


#if NN_PIA_CTR
void Profiler::RunIdleThread(void* pProfiler)
{
    reinterpret_cast<Profiler*>(pProfiler)->Count();
}

void Profiler::SleepProfileSpan()
{
    nn::os::Thread::Sleep(m_ProfileSpan);
}

void Profiler::SpinWait(int32_t msec)
{
    const int32_t cTicks = static_cast<int32_t>(nn::os::Tick::TICKS_PER_SECOND / 1000);
    for (int32_t i = 0; i < msec; ++i)
    {
        nn::os::ARM::SpinWaitCpuCycles(cTicks);
    }
}
#elif NN_PIA_CAFE
int Profiler::RunIdleThread(int, void* pProfiler)
{
    reinterpret_cast<Profiler*>(pProfiler)->Count();
    return 0;
}

void Profiler::SleepProfileSpan()
{
    OSSleepTicks(m_ProfileSpan);
}

void Profiler::SpinWait(int32_t msec)
{
    OSWaitMicroseconds(1000 * msec);
}
#elif NN_PIA_NINTENDOSDK
void Profiler::RunIdleThread(void* pProfiler)
{
    reinterpret_cast<Profiler*>(pProfiler)->Count();
}

void Profiler::SleepProfileSpan()
{
    nn::os::SleepThread(m_ProfileSpan);
}

void Profiler::SpinWait(int32_t msec)
{
    for (int32_t i = 0; i < msec; ++i)
    {
        Loop(s_BusyLoopTimes);
    }
}

// ビジーループの回数と経過時間の対応が一定していない場合に
// 実際にビジーループを回し、1msec 経過するのに必要な
// ビジーループ回数を調整します。
void Profiler::CalibrateSpinWait()
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("s_BusyLoopTimes(before) = %lld\n", s_BusyLoopTimes);

    static PIA_ATTRIBUTE_ALIGN(4096) uint8_t s_Stack[4096];  // nn::os::ThreadStackAlignment = 4096
    PIA_ATTRIBUTE_ALIGN(8) Thread thread;

    // 他のスレッドに邪魔されないよう、最高優先度で回るスレッドを立てます。
    thread.Start(CalibrateThreadFunc, NULL, s_Stack, sizeof(s_Stack), nn::os::HighestThreadPriority);

    thread.Join();
    thread.Finalize();

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("s_BusyLoopTimes(after)  = %lld\n", s_BusyLoopTimes);
}

void Profiler::CalibrateThreadFunc(void*)
{
    nn::os::Tick s;
    nn::os::Tick e;
    s = nn::os::GetSystemTick();
    s = nn::os::GetSystemTick();

    Loop(1000ul * 1000ul * 1000ul);
    e = nn::os::GetSystemTick();
    uint64_t times = (1000ull * 1000ull * 1000ull) / 1000 * nn::os::GetSystemTickFrequency() / nn::os::Tick(e - s).GetInt64Value();
    PIA_ASSERT(times <= 0xffffffff);
    s_BusyLoopTimes = static_cast<uintptr_t>(times);
}

void Profiler::Loop(uintptr_t count)
{
#if NN_PIA_NINTENDOSDK_NX
    asm volatile(
        "    cmp  %0, #0;"
        "    beq  2f;"
        "1:  subs  %0, %0,#1;"
        "    bne   1b;"
        "2:;"
        : "+r"(count)
        :
        : "cc");
#elif NN_PIA_NINTENDOSDK_WIN
    volatile uintptr_t dmy = 0;
    for (uintptr_t i = 0; i < count; ++i)
    {
        dmy += i;
    }
#else
#error "Not supported platform."
#endif
}

#elif NN_PIA_A || NN_PIA_B
//n1849:todo:not implemented
//int Profiler::RunIdleThread(int, void* pProfiler)

void Profiler::SleepProfileSpan()
{
    //n1849:todo:not implemented
}

void Profiler::SpinWait(int32_t msec)
{
    //n1849:todo:not implemented
}

#else
#error "Not supported platform."
#endif


void Profiler::StartProfilePacketLoss()
{
    nn::pia::transport::StationManager* pStationManager = nn::pia::transport::StationManager::GetInstance();
    if (PIA_IS_VALID_POINTER(pStationManager))
    {
        for (nn::pia::transport::StationManager::Iterator it = pStationManager->Begin(); it != pStationManager->End(); ++it)
        {
            nn::pia::transport::Station* pStation = &(*it);
            if (PIA_IS_VALID_POINTER(pStation))
            {
                pStation->GetSequenceIdController()->StartProfile();
            }
        }
    }
}

void Profiler::StopProfilePacketLoss()
{
    nn::pia::transport::StationManager* pStationManager = nn::pia::transport::StationManager::GetInstance();
    if (PIA_IS_VALID_POINTER(pStationManager))
    {
        for (nn::pia::transport::StationManager::Iterator it = pStationManager->Begin(); it != pStationManager->End(); ++it)
        {
            nn::pia::transport::Station* pStation = &(*it);
            if (PIA_IS_VALID_POINTER(pStation))
            {
                pStation->GetSequenceIdController()->StopProfile();
            }
        }
    }
}

void Profiler::ReportPacketLoss()
{
    nn::pia::transport::StationManager* pStationManager = nn::pia::transport::StationManager::GetInstance();
    if (PIA_IS_VALID_POINTER(pStationManager))
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("-- Packet Loss --\n");
        for (nn::pia::transport::StationManager::Iterator it = pStationManager->Begin(); it != pStationManager->End(); ++it)
        {
            nn::pia::transport::Station* pStation = &(*it);
            if (PIA_IS_VALID_POINTER(pStation))
            {
                uint16_t totalPacketNum = pStation->GetSequenceIdController()->GetProfiledTotalPacketNum();
                uint16_t packetLossNum = pStation->GetSequenceIdController()->GetProfiledPacketLossNum();
                float rate = (totalPacketNum > 0) ? static_cast<float>(packetLossNum) * 100.0f / totalPacketNum : 0.0f;
                uint16_t multicastPacketNum = pStation->GetSequenceIdController()->GetProfiledMulticastPacketNum();
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("StationId:%lld  SendTo(Lost/Total (ratio)): (%3d / %3d (%3.2f%%))  Multicast: %d\n",
                                 pStation->GetStationId(),
                                 packetLossNum,
                                 totalPacketNum,
                                 rate,
                                 multicastPacketNum);
            }
        }
    }
}


static size_t s_ReceivePacket = 0;
static size_t s_SendPacket = 0;

void Profiler::StartPacketProfile()
{
    nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
    if (PIA_IS_VALID_POINTER(pMgr))
    {
        pMgr->ClearReceiveThreadReceivePacketNum();
        pMgr->ClearSendThreadSendPacketNum();
    }
}

void Profiler::StopPacketProfile()
{
    nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
    if (PIA_IS_VALID_POINTER(pMgr))
    {
        s_ReceivePacket = pMgr->GetReceiveThreadReceivePacketNum();
        s_SendPacket = pMgr->GetSendThreadSendPacketNum();
    }
}

/*!
  n2908: Profile したデータを取得する
*/
void Profiler::GetPacketLoss(nn::pia::StationId stationId, uint16_t& totalPacketNum, uint16_t& packetLossNum, uint16_t& multicastPacketNum)
{
    nn::pia::transport::StationManager* pStationManager = nn::pia::transport::StationManager::GetInstance();
    if (PIA_IS_VALID_POINTER(pStationManager))
    {
        nn::pia::transport::Station* pStation = pStationManager->GetStation(stationId);
        if (PIA_IS_VALID_POINTER(pStation))
        {
            totalPacketNum = pStation->GetSequenceIdController()->GetProfiledTotalPacketNum();
            packetLossNum = pStation->GetSequenceIdController()->GetProfiledPacketLossNum();
            multicastPacketNum = pStation->GetSequenceIdController()->GetProfiledMulticastPacketNum();
        }
    }
}

void Profiler::ReportPacketProfile()
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("-- ThreadStreamManager --\n");
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("   ReceivePacket: %d\n", s_ReceivePacket);
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("   SendPacket:    %d\n", s_SendPacket);
}


#if 0

void Profiler::SetProfileSpan(const nn::fnd::TimeSpan& span)
{
    PIA_SAMPLE_ASSERTMSG(span > nn::fnd::TimeSpan(0), "span must be plus.");

    if (m_IsCounting)
    {
        return;
    }

    m_ProfileSpan = span;
    m_ProfileCountIdle = -1;
    m_ProfileCountBase = -1;
    m_ProfileCount = -1;
}

#endif
}
}  // end of namespace nns::pia
