﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>
#include <nn/pia/common.h>
#include "Thread.h"

namespace nns
{
namespace pia
{

class ProfileResult
{
public:
    int32_t count;
    int32_t predictedCount;
    nn::pia::common::TimeSpan period;

    ProfileResult()
    {
        count = 0;
        predictedCount = 0;
        period = 0;
    }

    void CalcPredictedCount(nn::pia::common::TimeSpan timeSpan)
    {
        if (period == 0)
        {
            predictedCount = 0;
        }
        else
        {
            predictedCount = (int32_t)(count * timeSpan.GetMSecS64() / period.GetMSecS64());
        }
    }
};

class Profiler
{
public:
    /*!
    @brief シングルトンインスタンスを生成します。

           この関数より先にnn::os::InitializeAlarmSystem()を呼んでおく必要があります。
     */
    static void CreateInstance();


    /*!
    @brief シングルトンインスタンスを破棄します。
     */
    static void DestroyInstance();


    /*!
    @brief シングルトンインスタンスを取得します。
     */
    static Profiler* GetInstance()
    {
        return s_pInstance;
    }


    /*!
    @brief 何もしない場合の負荷を計測します。指定時間の計測が終了してからこの関数を抜けます。
     */
    void StartProfilingIdle(uint32_t second);


    /*!
    @brief 通信を行っていない状態の負荷の計測を開始します。この関数を呼んでから指定時間の間を計測します。
     */
    void StartProfilingBase(uint32_t second);


    /*!
    @brief 通信を行っている状態の負荷の計測を開始します。この関数を呼んでから指定時間の間を計測します。
     */
    void StartProfilingTarget(uint32_t second);

    void FinishProfiling();

    /*!
    @brief 計測結果をコンソールに出力します。
     */
    void Report();


    /*!
    @brief 計測中かどうかを取得します。

    @return 計測中ならtrueを返します。
     */
    bool IsCounting() const
    {
        return m_IsCounting;
    }

    /*!
    @brief アプリに表示できるように総パケット数、パケットロス数を取得できるようにします。
    @param[in]  stationId 送信元ステーション
    @param[out] totalPacketNum 受信できたパケット数
    @param[out] packetLossNum パケットロス数
     */
    void GetPacketLoss(nn::pia::StationId stationId, uint16_t& totalPacketNum, uint16_t& packetLossNum, uint16_t& multicastPacketNum);


#if 0
    /*!
    @brief 計測する時間を設定します。設定しない場合のデフォルト値は10秒です。

    @param[in] span 計測する時間です。
     */
    void SetProfileSpan(const nn::fnd::TimeSpan& span);


    /*!
    @brief 何もしない場合の負荷測定の結果を取得します。

    @return 何もしない場合の負荷測定の結果です。
     */
    int32_t GetIdleCount() const {return m_ProfileCountIdle;}


    /*!
    @brief 通信を行っていない状態の負荷測定の結果を取得します。

    @return 通信を行っていない状態の負荷測定の結果です。
     */
    int32_t GetStandAloneCount() const {return m_ProfileCountBase;}


    /*!
    @brief 通信を行っている状態の負荷測定の結果を取得します。

    @return 通信を行っている状態の負荷測定の結果です。
     */
    int32_t GetCommunicationCount() const {return m_ProfileCount;}


    /*!
    @brief 何もしない場合の負荷測定の結果を設定します。

    @param[in] count 何もしない場合の負荷測定の結果です。
     */
    void PresetIdleCount(int32_t count) {m_ProfileCountIdle = count;}


    /*!
    @brief 通信を行っていない状態の負荷測定の結果を設定します。

    @param[in] count 通信を行っていない状態の負荷測定の結果です。
     */
    void PresetStandAloneCount(int32_t count) {m_ProfileCountBase = count;}


    /*!
    @brief 通信を行っている状態の負荷測定の結果を設定します。

    @param[in] count 通信を行っている状態の負荷測定の結果です。
     */
    void PresetCommunicationCount(int32_t count) {m_ProfileCount = count;}
#endif

private:
    Profiler();

    ~Profiler();

    void StartCount(uint32_t second, ProfileResult* pProfileResult);

    void StopCount();

    void Count();

    void StartProfilePacketLoss();

    void StopProfilePacketLoss();

    void ReportPacketLoss();

    void StartPacketProfile();  //n1769

    void StopPacketProfile();  //n1769

    void ReportPacketProfile();  //n1769

    void SleepProfileSpan();  //n1769

    static void SpinWait(int32_t msec);  //n1769

    static void AlarmHandler(void* param);  //n1769

    PIA_ATTRIBUTE_ALIGN(8) static Profiler* s_pInstance;

    PIA_ATTRIBUTE_ALIGN(8) Thread m_CountThread;

#if NN_PIA_CTR
    static void RunIdleThread(void* pProfiler);
    nn::fnd::TimeSpan m_ProfileSpan;
#elif NN_PIA_CAFE
    static int RunIdleThread(int, void* pProfiler);
    OSTime m_ProfileSpan;
#elif NN_PIA_NINTENDOSDK
    static void RunIdleThread(void* pProfiler);
    static void CalibrateSpinWait();
    static void CalibrateThreadFunc(void*);
    static void Loop(uintptr_t count);
    nn::TimeSpan m_ProfileSpan;
    static uintptr_t s_BusyLoopTimes;
#endif

    volatile int32_t m_CurrentProfileCount;
    volatile bool m_IsCounting;
    volatile bool m_IsExitIdleThread;

    nn::pia::common::Time m_CurrentProfileStartTime;

    ProfileResult m_IdleProfileResult;
    ProfileResult m_BaseProfileResult;
    ProfileResult m_TargetProfileResult;
    ProfileResult* m_pCurrentProfileResult;
};
}
}  // end of namespace nns::pia
