﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Session_Mesh.h"

#if NN_PIA_CAFE
#include <cafe.h>
#include <nn/act.h>
#include <nn/fp.h>
#elif NN_PIA_CTR
#include <nn/cfg.h>
#include <nn/friends.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/oe.h>
#include <nn/settings.h>
#endif

#include <nn/pia/transport.h>

// ネットワーク接続断絶を検出したら、while ループを脱出するためのマクロ。
#define BREAK_IF_CONNECTION_IS_LOST()                                                                                  \
                                                                                                                       \
{                                                                                                                      \
        nn::pia::Result connectionStatusResult = nn::pia::transport::Transport::GetInstance()->CheckConnectionError(); \
        if (connectionStatusResult.IsSuccess())                                                                        \
        {                                                                                                              \
        }                                                                                                              \
        else if (connectionStatusResult == nn::pia::ResultNetworkConnectionIsLost())                                   \
        {                                                                                                              \
            PIA_SAMPLE_PRINT("ResultNetworkConnectionIsLost is detected.  Break.\n");                                  \
            break;                                                                                                     \
        }                                                                                                              \
        else                                                                                                           \
        {                                                                                                              \
            PIA_SAMPLE_PRINT_RESULT(connectionStatusResult);                                                           \
            PIA_SAMPLE_ASSERTMSG(false, "Transport::CheckConnectionError() returned unexpected Result.");              \
        }                                                                                                              \
    \
}

namespace
{
const uint32_t PlayerNameListSize = 16;
const SampleStringChar_t* PlayerNameList[PlayerNameListSize] = {
    SAMPLE_STRING("Apple"),
    SAMPLE_STRING("Banana"),
    SAMPLE_STRING("Cherry"),
    SAMPLE_STRING("Durian"),
    SAMPLE_STRING("Eggfruit"),
    SAMPLE_STRING("Fig"),
    SAMPLE_STRING("Grape"),
    SAMPLE_STRING("HoneydewMelon"),
    SAMPLE_STRING("Iyokan"),
    SAMPLE_STRING("Jujube"),
    SAMPLE_STRING("Kiwifruit"),
    SAMPLE_STRING("Lemon"),
    SAMPLE_STRING("Mango"),
    SAMPLE_STRING("Navel"),
    SAMPLE_STRING("Orange"),
    SAMPLE_STRING("Pineapple")
};
}

namespace nns
{
namespace pia
{

nn::pia::Result Session_InitializePiaSession_ForMesh(nn::pia::transport::NetworkFactory* pFactory,
                                                     nn::pia::session::NetworkTopology topology,
                                                     nn::pia::session::BitRateCheckMode bitRateCheckMode)
{
    nn::pia::session::Mesh::Setting setting;
    setting.pNetworkFactory = pFactory;
    setting.networkTopology = topology;
    setting.bitRateCheckMode = bitRateCheckMode;

    return Session_InitializePiaSession_ForMesh(setting);
}

nn::pia::Result Session_InitializePiaSession_ForMesh(const nn::pia::session::Mesh::Setting& setting)
{
    return Session_InitializePiaSession_ForMesh(setting, false);
}

nn::pia::Result Session_InitializePiaSession_ForMesh(
    const nn::pia::session::Mesh::Setting& setting,
    bool isRelayEmulationEnabled)
{
    PIA_SAMPLE_PRINT("Session_InitializePiaSession_ForMesh\n");
    nn::pia::Result result;

    result = nn::pia::session::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    {
        // デバッグ機能設定用
        nn::pia::session::Mesh::DebugSetting debugSetting;
        // デバッグ用接続経路エミュレーションの設定
        debugSetting.isRelayEmulationEnabled = isRelayEmulationEnabled;

        // 製品版 ROM では呼び出さないように注意してください
        result = nn::pia::session::Mesh::SetDebugSetting(debugSetting);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }

    result = nn::pia::session::Mesh::CreateInstance(setting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // メモリ消費量をプリント
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Session_InitializePiaSession_ForMesh:Pia current memory usage: %d\n", nn::pia::common::GetMemoryUsage());
    return result;
}

void Session_FinalizePiaSession_ForMesh()
{
    PIA_SAMPLE_PRINT("Session_FinalizePiaSession_ForMesh\n");

    // PiaではFinalize、DestroyInstanceは常に成功します
    if (nn::pia::session::Mesh::GetInstance() != NULL)
    {
        nn::pia::session::Mesh::DestroyInstance();
    }
    nn::pia::session::Finalize();
}

uint8_t Session_GetLocalPlayerInfo_ForMesh(nn::pia::transport::Station::PlayerInfo* pPlayerInfo, uint8_t playerInfoSize, bool isUserNameInAppilcationUsed)
{
    uint8_t playerInfoNum = 1;
    if (pPlayerInfo == NULL || playerInfoSize == 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "pPlayerInfo is null or playerInfoSize is zero.");
        return 0;
    }

#if NN_PIA_CAFE
    uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
    const SampleStringChar_t* pScreenName = NULL;
    SampleStringChar_t miiName[ACT_MII_NAME_SIZE];
    memset(miiName, 0, sizeof(miiName));
    SampleStringChar_t screenName[FP_IN_GAME_SCREEN_NAME_SIZE];
    memset(screenName, 0, sizeof(screenName));
    SampleStringChar_t nullName[FP_IN_GAME_SCREEN_NAME_SIZE];
    memset(nullName, 0, sizeof(nullName));

    // アプリケーションで設定したユーザー名を使用する場合は名前取得処理をスキップする
    if (isUserNameInAppilcationUsed)
    {
        pScreenName = PlayerNameList[nameIndex];
    }
    else
    {
        // 画面表示用の名前を取得する
        if (nn::fp::HasLoggedIn() && nn::fp::GetMyScreenName(screenName).IsSuccess() && screenName[0] != 0)
        {
            pScreenName = screenName;
        }
        // 取得できない場合は Mii の名前を取得する
        else if (nn::act::GetMiiName(miiName).IsSuccess() && miiName[0] != 0)
        {
            pScreenName = miiName;
        }
        // 上記いずれかで名前取得に成功するはずだが、念の為、NULL 文字列だった場合はダミーの名前をセットしておく
        if (pScreenName == NULL)
        {
            pScreenName = SAMPLE_STRING("DummyName");
        }
    }
    uint8_t language = static_cast<uint8_t>(nn::fp::LANGUAGE_ENGLISH);
    nn::fp::MyPresence myPresence;
    if (nn::fp::GetMyPresence(&myPresence).IsSuccess())
    {
        language = myPresence.consoleInfo.language;
    }

    pPlayerInfo->screenName.Copy(pScreenName);
    pPlayerInfo->nameStringLanguage = language;
#elif NN_PIA_CTR
    uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
    const SampleStringChar_t* pScreenName = NULL;
    SampleStringChar_t screenName[nn::friends::SCREEN_NAME_SIZE];
    std::memset(screenName, 0, sizeof(screenName));
    nn::cfg::CTR::UserName userName;
    nn::cfg::CTR::CfgLanguageCode language = nn::cfg::CTR::CFG_LANGUAGE_ENGLISH;

    // アプリケーションで設定したユーザー名を使用する場合は名前取得処理をスキップする
    if (isUserNameInAppilcationUsed)
    {
        pScreenName = PlayerNameList[nameIndex];
    }
    else
    {
        // 画面表示用の名前を取得する
        nn::Result result = nn::friends::Initialize();
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        result = nn::friends::GetMyScreenName(screenName);
        nn::friends::Finalize();
        if (result.IsSuccess() && screenName[0] != 0)  // Miiが設定されていない場合も除く
        {
            pScreenName = screenName;
        }
        // 取得できない場合はユーザー名を取得する
        else
        {
            nn::cfg::CTR::Initialize();
            nn::cfg::CTR::GetUserName(&userName);
            language = nn::cfg::CTR::GetLanguage();
            nn::cfg::CTR::Finalize();
            if (!userName.isNgUserName && userName.userName[0] != 0)
            {
                pScreenName = userName.userName;
            }
        }
        // 上記いずれかで名前取得に成功するはずだが、念の為、NULL 文字列だった場合はダミーの名前をセットしておく
        if (pScreenName == NULL)
        {
            pScreenName = SAMPLE_STRING("DummyName");
        }
    }

    pPlayerInfo->screenName.Copy(pScreenName);
    pPlayerInfo->nameStringLanguage = static_cast<uint8_t>(language);
#elif NN_PIA_NINTENDOSDK
#if NN_PIA_NINTENDOSDK_NX
    // 言語
    const nn::settings::Language langs[] = {
        nn::settings::Language_Japanese,
        nn::settings::Language_AmericanEnglish,
        nn::settings::Language_French,
        nn::settings::Language_German,
        nn::settings::Language_Italian,
        nn::settings::Language_Spanish,
        nn::settings::Language_Chinese,
        nn::settings::Language_Korean,
        nn::settings::Language_Dutch,
        nn::settings::Language_Portuguese,
        nn::settings::Language_Russian,
        nn::settings::Language_Taiwanese,
        nn::settings::Language_BritishEnglish,
        nn::settings::Language_CanadianFrench,
        nn::settings::Language_LatinAmericanSpanish
    };

    // アカウント情報を取得し、利用可能なユーザーアカウントをセット
    int accountInfoNum = Net_GetUserAccountInfoSize();
    if (accountInfoNum < 1)
    {
        PIA_SAMPLE_ASSERTMSG(false, "No user account is found.");
        return 0;
    }
    uint8_t playerInfoIndex = 0;
    nn::pia::transport::Station::PlayerInfo* pInfo = pPlayerInfo;
    for (int i = 0; i < accountInfoNum && playerInfoIndex < playerInfoSize; ++i)
    {
        nns::pia::UserAccountInfo* pUserAccountInfo = Net_GetUserAccountInfo(i);
        if (pUserAccountInfo != NULL)
        {
            // アプリケーションで設定したユーザー名を使用する
            if (isUserNameInAppilcationUsed)
            {
                uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
                pInfo->screenName.Copy(PlayerNameList[nameIndex]);
            }
            else
            {
                // アカウント情報からニックネームを取得する
                nn::account::Nickname nickName;
                nn::Result result = nn::account::GetNickname(&nickName, pUserAccountInfo->userUid);
                if (result.IsFailure())
                {
                    // 失敗しないはずだが、ダミーの名前をセットする
                    PIA_SAMPLE_PRINT("nn::account::GetNickname is failed. [%08x]\n", result.GetInnerValueForDebug());
                    pInfo->screenName.Copy(SAMPLE_STRING("DummyName"));
                }
                else
                {
                    pInfo->screenName.Copy(nickName.name);
                }
            }
            pInfo->playerId.id[0] = pUserAccountInfo->userUid._data[0];
            pInfo->playerId.id[1] = pUserAccountInfo->userUid._data[1];

            // 現在の設定されている言語の言語コードを取得
            nn::oe::Initialize();
            nn::settings::Language language = nn::settings::Language_AmericanEnglish;
            nn::settings::LanguageCode code = nn::oe::GetDesiredLanguage();
            for (int i = 0; i < sizeof(langs) / sizeof(nn::settings::Language); ++i)
            {
                if (code == langs[i])
                {
                    language = langs[i];
                    break;
                }
            }
            pInfo->nameStringLanguage = static_cast<uint8_t>(language);
            ++playerInfoIndex;
            ++pInfo;
        }
    }
    playerInfoNum = playerInfoIndex;
#else
    uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
    nn::pia::transport::Station::PlayerInfo* pInfo = pPlayerInfo;
    for (uint8_t i = 0; i < playerInfoSize; ++i)
    {
        // アプリケーションで設定したユーザー名を使用する
        if (isUserNameInAppilcationUsed)
        {
            pInfo->screenName.Copy(PlayerNameList[(nameIndex + i) % PlayerNameListSize]);
        }
        // ダミーの名前を使用する
        else
        {
            pInfo->screenName.Copy(SAMPLE_STRING("DummyName"));
        }
        pInfo->nameStringLanguage = nn::settings::Language_AmericanEnglish;
        pInfo->playerId.id[0] = nn::pia::common::Time::GetTime().GetTick();
        pInfo->playerId.id[1] = i;
        ++pInfo;
    }
    playerInfoNum = playerInfoSize;
#endif
#else
    const SampleStringChar_t* pScreenName = SAMPLE_STRING("DummyScreen");
    pPlayerInfo->screenName.Copy(pScreenName);
    pPlayerInfo->nameStringLanguage = 0;
#endif
    return playerInfoNum;
}

nn::pia::Result Session_StartupMesh(nn::pia::session::Mesh::StartupSetting startupSetting)
{
    PIA_SAMPLE_PRINT("signature GetKeySize : %d\n", startupSetting.signatureSetting.GetKeySize());

    nn::pia::Result result;
    nn::pia::transport::Station::PlayerInfo playerInfo;

    Session_GetLocalPlayerInfo_ForMesh(&playerInfo, 1);
    startupSetting.pPlayerInfo = &playerInfo;
    startupSetting.playerInfoSize = 1;

    result = nn::pia::session::Mesh::GetInstance()->Startup(startupSetting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    return result;
}


void Session_CleanupMesh()
{
    PIA_SAMPLE_PRINT("Session_CleanupMesh\n");
    nn::pia::session::Mesh::GetInstance()->Cleanup();
#if NN_PIA_CTR
    nn::friends::Finalize();
#endif
}


#if !NN_PIA_A && !NN_PIA_B
bool Session_CreateMesh()
{
    PIA_SAMPLE_PRINT("Session_CreateMesh\n");

    nn::pia::Result result;

    // メッシュの作成
    result = nn::pia::session::Mesh::GetInstance()->CreateMeshAsync();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // CreateMeshAsync の完了待ち
    int loopCounter = 0;
    bool isExit = false;
    while (!isExit)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();
// ディスパッチ処理
#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "creating session... %d", loopCounter++);

        sampleframework::SampleFramework::GetInstance()->EndFrame();

        BREAK_IF_CONNECTION_IS_LOST();

        if (nn::pia::session::Mesh::GetInstance()->IsCreateMeshCompleted())
        {
            result = nn::pia::session::Mesh::GetInstance()->GetCreateMeshResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("Session_CreateMesh: CreateMeshAsync failed\n");
                PIA_SAMPLE_PRINT_RESULT(result);
                return false;
            }

            PIA_SAMPLE_PRINT("Session_CreateMesh: CreateMeshAsync success\n");
            return true;
        }
    }

    PIA_SAMPLE_PRINT("Session_CreateMesh: Connection is lost.\n");
    return false;
}
#endif


#if !NN_PIA_A && !NN_PIA_B
void Session_DestroyMesh()
{
    PIA_SAMPLE_PRINT("Session_DestroyMesh\n");

    if (nn::pia::session::Mesh::GetInstance()->CheckConnectionError() == nn::pia::ResultMeshConnectionIsLost())
    {
        PIA_SAMPLE_PRINT("Session_DestroyMesh: ResultMeshConnectionIsLost.  DestroyMeshAsync() is not called.\n");
        return;
    }

    // メッシュの破棄
    nn::pia::Result result;
    result = nn::pia::session::Mesh::GetInstance()->DestroyMeshAsync();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // DestroyMeshの完了待ち
    bool isExit = false;
    while (!isExit)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

// ディスパッチ処理
#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "destroying session...");

        sampleframework::SampleFramework::GetInstance()->EndFrame();

        BREAK_IF_CONNECTION_IS_LOST();

        if (nn::pia::session::Mesh::GetInstance()->IsDestroyMeshCompleted())
        {
            // メッシュの破棄が完了
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(nn::pia::session::Mesh::GetInstance()->GetDestroyMeshResult());
            PIA_SAMPLE_PRINT("Session_DestroyMesh: DestroyMeshAsync success\n");
            return;
        }
    }
}
#endif


#if !NN_PIA_A && !NN_PIA_B
void Session_LeaveMeshWithHostMigration()
{
    PIA_SAMPLE_PRINT("Session_LeaveMeshWithHostMigration\n");

    if (nn::pia::session::Mesh::GetInstance()->CheckConnectionError() == nn::pia::ResultMeshConnectionIsLost())
    {
        PIA_SAMPLE_PRINT("Session_LeaveMeshWithHostMigration: ResultMeshConnectionIsLost.  LeaveMeshWithHostMigrationAsync() is not called.\n");
        return;
    }

    // ホストマイグレーション開始指示を出してメッシュ離脱
    nn::pia::Result result;
    result = nn::pia::session::Mesh::GetInstance()->LeaveMeshWithHostMigrationAsync();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // LeaveMeshWithHostMigrationの完了待ち
    bool isExit = false;
    while (!isExit)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();
// ディスパッチ処理
#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "leaving session(with HM)...");

        sampleframework::SampleFramework::GetInstance()->EndFrame();

        BREAK_IF_CONNECTION_IS_LOST();

        if (nn::pia::session::Mesh::GetInstance()->IsLeaveMeshWithHostMigrationCompleted())
        {
            // メッシュ離脱が完了
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(nn::pia::session::Mesh::GetInstance()->GetLeaveMeshWithHostMigrationResult());
            PIA_SAMPLE_PRINT("Session_LeaveMeshWithHostMigration: LeaveMeshWithHostMigrationAsync success\n");
            return;
        }
    }
}
#endif


#if !NN_PIA_A && !NN_PIA_B
bool Session_JoinMesh(const nn::pia::transport::StationConnectionInfo& stationConnectionInfo)
{
    PIA_SAMPLE_PRINT("Session_JoinMesh\n");

    nn::pia::Result result;

    // メッシュに参加します。
    result = nn::pia::session::Mesh::GetInstance()->JoinMeshAsync(stationConnectionInfo);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Session_JoinMesh: JoinMeshAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(result);
        PIA_SAMPLE_ASSERT(result == nn::pia::ResultStationConnectionFailed());
        return false;
    }

    // JoinMeshAsync() の完了待ち
    int loopCounter = 0;
    bool isExit = false;
    while (!isExit)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

// ジョブのディスパッチ処理（JoinMeshによって投入されたジョブの実行）
#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "joinning session... %d", loopCounter++);

        sampleframework::SampleFramework::GetInstance()->EndFrame();

        BREAK_IF_CONNECTION_IS_LOST();

        if (nn::pia::session::Mesh::GetInstance()->IsJoinMeshCompleted())
        {
            result = nn::pia::session::Mesh::GetInstance()->GetJoinMeshResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("Session_JoinMesh: JoinMeshAsync() failed\n");
                PIA_SAMPLE_PRINT_RESULT(result);
                PIA_SAMPLE_ASSERT(result == nn::pia::ResultStationConnectionFailed() ||
                                  result == nn::pia::ResultStationConnectionNatTraversalFailedUnknown() ||
                                  result == nn::pia::ResultNatTraversalFailedBothEim() ||
                                  result == nn::pia::ResultNatTraversalFailedBothEdm() ||
                                  result == nn::pia::ResultNatTraversalFailedLocalEimRemoteEdm() ||
                                  result == nn::pia::ResultNatTraversalFailedLocalEdmRemoteEim() ||
                                  result == nn::pia::ResultNatTraversalFailedBothEimSamePublicAddress() ||
                                  result == nn::pia::ResultNatTraversalFailedBothEdmSamePublicAddress() ||
                                  result == nn::pia::ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress() ||
                                  result == nn::pia::ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress() ||
                                  result == nn::pia::ResultJoinRequestDenied() ||
                                  result == nn::pia::ResultMeshIsFull() ||
                                  result == nn::pia::ResultRelayFailedNoCandidate() ||
                                  result == nn::pia::ResultRelayFailedRttLimit() ||
                                  result == nn::pia::ResultRelayFailedRelayNumLimit() ||
                                  result == nn::pia::ResultRelayFailedUnknown() ||
                                  result == nn::pia::ResultNatTraversalRequestTimeout() ||
                                  result == nn::pia::ResultInvalidSystemMessage());
                return false;
            }

            PIA_SAMPLE_PRINT("Session_JoinMesh: JoinMeshAsync() success\n");
            return true;
        }
    }

    PIA_SAMPLE_PRINT("Session_JoinMesh: Connection is lost.\n");
    return false;
}
#endif


#if !NN_PIA_A && !NN_PIA_B
void Session_LeaveMesh()
{
    PIA_SAMPLE_PRINT("Session_LeaveMesh\n");

    if (nn::pia::session::Mesh::GetInstance()->CheckConnectionError() == nn::pia::ResultMeshConnectionIsLost())
    {
        PIA_SAMPLE_PRINT("Session_LeaveMesh: ResultMeshConnectionIsLost.  LeaveMeshAsync() is not called.\n");
        return;
    }

    // メッシュから離脱
    nn::pia::Result result = nn::pia::session::Mesh::GetInstance()->LeaveMeshAsync();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // LeaveMesh() の完了待ち
    bool isExit = false;
    while (!isExit)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

// ディスパッチ処理
#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "leaving session...");

        sampleframework::SampleFramework::GetInstance()->EndFrame();

        BREAK_IF_CONNECTION_IS_LOST();

        // LeaveMesh()が成功したかどうかの判定
        if (nn::pia::session::Mesh::GetInstance()->IsLeaveMeshCompleted())
        {
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(nn::pia::session::Mesh::GetInstance()->GetLeaveMeshResult());
            PIA_SAMPLE_PRINT("Session_LeaveMesh: LeaveMeshAsync success\n");
            return;
        }
    }
}
#endif
}
}  // end of namespace nns::pia
