﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Session_Setup.h"

#if NN_PIA_CAFE
#include <cafe.h>
#include <nn/act.h>
#include <nn/fp.h>
#elif NN_PIA_CTR
#include <nn/cfg.h>
#include <nn/friends.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/oe.h>
#include <nn/settings.h>
#endif
#include <nn/pia/transport.h>

namespace
{
nns::pia::SampleSessionCallback s_EndFrameSampleSessionCallback = NULL;
nns::pia::SampleSessionCallback s_DispatchSampleSessionCallback = NULL;

const uint32_t PlayerNameListSize = 16;
const SampleStringChar_t* PlayerNameList[PlayerNameListSize] = {
    SAMPLE_STRING("Apple"),
    SAMPLE_STRING("Banana"),
    SAMPLE_STRING("Cherry"),
    SAMPLE_STRING("Durian"),
    SAMPLE_STRING("Eggfruit"),
    SAMPLE_STRING("Fig"),
    SAMPLE_STRING("Grape"),
    SAMPLE_STRING("HoneydewMelon"),
    SAMPLE_STRING("Iyokan"),
    SAMPLE_STRING("Jujube"),
    SAMPLE_STRING("Kiwifruit"),
    SAMPLE_STRING("Lemon"),
    SAMPLE_STRING("Mango"),
    SAMPLE_STRING("Navel"),
    SAMPLE_STRING("Orange"),
    SAMPLE_STRING("Pineapple")
};
}

namespace nns
{
namespace pia
{
SampleSessionCallback RegisterEndFrameSampleSessionCallback(SampleSessionCallback cb)
{
    SampleSessionCallback prev = s_EndFrameSampleSessionCallback;
    s_EndFrameSampleSessionCallback = cb;
    return prev;
}

SampleSessionCallback RegisterDispatchSampleSessionCallback(SampleSessionCallback cb)
{
    SampleSessionCallback prev = s_DispatchSampleSessionCallback;
    s_DispatchSampleSessionCallback = cb;
    return prev;
}

nn::pia::Result Session_InitializePiaSession(const nn::pia::session::Session::Setting& setting)
{
    PIA_SAMPLE_PRINT("Session_InitializePiaSession\n");
    nn::pia::Result result;

    result = nn::pia::session::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::Session::CreateInstance(setting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // メモリ消費量をプリント
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Session_InitializePiaSession:Pia current memory usage: %d\n", nn::pia::common::GetMemoryUsage());

    return result;
}

void Session_FinalizePiaSession()
{
    PIA_SAMPLE_PRINT("Session_FinalizePiaSession\n");

    // PiaではFinalize、DestroyInstanceは常に成功します
    if (nn::pia::session::Session::GetInstance() != NULL)
    {
        nn::pia::session::Session::DestroyInstance();
    }
    else
    {
        PIA_SAMPLE_PRINT("Session_FinalizeSession:Session instance is not created yet.\n");
    }
    nn::pia::session::Finalize();
}

uint8_t Session_GetLocalPlayerInfo(nn::pia::transport::Station::PlayerInfo* pPlayerInfo, uint8_t playerInfoSize, bool isUserNameInAppilcationUsed)
{
    uint8_t playerInfoNum = 1;
    if (pPlayerInfo == NULL || playerInfoSize == 0)
    {
        PIA_SAMPLE_ASSERTMSG(false, "pPlayerInfo is null or playerInfoSize is zero.");
        return 0;
    }

#if NN_PIA_CAFE
    uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
    const SampleStringChar_t* pScreenName = NULL;
    SampleStringChar_t miiName[ACT_MII_NAME_SIZE];
    memset(miiName, 0, sizeof(miiName));
    SampleStringChar_t screenName[FP_IN_GAME_SCREEN_NAME_SIZE];
    memset(screenName, 0, sizeof(screenName));
    SampleStringChar_t nullName[FP_IN_GAME_SCREEN_NAME_SIZE];
    memset(nullName, 0, sizeof(nullName));

    // アプリケーションで設定したユーザー名を使用する場合は名前取得処理をスキップする
    if (isUserNameInAppilcationUsed)
    {
        pScreenName = PlayerNameList[nameIndex];
    }
    else
    {
        // 画面表示用の名前を取得する
        if (nn::fp::HasLoggedIn() && nn::fp::GetMyScreenName(screenName).IsSuccess() && screenName[0] != 0)
        {
            pScreenName = screenName;
        }
        // 取得できない場合は Mii の名前を取得する
        else if (nn::act::GetMiiName(miiName).IsSuccess() && miiName[0] != 0)
        {
            pScreenName = miiName;
        }
        // 上記いずれかで名前取得に成功するはずだが、念の為、NULL 文字列だった場合はダミーの名前をセットしておく
        if (pScreenName == NULL)
        {
            pScreenName = SAMPLE_STRING("DummyName");
        }
    }
    uint8_t language = static_cast<uint8_t>(nn::fp::LANGUAGE_ENGLISH);
    nn::fp::MyPresence myPresence;
    if (nn::fp::GetMyPresence(&myPresence).IsSuccess())
    {
        language = myPresence.consoleInfo.language;
    }

    pPlayerInfo->screenName.Copy(pScreenName);
    pPlayerInfo->nameStringLanguage = language;
#elif NN_PIA_CTR
    uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
    const SampleStringChar_t* pScreenName = NULL;
    SampleStringChar_t screenName[nn::friends::SCREEN_NAME_SIZE];
    std::memset(screenName, 0, sizeof(screenName));
    nn::cfg::CTR::UserName userName;
    nn::cfg::CTR::CfgLanguageCode language = nn::cfg::CTR::CFG_LANGUAGE_ENGLISH;

    // アプリケーションで設定したユーザー名を使用する場合は名前取得処理をスキップする
    if (isUserNameInAppilcationUsed)
    {
        pScreenName = PlayerNameList[nameIndex];
    }
    else
    {
        // 画面表示用の名前を取得する
        nn::Result result = nn::friends::Initialize();
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        result = nn::friends::GetMyScreenName(screenName);
        nn::friends::Finalize();
        if (result.IsSuccess() && screenName[0] != 0)  // Miiが設定されていない場合も除く
        {
            pScreenName = screenName;
        }
        // 取得できない場合はユーザー名を取得する
        else
        {
            nn::cfg::CTR::Initialize();
            nn::cfg::CTR::GetUserName(&userName);
            language = nn::cfg::CTR::GetLanguage();
            nn::cfg::CTR::Finalize();
            if (!userName.isNgUserName && userName.userName[0] != 0)
            {
                pScreenName = userName.userName;
            }
        }
        // 上記いずれかで名前取得に成功するはずだが、念の為、NULL 文字列だった場合はダミーの名前をセットしておく
        if (pScreenName == NULL)
        {
            pScreenName = SAMPLE_STRING("DummyName");
        }
    }

    pPlayerInfo->screenName.Copy(pScreenName);
    pPlayerInfo->nameStringLanguage = static_cast<uint8_t>(language);
#elif NN_PIA_NINTENDOSDK
#if NN_PIA_NINTENDOSDK_NX
    // 言語
    const nn::settings::Language langs[] = {
        nn::settings::Language_Japanese,
        nn::settings::Language_AmericanEnglish,
        nn::settings::Language_French,
        nn::settings::Language_German,
        nn::settings::Language_Italian,
        nn::settings::Language_Spanish,
        nn::settings::Language_Chinese,
        nn::settings::Language_Korean,
        nn::settings::Language_Dutch,
        nn::settings::Language_Portuguese,
        nn::settings::Language_Russian,
        nn::settings::Language_Taiwanese,
        nn::settings::Language_BritishEnglish,
        nn::settings::Language_CanadianFrench,
        nn::settings::Language_LatinAmericanSpanish
    };

    // アカウント情報を取得し、利用可能なユーザーアカウントをセット
    int accountInfoNum = Net_GetUserAccountInfoSize();
    if (accountInfoNum < 1)
    {
        PIA_SAMPLE_ASSERTMSG(false, "No user account is found.");
        return 0;
    }
    uint8_t playerInfoIndex = 0;
    nn::pia::transport::Station::PlayerInfo* pInfo = pPlayerInfo;
    for (int i = 0; i < accountInfoNum && playerInfoIndex < playerInfoSize; ++i)
    {
        nns::pia::UserAccountInfo* pUserAccountInfo = Net_GetUserAccountInfo(i);
        if (pUserAccountInfo != NULL)
        {
            // アプリケーションで設定したユーザー名を使用する
            if (isUserNameInAppilcationUsed)
            {
                uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
                pInfo->screenName.Copy(PlayerNameList[nameIndex]);
            }
            else
            {
                // アカウント情報からニックネームを取得する
                nn::account::Nickname nickName;
                nn::Result result = nn::account::GetNickname(&nickName, pUserAccountInfo->userUid);
                if (result.IsFailure())
                {
                    // 失敗しないはずだが、ダミーの名前をセットする
                    PIA_SAMPLE_PRINT("nn::account::GetNickname is failed. [%08x]\n", result.GetInnerValueForDebug());
                    pInfo->screenName.Copy(SAMPLE_STRING("DummyName"));
                }
                else
                {
                    pInfo->screenName.Copy(nickName.name);
                }
            }
            pInfo->playerId.id[0] = pUserAccountInfo->userUid._data[0];
            pInfo->playerId.id[1] = pUserAccountInfo->userUid._data[1];

            // 現在の設定されている言語の言語コードを取得
            nn::oe::Initialize();
            nn::settings::Language language = nn::settings::Language_AmericanEnglish;
            nn::settings::LanguageCode code = nn::oe::GetDesiredLanguage();
            for (int i = 0; i < sizeof(langs) / sizeof(nn::settings::Language); ++i)
            {
                if (code == langs[i])
                {
                    language = langs[i];
                    break;
                }
            }
            pInfo->nameStringLanguage = static_cast<uint8_t>(language);
            ++playerInfoIndex;
            ++pInfo;
        }
    }
    playerInfoNum = playerInfoIndex;
#else
    uint32_t nameIndex = static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick() % PlayerNameListSize);
    nn::pia::transport::Station::PlayerInfo* pInfo = pPlayerInfo;
    for (uint8_t i = 0; i < playerInfoSize; ++i)
    {
        // アプリケーションで設定したユーザー名を使用する
        if (isUserNameInAppilcationUsed)
        {
            pInfo->screenName.Copy(PlayerNameList[(nameIndex + i) % PlayerNameListSize]);
        }
        // ダミーの名前を使用する
        else
        {
            pInfo->screenName.Copy(SAMPLE_STRING("DummyName"));
        }
        pInfo->nameStringLanguage = nn::settings::Language_AmericanEnglish;
        pInfo->playerId.id[0] = nn::pia::common::Time::GetTime().GetTick();
        pInfo->playerId.id[1] = i;
        ++pInfo;
    }
    playerInfoNum = playerInfoSize;
#endif
#else
    const SampleStringChar_t* pScreenName = SAMPLE_STRING("DummyScreen");
    pPlayerInfo->screenName.Copy(pScreenName);
    pPlayerInfo->nameStringLanguage = 0;
#endif
    return playerInfoNum;
}

nn::pia::Result Session_StartupSession(const nn::pia::session::Session::StartupSetting& startupSetting)
{
    PIA_SAMPLE_PRINT("Session_StartupSession\n");

    nn::pia::Result result = nn::pia::session::Session::GetInstance()->Startup(startupSetting);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Session_StartupSession: Startup failed\n");
        PIA_SAMPLE_PRINT_RESULT(result);
        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() || 
                          result == nn::pia::ResultWifiOff() ||
                          result == nn::pia::ResultSleep());
    }
    return result;
}


void Session_CleanupSession()
{
    if (nn::pia::session::Session::GetInstance())
    {
        PIA_SAMPLE_PRINT("Session_CleanupSession\n");
        nn::pia::session::Session::GetInstance()->Cleanup();
    }
    else
    {
        PIA_SAMPLE_PRINT("Session_CleanupSession:Session instance is not created yet.\n");
    }
}

nn::pia::Result Session_JoinRandomSession(nn::pia::session::JoinRandomSessionSetting* pJoinRandomSessionSetting, bool bScore, bool bRender)
{
    PIA_SAMPLE_PRINT("Session_JoinRandomSession\n");

    // 非同期処理が未完了で関数を抜けた場合は ResultSuccess を返すこととします。
    nn::pia::Result result = nn::pia::ResultSuccess();

    // ランダムマッチメイクの開始
    result = nn::pia::session::Session::GetInstance()->JoinRandomSessionAsync(pJoinRandomSessionSetting);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Session_JoinRandomSession: call JoinRandomSessionAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(result);
        PIA_SAMPLE_ASSERT(false);
        return result;
    }

    // JoinRandomSessionAsync の非同期処理完了待ち
    int loopCounter = 0;
    bool isCanceled = false;
    while (true)
    {
        if (bRender)
        {
            sampleframework::SampleFramework::GetInstance()->BeginFrame();
        }

#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        if (PIA_IS_VALID_POINTER(s_DispatchSampleSessionCallback))
        {
            s_DispatchSampleSessionCallback();
        }

        if (bRender)
        {
            sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
            sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "random matchmaking session... %d (%d)", loopCounter++, nn::pia::session::Session::GetInstance()->GetJoinAutoMatchmakeSequence());
            sampleframework::SampleFramework::GetInstance()->DrawString(10, 30, "  B: Cancel");

            sampleframework::SampleFramework::GetInstance()->EndFrame();
        }
        else
        {
#if NN_PIA_CTR
            nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(16));
#elif NN_PIA_CAFE
            OSSleepMilliseconds(16);
#elif NN_PIA_WIN
            Sleep(16);
#elif NN_PIA_NINTENDOSDK
            nn::os::SleepThread(nn::TimeSpan::FromMilliSeconds(16));
#endif
        }

#if PIA_TEST_CODE
        if (PIA_IS_VALID_POINTER(s_EndFrameSampleSessionCallback))
        {
            s_EndFrameSampleSessionCallback();
        }
#endif

        if (nn::pia::session::Session::GetInstance()->IsJoinRandomSessionCompleted())
        {
            result = nn::pia::session::Session::GetInstance()->GetJoinRandomSessionResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("Session_JoinRandomSession: JoinRandomSessionAsync failed: 0x%08x\n", result.GetValue());
            }
            else
            {
                PIA_SAMPLE_PRINT("Session_JoinRandomSession: JoinRandomSessionAsync success\n");
                const nn::pia::session::ISessionInfo* pSessionInfo = nn::pia::session::Session::GetInstance()->GetSessionInfo();
                PIA_SAMPLE_ASSERT(pSessionInfo != NULL);
                pSessionInfo->Trace(nn::pia::common::TraceFlagAlways);

                int32_t bitRate = 0;
                if (nn::pia::session::Session::GetInstance()->IsBitRateCheckEnabled())
                {
                    if (nn::pia::session::Session::GetInstance()->IsBitRateCheckCompleted())
                    {
                        if (nn::pia::session::Session::GetInstance()->GetUplinkBitRate(&bitRate).IsSuccess())
                        {
                            PIA_SAMPLE_PRINT("bitRate: %d\n", bitRate);
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("measuring bitRate is failed.\n");
                        }
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("measuring bitRate is not completed.\n");
                    }
                }

                if (!nn::pia::session::Session::GetInstance()->IsHost() && bScore)
                {
                    uint32_t joinRandomScore;
                    if (nn::pia::session::Session::GetInstance()->GetJoinRandomSessionScore(&joinRandomScore).IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("Session_JoinRandomSession: JoinRandomScore = %d\n", joinRandomScore);
                    }
                }
            }
            return result;
        }
        else
        {
            // キャンセル処理
            if (!isCanceled && sampleframework::SampleFramework::GetInstance()->IsTrigger(sampleframework::ButtonB))
            {
                result = nn::pia::session::Session::GetInstance()->CancelJoinRandomSessionAsync();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("Session_JoinRandomSession: CancelJoinRandomSessionAsync failed\n");
                    PIA_SAMPLE_ASSERT(false);
                }
                else
                {
                    PIA_SAMPLE_PRINT("Session_JoinRandomSession: CancelJoinRandomSessionAsync success. Begin Canceling...\n");
                    isCanceled = true;
                }
            }
        }
    }
}


nn::pia::Result Session_CreateSession(nn::pia::session::CreateSessionSetting* pCreateSessionSetting)
{
    PIA_SAMPLE_PRINT("Session_CreateSession\n");

    // 非同期処理が未完了で関数を抜けた場合は ResultSuccess を返すこととします。
    nn::pia::Result result = nn::pia::ResultSuccess();

    // セッションの構築
    result = nn::pia::session::Session::GetInstance()->CreateSessionAsync(pCreateSessionSetting);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Session_CreateSession: call CreateSessionAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(result);
        PIA_SAMPLE_ASSERT(false);
        return result;
    }

    // CreateSessionAsync の非同期処理完了待ち
    int loopCounter = 0;
    while (true)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        if (PIA_IS_VALID_POINTER(s_DispatchSampleSessionCallback))
        {
            s_DispatchSampleSessionCallback();
        }

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "creating session... %d", loopCounter++);

        sampleframework::SampleFramework::GetInstance()->EndFrame();

#if PIA_TEST_CODE
        if (PIA_IS_VALID_POINTER(s_EndFrameSampleSessionCallback))
        {
            s_EndFrameSampleSessionCallback();
        }
#endif

        if (nn::pia::session::Session::GetInstance()->IsCreateSessionCompleted())
        {
            result = nn::pia::session::Session::GetInstance()->GetCreateSessionResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("Session_CreateSession: CreateSessionAsync failed\n");
                PIA_SAMPLE_PRINT_RESULT(result);
            }
            else
            {
                PIA_SAMPLE_PRINT("Session_CreateSession: CreateSessionAsync success\n");
                const nn::pia::session::ISessionInfo* pSessionInfo = nn::pia::session::Session::GetInstance()->GetSessionInfo();
                PIA_SAMPLE_ASSERT(pSessionInfo != NULL);
                pSessionInfo->Trace(nn::pia::common::TraceFlagAlways);
            }
            return result;
        }
    }
}


nn::pia::Result Session_BrowseSession(nn::pia::session::SessionSearchCriteria* pSearchCriteria)
{
    PIA_SAMPLE_PRINT("Session_BrowseSession\n");

    // 非同期処理が未完了で関数を抜けた場合は ResultSuccess を返すこととします。
    nn::pia::Result result = nn::pia::ResultSuccess();

    // セッションの検索
    result = nn::pia::session::Session::GetInstance()->BrowseSessionAsync(pSearchCriteria);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Session_BrowseSession: call BrowseSessionAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(result);
        PIA_SAMPLE_ASSERT(false);
        return result;
    }

    // BrowseSessionAsync の非同期処理完了待ち
    int loopCounter = 0;
    while (true)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        if (PIA_IS_VALID_POINTER(s_DispatchSampleSessionCallback))
        {
            s_DispatchSampleSessionCallback();
        }

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "browsing session... %d", loopCounter++);

        sampleframework::SampleFramework::GetInstance()->EndFrame();

#if PIA_TEST_CODE
        if (PIA_IS_VALID_POINTER(s_EndFrameSampleSessionCallback))
        {
            s_EndFrameSampleSessionCallback();
        }
#endif

        if (nn::pia::session::Session::GetInstance()->IsBrowseSessionCompleted())
        {
            result = nn::pia::session::Session::GetInstance()->GetBrowseSessionResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("Session_BrowseSession: BrowseSessionAsync failed\n");
                PIA_SAMPLE_PRINT_RESULT(result);
            }
            else
            {
                PIA_SAMPLE_PRINT("Session_BrowseSession: BrowseSessionAsync success\n");
            }
            return result;
        }
    }
}


nn::pia::Result Session_JoinSession(nn::pia::session::JoinSessionSetting* pJoinSessionSetting)
{
    PIA_SAMPLE_PRINT("Session_JoinSession\n");

    // 非同期処理が未完了で関数を抜けた場合は ResultSuccess を返すこととします。
    nn::pia::Result result = nn::pia::ResultSuccess();

    // セッションへ参加
    result = nn::pia::session::Session::GetInstance()->JoinSessionAsync(pJoinSessionSetting);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Session_JoinSession: call JoinSessionAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(result);
        PIA_SAMPLE_ASSERT(false);
        return result;
    }

    // JoinSessionAsync の非同期処理完了待ち
    int loopCounter = 0;
    bool isCanceled = false;
    while (true)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        if (PIA_IS_VALID_POINTER(s_DispatchSampleSessionCallback))
        {
            s_DispatchSampleSessionCallback();
        }

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "joining session... %d (%d)", loopCounter++, nn::pia::session::Session::GetInstance()->GetJoinSequence());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 30, "  B: Cancel");

        sampleframework::SampleFramework::GetInstance()->EndFrame();

#if PIA_TEST_CODE
        if (PIA_IS_VALID_POINTER(s_EndFrameSampleSessionCallback))
        {
            s_EndFrameSampleSessionCallback();
        }
#endif

        if (nn::pia::session::Session::GetInstance()->IsJoinSessionCompleted())
        {
            result = nn::pia::session::Session::GetInstance()->GetJoinSessionResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("Session_JoinSession: JoinSessionAsync failed: 0x%08x\n", result.GetValue());
            }
            else
            {
                PIA_SAMPLE_PRINT("Session_JoinSession: JoinSessionAsync success\n");
                const nn::pia::session::ISessionInfo* pSessionInfo = nn::pia::session::Session::GetInstance()->GetSessionInfo();
                PIA_SAMPLE_ASSERT(pSessionInfo != NULL);
                pSessionInfo->Trace(nn::pia::common::TraceFlagAlways);

                if (nn::pia::session::Session::GetInstance()->IsBitRateCheckEnabled())
                {
                    int32_t bitRate = 0;
                    if (nn::pia::session::Session::GetInstance()->GetUplinkBitRate(&bitRate).IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("bitRate: %d\n", bitRate);
                    }
                }
            }
            return result;
        }
        else
        {
            // キャンセル処理
            if (!isCanceled && sampleframework::SampleFramework::GetInstance()->IsTrigger(sampleframework::ButtonB))
            {
                result = nn::pia::session::Session::GetInstance()->CancelJoinSessionAsync();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("Session_JoinSession: CancelJoinSessionAsync failed\n");
                    PIA_SAMPLE_ASSERT(false);
                }
                else
                {
                    PIA_SAMPLE_PRINT("Session_JoinSession: CancelJoinSessionAsync success. Begin Canceling...\n");
                    isCanceled = true;
                }
            }
        }
    }
}


nn::pia::Result Session_LeaveSession()
{
    PIA_SAMPLE_PRINT("Session_LeaveSession\n");

    // セッションの破棄
    nn::pia::Result result;
    result = nn::pia::session::Session::GetInstance()->LeaveSessionAsync();
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Session_LeaveSession: call LeaveSessionAsync failed\n");
        PIA_SAMPLE_PRINT_RESULT(result);
        if (result != nn::pia::ResultNetworkConnectionIsLost())
        {
            PIA_SAMPLE_ASSERT(false);
        }

        // ネットワーク切断時は非同期処理開始に失敗する
        return result;
    }

    // LeaveSessionAsync の非同期処理完了待ち
    int loopCounter = 0;
    while (true)
    {
        sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if NNS_PIA_ENABLE_INET
        Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        if (PIA_IS_VALID_POINTER(s_DispatchSampleSessionCallback))
        {
            s_DispatchSampleSessionCallback();
        }

        sampleframework::SampleFramework::GetInstance()->DrawString(10, 10, Basic_GetProgramTitle());
        sampleframework::SampleFramework::GetInstance()->DrawString(10, 20, "leaving session... %d", loopCounter++);

        sampleframework::SampleFramework::GetInstance()->EndFrame();

#if PIA_TEST_CODE
        if (PIA_IS_VALID_POINTER(s_EndFrameSampleSessionCallback))
        {
            s_EndFrameSampleSessionCallback();
        }
#endif

        if (nn::pia::session::Session::GetInstance()->IsLeaveSessionCompleted())
        {
            // セッションの破棄が完了
            result = nn::pia::session::Session::GetInstance()->GetLeaveSessionResult();
            if (result.IsFailure())
            {
                // 失敗時もログを出すだけ
                PIA_SAMPLE_PRINT("Session_LeaveSession: LeaveSessionAsync failed\n");
                PIA_SAMPLE_PRINT_RESULT(result);
            }
            else
            {
                PIA_SAMPLE_PRINT("Session_LeaveSession: LeaveSessionAsync success\n");
            }
            return result;
        }
    }
}
}
}  // end of namespace nns::pia
