﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common.h>
#include "Print.h"
#include <float.h>
#include <limits.h>  // for LLONG_MIN, LLONG_MAX, ULLONG_MAX

#define STATISTIC_DETAIL 1

namespace nns
{
namespace pia
{

template <typename T>
class Statistic
{
public:
    Statistic()
    {
        Reset();
    }

    void Reset();

    void Record(T value);

    inline uint32_t GetCount() const
    {
        return m_Count;
    }

    inline T GetMax() const
    {
        return m_Max;
    }

    inline T GetMin() const
    {
        return m_Min;
    }

    inline T GetAverage() const;

#if STATISTIC_DETAIL
    static const uint32_t MaxDataNum = 8;
    void Print(const char* pCaption, uint32_t printMadDataNum = MaxDataNum);
#else
    void Print(const char* pCaption, uint32_t printMadDataNum);
#endif

private:
    static inline T MinValue();
    static inline T MaxValue();
    static inline T ZeroValue();
    static inline void PrintValue(const T& v);

private:
    uint32_t m_Count;
    T m_Sum;
    T m_Max;
    T m_Min;

#if STATISTIC_DETAIL
    struct Data
    {
        uint32_t index;
        T value;
    };
    Data m_aMaxData[MaxDataNum];
#endif
};

template <typename T>
void Statistic<T>::Reset()
{
    m_Count = 0;
    m_Sum = ZeroValue();
    m_Max = MinValue();
    m_Min = MaxValue();

#if STATISTIC_DETAIL
    for (int i = 0; i < MaxDataNum; i++)
    {
        m_aMaxData[i].index = 0xffffffff;
        m_aMaxData[i].value = MinValue();
    }
#endif
}

template <typename T>
void Statistic<T>::Record(T value)
{
    m_Sum += value;
    if (value < m_Min)
    {
        m_Min = value;
    }
    if (value > m_Max)
    {
        m_Max = value;
    }
    ++m_Count;

#if STATISTIC_DETAIL
    for (int i = 0; i < MaxDataNum; i++)
    {
        if (value > m_aMaxData[i].value)
        {
            for (int j = MaxDataNum - 1; j > i; j--)
            {
                m_aMaxData[j].index = m_aMaxData[j - 1].index;
                m_aMaxData[j].value = m_aMaxData[j - 1].value;
            }
            m_aMaxData[i].index = m_Count - 1;
            m_aMaxData[i].value = value;
            break;
        }
    }
#endif
}

template <typename T>
#if STATISTIC_DETAIL
void Statistic<T>::Print(const char* pCaption, uint32_t printMaxDataNum)
#else
void Statistic<T>::Print(const char* pCaption)
#endif
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%s min:", pCaption);
    PrintValue(GetMin());
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" max:");
    PrintValue(GetMax());
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" ave:");
    PrintValue(GetAverage());
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" cnt:%u\n", GetCount());

#if STATISTIC_DETAIL
    uint32_t num = (printMaxDataNum < MaxDataNum) ? printMaxDataNum : MaxDataNum;
    for (uint32_t i = 0; i < num; ++i)
    {
        if (m_aMaxData[i].index >= m_Count)
        {
            break;
        }
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("  %2d: %6u : ", i, m_aMaxData[i].index);
        PrintValue(m_aMaxData[i].value);
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("\n");
    }
#endif
}


template <>
inline int32_t Statistic<int32_t>::MinValue()
{
    // ここでストレートに -2147483648 と書くと、
    // warning C4146: 符号付きの値を代入する変数は、符号付き型にキャストしなければなりません。
    // といった警告が出てしまうので、このような表記としている。
    return -2147483647 - 1;
}
template <>
inline int32_t Statistic<int32_t>::MaxValue()
{
    return 2147483647;
}
template <>
inline int32_t Statistic<int32_t>::ZeroValue()
{
    return 0;
}
template <>
inline void Statistic<int32_t>::PrintValue(const int32_t& v)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%d", v);
}

template <>
inline uint32_t Statistic<uint32_t>::MinValue()
{
    return 0;
}
template <>
inline uint32_t Statistic<uint32_t>::MaxValue()
{
    return 0xffffffff;
}
template <>
inline uint32_t Statistic<uint32_t>::ZeroValue()
{
    return 0;
}
template <>
inline void Statistic<uint32_t>::PrintValue(const uint32_t& v)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%u", v);
}

template <>
inline int64_t Statistic<int64_t>::MinValue()
{
    return LLONG_MIN;
}
template <>
inline int64_t Statistic<int64_t>::MaxValue()
{
    return LLONG_MAX;
}
template <>
inline int64_t Statistic<int64_t>::ZeroValue()
{
    return 0;
}
template <>
inline void Statistic<int64_t>::PrintValue(const int64_t& v)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%lld", v);
}

template <>
inline uint64_t Statistic<uint64_t>::MinValue()
{
    return 0;
}
template <>
inline uint64_t Statistic<uint64_t>::MaxValue()
{
    return ULLONG_MAX;
}
template <>
inline uint64_t Statistic<uint64_t>::ZeroValue()
{
    return 0;
}
template <>
inline void Statistic<uint64_t>::PrintValue(const uint64_t& v)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%llu", v);
}

template <>
inline float Statistic<float>::MinValue()
{
    return FLT_MIN;
}
template <>
inline float Statistic<float>::MaxValue()
{
    return FLT_MAX;
}
template <>
inline float Statistic<float>::ZeroValue()
{
    return 0.0f;
}
template <>
inline void Statistic<float>::PrintValue(const float& v)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%f", v);
}

template <>
inline double Statistic<double>::MinValue()
{
    return DBL_MIN;
}
template <>
inline double Statistic<double>::MaxValue()
{
    return DBL_MAX;
}
template <>
inline double Statistic<double>::ZeroValue()
{
    return 0.0;
}
template <>
inline void Statistic<double>::PrintValue(const double& v)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%f", v);
}

template <>
inline nn::pia::common::TimeSpan Statistic<nn::pia::common::TimeSpan>::MinValue()
{
    nn::pia::common::TimeSpan t;
    t.SetTick(LLONG_MIN);
    return t;
}
template <>
inline nn::pia::common::TimeSpan Statistic<nn::pia::common::TimeSpan>::MaxValue()
{
    nn::pia::common::TimeSpan t;
    t.SetTick(LLONG_MAX);
    return t;
}
template <>
inline nn::pia::common::TimeSpan Statistic<nn::pia::common::TimeSpan>::ZeroValue()
{
    return nn::pia::common::TimeSpan(0);
}
template <>
inline void Statistic<nn::pia::common::TimeSpan>::PrintValue(const nn::pia::common::TimeSpan& v)
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("%fms", v.GetSecF32() * 1000.0f);
}

template <typename T>
inline T Statistic<T>::GetAverage() const
{
    return (m_Count > 0) ? (m_Sum / static_cast<T>(m_Count)) : ZeroValue();
}

template <>
inline nn::pia::common::TimeSpan Statistic<nn::pia::common::TimeSpan>::GetAverage() const
{
    nn::pia::common::TimeSpan t;
    t.SetTick((m_Count > 0) ? (m_Sum.GetTick() / m_Count) : 0);
    return t;
}
}
}  // end of namespace nns::pia
