﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "Thread.h"
#include "Defines.h"
#include "Assert.h"
#include <nn/pia/common.h>


#if NN_PIA_CTR
#include <nn/os.h>
#include <nn/fnd.h>
#endif


namespace
{
}  // end of anonymous namespace

namespace nns
{
namespace pia
{

///< コンストラクタ
Thread::Thread(void)
{
#if NN_PIA_NINTENDOSDK
    m_ThreadIsAlive = false;
#endif
}


///< デストラクタ
Thread::~Thread(void)
{
}


#if NN_PIA_CTR
void Thread::Start(ThreadFunc pFunc, void* pArg, size_t stackSize, int32_t priority)
{
    m_Thread.StartUsingAutoStack(pFunc, pArg, stackSize, priority);
}


bool Thread::IsAlive()
{
    return m_Thread.IsAlive();
}


void Thread::Join(void)
{
    m_Thread.Join();
}


void Thread::Finalize(void)
{
    m_Thread.Finalize();
}


void Thread::Detach(void)
{
    m_Thread.Detach();
}


void Thread::Sleep(int32_t msec)
{
    nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(msec));
}


#elif NN_PIA_WIN


#error "not implemented."


#elif NN_PIA_NINTENDOSDK


void Thread::Start(ThreadFunc pFunc, void* pArg, void* pStackAddr, size_t stackSize, int32_t priority)
{
    // スレッド生成
    nn::Result result = nn::os::CreateThread(&m_ThreadType, pFunc, pArg, pStackAddr, stackSize, priority);
    NN_PIA_UNUSED_IN_RELEASE(result);
    PIA_SAMPLE_ASSERTMSG(result.IsSuccess(), "nn::os::CreateThread() failed.");

    // スレッド実行開始
    nn::os::StartThread(&m_ThreadType);

    m_ThreadIsAlive = true;
}


bool Thread::IsAlive()
{
    return m_ThreadIsAlive;
}


void Thread::Join()
{
    nn::os::WaitThread(&m_ThreadType);
    m_ThreadIsAlive = false;
}


void Thread::Detach()
{
    PIA_SAMPLE_ASSERTMSG(false, "detach operation is not implemented.");
}


void Thread::Finalize()
{
    nn::os::DestroyThread(&m_ThreadType);
    m_ThreadIsAlive = false;
}


void Thread::Sleep(int32_t msec)
{
    nn::os::SleepThread(nn::TimeSpan::FromMilliSeconds(msec));
}


#elif NN_PIA_CAFE


void Thread::Start(ThreadFunc pFunc, void* pArg, void* pStackAddr, size_t stackSize, uint16_t attr, OSPriority priority)
{
    uint16_t attribute = attr;

    // アフィニティが設定されていない場合はサンプルのメインコアでスレッドを起動させる
    if ((attribute & (OS_THREAD_ATTR_AFFINITY_CORE0 | OS_THREAD_ATTR_AFFINITY_CORE1 | OS_THREAD_ATTR_AFFINITY_CORE2)) == 0)
    {
        switch (PIA_SAMPLE_MAIN_CORE)
        {
        case 0:
        {
            attribute |= OS_THREAD_ATTR_AFFINITY_CORE0;
        }
        break;
        case 1:
        {
            attribute |= OS_THREAD_ATTR_AFFINITY_CORE1;
        }
        break;
        case 2:
        {
            attribute |= OS_THREAD_ATTR_AFFINITY_CORE2;
        }
        break;
        }
    }

    OSCreateThread(&m_Thread,
                   pFunc,
                   1,
                   pArg,
                   static_cast<uint8_t*>(pStackAddr) + stackSize,
                   stackSize,
                   priority,
                   attribute);

    OSResumeThread(&m_Thread);
}


bool Thread::IsAlive()
{
    return (!OSIsThreadTerminated(&m_Thread));
}

void Thread::Join(void)
{
    BOOL ret = OSJoinThread(&m_Thread, NULL);
    PIA_SAMPLE_ASSERTMSG(ret, "OSJoinThread() failed.");
}


void Thread::Detach(void)
{
    OSDetachThread(&m_Thread);
}


void Thread::Finalize(void)
{
    //n1769 Cafeの場合は特に何もする必要なし？
}


void Thread::Sleep(int32_t msec)
{
    OSSleepMilliseconds(msec);
}

#elif NN_PIA_A || NN_PIA_B
bool Thread::IsAlive()
{
    //n1849:todo:not implemented
    return false;
}

void Thread::Join(void)
{
    //n1849:todo:not implemented
}

void Thread::Detach(void)
{
    //n1849:todo:not implemented
}

void Thread::Finalize(void)
{
    //n1849:todo:not implemented
}

void Thread::Sleep(int32_t msec)
{
    //n1849:todo:not implemented
}
#else

#error "not implemented."

#endif  // end of NN_PIA_(platform)
}
}  // end of namespace nns::pia
