﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Transport_Setup.h"

namespace
{
}

namespace nns
{
namespace pia
{

// 共通処理のプロトタイプ宣言。
nn::pia::Result transport_InitializePiaTransportImpl(
    const nn::pia::transport::Transport::Setting& setting,
    uint16_t unreliableProtocolPort,
    uint32_t* pUnreliableProtocolHandle,
    uint16_t reliableProtocolPort,
    uint32_t* pReliableProtocolHandle,
    uint32_t reliableSendBufNum,
    uint32_t reliableRecvBufNum,
    InitializeAdditionalProtocolFunction pAdditionalFunction,
    uint32_t unreliableRecvBufNum);


nn::pia::Result Transport_InitializePiaTransport(const nn::pia::transport::Transport::Setting& setting)
{
    return transport_InitializePiaTransportImpl(
        setting,
        0,
        NULL,
        0,
        NULL,
        0,
        0,
        NULL,
        0);
}


nn::pia::Result Transport_InitializePiaTransport(
    const nn::pia::transport::Transport::Setting& setting,
    uint16_t unreliableProtocolPort,
    uint32_t* pUnreliableProtocolHandle,  //n1769: NULL を渡すと、UnreliableProtocol の初期化処理をしない。
    uint16_t reliableProtocolPort,
    uint32_t* pReliableProtocolHandle,  //n1769: NULL を渡すと、ReliableProtocol の初期化処理をしない。
    uint32_t reliableSendBufNum,
    uint32_t reliableRecvBufNum,
    InitializeAdditionalProtocolFunction pAdditionalFunction,
    uint32_t unreliableRecvBufNum)
{
    return transport_InitializePiaTransportImpl(
        setting,
        unreliableProtocolPort,
        pUnreliableProtocolHandle,
        reliableProtocolPort,
        pReliableProtocolHandle,
        reliableSendBufNum,
        reliableRecvBufNum,
        pAdditionalFunction,
        unreliableRecvBufNum);
}


nn::pia::Result transport_InitializePiaTransportImpl(
    const nn::pia::transport::Transport::Setting& setting,
    uint16_t unreliableProtocolPort,
    uint32_t* pUnreliableProtocolHandle,
    uint16_t reliableProtocolPort,
    uint32_t* pReliableProtocolHandle,
    uint32_t reliableSendBufNum,
    uint32_t reliableRecvBufNum,
    InitializeAdditionalProtocolFunction pAdditionalFunction,
    uint32_t unreliableRecvBufNum)
{
    PIA_SAMPLE_PRINT("Transport_InitializePiaTransport\n");
    nn::pia::Result result;

    result = nn::pia::transport::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::transport::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::transport::Transport::CreateInstance(setting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    if (pUnreliableProtocolHandle != NULL)
    {
        *pUnreliableProtocolHandle = nn::pia::transport::Transport::GetInstance()->CreateProtocol<nn::pia::transport::UnreliableProtocol>(unreliableProtocolPort);
        result = nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(*pUnreliableProtocolHandle)->Initialize(unreliableRecvBufNum);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }

    //n1769: ReliableProtocolの初期化処理
    if (pReliableProtocolHandle != NULL)
    {
        const uint32_t SendBufNum = reliableSendBufNum;
        const uint32_t RecvBufNum = reliableRecvBufNum;
        *pReliableProtocolHandle = nn::pia::transport::Transport::GetInstance()->CreateProtocol<nn::pia::transport::ReliableProtocol>(reliableProtocolPort);
        PIA_SAMPLE_ASSERT(*pReliableProtocolHandle != 0);  // CreateProtocol()は、既に同じプロトコルIDが存在していた場合は0を返すので、そうではないことを確認
        result = nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::ReliableProtocol>(*pReliableProtocolHandle)->Initialize(SendBufNum, RecvBufNum);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }

    //m2004: 任意のプロトコルを使えるように
    if (pAdditionalFunction != NULL)
    {
        result = pAdditionalFunction();
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }

    result = nn::pia::transport::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // メモリ消費量をプリント
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Transport_InitializePiaTransport:Pia current memory usage: %d\n", nn::pia::common::GetMemoryUsage());
    return result;
}

void Transport_FinalizePiaTransport()
{
    Transport_FinalizePiaTransport(
        0,
        0,
        NULL);
}

void Transport_FinalizePiaTransport(
    uint32_t unreliableProtocolHandle,
    uint32_t reliableProtocolHandle,
    FinalizeAdditionalProtocolFunction pAdditionalFunction)
{
    PIA_SAMPLE_PRINT("Transport_FinalizePiaTransport\n");

    if (nn::pia::transport::Transport::GetInstance() != NULL)
    {
        //n1769: ReliableProtocolの終了処理
        if (reliableProtocolHandle != 0)
        {
            nn::pia::transport::ReliableProtocol* pReliableProtocol = nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::ReliableProtocol>(reliableProtocolHandle);
            if (pReliableProtocol != NULL)
            {
                pReliableProtocol->Finalize();
                nn::pia::transport::Transport::GetInstance()->DestroyProtocol(reliableProtocolHandle);
            }
        }

        if (unreliableProtocolHandle != 0)
        {
            nn::pia::transport::UnreliableProtocol* pUnreliableProtocol = nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(unreliableProtocolHandle);
            if (pUnreliableProtocol != NULL)
            {
                pUnreliableProtocol->Finalize();
                nn::pia::transport::Transport::GetInstance()->DestroyProtocol(unreliableProtocolHandle);
            }
        }
    }

    //n2004: 任意のプロトコルを使えるように
    if (pAdditionalFunction != NULL)
    {
        pAdditionalFunction();
    }

    // PiaではFinalize、DestroyInstanceは常に成功します
    if (nn::pia::transport::Transport::GetInstance() != NULL)
    {
        nn::pia::transport::Transport::DestroyInstance();
    }
    nn::pia::transport::Finalize();

    PIA_SAMPLE_PRINT("Transport_FinalizePiaTransport:finished\n");
}
}
}  // end of namespace nns::pia
