﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "Object.h"
#if NN_PIA_CAFE
#include <cafe/zipfile.h>
#endif
#include <math.h>
#include <cstdlib>

ReckoningStrategy Object::s_ReckoningStrategy;
SampleLargeData s_SampleLargeDataBuf;

// シリアル化のために SerializePolicy の各関数を定義します。
namespace nn
{
namespace pia
{
namespace clone
{

template<>
void SerializePolicy<StructValue>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    SerializePolicy<uint32_t>::Serialize(offset, value.v1);
    offset += 4;
    SerializePolicy<float>::Serialize(offset, value.v2);
    offset += 4;
#if PIA_TEST_CODE
    for (int i = 0; i < StructValueSizeMax; ++i)
    {
        SerializePolicy<uint32_t>::Serialize(offset, value.v3[i]);
        offset += 4;
    }
#endif
}

template<>
void SerializePolicy<StructValue>::Deserialize(Type* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    SerializePolicy<uint32_t>::Deserialize(&pValue->v1, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->v2, offset);
    offset += 4;
#if PIA_TEST_CODE
    for (int i = 0; i < StructValueSizeMax; ++i)
    {
        SerializePolicy<uint32_t>::Deserialize(&pValue->v3[i], offset);
        offset += 4;
    }
#endif
}

template<>
uint32_t SerializePolicy<StructValue>::GetSerializedSize()
{
#if PIA_TEST_CODE
    return 4 + 4 + StructValueSizeMax * 4;
#else
    return 4 + 4;
#endif
}


template<>
void SerializePolicy<SampleLargeData>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    memcpy(offset, value.dataArray, SampleLargeData::DataSize);
    offset += SampleLargeData::DataSize;
    common::ByteOrder::serializeU32(offset, value.hash);
    offset += 4;
}

template<>
void SerializePolicy<SampleLargeData>::Deserialize(Type* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    memcpy(pValue->dataArray, offset, SampleLargeData::DataSize);
    offset += SampleLargeData::DataSize;
    pValue->hash = common::ByteOrder::deserializeU32(offset);
    offset += 4;
}

template<>
uint32_t SerializePolicy<SampleLargeData>::GetSerializedSize()
{
    return SampleLargeData::DataSize + 4;
}

}
}
}  // end of namespace nn::pia::clone


void Object::Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
    {
        if (IsCloneRegistered())
        {
            UnregisterClone();
        }
        else
        {
            RegisterClone();
        }
    }
#if PIA_TEST_CODE
    OperateForPiaTest();
#else
    OperateElement();
#endif
}

void Object::Update()
{
    if (GetClone().GetState() == nn::pia::clone::CloneBase::State_Inactive)
    {
        ClearEventHistory();
    }
}

bool Object::CleanupCore(nn::pia::clone::CloneBase* pClone)
{
    if (pClone->IsRegisteredWithProtocol())
    {
        UnregisterClone();
    }
    else
    {
        nn::pia::Result r;
        // すでにCloneからUnregisterされている場合にのみUnregisterElementする
        if (m_ElementU1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementU1.GetId());
            r = pClone->UnregisterElement(&m_ElementU1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementU2.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementU2.GetId());
            r = pClone->UnregisterElement(&m_ElementU2);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementU3.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementU3.GetId());
            r = pClone->UnregisterElement(&m_ElementU3);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementR1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementR1.GetId());
            r = pClone->UnregisterElement(&m_ElementR1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementE1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementE1.GetId());
            r = pClone->UnregisterElement(&m_ElementE1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementC1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementC1.GetId());
            r = pClone->UnregisterElement(&m_ElementC1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
    }

    bool ret = !m_ElementU1.IsRegisteredWithCloneBase() &&
               !m_ElementU2.IsRegisteredWithCloneBase() &&
               !m_ElementU3.IsRegisteredWithCloneBase() &&
               !m_ElementR1.IsRegisteredWithCloneBase() &&
               !m_ElementE1.IsRegisteredWithCloneBase() &&
               !m_ElementC1.IsRegisteredWithCloneBase();
    if (ret)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Type:%s - Cleanup Process is Succeeded.\n", GetCloneTypeText());
    }
    return ret;
}
void Object::PrintValue(int y)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
#if PIA_TEST_CODE
    if (m_ElementU1.IsValidValue())
    {
        pSampleFramework->DrawString(50, y, "0x%08x", static_cast<uint32_t>(m_ElementU1.GetValue()));
    }
    if (m_ElementE1.GetValue() != NULL)
    {
        pSampleFramework->DrawString(170, y, "0x%08x", *m_ElementE1.GetValue());
    }
    if (m_ElementR1.IsValidValue())
    {
        pSampleFramework->DrawString(280, y, "0x%08x", m_ElementR1.GetValue());
    }
    if (m_ElementC1.IsValidValue())
    {
        pSampleFramework->DrawString(330, y, "%6.2f", m_ElementC1.GetValue());
    }
#else
#if NN_PIA_CTR
    if (m_ElementU1.IsValidValue())
    {
        pSampleFramework->DrawString(10, y, "%3lld", m_ElementU1.GetValue());
    }
    if (m_ElementU2.IsValidValue())
    {
        pSampleFramework->DrawString(35, y, "%6.2f", m_ElementU2.GetValue());
    }
    if (m_ElementU3.IsValidValue())
    {
        pSampleFramework->DrawString(80, y, "%3d %6.2f", m_ElementU3.GetValue().v1, m_ElementU3.GetValue().v2);
    }
    if (m_ElementR1.IsValidValue())
    {
        pSampleFramework->DrawString(180, y, "%3d", m_ElementR1.GetValue());
    }
    for (int i = 0; i < m_EventHistoryCount; ++i)
    {
        pSampleFramework->DrawString(210 + i * 70, y, "%08X", m_EventHistoryBuffer[m_EventHistoryCount - 1 - i]);
    }
    if (m_ElementL1.IsValidValue())
    {
        const SampleLargeData& sampleLargeData = m_ElementL1.GetValue();
        pSampleFramework->DrawString(280, y, "%08X", sampleLargeData.hash);
    }
    if (m_ElementC1.IsValidValue())
    {
        pSampleFramework->DrawString(350, y, "%6.2f", m_ElementC1.GetValue());
    }
#else
    if (m_ElementU1.IsValidValue())
    {
        pSampleFramework->DrawString(50, y, "%3lld", m_ElementU1.GetValue());
    }
    if (m_ElementU2.IsValidValue())
    {
        pSampleFramework->DrawString(65, y, "%6.2f", m_ElementU2.GetValue());
    }
    if (m_ElementU3.IsValidValue())
    {
        pSampleFramework->DrawString(95, y, "%3d %6.2f", m_ElementU3.GetValue().v1, m_ElementU3.GetValue().v2);
    }
    if (m_ElementR1.IsValidValue())
    {
        pSampleFramework->DrawString(150, y, "%3d", m_ElementR1.GetValue());
    }
    for (int i = 0; i < m_EventHistoryCount; ++i)
    {
        pSampleFramework->DrawString(170 + i * 40, y, "%08X", m_EventHistoryBuffer[m_EventHistoryCount - 1 - i]);
    }
    if (m_ElementL1.IsValidValue())
    {
        const SampleLargeData& sampleLargeData = m_ElementL1.GetValue();
        pSampleFramework->DrawString(340, y, "%08X", sampleLargeData.hash);
    }
    if (m_ElementC1.IsValidValue())
    {
        pSampleFramework->DrawString(390, y, "%6.2f", m_ElementC1.GetValue());
    }
#endif
#endif
}


const char* Object::ConvertCloneElementTypeToText(nn::pia::clone::CloneElementBase::Type type) const
{
    switch (type)
    {
    case nn::pia::clone::CloneElementBase::Type_Unreliable:
        return "U";
    case nn::pia::clone::CloneElementBase::Type_Reliable:
        return "R";
    case nn::pia::clone::CloneElementBase::Type_Event:
        return "E";
    case nn::pia::clone::CloneElementBase::Type_Reckoning:
        return "C";
    case nn::pia::clone::CloneElementBase::Type_ReliableLarge:
        return "L";
    default:
        PIA_SAMPLE_ASSERT(false);
        return NULL;
    }
}

void Object::OperateElement()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
    {
        if (m_ElementR1.IsReadyToSetValue())
        {
            uint32_t v = m_ElementR1.GetValue();
            m_ElementR1.SetValue(v < 100 ? v + 1 : 0);
        }
    }

    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
    {
        if (m_ElementE1.IsReadyToSetValue())
        {
            uint32_t v = ::std::rand();
            nn::pia::Result r = m_ElementE1.SetValue(v);
            if (r.IsSuccess())
            {
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendEvent CID:%s-%d Arg:%d Clock:%d UsingBufferNum:%d\n",
                                 GetCloneTypeText(),
                                 m_CloneId,
                                 v,
                                 pCloneProtocol->GetClock(),
                                 m_ElementE1.GetUsingBufferNum());
            }
        }
    }

    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
    {
        if (m_ElementL1.IsReadyToSetValue())
        {
            for (int i = 0; i < sizeof(s_SampleLargeDataBuf.dataArray) / sizeof(s_SampleLargeDataBuf.dataArray[0]); ++i)
            {
                s_SampleLargeDataBuf.dataArray[i] = ::std::rand();
            }
#if NN_PIA_CTR
            s_SampleLargeDataBuf.hash = nn::util::Crc32::Calculate(s_SampleLargeDataBuf.dataArray, sizeof(s_SampleLargeDataBuf.dataArray));
#elif NN_PIA_CAFE
            s_SampleLargeDataBuf.hash = ZIPFILE_CalcCRC(reinterpret_cast<const uint8_t*>(s_SampleLargeDataBuf.dataArray), sizeof(s_SampleLargeDataBuf.dataArray));
#elif NN_PIA_NINTENDOSDK
            s_SampleLargeDataBuf.hash = s_SampleLargeDataBuf.dataArray[0];
#else
            s_SampleLargeDataBuf.hash = s_SampleLargeDataBuf.dataArray[0];
#endif

            nn::pia::Result r = m_ElementL1.SetValue(s_SampleLargeDataBuf);
            if (r.IsSuccess())
            {
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendReliableLarge CID:%s-%d Hash:0x%08x Clock:%d\n",
                                 GetCloneTypeText(),
                                 m_CloneId,
                                 s_SampleLargeDataBuf.hash,
                                 pCloneProtocol->GetClock());
            }
        }
    }
}

void Object::UpdateElement()
{
    if (m_ElementU1.IsReadyToSetValue())
    {
        uint64_t v = m_ElementU1.GetValue();
        m_ElementU1.SetValue(v < 100 ? v + 1 : 0);
    }
    if (m_ElementU2.IsReadyToSetValue())
    {
        float v = m_ElementU2.GetValue();
        m_ElementU2.SetValue(v > 1.0f ? v * 0.9f : 100.0f);
    }
    if (m_ElementU3.IsReadyToSetValue())
    {
        const StructValue& v = m_ElementU3.GetValue();
        StructValue vv;
        vv.v1 = v.v1 < 100 ? v.v1 + 1 : 0;
        vv.v2 = v.v2 + v.v1;
        m_ElementU3.SetValue(vv);
    }
    while (true)
    {
        const uint32_t* cpArg = m_ElementE1.HandleNext();
        if (cpArg == NULL)
        {
            break;
        }
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RecvEvent CID:%s-%d Arg:%d Clock:%d SetterStationIndex:%d EventIndex:%d UsingBufferNum:%d\n",
                         GetCloneTypeText(),
                         m_CloneId,
                         *cpArg,
                         m_ElementE1.GetClock(),
                         m_ElementE1.GetSetterStationIndex(),
                         m_ElementE1.GetEventIndex(),
                         m_ElementE1.GetUsingBufferNum());

        // GetValueのチェック
        const uint32_t* cpArg2 = m_ElementE1.GetValue();
        PIA_SAMPLE_ASSERT(cpArg2 != NULL);
        PIA_SAMPLE_ASSERT(*cpArg == *cpArg2);

        AddEventHistory(*cpArg2);
    }
    if (m_ElementC1.IsReadyToSetValue())
    {
        float clock = static_cast<float>(nns::pia::Clone_GetCloneProtocol()->GetClock());
        float sin = sinf(clock * 0.001f) * 50.0f;
        m_ElementC1.SetValue(sin);
    }
    if (IsReceiveEnabled())
    {
        if (m_ElementL1.IsValidValue())
        {
            const SampleLargeData& sampleLargeData = m_ElementL1.GetValue();
            if (sampleLargeData.hash != m_SampleLargeDataPreHash)
            {
                m_SampleLargeDataPreHash = sampleLargeData.hash;
#if NN_PIA_CTR
                uint32_t hash = nn::util::Crc32::Calculate(sampleLargeData.dataArray, sizeof(sampleLargeData.dataArray));
#elif NN_PIA_CAFE
                uint32_t hash = ZIPFILE_CalcCRC(reinterpret_cast<const uint8_t*>(sampleLargeData.dataArray), sizeof(sampleLargeData.dataArray));
#elif NN_PIA_NINTENDOSDK
                uint32_t hash = sampleLargeData.dataArray[0];
#else
                uint32_t hash = sampleLargeData.dataArray[0];
#endif
                PIA_SAMPLE_ASSERT(hash == sampleLargeData.hash);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RecvReliableLarge CID:%s-%d Hash:0x%08x Clock:%d\n",
                                 GetCloneTypeText(),
                                 m_CloneId,
                                 hash,
                                 m_ElementL1.GetClock());
            }
        }
    }
}

void Object::AddEventHistory(uint32_t value)
{
    if (m_EventHistoryCount == EventHistoryCountMax)
    {
        for (int i = 0; i < EventHistoryCountMax - 1; ++i)
        {
            m_EventHistoryBuffer[i] = m_EventHistoryBuffer[i + 1];
        }
        m_EventHistoryBuffer[EventHistoryCountMax - 1] = value;
    }
    else
    {
        m_EventHistoryBuffer[m_EventHistoryCount] = value;
        ++m_EventHistoryCount;
    }
}

void Object::ClearEventHistory()
{
    for (int i = 0; i < EventHistoryCountMax; ++i)
    {
        m_EventHistoryBuffer[i] = 0;
    }
    m_EventHistoryCount = 0;
}

void Object::TraceProfiling() const
{
    if (GetClone().GetSendProfiler() != NULL || GetClone().GetReceiveProfiler() != NULL)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("==== Profiling Result (Clone) Begin CID:%s-%d ====\n", GetCloneTypeText(), m_CloneId);
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("                        Latest          Total\n");
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("                        Size  Count     Size  Count\n");

        if (GetClone().GetSendProfiler())
        {
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send                (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 GetClone().GetProfiledLatestSendSize(stationIndex),
                                 GetClone().GetProfiledLatestSendCount(stationIndex),
                                 GetClone().GetProfiledTotalSendSize(stationIndex),
                                 GetClone().GetProfiledTotalSendCount(stationIndex));
            }
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send [NoCompressed] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 GetClone().GetProfiledLatestSendNoCompressedSize(stationIndex),
                                 GetClone().GetProfiledLatestSendCount(stationIndex),
                                 GetClone().GetProfiledTotalSendNoCompressedSize(stationIndex),
                                 GetClone().GetProfiledTotalSendCount(stationIndex));
            }
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send      [Unicast] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 GetClone().GetProfiledLatestSendUnicastSize(stationIndex),
                                 GetClone().GetProfiledLatestSendUnicastCount(stationIndex),
                                 GetClone().GetProfiledTotalSendUnicastSize(stationIndex),
                                 GetClone().GetProfiledTotalSendUnicastCount(stationIndex));
            }
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send       [System] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 GetClone().GetProfiledLatestSendSystemSize(stationIndex),
                                 GetClone().GetProfiledLatestSendSystemCount(stationIndex),
                                 GetClone().GetProfiledTotalSendSystemSize(stationIndex),
                                 GetClone().GetProfiledTotalSendSystemCount(stationIndex));
            }
#if NN_PIA_ENABLE_CACHEDPRINT
            if (nn::pia::common::CachedPrint::GetInstance())
            {
                nn::pia::common::CachedPrint::GetInstance()->Flush();
            }
#endif // NN_PIA_ENABLE_CACHEDPRINT
        }

        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("--------------------------------------------------------\n");

        if (GetClone().GetReceiveProfiler())
        {
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Recv                (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 GetClone().GetProfiledLatestReceiveSize(stationIndex),
                                 GetClone().GetProfiledLatestReceiveCount(stationIndex),
                                 GetClone().GetProfiledTotalReceiveSize(stationIndex),
                                 GetClone().GetProfiledTotalReceiveCount(stationIndex));
            }
#if NN_PIA_ENABLE_CACHEDPRINT
            if (nn::pia::common::CachedPrint::GetInstance())
            {
                nn::pia::common::CachedPrint::GetInstance()->Flush();
            }
#endif // NN_PIA_ENABLE_CACHEDPRINT
        }
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("==== Profiling Result (Clone) End CID:%s-%d ====\n", GetCloneTypeText(), m_CloneId);
    }

    TraceCloneElementProfiling(&m_ElementU1);
    TraceCloneElementProfiling(&m_ElementU2);
    TraceCloneElementProfiling(&m_ElementU3);
    TraceCloneElementProfiling(&m_ElementR1);
    TraceCloneElementProfiling(&m_ElementE1);
    TraceCloneElementProfiling(&m_ElementC1);
    TraceCloneElementProfiling(&m_ElementL1);
}

void Object::TraceCloneElementProfiling(const nn::pia::clone::CloneElementBase* pCloneElement) const
{
    if (pCloneElement->GetSendProfiler() != NULL || pCloneElement->GetReceiveProfiler() != NULL)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("==== Profiling Result (CloneElement) Begin CID:%s-%d EID:%s(0x%02x) ====\n", GetCloneTypeText(), m_CloneId, ConvertCloneElementTypeToText(pCloneElement->GetType()), pCloneElement->GetId());
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("                        Latest          Total\n");
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("                        Size  Count     Size  Count\n");

        if (pCloneElement->GetSendProfiler())
        {
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send [NoCompressed] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 pCloneElement->GetProfiledLatestSendNoCompressedSize(stationIndex),
                                 pCloneElement->GetProfiledLatestSendCount(stationIndex),
                                 pCloneElement->GetProfiledTotalSendNoCompressedSize(stationIndex),
                                 pCloneElement->GetProfiledTotalSendCount(stationIndex));
            }
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send      [Unicast] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 pCloneElement->GetProfiledLatestSendUnicastSize(stationIndex),
                                 pCloneElement->GetProfiledLatestSendUnicastCount(stationIndex),
                                 pCloneElement->GetProfiledTotalSendUnicastSize(stationIndex),
                                 pCloneElement->GetProfiledTotalSendUnicastCount(stationIndex));
            }
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send       [Resend] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 pCloneElement->GetProfiledLatestResendSize(stationIndex),
                                 pCloneElement->GetProfiledLatestResendCount(stationIndex),
                                 pCloneElement->GetProfiledTotalResendSize(stationIndex),
                                 pCloneElement->GetProfiledTotalResendCount(stationIndex));
            }
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send       [System] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 pCloneElement->GetProfiledLatestSendSystemSize(stationIndex),
                                 pCloneElement->GetProfiledLatestSendSystemCount(stationIndex),
                                 pCloneElement->GetProfiledTotalSendSystemSize(stationIndex),
                                 pCloneElement->GetProfiledTotalSendSystemCount(stationIndex));
            }
#if NN_PIA_ENABLE_CACHEDPRINT
            if (nn::pia::common::CachedPrint::GetInstance())
            {
                nn::pia::common::CachedPrint::GetInstance()->Flush();
            }
#endif // NN_PIA_ENABLE_CACHEDPRINT
        }

        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("--------------------------------------------------------\n");

        if (pCloneElement->GetReceiveProfiler())
        {
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Recv [NoCompressed] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 pCloneElement->GetProfiledLatestReceiveNoCompressedSize(stationIndex),
                                 pCloneElement->GetProfiledLatestReceiveCount(stationIndex),
                                 pCloneElement->GetProfiledTotalReceiveNoCompressedSize(stationIndex),
                                 pCloneElement->GetProfiledTotalReceiveCount(stationIndex));
            }
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Recv       [System] (%02d) %8d %6d %8d %6d\n",
                                 i,
                                 pCloneElement->GetProfiledLatestReceiveSystemSize(stationIndex),
                                 pCloneElement->GetProfiledLatestReceiveSystemCount(stationIndex),
                                 pCloneElement->GetProfiledTotalReceiveSystemSize(stationIndex),
                                 pCloneElement->GetProfiledTotalReceiveSystemCount(stationIndex));
            }
#if NN_PIA_ENABLE_CACHEDPRINT
            if (nn::pia::common::CachedPrint::GetInstance())
            {
                nn::pia::common::CachedPrint::GetInstance()->Flush();
            }
#endif // NN_PIA_ENABLE_CACHEDPRINT
        }
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("==== Profiling Result (CloneElement) End CID:%s-%d EID:%s(%d) ====\n", GetCloneTypeText(), m_CloneId, ConvertCloneElementTypeToText(pCloneElement->GetType()), pCloneElement->GetId());
    }
}

#if PIA_TEST_CODE

void Object::UpdateForPiaTest()
{
}

void Object::OperateForPiaTest()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
    {
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            if (m_ElementR1.IsRegisteredWithCloneBase())
            {
                if (m_ElementR1.IsReadyToSetValue())
                {
                    // ランダムな値を送信する
                    uint32_t v = (static_cast<uint32_t>(nn::pia::common::Scheduler::GetInstance()->GetDispatchedTime().GetTick()) ^ 0x00decafe) + 1;
                    nn::pia::Result r = m_ElementR1.SetValue(v);
                    if (r.IsSuccess())
                    {
                        SetParamSendReliable(v);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendReliable CID:%s-%d Arg:0x%08x Clock:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock());
                    }
                    else
                    {
                        SetParamSendReliable(0);
                    }
                }
            }
        }
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            if (m_ElementE1.IsRegisteredWithCloneBase())
            {
                if (m_ElementE1.IsReadyToSetValue())
                {
                    // ランダムな値を送信する
                    uint32_t v = (static_cast<uint32_t>(nn::pia::common::Scheduler::GetInstance()->GetDispatchedTime().GetTick()) ^ 0xfeedbeef) + 1;
                    nn::pia::Result r = m_ElementE1.SetValue(v);
                    if (r.IsSuccess())
                    {
                        SetParamSendEvent(v);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendEvent CID:%s-%d Arg:0x%08x Clock:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock());
                    }
                    else
                    {
                        SetParamSendEvent(0);
                    }
                }
            }
        }
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            if (m_ElementU1.IsRegisteredWithCloneBase())
            {
                if (m_ElementU1.IsReadyToSetValue())
                {
                    // ランダムな値を送信する
                    uint32_t v = (static_cast<uint32_t>(nn::pia::common::Scheduler::GetInstance()->GetDispatchedTime().GetTick()) ^ 0xfacade55) + 1;
                    nn::pia::Result r = m_ElementU1.SetValue(v);
                    if (r.IsSuccess())
                    {
                        SetParamSendUnreliable(v);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendUnreliable CID:%s-%d Arg:0x%08x Clock:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock());
                    }
                    else
                    {
                        SetParamSendUnreliable(0);
                    }
                }
            }
        }
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            if (m_ElementC1.IsRegisteredWithCloneBase())
            {
                if (m_ElementC1.IsReadyToSetValue())
                {
                    // ランダムな値を送信する
                    uint32_t v = (static_cast<uint32_t>(nn::pia::common::Scheduler::GetInstance()->GetDispatchedTime().GetTick()) ^ 0xacdcdcdc) + 1;
                    nn::pia::Result r = m_ElementC1.SetValue(static_cast<float>(v));
                    if (r.IsSuccess())
                    {
                        SetParamSendReckoning(v);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendReckoning CID:%s-%d Arg:0x%08x Clock:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock());
                    }
                    else
                    {
                        SetParamSendReckoning(0);
                    }
                }
            }
        }
    }
    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
    {
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            if (m_ElementE1.IsRegisteredWithProtocol())
            {
                const uint32_t* cpArg = m_ElementE1.HandleNext();
                if (cpArg != NULL)
                {
                    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RecvEvent CID:%s-%d Arg:%d Clock:%d SetterStationIndex:%d EventIndex:%d UsingBufferNum:%d\n",
                                     GetCloneTypeText(),
                                     m_CloneId,
                                     *cpArg,
                                     m_ElementE1.GetClock(),
                                     m_ElementE1.GetSetterStationIndex(),
                                     m_ElementE1.GetEventIndex(),
                                     m_ElementE1.GetUsingBufferNum());
                }
            }
        }
    }
}

const char* Object::GetConnectionStatusText()
{
    static char connectionStatus[MaxStations + 1];
    for (int i = 0; i < MaxStations; ++i)
    {
        connectionStatus[i] = IsConnected(g_StationIdList[i]) ? '1' : '0';
    }
    connectionStatus[MaxStations] = '\0';
    return connectionStatus;
}
#endif


SendObject::SendObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);
    m_Clone.RegisterElement(&m_ElementE1, 0x21);
    m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    m_Clone.RegisterElement(&m_ElementC1, 0x31);
    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void SendObject::Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    NN_PIA_UNUSED(pSampleFramework);

    Object::Operate();

#if PIA_TEST_CODE
#else
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
    {
        if (m_Clone.IsConnectedWithAll())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendClone: Connected with all\n");
        }
        else
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SendClone: Connected with not all\n");
        }
    }
#endif
}

void SendObject::Update()
{
    Object::Update();
    UpdateElement();
}

bool SendObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void SendObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterSendClone(&m_Clone, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterSendClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterSendClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void SendObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterSendClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterSendClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterSendClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int SendObject::Print(int y, const nns::pia::sampleframework::Color& cColor)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    if (m_Clone.IsRegisteredWithProtocol())
    {
#if PIA_TEST_CODE
        pSampleFramework->DrawString(10, y, "S %1x   R %s", m_Clone.GetId(), GetConnectionStatusText());
#else
        char isConnected[8];
        for (int i = 0; i < 8; ++i)
        {
            if (i < MaxStations)
            {
                isConnected[i] = m_Clone.IsConnected(g_StationIdList[i]) ? '1' : '0';
            }
            else
            {
                isConnected[i] = '0';
            }
        }
        pSampleFramework->DrawString(10, y, "S %1x   R %c%c%c%c%c%c%c%c", m_Clone.GetId(), isConnected[0], isConnected[1], isConnected[2], isConnected[3], isConnected[4], isConnected[5], isConnected[6], isConnected[7]);
#endif
    }
    else
    {
        pSampleFramework->DrawString(10, y, "S %1x", m_CloneId);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void SendObject::UpdatePiaTestData()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(SEND_IS_REGISTERED, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    uint32_t isConnectedBitmap = 0;
    for (int i = 0; i < MaxStations; ++i)
    {
        nn::pia::StationId stationId;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, static_cast<nn::pia::StationIndex>(i));
        if (result.IsSuccess() && m_Clone.IsConnected(stationId))
        {
            isConnectedBitmap |= (0x1 << i);
        }
    }
    pSampleFramework->SetProgramParam(SEND_CONNECTED, isConnectedBitmap);
}

void SendObject::SetParamSendUnreliable(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_UNRELIABLE_SEND, v);
}

void SendObject::SetParamSendReliable(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_RELIABLE_SEND, v);
}

void SendObject::SetParamSendEvent(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_EVENT_SEND, v);
}

void SendObject::SetParamSendReckoning(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_RECKONING_SEND, v);
}
#endif


ReceiveObject::ReceiveObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);
    m_Clone.RegisterElement(&m_ElementE1, 0x21);
    m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    m_Clone.RegisterElement(&m_ElementC1, 0x31);
    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void ReceiveObject::Operate()
{
    Object::Operate();
}

void ReceiveObject::Update()
{
    Object::Update();
    UpdateElement();
}

bool ReceiveObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void ReceiveObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterReceiveClone(&m_Clone, m_StationIndex, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterReceiveClone Success (StationIndex:%d, cloneId:%d)\n", m_StationIndex, m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterReceiveClone Failure (StationIndex:%d, cloneId:%d)\n", m_StationIndex, m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void ReceiveObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterReceiveClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterReceiveClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterReceiveClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int ReceiveObject::Print(int y, const nns::pia::sampleframework::Color& cColor)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    if (m_Clone.IsRegisteredWithProtocol())
    {
        pSampleFramework->DrawString(10, y, "R %1d:%1d R %c", m_Clone.GetId(), m_Clone.GetSendStationIndex(), m_Clone.IsConnected() ? '1' : '0');

        // RTTを表示
        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
        nn::pia::StationId stationId;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, m_Clone.GetSendStationIndex());
        if (result.IsSuccess())
        {
            nn::pia::StationIndex localStationIndex;
            result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&localStationIndex, pSession->GetLocalStationId());
            nn::pia::transport::Station* pStation = nn::pia::transport::StationManager::GetInstance()->GetStation(stationId);
            if (result.IsSuccess() && pStation && localStationIndex != m_Clone.GetSendStationIndex())
            {
                int32_t rtt = pStation->GetRtt();
                pSampleFramework->DrawString(180, y, "RTT: %d", rtt);
            }
        }
    }
    else
    {
        pSampleFramework->DrawString(10, y, "R %1d:%1d", m_CloneId, m_StationIndex);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void ReceiveObject::UpdatePiaTestData()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(RECEIVE_IS_REGISTERED + m_StationIndex, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    pSampleFramework->SetProgramParam(RECEIVE_IS_CONNECTED + m_StationIndex, m_Clone.IsConnected() ? 1 : 0);

    pSampleFramework->SetProgramParam(RECEIVE_UNRELIABLE + m_StationIndex, m_ElementU1.IsValidValue() ? static_cast<uint32_t>(m_ElementU1.GetValue()) : 0);
    pSampleFramework->SetProgramParam(RECEIVE_RELIABLE + m_StationIndex, m_ElementR1.IsValidValue() ? m_ElementR1.GetValue() : 0);
    pSampleFramework->SetProgramParam(RECEIVE_EVENT + m_StationIndex, m_ElementE1.GetValue() != NULL ? *m_ElementE1.GetValue() : 0);
    pSampleFramework->SetProgramParam(RECEIVE_RECKONING + m_StationIndex, m_ElementC1.IsValidValue() ? static_cast<uint32_t>(m_ElementC1.GetValue()) : 0);
}
#endif

AtomicObject::AtomicObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);
    m_Clone.RegisterElement(&m_ElementE1, 0x21);
    m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    m_Clone.RegisterElement(&m_ElementC1, 0x31);
    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void AtomicObject::Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    Object::Operate();

    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
    {
        if (m_Clone.IsRegisteredWithProtocol())
        {
            if (m_Clone.GetLockStatus() == nn::pia::clone::AtomicSharingClone::LockStatus_Unlocked)
            {
                m_Clone.TryLock();
            }
            else
            {
                m_Clone.Unlock();
            }
        }
    }

#if PIA_TEST_CODE
#else
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
    {
        if (m_Clone.IsConnectedWithAll())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("AtomicSharingClone: Connected with all\n");
        }
        else
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("AtomicSharingClone: Connected with not all\n");
        }
    }
#endif
}

void AtomicObject::Update()
{
    Object::Update();
    UpdateElement();
    if (m_ElementE1.IsEventDropped())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("AtomicSharingClone: EventDropped\n");
    }
}

bool AtomicObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void AtomicObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterAtomicSharingClone(&m_Clone, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterAtomicSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterAtomicSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void AtomicObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterAtomicSharingClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterAtomicSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterAtomicSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int AtomicObject::Print(int y, const nns::pia::sampleframework::Color& cColor)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    if (m_Clone.IsRegisteredWithProtocol())
    {
        const char* cpLockStatusText = "";
        switch (m_Clone.GetLockStatus())
        {
        case nn::pia::clone::AtomicSharingClone::LockStatus_Unlocked:
        {
            cpLockStatusText = "Unlocked";
        }
        break;
        case nn::pia::clone::AtomicSharingClone::LockStatus_TryLock:
        {
            cpLockStatusText = "TryLock";
        }
        break;
        case nn::pia::clone::AtomicSharingClone::LockStatus_Locked:
        {
            cpLockStatusText = "Locked";
        }
        break;
        }
#if PIA_TEST_CODE
        pSampleFramework->DrawString(10, y, "A %1d   R %s %s", m_Clone.GetId(), GetConnectionStatusText(), cpLockStatusText);
#else
        char isConnected[8];
        for (int i = 0; i < 8; ++i)
        {
            if (i < MaxStations)
            {
                isConnected[i] = m_Clone.IsConnected(g_StationIdList[i]) ? '1' : '0';
            }
            else
            {
                isConnected[i] = '0';
            }
        }
        pSampleFramework->DrawString(10, y, "A %1d   R %c%c%c%c%c%c%c%c %s", m_Clone.GetId(), isConnected[0], isConnected[1], isConnected[2], isConnected[3], isConnected[4], isConnected[5], isConnected[6], isConnected[7], cpLockStatusText);
#endif
    }
    else
    {
        pSampleFramework->DrawString(10, y, "A %1d", m_CloneId);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void AtomicObject::UpdatePiaTestData()
{
    if (m_CloneId != 0)
    {
        return;
    }

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(ATOMIC_IS_REGISTERED, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    uint32_t isConnectedBitmap = 0;
    for (int i = 0; i < MaxStations; ++i)
    {
        nn::pia::StationId stationId;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, static_cast<nn::pia::StationIndex>(i));
        if (result.IsSuccess() && m_Clone.IsConnected(stationId))
        {
            isConnectedBitmap |= (0x1 << i);
        }
    }
    pSampleFramework->SetProgramParam(ATOMIC_CONNECTED, isConnectedBitmap);

    pSampleFramework->SetProgramParam(ATOMIC_LOCK_STATUS, m_Clone.GetLockStatus());

    pSampleFramework->SetProgramParam(ATOMIC_UNRELIABLE, m_ElementU1.IsValidValue() ? static_cast<uint32_t>(m_ElementU1.GetValue()) : 0);
    pSampleFramework->SetProgramParam(ATOMIC_RELIABLE, m_ElementR1.IsValidValue() ? m_ElementR1.GetValue() : 0);
    pSampleFramework->SetProgramParam(ATOMIC_EVENT, m_ElementE1.GetValue() != NULL ? *m_ElementE1.GetValue() : 0);
    pSampleFramework->SetProgramParam(ATOMIC_RECKONING, m_ElementC1.IsValidValue() ? static_cast<uint32_t>(m_ElementC1.GetValue()) : 0);
}

void AtomicObject::SetParamSendUnreliable(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_UNRELIABLE_SEND, v);
}

void AtomicObject::SetParamSendReliable(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_RELIABLE_SEND, v);
}

void AtomicObject::SetParamSendEvent(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_EVENT_SEND, v);
}

void AtomicObject::SetParamSendReckoning(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_RECKONING_SEND, v);
}
#endif


SequentialObject::SequentialObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);

    // SequentialSharingCloneでEventCloneElement, ReckoningCloneElementは使えません
    //m_Clone.RegisterElement(&m_ElementE1, 0x21);
    //m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    //m_Clone.RegisterElement(&m_ElementC1, 0x31);

    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void SequentialObject::Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    NN_PIA_UNUSED(pSampleFramework);

    Object::Operate();

#if PIA_TEST_CODE
#else
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
    {
        if (m_Clone.IsConnectedWithAll())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SequentialSharingClone: Connected with all\n");
        }
        else
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("SequentialSharingClone: Connected with not all\n");
        }
    }
#endif
}

void SequentialObject::Update()
{
    Object::Update();
    UpdateElement();
}

bool SequentialObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void SequentialObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterSequentialSharingClone(&m_Clone, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterSequentialSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterSequentialSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void SequentialObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterSequentialSharingClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterSequentialSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterSequentialSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int SequentialObject::Print(int y, const nns::pia::sampleframework::Color& cColor)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    if (m_Clone.IsRegisteredWithProtocol())
    {
#if PIA_TEST_CODE
        pSampleFramework->DrawString(10, y, "Q %1d   R %s", m_Clone.GetId(), GetConnectionStatusText());
#else
        char isConnected[8];
        for (int i = 0; i < 8; ++i)
        {
            if (i < MaxStations)
            {
                isConnected[i] = m_Clone.IsConnected(g_StationIdList[i]) ? '1' : '0';
            }
            else
            {
                isConnected[i] = '0';
            }
        }
        pSampleFramework->DrawString(10, y, "Q %1d   R %c%c%c%c%c%c%c%c", m_Clone.GetId(), isConnected[0], isConnected[1], isConnected[2], isConnected[3], isConnected[4], isConnected[5], isConnected[6], isConnected[7]);
#endif
    }
    else
    {
        pSampleFramework->DrawString(10, y, "Q %1d", m_CloneId);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void SequentialObject::UpdatePiaTestData()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(SEQUENTIAL_IS_REGISTERED, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    uint32_t isConnectedBitmap = 0;
    for (int i = 0; i < MaxStations; ++i)
    {
        nn::pia::StationId stationId;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, static_cast<nn::pia::StationIndex>(i));
        if (result.IsSuccess() && m_Clone.IsConnected(stationId))
        {
            isConnectedBitmap |= (0x1 << i);
        }
    }
    pSampleFramework->SetProgramParam(SEQUENTIAL_CONNECTED, isConnectedBitmap);

    pSampleFramework->SetProgramParam(SEQUENTIAL_UNRELIABLE, m_ElementU1.IsValidValue() ? m_ElementU1.GetValue() : 0);
    pSampleFramework->SetProgramParam(SEQUENTIAL_RELIABLE, m_ElementR1.IsValidValue() ? m_ElementR1.GetValue() : 0);
}

void SequentialObject::SetParamSendUnreliable(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEQUENTIAL_UNRELIABLE_SEND, v);
}

void SequentialObject::SetParamSendReliable(uint32_t v)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEQUENTIAL_RELIABLE_SEND, v);
}
#endif
