﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "ReckoningStrategy.h"
#include "SampleConfig.h"

#if PIA_TEST_CODE
const int StructValueSizeMax = 1000 / 4;
#endif
// 共有するデータの型です。
struct StructValue
{
    uint32_t v1;
    float v2;
#if PIA_TEST_CODE
    uint32_t v3[StructValueSizeMax];
#endif
};

struct SampleLargeData
{
    static const uint32_t DataSize = 100 * 1024;
    static const uint32_t SerializedSize = DataSize + 4;

    uint32_t dataArray[DataSize / sizeof(uint32_t)];
    uint32_t hash;
    void Clear()
    {
        memset(dataArray, 0, sizeof(dataArray));
        hash = 0;
    }
};

// Clone に対応するゲームオブジェクトの基底クラスです。
class Object
{
public:
    Object()
        : m_SampleLargeDataPreHash(0)
        , m_EventHistoryCount(0)
    {
    }
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup()
    {
        return true;
    }
    virtual void RegisterClone() = 0;
    virtual void UnregisterClone() = 0;
    virtual bool IsCloneRegistered() const = 0;
    bool CleanupCore(nn::pia::clone::CloneBase* pClone);
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor)
    {
        NN_PIA_UNUSED(cColor);
        return y;
    }
    void PrintValue(int y);
    void ResetProfiler()
    {
        GetClonePtr()->ResetProfiler();
        m_ElementU1.ResetProfiler();
        m_ElementU2.ResetProfiler();
        m_ElementU3.ResetProfiler();
        m_ElementR1.ResetProfiler();
        m_ElementE1.ResetProfiler();
        m_ElementC1.ResetProfiler();
        m_ElementL1.ResetProfiler();
    }
    void TraceProfiling() const;
    void TraceCloneElementProfiling(const nn::pia::clone::CloneElementBase* pCloneElement) const;
    virtual bool IsReceiveEnabled() const
    {
        return true;
    }

    nn::pia::clone::CloneBase::Id m_CloneId;
    nn::pia::clone::UnreliableCloneElement<uint64_t> m_ElementU1;
    nn::pia::clone::UnreliableCloneElement<float> m_ElementU2;
    nn::pia::clone::UnreliableCloneElement<StructValue> m_ElementU3;
    nn::pia::clone::ReliableCloneElement<uint32_t> m_ElementR1;
    nn::pia::clone::EventCloneElement<uint32_t, 8> m_ElementE1;
    nn::pia::reckoning::ReckoningCloneElement<ReckoningStrategy> m_ElementC1;

    nn::pia::clone::ReliableLargeCloneElement<SampleLargeData, SampleLargeData::SerializedSize> m_ElementL1;

    nn::pia::clone::CloneProfiler<MaxStations, 120> m_SendProfiler;
    nn::pia::clone::CloneProfiler<MaxStations, 120> m_ReceiveProfiler;

    nn::pia::clone::CloneProfiler<MaxStations, 120> m_SendProfilerForElement;
    nn::pia::clone::CloneProfiler<MaxStations, 120> m_ReceiveProfilerForElement;

    static ReckoningStrategy s_ReckoningStrategy;

protected:
    virtual nn::pia::clone::CloneBase* GetClonePtr() = 0;
    virtual const nn::pia::clone::CloneBase& GetClone() const = 0;
    virtual const char* GetCloneTypeText() const = 0;
    const char* ConvertCloneElementTypeToText(nn::pia::clone::CloneElementBase::Type type) const;
    void OperateElement();
    void UpdateElement();

    void AddEventHistory(uint32_t value);
    void ClearEventHistory();

    uint32_t m_SampleLargeDataPreHash;

#if NN_PIA_CTR
    static const int EventHistoryCountMax = 1;
#else
    static const int EventHistoryCountMax = 4;
#endif
    uint32_t m_EventHistoryBuffer[EventHistoryCountMax];
    int m_EventHistoryCount;

#if PIA_TEST_CODE
public:
    virtual void OperateForPiaTest();
    virtual void UpdateForPiaTest();
    virtual void UpdatePiaTestData() = 0;

protected:
    virtual nn::pia::StationIndex GetSendStationIndex()
    {
        return nn::pia::StationIndex_Invalid;
    }
    virtual void SetParamSendUnreliable(uint32_t v)
    {
        NN_PIA_UNUSED(v);
    }
    virtual void SetParamSendReliable(uint32_t v)
    {
        NN_PIA_UNUSED(v);
    }
    virtual void SetParamSendEvent(uint32_t v)
    {
        NN_PIA_UNUSED(v);
    }
    virtual void SetParamSendReckoning(uint32_t v)
    {
        NN_PIA_UNUSED(v);
    }
    const char* GetConnectionStatusText();
    virtual bool IsConnected(nn::pia::StationId sid)
    {
        NN_PIA_UNUSED(sid);
        return false;
    }
#endif
};


// SendClone を使ったゲームオブジェクトです。
class SendObject : public Object
{
public:
    SendObject();
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    virtual bool IsCloneRegistered() const
    {
        return m_Clone.IsRegisteredWithProtocol();
    }
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor);
    virtual bool IsReceiveEnabled() const
    {
        return false;
    }

protected:
    virtual nn::pia::clone::CloneBase* GetClonePtr()
    {
        return &m_Clone;
    }
    virtual const nn::pia::clone::CloneBase& GetClone() const
    {
        return m_Clone;
    }
    virtual const char* GetCloneTypeText() const
    {
        return "S";
    }
    nn::pia::clone::SendClone m_Clone;

#if PIA_TEST_CODE
public:
    virtual void UpdatePiaTestData();

protected:
    virtual void SetParamSendUnreliable(uint32_t v);
    virtual void SetParamSendEvent(uint32_t v);
    virtual void SetParamSendReliable(uint32_t v);
    virtual void SetParamSendReckoning(uint32_t v);
    virtual bool IsConnected(nn::pia::StationId sid)
    {
        return m_Clone.IsConnected(sid);
    }
#endif
};

// ReceiveClone を使ったゲームオブジェクトです。
class ReceiveObject : public Object
{
public:
    ReceiveObject();
    void InitCloneTypeText()
    {
        PIA_SNPRINTF(m_CloneTypeText, sizeof(m_CloneTypeText), "R%1d", m_StationIndex);
    }
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    virtual bool IsCloneRegistered() const
    {
        return m_Clone.IsRegisteredWithProtocol();
    }
    virtual void SetStationIndex(nn::pia::StationIndex index)
    {
        m_StationIndex = index;
    }
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor);

protected:
    virtual nn::pia::clone::CloneBase* GetClonePtr()
    {
        return &m_Clone;
    }
    virtual const nn::pia::clone::CloneBase& GetClone() const
    {
        return m_Clone;
    }
    virtual const char* GetCloneTypeText() const
    {
        return m_CloneTypeText;
    }
    char m_CloneTypeText[4];

    nn::pia::clone::ReceiveClone m_Clone;
    nn::pia::StationIndex m_StationIndex;

#if PIA_TEST_CODE
public:
    virtual void UpdatePiaTestData();

protected:
    virtual nn::pia::StationIndex GetSendStationIndex()
    {
        return m_Clone.GetSendStationIndex();
    }
#endif
};

// AtomicSharingClone を使ったゲームオブジェクトです。
class AtomicObject : public Object
{
public:
    AtomicObject();
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    virtual bool IsCloneRegistered() const
    {
        return m_Clone.IsRegisteredWithProtocol();
    }
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor);

protected:
    virtual nn::pia::clone::CloneBase* GetClonePtr()
    {
        return &m_Clone;
    }
    virtual const nn::pia::clone::CloneBase& GetClone() const
    {
        return m_Clone;
    }
    virtual const char* GetCloneTypeText() const
    {
        return "A";
    }
    nn::pia::clone::AtomicSharingClone m_Clone;

#if PIA_TEST_CODE
public:
    virtual void UpdatePiaTestData();

protected:
    virtual void SetParamSendUnreliable(uint32_t v);
    virtual void SetParamSendEvent(uint32_t v);
    virtual void SetParamSendReliable(uint32_t v);
    virtual void SetParamSendReckoning(uint32_t v);
    virtual bool IsConnected(nn::pia::StationId sid)
    {
        return m_Clone.IsConnected(sid);
    }
#endif
};

// SequentialSharingClone を使ったゲームオブジェクトです。
class SequentialObject : public Object
{
public:
    SequentialObject();
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    virtual bool IsCloneRegistered() const
    {
        return m_Clone.IsRegisteredWithProtocol();
    }
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor);

protected:
    virtual nn::pia::clone::CloneBase* GetClonePtr()
    {
        return &m_Clone;
    }
    virtual const nn::pia::clone::CloneBase& GetClone() const
    {
        return m_Clone;
    }
    virtual const char* GetCloneTypeText() const
    {
        return "Q";
    }
    nn::pia::clone::SequentialSharingClone m_Clone;

#if PIA_TEST_CODE
public:
    virtual void UpdatePiaTestData();

protected:
    virtual void SetParamSendUnreliable(uint32_t v);
    virtual void SetParamSendReliable(uint32_t v);
    virtual bool IsConnected(nn::pia::StationId sid)
    {
        return m_Clone.IsConnected(sid);
    }
#endif
};
