﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"

#if NNS_PIA_ENABLE_INET

class EventChecker
{
public:
    enum EventBit
    {
        EventBit_SessionHostChanged = 0x1 << 0,
        EventBit_HostMigrationFailed = 0x1 << 1,
        EventBit_JointSessionHostChanged = 0x1 << 2,
        EventBit_StartCreateJointSession = 0x1 << 3,
        EventBit_EndCreateJointSession = 0x1 << 4,
        EventBit_StartJoinJointSession = 0x1 << 5,
        EventBit_EndJoinJointSession = 0x1 << 6,
        EventBit_StartJoinRandomJointSession = 0x1 << 7,
        EventBit_EndJoinRandomJointSession = 0x1 << 8,
        EventBit_StartDestroyJointSession = 0x1 << 9,
        EventBit_EndDestroyJointSession = 0x1 << 10,
        EventBit_StartLeaveJointSession = 0x1 << 11,
        EventBit_EndLeaveJointSession = 0x1 << 12,
        EventBit_SessionMigrationFailed = 0x1 << 13
    };

    enum CheckTarget
    {
        CheckTarget_None = 0,
        CheckTarget_SessionMigration,
        CheckTarget_HostMigration
    };

    EventChecker()
    {
        Reset();
    }

    void Reset()
    {
        m_CheckTarget = CheckTarget_None;
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            m_JoinStationIdList[i] = nn::pia::StationIdInvalid;
            m_LeaveStationIdList[i] = nn::pia::StationIdInvalid;
            m_ExpectedLeaveStationIdList[i] = nn::pia::StationIdInvalid;
        }
        UpdateValidStationIdList();
        m_EventFlag = 0;
        m_ExpectedEventFlag = 0;
    }

    bool IsCheckEnabled() const
    {
        return m_CheckTarget != CheckTarget_None;
    }

    CheckTarget GetCheckTarget() const
    {
        return m_CheckTarget;
    }

    void CheckBegin(CheckTarget target)
    {
        PIA_SAMPLE_PRINT("[EventChecker] CheckBegin %d\n", target);
        PIA_SAMPLE_ASSERT(m_CheckTarget != target);
        Reset();
        m_CheckTarget = target;
    }

    void CheckEnd(CheckTarget target)
    {
        NN_PIA_UNUSED_IN_RELEASE(target);
        PIA_SAMPLE_PRINT("[EventChecker] CheckEnd %d\n", target);
        PIA_SAMPLE_ASSERT(m_CheckTarget == target);
        PIA_SAMPLE_ASSERT(IsCheckEnabled());
        Reset();
    }

    uint32_t GetEventFlag() const
    {
        return m_EventFlag;
    }

    uint32_t GetExpectedEventFlag() const
    {
        return m_ExpectedEventFlag;
    }

    void SetEventFlag(EventBit eventBit)
    {
        if (!IsCheckEnabled())
        {
            return;
        }
        if (eventBit != EventBit_SessionHostChanged && eventBit != EventBit_JointSessionHostChanged)
        {
            PIA_SAMPLE_ASSERT((m_EventFlag & eventBit) == 0);
        }
        m_EventFlag |= eventBit;
    }

    void SetExpectedEventFlag(EventBit eventBit)
    {
        m_ExpectedEventFlag |= eventBit;
    }

    void UnsetExpectedEventFlag(EventBit eventBit)
    {
        m_ExpectedEventFlag &= ~eventBit;
    }

    void SetJoinEventStationId(nn::pia::StationId stationId)
    {
        if (!IsCheckEnabled())
        {
            return;
        }
        PIA_SAMPLE_ASSERT(stationId != nn::pia::StationIdInvalid);
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            PIA_SAMPLE_ASSERT(m_JoinStationIdList[i] != stationId);
        }
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            if (m_JoinStationIdList[i] == nn::pia::StationIdInvalid)
            {
                m_JoinStationIdList[i] = stationId;
                return;
            }
        }
        PIA_SAMPLE_ASSERT(false);
    }

    void SetLeaveEventStationId(nn::pia::StationId stationId)
    {
        if (!IsCheckEnabled())
        {
            return;
        }
        PIA_SAMPLE_ASSERT(stationId != nn::pia::StationIdInvalid);
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            PIA_SAMPLE_ASSERT(m_LeaveStationIdList[i] != stationId);
        }
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            if (m_LeaveStationIdList[i] == nn::pia::StationIdInvalid)
            {
                m_LeaveStationIdList[i] = stationId;
                return;
            }
        }
        PIA_SAMPLE_ASSERT(false);
    }

    void SetExpectedLeaveEventStationId(nn::pia::StationId stationId)
    {
        PIA_SAMPLE_ASSERT(stationId != nn::pia::StationIdInvalid);
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            PIA_SAMPLE_ASSERT(m_ExpectedLeaveStationIdList[i] != stationId);
        }
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            if (m_ExpectedLeaveStationIdList[i] == nn::pia::StationIdInvalid)
            {
                m_ExpectedLeaveStationIdList[i] = stationId;
                return;
            }
        }
        PIA_SAMPLE_ASSERT(false);
    }

    void UpdateValidStationIdList()
    {
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            m_ValidStationIdList[i] = nn::pia::StationIdInvalid;
        }
        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
        if (!PIA_IS_VALID_POINTER(pSession))
        {
            return;
        }
        int index = 0;
        nn::pia::session::Session::StationIdIterator itEnd = pSession->End();
        for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != itEnd; ++it)
        {
            m_ValidStationIdList[index] = *it;
            ++index;
        }
    }

    void CheckStationIdList() const
    {
        // Valid + Join - Leave = current

        nn::pia::StationId tmpValidStationIdList[GamePlayerMax];
        nn::pia::StationId tmpJoinStationIdList[GamePlayerMax];
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            tmpValidStationIdList[i] = m_ValidStationIdList[i];
            tmpJoinStationIdList[i] = m_JoinStationIdList[i];
        }

        // 通常 tmpValid と tmpJoin の両方に含まれることはないはず
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            if (tmpValidStationIdList[i] != nn::pia::StationIdInvalid)
            {
                for (int j = 0; j < GamePlayerMax; ++j)
                {
                    // tmpValid と tmpJoin の両方に含まれる場合は途中で抜けて、
                    // 相手側から参加した場合になるが、このとき Leave も発生しているはず
                    if (tmpValidStationIdList[i] == tmpJoinStationIdList[j])
                    {
                        bool isFound = false;
                        NN_PIA_UNUSED_IN_RELEASE(isFound);
                        for (int k = 0; k < GamePlayerMax; ++k)
                        {
                            if (tmpValidStationIdList[i] == m_LeaveStationIdList[k])
                            {
                                isFound = true;
                                break;
                            }
                            if (tmpJoinStationIdList[j] == m_LeaveStationIdList[k])
                            {
                                isFound = true;
                                break;
                            }
                        }
                        PIA_SAMPLE_ASSERT(isFound);
                    }
                }
            }
        }

        // Leave は tmpValid か tmpJoin に存在するはず
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            if (m_LeaveStationIdList[i] != nn::pia::StationIdInvalid)
            {
                bool isFound = false;
                NN_PIA_UNUSED_IN_RELEASE(isFound);
                for (int j = 0; j < GamePlayerMax; ++j)
                {
                    if (tmpValidStationIdList[j] == m_LeaveStationIdList[i])
                    {
                        isFound = true;
                        tmpValidStationIdList[j] = nn::pia::StationIdInvalid;
                        break;
                    }
                    if (tmpJoinStationIdList[j] == m_LeaveStationIdList[i])
                    {
                        isFound = true;
                        tmpJoinStationIdList[j] = nn::pia::StationIdInvalid;
                        break;
                    }
                }
                PIA_SAMPLE_ASSERT(isFound);
            }
        }

        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();

        for (int i = 0; i < GamePlayerMax; ++i)
        {
            // tmpValid と tmpJoin は現在有効なはず
            if (tmpValidStationIdList[i] != nn::pia::StationIdInvalid)
            {
                PIA_SAMPLE_ASSERT(pSession->IsValidStationId(tmpValidStationIdList[i]));
            }
            if (tmpJoinStationIdList[i] != nn::pia::StationIdInvalid)
            {
                PIA_SAMPLE_ASSERT(pSession->IsValidStationId(tmpJoinStationIdList[i]));
            }
        }

        nn::pia::session::Session::StationIdIterator itEnd = pSession->End();
        for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != itEnd; ++it)
        {
            PIA_SAMPLE_ASSERT(*it != nn::pia::StationIdInvalid);
            // tmpValid か tmpJoin に存在するはず
            bool isFound = false;
            NN_PIA_UNUSED_IN_RELEASE(isFound);
            for (int i = 0; i < GamePlayerMax; ++i)
            {
                if (tmpValidStationIdList[i] == *it)
                {
                    isFound = true;
                    tmpValidStationIdList[i] = nn::pia::StationIdInvalid;
                    break;
                }
                if (tmpJoinStationIdList[i] == *it)
                {
                    isFound = true;
                    tmpJoinStationIdList[i] = nn::pia::StationIdInvalid;
                    break;
                }
            }
            PIA_SAMPLE_ASSERT(isFound);
        }

        for (int i = 0; i < GamePlayerMax; ++i)
        {
            // tmpValid と tmpJoin は空になっているはず
            PIA_SAMPLE_ASSERT(tmpValidStationIdList[i] == nn::pia::StationIdInvalid);
            PIA_SAMPLE_ASSERT(tmpJoinStationIdList[i] == nn::pia::StationIdInvalid);
        }
    }

    void CheckExpectedLeaveStationIdList() const
    {
        // ExpectedLeave が Leave に含まれているか調べる
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            if (m_ExpectedLeaveStationIdList[i] != nn::pia::StationIdInvalid)
            {
                bool isFound = false;
                NN_PIA_UNUSED_IN_RELEASE(isFound);
                for (int j = 0; j < GamePlayerMax; ++j)
                {
                    if (m_ExpectedLeaveStationIdList[i] == m_LeaveStationIdList[j])
                    {
                        isFound = true;
                        break;
                    }
                }
                PIA_SAMPLE_ASSERT(isFound);
            }
        }
    }

    void Trace() const
    {
        PIA_SAMPLE_PRINT("--- EventChecker Trace Begin ---\n");
        PIA_SAMPLE_PRINT("Target:%d Event:0x%08x ExpectedEvent:0x%08x\n", m_CheckTarget, m_EventFlag, m_ExpectedEventFlag);
        PIA_SAMPLE_PRINT("Valid: ");
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            PIA_SAMPLE_PRINT("[%d]%llu ", i, m_ValidStationIdList[i]);
        }
        PIA_SAMPLE_PRINT("\n");
        PIA_SAMPLE_PRINT("Join: ");
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            PIA_SAMPLE_PRINT("[%d]%llu ", i, m_JoinStationIdList[i]);
        }
        PIA_SAMPLE_PRINT("\n");
        PIA_SAMPLE_PRINT("Leave: ");
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            PIA_SAMPLE_PRINT("[%d]%llu ", i, m_LeaveStationIdList[i]);
        }
        PIA_SAMPLE_PRINT("\n");
        PIA_SAMPLE_PRINT("ExpectedLeave: ");
        for (int i = 0; i < GamePlayerMax; ++i)
        {
            PIA_SAMPLE_PRINT("[%d]%llu ", i, m_ExpectedLeaveStationIdList[i]);
        }
        PIA_SAMPLE_PRINT("\n");
        PIA_SAMPLE_PRINT("--- EventChecker Trace End ---\n");
    }

private:
    CheckTarget m_CheckTarget;
    uint32_t m_EventFlag;
    uint32_t m_ExpectedEventFlag;
    nn::pia::StationId m_ValidStationIdList[GamePlayerMax];
    nn::pia::StationId m_JoinStationIdList[GamePlayerMax];
    nn::pia::StationId m_LeaveStationIdList[GamePlayerMax];
    nn::pia::StationId m_ExpectedLeaveStationIdList[GamePlayerMax];
};

// イベントチェック用
extern EventChecker g_SessionMigrationEventChecker;
extern EventChecker g_HostMigrationEventChecker;

#endif
