﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/common.h>
#include <nn/pia/common/common_Event.h>
#include <nn/pia/inet.h>
#include <nn/pia/transport/transport_StationConnectionInfo.h>

#include "SampleLib/Defines.h"
#include "SampleLib/ErrorHandling.h"
#include "SampleLib/SampleLib.h"
#include "SampleLib/Thread.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"

#include "SampleConfig.h"

#if NN_PIA_CTR
#include <nn/ac.h>
#include <nn/os.h>
#endif


/*!

    @brief インターネット通信固有の処理をまとめた *.cpp

*/


namespace
{
// InitializePiaModules の進捗を画面表示する
void InitializingProcessRender(uint32_t processingStep)
{
    // 電波強度情報の取得
    uint32_t linkLevel = GetLinkLevel();
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_PrepareInet,
        false,
        false,
        0,
        linkLevel);

    nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
    int line = 0;
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo]", ProgramNameString);
    line += FontSize;
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Inet Initializing process.");
    line += FontSize;

    line += FontSize;

    // 進捗の表示
    // 処理済みステップを白、未処理ステップを赤で表示
    {
        const char* StepString[] = {
            "Initialize NEX",
            "Initialize Pia modules",
            "Initialize Network",
            "Login server"
        };  // 4 step
        const uint32_t StepMax = 4;

        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   START");
        line += FontSize;
        for (uint32_t step = 0; step < StepMax; step++)
        {
            if (step == processingStep)
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "     |");
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorRed);
            }
            else
            {
                line += FontSize;
            }
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   %s", StepString[step]);
            line += FontSize;
        }
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   END");
        line += FontSize;
    }
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

    line = ScreenHeight - FontSize;
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
    line += FontSize;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

    nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
}

}  // end of anonymous namespace

#if NN_PIA_CTR
// AP との通信電波強度を取得し、0-3 の値に変換して返す関数
uint32_t GetInternetLinkLevel()
{
    nn::ac::CTR::LinkLevel acLinkLevel = nn::ac::CTR::GetLinkLevel();

    switch (acLinkLevel)
    {
    case nn::ac::CTR::LINK_LEVEL_3:
        return 3;
    case nn::ac::CTR::LINK_LEVEL_2:
        return 2;
    case nn::ac::CTR::LINK_LEVEL_1:
        return 1;
    case nn::ac::CTR::LINK_LEVEL_0:
    default:
        return 0;
    }
}
#endif

/* ---< インターネット通信時のセットアップ >--- */
// NEX の初期化処理、Pia モジュールの初期化処理、インターネット接続処理、ゲームサーバへのログインを行います。
bool InitializePiaModules()
{
    nn::pia::Result result;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_PrepareInet,
        false,
        false,
        0);

    // 長時間ブロックする処理が続くため、進捗を画面表示しておく
    uint32_t step = 0;
    InitializingProcessRender(step);
    step++;

    /* NEX 関連初期化処理 開始 */

    result = nns::pia::Nex_InitializeNex(
        MemoryBlockSizeForNex,
        MemoryBlockSizeForNexEmergency);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    /* NEX 関連初期化処理 終了 */

    InitializingProcessRender(step);
    step++;

    PIA_SAMPLE_PRINT("NEX initialize completed.\n");

    /* Pia モジュールの初期化処理 開始 */

    result = nns::pia::Inet_InitializePiaInet();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    InitializePiaTransport();

    InitializePiaClone();

    InitializePiaSession();

    /* Pia モジュールの初期化処理 終了 */

    InitializingProcessRender(step);
    step++;

    PIA_SAMPLE_PRINT("Pia modules initialize completed.\n");

    /* インターネット通信準備処理 */
    // 長期間ブロックするため、メインスレッドでの実行は推奨されませんが
    // このサンプルプログラムでは処理の可読性を重視して、ここで行います。
    nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(nns::pia::NetworkType_Internet);

    if (netResult != nns::pia::NetResult_Success)
    {
        if (netResult == nns::pia::NetResult_AccountError)
        {
            SetCommunicationErrorType(SampleErrorType_InitializeByAccount);
        }
        else if (netResult == nns::pia::NetResult_AcError || 
                 netResult == nns::pia::NetResult_NicDown || 
                 netResult == nns::pia::NetResult_SocketError)
        {
            SetCommunicationErrorType(SampleErrorType_InitializeByNetworkInterfaceUp);
        }
        else
        {
            SetCommunicationErrorType(SampleErrorType_InitializeByOtherReason);
        }
        PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork failed.\n");
        SetNetworkErrorResult(netResult);
        return false;
    }

    /* インターネット接続準備処理 終了 */

    InitializingProcessRender(step);
    step++;

    PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork complete.\n");

    /* サーバへのログイン処理 開始 */

    // 内部でフレンドサーバー、ゲームサーバーへのログイン API（非同期）を呼び出します。
    // このサンプルプログラムでは処理の可読性を重視して、非同期処理の完了を内部で待ちます。
    if (!nns::pia::Inet_Login())
    {
        // ログイン処理に失敗
        SetCommunicationErrorType(SampleErrorType_InitializeByOtherReason);
        PIA_SAMPLE_PRINT("nns::pia::Inet_Login failed.\n");
        return false;
    }

    /* サーバへのログイン処理 終了 */

    InitializingProcessRender(step);
    step++;

    PIA_SAMPLE_PRINT("nns::pia::Inet_Login completed.\n");

    return true;
}


void FinalizePiaModules()
{
    PIA_SAMPLE_PRINT("FinalizePiaModules\n");

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_Reset,
        false,
        false,
        0);

    // Facade の Cleanup 前に Session の Cleanup を終えておく
    nns::pia::Session_CleanupSession();

    /* サーバからのログアウト処理 */
    nns::pia::Inet_Logout();

    /* インターネット終了（切断）処理 */
    nns::pia::Net_FinalizeNetwork();

    // PiaSession の終了
    FinalizePiaSession();

    // PiaClone の終了
    FinalizePiaClone();

    // PiaTransport の終了
    FinalizePiaTransport();

    /* PiaInet の終了処理 */
    nns::pia::Inet_FinalizePiaInet();

    /* NEX 関連終了処理 */
    nns::pia::Nex_FinalizeNex();
}

#endif
