﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleLib/SampleLib.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Net_Setup.h"

#include <nn/pia/pia_Platform.h>

#include <nn/pia/inet.h>
#include <nn/pia/inet/inet_NexFacade.h>
#include <nn/pia/inet/inet_NexNetworkFactory.h>

#include <nn/pia/transport.h>
#include <nn/pia/session.h>
#include <nn/pia/clone.h>
#include <nn/pia/reckoning.h>


#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

const uint32_t FramePerSecond = 60;
const char ProgramNameString[] = "clone/TeamMatchmake";

/*!
    @brief サンプルプログラムのシーンを示す列挙型です。
 */
enum SampleSceneNo
{
    SampleSceneNo_PrepareInet,                  //!< インターネット通信を行う準備を行うシーンです。P2P 通信とは無関係です。
    SampleSceneNo_SelectTeam,                   //!< 自分の所属するチームを選択するシーンです。
    SampleSceneNo_MatchmakeTeam,                //!< 参加するチームセッションを作る/参加する処理を行うシーンです。
    SampleSceneNo_CommunicateTeam,              //!< チームセッション内の相手と通信を行うシーンです。別チームとのマッチメイクを行い、ゲームセッションへ移行することもできます。
    SampleSceneNo_SessionMigrationPreprocess,   //!< チームセッション - ゲームセッション間の移行処理の前処理を行うシーンです。
    SampleSceneNo_SessionMigration,             //!< チームセッション - ゲームセッション間の移行処理待ちのシーンです。
    SampleSceneNo_SessionMigrationPostprocess,  //!< チームセッション - ゲームセッション間の移行処理の後処理を行うシーンです。
    SampleSceneNo_CommunicateGame,              //!< ゲームセッション内の相手と通信を行うシーンです。
    SampleSceneNo_NetworkError,                 //!< ネットワークエラーコードを表示させるシーンです。本来はこのシーンでエラー・EULAアプレットを起動することを想定しています。
    SampleSceneNo_Reset                         //!< デモが管理するすべての状態をリセットし、初期状態に戻すシーンです。
};

/*!
    @brief サンプルプログラムでハンドリングする通信エラーの種別です。
 */
enum SampleErrorType
{
    SampleErrorType_Unknown = 0,                     //!< エラーは発生していません
    SampleErrorType_InitializeByNetworkInterfaceUp,  //!< 無線 OFF モードのため、初期化に失敗
    SampleErrorType_InitializeByAccount,             //!< アカウントが必要な場合に有効なアカウントが存在しなかった
    SampleErrorType_InitializeByOtherReason,         //!< その他の理由で、初期化に失敗
    SampleErrorType_Matchmake,                       //!< セッションを確立できませんでした
    SampleErrorType_SessionLost,                     //!< 通信中にセッションから切断されました
    SampleErrorType_ConnectionLost,                  //!< ネットワーク接続が断絶しました
    SampleErrorType_Application                      //!< アプリケーションのエラー
};

void SetCommunicationErrorType(SampleErrorType error);   //通信エラーの内容をセットする関数です。
void SetNetworkErrorResult(nn::pia::Result result);      //Pia API 呼び出しが失敗した場合に返される nn::pia::Result 値をセットする関数です。
void SetNetworkErrorResult(nns::pia::NetResult result);  //非 Pia API 呼び出しが失敗した場合に返される nns::pia::NetResult 値をセットする関数です。
void SetResetOfNetworkIsNecessary(bool isNecessary);     //通信エラー時にネットワークのリセット（再ログインなど）が必要か否かをセットする関数です。

void InitializePiaTransport();
void FinalizePiaTransport();
void InitializePiaSession();
void FinalizePiaSession();
void InitializePiaClone();
void FinalizePiaClone();
bool InitializePiaModules();
void FinalizePiaModules();

/* ---< 処理別シーン >--- */
SampleSceneNo SceneSelectTeam();
SampleSceneNo SceneMatchmake();
SampleSceneNo SceneCommunicationTeam();
SampleSceneNo SceneChangingSessionPreprocessing();
SampleSceneNo SceneChangingSession();
SampleSceneNo SceneChangingSessionPostprocessing();
SampleSceneNo SceneCommunicationGame();
SampleSceneNo SceneNetworkError();

/* ---< 所属するチームを区別するための列挙型 >--- */
enum SampleTeamType
{
    SampleTeamType_Alpha = 0,
    SampleTeamType_Bravo,
    SampleTeamType_Charlie,
    SampleTeamType_Delta,
    SampleTeamType_Max
};
SampleTeamType GetCurrentTeamType();

/* ---< サンプルで使用する構造体定義 >--- */
struct RectPosition  //!< 矩形の位置座標
{
    int32_t x;
    int32_t y;
};

//セッション内での役割画面表示用文字列
const char* GetSessionRoleString(int index);

// セッション内の相手と通信を行うシーンで、自分の情報を送信する間隔[frame]の設定です。
// 0 にすると毎ゲームフレーム送信します。
const uint32_t IntervalSendingStationStatus = 3;

const int TeamMemberMax = 4;                        //!< チーム 1 つの最大参加人数。
const int TeamMax = 3;                              //!< 対戦マッチメイクで合流するチーム数
const int GamePlayerMax = TeamMemberMax * TeamMax;  //!< 別チームと合流後の最大人数。

const bool HostMigrationSetting = true;  //!< 親交代機能の ON/OFF。

/* NEX 関連設定 */
// NEX ゲームサーバでマッチメイクセッションを構築・検索する際に使用する条件群
const uint32_t MatchmakeGameModeOffset = 0;
// チーム結成用ゲームモード
const uint32_t MatchmakeGameModeBaseForTeam = 45100;
const uint32_t MatchmakeGameModeForTeam = MatchmakeGameModeBaseForTeam + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;
// 対戦用（ゲームセッション用）ゲームモード
const uint32_t MatchmakeGameModeBaseForGame = 45200;
const uint32_t MatchmakeGameModeForGame = MatchmakeGameModeBaseForGame + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;
// チームタイプに使用する属性のインデックス
const uint32_t MatchmakeAttributeIndexTeamType = 0;
const uint32_t MatchmakeAttributeIndexUserPassword = 1;

/* メモリ関連 */
const uint32_t MemoryBlockSizeForPia = 2 * 1024 * 1024;     //!< Pia に与えるメモリのサイズです。
const uint32_t MemoryBlockSizeForNex = 1024 * 1024;         //!< NEX に与えるメモリのサイズです。
const uint32_t MemoryBlockSizeForNexEmergency = 50 * 1024;  //!< NEX に与える緊急時用メモリのサイズです。

/* 描画関連 */
const int ScreenWidth = 400;   //!< 画面の解像度(横)
const int ScreenHeight = 240;  //!< 上画面の解像度(縦)

const int FontSize = 8;  //!< 描画するフォントのサイズ

const int UserNameLength = 11;  //!< UserName 描画の最大文字数（終端文字を含む）

const char* GetPiaVersionString();  //!< Piaライブラリバージョン情報を描画するための文字列取得関数

// インターネット系エラーの Result をセットするための関数
void SetNetworkErrorResult(nn::pia::Result result);

// 電波強度を取得するための処理をまとめた関数
uint32_t GetLinkLevel();

// 状態同期オブジェクトの初期化
void InitObjects();

// ゲームの状態
enum GameState
{
    GameState_None = 0,
    GameState_Team,
    GameState_PreparingGame,
    GameState_Game,
    GameState_PreparingTeam
};
extern GameState g_GameState;

// ジョイントセッションのタイプ
enum JointSessionType
{
    JointSessionType_None = 0,
    JointSessionType_Random,
    JointSessionType_Create,
    JointSessionType_CreatePrivate,
    JointSessionType_Join,
    JointSessionType_JoinPrivate,
    JointSessionType_Destroy,
    JointSessionType_Leave
};
extern JointSessionType g_JointSessionType;

// セッションの状態
bool IsInSession();
bool IsInJointSession();
bool IsSessionMigratingToSession();
bool IsSessionMigratingToJointSession();
bool IsSessionMigrating();

// ジョイン要求承認/拒否判定コールバック
bool SampleJoinApprovalCallback(const nn::pia::transport::Station::IdentificationToken* pToken);

// トークンデータ
struct SampleTokenData
{
#if 0
    uint32_t sessionId;
#endif
    uint8_t teamType;
};

// ユーザーパスワード
extern const SampleStringChar_t* UserPasswordString[];
extern uint8_t g_UserPasswordIndexForJoint;

void ResetErrorOccurredFlag();

#if PIA_TEST_CODE
bool IsHostMigrationEnabled();
void SetHostMigrationEnabled(bool b);

void SetMaxPlayerNum(uint16_t num);
uint16_t GetMaxPlayerNum();

enum TestParamIndex
{
    TestParamIndex_LatencyEmulation = 0,
    TestParamIndex_PacketLossEmulation,
    TestParamIndex_HostMigration,
    TestParamIndex_JointSessionSequence,
    TestParamIndex_LocalStationIndex,
    TestParamIndex_ApplicationDataHead,
    TestParamIndex_ApplicationDataSize,
    TestParamIndex_JoinJointSessionId
};

const int DispatchDelayCount = 100;
const int ProgramParamGetJoinSequence = 101;
#endif

// クローン関連
enum CloneIdOffset
{
    CloneIdOffset_Station = 0x0,
    CloneIdOffset_CommonObject = 0x10000,
    CloneIdOffset_TeamConstitutionManager = 0x20000
};
const int CommonObjectNum = TeamMax * (1 + 1) + (1 + 1);  //!< 各チーム Atom、Seq + ゲーム内共通 Atom、Seq

void UpdateStatusManager();

#endif
