﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/common.h>
#include <nn/pia/inet.h>
#include <nn/pia/transport.h>
#include <nn/pia/session.h>

#include "SampleLib/SampleLib.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Clone_Setup.h"
#include "SampleObjectStatus.h"

#include "EventChecker.h"
#include "TeamConstitutionManager.h"
#include "ObjectManager.h"
#include "StatusManager.h"

#include "SampleLib/Nex_Setup.h"

/*!

    @brief ネットワーク種別を問わない処理 (session + transport) をまとめた *.cpp

*/


SampleTeamType GetCurrentTeamType();
const nns::pia::sampleframework::Color GetColorOfTeamType(SampleTeamType teamType);

//
// グローバル変数
//

// ステーションのステータス管理用
StatusManager g_StatusManager;

// イベントチェック用
EventChecker g_SessionMigrationEventChecker;
EventChecker g_HostMigrationEventChecker;


namespace
{

enum SessionState
{
    SessionState_None = 0,
    SessionState_Session,
    SessionState_JointSession,
    SessionState_MigratingToSession,
    SessionState_MigratingToJointSession
};
SessionState s_SessionState = SessionState_None;

bool s_IsNeedModifyAttribute = false;
bool s_HostChangedEventOccurred = false;
bool s_IsLeaveEventForLocalOccurred = false;
bool s_IsErrorOccurred = false;
bool s_IsLeaveProcessing = false;

const uint32_t SendThreadBufferSize = nns::pia::MaxSendThreadBufferNum;
const uint32_t ReceiveThreadBufferSize = nns::pia::MaxReceiveThreadBufferNum;

void BindObjectToStatus()
{
    // StationIndex が変化するタイミングで、StationStatus と Object の結び付けをし直す必要がある
    for (int i = 0; i < GamePlayerMax; ++i)
    {
        SampleObjectStatus* pStatus = g_StatusManager.GetStationStatus(i);
        pStatus->pObject = g_ObjectManager.GetReceiveObject(pStatus->stationId);
    }
}

void AssignCommonObjectsStatus()
{
    for (int i = 0; i < TeamMax; ++i)
    {
        g_StatusManager.AssignCommonObjectStatus(SampleObjectStatusType_TeamAtomic, i, g_ObjectManager.GetTeamAtomicObject(i));
        g_StatusManager.AssignCommonObjectStatus(SampleObjectStatusType_TeamSequential, i, g_ObjectManager.GetTeamSequentialObject(i));
    }
    g_StatusManager.AssignCommonObjectStatus(SampleObjectStatusType_CommonAtomic, TeamMax, g_ObjectManager.GetCommonAtomicObject());
    g_StatusManager.AssignCommonObjectStatus(SampleObjectStatusType_CommonSequential, TeamMax, g_ObjectManager.GetCommonSequentialObject());
}

const int ColorOfRectangleNum = 8;
const nns::pia::sampleframework::Color RectangleColorArray[ColorOfRectangleNum] =
    {
        nns::pia::sampleframework::ColorSilver,
        nns::pia::sampleframework::ColorRed,
        nns::pia::sampleframework::ColorLime,
        nns::pia::sampleframework::ColorOlive,
        nns::pia::sampleframework::ColorPurple,
        nns::pia::sampleframework::ColorTeal,
        nns::pia::sampleframework::ColorMaroon,
        nns::pia::sampleframework::ColorGreen
    };

// 矩形描画時のフィールド設定
const int32_t TeamFieldWidth = 340 / TeamMax;
const int32_t TeamFieldHeight = 70;
const int32_t TeamFieldMargin = 5;
const int32_t LeftTopX = 20;
const int32_t LeftTopY = 48;

void DrawFrame(int32_t x, int32_t y, int32_t width, int32_t height, const nns::pia::sampleframework::Color& col)
{
    int32_t w = nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontWidth();
    int32_t h = nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontHeight();

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(col);
    // 水平方向は '-' を書く
    int32_t topy = y - h / 2;
    int32_t bottomy = y + height - h;
    for (int posx = x + w / 2; posx < x + width - w / 2; posx += w / 2)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(posx, topy, "-");
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(posx, bottomy, "-");
    }
    // 垂直方向は '|' を書く
    int32_t leftx = x;
    int32_t rightx = x + width - w / 2;
    for (int posy = y; posy < y + height - h; posy += h / 2)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(leftx, posy, "|");
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(rightx, posy, "|");
    }
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
}

void DrawTeamFrame(int teamIndex, const nns::pia::sampleframework::Color& teamColor)
{
    int32_t posLeftTopX = LeftTopX + (TeamFieldWidth + TeamFieldMargin) * teamIndex;
    int32_t posLeftTopY = LeftTopY;

    int32_t width = TeamFieldWidth;
    int32_t height = TeamFieldHeight;

    // 想定した領域の外側に枠線が引かれるように、枠線の太さの分だけ値を調整
    DrawFrame(posLeftTopX - 2, posLeftTopY - 2, width + 4, height + 4, teamColor);
}

RectPosition MoveRectPosition(SampleObjectStatus* pStatus, int32_t diffX, int32_t diffY)
{
    int32_t xBuf;
    int32_t yBuf;

    if (pStatus->type == SampleObjectStatusType_CommonSequential || pStatus->type == SampleObjectStatusType_CommonAtomic)
    {
        const int32_t Width = TeamFieldWidth * TeamMax;
        xBuf = (pStatus->pos.x + diffX + Width) % Width;
        yBuf = pStatus->pos.y;
    }
    else
    {
        // フィールドの端ではみ出さないように、最大値を調整
        const int32_t Width = TeamFieldWidth - nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontWidth();
        const int32_t Height = TeamFieldHeight - nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontHeight();

        xBuf = (pStatus->pos.x + diffX + Width) % Width;
        yBuf = (pStatus->pos.y + diffY + Height) % Height;
    }
    RectPosition ret = { xBuf, yBuf };
    return ret;
}

void StartCloneProtocol()
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(nns::pia::Clone_GetCloneProtocol()));
    PIA_SAMPLE_ASSERT(nns::pia::Clone_GetCloneProtocol()->GetState() == nn::pia::clone::CloneProtocol::State_Inactive);
    nn::pia::Result r = nns::pia::Clone_GetCloneProtocol()->Start();
    PIA_SAMPLE_ASSERT(r.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(r);
}

void DrawStationStatus(const SampleObjectStatus& target)
{
    int teamIndex = g_TeamConstitutionManager.GetTeamIndex(target.stationId);
    // チームが決定していないステーションは、描画しない
    if (target.type == SampleObjectStatusType_Station && teamIndex == -1)
    {
        return;
    }
    if ((target.type == SampleObjectStatusType_TeamAtomic || target.type == SampleObjectStatusType_TeamSequential) && target.teamIndex == -1)
    {
        return;
    }

    // チームによって描画位置を調整
    int32_t offsetX, offsetY;
    if (target.type == SampleObjectStatusType_Station)
    {
        offsetX = LeftTopX + (TeamFieldWidth + TeamFieldMargin) * teamIndex;
        offsetY = LeftTopY;
    }
    else if (target.type == SampleObjectStatusType_TeamAtomic || target.type == SampleObjectStatusType_TeamSequential)
    {
        offsetX = LeftTopX + (TeamFieldWidth + TeamFieldMargin) * target.teamIndex;
        offsetY = LeftTopY;
    }
    else
    {
        offsetX = LeftTopX;
        offsetY = LeftTopY;
    }

    int32_t targetId;
    if (target.type == SampleObjectStatusType_Station)
    {
        // チーム内インデックスを取得して利用する
        targetId = g_TeamConstitutionManager.GetMemberIndex(target.stationId);
        if (targetId == -1)
        {
            return;
        }
    }
    else
    {
        // ステーション以外のオブジェクト
        targetId = static_cast<int32_t>(TeamMemberMax);
    }

    // 位置情報で矩形描画
    int32_t xBuf = offsetX + target.pos.x;
    int32_t yBuf = offsetY + target.pos.y;
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(RectangleColorArray[targetId % ColorOfRectangleNum]);
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
        xBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontWidth() / 2 - 2,
        yBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontHeight() / 2 - 2,
        "*");
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
    if (&target == g_StatusManager.GetOperationTargetStatus())
    {
        const nns::pia::sampleframework::Color cOperationTargetColor(1.0f, 0.3f, 0.0f, 1.0f);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(cOperationTargetColor);
    }

    xBuf += nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontWidth();

    switch (target.type)
    {
    case SampleObjectStatusType_Station:
    {

#if PIA_TEST_CODE
        nn::pia::transport::Station* pStationBuf = nn::pia::transport::StationManager::GetInstance()->GetStation(target.stationId);
        int rtt = 0;
        if (PIA_IS_VALID_POINTER(pStationBuf))
        {
            // StationId に対応する Station インスタンスから RTT を取得
            rtt = pStationBuf->GetRtt();
            if (rtt == nn::pia::transport::Station::InvalidRtt)
            {
                rtt = 0;
            }
        }
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
            xBuf, yBuf, "<-%X,RTT:0x%03X", targetId, rtt);
#else
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
            xBuf, yBuf, "<-%X", targetId);
#endif
    }
    break;
    case SampleObjectStatusType_TeamAtomic:
    {
        if (target.pObject->GetLockStatus() == nn::pia::clone::AtomicSharingClone::LockStatus_Locked)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                xBuf, yBuf, "<-A!");
        }
        else if (target.pObject->GetLockStatus() == nn::pia::clone::AtomicSharingClone::LockStatus_TryLock)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                xBuf, yBuf, "<-A?");
        }
        else
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                xBuf, yBuf, "<-A");
        }
    }
    break;
    case SampleObjectStatusType_CommonAtomic:
    {
        if (target.pObject->GetLockStatus() == nn::pia::clone::AtomicSharingClone::LockStatus_Locked)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                xBuf, yBuf, "<-CA!");
        }
        else if (target.pObject->GetLockStatus() == nn::pia::clone::AtomicSharingClone::LockStatus_TryLock)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                xBuf, yBuf, "<-CA?");
        }
        else
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                xBuf, yBuf, "<-CA");
        }
    }
    break;
    case SampleObjectStatusType_TeamSequential:
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
            xBuf, yBuf, "<-Q");
    }
    break;
    case SampleObjectStatusType_CommonSequential:
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
            xBuf, yBuf, "<-CQ");
    }
    break;
    default:
    {
    }
    break;
    }
    if (&target == g_StatusManager.GetOperationTargetStatus())
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
    }
}

enum SampleWaitingAsyncProcessing
{
    SampleWaitingAsyncProcessing_LeaveSession,
    SampleWaitingAsyncProcessing_OpenSession,
    SampleWaitingAsyncProcessing_CloseSession,
    SampleWaitingAsyncProcessing_OpenJointSession,
    SampleWaitingAsyncProcessing_CloseJointSession,
    SampleWaitingAsyncProcessing_UpdateJointSessionSetting,
    SampleWaitingAsyncProcessing_UpdateSessionSetting,
    SampleWaitingAsyncProcessing_RequestJointSessionInfo,
    SampleWaitingAsyncProcessing_RequestSessionInfo,
    SampleWaitingAsyncProcessing_Max
};
}  // end of anonymous namespace


//セッション内での役割画面表示用文字列
const char* GetSessionRoleString(int index)
{
    switch (index)
    {
    case 1:
        return "Host";
    case 2:
        return "Client";
    }
    return "Unknown";
}

//ジョイントセッション内での役割画面表示用文字列
const char* JOINT_GetSessionRoleString(int index)
{
    switch (index)
    {
    case 1:
        return "JointSessionHost";
    case 2:
        return "JointSessionClient";
    }
    return "Unknown";
}

void ResetErrorOccurredFlag()
{
    s_IsErrorOccurred = false;
}

void UpdateSendData(int32_t diffX, int32_t diffY)
{
    // オブジェクトにデータをセット
    nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
    SampleObjectStatus* pStationStatus = g_StatusManager.GetStationStatus(localStationId);
    PIA_SAMPLE_ASSERT(pStationStatus != NULL);
    SampleObjectStatus* pOperationTargetStatus = g_StatusManager.GetOperationTargetStatus();
    PIA_SAMPLE_ASSERT(pOperationTargetStatus != NULL);

    if (g_ObjectManager.GetSendObject()->elementIncrementNumber.IsReadyToSetValue())
    {
        g_ObjectManager.GetSendObject()->elementIncrementNumber.SetValue(pStationStatus->latestIncrementNumber + 1);
    }

    if (g_ObjectManager.GetSendObject()->elementIsJointSessionStarted.IsReadyToSetValue())
    {
        g_ObjectManager.GetSendObject()->elementIsJointSessionStarted.SetValue(pStationStatus->isStartJointSession);
    }

    pStationStatus->stationNumber = nn::pia::session::Session::GetInstance()->GetStationNum();
    if (g_ObjectManager.GetSendObject()->elementStationNumber.IsReadyToSetValue())
    {
        g_ObjectManager.GetSendObject()->elementStationNumber.SetValue(pStationStatus->stationNumber);
    }

    if (g_StatusManager.GetOperationTargetStatus()->type == SampleObjectStatusType_Station && g_StatusManager.GetOperationTargetStatus()->stationId == localStationId)
    {
        if (g_ObjectManager.GetSendObject()->elementRectPosition.IsReadyToSetValue())
        {
            g_ObjectManager.GetSendObject()->elementRectPosition.SetValue(MoveRectPosition(pStationStatus, diffX, diffY));
        }
    }
    else
    {
        if (g_ObjectManager.GetSendObject()->elementRectPosition.IsReadyToSetValue())
        {
            g_ObjectManager.GetSendObject()->elementRectPosition.SetValue(MoveRectPosition(pStationStatus, 1, 0));
        }
    }

    if (pOperationTargetStatus->type == SampleObjectStatusType_TeamAtomic ||
        pOperationTargetStatus->type == SampleObjectStatusType_TeamSequential ||
        pOperationTargetStatus->type == SampleObjectStatusType_CommonSequential ||
        pOperationTargetStatus->type == SampleObjectStatusType_CommonAtomic)
    {
        SampleObjectStatus* pCommonObjectStatus = g_StatusManager.GetCommonObjectStatusByType(pOperationTargetStatus->type, GetCurrentTeamType());
        PIA_SAMPLE_ASSERT(pCommonObjectStatus != NULL);
        Object* pObj = pCommonObjectStatus->pObject;
        PIA_SAMPLE_ASSERT(pObj != NULL);
        if (pObj->elementIncrementNumber.IsReadyToSetValue())
        {
            pObj->elementIncrementNumber.SetValue(pCommonObjectStatus->latestIncrementNumber + 1);
        }
        if (pObj->elementRectPosition.IsReadyToSetValue())
        {
            pObj->elementRectPosition.SetValue(MoveRectPosition(pCommonObjectStatus, diffX, diffY));
        }
        if (pObj->elementStationNumber.IsReadyToSetValue())
        {
            pObj->elementStationNumber.SetValue(pCommonObjectStatus->stationNumber);
        }
        if (pObj->elementIsJointSessionStarted.IsReadyToSetValue())
        {
            pObj->elementIsJointSessionStarted.SetValue(pCommonObjectStatus->isStartJointSession);
        }
    }
}

void UpdateStatusManager()
{
    nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
    SampleObjectStatus* pStationStatus = g_StatusManager.GetStationStatus(localStationId);
    if (pStationStatus != NULL)
    {
        UpdateSendData(0, 0);
    }
    g_StatusManager.Update();
}

bool CheckSessionStatus()
{
    bool retval = true;
#if PIA_TEST_CODE
#else
    uint16_t stationNum = nn::pia::session::Session::GetInstance()->GetStationNum();
    uint16_t participantNum = nn::pia::session::Session::GetInstance()->GetMatchmakeSessionStationNum();

    // 参加、離脱しようとしているステーションが存在するか
    if (stationNum != participantNum)
    {
        retval = false;
    }
    // 参加しているステーション間でステーション数の同期が取れているか（接続処理が完了しているか）
    for (nn::pia::session::Session::StationIdIterator it = nn::pia::session::Session::GetInstance()->Begin();
         it != nn::pia::session::Session::GetInstance()->End();
         ++it)
    {
        SampleObjectStatus* pStationStatus = g_StatusManager.GetStationStatus(*it);
        if (pStationStatus->stationId != nn::pia::StationIdInvalid)
        {
            if (pStationStatus->stationNumber != stationNum)
            {
                retval = false;
            }
        }
    }
#endif
    return retval;
}

void SampleSessionEventCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam)
{
    PIA_SAMPLE_PRINT("EventType:%d, StationId:%llu\n", eventType, stationId);
    NN_PIA_UNUSED(pApplicationParam);

    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
    {
        PIA_SAMPLE_PRINT(" EventType: EventType_EventJoin\n");
        nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
        if (s_SessionState == SessionState_None || IsInSession() || IsInJointSession())
        {
            // セッションへの参加イベント

            if (stationId == localStationId)
            {
                // 自分の、セッション参加イベント
                PIA_SAMPLE_ASSERT(s_SessionState == SessionState_None);
                if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedSession)
                {
                    s_SessionState = SessionState_Session;
                }
                else if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedJointSession)
                {
                    s_SessionState = SessionState_JointSession;
                }
                else
                {
                    PIA_SAMPLE_ASSERTMSG(false, "Session status is invalid. [%d]", nn::pia::session::Session::GetInstance()->GetStatus());
                }
                s_IsNeedModifyAttribute = false;
                s_IsLeaveEventForLocalOccurred = false;

                // チーム構成をリセット
                g_TeamConstitutionManager.Reset();

                // 共有オブジェクトのステータスを割り当て
                AssignCommonObjectsStatus();

                // 状態同期通信を開始する
                StartCloneProtocol();
            }
        }
        else if (IsSessionMigrating())
        {
            // セッションマイグレーション中
        }

        // 新規参加イベントが来たので StationStatus を新規割り当て
        g_StatusManager.AssignStationStatus(stationId, g_ObjectManager.GetReceiveObject(stationId));

        // チーム構成を更新
        if (!g_TeamConstitutionManager.JoinEvent(stationId))
        {
            // 同じチームに異なるセッションが参加してきた
            // エラーとして扱う
            s_IsErrorOccurred = true;
        }

        g_SessionMigrationEventChecker.SetJoinEventStationId(stationId);
        // 有効なステーションインデックスに変換できることをチェック
        nn::pia::StationIndex stationIndex;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        PIA_SAMPLE_ASSERT(stationIndex != nn::pia::StationIndex_Invalid);
        NN_PIA_UNUSED_IN_RELEASE(result);
    }
    break;
    case nn::pia::session::Session::EventType_EventLeave:
    {
        PIA_SAMPLE_PRINT(" EventType: EventType_EventLeave\n");
        PIA_SAMPLE_PRINT("host:%llu, jointSessionHost:%llu\n",
                       nn::pia::session::Session::GetInstance()->GetHostStationId(),
                       nn::pia::session::Session::GetInstance()->GetJointSessionHostStationId());
        // 離脱イベントが来た StationStatus を片づける
        SampleObjectStatus* pStatus = g_StatusManager.GetStationStatus(stationId);
        PIA_SAMPLE_ASSERT(pStatus != NULL);  // 接続していない相手の離脱イベントが来ていないか？
        pStatus->Clear();

        // チーム構成を更新
        g_TeamConstitutionManager.LeaveEvent(stationId);

        nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
        if (stationId == localStationId)
        {
            // 自分の、セッション離脱イベント
            s_SessionState = SessionState_None;
            g_StatusManager.ClearCommonObjectStatus();
            s_IsLeaveEventForLocalOccurred = true;
            s_IsNeedModifyAttribute = false;
        }

        if ((stationId == nn::pia::session::Session::GetInstance()->GetHostStationId() ||
             stationId == nn::pia::session::Session::GetInstance()->GetJointSessionHostStationId()) &&
            !s_IsLeaveProcessing)
        {
            // ホスト、または、ジョイントセッションホストが離脱した
            if (!g_HostMigrationEventChecker.IsCheckEnabled())
            {
                // ホストマイグレーションのイベントチェック開始
                g_HostMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_HostMigration);
            }
            PIA_SAMPLE_ASSERT(g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration);
            if (stationId == nn::pia::session::Session::GetInstance()->GetHostStationId())
            {
                g_HostMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_SessionHostChanged);
            }
            if (stationId == nn::pia::session::Session::GetInstance()->GetJointSessionHostStationId())
            {
                g_HostMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_JointSessionHostChanged);
            }
        }

        g_SessionMigrationEventChecker.SetLeaveEventStationId(stationId);
        //有効なステーションインデックスに変換できることをチェック
        nn::pia::StationIndex stationIndex;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->CheckTransportStatus();
        if (result.IsSuccess())
        {
            result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
            if (result.IsSuccess())
            {
                PIA_SAMPLE_ASSERT(stationIndex != nn::pia::StationIndex_Invalid);
            }
        }
    }
    break;
    case nn::pia::session::Session::EventType_SessionHostChanged:
    {
        // ホストが変わった
        PIA_SAMPLE_PRINT(" EventType: EventType_SessionHostChanged\n");

        if (!IsSessionMigrating())
        {
            // チーム構成を更新
            if (!g_TeamConstitutionManager.SessionHostChangedEvent())
            {
                s_IsErrorOccurred = true;
            }
        }
        else
        {
            // セッション移行処理終了後にチーム構成を更新
            s_HostChangedEventOccurred = true;
        }

        PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_SessionHostChanged) != 0);

        g_HostMigrationEventChecker.SetEventFlag(EventChecker::EventBit_SessionHostChanged);
        // ホスト情報のチェック
        PIA_SAMPLE_ASSERT(stationId != nn::pia::StationIdInvalid);
        PIA_SAMPLE_ASSERT(stationId == nn::pia::session::Session::GetInstance()->GetHostStationId());
        if (stationId == nn::pia::session::Session::GetInstance()->GetLocalStationId())
        {
            PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->IsHost());
        }
    }
    break;
    case nn::pia::session::Session::EventType_JointSessionHostChanged:
    {
        // ジョイントセッションホストが変わった
        PIA_SAMPLE_PRINT(" EventType: EventType_JointSessionHostChanged\n");

        // チーム構成を更新
        if (!g_TeamConstitutionManager.SessionHostChangedEvent())
        {
            s_IsErrorOccurred = true;
        }

        if (nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
            // マッチメイクセッションの属性をジョイントセッションホストのチームに変更
            s_IsNeedModifyAttribute = true;
        }

        PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_JointSessionHostChanged) != 0);

        g_HostMigrationEventChecker.SetEventFlag(EventChecker::EventBit_JointSessionHostChanged);
        // ホスト情報のチェック
        PIA_SAMPLE_ASSERT(stationId != nn::pia::StationIdInvalid);
        PIA_SAMPLE_ASSERT(stationId == nn::pia::session::Session::GetInstance()->GetJointSessionHostStationId());
        if (stationId == nn::pia::session::Session::GetInstance()->GetLocalStationId())
        {
            PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->IsJointSessionHost());
        }
    }
    break;
    // 以下はセッションマイグレーション関連イベント
    case nn::pia::session::Session::EventType_StartSessionJointRandom:
    {
        // ジョイントセッション処理開始イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_StartSessionJointRandom\n");
        PIA_SAMPLE_ASSERT(IsInSession());
        s_SessionState = SessionState_MigratingToJointSession;
        g_JointSessionType = JointSessionType_Random;
        s_HostChangedEventOccurred = false;

        // 想定されるイベントをセット
        if (nn::pia::session::Session::GetInstance()->IsHost() == false)
        {
            g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartJoinRandomJointSession);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndJoinRandomJointSession);
        }
        // 発生したイベントをセット
        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_StartJoinRandomJointSession);
    }
    break;
    case nn::pia::session::Session::EventType_EndSessionJointRandom:
    {
        // ジョイントセッション処理完了イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_EndSessionJointRandom\n");
        PIA_SAMPLE_ASSERT(IsSessionMigratingToJointSession());
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartJoinRandomJointSession) != 0);
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_EndJoinRandomJointSession) != 0);
        s_SessionState = SessionState_JointSession;

        // StationIndex が変化したので、StationStatus と Object の結び付けをし直す
        BindObjectToStatus();

        if (s_HostChangedEventOccurred)
        {
            // チーム構成を更新
            if (!g_TeamConstitutionManager.SessionHostChangedEvent())
            {
                s_IsErrorOccurred = true;
            }
        }

        // 状態同期通信を開始する
        StartCloneProtocol();

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_EndJoinRandomJointSession);

        // イベントチェック
        g_SessionMigrationEventChecker.Trace();
        PIA_SAMPLE_ASSERT(g_SessionMigrationEventChecker.GetEventFlag() == g_SessionMigrationEventChecker.GetExpectedEventFlag());
        g_SessionMigrationEventChecker.CheckStationIdList();
        g_SessionMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_SessionMigration);
    }
    break;
    case nn::pia::session::Session::EventType_StartSessionCreateJoint:
    {
        // ジョイントセッション処理開始イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_StartSessionCreateJoint\n");
        PIA_SAMPLE_ASSERT(IsInSession());
        s_SessionState = SessionState_MigratingToJointSession;
        g_JointSessionType = JointSessionType_Create;
        s_HostChangedEventOccurred = false;

        // 想定されるイベントをセット
        if (nn::pia::session::Session::GetInstance()->IsHost() == false)
        {
            g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartCreateJointSession);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndCreateJointSession);
        }
        // 発生したイベントをセット
        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_StartCreateJointSession);
    }
    break;
    case nn::pia::session::Session::EventType_EndSessionCreateJoint:
    {
        // ジョイントセッション処理完了イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_EndSessionCreateJoint\n");
        PIA_SAMPLE_ASSERT(IsSessionMigratingToJointSession());
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartCreateJointSession) != 0);
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_EndCreateJointSession) != 0);
        s_SessionState = SessionState_JointSession;

        // StationIndex が変化したので、StationStatus と Object の結び付けをし直す
        BindObjectToStatus();

        if (s_HostChangedEventOccurred)
        {
            // チーム構成を更新
            if (!g_TeamConstitutionManager.SessionHostChangedEvent())
            {
                s_IsErrorOccurred = true;
            }
        }

        // 状態同期通信を開始する
        StartCloneProtocol();

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_EndCreateJointSession);

        // イベントチェック
        g_SessionMigrationEventChecker.Trace();
        PIA_SAMPLE_ASSERT(g_SessionMigrationEventChecker.GetEventFlag() == g_SessionMigrationEventChecker.GetExpectedEventFlag());
        g_SessionMigrationEventChecker.CheckStationIdList();
        g_SessionMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_SessionMigration);
    }
    break;
    case nn::pia::session::Session::EventType_StartSessionJoinJoint:
    {
        // ジョイントセッション処理開始イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_StartSessionJoinJoint\n");
        PIA_SAMPLE_ASSERT(IsInSession());
        s_SessionState = SessionState_MigratingToJointSession;
        g_JointSessionType = JointSessionType_Join;
        s_HostChangedEventOccurred = false;

        // 想定されるイベントをセット
        if (nn::pia::session::Session::GetInstance()->IsHost() == false)
        {
            g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartJoinJointSession);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndJoinJointSession);
        }
        // 発生したイベントをセット
        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_StartJoinJointSession);
    }
    break;
    case nn::pia::session::Session::EventType_EndSessionJoinJoint:
    {
        // ジョイントセッション処理完了イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_EndSessionJoinJoint\n");
        PIA_SAMPLE_ASSERT(IsSessionMigratingToJointSession());
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartJoinJointSession) != 0);
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_EndJoinJointSession) != 0);
        s_SessionState = SessionState_JointSession;

        // StationIndex が変化したので、StationStatus と Object の結び付けをし直す
        BindObjectToStatus();

        if (s_HostChangedEventOccurred)
        {
            // チーム構成を更新
            if (!g_TeamConstitutionManager.SessionHostChangedEvent())
            {
                s_IsErrorOccurred = true;
            }
        }

        // 状態同期通信を開始する
        StartCloneProtocol();

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_EndJoinJointSession);

        // イベントチェック
        g_SessionMigrationEventChecker.Trace();
        PIA_SAMPLE_ASSERT(g_SessionMigrationEventChecker.GetEventFlag() == g_SessionMigrationEventChecker.GetExpectedEventFlag());
        g_SessionMigrationEventChecker.CheckStationIdList();
        g_SessionMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_SessionMigration);
    }
    break;
    case nn::pia::session::Session::EventType_StartSessionDestroyJoint:
    {
        // ジョイントセッションの解散処理開始イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_StartSessionDestroyJoint\n");
        PIA_SAMPLE_ASSERT(IsInJointSession());
        s_SessionState = SessionState_MigratingToSession;
        g_JointSessionType = JointSessionType_Destroy;
        s_HostChangedEventOccurred = false;
        if (!nn::pia::session::Session::GetInstance()->IsJointSessionHost() && nn::pia::session::Session::GetInstance()->IsHost())
        {
            // ジョイントセッションホストではないセッションホストは、ジョイントセッション解散後に、
            // 離脱したもう一方のセッションのステーションをチーム構成から削除するため、フラグを立てておく
            s_HostChangedEventOccurred = true;
        }

        // 想定されるイベントをセット
        if (!nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
            g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartDestroyJointSession);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndDestroyJointSession);

            // この時点で他のチームのステーションは離脱済みのはず（まだチーム情報を受信していない場合もある）
            nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
            nn::pia::session::Session::StationIdIterator itEnd = pSession->End();
            int localTeamIndex = g_TeamConstitutionManager.GetTeamIndex(pSession->GetLocalStationId());
            NN_PIA_UNUSED_IN_RELEASE(localTeamIndex);
            for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != itEnd; ++it)
            {
                int teamIndex = g_TeamConstitutionManager.GetTeamIndex(*it);
                NN_PIA_UNUSED_IN_RELEASE(teamIndex);
                PIA_SAMPLE_PRINT(" StationId:%llu is in Team%d. Myteam is %d.\n", *it, teamIndex, localTeamIndex);
                PIA_SAMPLE_ASSERT(localTeamIndex == -1 || teamIndex == -1 || teamIndex == localTeamIndex);
            }

            // ジョイントセッションの解散による離脱により起こったかもしれないホストマイグレーションのチェックのうち、EVENT_JOINT_SESSION_HOST_CHANGED のチェックは終了する
            if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
            {
                g_HostMigrationEventChecker.UnsetExpectedEventFlag(EventChecker::EventBit_JointSessionHostChanged);
                if (g_HostMigrationEventChecker.GetExpectedEventFlag() == 0)
                {
                    g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
                }
            }
        }

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_StartDestroyJointSession);
    }
    break;
    case nn::pia::session::Session::EventType_EndSessionDestroyJoint:
    {
        // ジョイントセッションの解散処理完了イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_EndSessionDestroyJoint\n");
        PIA_SAMPLE_ASSERT(IsSessionMigratingToSession());
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartDestroyJointSession) != 0);
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_EndDestroyJointSession) != 0);
        s_SessionState = SessionState_Session;
        s_IsNeedModifyAttribute = false;

        // StationIndex が変化したので、StationStatus と Object の結び付けをし直す
        BindObjectToStatus();

        if (s_HostChangedEventOccurred)
        {
            // チーム構成を更新
            if (!g_TeamConstitutionManager.SessionHostChangedEvent())
            {
                s_IsErrorOccurred = true;
            }
        }

        // 状態同期通信を開始する
        StartCloneProtocol();

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_EndDestroyJointSession);

        // イベントチェック
        g_SessionMigrationEventChecker.Trace();
        PIA_SAMPLE_ASSERT(g_SessionMigrationEventChecker.GetEventFlag() == g_SessionMigrationEventChecker.GetExpectedEventFlag());
        g_SessionMigrationEventChecker.CheckStationIdList();
        g_SessionMigrationEventChecker.CheckExpectedLeaveStationIdList();
        g_SessionMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_SessionMigration);
    }
    break;
    case nn::pia::session::Session::EventType_StartSessionLeaveJoint:
    {
        // ジョイントセッションの解散処理開始イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_StartSessionLeaveJoint\n");
        PIA_SAMPLE_ASSERT(IsInJointSession());
        s_SessionState = SessionState_MigratingToSession;
        g_JointSessionType = JointSessionType_Leave;
        s_HostChangedEventOccurred = false;
        if (!nn::pia::session::Session::GetInstance()->IsJointSessionHost() && nn::pia::session::Session::GetInstance()->IsHost())
        {
            // ジョイントセッションホストではないセッションホストは、ジョイントセッション解散後に、
            // 離脱したもう一方のセッションのステーションをチーム構成から削除するため、フラグを立てておく
            s_HostChangedEventOccurred = true;
        }

        // 想定されるイベントをセット
        if (!nn::pia::session::Session::GetInstance()->IsHost())
        {
            g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartLeaveJointSession);
            g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndLeaveJointSession);

            // この時点で他のチームのステーションは離脱済みのはず（まだチーム情報を受信していない場合もある）
            nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
            nn::pia::session::Session::StationIdIterator itEnd = pSession->End();
            int localTeamIndex = g_TeamConstitutionManager.GetTeamIndex(pSession->GetLocalStationId());
            NN_PIA_UNUSED_IN_RELEASE(localTeamIndex);
            for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != itEnd; ++it)
            {
                int teamIndex = g_TeamConstitutionManager.GetTeamIndex(*it);
                NN_PIA_UNUSED_IN_RELEASE(teamIndex);
                PIA_SAMPLE_PRINT(" StationId:%llu is in Team%d. Myteam is %d.\n", *it, teamIndex, localTeamIndex);
                PIA_SAMPLE_ASSERT(localTeamIndex == -1 || teamIndex == -1 || teamIndex == localTeamIndex);
            }
        }

        // ジョイントセッションからの離脱により起こったかもしれないホストマイグレーションのチェックのうち、EVENT_JOINT_SESSION_HOST_CHANGED のチェックは終了する
        if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
        {
            g_HostMigrationEventChecker.UnsetExpectedEventFlag(EventChecker::EventBit_JointSessionHostChanged);
            if (g_HostMigrationEventChecker.GetExpectedEventFlag() == 0)
            {
                g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
            }
        }

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_StartLeaveJointSession);
    }
    break;
    case nn::pia::session::Session::EventType_EndSessionLeaveJoint:
    {
        // ジョイントセッションの解散処理完了イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_EndSessionLeaveJoint\n");
        PIA_SAMPLE_ASSERT(IsSessionMigratingToSession());
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartLeaveJointSession) != 0);
        PIA_SAMPLE_ASSERT((g_SessionMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_EndLeaveJointSession) != 0);
        s_SessionState = SessionState_Session;
        s_IsNeedModifyAttribute = false;

        // StationIndex が変化したので、StationStatus と Object の結び付けをし直す
        BindObjectToStatus();

        if (s_HostChangedEventOccurred)
        {
            // チーム構成を更新
            if (!g_TeamConstitutionManager.SessionHostChangedEvent())
            {
                s_IsErrorOccurred = true;
            }
        }

        // 状態同期通信を開始する
        StartCloneProtocol();

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_EndLeaveJointSession);

        // イベントチェック
        g_SessionMigrationEventChecker.Trace();
        PIA_SAMPLE_ASSERT(g_SessionMigrationEventChecker.GetEventFlag() == g_SessionMigrationEventChecker.GetExpectedEventFlag());
        g_SessionMigrationEventChecker.CheckStationIdList();
        g_SessionMigrationEventChecker.CheckExpectedLeaveStationIdList();
        g_SessionMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_SessionMigration);
    }
    break;
    case nn::pia::session::Session::EventType_SessionMigrationFailed:
    {
        // セッション移行処理失敗イベント
        PIA_SAMPLE_PRINT(" EventType: EventType_SessionMigrationFailed\n");
        PIA_SAMPLE_ASSERT(s_SessionState == SessionState_None && s_IsLeaveEventForLocalOccurred);
        NN_PIA_UNUSED_IN_RELEASE(s_IsLeaveEventForLocalOccurred);

        g_SessionMigrationEventChecker.SetEventFlag(EventChecker::EventBit_SessionMigrationFailed);
    }
    break;
    default:
    {
    }
    break;
    }
}

bool SampleJoinApprovalCallback(const nn::pia::transport::Station::IdentificationToken* pToken)
{
    NN_PIA_UNUSED(pToken);
    return true;

#if 0
    // ジョイン要求承認/拒否判定コールバック（セッションホストでのみ呼ばれる）
    // このサンプルでは、ジョイントセッション参加中に同じチームの異なるセッションが
    // 2 つ以上参加しないようにするために使用しています。

    if (pToken == NULL)
    {
        PIA_SAMPLE_ASSERT(false);
        return false;
    }

    SampleTokenData tokenData;
    memcpy(&tokenData, pToken->data, sizeof(tokenData));
    
    uint32_t curTeamSessionId = g_TeamConstitutionManager.GetTeamSessionId(tokenData.teamType);
    if (curTeamSessionId == 0 || curTeamSessionId == tokenData.sessionId)
    {
        return true;
    }
    else
    {
        return false;
    }
#endif
}

bool IsInSession()
{
    return s_SessionState == SessionState_Session;
}

bool IsInJointSession()
{
    return s_SessionState == SessionState_JointSession;
}

bool IsSessionMigratingToSession()
{
    return s_SessionState == SessionState_MigratingToSession;
}

bool IsSessionMigratingToJointSession()
{
    return s_SessionState == SessionState_MigratingToJointSession;
}

bool IsSessionMigrating()
{
    return IsSessionMigratingToSession() || IsSessionMigratingToJointSession();
}

void InitializePiaTransport()
{
    nn::pia::Result result;

    // *Factory インスタンスは初期化以外に必要としないためオート変数で確保しています。
    nn::pia::inet::NexNetworkFactory nexFactory;

    // ここでは各パラメータを transport のデモから拝借しています。
    // 各パラメータの詳細については プログラミングマニュアル を参照して下さい。
    nn::pia::transport::Transport::Setting transportSetting;

    transportSetting.pFactory = &nexFactory;
    transportSetting.maxStationNum = GamePlayerMax;
    transportSetting.maxSendThreadBufferNum = SendThreadBufferSize;
    transportSetting.maxReceiveThreadBufferNum = ReceiveThreadBufferSize;

#if PIA_TEST_CODE
    nn::pia::transport::Transport::DebugSetting debugSetting;
    debugSetting.sendThreadLatencyEmulationPacketNum = 128;
    debugSetting.receiveThreadLatencyEmulationPacketNum = 128;
    debugSetting.isPacketLossEmulationEnabled = true;
    result = nn::pia::transport::Transport::SetDebugSetting(debugSetting);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
#endif

    result = nns::pia::Transport_InitializePiaTransport(transportSetting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

#if PIA_TEST_CODE
    nn::pia::transport::ThreadStreamManager* pThreadStreamManager = nn::pia::transport::ThreadStreamManager::GetInstance();
    PIA_SAMPLE_ASSERT(pThreadStreamManager);  //transport の初期化が終わっていればインスタンスが生成されているはず

    pThreadStreamManager->SetSendThreadLatencyEmulation(100, 100);
    pThreadStreamManager->SetReceiveThreadLatencyEmulation(0, 0);
    PIA_SAMPLE_PRINT("Enable latency emulation.\n");

    pThreadStreamManager->SetSendThreadPacketLossRatio(10);
    pThreadStreamManager->SetReceiveThreadPacketLossRatio(10);
    PIA_SAMPLE_PRINT("Enable pakcet loss emulation.\n");
#endif
}

void FinalizePiaTransport()
{
    nns::pia::Transport_FinalizePiaTransport(0, 0, NULL);
}

void InitializePiaSession()
{
    nn::pia::Result result;

    // *Factory インスタンスは初期化以外に必要としないためオート変数で確保しています。
    nn::pia::inet::NexNetworkFactory nexFactory;

    nn::pia::session::Session::Setting sessionSetting;
    sessionSetting.pNetworkFactory = &nexFactory;
    sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
    sessionSetting.browsedSessionInfoListNum = 4;
#if PIA_TEST_CODE
    {
        nn::pia::session::Session::DebugSetting debugSetting(false, false, false);
        result = nn::pia::session::Session::SetDebugSetting(debugSetting);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }
#endif
    result = nns::pia::Session_InitializePiaSession(sessionSetting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // イベントコールバック登録前にステータスを初期化しておく
    g_StatusManager.ClearStationStatus();
    g_StatusManager.ClearCommonObjectStatus();

    // Session 用イベントコールバックを登録
    nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionEventCallback);
}

void FinalizePiaSession()
{
    if (nn::pia::session::Session::GetInstance() != NULL)
    {
        // Session 用イベントコールバックの登録解除
        nn::pia::session::Session::GetInstance()->UnregisterSessionEventCallback();
    }
    nns::pia::Session_FinalizePiaSession();
}

void InitializePiaClone()
{
    nns::pia::Clone_InitializePiaClone();
}

void FinalizePiaClone()
{
    nns::pia::Clone_FinalizePiaClone();
}

/*!
    @brief PiaTransport を使った通信を行うシーン。
    
    ホストがネットワークを終了するとホスト自身はネットワーク選択シーンへ戻りますが、クライアントはエラーシーンへ進みます。
    
    @return 次に実行するべきシーン No を返します。
 */

SampleSceneNo SceneCommunicationTeam()
{
    nn::pia::Result result = nn::pia::ResultSuccess();

    uint32_t frameNo = 0;
    bool isExit = false;
    bool goToGame = false;

#if PIA_TEST_CODE
    bool isDelayDispatch = false;
    bool isStopDispatch = false;
    int delayCount = 0;
#endif

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_CommunicateTeam,
        true,
        nn::pia::session::Session::GetInstance()->IsHost(),
        nn::pia::session::Session::GetInstance()->GetStationNum(),
        0);

    nn::pia::transport::StationConnectionInfo localInfo;
    nn::pia::transport::StationConnectionInfo hostInfo;

    if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedSession)
    {
        result = nn::pia::session::Session::GetInstance()->GetLocalStationConnectionInfo(&localInfo);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        result = nn::pia::session::Session::GetInstance()->GetHostStationConnectionInfo(&hostInfo);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
    }

    // 自身のチームタイプを基にチームカラーを取得
    nns::pia::sampleframework::Color myTeamColor = GetColorOfTeamType(GetCurrentTeamType());

    nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();

    SampleWaitingAsyncProcessing waitingProcess = SampleWaitingAsyncProcessing_Max;
    bool asyncProcessingFlag = false;
    nn::pia::Result::HandlingType handlingType = nn::pia::Result::HandlingType_Void;
    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        if (!asyncProcessingFlag && s_IsErrorOccurred)
        {
            SetCommunicationErrorType(SampleErrorType_Application);
            handlingType = nn::pia::Result::HandlingType_CleanupWithLeave;
            isExit = true;
        }

        if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
        {
            // ホストマイグレーションのイベントチェック中

            // ジョイントセッションのホストマイグレーションは待っていないはず
            PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_JointSessionHostChanged) == 0);
            nn::pia::StationId hostStationId = nn::pia::session::Session::GetInstance()->GetHostStationId();
            if (nn::pia::session::Session::GetInstance()->IsValidStationId(hostStationId))
            {
                // ホストのステーション ID が有効
                if (hostStationId == nn::pia::session::Session::GetInstance()->GetLocalStationId())
                {
                    // 自分は新ホスト
                    if (nn::pia::session::Session::GetInstance()->IsHost())
                    {
                        // EVENT_SESSION_HOST_CHANGED イベントが発生済みのはず
                        PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_SessionHostChanged) != 0);
                        g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
                    }
                }
                else
                {
                    // 自分は新クライアント
                    // EVENT_SESSION_HOST_CHANGED イベントが発生済みのはず
                    PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_SessionHostChanged) != 0);
                    g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
                }
            }
        }

        if (!asyncProcessingFlag && !s_IsErrorOccurred)
        {
            if (!asyncProcessingFlag)
            {
                // X: JointSession (Host)
                if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonX))
                {
                    if (nn::pia::session::Session::GetInstance()->IsHost() && CheckSessionStatus())
                    {
                        goToGame = true;
                        isExit = true;
#if PIA_TEST_CODE
                        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                        {
                            g_JointSessionType = JointSessionType_Join;
                        }
                        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
                        {
                            g_JointSessionType = JointSessionType_JoinPrivate;
                            g_UserPasswordIndexForJoint = SampleTeamType_Max;
                            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonStart))
                            {
                                g_UserPasswordIndexForJoint = static_cast<uint8_t>(GetCurrentTeamType());
                            }
                        }
                        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
                        {
                            g_JointSessionType = JointSessionType_Create;
                        }
                        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
                        {
                            g_JointSessionType = JointSessionType_CreatePrivate;
                            g_UserPasswordIndexForJoint = SampleTeamType_Max;
                        }
                        else
#endif
                        {
                            g_JointSessionType = JointSessionType_Random;
                        }
                        // Joint 開始通知
                        nn::pia::StationId mylocalStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
                        SampleObjectStatus* pStationStatus = g_StatusManager.GetStationStatus(mylocalStationId);
                        PIA_SAMPLE_ASSERT(pStationStatus != NULL);

                        pStationStatus->isStartJointSession = true;
                        pStationStatus->isUpdataStartJointSession = true;
                        UpdateSendData(0, 0);
                    }
                }
                // B: LeaveSession
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonB))
                {
                    PIA_SAMPLE_PRINT("Call LeaveSessionAsync.\n");
                    result = nn::pia::session::Session::GetInstance()->LeaveSessionAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("Call LeaveSession is failure.\n");
                        PIA_SAMPLE_ASSERT(false);
                        s_IsErrorOccurred = true;
                        isExit = true;
                    }
                    else
                    {
                        asyncProcessingFlag = true;
                        waitingProcess = SampleWaitingAsyncProcessing_LeaveSession;
                        s_IsLeaveProcessing = true;

                        // ホストマイグレーション中にセッションから離脱を開始した場合は、EVENT_SESSION_HOST_CHANGED イベントが通知されないので、ホストマイグレーションのチェックは終了する
                        if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
                        {
                            g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
                        }
                    }
                }
                // Cleanup ログアウトなどの必要な Result に対してはまだ未対処
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonL))
                {
                    if (nn::pia::session::Session::GetInstance()->IsHost())
                    {
                        if (nn::pia::session::Session::GetInstance()->GetSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Open)
                        {
                            result = nn::pia::session::Session::GetInstance()->CloseSessionAsync();
                            if (result.IsSuccess())
                            {
                                PIA_SAMPLE_PRINT("Call CloseSessionAsync\n");
                                asyncProcessingFlag = true;
                                waitingProcess = SampleWaitingAsyncProcessing_CloseSession;
                            }
                            else
                            {
                                PIA_SAMPLE_PRINT("Call CloseSessionAsync is failure. Result:0x%08X\n", result.GetValue());
                                PIA_SAMPLE_ASSERT(false);
                                s_IsErrorOccurred = true;
                                isExit = true;
                            }
                        }
                        else if (nn::pia::session::Session::GetInstance()->GetSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Close)
                        {
                            result = nn::pia::session::Session::GetInstance()->OpenSessionAsync();
                            if (result.IsSuccess())
                            {
                                PIA_SAMPLE_PRINT("Call OpenSessionAsync\n");
                                asyncProcessingFlag = true;
                                waitingProcess = SampleWaitingAsyncProcessing_OpenSession;
                            }
                            else
                            {
                                PIA_SAMPLE_PRINT("Call OpenSessionAsync is failure. Result:0x%08X\n", result.GetValue());
                                PIA_SAMPLE_ASSERT(false);
                                s_IsErrorOccurred = true;
                                isExit = true;
                            }
                        }
                        else
                        {
                            // セッションホストはセッションの開閉状態を取得できるので、ここに来ることはないはず。
                            PIA_SAMPLE_ASSERT(false);
                        }
                    }
                }
            }
        }

#if PIA_TEST_CODE
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
            PIA_SAMPLE_PRINT("[JointSequence] %d\n", nn::pia::session::Session::GetInstance()->GetJointSequence());
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 100%, Receive 10%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 10%, Receive 100%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 10%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                pMgr->SetSendThreadPacketLossRatio(0);
                pMgr->SetReceiveThreadPacketLossRatio(0);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 0%).\n");
            }
            else
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 100%).\n");
            }
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
            {
                nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->InitiateCall();
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
            {
                if (nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->IsInProgress())
                {
                    nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->SignalCancel();
                }
                nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->Reset();
            }
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight) && 
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            isDelayDispatch = !isDelayDispatch;
            delayCount = 0;
            isStopDispatch = false;
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp) && 
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            PIA_SAMPLE_PRINT("Dispatch stoped.");
            isStopDispatch = !isStopDispatch;
            isDelayDispatch = false;
            delayCount = 0;
        }
#endif

        if (asyncProcessingFlag)
        {
            // 非同期処理の進捗チェック
            switch (waitingProcess)
            {
            case SampleWaitingAsyncProcessing_LeaveSession:
            {
                if (nn::pia::session::Session::GetInstance()->IsLeaveSessionCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetLeaveSessionResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("LeaveSession is success.\n");
                        isExit = true;
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("LeaveSession is failure. Result:0x%08X\n", result.GetValue());
                    }
                    asyncProcessingFlag = false;
                    s_IsLeaveProcessing = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_OpenSession:
            {
                if (nn::pia::session::Session::GetInstance()->IsOpenSessionCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetOpenSessionResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("OpenSession is success.\n");
                    }
                    else if (result == nn::pia::ResultSessionConnectionIsLost())
                    {
                        PIA_SAMPLE_PRINT("CloseSession is failed. ResultSessionConnectionIsLost\n");
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("OpenSession is failed. Result:0x%08X\n", result.GetValue());
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() || 
                                          result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                                          result == nn::pia::ResultWifiOff() ||
                                          result == nn::pia::ResultSleep());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_CloseSession:
            {
                if (nn::pia::session::Session::GetInstance()->IsCloseSessionCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetCloseSessionResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("CloseSession is success.\n");
                    }
                    else if (result == nn::pia::ResultSessionConnectionIsLost())
                    {
                        PIA_SAMPLE_PRINT("CloseSession is failed. ResultSessionConnectionIsLost\n");
                    }
                    else if (result == nn::pia::ResultNegligibleFault())
                    {
                        PIA_SAMPLE_PRINT("CloseSession is failed. ResultNegligibleFault\n");
                        // クローズに失敗したが、無視可能なエラーのため、サンプルの都合上 Result を成功に上書きしておく
                        result = nn::pia::ResultSuccess();
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("CloseSession is failed. Result:0x%08X\n", result.GetValue());
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                          result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                                          result == nn::pia::ResultWifiOff() ||
                                          result == nn::pia::ResultSleep());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            default:
            {
            }
            break;
            }
            if (result.IsFailure())
            {
                s_IsErrorOccurred = true;
                isExit = true;
            }
        }

        if (!s_IsErrorOccurred)
        {
            // R: ChangeOperationTarget
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                g_StatusManager.ChangeOperationTarget(GetCurrentTeamType());
            }
            // A: AtomicLockOperation
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                g_StatusManager.AtomicLockOperation();
            }

            int32_t diffX = 0, diffY = 0;
            // 矩形の移動入力チェック
            {
                float analogX, analogY;
                nns::pia::sampleframework::SampleFramework::GetInstance()->GetAnalog(&analogX, &analogY);
                if (analogX != 0.0f || analogY != 0.0f)
                {
                    const float cAnalogMoveBase = 3.0f;
                    diffX = static_cast<int32_t>(cAnalogMoveBase * analogX);
                    diffY = static_cast<int32_t>(cAnalogMoveBase * analogY * (-1.0f));  // アナログスティックの上下と描画の上下では正負が逆
                }
                else
                {
                    const int32_t cMoveBase = 2;
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
                    {
                        diffX -= cMoveBase;
                    }
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
                    {
                        diffX += cMoveBase;
                    }
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                    {
                        diffY -= cMoveBase;
                    }
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
                    {
                        diffY += cMoveBase;
                    }

                    // 移動入力が無い場合でも、通信をわかりやすくするために常に移動させる
                    if (diffX == 0 && diffY == 0)
                    {
                        diffX = 1;
                    }
                }
            }

            // Dispatch 処理前に送信用処理
            if (IsInSession())  // セッション移行処理中は送信しない
            {
                // 送信間隔チェック
                if ((frameNo % (IntervalSendingStationStatus + 1)) == 0)
                {
                    // 送信するデータを更新
                    UpdateSendData(diffX, diffY);
                }
            }

            // CloneProtocol のクロックを更新
            nns::pia::Clone_GetCloneProtocol()->UpdateClock();

#if PIA_TEST_CODE
            // テストのためディスパッチの頻度を下げて、特定のタイミングを狙い易くします。
            if (isDelayDispatch)
            {
                ++delayCount;
                delayCount = delayCount % DispatchDelayCount;
            }

            if ((!isDelayDispatch || (isDelayDispatch && delayCount == DispatchDelayCount - 1)) && !isStopDispatch)
            {
#endif
                // Dispatch 処理
                nn::pia::common::Scheduler::GetInstance()->Dispatch();
#if PIA_TEST_CODE
            }
#endif
            nns::pia::Nex_SchedulerDispatch();

            // チーム構成が変更されていないか調べる
            g_TeamConstitutionManager.Update();

            // Dispatch 処理後に受信処理
            // 受信データをもとに状態を更新
            g_StatusManager.Update();

            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
                SampleSceneNo_CommunicateTeam,
                true,
                nn::pia::session::Session::GetInstance()->IsHost(),
                nn::pia::session::Session::GetInstance()->GetStationNum(),
                0);

            // 非同期処理待ち中は Session の状態チェックを行わない
            if (!asyncProcessingFlag)
            {
                // セッション移行処理中フラグのチェック
                if (IsSessionMigrating())
                {
                    // ゲームセッションへ合流する際に発生
                    PIA_SAMPLE_PRINT("ChangingSessionFlag became true. GoToGame.\n");
                    goToGame = true;
                    isExit = true;
                }
                // 被切断チェックは CheckConnection で行う

                // Session の状態チェック
                result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
                if (result.IsFailure())
                {
                    if (result == nn::pia::ResultNetworkConnectionIsLost())
                    {
                        PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK\n");
                    }
                    else if (result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin())
                    {
                        PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK by duplicate login\n");
                    }
                    else if (result == nn::pia::ResultWifiOff())
                    {
                        PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK by WifiOff\n");
                    }
                    else if (result == nn::pia::ResultSleep())
                    {
                        PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK by Sleep\n");
                    }
                    else if (result == nn::pia::ResultSessionConnectionIsLost())
                    {
                        PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected SESSION\n");
                    }
                    else
                    {
                        // ありえない値
                        PIA_SAMPLE_ASSERT(false);
                        PIA_SAMPLE_PRINT("Session::CheckConnection: Unexpected result:0x%08x\n", result.GetValue());
                    }
                    s_IsErrorOccurred = true;
                    isExit = true;
                }
            }
        }

        //画面描画処理
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

            //文字列
            {
                //自分自身の役割チェック
                int roleIndex = 0;  //Unknown;
                localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
                if (localStationId != nn::pia::StationIdInvalid)
                {
                    if (nn::pia::session::Session::GetInstance()->IsHost())
                    {
                        roleIndex = 1;  // 1:Host
                    }
                    else
                    {
                        roleIndex = 2;  // 2:Client
                    }
                }

                int line = 0;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Communication (Team)");
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationNum:%2d  Role:%-6s", nn::pia::session::Session::GetInstance()->GetStationNum(), GetSessionRoleString(roleIndex));
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationId:%llu TeamMemberIndex:%2d", localStationId, g_TeamConstitutionManager.GetMemberIndex(localStationId));
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  SessionId:%d", nn::pia::session::Session::GetInstance()->GetSessionId());
                line += FontSize;

                line = ScreenHeight - (FontSize * 9);
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  X: JointSession (Host)");
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  B: LeaveSession");
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  R: ChangeOperationTarget");
                line += FontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: TryLock (Atomic)");
                line += FontSize;
                if (nn::pia::session::Session::GetInstance()->IsHost())
                {
                    if (nn::pia::session::Session::GetInstance()->GetSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Close)
                    {
                        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  L: OpenSessionAsync");
                    }
                    else if (nn::pia::session::Session::GetInstance()->GetSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Open)
                    {
                        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  L: CloseSessionAsync");
                    }
                }
                line += FontSize;

                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
                line += FontSize;
            }

            // チーム区別用枠線の描画
            DrawTeamFrame(GetCurrentTeamType(), myTeamColor);

            // StationStatus の描画
            {
                for (int i = 0; i < GamePlayerMax; ++i)
                {
                    // 有効な StationId が記録された StationStatus は全て描画
                    if (g_StatusManager.GetStationStatus(i)->stationId != nn::pia::StationIdInvalid)
                    {
                        DrawStationStatus(*g_StatusManager.GetStationStatus(i));
                    }
                }
            }

            // 共有オブジェクトの描画
            for (int i = 0; i < CommonObjectNum; ++i)
            {
                // 自分のチームオブジェクトのみ描画
                if (g_StatusManager.GetCommonObjectStatus(i)->type == SampleObjectStatusType_TeamAtomic ||
                    g_StatusManager.GetCommonObjectStatus(i)->type == SampleObjectStatusType_TeamSequential)
                {
                    if (g_StatusManager.GetCommonObjectStatus(i)->teamIndex == GetCurrentTeamType())
                    {
                        DrawStationStatus(*g_StatusManager.GetCommonObjectStatus(i));
                    }
                }
            }
        }
        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();
        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        frameNo++;
    }

    if (s_IsErrorOccurred)
    {
        PIA_SAMPLE_ASSERT(asyncProcessingFlag == false);

        if (handlingType == nn::pia::Result::HandlingType_Void)
        {
            handlingType = result.GetHandlingType();
            SetNetworkErrorResult(result);

            if (result == nn::pia::ResultSessionConnectionIsLost())
            {
                SetCommunicationErrorType(SampleErrorType_SessionLost);
            }
            else if (result == nn::pia::ResultNetworkConnectionIsLost() || 
                     result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                     result == nn::pia::ResultWifiOff() ||
                     result == nn::pia::ResultSleep())
            {
                SetCommunicationErrorType(SampleErrorType_ConnectionLost);
            }
            else
            {
                SetCommunicationErrorType(SampleErrorType_Matchmake);
            }
        }

        // このサンプルデモでは、これらのハンドリングタイプの Result を本関数に渡されることはない
        if (handlingType == nn::pia::Result::HandlingType_Void ||
            handlingType == nn::pia::Result::HandlingType_Ignorable ||
            handlingType == nn::pia::Result::HandlingType_Retry ||
            handlingType == nn::pia::Result::HandlingType_ProgrammingError)
        {
            PIA_SAMPLE_ASSERT(false);
        }
        else if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
        {
            // ホストマイグレーション中にセッションから離脱を開始した場合は、EVENT_SESSION_HOST_CHANGED イベントが通知されないので、ホストマイグレーションのチェックは終了する
            if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
            {
                PIA_SAMPLE_PRINT("cancel HostMigrationEventChecker (Team)");
                g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
            }

#if PIA_TEST_CODE
            if (nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->IsInProgress())
            {
                nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->SignalCancel();
            }
            nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->Reset();
#endif

            nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
            if (leaveResult.IsSuccess())
            {
                // Session のクリーンアップ＆再スタートアップをすればマッチメイク処理に戻れる状態であるため、ネットワークのリセット不要状態にしておく
                SetResetOfNetworkIsNecessary(false);
            }
        }
        else if (handlingType == nn::pia::Result::HandlingType_Cleanup)
        {
            SetResetOfNetworkIsNecessary(false);
        }
        else if (handlingType == nn::pia::Result::HandlingType_LogoutWithLeave)
        {
            nns::pia::Session_LeaveSession();
        }

        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }

    if (goToGame)
    {
        return SampleSceneNo_SessionMigrationPreprocess;
    }

    // チーム選択へ戻る
    nns::pia::Session_CleanupSession();
    return SampleSceneNo_SelectTeam;
}


SampleSceneNo SceneCommunicationGame()
{
    nn::pia::Result result = nn::pia::ResultSuccess();

    uint32_t frameNo = 0;
    bool isExit = false;
    bool backToTeam = false;

#if PIA_TEST_CODE
    bool isDelayDispatch = false;
    bool isStopDispatch = false;
    int delayCount = 0;
#endif

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_CommunicateGame,
        true,
        nn::pia::session::Session::GetInstance()->IsJointSessionHost(),
        nn::pia::session::Session::GetInstance()->GetStationNum(),
        0);

    nn::pia::transport::StationConnectionInfo localInfo;
    nn::pia::transport::StationConnectionInfo jointSessionHostInfo;

    if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedJointSession)
    {
        result = nn::pia::session::Session::GetInstance()->GetLocalStationConnectionInfo(&localInfo);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        result = nn::pia::session::Session::GetInstance()->GetJointSessionHostStationConnectionInfo(&jointSessionHostInfo);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
    }

    SampleWaitingAsyncProcessing waitingProcess = SampleWaitingAsyncProcessing_Max;
    bool asyncProcessingFlag = false;

    nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
    nn::pia::Result::HandlingType handlingType = nn::pia::Result::HandlingType_Void;

    //
    nn::pia::StationId mylocalStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
    SampleObjectStatus* pStationStatus = g_StatusManager.GetStationStatus(mylocalStationId);
    PIA_SAMPLE_ASSERT(pStationStatus != NULL);

    pStationStatus->isStartJointSession = false;
    pStationStatus->isUpdataStartJointSession = true;
    g_StatusManager.Update();

    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
        {
            // ホストマイグレーションのイベントチェック中
            bool isHostCheckEnd = false;
            bool isJointSessionHostCheckEnd = false;

            if ((g_HostMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_SessionHostChanged) != 0)
            {
                // ホストのホストマイグレーションはチェック対象
                nn::pia::StationId hostStationId = nn::pia::session::Session::GetInstance()->GetHostStationId();
                if (nn::pia::session::Session::GetInstance()->IsValidStationId(hostStationId))
                {
                    // ホストのステーション ID が有効
                    if (hostStationId == nn::pia::session::Session::GetInstance()->GetLocalStationId())
                    {
                        // 自分は新ホスト
                        if (nn::pia::session::Session::GetInstance()->IsHost())
                        {
                            // EVENT_SESSION_HOST_CHANGED イベントが発生済みのはず
                            PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_SessionHostChanged) != 0);
                            isHostCheckEnd = true;
                        }
                    }
                    else
                    {
                        // 自分は新クライアント
                        // EVENT_SESSION_HOST_CHANGED イベントが発生済みのはず
                        PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_SessionHostChanged) != 0);
                        isHostCheckEnd = true;
                    }
                }
            }
            else
            {
                isHostCheckEnd = true;
            }

            if ((g_HostMigrationEventChecker.GetExpectedEventFlag() & EventChecker::EventBit_JointSessionHostChanged) != 0)
            {
                // ジョイントセッションホストのホストマイグレーションはチェック対象
                nn::pia::StationId jointSessionHostStationId = nn::pia::session::Session::GetInstance()->GetJointSessionHostStationId();
                if (nn::pia::session::Session::GetInstance()->IsValidStationId(jointSessionHostStationId))
                {
                    // ジョイントセッションホストのステーション ID が有効
                    if (jointSessionHostStationId == nn::pia::session::Session::GetInstance()->GetLocalStationId())
                    {
                        // 自分は新ジョイントセッションホスト
                        if (nn::pia::session::Session::GetInstance()->IsJointSessionHost())
                        {
                            // EVENT_JOINT_SESSION_HOST_CHANGED イベントが発生済みのはず
                            PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_JointSessionHostChanged) != 0);
                            isJointSessionHostCheckEnd = true;
                        }
                    }
                    else
                    {
                        // 自分は新ジョイントセッションホストではない
                        // EVENT_JOINT_SESSION_HOST_CHANGED イベントが発生済みのはず
                        PIA_SAMPLE_ASSERT((g_HostMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_JointSessionHostChanged) != 0);
                        isJointSessionHostCheckEnd = true;
                    }
                }
            }
            else
            {
                isJointSessionHostCheckEnd = true;
            }

            if (isHostCheckEnd && isJointSessionHostCheckEnd)
            {
                g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
            }
        }

        if (!asyncProcessingFlag && s_IsErrorOccurred)
        {
            SetCommunicationErrorType(SampleErrorType_Application);
            handlingType = nn::pia::Result::HandlingType_CleanupWithLeave;
            isExit = true;
        }
        else if (!asyncProcessingFlag && !s_IsErrorOccurred)
        {
#if PIA_TEST_CODE
#else
            if (nn::pia::session::Session::GetInstance()->IsJointSessionHost() && CheckSessionStatus())
            {
                // サンプルで指定したチーム数以上が参加していることを検知したらクローズします
                // しかしながら、同時に複数チームが参加してきた場合はクローズが間に合わないので、
                // 参加完了後にチーム数のチェックをしてください
                if (g_TeamConstitutionManager.GetTeamNum() >= TeamMax)
                {
                    if (nn::pia::session::Session::GetInstance()->GetJointSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Open)
                    {
                        PIA_SAMPLE_PRINT("Call CloseJointSessionAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->CloseJointSessionAsync();
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_CloseJointSession;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call CloseJointSessionAsync is failure. Result:0x%08X\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                }
                // サンプルで指定したチーム数より少ないチーム数になったのでオープンします
                else if (g_TeamConstitutionManager.GetTeamNum() < TeamMax)
                {
                    if (nn::pia::session::Session::GetInstance()->GetJointSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Close)
                    {
                        PIA_SAMPLE_PRINT("Call OpenJointSessionAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->OpenJointSessionAsync();
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_OpenJointSession;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call OpenJointSessionAsync is failure. Result:0x%08X\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                }
            }
#endif
            if (!asyncProcessingFlag)
            {
                // X: DestroyJointSession (JointSessionHost)
                if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonX))
                {
#if PIA_TEST_CODE
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                    {
                        if (nn::pia::session::Session::GetInstance()->IsHost() && CheckSessionStatus())
                        {
                            g_JointSessionType = JointSessionType_Leave;
                            backToTeam = true;
                            isExit = true;
                        }
                    }
                    else
#endif
                        if (nn::pia::session::Session::GetInstance()->IsJointSessionHost() && CheckSessionStatus())
                    {
                        g_JointSessionType = JointSessionType_Destroy;
                        backToTeam = true;
                        isExit = true;
                    }

                    if (backToTeam)
                    {
                        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
                        nn::pia::session::Session::StationIdIterator itEnd = pSession->End();
                        for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != itEnd; ++it)
                        {
                            if (g_TeamConstitutionManager.GetTeamIndex(*it) != g_TeamConstitutionManager.GetTeamIndex(pSession->GetLocalStationId()))
                            {
                                g_SessionMigrationEventChecker.SetExpectedLeaveEventStationId(*it);
                            }
                        }
                    }
                }
                // B: LeaveSession
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonB))
                {
                    PIA_SAMPLE_PRINT("Call LeaveSessionAsync.\n");
                    result = nn::pia::session::Session::GetInstance()->LeaveSessionAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("Call LeaveSession is failure.\n");
                        PIA_SAMPLE_ASSERT(false);
                        s_IsErrorOccurred = true;
                        isExit = true;
                    }
                    else
                    {
                        asyncProcessingFlag = true;
                        waitingProcess = SampleWaitingAsyncProcessing_LeaveSession;
                        s_IsLeaveProcessing = true;

                        // ホストマイグレーション中にセッションから離脱を開始した場合は、EVENT_SESSION_HOST_CHANGED イベントが通知されないので、ホストマイグレーションのチェックは終了する
                        if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
                        {
                            g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
                        }
                    }
                }
                // ホストマイグレーション後にマッチメイクセッションの属性のチームタイプを変更する
                else if (s_IsNeedModifyAttribute)
                {
                    PIA_SAMPLE_PRINT("Call UpdateJointSessionSettingAsync.\n");
                    nn::pia::inet::NexUpdateSessionSetting updateSetting;

                    updateSetting.SetAttribute(MatchmakeAttributeIndexTeamType, static_cast<uint32_t>(GetCurrentTeamType()));

                    result = nn::pia::session::Session::GetInstance()->UpdateJointSessionSettingAsync(&updateSetting);
                    if (result.IsSuccess())
                    {
                        asyncProcessingFlag = true;
                        waitingProcess = SampleWaitingAsyncProcessing_UpdateJointSessionSetting;
                        s_IsNeedModifyAttribute = false;
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("Call UpdateJointSessionSettingAsync is failure. Result:0x%08X\n", result.GetValue());
                        PIA_SAMPLE_ASSERT(false);
                        s_IsErrorOccurred = true;
                        isExit = true;
                    }
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonL))
                {
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                    {
                        PIA_SAMPLE_PRINT("Call RequestJointSessionInfoAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->RequestJointSessionInfoAsync();
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_RequestJointSessionInfo;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call RequestJointSessionInfoAsync is failed. Result:%08x\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                    else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
                    {
                        PIA_SAMPLE_PRINT("Call RequestSessionInfoAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->RequestSessionInfoAsync();
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_RequestSessionInfo;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call RequestSessionInfoAsync is failed. Result:%08x\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                }
#if PIA_TEST_CODE
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonR))
                {
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                    {
                        PIA_SAMPLE_PRINT("Call OpenJointSessionAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->OpenJointSessionAsync();
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_OpenJointSession;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call OpenJointSessionAsync is failure. Result:0x%08X\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                    else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
                    {
                        PIA_SAMPLE_PRINT("Call CloseJointSessionAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->CloseJointSessionAsync();
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_CloseJointSession;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call CloseJointSessionAsync is failure. Result:0x%08X\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                    else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
                    {
                        nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->InitiateCall();
                    }
                    else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
                    {
                        if (nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->IsInProgress())
                        {
                            nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->SignalCancel();
                        }
                        nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->Reset();
                    }
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonStart))
                {
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
                    {
                        // 一部のパラメータのみの更新の場合、必要ないものに関しては設定は不要です
                        nn::pia::inet::NexUpdateSessionSetting updateSetting;
                        const uint32_t MatchmakeAttributeIndex = 6;
                        uint32_t attr[nn::pia::inet::AttributeSizeMax];
                        for (int i = 0; i < nn::pia::inet::AttributeSizeMax; i++)
                        {
                            if (i != MatchmakeAttributeIndex)
                            {
                                attr[i] = 111 * i;
                            }
                        }
                        updateSetting.SetAttributes(attr);
                        const nn::pia::common::String pass(SAMPLE_STRING("newpassword"));
                        updateSetting.SetSessionUserPassword(pass);
                        updateSetting.SetSelectionPriority(3);
                        updateSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
                        updateSetting.SetMaxParticipantNum(8);
                        updateSetting.SetMinParticipantNum(2);

                        const uint32_t applicationDataSize = 16;
                        uint8_t applicationData[applicationDataSize];

                        for (int i = 0; i < applicationDataSize; i++)
                        {
                            applicationData[i] = 100 + i;
                        }
                        updateSetting.SetApplicationData(applicationData, sizeof(applicationData));

                        PIA_SAMPLE_PRINT("Call UpdateJointSessionSettingAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->UpdateJointSessionSettingAsync(&updateSetting);
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_UpdateJointSessionSetting;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call UpdateJointSessionSettingAsync is failure. Result:0x%08X\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                    else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
                    {
                        nn::pia::inet::NexUpdateSessionSetting updateSetting;
                        const uint32_t applicationDataSize = 16;
                        uint8_t applicationData[applicationDataSize];

                        for (int i = 0; i < applicationDataSize; i++)
                        {
                            applicationData[i] = 100 + i;
                        }
                        updateSetting.SetApplicationData(applicationData, sizeof(applicationData));

                        PIA_SAMPLE_PRINT("Call UpdateJointSessionSettingAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->UpdateJointSessionSettingAsync(&updateSetting);
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_UpdateJointSessionSetting;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call UpdateJointSessionSettingAsync is failure. Result:0x%08X\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                    else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                    {
                        // 一部のパラメータのみの更新の場合、必要ないものに関しては設定は不要です
                        nn::pia::inet::NexUpdateSessionSetting updateSetting;
                        const uint32_t MatchmakeAttributeIndex = 6;
                        uint32_t attr[nn::pia::inet::AttributeSizeMax];
                        for (int i = 0; i < nn::pia::inet::AttributeSizeMax; i++)
                        {
                            if (i != MatchmakeAttributeIndex)
                            {
                                attr[i] = 111 * i;
                            }
                        }
                        updateSetting.SetAttributes(attr);
                        const nn::pia::common::String pass(SAMPLE_STRING("newpassword"));
                        updateSetting.SetSessionUserPassword(pass);
                        updateSetting.SetSelectionPriority(3);
                        updateSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
                        updateSetting.SetMaxParticipantNum(8);
                        updateSetting.SetMinParticipantNum(2);

                        const uint32_t applicationDataSize = 16;
                        uint8_t applicationData[applicationDataSize];

                        for (int i = 0; i < applicationDataSize; i++)
                        {
                            applicationData[i] = 100 + i;
                        }
                        updateSetting.SetApplicationData(applicationData, sizeof(applicationData));

                        PIA_SAMPLE_PRINT("Call UpdateSessionSettingAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->UpdateSessionSettingAsync(&updateSetting);
                        if (result.IsSuccess())
                        {
                            asyncProcessingFlag = true;
                            waitingProcess = SampleWaitingAsyncProcessing_UpdateSessionSetting;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("Call UpdateSessionSettingAsync is failure. Result:0x%08X\n", result.GetValue());
                            PIA_SAMPLE_ASSERT(false);
                            s_IsErrorOccurred = true;
                            isExit = true;
                        }
                    }
                }
#endif
            }
        }

#if PIA_TEST_CODE
        // 非同期処理中にもディスパッチ間隔を調整することがあるので、非同期処理のチェックから外れた箇所で操作
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight) &&
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            isDelayDispatch = !isDelayDispatch;
            delayCount = 0;
            isStopDispatch = false;
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp) &&
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            PIA_SAMPLE_PRINT("Dispatch stoped.");
            isStopDispatch = !isStopDispatch;
            isDelayDispatch = false;
            delayCount = 0;
        }
#endif

        if (asyncProcessingFlag)
        {
            // 非同期処理の進捗チェック
            switch (waitingProcess)
            {
            case SampleWaitingAsyncProcessing_LeaveSession:
            {
                if (nn::pia::session::Session::GetInstance()->IsLeaveSessionCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetLeaveSessionResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("LeaveSession is success.\n");
                        isExit = true;
                    }
                    else
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                            result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                            result == nn::pia::ResultWifiOff() ||
                            result == nn::pia::ResultSleep());
                    }
                    asyncProcessingFlag = false;
                    s_IsLeaveProcessing = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_OpenJointSession:
            {
                if (nn::pia::session::Session::GetInstance()->IsOpenJointSessionCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetOpenJointSessionResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("OpenJointSession is success.\n");
                    }
                    else
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                          result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_CloseJointSession:
            {
                if (nn::pia::session::Session::GetInstance()->IsCloseJointSessionCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetCloseJointSessionResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("CloseJointSession is success.\n");
                    }
                    else if (result == nn::pia::ResultNegligibleFault())
                    {
                        PIA_SAMPLE_PRINT("CloseJointSession is failed. ResultNegligibleFault\n");
                        // クローズに失敗したが、無視可能なエラーのため、サンプルの都合上 Result を成功に上書きしておく
                        result = nn::pia::ResultSuccess();
                    }
                    else
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                          result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_UpdateJointSessionSetting:
            {
                if (nn::pia::session::Session::GetInstance()->IsUpdateJointSessionSettingCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetUpdateJointSessionSettingResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("UpdateJointSessionSetting is success.\n");
                    }
                    else
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                          result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_UpdateSessionSetting:
            {
                if (nn::pia::session::Session::GetInstance()->IsUpdateSessionSettingCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetUpdateSessionSettingResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("UpdateSessionSetting is success.\n");
                    }
                    else
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                            result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                            result == nn::pia::ResultWifiOff() ||
                            result == nn::pia::ResultSleep());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_RequestJointSessionInfo:
            {
                if (nn::pia::session::Session::GetInstance()->IsRequestJointSessionInfoCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetRequestJointSessionInfoResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("RequestJointSessionInfo Complete success\n");
                        const_cast<nn::pia::session::ISessionInfo*>(nn::pia::session::Session::GetInstance()->GetJointSessionInfo())->Trace(nn::pia::common::TraceFlagAlways);
                    }
                    else
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                          result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            case SampleWaitingAsyncProcessing_RequestSessionInfo:
            {
                if (nn::pia::session::Session::GetInstance()->IsRequestSessionInfoCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetRequestSessionInfoResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("RequestSessionInfo Complete success\n");
                        const_cast<nn::pia::session::ISessionInfo*>(nn::pia::session::Session::GetInstance()->GetSessionInfo())->Trace(nn::pia::common::TraceFlagAlways);
                    }
                    else
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                            result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                            result == nn::pia::ResultWifiOff() ||
                            result == nn::pia::ResultSleep());
                    }
                    asyncProcessingFlag = false;
                }
            }
            break;
            default:
            {
            }
            break;
            }
            if (result.IsFailure())
            {
                s_IsErrorOccurred = true;
                isExit = true;
                PIA_SAMPLE_PRINT("Process(%d) ErrorCode:%d\n", waitingProcess, result.GetErrorCode());
            }
        }
        else
        {
            // セッション移行処理中フラグのチェック
            if (IsSessionMigrating())
            {
                // ゲームセッションからチームセッションへ分裂する際に発生
                PIA_SAMPLE_PRINT("ChangingSessionFlag became true. GoBackToTeam.\n");
                backToTeam = true;
                isExit = true;
            }

            // 被切断チェックは CheckConnection で行う
            result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
            SetNetworkErrorResult(result);
            if (result.IsFailure())
            {
                if (result == nn::pia::ResultNetworkConnectionIsLost())
                {
                    PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK\n");
                }
                else if (result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin())
                {
                    PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK by duplicate login\n");
                }
                else if (result == nn::pia::ResultWifiOff())
                {
                    PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK by WifiOff\n");
                }
                else if (result == nn::pia::ResultSleep())
                {
                    PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected NETWORK by Sleep\n");
                }
                else if (result == nn::pia::ResultSessionConnectionIsLost())
                {
                    PIA_SAMPLE_PRINT("CheckConnection: Session::CheckConnection: Disconnected SESSION\n");
                }
                else
                {
                    // ありえない値
                    PIA_SAMPLE_PRINT("Session::CheckConnection: Unexpected result:0x%08x\n", result.GetErrorCode());
                    PIA_SAMPLE_ASSERT(false);
                }
                s_IsErrorOccurred = true;
                isExit = true;
            }
        }

        if (!s_IsErrorOccurred)
        {
            // R: ChangeOperationTarget
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                g_StatusManager.ChangeOperationTarget(GetCurrentTeamType());
            }
            // A: AtomicLockOperation
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                g_StatusManager.AtomicLockOperation();
            }

#if PIA_TEST_CODE
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
                PIA_SAMPLE_PRINT("[JointSequence] %d\n", nn::pia::session::Session::GetInstance()->GetJointSequence());
                if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
                {
                    pMgr->SetSendThreadPacketLossRatio(100);
                    pMgr->SetReceiveThreadPacketLossRatio(10);
                    PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 100%, Receive 10%).\n");
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
                {
                    pMgr->SetSendThreadPacketLossRatio(10);
                    pMgr->SetReceiveThreadPacketLossRatio(100);
                    PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 10%, Receive 100%).\n");
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight))
                {
                    pMgr->SetSendThreadPacketLossRatio(10);
                    pMgr->SetReceiveThreadPacketLossRatio(10);
                    PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 10%).\n");
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
                {
                    pMgr->SetSendThreadPacketLossRatio(0);
                    pMgr->SetReceiveThreadPacketLossRatio(0);
                    PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 0%).\n");
                }
                else
                {
                    pMgr->SetSendThreadPacketLossRatio(100);
                    pMgr->SetReceiveThreadPacketLossRatio(100);
                    PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 100%).\n");
                }
            }
#endif

            int32_t diffX = 0, diffY = 0;
            // 矩形の移動入力チェック
            {
                float analogX, analogY;
                nns::pia::sampleframework::SampleFramework::GetInstance()->GetAnalog(&analogX, &analogY);
                if (analogX != 0.0f || analogY != 0.0f)
                {
                    const float cAnalogMoveBase = 3.0f;
                    diffX = static_cast<int32_t>(cAnalogMoveBase * analogX);
                    diffY = static_cast<int32_t>(cAnalogMoveBase * analogY * (-1.0f));  // アナログスティックの上下と描画の上下では正負が逆
                }
                else
                {
                    const int32_t cMoveBase = 2;
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
                    {
                        diffX -= cMoveBase;
                    }
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
                    {
                        diffX += cMoveBase;
                    }
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                    {
                        diffY -= cMoveBase;
                    }
                    if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
                    {
                        diffY += cMoveBase;
                    }

                    // 移動入力が無い場合でも、通信をわかりやすくするために常に移動させる
                    if (diffX == 0 && diffY == 0)
                    {
                        diffX = 1;
                    }
                }
            }

            // Dispatch 処理前に送信用処理
            if (IsInJointSession())  // セッション移行処理中は送信しない
            {
                // 送信間隔チェック
                if ((frameNo % (IntervalSendingStationStatus + 1)) == 0)
                {
                    // 送信するデータを更新
                    UpdateSendData(diffX, diffY);
                }
            }

            // CloneProtocol のクロックを更新
            nns::pia::Clone_GetCloneProtocol()->UpdateClock();

#if PIA_TEST_CODE
            // テストのためディスパッチの頻度を下げて、特定のタイミングを狙い易くします。
            if (isDelayDispatch)
            {
                ++delayCount;
                delayCount = delayCount % DispatchDelayCount;
            }

            if ((!isDelayDispatch || (isDelayDispatch && delayCount == DispatchDelayCount - 1)) && !isStopDispatch)
            {
#endif

                // Dispatch 処理
                nn::pia::common::Scheduler::GetInstance()->Dispatch();
#if PIA_TEST_CODE
            }
#endif
            nns::pia::Nex_SchedulerDispatch();


            // チーム構成が変更されていないか調べる
            g_TeamConstitutionManager.Update();

            // Dispatch 処理後に受信処理
            // 受信データをもとに状態を更新
            g_StatusManager.Update();

            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
                SampleSceneNo_CommunicateGame,
                true,
                nn::pia::session::Session::GetInstance()->IsJointSessionHost(),
                nn::pia::session::Session::GetInstance()->GetStationNum(),
                0);

            //画面描画処理
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

                //文字列
                {
                    //自分自身の役割チェック
                    int roleIndex = 0;       //Unknown;
                    int jointRoleIndex = 0;  //Unknown;
                    localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
                    if (localStationId != nn::pia::StationIdInvalid)
                    {
                        if (nn::pia::session::Session::GetInstance()->IsHost())
                        {
                            roleIndex = 1;  // 1:Host
                        }
                        else
                        {
                            roleIndex = 2;  // 2:Client
                        }

                        if (nn::pia::session::Session::GetInstance()->IsJointSessionHost())
                        {
                            jointRoleIndex = 1;  // 1:JointSessionHost
                        }
                        else
                        {
                            jointRoleIndex = 2;  // 2:JointSessionClient
                        }
                    }

                    int line = 0;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Communication (Game)  TeamNum:%d", g_TeamConstitutionManager.GetTeamNum());
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationNum:%2d  Role:%-6s %-18s", nn::pia::session::Session::GetInstance()->GetStationNum(), GetSessionRoleString(roleIndex), JOINT_GetSessionRoleString(jointRoleIndex));
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationId:%llu TeamMemberIndex:%2d", localStationId, g_TeamConstitutionManager.GetMemberIndex(localStationId));
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  SessionId:%d JointSessionId:%d", nn::pia::session::Session::GetInstance()->GetSessionId(), nn::pia::session::Session::GetInstance()->GetJointSessionId());
                    line += FontSize;

                    line = ScreenHeight - (FontSize * 8);
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  X: DestroyJointSession (JointSessionHost)");
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  B: LeaveSession");
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  R: ChangeOperationTarget");
                    line += FontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: TryLock (Atomic)");
                    line += FontSize;

                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
                    line += FontSize;
                }

                // StationStatus の描画
                uint8_t joinedTeamFlag = 0x0;
                const uint8_t FlagBit = 0x1;
                for (int i = 0; i < GamePlayerMax; ++i)
                {
                    SampleObjectStatus* pStatus = g_StatusManager.GetStationStatus(i);
                    if (pStatus->stationId != nn::pia::StationIdInvalid)
                    {
                        DrawStationStatus(*pStatus);
                        int teamIndex = g_TeamConstitutionManager.GetTeamIndex(pStatus->stationId);
                        if (teamIndex >= 0)
                        {
                            joinedTeamFlag |= FlagBit << teamIndex;
                        }
                    }
                }

                // チーム区別用枠線の描画
                for (int i = 0; i < TeamMax; ++i)
                {
                    if (joinedTeamFlag & (FlagBit << i))
                    {
                        DrawTeamFrame(i, GetColorOfTeamType(static_cast<SampleTeamType>(i)));
                    }
                }

                // 共有オブジェクトの描画
                for (int i = 0; i < CommonObjectNum; ++i)
                {
                    SampleObjectStatus* pStatus = g_StatusManager.GetCommonObjectStatus(i);
                    if (pStatus->type == SampleObjectStatusType_TeamAtomic || pStatus->type == SampleObjectStatusType_TeamSequential)
                    {
                        if (pStatus->teamIndex < 0 || (joinedTeamFlag & (FlagBit << pStatus->teamIndex)) == 0)
                        {
                            continue;
                        }
                    }
                    DrawStationStatus(*pStatus);
                }
            }
        }
#if PIA_TEST_CODE
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_LocalStationIndex, nn::pia::session::Session::GetInstance()->GetLocalStationIndex());
#endif

        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();
        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        frameNo++;
    }

    if (s_IsErrorOccurred)
    {
        PIA_SAMPLE_ASSERT(asyncProcessingFlag == false);

        if (handlingType == nn::pia::Result::HandlingType_Void)
        {
            handlingType = result.GetHandlingType();
            SetNetworkErrorResult(result);

            if (result == nn::pia::ResultSessionConnectionIsLost())
            {
                SetCommunicationErrorType(SampleErrorType_SessionLost);
            }
            else if (result == nn::pia::ResultNetworkConnectionIsLost() || 
                     result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                     result == nn::pia::ResultWifiOff() ||
                     result == nn::pia::ResultSleep())
            {
                SetCommunicationErrorType(SampleErrorType_ConnectionLost);
            }
            else
            {
                SetCommunicationErrorType(SampleErrorType_Matchmake);
            }
        }

        PIA_SAMPLE_PRINT("HandlingType=%d\n", handlingType);

        // このサンプルデモでは、これらのハンドリングタイプの Result を本関数に渡されることはない
        if (handlingType == nn::pia::Result::HandlingType_Void ||
            handlingType == nn::pia::Result::HandlingType_Ignorable ||
            handlingType == nn::pia::Result::HandlingType_Retry ||
            handlingType == nn::pia::Result::HandlingType_ProgrammingError)
        {
            PIA_SAMPLE_ASSERT(false);
        }
        else if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
        {
            // ホストマイグレーション中にセッションから離脱を開始した場合は、EVENT_SESSION_HOST_CHANGED イベントが通知されないので、ホストマイグレーションのチェックは終了する
            if (g_HostMigrationEventChecker.GetCheckTarget() == EventChecker::CheckTarget_HostMigration)
            {
                PIA_SAMPLE_PRINT("cancel HostMigrationEventChecker (Game)");
                g_HostMigrationEventChecker.CheckEnd(EventChecker::CheckTarget_HostMigration);
            }

#if PIA_TEST_CODE
            if (nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->IsInProgress())
            {
                nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->SignalCancel();
            }
            nn::pia::session::Session::GetInstance()->GetInternalAsyncContext()->Reset();
#endif

            nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
            if (leaveResult.IsSuccess())
            {
                // Session のクリーンアップ＆再スタートアップをすればマッチメイク処理に戻れる状態であるため、ネットワークのリセット不要状態にしておく
                SetResetOfNetworkIsNecessary(false);
            }
        }
        else if (handlingType == nn::pia::Result::HandlingType_Cleanup)
        {
            SetResetOfNetworkIsNecessary(false);
        }
        else if (handlingType == nn::pia::Result::HandlingType_LogoutWithLeave)
        {
            nns::pia::Session_LeaveSession();
        }

        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }

    if (backToTeam)
    {
        return SampleSceneNo_SessionMigrationPreprocess;
    }

    // チーム選択へ戻る
    nns::pia::Session_CleanupSession();
    return SampleSceneNo_SelectTeam;
}

#endif
