﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/common.h>
#if NN_PIA_CTR
#include <nn/pia/local.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/err.h>
#endif

#include "SampleLib/SampleLib.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Nex_Setup.h"

#include "SampleConfig.h"

namespace
{
SampleErrorType s_ErrorType = SampleErrorType_Unknown;  //直前に発生したエラーを保持しておく変数

const char* ERROR_INFO_STRING[] =
    {
        "Unknown Reason",
        "Initialize Error (wireless OFF)",
        "Initialize Error (Account)",
        "Initialize Error (other reason)",
        "Matchmake is Failed",
        "Session is Lost",
        "Connection is Lost",
        "Application Error"
    };

enum nns::pia::NetResult s_NetResult = nns::pia::NetResult_Success;
nn::pia::Result s_PiaResult = nn::pia::ResultSuccess();

bool s_ResetOfNetworkIsNecessary = true;
}

/*!
    @brief エラーシーンに飛ぶ前にエラー理由をセットする関数です。
 */
void SetCommunicationErrorType(SampleErrorType error)
{
    s_ErrorType = error;
}

void ClearNetworkErrorResult()
{
    s_PiaResult = nn::pia::ResultSuccess();
    s_NetResult = nns::pia::NetResult_Success;
}


// インターネット接続時に発生したエラーのResultを渡してください。
// ここで渡されたResultがネットワークエラーコード変換APIに渡され、
// 画面に表示されます。
void SetNetworkErrorResult(nn::pia::Result result)
{
    s_PiaResult = result;
}

void SetNetworkErrorResult(enum nns::pia::NetResult result)
{
    s_NetResult = result;
}


uint32_t GetNetworkErrorCode(void)
{
    // Pia の Result の方が優先
    if (s_PiaResult.IsFailure())
    {
        return s_PiaResult.GetErrorCode();
    }
    else if (s_NetResult != nns::pia::NetResult_Success && s_NetResult != nns::pia::NetResult_AccountError)
    {
        return nns::pia::Net_GetNetworkErrorCode();
    }
    else
    {
        return 0;
    }
}


//エラーシーンに飛ぶ前にネットワークのリセット（再ログインなど）が必要か否かをセットする関数です。
void SetResetOfNetworkIsNecessary(bool isNecessary)
{
    s_ResetOfNetworkIsNecessary = isNecessary;
}

bool GetResetOfNetworkIsNecessary()
{
    return s_ResetOfNetworkIsNecessary;
}


/*!
    @brief インターネット接続環境でエラーが発生した場合に、そのネットワークエラーコードを表示するシーンです。
    
    本来はネットワークエラーコードはエラー・EULAアプレットに渡してください。
 */
SampleSceneNo SceneNetworkError()
{
    // エラーの Result をネットワークエラーコードに変換します。
    // ここで得られたコードは、エラー・EULAアプレットに渡されるべきですが、
    // エラー・EULAアプレットは開発機材が prod モードに設定されていないと
    // 利用できませんので、利便性を考慮し、本サンプルデモではエラー・EULAアプレットを
    // 使用していません。
    uint32_t networkErrorCode = GetNetworkErrorCode();

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(SampleSceneNo_NetworkError, false);

    int frameNo = 0;
    bool isExit = false;

    // 描画
    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // Aボタンでループ脱出
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            isExit = true;
        }

        int line;

        // 画面描画
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

        line = 0;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, " Scene: Error occured.");
        line += FontSize;
        line += FontSize;

        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  %s", ERROR_INFO_STRING[s_ErrorType]);
        line += FontSize;
        line += FontSize;

        if (s_ErrorType != SampleErrorType_Application)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Result: 0x%08X", s_PiaResult.GetValue());
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   HandlingType:    %4d", s_PiaResult.GetHandlingType());
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   ViewerType:      %4d", s_PiaResult.GetViewerType());
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   Description:     %4d", s_PiaResult.GetDescription());
            line += FontSize;
            line += FontSize;

#if NN_PIA_NINTENDOSDK && !PIA_TEST_CODE
            if (s_PiaResult.IsFailure() && s_PiaResult.GetViewerType() != nn::pia::Result::ViewerType_ShouldNotUse)
            {
                nn::err::ErrorCode errCode;
                if (s_PiaResult.GetErrorCode(&errCode).IsSuccess())
                {
                    nn::err::ShowError(errCode);
                    networkErrorCode = 0;
                    isExit = true;
                }
            }
#endif
            if (networkErrorCode != 0)
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Network error code: %07d", networkErrorCode);
                line += FontSize;
            }

#if NN_PIA_NINTENDOSDK
            if (nns::pia::Inet_GetLoginError().IsFailure())
            {
#if PIA_TEST_CODE
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Login error code: %08d", nns::pia::Inet_GetLoginError().GetInnerValueForDebug());
                line += FontSize;
#else
                // ログインエラーはエラービューアアプレットで表示済みなので、次のシーンに進む
                isExit = true;
#endif
            }
#else
            if (nns::pia::Inet_GetLoginError() != 0)
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Login error code: %07d", nns::pia::Inet_GetLoginError());
                line += FontSize;
            }
#endif // NN_PIA_NINTENDOSDK
        }
        line = ScreenHeight - (FontSize * 4);
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: Exit and Restart.");
        line += FontSize;
        line += FontSize;

        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
        line += FontSize;

        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        nns::pia::Nex_SchedulerDispatch();
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        frameNo++;
    }

    //念のためエラー情報をクリアしておく
    s_ErrorType = SampleErrorType_Unknown;
    ClearNetworkErrorResult();

    SampleSceneNo nextSceneNo;
    if (GetResetOfNetworkIsNecessary())
    {
        nextSceneNo = SampleSceneNo_Reset;
    }
    else
    {
        nextSceneNo = SampleSceneNo_SelectTeam;
    }

    // ネットワークのリセットが必要であるか否かのフラグは初期化しておく
    SetResetOfNetworkIsNecessary(true);

    return nextSceneNo;
}

#endif
