﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "StatusManager.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Clone_Setup.h"

namespace
{
}


StatusManager::StatusManager()
{
    m_pOperationTargetStatus = &m_StationStatus[0];
}

void StatusManager::ClearStationStatus()
{
    for (int i = 0; i < GamePlayerMax; ++i)
    {
        m_StationStatus[i].Clear();
    }
}

void StatusManager::ClearCommonObjectStatus()
{
    for (int i = 0; i < CommonObjectNum; ++i)
    {
        m_CommonObjectStatus[i].Clear();
    }
}

void StatusManager::Update()
{
    // 受信データに基づき、ステータスを更新
    for (int i = 0; i < GamePlayerMax; ++i)
    {
        m_StationStatus[i].Update();
    }

    for (int i = 0; i < CommonObjectNum; ++i)
    {
        m_CommonObjectStatus[i].Update();
    }
}

void StatusManager::AssignStationStatus(nn::pia::StationId stationId, ReceiveObject* pReceiveObject)
{
    // 未使用な StationStatus を探す
    int idx = -1;
    for (int i = 0; i < GamePlayerMax; ++i)
    {
        if (m_StationStatus[i].stationId == nn::pia::StationIdInvalid)
        {
            idx = i;
            break;
        }
    }
    PIA_SAMPLE_ASSERT(idx != -1);
    m_StationStatus[idx].Clear();
    m_StationStatus[idx].type = SampleObjectStatusType_Station;
    m_StationStatus[idx].stationId = stationId;
    m_StationStatus[idx].pObject = pReceiveObject;
}

SampleObjectStatus* StatusManager::GetStationStatus(nn::pia::StationId stationId)
{
    // 指定された StationId を持つ StationStatus を返す。
    for (int i = 0; i < GamePlayerMax; ++i)
    {
        if (m_StationStatus[i].stationId == stationId)
        {
            return &m_StationStatus[i];
        }
    }
    return NULL;
}

SampleObjectStatus* StatusManager::GetStationStatus(int idx)
{
    PIA_SAMPLE_ASSERT(idx < GamePlayerMax);
    return &m_StationStatus[idx];
}

void StatusManager::AssignCommonObjectStatus(SampleObjectStatusType type, int teamIndex, Object* pObject)
{
    // 未使用な CommonObjectStatus を探す
    int idx = -1;
    for (int i = 0; i < CommonObjectNum; ++i)
    {
        if (m_CommonObjectStatus[i].type == SampleObjectStatusType_None)
        {
            idx = i;
            break;
        }
    }
    PIA_SAMPLE_ASSERT(idx != -1);
    m_CommonObjectStatus[idx].Clear();
    m_CommonObjectStatus[idx].teamIndex = teamIndex;
    m_CommonObjectStatus[idx].type = type;
    m_CommonObjectStatus[idx].pObject = pObject;
    m_CommonObjectStatus[idx].pos.y = static_cast<uint16_t>(24 + type * 12);
}

SampleObjectStatus* StatusManager::GetCommonObjectStatus(int idx)
{
    PIA_SAMPLE_ASSERT(idx < CommonObjectNum);
    return &m_CommonObjectStatus[idx];
}

SampleObjectStatus* StatusManager::GetCommonObjectStatusByType(SampleObjectStatusType type, int teamIndex)
{
    for (int i = 0; i < CommonObjectNum; ++i)
    {
        if (m_CommonObjectStatus[i].type == type)
        {
            if (type == SampleObjectStatusType_TeamAtomic || type == SampleObjectStatusType_TeamSequential)
            {
                PIA_SAMPLE_ASSERT(teamIndex != -1);
                if (teamIndex != m_CommonObjectStatus[i].teamIndex)
                {
                    continue;
                }
            }
            return &m_CommonObjectStatus[i];
        }
    }
    return NULL;
}

void StatusManager::ChangeOperationTarget(int localTeamIndex)
{
    PIA_SAMPLE_ASSERT(m_pOperationTargetStatus != NULL);
    switch (m_pOperationTargetStatus->type)
    {
    case SampleObjectStatusType_Station:
    {
        m_pOperationTargetStatus = GetCommonObjectStatusByType(SampleObjectStatusType_TeamAtomic, localTeamIndex);
    }
    break;
    case SampleObjectStatusType_TeamAtomic:
    {
        m_pOperationTargetStatus = GetCommonObjectStatusByType(SampleObjectStatusType_TeamSequential, localTeamIndex);
    }
    break;
    case SampleObjectStatusType_TeamSequential:
    {
        if (g_GameState == GameState_Game)
        {
            m_pOperationTargetStatus = GetCommonObjectStatusByType(SampleObjectStatusType_CommonAtomic);
        }
        else
        {
            m_pOperationTargetStatus = GetStationStatus(nn::pia::session::Session::GetInstance()->GetLocalStationId());
        }
    }
    break;
    case SampleObjectStatusType_CommonAtomic:
    {
        m_pOperationTargetStatus = GetCommonObjectStatusByType(SampleObjectStatusType_CommonSequential);
    }
    break;
    case SampleObjectStatusType_CommonSequential:
    default:
    {
        m_pOperationTargetStatus = GetStationStatus(nn::pia::session::Session::GetInstance()->GetLocalStationId());
    }
    }
    PIA_SAMPLE_ASSERT(m_pOperationTargetStatus != NULL);
    return;
}

void StatusManager::AtomicLockOperation()
{
    if (m_pOperationTargetStatus->type != SampleObjectStatusType_TeamAtomic &&
        m_pOperationTargetStatus->type != SampleObjectStatusType_CommonAtomic)
    {
        return;
    }

    nn::pia::Result result;
    nn::pia::clone::AtomicSharingClone::LockStatus lockStatus = m_pOperationTargetStatus->pObject->GetLockStatus();
    switch (lockStatus)
    {
    case nn::pia::clone::AtomicSharingClone::LockStatus_Locked:
    case nn::pia::clone::AtomicSharingClone::LockStatus_TryLock:
    {
        result = m_pOperationTargetStatus->pObject->Unlock();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("Unlock failed\n");
        }
    }
    break;
    case nn::pia::clone::AtomicSharingClone::LockStatus_Unlocked:
    {
        result = m_pOperationTargetStatus->pObject->TryLock();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("TryLock failed\n");
        }
    }
    break;
    }
}

#endif
