﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "TeamConstitutionManager.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Clone_Setup.h"

TeamConstitutionManager g_TeamConstitutionManager;

namespace
{
int GetTeamType(nn::pia::StationId stationId)
{
    nn::pia::transport::Station* pStation = nn::pia::transport::StationManager::GetInstance()->GetStation(stationId);
    if (pStation == NULL)
    {
        return TeamMax;
    }
    nn::pia::transport::Station::IdentificationToken token;
    nn::pia::Result result = pStation->GetIdentificationToken(&token);
    if (result.IsFailure())
    {
        return TeamMax;
    }
    SampleTokenData tokenData;
    memcpy(&tokenData, token.data, sizeof(tokenData));
    return tokenData.teamType;
}
}

TeamConstitutionManager::TeamConstitutionManager()
{
    m_IsNeedSend = false;
    m_LatestInfoClock = nn::pia::clone::InvalidClock;
    clone.RegisterElement(&m_ElementR, 0x01);
}


void TeamConstitutionManager::Setup()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterAtomicSharingClone(&clone, CloneIdOffset_TeamConstitutionManager + 0x0);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(result);
}

void TeamConstitutionManager::Reset()
{
    m_TeamConstitution.Clear();
    m_IsNeedSend = false;
    m_LatestInfoClock = nn::pia::clone::InvalidClock;
}

void TeamConstitutionManager::Update()
{
    if (nns::pia::Clone_GetCloneProtocol()->GetState() != nn::pia::clone::CloneProtocol::State_Active)
    {
        return;
    }

    nn::pia::Result result;
    if (nn::pia::session::Session::GetInstance()->IsJointSessionHost() || (IsInSession() && nn::pia::session::Session::GetInstance()->IsHost()))
    {
        // チーム構成情報が更新されていれば、送信
        if (m_IsNeedSend)
        {
            if (clone.GetLockStatus() == nn::pia::clone::AtomicSharingClone::LockStatus_Locked)
            {
                // 全員が同期状態であれば送信
                nn::pia::transport::StationManager::Iterator itEnd = nn::pia::transport::StationManager::GetInstance()->End();
                for (nn::pia::transport::StationManager::Iterator it = nn::pia::transport::StationManager::GetInstance()->Begin(); it != itEnd; ++it)
                {
                    if (!nns::pia::Clone_GetCloneProtocol()->IsActiveStation(it->GetStationId()))
                    {
                        return;
                    }
                }
                if (!clone.IsConnectedWithAll())
                {
                    return;
                }

                m_TeamConstitution.Trace();
                PIA_SAMPLE_ASSERT(m_ElementR.IsReadyToSetValue());
                result = m_ElementR.SetValue(m_TeamConstitution);
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("Send Team Constitution Success\n");
                    m_IsNeedSend = false;
                }
                else
                {
                    PIA_SAMPLE_PRINT("Send Team Constitution Failed\n");
                    PIA_SAMPLE_ASSERT(result == nn::pia::ResultTemporaryUnavailable());
                }
            }
            else if (clone.GetLockStatus() == nn::pia::clone::AtomicSharingClone::LockStatus_Unlocked)
            {
                result = clone.TryLock();
                PIA_SAMPLE_ASSERT(result.IsSuccess());
            }
        }
    }
    else
    {
        // 新しいチーム構成情報を受信していたら、チーム情報を更新
        if (m_ElementR.IsValidValue())
        {
            nn::pia::clone::ClockValue clock = m_ElementR.GetClock();
            if (clock != m_LatestInfoClock)
            {
                m_LatestInfoClock = clock;
                m_TeamConstitution = m_ElementR.GetValue();
                PIA_SAMPLE_PRINT("Receive Team Constitution\n");
                m_TeamConstitution.Trace();
                return;
            }
        }
    }
    return;
}

bool TeamConstitutionManager::JoinEvent(nn::pia::StationId stationId)
{
    PIA_SAMPLE_PRINT("JoinEvent called GameState:%d\n", g_GameState);
    if (IsInSession() || IsSessionMigratingToSession() || IsSessionMigratingToJointSession())
    {
        if (!nn::pia::session::Session::GetInstance()->IsHost())
        {
            return true;
        }
    }
    else if (IsInJointSession())
    {
        if (!nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
            return true;
        }
    }
    else
    {
        PIA_SAMPLE_ASSERT(false);
    }

    bool isSuccess = AssignTeamMember(stationId);
    if (!isSuccess)
    {
        // 同じチームに異なるセッションが参加してきた
        return false;
    }

    m_IsNeedSend = true;
    return true;
}

void TeamConstitutionManager::LeaveEvent(nn::pia::StationId stationId)
{
    PIA_SAMPLE_PRINT("LeaveEvent called GameState:%d\n", g_GameState);
    if (IsInSession() || IsSessionMigratingToSession() || IsSessionMigratingToJointSession())
    {
        if (!nn::pia::session::Session::GetInstance()->IsHost())
        {
            return;
        }
    }
    else if (IsInJointSession())
    {
        if (!nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
            return;
        }
    }
    else
    {
        PIA_SAMPLE_ASSERT(false);
    }

    bool isSuccess = CancelTeamMember(stationId);
    PIA_SAMPLE_ASSERT(isSuccess);
    NN_PIA_UNUSED_IN_RELEASE(isSuccess);

    m_IsNeedSend = true;
}

bool TeamConstitutionManager::SessionHostChangedEvent()
{
    PIA_SAMPLE_PRINT("SessionHostChangedEvent called GameState:%d\n", g_GameState);
    if (IsInSession() || IsSessionMigratingToSession() || IsSessionMigratingToJointSession())
    {
        if (!nn::pia::session::Session::GetInstance()->IsHost())
        {
            return true;
        }
    }
    else if (IsInJointSession())
    {
        if (!nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
            return true;
        }
    }
    else
    {
        PIA_SAMPLE_ASSERT(false);
    }

    // セッション移行処理中はスキップ
    if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_MigratingSession)
    {
        return true;
    }

    // 新ホストはチーム編成を最新状態に更新してクライアントに送信する
    bool isSuccess = true;

    // 自分が持っているチーム編成情報に含まれているが既に存在しないステーションは削除する
    // 別のチームに再参加している場合があるので、チームが誤っている場合も削除する
    for (int i = 0; i < TeamMax; ++i)
    {
        for (int j = 0; j < TeamMemberMax; ++j)
        {
            nn::pia::StationId stationId = GetStationId(i, j);
            if (stationId != nn::pia::StationIdInvalid)
            {
                if (nn::pia::session::Session::GetInstance()->IsValidStationId(stationId) == false)
                {
                    isSuccess &= CancelTeamMember(stationId);
                }
                else if (GetTeamType(stationId) != i)
                {
                    PIA_SAMPLE_PRINT("Different team type: %llu (%d != %d)\n", stationId, GetTeamType(stationId), i);
                    isSuccess &= CancelTeamMember(stationId);
                }
            }
        }
    }
    PIA_SAMPLE_ASSERT(isSuccess);

    // 自分が持っているチーム編成情報に含まれていないステーションがいた場合は、追加する
    nn::pia::session::Session::StationIdIterator itEnd = nn::pia::session::Session::GetInstance()->End();
    for (nn::pia::session::Session::StationIdIterator it = nn::pia::session::Session::GetInstance()->Begin(); it != itEnd; ++it)
    {
        if (nn::pia::transport::StationManager::GetInstance()->GetStation(*it) == NULL)
        {
            // すでに離脱している
            continue;
        }
        if (g_TeamConstitutionManager.GetTeamIndex(*it) == -1)
        {
            // チーム編成上に含まれていないので追加する
            isSuccess &= AssignTeamMember(*it);
        }
    }
    if (!isSuccess)
    {
        // 同じチームに異なるセッションが参加してきた
        return false;
    }
    m_IsNeedSend = true;
    return true;
}

bool TeamConstitutionManager::AssignTeamMember(nn::pia::StationId stationId)
{
    int teamType = GetTeamType(stationId);
    PIA_SAMPLE_ASSERT(teamType < TeamMax);

    uint32_t sessionId = nn::pia::session::Session::GetInstance()->GetSessionId(stationId);
    if (m_TeamConstitution.sessionIdTable[teamType] != nn::pia::session::InvalidSessionId && m_TeamConstitution.sessionIdTable[teamType] != sessionId)
    {
        // 同じチームに異なるセッションが参加してきた。
        PIA_SAMPLE_PRINT("different sessionId:0x%08x (teamType:%d sessionId:0x%08x)\n", sessionId, teamType, m_TeamConstitution.sessionIdTable[teamType]);
        return false;
    }
    m_TeamConstitution.sessionIdTable[teamType] = sessionId;

    for (int i = 0; i < TeamMemberMax; ++i)
    {
        if (m_TeamConstitution.stationIdTable[teamType][i] == nn::pia::StationIdInvalid)
        {
            m_TeamConstitution.stationIdTable[teamType][i] = stationId;
            return true;
        }
    }
    PIA_SAMPLE_ASSERT(false);
    return false;
}

bool TeamConstitutionManager::CancelTeamMember(nn::pia::StationId stationId)
{
    if (stationId == nn::pia::StationIdInvalid)
    {
        return false;
    }

    for (int i = 0; i < TeamMax; ++i)
    {
        for (int j = 0; j < TeamMemberMax; ++j)
        {
            if (m_TeamConstitution.stationIdTable[i][j] == stationId)
            {
                m_TeamConstitution.stationIdTable[i][j] = nn::pia::StationIdInvalid;

                // このチームに、他にメンバーがいるかをチェック
                bool isEmpty = true;
                for (int k = 0; k < TeamMemberMax; ++k)
                {
                    if (m_TeamConstitution.stationIdTable[i][k] != nn::pia::StationIdInvalid)
                    {
                        isEmpty = false;
                        break;
                    }
                }
                if (isEmpty)
                {
                    // 誰もいなくなったので、sessionIdTable を更新
                    m_TeamConstitution.sessionIdTable[i] = nn::pia::session::InvalidSessionId;
                }
                return true;
            }
        }
    }

    // ホストマイグレーション時には、新ホストがチーム編成を最新状態に更新後に、チーム編成上存在しないステーションの離脱イベントが発生することがあるので、成功を返しておく
    return true;
}

nn::pia::StationId TeamConstitutionManager::GetStationId(int teamIndex, int memberIndex) const
{
    if (teamIndex >= TeamMax)
    {
        return nn::pia::StationIdInvalid;
    }

    if (memberIndex >= TeamMemberMax)
    {
        return nn::pia::StationIdInvalid;
    }

    return nn::pia::StationId(m_TeamConstitution.stationIdTable[teamIndex][memberIndex]);
}

int TeamConstitutionManager::GetTeamIndex(nn::pia::StationId stationId) const
{
    if (stationId == nn::pia::StationIdInvalid)
    {
        return -1;
    }

    for (int i = 0; i < TeamMax; ++i)
    {
        for (int j = 0; j < TeamMemberMax; ++j)
        {
            if (m_TeamConstitution.stationIdTable[i][j] == stationId)
            {
                return i;
            }
        }
    }
    return -1;
}

int TeamConstitutionManager::GetMemberIndex(nn::pia::StationId stationId) const
{
    if (stationId == nn::pia::StationIdInvalid)
    {
        return -1;
    }

    for (int i = 0; i < TeamMax; ++i)
    {
        for (int j = 0; j < TeamMemberMax; ++j)
        {
            if (m_TeamConstitution.stationIdTable[i][j] == stationId)
            {
                return j;
            }
        }
    }
    return -1;
}

uint32_t TeamConstitutionManager::GetTeamSessionId(int teamIndex) const
{
    if (teamIndex >= TeamMax)
    {
        return nn::pia::session::InvalidSessionId;
    }
    return m_TeamConstitution.sessionIdTable[teamIndex];
}

int TeamConstitutionManager::GetTeamMemberNum(int teamIndex) const
{
    if (teamIndex >= TeamMax)
    {
        return 0;
    }
    int num = 0;
    for (int i = 0; i < TeamMemberMax; ++i)
    {
        if (GetStationId(teamIndex, i) != nn::pia::StationIdInvalid)
        {
            ++num;
        }
    }
    return num;
}

int TeamConstitutionManager::GetTeamNum() const
{
    int teamNum = 0;
    for (int i = 0; i < TeamMax; i++)
    {
        if (m_TeamConstitution.sessionIdTable[i] != nn::pia::session::InvalidSessionId)
        {
            for (int j = 0; j < TeamMemberMax; j++)
            {
                if (m_TeamConstitution.stationIdTable[i][j] != nn::pia::StationIdInvalid)
                {
                    ++teamNum;
                    break;
                }
            }
        }
    }
    return teamNum;
}

// シリアル化のために SerializePolicy を定義します。
namespace nn
{
namespace pia
{
namespace clone
{

template<>
void SerializePolicy<TeamConstitutionManager::TeamConstitution>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    for (int i = 0; i < TeamMax; ++i)
    {
        SerializePolicy<uint32_t>::Serialize(offset, value.sessionIdTable[i]);
        offset += 4;
    }
    for (int i = 0; i < TeamMax; ++i)
    {
        for (int j = 0; j < TeamMemberMax; ++j)
        {
            SerializePolicy<uint64_t>::Serialize(offset, value.stationIdTable[i][j]);
            offset += 8;
        }
    }
}

template<>
void SerializePolicy<TeamConstitutionManager::TeamConstitution>::Deserialize(Type* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    for (int i = 0; i < TeamMax; ++i)
    {
        SerializePolicy<uint32_t>::Deserialize(&pValue->sessionIdTable[i], offset);
        offset += 4;
    }
    for (int i = 0; i < TeamMax; ++i)
    {
        for (int j = 0; j < TeamMemberMax; ++j)
        {
            SerializePolicy<uint64_t>::Deserialize(&pValue->stationIdTable[i][j], offset);
            offset += 8;
        }
    }
}

template<>
uint32_t SerializePolicy<TeamConstitutionManager::TeamConstitution>::GetSerializedSize()
{
    return 4 * TeamMax + 8 * TeamMax * TeamMemberMax;
}

}
}
}  // end of namespace nn::pia::clone

#endif
