﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/pia_Version.h>
#include <nn/pia/common.h>

#include "SampleLib/Defines.h"
#include "SampleLib/SampleLib.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Common_Setup.h"

#include "SampleConfig.h"
#include "TeamConstitutionManager.h"
#include "ObjectManager.h"

GameState g_GameState = GameState_None;
JointSessionType g_JointSessionType = JointSessionType_None;
const SampleStringChar_t* UserPasswordString[] = {
    SAMPLE_STRING("alpha"),
    SAMPLE_STRING("bravo"),
    SAMPLE_STRING("charlie"),
    SAMPLE_STRING("delta"),
    SAMPLE_STRING("joint")
};
uint8_t g_UserPasswordIndexForJoint = 0;

namespace
{
/* ---< Pia に与えるメモリの用意 >--- */
// Pia に与えるメモリ領域です。 4バイト整合されていないといけません。
// このサンプルでは十分余裕を持たせた数値にしています。
PIA_ATTRIBUTE_ALIGN(8) uint8_t s_PiaMemoryBlock[MemoryBlockSizeForPia];
/* ---< end of Pia に与えるメモリの用意 >--- */

// true にするとPiaライブラリのトレースフラグを多数有効にした場合の処理速度への影響を抑えられます。
// ただし、他のライブラリ(NEXなど)のログとの順序関係が入れ替わる可能性があります
const bool IsCachedPrintEnabled = true;

// Piaライブラリバージョン情報文字列保存用変数
char s_SamplePiaVersionInfoString[ScreenWidth / FontSize] = {};

#if PIA_TEST_CODE
bool s_IsHostMigrationEnabled = true;
uint16_t s_MaxPlayerNum = static_cast<uint16_t>(GamePlayerMax);
#endif

}  // end of anonymous namespace

// Piaライブラリバージョン情報文字列取得関数
const char* GetPiaVersionString()
{
    return s_SamplePiaVersionInfoString;
}

#if PIA_TEST_CODE
bool IsHostMigrationEnabled()
{
    return s_IsHostMigrationEnabled;
}

void SetHostMigrationEnabled(bool b)
{
    s_IsHostMigrationEnabled = b;
}

void SetMaxPlayerNum(uint16_t num)
{
    s_MaxPlayerNum = num;
}

uint16_t GetMaxPlayerNum()
{
    return s_MaxPlayerNum;
}
#endif

#if NN_PIA_CTR
uint32_t GetInternetLinkLevel();
#endif

// 電波強度を取得するための処理をまとめた関数
uint32_t GetLinkLevel()
{
#if NN_PIA_CTR
    // 通信方式に合わせて電波強度を取得し、0-3 の数値として返します
    return GetInternetLinkLevel();
#elif NN_PIA_WIN
    // 常に 0 とします
    return 0;
#elif NN_PIA_CAFE
    // 電波強度を取得する方法が無いため、常に 0 とします
    return 0;
#elif NN_PIA_NINTENDOSDK
    // 暫定的に常に 0 とします。
    return 0;
#else
#error "illegal platform."
#endif
}

PIA_SAMPLE_MAIN_FUNC()
{
    PIA_SAMPLE_PRINT("Sample program Start.\n");

    // Piaライブラリバージョン情報文字列の作成
    PIA_SNPRINTF(s_SamplePiaVersionInfoString, ScreenWidth / FontSize, "Pia %d.%d.%d r%d", PIA_VERSION_MAJOR, PIA_VERSION_MINOR, PIA_VERSION_MICRO, PIA_VERSION_REVISION);

    // Pia のサンプルプログラム用描画フレームワークの初期化
    nns::pia::sampleframework::SampleFramework::Setting setting;
    setting.isUpperScreenOnly = true;
    nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(FramePerSecond);
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetClearColor(nns::pia::sampleframework::Color(0.0f, 0.0f, 0.2f, 1.0f));
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(SampleSceneNo_PrepareInet, false);
#if PIA_TEST_CODE
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_HostMigration, IsHostMigrationEnabled() ? 1 : 0);
#endif

    while (1)
    {
        SampleSceneNo nextSceneNo = SampleSceneNo_PrepareInet;

        /* ---< PiaCommon モジュールで利用する機能のセットアップ >--- */
        uint64_t traceFlags = 0;
        if (IsCachedPrintEnabled)
        {
            traceFlags |=
                nn::pia::common::TraceFlagCommon |
                nn::pia::common::TraceFlagLocal |
                nn::pia::common::TraceFlagTransport |
                nn::pia::common::TraceFlagSession |
                nn::pia::common::TraceFlagInet |
                nn::pia::common::TraceFlagClone |
                nn::pia::common::TraceFlagSessionJoint |
#if PIA_TEST_CODE
                nn::pia::common::TraceFlagSessionConnect |
                nn::pia::common::TraceFlagSessionLeave |
                nn::pia::common::TraceFlagSessionUpdate |
                nn::pia::common::TraceFlagSessionMigration |
                nn::pia::common::TraceFlagInetInit |
                nn::pia::common::TraceFlagInetNat |
#endif
#if 0
                nn::pia::common::TraceFlagCloneDetail |
                nn::pia::common::TraceFlagTransportPacketSend |
                nn::pia::common::TraceFlagTransportPacketReceive |
                nn::pia::common::TraceFlagInetNatDetail |
                nn::pia::common::TraceFlagInetNatProbe |
#endif
                nn::pia::common::TraceFlagAlways;
        }
        else
        {
            traceFlags |= nn::pia::common::TraceFlagAlways;
        }
        nns::pia::Common_InitializePiaCommon(s_PiaMemoryBlock, MemoryBlockSizeForPia, traceFlags, IsCachedPrintEnabled);

        /* ---< end of PiaCommon モジュールで利用する機能のセットアップ >--- */

        bool isExit = false;

        /* ゲームループ */
        while (isExit == false)
        {
            switch (nextSceneNo)
            {
            case SampleSceneNo_PrepareInet:
            {
                if (InitializePiaModules())
                {
                    g_ObjectManager.RegisterAll();
                    g_TeamConstitutionManager.Setup();
                    nextSceneNo = SampleSceneNo_SelectTeam;
                }
                else
                {
                    nextSceneNo = SampleSceneNo_NetworkError;
                }
            }
            break;
            case SampleSceneNo_SelectTeam:
            {
                nextSceneNo = SceneSelectTeam();
            }
            break;
            case SampleSceneNo_MatchmakeTeam:
            {
                nextSceneNo = SceneMatchmake();
            }
            break;
            case SampleSceneNo_CommunicateTeam:
            {
                nextSceneNo = SceneCommunicationTeam();
            }
            break;
            case SampleSceneNo_SessionMigrationPreprocess:
            {
                nextSceneNo = SceneChangingSessionPreprocessing();
            }
            break;
            case SampleSceneNo_SessionMigration:
            {
                nextSceneNo = SceneChangingSession();
            }
            break;
            case SampleSceneNo_SessionMigrationPostprocess:
            {
                nextSceneNo = SceneChangingSessionPostprocessing();
            }
            break;
            case SampleSceneNo_CommunicateGame:
            {
                nextSceneNo = SceneCommunicationGame();
            }
            break;
            case SampleSceneNo_NetworkError:
            {
                nextSceneNo = SceneNetworkError();
            }
            break;
            case SampleSceneNo_Reset:
            {
                FinalizePiaModules();
                isExit = true;
            }
            break;
            }
        }

        /* ---< PiaCommon 関連の終了処理 >--- */
        nns::pia::Common_FinalizePiaCommon();
        /* ---< end of PiaCommon 関連の終了処理 >--- */
    }
    //ここには到達しません
    //nns::pia::sampleframework::SampleFramework::DestroyInstance();
    //PIA_SAMPLE_PRINT("Sample program end.\n");

}  // end of nnMain()
#else
PIA_SAMPLE_MAIN_FUNC()
{
}
#endif
