﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include <nn/pia/common.h>
#include <nn/pia/clone.h>
#include <nn/pia/session.h>
#include <nn/pia/lan.h>
#include <nn/pia/transport.h>
#include <nn/pia/framework.h>

#if NN_PIA_NINTENDOSDK
#include <nn/account.h>
#include <nn/account/account_Selector.h>
#endif

#include "SampleLib/SampleLib.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Local_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleFramework/include/SampleFramework.h"

#define NN_PIA_BROWSE_ONLY 0        // 0: ランダムマッチメイク、1: ブラウズマッチメイク
#define NN_PIA_AUTO_NEX_SETTING 1   // NEX の初期化とログインを自動で行うかどうか

const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 65000;
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;

static const uint16_t MaxStationNum = 8;  // 最大 Station 数
static const int MaxBrowsedSessionNum = 8;

// NEX 用
#if NN_PIA_FRAMEWORK_ENABLE_NEX
static const size_t MemoryBlockSizeForNex = 1 * 1024 * 1024;
static const size_t MemoryBlockSizeForNexEmergency = 50 * 1024;
#endif

// ローカル用
#if NN_PIA_NINTENDOSDK || NN_PIA_CTR
#include <nn/pia/local.h>
static const char PassPhrase[] = "FrameworkSimple";
static const int PassPhraseSize = sizeof(PassPhrase);
#if NN_PIA_NINTENDOSDK
static uint16_t LocalSceneId = MatchmakeGameModeOffset + 10000;
#elif NN_PIA_CTR
static const uint32_t LocalTitleId = 0xFF350;
static uint8_t LocalSubId = MatchmakeGameModeOffset;
#endif
#endif

enum State
{
    State_Title,
    State_InitializeFramework,
    State_RandomSession,
    State_CreateOrJoinSession,
    State_Session,
    State_Leave,
    State_FinalizeFramework,
    State_Max
};

enum SendProtocol
{
    SendProtocol_Unreliable,
    SendProtocol_UnreliableAll,
    SendProtocol_Reliable,
    SendProtocol_ReliableAll,
    SendProtocol_Max
};

static int s_CursorY = 0;
static SendProtocol s_SendProtocol = SendProtocol_Unreliable;
static uint32_t s_sendCount[MaxStationNum];
static uint32_t s_receiveCount[MaxStationNum];
static uint32_t s_sendAllCount = 0;

nn::pia::StationIndex ConvertStationIdToStationIndex(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
    if (result.IsFailure())
    {
        stationIndex = nn::pia::StationIndex_Invalid;
    }
    return stationIndex;
}

nn::pia::StationId ConvertStationIndexToStationId(nn::pia::StationIndex stationIndex)
{
    nn::pia::StationId stationId;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, stationIndex);
    if (result.IsFailure())
    {
        stationId = nn::pia::StationIdInvalid;
    }
    return stationId;
}

nns::pia::sampleframework::SampleFramework* GetSampleFramework()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = NULL;
    pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    if (!pSampleFramework)
    {
        PIA_SAMPLE_ASSERT(false);
    }
    return pSampleFramework;
}

// PiaFramework の非同期処理が完了するまで（Framework::IsXxxCompleted() が true を返すまで）ブロックします
void WaitForAsyncProcess(const char* asyncProcessName, bool (nn::pia::framework::Framework::*checkCompletionFunc)() const, State state)
{
    nn::pia::Result result;
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();
    int frameNo = 0;
    while (!(nn::pia::framework::Framework::GetInstance()->*checkCompletionFunc)())
    {
        pSampleFramework->BeginFrame();
        pSampleFramework->DrawString(10, 20, "Sample state : %d", state);
        pSampleFramework->DrawString(10, 30, "PiaFramework state : %d", nn::pia::framework::Framework::GetInstance()->GetState());
        pSampleFramework->DrawString(10, 40, "%s... (frame count: %d)", asyncProcessName, frameNo++);
        result = nn::pia::framework::Framework::GetInstance()->Update();
        pSampleFramework->EndFrame();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->Update() failed.\n");
        }
    }
}

// 失敗した Result が与えられた場合、 HandleErrorAsync でエラーハンドリングを行う。
void ResultCheck(nn::pia::Result result, State* pState, const nn::pia::framework::Framework::State& transitionState)
{
    if (result.IsFailure())
    {
#if NN_PIA_NINTENDOSDK
        // エラービューアで表示すべきエラーのハンドリング
        if (result.GetViewerType() == nn::pia::Result::ViewerType_ShouldUse)
        {
            if (result == nn::pia::ResultSdkViewerResultError())
            {
                nn::err::ShowError(nn::pia::framework::Framework::GetInstance()->GetLastSdkResult());
            }
            else
            {
                nn::err::ErrorCode errorCode;
                result.GetErrorCode(&errorCode);
                nn::err::ShowError(errorCode);
            }
        }
#endif
        PIA_SAMPLE_PRINT("----------------ErrorHandlingAsync start---------------\n");
        result = nn::pia::framework::Framework::GetInstance()->HandleErrorAsync(transitionState, result);
        if (result.IsFailure())
        {
            *pState = State_FinalizeFramework;
            return;
        }

        WaitForAsyncProcess("HandleErrorAsync", &nn::pia::framework::Framework::IsHandleErrorCompleted, *pState);

        result = nn::pia::framework::Framework::GetInstance()->GetHandleErrorResult();
        PIA_SAMPLE_ASSERTMSG(result.IsSuccess(), "Result(%08x)\n    Description: (%4d)\n    HandlingType:(%2d)", result.GetValue(), result.GetDescription(), result.GetHandlingType());
        if (nn::pia::framework::Framework::GetInstance()->GetState() >= nn::pia::framework::Framework::State_SessionJoined)
        {
            *pState = State_Leave;
        }
        else
        {
            *pState = State_FinalizeFramework;
        }
        PIA_SAMPLE_PRINT("[Ape] ----------------ErrorHandlingAsync end---------------\n");
    }

}

// 失敗した Result が与えられた場合、次に遷移すべき状態を求めて HandleErrorAsync でエラーハンドリングを行う。
void ResultCheck(nn::pia::Result result, State* pState)
{
    if (result.IsFailure())
    {
        const nn::pia::framework::Framework::State transitionState = nn::pia::framework::Framework::GetInstance()->ConvertHandlingTypeToState(result.GetHandlingType());
        ResultCheck(result, pState, transitionState);
    }
}

// ChangeStateAsync による通信の状態遷移を行う。
// ChangeStateAsync は非同期処理なので、この関数で終了を待つ。
nn::pia::Result ChangeState(State* pState, const nn::pia::framework::Framework::State& transitionState)
{
    nn::pia::Result result;

    PIA_SAMPLE_PRINT("----------------ChangeState start---------------\n");

    // 状態遷移の開始
    result = nn::pia::framework::Framework::GetInstance()->ChangeStateAsync(transitionState);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->ChangeStateAsync() failed.\n");
        return result;
    }

    WaitForAsyncProcess("ChangeStateAsync", &nn::pia::framework::Framework::IsChangeStateCompleted, *pState);

    result = nn::pia::framework::Framework::GetInstance()->GetChangeStateResult();
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->GetChangeStateResult() failed.\n");
        return result;
    }
    else
    {
        PIA_SAMPLE_ASSERTMSG(result.IsSuccess(), "Result(%08x)\n    Description: (%4d)\n    HandlingType:(%2d)", result.GetValue(), result.GetDescription(), result.GetHandlingType());
    }
    *pState = State_RandomSession;
    PIA_SAMPLE_PRINT("----------------ChangeState end---------------\n");

    return nn::pia::ResultSuccess();
}

// 各 setting の初期化
void InitializeSetting(nn::pia::framework::NetworkType networkType)
{
    // framework 設定
    {
        const int RootHeapSize = 10 * 1024 * 1024;
        PIA_ATTRIBUTE_ALIGN(8) static uint8_t s_memory[RootHeapSize];
        nn::pia::framework::InitializeFrameworkSetting setting;
        setting.piaBuffer = s_memory;
        setting.piaBufferSize = RootHeapSize;
        setting.networkType = networkType;
        setting.traceFlags = nn::pia::common::TraceFlagFramework |
                             nn::pia::common::TraceFlagSession |
                             nn::pia::common::TraceFlagSessionConnect |
                             nn::pia::common::TraceFlagSessionLeave |
                             nn::pia::common::TraceFlagSessionInit |
                             nn::pia::common::TraceFlagLocal |
                             nn::pia::common::TraceFlagInet |
                             nn::pia::common::TraceFlagLan |
                             nn::pia::common::TraceFlagPlugin;
        PIA_SAMPLE_PRINT("---------------Initialize start----------------\n");
        nn::pia::Result result = nn::pia::framework::Initialize(setting);
        if (result.IsFailure())
        {
            PIA_SAMPLE_ASSERT(false);
        }
    }
    // Network 設定
    {
        nn::pia::framework::StartupNetworkSetting setting;
        setting.isAutoInitializeNetworkInterface = true;
        setting.isAutoInitializeSocket = true;
#if NN_PIA_CTR
        const int32_t SocketSessionMax = 5;
        const int32_t SocketBufferSize = 64 * 1024;                                      // １つのソケットの送受信バッファの割り当てに使用するサイズ
        const uint32_t SocketBufferTotalSize = SocketBufferSize * SocketSessionMax;      // ソケットライブラリ用ワークメモリのサイズ
        PIA_ATTRIBUTE_ALIGN(4096) static uint8_t s_SocketBuffer[SocketBufferTotalSize];  // ソケットライブラリに渡すワークメモリ（4096Byteアライメントされたバッファが必要
        setting.memoryPool = s_SocketBuffer;
        setting.allocatorPoolSize = SocketBufferSize;
        setting.memoryPoolSize = sizeof(s_SocketBuffer);
#elif NN_PIA_NINTENDOSDK
        const uint32_t SocketBufferTotalSize = 6 * 1024 * 1024;                          // ソケットライブラリ用ワークメモリのサイズ(とりあえず 6MB)
        PIA_ATTRIBUTE_ALIGN(4096) static uint8_t s_SocketBuffer[SocketBufferTotalSize];  // ソケットライブラリに渡すワークメモリ（4096Byteアライメントされたバッファが必要）
        setting.memoryPool = s_SocketBuffer;
        setting.allocatorPoolSize = nn::socket::DefaultSocketAllocatorSize;
        setting.memoryPoolSize = sizeof(s_SocketBuffer);
#endif
        nn::pia::framework::Framework::GetInstance()->RegisterStartupNetworkSetting(setting);
    }
#if NN_PIA_AUTO_NEX_SETTING
    // inet 設定
    {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        const size_t TotalMemoryBlockSizeForNex = MemoryBlockSizeForNex + MemoryBlockSizeForNexEmergency;
        PIA_ATTRIBUTE_ALIGN(4096) static uint8_t s_NexMemory[TotalMemoryBlockSizeForNex];
        nn::pia::framework::InitializeInetSetting setting;
        setting.isAutoInitializeAndLogin = true;
        setting.nexMemory = s_NexMemory;          // NEX で使用するメモリ
        setting.emergencyMemorySize = MemoryBlockSizeForNexEmergency;  // ヒープ枯渇時に使用する緊急メモリのサイズ
        setting.totalMemorySize = TotalMemoryBlockSizeForNex;
        setting.gameId = NNS_PIA_LOGIN_GAME_ID;
#if NN_PIA_CTR
        setting.accessKey.Copy(NNS_PIA_LOGIN_ACCESS_KEY);
#elif NN_PIA_NINTENDOSDK
#if NN_PIA_NINTENDOSDK_WIN
        setting.hostName.Copy(NNS_PIA_DEBUGLOGIN_HOST_ADDRESS);
        setting.port = static_cast<uint16_t>(NNS_PIA_DEBUGLOGIN_HOST_PORT);
        setting.hostUri.Copy(NNS_PIA_DEBUGLOGIN_HOST_URL);
        nn::nex::StringStream name;
        name << "testuser_" << nn::nex::Platform::GetRandomNumber(10000);
        setting.userName.Copy(name.CStr());
        setting.password.Copy(NNS_PIA_DEBUGLOGIN_USER_PASSWORD);
#endif
        setting.accessKey.Copy(NNS_PIA_LOGIN_ACCESS_KEY);
        nns::pia::UserAccountInfo* accountInfo = nns::pia::Net_GetUserAccountInfo(0);
        PIA_SAMPLE_ASSERTMSG(accountInfo != NULL, "accountInfo is not found.");
        setting.userHandle = accountInfo->userHandle;
#endif
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeInetSetting(setting);
#endif
    }
#endif 
    // JoinRandomSessionSetting 設定
    {
        switch (networkType)
        {
        case nn::pia::framework::NetworkType_Local:
            // nothing to do
            break;
        case nn::pia::framework::NetworkType_Inet:
        {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
            nn::pia::framework::NexJoinRandomSessionSetting setting;
            nn::pia::inet::NexCreateSessionSetting nexCreateSessionSetting;
            nexCreateSessionSetting.SetGameMode(MatchmakeGameMode);                // ゲームモード
            nexCreateSessionSetting.SetMaxParticipantNum(MaxStationNum);  // 最大参加人数
            nexCreateSessionSetting.SetOpenSession(true);                 // 作成されてすぐに参加可能状態にします。
            nexCreateSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);

            // 検索条件の設定
            nn::pia::inet::NexSessionSearchCriteria nexSessionSearchCriteria;
            nexSessionSearchCriteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
            nexSessionSearchCriteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
            nexSessionSearchCriteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数

            setting.nexJoinRandomSessionSetting.SetCreateSessionSetting(&nexCreateSessionSetting);
            setting.nexJoinRandomSessionSetting.SetSessionSearchCriteria(&nexSessionSearchCriteria, 1);

            nn::pia::framework::Framework::GetInstance()->RegisterJoinRandomSessionSetting(setting);
#endif
        }
        break;
        case nn::pia::framework::NetworkType_Lan:
        {
            nn::pia::framework::LanJoinRandomSessionSetting setting;
            nn::pia::lan::LanCreateSessionSetting lanCreateSessionSetting;
            lanCreateSessionSetting.SetGameMode(MatchmakeGameMode);                // ゲームモード
            lanCreateSessionSetting.SetMaxParticipantNum(MaxStationNum);  // 最大参加人数
            lanCreateSessionSetting.SetOpenSession(true);                 // 作成されてすぐに参加可能状態にします。

            // 検索条件の設定
            nn::pia::lan::LanSessionSearchCriteria lanSessionSearchCriteria;
            lanSessionSearchCriteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
            lanSessionSearchCriteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
            lanSessionSearchCriteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数

            setting.lanJoinRandomSessionSetting.SetCreateSessionSetting(&lanCreateSessionSetting);
            setting.lanJoinRandomSessionSetting.SetSessionSearchCriteria(&lanSessionSearchCriteria, 1);

            nn::pia::framework::Framework::GetInstance()->RegisterJoinRandomSessionSetting(setting);
        }
        break;
        }
    }
    // InitializeSessionSetting 設定
    {
        nn::pia::framework::InitializeSessionSetting setting;
        setting.browsedSessionInfoListNum = MaxBrowsedSessionNum;
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeSessionSetting(setting);
    }
    // SessionStartupSetting 設定
    {
        nn::pia::framework::StartupSessionSetting setting;

        nns::pia::Session_GetLocalPlayerInfo(&setting.playerInfo, 1);
        setting.signatureKeyData.Copy("FrameworkSimple Key");
        setting.signatureKeyDataSize = sizeof("FrameworkSimple Key");
#if NN_PIA_NINTENDOSDK
        setting.isAutoInitializeLdn = true;
#endif
        nn::pia::framework::Framework::GetInstance()->RegisterStartupSessionSetting(setting);
    }
    // InitializeCloneSetting 設定
    {
        nn::pia::framework::InitializeCloneSetting setting;
        setting.isAutoInitialize = true;
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeCloneSetting(setting);
    }
}

void Send()
{
    switch (s_SendProtocol)
    {
    case SendProtocol_Unreliable:     // transport::UnreliableProtocol を用いた特定のステーションに対するデータの送信を行います
    {
        uint8_t sendData[] = { 7, 4, 6 }; // 送信する値
        nn::pia::Result result = nn::pia::framework::Framework::GetInstance()->SendUnreliable(ConvertStationIndexToStationId(nn::pia::StationIndex(s_CursorY)), sendData, sizeof(sendData));
        if (result.IsSuccess())
        {
            ++s_sendCount[s_CursorY];
        }
        else
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->SendUnreliable() failed\n");
        }
    }
    break;
    case SendProtocol_UnreliableAll:  // transport::UnreliableProtocol を用いたデータの送信を行います
    {
        uint8_t sendData[] = { 5, 3, 2 }; // 送信する値
        nn::pia::Result result = nn::pia::framework::Framework::GetInstance()->SendToAllUnreliable(sendData, sizeof(sendData));
        if (result.IsSuccess())
        {
            ++s_sendAllCount;
        }
        else
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->SendToAllUnreliable() failed\n");
        }
    }
    break;
    case SendProtocol_Reliable:       // transport::ReliableProtocol を用いた特定のステーションに対するデータの送信を行います
    {
        uint8_t sendData[] = { 1, 6, 9 }; // 送信する値
        nn::pia::Result result = nn::pia::framework::Framework::GetInstance()->SendReliable(ConvertStationIndexToStationId(nn::pia::StationIndex(s_CursorY)), sendData, sizeof(sendData));
        if (result.IsSuccess())
        {
            ++s_sendCount[s_CursorY];
        }
        else
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->SendReliable() failed\n");
        }
    }
    break;
    case SendProtocol_ReliableAll:    // transport::ReliableProtocol を用いたデータの送信を行います
    {
        uint8_t sendData[] = { 4, 8, 7 }; // 送信する値
        nn::pia::Result result = nn::pia::framework::Framework::GetInstance()->SendToAllReliable(sendData, sizeof(sendData));
        if (result.IsSuccess())
        {
            ++s_sendAllCount;
        }
        else
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->SendToAllReliable() failed\n");
        }
    }
    default:
        break;
    }



}

void Receive()
{
    nn::pia::Result result;
    nn::pia::StationId srcId;
    uint8_t buf[nn::pia::transport::UnreliableProtocol::MaxDataSize];  // 到着する可能性のある最大サイズで配列を確保しておきます。
    uint32_t receivedSize;

    // transport::UnreliableProtocol を用いたデータの受信を行います
    {
        result = nn::pia::framework::Framework::GetInstance()->ReceiveUnreliable(&srcId, buf, &receivedSize, sizeof(buf));
        if (result.IsSuccess())
        {
            // データを無事に受信できたケースです。
            PIA_SAMPLE_PRINT("ReceiveUnreliable : Received %d bytes from Station %llu. ", receivedSize, srcId);
            PIA_SAMPLE_PRINT("buf[0] : %d, buf[1] : %d, buf[2] : %d\n", buf[0], buf[1], buf[2]);
            ++s_receiveCount[ConvertStationIdToStationIndex(srcId)];
        }
        else
        {
            if (result != nn::pia::ResultNoData())
            {
                PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->ReceiveUnreliable() failed. Result: %s\n", result.ToString());
            }
        }
    }

    // transport::ReliableProtocol を用いたデータの受信を行います
    {
        result = nn::pia::framework::Framework::GetInstance()->ReceiveReliable(&srcId, buf, &receivedSize, sizeof(buf));
        if (result.IsSuccess())
        {
            // データを無事に受信できたケースです。
            PIA_SAMPLE_PRINT("ReceiveReliable : Received %d bytes from Station %llu. ", receivedSize, srcId);
            PIA_SAMPLE_PRINT("buf[0] : %d, buf[1] : %d, buf[2] : %d\n", buf[0], buf[1], buf[2]);
            ++s_receiveCount[ConvertStationIdToStationIndex(srcId)];
        }
        else
        {
            if (result != nn::pia::ResultNoData())
            {
                PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->ReceiveReliable() failed. Result: %s\n", result.ToString());
            }
        }
    }
}

void ClearSendReceiveCount()
{ 
    s_CursorY = 0;
    s_sendAllCount = 0;
    for (int i = 0; i < MaxStationNum; ++i)
    {
        s_sendCount[i] = 0;
        s_receiveCount[i] = 0;
    }
}

#if !NN_PIA_AUTO_NEX_SETTING
void Login(State* pState)
{
#if NN_PIA_NINTENDOSDK
    nn::Result loginError = nn::ResultSuccess();
#else
    int32_t loginError = 0;
#endif
    if (!nns::pia::Nex_Login(&loginError))
    {
#if NN_PIA_NINTENDOSDK
        if (loginError.IsFailure())
        {
            PIA_SAMPLE_PRINT("Login failed. code: %08d\n", loginError.GetInnerValueForDebug());
            PIA_SAMPLE_ASSERT(false);
        }
#else
        if (loginError != 0)
        {
            PIA_SAMPLE_PRINT("Login failed. code: %08d\n", loginError);
            PIA_SAMPLE_ASSERT(false);
        }
#endif        
        PIA_SAMPLE_ASSERT(false);
        *pState = State_FinalizeFramework;
        return;
    }
}

void Logout()
{
    bool result = nns::pia::Nex_Logout();
    if (result == true)
    {
        PIA_SAMPLE_PRINT("Nex_Logout:Logout succeeded\n");
    }
    else
    {
        PIA_SAMPLE_PRINT("Nex_Logout:Logout failed\n");
    }
}

void BindLoginInfo()
{
    nn::pia::inet::NexFacade::LoginInfo loginInfo;
    nn::pia::common::String accessKey(NNS_PIA_LOGIN_ACCESS_KEY);
    loginInfo.gameServerId = NNS_PIA_LOGIN_GAME_ID;
    loginInfo.pAccessKey = &accessKey;
    loginInfo.pNgsFacade = nns::pia::Nex_GetNgsFacadeInstance();
   
    nn::pia::inet::NexFacade* pNexFacade = nn::pia::inet::NexFacade::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pNexFacade));
    nn::pia::Result result = pNexFacade->Bind(&loginInfo);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
}

void UnbindLoginInfo()
{
    nn::pia::inet::NexFacade* pNexFacade = nn::pia::inet::NexFacade::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pNexFacade));
    pNexFacade->Unbind();
}
#endif

void SceneTitle(State* pState, nn::pia::framework::NetworkType* pNetworkType, bool* pIsLatencyEmulationEnabled, bool* pIsPacketLossEmulationEnabled)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();

    bool isExit = false;

    while (!isExit)
    {
        pSampleFramework->BeginFrame();
        pSampleFramework->DrawString(10, 20, "state : State_Title");
#if NN_PIA_BROWSE_ONLY
        pSampleFramework->DrawString(10, 40, "A : [Inet]State_BrowseSession");
        pSampleFramework->DrawString(10, 50, "Y : [Lan]State_BrowseSession");
#else
        pSampleFramework->DrawString(10, 40, "A : [Inet]State_RandomSession");
        pSampleFramework->DrawString(10, 50, "Y : [Lan]State_RandomSession");
#endif
        pSampleFramework->DrawString(10, 60, "X : [Local]State_CreateOrJoinSession");
        if (*pIsLatencyEmulationEnabled)
        {
            pSampleFramework->DrawString(10, 80, "L : Latency Emulation [ on ]");
        }
        else
        {
            pSampleFramework->DrawString(10, 80, "L : Latency Emulation [ off ]");
        }
        if (*pIsPacketLossEmulationEnabled)
        {
            pSampleFramework->DrawString(10, 90, "R : Packet Loss Emulation [ on ]");
        }
        else
        {
            pSampleFramework->DrawString(10, 90, "R : Packet Loss Emulation [ off ]");
        }
        pSampleFramework->DrawString(10, 110, "START : End");
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            *pState = State_InitializeFramework;
            *pNetworkType = nn::pia::framework::NetworkType_Inet;
#if NN_PIA_BROWSE_ONLY
            pSampleFramework->DrawString(10, 130, "[Inet]State_BrowseSession...");
#else
            pSampleFramework->DrawString(10, 130, "[Inet]State_RandomSession...");
#endif
            isExit = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            *pState = State_InitializeFramework;
            *pNetworkType = nn::pia::framework::NetworkType_Lan;
#if NN_PIA_BROWSE_ONLY
            pSampleFramework->DrawString(10, 130, "[Lan]State_BrowseSession...");
#else
            pSampleFramework->DrawString(10, 130, "[Lan]State_RandomSession...");
#endif            
            isExit = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            *pState = State_InitializeFramework;
            *pNetworkType = nn::pia::framework::NetworkType_Local;
            pSampleFramework->DrawString(10, 130, "[Local]State_CreateOrJoinSession...");
            isExit = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
        {
            *pIsLatencyEmulationEnabled = !(*pIsLatencyEmulationEnabled);
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            *pIsPacketLossEmulationEnabled = !(*pIsPacketLossEmulationEnabled);
        }
        pSampleFramework->EndFrame();
    }
}

void SceneInitializeFramework(State* pState, nn::pia::framework::NetworkType* pNetworkType, bool* pIsLatencyEmulationEnabled, bool* pIsPacketLossEmulationEnabled)
{
    PIA_SAMPLE_PRINT("---------------SceneInitializeFramework start----------------\n");
    nn::pia::Result result;

    // アカウントライブラリの初期化、アカウント選択
#if NN_PIA_NINTENDOSDK
    nns::pia::NetworkType nnsNetworkType = nns::pia::NetworkType_Internet;
    switch (*pNetworkType)
    {
    case nn::pia::framework::NetworkType_Inet:
        nnsNetworkType = nns::pia::NetworkType_Internet;
        break;
    case nn::pia::framework::NetworkType_Lan:
        nnsNetworkType = nns::pia::NetworkType_Lan;
        break;
    case nn::pia::framework::NetworkType_Local:
        nnsNetworkType = nns::pia::NetworkType_Local;
        break;
    default:
        PIA_SAMPLE_ASSERT(false);
        break;
    }

    nns::pia::NetResult netResult = nns::pia::Net_InitializeAccount(nnsNetworkType);
    if (netResult != nns::pia::NetResult_Success)
    {
        *pState = State_FinalizeFramework;
        return;
    }
#endif
    // 各 setting の初期化
    InitializeSetting(*pNetworkType);

#if NN_PIA_AUTO_NEX_SETTING
    // State_PiaStartedUp まで進める
    result = ChangeState(pState, nn::pia::framework::Framework::State_SessionStartedUp);
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        return;
    }
#else

    // NEX を初期化する
    result = nns::pia::Nex_InitializeNex(
        MemoryBlockSizeForNex,
        MemoryBlockSizeForNexEmergency);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // State_NetworkStartedUp まで進める
    result = ChangeState(pState, nn::pia::framework::Framework::State_NetworkStartedUp);
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(false);
    }

    nns::pia::Net_InitializeFriends();

    // ゲームサーバにログインします。
    Login(pState);

    // State_SessionInitialized まで進める
    result = ChangeState(pState, nn::pia::framework::Framework::State_SessionInitialized);
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(false);
    }

    // nn::pia::inet::NexFacade に NEX ゲームサーバ情報を設定
    // NexFacade のインスタンスは State_LoggedIn から State_SessionInitialized に遷移するときに生成される
    BindLoginInfo();

    // State_SessionStartedUp まで進める
    result = ChangeState(pState, nn::pia::framework::Framework::State_SessionStartedUp);
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(false);
    }
#endif
    // パケロス・レイテンシエミュレーションの設定
    nn::pia::framework::SetEmulationParamArg arg;
    if (*pIsLatencyEmulationEnabled)
    {
        arg.sendThreadLatencyEmulationMin = 0;
        arg.sendThreadLatencyEmulationMax = 500;
        arg.receiveThreadLatencyEmulationMin = 0;
        arg.receiveThreadLatencyEmulationMax = 500;
    }
    if (*pIsPacketLossEmulationEnabled)
    {
        arg.sendThreadPacketLossRatio = 10;
        arg.receiveThreadPacketLossRatio = 10;
    }
    nn::pia::framework::Framework::GetInstance()->SetEmulationParam(arg);

#if NN_PIA_FRAMEWORK_ENABLE_NEX
    if (*pNetworkType == nn::pia::framework::NetworkType_Inet)
    {
        nn::pia::framework::NexService* pNexService = static_cast<nn::pia::framework::NexService*>(nn::pia::framework::Framework::GetInstance()->GetInetService());
        NN_PIA_UNUSED_IN_RELEASE(pNexService);
        PIA_SAMPLE_PRINT("nex::Credentials:0x%08x\n", pNexService->GetCredential());
        PIA_SAMPLE_PRINT("nex::NgsFacade:0x%08x\n", pNexService->GetNgsFacade());
    }
#endif

    if (*pNetworkType == nn::pia::framework::NetworkType_Local)
    {
        *pState = State_CreateOrJoinSession;
    }
    else
    {
#if NN_PIA_BROWSE_ONLY
        *pState = State_CreateOrJoinSession;
#else
        *pState = State_RandomSession;
#endif
    }

    PIA_SAMPLE_PRINT("---------------SceneInitializeFramework end----------------\n");
}

void SceneRandomSession(State* pState)
{
    PIA_SAMPLE_PRINT("----------------SceneRandomSession start---------------\n");
    nn::pia::Result result;
    
    ClearSendReceiveCount();
    // 通信の進行状態を State_SessionJoined に遷移すると自動で RandomSession が行われる
    result = ChangeState(pState, nn::pia::framework::Framework::State_SessionJoined);
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        return;
    }
    *pState = State_Session;
    PIA_SAMPLE_PRINT("----------------SceneRandomSession end---------------\n");
}

#if NN_PIA_NINTENDOSDK || NN_PIA_CTR
void SceneCreateOrJoinSession(State* pState, nn::pia::framework::NetworkType* pNetworkType)
{
    PIA_SAMPLE_PRINT("----------------SceneCreateOrJoinSession start---------------\n");
    nn::pia::Result result;

    ClearSendReceiveCount();
    // まずはセッションを探索する
    switch (*pNetworkType)
    {
    case nn::pia::framework::NetworkType_Local:
#if NN_PIA_NINTENDOSDK
    {
        nn::pia::local::LdnSessionSearchCriteria criteria;
        {
            criteria.SetLocalCommunicationId(nns::pia::LocalCommunicationId);  // ローカル通信ID
            criteria.SetSceneId(LocalSceneId);                            // 通信モード識別用 ID です
            criteria.SetMaxParticipantNum(MaxStationNum);            // 最大参加人数
            criteria.SetResultRange(0, MaxBrowsedSessionNum);        // 検索結果取得数
        }
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
    }
#elif NN_PIA_CTR
    {
        nn::pia::local::UdsSessionSearchCriteria criteria;
        {
            criteria.SetLocalCommunicationId(nn::pia::framework::Framework::GetInstance()->CreateLocalCommunicationId(LocalTitleId));  // ローカル通信ID
            criteria.SetSubId(LocalSubId);                                // 通信モード識別用 ID です
            criteria.SetMaxParticipantNum(MaxStationNum);            // 最大参加人数
            criteria.SetResultRange(0, MaxBrowsedSessionNum);        // 検索結果取得数
        }
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
    }
#endif
    break;
    case nn::pia::framework::NetworkType_Inet:
    {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        nn::pia::inet::NexSessionSearchCriteria criteria;
        criteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
        criteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
        criteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
#endif
    }
    break;
    case nn::pia::framework::NetworkType_Lan:
    {
        nn::pia::lan::LanSessionSearchCriteria criteria;
        criteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
        criteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
        criteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
    }
    break;
    }

    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync() failed. line: %d\n", __LINE__);
        return;
    }

    // 非同期処理 BrowseSessionAsync の終了を待つ
    WaitForAsyncProcess("BrowseSessionAsync", &nn::pia::framework::Framework::IsBrowseSessionCompleted, *pState);

    result = nn::pia::framework::Framework::GetInstance()->GetBrowseSessionResult();
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->GetBrowseSessionResult() failed. line: %d\n", __LINE__);
        return;
    }

    nn::pia::session::ISessionInfoList* pSessionInfoList = nn::pia::framework::Framework::GetInstance()->GetBrowsedSessionInfoList();
    if (pSessionInfoList != NULL && pSessionInfoList->GetSize() > 0)
    {
        PIA_SAMPLE_PRINT("Join session\n");
        // セッションが見つかった場合は、見つけたセッションに参加する
        switch (*pNetworkType)
        {
        case nn::pia::framework::NetworkType_Local:
        {
#if NN_PIA_NINTENDOSDK
            nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
#elif NN_PIA_CTR
            nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#endif
            joinSessionSetting.SetWirelessCryptoKey(PassPhrase, PassPhraseSize);
            joinSessionSetting.SetSessionInfoPtr(*(pSessionInfoList->Begin()));
            joinSessionSetting.SetMaxParticipantNum(MaxStationNum);
            result = nn::pia::framework::Framework::GetInstance()->JoinSessionAsync(&joinSessionSetting);
        }
        break;
        case nn::pia::framework::NetworkType_Inet:
        {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
            nn::pia::inet::NexJoinSessionSetting joinSessionSetting;
            joinSessionSetting.SetSessionInfoPtr(*(pSessionInfoList->Begin()));
            joinSessionSetting.SetMaxParticipantNum(MaxStationNum);
            result = nn::pia::framework::Framework::GetInstance()->JoinSessionAsync(&joinSessionSetting);
#endif
        }
        break;
        case nn::pia::framework::NetworkType_Lan:
        {
            nn::pia::lan::LanJoinSessionSetting joinSessionSetting;
            joinSessionSetting.SetSessionInfoPtr(*(pSessionInfoList->Begin()));
            joinSessionSetting.SetMaxParticipantNum(MaxStationNum);
            result = nn::pia::framework::Framework::GetInstance()->JoinSessionAsync(&joinSessionSetting);
        }
        break;
        }

        ResultCheck(result, pState);
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->JoinSessionAsync() failed. line: %d\n", __LINE__);
            return;
        }

        // JoinSessionAsync の終了を待つ
        WaitForAsyncProcess("JoinSessionAsync", &nn::pia::framework::Framework::IsJoinSessionCompleted, *pState);
        result = nn::pia::framework::Framework::GetInstance()->GetJoinSessionResult();
        ResultCheck(result, pState);
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->GetJoinSessionResult() failed. line: %d\n", __LINE__);
            return;
        }
    }
    else
    {
        PIA_SAMPLE_PRINT("Create session\n");
        // セッションが見つからなかった場合は、セッションを作成する
        switch (*pNetworkType)
        {
        case nn::pia::framework::NetworkType_Local:
        {
#if NN_PIA_NINTENDOSDK
            nn::pia::local::LdnCreateSessionSetting createSessionSetting;
            {
                createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                createSessionSetting.SetSceneId(LocalSceneId);
                createSessionSetting.SetWirelessCryptoKey(PassPhrase, PassPhraseSize);
                createSessionSetting.SetMaxParticipantNum(MaxStationNum);
            }
#elif NN_PIA_CTR
            nn::pia::local::UdsCreateSessionSetting createSessionSetting;
            {
                createSessionSetting.SetLocalCommunicationId(nn::pia::framework::Framework::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                createSessionSetting.SetSubId(LocalSubId);
                createSessionSetting.SetWirelessCryptoKey(PassPhrase, PassPhraseSize);
                createSessionSetting.SetMaxParticipantNum(MaxStationNum);
            }
#endif
            result = nn::pia::framework::Framework::GetInstance()->CreateSessionAsync(&createSessionSetting);
        }
        break;
        case nn::pia::framework::NetworkType_Inet:
        {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
            nn::pia::inet::NexCreateSessionSetting createSessionSetting;
            createSessionSetting.SetGameMode(MatchmakeGameMode);
            createSessionSetting.SetMaxParticipantNum(MaxStationNum);
            createSessionSetting.SetOpenSession(true);
            result = nn::pia::framework::Framework::GetInstance()->CreateSessionAsync(&createSessionSetting);
#endif
        }
        break;
        case nn::pia::framework::NetworkType_Lan:
        {
            nn::pia::lan::LanCreateSessionSetting createSessionSetting;
            createSessionSetting.SetGameMode(MatchmakeGameMode);
            createSessionSetting.SetMaxParticipantNum(MaxStationNum);
            createSessionSetting.SetOpenSession(true);
            result = nn::pia::framework::Framework::GetInstance()->CreateSessionAsync(&createSessionSetting);
        }
        break;
        }

        ResultCheck(result, pState);
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->CreateSessionAsync() failed. line: %d\n", __LINE__);
            return;
        }

        // CreateSessionAsync の終了を待つ
        WaitForAsyncProcess("CreateSessionAsync", &nn::pia::framework::Framework::IsCreateSessionCompleted, *pState);

        result = nn::pia::framework::Framework::GetInstance()->GetCreateSessionResult();
        ResultCheck(result, pState);
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->GetCreateSessionResult() failed. line: %d\n", __LINE__);
            return;
        }
    }
    *pState = State_Session;
    PIA_SAMPLE_PRINT("----------------SceneCreateOrJoinSession end---------------\n");
}
#endif

void SceneSession(State* pState, nn::pia::framework::NetworkType* pNetworkType, int* pErrorHandlingCase)
{
    nn::pia::Result result;
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();

    bool isExit = false;

    while (!isExit)
    {
        pSampleFramework->BeginFrame();
        result = nn::pia::framework::Framework::GetInstance()->Update();

        if (result.IsFailure())
        {
            pSampleFramework->EndFrame();
            ResultCheck(result, pState);
            return;
        }

        // セッション参加時の処理
        pSampleFramework->DrawString(10, 20, "state : State_Session");
        pSampleFramework->DrawString(10, 50, "B : Leave");
        pSampleFramework->DrawString(10, 60, "A : Send");
        pSampleFramework->DrawString(10, 70, "Up/Down : Change sta. to send");
        {
            pSampleFramework->DrawString(10, 90, "X : Change send protocol");
            pSampleFramework->DrawString(22, 100, "[                    ]");
            int posX = 27;
            int posY = 100;
            switch (s_SendProtocol)
            {
            case SendProtocol_Unreliable:
                pSampleFramework->DrawString(posX, posY, "Unreliable");
                break;
            case SendProtocol_UnreliableAll:
                pSampleFramework->DrawString(posX, posY, "Unreliable All");
                break;
            case SendProtocol_Reliable:
                pSampleFramework->DrawString(posX, posY, "Reliable");
                break;
            case SendProtocol_ReliableAll:
                pSampleFramework->DrawString(posX, posY, "Reliable All");
                break;
            default:
                break;
            }
        }

        // transport::UnreliableProtocol を用いたデータの受信を行います
        Receive();

        {
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                pSampleFramework->DrawString(10, 120, "I'm host.");
            }
            else
            {
                pSampleFramework->DrawString(10, 120, "I'm client.");
            }
            nn::pia::session::Session* session = nn::pia::session::Session::GetInstance();
            pSampleFramework->DrawString(10, 130, "station num : %u(MyID : %d)", session->GetStationNum(), ConvertStationIdToStationIndex(session->GetLocalStationId()));
            pSampleFramework->DrawString(10, 140, "hostId : %u", session->GetHostStationId());
            pSampleFramework->DrawString(10, 150, "localId : %u", session->GetLocalStationId());
#if NN_PIA_FRAMEWORK_ENABLE_NEX
            if (*pNetworkType == nn::pia::framework::NetworkType_Inet)
            {
                pSampleFramework->DrawString(10, 160, "GetLocalURLs : %d", nn::nex::Network::GetInstance()->GetLocalURLs().size());
            }
#endif
#if NN_PIA_AUTO_NEX_SETTING
            pSampleFramework->DrawString(10, 190, "errorHandlingCase(L, R, Y) : %d", *pErrorHandlingCase);
#endif

            int32_t offset = 0;
            pSampleFramework->DrawString(260, offset + 20, "sendAllCount : %d", s_sendAllCount);
            pSampleFramework->DrawString(260, offset + 30, "---------------------");
            for (int i = 0; i < MaxStationNum; ++i)
            {
                // カーソルの初期位置が自分自身にならないよう調整
                if (static_cast<int>(nn::pia::session::Session::GetInstance()->GetLocalStationIndex()) == s_CursorY)
                {
                    s_CursorY = (s_CursorY + 1) % MaxStationNum;
                }
                else if (static_cast<int>(nn::pia::session::Session::GetInstance()->GetLocalStationIndex() == nn::pia::StationIndex_Invalid))
                {
                    break;
                }

                if (i == s_CursorY)
                {
                    pSampleFramework->SetFontColor(nns::pia::sampleframework::Color(1.0f, 0.3f, 0.3f, 1.0f));
                }
                else if (i == static_cast<int>(nn::pia::session::Session::GetInstance()->GetLocalStationIndex()))
                {
                    pSampleFramework->SetFontColor(nns::pia::sampleframework::Color(0.2f, 0.2f, 0.2f, 1.0f));
                }
                pSampleFramework->DrawString(260, offset + 40 + (i * 25), "sendCount[%d] : %d", i, s_sendCount[i]);
                pSampleFramework->DrawString(260, offset + 50 + (i * 25), "recvCount[%d] : %d", i, s_receiveCount[i]);

                pSampleFramework->SetFontColor(nns::pia::sampleframework::Color(1.0f, 1.0f, 1.0f, 1.0f));
            }
        }
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
        {
            (*pErrorHandlingCase)--;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            (*pErrorHandlingCase)++;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            Send();
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            s_CursorY = (s_CursorY + 1) % MaxStationNum;
            if (static_cast<int>(nn::pia::session::Session::GetInstance()->GetLocalStationIndex()) == s_CursorY)
            {
                s_CursorY = (s_CursorY + 1) % MaxStationNum;
            }
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            s_CursorY = (s_CursorY + MaxStationNum - 1) % MaxStationNum;
            if (static_cast<int>(nn::pia::session::Session::GetInstance()->GetLocalStationIndex()) == s_CursorY)
            {
                s_CursorY = (s_CursorY + MaxStationNum - 1) % MaxStationNum;
            }
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            switch (s_SendProtocol)
            {
            case SendProtocol_Unreliable:
                s_SendProtocol = SendProtocol_UnreliableAll;
                break;
            case SendProtocol_UnreliableAll:
                s_SendProtocol = SendProtocol_Reliable;
                break;
            case SendProtocol_Reliable:
                s_SendProtocol = SendProtocol_ReliableAll;
                break;
            case SendProtocol_ReliableAll:
                s_SendProtocol = SendProtocol_Unreliable;
                break;
            default:
                break;
            }
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            pSampleFramework->DrawString(10, 60, "State_Leave...");
            *pState = State_Leave;
            isExit = true;
        }
#if NN_PIA_AUTO_NEX_SETTING
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            // Yボタンで HandleErrorAsync を用いた任意のエラーに対するエラーハンドリングをテストします
            switch (*pErrorHandlingCase)
            {
            case 0:
                // HandlingType_Cleanup
                ResultCheck(nn::pia::ResultCancelled(), pState);
                break;
            case 1:
                // HandlingType_CleanupWithLeave
                ResultCheck(nn::pia::ResultSessionConnectionIsLost(), pState);
                break;
            case 2:
                // HandlingType_LogoutWithLeave
                ResultCheck(nn::pia::ResultGameServerMaintenance(), pState);
                break;
            case 3:
                // HandlingType_ShutdownNetwork
                ResultCheck(nn::pia::ResultNetworkConnectionIsLost(), pState);
                break;
            case 4:
                // HandlingType_Finalize
                ResultCheck(nn::pia::ResultNexInternalError(), pState);
                break;
            case 5:
                // success -> State_LoggedIn
                result = ChangeState(pState, nn::pia::framework::Framework::State_LoggedIn);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                *pState = State_Leave;
                break;
            case 6:
                // HandlingType_LogoutWithLeave -> State_LoggedIn
                ResultCheck(nn::pia::ResultGameServerMaintenance(), pState, nn::pia::framework::Framework::State_LoggedIn);
                break;
            case 7:
                // HandlingType_Cleanup -> State_LoggedIn
                ResultCheck(nn::pia::ResultCancelled(), pState, nn::pia::framework::Framework::State_LoggedIn);
                break;
            case 8:
#if NN_PIA_CTR
                nn::math::RandomGenerator random;
                switch (random.Generate(6))
                {
                case 0:
                    result = nn::pia::ResultCancelled();
                    break;
                case 1:
                    result = nn::pia::ResultSessionConnectionIsLost();
                    break;
                case 2:
                    result = nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin();
                    break;
                case 3:
                    result = nn::pia::ResultGameServerMaintenance();
                    break;
                case 4:
                    result = nn::pia::ResultNetworkConnectionIsLost();
                    break;
                case 5:
                    result = nn::pia::ResultNexInternalError();
                    break;
                default:
                    result = nn::pia::ResultSuccess();
                    break;
                }
                nn::pia::framework::Framework::State transitionState = static_cast<nn::pia::framework::Framework::State>(random.Generate(8));
                PIA_SAMPLE_PRINT("result : %d, state : %d -> %d", result.GetHandlingType(), nn::pia::framework::Framework::GetInstance()->GetState(), transitionState);
                ResultCheck(result, pState, transitionState);
#endif
                break;
            }
            isExit = true;
        }
#endif
        result = nn::pia::framework::Framework::GetInstance()->Update();

        pSampleFramework->EndFrame();

        if (result.IsFailure())
        {
            ResultCheck(result, pState);
            return;
        }
    }
}

void SceneLeave(State* pState)
{
    PIA_SAMPLE_PRINT("---------------SceneLeave start----------------\n");
    nn::pia::Result result;

    // セッションから離脱します
    result = nn::pia::framework::Framework::GetInstance()->ChangeStateAsync(nn::pia::framework::Framework::State_SessionLeft);
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        return;
    }

    WaitForAsyncProcess("ChangeStateAsync", &nn::pia::framework::Framework::IsChangeStateCompleted, *pState);

    result = nn::pia::framework::Framework::GetInstance()->GetChangeStateResult();
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        return;
    }

    *pState = State_FinalizeFramework;
    PIA_SAMPLE_PRINT("---------------SceneLeave end----------------\n");
}

void SceneFinalizeFramework(State* pState)
{
    PIA_SAMPLE_PRINT("---------------SceneFinalizeFrameowork start----------------\n");
#if !NN_PIA_AUTO_NEX_SETTING
    nn::pia::Result result;

    // State_SessionCleanedUp まで後退する
    result = ChangeState(pState, nn::pia::framework::Framework::State_SessionCleanedUp);
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(false);
    }

    // NexFacade に登録した NEX ゲームサーバ情報を設定解除
    UnbindLoginInfo();

    // State_SessionFinalized まで後退する
    result = ChangeState(pState, nn::pia::framework::Framework::State_SessionFinalized);
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(false);
    }

    // ゲームサーバからログアウトします。
    Logout();

    nns::pia::Net_FinalizeFriends();

    // State_PiaFinalized まで後退する
    result = ChangeState(pState, nn::pia::framework::Framework::State_LoggedOut);
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(false);
    }

    nns::pia::Inet_FinalizePiaInet();

    // NEX の終了処理を行う
    nns::pia::Nex_FinalizeNex();

    // State_NetworkCleanedUp まで後退する
    result = ChangeState(pState, nn::pia::framework::Framework::State_NetworkCleanedUp);
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(false);
    }
#endif
    nn::pia::framework::Finalize();
#if NN_PIA_NINTENDOSDK
    nns::pia::Net_FinalizeAccount();
#endif

    *pState = State_Title;
    PIA_SAMPLE_PRINT("---------------SceneFinalizeFrameowork end----------------\n");
}

// エントリポイント。
PIA_SAMPLE_MAIN_FUNC()
{
    nn::pia::framework::NetworkType networkType = nn::pia::framework::NetworkType_Inet;

    int errorHandlingCase = 0;
    nn::pia::Result result;
    bool isLatencyEmulationEnabled = false;
    bool isPacketLossEmulationEnabled = false;

    nns::pia::sampleframework::SampleFramework* pSampleFramework = NULL;
    nns::pia::sampleframework::SampleFramework::Setting setting;
    setting.isUpperScreenOnly = true;
    nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
    pSampleFramework = GetSampleFramework();
    pSampleFramework->SetFontColor(nns::pia::sampleframework::ColorWhite);
    pSampleFramework->SetFramePerSecond(60);

    State state = State_Title;

    while (true)
    {
        switch (state)
        {
        case State_Title:
        {
            SceneTitle(&state, &networkType, &isLatencyEmulationEnabled, &isPacketLossEmulationEnabled);
        }
        break;
        case State_InitializeFramework:
        {
            SceneInitializeFramework(&state, &networkType, &isLatencyEmulationEnabled, &isPacketLossEmulationEnabled);
        }
        break;
        case State_RandomSession:
        {
            SceneRandomSession(&state);
        }
        break;
        case State_CreateOrJoinSession:
#if NN_PIA_NINTENDOSDK || NN_PIA_CTR
        {
            SceneCreateOrJoinSession(&state, &networkType);
        }
#endif
        break;
        case State_Session:
        {
            SceneSession(&state, &networkType, &errorHandlingCase);
        }
        break;
        case State_Leave:
        {
            SceneLeave(&state);
        }
        break;
        case State_FinalizeFramework:
        {
            SceneFinalizeFramework(&state);
        }
        break;
        default:
            break;
        }
    }
}
