﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "Object.h"
#include <nn/pia/session.h>
#include "SampleLib/Clone_Setup.h"
#include <nn/pia/reckoning/reckoning_Simple3dReckoningStrategy.h>
#include <math.h>

namespace
{

const float VelMax = 6.f;
const float DefAcc = 0.5f;


const float DecreaseVel = 0.15f;

const float SquareSize = 30.f;
const float DisplayWidth = 400.f - SquareSize;
const float DisplayHeight = 240.f - SquareSize;


EventStatusArg s_EventStatusArg;


int32_t s_EventStatusIndex;

enum EventStatusType
{
    EventStatusType_Unreliable,
    EventStatusType_Latency,
    EventStatusType_ChaseType,
    EventStatusType_ChaseRatio,
    EventStatusType_VelScale,
    EventStatusType_StopThreshold,
    EventStatusType_ReckoningThreshold,

#if RECKONING_DELAY_TEST_ON
    EventStatusType_DelayClock,
#endif
    EventStatusType_Max
};

/// 位置の補間方法
enum EventChaseType
{
    EventChaseType_None,   //< 補完しない
    EventChaseType_Chase,  //< 割合で追従
    EventChaseType_Max
};


}  // end of anonymous namespace

// シリアル化のために SerializePolicy の各関数を定義します。
namespace nn
{
namespace pia
{
namespace clone
{

template<>
void SerializePolicy<Status>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    SerializePolicy<float>::Serialize(offset, value.x);
    offset += 4;
    SerializePolicy<float>::Serialize(offset, value.y);
    offset += 4;
}

template<>
void SerializePolicy<Status>::Deserialize(Type* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    SerializePolicy<float>::Deserialize(&pValue->x, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->y, offset);
    offset += 4;
}

template<>
uint32_t SerializePolicy<Status>::GetSerializedSize()
{
    return 8;
}

template<>
void SerializePolicy<EventStatusArg>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    SerializePolicy<uint8_t>::Serialize(offset, value.isUnreliable);
    offset += 1;
    SerializePolicy<int32_t>::Serialize(offset, value.latency);
    offset += 4;
    SerializePolicy<int32_t>::Serialize(offset, value.chaseType);
    offset += 4;
    SerializePolicy<float>::Serialize(offset, value.chaseRatio);
    offset += 4;
    SerializePolicy<float>::Serialize(offset, value.velScale);
    offset += 4;
    SerializePolicy<float>::Serialize(offset, value.stopThreshold);
    offset += 4;
    SerializePolicy<float>::Serialize(offset, value.reckoningThreshold);
    offset += 4;
#if RECKONING_DELAY_TEST_ON
    SerializePolicy<float>::Serialize(offset, value.delayClock);
    offset += 4;
#endif
}

template<>
void SerializePolicy<EventStatusArg>::Deserialize(EventStatusArg* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    SerializePolicy<uint8_t>::Deserialize(reinterpret_cast<uint8_t*>(&pValue->isUnreliable), offset);
    offset += 1;
    SerializePolicy<int32_t>::Deserialize(&pValue->latency, offset);
    offset += 4;
    SerializePolicy<int32_t>::Deserialize(&pValue->chaseType, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->chaseRatio, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->velScale, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->stopThreshold, offset);
    offset += 4;
    SerializePolicy<float>::Deserialize(&pValue->reckoningThreshold, offset);
    offset += 4;
#if RECKONING_DELAY_TEST_ON
    SerializePolicy<float>::Deserialize(&pValue->delayClock, offset);
    offset += 4;
#endif
}

template<>
uint32_t SerializePolicy<EventStatusArg>::GetSerializedSize()
{
#if RECKONING_DELAY_TEST_ON
    return 9 + 4 * 5;
#else
    return 9 + 4 * 4;
#endif
}

}
}
}  // end of namespace nn::pia::clone


// クランプ。orgValue を minValue 以上、maxValue 以下の値に収める。
template <typename T>
static T Clamp(T orgValue, T minValue, T maxValue)
{
    if (orgValue < minValue)
    {
        return minValue;
    }
    if (maxValue < orgValue)
    {
        return maxValue;
    }
    return orgValue;
}

static float GetMaxValue()
{

    return VelMax * s_EventStatusArg.velScale;
}

static float GetDefAcc()
{

    return DefAcc * s_EventStatusArg.velScale;
}

static float GetDecreaseVel()
{

    return (1.0f + DecreaseVel * s_EventStatusArg.velScale * s_EventStatusArg.velScale);
}

/// sead からコピーしてきたもの
static bool ChaseUtl(float* current, float target, const float step)
{

    //Vectorの実装と似たdiffを計算するような実装だとコードは短くなるのですが、関数を一回通るときの計算や比較が多くなるので、このような実装にしています。
    const float before = *current;
    float after;
    if (before < target)
    {
        // 正の方向に近づく
        after = before + step;
        if (target <= after ||  // targetに到達した場合
            after < before)     // afterの計算結果がオーバーフローした場合
        {
            *current = target;
            return true;
        }
        else
        {
            *current = after;
            return false;
        }
    }
    else if (before > target)
    {
        // 負の方向に近づく
        after = before - step;
        if (after <= target ||  // targetに到達した場合
            before < after)     // afterの計算結果がアンダーフローした場合
        {
            *current = target;
            return true;
        }
        else
        {
            *current = after;
            return false;
        }
    }
    else
    {
        // currentとtargetは初めから等しい
        return true;
    }
}

static bool VecChaseDecrease(nn::pia::reckoning::Vector3f* pVec,
                             const nn::pia::reckoning::Vector3f& rTarget,
                             float ratio,
                             float min,
                             float max /* = std::numeric_limits<float>::max()*/)
{

    PIA_ASSERT(0.0f <= ratio);
    PIA_ASSERT(1.0f >= ratio);
    PIA_ASSERT(min >= 0);
    PIA_ASSERT(max >= 0);
    PIA_ASSERT(min <= max);

    const float stepX = fabs(pVec->x - rTarget.x) * ratio;  //sqrt( pVec->SquaredDistance( rTarget )  ) * ratio;

    const float stepY = fabs(pVec->y - rTarget.y) * ratio;

    const float stepZ = fabs(pVec->z - rTarget.z) * ratio;

    bool blX = ChaseUtl(&(pVec->x), rTarget.x, Clamp(stepX, min, max));
    bool blY = ChaseUtl(&(pVec->y), rTarget.y, Clamp(stepY, min, max));
    bool blZ = ChaseUtl(&(pVec->z), rTarget.z, Clamp(stepZ, min, max));
    return blX && blY && blZ;
}


// スムーズに動くようにStrategyの初期値を調整してあります
// 10.fにするとカクカクする様子が分かります
//nn::pia::reckoning::Simple3dReckoningStrategy Object::s_ReckoningStrategy(1.f);

nn::pia::reckoning::Simple3dReckoningStrategy Object::s_ReckoningStrategy(1.f);


////////////////////////////////////////////////////////////////////


Operation Object::s_Operation;

void Operation::Update(bool isHold)
{
    nn::pia::reckoning::Vector3f::Mul(&acc, GetDefAcc());
    nn::pia::reckoning::Vector3f::Add(&vel, acc);
    nn::pia::reckoning::Vector3f::Add(&pos, vel);

    if (!isHold)
    {
        acc.MakeZero();
        vel /= GetDecreaseVel();
    }
}

void Operation::ClampValues()
{
    vel.x = Clamp(vel.x, -GetMaxValue(), GetMaxValue());
    vel.y = Clamp(vel.y, -GetMaxValue(), GetMaxValue());
    pos.x = Clamp(pos.x, 0.f, DisplayWidth);
    pos.y = Clamp(pos.y, 0.f, DisplayHeight);
}

bool Operation::IsStop()
{

    float distance = pos.SquaredDistance(oldPos);
    float thresholdStop = m_ThresholdStop * m_ThresholdStop;

    return (distance <= thresholdStop);
}

void Operation::SetThresholdStop(float th)
{
    if (th < 0.f)
    {
        m_ThresholdStop = 0.f;
    }
    else
    {
        m_ThresholdStop = th;
    }
}

void Object::RegisterElementCore(nn::pia::clone::CloneBase* pClone)
{
    m_ElementR.SetReckoningStrategy(&s_ReckoningStrategy);
    pClone->RegisterElement(&m_ElementR, 0x01);
    pClone->RegisterElement(&m_ElementU, 0x02);


    s_EventStatusArg.isUnreliable = false;
    s_EventStatusArg.latency = 400;
    s_EventStatusArg.chaseType = EventChaseType_Chase;
    s_EventStatusArg.chaseRatio = 0.06f;
    s_EventStatusArg.velScale = 1.0f;
    s_EventStatusArg.stopThreshold = 0.1f;
    s_EventStatusArg.reckoningThreshold = 1.0f;

#if RECKONING_DELAY_TEST_ON
    s_EventStatusArg.delayClock = 0.f;
#endif
    s_EventStatusIndex = 0;
}

bool Object::CleanupCore(nn::pia::clone::CloneBase* pClone)
{
    if (pClone->IsRegisteredWithProtocol())
    {
        Unregister();
    }
    else
    {
        nn::pia::Result r;
        // すでにCloneからUnregisterされている場合にのみUnregisterElementする
        if (m_ElementR.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementR.GetId());
            r = pClone->UnregisterElement(&m_ElementR);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementU.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementU.GetId());
            r = pClone->UnregisterElement(&m_ElementU);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
    }
    bool ret = !m_ElementR.IsRegisteredWithCloneBase() &&
               !m_ElementU.IsRegisteredWithCloneBase();
    if (ret)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Type:%s - Cleanup Process is Succeeded.\n", GetCloneTypeText());
    }
    return ret;
}

void Object::ResetReckoningStrategy()
{
    nn::pia::Result r = m_ElementR.ResetReckoningStrategy();
    PIA_SAMPLE_ASSERT(r.IsSuccess());
    NN_PIA_UNUSED(r);
}

bool Object::IsLocalStation()
{
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    if (pSession && m_StationIndex == pSession->GetLocalStationIndex())
    {
        return true;
    }
    return false;
}

void Object::UpdateElement()
{
    if (!s_EventStatusArg.isUnreliable && m_ElementR.IsReadyToSetValue())
    {

        bool isStop = s_Operation.IsStop();

        m_ElementR.SetValue(s_Operation.pos, &isStop);

        s_Operation.oldPos = s_Operation.pos;
    }
    if (s_EventStatusArg.isUnreliable && m_ElementU.IsReadyToSetValue())
    {
        Status v;
        v.x = s_Operation.pos.x;
        v.y = s_Operation.pos.y;
        m_ElementU.SetValue(v);
    }
}

const char* Object::GetCurrentMode()
{
    return s_EventStatusArg.isUnreliable ? "Unr" : "Rck";
}

bool Object::IsActive()
{

    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

    return (pCloneProtocol->GetState() == nn::pia::clone::CloneProtocol::State_Active);
}


SendObject::SendObject()
{
    RegisterElementCore(&m_Clone);
}

void SendObject::Register()
{
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();
    pCloneProtocol->RegisterSendClone(&m_Clone, m_CloneId);
}

void SendObject::Unregister()
{
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();
    pCloneProtocol->UnregisterSendClone(m_CloneId);
}


void SendObject::Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();


    // 自ステーションの[*]を操作
    bool isHold = false;

    float accX;
    float accY;
    pSampleFramework->GetAnalog(&accX, &accY);
    const float min = 0.1f;
    if (accX >= min || accX <= -min)
    {
        s_Operation.acc.x = accX;
        isHold = true;
    }
    if (accY >= min || accY <= -min)
    {
        s_Operation.acc.y = -accY;
        isHold = true;
    }
    s_Operation.Update(isHold);
    s_Operation.ClampValues();
}

void SendObject::Update()
{
    UpdateElement();
}

bool SendObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

int SendObject::Print(int y, const nns::pia::sampleframework::Color& cColor)
{
    PrintValue(y, cColor);
    return y;
}

void SendObject::PrintValue(int y, const nns::pia::sampleframework::Color& cColor)
{
    // 自ステーションで操作している[*]をローカル座標を使って表示
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    nn::pia::reckoning::Vector3f* pos = &s_Operation.pos;
    pSampleFramework->DrawString(40, y, "%6.2f, %6.2f", pos->x, pos->y);

    int32_t xBuf = static_cast<int32_t>(pos->x);
    int32_t yBuf = static_cast<int32_t>(pos->y);
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(cColor);
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
        xBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontWidth() / 2 - 2,
        yBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontHeight() / 2 - 2,
        "[@]");
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
}

ReceiveObject::ReceiveObject()
{
    RegisterElementCore(&m_Clone);
    m_DrawPos = s_Operation.pos;
}

void ReceiveObject::Register()
{
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();
    pCloneProtocol->RegisterReceiveClone(&m_Clone, m_StationIndex, m_CloneId);
}

void ReceiveObject::Unregister()
{
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();
    pCloneProtocol->UnregisterReceiveClone(m_StationIndex, m_CloneId);
}

void ReceiveObject::Operate()
{
}

void ReceiveObject::Update()
{
    UpdateElement();
}

bool ReceiveObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

int ReceiveObject::Print(int y, const nns::pia::sampleframework::Color& cColor)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetFontColor(cColor);
    if (m_Clone.IsRegisteredWithProtocol())
    {
        pSampleFramework->DrawString(0, y, "R %1d ", m_StationIndex);

        // RTTを表示
        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
        nn::pia::StationId stationId;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, m_Clone.GetSendStationIndex());
        if (result.IsSuccess())
        {
            nn::pia::transport::Station* pStation = nn::pia::transport::StationManager::GetInstance()->GetStation(stationId);
            if (pStation && pSession->GetLocalStationIndex() != m_Clone.GetSendStationIndex())
            {
                int32_t rtt = pStation->GetRtt(3);
                pSampleFramework->DrawString(170, y, "RTT: %d", rtt);
            }
        }
    }
    PrintValue(y, cColor);
    y += 15;
    return y;
}

void ReceiveObject::PrintValue(int y, const nns::pia::sampleframework::Color& cColor)
{
    NN_PIA_UNUSED(cColor);

    if (IsLocalStation())
    {
        return;
    }
    // 自ステーション以外から受け取った[*]の座標を表示
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::reckoning::Vector3f v;
    bool isValid = false;

#if RECKONING_DELAY_TEST_ON
    s_ReckoningStrategy.setGetTiming(s_EventStatusArg.delayClock);
#endif

    if (!s_EventStatusArg.isUnreliable && m_ElementR.IsValidValue())
    {

        v = m_ElementR.GetValue();
        if (s_EventStatusArg.chaseType == EventChaseType_None)
        {
            m_DrawPos = v;
        }
        else if (s_EventStatusArg.chaseType == EventChaseType_Chase)
        {
            VecChaseDecrease(&m_DrawPos, v, s_EventStatusArg.chaseRatio, 0.1f, 10.0f);
        }

        isValid = true;
    }
    if (m_ElementU.IsValidValue())
    {
        Status s = m_ElementU.GetValue();

        if (s_EventStatusArg.isUnreliable)
        {
            v.x = s.x;
            v.y = s.y;
            m_DrawPos.x = v.x;
            m_DrawPos.y = v.y;
            isValid = true;
        }
    }
    if (isValid)
    {
        pSampleFramework->DrawString(40, y, "%6.2f, %6.2f", v.x, v.y);
        pSampleFramework->DrawString(static_cast<int32_t>(m_DrawPos.x), static_cast<int32_t>(m_DrawPos.y), " @ ");
    }
#if RECKONING_DELAY_TEST_ON
    s_ReckoningStrategy.offGetTiming();
#endif
}


SequentialObject::SequentialObject()
{

    m_Clone.RegisterElement(&m_ElementREX, 0x4);
}

void SequentialObject::Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    NN_PIA_UNUSED(pSampleFramework);


    if (m_ElementREX.IsReadyToSetValue())
    {

        /// L を押したら いじれる項目がずれる。
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {

            s_EventStatusIndex = (s_EventStatusIndex - 1 + EventStatusType_Max) % EventStatusType_Max;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {

            s_EventStatusIndex = (s_EventStatusIndex + 1) % EventStatusType_Max;
        }


        // EventStatusArg を切り替える
        int32_t count = 0;
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
        {
            count++;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            count--;
        }

        if (count != 0)
        {


            switch (s_EventStatusIndex)
            {

            case EventStatusType_Unreliable:
            {
                s_EventStatusArg.isUnreliable = !s_EventStatusArg.isUnreliable;
            }

            break;
            case EventStatusType_Latency:
            {
                s_EventStatusArg.latency += (50 * count);

                s_EventStatusArg.latency = Clamp(static_cast<int32_t>(s_EventStatusArg.latency), 0, 600);
            }
            break;
            case EventStatusType_ChaseType:
                s_EventStatusArg.chaseType += (count + EventChaseType_Max);
                s_EventStatusArg.chaseType %= EventChaseType_Max;

                break;


            case EventStatusType_ChaseRatio:
                s_EventStatusArg.chaseRatio += 0.01f * count;

                s_EventStatusArg.chaseRatio = Clamp(static_cast<float>(s_EventStatusArg.chaseRatio), 0.02f, 0.4f);

                break;

            case EventStatusType_VelScale:
                s_EventStatusArg.velScale += 0.1f * count;

                s_EventStatusArg.velScale = Clamp(static_cast<float>(s_EventStatusArg.velScale), 0.7f, 1.5f);
                break;

            case EventStatusType_StopThreshold:
                s_EventStatusArg.stopThreshold += 0.05f * count;

                s_EventStatusArg.stopThreshold = Clamp(static_cast<float>(s_EventStatusArg.stopThreshold), 0.0f, 1.0f);

                break;

            case EventStatusType_ReckoningThreshold:
                s_EventStatusArg.reckoningThreshold += 0.1f * count;
                s_EventStatusArg.reckoningThreshold = Clamp(static_cast<float>(s_EventStatusArg.reckoningThreshold), 0.0f, 3.0f);

                break;
#if RECKONING_DELAY_TEST_ON
            case EventStatusType_DelayClock:
                s_EventStatusArg.delayClock += 40.f * count;

                s_EventStatusArg.delayClock = Clamp(static_cast<float>(s_EventStatusArg.delayClock), 0.0f, 280.0f);

                break;
#endif
            }

            /// 一台ずつでの設定 相手のCAT-DEVでも設定する必要がある。
            nn::pia::transport::ThreadStreamManager::GetInstance()->SetSendThreadLatencyEmulation(s_EventStatusArg.latency / 2, s_EventStatusArg.latency / 2);

            nn::pia::transport::ThreadStreamManager::GetInstance()->SetReceiveThreadLatencyEmulation(0, 0);

            nn::pia::Result r = m_ElementREX.SetValue(s_EventStatusArg);


            if (r.IsSuccess())
            {
                //                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send SequentialObject \n");
            }
        }
    }
}

void SequentialObject::Update()
{
    //    UpdateElement();

    if (m_ElementREX.IsValidValue())
    {
        s_EventStatusArg = m_ElementREX.GetValue();
        /// 一台ずつでの設定 相手側でも 設定する必要がある。
        nn::pia::transport::ThreadStreamManager::GetInstance()->SetSendThreadLatencyEmulation(s_EventStatusArg.latency / 2, s_EventStatusArg.latency / 2);

        nn::pia::transport::ThreadStreamManager::GetInstance()->SetReceiveThreadLatencyEmulation(0, 0);


        Object::GetOperation()->SetThresholdStop(s_EventStatusArg.stopThreshold);
        Object::GetReckoningStrategy()->SetThreshold(s_EventStatusArg.reckoningThreshold);
    }
}

bool SequentialObject::Cleanup()
{

    nn::pia::Result r;
    if (m_ElementREX.IsRegisteredWithCloneBase())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Element Id:%x - Unregister\n", m_ElementREX.GetId());
        r = m_Clone.UnregisterElement(&m_ElementREX);
        PIA_SAMPLE_ASSERT(r.IsSuccess());
    }

    bool ret = !m_ElementREX.IsRegisteredWithCloneBase();
    if (ret)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Clone Type:%s - Cleanup Process is Succeeded.\n", GetCloneTypeText());
    }
    //    return CleanupCore(&m_Clone);
    return ret;
}

void SequentialObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterSequentialSharingClone(&m_Clone, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterSequentialSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("RegisterSequentialSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void SequentialObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterSequentialSharingClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterSequentialSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("UnregisterSequentialSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int SequentialObject::Print(int y, const nns::pia::sampleframework::Color& cColor)
{

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    nns::pia::sampleframework::Color cColorLY = cColor;

    if (!IsActive())
    {
        cColorLY = nns::pia::sampleframework::Color(0.4f, 0.4f, 0.4f, 1.0f);
    }

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(cColorLY);

    const int32_t beginX = 250;
    int32_t posY = 155;

    pSampleFramework->DrawString(beginX - 10, posY + 10 * s_EventStatusIndex, "* ");

    if (s_EventStatusArg.isUnreliable)
    {
        pSampleFramework->DrawString(beginX, posY, "Unreliable ");
    }
    else
    {
        pSampleFramework->DrawString(beginX, posY, "Reckoning ");
    }

    posY += 10;
    pSampleFramework->DrawString(beginX, posY, "Latency       %d ", s_EventStatusArg.latency);

    posY += 10;
    if (s_EventStatusArg.chaseType == EventChaseType_None)
    {
        pSampleFramework->DrawString(beginX, posY, "ChaseType     None");
    }
    else
    {
        pSampleFramework->DrawString(beginX, posY, "ChaseType    Chase");
    }

    posY += 10;
    pSampleFramework->DrawString(beginX, posY, "ChaseRatio    %0.2f ", s_EventStatusArg.chaseRatio);

    posY += 10;
    pSampleFramework->DrawString(beginX, posY, "VelScale      %1.1f ", s_EventStatusArg.velScale);

    posY += 10;
    pSampleFramework->DrawString(beginX, posY, "StopThreshold %1.1f ", s_EventStatusArg.stopThreshold);

    posY += 10;
    pSampleFramework->DrawString(beginX, posY, "Threshold     %1.1f ", s_EventStatusArg.reckoningThreshold);

    posY += 10;
#if RECKONING_DELAY_TEST_ON
    pSampleFramework->DrawString(beginX, posY, "DelayClock    %3.0f ", s_EventStatusArg.delayClock);
#endif

    return y;
}
