﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/reckoning.h>
#include <nn/pia/clone.h>
#include <nn/pia/transport.h>
#include "SampleLib/SampleLib.h"

#define RECKONING_DELAY_TEST_ON 0

struct EventStatusArg
{
public:
    bool isUnreliable;
    int32_t latency;

    int32_t chaseType;  //< 送られて来たデータをなめらかにする。

    float chaseRatio;
    float velScale;
    float stopThreshold;
    float reckoningThreshold;
#if RECKONING_DELAY_TEST_ON
    float delayClock;
#endif
};


class Operation
{
public:
    nn::pia::reckoning::Vector3f pos;
    nn::pia::reckoning::Vector3f oldPos;
    nn::pia::reckoning::Vector3f vel;
    nn::pia::reckoning::Vector3f acc;

    Operation()
    {
        pos.MakeZero();
        oldPos.MakeZero();
        vel.MakeZero();
        acc.MakeZero();
        m_ThresholdStop = 0.1f;
    }
    void Update(bool isHold);
    void ClampValues();
    bool IsStop();
    float GetThresholdStop()
    {
        return m_ThresholdStop;
    }
    void SetThresholdStop(float th);

private:
    float m_ThresholdStop;
};

struct Status
{
    float x;
    float y;
};

class Object
{
public:
    virtual void Register()
    {
    }
    virtual void Unregister()
    {
    }
    virtual void RegisterElement()
    {
    }
    void RegisterElementCore(nn::pia::clone::CloneBase* pClone);
    virtual void Operate()
    {
    }
    virtual void Update()
    {
    }
    virtual bool Cleanup()
    {
        return true;
    }
    bool CleanupCore(nn::pia::clone::CloneBase* pClone);
    void ResetReckoningStrategy();
    bool IsLocalStation();
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor)
    {
        NN_PIA_UNUSED(cColor);
        return y;
    }
    virtual void PrintValue(int, const nns::pia::sampleframework::Color& cColor)
    {
        NN_PIA_UNUSED(cColor);
    }

    void SetStationIndex(nn::pia::StationIndex index)
    {
        m_StationIndex = index;
    }
    nn::pia::StationIndex GetStationIndex()
    {
        return m_StationIndex;
    }
    void SetCloneId(nn::pia::clone::CloneBase::Id id)
    {
        m_CloneId = id;
    }
    nn::pia::clone::CloneBase::Id GetCloneId()
    {
        return m_CloneId;
    }


    static nn::pia::reckoning::Simple3dReckoningStrategy* GetReckoningStrategy()
    {
        return &s_ReckoningStrategy;
    }

    static Operation* GetOperation()
    {
        return &s_Operation;
    }


protected:
    virtual const char* GetCloneTypeText() const = 0;
    void UpdateElement();
    const char* GetCurrentMode();

    bool IsActive();

protected:
    nn::pia::clone::CloneBase::Id m_CloneId;
    nn::pia::StationIndex m_StationIndex;

    nn::pia::reckoning::ReckoningCloneElement<nn::pia::reckoning::Simple3dReckoningStrategy> m_ElementR;
    static nn::pia::reckoning::Simple3dReckoningStrategy s_ReckoningStrategy;

    // 比較用エレメント
    nn::pia::clone::UnreliableCloneElement<Status> m_ElementU;
    // 通知用エレメント

    nn::pia::clone::ReliableCloneElement<EventStatusArg> m_ElementREX;

    static Operation s_Operation;
};

class SendObject : public Object
{
public:
    SendObject();
    virtual void Register();
    virtual void Unregister();
    virtual void RegisterElement()
    {
        RegisterElementCore(&m_Clone);
    }
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup();
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor);
    virtual void PrintValue(int y, const nns::pia::sampleframework::Color& cColor);
    nn::pia::clone::SendClone m_Clone;

protected:
    virtual const char* GetCloneTypeText() const
    {
        return "S";
    }
};

class ReceiveObject : public Object
{
public:
    ReceiveObject();
    void InitCloneTypeText()
    {
        PIA_SNPRINTF(m_CloneTypeText, sizeof(m_CloneTypeText), "R%1d", m_StationIndex);
    }
    virtual void Register();
    virtual void Unregister();
    virtual void RegisterElement()
    {
        RegisterElementCore(&m_Clone);
    }
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup();
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor);
    virtual void PrintValue(int y, const nns::pia::sampleframework::Color& cColor);

protected:
    virtual const char* GetCloneTypeText() const
    {
        return m_CloneTypeText;
    }
    char m_CloneTypeText[4];

    nn::pia::clone::ReceiveClone m_Clone;
    nn::pia::reckoning::Vector3f m_DrawPos;
};


// SequentialSharingClone を使ったゲームオブジェクトです。
// ゲーム状態の共有のため
class SequentialObject : public Object
{
public:
    SequentialObject();
    virtual void Operate();
    virtual void Update();
    virtual bool Cleanup();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    virtual bool IsCloneRegistered() const
    {
        return m_Clone.IsRegisteredWithProtocol();
    }
    virtual int Print(int y, const nns::pia::sampleframework::Color& cColor);

protected:
    virtual nn::pia::clone::CloneBase* GetClonePtr()
    {
        return &m_Clone;
    }
    virtual const nn::pia::clone::CloneBase& GetClone() const
    {
        return m_Clone;
    }
    virtual const char* GetCloneTypeText() const
    {
        return "Q";
    }
    nn::pia::clone::SequentialSharingClone m_Clone;
};
