﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "Object.h"

#include <cstdio>

#if NN_PIA_CTR || NN_PIA_NINTENDOSDK_NX
#define USE_PIA_LOCAL 1
#else
#define USE_PIA_LOCAL 0
#endif


#if USE_PIA_LOCAL
#include <nn/pia/local.h>
#include "SampleLib/Local_Setup.h"
#endif

#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Clone_Setup.h"
#include "SampleLib/Common_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Reckoning_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Transport_Setup.h"

#define PROGRAM_TITLE "SimpleReckoning"

#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif


namespace
{
#if NNS_PIA_ENABLE_INET
const bool IsHostMigrationEnabled = false;  //!< 親交代機能の ON/OFF。このデモでは OFF 固定です。
#endif
const int MaxStations = 8;                  // 最大 Station 数
const int ObjectNum = 1 + MaxStations;
const nn::pia::clone::CloneBase::Id CloneId = 0;

static SendObject s_SendObject;
static ReceiveObject s_ReceiveObject[MaxStations];
static Object* s_pObjectArray[ObjectNum];
static SequentialObject s_SeqObject;

static int s_Cursor = ObjectNum;
static int s_HoldCounterUp = 0;
static int s_HoldCounterDown = 0;
static int s_HoldCounterRight = 0;
static int s_HoldCounterLeft = 0;
static int s_PrintSendPacketNum = 0;
static int s_PrintReceivePacketNum = 0;
static int s_LastSendPacketNum = 0;
static int s_LastReceivePacketNum = 0;

// 実行中のモードを表します。
// inet, lan では、当初ランダムマッチメイクで始まって、マッチング後にホストになるかクライアントになるか決まります。
enum GameMode
{
#if USE_PIA_LOCAL
    GameMode_LocalHost,
    GameMode_LocalClient,
    GameMode_LocalError,
#endif
#if NNS_PIA_ENABLE_INET
    GameMode_InetRandom,
    GameMode_InetHost,
    GameMode_InetClient,
    GameMode_InetError,
#endif
    GameMode_LanRandom,
    GameMode_LanHost,
    GameMode_LanClient,
    GameMode_LanError,
    GameMode_Exit
};

const uint32_t MemoryBlockSize = 10 * 1024 * 1024;                // nn::pia::common::Initialize()で供給するメモリサイズ（単位はバイト）
PIA_ATTRIBUTE_ALIGN(8) static uint8_t s_Memory[MemoryBlockSize];  // nn::pia::common::Initialize()で供給するメモリ

#if NNS_PIA_ENABLE_INET
// アプリケーションの仕様に応じて調整して下さい。
const uint32_t MemoryBlockSizeForNex = 1 * 1024 * 1024;
const uint32_t MemoryBlockSizeForNexEmergency = 50 * 1024;
#endif
const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 55000;
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;

#if USE_PIA_LOCAL
#if NN_PIA_CTR
const uint32_t LocalTitleId = 0xFF340;
const uint8_t LocalSubId = MatchmakeGameModeOffset;
#elif NN_PIA_NINTENDOSDK
const uint16_t LocalSceneId = MatchmakeGameModeOffset + 8000 + 30000 * PIA_TEST_CODE;
#endif
//  sizeofするのでcharでも[]で定義する
const char LocalWirelessCryptoKey[] = "aaabbccc0123456789";
const uint32_t LocalWirelessCryptoKeySize = sizeof(LocalWirelessCryptoKey);
const char SignatureKey[] = "simplereckoning_key";
const uint32_t SignatureKeySize = sizeof(SignatureKey);
#endif

}  // end of anonymous namespace


static nn::pia::StationIndex ConvertStationIdToDataIndex(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_ASSERT(stationIndex < MaxStations);
        return stationIndex;
    }
    else
    {
        // 親がいなくなってセッション強制終了するとき、一瞬だけ
        // シーン終了前にDrawでここが呼ばれてしまうのでresult.IsSuccess()でASSERTしない
        return nn::pia::StationIndex_Invalid;
    }
}

// 実行するモードを決定します。
static GameMode DecideGameMode()
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Please select game mode.\n [START]Quit\n [Y]LanRandom\n");
#if NNS_PIA_ENABLE_INET
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" [X]InetRandom\n");
#endif
#if USE_PIA_LOCAL
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" [A]LocalHost\n [B]LocalClient\n");
#endif
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" Waiting input\n");

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    GameMode gameMode = GameMode_Exit;
    bool isDecided = false;
    while (!isDecided)
    {
        pSampleFramework->BeginFrame();

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
#if NNS_PIA_ENABLE_INET
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: InetRandom\n");
            gameMode = GameMode_InetRandom;
            isDecided = true;
#endif
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LanRandom\n");
            gameMode = GameMode_LanRandom;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Exit\n");
            gameMode = GameMode_Exit;
            isDecided = true;
        }
#if USE_PIA_LOCAL
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalHost\n");
            gameMode = GameMode_LocalHost;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalClient\n");
            gameMode = GameMode_LocalClient;
            isDecided = true;
        }
#endif
        int y = 10;
        pSampleFramework->DrawString(10, y, PROGRAM_TITLE);
        y += 10;
        pSampleFramework->DrawString(10, y, "Select game mode.");
        y += 10;
#if USE_PIA_LOCAL
        pSampleFramework->DrawString(10, y, "A: LocalHost");
        y += 10;
        pSampleFramework->DrawString(10, y, "B: LocalClient");
        y += 10;
#endif
#if NNS_PIA_ENABLE_INET
        pSampleFramework->DrawString(10, y, "X: InetRandom");
        y += 10;
#endif
        pSampleFramework->DrawString(10, y, "Y: LanRandom");
        y += 10;
        pSampleFramework->DrawString(10, y, "START: Quit this application");

        pSampleFramework->EndFrame();
    }
    return gameMode;
}

static void RegisterStationId(nn::pia::StationId stationId)
{
    nn::pia::StationIndex index = ConvertStationIdToDataIndex(stationId);
    PIA_SAMPLE_ASSERT(index < MaxStations);

    ReceiveObject& r = s_ReceiveObject[index];
    r.Register();
}

static void UnregisterStationId(nn::pia::StationId stationId)
{
    nn::pia::StationIndex index = ConvertStationIdToDataIndex(stationId);
    PIA_SAMPLE_ASSERT(index < MaxStations);

    ReceiveObject& r = s_ReceiveObject[index];
    r.Unregister();
}

// コールバック関数の宣言
static void SampleSessionCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam)
{
    NN_PIA_UNUSED(pApplicationParam);
    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
    {
        PIA_SAMPLE_PRINT("Sample Event JOIN detect. StationId(%016llx)\n", stationId);
        RegisterStationId(stationId);
    }
    break;
    case nn::pia::session::Session::EventType_EventLeave:
    {
        PIA_SAMPLE_PRINT("Sample Event LEAVE detect. StationId(%016llx)\n", stationId);
        UnregisterStationId(stationId);
    }
    break;
    default:
    {
        PIA_SAMPLE_PRINT("Sample Event(%d) detect. StationId(0x%016llx)\n", eventType, stationId);
    }
    }
}

// 各オブジェクトを初期化します。
static void InitObjects()
{
    int i = 0;
    {
        s_SendObject.SetCloneId(CloneId);
        s_pObjectArray[i] = &s_SendObject;
        ++i;
    }
    for (int j = 0; j < MaxStations; ++j)
    {
        s_ReceiveObject[j].SetCloneId(CloneId);
        s_ReceiveObject[j].SetStationIndex(static_cast<nn::pia::StationIndex>(j));
        s_ReceiveObject[j].InitCloneTypeText();
        s_pObjectArray[i] = &s_ReceiveObject[j];
        ++i;
    }

    {
        s_SeqObject.SetCloneId(CloneId);
    }
    PIA_SAMPLE_ASSERT(i == ObjectNum);
}

static void RegisterSendObject()
{
    nn::pia::StationIndex sidx;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&sidx, nn::pia::session::Session::GetInstance()->GetLocalStationId());
    if (result.IsFailure())
    {
        sidx = nn::pia::StationIndex_Invalid;
    }

    s_SendObject.SetStationIndex(sidx);
    s_SendObject.Register();

    if (!s_SeqObject.IsCloneRegistered())
    {
        s_SeqObject.RegisterClone();
    }
}

// キー入力による操作です。
static bool Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    // START で GameLoop() を抜けます。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
    {
        return true;
    }

    s_SendObject.Operate();
    s_SeqObject.Operate();
    {
        //        float threshold = Object::GetReckoningStrategy()->GetThreshold();
        //        float thStop = Object::GetOperation()->GetThresholdStop();

        if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonUp))
        {
            ++s_HoldCounterUp;
        }
        else if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonDown))
        {
            ++s_HoldCounterDown;
        }
        else
        {
            s_HoldCounterUp = 0;
            s_HoldCounterDown = 0;
        }

        if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonRight))
        {
            ++s_HoldCounterRight;
        }
        else if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonLeft))
        {
            ++s_HoldCounterLeft;
        }
        else
        {
            s_HoldCounterRight = 0;
            s_HoldCounterLeft = 0;
        }
    }
    {
        nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            switch (pCloneProtocol->GetState())
            {
            case nn::pia::clone::CloneProtocol::State_Inactive:
            {
                nn::pia::Result r = pCloneProtocol->Start();
                PIA_SAMPLE_ASSERT(r.IsSuccess());
                NN_PIA_UNUSED_IN_RELEASE(r);
            }
            break;

            case nn::pia::clone::CloneProtocol::State_Start:
            case nn::pia::clone::CloneProtocol::State_Active:
            {
                nn::pia::Result r = pCloneProtocol->Stop();
                PIA_SAMPLE_ASSERT(r.IsSuccess());
                NN_PIA_UNUSED_IN_RELEASE(r);
            }
            break;
            default:
                break;
            }
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            if (pCloneProtocol->GetState() == nn::pia::clone::CloneProtocol::State_Active &&
                pCloneProtocol->IsRegulatingClock() == false)
            {
                nn::pia::Result r = pCloneProtocol->RegulateClock();
                PIA_SAMPLE_ASSERT(r.IsSuccess());
                NN_PIA_UNUSED_IN_RELEASE(r);
            }
        }
    }

    return false;
}

static nns::pia::sampleframework::Color GetColor(int index)
{
    nns::pia::sampleframework::Color result(1.f, 1.f, 1.f, 1.f);
    switch (index)
    {
    case 0:
    {
        result = nns::pia::sampleframework::Color(1.f, 0.0f, 0.0f, 1.f);
    }
    break;
    case 1:
    {
        result = nns::pia::sampleframework::Color(0.4f, 0.4f, 0.9f, 1.f);
    }
    break;
    case 2:
    {
        result = nns::pia::sampleframework::Color(0.f, 0.5f, 0.f, 1.f);
    }
    break;
    case 3:
    {
        result = nns::pia::sampleframework::Color(0.5f, 0.5f, 0.f, 1.f);
    }
    break;
    case 4:
    {
        result = nns::pia::sampleframework::Color(0.f, 1.f, 1.f, 1.f);
    }
    break;
    case 5:
    {
        result = nns::pia::sampleframework::Color(1.f, 0.f, 1.f, 1.f);
    }
    break;
    case 6:
    {
        result = nns::pia::sampleframework::Color(1.f, 1.f, 0.f, 1.f);
    }
    break;
    default:
    {
        result = nns::pia::sampleframework::Color(0.5f, 0.5f, 0.5f, 1.f);
    }
    break;
    }
    return result;
}

// 画面の描画処理です。
static void Draw(GameMode gameMode)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

    const nns::pia::sampleframework::Color DefaultColor(1.0f, 1.0f, 1.0f, 1.0f);
    const nns::pia::sampleframework::Color CursorColor(1.0f, 0.3f, 0.3f, 1.0f);
    const nns::pia::sampleframework::Color InvalidColor(0.4f, 0.4f, 0.4f, 1.0f);
    const nns::pia::sampleframework::Color ColorR(1.0f, 0.4f, 0.4f, 1.0f);
    const nns::pia::sampleframework::Color ColorG(0.4f, 1.0f, 0.4f, 1.0f);
    const nns::pia::sampleframework::Color ColorB(0.4f, 0.4f, 1.0f, 1.0f);

    pSampleFramework->SetFontColor(DefaultColor);
    {
        pSampleFramework->DrawString(10, 10, PROGRAM_TITLE);
        const char* GameModeString = NULL;
        switch (gameMode)
        {
#if USE_PIA_LOCAL
        case GameMode_LocalHost:
        case GameMode_LocalClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "LocalHost";
            }
            else
            {
                GameModeString = "LocalClient";
            }
        }
        break;
#endif
#if NNS_PIA_ENABLE_INET
        case GameMode_InetRandom:
        {
            GameModeString = "InetRandom";
        }
        break;
        case GameMode_InetHost:
        case GameMode_InetClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "InetHost";
            }
            else
            {
                GameModeString = "InetClient";
            }
        }
        break;
#endif
        case GameMode_LanRandom:
        {
            GameModeString = "LanRandom";
        }
        break;
        case GameMode_LanHost:
        case GameMode_LanClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "LanHost";
            }
            else
            {
                GameModeString = "LanClient";
            }
        }
        break;
        default:
        {
            PIA_SAMPLE_ASSERT(false);
        }
        break;
        }
        PIA_SAMPLE_ASSERT(GameModeString != NULL);
        pSampleFramework->DrawString(10, 20, "GameMode:%s", GameModeString);
        pSampleFramework->DrawString(180, 20, "StationNum:%d", pSession->GetStationNum());
        pSampleFramework->DrawString(300, 20, "Index:%d", ConvertStationIdToDataIndex(pSession->GetLocalStationId()));
    }

    {
        int idx = (pSampleFramework->GetFrameCount() / 4) % 4;
        const char AnimLetter[] = { '-', '\\', '|', '/' };
        pSampleFramework->DrawString(380, 10, "%c", AnimLetter[idx]);
    }

    {
        pSampleFramework->SetFontColor(s_Cursor == ObjectNum ? CursorColor : DefaultColor);

        const char* cpStateText = "";
        switch (pCloneProtocol->GetState())
        {
        case nn::pia::clone::CloneProtocol::State_Inactive:
        {
            cpStateText = "Inactive";
        }
        break;
        case nn::pia::clone::CloneProtocol::State_Start:
        {
            cpStateText = "Start";
        }
        break;
        case nn::pia::clone::CloneProtocol::State_Active:
        {
            cpStateText = "Active";
        }
        break;
        case nn::pia::clone::CloneProtocol::State_Stop:
        {
            cpStateText = "Stop";
        }
        break;
        }
        pSampleFramework->DrawString(10, 225, "State:%s", cpStateText);

        pSampleFramework->SetFontColor(DefaultColor);
    }

    {
        nn::pia::clone::ClockValue clock = pCloneProtocol->GetClock();
        if (clock == nn::pia::clone::InvalidClock)
        {
            pSampleFramework->DrawString(10, 210, "Clock:--------");
        }
        else
        {
            pSampleFramework->DrawString(10, 210, "Clock:%8.2f%c", static_cast<float>(clock) / 1000.0f, pCloneProtocol->IsRegulatingClock() ? 'R' : ' ');
        }
    }
    /*
    {
        pSampleFramework->DrawString(10, 205, "Stop Threshold:%6.2f", Object::GetOperation()->GetThresholdStop());
        pSampleFramework->DrawString(10, 215, "Threshold:%6.2f", Object::GetReckoningStrategy()->GetThreshold());
    }
    */
    {
        if (pSampleFramework->GetFrameCount() % 30 == 0)
        {
            {
                int packetNum = nn::pia::transport::ThreadStreamManager::GetInstance()->GetSendThreadSendPacketNum();
                s_PrintSendPacketNum = packetNum - s_LastSendPacketNum;
                s_LastSendPacketNum = packetNum;
            }
            {
                int packetNum = nn::pia::transport::ThreadStreamManager::GetInstance()->GetReceiveThreadReceivePacketNum();
                s_PrintReceivePacketNum = packetNum - s_LastReceivePacketNum;
                s_LastReceivePacketNum = packetNum;
            }
        }

        pSampleFramework->DrawString(150, 225 - 8, "Send : %3d", s_PrintSendPacketNum);
        pSampleFramework->DrawString(150, 225, "Recv : %3d", s_PrintReceivePacketNum);
    }

    {
        int y = 30;
        for (uint32_t i = 0; i < MaxStations; ++i)
        {

            Object* o = &s_ReceiveObject[i];
            nns::pia::sampleframework::Color color = GetColor(i);
            nn::pia::StationIndex idx = o->GetStationIndex();
            // 改行される前にSendをPrint
            if (s_SendObject.GetStationIndex() == idx)
            {
                y = s_SendObject.Print(y, color);
            }
            y = o->Print(y, color);  // ここで改行
        }
    }
    pSampleFramework->SetFontColor(DefaultColor);

    s_SeqObject.Print(0, DefaultColor);
}

static void Update()
{
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

    if (pCloneProtocol->GetState() != nn::pia::clone::CloneProtocol::State_Inactive)
    {
        pCloneProtocol->UpdateClock();
    }

    if (pCloneProtocol->GetState() == nn::pia::clone::CloneProtocol::State_Start ||
        pCloneProtocol->GetState() == nn::pia::clone::CloneProtocol::State_Active)
    {
        if (pCloneProtocol->GetError() != nn::pia::clone::CloneProtocol::ErrorType_Nothing)
        {
            // エラー発生を検知したので終了
            const char* cpErrorText = "";
            switch (pCloneProtocol->GetError())
            {
            case nn::pia::clone::CloneProtocol::ErrorType_Nothing:
                break;
            case nn::pia::clone::CloneProtocol::ErrorType_ClockOverflow:
            {
                cpErrorText = "ClockOverflow";
            }
            break;
            case nn::pia::clone::CloneProtocol::ErrorType_EventDropped:
            {
                cpErrorText = "EventDropped";
            }
            break;
            }
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Error: %s\n", cpErrorText);
            pCloneProtocol->Stop();
        }
    }

    for (int i = 0; i < ObjectNum; ++i)
    {
        s_pObjectArray[i]->Update();
    }

    s_SeqObject.Update();
}

// 通信中のメインループです。
static void MainLoop(GameMode gameMode, bool* pIsLeaveNeeded)
{
    RegisterSendObject();

    {
        // 送信遅延を設定
        nn::pia::transport::ThreadStreamManager::GetInstance()->SetSendThreadLatencyEmulation(200, 200);
        nn::pia::transport::ThreadStreamManager::GetInstance()->SetReceiveThreadLatencyEmulation(0, 0);
    }

    nn::pia::Result result;
    while (true)
    {
        bool isExit = false;

        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        isExit |= Operate();

#if NNS_PIA_ENABLE_INET
        // GameMode_InetXXX の場合は nn::nex::Scheduler が作られていて、そのディスパッチも行う必要があります。
        nns::pia::Nex_SchedulerDispatch();
#endif
        // nn::pia::common::Scheduler::Dispatch() を定期的に呼び出す必要があります。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

#if NNS_PIA_ENABLE_INET
        if (gameMode == GameMode_LanHost || gameMode == GameMode_LanClient || gameMode == GameMode_InetHost || gameMode == GameMode_InetClient)
#else
        if (gameMode == GameMode_LanHost || gameMode == GameMode_LanClient)
#endif
        {
            if (nns::pia::Net_IsConnected() == false)
            {
                // AC が切断したので終了します。
                isExit = true;
                *pIsLeaveNeeded = false;
            }
        }

        result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
        if (result.IsSuccess() &&
            nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedSession)
        {
            Update();
        }
        else
        {
            // セッションが切れたので終了します。
            isExit = true;
            nn::pia::Result::HandlingType handlingType = result.GetHandlingType();
            if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
            {
                *pIsLeaveNeeded = true;
            }
            else if (handlingType == nn::pia::Result::HandlingType_ShutdownNetwork)
            {
                *pIsLeaveNeeded = false;
            }
        }


        // プロトコルに対して送信したものをすぐに処理するために、もう一度 nn::pia::common::Scheduler::Dispatch() を呼びます。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        Draw(gameMode);

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        if (isExit)
        {
            return;
        }
    }
}

static void ResetReckoningStrategy()
{
    bool isCleanupAll = true;

    do
    {
        for (int i = 0; i < ObjectNum; ++i)
        {
            // 各Cloneの各ElementをUnregisterする
            isCleanupAll &= s_pObjectArray[i]->Cleanup();
        }
    } while (!isCleanupAll);

    if (isCleanupAll)
    {
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Called ResetReckoningStrategy()\n");
        for (int i = 0; i < ObjectNum; ++i)
        {
            s_pObjectArray[i]->ResetReckoningStrategy();
        }
        // もう一度登録し直す
        for (int i = 0; i < ObjectNum; ++i)
        {
            s_pObjectArray[i]->RegisterElement();
        }
    }
}

// エントリポイント。
PIA_SAMPLE_MAIN_FUNC()
{
    nn::pia::Result result;
    {
        nns::pia::sampleframework::SampleFramework::Setting setting;
        setting.isUpperScreenOnly = true;
        nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(60);
    }

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(PROGRAM_TITLE "\n");
    nns::pia::Basic_SetProgramTitle(PROGRAM_TITLE);

    InitObjects();


    while (true)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetClearColor(nns::pia::sampleframework::Color(0.1f, 0.1f, 0.1f, 1.0f));

        GameMode gameMode = DecideGameMode();
        if (gameMode == GameMode_Exit)
        {
            break;
        }

        // Piaの各モジュールの初期化処理
        // この順番で行ってください。
        {
            result = nns::pia::Common_InitializePiaCommon(s_Memory, MemoryBlockSize);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            nn::pia::common::Report::GetGlobalObject()->SetMask(nn::pia::common::Report::LevelError);

            nn::pia::transport::NetworkFactory* pFactory = NULL;
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            {
#if NN_PIA_CTR
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(LocalTitleId, LocalSubId, false, nn::uds::NO_WAIT | nn::uds::FORCE_DIRECT_BC));
#elif NN_PIA_NINTENDOSDK
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(nns::pia::LocalCommunicationId, LocalSceneId));
#endif
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                pFactory = nns::pia::Local_GetNetworkFactory();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                // PiaInet を初期化するより前に NEX を初期化してください
                result = nns::pia::Nex_InitializeNex(
                    MemoryBlockSizeForNex,
                    MemoryBlockSizeForNexEmergency);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                result = nns::pia::Inet_InitializePiaInet();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Inet_GetNetworkFactory();
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                result = nns::pia::Lan_InitializePiaLan();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Lan_GetNetworkFactory();
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

#if NN_PIA_UNDER_DEVELOP
            {
                // debugSetting をいれる。
                nn::pia::transport::Transport::DebugSetting debugSetting;
                debugSetting.sendThreadLatencyEmulationPacketNum = 128;
                debugSetting.receiveThreadLatencyEmulationPacketNum = 128;
                debugSetting.isPacketLossEmulationEnabled = false;
                result = nn::pia::transport::Transport::SetDebugSetting(debugSetting);
                PIA_SAMPLE_ASSERT(result.IsSuccess());
            }
#endif

            // Factoryを利用して、Transportをセットアップします。
            nn::pia::transport::Transport::Setting setting;
            setting.pFactory = pFactory;
            setting.maxStationNum = MaxStations;
            setting.maxSendThreadBufferNum = nns::pia::MaxSendThreadBufferNum;
            setting.maxReceiveThreadBufferNum = nns::pia::MaxReceiveThreadBufferNum;


            result = nns::pia::Transport_InitializePiaTransport(setting);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            // cloneのセットアップを行います。
            nns::pia::Clone_InitializePiaClone();

            // reckoningのセットアップを行います。
            nns::pia::Reckoning_InitializePiaReckoning();

            // sessionのセットアップを行います。
            {
                nn::pia::session::Session::Setting sessionSetting;
                sessionSetting.pNetworkFactory = pFactory;
                sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
                sessionSetting.browsedSessionInfoListNum = 8;
                result = nns::pia::Session_InitializePiaSession(sessionSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                result = nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionCallback);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
        }

        // ネットワークの接続、セッションの確立の処理
        {
            // ネットワークの初期化
            nns::pia::NetworkType networkType = nns::pia::NetworkType_Internet;
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            {
                networkType = nns::pia::NetworkType_Local;
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                networkType = nns::pia::NetworkType_Internet;
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                networkType = nns::pia::NetworkType_Lan;
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(networkType);
            if (netResult != nns::pia::NetResult_Success)
            {
                PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork:failed. (netResult = %d)\n", netResult);

                switch (gameMode)
                {
#if USE_PIA_LOCAL
                case GameMode_LocalHost:
                case GameMode_LocalClient:
                {
                    gameMode = GameMode_LocalError;
                }
                break;
#endif
#if NNS_PIA_ENABLE_INET
                case GameMode_InetRandom:
                {
                    gameMode = GameMode_InetError;
                }
                break;
#endif
                case GameMode_LanRandom:
                {
                    gameMode = GameMode_LanError;
                }
                break;
                default:
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                break;
                }
            }

            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, SignatureKeySize);
                startupSetting.signatureKeyDataSize = SignatureKeySize;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

// セッションを構築します。
// 作成するマッチメイクセッションの設定
#if NN_PIA_CTR
                nn::pia::local::UdsCreateSessionSetting createSessionSetting;
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnCreateSessionSetting createSessionSetting;
#endif
                {
#if NN_PIA_CTR
                    createSessionSetting.SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    createSessionSetting.SetSubId(LocalSubId);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, LocalWirelessCryptoKeySize);
#elif NN_PIA_NINTENDOSDK
                    createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    createSessionSetting.SetSceneId(LocalSceneId);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, LocalWirelessCryptoKeySize);
#endif
                    createSessionSetting.SetMaxParticipantNum(MaxStations);
                }

                result = nns::pia::Session_CreateSession(&createSessionSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_CreateSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LocalError;
                    break;
                }
            }
            break;

            case GameMode_LocalClient:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, SignatureKeySize);
                startupSetting.signatureKeyDataSize = SignatureKeySize;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = GameMode_LocalError;
                    break;
                }
                // セッションを検索します。
                // 検索条件の設定
                const uint32_t SearchSettingNum = 1;
#if NN_PIA_CTR
                nn::pia::local::UdsSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    searchCriteriaList[0].SetSubId(LocalSubId);
                }
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    searchCriteriaList[0].SetSceneId(LocalSceneId);
                }
#endif
                searchCriteriaList[0].SetMaxParticipantNum(MaxStations);
                searchCriteriaList[0].SetResultRange(0, 8);

                result = nns::pia::Session_BrowseSession(&searchCriteriaList[0]);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LocalError;
                    break;
                }

                // 検索したセッションに参加します。
                uint32_t foundSessionNum = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize();
                if (foundSessionNum > 0)
                {
                    PIA_SAMPLE_PRINT("session found\n");
                    nn::pia::session::ISessionInfoList::Iterator it = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->Begin();
                    nn::pia::session::ISessionInfo* pTargetSessionInfo = *it;
                    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTargetSessionInfo));

#if NN_PIA_CTR
                    nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#elif NN_PIA_NINTENDOSDK
                    nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
#endif
                    joinSessionSetting.SetSessionInfoPtr(pTargetSessionInfo);
                    joinSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, LocalWirelessCryptoKeySize);
                    result = nns::pia::Session_JoinSession(&joinSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("nns::pia::Session_JoinSession:failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                        gameMode = GameMode_LocalError;
                        break;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("no session found\n");
                    gameMode = GameMode_LocalError;
                    break;
                }
            }
            break;
            case GameMode_LocalError:
            {
            }
            break;
#endif  // USE_PIA_LOCAL
#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                // ゲームサーバにログイン
                if (!nns::pia::Inet_Login())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Inet_Login: nns::pia::Inet_Login:failed\n");
                    gameMode = GameMode_InetError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::inet::NexSessionStartupSetting startupSetting;
                startupSetting.isHostMigrationEnabled = IsHostMigrationEnabled;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_InetError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::inet::NexCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const uint32_t SearchSettingNum = 1;
                nn::pia::inet::NexSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::inet::NexJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_InetError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = GameMode_InetHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = GameMode_InetClient;
                }
            }
            break;
            case GameMode_InetError:
            {
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                if (nns::pia::Lan_StartupPiaLan().IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Lan_StartupLan: failed\n");
                    gameMode = GameMode_LanError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::lan::LanSessionStartupSetting startupSetting;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LanError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::lan::LanCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const uint32_t SearchSettingNum = 1;
                nn::pia::lan::LanSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::lan::LanJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LanError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = GameMode_LanHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = GameMode_LanClient;
                }
            }
            break;
            case GameMode_LanError:
            {
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }
        }

        bool isLeaveNeeded = false;

        // メインの処理
        switch (gameMode)
        {
#if USE_PIA_LOCAL
        case GameMode_LocalHost:
        case GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
        case GameMode_InetHost:
        case GameMode_InetClient:
#endif
        case GameMode_LanHost:
        case GameMode_LanClient:
        {
            MainLoop(gameMode, &isLeaveNeeded);

            // Watermark機能による計測結果プリント。
            if (PIA_IS_VALID_POINTER(nn::pia::common::WatermarkManager::GetInstance()))
            {
                nn::pia::common::WatermarkManager::GetInstance()->Trace(nn::pia::common::TraceFlagAlways);
            }
        }
        break;
        default:
            break;
        }


        // セッションの終了、ネットワークからの切断の処理
        {
            // Session 終了処理。
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            {
                if (isLeaveNeeded)
                {
                    // セッションから離脱します。
                    // 本サンプルでは、この後終了処理を行うため、返り値は無視しています。
                    nns::pia::Session_LeaveSession();
                }
            }
            break;
#if USE_PIA_LOCAL
            case GameMode_LocalError:
            {
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetError:
            {
            }
            break;
#endif
            case GameMode_LanError:
            {
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            // Session クラスのクリーンアップ処理。
            nns::pia::Session_CleanupSession();

            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            case GameMode_LocalError:
            {
                nns::pia::Local_FinalizeLocalCommunication();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
            case GameMode_InetError:
            {
                nns::pia::Inet_Logout();
            }
            break;
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            case GameMode_LanError:
            {
                nns::pia::Lan_CleanupPiaLan();
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            nns::pia::Net_FinalizeNetwork();
        }

        // Piaの各モジュールの終了処理
        // 初期化と逆順で行ってください。
        {
            nns::pia::Session_FinalizePiaSession();
            nns::pia::Reckoning_FinalizePiaReckoning();
            nns::pia::Clone_FinalizePiaClone();
            ResetReckoningStrategy();
            nns::pia::Transport_FinalizePiaTransport();
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            case GameMode_LocalError:
            {
                nns::pia::Local_FinalizePiaLocal();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
            case GameMode_InetError:
            {
                nns::pia::Inet_FinalizePiaInet();
                nns::pia::Nex_FinalizeNex();
            }
            break;
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            case GameMode_LanError:
            {
                nns::pia::Lan_FinalizePiaLan();
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }
            nns::pia::Common_FinalizePiaCommon();
        }
    }

    nns::pia::sampleframework::SampleFramework::DestroyInstance();

    PIA_SAMPLE_PRINT("end.\n");
}
