﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/Session_Setup.h"
#if NNS_PIA_ENABLE_INET
#include <nn/pia/inet.h>
#endif
#include <nn/pia/lan.h>
#if NNS_PIA_ENABLE_LOCAL
#include "SampleLib/Local_Setup.h"
#endif

namespace
{
struct RectPosition  //!< 矩形の位置座標
{
    int16_t x;
    int16_t y;
};

// ステーションのステータス情報
struct SampleStationStatus
{
    nn::pia::StationId stationId;
    uint8_t colorIndex;
    RectPosition pos;
    uint32_t latestIncrementNumber;
    int32_t rtt;
    nn::pia::transport::Station::IdentificationToken token;
    nn::pia::transport::Station::PlayerInfo playerInfo[nn::pia::MaxPlayerInfoSize];
    uint8_t playerCount;

    SampleStationStatus()
    {
        stationId = nn::pia::StationIdInvalid;
        colorIndex = 0;
        pos.x = 0;
        pos.y = 0;
        latestIncrementNumber = 0;
        rtt = 0;
        memset(&token, 0, sizeof(nn::pia::transport::Station::IdentificationToken));
        for (int i = 0; i < nn::pia::MaxPlayerInfoSize; ++i)
        {
            playerInfo[i].screenName.Clear();
            playerInfo[i].nameStringLanguage = 0xff;
        }
        playerCount = 0;
    }

    // stationId, colorIdex, pos, latestIncrementNumber
    static const uint32_t SerializedSize = sizeof(uint64_t) + sizeof(uint8_t) + sizeof(int16_t) + sizeof(int16_t) + sizeof(uint32_t);

    void Serialize(void* pBuf)
    {
        PIA_SAMPLE_ASSERT(pBuf != NULL);
        PIA_SAMPLE_ASSERT(reinterpret_cast<uintptr_t>(pBuf) % 8 == 0);

        uint8_t* p = static_cast<uint8_t*>(pBuf);

        *reinterpret_cast<uint64_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork64(stationId);
        p += 8;
        *p = colorIndex;
        p += 1;
        *reinterpret_cast<int16_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork16(pos.x);
        p += 2;
        *reinterpret_cast<int16_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork16(pos.y);
        p += 2;
        *reinterpret_cast<uint32_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork32(latestIncrementNumber);
    }

    void Deserialize(const void* pData)
    {
        PIA_SAMPLE_ASSERT(pData != NULL);
        PIA_SAMPLE_ASSERT(reinterpret_cast<uintptr_t>(pData) % 8 == 0);

        const uint8_t* p = static_cast<const uint8_t*>(pData);

        stationId = nn::pia::common::ByteOrder::NetworkToHost64(*reinterpret_cast<const uint64_t*>(p));
        p += 8;
        colorIndex = *p;
        p += 1;
        pos.x = nn::pia::common::ByteOrder::NetworkToHost16(*reinterpret_cast<const int16_t*>(p));
        p += 2;
        pos.y = nn::pia::common::ByteOrder::NetworkToHost16(*reinterpret_cast<const int16_t*>(p));
        p += 2;
        latestIncrementNumber = nn::pia::common::ByteOrder::NetworkToHost32(*reinterpret_cast<const uint32_t*>(p));
    }

    void UpdateInfo()
    {
        // RTT の更新
        nn::pia::transport::Station* pStationBuf = nn::pia::transport::StationManager::GetInstance()->GetStation(stationId);
        if (PIA_IS_VALID_POINTER(pStationBuf))
        {
            // StationId に対応する Station インスタンスから RTT を取得
            rtt = pStationBuf->GetRtt();
        }
        else
        {
            // Station インスタンスが取得できなかった場合は無効な値にしておきます
            rtt = nn::pia::transport::Station::InvalidRtt;
        }
    }
};

// ステーションのステータス情報の配列
SampleStationStatus s_StationStatus[MaxParticipantNum];

// アンリライアブルプロトコルのハンドル
uint32_t s_UnreliableProtocolHandle = 0;

#if NNS_PIA_ENABLE_INET
const uint32_t SessionSystemPasswordLength = nn::pia::inet::SessionSystemPasswordLengthMax;
#else
const uint32_t SessionSystemPasswordLength = 32;
#endif
// セッションシステムパスワードを設定したセッション ID とパスワード
uint32_t s_SessionId = nn::pia::session::InvalidSessionId;
nn::pia::common::FixedString<SessionSystemPasswordLength> s_SystemPassword;

// ステーションインデックスに関連付ける矩形のカラー
const nns::pia::sampleframework::Color RectangleColorArray[] = {
    nns::pia::sampleframework::ColorBlack,
    nns::pia::sampleframework::ColorFuchsia,
    nns::pia::sampleframework::ColorGray,
    nns::pia::sampleframework::ColorWhite,
    nns::pia::sampleframework::ColorMaroon,
    nns::pia::sampleframework::ColorRed,
    nns::pia::sampleframework::ColorOlive,
    nns::pia::sampleframework::ColorYellow,
    nns::pia::sampleframework::ColorGreen,
    nns::pia::sampleframework::ColorLime,
    nns::pia::sampleframework::ColorTeal,
    nns::pia::sampleframework::ColorNavy,
    nns::pia::sampleframework::ColorPurple
};

// セッション内での役割画面表示用文字列
const char* SessionRole[] = {
    "Unknown",
    "Host",
    "Client"
};

// セッションのオープン状態を示す
enum SessionOpenStatus
{
    SessionOpenStatus_Unknown,
    SessionOpenStatus_Closing,
    SessionOpenStatus_Closed,
    SessionOpenStatus_Opening,
    SessionOpenStatus_Opened
};

// セッションのオープン状態の画面表示用文字列
const char* SessionOpenStatusString[] = {
    "Unknown",
    "Closing",
    "Closed",
    "Opening",
    "Opened"
};

SessionOpenStatus s_SessionOpenStatus = SessionOpenStatus_Unknown;

// UpdateSessionSettingAsync や OpenSessionAsync で使用するセッションの更新設定
const uint32_t UpdateSessionSettingNum = 3;
#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
nn::pia::local::UdsUpdateSessionSetting s_UdsUpdateSessionSetting[UpdateSessionSettingNum];
#elif NN_PIA_NINTENDOSDK
nn::pia::local::LdnUpdateSessionSetting s_LdnUpdateSessionSetting[UpdateSessionSettingNum];
#endif
#endif
#if NNS_PIA_ENABLE_INET
nn::pia::inet::NexUpdateSessionSetting s_NexUpdateSessionSetting[UpdateSessionSettingNum];
#endif
nn::pia::lan::LanUpdateSessionSetting s_LanUpdateSessionSetting[UpdateSessionSettingNum];
}  // end of anonymous namespace


// PiaTransport 初期化時に取得する UnreliableProtocolHandle をセットします
void SetUnreliableProtocolHandle(uint32_t handle)
{
    s_UnreliableProtocolHandle = handle;
}

uint32_t GetUnreliableProtocolHandle()
{
    return s_UnreliableProtocolHandle;
}

nn::pia::transport::UnreliableProtocol* GetUnreliableProtocol()
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    if (pTransport)
    {
        return pTransport->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_UnreliableProtocolHandle);
    }
    return NULL;
}

// セッション参加処理が長引いた場合に処理中に届く UnreliableProtocol のパケットでバッファが埋まるのを避けるためのコールバック
void SampleSessionCallbackForDispatch()
{
    PIA_ATTRIBUTE_ALIGN(8) static uint8_t receiveBuffer[nn::pia::transport::UnreliableProtocol::MaxDataSize];

    nn::pia::Result result;
    //受信した数は分からないので、バッファが空になるまで受信処理を繰り返します。
    do
    {
        nn::pia::StationId sourceStationId;
        uint32_t receivedDataSize = 0;

        result = GetUnreliableProtocol()->Receive(&sourceStationId, receiveBuffer, &receivedDataSize, sizeof(receiveBuffer));
        if (result == nn::pia::ResultNotInCommunication())
        {
            break;
        }
    } while (result != nn::pia::ResultNoData());
}

uint16_t GetStationStatusArrayIndex(nn::pia::StationId stationId)
{
    // 一致するものが無いかを最初に確認
    for (uint16_t i = 0; i < GetMaxParticipantNum(); i++)
    {
        if (s_StationStatus[i].stationId == stationId)
        {
            return i;
        }
    }
    // 一致するものが無い場合は空いてる要素を確認
    for (uint16_t i = 0; i < GetMaxParticipantNum(); i++)
    {
        if (s_StationStatus[i].stationId == nn::pia::StationIdInvalid)
        {
            return i;
        }
    }
    // ここに来ることはないはず
    return 0xffff;
}

void ActivateStationStatus(nn::pia::StationId stationId)
{
    nn::pia::Result result;
    uint16_t idx = GetStationStatusArrayIndex(stationId);
    PIA_SAMPLE_PRINT("ActivateStationStatus: idx=%d, stationId:%llu\n", idx, stationId);
    PIA_SAMPLE_ASSERT(idx < GetMaxParticipantNum());
    PIA_SAMPLE_PRINT("ActivateStationStatus: stationId : %ld sessionId: %d\n", stationId, nn::pia::session::Session::GetInstance()->GetSessionId(stationId));

#if NNS_PIA_ENABLE_LOCAL
    // ローカルマッチメイクの場合はステーション情報をログに出力する
    if (g_NetworkType == nns::pia::NetworkType_Local)
    {
#if NN_PIA_CTR
        nn::pia::common::StationAddress stationAddress;

        result = nn::pia::transport::StationManager::GetInstance()->GetStationAddress(&stationAddress, stationId);
        if (result.IsSuccess())
        {
            nn::pia::local::UdsStationInfo udsStationInfo;

            result = nn::pia::local::LocalFacade::GetInstance()->GetStationInfo(&udsStationInfo, stationAddress);
            if (result.IsSuccess())
            {
                char userNameString[nn::cfg::CFG_USER_NAME_LENGTH + 1];
                NN_PIA_UNUSED_IN_RELEASE(userNameString);
                for (int i = 0; i < nn::cfg::CFG_USER_NAME_LENGTH; ++i)
                {
                    if ((udsStationInfo.userName.userName[i] >= 0x0020 && udsStationInfo.userName.userName[i] < 0x007f) || udsStationInfo.userName.userName[i] == 0)
                    {
                        userNameString[i] = static_cast<char>(udsStationInfo.userName.userName[i]);
                    }
                    else
                    {
                        userNameString[i] = '?';
                    }
                }
                userNameString[nn::cfg::CFG_USER_NAME_LENGTH] = '\0';
                PIA_SAMPLE_PRINT("ActivateStationStatus: Role:%d Name:%s Code:%04x %04x %04x %04x %04x %04x\n",
                    udsStationInfo.GetRole(),
                    userNameString,
                    udsStationInfo.scrambledLocalFriendCode.value[0],
                    udsStationInfo.scrambledLocalFriendCode.value[1],
                    udsStationInfo.scrambledLocalFriendCode.value[2],
                    udsStationInfo.scrambledLocalFriendCode.value[3],
                    udsStationInfo.scrambledLocalFriendCode.value[4],
                    udsStationInfo.scrambledLocalFriendCode.value[5]);
            }
        }
#elif NN_PIA_NINTENDOSDK
#endif
    }
#endif

    if (idx < GetMaxParticipantNum())
    {
        //同一ステーションの接続イベントの通知は複数回来ないはず
        PIA_SAMPLE_ASSERT(s_StationStatus[idx].stationId == nn::pia::StationIdInvalid);

        nn::pia::StationIndex stationIndex;
        uint8_t colorIndex = 0;
        result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        if (result.IsSuccess())
        {
            colorIndex = static_cast<uint8_t>(stationIndex) + 1;
        }

        s_StationStatus[idx].stationId = stationId;
        s_StationStatus[idx].colorIndex = colorIndex;
        s_StationStatus[idx].pos.x = 0;
        s_StationStatus[idx].pos.y = FontSize * colorIndex;
        s_StationStatus[idx].latestIncrementNumber = 0;
        s_StationStatus[idx].rtt = nn::pia::transport::Station::InvalidRtt;

        nn::pia::transport::StationManager* pManager = nn::pia::transport::StationManager::GetInstance();
        PIA_SAMPLE_ASSERT(pManager != NULL);

        if (pManager)
        {
            nn::pia::transport::Station* pStation = pManager->GetStation(stationId);
            PIA_SAMPLE_ASSERT(pStation != NULL);

            if (pStation)
            {
                if (pStation->GetState() == nn::pia::transport::Station::StationState_Participating)
                {
                    result = pStation->GetIdentificationToken(&s_StationStatus[idx].token);
                    PIA_SAMPLE_ASSERT(result.IsSuccess());
                    for (int i = 0; i < nn::pia::MaxPlayerInfoSize; i++)
                    {
                        result = pStation->GetPlayerInfo(static_cast<uint8_t>(i), &s_StationStatus[idx].playerInfo[i]);
                        PIA_SAMPLE_ASSERT(result.IsSuccess());
                    }
                    result = pStation->GetPlayerCount(&s_StationStatus[idx].playerCount);
                    PIA_SAMPLE_ASSERT(result.IsSuccess());

                    PIA_SAMPLE_PRINT("token:%s\n", reinterpret_cast<char*>(s_StationStatus[idx].token.data));
                    for (int i = 0; i < nn::pia::MaxPlayerInfoSize; i++)
                    {
                        if (s_StationStatus[idx].playerInfo[i].screenName.GetStringLength() == 0 &&
                            s_StationStatus[idx].playerInfo[i].nameStringLanguage == 0xff)
                        {
                            PIA_SAMPLE_PRINT("playerInfo%d is not registered.\n", i);
                        }
                        else
                        {
                            // プレイヤー名を取得
                            char screenName[nn::pia::MaxPlayerNameLength + 1];

                            nns::pia::StringUtil::ConvertToAscii(screenName, nn::pia::MaxPlayerNameLength, s_StationStatus[idx].playerInfo[i].screenName);
                            screenName[nn::pia::MaxPlayerNameLength] = '\0';  // NULL 終端
                            PIA_SAMPLE_PRINT("playerInfo%d: %s\n", i, screenName);
                        }
                    }
                    PIA_SAMPLE_PRINT("playerCount: %d\n", s_StationStatus[idx].playerCount);
                }
                else
                {
                    PIA_SAMPLE_ASSERT(false);
                }
            }
        }
    }
}

void DeactivateStationStatus(nn::pia::StationId stationId)
{
    uint16_t idx = GetStationStatusArrayIndex(stationId);
    PIA_SAMPLE_PRINT("DeactivateStationStatus: idx=%d, stationId:%llu\n", idx, stationId);
    PIA_SAMPLE_ASSERT(idx < GetMaxParticipantNum());

    if (idx < GetMaxParticipantNum())
    {
        //同一ステーションの切断イベントの通知は複数回来ないはず
        PIA_SAMPLE_ASSERT(s_StationStatus[idx].stationId != nn::pia::StationIdInvalid);

        s_StationStatus[idx].stationId = nn::pia::StationIdInvalid;
        s_StationStatus[idx].colorIndex = 0;
        s_StationStatus[idx].pos.x = 0;
        s_StationStatus[idx].pos.y = 0;
        s_StationStatus[idx].latestIncrementNumber = 0;
        s_StationStatus[idx].rtt = nn::pia::transport::Station::InvalidRtt;

        memset(&s_StationStatus[idx].token, 0, sizeof(nn::pia::transport::Station::IdentificationToken));
        for (uint8_t i = 0; i < nn::pia::MaxPlayerInfoSize; i++)
        {
            s_StationStatus[idx].playerInfo[i].screenName.Clear();
            s_StationStatus[idx].playerInfo[i].nameStringLanguage = 0xff;
        }
        s_StationStatus[idx].playerCount = 0;
    }
}

bool CheckStationStatus()
{
    bool bResult = true;
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != pSession->End(); ++it)
    {
        uint16_t idx = GetStationStatusArrayIndex(*it);
        PIA_SAMPLE_PRINT("CheckStationStatus: StationIndex=%d, StationId:%llu(%llu)\n", idx, s_StationStatus[idx].stationId, *it);
        if (s_StationStatus[idx].stationId != *it)
        {
            bResult = false;
        }
    }
    return bResult;
}

bool CheckGetLocalAndHostStationId()
{
    bool bResult = true;
    nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
    if (localStationId == nn::pia::StationIdInvalid)
    {
        bResult = false;
    }
    else
    {
        nn::pia::transport::StationConnectionInfo stationConnectionInfo;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->GetLocalStationConnectionInfo(&stationConnectionInfo);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        NN_PIA_UNUSED(result);
        stationConnectionInfo.Trace(nn::pia::common::TraceFlagAlways);
    }
    nn::pia::StationId hostStationId = nn::pia::session::Session::GetInstance()->GetHostStationId();
    if (hostStationId == nn::pia::StationIdInvalid)
    {
        bResult = false;
    }
    else
    {
        nn::pia::transport::StationConnectionInfo stationConnectionInfo;
        nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->GetHostStationConnectionInfo(&stationConnectionInfo);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        NN_PIA_UNUSED(result);
        stationConnectionInfo.Trace(nn::pia::common::TraceFlagAlways);
    }
    PIA_SAMPLE_PRINT("CheckGetLocalAndHostStationId: Local=%llu, Host:%llu\n", localStationId, hostStationId);
    return bResult;
}

uint32_t GetSessionId()
{
    return s_SessionId;
}

const nn::pia::common::String& GetSystemPassword()
{
    return s_SystemPassword;
}

void PrintSystemPassword()
{
    PIA_SAMPLE_PRINT("-----------------------------------\n");
    PIA_SAMPLE_PRINT("s_SessionId : %d\n", s_SessionId);
    PIA_SAMPLE_PRINT("s_SystemPassword: Length=%d Bytes=%d EncType=%d\n",
                   s_SystemPassword.GetStringLength(),
                   s_SystemPassword.GetBytes(),
                   s_SystemPassword.GetEncodingType());
    const char* p = static_cast<const char*>(s_SystemPassword.GetBuffer());
    for (uint32_t i = 0; i < s_SystemPassword.GetBytes(); i++)
    {
        PIA_SAMPLE_PRINT("s_SystemPassword[%d] : %u\n", i, *p);
        ++p;
    }
    PIA_SAMPLE_PRINT("-----------------------------------\n");
}

void ResetRejoinSettings()
{
    PIA_SAMPLE_PRINT("Reset settings for rejoin.\n");
    s_SessionId = nn::pia::session::InvalidSessionId;
    s_SystemPassword.Clear();
}

int32_t GetSessionTime()
{
    int32_t time = -1;
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    if (pSession)
    {
        time = static_cast<int32_t>(pSession->GetTime());
    }
    return time;
}

// コールバック関数の宣言
void SampleSessionCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam)
{
    NN_PIA_UNUSED(pApplicationParam);
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    PIA_SAMPLE_PRINT("[%d] Sample Event Callback called. State:%d SessionId:%d StationNum:%d(%d) ClockPulse:%d\n",
                   GetSessionTime(),
                   pSession->GetStatus(),
                   pSession->GetSessionId(),
                   pSession->GetStationNum(),
                   pSession->GetValidStationIdNum(),
                   pSession->GetSyncClockPulseInterval());
    PIA_SAMPLE_PRINT("    LocalStationId:%llu HostStationId:%llu\n",
                     pSession->GetLocalStationId(),
                     pSession->GetHostStationId());

    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
    {
        PIA_SAMPLE_ASSERT(CheckGetLocalAndHostStationId());
        PIA_SAMPLE_PRINT("Sample Event JOIN detect. StationId(%llu)\n", stationId);
        ActivateStationStatus(stationId);
#if NN_PIA_NINTENDOSDK && NNS_PIA_ENABLE_FRIENDS
        if (stationId == pSession->GetLocalStationId())
        {
            DeclareOpenOnlinePlaySession();
        }
#endif
    }
    break;
    case nn::pia::session::Session::EventType_EventLeave:
    {
        PIA_SAMPLE_PRINT("Sample Event LEAVE detect. StationId(%llu)\n", stationId);
        DeactivateStationStatus(stationId);
#if NN_PIA_NINTENDOSDK && NNS_PIA_ENABLE_FRIENDS
        if (stationId == pSession->GetLocalStationId())
        {
            DeclareCloseOnlinePlaySession();
        }
#endif
    }
    break;
    case nn::pia::session::Session::EventType_SessionHostChanged:
    {
        PIA_SAMPLE_PRINT("Sample Event Host Change detect. StationId(%llu)\n", stationId);
        if (nn::pia::session::Session::GetInstance()->GetLocalStationId() == stationId)
        {
            if (!nn::pia::session::Session::GetInstance()->IsHost())
            {
                PIA_SAMPLE_PRINT("HostChangeEvent that localStaionId is host detect but IsHost is false.\n");
            }
        }

        nn::pia::session::Session::SessionOpenStatus openStatus = nn::pia::session::Session::GetInstance()->GetSessionOpenStatus();
        if (openStatus == nn::pia::session::Session::SessionOpenStatus_Open)
        {
            s_SessionOpenStatus = SessionOpenStatus_Opened;
        }
        else if (openStatus == nn::pia::session::Session::SessionOpenStatus_Close)
        {
            s_SessionOpenStatus = SessionOpenStatus_Closed;
        }
        else if (openStatus == nn::pia::session::Session::SessionOpenStatus_Unknown)
        {
            s_SessionOpenStatus = SessionOpenStatus_Unknown;
        }
#if PIA_TEST_CODE
        GetFramework()->SetProgramParam(ProgramParamStationId, (uint32_t)stationId);
#endif
    }
    break;
    case nn::pia::session::Session::EventType_HostMigrationFailed:
    {
        // ホストマイグレーション失敗での切断は CheckConnectionError() で検出するので、ここではログ出力のみにしておく
        PIA_SAMPLE_PRINT("Sample Event Host Migration Failed detect. StationId(%llu)\n", stationId);
    }
    break;
    case nn::pia::session::Session::EventType_SetSessionSystemPassword:
    {
        PIA_SAMPLE_PRINT("Sample Event System Password Set detect. StationId(%llu)\n", stationId);
        s_SessionId = nn::pia::session::Session::GetInstance()->GetSessionId();
        s_SystemPassword.Copy(pSession->GetSessionSystemPassword());
        PrintSystemPassword();
    }
    break;
    case nn::pia::session::Session::EventType_ClearSessionSystemPassword:
    {
        PIA_SAMPLE_PRINT("Sample Event System Password Clear detect. StationId(%llu)\n", stationId);
        ResetRejoinSettings();
    }
    break;
    default:
    {
    }
    break;
    }
}

bool SampleJoinApprovalCallback(const nn::pia::transport::Station::IdentificationToken* pToken)
{
    // ジョイン要求承認/拒否判定コールバック（セッションホストでのみ呼ばれる）
    bool isJoinApproval = false;

    // このサンプルでは、トークンデータの内容をチェックせず、設定されていたら承認と判定します。
    if (pToken != NULL)
    {
        uint8_t zeroBuffer[nn::pia::transport::Station::IdentificationTokenMaxDataSize];

        memset(zeroBuffer, 0, nn::pia::transport::Station::IdentificationTokenMaxDataSize);
        if (memcmp(pToken->data, zeroBuffer, nn::pia::transport::Station::IdentificationTokenMaxDataSize))
        {
            // 渡された要求元ステーションの識別トークンの中身をログ出力
            PIA_SAMPLE_PRINT("Sample Join Approval Callback RequesterToken Data:%s\n", pToken->data);
            isJoinApproval = true;
        }
    }
    PIA_SAMPLE_PRINT("Sample Join Approval Callback %s\n", (isJoinApproval) ? "Approved" : "Denied");

    return isJoinApproval;
}

// Session::DisconnectReason を取得し、ログ出力します。
void PrintSessionDisconnectReason()
{
    nn::pia::session::Session::DisconnectReason reason = nn::pia::session::Session::GetInstance()->GetSessionDisconnectReason();
    switch (reason)
    {
    case nn::pia::session::Session::DisconnectReason_NotYetCommunicated:
        PIA_SAMPLE_PRINT("DisconnectReason_NotYetCommunicated\n");
        break;
    case nn::pia::session::Session::DisconnectReason_OperationOfOwn:
        PIA_SAMPLE_PRINT("DisconnectReason_OperationOfOwn\n");
        break;
    case nn::pia::session::Session::DisconnectReason_OperationOfOther:
        PIA_SAMPLE_PRINT("DisconnectReason_OperationOfOther\n");
        break;
    case nn::pia::session::Session::DisconnectReason_KickoutByUser:
        PIA_SAMPLE_PRINT("DisconnectReason_KickoutByUser\n");
        break;
    case nn::pia::session::Session::DisconnectReason_KickoutBySystem:
        PIA_SAMPLE_PRINT("DisconnectReason_KickoutBySystem\n");
        break;
    case nn::pia::session::Session::DisconnectReason_InconsistentInfo:
        PIA_SAMPLE_PRINT("DisconnectReason_InconsistentInfo\n");
        break;
    case nn::pia::session::Session::DisconnectReason_MigrationFail:
        PIA_SAMPLE_PRINT("DisconnectReason_MigrationFail\n");
        break;
    case nn::pia::session::Session::DisconnectReason_ExternalFactor:
        PIA_SAMPLE_PRINT("DisconnectReason_ExternalFactor\n");
        break;
    case nn::pia::session::Session::DisconnectReason_MigrationFatalError:
        PIA_SAMPLE_PRINT("DisconnectReason_MigrationFatalError\n");
        break;
    case nn::pia::session::Session::DisconnectReason_UnknownReason:
    default:
        PIA_SAMPLE_PRINT("DisconnectReason_UnknownReason\n");
        break;
    }
}

// セッション内の相手と通信を行うシーンで、自分の情報を送信する間隔[frame]の設定です。
// 0 にすると毎ゲームフレーム送信します。
uint32_t GetSendingInterval(uint16_t stationNum)
{
#if NNS_PIA_ENABLE_INET
    if (g_NetworkType == nns::pia::NetworkType_Internet)
    {
        if (stationNum > 8)
        {
            // 20回/秒
            return 2;
        }
        else
        {
            // 30回/秒
            return 1;
        }
    }
#endif
    if (g_NetworkType == nns::pia::NetworkType_Lan)
    {
        if (stationNum > 8)
        {
            // 15回/秒
            return 3;
        }
        else
        {
            // 30回/秒
            return 1;
        }
    }
#if NNS_PIA_ENABLE_LOCAL
    if (g_NetworkType == nns::pia::NetworkType_Local)
    {
        // 30回 / 秒
        return 1;
    }
#endif
    return 8;
}

/*!
    @brief PiaTransport を使った通信を行うシーン。
    
    ホストがネットワークを終了すると自分はネットワーク選択シーンへ戻りますが、クライアントはエラーシーンへ進みます。
    
    @return 次に実行するべきシーン No を返します。
 */
SampleSceneNo SceneCommunication()
{
    bool isExit = false;

#if PIA_TEST_CODE
    int32_t sessionTime = -1;
#endif

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();

#if NNS_PIA_ENABLE_INET
    nn::pia::inet::NexSessionInfo nexSessionInfo;
#endif
#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
    nn::pia::local::UdsSessionInfo udsSessionInfo;
#elif NN_PIA_NINTENDOSDK
    nn::pia::local::LdnSessionInfo ldnSessionInfo;
#endif
#endif
    nn::pia::lan::LanSessionInfo lanSessionInfo;

    // UpdateSessionSettingAsync や OpenSessionAsync で使用するセッションの更新設定の準備
#if NNS_PIA_ENABLE_INET
    {
        uint8_t appData[4] = { 0x12, 0x34, 0x56, 0x78 };
        uint32_t attributes[nn::pia::inet::AttributeSizeMax] = { 0xa, 0xb, 0xc, 0xd, 0xe, 0xf };
        s_NexUpdateSessionSetting[0].Reset();
        for (uint32_t i = 0; i < nn::pia::inet::AttributeSizeMax; ++i)
        {
            s_NexUpdateSessionSetting[0].SetAttribute(i, attributes[i]);
        }

        s_NexUpdateSessionSetting[1].Reset();
        s_NexUpdateSessionSetting[1].SetApplicationData(appData, 4);

        s_NexUpdateSessionSetting[2].Reset();
        s_NexUpdateSessionSetting[2].SetMinParticipantNum(1);
        s_NexUpdateSessionSetting[2].SetMaxParticipantNum(GetMaxParticipantNum());
        attributes[0] = 0;
        attributes[1] = g_IsHostMigrationEnabled;
        attributes[2] = g_NetworkTopologyType;
        attributes[3] = 0;
        attributes[4] = 0;
        attributes[5] = 0;
        s_NexUpdateSessionSetting[2].SetAttributes(attributes);
        s_NexUpdateSessionSetting[2].SetSessionType(nn::pia::inet::SessionType_Anybody, true);
        s_NexUpdateSessionSetting[2].SetSessionUserPassword(SAMPLE_STRING("Update"));
        s_NexUpdateSessionSetting[2].SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[1]);
        s_NexUpdateSessionSetting[2].SetSelectionPriority(90);
        // 未来の日時に更新すると、サーバー側で現在の日時に更新されます
        //s_NexUpdateSessionSetting[2].SetStartedTime(nn::pia::common::DateTime(2035, 12, 31, 23, 59, 59));
        s_NexUpdateSessionSetting[2].SetStartedTime(nn::pia::common::DateTime(2016, 12, 31, 23, 59, 59));
    }
#endif
#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
    nn::pia::local::UdsUpdateSessionSetting* pLocalUpdateSessionSetting = &s_UdsUpdateSessionSetting[0];
#elif NN_PIA_NINTENDOSDK
    nn::pia::local::LdnUpdateSessionSetting* pLocalUpdateSessionSetting = &s_LdnUpdateSessionSetting[0];
#endif
    {
        SampleBeaconApplicationData appData;

        nn::pia::transport::Station::PlayerInfo playerInfo;
        nns::pia::Session_GetLocalPlayerInfo(&playerInfo, 1);
        nns::pia::StringUtil::ConvertToAscii(appData.playerName, nn::pia::MaxPlayerNameLength, playerInfo.screenName);

        ::std::memset(&appData, 0x00, sizeof(appData));
        const char updateStr[] = " Update session/Advanced";
        ::std::memcpy(appData.programName, updateStr, sizeof(updateStr));

        pLocalUpdateSessionSetting[0].Reset();
        pLocalUpdateSessionSetting[0].SetApplicationData(&appData, sizeof(appData));

        ::std::memset(&appData, 0x00, sizeof(appData));
        const char update2Str[] = " Update2 session/Advanced";
        ::std::memcpy(appData.programName, update2Str, sizeof(update2Str));

        pLocalUpdateSessionSetting[1].Reset();
        pLocalUpdateSessionSetting[1].SetApplicationData(&appData, sizeof(appData));

        ::std::memset(&appData, 0x00, sizeof(appData));
        ::std::memcpy(appData.programName, ProgramNameString, sizeof(ProgramNameString));

        pLocalUpdateSessionSetting[2].Reset();
        pLocalUpdateSessionSetting[2].SetApplicationData(&appData, sizeof(appData));
    }
#endif
    {
        uint8_t appData[4] = { 0x12, 0x34, 0x56, 0x78 };
        uint32_t attributes[nn::pia::lan::LanAttributeSizeMax] = { 0xa, 0xb, 0xc, 0xd, 0xe, 0xf };

        s_LanUpdateSessionSetting[0].Reset();
        for (uint32_t i = 0; i < nn::pia::lan::LanAttributeSizeMax; ++i)
        {
            s_LanUpdateSessionSetting[0].SetAttribute(i, attributes[i]);
        }
        s_LanUpdateSessionSetting[0].SetMinParticipantNum(1);
        s_LanUpdateSessionSetting[0].SetMaxParticipantNum(GetMaxParticipantNum()/2);
        s_LanUpdateSessionSetting[0].SetSessionType(nn::pia::lan::LanSessionType_Anybody);

        s_LanUpdateSessionSetting[1].Reset();
        s_LanUpdateSessionSetting[1].SetApplicationData(appData, 4);

        s_LanUpdateSessionSetting[2].Reset();
        s_LanUpdateSessionSetting[2].SetMaxParticipantNum(GetMaxParticipantNum());
        s_LanUpdateSessionSetting[2].SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
        s_LanUpdateSessionSetting[2].SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
    }

    nn::pia::transport::StationConnectionInfo localInfo;
    nn::pia::transport::StationConnectionInfo masterInfo;

    pTransport->GetLocalStationConnectionInfo(&localInfo);
    pTransport->GetHostStationConnectionInfo(&masterInfo);

    SampleSceneNo nextSceneNo = SampleSceneNo_SelectSessionType;

    bool isAsyncProcessInProgress = false;
    bool isLeaveSessionByUser = false;

    uint8_t updateType = 0;

    // ホストの場合、ビットレート判定機能の測定完了タイミングがセッション参加成功時ではないので、一度取得できるまでチェックするためのフラグ
    bool isCompletedMeasuringBitRate = false;
    if (!pSession->IsHost())
    {
        // クライアントの場合参加完了時に取得済み
        isCompletedMeasuringBitRate = true;
    }

    // セッション参加／拒否判定コールバックを設定します。
    pSession->RegisterJoiningApprovalCallback(SampleJoinApprovalCallback);

    GetFramework()->SetProgramStatus(SampleSceneNo_Communication, true, pSession->IsHost(), pSession->GetStationNum());

#if PIA_TEST_CODE
    if (g_IsLatencyEmulationEnabled)
    {
        // PiaTransport の送受信スレッドのレイテンシエミュレーション時間を指定します。
        nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
        PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
        pMgr->SetReceiveThreadLatencyEmulation(200, 200);
    }
#endif

    while (!isExit)
    {
        GetFramework()->BeginFrame();

        if (isAsyncProcessInProgress)
        {
            // PiaSession の非同期処理確認
            if (pSession->IsOpenSessionCompleted())
            {
                nn::pia::Result result = pSession->GetOpenSessionResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("Open Session Complete success\n");
                }
                else
                {
                    PIA_SAMPLE_PRINT("Open Session Complete failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                }
                isAsyncProcessInProgress = false;
            }
            if (pSession->IsCloseSessionCompleted())
            {
                nn::pia::Result result = pSession->GetCloseSessionResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("Close Session Complete success\n");
                }
                else
                {
                    PIA_SAMPLE_PRINT("Close Session Complete failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                }
                isAsyncProcessInProgress = false;
            }
            if (pSession->IsUpdateSessionSettingCompleted())
            {
                nn::pia::Result result = pSession->GetUpdateSessionSettingResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("UpdateSessionSetting Complete success\n");
                }
                else
                {
                    PIA_SAMPLE_PRINT("UpdateSessionSetting Complete failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                }
                isAsyncProcessInProgress = false;
            }
            if (pSession->IsGenerateSessionSystemPasswordCompleted())
            {
                nn::pia::Result result = pSession->GetGenerateSessionSystemPasswordResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("Generate System Password Complete success\n");
                }
                else
                {
                    PIA_SAMPLE_PRINT("Generate System Password Complete failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                }
                isAsyncProcessInProgress = false;
            }
            if (pSession->IsClearSystemPasswordCompleted())
            {
                nn::pia::Result result = pSession->GetClearSystemPasswordResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("Clear System Password Complete success\n");
                }
                else
                {
                    PIA_SAMPLE_PRINT("Clear System Password Complete failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                }
                isAsyncProcessInProgress = false;
            }
            if (pSession->IsRequestSessionInfoCompleted())
            {
                nn::pia::Result result = pSession->GetRequestSessionInfoResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("RequestSessionInfo Complete success\n");
#if NNS_PIA_ENABLE_INET
                    if (g_NetworkType == nns::pia::NetworkType_Internet)
                    {
                        nexSessionInfo.Copy(*(static_cast<const nn::pia::inet::NexSessionInfo*>(pSession->GetSessionInfo())));
                        nexSessionInfo.Trace(nn::pia::common::TraceFlagAlways);
                    }
#endif
#if NNS_PIA_ENABLE_LOCAL
                    if (g_NetworkType == nns::pia::NetworkType_Local)
                    {
#if NN_PIA_CTR
                        udsSessionInfo.Copy(*(static_cast<const nn::pia::local::UdsSessionInfo*>(pSession->GetSessionInfo())));
                        udsSessionInfo.Trace(nn::pia::common::TraceFlagAlways);
#elif NN_PIA_NINTENDOSDK
                        ldnSessionInfo.Copy(*(static_cast<const nn::pia::local::LdnSessionInfo*>(pSession->GetSessionInfo())));
                        ldnSessionInfo.Trace(nn::pia::common::TraceFlagAlways);
#endif
                    }
#endif
                    if (g_NetworkType == nns::pia::NetworkType_Lan)
                    {
                        lanSessionInfo.Copy(*(static_cast<const nn::pia::lan::LanSessionInfo*>(pSession->GetSessionInfo())));
                        lanSessionInfo.Trace(nn::pia::common::TraceFlagAlways);
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("RequestSessionInfo Complete failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                }
                isAsyncProcessInProgress = false;
            }
#if NN_PIA_CTR && !PIA_TEST_CODE && NNS_PIA_ENABLE_INET
            if (g_NetworkType == nns::pia::NetworkType_Internet)
            {
                if (nn::pia::inet::NexFacade::GetInstance()->IsAddToBlockListCompleted())
                {
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->GetAddToBlockListResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("AddToBlockList Complete success\n");
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("AddToBlockList Complete failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                    }
                    isAsyncProcessInProgress = false;
                }
                if (nn::pia::inet::NexFacade::GetInstance()->IsRemoveFromBlockListCompleted())
                {
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->GetRemoveFromBlockListResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("RemoveFromBlockList Complete success\n");
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("RemoveFromBlockList Complete failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                    }
                    isAsyncProcessInProgress = false;
                }
                if (nn::pia::inet::NexFacade::GetInstance()->IsClearBlockListCompleted())
                {
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->GetClearBlockListResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("ClearBlockList Complete success\n");
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("ClearBlockList Complete failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                    }
                    isAsyncProcessInProgress = false;
                }
                if (nn::pia::inet::NexFacade::GetInstance()->IsRequestBlockListCompleted())
                {
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->GetRequestBlockListResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("RequestMyBlockList Complete success\n");
                        nn::pia::PrincipalId principalIdArray[nn::pia::inet::PrincipalIdArraySizeMax];
                        uint32_t myBlockListSize;
                        nn::pia::inet::NexFacade::GetInstance()->GetBlockList(principalIdArray, nn::pia::inet::PrincipalIdArraySizeMax, &myBlockListSize);
                        PIA_SAMPLE_PRINT("MyBlockList size = %d\n", myBlockListSize);
                        for (uint32_t i = 0; i < myBlockListSize; ++i)
                        {
                            PIA_SAMPLE_PRINT("  %llu\n", principalIdArray[i]);
                        }
                    }
                    else
                    {
                        PIA_SAMPLE_PRINT("RequestMyBlockList Complete failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                    }
                    isAsyncProcessInProgress = false;
                }
            }
#endif // NN_PIA_CTR && !PIA_TEST_CODE && NNS_PIA_ENABLE_INET
        }
        else
        {
            // セッション状態チェック
            nn::pia::Result result = pSession->CheckConnectionError();
            if (result.IsFailure())
            {
                if (result == nn::pia::ResultSessionConnectionIsLost())
                {
                    PrintSessionDisconnectReason();
                    nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_SessionLost);
                }
                else
                {
                    nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_ConnectionLost);
                }
                isExit = true;
                GetFramework()->SetProgramStatus(SampleSceneNo_Communication, false);
            }
            else
            {
                GetFramework()->SetProgramStatus(SampleSceneNo_Communication, true, pSession->IsHost(), pSession->GetStationNum());

#if PIA_TEST_CODE
                nn::pia::session::Mesh* pMesh = nn::pia::session::Mesh::GetInstance();
                GetFramework()->SetProgramParam(ProgramParamMatchmakeSessionParticipant, pSession->GetMatchmakeSessionStationNum());
                GetFramework()->SetProgramParam(ProgramParamHostMigrationSequence, static_cast<uint32_t>(pMesh->GetHostMigrationSequence()));
                GetFramework()->SetProgramParam(ProgramParamHostMigrationState, reinterpret_cast<nn::pia::common::Job*>(pMesh->GetProcessHostMigrationJobPtr())->GetState());
#endif
            }
        }

        switch (pSession->GetSessionOpenStatus())
        {
        case nn::pia::session::Session::SessionOpenStatus_Open:
            s_SessionOpenStatus = SessionOpenStatus_Opened;
            break;
        case nn::pia::session::Session::SessionOpenStatus_Close:
            s_SessionOpenStatus = SessionOpenStatus_Closed;
            break;
        case nn::pia::session::Session::SessionOpenStatus_Unknown:
            s_SessionOpenStatus = SessionOpenStatus_Unknown;
            break;
        }

        //下画面描画処理
        {
            GetFramework()->ChangeRenderTarget(false);
            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);

            //文字列
            {
                //自分自身の役割チェック
                int roleIndex = 0;  //Unknown;
                if (pSession->GetLocalStationId() != nn::pia::StationIdInvalid)
                {
                    roleIndex = (pSession->IsHost() ? 1 : 2);  // 1:Host, 2:Client
                }

                int line = 0;
                GetFramework()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, g_FrameNo);
                line += FontSize;
                GetFramework()->DrawString(0, line, "  Scene: Communication");
                line += FontSize;
                GetFramework()->DrawString(0, line, "  Time=%08d SessionId=%d", GetSessionTime(), pSession->GetSessionId());
                line += FontSize;
                if (pSession->GetMatchmakeSessionStationNum() != 0)
                {
                    GetFramework()->DrawString(0, line, "  STA=%d PLY=%d MPT=%d MST=%d Type:%d", pSession->GetStationNum(), pSession->GetPlayerNum(), pSession->GetMatchmakeSessionParticipantNum(), pSession->GetMatchmakeSessionStationNum(), g_LocalMultiPlayType);
                }
                else
                {
                    GetFramework()->DrawString(0, line, "  Station=%d Player=%d", pSession->GetStationNum(), pSession->GetPlayerNum());
                }
                line += FontSize;
                GetFramework()->DrawString(0, line, "  Participation=%s Role=%s", SessionOpenStatusString[s_SessionOpenStatus], SessionRole[roleIndex]);
                line += FontSize;

                for (int i = 0; i < GetMaxParticipantNum(); i++)
                {
                    int xBuf = 10;
                    int yBuf = line + 10 * i;
                    GetFramework()->SetFontColor(RectangleColorArray[s_StationStatus[i].colorIndex]);
                    GetFramework()->DrawString(xBuf + GetFramework()->GetFontWidth() / 2 - 2, yBuf + GetFramework()->GetFontHeight() / 2 - 2, "*");
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);

                    if (s_StationStatus[i].stationId == nn::pia::StationIdInvalid)
                    {
                        GetFramework()->DrawString(xBuf + 10, yBuf, "No Connection");
                        continue;
                    }

                    s_StationStatus[i].UpdateInfo();
                    if (s_StationStatus[i].stationId == pSession->GetLocalStationId())
                    {
                        GetFramework()->DrawString(xBuf + 10, yBuf, "Id=%llu, RTT:%4d, Local", s_StationStatus[i].stationId, s_StationStatus[i].rtt);
                    }
                    else if (g_NetworkTopologyType == nn::pia::session::NetworkTopology_RelayMesh)
                    {
                        nn::pia::StationId relayId;

                        // 接続経路情報を取得
                        nn::pia::Result result = pSession->GetConnectionRoute(s_StationStatus[i].stationId, &relayId);
                        if (result.IsSuccess())
                        {
                            // 直接接続
                            if (s_StationStatus[i].stationId == relayId)
                            {
                                GetFramework()->DrawString(xBuf + 10, yBuf, "Id=%llu, RTT:%4d, Direct", s_StationStatus[i].stationId, s_StationStatus[i].rtt);
                            }
                            // リレー接続
                            else
                            {
                                GetFramework()->DrawString(xBuf + 10, yBuf, "Id=%llu, RTT:%4d, RelayId:%llu", s_StationStatus[i].stationId, s_StationStatus[i].rtt, relayId);
                            }
                        }
                    }
                    else
                    {
                        GetFramework()->DrawString(xBuf + 10, yBuf, "Id=%llu, RTT:%4d, %05X", s_StationStatus[i].stationId, s_StationStatus[i].rtt, s_StationStatus[i].latestIncrementNumber);
                    }
                }

#if PIA_TEST_CODE
                line = LowerScreenHeight - (FontSize * 11);
#else
                line = LowerScreenHeight - (FontSize * 10);
#endif
                GetFramework()->DrawString(0, line, "[ Network Information ]");
                line += FontSize;

                //通信関連データの画面表示（デバッグ用）
                pTransport->GetLocalStationConnectionInfo(&localInfo);
                pTransport->GetHostStationConnectionInfo(&masterInfo);

                nn::pia::common::FixedString<32> mainStr;
                nn::pia::common::FixedString<32> subStr;
                localInfo.GetMainLocation().GetStationAddress().GetInetAddress().GetAddressString(&mainStr);
                localInfo.GetSubLocation().GetStationAddress().GetInetAddress().GetAddressString(&subStr);
                GetFramework()->DrawString(0, line, "  Local Main:%s", mainStr.GetCharArray());
                line += FontSize;
                GetFramework()->DrawString(0, line, "         Sub:%s", subStr.GetCharArray());
                line += FontSize;
                masterInfo.GetMainLocation().GetStationAddress().GetInetAddress().GetAddressString(&mainStr);
                masterInfo.GetSubLocation().GetStationAddress().GetInetAddress().GetAddressString(&subStr);
                GetFramework()->DrawString(0, line, "  Host  Main:%s", mainStr.GetCharArray());
                line += FontSize;
                GetFramework()->DrawString(0, line, "         Sub:%s", subStr.GetCharArray());
                line += FontSize;

                GetFramework()->DrawString(0, line, "[ Manual ]");
                line += FontSize;
                GetFramework()->DrawString(0, line, "    B:LeaveSession");
                line += FontSize;
                GetFramework()->DrawString(0, line, "    Y:Request SessionInfo");
                line += FontSize;
                if (pSession->IsHost())
                {
                    if (g_NetworkType != nns::pia::NetworkType_Local)
                    {
                        GetFramework()->DrawString(0, line, "    X:Update SessionSetting");
                        line += FontSize;
                    }
                    if (s_SessionOpenStatus == SessionOpenStatus_Closed)
                    {
                        GetFramework()->DrawString(0, line, "    L:OpenSession");
                    }
                    else if (s_SessionOpenStatus == SessionOpenStatus_Opened)
                    {
                        GetFramework()->DrawString(0, line, "    L:CloseSession");
                    }
                    line += FontSize;
                    if (g_NetworkType == nns::pia::NetworkType_Internet)
                    {
                        GetFramework()->DrawString(0, line, "START:Generate/Clear SystemPassword");
                        line += FontSize;
                    }
                }
#if NN_PIA_CTR && !PIA_TEST_CODE && NNS_PIA_ENABLE_INET
                GetFramework()->DrawString(0, line, "    R:Get MyBlockList");
                line += FontSize;
                GetFramework()->DrawString(0, line, "    R+A:Add participants to MyBlockList");
                line += FontSize;
                GetFramework()->DrawString(0, line, "    R+LEFT:Remove participants from MyBlockList");
                line += FontSize;
                GetFramework()->DrawString(0, line, "    R+RIGHT:Clear MyBlockList");
                line += FontSize;
#endif
                GetFramework()->SwapBuffers();
            }
        }
        //上画面描画処理
        {
            GetFramework()->ChangeRenderTarget(true);

            for (uint16_t i = 0; i < GetMaxParticipantNum(); i++)
            {
                if (s_StationStatus[i].stationId != nn::pia::StationIdInvalid)
                {
                    int xBuf = s_StationStatus[i].pos.x;
                    int yBuf = s_StationStatus[i].pos.y;
                    GetFramework()->SetFontColor(RectangleColorArray[s_StationStatus[i].colorIndex]);
                    GetFramework()->DrawString(xBuf + GetFramework()->GetFontWidth() / 2 - 2, yBuf + GetFramework()->GetFontHeight() / 2 - 2, "*");
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                    GetFramework()->DrawString(xBuf + 10, yBuf, "<-ID:%llu (%d)", s_StationStatus[i].stationId, s_StationStatus[i].playerCount);

                    for (uint8_t j = 0; j < nn::pia::MaxPlayerInfoSize; j++)
                    {
                        if (s_StationStatus[i].playerInfo[j].screenName.GetStringLength() == 0 && s_StationStatus[i].playerInfo[j].nameStringLanguage == 0xff)
                        {
                            continue;
                        }
                        else
                        {
                            // プレイヤー名を取得
                            char screenName[nn::pia::MaxPlayerNameLength + 1];

                            nns::pia::StringUtil::ConvertToAscii(screenName, nn::pia::MaxPlayerNameLength, s_StationStatus[i].playerInfo[j].screenName);
                            screenName[nn::pia::MaxPlayerNameLength] = '\0';  // NULL 終端

                            yBuf += FontSize;
                            GetFramework()->DrawString(xBuf + 10, yBuf, "  %s", screenName);
                        }
                    }
                }
            }

            int line = UpperScreenHeight - FontSize;
            GetFramework()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
            line += FontSize;

            GetFramework()->SwapBuffers();
        }

        // Dispatch 処理
        Dispatch();
        if (!isAsyncProcessInProgress && !isExit)
        {
            // セッション状態チェック
            nn::pia::Result result = pSession->CheckConnectionError();
            if (result.IsFailure())
            {
                if (result == nn::pia::ResultSessionConnectionIsLost())
                {
                    PrintSessionDisconnectReason();
                    nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_SessionLost);
                }
                else
                {
                    nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_ConnectionLost);
                }
                isExit = true;
                GetFramework()->SetProgramStatus(SampleSceneNo_Communication, false);
            }
            else
            {
                GetFramework()->SetProgramStatus(SampleSceneNo_Communication, true, pSession->IsHost(), pSession->GetStationNum());
            }
        }

        // ビットレート判定機能による測定結果取得
        if (pSession->IsHost() && !isCompletedMeasuringBitRate)
        {
            int32_t bitRate = 0;
            if (pSession->IsBitRateCheckCompleted())
            {
                if (pSession->GetUplinkBitRate(&bitRate).IsSuccess())
                {
                    isCompletedMeasuringBitRate = true;
                    PIA_SAMPLE_PRINT("bitRate: %d\n", bitRate);
                }
            }
        }

#if PIA_TEST_CODE
        nn::pia::session::Mesh* pMesh = nn::pia::session::Mesh::GetInstance();
        GetFramework()->SetProgramParam(ProgramParamMatchmakeSessionParticipant, pSession->GetMatchmakeSessionStationNum());
        GetFramework()->SetProgramParam(ProgramParamIsCloseSessionCompleted, pSession->IsCloseSessionCompleted() ? 1 : 0);
        GetFramework()->SetProgramParam(ProgramParamHostMigrationSequence, static_cast<uint32_t>(pMesh->GetHostMigrationSequence()));
        GetFramework()->SetProgramParam(ProgramParamHostMigrationState, reinterpret_cast<nn::pia::common::Job*>(pMesh->GetProcessHostMigrationJobPtr())->GetState());
        GetFramework()->SetProgramParam(ProgramParamSessionTime, sessionTime);
        if (g_NetworkType == nns::pia::NetworkType_Lan)
        {
            GetFramework()->SetProgramParam(ProgramParamMinParticipant, lanSessionInfo.GetMinParticipantNum());
            GetFramework()->SetProgramParam(ProgramParamMaxParticipant, lanSessionInfo.GetMaxParticipantNum());
            for (uint32_t i = 0; i < nn::pia::lan::LanAttributeSizeMax; ++i)
            {
                uint32_t attrBuf;
                lanSessionInfo.GetAttribute(i, &attrBuf);
                GetFramework()->SetProgramParam(ProgramParamAttribute0 + i, attrBuf);
            }
            uint8_t appData[4] = { 0x12, 0x34, 0x56, 0x78 };
            uint8_t appDataBuf[nn::pia::lan::LanApplicationDataBufferSizeMax];
            lanSessionInfo.GetApplicationData(appDataBuf, nn::pia::lan::LanApplicationDataBufferSizeMax);
            GetFramework()->SetProgramParam(ProgramParamAppData, memcmp(appData, appDataBuf, 4));
        }
#if NNS_PIA_ENABLE_INET
        else if (g_NetworkType == nns::pia::NetworkType_Internet)
        {
            GetFramework()->SetProgramParam(ProgramParamMinParticipant, nexSessionInfo.GetMinParticipantNum());
            GetFramework()->SetProgramParam(ProgramParamMaxParticipant, nexSessionInfo.GetMaxParticipantNum());
            GetFramework()->SetProgramParam(ProgramParamSessionType, nexSessionInfo.GetSessionType());
            GetFramework()->SetProgramParam(ProgramParamAutoClose, nexSessionInfo.IsAutoCloseSession());
            for (uint32_t i = 0; i < nn::pia::inet::AttributeSizeMax; ++i)
            {
                uint32_t attrBuf;
                nexSessionInfo.GetAttribute(i, &attrBuf);
                GetFramework()->SetProgramParam(ProgramParamAttribute0 + i, attrBuf);
            }
            GetFramework()->SetProgramParam(ProgramParamUserPassword, nexSessionInfo.IsRestrictedByUserPassword());
            GetFramework()->SetProgramParam(ProgramParamPriority, nexSessionInfo.GetSelectionPriority());
            uint8_t appData[4] = { 0x12, 0x34, 0x56, 0x78 };
            uint8_t appDataBuf[nn::pia::inet::ApplicationDataBufferSizeMax];
            nexSessionInfo.GetApplicationData(appDataBuf, nn::pia::inet::ApplicationDataBufferSizeMax);
            GetFramework()->SetProgramParam(ProgramParamAppData, memcmp(appData, appDataBuf, 4));
            GetFramework()->SetProgramParam(ProgramParamStartYear, nexSessionInfo.GetStartedTime().GetYear());
            GetFramework()->SetProgramParam(ProgramParamStartMonth, nexSessionInfo.GetStartedTime().GetMonth());
            GetFramework()->SetProgramParam(ProgramParamStartDay, nexSessionInfo.GetStartedTime().GetDay());
            GetFramework()->SetProgramParam(ProgramParamStartHour, nexSessionInfo.GetStartedTime().GetHour());
            GetFramework()->SetProgramParam(ProgramParamStartMin, nexSessionInfo.GetStartedTime().GetMinute());
            GetFramework()->SetProgramParam(ProgramParamStartSec, nexSessionInfo.GetStartedTime().GetSecond());
        }
#endif
#if NNS_PIA_ENABLE_LOCAL
        else if (g_NetworkType == nns::pia::NetworkType_Local)
        {
            SampleBeaconApplicationData appData;
            const char updateStr[] = " Update session/Advanced";
#if NN_PIA_CTR
            udsSessionInfo.GetApplicationData(&appData, sizeof(SampleBeaconApplicationData));
#elif NN_PIA_NINTENDOSDK
            ldnSessionInfo.GetApplicationData(&appData, sizeof(SampleBeaconApplicationData));
#endif
            GetFramework()->SetProgramParam(ProgramParamAppData, memcmp(appData.programName, updateStr, sizeof(updateStr)));
        }
#endif
#endif

        nn::pia::StationId myStationId = pSession->GetLocalStationId();

        //受信処理
        // Scheduler::Dispatch() で受信処理が行われる都合上、 Dispatch（１回目）の後に実行
        if (isExit == false)
        {
            // 相手が送信したデータのサイズは基本的に想定できないので、UnreliableProtocol の MAX_DATA_SIZE
            // 分の受信バッファを確保します。
            PIA_ATTRIBUTE_ALIGN(8) static uint8_t receiveBuffer[nn::pia::transport::UnreliableProtocol::MaxDataSize];

            nn::pia::Result result;
            //受信した数は分からないので、バッファが空になるまで受信処理を繰り返します。
            do
            {
                nn::pia::StationId sourceStationId;
                uint32_t receivedDataSize = 0;

                result = GetUnreliableProtocol()->Receive(&sourceStationId, receiveBuffer, &receivedDataSize, sizeof(receiveBuffer));
                if (result.IsSuccess())
                {
                    uint16_t id = GetStationStatusArrayIndex(sourceStationId);
                    if (id < GetMaxParticipantNum())
                    {
                        //受信したデータが正しいかをチェックする必要があるかもしれません
                        s_StationStatus[id].Deserialize(&receiveBuffer);
                    }
                }
                else if (result == nn::pia::ResultNotInCommunication())
                {
                    //通信切断による受信失敗
                    //直前の Dispatch 処理で非同期のセッション破棄／離脱処理が終了した場合に発生することがあります
                    //正常状態であれば、次ゲームフレーム開始時の Session の状態チェックで DISCONNECTED になっています
                    PIA_SAMPLE_PRINT("UnreliableProtocol::Receive failed. (IsNotInCommunication)\n");
                    break;
                }
                else if (result != nn::pia::ResultNoData())
                {
                    //その他の受信失敗
                    //このデモではあまり気にしない
                }
            } while (result != nn::pia::ResultNoData());
        }

        int move_x = 0;  //自身の矩形移動量(x)
        int move_y = 0;  //自身の矩形移動量(y)

#if PIA_TEST_CODE
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            nn::pia::transport::ThreadStreamManager* pThreadStreamManager = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(pThreadStreamManager);  //transport の初期化が終わっていればインスタンスが生成されているはず

            int32_t ratio = (pThreadStreamManager->GetSendThreadPacketLossRatio() == 100) ? 0 : 100;
            pThreadStreamManager->SetSendThreadPacketLossRatio(ratio);
            pThreadStreamManager->SetReceiveThreadPacketLossRatio(ratio);
            PIA_SAMPLE_PRINT("Packet dropped emulation ratio (send) = %d\n", ratio);
            PIA_SAMPLE_PRINT("Packet dropped emulation ratio (recv) = %d\n", ratio);
        }
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            sessionTime = GetSessionTime();
            PIA_SAMPLE_PRINT("SessionTime = %d\n", sessionTime);
        }
#endif

        if (!pSession->IsAsyncProcessInProgress())
        {
            // セッションから離脱
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                //ユーザー操作で離脱する場合は再参加情報をクリアしてから離脱
                ResetRejoinSettings();
#if NNS_PIA_ENABLE_FRIENDS
                ResetGameMode();
#endif
                isLeaveSessionByUser = true;
                isExit = true;
            }
            //ユーザー操作のハンドリング
            // セッションのオープン／クローズ
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonL) && pSession->IsHost())
            {
                if (s_SessionOpenStatus == SessionOpenStatus_Opened)
                {
                    PIA_SAMPLE_PRINT("CloseSessionAsync is called.\n");
                    nn::pia::Result result = pSession->CloseSessionAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("CloseSession failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        s_SessionOpenStatus = SessionOpenStatus_Closing;
                        isAsyncProcessInProgress = true;
                    }
                }
                else if (s_SessionOpenStatus == SessionOpenStatus_Closed)
                {
                    PIA_SAMPLE_PRINT("OpenSessionAsync is called.\n");
                    nn::pia::session::UpdateSessionSetting* pUpdateSessionSetting = NULL;
#if NNS_PIA_ENABLE_INET
                    nn::pia::inet::NexUpdateSessionSetting nexUpdateSessionSetting;
                    if (g_NetworkType == nns::pia::NetworkType_Internet)
                    {
                        uint32_t attributes[] = { 0, g_IsHostMigrationEnabled, g_NetworkTopologyType, 0, 0, 0 };
                        nexUpdateSessionSetting.SetAttributes(attributes);
                        nexUpdateSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody, true);
                        nexUpdateSessionSetting.SetSessionUserPassword(SAMPLE_STRING(""));
                        nexUpdateSessionSetting.SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[0]);
                        pUpdateSessionSetting = &nexUpdateSessionSetting;
                    }
#endif
                    nn::pia::lan::LanUpdateSessionSetting lanUpdateSessionSetting;
                    if (g_NetworkType == nns::pia::NetworkType_Lan)
                    {
                        lanUpdateSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
                        lanUpdateSessionSetting.SetAttribute(0, 0);
                        lanUpdateSessionSetting.SetAttribute(1, g_IsHostMigrationEnabled);
                        lanUpdateSessionSetting.SetAttribute(2, g_NetworkTopologyType);
                        lanUpdateSessionSetting.SetAttribute(3, 0);
                        lanUpdateSessionSetting.SetAttribute(4, 0);
                        lanUpdateSessionSetting.SetAttribute(5, 0);
                        pUpdateSessionSetting = &lanUpdateSessionSetting;
                    }
#if NNS_PIA_ENABLE_LOCAL
                    if (g_NetworkType == nns::pia::NetworkType_Local)
                    {
#if NN_PIA_CTR
                        pUpdateSessionSetting = &s_UdsUpdateSessionSetting[2];
#elif NN_PIA_NINTENDOSDK
                        pUpdateSessionSetting = &s_LdnUpdateSessionSetting[2];
#endif                    
                    }
#endif
                    // セッションのオープンと同時にセッションの設定を更新します
                    nn::pia::Result result = pSession->OpenSessionAsync(pUpdateSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("OpenSession failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        s_SessionOpenStatus = SessionOpenStatus_Opening;
                        isAsyncProcessInProgress = true;
                    }
                    updateType = (updateType + 1) % UpdateSessionSettingNum;
                }
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                PIA_SAMPLE_PRINT("RequestSessionInfoAsync is called.\n");
                nn::pia::Result result = pSession->RequestSessionInfoAsync();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT_RESULT(result);
                    PIA_SAMPLE_PRINT("RequestSessionInfo failed\n");
                    nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                    isExit = true;
                }
                else
                {
                    isAsyncProcessInProgress = true;
                }
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonX) && pSession->IsHost())
            {
                PIA_SAMPLE_PRINT("UpdateSessionSettingAsync is called.\n");
                nn::pia::session::UpdateSessionSetting* pUpdateSessionSetting = NULL;
#if NNS_PIA_ENABLE_INET
                if (g_NetworkType == nns::pia::NetworkType_Internet)
                {
                    pUpdateSessionSetting = &s_NexUpdateSessionSetting[updateType];
                }
#endif
                if (g_NetworkType == nns::pia::NetworkType_Lan)
                {
                    pUpdateSessionSetting = &s_LanUpdateSessionSetting[updateType];
                }
#if NNS_PIA_ENABLE_LOCAL
                if (g_NetworkType == nns::pia::NetworkType_Local)
                {
#if NN_PIA_CTR
                    pUpdateSessionSetting = &s_UdsUpdateSessionSetting[updateType];
#elif NN_PIA_NINTENDOSDK
                    pUpdateSessionSetting = &s_LdnUpdateSessionSetting[updateType];
#endif                    
                }
#endif
                // セッションの設定を更新します（３つの更新設定を順に適用します）
                // ネットワークタイプがインターネットの場合、更新設定に「あいことば」が含まれます
                // 検索時に一致する「あいことば」を指定することで検索できます
                nn::pia::Result result = pSession->UpdateSessionSettingAsync(pUpdateSessionSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT_RESULT(result);
                    PIA_SAMPLE_PRINT("UpdateSessionSetting failed\n");
                    nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                    isExit = true;
                }
                else
                {
                    isAsyncProcessInProgress = true;
                }
                updateType = (updateType + 1) % UpdateSessionSettingNum;
            }

            // システムパスワードの設定／解除
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonStart) && pSession->IsHost() && g_NetworkType == nns::pia::NetworkType_Internet)
            {
                if (s_SessionId == nn::pia::session::InvalidSessionId)
                {
                    PIA_SAMPLE_PRINT("GenerateSessionSystemPasswwordAsync is called.\n");
                    nn::pia::Result result = pSession->GenerateSessionSystemPasswordAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("GenerateSystemPassword failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        isAsyncProcessInProgress = true;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("ClearSystemPasswwordAsync is called.\n");
                    nn::pia::Result result = pSession->ClearSessionSystemPasswordAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("ClearSystemPassword failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        isAsyncProcessInProgress = true;
                    }
                }
            }
#if NN_PIA_CTR && !PIA_TEST_CODE && NNS_PIA_ENABLE_INET
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonR)&& g_NetworkType == nns::pia::NetworkType_Internet)
            {
                if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonA))
                {
                    PIA_SAMPLE_PRINT("AddToBlockListAsync is called.\n");
                    nn::pia::PrincipalId principalIdArray[nn::pia::inet::PrincipalIdArraySizeMax];
                    uint32_t principalIdArraySize = 0;
                    for (uint32_t i = 0; i < GetMaxParticipantNum(); ++i)
                    {
                        if (s_StationStatus[i].stationId != nn::pia::StationIdInvalid &&
                            s_StationStatus[i].stationId != pSession->GetLocalStationId() &&
                            principalIdArraySize < nn::pia::inet::PrincipalIdArraySizeMax)
                        {
                            principalIdArray[principalIdArraySize] = s_StationStatus[i].stationId;
                            ++principalIdArraySize;
                        }
                    }
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->AddToBlockListAsync(principalIdArray, principalIdArraySize);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("AddToBlockListAsync failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        isAsyncProcessInProgress = true;
                    }
                }
                else if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonLeft))
                {
                    PIA_SAMPLE_PRINT("RemoveFromBlockListAsync is called.\n");
                    nn::pia::PrincipalId principalIdArray[nn::pia::inet::PrincipalIdArraySizeMax];
                    uint32_t principalIdArraySize = 0;
                    for (uint32_t i = 0; i < GetMaxParticipantNum(); ++i)
                    {
                        if (s_StationStatus[i].stationId != nn::pia::StationIdInvalid &&
                            s_StationStatus[i].stationId != pSession->GetLocalStationId() &&
                            principalIdArraySize < nn::pia::inet::PrincipalIdArraySizeMax)
                        {
                            principalIdArray[principalIdArraySize] = s_StationStatus[i].stationId;
                            ++principalIdArraySize;
                        }
                    }
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->RemoveFromBlockListAsync(principalIdArray, principalIdArraySize);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("RemoveFromBlockListAsync failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        isAsyncProcessInProgress = true;
                    }
                }
                else if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonRight))
                {
                    PIA_SAMPLE_PRINT("ClearBlockListAsync is called.\n");
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->ClearBlockListAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("ClearBlockListAsync failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        isAsyncProcessInProgress = true;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("RequestBlockListAsync is called.\n");
                    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->RequestBlockListAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT_RESULT(result);
                        PIA_SAMPLE_PRINT("RequestBlockListAsync failed\n");
                        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
                        isExit = true;
                    }
                    else
                    {
                        isAsyncProcessInProgress = true;
                    }
                }
            }
#endif
        }
        //矩形の移動
        {
            if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonLeft))
            {
                move_x = -1;
            }
            if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonRight))
            {
                move_x = 1;
            }
            if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonUp))
            {
                move_y = -1;
            }
            if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonDown))
            {
                move_y = 1;
            }
        }

        //送信処理 当然排他!
        // Scheduler::Dispatch() で送信処理が行われる都合上、 Dispatch（２回目）の前に実行
        if (isExit == false)
        {
            uint16_t id = GetStationStatusArrayIndex(myStationId);
            if (id < GetMaxParticipantNum())
            {
                //自身のデータを更新
                if (move_x != 0)
                {
                    s_StationStatus[id].pos.x = (s_StationStatus[id].pos.x + UpperScreenWidth + move_x) % UpperScreenWidth;
                }
                else
                {
                    //操作していない場合にも、常時通信が行われていることを示すため 矩形を動かします
                    s_StationStatus[id].pos.x = (s_StationStatus[id].pos.x + UpperScreenWidth + 1) % UpperScreenWidth;
                }
                if (move_y != 0)
                {
                    s_StationStatus[id].pos.y = (s_StationStatus[id].pos.y + UpperScreenHeight + move_y) % UpperScreenHeight;
                }
                s_StationStatus[id].latestIncrementNumber = g_FrameNo;

                uint16_t stationNum = pSession->GetStationNum();
                if ((g_FrameNo % (GetSendingInterval(stationNum) + 1) == 0))
                {
                    if (stationNum > 1)
                    {
                        // SendToAll を使って一斉送信
                        PIA_ATTRIBUTE_ALIGN(8) uint8_t sendDataBuffer[SampleStationStatus::SerializedSize];
                        s_StationStatus[id].Serialize(sendDataBuffer);
                        if (GetUnreliableProtocol()->SendToAll(sendDataBuffer, SampleStationStatus::SerializedSize).IsFailure())
                        {
                            //送信失敗。
                            //このデモではあまり気にしない
                            //PIA_SAMPLE_PRINT("Send Fail\n");
                        }
                    }
                }
            }
        }

        // Dispatch 処理（２回目：送信処理用）
        Dispatch();

        GetFramework()->EndFrame();
        g_FrameNo++;
    }

    // ユーザー操作による離脱
    if (isLeaveSessionByUser)
    {
        nn::pia::Result result = nns::pia::Session_LeaveSession();
        if (result.IsFailure())
        {
            nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Leave);
        }
    }

    pSession->UnregisterJoiningApprovalCallback();

#if PIA_TEST_CODE
    GetFramework()->SetProgramParam(
        ProgramParamMatchmakeSessionParticipant,
        pSession->GetMatchmakeSessionStationNum());
    {
        nn::pia::transport::ThreadStreamManager* pThreadStreamManager = nn::pia::transport::ThreadStreamManager::GetInstance();
        PIA_SAMPLE_ASSERT(pThreadStreamManager);  //transport の初期化が終わっていればインスタンスが生成されているはず

        int32_t ratio = 0;
        pThreadStreamManager->SetSendThreadPacketLossRatio(ratio);
        pThreadStreamManager->SetReceiveThreadPacketLossRatio(ratio);
        PIA_SAMPLE_PRINT("Packet dropped emulation ratio (send) = %d\n", ratio);
        PIA_SAMPLE_PRINT("Packet dropped emulation ratio (recv) = %d\n", ratio);
    }
#endif

    // ユーザー操作による離脱が成功した場合はここでクリーンアップを行っておく
    if (isLeaveSessionByUser && nextSceneNo == SampleSceneNo_SelectSessionType)
    {
        CleanupPia();
    }

    return nextSceneNo;
}
