﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include "SampleConfig.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Inet_Setup.h"
#if NNS_PIA_ENABLE_LOCAL
#include "SampleLib/Local_Setup.h"
#endif
#include "SampleLib/Session_Setup.h"
#if NN_PIA_NINTENDOSDK
#include <nn/err.h>
#endif

namespace
{
SampleErrorType s_ErrorType = SampleErrorType_Unknown;  //直前に発生したエラーを保持しておく変数

const char* ErrorInfoString[] =
    {
        "Unknown Reason",
        "Initialize Error (Network IF up)",
        "Initialize Error (Account)",
        "Initialize Error (other reason)",
        "Login is failed",
        "Connection is lost",
        "Connection is lost",
        "Session is lost",
        "StartupSession is failed",
        "CreateSession is failed",
        "BrowseSession is failed",
        "JoinSession is failed",
        "LeaveSession is failed",
        "DestroySession is failed",
        "JoinCommunity is failed"
    };

#if NNS_PIA_ENABLE_FRIENDS
uint32_t s_FriendResult = 0;
#endif
nn::pia::Result s_PiaResult = nn::pia::ResultSuccess();
}

nn::pia::Result::HandlingType GetNetworkErrorHandlingType()
{
    if (s_PiaResult.IsFailure())
    {
        return s_PiaResult.GetHandlingType();
    }
#if NNS_PIA_ENABLE_FRIENDS
    else if (s_FriendResult != 0)
    {
        return nn::pia::Result::HandlingType_Finalize;
    }
#endif
    else
    {
        return nn::pia::Result::HandlingType_Void;
    }
}

/*!
    @brief エラーシーンに飛ぶ前にエラー理由をセットする関数です。
 */
void SetCommunicationErrorType(SampleErrorType error)
{
    s_ErrorType = error;
}

/*!
    @brief 通信エラーなど復帰可能なエラーが発生した場合にエラー内容を表示するシーンです。
    
    エラー発生時に収集したエラー情報を表示するだけの処理をしています。
 */
SampleSceneNo SceneError()
{
    bool isExit = false;

    GetFramework()->SetProgramStatus(SampleSceneNo_Error, false);
#if NNS_PIA_ENABLE_FRIENDS
    ResetGameMode();
#endif

    // メインループ
    while (isExit == false)
    {
        // アカウントが無かった場合はメッセージを表示せずに先に進める
        if (s_ErrorType == SampleErrorType_InitializeByAccount)
        {
            break;
        }
        // キャンセルの場合はメッセージを表示せずに先に進める
        if (s_PiaResult.IsFailure() && s_PiaResult == nn::pia::ResultCancelled())
        {
            break;
        }
    
        GetFramework()->BeginFrame();

        //ユーザー操作を処理 ※パッド情報の収集は BeginFrame() で行われている
        {
            // A:    Restart
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                isExit = true;
            }
        }

        // 描画処理
        int line;

        // 下画面(通常)
        GetFramework()->ChangeRenderTarget(false);
        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);

        line = 0;
        GetFramework()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, g_FrameNo);
        line += FontSize;
        GetFramework()->DrawString(0, line, " Scene: A Communication Error Occurred.");
        line += FontSize;
        line += FontSize;

        GetFramework()->DrawString(0, line, "  %s", ErrorInfoString[s_ErrorType]);
        line += FontSize;

        line = LowerScreenHeight - (FontSize * 2);
        GetFramework()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  A: Exit and Restart.");
        line += FontSize;

        GetFramework()->SwapBuffers();

        // 上画面
        GetFramework()->ChangeRenderTarget(true);

        line = UpperScreenHeight - FontSize;
        GetFramework()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
        line += FontSize;

        GetFramework()->SwapBuffers();

        GetFramework()->EndFrame();

        // Dispatch 処理
        Dispatch();
        g_FrameNo++;
    }

    // エラー情報がクリアされる前にハンドリングタイプを取得しておく
    nn::pia::Result::HandlingType type = GetNetworkErrorHandlingType();
    SampleErrorType errorType = s_ErrorType;

    //念のためエラー情報をクリアしておく
    s_ErrorType = SampleErrorType_Unknown;
    ClearErrorResult();

    return HandlingErrorResult(type, errorType);
}


void ClearErrorResult()
{
    s_PiaResult = nn::pia::ResultSuccess();
#if NNS_PIA_ENABLE_FRIENDS
    s_FriendResult = 0;
#endif
}


// インターネット接続時に発生したエラーのResultを渡してください。
// ここで渡されたResultがネットワークエラーコード変換APIに渡され、
// 画面に表示されます。
void SetPiaErrorResult(nn::pia::Result result)
{
    s_PiaResult = result;
}

#if NNS_PIA_ENABLE_FRIENDS
void SetFriendErrorResult(uint32_t result)
{
    s_FriendResult = result;
}
#endif

uint32_t GetNetworkErrorCode(void)
{
    // Pia の Result の方が優先
    if (s_PiaResult.IsFailure())
    {
        return s_PiaResult.GetErrorCode();
    }
#if NNS_PIA_ENABLE_FRIENDS
    else if (s_FriendResult != 0)
    {
        return s_FriendResult;
    }
#endif
    else
    {
        return 0;
    }
}

/*!
    @brief インターネット接続環境でエラーが発生した場合に、そのネットワークエラーコードを表示するシーンです。
    再度ログインからやり直す必要があるエラーが発生した場合にこのシーンへ遷移します。
    本来はネットワークエラーコードはエラー・EULAアプレットに渡してください。
 */
SampleSceneNo SceneNetworkError()
{
    GetFramework()->SetProgramStatus(SampleSceneNo_NetworkError, false);
#if NNS_PIA_ENABLE_FRIENDS
    ResetGameMode();
#endif

    // ネットワークエラーコードを取得します。
    // ここで得られたコードは、エラー・EULAアプレットに渡されるべきですが、
    // CTR の場合、エラー・EULAアプレットは開発機材が prod モードに設定されていないと
    // 利用できませんので、利便性を考慮し、本サンプルデモではエラー・EULAアプレットを
    // 使用していません。
    uint32_t networkErrorCode = GetNetworkErrorCode();

    bool isExit = false;

    // 描画
    while (isExit == false)
    {
        GetFramework()->BeginFrame();

        // Aボタンでループ脱出
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            isExit = true;
        }

        int line;

        // 下画面描画
        GetFramework()->ChangeRenderTarget(false);
        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);

        line = 0;
        GetFramework()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, g_FrameNo);
        line += FontSize;
        GetFramework()->DrawString(0, line, " Scene: A Network Error Occurred.");
        line += FontSize;
        line += FontSize;

        GetFramework()->DrawString(0, line, "  %s", ErrorInfoString[s_ErrorType]);
        line += FontSize;
        line += FontSize;

#if NN_PIA_NINTENDOSDK && !PIA_TEST_CODE
        if (s_PiaResult.IsFailure())
        {
            nn::err::ErrorCode errCode;
            if (s_PiaResult.GetErrorCode(&errCode).IsSuccess())
            {
                nn::err::ShowError(errCode);
                isExit = true;
                networkErrorCode = 0;
            }
        }
#endif
        if (networkErrorCode != 0)
        {
            GetFramework()->DrawString(0, line, "  Network error code: %08d", networkErrorCode);
            line += FontSize;
        }

#if NNS_PIA_ENABLE_INET
#if NN_PIA_NINTENDOSDK
        if (g_LoginError.IsFailure())
        {
#if PIA_TEST_CODE
            GetFramework()->DrawString(0, line, "  Login error code: %08d", g_LoginError.GetInnerValueForDebug());
            line += FontSize;
#else
            // ログインエラーはエラービューアアプレットで表示済みなので、次のシーンに進む
            isExit = true;
#endif // PIA_TEST_CODE
        }
#else
        if (g_LoginError != 0)
        {
            GetFramework()->DrawString(0, line, "  Login error code: %08d", g_LoginError);
            line += FontSize;
        }
#endif // NN_PIA_NINTENDOSDK
#endif // NNS_PIA_ENABLE_INET

        line = LowerScreenHeight - (FontSize * 2);
        GetFramework()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  A: Exit and Restart.");
        line += FontSize;

        GetFramework()->SwapBuffers();

        // 上画面
        GetFramework()->ChangeRenderTarget(true);

        line = UpperScreenHeight - FontSize;
        GetFramework()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
        line += FontSize;

        GetFramework()->SwapBuffers();

        GetFramework()->EndFrame();

        // Dispatch 処理
        Dispatch();

        g_FrameNo++;
    }

    // エラー情報がクリアされる前にハンドリングタイプを取得しておく
    nn::pia::Result::HandlingType type = GetNetworkErrorHandlingType();
    SampleErrorType errorType = s_ErrorType;

    //念のためエラー情報をクリアしておく
    s_ErrorType = SampleErrorType_Unknown;
    ClearErrorResult();

    return HandlingErrorResult(type, errorType);
}
