﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Session_Setup.h"
#include <nn/pia/common/common_FixedObjList.h>

#if NNS_PIA_ENABLE_FRIENDS && NN_PIA_CTR

#include <nn/friends.h>

typedef nn::pia::common::FixedObjList<nn::friends::FriendKey, MaxFriendSessionNum> OnlineFriendsList;

namespace
{
const uint32_t FrinedMatchmakeGameMode = 0;
const uint64_t FrinedMatchmakeGameModeMask = 0x0000000000000001;
const uint32_t FriendJoinAvailabilityJoinable = nn::friends::JOIN_AVAILABILITY_JOINABLE;
const uint32_t FriendJoinAvailabilityNotJoinable = nn::friends::JOIN_AVAILABILITY_NOT_JOINABLE;
const uint32_t GameModeType_Invalid = nn::friends::CTR::MATCHMAKE_SYSTEM_TYPE_INVALID;
#if NN_PIA_ENABLE_SESSION_TYPE_FRIEND
const uint32_t GameModeType_Friends = nn::friends::CTR::MATCHMAKE_SYSTEM_TYPE_FRIEND;
#else
const uint32_t GameModeType_Friends = nn::friends::CTR::MATCHMAKE_SYSTEM_TYPE_ANYBODY;
#endif

nn::friends::GameMode s_FriendGameMode;
nn::friends::FriendKey s_FriendKeyForJoinIn;
bool s_IsFromFriendList = false;
OnlineFriendsList s_OnlineFriendsList;
}

// フレンドリストからの起動であるかどうかを判定
void CheckJoinFromFriendList()
{
#if NNS_PIA_ENABLE_DIRECT_JOIN_FROM_FRIENDLIST
    s_IsFromFriendList = false;
    memset(&s_FriendKeyForJoinIn, 0, sizeof(nn::friends::FriendKey));
    if (nn::friends::IsFromFriendList(&s_FriendKeyForJoinIn))
    {
        // プレゼンス情報を取得して合流可能であるかは後程確認する
        s_IsFromFriendList = true;
    }
#endif
}

// フレンドリストからの起動であるかどうか
bool IsJoinFromFriendList()
{
    return s_IsFromFriendList;
}

// フレンドプレゼンスのゲームモード情報を更新する
uint32_t UpdateGameMode(const nn::pia::common::String& description)
{
    PIA_SAMPLE_ASSERT(description.GetEncodingType() == nn::pia::common::String::EncodingType_Utf16);
    if (!nn::friends::IsInitialized())
    {
        PIA_SAMPLE_PRINT("UpdateGameMode is skipped. nn::friends::Initialize() is not called.\n");
        return 0;
    }

    nn::Result result = nn::friends::UpdateGameMode(s_FriendGameMode, static_cast<const SampleStringChar_t*>(description.GetBuffer()));
    if (result.IsFailure())
    {
        uint32_t errorCode = nn::friends::ResultToErrorCode(result);
        PIA_SAMPLE_PRINT("UpdateGameMode is failed. [%d]\n", errorCode);
        return errorCode;
    }
    else
    {
        return 0;
    }
}

void ResetGameMode()
{
    s_FriendGameMode.joinGameMode = 0;
    s_FriendGameMode.matchmakeSystemType = GameModeType_Invalid;
    s_FriendGameMode.joinGroupId = 0;
    s_FriendGameMode.ownerPrincipalId = 0;
    s_FriendGameMode.joinAvailabilityFlag = FriendJoinAvailabilityNotJoinable;
    UpdateGameMode(SAMPLE_STRING("NotInCommunication"));
}

SampleSceneNo SetGameMode(uint32_t mode, uint32_t type, bool isJoinable, const nn::pia::common::String& description)
{
    s_FriendGameMode.joinGameMode = mode;
    s_FriendGameMode.matchmakeSystemType = type;
    if (isJoinable)
    {
        s_FriendGameMode.joinGroupId = nn::pia::session::Session::GetInstance()->GetSessionId();
        nn::pia::StationId hostStationId = nn::pia::session::Session::GetInstance()->GetHostStationId();
        nn::pia::transport::Station* pHostStation = nn::pia::transport::StationManager::GetInstance()->GetStation(hostStationId);
        nn::pia::PrincipalId hostPrincipalId;
        pHostStation->GetPrincipalId(&hostPrincipalId);
        s_FriendGameMode.ownerPrincipalId = static_cast<uint32_t>(hostPrincipalId);
        s_FriendGameMode.joinAvailabilityFlag = FriendJoinAvailabilityJoinable;
    }
    else
    {
        s_FriendGameMode.joinGroupId = 0;
        s_FriendGameMode.ownerPrincipalId = 0;
        s_FriendGameMode.joinAvailabilityFlag = FriendJoinAvailabilityNotJoinable;
    }

    uint32_t errorCode = UpdateGameMode(description);
    if (errorCode != 0)
    {
        SetFriendErrorResult(errorCode);
        return SampleSceneNo_NetworkError;
    }
    return SampleSceneNo_Communication;
}

void UpdateFriendPresence()
{
    size_t friendsNum = 0;
    nn::friends::FriendKey friendIdList[MaxFriendSessionNum];
    nn::friends::FriendPresence friendPresenceList[MaxFriendSessionNum];

    s_OnlineFriendsList.Clear();

    nn::friends::GetFriendKeyList(friendIdList, &friendsNum, 0, MaxFriendSessionNum);
    nn::friends::GetFriendPresence(friendPresenceList, friendIdList, friendsNum);

    for (size_t i = 0; i < friendsNum; i++)
    {
        PIA_SAMPLE_PRINT("Presence[%d]\n", i);
        PIA_SAMPLE_PRINT("  IsOnline              : %d\n", friendPresenceList[i].isOnline);
        PIA_SAMPLE_PRINT("  GameId                : %08x\n", friendPresenceList[i].gameMode.joinGameId);
        PIA_SAMPLE_PRINT("  GameMode              : %u\n", friendPresenceList[i].gameMode.joinGameMode);
        PIA_SAMPLE_PRINT("  GroupId(GatheringId)  : %u\n", friendPresenceList[i].gameMode.joinGroupId);
        PIA_SAMPLE_PRINT("  MatchmakeSystemType   : %u\n", friendPresenceList[i].gameMode.matchmakeSystemType);
        PIA_SAMPLE_PRINT("  Owner                 : %d\n", friendPresenceList[i].gameMode.ownerPrincipalId);
        PIA_SAMPLE_PRINT("  JoinAvailabilityFlag  : %08x\n", friendPresenceList[i].gameMode.joinAvailabilityFlag);
        PIA_SAMPLE_PRINT("  IsInviteMe            : %d\n", friendPresenceList[i].isInvitingMe);
        PIA_SAMPLE_PRINT("  IsJoinable            : %d\n", friendPresenceList[i].IsJoinable(FrinedMatchmakeGameModeMask));

        if (friendPresenceList[i].isOnline && friendPresenceList[i].IsJoinable(FrinedMatchmakeGameModeMask))
        {
            nn::friends::FriendKey* pNode = s_OnlineFriendsList.TryBirthBack();
            if (pNode)
            {
                *pNode = friendIdList[i];
            }
        }
    }
}

SampleSceneNo SceneFriendsMatchmake()
{
    nn::pia::Result result;
    uint32_t target = 0;
    MatchmakeCommand command = MatchmakeCommand_None;
    SampleSceneNo nextSceneNo = SampleSceneNo_Communication;

    GetFramework()->SetProgramStatus(SampleSceneNo_JoinFriendSession, false, false, 0);

    nn::pia::inet::NexCreateSessionSetting createSessionSetting;
    nn::pia::inet::NexJoinSessionSetting joinSessionSetting;

    // フレンドプレゼンスと連動させるため、ゲームモードは 0-63 の範囲に設定します
    createSessionSetting.SetGameMode(FrinedMatchmakeGameMode + GetIncrementalGameModeValueByBitRateCheckMode());
#if NN_PIA_ENABLE_SESSION_TYPE_FRIEND
    createSessionSetting.SetSessionType(nn::pia::inet::SessionType_Friend);
#else
    createSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
#endif
    createSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
    createSessionSetting.SetSessionUserPassword(SAMPLE_STRING("password"));
    createSessionSetting.SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
    createSessionSetting.SetOpenSession(true);

#if NNS_PIA_ENABLE_DIRECT_JOIN_FROM_FRIENDLIST
    // フレンドプレゼンスからの起動
    if (s_IsFromFriendList)
    {
        s_IsFromFriendList = false;

        // フレンドプレゼンスから合流先のセッションIDを取得
        nn::friends::FriendPresence friendPresence;
        nn::friends::GetFriendPresence(&friendPresence, &s_FriendKeyForJoinIn);
        if (friendPresence.IsJoinable(FrinedMatchmakeGameModeMask))
        {
            joinSessionSetting.SetSessionId(friendPresence.gameMode.joinGroupId);
            command = MatchmakeCommand_Join;
        }
    }
#endif

    joinSessionSetting.SetSessionUserPassword(SAMPLE_STRING("password"));

    s_OnlineFriendsList.Clear();

    while (command == MatchmakeCommand_None)
    {
        GetFramework()->BeginFrame();

        // 上下: セッションの選択
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            target = (MaxFriendSessionNum + target - 1) % MaxFriendSessionNum;
        }
        else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            target = (target + 1) % MaxFriendSessionNum;
        }
        // B: リセット
        else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            command = MatchmakeCommand_Back;
        }
        // Y: フレンドプレゼンス更新
        else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            UpdateFriendPresence();
        }
        // X: セッション構築
        else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            command = MatchmakeCommand_Create;
        }
        // A: セッション参加
        else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            OnlineFriendsList::Iterator it = s_OnlineFriendsList.Begin();
            for (int i = 0; i < MaxFriendSessionNum; i++)
            {
                if (it == s_OnlineFriendsList.End())
                {
                    break;
                }
                if (i == target)
                {
                    // フレンドプレゼンスから合流先のセッションIDを取得
                    nn::friends::FriendPresence friendPresence;
                    nn::friends::GetFriendPresence(&friendPresence, &(*it));
                    if (friendPresence.IsJoinable(FrinedMatchmakeGameModeMask))
                    {
                        joinSessionSetting.SetSessionId(friendPresence.gameMode.joinGroupId);
                        command = MatchmakeCommand_Join;
                    }
                }
                ++it;
            }
        }

        // 下画面
        GetFramework()->ChangeRenderTarget(false);
        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        int line = 0;
        GetFramework()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, g_FrameNo);
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Scene: Select Friend's Session.");
        line += FontSize;
        line += FontSize;

        GetFramework()->DrawString(0, line, "[ Online Friend PrincipalID List ] Found:%01d ", s_OnlineFriendsList.Size());
        line += FontSize;
        OnlineFriendsList::Iterator it = s_OnlineFriendsList.Begin();
        for (int i = 0; i < MaxFriendSessionNum; i++)
        {
            if (it != s_OnlineFriendsList.End())
            {
                if (i == target)
                {
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                    GetFramework()->DrawString(0, line, " PID=%llu", static_cast<nn::pia::PrincipalId>(it->principalId));
                    line += FontSize;
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);

                    nn::friends::FriendPresence friendPresence;
                    nn::friends::GetFriendPresence(&friendPresence, &(*it));
                    int y = LowerScreenHeight - (FontSize * 14);
                    GetFramework()->DrawString(10, y, "  GameId   : %08x\n", friendPresence.gameMode.joinGameId);
                    y += FontSize;
                    GetFramework()->DrawString(10, y, "  GameMode : %u\n", friendPresence.gameMode.joinGameMode);
                    y += FontSize;
                    GetFramework()->DrawString(10, y, "  GroupId  : %u\n", friendPresence.gameMode.joinGroupId);
                    y += FontSize;
                    GetFramework()->DrawString(10, y, "  Type     : %u\n", friendPresence.gameMode.matchmakeSystemType);
                    y += FontSize;
                    GetFramework()->DrawString(10, y, "  Owner    : %d\n", friendPresence.gameMode.ownerPrincipalId);
                    y += FontSize;
                    GetFramework()->DrawString(10, y, "  Flag     : %08x\n", friendPresence.gameMode.joinAvailabilityFlag);
                    y += FontSize;
                }
                else
                {
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                    GetFramework()->DrawString(0, line, " PID=%llu", static_cast<nn::pia::PrincipalId>(it->principalId));
                    line += FontSize;
                }
                ++it;
            }
            else
            {
                if (i == target)
                {
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                }
                else
                {
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                }
                GetFramework()->DrawString(0, line, "  -----");
                GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                line += FontSize;
            }
        }

        line = LowerScreenHeight - (FontSize * 5);
        GetFramework()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  UP/DOWN: Select Friend's Session");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  X: Create New Session for Friends");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Y: Update Friend Presence");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  A: Start Join-in Friend's Session");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  B: Back");
        line += FontSize;

        GetFramework()->SwapBuffers();

        // 上画面
        GetFramework()->ChangeRenderTarget(true);
        GetFramework()->SwapBuffers();

        ++g_FrameNo;
        Dispatch();

        GetFramework()->EndFrame();

        // 接続状態チェック
        result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
        if (result.IsFailure())
        {
            s_OnlineFriendsList.Clear();
            // セッション参加前に ResultSessionConnectionIsLost が返ることはない
            PIA_SAMPLE_ASSERT(result != nn::pia::ResultSessionConnectionIsLost());
            return GetNextSceneByErrorResult(result, SampleErrorType_ConnectionLost);
        }
    }

    if (command == MatchmakeCommand_Create)
    {
        result = nns::pia::Session_CreateSession(&createSessionSetting);
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT_RESULT(result);
            nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Create);
        }
        else
        {
            // フレンドプレゼンスのゲームモード情報を更新する
            nextSceneNo = SetGameMode(FrinedMatchmakeGameMode + GetIncrementalGameModeValueByBitRateCheckMode(), GameModeType_Friends, true, DescriptionJoinFriendSession);
        }
    }
    else if (command == MatchmakeCommand_Join)
    {
        nns::pia::RegisterDispatchSampleSessionCallback(SampleSessionCallbackForDispatch);
        result = nns::pia::Session_JoinSession(&joinSessionSetting);
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT_RESULT(result);
            nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Join);
        }
        else
        {
            // フレンドプレゼンスのゲームモード情報を更新する
            nextSceneNo = SetGameMode(FrinedMatchmakeGameMode + GetIncrementalGameModeValueByBitRateCheckMode(), GameModeType_Friends, true, DescriptionJoinFriendSession);
        }
    }
    else if (command == MatchmakeCommand_Back)
    {
        nextSceneNo = SampleSceneNo_SelectSessionType;
        nns::pia::Session_CleanupSession();
    }

    s_OnlineFriendsList.Clear();

    return nextSceneNo;
}
#endif
