﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include "SampleConfig.h"

#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"

#if NNS_PIA_ENABLE_INET

/*!
    @brief インターネット通信固有の処理をまとめた *.cpp
*/
namespace
{
const char* StepString[] = {
    "Initialize NEX",
    "Initialize Pia modules",
    "Initialize internet network",
    "Login server",
    "Retrieve Notificaion Event"
};

enum InitializeStep
{
    InitializeStep_Nex = 0,
    InitializeStep_PiaModules,
    InitializeStep_Network,
    InitializeStep_Login,
    InitializeStep_RetriveEvent,
    InitializeStep_Max
};

bool s_IsNgsLogined = false;
bool s_IsPiaInitialized = false;

#if NNS_PIA_ENABLE_FRIENDS
class SampleNexNofiticationEventHandler : public nn::nex::NotificationEventHandler
{
public:
    SampleNexNofiticationEventHandler()
    {
    }
    virtual ~SampleNexNofiticationEventHandler()
    {
    }

    virtual void ProcessNotificationEvent(const nn::nex::NotificationEvent& oEvent)
    {
        PIA_SAMPLE_PRINT("Received notification event. Type:%u, Param1:%llu, Param2:%llu\n",
                       oEvent.GetType(),
                       static_cast<uint64_t>(oEvent.GetParam1()),
                       static_cast<uint64_t>(oEvent.GetParam2()));
        NN_PIA_UNUSED_IN_RELEASE(oEvent);
    }
};
SampleNexNofiticationEventHandler s_NexNotificationHandler;
#endif
}  // end of anonymous namespace

SampleSceneNo ScenePrepareInternet()
{
    GetFramework()->SetProgramStatus(
        SampleSceneNo_PrepareInet,
        false,
        false,
        0,
        0);

    nn::pia::Result result;
    uint32_t step = InitializeStep_Nex;
    nn::pia::inet::NexNetworkFactory nexNetworkFactroy;
    PIA_SAMPLE_ASSERT(g_NetworkType == nns::pia::NetworkType_Internet);

    if (g_NetworkType == nns::pia::NetworkType_Internet)
    {
        result = nns::pia::Nex_InitializeNex(
            MemoryBlockSizeForNex,
            MemoryBlockSizeForNexEmergency);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }
    ++step;

    nn::nex::qList<nn::nex::NotificationEvent> retrievedEventList;

    while (step < InitializeStep_Max)
    {
        switch (step)
        {
        case InitializeStep_PiaModules:
        {
            if (!s_IsPiaInitialized)
            {
                // PiaInet の初期化
                nns::pia::Inet_InitializePiaInet();

                // PiaTransport の初期化
                uint32_t unreliableProtocolHandle;
                nn::pia::transport::Transport::Setting transportSetting;
                transportSetting.pFactory = &nexNetworkFactroy;
                transportSetting.maxStationNum = GetMaxParticipantNum();
                transportSetting.maxSendThreadBufferNum = GetMaxParticipantNum() * 32;
                transportSetting.maxReceiveThreadBufferNum = GetMaxParticipantNum() * 32;

                result = nns::pia::Transport_InitializePiaTransport(
                    transportSetting,
                    0,
                    &unreliableProtocolHandle,
                    0,
                    NULL);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                // 通信に使用する UnreliableProtocolHandle を設定
                SetUnreliableProtocolHandle(unreliableProtocolHandle);

                bool isRelayEmulationEnabled = (g_NetworkTopologyType == nn::pia::session::NetworkTopology_RelayMesh);
#if NN_PIA_UNDER_DEVELOP || PIA_TEST_CODE
                {
                    nn::pia::session::Session::DebugSetting debugSetting;
                    debugSetting.isRelayEmulationEnabled = isRelayEmulationEnabled;
                    result = nn::pia::session::Session::SetDebugSetting(debugSetting);
                    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                }
#endif
                // PiaSession の初期化
                nn::pia::session::Session::Setting sessionSetting;
                sessionSetting.pNetworkFactory = &nexNetworkFactroy;
                sessionSetting.browsedCommunityInfoListNum = 16;
                sessionSetting.networkTopology = g_NetworkTopologyType;
                sessionSetting.browsedSessionInfoListNum = MaxBrowsedSessionNum;

                if (g_IsBitRateCheckEnabled)
                {
                    sessionSetting.bitRateCheckMode = nn::pia::session::BitRateCheckMode_Enable;
                }
                else
                {
                    sessionSetting.bitRateCheckMode = nn::pia::session::BitRateCheckMode_Disable;
                }
                PIA_SAMPLE_PRINT("BitRate check mode: %s\n", g_IsBitRateCheckEnabled ? "Enable" : "Disable");
                PIA_SAMPLE_PRINT("Relay emulation   : %s\n", isRelayEmulationEnabled ? "Enable" : "Disable");

                result = nns::pia::Session_InitializePiaSession(sessionSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                // イベントコールバックを設定します。
                result = nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionCallback);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                if (isRelayEmulationEnabled)
                {
                    // リレー接続エミュレーション設定をリセット
                    nn::pia::session::Session::GetInstance()->ResetRelayEmulation();
                    // リレー接続のエミュレーション設定を行います。
                    // リレー経路の設定は StationIndex 指定となります。
                    // ここでは StationIndex2 と StationIndex3 の間がリレー接続となるように設定します。
                    nn::pia::session::Session::GetInstance()->RelayEmulation(nn::pia::StationIndex_2, nn::pia::StationIndex_3);
                }
                s_IsPiaInitialized = true;
            }
        }
        ++step;
        break;

        case InitializeStep_Network:
        {
            if (!nns::pia::Net_IsInitialized())
            {
                nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(nns::pia::NetworkType_Internet);
                if (netResult != nns::pia::NetResult_Success)
                {
                    if (netResult == nns::pia::NetResult_AccountError)
                    {
                        return GetNextSceneByErrorResult(nn::pia::ResultSdkError(), SampleErrorType_InitializeByAccount);
                    }
                    else if (netResult == nns::pia::NetResult_AcError || netResult == nns::pia::NetResult_NicDown || netResult == nns::pia::NetResult_SocketError)
                    {
                        return GetNextSceneByErrorResult(nn::pia::ResultNetworkConnectionIsLost(), SampleErrorType_InitializeByNetworkInterfaceUp);
                    }
                    else
                    {
                        return GetNextSceneByErrorResult(nn::pia::ResultNetworkConnectionIsLost(), SampleErrorType_InitializeByOtherReason);
                    }
                }
            }
        }
        ++step;
        break;

        case InitializeStep_Login:
        {
            NN_PIA_COMMON_SET_PROGRAM_INFORMATION();

            if (!s_IsNgsLogined)
            {
#if NN_PIA_NINTENDOSDK
                g_LoginError = nn::ResultSuccess();
#else
                g_LoginError = 0;
#endif
                if (!nns::pia::Nex_Login(&g_LoginError, 30000))
                {
                    // ログイン処理に失敗
                    SetCommunicationErrorType(SampleErrorType_Login);
                    return SampleSceneNo_NetworkError;
                }
                s_IsNgsLogined = true;

#if NNS_PIA_ENABLE_FRIENDS
                // ゲームサーバーからのイベント通知を受信するイベントハンドラを登録
                nns::pia::Nex_GetNgsFacadeInstance()->RegisterNotificationEventHandler(&s_NexNotificationHandler);
#endif
                // NexFacade に NEX ゲームサーバ情報を設定
                nn::pia::inet::NexFacade::LoginInfo loginInfo;
                nn::pia::common::String accessKey(NNS_PIA_LOGIN_ACCESS_KEY);

                loginInfo.gameServerId = NNS_PIA_LOGIN_GAME_ID;
                loginInfo.pAccessKey = &accessKey;
                loginInfo.pNgsFacade = nns::pia::Nex_GetNgsFacadeInstance();

                result = nn::pia::inet::NexFacade::GetInstance()->Bind(&loginInfo);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
#if NNS_PIA_ENABLE_FRIENDS
            // 既にゲームサーバーに通知済みのイベントを取得
            retrievedEventList.clear();
            result = nn::pia::inet::NexFacade::GetInstance()->RetrieveNotificationDataAsync(0xf, &retrievedEventList);
            if (result.IsFailure())
            {
                return GetNextSceneByErrorResult(result, SampleErrorType_Login);
            }
#endif
        }
        ++step;
        break;

        case InitializeStep_RetriveEvent:
        {
#if NNS_PIA_ENABLE_FRIENDS
            if (nn::pia::inet::NexFacade::GetInstance()->IsRetrieveNotificationDataCompleted())
            {
                result = nn::pia::inet::NexFacade::GetInstance()->GetRetrieveNotificationDataResult();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("Retrieve notification event process is failed. Result:%d\n", result.GetErrorCode());
                    return GetNextSceneByErrorResult(result, SampleErrorType_Login);
                }
                else
                {
                    for (nn::nex::qList<nn::nex::NotificationEvent>::const_iterator it = retrievedEventList.begin(); it != retrievedEventList.end(); ++it)
                    {
                        PIA_SAMPLE_PRINT("Retrieved notification event. Type:%u, Param1:0x%08x, Param2:0x%08x\n",
                            it->GetType(),
                            static_cast<uint32_t>(it->GetParam1()),
                            static_cast<uint32_t>(it->GetParam2()));
                    }
                }
                ++step;
            }
#else
            ++step;
#endif
        }
        break;
        }

        GetFramework()->BeginFrame();

        // 下画面
        GetFramework()->ChangeRenderTarget(false);
        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        int line = 0;
        GetFramework()->DrawString(0, line, "[%s Demo]", ProgramNameString);
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Scene: Inet Initializing process.");
        line += FontSize;

        line += FontSize;

        // 進捗の表示
        // 処理済みステップを白、未処理ステップを赤で表示
        {
            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
            GetFramework()->DrawString(0, line, "   START");
            line += FontSize;
            for (uint32_t i = 0; i < InitializeStep_Max; i++)
            {
                if (i == step)
                {
                    GetFramework()->DrawString(0, line, "     |");
                    line += FontSize;
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                }
                else
                {
                    line += FontSize;
                }
                GetFramework()->DrawString(0, line, "   %s", StepString[i]);
                line += FontSize;
            }
            line += FontSize;
            GetFramework()->DrawString(0, line, "   END");
            line += FontSize;
        }
        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);

        GetFramework()->SwapBuffers();

        // 上画面
        GetFramework()->ChangeRenderTarget(true);

        line = UpperScreenHeight - FontSize;
        GetFramework()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
        line += FontSize;

        GetFramework()->SwapBuffers();

        GetFramework()->EndFrame();

        Dispatch();
        ++g_FrameNo;
    }
    return SampleSceneNo_SelectSessionType;
}

void Logout()
{
    // NexFacade から NEX ゲームサーバ情報を解除
    if (nn::pia::inet::NexFacade::GetInstance())
    {
        nn::pia::inet::NexFacade::GetInstance()->Unbind();
    }

    if (s_IsNgsLogined)
    {
#if NNS_PIA_ENABLE_FRIENDS
        // ゲームサーバーからのイベント通知を受信するイベントハンドラを削除
        nns::pia::Nex_GetNgsFacadeInstance()->UnregisterNotificationEventHandler(&s_NexNotificationHandler);
#endif
        s_IsNgsLogined = false;
    }
    nns::pia::Nex_Logout();
}

void FinalizeInet()
{
#if NN_PIA_UNDER_DEVELOP || PIA_TEST_CODE
    {
        nn::pia::session::Session::DebugSetting debugSetting;
        nn::pia::Result result = nn::pia::session::Session::SetDebugSetting(debugSetting);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }
#endif
    nns::pia::Inet_FinalizePiaInet();
    nns::pia::Nex_FinalizeNex();
    s_IsPiaInitialized = false;
}
#endif
