﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"

#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Local_Setup.h"
#include "SampleLib/Net_Setup.h"

#if NNS_PIA_ENABLE_LOCAL

/*!

    @brief ローカル通信固有の処理をまとめた *.cpp

*/

namespace
{
const char* StepString[] = {
    "Initialize Pia modules",
    "Initialize local network",
    "Completed"
};  // 1 step

enum InitializeStep
{
    InitializeStep_PiaModules = 0,
    InitializeStep_Network,
    InitializeStep_Max
};

bool s_IsPiaInitialized = false;

#if NN_PIA_CTR
const nns::pia::LocalSetting s_LocalSetting(LocalTitleId, LocalSubId, false);
#elif NN_PIA_NINTENDOSDK
const nns::pia::LocalSetting s_LocalSetting(nns::pia::LocalCommunicationId, LocalSceneId, false);
#endif
}

// InitializePiaLocal の進捗を画面表示する
SampleSceneNo ScenePrepareLocal()
{
    GetFramework()->SetProgramStatus(
        SampleSceneNo_PrepareLocal,
        false,
        false,
        0,
        0);

    uint32_t step = 0;
    nn::pia::Result result;

    while (step < InitializeStep_Max)
    {
        switch (step)
        {
        case InitializeStep_PiaModules:
            if (!s_IsPiaInitialized)
            {
                // PiaLocal の初期化
                result = nns::pia::Local_InitializePiaLocal(s_LocalSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                // PiaTransport の初期化
#if PIA_TEST_CODE
                if (g_IsLatencyEmulationEnabled)
                {
                    nn::pia::transport::Transport::DebugSetting debugSetting;
                    debugSetting.receiveThreadLatencyEmulationPacketNum = GetMaxParticipantNum() * 32;
                    debugSetting.sendThreadLatencyEmulationPacketNum = 32;
                    nn::pia::transport::Transport::SetDebugSetting(debugSetting);
                }
#endif
                {
                    uint32_t unreliableProtocolHandle;
                    nn::pia::transport::Transport::Setting transportSetting;
                    transportSetting.pFactory = nns::pia::Local_GetNetworkFactory();
                    transportSetting.maxStationNum = GetMaxParticipantNum();
                    transportSetting.maxSendThreadBufferNum = GetMaxParticipantNum() * 32;
                    transportSetting.maxReceiveThreadBufferNum = GetMaxParticipantNum() * 32;
                    result = nns::pia::Transport_InitializePiaTransport(
                        transportSetting,
                        0,
                        &unreliableProtocolHandle,
                        0,
                        NULL);
                    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                    // 通信に使用する UnreliableProtocolHandle を設定
                    SetUnreliableProtocolHandle(unreliableProtocolHandle);
                }
                // PiaSession の初期化
                {
                    nn::pia::session::Session::Setting sessionSetting;
                    sessionSetting.pNetworkFactory = nns::pia::Local_GetNetworkFactory();
                    sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
                    sessionSetting.browsedSessionInfoListNum = MaxBrowsedSessionNum;
                    result = nns::pia::Session_InitializePiaSession(sessionSetting);
                    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                    // イベントコールバックを設定します。
                    result = nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionCallback);
                    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                }
                s_IsPiaInitialized = true;
            }
            ++step;
            break;

        case InitializeStep_Network:
            if (!nns::pia::Net_IsInitialized())
            {
                nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(nns::pia::NetworkType_Local);
                if (netResult != nns::pia::NetResult_Success)
                {
                    if (netResult == nns::pia::NetResult_AccountError)
                    {
                        return GetNextSceneByErrorResult(nn::pia::ResultSdkError(), SampleErrorType_InitializeByAccount);
                    }
                    else if (netResult == nns::pia::NetResult_AcError || netResult == nns::pia::NetResult_NicDown || netResult == nns::pia::NetResult_SocketError)
                    {
                        return GetNextSceneByErrorResult(nn::pia::ResultNetworkConnectionIsLost(), SampleErrorType_InitializeByNetworkInterfaceUp);
                    }
                    else
                    {
                        return GetNextSceneByErrorResult(nn::pia::ResultNetworkConnectionIsLost(), SampleErrorType_InitializeByOtherReason);
                    }
                }
            }
            ++step;
            break;
        }
    }


    GetFramework()->BeginFrame();

    // 下画面
    GetFramework()->ChangeRenderTarget(false);
    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
    int line = 0;
    GetFramework()->DrawString(0, line, "[%s Demo]", ProgramNameString);
    line += FontSize;
    GetFramework()->DrawString(0, line, "  Scene: Local Initializing process.");
    line += FontSize;
    line += FontSize;

    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
    GetFramework()->DrawString(0, line, "   START");
    line += FontSize;
    for (uint32_t i = 0; i < InitializeStep_Max; i++)
    {
        if (i == step)
        {
            GetFramework()->DrawString(0, line, "     |");
            line += FontSize;
            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
        }
        else
        {
            line += FontSize;
        }
        GetFramework()->DrawString(0, line, "   %s", StepString[step]);
        line += FontSize;
    }
    line += FontSize;
    GetFramework()->DrawString(0, line, "   END");
    line += FontSize;
    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
    GetFramework()->SwapBuffers();

    // 上画面
    GetFramework()->ChangeRenderTarget(true);

    line = UpperScreenHeight - FontSize;
    GetFramework()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
    line += FontSize;

    GetFramework()->SwapBuffers();

    GetFramework()->EndFrame();

    return SampleSceneNo_SelectSessionType;
}

void CleanupLocal()
{
    nns::pia::Local_FinalizeLocalCommunication();
}

void FinalizeLocal()
{
    nns::pia::Local_FinalizePiaLocal();
    s_IsPiaInitialized = false;
}
#endif
