﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


/* 
session/Advanced サンプルデモ

概要
  nn::pia::session::Session の機能を利用した、より高機能なデモです。
  インターネット通信で、ランダムマッチメイク、ブラウズマッチメイク、
  フレンド合流マッチメイクを行えます。

  フレンド合流マッチメイクは同じゲームを起動中のフレンドセッションへの合流の他、
  HOME メニューからフレンドセッションへの合流にも対応しています。

  通信シーンでは以下の機能をサポートしています。
   ホストマイグレーション
   セッションのオープン／クローズ
   マッチメイクシステムパスワードを利用した再参加

  3DS ではインターネット通信の他、ローカル通信でのブラウズマッチメイクも行えます。

  Wii U では ENABLE_SERVERLESS の設定により LAN マッチメイクに切り替えることもできます。

注意
  プレゼンスを更新しても、前回の更新から10秒程度経過するまではフレンドに通知されません。
  またネットワークの状態によってはプレゼンス変更通知が遅れることもあるため、フレンド
  プレゼンスライブラリで合流可能と判定されても必ず合流可能であるとは限らないことに
  注意してください。

  3DS では、HOME メニューのアイコンキャッシュがシステムに残っているとフレンド
  プレゼンスによる合流機能が正常に機能しない場合があります。合流機能を使用する際は
  アイコンキャッシュをリセットしてください。

  本サンプルでは、フレンドプレゼンスの合流機能を利用して、フレンドの参加している
  セッションに合流するため、合流ゲーム ID、合流ゲームモードマスクについて適切な
  設定を行ったマスター ROM を作成し、HOME メニューから起動する必要があります。
  マスター ROM 作成手順についての詳細はプラットフォーム毎の
  「マスター ROM 作成・提出手順書」を参照してください。
*/


#include <nn/pia/pia_Version.h>

#include "SampleConfig.h"

#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Common_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Local_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"

#if NN_PIA_A
#include "SampleFramework/include/AdrMainHelper.h"
#endif

namespace
{
// Pia に与えるメモリ領域です。32bitの場合は4バイト、32bitの場合は8バイト整合されていないといけません。
// このサンプルでは十分余裕を持たせた数値にしています。
PIA_ATTRIBUTE_ALIGN(8) uint8_t s_PiaMemoryBlock[MemoryBlockSizeForPia];

// Piaライブラリバージョン情報文字列保存用変数
char s_SamplePiaVersionInfoString[UpperScreenWidth / FontSize] = {};

#if PIA_TEST_CODE
bool s_IsLanMatchmakeEnabled = false;
#endif

}  // end of anonymous namespace

// Global 変数宣言
uint32_t g_FrameNo = 0;
nns::pia::NetworkType g_NetworkType = nns::pia::NetworkType_Internet;
nn::pia::session::NetworkTopology g_NetworkTopologyType = nn::pia::session::NetworkTopology_FullMesh;
bool g_IsHostMigrationEnabled = false;
bool g_IsBitRateCheckEnabled = false;
#if PIA_TEST_CODE
bool g_IsLatencyEmulationEnabled = false;
#endif

#if TEST_PIA_LOCAL_USER_PASSWORD
// ローカルのユーザーパスワード変更テスト用
uint8_t g_UserPasswordType = 0;
#endif

#if NN_PIA_NINTENDOSDK
nn::Result g_LoginError = nn::ResultSuccess();
#else
int32_t g_LoginError = 0;
#endif
LocalMultiPlayType g_LocalMultiPlayType = LocalMultiPlayType_None;
MatchmakeKeywordType g_MatchmakeKeywordType = MatchmakeKeywordType_None;

#if NNS_PIA_ENABLE_INET
const SampleStringChar_t* SampleMatchmakeKeyword[] = {
    SAMPLE_STRING(""),
    SAMPLE_STRING("KEY_TYPE_A"),
    SAMPLE_STRING("KEY_TYPE_B")
};
#endif


nns::pia::sampleframework::SampleFramework* GetFramework()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    // この関数を呼び出す前に SampleFramework の初期化が完了している前提なので、完了していない場合はアサートで止める
    PIA_SAMPLE_ASSERT(pSampleFramework != NULL);
    return pSampleFramework;
}

// Piaライブラリバージョン情報文字列取得関数
const char* GetPiaVersionString()
{
    return s_SamplePiaVersionInfoString;
}

uint32_t GetIncrementalGameModeValueByBitRateCheckMode()
{
    if (g_IsBitRateCheckEnabled)
    {
        return 1000;
    }
    else
    {
        return 0;
    }
}

void Dispatch()
{
#if NNS_PIA_ENABLE_INET
    if (g_NetworkType == nns::pia::NetworkType_Internet && (g_FrameNo % 2) == 0)
    {
        nns::pia::Nex_SchedulerDispatch();
    }
#endif
    nn::pia::common::Scheduler::GetInstance()->Dispatch();
}

PIA_SAMPLE_MAIN_FUNC()
{
    PIA_SAMPLE_PRINT("Sample program Start.\n");

    // Piaライブラリバージョン情報文字列の作成
    PIA_SNPRINTF(s_SamplePiaVersionInfoString,
                 UpperScreenWidth / FontSize,
                 "Pia %d.%d.%d r%d",
                 PIA_VERSION_MAJOR,
                 PIA_VERSION_MINOR,
                 PIA_VERSION_MICRO,
                 PIA_VERSION_REVISION);

    // 再参加用の情報をクリア
    ResetRejoinSettings();

#if NNS_PIA_ENABLE_FRIENDS
    ResetGameMode();
#if NNS_PIA_ENABLE_DIRECT_JOIN_FROM_FRIENDLIST
    // フレンドリストからの合流かどうかをチェック
    CheckJoinFromFriendList();
#endif
#endif

    nn::pia::Result result;

#if PIA_TEST_CODE
    // デバッグ機能（パケロスエミュレーション）
    // 製品版 ROM では呼び出さないように注意してください
    {
        nn::pia::transport::Transport::DebugSetting transportDebugSetting;
        transportDebugSetting.isPacketLossEmulationEnabled = true;
        result = nn::pia::transport::Transport::SetDebugSetting(transportDebugSetting);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
    }
#endif

    // Pia のサンプルプログラム用描画フレームワークの初期化
    nns::pia::sampleframework::SampleFramework::CreateInstance();

    GetFramework()->SetFramePerSecond(FramePerSecond);
    GetFramework()->SetClearColor(nns::pia::sampleframework::Color(0.0f, 0.0f, 0.2f, 1.0f));
    GetFramework()->SetProgramStatus(SampleSceneNo_Start, false);

    nns::pia::Basic_SetProgramTitle("Session Advanced");

    bool isExit = false;

    while (isExit == false)
    {
        SampleSceneNo nextSceneNo = SampleSceneNo_Start;

        // PiaCommon モジュールで利用する機能のセットアップ
        result = nns::pia::Common_InitializePiaCommon(
            s_PiaMemoryBlock,
            MemoryBlockSizeForPia,
            nn::pia::common::TraceFlagCommon |
                //nn::pia::common::TraceFlagCommonSocket |
                nn::pia::common::TraceFlagLocal |
                nn::pia::common::TraceFlagTransport |
                //nn::pia::common::TraceFlagTransportPacketSend |
                //nn::pia::common::TraceFlagTransportPacketReceive |
                //nn::pia::common::TraceFlagTransportProtocolmessageSend |
                //nn::pia::common::TraceFlagTransportProtocolmessageReceive |
                nn::pia::common::TraceFlagTransportRelay |
                //nn::pia::common::TraceFlagTransportRelayDetail |
                nn::pia::common::TraceFlagSession |
                nn::pia::common::TraceFlagSessionConnect |
                nn::pia::common::TraceFlagSessionLeave |
                nn::pia::common::TraceFlagSessionUpdate |
                nn::pia::common::TraceFlagSessionMigration |
                //nn::pia::common::TraceFlagSessionDetail |
                nn::pia::common::TraceFlagInet |
                //nn::pia::common::TraceFlagInetReport |
                //nn::pia::common::TraceFlagInetStream |
                //nn::pia::common::TraceFlagInetNat |
                //nn::pia::common::TraceFlagInetNatProbe |
                //nn::pia::common::TraceFlagInetNatDetail |
                nn::pia::common::TraceFlagLan |
                //nn::pia::common::TraceFlagLanDetail |
                nn::pia::common::TraceFlagAlways,
            false,
            true);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

        // このサンプルでは機能ごとの処理を明確に分けるため、メインスレッドの処理をいくつかのシーンに分割しています。

        /* ゲームループ */
        while (isExit == false)
        {
            switch (nextSceneNo)
            {
            case SampleSceneNo_Start:
            {
                nextSceneNo = SceneStart();
            }
            break;
            case SampleSceneNo_PrepareLan:
            {
                nextSceneNo = ScenePrepareLan();
            }
            break;
#if NNS_PIA_ENABLE_INET
            case SampleSceneNo_PrepareInet:
            {
                nextSceneNo = ScenePrepareInternet();
            }
            break;
            case SampleSceneNo_DummySession:
            {
                nextSceneNo = SceneDummySession();
            }
            break;
#endif
#if NNS_PIA_ENABLE_LOCAL
            case SampleSceneNo_PrepareLocal:
            {
                nextSceneNo = ScenePrepareLocal();
            }
            break;
#endif
            case SampleSceneNo_SelectSessionType:
            {
                nextSceneNo = SceneSelectMatchmakeType();
            }
            break;
            case SampleSceneNo_Community:
            {
                nextSceneNo = SceneCommunity();
            }
            break;
            case SampleSceneNo_JoinRandomSession:
            {
                nextSceneNo = SceneRandomMatchmake();
            }
            break;
            case SampleSceneNo_JoinSession:
            {
                nextSceneNo = SceneBrowseMatchmake();
            }
            break;
#if NNS_PIA_ENABLE_FRIENDS
            case SampleSceneNo_JoinFriendSession:
            {
                nextSceneNo = SceneFriendsMatchmake();
            }
            break;
#endif
            case SampleSceneNo_Communication:
            {
                nextSceneNo = SceneCommunication();
            }
            break;
            case SampleSceneNo_Error:
            {
                nextSceneNo = SceneError();
            }
            break;
            case SampleSceneNo_NetworkError:
            {
                nextSceneNo = SceneNetworkError();
            }
            break;
            case SampleSceneNo_Exit:
            {
                isExit = true;
            }
            break;
            default:
                break;
            }
        }

        // PiaCommon の終了処理
        nns::pia::Common_FinalizePiaCommon();
    }
    nns::pia::sampleframework::SampleFramework::DestroyInstance();
    //PIA_SAMPLE_PRINT("Sample program end.\n");
}

SampleSceneNo SceneStart()
{
    nn::pia::Result result;
    SampleSceneNo nextSceneNo = SampleSceneNo_Start;

    GetFramework()->SetProgramStatus(SampleSceneNo_Start, false, false, 0);
    g_FrameNo = 0;

    PIA_SAMPLE_PRINT("Called SceneStart().\n");

#if PIA_TEST_CODE
    g_IsLatencyEmulationEnabled = false;
    GetFramework()->SetProgramParam(ProgramParamLanMatchmakeState, s_IsLanMatchmakeEnabled);
#endif

#if NNS_PIA_ENABLE_INET
    // 参加していたセッションへの再参加用情報を保持している場合はブラウズマッチメイクシーンを経由してセッションに参加
    if (GetSessionId() != nn::pia::session::InvalidSessionId)
    {
        // NetworkType, NetworkTopologyType は前回の値をそのまま使用する
        nextSceneNo = SampleSceneNo_PrepareInet;
    }
#if NNS_PIA_ENABLE_DIRECT_JOIN_FROM_FRIENDLIST
    else if (IsJoinFromFriendList())
    {
        // サンプルデモの実装上の都合でフレンド合流はフルメッシュ固定にしています。
        g_NetworkType = nns::pia::NetworkType_Internet;
        g_NetworkTopologyType = nn::pia::session::NetworkTopology_FullMesh;
        nextSceneNo = SampleSceneNo_PrepareInet;
    }
#endif
#endif

    while (nextSceneNo == SampleSceneNo_Start)
    {
        GetFramework()->BeginFrame();

#if NNS_PIA_ENABLE_LOCAL
        // A:
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            g_NetworkType = nns::pia::NetworkType_Local;
            g_NetworkTopologyType = nn::pia::session::NetworkTopology_FullMesh;
            nextSceneNo = SampleSceneNo_PrepareLocal;
        }
#endif
        // X:
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            g_NetworkType = nns::pia::NetworkType_Lan;
            g_NetworkTopologyType = nn::pia::session::NetworkTopology_FullMesh;
            nextSceneNo = SampleSceneNo_PrepareLan;
        }
#if NNS_PIA_ENABLE_INET
        // Y:
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            g_NetworkType = nns::pia::NetworkType_Internet;
            nextSceneNo = SampleSceneNo_PrepareInet;
        }
        // B:
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            g_NetworkType = nns::pia::NetworkType_Internet;
            nextSceneNo = SampleSceneNo_DummySession;
        }
#endif
        // L:
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonL))
        {
            g_IsBitRateCheckEnabled = !g_IsBitRateCheckEnabled;
            PIA_SAMPLE_PRINT("Change BitRate check mode: %s\n", g_IsBitRateCheckEnabled ? "Enable" : "Disable");
        }
        // R:
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            if (g_NetworkTopologyType == nn::pia::session::NetworkTopology_FullMesh)
            {
                g_NetworkTopologyType = nn::pia::session::NetworkTopology_RelayMesh;
            }
            else
            {
                g_NetworkTopologyType = nn::pia::session::NetworkTopology_FullMesh;
            }
        }
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonStart))
        {
            nextSceneNo = SampleSceneNo_Exit;
        }

#if PIA_TEST_CODE
        if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            g_IsLatencyEmulationEnabled = !g_IsLatencyEmulationEnabled;
        }
        GetFramework()->SetProgramParam(ProgramParamNetworkTopologyType, g_NetworkTopologyType);
#endif

        // 下画面
        GetFramework()->ChangeRenderTarget(false);
        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        int line = 0;
        GetFramework()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, g_FrameNo);
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Scene: Select NetworkType.");
        line += FontSize;
        line += FontSize;

        line = LowerScreenHeight - (FontSize * 9);
        GetFramework()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  L: Change BitRateCheck mode for Internet session");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  R: Change NetworkTopology for Internet session");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  X: Start Lan session");
        line += FontSize;
#if NNS_PIA_ENABLE_INET
        GetFramework()->DrawString(0, line, "  Y: Start Internet session (%s)", (g_NetworkTopologyType == nn::pia::session::NetworkTopology_FullMesh) ? "FullMesh" : "RelayMesh");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  B: Start Dummy session");
        line += FontSize;
#endif
#if NNS_PIA_ENABLE_LOCAL
        GetFramework()->DrawString(0, line, "  A: Start Local session");
        line += FontSize;
#endif
        line += FontSize;

        GetFramework()->SwapBuffers();

        GetFramework()->EndFrame();
        ++g_FrameNo;
    }

    return nextSceneNo;
}

SampleSceneNo GetNextSceneByErrorResult(nn::pia::Result result, SampleErrorType errorType)
{
    SampleSceneNo nextScene;

    if (result.GetViewerType() == nn::pia::Result::ViewerType_ShouldNotUse)
    {
        SetCommunicationErrorType(errorType);
        SetPiaErrorResult(result);
        nextScene = SampleSceneNo_Error;
    }
    // nn::pia::Result::ViewerType_ShouldUse or nn::pia::Result::ViewerType_MayUse
    else
    {
        SetCommunicationErrorType(errorType);
        SetPiaErrorResult(result);
        nextScene = SampleSceneNo_NetworkError;
    }
    return nextScene;
}

void CleanupPia()
{
    if (nn::pia::session::Session::GetInstance())
    {
        nns::pia::Session_CleanupSession();
    }
#if NNS_PIA_ENABLE_LOCAL
    if (g_NetworkType == nns::pia::NetworkType_Local)
    {
        CleanupLocal();
    }
#endif
    if (g_NetworkType == nns::pia::NetworkType_Lan)
    {
        CleanupLan();
    }
}

void FinalizePia()
{
    nn::pia::session::Session::GetInstance()->UnregisterSessionEventCallback();
    nns::pia::Session_FinalizePiaSession();
    nns::pia::Transport_FinalizePiaTransport(GetUnreliableProtocolHandle(), 0);
#if NNS_PIA_ENABLE_INET
    if (g_NetworkType == nns::pia::NetworkType_Internet)
    {
        FinalizeInet();
    }
#endif
#if NNS_PIA_ENABLE_LOCAL
    if (g_NetworkType == nns::pia::NetworkType_Local)
    {
        FinalizeLocal();
    }
#endif
    if (g_NetworkType == nns::pia::NetworkType_Lan)
    {
        FinalizeLan();
    }
}

SampleSceneNo HandlingErrorResult(nn::pia::Result::HandlingType handlingType, SampleErrorType errorType)
{
    SampleSceneNo nextScene;

    PIA_SAMPLE_PRINT("HandlingErrorResult: HandlingType=%d\n", handlingType);

    // このサンプルデモでは、これらのハンドリングタイプの Result を本関数に渡されることはない
    if (handlingType == nn::pia::Result::HandlingType_Void ||
        handlingType == nn::pia::Result::HandlingType_Ignorable ||
        handlingType == nn::pia::Result::HandlingType_Retry ||
        handlingType == nn::pia::Result::HandlingType_ProgrammingError)
    {
        PIA_SAMPLE_ASSERT(false);
    }

    // セッション離脱が必要な場合
    if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave ||
        handlingType == nn::pia::Result::HandlingType_LogoutWithLeave)
    {
        // このサンプルデモではゲームサーバーへのログインからセッションのスタートアップまでの間、
        // NEX API でゲームサーバーとの接続状態をチェックしています。そこで切断状態を検知した場合も
        // ここでハンドリングできるように NEX のエラーと同等となる Pia の Result に変換してハンドリングを
        // 行っています。変換後の Result が ResultNetworkConnectionIsLostByDuplicateLogin となる場合、
        // ハンドリングタイプではセッション離脱が必要となりますが、セッションのスタートアップ前のため、
        // LeaveSession を呼び出さないようにしています。
        if (errorType != SampleErrorType_ConnectionLostBeforeStartup)
        {
            nns::pia::Session_LeaveSession();
        }
    }

    // クリーンアップはどのハンドリングタイプでも行う
    CleanupPia();

    // Cleanup, CleanupWithLeave 以外はインターネット通信時はログアウト処理を行う
    if (handlingType != nn::pia::Result::HandlingType_Cleanup &&
        handlingType != nn::pia::Result::HandlingType_CleanupWithLeave)
    {
#if NNS_PIA_ENABLE_INET
        if (g_NetworkType == nns::pia::NetworkType_Internet)
        {
            Logout();
        }
#endif
        if (handlingType == nn::pia::Result::HandlingType_ShutdownNetwork)
        {
            nns::pia::Net_FinalizeNetwork();
        }
        // nn::pia::Result::HandlingType_Finalize
        else
        {
            nns::pia::Net_FinalizeNetwork();
            FinalizePia();
        }
    }

    // ハンドリング後に進むシーンを決める
    if (handlingType == nn::pia::Result::HandlingType_Cleanup ||
        handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
    {
        nextScene = SampleSceneNo_SelectSessionType;
    }
    else if (handlingType == nn::pia::Result::HandlingType_LogoutWithLeave ||
             handlingType == nn::pia::Result::HandlingType_ShutdownNetwork)
    {
        if (g_NetworkType == nns::pia::NetworkType_Internet)
        {
            nextScene = SampleSceneNo_PrepareInet;
        }
        if (g_NetworkType == nns::pia::NetworkType_Local)
        {
            nextScene = SampleSceneNo_PrepareLocal;
        }
        if (g_NetworkType == nns::pia::NetworkType_Lan)
        {
            nextScene = SampleSceneNo_PrepareLan;
        }
    }
    // handlingType == nn::pia::Result::HandlingType_Finalize
    else
    {
        nextScene = SampleSceneNo_Start;
    }

    PIA_SAMPLE_PRINT("HandlingErrorResult: nextScene=%d\n", nextScene);
    return nextScene;
}

uint16_t GetMaxParticipantNum()
{
#if NNS_PIA_ENABLE_INET
    if (g_NetworkType == nns::pia::NetworkType_Internet)
    {
#if NN_PIA_NINTENDOSDK
        return 16;
#elif NN_PIA_CTR
        return 12;
#else
        return 16;
#endif
    }
#endif
    if (g_NetworkType == nns::pia::NetworkType_Lan)
    {
#if NN_PIA_NINTENDOSDK
        return 16;
#elif NN_PIA_CTR
        return 12;
#else
        return 16;
#endif
    }
#if NNS_PIA_ENABLE_LOCAL
    if (g_NetworkType == nns::pia::NetworkType_Local)
    {
        return 8;
    }
#endif
    return 8;
}
