﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>

#include <nn/pia/common.h>
#include <nn/pia/inet.h>
#include <nn/pia/transport.h>
#include <nn/pia/session.h>

#include "SampleLib/SampleLib.h"
#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Common_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"

#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

// マッチメイク分析機能を使用する際は 1 に設定します。
#define ENABLE_MATCHMAKE_ANALYSE 0 && NNS_PIA_ENABLE_INET
// ビットレート計測機能を使用する際は 1 に設定します。
#define ENABLE_BIT_RATE_CHECK 0 && NNS_PIA_ENABLE_INET
// LAN マッチメイクを使用する際は 1 に設定します。（NN_PIA_ENABLE_NEX はプラットフォーム毎に定義されます）
#define ENABLE_LAN_MATCHMAKE !NNS_PIA_ENABLE_INET

/*!
    @brief サンプルプログラムのシーンを示す列挙型です。
 */
enum SampleSceneNo
{
    SampleSceneNo_Preparation,      //!< 準備を行うシーンです。通信とは無関係です。
    SampleSceneNo_Matchmake,        //!< 指定したネットワーク種別毎に、参加するセッションを作る/参加する処理を行うシーンです。
    SampleSceneNo_Communicate,      //!< セッション内の相手と通信を行うシーンです。
    SampleSceneNo_ShowErrorApp,     //!< 問題が発生した場合に遷移するシーンです。アプリケーションで用意するエラーメッセージウィンドウを表示する処理を行います。
    SampleSceneNo_ShowErrorViewer,  //!< 問題が発生した場合に遷移するシーンです。エラー・EULAアプレット（エラービューア）でエラーを表示する処理を行います。
    SampleSceneNo_Reset             //!< デモが管理するすべての状態をリセットし、初期状態に戻すシーンです。
};

/*!
    @brief サンプルプログラムでハンドリングする通信エラーの種別です。
 */
enum SampleErrorType
{
    SampleErrorType_None = 0,                        //!< エラーは発生していません
    SampleErrorType_InitializeByNetworkInterfaceUp,  //!< 無線 OFF モードのため、初期化に失敗
    SampleErrorType_InitializeByAccount,             //!< アカウントが必要な場合に有効なアカウントが存在しなかった
    SampleErrorType_InitializeByOtherReason,         //!< その他の理由で、初期化に失敗
    SampleErrorType_Matchmake,                       //!< セッションを確立できませんでした
    SampleErrorType_SessionLost,                     //!< 通信中にセッションから切断されました
    SampleErrorType_ConnectionLost                   //!< ネットワーク接続が断絶しました
};

void SetCommunicationErrorType(SampleErrorType error);  //通信エラーの内容をセットする関数です。
void ClearNetworkErrorResult();
void SetNetworkErrorResult(nn::pia::Result result);      //Pia API 呼び出しが失敗した場合に返される nn::pia::Result 値をセットする関数です。
void SetNetworkErrorResult(nns::pia::NetResult result);  //非 Pia API 呼び出しが失敗した場合に返される nns::pia::NetResult 値をセットする関数です。

void SetUnreliableProtocolHandle(uint32_t handle);
uint32_t GetUnreliableProtocolHandle();

bool SampleJoinApprovalCallback(const nn::pia::transport::Station::IdentificationToken* pRequesterToken);
void SampleSessionCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam);

bool Login();
void Logout();

bool InitializePia();
void CleanupPia();
void FinalizePia();

/* ---< 処理別シーン >--- */
SampleSceneNo SceneMatchmake();
SampleSceneNo SceneCommunication();
SampleSceneNo SceneShowErrorByApp();
SampleSceneNo SceneShowErrorByViewr();

/* ---< サンプルで使用する構造体定義 >--- */

struct RectPosition  //!< 矩形の位置座標
{
    uint16_t x;
    uint16_t y;
};

struct SampleStationStatus
{
    nn::pia::StationId stationId;
    RectPosition pos;
    uint32_t latestIncrementNumber;
    int32_t rtt;

    static const uint32_t SerializedSize = sizeof(uint64_t) + sizeof(uint16_t) * 2 + sizeof(uint32_t);

    SampleStationStatus()
    {
        stationId = nn::pia::StationIdInvalid;
        pos.x = 0;
        pos.y = 0;
        latestIncrementNumber = 0;
        rtt = nn::pia::transport::Station::InvalidRtt;
    }

    void Serialize(void* pBuf)
    {
        PIA_SAMPLE_ASSERT(pBuf != NULL);
        PIA_SAMPLE_ASSERT(reinterpret_cast<uintptr_t>(pBuf) % 8 == 0);

        uint8_t* p = static_cast<uint8_t*>(pBuf);

        *reinterpret_cast<uint64_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork64(stationId);
        p += 8;
        *reinterpret_cast<uint16_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork16(pos.x);
        p += 2;
        *reinterpret_cast<uint16_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork16(pos.y);
        p += 2;
        *reinterpret_cast<uint32_t*>(p) = nn::pia::common::ByteOrder::HostToNetwork32(latestIncrementNumber);
    }

    void Deserialize(const void* pData)
    {
        PIA_SAMPLE_ASSERT(pData != NULL);
        PIA_SAMPLE_ASSERT(reinterpret_cast<uintptr_t>(pData) % 8 == 0);

        const uint8_t* p = static_cast<const uint8_t*>(pData);

        stationId = nn::pia::common::ByteOrder::NetworkToHost64(*reinterpret_cast<const uint64_t*>(p));
        p += 8;
        pos.x = nn::pia::common::ByteOrder::NetworkToHost16(*reinterpret_cast<const uint16_t*>(p));
        p += 2;
        pos.y = nn::pia::common::ByteOrder::NetworkToHost16(*reinterpret_cast<const uint16_t*>(p));
        p += 2;
        latestIncrementNumber = nn::pia::common::ByteOrder::NetworkToHost32(*reinterpret_cast<const uint32_t*>(p));
    }

    void UpdateInfo()
    {
        // RTT の更新
        nn::pia::transport::Station* pStationBuf = nn::pia::transport::StationManager::GetInstance()->GetStation(stationId);
        if (PIA_IS_VALID_POINTER(pStationBuf))
        {
            // StationId に対応する Station インスタンスから RTT を取得
            rtt = pStationBuf->GetRtt();
        }
        else
        {
            // Station インスタンスが取得できなかった場合は無効な値にしておきます
            rtt = nn::pia::transport::Station::InvalidRtt;
        }
    }
};

const int32_t FramePerSecond = 60;
const char ProgramNameString[] = "session/RandomMatchmake";

// セッション内の相手と通信を行うシーンで、自分の情報を送信する間隔[frame]の設定です。
// 0 にすると毎ゲームフレーム送信します。
const uint32_t IntervalOfSendingNodeStatus = 0;

const uint8_t MaxParticipantNum = 5;      //!< 最大接続数。実アプリではアプリの仕様に併せて設定してください。
const uint8_t MaxBrowsedSessionList = 5;  //!< 検索したネットワークの表示数最大

const uint32_t InitialRatingValue = 1500;
const uint8_t InitialSelectionPriority = 0;

const uint32_t MinScoreSettingIndex = 2;

const bool IsHostMigrationEnabled = false;  //!< 親交代機能の ON/OFF。PiaSession 親交代機能を使用する場合は ON 固定です。

/* スレッド優先度 */
#if NN_PIA_CTR
const int32_t SendThreadPriority = nn::os::DEFAULT_THREAD_PRIORITY - 1;  //transport の送信スレッド優先度。メインの処理よりも高優先度が望ましい
const int32_t ReceiveThreadPriority = SendThreadPriority - 1;            //transport の送信スレッド優先度。 機会損失を避けるために優先度を高く設定する
#elif NN_PIA_CAFE
const int32_t SendThreadPriority = OS_PRIORITY_APP_DEFAULT - 1;  //transport の送信スレッド優先度。メインの処理よりも高優先度が望ましい
const int32_t ReceiveThreadPriority = SendThreadPriority - 1;  //transport の送信スレッド優先度。 機会損失を避けるために優先度を高く設定する
#endif

/* NEX 関連設定 */
// NEX ゲームサーバでマッチメイクセッションを構築・検索する際に使用する条件群
const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 35500;  // ゲームモード
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;
const uint32_t MatchmakeAttributeIndex = 0;   //!< 使用する属性のインデックス
const uint32_t MatchmakeAttributeValue = 72;  //!< 使用する属性にサンプルとして設定する値

/* メモリ関連 */
const uint32_t MemoryBlockSizeForPia = 2 * 1024 * 1024;     //!< Pia に与えるメモリのサイズです。
const uint32_t MemoryBlockSizeForNex = 1024 * 1024;         //!< NEX に与えるメモリのサイズです。
const uint32_t MemoryBlockSizeForNexEmergency = 50 * 1024;  //!< NEX に与える緊急時用メモリのサイズです。

/* 描画関連 */
const uint32_t UserNameLength = 11;  //!< UserName 描画の最大文字数（終端文字を含む）

// 電波強度を取得するための処理をまとめた関数
uint32_t GetLinkLevel();

const int ProgramParamMatchmakeSessionParticipant = 102;
const int ProgramParamMatchmakeSessionSelectionMethod = 105;
const int ProgramParamMatchmakeSessionScoreBasedParam = 106;
