﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/common/common_CriticalSection.h>
#include <nn/pia/common/common_ScopedLock.h>

#if NN_PIA_CAFE
#include <nn/act.h>
#endif

#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/SampleLib.h"
#include "SampleConfig.h"

/*!

    @brief ネットワーク種別を問わない処理 (session + transport) をまとめた *.cpp

*/

namespace
{
const nns::pia::sampleframework::Color RectangleColorArray[12] =
    {
        nns::pia::sampleframework::ColorFuchsia,
        nns::pia::sampleframework::ColorGray,
        nns::pia::sampleframework::ColorWhite,
        nns::pia::sampleframework::ColorMaroon,
        nns::pia::sampleframework::ColorRed,
        nns::pia::sampleframework::ColorOlive,
        nns::pia::sampleframework::ColorYellow,
        nns::pia::sampleframework::ColorGreen,
        nns::pia::sampleframework::ColorLime,
        nns::pia::sampleframework::ColorTeal,
        nns::pia::sampleframework::ColorNavy,
        nns::pia::sampleframework::ColorPurple
    };

//セッション内での役割画面表示用文字列
const char* SessionRoleString[] =
    {
        "Unknown",
        "Host",
        "Client"
    };

static SampleStationStatus s_StationStatus[MaxParticipantNum];
static uint32_t s_UnreliableProtocolHandle = 0;

#if ENABLE_MATCHMAKE_ANALYSE
static uint8_t s_TokenLock[nn::pia::transport::Station::IdentificationTokenMaxDataSize] = "Lock";
static uint8_t s_TokenUnlock[nn::pia::transport::Station::IdentificationTokenMaxDataSize] = "Unlock";
static bool s_IsApproved = true;
#endif
}  // end of anonymous namespace

void SetUnreliableProtocolHandle(uint32_t handle)
{
    s_UnreliableProtocolHandle = handle;
}

uint32_t GetUnreliableProtocolHandle()
{
    return s_UnreliableProtocolHandle;
}

void ActivateStationStatus(const nn::pia::StationId stationId)
{
    nn::pia::StationIndex index;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&index, stationId);
    NN_PIA_UNUSED(result);
    PIA_SAMPLE_ASSERT(result.IsSuccess());

    s_StationStatus[index].stationId = stationId;
    s_StationStatus[index].pos.x = 0;
    s_StationStatus[index].pos.y = 0;
    s_StationStatus[index].latestIncrementNumber = 0;
    s_StationStatus[index].rtt = nn::pia::transport::Station::InvalidRtt;
}

void DeactivateStationStatus(const nn::pia::StationId stationId)
{
    nn::pia::StationIndex index;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&index, stationId);
    NN_PIA_UNUSED(result);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    PIA_SAMPLE_ASSERT(index < MaxParticipantNum);  // 接続していない相手の離脱イベントが来ていないか？

    s_StationStatus[index].stationId = nn::pia::StationIdInvalid;  //有効か否かを示すのみです。
    s_StationStatus[index].pos.x = 0;
    s_StationStatus[index].pos.y = 0;
    s_StationStatus[index].latestIncrementNumber = 0;
    s_StationStatus[index].rtt = nn::pia::transport::Station::InvalidRtt;
}

void SampleSessionCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam)
{
    NN_PIA_UNUSED(pApplicationParam);
    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
    {
        PIA_SAMPLE_PRINT("Sample Event JOIN detect.\n");
        ActivateStationStatus(stationId);
    }
    break;
    case nn::pia::session::Session::EventType_EventLeave:
    {
        PIA_SAMPLE_PRINT("Sample Event LEAVE detect.\n");
        DeactivateStationStatus(stationId);
    }
    break;
    case nn::pia::session::Session::EventType_SessionHostChanged:
    {
        PIA_SAMPLE_PRINT("Sample Event Host Change detect.\n");
    }
    break;
    case nn::pia::session::Session::EventType_HostMigrationFailed:
    {
        PIA_SAMPLE_PRINT("Sample Event Host Migration Failed detect.\n");
        // Session::CheckConnectionError() でこのエラーのハンドリングを行っているのでログのみ
    }
    break;
    default:
    {
        // このサンプルでは他のイベントが通知されることはありません。
        PIA_SAMPLE_ASSERT(false);
    }
    break;
    }
}

bool SampleJoinApprovalCallback(const nn::pia::transport::Station::IdentificationToken* pRequesterToken)
{
#if ENABLE_MATCHMAKE_ANALYSE
    uint8_t header = 'L';
    if (*(pRequesterToken->data) == header)
    {
        return false;
    }
#else
    NN_PIA_UNUSED(pRequesterToken);
#endif
    return true;
}


/*!
    @brief PiaTransport を使った通信を行うシーン。
    
    ホストがネットワークを終了すると自分はネットワーク選択シーンへ戻りますが、クライアントはエラーシーンへ進みます。
    
    @return 次に実行するべきシーン No を返します。
 */
SampleSceneNo SceneCommunication()
{
    uint32_t frameNo = 0;
    bool isExit = false;

    nn::pia::transport::StationConnectionInfo localInfo;
    nn::pia::transport::StationConnectionInfo masterInfo;

    nn::pia::transport::Transport::GetInstance()->GetLocalStationConnectionInfo(&localInfo);
    nn::pia::transport::Transport::GetInstance()->GetHostStationConnectionInfo(&masterInfo);

    SampleSceneNo nextSceneNo = SampleSceneNo_Matchmake;

    bool isAsyncProcessInProgress = false;

    uint8_t priority = InitialSelectionPriority;
    uint32_t matchmakeAttribute = MatchmakeAttributeValue;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_Communicate,
        true,
        nn::pia::session::Session::GetInstance()->IsHost(),
        nn::pia::session::Session::GetInstance()->GetStationNum());

#if ENABLE_MATCHMAKE_ANALYSE
    bool isOpenParticipation = true;
#endif

    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();
        nn::pia::session::Session::SessionOpenStatus openStatus = nn::pia::session::Session::GetInstance()->GetSessionOpenStatus();
#if ENABLE_MATCHMAKE_ANALYSE
        isOpenParticipation = (openStatus != nn::pia::session::Session::SessionOpenStatus_Close);
#endif

        // PiaSession の非同期処理確認
        if (isAsyncProcessInProgress)
        {
#if ENABLE_MATCHMAKE_ANALYSE
            if (nn::pia::session::Session::GetInstance()->IsOpenSessionCompleted())
            {
                nn::pia::Result result = nn::pia::session::Session::GetInstance()->GetOpenSessionResult();
                if (result.IsSuccess() && !isOpenParticipation)
                {
                    PIA_SAMPLE_PRINT("Open session complete and success\n");
                    // 参加可 / 不可のフラグを持って置いて画面に表示
                    isOpenParticipation = true;
                }
                isAsyncProcessInProgress = false;
            }
            if (nn::pia::session::Session::GetInstance()->IsCloseSessionCompleted())
            {
                nn::pia::Result result = nn::pia::session::Session::GetInstance()->GetCloseSessionResult();
                if (result.IsSuccess() && isOpenParticipation)
                {
                    PIA_SAMPLE_PRINT("Close session complete and success\n");
                    // 参加可 / 不可のフラグを持って置いて画面に表示
                    isOpenParticipation = false;
                }
                isAsyncProcessInProgress = false;
            }
#else
            if (nn::pia::session::Session::GetInstance()->IsOpenSessionCompleted())
            {
                nn::pia::Result result = nn::pia::session::Session::GetInstance()->GetOpenSessionResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("Open session complete and success\n");
                }
                isAsyncProcessInProgress = false;
            }
            if (nn::pia::session::Session::GetInstance()->IsCloseSessionCompleted())
            {
                nn::pia::Result result = nn::pia::session::Session::GetInstance()->GetCloseSessionResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("Close session complete and success\n");
                }
                isAsyncProcessInProgress = false;
            }
#endif
            if (nn::pia::session::Session::GetInstance()->IsUpdateSessionSettingCompleted())
            {
                nn::pia::Result result = nn::pia::session::Session::GetInstance()->GetUpdateSessionSettingResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("UpdateSessionSetting Complete and Success\n");
                }
                isAsyncProcessInProgress = false;
            }
        }
        else
        {
            nn::pia::Result result = nn::pia::session::Session::GetInstance()->CheckConnectionError();

            if (result.IsFailure())
            {
                if (result == nn::pia::ResultNetworkConnectionIsLost() || 
                    result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                    result == nn::pia::ResultWifiOff() ||
                    result == nn::pia::ResultSleep())
                {
                    PIA_SAMPLE_PRINT("Session::CheckConnection: Disconnected NETWORK\n");
                    nextSceneNo = SampleSceneNo_ShowErrorViewer;
                    SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                    SetNetworkErrorResult(result);
                }
                else if (result == nn::pia::ResultSessionConnectionIsLost())
                {
                    PIA_SAMPLE_PRINT("Session::CheckConnection: Disconnected SESSION\n");
                    nextSceneNo = SampleSceneNo_ShowErrorApp;
                    SetCommunicationErrorType(SampleErrorType_SessionLost);
                    SetNetworkErrorResult(result);
                }
                isExit = true;
            }
#if PIA_TEST_CODE
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
                SampleSceneNo_Communicate,
                result.IsSuccess(),
                nn::pia::session::Session::GetInstance()->IsHost(),
                nn::pia::session::Session::GetInstance()->GetStationNum());

            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(
                ProgramParamMatchmakeSessionParticipant,
                nn::pia::session::Session::GetInstance()->GetMatchmakeSessionStationNum());
#endif
        }

        //下画面描画処理
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

            //文字列
            {
                int line = 0;

                {
                    for (int i = 0; i < MaxParticipantNum; i++)
                    {
                        int32_t xBuf = 10;
                        int32_t yBuf = line + 10 * i;
                        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(RectangleColorArray[i]);
                        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                            xBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontWidth() / 2 - 2,
                            yBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontHeight() / 2 - 2,
                            "*");
                        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                        xBuf += 10;

                        if (s_StationStatus[i].stationId != nn::pia::StationIdInvalid)
                        {
                            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(xBuf, yBuf, "Idx=%02X, %05X, RTT:%4d", i, s_StationStatus[i].latestIncrementNumber, s_StationStatus[i].rtt);
                        }
                        else
                        {
                            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(xBuf, yBuf, "No Connection");
                        }
                    }
                }

                line = nns::pia::BasicLowerScreenHeight - (nns::pia::BasicFontSize * 14);
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Network Information ]");
                line += nns::pia::BasicFontSize;

                //通信関連データの画面表示（デバッグ用）
                nn::pia::transport::Transport::GetInstance()->GetLocalStationConnectionInfo(&localInfo);
                nn::pia::transport::Transport::GetInstance()->GetHostStationConnectionInfo(&masterInfo);

                nn::pia::common::FixedString<32> mainStr;
                nn::pia::common::FixedString<32> subStr;
                localInfo.GetMainLocation().GetStationAddress().GetInetAddress().GetAddressString(&mainStr);
                localInfo.GetSubLocation().GetStationAddress().GetInetAddress().GetAddressString(&subStr);
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Local  Main:%s", mainStr.GetCharArray());
                line += nns::pia::BasicFontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "          Sub:%s", subStr.GetCharArray());
                line += nns::pia::BasicFontSize;
                masterInfo.GetMainLocation().GetStationAddress().GetInetAddress().GetAddressString(&mainStr);
                masterInfo.GetSubLocation().GetStationAddress().GetInetAddress().GetAddressString(&subStr);
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Host Main:%s", mainStr.GetCharArray());
                line += nns::pia::BasicFontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "          Sub:%s", subStr.GetCharArray());
                line += nns::pia::BasicFontSize;

                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
                line += nns::pia::BasicFontSize;

                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  B: LeaveSession");
                line += nns::pia::BasicFontSize;
                if (nn::pia::session::Session::GetInstance()->IsHost())
                {
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Y: Change Matchmake Selection Priority");
                    line += nns::pia::BasicFontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  RIGHT+L: Update SessionSettings");
                    line += nns::pia::BasicFontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  LEFT +L: Update ApplicationData");
                    line += nns::pia::BasicFontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  DOWN +L: Update AllAttributes");
                    line += nns::pia::BasicFontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  UP   +L: Update Attribute[%d]", MatchmakeAttributeIndex);
                    line += nns::pia::BasicFontSize;
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  R: %s", (openStatus == nn::pia::session::Session::SessionOpenStatus_Open) ? "CloseSession" : "OpenSession");
                    line += nns::pia::BasicFontSize;
#if ENABLE_MATCHMAKE_ANALYSE
                    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  START: ApprovalCallback(%s)", s_IsApproved ? "Set Deny" : "Set approval");
                    line += nns::pia::BasicFontSize;
#endif
                }

                nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();
            }
        }
        //上画面描画処理
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);

            {
                for (int i = 0; i < MaxParticipantNum; i++)
                {
                    if (s_StationStatus[i].stationId != nn::pia::StationIdInvalid)
                    {
                        s_StationStatus[i].UpdateInfo();

                        int32_t xBuf = s_StationStatus[i].pos.x;
                        int32_t yBuf = s_StationStatus[i].pos.y;
                        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(RectangleColorArray[i]);
                        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
                            xBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontWidth() / 2 - 2,
                            yBuf + nns::pia::sampleframework::SampleFramework::GetInstance()->GetFontHeight() / 2 - 2,
                            "*");
                        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(s_StationStatus[i].pos.x + 10, s_StationStatus[i].pos.y, "<-ID:%02X", i);
                    }
                }
            }

            int line = nns::pia::BasicUpperScreenHeight - nns::pia::BasicFontSize * 8;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", nns::pia::Basic_GetPiaVersionString());
            line += nns::pia::BasicFontSize;

            //自分自身の役割チェック
            int roleIndex = 0;  //Unknown;

            if (nn::pia::session::Session::GetInstance()->GetLocalStationId() != nn::pia::StationIdInvalid)
            {
                roleIndex = (nn::pia::session::Session::GetInstance()->IsHost() ? 1 : 2);  // 1:Host, 2:Client
            }

            // マッチメイク情報
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Communication");
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationNum %d   Role=%s", nn::pia::session::Session::GetInstance()->GetStationNum(), SessionRoleString[roleIndex]);
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  SessionID %d", nn::pia::session::Session::GetInstance()->GetSessionId());
            line += nns::pia::BasicFontSize;
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  SessionOpenStatus=%s", (openStatus == nn::pia::session::Session::SessionOpenStatus_Open) ? "open" : "close");
                line += nns::pia::BasicFontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Matchmake Selection Priority=%d", priority);
                line += nns::pia::BasicFontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Attribute[%d] Value:%d", MatchmakeAttributeIndex, matchmakeAttribute);
                line += nns::pia::BasicFontSize;
            }
            nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();
        }

        // Dispatch 処理
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
#if !ENABLE_LAN_MATCHMAKE
        nns::pia::Nex_SchedulerDispatch();
#endif
        //受信処理
        // Scheduler::Dispatch() で受信処理が行われる都合上、 Dispatch（１回目）の後に実行
        {
            // 相手が送信したデータのサイズは基本的に想定できないので、UnreliableProtocol の MAX_DATA_SIZE
            // 分の受信バッファを確保します。
            PIA_ATTRIBUTE_ALIGN(8) static uint8_t receiveBuffer[nn::pia::transport::UnreliableProtocol::MaxDataSize];

            nn::pia::Result result;
            //受信した数は分からないので、バッファが空になるまで受信処理を繰り返します。
            do
            {
                nn::pia::StationId sourceStationId;
                uint32_t receivedDataSize = 0;

                result = nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_UnreliableProtocolHandle)->Receive(&sourceStationId, receiveBuffer, &receivedDataSize, sizeof(receiveBuffer));

                if (result.IsSuccess())
                {
                    if (sourceStationId != nn::pia::StationIdInvalid)
                    {
                        //受信したデータが正しいかをチェックする必要があるかもしれません
                        nn::pia::StationIndex sourceStatinIndex;
                        result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&sourceStatinIndex, sourceStationId);
                        if (result.IsSuccess() && s_StationStatus[sourceStatinIndex].stationId != nn::pia::StationIdInvalid)
                        {
                            s_StationStatus[sourceStatinIndex].Deserialize(&receiveBuffer);
                        }
                    }
                }
                else if (result == nn::pia::ResultNotInCommunication())
                {
                    //通信切断による受信失敗
                    //直前の Dispatch 処理で非同期のセッション破棄／離脱処理が終了した場合に発生することがあります
                    //正常状態であれば、次ゲームフレーム開始時の Session の状態チェックで DISCONNECTED になっています
                    PIA_SAMPLE_PRINT("UnreliableProtocol::Receive failed. (IsNotInCommunication)\n");
                    break;
                }
                else if (result != nn::pia::ResultNoData())
                {
                    //その他の受信失敗
                    //このデモではあまり気にしない
                }
            } while (result != nn::pia::ResultNoData());
        }
        int move_x = 0;  //自身の矩形移動量(x)
        int move_y = 0;  //自身の矩形移動量(y)

        //矩形の移動
        {
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
            {
                move_x = -1;
            }
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
            {
                move_x = 1;
            }
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
            {
                move_y = -1;
            }
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
            {
                move_y = 1;
            }
        }

        //送信処理 当然排他!
        // Scheduler::Dispatch() で送信処理が行われる都合上、 Dispatch（２回目）の前に実行
        {
            nn::pia::StationId myStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
            nn::pia::StationIndex myStationIndex;
            nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&myStationIndex, myStationId);

            // ConvertToStationIndex() に失敗していたら、既にセッションから切断されているので、送信処理を行わない
            if (result.IsSuccess())
            {
                //自身のデータを更新
                if (move_x != 0)
                {
                    s_StationStatus[myStationIndex].pos.x = (s_StationStatus[myStationIndex].pos.x + nns::pia::BasicUpperScreenWidth + move_x) % nns::pia::BasicUpperScreenWidth;
                }
                else
                {
                    //操作していない場合にも、常時通信が行われていることを示すため 矩形を動かします
                    s_StationStatus[myStationIndex].pos.x = (s_StationStatus[myStationIndex].pos.x + nns::pia::BasicUpperScreenWidth + 1) % nns::pia::BasicUpperScreenWidth;
                }
                if (move_y != 0)
                {
                    s_StationStatus[myStationIndex].pos.y = (s_StationStatus[myStationIndex].pos.y + nns::pia::BasicUpperScreenHeight + move_y) % nns::pia::BasicUpperScreenHeight;
                }
                s_StationStatus[myStationIndex].latestIncrementNumber = frameNo;

                if ((frameNo % (IntervalOfSendingNodeStatus + 1)) == 0)
                {
#if 1
                    uint16_t stationNum = nn::pia::session::Session::GetInstance()->GetStationNum();
                    if (stationNum > 1)
                    {
                        // SendToAll を使って一斉送信
                        PIA_ATTRIBUTE_ALIGN(8) uint8_t sendDataBuffer[SampleStationStatus::SerializedSize];
                        s_StationStatus[myStationIndex].Serialize(sendDataBuffer);
                        if (nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_UnreliableProtocolHandle)->SendToAll(sendDataBuffer, SampleStationStatus::SerializedSize).IsFailure())
                        {
                            //送信失敗。
                            //このデモではあまり気にしない
                            //PIA_SAMPLE_PRINT("Send Fail\n");
                        }
                    }
#else
                    // Send を使って有効な相手にのみ送信
                    for (int i = 0; i < MaxParticipantNum; i++)
                    {
                        // 自分以外の有効な StationId へ送信
                        if (s_StationStatus[i].stationId != nn::pia::StationIdInvalid && s_StationStatus[i].stationId != myStationId)
                        {
                            PIA_ATTRIBUTE_ALIGN(8) uint8_t sendDataBuffer[SampleStationStatus::SerializedSize];
                            s_StationStatus[myStationIndex].Serialize(sendDataBuffer);
                            if (nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_UnreliableProtocolHandle)->Send(s_StationStatus[i].stationId, sendDataBuffer, SampleStationStatus::SerializedSize).IsFailure())
                            {
                                //送信失敗。
                                //このデモではあまり気にしない
                                //PIA_SAMPLE_PRINT("Send failed. Dest StationIndex:%d\n", index);
                            }
                        }
                    }
#endif
                }
            }
        }

        // Dispatch 処理（２回目：送信処理用）
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        // Session 接続状態チェック
        {
            nn::pia::Result result = nn::pia::session::Session::GetInstance()->CheckConnectionError();

            if (result.IsFailure())
            {
                if (result == nn::pia::ResultNetworkConnectionIsLost() || 
                    result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                    result == nn::pia::ResultWifiOff() ||
                    result == nn::pia::ResultSleep())
                {
                    PIA_SAMPLE_PRINT("Session::CheckConnection: Disconnected NETWORK\n");
                    nextSceneNo = SampleSceneNo_ShowErrorViewer;
                    SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                    SetNetworkErrorResult(result);
                }
                else if (result == nn::pia::ResultSessionConnectionIsLost())
                {
                    PIA_SAMPLE_PRINT("Session::CheckConnection: Disconnected SESSION\n");
                    nextSceneNo = SampleSceneNo_ShowErrorApp;
                    SetCommunicationErrorType(SampleErrorType_SessionLost);
                    SetNetworkErrorResult(result);
                }
                isExit = true;
            }
#if PIA_TEST_CODE
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
                SampleSceneNo_Communicate,
                result.IsSuccess(),
                nn::pia::session::Session::GetInstance()->IsHost(),
                nn::pia::session::Session::GetInstance()->GetStationNum());

            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(
                ProgramParamMatchmakeSessionParticipant,
                nn::pia::session::Session::GetInstance()->GetMatchmakeSessionStationNum());
#endif
        }


        //ユーザー操作のハンドリング
        {
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                if (!isAsyncProcessInProgress)
                {
                    nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
                    NN_PIA_UNUSED(leaveResult);
                    PIA_SAMPLE_ASSERT(leaveResult.IsSuccess());
                    CleanupPia();
                    nextSceneNo = SampleSceneNo_Matchmake;
                    isExit = true;
                }
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                uint8_t temporaryPriority = 1 - priority;
                if (nn::pia::session::Session::GetInstance()->IsUpdatableSelectionPriority(temporaryPriority))
                {
                    priority = temporaryPriority;
                    nn::pia::Result result = nn::pia::session::Session::GetInstance()->UpdateSelectionPriority(priority);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                          result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
                        nextSceneNo = SampleSceneNo_ShowErrorViewer;
                        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                        SetNetworkErrorResult(result);
                        isExit = true;
                    }
                }
            }
#if ENABLE_MATCHMAKE_ANALYSE
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonStart))
            {
                nn::pia::transport::Station::IdentificationToken token;
                uint8_t* pTokenData;
                if (s_IsApproved)
                {
                    s_IsApproved = false;
                    pTokenData = s_TokenLock;
                }
                else
                {
                    s_IsApproved = true;
                    pTokenData = s_TokenUnlock;
                }
                ::std::memcpy(token.data, pTokenData, nn::pia::transport::Station::IdentificationTokenMaxDataSize);

                nn::pia::session::Session::GetInstance()->SetMatchmakeAnalysisToken(&token);
            }
#endif
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                if (nn::pia::session::Session::GetInstance()->IsHost())
                {
                    if (!isAsyncProcessInProgress)
                    {
                        nn::pia::Result result;
                        if (openStatus == nn::pia::session::Session::SessionOpenStatus_Open)
                        {
                            result = nn::pia::session::Session::GetInstance()->CloseSessionAsync();
                            if (result.IsFailure())
                            {
                                PIA_SAMPLE_PRINT("CloseSession failed\n");
                                PIA_SAMPLE_ASSERT(false);
                                nextSceneNo = SampleSceneNo_ShowErrorViewer;
                                SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                                SetNetworkErrorResult(result);
                                isExit = true;
                            }
                            else
                            {
                                isAsyncProcessInProgress = true;
                            }
                        }
                        else
                        {
                            result = nn::pia::session::Session::GetInstance()->OpenSessionAsync();
                            if (result.IsFailure())
                            {
                                PIA_SAMPLE_PRINT("OpenSession failed\n");
                                PIA_SAMPLE_ASSERT(false);
                                nextSceneNo = SampleSceneNo_ShowErrorViewer;
                                SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                                SetNetworkErrorResult(result);
                                isExit = true;
                            }
                            else
                            {
                                isAsyncProcessInProgress = true;
                            }
                        }
                    }
                }
            }
#if !ENABLE_LAN_MATCHMAKE
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonL))
            {
                if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
                {
                    if (nn::pia::session::Session::GetInstance()->IsHost() && !isAsyncProcessInProgress)
                    {
                        // 一部のパラメータのみの更新の場合、必要ないものに関しては設定は不要です
                        nn::pia::inet::NexUpdateSessionSetting updateSetting;
                        matchmakeAttribute = 100 - matchmakeAttribute;  // MatchmakeAttributeValue と 100 - MatchmakeAttributeValue をループ
                        uint32_t attr[nn::pia::inet::AttributeSizeMax];
                        attr[MatchmakeAttributeIndex] = matchmakeAttribute;
                        for (uint32_t i = 0; i < nn::pia::inet::AttributeSizeMax; i++)
                        {
                            if (i != MatchmakeAttributeIndex)
                            {
                                attr[i] = 111 * i;
                            }
                        }
                        updateSetting.SetAttributes(attr);
                        const nn::pia::common::String pass(SAMPLE_STRING("newpassword"));
                        updateSetting.SetSessionUserPassword(pass);
                        updateSetting.SetSelectionPriority(3);
                        updateSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
                        updateSetting.SetMaxParticipantNum(8);
                        updateSetting.SetMinParticipantNum(2);

                        const uint32_t applicationDataSize = 16;
                        uint8_t applicationData[applicationDataSize];

                        for (uint32_t i = 0; i < applicationDataSize; i++)
                        {
                            applicationData[i] = static_cast<uint8_t>(100 + i);
                        }
                        updateSetting.SetApplicationData(applicationData, sizeof(applicationData));

                        const uint32_t applicationDataVerificationBufSize = 32;
                        uint8_t applicationDataVerificationBuf[applicationDataVerificationBufSize];
                        uint32_t dataSize = updateSetting.GetApplicationDataSize();
                        updateSetting.GetApplicationData(applicationDataVerificationBuf, applicationDataVerificationBufSize);

// スコア付けマッチメイク用のパラメータ更新
#if NN_PIA_CTR
                        nn::cfg::CTR::Initialize();
                        nn::cfg::CTR::SimpleAddressId addrId;
                        nn::cfg::CTR::GetSimpleAddressId(&addrId);
                        uint32_t cfgCountryCode = static_cast<uint32_t>(addrId.GetCountryCode());
                        nn::cfg::CTR::Finalize();
#elif NN_PIA_CAFE
                        nn::act::Initialize();
                        uint32_t addrId = nn::act::GetSimpleAddressId();
                        uint32_t cfgCountryCode = 0;
                        if (addrId != ACT_INVALID_SIMPLE_ADDRESS_ID)
                        {
                            cfgCountryCode = static_cast<uint8_t>((addrId & 0xff000000) >> 24);
                        }
                        nn::act::Finalize();
#elif NN_PIA_ENABLE_ISO_COUNTRY_CODE
                        const nn::pia::common::String cfgCountryCode(SAMPLE_STRING("JP"));
#else
                        uint32_t cfgCountryCode = 0;
#endif
                        updateSetting.SetViolationRate(1);
                        updateSetting.SetDisconnectionRate(1);
                        updateSetting.SetRatingValue(2015);
                        updateSetting.SetCountryCode(cfgCountryCode);
                        updateSetting.SetUpdateGeoIp(true);

                        NN_PIA_UNUSED(applicationDataVerificationBuf);
                        NN_PIA_UNUSED(dataSize);
                        PIA_SAMPLE_PRINT("--- Confirm ApplicationData ----\n");
                        for (uint32_t i = 0; i < dataSize; i++)
                        {
                            PIA_SAMPLE_PRINT("%d,", applicationDataVerificationBuf[i]);
                        }
                        PIA_SAMPLE_PRINT("\n--- Confirm ApplicationData ----\n");

                        nn::pia::Result result = nn::pia::session::Session::GetInstance()->UpdateSessionSettingAsync(&updateSetting);
                        isAsyncProcessInProgress = true;
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("UpdateSessionSettingAsync failed\n");
                            PIA_SAMPLE_ASSERT(false);
                            nextSceneNo = SampleSceneNo_ShowErrorViewer;
                            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                            SetNetworkErrorResult(result);
                            isExit = true;
                            isAsyncProcessInProgress = false;
                        }
                    }
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonLeft))
                {
                    if (nn::pia::session::Session::GetInstance()->IsHost() && !isAsyncProcessInProgress)
                    {
                        nn::pia::inet::NexUpdateSessionSetting updateSetting;
                        const uint32_t applicationDataSize = 16;
                        uint8_t applicationData[applicationDataSize];

                        for (uint32_t i = 0; i < applicationDataSize; i++)
                        {
                            applicationData[i] = static_cast<uint8_t>(100 + i);
                        }
                        updateSetting.SetApplicationData(applicationData, applicationDataSize);

                        nn::pia::Result result = nn::pia::session::Session::GetInstance()->UpdateSessionSettingAsync(&updateSetting);
                        isAsyncProcessInProgress = true;
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("UpdateSessionSettingAsync failed\n");
                            PIA_SAMPLE_ASSERT(false);
                            nextSceneNo = SampleSceneNo_ShowErrorViewer;
                            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                            SetNetworkErrorResult(result);
                            isExit = true;
                            isAsyncProcessInProgress = false;
                        }
                    }
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                {
                    if (nn::pia::session::Session::GetInstance()->IsHost() && !isAsyncProcessInProgress)
                    {
                        nn::pia::inet::NexUpdateSessionSetting updateSetting;

                        matchmakeAttribute = 100 - matchmakeAttribute;  // MatchmakeAttributeValue と 100 - MatchmakeAttributeValue をループ
                        updateSetting.SetAttribute(MatchmakeAttributeIndex, matchmakeAttribute);

                        nn::pia::Result result = nn::pia::session::Session::GetInstance()->UpdateSessionSettingAsync(&updateSetting);
                        isAsyncProcessInProgress = true;
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("UpdateSessionSettingAsync failed\n");
                            PIA_SAMPLE_ASSERT(false);
                            nextSceneNo = SampleSceneNo_ShowErrorViewer;
                            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                            SetNetworkErrorResult(result);
                            isExit = true;
                            isAsyncProcessInProgress = false;
                        }
                    }
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
                {
                    if (nn::pia::session::Session::GetInstance()->IsHost() && !isAsyncProcessInProgress)
                    {
                        nn::pia::inet::NexUpdateSessionSetting updateSetting;
                        uint32_t attributes[nn::pia::inet::AttributeSizeMax];

                        matchmakeAttribute = 100 - matchmakeAttribute;  // MatchmakeAttributeValue と 100 - MatchmakeAttributeValue をループ
                        attributes[MatchmakeAttributeIndex] = matchmakeAttribute;
                        updateSetting.SetAttributes(attributes);

                        nn::pia::Result result = nn::pia::session::Session::GetInstance()->UpdateSessionSettingAsync(&updateSetting);
                        isAsyncProcessInProgress = true;
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("UpdateSessionSettingAsync failed\n");
                            PIA_SAMPLE_ASSERT(false);
                            nextSceneNo = SampleSceneNo_ShowErrorViewer;
                            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
                            SetNetworkErrorResult(result);
                            isExit = true;
                            isAsyncProcessInProgress = false;
                        }
                    }
                }
            }
#endif // !ENABLE_LAN_MATCHMAKE
#if PIA_TEST_CODE
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                nn::pia::transport::ThreadStreamManager* pThreadStreamManager = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_SAMPLE_ASSERT(pThreadStreamManager);  //transport の初期化が終わっていればインスタンスが生成されているはず

                int32_t ratio = 100;
                pThreadStreamManager->SetSendThreadPacketLossRatio(ratio);
                pThreadStreamManager->SetReceiveThreadPacketLossRatio(ratio);
                PIA_SAMPLE_PRINT("Packet dropped emulation ratio (send) = %d\n", ratio);
                PIA_SAMPLE_PRINT("Packet dropped emulation ratio (recv) = %d\n", ratio);
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                nn::pia::transport::ThreadStreamManager* pThreadStreamManager = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_SAMPLE_ASSERT(pThreadStreamManager);  //transport の初期化が終わっていればインスタンスが生成されているはず

                int32_t ratio = 0;
                pThreadStreamManager->SetSendThreadPacketLossRatio(ratio);
                pThreadStreamManager->SetReceiveThreadPacketLossRatio(ratio);
                PIA_SAMPLE_PRINT("Packet dropped emulation ratio (send) = %d\n", ratio);
                PIA_SAMPLE_PRINT("Packet dropped emulation ratio (recv) = %d\n", ratio);
            }
#endif
        }

        frameNo++;
    }

    // 上画面描画クリア
    nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);
    nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

#if PIA_TEST_CODE
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(
        ProgramParamMatchmakeSessionParticipant,
        nn::pia::session::Session::GetInstance()->GetMatchmakeSessionStationNum());
#endif

    for (int i = 0; i < MaxParticipantNum; i++)
    {
        s_StationStatus[i].stationId = nn::pia::StationIdInvalid;
        s_StationStatus[i].pos.x = 0;
        s_StationStatus[i].pos.y = 0;
        s_StationStatus[i].latestIncrementNumber = 0;
        s_StationStatus[i].rtt = nn::pia::transport::Station::InvalidRtt;
    }

    return nextSceneNo;
}
