﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"

#if NN_PIA_NINTENDOSDK
#include <nn/err.h>
#endif

namespace
{

const char* ErrorInfoString[] =
    {
        "Unknown Reason",
        "Initialize Error (wireless OFF)",
        "Initialize Error (Account)",
        "Initialize Error (other reason)",
        "Matchmake is Failed",
        "Session is Lost",
        "Connection is Lost"
    };

static SampleErrorType s_ErrorType = SampleErrorType_None;  //直前に発生したエラーを保持しておく変数
static nns::pia::NetResult s_NetResult = nns::pia::NetResult_Success;
static nn::pia::Result s_PiaResult = nn::pia::ResultSuccess();
}

/*!
    @brief エラーシーンに飛ぶ前にエラー理由をセットする関数です。
 */
void SetCommunicationErrorType(SampleErrorType error)
{
    s_ErrorType = error;
}

void ClearNetworkErrorResult()
{
    s_PiaResult = nn::pia::ResultSuccess();
    s_NetResult = nns::pia::NetResult_Success;
}

// インターネット接続時に発生したエラーのResultを渡してください。
// ここで渡されたResultがネットワークエラーコード変換APIに渡され、
// 画面に表示されます。
void SetNetworkErrorResult(nn::pia::Result result)
{
    s_PiaResult = result;
}

void SetNetworkErrorResult(enum nns::pia::NetResult result)
{
    s_NetResult = result;
}

uint32_t GetNetworkErrorCode(void)
{
    // Pia の Result の方が優先
    if (s_PiaResult.IsFailure())
    {
        return s_PiaResult.GetErrorCode();
    }
    else if (s_NetResult != nns::pia::NetResult_Success && s_NetResult != nns::pia::NetResult_AccountError)
    {
#if ENABLE_LAN_MATCHMAKE
        return 0;
#else
        return nns::pia::Net_GetNetworkErrorCode();
#endif
    }
    else
    {
        return 0;
    }
}

void HandlingErrorResult(nn::pia::Result::HandlingType handlingType)
{
    PIA_SAMPLE_PRINT("HandlingErrorResult: HandlingType=%d\n", handlingType);

    // このサンプルデモでは、これらのハンドリングタイプの Result を本関数に渡されることはない
    if (handlingType == nn::pia::Result::HandlingType_Void ||
        handlingType == nn::pia::Result::HandlingType_Ignorable ||
        handlingType == nn::pia::Result::HandlingType_Retry ||
        handlingType == nn::pia::Result::HandlingType_ProgrammingError)
    {
        PIA_SAMPLE_ASSERT(false);
    }

    // セッション離脱が必要な場合
    if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave ||
        handlingType == nn::pia::Result::HandlingType_LogoutWithLeave)
    {
        nns::pia::Session_LeaveSession();
    }

#if PIA_TEST_CODE
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(
        ProgramParamMatchmakeSessionParticipant,
        nn::pia::session::Session::GetInstance()->GetMatchmakeSessionStationNum());
#endif

    // クリーンアップはどのハンドリングタイプでも行う
    CleanupPia();

    // Cleanup, CleanupWithLeave 以外はインターネット通信時はログアウト処理を行う
    if (handlingType != nn::pia::Result::HandlingType_Cleanup &&
        handlingType != nn::pia::Result::HandlingType_CleanupWithLeave)
    {
#if !ENABLE_LAN_MATCHMAKE
        Logout();
#endif
        if (handlingType == nn::pia::Result::HandlingType_ShutdownNetwork)
        {
            nns::pia::Net_FinalizeNetwork();
        }
        // nn::pia::Result::HandlingType_Finalize
        else
        {
            nns::pia::Net_FinalizeNetwork();
            FinalizePia();
        }
    }
}

/*!
    @brief アプリケーションのエラーメッセージウィンドウでエラー内容を表示することを想定したシーンです。
 */
SampleSceneNo SceneShowErrorByApp()
{
    bool isExit = false;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(SampleSceneNo_ShowErrorApp, false);

    int frameNo = 0;
    // メインループ
    while (isExit == false)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        //ユーザー操作を処理 ※パッド情報の収集は BeginFrame() で行われている
        {
            // A:    Restart
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                isExit = true;
            }
        }

        // 描画処理
        int line;
        {
            // 下画面(通常)
            nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

            line = 0;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, " Scene: A Communication Error Occurred.");
            line += nns::pia::BasicFontSize;
            line += nns::pia::BasicFontSize;

            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  %s", ErrorInfoString[s_ErrorType]);
            line += nns::pia::BasicFontSize;
            line += nns::pia::BasicFontSize;

            line = nns::pia::BasicLowerScreenHeight - (nns::pia::BasicFontSize * 2);
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: Exit and Restart.");
            line += nns::pia::BasicFontSize;
        }
        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();
        {
            // 上画面
            nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);

            line = nns::pia::BasicUpperScreenHeight - nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", nns::pia::Basic_GetPiaVersionString());
            line += nns::pia::BasicFontSize;
        }
        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        // Dispatch 処理
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
#if !ENABLE_LAN_MATCHMAKE
        nns::pia::Nex_SchedulerDispatch();
#endif
        frameNo++;
    }
    HandlingErrorResult(s_PiaResult.GetHandlingType());

    //念のためエラー情報をクリアしておく
    s_ErrorType = SampleErrorType_None;
    ClearNetworkErrorResult();
    return SampleSceneNo_Matchmake;
}


/*!
    @brief エラー・EULAアプレット（エラービューア）でネットワークエラーコードを表示することを想定したシーンです。
 */
SampleSceneNo SceneShowErrorByViewr()
{
    // エラーの Result をネットワークエラーコードに変換します。
    // ここで得られたコードは、エラー・EULAアプレット（エラービューア）に渡されるべきですが、
    // CTR では開発機材が prod モードに設定されていないと利用できませんので、利便性を考慮し、
    // 本サンプルデモではエラー・EULAアプレットを使用していません。
    uint32_t networkErrorCode = GetNetworkErrorCode();

    bool isExit = false;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(SampleSceneNo_ShowErrorViewer, false);

    int frameNo = 0;
    // 描画
    while (isExit == false)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // Aボタンでループ脱出
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            isExit = true;
        }

        int line;
        {
            // 下画面描画
            nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

            line = 0;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, " Scene: A Network Error Occurred.");
            line += nns::pia::BasicFontSize;
            line += nns::pia::BasicFontSize;

            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  %s", ErrorInfoString[s_ErrorType]);
            line += nns::pia::BasicFontSize;
            line += nns::pia::BasicFontSize;

#if NN_PIA_NINTENDOSDK && !PIA_TEST_CODE
            if (s_PiaResult.IsFailure())
            {
                nn::err::ErrorCode errCode;
                if (s_PiaResult.GetErrorCode(&errCode).IsSuccess())
                {
                    nn::err::ShowError(errCode);
                    isExit = true;
                    networkErrorCode = 0;
                }
            }
#endif
            if (networkErrorCode != 0)
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Network error code: %07d", networkErrorCode);
                line += nns::pia::BasicFontSize;
            }
#if !ENABLE_LAN_MATCHMAKE
#if NN_PIA_NINTENDOSDK
#if PIA_TEST_CODE
            if (nns::pia::Inet_GetLoginError().IsFailure())
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Login error code: %08d", nns::pia::Inet_GetLoginError().GetInnerValueForDebug());
                line += nns::pia::BasicFontSize;
            }
#else
            // ログインエラーはエラービューアアプレットで表示済みなので、次のシーンに進む
            isExit = true;
#endif
#else
            if (nns::pia::Inet_GetLoginError() != 0)
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Login error code: %07d", nns::pia::Inet_GetLoginError());
                line += nns::pia::BasicFontSize;
            }
#endif
#endif
            line = nns::pia::BasicLowerScreenHeight - (nns::pia::BasicFontSize * 2);
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: Exit and Restart.");
            line += nns::pia::BasicFontSize;
        }
        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();
        {
            // 上画面
            nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);

            line = nns::pia::BasicUpperScreenHeight - nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", nns::pia::Basic_GetPiaVersionString());
            line += nns::pia::BasicFontSize;
        }

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        frameNo++;
    }
    nn::pia::Result::HandlingType handlingType = nn::pia::Result::HandlingType_Finalize;
    if (s_PiaResult.IsFailure())
    {
        handlingType = s_PiaResult.GetHandlingType();
    }
    HandlingErrorResult(handlingType);

    //念のためエラー情報をクリアしておく
    s_ErrorType = SampleErrorType_None;
    ClearNetworkErrorResult();

    if (handlingType == nn::pia::Result::HandlingType_Cleanup ||
        handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
    {
        return SampleSceneNo_Matchmake;
    }
    else if (handlingType == nn::pia::Result::HandlingType_LogoutWithLeave ||
             handlingType == nn::pia::Result::HandlingType_ShutdownNetwork)
    {
        return SampleSceneNo_Preparation;
    }
    else
    {
        return SampleSceneNo_Reset;
    }
}
