﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"


// 同期通信するデータのID
enum DataId
{
    DataId_Key = 0,
    DataId_Touch,
    DataId_Stick,

#if PIA_TEST_CODE
    DataId_SyncCheck,  // 同期ずれ検知用
#endif
    DataId_Max
};

// キーのID
enum Key
{
    Key_Up = 0,
    Key_Down,
    Key_Left,
    Key_Right,
    Key_A,
    Key_B,
    Key_X,
    Key_Y,
    Key_Max
};


// 同期するキーデータ
struct KeyData
{
    uint32_t holdFlag;
    uint32_t triggerFlag;
    uint32_t releaseFlag;
};

// 同期するタッチパネルデータ
struct TouchData
{
    uint16_t x;
    uint16_t y;
    uint8_t touch;
};

// 同期するアナログパッド／スティックデータ
struct StickData
{
    float x;
    float y;
};


#if PIA_TEST_CODE
const uint32_t SyncCheckBufferSize = MaxStations;

// 同期ずれ検知用データ
struct SyncCheckData
{
    KeyData keyDataArray[SyncCheckBufferSize];
    TouchData touchDataArray[SyncCheckBufferSize];
    StickData stickDataArray[SyncCheckBufferSize];
};
#endif

// コントローラデータ
class ControllerData
{
public:
    void Clear();

protected:
    static uint32_t KeyToButton(Key key);

protected:
    KeyData m_KeyData;
    TouchData m_TouchData;
    StickData m_StickData;

#if PIA_TEST_CODE
    SyncCheckData m_SyncCheckData;
#endif
};


// 送信するためのコントローラデータを管理する
class ControllerReader : public ControllerData
{
public:
    const void* GetData(DataId dataID) const;
    void Read();

#if PIA_TEST_CODE
    // 同期ずれ検知用データをセット
    void SetSyncCheckKeyData(uint32_t n, uint32_t holdFlag, uint32_t triggerFlag, uint32_t releaseFlag);
    void SetSyncCheckTouchData(uint32_t n, uint16_t x, uint16_t y, uint8_t touch);
    void SetSyncCheckStickData(uint32_t n, float x, float y);
#endif
};


// 受信したコントローラデータを管理する
class ControllerAccessor : public ControllerData
{
public:
    void* GetDataPtr(DataId dataID);

    bool IsHold(Key key) const
    {
        return (m_KeyData.holdFlag & (0x1 << key)) != 0;
    }
    bool IsTrigger(Key key) const
    {
        return (m_KeyData.triggerFlag & (0x1 << key)) != 0;
    }
    bool IsRelease(Key key) const
    {
        return (m_KeyData.releaseFlag & (0x1 << key)) != 0;
    }

    uint32_t GetHoldFlag() const
    {
        return m_KeyData.holdFlag;
    }
    uint32_t GetTriggerFlag() const
    {
        return m_KeyData.triggerFlag;
    }
    uint32_t GetReleaseFlag() const
    {
        return m_KeyData.releaseFlag;
    }

    bool IsTouch() const
    {
        return (m_TouchData.touch > 0);
    }
    uint16_t GetTouchX() const
    {
        return m_TouchData.x;
    }
    uint16_t GetTouchY() const
    {
        return m_TouchData.y;
    }

    float GetStickX() const
    {
        return m_StickData.x;
    }
    float GetStickY() const
    {
        return m_StickData.y;
    }

#if PIA_TEST_CODE
    // 同期ずれ検知用データを取得
    uint32_t GetSyncCheckHoldFlag(uint32_t n) const
    {
        return m_SyncCheckData.keyDataArray[n].holdFlag;
    }
    uint32_t GetSyncCheckTriggerFlag(uint32_t n) const
    {
        return m_SyncCheckData.keyDataArray[n].triggerFlag;
    }
    uint32_t GetSyncCheckReleaseFlag(uint32_t n) const
    {
        return m_SyncCheckData.keyDataArray[n].releaseFlag;
    }
    uint16_t GetSyncCheckTouchX(uint32_t n) const
    {
        return m_SyncCheckData.touchDataArray[n].x;
    }
    uint16_t GetSyncCheckTouchY(uint32_t n) const
    {
        return m_SyncCheckData.touchDataArray[n].y;
    }
    uint8_t GetSyncCheckIsTouch(uint32_t n) const
    {
        return m_SyncCheckData.touchDataArray[n].touch;
    }
    float GetSyncCheckStickX(uint32_t n) const
    {
        return m_SyncCheckData.stickDataArray[n].x;
    }
    float GetSyncCheckStickY(uint32_t n) const
    {
        return m_SyncCheckData.stickDataArray[n].y;
    }
#endif
};
