﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"

#include "SampleConfig.h"
#include "ControllerData.h"

#include <cstdio>

#if USE_PIA_LOCAL
#include "SampleLib/Local_Setup.h"
#endif

#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Common_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Sync_Setup.h"

namespace
{
PIA_ATTRIBUTE_ALIGN(8) static uint8_t s_Memory[MemoryBlockSize];  // nn::pia::common::Initialize()で供給するメモリ

static ControllerReader* s_pControllerReader;
static ControllerAccessor s_ControllerAccessor[MaxStations];

static uint32_t s_UsingDataIdBitmapAll = (0x1 << DataId_Max) - 1;
static uint32_t s_MyUsingDataIdBitmap;
static int32_t s_Delay;
static uint32_t s_SendPeriod;
static int s_CursorConfigId;

// 遅延変更用
static uint32_t s_ChangeDelayInput;
static uint32_t s_ChangeDelay;

// 送信間隔変更用
static uint32_t s_ChangeSendPeriod;

nn::pia::StationId s_StationIdList[MaxStations];

enum ConfigId
{
#if PIA_TEST_CODE
    ConfigId_Delay = DataId_Max - 1,
#else
    ConfigId_Delay = DataId_Max,
#endif
    ConfigId_SendPeriod,
    ConfigId_Max
};

// 実行中のモードを表します。
// inet, lan では、当初ランダムマッチメイクで始まって、マッチング後にホストになるかクライアントになるか決まります。
enum GameMode
{
#if USE_PIA_LOCAL
    GameMode_LocalHost,
    GameMode_LocalClient,
    GameMode_LocalError,
#endif
#if NNS_PIA_ENABLE_INET
    GameMode_InetRandom,
    GameMode_InetHost,
    GameMode_InetClient,
    GameMode_InetError,
#endif
    GameMode_LanRandom,
    GameMode_LanHost,
    GameMode_LanClient,
    GameMode_LanError,
    GameMode_Exit
};

nn::pia::StationIndex ConvertStationIdToStationIndex(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
    if (result.IsFailure())
    {
        stationIndex = nn::pia::StationIndex_Invalid;
    }
    return stationIndex;
}

void ClearStationIdList()
{
    for (int i = 0; i < MaxStations; ++i)
    {
        s_StationIdList[i] = nn::pia::StationIdInvalid;
    }
}

void RegisterStationId(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex = ConvertStationIdToStationIndex(stationId);
    PIA_SAMPLE_ASSERT(stationIndex < MaxStations);
    PIA_SAMPLE_ASSERT(s_StationIdList[stationIndex] == nn::pia::StationIdInvalid);
    s_StationIdList[stationIndex] = stationId;
}

void UnregisterStationId(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex = ConvertStationIdToStationIndex(stationId);
    PIA_SAMPLE_ASSERT(stationIndex < MaxStations);
    PIA_SAMPLE_ASSERT(s_StationIdList[stationIndex] != nn::pia::StationIdInvalid);
    s_StationIdList[stationIndex] = nn::pia::StationIdInvalid;
}

// コールバック関数の宣言
void SampleSessionCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam)
{
    NN_PIA_UNUSED(pApplicationParam);
    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
    {
        PIA_SAMPLE_PRINT("Sample Event JOIN detect. StationId(0x%016llx)\n", stationId);
        RegisterStationId(stationId);
    }
    break;
    case nn::pia::session::Session::EventType_EventLeave:
    {
        PIA_SAMPLE_PRINT("Sample Event LEAVE detect. StationId(0x%016llx)\n", stationId);
        UnregisterStationId(stationId);
    }
    break;
    default:
    {
        PIA_SAMPLE_PRINT("Sample Event(%d) detect. StationId(0x%016llx)\n", eventType, stationId);
    }
    }
}

// 実行するモードを決定します。
GameMode DecideGameMode()
{
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Please select game mode.\n [START]Quit\n [Y]LanRandom\n");
#if NNS_PIA_ENABLE_INET
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" [X]InetRandom\n");
#endif
#if USE_PIA_LOCAL
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" [A]LocalHost\n [B]LocalClient\n");
#endif
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" Waiting input\n");

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    GameMode gameMode = GameMode_Exit;
    bool isDecided = false;
    while (!isDecided)
    {
        pSampleFramework->BeginFrame();

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
#if NNS_PIA_ENABLE_INET
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: InetRandom\n");
            gameMode = GameMode_InetRandom;
            isDecided = true;
#endif
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LanRandom\n");
            gameMode = GameMode_LanRandom;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Exit\n");
            gameMode = GameMode_Exit;
            isDecided = true;
        }
#if USE_PIA_LOCAL
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalHost\n");
            gameMode = GameMode_LocalHost;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalClient\n");
            gameMode = GameMode_LocalClient;
            isDecided = true;
        }
#endif

        int y = 10;
        pSampleFramework->DrawString(10, y, PROGRAM_TITLE);
        y += 10;
        pSampleFramework->DrawString(10, y, "Select game mode.");
        y += 10;
#if USE_PIA_LOCAL
        pSampleFramework->DrawString(10, y, "A: LocalHost");
        y += 10;
        pSampleFramework->DrawString(10, y, "B: LocalClient");
        y += 10;
#endif
#if NNS_PIA_ENABLE_INET
        pSampleFramework->DrawString(10, y, "X: InetRandom");
        y += 10;
#endif
        pSampleFramework->DrawString(10, y, "Y: LanRandom");
        y += 10;
        pSampleFramework->DrawString(10, y, "START: Quit this application");

#if PIA_TEST_CODE
        pSampleFramework->SetProgramStatus(isDecided ? 0 : 1, false);
        pSampleFramework->SetProgramParam(0x01, isDecided ? 1 : 0);
#endif

        pSampleFramework->EndFrame();
    }
    return gameMode;
}


#if PIA_TEST_CODE
//UP:1, RIGHT:2, DOWN:4, LEFT:8
static uint32_t s_KeyBit[4] = {
    nns::pia::sampleframework::ButtonUp,
    nns::pia::sampleframework::ButtonRight,
    nns::pia::sampleframework::ButtonDown,
    nns::pia::sampleframework::ButtonLeft
};

void CheckTestInput()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL) &&
        pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
    {
        uint32_t changeDelay = 0;
        for (uint32_t i = 0; i < 4; i++)
        {
            if (pSampleFramework->IsTrigger(s_KeyBit[i]))
            {
                changeDelay += (0x01 << i);
            }
        }

        s_ChangeDelayInput = changeDelay;
        s_ChangeDelay = s_ChangeDelayInput;
    }
}
#endif


// キー入力による操作です。
bool Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::sync::SyncProtocol* pSyncProtocol = nns::pia::Sync_GetSyncProtocol();

    nn::pia::Result r;

    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
    {
        // START で GameLoop() を抜けます。
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Exit GameLoop\n");
        return true;
    }

    switch (pSyncProtocol->GetState())
    {
    case nn::pia::sync::SyncProtocol::State_NotSynchronized:
    {
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            // A で同期通信を開始します。
            r = pSyncProtocol->Start(s_MyUsingDataIdBitmap, s_Delay, s_SendPeriod);
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Start %s\n", r.IsSuccess() ? "Success" : "Failure");
        }

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            --s_CursorConfigId;
            if (s_CursorConfigId < 0)
            {
                s_CursorConfigId = ConfigId_Max - 1;
            }
        }
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            ++s_CursorConfigId;
            if (s_CursorConfigId >= ConfigId_Max)
            {
                s_CursorConfigId = 0;
            }
        }

        if (s_CursorConfigId < ConfigId_Delay)
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft) || pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                s_MyUsingDataIdBitmap ^= (0x1 << s_CursorConfigId);
            }
        }
        else if (s_CursorConfigId == ConfigId_Delay)
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                if (s_Delay > 1)
                {
                    --s_Delay;
                }
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                if (s_Delay < DelayMax)
                {
                    ++s_Delay;
                }
            }
        }
        else if (s_CursorConfigId == ConfigId_SendPeriod)
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                if (s_SendPeriod > 1)
                {
                    --s_SendPeriod;
                }
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                ++s_SendPeriod;
            }
        }
    }
    break;

    case nn::pia::sync::SyncProtocol::State_Waiting:
    case nn::pia::sync::SyncProtocol::State_Starting:
    case nn::pia::sync::SyncProtocol::State_Synchronized:
    {
        if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonR) && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            // R + X で同期通信を終了します。
            r = pSyncProtocol->End();
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("End %s\n", r.IsSuccess() ? "Success" : "Failure");
        }

        if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_Synchronized)
        {
            if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonR) && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                // R + Y で自分だけ同期通信を終了します。
                // 同期ずれの原因となるため、受信した同期データを取得後である必要があります。
                r = pSyncProtocol->EndAlone();
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("EndAlone %s\n", r.IsSuccess() ? "Success" : "Failure");
            }
        }

        if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonR))
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                // R + > で依頼する遅延の値を +1 します
                if (s_ChangeDelayInput < pSyncProtocol->GetMaxDelay())
                {
                    ++s_ChangeDelayInput;
                }
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                // R + < で依頼する遅延の値を -1 します
                if (s_ChangeDelayInput > 0)
                {
                    --s_ChangeDelayInput;
                }
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                // R + A で遅延変更を依頼します
                s_ChangeDelay = s_ChangeDelayInput;
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                // R + ^ で設定する送信間隔の値を +1 します
                ++s_ChangeSendPeriod;
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                // R + v で設定する送信間隔の値を -1 します
                if (s_ChangeSendPeriod > 1)
                {
                    --s_ChangeSendPeriod;
                }
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                // R + B で送信間隔を変更します
                pSyncProtocol->SetSendPeriod(s_ChangeSendPeriod);
            }
        }

#if PIA_TEST_CODE
        if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonL))
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                nn::pia::Result result;
                nn::pia::transport::ThreadStreamManager* threadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_ASSERT(PIA_IS_VALID_POINTER(threadStreamMgr));
                result = threadStreamMgr->SetSendThreadPacketLossRatio(0);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                result = threadStreamMgr->SetReceiveThreadPacketLossRatio(0);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                nn::pia::Result result;
                nn::pia::transport::ThreadStreamManager* threadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_ASSERT(PIA_IS_VALID_POINTER(threadStreamMgr));
                result = threadStreamMgr->SetSendThreadPacketLossRatio(100);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                result = threadStreamMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
        }
#endif

#if PIA_TEST_CODE
        CheckTestInput();
#endif
    }
    break;
    case nn::pia::sync::SyncProtocol::State_Ending:
    case nn::pia::sync::SyncProtocol::State_EndedAlone:
    {
#if PIA_TEST_CODE
        if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonL))
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                nn::pia::Result result;
                nn::pia::transport::ThreadStreamManager* threadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_ASSERT(PIA_IS_VALID_POINTER(threadStreamMgr));
                result = threadStreamMgr->SetSendThreadPacketLossRatio(0);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                result = threadStreamMgr->SetReceiveThreadPacketLossRatio(0);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                nn::pia::Result result;
                nn::pia::transport::ThreadStreamManager* threadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_ASSERT(PIA_IS_VALID_POINTER(threadStreamMgr));
                result = threadStreamMgr->SetSendThreadPacketLossRatio(100);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                result = threadStreamMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
        }
#endif
    }
    break;
    }

    return false;
}


static uint32_t s_DispatchCount = 0;
static int32_t s_DispatchTime = 0;

// 画面の描画処理です。
void Draw(GameMode gameMode)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    nn::pia::sync::SyncProtocol* pSyncProtocol = nns::pia::Sync_GetSyncProtocol();

    static const nns::pia::sampleframework::Color DefaultColor(1.0f, 1.0f, 1.0f, 1.0f);
    static const nns::pia::sampleframework::Color InvalidColor(0.4f, 0.4f, 0.4f, 1.0f);

    pSampleFramework->SetFontColor(DefaultColor);

    {
        pSampleFramework->DrawString(10, 10, PROGRAM_TITLE);
        const char* GameModeString = NULL;
        switch (gameMode)
        {
#if USE_PIA_LOCAL
        case GameMode_LocalHost:
        case GameMode_LocalClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "LocalHost";
            }
            else
            {
                GameModeString = "LocalClient";
            }
        }
        break;
#endif
#if NNS_PIA_ENABLE_INET
        case GameMode_InetRandom:
        {
            GameModeString = "InetRandom";
        }
        break;
        case GameMode_InetHost:
        case GameMode_InetClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "InetHost";
            }
            else
            {
                GameModeString = "InetClient";
            }
        }
        break;
#endif
        case GameMode_LanRandom:
        {
            GameModeString = "LanRandom";
        }
        break;
        case GameMode_LanHost:
        case GameMode_LanClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "LanHost";
            }
            else
            {
                GameModeString = "LanClient";
            }
        }
        break;
        default:
        {
        }
        break;
        }
        PIA_SAMPLE_ASSERT(GameModeString != NULL);
        pSampleFramework->DrawString(10, 20, "GameMode:%s", GameModeString);
        pSampleFramework->DrawString(180, 20, "StationNum:%d", pSession->GetStationNum());
        pSampleFramework->DrawString(290, 20, "Index:%d", ConvertStationIdToStationIndex(pSession->GetLocalStationId()));
    }

    {
        uint32_t idx = (pSampleFramework->GetFrameCount() / 4) % 4;
        const char AnimLetter[] = { '-', '\\', '|', '/' };
        pSampleFramework->DrawString(380, 10, "%c", AnimLetter[idx]);
    }

    {
        // SyncProtocol のステートを表示します。
        const char* cpState = "";
        switch (pSyncProtocol->GetState())
        {
        case nn::pia::sync::SyncProtocol::State_NotSynchronized:
            cpState = "NOT_SYNCHRONIZED";
            break;
        case nn::pia::sync::SyncProtocol::State_Waiting:
            cpState = "WAITING";
            break;
        case nn::pia::sync::SyncProtocol::State_Starting:
            cpState = "STARTING";
            break;
        case nn::pia::sync::SyncProtocol::State_Synchronized:
            cpState = "SYNCHRONIZED";
            break;
        case nn::pia::sync::SyncProtocol::State_Ending:
            cpState = "ENDING";
            break;
        case nn::pia::sync::SyncProtocol::State_EndedAlone:
            cpState = "ENDED_ALONE";
            break;
        }
        pSampleFramework->DrawString(10, 30, "State:%s", cpState);

        if (pSyncProtocol->GetLastEndReason() != nn::pia::sync::SyncProtocol::EndReason_Nothing)
        {
            const char* cpEndReason = "";
            switch (pSyncProtocol->GetLastEndReason())
            {
            case nn::pia::sync::SyncProtocol::EndReason_Nothing:
                break;
            case nn::pia::sync::SyncProtocol::EndReason_End:
                cpEndReason = "END";
                break;
            case nn::pia::sync::SyncProtocol::EndReason_OtherStation:
                cpEndReason = "OTHER_STATION";
                break;
            case nn::pia::sync::SyncProtocol::EndReason_ChangeConnection:
                cpEndReason = "CHANGE_CONNECTION";
                break;
            case nn::pia::sync::SyncProtocol::EndReason_Timeout:
                cpEndReason = "TIMEOUT";
                break;
            case nn::pia::sync::SyncProtocol::EndReason_EndAlone:
                cpEndReason = "END_ALONE";
                break;
            }
            pSampleFramework->DrawString(200, 30, "EndReason:%s", cpEndReason);
        }
    }

    {
        // ステートに応じて可能な操作を表示します。
        switch (pSyncProtocol->GetState())
        {
        case nn::pia::sync::SyncProtocol::State_NotSynchronized:
        {
            pSampleFramework->DrawString(10, 40, "A:Start START:Exit");
        }
        break;
        case nn::pia::sync::SyncProtocol::State_Waiting:
        case nn::pia::sync::SyncProtocol::State_Starting:
        case nn::pia::sync::SyncProtocol::State_Synchronized:
        {
            pSampleFramework->DrawString(10, 40, "R+X:End R+Y:EndAlone");
            pSampleFramework->DrawString(10, 50, "START:Exit");
        }
        break;
        case nn::pia::sync::SyncProtocol::State_Ending:
        case nn::pia::sync::SyncProtocol::State_EndedAlone:
        {
            pSampleFramework->DrawString(10, 40, "START:Exit");
        }
        break;
        }
    }

    if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_NotSynchronized)
    {
        pSampleFramework->DrawString(10, 60, "Key");
        pSampleFramework->DrawString(10, 70, "Touch");
        pSampleFramework->DrawString(10, 80, "Stick");
        pSampleFramework->DrawString(10, 90, "Delay");
        pSampleFramework->DrawString(10, 100, "SendPeriod");

        for (int32_t i = 0; i < ConfigId_Max; ++i)
        {
            int32_t y = 60 + i * 10;
            if (i < ConfigId_Delay)
            {
                pSampleFramework->DrawString(100, y, "%c %s", (i == s_CursorConfigId) ? '>' : ' ', ((0x1 << i) & s_MyUsingDataIdBitmap) ? "ON" : "OFF");
            }
            else if (i == ConfigId_Delay)
            {
                pSampleFramework->DrawString(100, y, "%c %d", (i == s_CursorConfigId) ? '>' : ' ', s_Delay);
            }
            else if (i == ConfigId_SendPeriod)
            {
                pSampleFramework->DrawString(100, y, "%c %d", (i == s_CursorConfigId) ? '>' : ' ', s_SendPeriod);
            }
        }
    }
    else
    {

        {
            // 列タイトルを表示します。
            pSampleFramework->DrawString(5, 70, "UP");
            pSampleFramework->DrawString(5, 80, "DOWN");
            pSampleFramework->DrawString(5, 90, "LEFT");
            pSampleFramework->DrawString(5, 100, "RIGHT");
            pSampleFramework->DrawString(5, 110, "A");
            pSampleFramework->DrawString(5, 120, "B");
            pSampleFramework->DrawString(5, 130, "X");
            pSampleFramework->DrawString(5, 140, "Y");
            pSampleFramework->DrawString(5, 155, "TouchX");
            pSampleFramework->DrawString(5, 165, "TouchY");
            pSampleFramework->DrawString(5, 180, "StickX");
            pSampleFramework->DrawString(5, 190, "StickY");
            pSampleFramework->DrawString(5, 200, "EndAln");
            pSampleFramework->DrawString(5, 210, "RTT");
        }

        {
            // 各ステーションの同期されたコントローラ情報を表示します。
            for (int32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationId stationId = s_StationIdList[i];

                int32_t x = 55 + i * 43;

                pSampleFramework->SetFontColor(pSyncProtocol->CheckEntry(stationId) ? DefaultColor : InvalidColor);
                pSampleFramework->DrawString(x, 60, " [%dP]", i + 1);

                if (pSyncProtocol->CheckEntry(stationId) == false)
                {
                    continue;
                }

                uint32_t usingDataIdBitmap = pSyncProtocol->GetUsingDataIdBitmap(stationId);

                if (usingDataIdBitmap & (0x1 << DataId_Key))
                {
                    for (int32_t j = 0; j < Key_Max; ++j)
                    {
                        int32_t y = 70 + j * 10;
                        Key key = static_cast<Key>(j);
                        pSampleFramework->DrawString(x, y, " %c%c%c", s_ControllerAccessor[i].IsTrigger(key) ? 'T' : '-', s_ControllerAccessor[i].IsHold(key) ? 'H' : '-', s_ControllerAccessor[i].IsRelease(key) ? 'R' : '-');
                    }
                }

                if (usingDataIdBitmap & (0x1 << DataId_Touch))
                {
                    if (s_ControllerAccessor[i].IsTouch())
                    {
                        pSampleFramework->DrawString(x, 155, "%4d", s_ControllerAccessor[i].GetTouchX());
                        pSampleFramework->DrawString(x, 165, "%4d", s_ControllerAccessor[i].GetTouchY());
                    }
                    else
                    {
                        pSampleFramework->DrawString(x, 155, " ---");
                        pSampleFramework->DrawString(x, 165, " ---");
                    }
                }

                if (usingDataIdBitmap & (0x1 << DataId_Stick))
                {
                    pSampleFramework->DrawString(x, 180, "%5.2f", s_ControllerAccessor[i].GetStickX());
                    pSampleFramework->DrawString(x, 190, "%5.2f", s_ControllerAccessor[i].GetStickY());
                }
            }

            pSampleFramework->SetFontColor(DefaultColor);
        }

        {
            // 終了フレームを表示します。
            for (int32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationId stationId = s_StationIdList[i];
                if (stationId == nn::pia::StationIdInvalid)
                {
                    continue;
                }

                int32_t x = 55 + i * 43;
                uint32_t endFrameNo = 0;
                if (pSyncProtocol->GetState() != nn::pia::sync::SyncProtocol::State_NotSynchronized &&
                    pSyncProtocol->GetState() != nn::pia::sync::SyncProtocol::State_Waiting)
                {
                    nn::pia::Result result = pSyncProtocol->GetEndFrameNo(stationId, &endFrameNo);
                    if (result.IsSuccess())
                    {
                        pSampleFramework->DrawString(x, 200, "%4d", endFrameNo);
                    }
                    else
                    {
                        if (pSyncProtocol->CheckEntry(stationId) == false)
                        {
                            continue;
                        }
                        pSampleFramework->DrawString(x, 200, " ---");
                    }
                }
            }
        }

        {
            // RTT を表示します。
            for (int32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationId stationId = s_StationIdList[i];
                if (stationId == nn::pia::StationIdInvalid)
                {
                    continue;
                }

                int32_t x = 55 + i * 43;

                nn::pia::transport::Station* pStation = nn::pia::transport::StationManager::GetInstance()->GetStation(stationId);
                if (pStation != NULL)
                {
                    int rtt = pStation->GetRtt();
                    if (rtt == nn::pia::transport::Station::InvalidRtt)
                    {
                        pSampleFramework->DrawString(x, 210, " ---", rtt);
                    }
                    else
                    {
                        pSampleFramework->DrawString(x, 210, "%4d", rtt);
                    }
                }
            }
        }

        {
            // 遅延を表示します。
            if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_Synchronized)
            {
                pSampleFramework->DrawString(180, 30, "Delay:%d", pSyncProtocol->GetDelay());
            }

            // 変更を依頼する遅延を表示します。
            if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_Synchronized)
            {
                if (IsChangeDelayEnabled)
                {
                    pSampleFramework->DrawString(180, 40, "R+[<>A]:ChangeDelay:%d", s_ChangeDelayInput);
                }
                else
                {
                    pSampleFramework->DrawString(180, 40, "R+[<>A]:ChangeDelay:-");
                }
            }

            // 送信間隔を表示します。
            if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_Synchronized)
            {
                pSampleFramework->DrawString(290, 30, "SendPeriod:%d", pSyncProtocol->GetSendPeriod());
            }

            // 設定する送信間隔を表示します。
            if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_Synchronized)
            {
                pSampleFramework->DrawString(180, 50, "R+[^vB]:ChangeSendPeriod:%d", s_ChangeSendPeriod);
            }

            // フレーム番号を表示します。
            if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_Synchronized)
            {
                pSampleFramework->DrawString(260, 10, "Frame: %d", pSyncProtocol->GetFrameNo());
            }

#if 0
            // 先行フレーム数を表示します。
            if (pSyncProtocol->GetState() == nn::pia::sync::SyncProtocol::State_Synchronized)
            {
                pSampleFramework->DrawString(180, 40, "LeadingFrame:");
                for (uint32_t i = 0; i < pSession->GetStationNum(); i++)
                {
                    pSampleFramework->DrawString(280 + i*20, 40, " %d", pSyncProtocol->EstimateLeadingFrameNum(s_StationIdList[i], FramePerSeconds));
                }
            }
#endif

            // 処理落ちを確認するため、Dispatch の試行回数と所要時間を表示します。
            pSampleFramework->DrawString(10, 230, "Dispatch: %3d (%4dms)", s_DispatchCount, s_DispatchTime);
        }
    }
}

// 同期データをセットする
void SetSyncData()
{
    nn::pia::sync::SyncProtocol* pSyncProtocol = nns::pia::Sync_GetSyncProtocol();

    if (pSyncProtocol->NeedsSetData())
    {
        // 送信するデータを用意してSyncProtocolに設定します。
        s_pControllerReader->Read();

#if PIA_TEST_CODE
        // 同期ずれ検知用データを設定します。
        for (uint32_t i = 0; i < SyncCheckBufferSize; ++i)
        {
            nn::pia::StationId stationId = s_StationIdList[i];
            if (pSyncProtocol->CanGetData() && pSyncProtocol->CheckEntry(stationId))
            {
                s_pControllerReader->SetSyncCheckKeyData(
                    i,
                    s_ControllerAccessor[i].GetHoldFlag(),
                    s_ControllerAccessor[i].GetTriggerFlag(),
                    s_ControllerAccessor[i].GetReleaseFlag());
                s_pControllerReader->SetSyncCheckTouchData(
                    i,
                    s_ControllerAccessor[i].GetTouchX(),
                    s_ControllerAccessor[i].GetTouchY(),
                    s_ControllerAccessor[i].IsTouch());
                s_pControllerReader->SetSyncCheckStickData(
                    i,
                    s_ControllerAccessor[i].GetStickX(),
                    s_ControllerAccessor[i].GetStickY());
            }
        }

#if 0
        // わざと同期ずれ検知に引っかかります。
        if (pSyncProtocol->GetFrameNo() == 100 &&
            nn::pia::session::Session::GetInstance()->IsHost())
        {
            s_pControllerReader->SetSyncCheckKeyData(
                0,
                s_ControllerAccessor[0].GetHoldFlag() + 1,
                s_ControllerAccessor[0].GetTriggerFlag(),
                s_ControllerAccessor[0].GetReleaseFlag()
            );
        }
#endif

#endif

        for (uint32_t i = 0; i < DataId_Max; ++i)
        {
            DataId dataId = static_cast<DataId>(i);
            if (pSyncProtocol->NeedsSetData(dataId))
            {
                nn::pia::Result result = pSyncProtocol->SetData(dataId, s_pControllerReader->GetData(dataId));
                PIA_SAMPLE_ASSERT(result.IsSuccess());
                NN_PIA_UNUSED_IN_RELEASE(result);
            }
        }

        if (IsChangeDelayEnabled)
        {
            // 遅延変更を依頼します。
            nn::pia::Result result = pSyncProtocol->RequestToChangeDelay(s_ChangeDelay);
            PIA_SAMPLE_ASSERT(result.IsSuccess());
            NN_PIA_UNUSED_IN_RELEASE(result);
            s_ChangeDelay = 0;
        }
    }
}

#if PIA_TEST_CODE
void CheckSyncData()
{
    nn::pia::sync::SyncProtocol* pSyncProtocol = nns::pia::Sync_GetSyncProtocol();

    // 比較の基準となるステーション
    uint32_t baseIndex = 0;
    for (uint32_t i = 0; i < SyncCheckBufferSize; ++i)
    {
        if (pSyncProtocol->CheckEntry(s_StationIdList[i]))
        {
            baseIndex = i;
            break;
        }
    }

    // ステーション j から送られてきた、ステーション i についてのデータをチェックする
    for (uint32_t i = 0; i < SyncCheckBufferSize; ++i)
    {
        if (!pSyncProtocol->CheckEntry(s_StationIdList[i]))
        {
            continue;
        }

        // ステーション baseIndex から送られてきたデータを基準にチェックする
        uint32_t holdFlag = s_ControllerAccessor[baseIndex].GetSyncCheckHoldFlag(i);
        uint32_t triggerFlag = s_ControllerAccessor[baseIndex].GetSyncCheckTriggerFlag(i);
        uint32_t releaseFlag = s_ControllerAccessor[baseIndex].GetSyncCheckReleaseFlag(i);
        uint16_t touchX = s_ControllerAccessor[baseIndex].GetSyncCheckTouchX(i);
        uint16_t touchY = s_ControllerAccessor[baseIndex].GetSyncCheckTouchY(i);
        uint8_t isTouch = s_ControllerAccessor[baseIndex].GetSyncCheckIsTouch(i);
        float stickX = s_ControllerAccessor[baseIndex].GetSyncCheckStickX(i);
        float stickY = s_ControllerAccessor[baseIndex].GetSyncCheckStickY(i);

        for (uint32_t j = baseIndex + 1; j < MaxStations; ++j)
        {
            if (!pSyncProtocol->CheckEntry(s_StationIdList[j]))
            {
                continue;
            }

            if (holdFlag != s_ControllerAccessor[j].GetSyncCheckHoldFlag(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
            if (triggerFlag != s_ControllerAccessor[j].GetSyncCheckTriggerFlag(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
            if (releaseFlag != s_ControllerAccessor[j].GetSyncCheckReleaseFlag(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
            if (touchX != s_ControllerAccessor[j].GetSyncCheckTouchX(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
            if (touchY != s_ControllerAccessor[j].GetSyncCheckTouchY(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
            if (isTouch != s_ControllerAccessor[j].GetSyncCheckIsTouch(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
            if (stickX != s_ControllerAccessor[j].GetSyncCheckStickX(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
            if (stickY != s_ControllerAccessor[j].GetSyncCheckStickY(i))
            {
                PIA_SAMPLE_ASSERT(false);
            }
        }
    }
}

void UpdatePiaTestData(GameMode gameMode, bool isExit)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();

    nn::pia::session::Session::Status sessionStatus = pSession->GetStatus();
    bool isConnected = pSession->CheckConnectionError().IsSuccess() &&
                       sessionStatus == nn::pia::session::Session::Status_ConnectedSession;

    uint32_t sceneNo = 0;
    if (!isExit)
    {
        if (gameMode == GameMode_LanHost || gameMode == GameMode_LanClient)
        {
            sceneNo = 5;
        }
#if NNS_PIA_ENABLE_INET
        else if (gameMode == GameMode_InetHost || gameMode == GameMode_InetClient)
        {
            sceneNo = 4;
        }
#endif
#if USE_PIA_LOCAL
        else if (gameMode == GameMode_LocalHost || gameMode == GameMode_LocalClient)
        {
            sceneNo = 3;
        }
#endif
    }
    pSampleFramework->SetProgramStatus(
        sceneNo,
        isConnected,
        pSession->GetLocalStationId() == pSession->GetHostStationId(),
        pSession->GetStationNum());


    nn::pia::sync::SyncProtocol* pSyncProtocol = nns::pia::Sync_GetSyncProtocol();

    // Session
    pSampleFramework->SetProgramParam(0x00, ConvertStationIdToStationIndex(pSession->GetLocalStationId()));

    // SyncProtocol全体
    pSampleFramework->SetProgramParam(0x10, pSyncProtocol->GetState());
    pSampleFramework->SetProgramParam(0x11, pSyncProtocol->GetLastEndReason());
    pSampleFramework->SetProgramParam(0x12, pSyncProtocol->GetFrameNo());
    pSampleFramework->SetProgramParam(0x13, pSyncProtocol->GetDelay());
    pSampleFramework->SetProgramParam(0x14, pSyncProtocol->GetSendPeriod());

    // 自分の設定
    pSampleFramework->SetProgramParam(0x20, s_CursorConfigId);
    pSampleFramework->SetProgramParam(0x21, s_MyUsingDataIdBitmap);
    pSampleFramework->SetProgramParam(0x22, s_Delay);
    pSampleFramework->SetProgramParam(0x23, s_SendPeriod);
    pSampleFramework->SetProgramParam(0x24, s_ChangeDelayInput);
    pSampleFramework->SetProgramParam(0x25, s_ChangeSendPeriod);

    // 個々のステーション
    for (uint32_t i = 0; i < MaxStations; ++i)
    {
        nn::pia::StationId stationId = s_StationIdList[i];
        uint32_t idOfs = (i + 8) << 4;

        pSampleFramework->SetProgramParam(idOfs + 0x0, pSyncProtocol->CheckEntry(stationId) ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0x1, pSyncProtocol->GetUsingDataIdBitmap(stationId));

        pSampleFramework->SetProgramParam(idOfs + 0x2, s_ControllerAccessor[i].GetHoldFlag());
        pSampleFramework->SetProgramParam(idOfs + 0x3, s_ControllerAccessor[i].GetTriggerFlag());
        pSampleFramework->SetProgramParam(idOfs + 0x4, s_ControllerAccessor[i].GetReleaseFlag());
        pSampleFramework->SetProgramParam(idOfs + 0x5, s_ControllerAccessor[i].IsTouch() ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0x6, s_ControllerAccessor[i].GetTouchX());
        pSampleFramework->SetProgramParam(idOfs + 0x7, s_ControllerAccessor[i].GetTouchY());
        pSampleFramework->SetProgramParam(idOfs + 0x8, static_cast<int32_t>(s_ControllerAccessor[i].GetStickX() * 1000.0f));
        pSampleFramework->SetProgramParam(idOfs + 0x9, static_cast<int32_t>(s_ControllerAccessor[i].GetStickY() * 1000.0f));
    }
}
#endif

// 通信中のメインループです。
void MainLoop(GameMode gameMode, bool* pIsLeaveNeeded)
{
    nn::pia::Result r;

    s_pControllerReader->Clear();
    for (int i = 0; i < MaxStations; ++i)
    {
        s_ControllerAccessor[i].Clear();
    }

    nn::pia::sync::SyncProtocol* pSyncProtocol = nns::pia::Sync_GetSyncProtocol();

    s_DispatchCount = 0;
    nn::pia::common::Time start;

    while (true)
    {
        bool isExit = false;

        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        if (s_DispatchCount == 0)
        {
            start.SetNow();
        }
        ++s_DispatchCount;

#if NNS_PIA_ENABLE_INET
        // GameMode_InetXXX の場合は nn::nex::Scheduler が作られていて、そのディスパッチも行う必要があります。
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->GetFrameCount() % NexDispatchPeriod == 0)
        {
            nns::pia::Nex_SchedulerDispatch();
        }
#endif

        // nn::pia::common::Scheduler::Dispatch() を定期的に呼び出す必要があります。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

#if NNS_PIA_ENABLE_INET
        if (gameMode == GameMode_LanHost || gameMode == GameMode_LanClient || gameMode == GameMode_InetHost || gameMode == GameMode_InetClient)
#else
        if (gameMode == GameMode_LanHost || gameMode == GameMode_LanClient)
#endif
        {
            if (nns::pia::Net_IsConnected() == false)
            {
                // AC が切断したので終了します。
                isExit = true;
                *pIsLeaveNeeded = false;
            }
        }

        nn::pia::Result result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
        if (result.IsFailure())
        {
            // 通信が切断したので終了します。
            isExit = true;
            nn::pia::Result::HandlingType handlingType = result.GetHandlingType();
            PIA_SAMPLE_ASSERT(handlingType == nn::pia::Result::HandlingType_CleanupWithLeave || handlingType == nn::pia::Result::HandlingType_ShutdownNetwork);
            if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
            {
                *pIsLeaveNeeded = true;
            }
            else
            {
                *pIsLeaveNeeded = false;
            }
        }

        if (!isExit)
        {
            // 操作を受け付けます。
            isExit = Operate();
            if (isExit)
            {
                *pIsLeaveNeeded = true;
            }
        }

        if (isExit)
        {
            // 通信中のメインループを抜けます
            nn::pia::common::Scheduler::GetInstance()->Dispatch();

            Draw(gameMode);

#if PIA_TEST_CODE
            UpdatePiaTestData(gameMode, isExit);
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(0x01, isExit ? 1 : 0);
#endif

            nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
            return;
        }

        // 1 フレーム進めます。
        r = pSyncProtocol->Step();
        if (r.IsFailure())
        {
            if (r == nn::pia::ResultDataIsNotArrivedYet())
            {
                // まだデータが届いていないのでゲームフレームをドロップさせます
                // データを設定してからフレームが進んでいないので、ここで設定する必要はありません。
                PIA_SAMPLE_ASSERT(pSyncProtocol->NeedsSetData() == false);
                nn::pia::common::Scheduler::GetInstance()->Dispatch();
                Draw(gameMode);
#if PIA_TEST_CODE
                UpdatePiaTestData(gameMode, isExit);
#endif
                nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

                continue;
            }
            else if (r == nn::pia::ResultDataIsNotSet())
            {
                // 既にデータを設定してあるので、この Result にはなりません。
                PIA_SAMPLE_ASSERT(false);
            }
            else if (r == nn::pia::ResultInvalidState())
            {
                // 通信中なので、この Result にはなりません。
                PIA_SAMPLE_ASSERT(false);
            }
            else if (r == nn::pia::ResultTemporaryUnavailable())
            {
                // 単独同期終了処理中には、この Result が返りますが、ハンドリングの必要はありません。
            }
            else
            {
                // その他の Result が返ることはありません。
                PIA_SAMPLE_ASSERT(false);
            }
        }

        // 同期通信中の遅延変更機能を利用している場合は、データを取得できないフレームが生じる可能性があるため、
        // 現在のフレームはデータを取得できるフレームかをチェックする必要があります。
        bool isNoDataFrame = false;
        if (pSyncProtocol->CanGetData())
        {
            r = pSyncProtocol->GetIsNoDataFrame(&isNoDataFrame);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }

        // 受信したデータを取得します。自分のデータもここで取得したものを使います。
        for (int i = 0; i < MaxStations; ++i)
        {
            nn::pia::StationId stationId = s_StationIdList[i];
            s_ControllerAccessor[i].Clear();

            if (isNoDataFrame)
            {
                continue;
            }

            if (pSyncProtocol->CanGetData() && pSyncProtocol->CheckEntry(stationId))
            {
                uint32_t usingDataIdBitmap = pSyncProtocol->GetUsingDataIdBitmap(stationId);
                for (uint32_t j = 0; j < DataId_Max; ++j)
                {
                    DataId dataId = static_cast<DataId>(j);
                    if ((0x1 << dataId) & usingDataIdBitmap)
                    {
                        r = pSyncProtocol->GetData(stationId, dataId, s_ControllerAccessor[i].GetDataPtr(dataId));
                        PIA_SAMPLE_ASSERT(r.IsSuccess());

                        PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSyncProtocol->GetDataPtr(stationId, dataId)));
                        if (dataId == DataId_Key)
                        {
                            PIA_SAMPLE_ASSERT(*static_cast<const uint32_t*>(pSyncProtocol->GetDataPtr(stationId, dataId)) == *static_cast<uint32_t*>(s_ControllerAccessor[i].GetDataPtr(dataId)));
                        }
                    }
                }
            }
        }

#if PIA_TEST_CODE
        // 同期ずれ検知用データを使用して、同期ずれチェックを行います。
        if (pSyncProtocol->CanGetData())
        {
            CheckSyncData();
        }
#endif

        // 同期データをセットします
        SetSyncData();

        nn::pia::common::Time end;
        end.SetNow();
        s_DispatchTime = (end - start).GetMSec();

        // 同期データの送信処理をすぐに行うために、もう一度 nn::pia::common::Scheduler::Dispatch() を呼びます。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        Draw(gameMode);

#if PIA_TEST_CODE
        UpdatePiaTestData(gameMode, isExit);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(0x01, isExit ? 1 : 0);
#endif
        s_DispatchCount = 0;

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
    }
}


}  // end of anonymous namespace


// エントリポイント。
PIA_SAMPLE_MAIN_FUNC()
{
    nn::pia::Result result;

    {
        nns::pia::sampleframework::SampleFramework::Setting setting;
        setting.isUpperScreenOnly = true;
        setting.isRawTouchpad = true;
        nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
    }
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(PROGRAM_TITLE "\n");
    nns::pia::Basic_SetProgramTitle(PROGRAM_TITLE);

    // SampleFramework::CreateInstance() でコントローラ関連の初期化が行われるので、
    // それより後でControllerReaderのインスタンスを作ります。
    ControllerReader controllerReader;
    s_pControllerReader = &controllerReader;

    while (true)
    {
        uint32_t timeoutFrame = 0;

        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(FramePerSeconds);

        ClearStationIdList();

        GameMode gameMode = DecideGameMode();
        if (gameMode == GameMode_Exit)
        {
            break;
        }

        // Piaの各モジュールの初期化処理
        // この順番で行ってください。
        {
#if PIA_TEST_CODE
            uint64_t traceFlags =
                nn::pia::common::TraceFlagSync |
                nn::pia::common::TraceFlagLocal |
                nn::pia::common::TraceFlagInet |
                nn::pia::common::TraceFlagLan |
                nn::pia::common::TraceFlagTransport |
                nn::pia::common::TraceFlagSession |
                nn::pia::common::TraceFlagSessionConnect |
                nn::pia::common::TraceFlagSessionLeave |
                nn::pia::common::TraceFlagSessionUpdate;
#else
            uint64_t traceFlags = 0;
#endif
            result = nns::pia::Common_InitializePiaCommon(s_Memory, MemoryBlockSize, traceFlags, true);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            nn::pia::transport::NetworkFactory* pFactory = NULL;
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            {
#if NN_PIA_CTR
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(LocalTitleId, LocalSubId));
#elif NN_PIA_NINTENDOSDK
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(nns::pia::LocalCommunicationId, LocalSceneId));
#endif
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Local_GetNetworkFactory();
                timeoutFrame = 100;
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                // PiaInet を初期化するより前に NEX を初期化してください
                result = nns::pia::Nex_InitializeNex(
                    MemoryBlockSizeForNex,
                    MemoryBlockSizeForNexEmergency);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                result = nns::pia::Inet_InitializePiaInet();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Inet_GetNetworkFactory();
                timeoutFrame = 240;
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                result = nns::pia::Lan_InitializePiaLan();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Lan_GetNetworkFactory();
                timeoutFrame = 240;
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            // Factoryを利用して、Transportをセットアップします。
            // ローカル通信では nn::pia::local::UdsNetworkFactory を、インターネット通信では nn::pia::inet::NexNetworkFactory を使用します。
            {
                nn::pia::transport::Transport::Setting setting;
                setting.pFactory = pFactory;
                setting.maxStationNum = MaxStations;
                setting.maxSendThreadBufferNum = nns::pia::MaxSendThreadBufferNum;
                setting.maxReceiveThreadBufferNum = nns::pia::MaxReceiveThreadBufferNum;

#if PIA_TEST_CODE
                nn::pia::transport::Transport::DebugSetting debugSetting;
                debugSetting.sendThreadLatencyEmulationPacketNum = 128;
                debugSetting.receiveThreadLatencyEmulationPacketNum = 512;
                debugSetting.isPacketLossEmulationEnabled = true;
                result = nn::pia::transport::Transport::SetDebugSetting(debugSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#endif

                result = nns::pia::Transport_InitializePiaTransport(setting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }

            // syncのセットアップを行います。
            {
                nn::pia::sync::SyncProtocol::Setting setting;
                setting.maxDelay = DelayMax;
                setting.timeoutFrame = timeoutFrame;
                setting.dataUnitSize[DataId_Key] = sizeof(KeyData);
                setting.dataUnitSize[DataId_Touch] = sizeof(TouchData);
                setting.dataUnitSize[DataId_Stick] = sizeof(StickData);
#if PIA_TEST_CODE
                setting.dataUnitSize[DataId_SyncCheck] = sizeof(SyncCheckData);
#endif
                // 同期通信中の遅延変更機能を使用するかを設定します。
                // 使用する場合は、毎フレーム SyncProtocol::RequestToChangeDelay を呼ぶ必要があります。
                // この機能を使用する場合の注意点は、Pia プログラミングマニュアル PiaSync 編 3. PiaSync 応用機能 を参照してください。
                setting.isChangeDelayEnabled = IsChangeDelayEnabled;

#if PIA_TEST_CODE
                setting.dataCompressionLevel = nn::pia::sync::SyncProtocol::CompressionLevelNone;
#else
                setting.dataCompressionLevel = nn::pia::sync::SyncProtocol::CompressionLevelMiddle;
#endif

                nns::pia::Sync_InitializePiaSync(setting);
            }
            {
                nn::pia::session::Session::Setting sessionSetting;
                sessionSetting.pNetworkFactory = pFactory;
                sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
                sessionSetting.browsedSessionInfoListNum = 8;
                // sessionのセットアップを行います。
                result = nns::pia::Session_InitializePiaSession(sessionSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                result = nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionCallback);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
        }


        // ネットワークの接続、セッションの確立の処理
        {
            // ネットワークの初期化
            nns::pia::NetworkType networkType = nns::pia::NetworkType_Internet;
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            {
                networkType = nns::pia::NetworkType_Local;
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                networkType = nns::pia::NetworkType_Internet;
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                networkType = nns::pia::NetworkType_Lan;
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(networkType);
            if (netResult != nns::pia::NetResult_Success)
            {
                PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork:failed. (netResult = %d)\n", netResult);

                switch (gameMode)
                {
#if USE_PIA_LOCAL
                case GameMode_LocalHost:
                case GameMode_LocalClient:
                {
                    gameMode = GameMode_LocalError;
                }
                break;
#endif
#if NNS_PIA_ENABLE_INET
                case GameMode_InetRandom:
                {
                    gameMode = GameMode_InetError;
                }
                break;
#endif
                case GameMode_LanRandom:
                {
                    gameMode = GameMode_LanError;
                }
                break;
                default:
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                break;
                }
            }

            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

// セッションを構築します。
// 作成するセッションの設定
#if NN_PIA_CTR
                nn::pia::local::UdsCreateSessionSetting createSessionSetting;
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnCreateSessionSetting createSessionSetting;
#endif
                {
#if NN_PIA_CTR
                    createSessionSetting.SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    createSessionSetting.SetSubId(LocalSubId);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
#elif NN_PIA_NINTENDOSDK
                    createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    createSessionSetting.SetSceneId(LocalSceneId);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
#endif
                    createSessionSetting.SetMaxParticipantNum(MaxStations);
                }

                result = nns::pia::Session_CreateSession(&createSessionSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_CreateSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LocalError;
                    break;
                }
            }
            break;

            case GameMode_LocalClient:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

                // セッションを検索します。
                // 検索条件の設定
                const uint8_t SearchSettingNum = 1;
#if NN_PIA_CTR
                nn::pia::local::UdsSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    searchCriteriaList[0].SetSubId(LocalSubId);
                    searchCriteriaList[0].SetMaxParticipantNum(MaxStations);
                    searchCriteriaList[0].SetResultRange(0, 8);
                }
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    searchCriteriaList[0].SetSceneId(LocalSceneId);
                    searchCriteriaList[0].SetMaxParticipantNum(MaxStations);
                    searchCriteriaList[0].SetResultRange(0, 8);
                }
#endif
                result = nns::pia::Session_BrowseSession(&searchCriteriaList[0]);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_BrowseSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LocalError;
                    break;
                }

                // 検索したセッションに参加します。
                uint32_t foundSessionNum = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize();
                if (foundSessionNum > 0)
                {
                    PIA_SAMPLE_PRINT("session found\n");
                    nn::pia::session::ISessionInfoList::Iterator it = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->Begin();
                    nn::pia::session::ISessionInfo* pTargetSessionInfo = *it;
                    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTargetSessionInfo));

#if NN_PIA_CTR
                    nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#elif NN_PIA_NINTENDOSDK
                    nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
#endif
                    joinSessionSetting.SetSessionInfoPtr(pTargetSessionInfo);
                    joinSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    result = nns::pia::Session_JoinSession(&joinSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("nns::pia::Session_JoinSession:failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                        gameMode = GameMode_LocalError;
                        break;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("no session found\n");
                    gameMode = GameMode_LocalError;
                    break;
                }
            }
            break;
            case GameMode_LocalError:
            {
            }
            break;
#endif  // USE_PIA_LOCAL
#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                // ゲームサーバにログイン
                if (!nns::pia::Inet_Login())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Inet_Login:failed\n");
                    gameMode = GameMode_InetError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::inet::NexSessionStartupSetting startupSetting;
                startupSetting.isHostMigrationEnabled = IsHostMigrationEnabled;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_InetError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::inet::NexCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const uint32_t SearchSettingNum = 1;
                nn::pia::inet::NexSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::inet::NexJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_InetError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = GameMode_InetHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = GameMode_InetClient;
                }
            }
            break;
            case GameMode_InetError:
            {
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                if (nns::pia::Lan_StartupPiaLan().IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Lan_StartupLan: failed\n");
                    gameMode = GameMode_LanError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::lan::LanSessionStartupSetting startupSetting;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);
                startupSetting.maxSilenceTime = 15000;
                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LanError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::lan::LanCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const uint32_t SearchSettingNum = 1;
                nn::pia::lan::LanSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::lan::LanJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LanError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = GameMode_LanHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = GameMode_LanClient;
                }
            }
            break;
            case GameMode_LanError:
            {
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }
        }

        s_MyUsingDataIdBitmap = s_UsingDataIdBitmapAll;
        s_Delay = 8;
        s_SendPeriod = FramePerSeconds / 30;
        s_CursorConfigId = 0;
        s_ChangeDelay = 0;
        s_ChangeDelayInput = 0;
        s_ChangeSendPeriod = 1;

        // 送信する同期データを1パケットに収めるための、各データID のデータサイズの合計の最大値を出力
        {
            uint32_t size = nns::pia::Sync_GetSyncProtocol()->CalcMaxSyncDataSizeSentInOnePacket(s_Delay, DataId_Max);
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("MaxSyncDataSizeSentInOnePacket = %d\n", size);
        }

        bool isLeaveNeeded = false;

        // メインの処理
        switch (gameMode)
        {
#if USE_PIA_LOCAL
        case GameMode_LocalHost:
        case GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
        case GameMode_InetHost:
        case GameMode_InetClient:
#endif
        case GameMode_LanHost:
        case GameMode_LanClient:
        {
            MainLoop(gameMode, &isLeaveNeeded);

            // Watermark機能による計測結果プリント。
            if (PIA_IS_VALID_POINTER(nn::pia::common::WatermarkManager::GetInstance()))
            {
                nn::pia::common::WatermarkManager::GetInstance()->Trace(nn::pia::common::TraceFlagAlways);
            }
        }
        break;
        default:
        {
        }
        break;
        }

        // セッションの終了、ネットワークからの切断の処理
        {
            // Session 終了処理。
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            {
                if (isLeaveNeeded)
                {
                    // セッションから離脱
                    // 本サンプルでは、この後終了処理を行うため、返り値は無視しています。
                    nns::pia::Session_LeaveSession();
                }
            }
            break;
#if USE_PIA_LOCAL
            case GameMode_LocalError:
            {
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetError:
            {
            }
            break;
#endif
            case GameMode_LanError:
            {
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            // Session クラスのクリーンアップ処理。
            nns::pia::Session_CleanupSession();

            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            case GameMode_LocalError:
            {
                nns::pia::Local_FinalizeLocalCommunication();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
            case GameMode_InetError:
            {
                nns::pia::Inet_Logout();
            }
            break;
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            case GameMode_LanError:
            {
                nns::pia::Lan_CleanupPiaLan();
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            nns::pia::Net_FinalizeNetwork();
        }

#if PIA_TEST_CODE
        UpdatePiaTestData(gameMode, true);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(0x01, 0);
#endif

        // Piaの各モジュールの終了処理
        // 初期化と逆順で行ってください。
        {
            nns::pia::Session_FinalizePiaSession();
            nns::pia::Sync_FinalizePiaSync();
            nns::pia::Transport_FinalizePiaTransport();
            switch (gameMode)
            {
#if USE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            case GameMode_LocalError:
            {
                nns::pia::Local_FinalizePiaLocal();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
            case GameMode_InetError:
            {
                nns::pia::Inet_FinalizePiaInet();
                nns::pia::Nex_FinalizeNex();
            }
            break;
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            case GameMode_LanError:
            {
                nns::pia::Lan_FinalizePiaLan();
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }
            nns::pia::Common_FinalizePiaCommon();
        }
    }

    nns::pia::sampleframework::SampleFramework::DestroyInstance();

    PIA_SAMPLE_PRINT("end.\n");
}
