﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"

#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Common_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Demo.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Local_Setup.h"

#include <cstdio>
#include <cstdlib>
#include <cstring>


#ifndef ENABLE_PROFILING
#define ENABLE_PROFILING 0
#endif

#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

#define PROGRAM_TITLE "ReliableAndUnreliable"

#define ENABLE_PRINT_CALLBACK 0        // ユーザー独自のプリントコールバックを使用したい場合は 1
#define TRANSPORT_ANALYSIS_INTERVAL 5  // トランスポート分析結果のプリントを何秒おきに実行するか

namespace
{
const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 30000;  // この値は、Pia サンプルプログラム毎に違えてください。
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;

const int MaxStations = 4;  // 最大 Station 数

#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
const uint32_t LocalTitleId = 0xFF300;  // for transport sample
const uint8_t LocalSubId = MatchmakeGameModeOffset;
#elif NN_PIA_NINTENDOSDK
const uint16_t LocalSceneId = MatchmakeGameModeOffset + 4000 + 30000 * PIA_TEST_CODE;     //!< Scene ID です。
#endif
const int16_t LocalChannel = 0;

const char LocalWirelessCryptoKey[] = "aaabbccc0123456789";
const char SignatureKey[] = "ReliableAndUnreliable_key";
#endif

#if NNS_PIA_ENABLE_INET
// アプリケーションの仕様に応じて調整して下さい。
const uint32_t MemoryBlockSizeForNex = 1 * 1024 * 1024;
const uint32_t MemoryBlockSizeForNexEmergency = 50 * 1024;
#endif

const uint32_t LargeBufferSize = 1 * 1024 * 1024;  // ReliableLarge で大きめのデータを送受信する際の最大サイズです。
static uint32_t s_LargeBuffer[LargeBufferSize / sizeof(uint32_t)];
const uint32_t LargeDataOffSetMax = 1 * 1024 * 1024;
static uint32_t s_BasicLargeData[(LargeDataOffSetMax + LargeBufferSize) / sizeof(uint32_t)];

const uint32_t MemoryBlockSize = 10 * 1024 * 1024;                // nn::pia::common::Initialize()で供給するメモリサイズ（単位はバイト）
PIA_ATTRIBUTE_ALIGN(8) static uint8_t s_Memory[MemoryBlockSize];  // nn::pia::common::Initialize()で供給するメモリ


const uint16_t UnreliableProtocolPort = 0;            // UnreliableProtocol が使用するポート番号。
const uint16_t ReliableProtocolPort = 0;              // ReliableProtocol が使用するポート番号。
const uint16_t ReliableProtocolPortForLargeData = 1;  // 大きいデータ送信用の ReliableProtocol が使用するポート番号。

static uint32_t s_hUnreliable = 0;
static uint32_t s_hReliable = 0;
static uint32_t s_hReliableLarge = 0;

#if PIA_TEST_CODE
struct nns::pia::OptionalParameter s_OptionalParam;
const int32_t OptionalParamOffset = 200;
#endif

nn::pia::StationId ConvertToStationId(nn::pia::StationIndex idx)
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    if (!PIA_IS_VALID_POINTER(pTransport))
    {
        PIA_SAMPLE_ASSERTMSG(false, "ConvertToStationId(): Transport instance is not ready?");
        return nn::pia::StationIdInvalid;
    }

    nn::pia::StationId id;
    nn::pia::Result ret = pTransport->ConvertToStationId(&id, idx);
    if (ret.IsFailure())
    {
        if (ret == nn::pia::ResultNotFound())
        {
            return nn::pia::StationIdInvalid;
        }
        else
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Result = 0x%08x\n", ret.GetValue());
            PIA_SAMPLE_ASSERTMSG(false, "ConvertToStationId(): ConvertToStationId() failed.");
            return nn::pia::StationIdInvalid;
        }
    }

    return id;
}


nn::pia::StationIndex ConvertToStationIndex(nn::pia::StationId id)
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    if (!PIA_IS_VALID_POINTER(pTransport))
    {
        PIA_SAMPLE_ASSERTMSG(false, "ConvertToStationIndex: Transport instance is not ready?");
        return nn::pia::StationIndex_Invalid;
    }

    nn::pia::StationIndex idx;
    nn::pia::Result ret = pTransport->ConvertToStationIndex(&idx, id);
    if (ret.IsFailure())
    {
        if (ret == nn::pia::ResultNotFound())
        {
            return nn::pia::StationIndex_Invalid;
        }
        else
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Result = 0x%08x\n", ret.GetValue());
            PIA_SAMPLE_ASSERTMSG(false, "ConvertToStationIndex: ConvertToStationIndex() failed.");
            return nn::pia::StationIndex_Invalid;
        }
    }

    return idx;
}


// プロトコル群の作成と初期化を行います。
nn::pia::Result CreateAndInitializeProtocols()
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // UnreliableProtocol（データ到達保証のない通信）の作成と初期化。
    {
        PIA_SAMPLE_ASSERT(s_hUnreliable == 0);
        s_hUnreliable = pTransport->CreateProtocol<nn::pia::transport::UnreliableProtocol>(UnreliableProtocolPort);
        PIA_SAMPLE_ASSERT(s_hUnreliable != 0);  // CreateProtocol()は、既に同じプロトコルIDが存在していた場合は0を返すので、そうではないことを確認
        pTransport->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_hUnreliable)->Initialize();
    }

    // ReliableProtocol（データ到達保証のある通信）の作成と初期化。
    {
        PIA_SAMPLE_ASSERT(s_hReliable == 0);
        s_hReliable = pTransport->CreateProtocol<nn::pia::transport::ReliableProtocol>(ReliableProtocolPort);
        PIA_SAMPLE_ASSERT(s_hReliable != 0);  // CreateProtocol()は、既に同じプロトコルIDが存在していた場合は0を返すので、そうではないことを確認
        const uint32_t SendBufNum = 64;
        const uint32_t RecvBufNum = 64;
        pTransport->GetProtocol<nn::pia::transport::ReliableProtocol>(s_hReliable)->Initialize(SendBufNum, RecvBufNum);
    }

    // ReliableProtocolLarge（データ到達保証のある通信）の作成と初期化。
    {
        PIA_SAMPLE_ASSERT(s_hReliableLarge == 0);
        s_hReliableLarge = pTransport->CreateProtocol<nn::pia::transport::ReliableProtocol>(ReliableProtocolPortForLargeData);
        PIA_SAMPLE_ASSERT(s_hReliableLarge != 0);  // CreateProtocol()は、既に同じプロトコルIDが存在していた場合は0を返すので、そうではないことを確認
        const uint32_t SendBufNum = 1000;
        const uint32_t RecvBufNum = 1000;
        pTransport->GetProtocol<nn::pia::transport::ReliableProtocol>(s_hReliableLarge)->Initialize(SendBufNum, RecvBufNum);
    }

    return nn::pia::ResultSuccess();
}


// プロトコル群の後始末をします。
void FinalizeAndDestroyProtocols()
{
    // Transportシングルトン取得。
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // ReliableProtocolLargeの終了処理
    nn::pia::transport::ReliableProtocol* pReliableLarge = pTransport->GetProtocol<nn::pia::transport::ReliableProtocol>(s_hReliableLarge);
    if (PIA_IS_VALID_POINTER(pReliableLarge))
    {
        pReliableLarge->Finalize();
        pTransport->DestroyProtocol(s_hReliableLarge);
        s_hReliableLarge = 0;
    }

    // ReliableProtocolの終了処理
    nn::pia::transport::ReliableProtocol* pReliable = pTransport->GetProtocol<nn::pia::transport::ReliableProtocol>(s_hReliable);
    if (PIA_IS_VALID_POINTER(pReliable))
    {
        pReliable->Finalize();
        pTransport->DestroyProtocol(s_hReliable);
        s_hReliable = 0;
    }

    // UnreliableProtocolの終了処理
    nn::pia::transport::UnreliableProtocol* pUnreliable = pTransport->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_hUnreliable);
    if (PIA_IS_VALID_POINTER(pUnreliable))
    {
        pUnreliable->Finalize();
        pTransport->DestroyProtocol(s_hUnreliable);
        s_hUnreliable = 0;
    }
}


nn::pia::transport::UnreliableProtocol* GetUnreliableProtocol()
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(nn::pia::transport::Transport::GetInstance()));
    return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_hUnreliable);
}


nn::pia::transport::ReliableProtocol* GetReliableProtocol()
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(nn::pia::transport::Transport::GetInstance()));
    return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::ReliableProtocol>(s_hReliable);
}


nn::pia::transport::ReliableProtocol* GetReliableProtocolLarge()
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(nn::pia::transport::Transport::GetInstance()));
    return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::ReliableProtocol>(s_hReliableLarge);
}


// 大きいデータの元データを作成します。
void InitBasicLargeData()
{
    std::srand(0x12345678);
    uint32_t size = sizeof(s_BasicLargeData) / sizeof(uint32_t);
    for (uint32_t i = 0; i < size; ++i)
    {
        s_BasicLargeData[i] = static_cast<uint32_t>(std::rand());
    }
}


// 大きいデータを作成します。内容に特に意味はありません。
uint32_t CreateLargeData(uint32_t key)
{
    // 最初の4バイトがkeyそのまま。それをもとにサイズと残りのデータを乱数で決めます。

    std::srand(key);

    uint32_t size;

#if PIA_TEST_CODE
    size = 128 * 1024;
    PIA_SAMPLE_ASSERT(size < LargeBufferSize);
#else
#if ENABLE_PROFILING
    if (true)
#else
    if (std::rand() & 0x1)
#endif
    {
        size = LargeBufferSize;
    }
    else
    {
        size = (std::rand() * sizeof(uint32_t)) % LargeBufferSize;
        if (size <= 0)
        {
            size = LargeBufferSize;
        }
    }
#endif

    uintptr_t offset = (std::rand() * sizeof(uint32_t)) % LargeDataOffSetMax;
    const void* cpData = reinterpret_cast<void*>(reinterpret_cast<uintptr_t>(s_BasicLargeData) + offset);
    std::memcpy(s_LargeBuffer, cpData, size);
    s_LargeBuffer[0] = key;

    return size;
}

// 大きいデータの内容をチェックします。
bool CheckLargeData(uint32_t dataSize)
{
    if (dataSize <= 0)
    {
        return false;
    }
    if (dataSize % 4 != 0)
    {
        return false;
    }

    // 最初の4バイトをkeyとしてそこから乱数で求められたサイズと残りのデータをチェックします。

    uint32_t key = s_LargeBuffer[0];
    std::srand(key);

    uint32_t size;
#if PIA_TEST_CODE
    size = 128 * 1024;
    PIA_SAMPLE_ASSERT(size < LargeBufferSize);
#else
#if ENABLE_PROFILING
    if (true)
#else
    if (std::rand() & 0x1)
#endif
    {
        size = LargeBufferSize;
    }
    else
    {
        size = (std::rand() * sizeof(uint32_t)) % LargeBufferSize;
        if (size <= 0)
        {
            size = LargeBufferSize;
        }
    }
#endif

    if (size != dataSize)
    {
        return false;
    }

    uintptr_t offset = (std::rand() * sizeof(uint32_t)) % LargeDataOffSetMax;
    const void* cpData = reinterpret_cast<void*>(reinterpret_cast<uintptr_t>(s_BasicLargeData) + offset + sizeof(uint32_t));
    return (std::memcmp(&(s_LargeBuffer[1]), cpData, size - sizeof(uint32_t)) == 0);
}


// Protocol を区別する列挙体です。
enum ProtocolType
{
    ProtocolType_Unreliable,
    ProtocolType_Reliable,
    ProtocolType_ReliableLarge,
    ProtocolType_Max
};

static nn::pia::common::Time s_CurrentTime;

// 送受信の状態を管理するデータです。
class StationData
{
public:
    bool m_bAllStation;              // 宛先が全ステーションであるとき true、一つのステーションであるときは false とします。
    nn::pia::StationId m_StationId;  // m_bAllStation が true の場合、この値は意味を持ちません。
    struct ProtocolData
    {
        bool isSendRepeat;
        bool isSendOnce;
        uint32_t sendNum;
        uint32_t receiveNum;
        nn::pia::common::Time lastReceivedTime;
    };
    ProtocolData m_ProtocolData[ProtocolType_Max];

    void Clear()
    {
        for (int i = 0; i < ProtocolType_Max; ++i)
        {
            m_ProtocolData[i].isSendRepeat = false;
            m_ProtocolData[i].isSendOnce = false;
            m_ProtocolData[i].sendNum = 0;
            m_ProtocolData[i].receiveNum = 0;
            m_ProtocolData[i].lastReceivedTime = s_CurrentTime;
        }
    }
};

const int StationDataMax = MaxStations + 1;
const int StationAllIndex = MaxStations;
static StationData s_StationData[StationDataMax];
static int s_TargetStationIdx;
static int s_TargetProtocolType;
static bool s_IsLogEnable;

#if ENABLE_PROFILING
static bool s_IsCountReceive = false;
static uint32_t s_ReceivedSize = 0;
static uint32_t s_ReceivedTime = 0;
#endif  // end of ENABLE_PROFILING


// 送受信の状態のデータを初期化します。
void InitializeStationData()
{
    for (int i = 0; i < StationDataMax; ++i)
    {
        s_StationData[i].m_bAllStation = false;
        s_StationData[i].m_StationId = nn::pia::StationIdInvalid;
        s_StationData[i].Clear();
    }
    s_StationData[StationAllIndex].m_bAllStation = true;
    s_StationData[StationAllIndex].m_StationId = nn::pia::StationIdInvalid;  // m_StationId が不定値となるのを避けるためだけに値をセットしています。
    s_StationData[StationAllIndex].Clear();

    s_TargetProtocolType = 0;
    s_TargetStationIdx = StationAllIndex;
    s_IsLogEnable = true;
}


// 接続、切断時の処理です。
void SessionEventCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam)
{
    NN_PIA_UNUSED(pApplicationParam);
    nn::pia::transport::TransportAnalyzer* pAnalyzer = nn::pia::transport::TransportAnalyzer::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pAnalyzer));

    nn::pia::Result r;
    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
        s_StationData[ConvertToStationIndex(stationId)].Clear();
        s_StationData[ConvertToStationIndex(stationId)].m_StationId = stationId;
        r = pAnalyzer->SetMarker("JoinEvent");
        PIA_SAMPLE_ASSERT(r.IsSuccess());
        break;

    case nn::pia::session::Session::EventType_EventLeave:
        if (stationId == s_StationData[s_TargetStationIdx].m_StationId)
        {
            // 現在カーソルのある送信先が切断されたので、Allのところにカーソルを動かします。
            s_TargetStationIdx = StationAllIndex;
        }
        r = pAnalyzer->SetMarker("LeaveEvent");
        PIA_SAMPLE_ASSERT(r.IsSuccess());
        break;
    case nn::pia::session::Session::EventType_SessionHostChanged:
        // セッションのセッションホスト権限が移譲された
        if (stationId == s_StationData[s_TargetStationIdx].m_StationId)
        {
            // 現在カーソルのある送信先が切断されたので、Allのところにカーソルを動かします。
            s_TargetStationIdx = StationAllIndex;
        }
        r = pAnalyzer->SetMarker("HostChangedEvent");
        PIA_SAMPLE_ASSERT(r.IsSuccess());
        break;
    case nn::pia::session::Session::EventType_HostMigrationFailed:
        // ホストマイグレーションに失敗
        r = pAnalyzer->SetMarker("HostMigrationFailedEvent");
        PIA_SAMPLE_ASSERT(r.IsSuccess());
        break;
    default:
        PIA_SAMPLE_ASSERTMSG(false, "Unsupported event(%d).", eventType);
        break;
    }
}


// キー入力による操作です。
bool Operate()
{
    bool isExit = false;

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();

    // 上下で操作対象の送信先を切り替えます。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp) || pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
    {
        int mlt = pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp) ? -1 : 1;
        nn::pia::StationId localStationId = pSession->GetLocalStationId();
        for (int i = 1; i < StationDataMax; i++)
        {
            int idx = (s_TargetStationIdx + i * mlt + StationDataMax) % StationDataMax;
            nn::pia::StationId stationId = s_StationData[idx].m_StationId;
            if ((stationId != localStationId && pSession->IsValidStationId(stationId)) || s_StationData[idx].m_bAllStation)
            {
                s_TargetStationIdx = idx;
                break;
            }
        }
    }

    // 左右で操作対象のProtocolを切り替えます。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
    {
        --s_TargetProtocolType;
        if (s_TargetProtocolType < 0)
        {
            s_TargetProtocolType = ProtocolType_Max - 1;
        }
    }
    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
    {
        ++s_TargetProtocolType;
        if (s_TargetProtocolType >= ProtocolType_Max)
        {
            s_TargetProtocolType = 0;
        }
    }

    // Aで1回送信のフラグをセットします。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
    {
        s_StationData[s_TargetStationIdx].m_ProtocolData[s_TargetProtocolType].isSendOnce = true;
    }

    // Xで連続送信のフラグを ON/OFF します。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
    {
        s_StationData[s_TargetStationIdx].m_ProtocolData[s_TargetProtocolType].isSendRepeat ^= true;
    }

    // Yで送受信時のログ出力を ON/OFF します。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
    {
        s_IsLogEnable ^= true;
    }

    // START で GameLoop() を抜けます。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
    {
        isExit = true;
    }

#if ENABLE_PROFILING
    // L でスループット計測を切り替えます。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
    {
        if (s_IsCountReceive)
        {
            s_IsCountReceive = false;
        }
        else
        {
            s_IsCountReceive = true;
            s_ReceivedSize = 0;
            s_ReceivedTime = 0;
        }
    }
#endif

    return isExit;
}


// UnreliableProtocol の送信処理です。
void SendUnreliable(nn::pia::transport::UnreliableProtocol* pUnreliable)
{
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();

    for (int i = 0; i < StationDataMax; ++i)
    {
        nn::pia::StationId stationId = s_StationData[i].m_StationId;
        if (stationId == pSession->GetLocalStationId())
        {
            continue;
        }

        StationData::ProtocolData* pProtocolData = &s_StationData[i].m_ProtocolData[ProtocolType_Unreliable];
        if (pProtocolData->isSendOnce || pProtocolData->isSendRepeat)
        {
            nn::pia::Result r;
            if (s_StationData[i].m_bAllStation)
            {
                // 接続している全員に送信します。
                // UnreliableProtocol で送信できる最大サイズのデータを送ります。
                const uint32_t BufferSize = nn::pia::transport::UnreliableProtocol::MaxDataSize;
                char buffer[BufferSize];
                memset(buffer, 0xff, BufferSize);
                int size = PIA_SNPRINTF(buffer, BufferSize, "Unreliable Broadcast %llu => All (No.%u)", pSession->GetLocalStationId(), pProtocolData->sendNum);
                uint32_t limit = pUnreliable->GetDataSizeLimit();
                PIA_SAMPLE_ASSERT(size < static_cast<int>(limit));
                NN_PIA_UNUSED_IN_RELEASE(size);

                // 実際の送信処理
                r = pUnreliable->SendToAll(reinterpret_cast<uint8_t*>(buffer), limit);
            }
            else
            {
                // 特定の1台に対して送信します。
                // ネットワークにある程度の負荷をかける目的で、用意したバッファサイズ分のデータを送信します。
                if (pSession->IsValidStationId(stationId) == false)
                {
                    continue;
                }

                const uint32_t BufferSize = 512;
                char buffer[BufferSize];
                memset(buffer, 0x00, BufferSize);
                int size = PIA_SNPRINTF(buffer, BufferSize, "Unreliable Unicast %llu => %llu (No.%u)", pSession->GetLocalStationId(), stationId, pProtocolData->sendNum);
                NN_PIA_UNUSED_IN_RELEASE(size);
                PIA_SAMPLE_ASSERT(size < BufferSize);

                // 実際の送信処理
                r = pUnreliable->Send(stationId, reinterpret_cast<uint8_t*>(buffer), BufferSize);
            }

            if (r.IsSuccess())
            {
                // 送信成功したので送信数をカウントします。
                ++(pProtocolData->sendNum);
                pProtocolData->isSendOnce = false;
            }
            else
            {
                // その他のエラーはプログラムの修正が必要です。
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
            }
        }
    }
}


// ReliableProtocol の送信処理です。
void SendReliable(nn::pia::transport::ReliableProtocol* pReliable)
{
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();

    for (int i = 0; i < StationDataMax; ++i)
    {
        nn::pia::StationId stationId = s_StationData[i].m_StationId;
        if (stationId == pSession->GetLocalStationId())
        {
            continue;
        }

        // 個人宛ての送信は、送信先と通信中であることを確認してから送信してください。
        if (!s_StationData[i].m_bAllStation && pReliable->IsInCommunication(stationId) == false)
        {
            continue;
        }

        StationData::ProtocolData* pProtocolData = &s_StationData[i].m_ProtocolData[ProtocolType_Reliable];
        if (pProtocolData->isSendOnce || pProtocolData->isSendRepeat)
        {
            static const uint32_t BufferSize = 128;
            char buffer[BufferSize];
            int size = PIA_SNPRINTF(buffer, BufferSize, "Reliable %llu => %llu (No.%u)", pSession->GetLocalStationId(), stationId, pProtocolData->sendNum);
            PIA_SAMPLE_ASSERT(size < BufferSize);

            // 実際の送信処理
            nn::pia::Result r;
            if (s_StationData[i].m_bAllStation)
            {
                r = pReliable->SendToAll(buffer, size + 1);
            }
            else
            {
                r = pReliable->Send(stationId, buffer, size + 1);
            }

            if (r.IsSuccess())
            {
                // 送信成功したので送信数をカウントします。
                ++(pProtocolData->sendNum);
                pProtocolData->isSendOnce = false;
            }
            else if (r == nn::pia::ResultBufferIsFull())
            {
                // 送信バッファがいっぱいです。
                // 次回再び送信するために、Once のフラグも立てたままにしておきます。
                if (s_IsLogEnable)
                {
                    PIA_SAMPLE_PRINT("ReliableProtocol BufferIsFull to:%llu No:%u\n", stationId, pProtocolData->sendNum);
                }
            }
            else if (r == nn::pia::ResultNotInCommunication())
            {
                // IsInCommunication() であらかじめチェックしておけばここには入りません。
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
            }
            else
            {
                // その他のエラーはプログラムの修正が必要です。
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
            }
        }
    }
}


// ReliableProtocol で大きいデータの送信処理です。送信データの作り方以外は SendReliable() と同じです。
void SendReliableLarge(nn::pia::transport::ReliableProtocol* pReliableLarge)
{
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();

    for (int i = 0; i < StationDataMax; ++i)
    {
        nn::pia::StationId stationId = s_StationData[i].m_StationId;
        if (stationId == pSession->GetLocalStationId())
        {
            continue;
        }

        // 個人宛ての送信は、送信先と通信中であることを確認してから送信してください。
        if (!s_StationData[i].m_bAllStation && pReliableLarge->IsInCommunication(stationId) == false)
        {
            continue;
        }

        StationData::ProtocolData* pProtocolData = &s_StationData[i].m_ProtocolData[ProtocolType_ReliableLarge];
        if (pProtocolData->isSendOnce || pProtocolData->isSendRepeat)
        {
            nn::pia::StationIndex targetIdx = ConvertToStationIndex(stationId);
            PIA_SAMPLE_ASSERT(targetIdx != nn::pia::StationIndex_Invalid || s_StationData[i].m_bAllStation);
            nn::pia::StationIndex localIdx = ConvertToStationIndex(pSession->GetLocalStationId());
            PIA_SAMPLE_ASSERT(localIdx != nn::pia::StationIndex_Invalid);

            // 個人宛ても全体宛ても同じインターフェースで送信します。
            uint32_t key = (static_cast<uint32_t>(targetIdx) << 24) | (static_cast<uint32_t>(localIdx) << 16) | (pProtocolData->sendNum & 0xffff);
            uint32_t size = CreateLargeData(key);
            PIA_SAMPLE_ASSERT(CheckLargeData(size));

            // 実際の送信処理
            nn::pia::Result r;
            if (s_StationData[i].m_bAllStation)
            {
                r = pReliableLarge->SendToAll(s_LargeBuffer, size);
            }
            else
            {
                r = pReliableLarge->Send(stationId, s_LargeBuffer, size);
            }

            if (r.IsSuccess())
            {
                // 送信成功したので送信数をカウントします。
                ++(pProtocolData->sendNum);
                pProtocolData->isSendOnce = false;
            }
            else if (r == nn::pia::ResultBufferIsFull())
            {
                // 送信バッファがいっぱいです。
                // 次回再び送信するために、Once のフラグも立てたままにしておきます。
                // 送信バッファに余裕ができるまでしばらく時間がかかるので、プリントしないでおきます。
            }
            else if (r == nn::pia::ResultNotInCommunication())
            {
                // IsInCommunication() であらかじめチェックしておけばここには入りません。
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
            }
            else
            {
                // その他のエラーはプログラムの修正が必要です。
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
            }
        }
    }
}


// UnreliableProtocol の受信処理です。
void ReceiveUnreliable(nn::pia::transport::UnreliableProtocol* pUnreliable)
{
    while (true)
    {
        const uint32_t BufferSize = nn::pia::transport::UnreliableProtocol::MaxDataSize;
        char buffer[BufferSize];
        nn::pia::StationId stationId;
        uint32_t receivedSize;

        // 実際の受信処理
        nn::pia::Result r = pUnreliable->Receive(&stationId, reinterpret_cast<uint8_t*>(buffer), &receivedSize, BufferSize);

        if (r.IsSuccess())
        {
            // 受信データが取得できたので、受信回数をカウントします。
            StationData::ProtocolData* pProtocolData = &s_StationData[ConvertToStationIndex(stationId)].m_ProtocolData[ProtocolType_Unreliable];
            if (s_IsLogEnable)
            {
                nn::pia::common::TimeSpan span = s_CurrentTime - pProtocolData->lastReceivedTime;
                NN_PIA_UNUSED_IN_RELEASE(span);
                PIA_SAMPLE_PRINT("Unreliable Received from %llu. Data(%d bytes; span %d ms): [%s]\n", stationId, receivedSize, span.GetMSec(), buffer);
            }
            ++(pProtocolData->receiveNum);
            pProtocolData->lastReceivedTime = s_CurrentTime;
        }
        else if (r == nn::pia::ResultNoData())
        {
            // 受信データがもうないので受信処理を終了します。
            break;
        }
        else
        {
            // その他のエラーはプログラムの修正が必要です。
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
        }
    }
}


// ReliableProtocol の受信処理です。
// 通信を継続できない状態になった場合は true を返します。
bool ReceiveReliable(nn::pia::transport::ReliableProtocol* pReliable)
{
    while (true)
    {
        const uint32_t BufferSize = 128;
        char buffer[BufferSize];
        nn::pia::StationId stationId;
        uint32_t receivedSize;

        // 実際の受信処理
        nn::pia::Result r = pReliable->Receive(&stationId, reinterpret_cast<uint8_t*>(buffer), &receivedSize, BufferSize);

        if (r.IsSuccess())
        {
            // 受信データが取得できたので、受信回数をカウントします。
            StationData::ProtocolData* pProtocolData = &s_StationData[ConvertToStationIndex(stationId)].m_ProtocolData[ProtocolType_Reliable];
            if (s_IsLogEnable)
            {
                nn::pia::common::TimeSpan span = s_CurrentTime - pProtocolData->lastReceivedTime;
                NN_PIA_UNUSED_IN_RELEASE(span);
                PIA_SAMPLE_PRINT("Reliable Received from %llu. Data(%d bytes; span %d ms): [%s]\n", stationId, receivedSize, span.GetMSec(), buffer);
            }
            ++(pProtocolData->receiveNum);
            pProtocolData->lastReceivedTime = s_CurrentTime;
        }
        else if (r == nn::pia::ResultNoData())
        {
            // 受信データがもうないので受信処理を終了します。
            break;
        }
        else if (r == nn::pia::ResultBrokenData())
        {
            // 想定しないサイズのデータが届いています。
            // この状態になると、この ReliableProtocol での受信を継続できないので、通信を終了します。
            return true;
        }
        else if (r == nn::pia::ResultNotInCommunication())
        {
            // セッションに参加中ならここには入りません。
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
        }
        else
        {
            // その他のエラーはプログラムの修正が必要です。
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
        }
    }

    return false;
}


// ReliableProtocol で大きいデータのの受信処理です。
// 通信を継続できない状態になった場合は true を返します。
// 受信データの扱いが違う以外は ReceiveReliable() と同じです。
bool ReceiveReliableLarge(nn::pia::transport::ReliableProtocol* pReliableLarge)
{
    while (true)
    {
        nn::pia::StationId stationId;
        uint32_t receivedSize;

        // 実際の受信処理
        nn::pia::Result r = pReliableLarge->Receive(&stationId, reinterpret_cast<uint8_t*>(s_LargeBuffer), &receivedSize, LargeBufferSize);
        if (r.IsSuccess())
        {
            // 受信データが取得できたので、受信回数をカウントします。
            bool isValidData = CheckLargeData(static_cast<uint32_t>(receivedSize));
            StationData::ProtocolData* pProtocolData = &s_StationData[ConvertToStationIndex(stationId)].m_ProtocolData[ProtocolType_ReliableLarge];
            nn::pia::common::TimeSpan span = s_CurrentTime - pProtocolData->lastReceivedTime;
            NN_PIA_UNUSED_IN_RELEASE(span);
            if (s_IsLogEnable)
            {
                PIA_SAMPLE_PRINT("ReliableLarge Received from %llu. Data(%d bytes; span %dms): %s\n", stationId, receivedSize, span.GetMSec(), isValidData ? "OK" : "NG");
            }
            NN_PIA_UNUSED_IN_RELEASE(isValidData);

            // このサンプル同士で通信する限りはチェックに通らないデータは来ないはずなので、アサートにかけておきます。
            PIA_SAMPLE_ASSERTMSG(isValidData, "received data is broken");

            ++(pProtocolData->receiveNum);
            pProtocolData->lastReceivedTime = s_CurrentTime;

#if ENABLE_PROFILING
            if (s_IsCountReceive)
            {
                s_ReceivedSize += receivedSize;
                s_ReceivedTime += span.GetMSec();
            }
#endif
        }
        else if (r == nn::pia::ResultNoData())
        {
            // 受信データがもうないので受信処理を終了します。
            break;
        }
        else if (r == nn::pia::ResultBrokenData())
        {
            // 想定しないサイズのデータが届いています。
            // この状態になると、この ReliableProtocol での受信を継続できないので、通信を終了します。
            return true;
        }
        else if (r == nn::pia::ResultNotInCommunication())
        {
            // セッションに参加中ならここには入りません。
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
        }
        else
        {
            // その他のエラーはプログラムの修正が必要です。
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
        }
    }

    return false;
}


// 画面の描画処理です。
void Draw(nns::pia::GameMode gameMode)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();

    static const nns::pia::sampleframework::Color DefaultColor(1.0f, 1.0f, 1.0f, 1.0f);
    static const nns::pia::sampleframework::Color CursorColor(1.0f, 0.3f, 0.3f, 1.0f);
    static const nns::pia::sampleframework::Color InvalidColor(0.4f, 0.4f, 0.4f, 1.0f);

    pSampleFramework->SetFontColor(DefaultColor);
    {
        pSampleFramework->DrawString(10, 10, "PiaTransport ReliableAndUnreliable");
        const char* GameModeString = NULL;
        switch (gameMode)
        {
#if NNS_PIA_ENABLE_LOCAL
        case nns::pia::GameMode_LocalHost:
        case nns::pia::GameMode_LocalClient:
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                GameModeString = "LocalHost";
            }
            else
            {
                GameModeString = "LocalClient";
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
        case nns::pia::GameMode_InetHost:
        case nns::pia::GameMode_InetClient:
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                GameModeString = "InetHost";
            }
            else
            {
                GameModeString = "InetClient";
            }
            break;
        case nns::pia::GameMode_InetRandom:
            PIA_SAMPLE_ASSERTMSG(false, "InetRandom?");
            GameModeString = "InetRandom";
            break;
#endif
        case nns::pia::GameMode_LanHost:
        case nns::pia::GameMode_LanClient:
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                GameModeString = "LanHost";
            }
            else
            {
                GameModeString = "LanClient";
            }
            break;
        case nns::pia::GameMode_LanRandom:
            PIA_SAMPLE_ASSERTMSG(false, "LanRandom?");
            GameModeString = "LanRandom";
            break;
        default:
            PIA_SAMPLE_ASSERTMSG(false, "Unknown game mode? (%d)", gameMode);
            GameModeString = "???";
            break;
        }
        PIA_SAMPLE_ASSERT(GameModeString != NULL);
        pSampleFramework->DrawString(10, 20, "GameMode:%s", GameModeString);
        pSampleFramework->DrawString(180, 20, "StationNum:%d", pSession->GetStationNum());
        pSampleFramework->DrawString(320, 20, "Log:%s", s_IsLogEnable ? "ON" : "OFF");
        pSampleFramework->DrawString(10, 30, "A: send once  X: toggle to send");
        pSampleFramework->DrawString(10, 40, "Y: toggle to log  START:quit");
    }

    {
        uint32_t idx = (pSampleFramework->GetFrameCount() / 4) % 4;
        const char AnimLetter[] = { '-', '\\', '|', '/' };
        pSampleFramework->DrawString(380, 10, "%c", AnimLetter[idx]);
    }

    {
        pSampleFramework->DrawString(10, 60, "Idx");
        pSampleFramework->DrawString(50, 60, "Unreliable");
        pSampleFramework->DrawString(150, 60, "Reliable");
        pSampleFramework->DrawString(250, 60, "ReliableLarge");
        pSampleFramework->DrawString(360, 60, "RTT");
    }

    nn::pia::StationId localStationId = pSession->GetLocalStationId();
    nn::pia::StationId masterStationId = pSession->GetHostStationId();

    for (int i = 0; i < MaxStations; ++i)
    {
        int y1 = 70 + i * 22;
        int y2 = 80 + i * 22;
        nn::pia::StationId stationId = s_StationData[i].m_StationId;

        if (stationId == masterStationId)
        {
            pSampleFramework->DrawString(2, y1, "*");
        }
        pSampleFramework->DrawString(10, y1, "%02x", ConvertToStationIndex(stationId));

        bool isValid = pSession->IsValidStationId(stationId);
        if (isValid == false)
        {
            pSampleFramework->SetFontColor(InvalidColor);
            pSampleFramework->DrawString(30, y1, "disconnect");
            pSampleFramework->SetFontColor(DefaultColor);
            continue;
        }

        if (i == ConvertToStationIndex(localStationId))
        {
            pSampleFramework->SetFontColor(InvalidColor);
            pSampleFramework->DrawString(30, y1, "me");
            pSampleFramework->SetFontColor(DefaultColor);
            continue;
        }

        for (int j = 0; j < ProtocolType_Max; ++j)
        {
            int x = 70 + j * 100;
            if (i == s_TargetStationIdx && j == s_TargetProtocolType)
            {
                pSampleFramework->SetFontColor(CursorColor);
                pSampleFramework->DrawString(x - 10, y1, ">");
            }
            pSampleFramework->DrawString(x, y1, "%6d%c", s_StationData[i].m_ProtocolData[j].sendNum, s_StationData[i].m_ProtocolData[j].isSendRepeat ? '@' : ' ');
            pSampleFramework->DrawString(x, y2, "%6d", s_StationData[i].m_ProtocolData[j].receiveNum);
            if (i == s_TargetStationIdx && j == s_TargetProtocolType)
            {
                pSampleFramework->SetFontColor(DefaultColor);
            }
        }

        // RTT(Round Trip Time)表示
        {
            int x = 360;
            nn::pia::transport::StationManager* pMgr = nn::pia::transport::StationManager::GetInstance();
            if (PIA_IS_VALID_POINTER(pMgr))
            {
                nn::pia::transport::Station* pStation = pMgr->GetStation(ConvertToStationId(static_cast<nn::pia::StationIndex>(i)));
                if (PIA_IS_VALID_POINTER(pStation))
                {
                    pSampleFramework->DrawString(x, y1, "%dms", pStation->GetRtt());
                    pSampleFramework->DrawString(x, y1 + 10, "[%d]", pStation->GetRttSamplingNum());
                }
            }
        }
    }

    {
        int i = StationAllIndex;
        int y1 = 70 + i * 22;
        pSampleFramework->DrawString(10, y1, "All");

        for (int j = 0; j < ProtocolType_Max; ++j)
        {
            int x = 70 + j * 100;
            if (MaxStations == s_TargetStationIdx && j == s_TargetProtocolType)
            {
                pSampleFramework->SetFontColor(CursorColor);
                pSampleFramework->DrawString(x - 10, y1, ">");
            }
            pSampleFramework->DrawString(x, y1, "%6d%c", s_StationData[i].m_ProtocolData[j].sendNum, s_StationData[i].m_ProtocolData[j].isSendRepeat ? '@' : ' ');
            if (MaxStations == s_TargetStationIdx && j == s_TargetProtocolType)
            {
                pSampleFramework->SetFontColor(DefaultColor);
            }
        }
    }

    // 送受信に成功したパケット数を画面に描画します。
    {
        // 描画位置
        int x = 10;
        int y = 200;

        nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
        PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));

        pSampleFramework->DrawString(x, y, "Send packets: %5d", pMgr->GetSendThreadSendPacketNum());
        pSampleFramework->DrawString(x, y + 12, "Recv packets: %5d", pMgr->GetReceiveThreadReceivePacketNum());
    }

    // 送受信に成功したデータ量を画面に描画します。
    {
        // 描画位置
        int x = 180;
        int y = 200;

        nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
        PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));

        pSampleFramework->DrawString(x, y, "Send data(bytes): %8lld", pMgr->GetSendThreadSendDataSize());
        pSampleFramework->DrawString(x, y + 12, "Recv data(bytes): %8lld", pMgr->GetReceiveThreadReceiveDataSize());
    }
}


// 送受信パケットのカウンタをクリアします。
void ClearSendRecvPacketNum(void)
{
    nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));

    pMgr->ClearSendThreadSendPacketNum();
    pMgr->ClearReceiveThreadReceivePacketNum();
}


// 送受信データのカウンタをクリアします。
void ClearSendRecvDataSize(void)
{
    nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));

    pMgr->ClearSendThreadSendDataSize();
    pMgr->ClearReceiveThreadReceiveDataSize();
}


#if PIA_TEST_CODE
void UpdatePiaTestData(nns::pia::GameMode gameMode, bool isExit)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    PIA_ASSERT(PIA_IS_VALID_POINTER(pSession));

    uint32_t sceneNo = 0;
    if (!isExit)
    {
        switch (gameMode)
        {
#if NNS_PIA_ENABLE_INET
        case nns::pia::GameMode_InetHost:
        case nns::pia::GameMode_InetClient:
        case nns::pia::GameMode_InetRandom:
            sceneNo = 4;
            break;
#endif
#if NNS_PIA_ENABLE_LOCAL
        case nns::pia::GameMode_LocalHost:
        case nns::pia::GameMode_LocalClient:
            sceneNo = 3;
            break;
#endif
        case nns::pia::GameMode_LanHost:
        case nns::pia::GameMode_LanClient:
        case nns::pia::GameMode_LanRandom:
            sceneNo = 5;
            break;
        default:
            PIA_SAMPLE_ASSERTMSG(false, "Unknown gamemode? (%d)", gameMode);
            break;
        }
    }
    pSampleFramework->SetProgramStatus(
        sceneNo,
        pSession->GetStatus() == nn::pia::session::Session::Status_ConnectedSession,
        pSession->GetLocalStationId() == pSession->GetHostStationId(),
        pSession->GetStationNum());

    // Session
    pSampleFramework->SetProgramParam(0x00, ConvertToStationIndex(pSession->GetLocalStationId()));

    // 自分の設定
    pSampleFramework->SetProgramParam(0x08, s_TargetStationIdx);
    pSampleFramework->SetProgramParam(0x09, s_TargetProtocolType);

#if ENABLE_PROFILING
    // 計測
    pSampleFramework->SetProgramParam(0x0c, s_IsCountReceive ? 1 : 0);
    pSampleFramework->SetProgramParam(0x0d, s_ReceivedSize);
    pSampleFramework->SetProgramParam(0x0e, s_ReceivedTime);
#endif

    // 個々のステーション
    for (int i = 0; i < MaxStations; ++i)
    {
        nn::pia::StationId sid = ConvertToStationId(static_cast<nn::pia::StationIndex>(i));
        int sIdx = i;
        uint32_t idOfs = (i + 1) << 4;

        bool isValid = pSession->IsValidStationId(sid);
        pSampleFramework->SetProgramParam(idOfs + 0x0, isValid ? 1 : 0);

        pSampleFramework->SetProgramParam(idOfs + 0x4, s_StationData[sIdx].m_ProtocolData[ProtocolType_Unreliable].isSendRepeat ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0x5, s_StationData[sIdx].m_ProtocolData[ProtocolType_Unreliable].receiveNum);
        pSampleFramework->SetProgramParam(idOfs + 0x6, s_StationData[sIdx].m_ProtocolData[ProtocolType_Unreliable].sendNum);

        pSampleFramework->SetProgramParam(idOfs + 0x8, s_StationData[sIdx].m_ProtocolData[ProtocolType_Reliable].isSendRepeat ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0x9, s_StationData[sIdx].m_ProtocolData[ProtocolType_Reliable].receiveNum);
        pSampleFramework->SetProgramParam(idOfs + 0xa, s_StationData[sIdx].m_ProtocolData[ProtocolType_Reliable].sendNum);

        pSampleFramework->SetProgramParam(idOfs + 0xc, s_StationData[sIdx].m_ProtocolData[ProtocolType_ReliableLarge].isSendRepeat ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0xd, s_StationData[sIdx].m_ProtocolData[ProtocolType_ReliableLarge].receiveNum);
        pSampleFramework->SetProgramParam(idOfs + 0xe, s_StationData[sIdx].m_ProtocolData[ProtocolType_ReliableLarge].sendNum);
    }

    // 全体宛て
    {
        uint32_t idOfs = (StationAllIndex + 1) << 4;

        pSampleFramework->SetProgramParam(idOfs + 0x4, s_StationData[StationAllIndex].m_ProtocolData[ProtocolType_Unreliable].isSendRepeat ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0x6, s_StationData[StationAllIndex].m_ProtocolData[ProtocolType_Unreliable].sendNum);

        pSampleFramework->SetProgramParam(idOfs + 0x8, s_StationData[StationAllIndex].m_ProtocolData[ProtocolType_Reliable].isSendRepeat ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0xa, s_StationData[StationAllIndex].m_ProtocolData[ProtocolType_Reliable].sendNum);

        pSampleFramework->SetProgramParam(idOfs + 0xc, s_StationData[StationAllIndex].m_ProtocolData[ProtocolType_ReliableLarge].isSendRepeat ? 1 : 0);
        pSampleFramework->SetProgramParam(idOfs + 0xe, s_StationData[StationAllIndex].m_ProtocolData[ProtocolType_ReliableLarge].sendNum);
    }
}
#endif


// 通信中のメインループです。
void MainLoop(nns::pia::GameMode gameMode)
{
    while (true)
    {
        bool isExit = false;
        bool bCallLeaveSession = false;

        s_CurrentTime.SetNow();

        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        isExit |= Operate();

#if NNS_PIA_ENABLE_INET
        // nns::pia::GameMode_InetXXX の場合は nn::nex::Scheduler が作られていて、そのディスパッチも行う必要があります。
        nns::pia::Nex_SchedulerDispatch();
#endif

        // nn::pia::common::Scheduler::Dispatch() を定期的に呼び出す必要があります。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        // Session 接続状態のチェック
        {
            nn::pia::Result result = nn::pia::session::Session::GetInstance()->CheckConnectionError();

            // エラーハンドリング。
            if (result.IsFailure())
            {
                enum nn::pia::Result::HandlingType handling = result.GetHandlingType();
                switch (handling)
                {
                case nn::pia::Result::HandlingType_CleanupWithLeave:
                    PIA_SAMPLE_PRINT("Exit MainLoop() with LeaveSession().\n");
                    bCallLeaveSession = true;
                    isExit = true;
                    break;
                case nn::pia::Result::HandlingType_ShutdownNetwork:
                    PIA_SAMPLE_PRINT("Exit MainLoop().\n");
                    isExit = true;
                    break;
                default:
                    // GetHandlingType() は HandlingType_CleanupWithLeave や HandlingType_ShutdownNetwork 以外の値を
                    // 返しうる関数ですが、Session::CheckConnectionError() が返しうる Result を考慮すると、
                    // これら以外の値が返ることはありません。そのため、アサート文を用意しています。
                    PIA_SAMPLE_ASSERTMSG(false, "Unexpected result.\n");
                    break;
                }
            }
        }

        if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedSession)
        {
            // 通信中であれば送受信処理を行います。

            SendUnreliable(GetUnreliableProtocol());
            SendReliable(GetReliableProtocol());
            SendReliableLarge(GetReliableProtocolLarge());

            ReceiveUnreliable(GetUnreliableProtocol());
            isExit |= ReceiveReliable(GetReliableProtocol());
            isExit |= ReceiveReliableLarge(GetReliableProtocolLarge());
        }
        else
        {
            // セッションが切れたので終了します。
            isExit = true;
        }

        // プロトコルに対して送信したものをすぐに処理するために、もう一度 nn::pia::common::Scheduler::Dispatch() を呼びます。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        Draw(gameMode);

#if PIA_TEST_CODE
        UpdatePiaTestData(gameMode, isExit);
#endif

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        if (isExit)
        {
            if (bCallLeaveSession)
            {
                nns::pia::Session_LeaveSession();
            }

            ClearSendRecvPacketNum();
            ClearSendRecvDataSize();

            return;
        }
    }
}


#if ENABLE_PRINT_CALLBACK
void MyPrintCallback(const char* str)
{
    PIA_DIRECT_PRINT_ALWAYS(str);
    PIA_DIRECT_PRINT_ALWAYS("\n");
}
#endif


}  // end of anonymous namespace


// エントリポイント。
PIA_SAMPLE_MAIN_FUNC()
{
    nn::pia::Result result;
    {
        nns::pia::sampleframework::SampleFramework::Setting setting;
        setting.isUpperScreenOnly = true;
        nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(30);
    }

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(PROGRAM_TITLE "\n");
    nns::pia::Basic_SetProgramTitle(PROGRAM_TITLE);

    InitBasicLargeData();

    s_CurrentTime.SetNow();

    while (true)
    {
        nn::pia::common::MemoryLeakChecker leakChecker;

#if PIA_TEST_CODE
        s_OptionalParam.Initialize(OptionalParamOffset);
        nns::pia::GameMode gameMode = nns::pia::DecideGameMode("PiaTransport ReliableAndUnreliable", &s_OptionalParam);
#else
        nns::pia::GameMode gameMode = nns::pia::DecideGameMode("PiaTransport ReliableAndUnreliable");
#endif
        if (gameMode == nns::pia::GameMode_Exit)
        {
            break;
        }

        // Piaの各モジュールの初期化処理
        // この順番で行ってください。
        {
#if 0
            uint64_t traceFlag = 0;
#else
            uint64_t traceFlag =
                nn::pia::common::TraceFlagCommon |
                nn::pia::common::TraceFlagTransport |
                nn::pia::common::TraceFlagLocal |
                nn::pia::common::TraceFlagSession;
#endif
            result = nns::pia::Common_InitializePiaCommon(s_Memory, MemoryBlockSize, traceFlag, true);
            PIA_SAMPLE_ASSERT(result.IsSuccess());

            result = leakChecker.Set();
            PIA_SAMPLE_ASSERT(result.IsSuccess());

            nn::pia::transport::NetworkFactory* pFactory = NULL;
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            {
#if NN_PIA_CTR
                PIA_SAMPLE_PRINT("subId: %d\n", LocalSubId);
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(LocalTitleId, LocalSubId));
#elif NN_PIA_NINTENDOSDK
                PIA_SAMPLE_PRINT("SceneId: %d\n", LocalSceneId);
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(nns::pia::LocalCommunicationId, LocalSceneId));
#endif
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Local_InitializePiaLocal() failed\n");
                    nns::pia::Local_FinalizePiaLocal();

                    result = leakChecker.Check();
                    PIA_SAMPLE_ASSERT(result.IsSuccess());

                    nns::pia::Common_FinalizePiaCommon();
                    continue;
                }
                pFactory = nns::pia::Local_GetNetworkFactory();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetRandom:
                // PiaInet を初期化するより前に NEX を初期化してください
                result = nns::pia::Nex_InitializeNex(
                    MemoryBlockSizeForNex,
                    MemoryBlockSizeForNexEmergency);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                result = nns::pia::Inet_InitializePiaInet();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Inet_GetNetworkFactory();
                break;
#endif
            case nns::pia::GameMode_LanRandom:
                result = nns::pia::Lan_InitializePiaLan();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                pFactory = nns::pia::Lan_GetNetworkFactory();
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            // Factoryを利用して、Transportをセットアップします。
            nn::pia::transport::Transport::Setting transportSetting;
            transportSetting.pFactory = pFactory;
            transportSetting.maxStationNum = MaxStations;
            transportSetting.maxSendThreadBufferNum = nns::pia::MaxSendThreadBufferNum;
            transportSetting.maxReceiveThreadBufferNum = nns::pia::MaxReceiveThreadBufferNum;
            transportSetting.analysisInterval = TRANSPORT_ANALYSIS_INTERVAL;  // 単位は秒
            transportSetting.isAnalysisAutoPrintDetail = true;
            transportSetting.isAnalysisAutoPrintTotalOnly = false;

#if PIA_TEST_CODE
            // レイテンシエミュレーション機能やパケロスエミュレーションなどの
            // デバッグ用機能を利用しない場合は、SetDebugSetting() を呼び出してはいけません。
            {
                nn::pia::transport::Transport::DebugSetting debugSetting;
                debugSetting.sendThreadLatencyEmulationPacketNum = 0;
                debugSetting.receiveThreadLatencyEmulationPacketNum = 0;
                debugSetting.isPacketLossEmulationEnabled = s_OptionalParam.bPacketLossEmulation;
                result = nn::pia::transport::Transport::SetDebugSetting(debugSetting);
                PIA_SAMPLE_ASSERT(result.IsSuccess());
            }
#endif

            result = nns::pia::Transport_InitializePiaTransport(
                transportSetting,
                0,
                NULL,
                0,
                NULL,
                64,
                64,
                CreateAndInitializeProtocols,
                32);
            PIA_SAMPLE_ASSERT(result.IsSuccess());

            {
                // PiaTransport の送受信スレッドの優先度、スリープ時間を明示的に指定しています。
                nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));

                pMgr->SetSendThreadPriority(8);
                pMgr->SetSendThreadSleepTimeSpan(5);
                pMgr->SetReceiveThreadPriority(8);
                pMgr->SetReceiveThreadSleepTimeSpan(5);

                // 設定した値の確認。
                PIA_SAMPLE_PRINT("Send thread priority:      %d\n", pMgr->GetSendThreadPriority());
                PIA_SAMPLE_PRINT("Send thread sleep span:    %d\n", pMgr->GetSendThreadSleepTimeSpan());
                PIA_SAMPLE_PRINT("Receive thread priority:   %d\n", pMgr->GetReceiveThreadPriority());
                PIA_SAMPLE_PRINT("Receive thread sleep span: %d\n", pMgr->GetReceiveThreadSleepTimeSpan());
            }

            {
                // RTT パルス送出の初期インターバルを明示的に指定／取得する例を示しています。
                // デフォルト設定のままで構わなければ、SetInitialRttPulseInterval() を呼び出す必要はありません。
                nn::pia::transport::Station::SetInitialRttPulseInterval(200);  // 単位はミリ秒です。
                PIA_SAMPLE_PRINT("Initial RTT pulse interval: %d\n", nn::pia::transport::Station::GetInitialRttPulseInterval());
            }

            {
                // トランスポート分析機能のテキストをアプリ独自の関数に渡したい場合は、
                // RegisterPrintCallback() に非 NULL のコールバック関数ポインタを与えてください。
                nn::pia::transport::TransportAnalyzer* pAnalyzer = nn::pia::transport::TransportAnalyzer::GetInstance();
                PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pAnalyzer));
#if ENABLE_PRINT_CALLBACK
                pAnalyzer->RegisterPrintCallback(MyPrintCallback);
#else
                pAnalyzer->RegisterPrintCallback(NULL);
#endif
            }

#if !ENABLE_PROFILING && NNS_PIA_ENABLE_LOCAL
            if (gameMode == nns::pia::GameMode_LocalHost || gameMode == nns::pia::GameMode_LocalClient)
            {
                GetReliableProtocolLarge()->SetThroughputLimit(3000);
            }
#endif

#if PIA_TEST_CODE
            if (s_OptionalParam.bPacketLossEmulation)
            {
                nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
                PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));

                pMgr->SetSendThreadPacketLossRatio(s_OptionalParam.sendPacketLossPercentage);
                pMgr->SetReceiveThreadPacketLossRatio(s_OptionalParam.recvPacketLossPercentage);
            }

            // Session の初期化を行います。
            nn::pia::session::Session::Setting sessionSetting;
            sessionSetting.pNetworkFactory = pFactory;
            sessionSetting.browsedSessionInfoListNum = 8;

            if (s_OptionalParam.bRelayEmulation)
            {
                // リレーエミュレーションを行う場合は、Session のインスタンス作成前に設定関数を呼び出す必要があります。
                nn::pia::session::Session::DebugSetting debugSetting;
                debugSetting.isRelayEmulationEnabled = true;
                nn::pia::Result ret = nn::pia::session::Session::SetDebugSetting(debugSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(ret);
                sessionSetting.networkTopology = nn::pia::session::NetworkTopology_RelayMesh;
            }
            else
            {
                nn::pia::session::Session::DebugSetting debugSetting;
                debugSetting.isRelayEmulationEnabled = false;
                nn::pia::Result ret = nn::pia::session::Session::SetDebugSetting(debugSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(ret);
                sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
            }
#else
            nn::pia::session::Session::Setting sessionSetting;
            sessionSetting.pNetworkFactory = pFactory;
            sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
            sessionSetting.browsedSessionInfoListNum = 8;
#endif
            result = nns::pia::Session_InitializePiaSession(sessionSetting);
        }

        // ネットワークの接続、セッションの確立の処理
        {
            nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SessionEventCallback);
            InitializeStationData();

            // ネットワークの初期化
            nns::pia::NetworkType networkType = nns::pia::NetworkType_Internet;
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            {
                networkType = nns::pia::NetworkType_Local;
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetRandom:
            {
                networkType = nns::pia::NetworkType_Internet;
            }
            break;
#endif
            case nns::pia::GameMode_LanRandom:
            {
                networkType = nns::pia::NetworkType_Lan;
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(networkType);
            if (netResult != nns::pia::NetResult_Success)
            {
                PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork:failed. (netResult = %d)\n", netResult);

                switch (gameMode)
                {
#if NNS_PIA_ENABLE_LOCAL
                case nns::pia::GameMode_LocalHost:
                case nns::pia::GameMode_LocalClient:
                {
                    gameMode = nns::pia::GameMode_LocalError;
                }
                break;
#endif
#if NNS_PIA_ENABLE_INET
                case nns::pia::GameMode_InetRandom:
                {
                    gameMode = nns::pia::GameMode_InetError;
                }
                break;
#endif
                case nns::pia::GameMode_LanRandom:
                {
                    gameMode = nns::pia::GameMode_LanError;
                }
                break;
                default:
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                break;
                }
            }

            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

#if ENABLE_PROFILING
                startupSetting.maxSilenceTime = nn::pia::session::SessionMaxSilenceTimeMax;
#endif

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

// セッションを構築します。
// 作成するマッチメイクセッションの設定
#if NN_PIA_CTR
                nn::pia::local::UdsCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    createSessionSetting.SetSubId(LocalSubId);
                    createSessionSetting.SetChannel(LocalChannel);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    createSessionSetting.SetMaxParticipantNum(MaxStations);
                }
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    createSessionSetting.SetSceneId(LocalSceneId);
                    createSessionSetting.SetChannel(LocalChannel);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    createSessionSetting.SetMaxParticipantNum(MaxStations);
                }
#endif
                result = nns::pia::Session_CreateSession(&createSessionSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_CreateSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }
            }
            break;

            case nns::pia::GameMode_LocalClient:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

#if ENABLE_PROFILING
                startupSetting.maxSilenceTime = nn::pia::session::SessionMaxSilenceTimeMax;
#endif

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }
                // セッションを検索します。
                // 検索条件の設定
                const int SearchSettingNum = 1;
#if NN_PIA_CTR
                nn::pia::local::UdsSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    searchCriteriaList[0].SetSubId(LocalSubId);
                    searchCriteriaList[0].SetMaxParticipantNum(MaxStations);
                }
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    searchCriteriaList[0].SetSceneId(LocalSceneId);
                    searchCriteriaList[0].SetMaxParticipantNum(MaxStations);
                }
#endif
                searchCriteriaList[0].SetResultRange(0, 8);
                result = nns::pia::Session_BrowseSession(&searchCriteriaList[0]);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_BrowseSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

                // 検索したセッションに参加します。
                uint32_t foundSessionNum = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize();
                if (foundSessionNum > 0)
                {
                    PIA_SAMPLE_PRINT("session found\n");
                    nn::pia::session::ISessionInfoList::Iterator it = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->Begin();
                    nn::pia::session::ISessionInfo* pTargetSessionInfo = *it;
                    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTargetSessionInfo));
#if NN_PIA_CTR
                    nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#elif NN_PIA_NINTENDOSDK
                    nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
#endif
                    joinSessionSetting.SetSessionInfoPtr(pTargetSessionInfo);
                    joinSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    result = nns::pia::Session_JoinSession(&joinSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("nns::pia::Session_JoinSession:failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                        gameMode = nns::pia::GameMode_LocalError;
                        break;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("no session found\n");
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }
            }
            break;

            case nns::pia::GameMode_LocalError:
            {
            }
            break;
#endif  // NNS_PIA_ENABLE_LOCAL
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetRandom:
            {
                // ゲームサーバにログイン
                if (!nns::pia::Inet_Login())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Inet_Login:failed\n");
                    gameMode = nns::pia::GameMode_InetError;
                    break;
                }
                // Session クラスのスタートアップ処理。
                nn::pia::inet::NexSessionStartupSetting startupSetting;
                startupSetting.isHostMigrationEnabled = true;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_InetError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::inet::NexCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const int SearchSettingNum = 1;
                nn::pia::inet::NexSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::inet::NexJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_InetError;
                    break;
                }

#if PIA_TEST_CODE
                if (s_OptionalParam.bRelayEmulation)
                {
                    nns::pia::SetRelayEmulation(nn::pia::StationIndex_2, nn::pia::StationIndex_3);
                }
#endif

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = nns::pia::GameMode_InetHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = nns::pia::GameMode_InetClient;
                }
            }
            break;

            case nns::pia::GameMode_InetError:
            {
            }
            break;
#endif
            case nns::pia::GameMode_LanRandom:
            {
                result = nns::pia::Lan_StartupPiaLan();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Lan_StartupPiaLan failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LanError;
                    break;
                }
                // Session クラスのスタートアップ処理。
                nn::pia::lan::LanSessionStartupSetting startupSetting;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

#if ENABLE_PROFILING
                startupSetting.maxSilenceTime = nn::pia::session::SessionMaxSilenceTimeMax;
#endif

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LanError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::lan::LanCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const int SearchSettingNum = 1;
                nn::pia::lan::LanSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::lan::LanJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LanError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = nns::pia::GameMode_LanHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = nns::pia::GameMode_LanClient;
                }
            }
            break;

            case nns::pia::GameMode_LanError:
            {
            }
            break;

            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }
        }

        // メインの処理
        switch (gameMode)
        {
#if NNS_PIA_ENABLE_LOCAL
        case nns::pia::GameMode_LocalHost:
        case nns::pia::GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
        case nns::pia::GameMode_InetHost:
        case nns::pia::GameMode_InetClient:
#endif
        case nns::pia::GameMode_LanHost:
        case nns::pia::GameMode_LanClient:
            MainLoop(gameMode);

            // Watermark機能による計測結果プリント。
            if (PIA_IS_VALID_POINTER(nn::pia::common::WatermarkManager::GetInstance()))
            {
                nn::pia::common::WatermarkManager::GetInstance()->Trace(nn::pia::common::TraceFlagAlways);
            }

            break;

        default:
            break;
        }


        // セッションの終了、ネットワークからの切断の処理
        {
            // Session 終了処理。
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetHost:
            case nns::pia::GameMode_InetClient:
#endif
            case nns::pia::GameMode_LanHost:
            case nns::pia::GameMode_LanClient:
                // セッションから離脱
                nns::pia::Session_LeaveSession();
                break;
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalError:
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetError:
#endif
            case nns::pia::GameMode_LanError:
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            // Session クラスのクリーンアップ処理。
            nns::pia::Session_CleanupSession();

            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            case nns::pia::GameMode_LocalError:
                nns::pia::Local_FinalizeLocalCommunication();
                break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetHost:
            case nns::pia::GameMode_InetClient:
            case nns::pia::GameMode_InetError:
                nns::pia::Inet_Logout();
                break;
#endif
            case nns::pia::GameMode_LanHost:
            case nns::pia::GameMode_LanClient:
            case nns::pia::GameMode_LanError:
                nns::pia::Lan_CleanupPiaLan();
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            nns::pia::Net_FinalizeNetwork();
        }


        // Piaの各モジュールの終了処理
        // 初期化と逆順で行ってください。
        {
            nns::pia::Session_FinalizePiaSession();
            nns::pia::Transport_FinalizePiaTransport(0, 0, FinalizeAndDestroyProtocols);
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            case nns::pia::GameMode_LocalError:
                nns::pia::Local_FinalizePiaLocal();
                break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetHost:
            case nns::pia::GameMode_InetClient:
            case nns::pia::GameMode_InetError:
                nns::pia::Inet_FinalizePiaInet();
                nns::pia::Nex_FinalizeNex();
                break;
#endif
            case nns::pia::GameMode_LanHost:
            case nns::pia::GameMode_LanClient:
            case nns::pia::GameMode_LanError:
                nns::pia::Lan_FinalizePiaLan();
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            result = leakChecker.Check();
            PIA_SAMPLE_ASSERT(result.IsSuccess());

            nns::pia::Common_FinalizePiaCommon();
        }
    }

    nns::pia::sampleframework::SampleFramework::DestroyInstance();

    PIA_SAMPLE_PRINT("end.\n");
}
