﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"

#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Common_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Demo.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Local_Setup.h"

#define ENABLE_HOST_MIGRATION 1

#include <cstdio>
#include <cstdlib>
#include <cstring>


#ifndef ENABLE_PROFILING
#define ENABLE_PROFILING 0
#endif

#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

#define PROGRAM_TITLE "ReliableBroadcast"


namespace
{
const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 30100;  // この値は、Pia サンプルプログラム毎に違えてください。
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;

const int MaxStations = 4;  // 最大 Station 数

const uint32_t OriginalDataSize = 10 * 1024 * 1024;                     // 送受信する元データのサイズ
static uint32_t s_OriginalData[OriginalDataSize / sizeof(uint32_t)];    // 送受信する元データ
const uint32_t ReceiveBufferSize = 8 * 1024 * 1024;                     // 受信バッファのサイズ
static uint32_t s_ReceiveBuffer[ReceiveBufferSize / sizeof(uint32_t)];  // 受信バッファ

#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
const uint32_t LocalTitleId = 0xFF301;  // for transport sample
const uint8_t LocalSubId = MatchmakeGameModeOffset;
#elif NN_PIA_NINTENDOSDK
const uint16_t LocalSceneId = MatchmakeGameModeOffset + 5000 + 30000 * PIA_TEST_CODE;
#endif
const int16_t LocalChannel = 0;
const char LocalWirelessCryptoKey[] = "aaabbccc0123456789";
const char SignatureKey[] = "ReliableBroadcast_key";
#endif

#if NNS_PIA_ENABLE_INET
// アプリケーションの仕様に応じて調整して下さい。
const uint32_t MemoryBlockSizeForNex = 1 * 1024 * 1024;
const uint32_t MemoryBlockSizeForNexEmergency = 50 * 1024;
#endif

const uint32_t MemoryBlockSize = 1 * 1024 * 1024;                 // nn::pia::common::Initialize()で供給するメモリサイズ（単位はバイト）
PIA_ATTRIBUTE_ALIGN(8) static uint8_t s_Memory[MemoryBlockSize];  // nn::pia::common::Initialize()で供給するメモリ


const uint16_t ReliableBroadcastProtocolPort = 0;  // ReliableBroadcastProtocol が使用するポート番号。

static uint32_t s_hReliableBroadcast = 0;

#if PIA_TEST_CODE
struct nns::pia::OptionalParameter s_OptionalParam;
const int32_t OptionalParamOffset = 200;
#endif


nn::pia::StationId conv2StationId(nn::pia::StationIndex idx)
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    if (!PIA_IS_VALID_POINTER(pTransport))
    {
        PIA_SAMPLE_ASSERTMSG(false, "conv2StationId(): Transport instance is not ready?");
        return nn::pia::StationIdInvalid;
    }

    nn::pia::StationId id;
    nn::pia::Result ret = pTransport->ConvertToStationId(&id, idx);
    if (ret.IsFailure())
    {
        if (ret == nn::pia::ResultNotFound())
        {
            return nn::pia::StationIdInvalid;
        }
        else
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Result = 0x%08x\n", ret.GetValue());
            PIA_SAMPLE_ASSERTMSG(false, "conv2StationId(): ConvertToStationId() failed.");
            return nn::pia::StationIdInvalid;
        }
    }

    return id;
}


nn::pia::StationIndex conv2StationIndex(nn::pia::StationId id)
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    if (!PIA_IS_VALID_POINTER(pTransport))
    {
        PIA_SAMPLE_ASSERTMSG(false, "conv2StationIndex: Transport instance is not ready?");
        return nn::pia::StationIndex_Invalid;
    }

    nn::pia::StationIndex idx;
    nn::pia::Result ret = pTransport->ConvertToStationIndex(&idx, id);
    if (ret.IsFailure())
    {
        if (ret == nn::pia::ResultNotFound())
        {
            return nn::pia::StationIndex_Invalid;
        }
        else
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Result = 0x%08x\n", ret.GetValue());
            PIA_SAMPLE_ASSERTMSG(false, "conv2StationIndex: ConvertToStationIndex() failed.");
            return nn::pia::StationIndex_Invalid;
        }
    }

    return idx;
}


// プロトコルの作成と初期化を行います。
nn::pia::Result CreateAndInitializeProtocol()
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // ReliableBroadcastProtocol の作成と初期化。
    {
        PIA_SAMPLE_ASSERT(s_hReliableBroadcast == 0);
        s_hReliableBroadcast = pTransport->CreateProtocol<nn::pia::transport::ReliableBroadcastProtocol>(ReliableBroadcastProtocolPort);
        PIA_SAMPLE_ASSERT(s_hReliableBroadcast != 0);  // CreateProtocol()は、既に同じプロトコルIDが存在していた場合は0を返すので、そうではないことを確認
        pTransport->GetProtocol<nn::pia::transport::ReliableBroadcastProtocol>(s_hReliableBroadcast)->Initialize();
    }

    return nn::pia::ResultSuccess();
}


// プロトコルの後始末をします。
void FinalizeAndDestroyProtocol()
{
    // Transportシングルトン取得。
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // ReliableBroadcastProtocol の終了処理
    nn::pia::transport::ReliableBroadcastProtocol* pReliableBroadcastProtocol = pTransport->GetProtocol<nn::pia::transport::ReliableBroadcastProtocol>(s_hReliableBroadcast);
    if (PIA_IS_VALID_POINTER(pReliableBroadcastProtocol))
    {
        pReliableBroadcastProtocol->Finalize();
        pTransport->DestroyProtocol(s_hReliableBroadcast);
        s_hReliableBroadcast = 0;
    }
}


nn::pia::transport::ReliableBroadcastProtocol* GetReliableBroadcastProtocol()
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(nn::pia::transport::Transport::GetInstance()));
    return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::ReliableBroadcastProtocol>(s_hReliableBroadcast);
}


// 送信する元データを作成します。
void InitOriginalData()
{
    std::srand(0x12345678);
    uint32_t size = OriginalDataSize / sizeof(uint32_t);
    for (uint32_t i = 0; i < size; ++i)
    {
        s_OriginalData[i] = static_cast<uint32_t>(std::rand());
    }
}


// データの内容をチェックします。
bool CheckReceivedData(uint32_t offset, uint32_t size)
{
    if (offset + size > OriginalDataSize)
    {
        // 元データの範囲外まで参照しようとするのは不正なデータです。
        return false;
    }

    const void* cpData = reinterpret_cast<void*>(reinterpret_cast<uintptr_t>(s_OriginalData) + offset);
    return (std::memcmp(s_ReceiveBuffer, cpData, size) == 0);
}


static nn::pia::StationId s_ReceiveStationId;                                      // 受信するデータの送信元の指定
static uint32_t s_ReceiveBufferSize;                                               // 指定した受信バッファのサイズ
static uint32_t s_SendDataSize;                                                    // 送信するデータのサイズ
static nn::pia::transport::ReliableBroadcastProtocol::Configuration s_SendConfig;  // 送信時の設定
static bool s_IsCompletionChecked;                                                 // 送受信の完了をチェックしたか
const uint32_t MessageSize = 32;                                                   // 表示用メッセージのサイズ
static char s_Message[MessageSize];                                                // 表示用メッセージ

#if ENABLE_PROFILING
static uint32_t s_TimeElapsed = 0;  // 前回の処理の所要時間(ms)
#endif

// 操作対象
enum OperationTarget
{
    OperationTarget_ReceiveStationId = 0,
    OperationTarget_ReceiveBufferSize,
    OperationTarget_StartReceive,
    OperationTarget_RejectRequest,
    OperationTarget_SendSize,
    OperationTarget_ConfigSendToNewcomer,
    OperationTarget_SynchronizeCompletion,
    OperationTarget_SendEndless,
    OperationTarget_ConfigFailedIfRejected,
    OperationTarget_MediateRequest,
    OperationTarget_StartSend,
    OperationTarget_ThroughputLimit,
    OperationTarget_Cancel,
    OperationTarget_FinishSendEndless,
    OperationTarget_Reset,
    OperationTarget_Max
};
OperationTarget s_OperationTarget;

enum OperationGroup
{
    OperationGroup_Receive = 0,
    OperationGroup_Send,
    OperationGroup_Progress,
    OperationGroup_Setting,
    OperationGroup_Max
};

OperationGroup GetOperationGroup(OperationTarget target)
{
    switch (target)
    {
    case OperationTarget_ReceiveStationId:
    case OperationTarget_ReceiveBufferSize:
    case OperationTarget_StartReceive:
    case OperationTarget_RejectRequest:
        return OperationGroup_Receive;

    case OperationTarget_SendSize:
    case OperationTarget_ConfigSendToNewcomer:
    case OperationTarget_SynchronizeCompletion:
    case OperationTarget_SendEndless:
    case OperationTarget_ConfigFailedIfRejected:
    case OperationTarget_MediateRequest:
    case OperationTarget_StartSend:
        return OperationGroup_Send;

    case OperationTarget_ThroughputLimit:
    case OperationTarget_Cancel:
    case OperationTarget_FinishSendEndless:
    case OperationTarget_Reset:
        return OperationGroup_Progress;

    default:
        PIA_SAMPLE_ASSERT(false);
        return OperationGroup_Progress;
    }
}

static uint32_t s_LineFeed[OperationGroup_Max];
static const uint32_t cLineFeedSize = 9;
static nn::pia::common::Time s_StartTime;

// アプリケーションの設定を初期化します。
void InitializeGameData()
{
    s_ReceiveStationId = nn::pia::StationIdInvalid;
    s_ReceiveBufferSize = 8 * 1024 * 1024;
    s_SendDataSize = 1 * 1024 * 1024;
    s_SendConfig = nn::pia::transport::ReliableBroadcastProtocol::Configuration();
    s_IsCompletionChecked = false;
    memset(s_Message, 0, sizeof(s_Message));
    s_OperationTarget = OperationTarget_ReceiveStationId;
    for (uint32_t i = 0; i < OperationGroup_Max; ++i)
    {
        s_LineFeed[i] = 0;
    }
    s_LineFeed[GetOperationGroup(s_OperationTarget)] = cLineFeedSize;

    std::srand(static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick()));
}


// 接続、切断時の処理です。
void SessionEventCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId, void* pApplicationParam)
{
    NN_PIA_UNUSED(pApplicationParam);
    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
        break;
    case nn::pia::session::Session::EventType_EventLeave:
        break;
    case nn::pia::session::Session::EventType_SessionHostChanged:
        break;
    case nn::pia::session::Session::EventType_HostMigrationFailed:
        // ホストマイグレーションに失敗
        break;
    default:
        PIA_SAMPLE_ASSERTMSG(false, "Unsupported event(%d).", eventType);
        break;
    }
}


// 表示用メッセージを設定します。
void SetMessage(const char* cpFormat, ...)
{
    va_list va;
    va_start(va, cpFormat);
    vsnprintf(s_Message, MessageSize - 1, cpFormat, va);
    va_end(va);
    s_Message[MessageSize - 1] = '\0';
    PIA_SAMPLE_PRINT("%s\n", s_Message);
}


// 送信を開始します。
void StartSend()
{
    PIA_SAMPLE_PRINT("StartSend dataSize:%d\n", s_SendDataSize);

    // 乱数でオフセット値を決めて、元データからそのオフセット値分ずれた位置のデータを送信データとします。
    // オフセット値はユーザーデータとして受信側に伝えます。

    uint32_t offset = static_cast<uint32_t>(std::rand() & 0xFFFFFFFC) % (OriginalDataSize - s_SendDataSize);
    const void* cpData = reinterpret_cast<void*>(reinterpret_cast<uintptr_t>(s_OriginalData) + offset);
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();
    nn::pia::Result r = pProtocol->StartSend(cpData, s_SendDataSize, s_SendConfig, offset);
    NN_PIA_UNUSED_IN_RELEASE(r);
    PIA_SAMPLE_ASSERT(r.IsSuccess());
    SetMessage("start send data");
    s_StartTime.SetNow();
#if ENABLE_PROFILING
    s_TimeElapsed = 0;
#endif
    s_OperationTarget = OperationTarget_Cancel;
    s_IsCompletionChecked = false;
}


// 受信を開始します。
void StartReceive()
{
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();

    if (s_ReceiveStationId == nn::pia::StationIdInvalid)
    {
        // 最優先の受信要求のものの受信を開始します。
        const nn::pia::transport::ReliableBroadcastProtocol::TransferSetting* cpRequest = pProtocol->GetRequest();
        if (cpRequest != NULL)
        {
            nn::pia::StationId srcId = cpRequest->GetSourceStationId();
            PIA_SAMPLE_ASSERT(srcId != nn::pia::StationIdInvalid);
            nn::pia::Result r = pProtocol->StartReceive(s_ReceiveBuffer, s_ReceiveBufferSize, srcId);
            NN_PIA_UNUSED_IN_RELEASE(r);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
            SetMessage("start receive data");
            s_StartTime.SetNow();
            s_OperationTarget = OperationTarget_Cancel;
            s_IsCompletionChecked = false;
        }
    }
    else if (nn::pia::session::Session::GetInstance()->IsValidStationId(s_ReceiveStationId))
    {
        // 指定したステーションからの受信を開始します。
        const nn::pia::transport::ReliableBroadcastProtocol::TransferSetting* cpRequest = pProtocol->GetRequest(s_ReceiveStationId);
        if (cpRequest != NULL)
        {
            nn::pia::Result r = pProtocol->StartReceive(s_ReceiveBuffer, s_ReceiveBufferSize, s_ReceiveStationId);
            NN_PIA_UNUSED_IN_RELEASE(r);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
            SetMessage("start receive data");
            s_StartTime.SetNow();
#if ENABLE_PROFILING
            s_TimeElapsed = 0;
#endif
            s_OperationTarget = OperationTarget_Cancel;
            s_IsCompletionChecked = false;
        }
        else
        {
            nn::pia::Result r = pProtocol->StartReceive(s_ReceiveBuffer, s_ReceiveBufferSize, s_ReceiveStationId);
            NN_PIA_UNUSED_IN_RELEASE(r);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
            SetMessage("start receive data");
            s_StartTime.SetNow();
#if ENABLE_PROFILING
            s_TimeElapsed = 0;
#endif
            s_OperationTarget = OperationTarget_Cancel;
            s_IsCompletionChecked = false;
        }
    }
}


// 受信要求を拒否します。
void RejectRequest()
{
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();
    nn::pia::Result r = pProtocol->RejectRequest();
    PIA_SAMPLE_ASSERT(r.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(r);
    SetMessage("reject request");
    s_OperationTarget = OperationTarget_ReceiveStationId;
    s_IsCompletionChecked = false;
}


// 現在の送受信を中断します。
void Cancel()
{
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();
    switch (pProtocol->GetState())
    {
    case nn::pia::transport::ReliableBroadcastProtocol::State_Sending:
    {
        // 送信を中断します。
        nn::pia::Result r = pProtocol->Cancel();
        NN_PIA_UNUSED_IN_RELEASE(r);
        PIA_SAMPLE_ASSERT(r.IsSuccess());
        SetMessage("cancel send data");
    }
    break;

    case nn::pia::transport::ReliableBroadcastProtocol::State_Receiving:
    {
        // 受信を中断します。
        nn::pia::Result r = pProtocol->Cancel();
        NN_PIA_UNUSED_IN_RELEASE(r);
        PIA_SAMPLE_ASSERT(r.IsSuccess());
        SetMessage("cancel receive data");
    }
    break;

    default:
        break;
    }
}


// エンドレス送信を終了させます。
void FinishSendEndless()
{
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();
    PIA_SAMPLE_ASSERT(pProtocol->GetState() == nn::pia::transport::ReliableBroadcastProtocol::State_Sending);
    const nn::pia::transport::ReliableBroadcastProtocol::TransferSetting* cpTransferSetting = pProtocol->GetCurrentTransferSetting();
    NN_PIA_UNUSED_IN_RELEASE(cpTransferSetting);
    PIA_SAMPLE_ASSERT(cpTransferSetting != NULL);
    PIA_SAMPLE_ASSERT(cpTransferSetting->m_Configuration.IsSendEndless());
    nn::pia::Result r = pProtocol->FinishSendEndless();
    NN_PIA_UNUSED_IN_RELEASE(r);
    PIA_SAMPLE_ASSERT(r.IsSuccess());
    SetMessage("finish SendEndless");
}


// 送受信完了状態をリセットして初期状態に戻します。
void Reset()
{
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();
    PIA_SAMPLE_ASSERT(pProtocol->IsRunning() == false);
    nn::pia::Result r = pProtocol->Reset();
    NN_PIA_UNUSED_IN_RELEASE(r);
    PIA_SAMPLE_ASSERT(r.IsSuccess());
    SetMessage("reset");
    s_IsCompletionChecked = false;
}


// 指定した操作対象が、現在操作できる状態かどうかを判定します。
bool IsValidOperationTarget(OperationTarget operationTarget)
{
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();

    switch (operationTarget)
    {
    case OperationTarget_ReceiveStationId:
    case OperationTarget_ReceiveBufferSize:
        return (pProtocol->IsRunning() == false);

    case OperationTarget_StartReceive:
        if (pProtocol->IsRunning())
        {
            return false;
        }
        if (s_ReceiveStationId == nn::pia::StationIdInvalid)
        {
            return (pProtocol->GetRequest() != NULL);
        }
        else
        {
            return true;
        }

    case OperationTarget_RejectRequest:
        if (pProtocol->IsRunning())
        {
            return false;
        }
        if (s_ReceiveStationId == nn::pia::StationIdInvalid)
        {
            return (pProtocol->GetRequest() != NULL);
        }
        else
        {
            return (pProtocol->GetRequest(s_ReceiveStationId) != NULL);
        }

    case OperationTarget_SendSize:
    case OperationTarget_ConfigSendToNewcomer:
    case OperationTarget_SynchronizeCompletion:
    case OperationTarget_SendEndless:
    case OperationTarget_ConfigFailedIfRejected:
    case OperationTarget_MediateRequest:
    case OperationTarget_StartSend:
        return (pProtocol->IsRunning() == false);

    case OperationTarget_ThroughputLimit:
        return true;

    case OperationTarget_Cancel:
        return pProtocol->IsRunning();

    case OperationTarget_FinishSendEndless:
        return ((pProtocol->GetState() == nn::pia::transport::ReliableBroadcastProtocol::State_Sending) &&
                (pProtocol->GetCurrentTransferSetting()->m_Configuration.IsSendEndless()));

    case OperationTarget_Reset:
        return (pProtocol->IsRunning() == false);

    default:
        return false;
    }
}


// キー入力による操作です。
bool Operate()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();

    // START で GameLoop() を抜けます。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
    {
        return true;
    }

    // キーリピート用のカウント
    static int32_t keyCountL = -1;
    static int32_t keyCountR = -1;
    {
        if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonLeft))
        {
            ++keyCountL;
            keyCountR = -1;
        }
        else if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonRight))
        {
            keyCountL = -1;
            ++keyCountR;
        }
        else
        {
            keyCountL = -1;
            keyCountR = -1;
        }
    }

    // Up / Down で操作対象を切り替えます。
    {
        int offset = 0;
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            offset = -1;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            offset = 1;
        }
        else if (IsValidOperationTarget(s_OperationTarget) == false)
        {
            offset = 1;
        }

        if (offset != 0)
        {
            int target = s_OperationTarget;
            while (true)
            {
                target += offset;
                if (target < 0)
                {
                    target = OperationTarget_Max - 1;
                }
                else if (target >= OperationTarget_Max)
                {
                    target = 0;
                }

                if (IsValidOperationTarget(static_cast<OperationTarget>(target)))
                {
                    s_OperationTarget = static_cast<OperationTarget>(target);
                    return false;
                }
            }
        }
    }

    switch (s_OperationTarget)
    {
    case OperationTarget_ReceiveStationId:
        // Left / Right で受信対象を切り替えます。
        {
            int offset = 0;
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                offset = 1;
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                offset = -1;
            }

            // s_ReceiveStationId が nn::pia::StationIdInvalid の時は、最優先の受信要求を指していることを意味します。
            // 自分を飛ばして、対象のステーションを切り替えます。

            if (offset != 0)
            {
                int stIdx = static_cast<int>(conv2StationIndex(s_ReceiveStationId));
                while (true)
                {
                    if (stIdx == nn::pia::StationIndex_Invalid)
                    {
                        if (offset > 0)
                        {
                            stIdx = 0;
                        }
                        else
                        {
                            stIdx = MaxStations - 1;
                        }
                    }
                    else
                    {
                        stIdx += offset;
                        if (stIdx < 0 || stIdx >= MaxStations)
                        {
                            s_ReceiveStationId = nn::pia::StationIdInvalid;
                            break;
                        }
                    }

                    if (stIdx != conv2StationIndex(pSession->GetLocalStationId()))
                    {
                        s_ReceiveStationId = conv2StationId(static_cast<nn::pia::StationIndex>(stIdx));
                        break;
                    }
                }
            }
        }
        break;

    case OperationTarget_ReceiveBufferSize:
    case OperationTarget_SendSize:
        // Left / Right で送受信のサイズを切り替えます。
        // 通常は 256KB 単位、 L を押しながらだと 1KB 単位、 R を押しながらだと 4B 単位で変化します。
        {
            uint32_t* pSize = (s_OperationTarget == OperationTarget_ReceiveBufferSize) ? &s_ReceiveBufferSize : &s_SendDataSize;

            uint32_t unit = 256 * 1024;
            if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonL))
            {
                unit = 1024;
            }
            else if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonR))
            {
                unit = 4;
            }

            if (keyCountL == 0 || keyCountL > 15)
            {
                if (*pSize > unit)
                {
                    if (*pSize % unit > 0)
                    {
                        *pSize -= (*pSize % unit);
                    }
                    else
                    {
                        *pSize -= unit;
                    }
                }
            }
            else if (keyCountR == 0 || keyCountR > 15)
            {
                *pSize = *pSize - (*pSize % unit) + unit;
                if (*pSize > ReceiveBufferSize)
                {
                    *pSize = ReceiveBufferSize;
                }
            }

#if PIA_TEST_CODE
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
            {
                *pSize = 1 * 1024 * 1024;
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                *pSize = 8 * 1024 * 1024;
            }
#endif
        }
        break;

    case OperationTarget_ThroughputLimit:
        // Left / Right でスループットの上限値を切り替えます。
        // 通常は 256B 単位、 R を押しながらだと 4B 単位で変化します。
        {
            uint32_t unit = 256;
            if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonR))
            {
                unit = 4;
            }
#if PIA_TEST_CODE
            else if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonL))
            {
                unit = 32;
            }
            else if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonY))
            {
                unit = 2 * 1024;
            }
            else if (pSampleFramework->IsHold(nns::pia::sampleframework::ButtonX))
            {
                unit = 16 * 1024;
            }
#endif

            uint32_t limit = pProtocol->GetThroughputLimit();
#if PIA_TEST_CODE
            const uint32_t THROUGHPUT_LIMIT_MAX = 120 * 1024;
#else
            const uint32_t THROUGHPUT_LIMIT_MAX = 10 * 1024;
#endif
            if (keyCountL == 0 || keyCountL > 15)
            {
                if (limit > 0)
                {
                    if (limit == nn::pia::transport::ReliableBroadcastProtocol::ThroughputLimitMax)
                    {
                        limit = THROUGHPUT_LIMIT_MAX;
                    }
                    else if (limit % unit > 0)
                    {
                        limit -= (limit % unit);
                    }
                    else
                    {
                        limit -= unit;
                    }
                }
            }
            else if (keyCountR == 0 || keyCountR > 15)
            {
                if (limit < THROUGHPUT_LIMIT_MAX)
                {
                    limit = limit - (limit % unit) + unit;
                    if (limit > THROUGHPUT_LIMIT_MAX)
                    {
                        limit = THROUGHPUT_LIMIT_MAX;
                    }
                }
                else if (limit == THROUGHPUT_LIMIT_MAX)
                {
                    limit = nn::pia::transport::ReliableBroadcastProtocol::ThroughputLimitMax;
                }
            }
#if PIA_TEST_CODE
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL) && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                limit = nn::pia::transport::ReliableBroadcastProtocol::ThroughputLimitMax;
            }
#endif
            pProtocol->SetThroughputLimit(limit);
        }
        break;

    case OperationTarget_ConfigSendToNewcomer:
    case OperationTarget_SynchronizeCompletion:
    case OperationTarget_SendEndless:
    case OperationTarget_ConfigFailedIfRejected:
    case OperationTarget_MediateRequest:
        // Left / Right で設定の yes / no を切り替えます。
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft) || pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                switch (s_OperationTarget)
                {
                case OperationTarget_ConfigSendToNewcomer:
                    s_SendConfig.SetSendToNewcomer(!s_SendConfig.IsSendToNewcomer());
                    break;
                case OperationTarget_SynchronizeCompletion:
                    s_SendConfig.SetSynchronizeCompletion(!s_SendConfig.IsSynchronizeCompletion());
                    break;
                case OperationTarget_SendEndless:
                    s_SendConfig.SetSendEndless(!s_SendConfig.IsSendEndless());
                    break;
                case OperationTarget_ConfigFailedIfRejected:
                    s_SendConfig.SetFailIfRejected(!s_SendConfig.IsFailIfRejected());
                    break;
                case OperationTarget_MediateRequest:
                    s_SendConfig.SetMediateRequest(!s_SendConfig.IsMediateRequest());
                    break;
                default:
                    break;
                }
            }
        }
        break;

    case OperationTarget_StartReceive:
    case OperationTarget_RejectRequest:
    case OperationTarget_StartSend:
    case OperationTarget_Cancel:
    case OperationTarget_FinishSendEndless:
    case OperationTarget_Reset:
        // A で処理を実行します。
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                switch (s_OperationTarget)
                {
                case OperationTarget_StartReceive:
                    StartReceive();
                    break;
                case OperationTarget_RejectRequest:
                    RejectRequest();
                    break;
                case OperationTarget_StartSend:
                    StartSend();
                    break;
                case OperationTarget_Cancel:
                    Cancel();
                    break;
                case OperationTarget_FinishSendEndless:
                    FinishSendEndless();
                    break;
                case OperationTarget_Reset:
                    Reset();
                    break;
                default:
                    break;
                }
            }
        }
        break;
    default:
        break;
    }

    return false;
}


// 送受信の完了をチェックします。
void CheckCompletion()
{
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();
    switch (pProtocol->GetState())
    {
    case nn::pia::transport::ReliableBroadcastProtocol::State_ReceiveSuccess:
        if (s_IsCompletionChecked == false)
        {
            const nn::pia::transport::ReliableBroadcastProtocol::TransferSetting* cpSetting = pProtocol->GetCurrentTransferSetting();
            PIA_SAMPLE_ASSERT(cpSetting != NULL);

            // 送られてきたデータの内容があっていることをチェック
            bool isValidData = CheckReceivedData(cpSetting->m_UserData, cpSetting->m_DataSize);
            NN_PIA_UNUSED_IN_RELEASE(isValidData);
            nn::pia::StationId srcId = cpSetting->GetSourceStationId();
            NN_PIA_UNUSED_IN_RELEASE(srcId);
            PIA_SAMPLE_ASSERT(srcId != nn::pia::StationIdInvalid);
            PIA_SAMPLE_ASSERTMSG(isValidData, "received a wrong data (source:0x%16llx offset:%d, size:%d)", srcId, cpSetting->m_UserData, cpSetting->m_DataSize);

            nn::pia::common::Time finishTime;
            finishTime.SetNow();

#if ENABLE_PROFILING
            s_TimeElapsed = (finishTime - s_StartTime).GetMSec();
#endif
            SetMessage("receive success (%.1fs)", (finishTime - s_StartTime).GetSecF32());
            s_IsCompletionChecked = true;
        }
        break;

    case nn::pia::transport::ReliableBroadcastProtocol::State_ReceiveFailure:
        if (s_IsCompletionChecked == false)
        {
            SetMessage("receive failure");
            s_IsCompletionChecked = true;
        }
        break;

    case nn::pia::transport::ReliableBroadcastProtocol::State_SendSuccess:
        if (s_IsCompletionChecked == false)
        {
            nn::pia::common::Time finishTime;
            finishTime.SetNow();

#if ENABLE_PROFILING
            s_TimeElapsed = (finishTime - s_StartTime).GetMSec();
#endif
            SetMessage("send success (%.1fs)", (finishTime - s_StartTime).GetSecF32());
            s_IsCompletionChecked = true;
        }
        break;

    case nn::pia::transport::ReliableBroadcastProtocol::State_SendFailure:
        if (s_IsCompletionChecked == false)
        {
            SetMessage("send failure");
            s_IsCompletionChecked = true;
        }
        break;
    default:
        break;
    }
}


#define PRINT(x, ...)                                      \
    {                                                      \
        pSampleFramework->DrawString((x), y, __VA_ARGS__); \
    }
#define NEXT_LINE()         \
    {                       \
        y += cLineFeedSize; \
    }
#define NEXT_LINE_SUB(group)    \
    {                           \
        y += s_LineFeed[group]; \
    }

#define PRINT_COLOR(color, x, ...)                         \
    \
{                                                   \
        pSampleFramework->SetFontColor(color);             \
        pSampleFramework->DrawString((x), y, __VA_ARGS__); \
        pSampleFramework->SetFontColor(DefaultColor);      \
    \
}

#define PRINT_CURSOR(target, x, ...)                      \
    \
{                                                  \
        if (s_OperationTarget == (target))                \
        {                                                 \
            PRINT_COLOR(CursorColor, (x), __VA_ARGS__);   \
        }                                                 \
        else if (IsValidOperationTarget(target) == false) \
        {                                                 \
            PRINT_COLOR(InvalidColor, (x), __VA_ARGS__);  \
        }                                                 \
        else                                              \
        {                                                 \
            PRINT((x), __VA_ARGS__);                      \
        }                                                 \
    \
}


// 画面の描画処理です。
void Draw(nns::pia::GameMode gameMode)
{
    // 改行幅の更新
    {
        OperationGroup group = GetOperationGroup(s_OperationTarget);
        for (int32_t i = 0; i < OperationGroup_Max; ++i)
        {
            if (i == group)
            {
                if (s_LineFeed[i] < cLineFeedSize)
                {
                    ++s_LineFeed[i];
                }
            }
            else
            {
                if (s_LineFeed[i] > 0)
                {
                    --s_LineFeed[i];
                }
            }
        }
    }

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();

    static const nns::pia::sampleframework::Color DefaultColor(1.0f, 1.0f, 1.0f, 1.0f);
    static const nns::pia::sampleframework::Color CursorColor(1.0f, 0.3f, 0.3f, 1.0f);
    static const nns::pia::sampleframework::Color InvalidColor(0.4f, 0.4f, 0.4f, 1.0f);
    static const nns::pia::sampleframework::Color ProgressDoneColor(0.8f, 0.8f, 0.2f, 1.0f);
    static const nns::pia::sampleframework::Color ProgressRemainColor(0.6f, 0.6f, 0.6f, 1.0f);

    pSampleFramework->SetFontColor(DefaultColor);

    int32_t y = 4;

    // 共通の表示
    {
        PRINT(4, "PiaTransport ReliableBroadcast");
        uint32_t idx = (pSampleFramework->GetFrameCount() / 4) % 4;
        const char AnimLetter[] = { '-', '\\', '|', '/' };
        PRINT(380, "%c", AnimLetter[idx]);
        NEXT_LINE();

        const char* GameModeString = NULL;
        switch (gameMode)
        {
#if NNS_PIA_ENABLE_LOCAL
        case nns::pia::GameMode_LocalHost:
        case nns::pia::GameMode_LocalClient:
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                GameModeString = "LocalHost";
            }
            else
            {
                GameModeString = "LocalClient";
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
        case nns::pia::GameMode_InetHost:
        case nns::pia::GameMode_InetClient:
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                GameModeString = "InetHost";
            }
            else
            {
                GameModeString = "InetClient";
            }
            break;
        case nns::pia::GameMode_InetRandom:
            PIA_SAMPLE_ASSERTMSG(false, "InetRandom?");
            GameModeString = "InetRandom";
            break;
#endif
        case nns::pia::GameMode_LanHost:
        case nns::pia::GameMode_LanClient:
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                GameModeString = "LanHost";
            }
            else
            {
                GameModeString = "LanClient";
            }
            break;
        case nns::pia::GameMode_LanRandom:
            PIA_SAMPLE_ASSERTMSG(false, "LanRandom?");
            GameModeString = "LanRandom";
            break;
        default:
            PIA_SAMPLE_ASSERTMSG(false, "Unknown game mode? (%d)", gameMode);
            GameModeString = "???";
            break;
        }
        PIA_SAMPLE_ASSERT(GameModeString != NULL);
        PRINT(4, "GameMode:%s", GameModeString);
        NEXT_LINE();

        PRINT(4, "StationNum:%d", pSession->GetStationNum());
        PRINT(180, "Local:0x%02x  Host:0x%02x", conv2StationIndex(pSession->GetLocalStationId()) & 0xff, conv2StationIndex(pSession->GetHostStationId()) & 0xff);
        NEXT_LINE();
    }

    y += 3;

    // ステート
    {
        const char* cpStateName = "";
        switch (pProtocol->GetState())
        {
        case nn::pia::transport::ReliableBroadcastProtocol::State_Invalid:
            cpStateName = "Invalid";
            break;
        case nn::pia::transport::ReliableBroadcastProtocol::State_Wait:
            cpStateName = "Wait";
            break;
        case nn::pia::transport::ReliableBroadcastProtocol::State_Sending:
            cpStateName = "Sending";
            break;
        case nn::pia::transport::ReliableBroadcastProtocol::State_SendSuccess:
            cpStateName = "Send Success";
            break;
        case nn::pia::transport::ReliableBroadcastProtocol::State_SendFailure:
            cpStateName = "Send Failure";
            break;
        case nn::pia::transport::ReliableBroadcastProtocol::State_Receiving:
            cpStateName = "Receiving";
            break;
        case nn::pia::transport::ReliableBroadcastProtocol::State_ReceiveSuccess:
            cpStateName = "Receive Success";
            break;
        case nn::pia::transport::ReliableBroadcastProtocol::State_ReceiveFailure:
            cpStateName = "Receive Failure";
            break;
        default:
            break;
        }
        PRINT(4, "State: %s", cpStateName);
        NEXT_LINE();
    }

    y += 3;

    // 受信設定
    if (s_LineFeed[OperationGroup_Receive] > 0)
    {
        PRINT(4, "-Receive");
        NEXT_LINE();

        PRINT(20, "request:");
        if (s_ReceiveStationId == nn::pia::StationIdInvalid)
        {
            PRINT_CURSOR(OperationTarget_ReceiveStationId, 220, "top priority");
        }
        else
        {
            PRINT_CURSOR(OperationTarget_ReceiveStationId, 220, "station 0x%02x", conv2StationIndex(s_ReceiveStationId) & 0xff);
            if (pSession->IsValidStationId(s_ReceiveStationId) == false)
            {
                PRINT(300, "disconnected");
            }
        }
        NEXT_LINE_SUB(OperationGroup_Receive);

        const nn::pia::transport::ReliableBroadcastProtocol::TransferSetting* cpRequest = (s_ReceiveStationId == nn::pia::StationIdInvalid) ? pProtocol->GetRequest() : pProtocol->GetRequest(s_ReceiveStationId);

        if (cpRequest != NULL)
        {
            PRINT(20, "source stationId:");
            nn::pia::StationId srcId = cpRequest->GetSourceStationId();
            PRINT(220, "0x%02x", conv2StationIndex(srcId) & 0xff);
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "data size:");
            PRINT(220, "%7dB (%4.2fMB)", cpRequest->m_DataSize, static_cast<float>(cpRequest->m_DataSize) / (1024.0f * 1024.0f));
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "offset:");
            PRINT(220, "%7dB", cpRequest->m_UserData);
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "send to newcomer:");
            PRINT(220, cpRequest->m_Configuration.IsSendToNewcomer() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "synchronize completion:");
            PRINT(220, cpRequest->m_Configuration.IsSynchronizeCompletion() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "send endless:");
            PRINT(220, cpRequest->m_Configuration.IsSendEndless() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "fail if rejected:");
            PRINT(220, cpRequest->m_Configuration.IsFailIfRejected() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "mediate request:");
            PRINT(220, cpRequest->m_Configuration.IsMediateRequest() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Receive);
        }
        else
        {
            PRINT(20, "source stationId:");
            PRINT(220, "-");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "data size:");
            PRINT(220, " ----- B ( -- MB)");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "offset:");
            PRINT(220, " ----- B");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "send to newcomer:");
            PRINT(220, "-");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "synchronize completion:");
            PRINT(220, "-");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "send endless:");
            PRINT(220, "-");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "fail if rejected:");
            PRINT(220, "-");
            NEXT_LINE_SUB(OperationGroup_Receive);

            PRINT(20, "mediate request:");
            PRINT(220, "-");
            NEXT_LINE_SUB(OperationGroup_Receive);
        }

        PRINT(20, "receive buffer size:");
        PRINT_CURSOR(OperationTarget_ReceiveBufferSize, 220, "%7dB (%4.2fMB)", s_ReceiveBufferSize, static_cast<float>(s_ReceiveBufferSize) / (1024.0f * 1024.0f));
        NEXT_LINE_SUB(OperationGroup_Receive);

        PRINT_CURSOR(OperationTarget_StartReceive, 20, "> Start receive");
        NEXT_LINE_SUB(OperationGroup_Receive);

        PRINT_CURSOR(OperationTarget_RejectRequest, 20, "> Reject request");
    }
    else
    {
        PRINT_COLOR(InvalidColor, 4, "+Receive");
        NEXT_LINE();

        PRINT_COLOR(InvalidColor, 20, "...");
    }
    NEXT_LINE();

    // 送信設定
    if (s_LineFeed[OperationGroup_Send] > 0)
    {
        PRINT(4, "-Send");
        NEXT_LINE();

        PRINT(20, "data size:");
        PRINT_CURSOR(OperationTarget_SendSize, 220, "%7dB (%4.2fMB)", s_SendDataSize, static_cast<float>(s_SendDataSize) / (1024.0f * 1024.0f));
        NEXT_LINE_SUB(OperationGroup_Send);

        PRINT(20, "send to newcomer:");
        PRINT_CURSOR(OperationTarget_ConfigSendToNewcomer, 220, s_SendConfig.IsSendToNewcomer() ? "yes" : "no");
        NEXT_LINE_SUB(OperationGroup_Send);

        PRINT(20, "synchronize completion:");
        PRINT_CURSOR(OperationTarget_SynchronizeCompletion, 220, s_SendConfig.IsSynchronizeCompletion() ? "yes" : "no");
        NEXT_LINE_SUB(OperationGroup_Send);

        PRINT(20, "send endless:");
        PRINT_CURSOR(OperationTarget_SendEndless, 220, s_SendConfig.IsSendEndless() ? "yes" : "no");
        NEXT_LINE_SUB(OperationGroup_Send);

        PRINT(20, "fail if rejected:");
        PRINT_CURSOR(OperationTarget_ConfigFailedIfRejected, 220, s_SendConfig.IsFailIfRejected() ? "yes" : "no");
        NEXT_LINE_SUB(OperationGroup_Send);

        PRINT(20, "mediate request:");
        PRINT_CURSOR(OperationTarget_MediateRequest, 220, s_SendConfig.IsMediateRequest() ? "yes" : "no");
        NEXT_LINE_SUB(OperationGroup_Send);

        PRINT_CURSOR(OperationTarget_StartSend, 20, "> Start send");
    }
    else
    {
        PRINT_COLOR(InvalidColor, 4, "+Send");
        NEXT_LINE();

        PRINT_COLOR(InvalidColor, 20, "...");
    }
    NEXT_LINE();

    // 進捗表示
    if (s_LineFeed[OperationGroup_Progress] > 0)
    {
        PRINT(4, "-Progress");
        NEXT_LINE();

        const nn::pia::transport::ReliableBroadcastProtocol::TransferSetting* cpTransferSetting = pProtocol->GetCurrentTransferSetting();
        if (cpTransferSetting != NULL)
        {
            PRINT(20, "source stationId:");
            nn::pia::StationId srcId = cpTransferSetting->GetSourceStationId();
            PRINT(220, "0x%02x", conv2StationIndex(srcId) & 0xff);
            NEXT_LINE_SUB(OperationGroup_Progress);

            {
                uint32_t total = cpTransferSetting->m_DataSize;
                uint32_t current = static_cast<uint32_t>(pProtocol->GetProgress());
                float rate = static_cast<float>(current) / static_cast<float>(total);
                int32_t progressMax = 20;
                int32_t progress = static_cast<int32_t>(rate * progressMax);
                int32_t w = pSampleFramework->GetFontWidth();
                pSampleFramework->SetFontColor(ProgressDoneColor);
                for (int32_t i = 0; i < progress; i++)
                {
                    pSampleFramework->DrawString(20 + w * i, y, "+");
                }
                pSampleFramework->SetFontColor(ProgressRemainColor);
                for (int32_t i = progress; i < progressMax; i++)
                {
                    pSampleFramework->DrawString(20 + w * i, y, "-");
                }
                pSampleFramework->SetFontColor(DefaultColor);
                NEXT_LINE_SUB(OperationGroup_Progress);

                PRINT(20, "%7dB / %7dB (%5.1f%%)", current, total, rate * 100.0f);
                NEXT_LINE_SUB(OperationGroup_Progress);
            }

            switch (pProtocol->GetState())
            {
            case nn::pia::transport::ReliableBroadcastProtocol::State_Sending:
            case nn::pia::transport::ReliableBroadcastProtocol::State_SendSuccess:
            case nn::pia::transport::ReliableBroadcastProtocol::State_SendFailure:
                for (uint32_t i = 0; i < MaxStations; ++i)
                {
                    nn::pia::StationId stationId = conv2StationId(static_cast<nn::pia::StationIndex>(i));
                    if (stationId == pSession->GetLocalStationId())
                    {
                        continue;
                    }

                    const nns::pia::sampleframework::Color& color = pProtocol->IsSendTarget(stationId) ? DefaultColor : InvalidColor;

                    PRINT_COLOR(color, 30, "%02x:", conv2StationIndex(stationId) & 0xff);
                    if (pSession->IsValidStationId(stationId))
                    {
                        uint32_t current = static_cast<uint32_t>(pProtocol->GetSendProgress(stationId));
                        float rate = static_cast<float>(current) / static_cast<float>(cpTransferSetting->m_DataSize);
                        int32_t progressMax = 20;
                        int32_t progress = static_cast<int32_t>(rate * progressMax);
                        int32_t w = pSampleFramework->GetFontWidth();
                        pSampleFramework->SetFontColor(ProgressDoneColor);
                        for (int32_t j = 0; j < progress; j++)
                        {
                            pSampleFramework->DrawString(50 + w * j, y, "+");
                        }
                        pSampleFramework->SetFontColor(ProgressRemainColor);
                        for (int32_t j = progress; j < progressMax; j++)
                        {
                            pSampleFramework->DrawString(50 + w * j, y, "-");
                        }
                        pSampleFramework->SetFontColor(DefaultColor);

                        PRINT_COLOR(color, 260, "%7dB (%5.1f%%)", current, rate * 100.0f);
                    }
                    else
                    {
                        PRINT_COLOR(color, 50, "disconnected");
                    }
                    NEXT_LINE_SUB(OperationGroup_Progress);
                }
                break;
            default:
                break;
            }

            PRINT(20, "offset:");
            PRINT(220, "%7dB", cpTransferSetting->m_UserData);
            NEXT_LINE_SUB(OperationGroup_Progress);

            PRINT(20, "send to newcomer:");
            PRINT(220, cpTransferSetting->m_Configuration.IsSendToNewcomer() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Progress);

            PRINT(20, "synchronize completion:");
            PRINT(220, cpTransferSetting->m_Configuration.IsSynchronizeCompletion() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Progress);

            PRINT(20, "send endless:");
            PRINT(220, cpTransferSetting->m_Configuration.IsSendEndless() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Progress);

            PRINT(20, "fail if rejected:");
            PRINT(220, cpTransferSetting->m_Configuration.IsFailIfRejected() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Progress);

            PRINT(20, "mediate request:");
            PRINT(220, cpTransferSetting->m_Configuration.IsMediateRequest() ? "yes" : "no");
            NEXT_LINE_SUB(OperationGroup_Progress);
        }
        else if (pProtocol->GetState() == nn::pia::transport::ReliableBroadcastProtocol::State_Receiving ||
                 pProtocol->GetState() == nn::pia::transport::ReliableBroadcastProtocol::State_ReceiveFailure)
        {

            PRINT(20, "source stationId:");
            PRINT(220, "0x%02x", conv2StationIndex(s_ReceiveStationId) & 0xff);
            NEXT_LINE_SUB(OperationGroup_Progress);
        }

        PRINT(20, "throughput limit:");
        PRINT_CURSOR(OperationTarget_ThroughputLimit, 220, "%6dB", pProtocol->GetThroughputLimit());
        NEXT_LINE_SUB(OperationGroup_Progress);

        if (pProtocol->IsRunning())
        {
            PRINT_CURSOR(OperationTarget_Cancel, 20, "> Cancel");
        }
        else
        {
            PRINT_CURSOR(OperationTarget_Reset, 20, "> Reset");
        }

        if (pProtocol->GetState() == nn::pia::transport::ReliableBroadcastProtocol::State_Sending)
        {
            NEXT_LINE_SUB(OperationGroup_Progress);
            PRINT_CURSOR(OperationTarget_FinishSendEndless, 20, "> Finish send endless");
        }
    }
    else
    {
        PRINT_COLOR(InvalidColor, 4, "+Progress");
        NEXT_LINE();

        PRINT_COLOR(InvalidColor, 20, "...");
    }

    // メッセージ
    {
        y = 226;
        PRINT(4, "Message: %s", s_Message);
    }
}


#if PIA_TEST_CODE
void UpdatePiaTestData(nns::pia::GameMode gameMode, bool isExit)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    nn::pia::transport::ReliableBroadcastProtocol* pProtocol = GetReliableBroadcastProtocol();

    uint32_t sceneNo = 0;
    if (!isExit)
    {
        switch (gameMode)
        {
#if NNS_PIA_ENABLE_INET
        case nns::pia::GameMode_InetHost:
        case nns::pia::GameMode_InetClient:
        case nns::pia::GameMode_InetRandom:
            sceneNo = 4;
            break;
#endif
#if NNS_PIA_ENABLE_LOCAL
        case nns::pia::GameMode_LocalHost:
        case nns::pia::GameMode_LocalClient:
            sceneNo = 3;
            break;
#endif
        case nns::pia::GameMode_LanHost:
        case nns::pia::GameMode_LanClient:
        case nns::pia::GameMode_LanRandom:
            sceneNo = 5;
            break;
        default:
            PIA_SAMPLE_ASSERTMSG(false, "Unknown gamemode? (%d)", gameMode);
            break;
        }
    }
    pSampleFramework->SetProgramStatus(
        sceneNo,
        pSession->GetStatus() == nn::pia::session::Session::Status_ConnectedSession,
        pSession->GetLocalStationId() == pSession->GetHostStationId(),
        pSession->GetStationNum());

    // Session
    pSampleFramework->SetProgramParam(0x00, conv2StationIndex(pSession->GetLocalStationId()));

    // 操作
    pSampleFramework->SetProgramParam(0x08, s_OperationTarget);

    // 状態
    pSampleFramework->SetProgramParam(0x10, pProtocol->GetState());
    pSampleFramework->SetProgramParam(0x11, pProtocol->IsRunning() ? 1 : 0);

    // 操作可能か
    for (int i = 0; i < OperationTarget_Max; ++i)
    {
        pSampleFramework->SetProgramParam(0x20 + i, IsValidOperationTarget(static_cast<OperationTarget>(i)) ? 1 : 0);
    }

    // 受信設定
    pSampleFramework->SetProgramParam(0x30, conv2StationIndex(s_ReceiveStationId)); // OperationTarget_ReceiveStationId
    pSampleFramework->SetProgramParam(0x31, s_ReceiveBufferSize); // OperationTarget_ReceiveBufferSize

    // 送信設定
    pSampleFramework->SetProgramParam(0x40, s_SendDataSize); // OperationTarget_SendSize
    pSampleFramework->SetProgramParam(0x41, s_SendConfig.IsSendToNewcomer()); // OperationTarget_ConfigSendToNewcomer
    pSampleFramework->SetProgramParam(0x42, s_SendConfig.IsSynchronizeCompletion()); // OperationTarget_SynchronizeCompletion:
    pSampleFramework->SetProgramParam(0x43, s_SendConfig.IsSendEndless()); // OperationTarget_SendEndless:
    pSampleFramework->SetProgramParam(0x44, s_SendConfig.IsFailIfRejected()); // OperationTarget_ConfigFailedIfRejected:
    pSampleFramework->SetProgramParam(0x45, s_SendConfig.IsMediateRequest()); // OperationTarget_MediateRequest:

    // 進捗
    pSampleFramework->SetProgramParam(0x50, pProtocol->GetThroughputLimit()); // OperationTarget_ThroughputLimit
    pSampleFramework->SetProgramParam(0x51, pProtocol->GetProgress());
#if ENABLE_PROFILING
    pSampleFramework->SetProgramParam(0x52, s_TimeElapsed);
#endif

    // 相手ステーションごとの状態
    for (uint32_t i = 0; i < MaxStations; ++i)
    {
        nn::pia::StationId sid = conv2StationId(static_cast<nn::pia::StationIndex>(i));
        uint32_t idOfs = 0x60 + 0x10 * i;

        if (pSession->IsValidStationId(sid) && sid != pSession->GetLocalStationId())
        {
            pSampleFramework->SetProgramParam(idOfs + 0x0, 1);

            pSampleFramework->SetProgramParam(idOfs + 0x4, pProtocol->GetSendProgress(sid));

            const nn::pia::transport::ReliableBroadcastProtocol::TransferSetting* cpRequest = pProtocol->GetRequest(sid);
            if (cpRequest != NULL)
            {
                pSampleFramework->SetProgramParam(idOfs + 0x8, 1);
                pSampleFramework->SetProgramParam(idOfs + 0x9, cpRequest->m_DataSize);
                pSampleFramework->SetProgramParam(idOfs + 0xa, cpRequest->m_Configuration.IsSendToNewcomer());
                pSampleFramework->SetProgramParam(idOfs + 0xb, cpRequest->m_Configuration.IsSynchronizeCompletion());
                pSampleFramework->SetProgramParam(idOfs + 0xc, cpRequest->m_Configuration.IsSendEndless());
                pSampleFramework->SetProgramParam(idOfs + 0xd, cpRequest->m_Configuration.IsFailIfRejected());
                pSampleFramework->SetProgramParam(idOfs + 0xe, cpRequest->m_Configuration.IsMediateRequest());
                pSampleFramework->SetProgramParam(idOfs + 0xf, cpRequest->m_UserData);
            }
            else
            {
                for (uint32_t j = 0x8; j <= 0xf; ++j)
                {
                    pSampleFramework->SetProgramParam(idOfs + j, 0);
                }
            }
        }
        else
        {
            for (uint32_t j = 0; j < 16; ++j)
            {
                pSampleFramework->SetProgramParam(idOfs + j, 0);
            }
        }
    }
}
#endif

// 通信中のメインループです。
void MainLoop(nns::pia::GameMode gameMode)
{
    while (true)
    {
        bool isExit = false;
        bool bCallLeaveSession = false;

        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if NNS_PIA_ENABLE_INET
        // nns::pia::GameMode_InetXXX の場合は nn::nex::Scheduler が作られていて、そのディスパッチも行う必要があります。
        nns::pia::Nex_SchedulerDispatch();
#endif

        // nn::pia::common::Scheduler::Dispatch() を定期的に呼び出す必要があります。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        // Session 接続状態のチェック
        {
            nn::pia::Result result = nn::pia::session::Session::GetInstance()->CheckConnectionError();

            // エラーハンドリング。
            if (result.IsFailure())
            {
                enum nn::pia::Result::HandlingType handling = result.GetHandlingType();
                switch (handling)
                {
                case nn::pia::Result::HandlingType_CleanupWithLeave:
                case nn::pia::Result::HandlingType_LogoutWithLeave:
                    PIA_SAMPLE_PRINT("Exit MainLoop() with LeaveSession().\n");
                    bCallLeaveSession = true;
                    isExit = true;
                    break;
                case nn::pia::Result::HandlingType_ShutdownNetwork:
                    PIA_SAMPLE_PRINT("Exit MainLoop().\n");
                    isExit = true;
                    break;
                default:
                    // GetHandlingType() は HandlingType_CleanupWithLeave や HandlingType_ShutdownNetwork 以外の値を
                    // 返しうる関数ですが、Session::CheckConnectionError() が返しうる Result を考慮すると、
                    // これら以外の値が返ることはありません。そのため、アサート文を用意しています。
                    PIA_SAMPLE_ASSERTMSG(false, "Unexpected result.\n");
                    break;
                }
            }
        }

        if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedSession)
        {
            // 通信中であれば送受信処理を行います。
            isExit |= Operate();

            CheckCompletion();
        }
        else
        {
            // セッションが切れたので終了します。
            isExit = true;
        }

        // プロトコルに対して送信したものをすぐに処理するために、もう一度 nn::pia::common::Scheduler::Dispatch() を呼びます。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        Draw(gameMode);

#if PIA_TEST_CODE
        UpdatePiaTestData(gameMode, isExit);
#endif

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        if (isExit)
        {
            if (bCallLeaveSession)
            {
                nns::pia::Session_LeaveSession();
            }

            return;
        }
    }
}

}  // end of anonymous namespace


// エントリポイント。
PIA_SAMPLE_MAIN_FUNC()
{
    nn::pia::Result result;
    {
        nns::pia::sampleframework::SampleFramework::Setting setting;
        setting.isUpperScreenOnly = true;
        nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(60);
    }

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(PROGRAM_TITLE "\n");
    nns::pia::Basic_SetProgramTitle(PROGRAM_TITLE);

    InitOriginalData();

    while (true)
    {
#if PIA_TEST_CODE
        s_OptionalParam.Initialize(OptionalParamOffset);
        nns::pia::GameMode gameMode = nns::pia::DecideGameMode("PiaTransport ReliableBroadcast", &s_OptionalParam);
#else
        nns::pia::GameMode gameMode = nns::pia::DecideGameMode("PiaTransport ReliableBroadcast");
#endif
        if (gameMode == nns::pia::GameMode_Exit)
        {
            break;
        }

        // Piaの各モジュールの初期化処理
        // この順番で行ってください。
        {
#if 0
            uint64_t traceFlag = 0;
#else
            uint64_t traceFlag =
                nn::pia::common::TraceFlagCommon |
                nn::pia::common::TraceFlagTransport |
                nn::pia::common::TraceFlagLocal |
                nn::pia::common::TraceFlagLan |
                nn::pia::common::TraceFlagInet |
                nn::pia::common::TraceFlagSession;
#endif
            result = nns::pia::Common_InitializePiaCommon(s_Memory, MemoryBlockSize, traceFlag, true);
            PIA_SAMPLE_ASSERT(result.IsSuccess());

            nn::pia::transport::NetworkFactory* pFactory = NULL;
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            {
#if NN_PIA_CTR
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(LocalTitleId, LocalSubId));
#elif NN_PIA_NINTENDOSDK
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(nns::pia::LocalCommunicationId, LocalSceneId));
#endif
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Local_InitializePiaLocal() failed\n");
                    nns::pia::Local_FinalizePiaLocal();
                    nns::pia::Common_FinalizePiaCommon();
                    continue;
                }
                pFactory = nns::pia::Local_GetNetworkFactory();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetRandom:
                // PiaInet を初期化するより前に NEX を初期化してください
                result = nns::pia::Nex_InitializeNex(
                    MemoryBlockSizeForNex,
                    MemoryBlockSizeForNexEmergency);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                result = nns::pia::Inet_InitializePiaInet();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Inet_GetNetworkFactory();
                break;
#endif
            case nns::pia::GameMode_LanRandom:
                result = nns::pia::Lan_InitializePiaLan();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                pFactory = nns::pia::Lan_GetNetworkFactory();
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            // Factoryを利用して、Transportをセットアップします。
            nn::pia::transport::Transport::Setting transportSetting;
            transportSetting.pFactory = pFactory;
            transportSetting.maxStationNum = MaxStations;
            transportSetting.maxSendThreadBufferNum = nns::pia::MaxSendThreadBufferNum;
            transportSetting.maxReceiveThreadBufferNum = nns::pia::MaxReceiveThreadBufferNum;
#if PIA_TEST_CODE
            transportSetting.analysisInterval = 5;
#endif

            result = nns::pia::Transport_InitializePiaTransport(
                transportSetting,
                0,
                NULL,
                0,
                NULL,
                64,
                64,
                CreateAndInitializeProtocol,
                32);
            PIA_SAMPLE_ASSERT(result.IsSuccess());

            // セッションのセットアップを行います。
            nn::pia::session::Session::Setting sessionSetting;
            sessionSetting.pNetworkFactory = pFactory;
            sessionSetting.browsedSessionInfoListNum = 8;
            sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
#if PIA_TEST_CODE
            if (s_OptionalParam.bRelayEmulation)
            {
                // リレーエミュレーションを行う場合は、Session のインスタンス作成前に設定関数を呼び出す必要があります。
                nn::pia::session::Session::DebugSetting debugSetting;
                debugSetting.isRelayEmulationEnabled = true;
                nn::pia::Result ret = nn::pia::session::Session::SetDebugSetting(debugSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(ret);
                sessionSetting.networkTopology = nn::pia::session::NetworkTopology_RelayMesh;
            }
            else
            {
                nn::pia::session::Session::DebugSetting debugSetting;
                debugSetting.isRelayEmulationEnabled = false;
                nn::pia::Result ret = nn::pia::session::Session::SetDebugSetting(debugSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(ret);
            }
#endif
            result = nns::pia::Session_InitializePiaSession(sessionSetting);
            PIA_SAMPLE_ASSERT(result.IsSuccess());
        }


        // ネットワークの接続、セッションの確立の処理
        {
            nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SessionEventCallback);
            InitializeGameData();

            // ネットワークの初期化
            nns::pia::NetworkType networkType = nns::pia::NetworkType_Internet;
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            {
                networkType = nns::pia::NetworkType_Local;
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetRandom:
            {
                networkType = nns::pia::NetworkType_Internet;
            }
            break;
#endif
            case nns::pia::GameMode_LanRandom:
            {
                networkType = nns::pia::NetworkType_Lan;
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(networkType);
            if (netResult != nns::pia::NetResult_Success)
            {
                PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork:failed. (netResult = %d)\n", netResult);

                switch (gameMode)
                {
#if NNS_PIA_ENABLE_LOCAL
                case nns::pia::GameMode_LocalHost:
                case nns::pia::GameMode_LocalClient:
                {
                    gameMode = nns::pia::GameMode_LocalError;
                }
                break;
#endif
#if NNS_PIA_ENABLE_INET
                case nns::pia::GameMode_InetRandom:
                {
                    gameMode = nns::pia::GameMode_InetError;
                }
                break;
#endif
                case nns::pia::GameMode_LanRandom:
                {
                    gameMode = nns::pia::GameMode_LanError;
                }
                break;
                default:
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                break;
                }
            }

            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

// セッションを構築します。
// 作成するマッチメイクセッションの設定
#if NN_PIA_CTR
                nn::pia::local::UdsCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    createSessionSetting.SetSubId(LocalSubId);
                    createSessionSetting.SetChannel(LocalChannel);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    createSessionSetting.SetMaxParticipantNum(MaxStations);
                }
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    createSessionSetting.SetSceneId(LocalSceneId);
                    createSessionSetting.SetChannel(LocalChannel);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    createSessionSetting.SetMaxParticipantNum(MaxStations);
                }
#endif
                result = nns::pia::Session_CreateSession(&createSessionSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_CreateSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }
            }
            break;

            case nns::pia::GameMode_LocalClient:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }
                // セッションを検索します。
                // 検索条件の設定
                const int SearchSettingNum = 1;
#if NN_PIA_CTR
                nn::pia::local::UdsSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    searchCriteriaList[0].SetSubId(LocalSubId);
                    searchCriteriaList[0].SetMaxParticipantNum(MaxStations);
                }
#elif NN_PIA_NINTENDOSDK
                nn::pia::local::LdnSessionSearchCriteria searchCriteriaList[SearchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    searchCriteriaList[0].SetSceneId(LocalSceneId);
                    searchCriteriaList[0].SetMaxParticipantNum(MaxStations);
                }
#endif
                searchCriteriaList[0].SetResultRange(0, 8);
                result = nns::pia::Session_BrowseSession(&searchCriteriaList[0]);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_BrowseSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }

                // 検索したセッションに参加します。
                uint32_t foundSessionNum = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize();
                if (foundSessionNum > 0)
                {
                    PIA_SAMPLE_PRINT("session found\n");
                    nn::pia::session::ISessionInfoList::Iterator it = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->Begin();
                    nn::pia::session::ISessionInfo* pTargetSessionInfo = *it;
                    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTargetSessionInfo));

#if NN_PIA_CTR
                    nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#elif NN_PIA_NINTENDOSDK
                    nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
#endif
                    joinSessionSetting.SetSessionInfoPtr(pTargetSessionInfo);
                    joinSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    result = nns::pia::Session_JoinSession(&joinSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("nns::pia::Session_JoinSession:failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                        gameMode = nns::pia::GameMode_LocalError;
                        break;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("no session found\n");
                    gameMode = nns::pia::GameMode_LocalError;
                    break;
                }
            }
            break;

            case nns::pia::GameMode_LocalError:
            {
            }
            break;
#endif  // NNS_PIA_ENABLE_LOCAL
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetRandom:
            {
                // ゲームサーバにログイン
                if (!nns::pia::Inet_Login())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Inet_Login:failed\n");
                    gameMode = nns::pia::GameMode_InetError;
                    break;
                }
                // Session クラスのスタートアップ処理。
                nn::pia::inet::NexSessionStartupSetting startupSetting;
                startupSetting.isHostMigrationEnabled = static_cast<bool>(ENABLE_HOST_MIGRATION);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_InetError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::inet::NexCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const int SearchSettingNum = 1;
                nn::pia::inet::NexSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::inet::NexJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_InetError;
                    break;
                }

#if PIA_TEST_CODE
                if (s_OptionalParam.bRelayEmulation)
                {
                    nns::pia::SetRelayEmulation(nn::pia::StationIndex_2, nn::pia::StationIndex_3);
                }
#endif

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = nns::pia::GameMode_InetHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = nns::pia::GameMode_InetClient;
                }
            }
            break;

            case nns::pia::GameMode_InetError:
            {
            }
            break;
#endif
            case nns::pia::GameMode_LanRandom:
            {
                result = nns::pia::Lan_StartupPiaLan();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Lan_StartupPiaLan failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LanError;
                    break;
                }
                // Session クラスのスタートアップ処理。
                nn::pia::lan::LanSessionStartupSetting startupSetting;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LanError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::lan::LanCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(MaxStations);
                    setting.SetOpenSession(true);
                }

                const int SearchSettingNum = 1;
                nn::pia::lan::LanSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(MaxStations);
                }

                nn::pia::lan::LanJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = nns::pia::GameMode_LanError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = nns::pia::GameMode_LanHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = nns::pia::GameMode_LanClient;
                }
            }
            break;

            case nns::pia::GameMode_LanError:
            {
            }
            break;

            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }
        }

        // メインの処理
        switch (gameMode)
        {
#if NNS_PIA_ENABLE_LOCAL
        case nns::pia::GameMode_LocalHost:
        case nns::pia::GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
        case nns::pia::GameMode_InetHost:
        case nns::pia::GameMode_InetClient:
#endif
        case nns::pia::GameMode_LanHost:
        case nns::pia::GameMode_LanClient:
            MainLoop(gameMode);

            // Watermark機能による計測結果プリント。
            if (PIA_IS_VALID_POINTER(nn::pia::common::WatermarkManager::GetInstance()))
            {
                nn::pia::common::WatermarkManager::GetInstance()->Trace(nn::pia::common::TraceFlagAlways);
            }

            break;
        default:
            break;
        }


        // セッションの終了、ネットワークからの切断の処理
        {
            // Session 終了処理。
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetHost:
            case nns::pia::GameMode_InetClient:
#endif
            case nns::pia::GameMode_LanHost:
            case nns::pia::GameMode_LanClient:
                // セッションから離脱
                nns::pia::Session_LeaveSession();
                break;
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalError:
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetError:
#endif
            case nns::pia::GameMode_LanError:
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            // Session クラスのクリーンアップ処理。
            nns::pia::Session_CleanupSession();

            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            case nns::pia::GameMode_LocalError:
                nns::pia::Local_FinalizeLocalCommunication();
                break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetHost:
            case nns::pia::GameMode_InetClient:
            case nns::pia::GameMode_InetError:
                nns::pia::Inet_Logout();
                break;
#endif
            case nns::pia::GameMode_LanHost:
            case nns::pia::GameMode_LanClient:
            case nns::pia::GameMode_LanError:
                nns::pia::Lan_CleanupPiaLan();
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            nns::pia::Net_FinalizeNetwork();
        }


        // Piaの各モジュールの終了処理
        // 初期化と逆順で行ってください。
        {
            nns::pia::Session_FinalizePiaSession();
            nns::pia::Transport_FinalizePiaTransport(0, 0, FinalizeAndDestroyProtocol);
            switch (gameMode)
            {
#if NNS_PIA_ENABLE_LOCAL
            case nns::pia::GameMode_LocalHost:
            case nns::pia::GameMode_LocalClient:
            case nns::pia::GameMode_LocalError:
                nns::pia::Local_FinalizePiaLocal();
                break;
#endif
#if NNS_PIA_ENABLE_INET
            case nns::pia::GameMode_InetHost:
            case nns::pia::GameMode_InetClient:
            case nns::pia::GameMode_InetError:
                nns::pia::Inet_FinalizePiaInet();
                nns::pia::Nex_FinalizeNex();
                break;
#endif
            case nns::pia::GameMode_LanHost:
            case nns::pia::GameMode_LanClient:
            case nns::pia::GameMode_LanError:
                nns::pia::Lan_FinalizePiaLan();
                break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }
            nns::pia::Common_FinalizePiaCommon();
        }
    }

    nns::pia::sampleframework::SampleFramework::DestroyInstance();

    PIA_SAMPLE_PRINT("end.\n");
}
