﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include <nn/pia/framework/private/framework_NetworkService.h>
#include <nn/pia/framework/framework_Framework.h>

#if NN_PIA_CAFE
#include <cafe/net/so.h>
#include <nn/ac.h>
#include <nn/act.h>
#include <nn/fp.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/nifm.h>
#include <nn/socket.h>

namespace nn
{
namespace nifm
{
/**
        * @brief 利用中の IP アドレスなどの情報を取得します。
        *
        * @param[out] pOutIpAddress       利用中の IP アドレスが格納されるバッファへのポインタ
        * @param[out] pOutSubnetMask      利用中のサブネットマスクが格納されるバッファへのポインタ
        * @param[out] pOutDefaultGateway  利用中のデフォルトゲートウェイアドレス格納されるバッファへのポインタ
        * @param[out] pOutPreferredDns    利用中の優先 DNS アドレスが格納されるバッファへのポインタ
        * @param[out] pOutAlternateDns    利用中の代替 DNS アドレスが格納されるバッファへのポインタ
        *
        * @return 処理の結果を表す nn::Result 値を返します。
        */
nn::Result GetCurrentIpConfigInfo(
    in_addr* pOutIpAddress,
    in_addr* pOutSubnetMask,
    in_addr* pOutDefaultGateway,
    in_addr* pOutPreferredDns,
    in_addr* pOutAlternateDns) NN_NOEXCEPT;
}
}
#elif NN_PIA_A
#include <sys/socket.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <net/if.h>
#elif NN_PIA_B
#include <arpa/inet.h>
#include <ifaddrs.h>
#endif

namespace
{
}  // end of anonymous namespace

namespace nn
{
namespace pia
{
namespace framework
{
// コンストラクタ。
NetworkService::NetworkService()
    : m_IsNetworkIfInitialized(false)
    , m_IsNetworkIfUp(false)
    , m_IsInitializedSocket(false)
    , m_NetworkErrorCode(0)
#if NN_PIA_CTR
    , m_ACDisconnectedEvent()
#elif NN_PIA_CAFE
    , m_ACConnected(false)
    , m_UpdateACStatusRun(false)
#elif NN_PIA_NINTENDOSDK
#endif
      
{
}

// デストラクタ。
NetworkService::~NetworkService()
{
}

Result NetworkService::InitializeNetworkInterface()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

#if NN_PIA_CTR
    uint32_t networkErrorCode = 0;

    // fs::Initializeはありますが、fs::Finalizeは呼ぶ必要はありません
    nn::fs::Initialize();

    nn::Result result = nn::ac::Initialize();
    PIA_ASSERT(result.IsSuccess());
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ac::Initialize failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        m_IsNetworkIfInitialized = false;
        PIA_ASSERT(result.GetLevel() != nn::Result::LEVEL_USAGE);

        result = nn::ac::GetLastErrorCode(&networkErrorCode);
        PIA_ASSERT(result.IsSuccess());
        PIA_TRACE_EX(common::TraceFlagFramework, "errCode:%d", networkErrorCode);

        Result::SetSdkErrorCode(networkErrorCode);
        PIA_RETURN_RESULT(ResultSdkViewerErrorCodeError);
    }
    m_IsNetworkIfInitialized = true;
    PIA_TRACE_EX(common::TraceFlagFramework, "ac::Initialize success");

    nn::ac::Config config;
    result = nn::ac::CreateDefaultConfig(&config);
    PIA_ASSERT(result.IsSuccess());
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ac::CreateDefaultConfig failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_ASSERT(result.GetLevel() != nn::Result::LEVEL_USAGE);

        result = nn::ac::GetLastErrorCode(&networkErrorCode);
        PIA_ASSERT(result.IsSuccess());
        PIA_TRACE_EX(common::TraceFlagFramework, "errCode:%d", networkErrorCode);

        Result::SetSdkErrorCode(networkErrorCode);
        PIA_RETURN_RESULT(ResultSdkViewerErrorCodeError);
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "ac::CreateDefaultConfig success");

    PIA_TRACE_EX(common::TraceFlagFramework, "ac::Connect begin");
    result = nn::ac::Connect(config);
    if (result.IsFailure())
    {
        if (result == nn::ac::ResultNotAgreeEula())
        {
            networkErrorCode = 30399;
            PIA_TRACE_EX(common::TraceFlagFramework, "ac::Connect failed (ac::ResultNotAgreeEula) errcode:%d", networkErrorCode);
        }
        else
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "ac::Connect failed");
            PIA_TRACE_RESULT_ALWAYS(result);
            PIA_ASSERT(result.GetLevel() != nn::Result::LEVEL_USAGE);

            result = nn::ac::GetLastErrorCode(&networkErrorCode);
            PIA_ASSERT(result.IsSuccess());
            PIA_TRACE_EX(common::TraceFlagFramework, "errCode:%d", networkErrorCode);
        }
        m_IsNetworkIfUp = false;
        Result::SetSdkErrorCode(networkErrorCode);
        PIA_RETURN_RESULT(ResultSdkViewerErrorCodeError);
    }
    m_IsNetworkIfUp = true;
    PIA_TRACE_EX(common::TraceFlagFramework, "ac::Connect success");

    // 切断通知を受ける Event を登録します
    m_ACDisconnectedEvent.Initialize(true);
    result = nn::ac::CTR::RegisterDisconnectEvent(&m_ACDisconnectedEvent);
    PIA_ASSERT(result.IsSuccess());
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ac::RegisterDisconnectEvent failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_ASSERT(result.GetLevel() != nn::Result::LEVEL_USAGE);

        result = nn::ac::GetLastErrorCode(&networkErrorCode);
        PIA_ASSERT(result.IsSuccess());
        PIA_TRACE_EX(common::TraceFlagFramework, "errCode:%d", networkErrorCode);

        Result::SetSdkErrorCode(networkErrorCode);
        PIA_RETURN_RESULT(ResultSdkViewerErrorCodeError);
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "ac::RegisterDisconnectEvent success");
#elif NN_PIA_CAFE
#elif NN_PIA_WIN
    // Windows ではネットワークインターフェースは OS 側で自動的に立ち上げてくれているはずなので何もしません
    m_IsNetworkIfInitialized = true;
#elif NN_PIA_NINTENDOSDK
    // NIFM ライブラリを初期化
    PIA_TRACE_EX(common::TraceFlagFramework, "nifm::Initialize begin");
    nn::Result result = nn::nifm::Initialize();
    PIA_TRACE_EX(common::TraceFlagFramework, "nifm::Initialize end");
    nn::nifm::SetLocalNetworkMode(true);
    PIA_ASSERT(result.IsSuccess());
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "nifm::Initialize failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        m_IsNetworkIfInitialized = false;
        PIA_RETURN_RESULT(ResultSdkError);
    }
    m_IsNetworkIfInitialized = true;
    PIA_TRACE_EX(common::TraceFlagFramework, "nifm::Initialize success");


    while (true)
    {
        // ネットワークインターフェース起動要求（ブロッキング API）
        PIA_TRACE_EX(common::TraceFlagFramework, "nifm::SubmitNetworkRequestAndWait begin");
        nn::nifm::SubmitNetworkRequestAndWait();
        PIA_TRACE_EX(common::TraceFlagFramework, "nifm::SubmitNetworkRequestAndWait end");
        // 起動処理の成否をチェック
        if (!nn::nifm::IsNetworkAvailable())
        {
            result = nn::nifm::HandleNetworkRequestResult();
            if (!result.IsSuccess())
            {
                if (!nn::nifm::ResultErrorHandlingCompleted::Includes(result))
                {
                    // エラーハンドリングで解消できないエラーが発生したら ResultSdkError
                    PIA_TRACE_EX(common::TraceFlagFramework, "Network is not available");
                    m_IsNetworkIfUp = false;
                    PIA_RETURN_RESULT(ResultSdkError);
                }
                else
                {
                    PIA_TRACE_RESULT_ALWAYS(result);
                }
                // エラーハンドリングに成功した場合はリトライ
                continue;
            }
        }
        break;
    }
    m_IsNetworkIfUp = true;
    PIA_TRACE_EX(common::TraceFlagFramework, "Network is available");
#elif NN_PIA_A || NN_PIA_B
    // 何もしない
    m_IsNetworkIfInitialized = true;
#endif
    return ResultSuccess();
}

// ソケットの初期化処理
Result NetworkService::InitializeSocket(const StartupNetworkSetting& setting)
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    if (m_IsInitializedSocket)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket is already initialized.");
        PIA_ASSERT(false);
        PIA_RETURN_RESULT(ResultAlreadyInitialized)
    }

#if NN_PIA_CTR
    const uint32_t RequiredMemorySize = nn::socket::GetRequiredMemorySize(setting.allocatorPoolSize, SocketSessionMax);
    PIA_TRACE_EX(common::TraceFlagFramework, "requiredMemorySize:%d, SocketBufferSize:%d, memoryPool:%d", RequiredMemorySize, setting.allocatorPoolSize, setting.memoryPoolSize);
    if (setting.memoryPoolSize < RequiredMemorySize)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPoolSize(%d) < RequiredMemorySize(%d)", setting.memoryPoolSize, RequiredMemorySize);
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (setting.memoryPool == NULL)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPool is NULL.");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (setting.memoryPoolSize % 4096 != 0 || setting.allocatorPoolSize % 4096 != 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPoolSize(%d) or allocatorPoolSize(%d) is not Aligned by 4KB.", setting.memoryPoolSize, setting.allocatorPoolSize);
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (reinterpret_cast<uintptr_t>(setting.memoryPool) % 4096 != 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPoolSize or allocatorPoolSize is not Aligned by 4KB.");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }

    NN_PIA_UNUSED_IN_RELEASE(RequiredMemorySize);

    nn::Result result = nn::socket::Initialize(reinterpret_cast<uptr>(setting.memoryPool), setting.memoryPoolSize, setting.allocatorPoolSize, SocketSessionMax);
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket::Initialize failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_ASSERT(result.GetLevel() != nn::Result::LEVEL_USAGE);
        m_IsInitializedSocket = false;
        PIA_RETURN_RESULT(ResultSdkError);
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "socket::Initialize success");
#elif NN_PIA_CAFE
#elif NN_PIA_WIN
    // Winsock を初期化します
    WORD wVersionRequested = MAKEWORD(2, 2);  // 使用するWinSockのバージョン要求
    WSADATA wsaData;                          // WinSockデータ構造体
    int nErrorStatus;

    nErrorStatus = WSAStartup(wVersionRequested, &wsaData);
    if (nErrorStatus != 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "Winsock initialization failed");
        m_IsNetworkIfInitialized = false;
        PIA_RETURN_RESULT(ResultTemporaryUnavailable);  //一時的に HandlingType_Retry を返す
    }
    else
    {
        // 要求したバージョンと同一か確認
        if (LOBYTE(wsaData.wVersion) == 2 && HIBYTE(wsaData.wVersion) == 2)
        {
            // バージョン情報の表示
            PIA_TRACE_EX(common::TraceFlagFramework, "version %d.%d is required", LOBYTE(wsaData.wVersion), HIBYTE(wsaData.wVersion));
            PIA_TRACE_EX(common::TraceFlagFramework, "latest version is %d.%d", LOBYTE(wsaData.wHighVersion), HIBYTE(wsaData.wHighVersion));
        }
    }
#elif NN_PIA_NINTENDOSDK
    if (setting.memoryPoolSize < nn::socket::MinSocketMemoryPoolSize)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPoolSize(%d) < socket::MinSocketMemoryPoolSize(%d)", setting.memoryPoolSize, nn::socket::MinSocketMemoryPoolSize);
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (setting.allocatorPoolSize < nn::socket::MinSocketAllocatorSize)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : allocatorPoolSize(%d) < socket::MinSocketAllocatorSize(%d)", setting.allocatorPoolSize, nn::socket::MinSocketAllocatorSize);
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (setting.memoryPool == NULL)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPool is NULL.");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (setting.memoryPoolSize % 4096 != 0 || setting.allocatorPoolSize % 4096 != 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPoolSize(%d) or allocatorPoolSize(%d) is not Aligned by 4KB.", setting.memoryPoolSize, setting.allocatorPoolSize);
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (reinterpret_cast<uintptr_t>(setting.memoryPool) % 4096 != 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "error : memoryPoolSize or allocatorPoolSize is not Aligned by 4KB.");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }

    nn::socket::Config config(setting.memoryPool,
        setting.memoryPoolSize,
        setting.allocatorPoolSize,
        nn::socket::ConfigDefault::DefaultTcpInitialSendBufferSize,
        nn::socket::ConfigDefault::DefaultTcpInitialReceiveBufferSize,
        nn::socket::ConfigDefault::DefaultTcpAutoSendBufferSizeMax,
        nn::socket::ConfigDefault::DefaultTcpAutoReceiveBufferSizeMax,
        nn::socket::ConfigDefault::DefaultUdpSendBufferSize,
        nn::socket::ConfigDefault::DefaultUdpReceiveBufferSize,
        nn::socket::ConfigDefault::DefaultSocketBufferEfficiency,
        nn::socket::DefaultConcurrencyLimit);
    nn::Result result = nn::socket::Initialize(config);
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket::Initialize failed, err=%d", nn::socket::GetLastErrno());
        PIA_TRACE_RESULT_ALWAYS(result);
        m_IsInitializedSocket = false;
        PIA_RETURN_RESULT(ResultSdkError);
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "socket::Initialize success");
#elif NN_PIA_A || NN_PIA_B
    // 何もしない
#endif
    m_IsInitializedSocket = true;
    return ResultSuccess();
}

void NetworkService::FinalizeNetworkInterface()
{
#if NN_PIA_CTR
    nn::Result result;

    if (m_IsNetworkIfUp)
    {
        // 切断通知を受ける Event を破棄します
        m_ACDisconnectedEvent.Finalize();

        PIA_TRACE_EX(common::TraceFlagFramework, "FinalizeNetwork:ac::Close");
        result = nn::ac::Close();
        if (result.IsFailure())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "FinalizeNetwork:ac::Close:failed");
            PIA_TRACE_RESULT_ALWAYS(result);
            //n1589:ここでリターンすると次のAC初期化に失敗する
        }
        m_IsNetworkIfUp = false;
    }

    if (m_IsNetworkIfInitialized)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "FinalizeNetwork:ac::Finalize");
        result = nn::ac::Finalize();
        if (result.IsFailure())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "FinalizeNetwork:ac::Finalize:failed");
            PIA_TRACE_RESULT_ALWAYS(result);
        }
        m_IsNetworkIfInitialized = false;
    }

// fs::Initializeはありますが、fs::Finalizeは呼ぶ必要はありません
#elif NN_PIA_CAFE
#elif NN_PIA_WIN
    // Winsock を停止します
    WSACleanup();
    // Windows ではネットワークインターフェースは OS 側が制御するので何もしない
    m_IsNetworkIfInitialized = false;
#elif NN_PIA_NINTENDOSDK
    if (m_IsInitializedSocket)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "Net_FinalizeNetwork:nn::socket::Finalize");
        nn::Result result = nn::socket::Finalize();
        if (result.IsFailure())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "Net_FinalizeNetwork:nn::socket::Finalize is failed. [%08x] [err=%d]", result.GetInnerValueForDebug(), nn::socket::GetLastErrno());
        }
        m_IsInitializedSocket = false;
    }
    // nn::nifm::Finalize を呼ぶ必要はありません
    m_IsNetworkIfUp = false;
    m_IsNetworkIfInitialized = false;
#elif NN_PIA_A || NN_PIA_B
    // 何もしない
    m_IsNetworkIfInitialized = false;
#endif
}

// ソケットの終了処理
void NetworkService::FinalizeSocket()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    if (!m_IsInitializedSocket)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket is not initialized.");
        return;
    }
    m_IsInitializedSocket = false;

#if NN_PIA_CTR
    nn::Result result;

    // Finilizing socket library
    PIA_TRACE_EX(common::TraceFlagFramework, "socket::Finalize begin");
    result = nn::socket::Finalize();
    PIA_ASSERT(result.IsSuccess());
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket::Finalize failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_ASSERT(result.GetLevel() != nn::Result::LEVEL_USAGE);
        return;
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "socket::Finalize success");
#elif NN_PIA_CAFE
    // ac::Connect が成功している場合はソケットライブラリを終了し、AC の状態をチェックしているスレッドを回収します
    if (m_UpdateACStatusRun)
    {
        // Finilizing socket library
        PIA_TRACE_EX(common::TraceFlagFramework, "FinalizeNetwork:SOFinish");
        int ret = SOFinish();
        if (ret != 0)
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "SOFinish:failed, err=%d", ret);
        }

        // AC の状態をチェックするスレッドを停止し、回収します
        m_UpdateACStatusRun = false;
        //s_UpdateACStatusThread.Join();
        //s_UpdateACStatusThread.Finalize();
    }
#elif NN_PIA_WIN
    // Winsock を停止します
    WSACleanup();
#elif NN_PIA_NINTENDOSDK
    PIA_TRACE_EX(common::TraceFlagFramework, "socket::Finalize begin");
    nn::Result result = nn::socket::Finalize();
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket::Finalize failed, err=%d", nn::socket::GetLastErrno());
        PIA_TRACE_RESULT_ALWAYS(result);
        return;
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "socket::Finalize success");
#elif NN_PIA_A || NN_PIA_B
    // 何もしない
    m_IsInitializedSocket = false;
#endif
}

// ローカルアドレスを取得します。
Result NetworkService::GetLocalAddress(uint32_t* pIpAddr, uint32_t* pSubnetMask)
{
#if NN_PIA_CTR
    int32_t ret;
    nn::socket::InAddr addr1, addr2;

    ret = nn::socket::GetPrimaryAddress(reinterpret_cast<uint8_t*>(&addr1), reinterpret_cast<uint8_t*>(&addr2));
    PIA_ASSERT(ret == 0);
    if (ret < 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket::GetPrimaryAddress failed, ret=%d", ret);
        PIA_RETURN_RESULT(ResultSdkError);
    }
    *pIpAddr = common::ByteOrder::NetworkToHost32(addr1.addr);
    *pSubnetMask = common::ByteOrder::NetworkToHost32(addr2.addr);
#elif NN_PIA_CAFE
    nn::Result result;
    nn::ac::IpAddress addr;

    result = nn::ac::GetAssignedAddress(&addr);
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ac::GetAssignedAddress failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_RETURN_RESULT(ResultSdkError);
    }
    *pIpAddr = common::ByteOrder::NetworkToHost32(addr);

    result = nn::ac::GetAssignedSubnet(&addr);
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ac::GetAssignedSubnet failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_RETURN_RESULT(ResultSdkError);
    }
    *pSubnetMask = common::ByteOrder::NetworkToHost32(addr);
#elif NN_PIA_WIN
    //ホスト名を取得する
    char hostname[256];
    if (gethostname(hostname, sizeof(hostname)) != 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "gethostname failed");
        PIA_RETURN_RESULT(ResultSdkError);
    }

    //ホスト名からIPアドレスを取得する
    HOSTENT* hostent = gethostbyname(hostname);
    if (hostent == NULL)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "gethostbyname failed");
        PIA_RETURN_RESULT(ResultSdkError);
    }
    IN_ADDR inaddr;
    memcpy(&inaddr, hostent->h_addr_list[0], sizeof(struct in_addr));
    *pIpAddr = ntohl(inaddr.S_un.S_addr);
    *pSubnetMask = 0xffffff00;
#elif NN_PIA_NINTENDOSDK
    nn::Result result;
    in_addr inAddrIpAddress;
    in_addr inAddrSubnetMask;
    in_addr inAddrDefaultGateway;
    in_addr inAddrPreferredDns;
    in_addr inAddrAlternateDns;

    PIA_TRACE_EX(common::TraceFlagFramework, "nifm::GetCurrentIpConfigInfo begin.");
    result = nn::nifm::GetCurrentIpConfigInfo(
        &inAddrIpAddress,
        &inAddrSubnetMask,
        &inAddrDefaultGateway,
        &inAddrPreferredDns,
        &inAddrAlternateDns);
    PIA_TRACE_EX(common::TraceFlagFramework, "nifm::GetCurrentIpConfigInfo end.");
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "nifm::GetCurrentIpConfigInfo failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_RETURN_RESULT(ResultSdkError);
    }

    *pIpAddr = common::ByteOrder::NetworkToHost32(inAddrIpAddress.s_addr);
    *pSubnetMask = common::ByteOrder::NetworkToHost32(inAddrSubnetMask.s_addr);

    NN_PIA_UNUSED(inAddrDefaultGateway);
    NN_PIA_UNUSED(inAddrPreferredDns);
    NN_PIA_UNUSED(inAddrAlternateDns);
#elif NN_PIA_A
    int dummySock = socket(AF_INET, SOCK_STREAM, 0);
    if (dummySock < 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "socket() is failed.[errno:%d]", errno);
        PIA_RETURN_RESULT(ResultInvalidState);
    }

    struct ifreq ifr;
    memset(&ifr, 0, sizeof(ifreq));
    ifr.ifr_addr.sa_family = AF_INET;
    strncpy(ifr.ifr_name, "wlan0", IFNAMSIZ - 1);

    if (ioctl(dummySock, SIOCGIFADDR, &ifr) < 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ioctl(dummySock, SIOCGIFADDR, &ifr) is failed.[errno:%d]", errno);
        PIA_RETURN_RESULT(ResultInvalidState);
    }
    *pIpAddr = common::ByteOrder::NetworkToHost32(((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr);

    if (ioctl(dummySock, SIOCGIFNETMASK, &ifr) < 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ioctl(dummySock, SIOCGIFNETMASK, &ifr) is failed.[errno:%d]", errno);
        PIA_RETURN_RESULT(ResultInvalidState);
    }
    *pSubnetMask = common::ByteOrder::NetworkToHost32(((struct sockaddr_in*)&ifr.ifr_addr)->sin_addr.s_addr);

    close(dummySock);
#elif NN_PIA_B
    struct ifaddrs *ifa_list;
    struct ifaddrs *ifa;

    int result = getifaddrs(&ifa_list);
    if (result != 0)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "getifaddrs is failed.");
        return; // 失敗
    }

    for (ifa = ifa_list; ifa != NULL; ifa = ifa->ifa_next)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "ifa_name : %s, addr : %08x, sa_family : %u", ifa->ifa_name, ntohl(((struct sockaddr_in *)ifa->ifa_addr)->sin_addr.s_addr), ifa->ifa_addr->sa_family);
        if (strcmp(ifa->ifa_name, "en0") == 0 && ifa->ifa_addr->sa_family == AF_INET)
        {
            *pIpAddr = ntohl(((struct sockaddr_in *)ifa->ifa_addr)->sin_addr.s_addr);
            *pSubnetMask = ntohl(((struct sockaddr_in *)ifa->ifa_netmask)->sin_addr.s_addr);
            PIA_TRACE_EX(common::TraceFlagFramework, "Input : pIpAddr:0x%08x, pSubnetMask:0x%08x", *pIpAddr, *pSubnetMask);
        }
    }
    freeifaddrs(ifa_list);
#else
    PIA_RETURN_RESULT(ResultNotImplemented);
#endif

    PIA_TRACE_EX(common::TraceFlagFramework, "Success : pIpAddr:0x%08x, pSubnetMask:0x%08x", *pIpAddr, *pSubnetMask);
    return ResultSuccess();
}


bool NetworkService::IsNetworkInterfaceConnected()
{
#if NN_PIA_CTR
    if (m_ACDisconnectedEvent.IsValid() == false)
    {
        return false;
    }

    if (m_ACDisconnectedEvent.Wait(0))
    {
        return false;
    }
    else
    {
        return true;
    }
#elif NN_PIA_CAFE
    return m_ACConnected;
#elif NN_PIA_WIN
    return true;
#elif NN_PIA_A || NN_PIA_B
    return true;
#elif NN_PIA_NINTENDOSDK
    return nn::nifm::IsNetworkAvailable();
#else
    return false;
#endif
}

}
}
}  // end of namespace nn::pia::framework
