﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/framework/framework_NexService.h>

#if NN_PIA_FRAMEWORK_ENABLE_NEX

#include <nn/pia/common/private/common_NewDelete.h>
#include <nn/pia/common/common_Api.h>
#include <nn/pia/framework/private/framework_NetworkService.h>
#include <nn/pia/framework/framework_framework.h>

#if NN_PIA_CTR
// n3389 : warning 抑制
#pragma diag_suppress 1300  //n1769 ARMCC 用プラグマ。
#endif
#include <nex.h>
#if NN_PIA_CTR
// n3389 : warning 抑制の解除
#pragma diag_warning 1300  //n1769 ARMCC 用プラグマ。
#endif

#if NN_PIA_CAFE
#include <cafe/mem.h>
#include <nn/fp.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/mem.h>
#endif



namespace
{
// メインメモリのサイズ
uint32_t s_MemorySize = 0;

// 緊急用メモリのサイズ
uint32_t s_EmergencyMemorySize = 0;

// メインメモリ
void* s_pMemory = NULL;

// ヒープ枯渇時に使用する緊急用メモリ
void* s_pEmergencyMemory = NULL;

// メモリ使用量
uint32_t s_MemUsage = 0;
uint32_t s_MemMaxUsage = 0;

#if NN_PIA_CTR
// NEX用ヒープ
nn::fnd::ThreadSafeExpHeap s_NexHeap;
#elif NN_PIA_CAFE
MEMHeapHandle s_NexHeap;
#elif NN_PIA_NINTENDOSDK
nn::mem::StandardAllocator s_NexHeap;
#endif

}  // end of anonymous namespace

namespace nn
{
namespace pia
{
namespace framework
{
#if NN_PIA_CAFE
OSEvent s_FriendPresenceLoginEvent;
//-----------------------------------------------------------------------------
// フレンドサーバへのログイン非同期処理の完了コールバック
//-----------------------------------------------------------------------------
void FriendPresenceLoginCompletionCallback(nn::Result result, void* pContext)
{
    nn::Result* pResult = static_cast<nn::Result*>(pContext);
    *pResult = result;

    OSSignalEvent(&s_FriendPresenceLoginEvent);
}
#endif

const nn::nex::Core::ThreadMode NexService::NexThreadMode = nn::nex::Core::ThreadModeUnsafeTransportBuffer;

// コンストラクタ。
NexService::NexService()
    : m_IsNexInitialized(false)
    , m_IsInetInitialized(false)
    , m_pNgsFacade(NULL)
    , m_pCallContext(NULL)
    , m_LoginInfo()
    , m_NexNetworkFactory()
{
}

// デストラクタ。
NexService::~NexService()
{
}

// inet 初期化処理
Result NexService::InitializeInet(const InitializeInetSetting& setting)
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");
    NN_PIA_UNUSED(setting);

    if (m_IsInetInitialized)
    {
        PIA_RETURN_RESULT_WITH_REPORT(ResultAlreadyInitialized, "InetService is already initialized.");
    }

    Result result;

    result = inet::Initialize();
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "inet::Initialize failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        return result;
    }

    result = inet::BeginSetup();
    if (result.IsFailure())
    {
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_ASSERTMSG(false, "inet::BeginSetup failed");
        return result;
    }

    result = inet::NexFacade::CreateInstance();
    if (result.IsFailure())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "NexService::CreateInstance failed");
        PIA_TRACE_RESULT_ALWAYS(result);
        inet::EndSetup();
        return result;
    }

    result = inet::EndSetup();
    if (result.IsFailure())
    {
        PIA_TRACE_RESULT_ALWAYS(result);
        PIA_ASSERTMSG(false, "inet::EndSetup failed");
        return result;
    }

    // メモリ消費量をプリント
    PIA_TRACE_EX(common::TraceFlagFramework, "Pia current memory usage: %d", common::GetMemoryUsage());

    m_IsInetInitialized = true;

    if (setting.isAutoInitializeAndLogin)
    {
        result = inet::NexFacade::GetInstance()->Bind(&m_LoginInfo);
        if (result.IsFailure())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "NexService::Bind failed");
            PIA_TRACE_RESULT_ALWAYS(result);
            return result;
        }
    }

    return ResultSuccess();
}

// Inet 終了処理
void NexService::FinalizeInet()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    if (!m_IsInetInitialized)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "already finalized");
        return;
    }

    // 先に NexService から NEX ゲームサーバ情報を解除
    if (inet::NexFacade::GetInstance() != NULL)
    {
        inet::NexFacade::GetInstance()->Unbind();
    }

    inet::NexFacade::DestroyInstance();

    inet::Finalize();

    m_IsInetInitialized = false;
}

Result NexService::InitializeInetService(const InitializeInetServiceSetting& base)
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    const InitializeNexServiceSetting& setting = static_cast<const InitializeNexServiceSetting&>(base);

    if (m_IsNexInitialized)
    {
        PIA_RETURN_RESULT_WITH_REPORT(ResultAlreadyInitialized, "NexService is already initialized.");
    }

    if (m_IsNexInitialized)
    {
        PIA_RETURN_RESULT_WITH_REPORT(ResultAlreadyInitialized, "NEX is already initialized.");
    }

    if (setting.nexMemory == NULL)
    {
        PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "nexMemory == NULL");
    }

    if (setting.totalMemorySize <= setting.emergencyMemorySize)
    {
        PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "totalMemorySize(%d) <= emergencyMemorySize(%d)", setting.totalMemorySize, setting.emergencyMemorySize);
    }

#if NN_PIA_NINTENDOSDK
    // 4KBアライン、16KB以上であること
    if (reinterpret_cast<uintptr_t>(setting.nexMemory) % (4 * 1024) != 0)
    {
        PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "nexMemory (0x%p) must be aligned by 4 KB", setting.nexMemory);
    }
    if (setting.totalMemorySize < 16 * 1024)
    {
        PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidArgument, "totalMemorySize (%d) must be more than 16 KB", setting.totalMemorySize);
    }
#endif

    // NEX 用ヒープの初期化処理
    InitializeRuntimeMemory(setting.nexMemory, setting.totalMemorySize, setting.emergencyMemorySize);

    // グローバルインスタンスなどの初期化処理
    nn::nex::GlobalVariables::AcquireInstance();

    // スレッドモード設定
    nn::nex::Core::SetThreadMode(NexThreadMode);

    // NgsFacade インスタンスの生成
    m_pNgsFacade = qNew nn::nex::NgsFacade();
    PIA_ASSERT(m_pNgsFacade != NULL);

    if (nn::nex::EventLog::GetInstance() == NULL || nn::nex::TraceLog::GetInstance() == NULL)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "Error : GetInstance() == NULL");
        PIA_RETURN_RESULT(ResultInvalidState);
    }

    // CallContext インスタンスの生成
    m_pCallContext = qNew nn::nex::ProtocolCallContext();
    PIA_ASSERT(m_pCallContext != NULL);

    // NEX トレース設定
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    m_IsNexInitialized = true;

    // メモリ消費量をプリント
    PIA_TRACE_EX(common::TraceFlagFramework, "Pia current memory usage: %d", common::GetMemoryUsage());

    return ResultSuccess();
}

void NexService::FinalizeInetService()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    if (!m_IsNexInitialized)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "already finalized");
        return;
    }

    // NgsFacade インスタンスの破棄
    if (m_pNgsFacade)
    {
        qDelete m_pNgsFacade;
        m_pNgsFacade = NULL;
    }

    // CallContext インスタンスの破棄
    if (m_pCallContext)
    {
        qDelete m_pCallContext;
        m_pCallContext = NULL;
    }

    // グローバルインスタンスなどの終了処理
    nn::nex::GlobalVariables::ReleaseInstance();

    // NEX 用ヒープの終了処理
    FinalizeRuntimeMemory();

    m_IsNexInitialized = false;
}

void NexService::InitializeRuntimeMemory(uint8_t* pNexMemory, uint32_t totalMemorySize, uint32_t emergencyMemorySize)
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called total:%d, emergency:%d", totalMemorySize, emergencyMemorySize);

    PIA_ASSERT(totalMemorySize > emergencyMemorySize);
    s_MemorySize = totalMemorySize - emergencyMemorySize;
    s_EmergencyMemorySize = emergencyMemorySize;
    s_pMemory = pNexMemory;
    NN_PIA_UNUSED(s_MemorySize);

    // ヒープを初期化する
#if NN_PIA_CTR
    s_NexHeap.Initialize(NN_ANY_TO_UPTR(s_pMemory), totalMemorySize);
#elif NN_PIA_NINTENDOSDK
    s_NexHeap.Initialize(s_pMemory, totalMemorySize);
#elif NN_PIA_CAFE
    s_NexHeap = MEMCreateExpHeapEx(s_pMemory, totalMemorySize, MEM_HEAP_OPT_0_CLEAR | MEM_HEAP_OPT_THREAD_SAFE);
#endif

    // メモリ関数を登録する
    nn::nex::MemoryManager::SetBasicMemoryFunctions(AllocFunction, FreeFunction);

    if (s_EmergencyMemorySize != 0)
    {
        // 緊急用メモリを確保する
        AcquireReservedMemory();
    }
}

//-----------------------------------------------------------------------------------------
// アプリケーション定義のメモリ確保関数
//-----------------------------------------------------------------------------------------
void* NexService::AllocFunction(size_t size)
{
    void* p = InnerAlloc(size);

    if (p == NULL)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "NexService::AllocFunction: No enough memory. Reserved memory will be used. Please shutdown NEX library.");
        ReleaseReservedMemory();

        p = InnerAlloc(size);
        PIA_ASSERTMSG(p != NULL, "Reserved memory is not enough");
    }

    return p;
}

//-----------------------------------------------------------------------------------------
// アプリケーション定義のメモリ解放関数
//-----------------------------------------------------------------------------------------
void NexService::FreeFunction(void* pointer)
{
    InnerFree(pointer);
}

//-----------------------------------------------------------------------------------------
// AllocFunction から呼ばれる内部メモリ関数
//-----------------------------------------------------------------------------------------
void* NexService::InnerAlloc(uint32_t size)
{
#if NN_PIA_CTR
    void* pBuffer = s_NexHeap.Allocate(size);
#elif NN_PIA_NINTENDOSDK
    void* pBuffer = s_NexHeap.Allocate(size);
#elif NN_PIA_CAFE
    void* pBuffer = MEMAllocFromExpHeapEx(s_NexHeap, size, 8);
#endif
    if (pBuffer == NULL)
    {
        return NULL;
    }

#if NN_PIA_CTR
    s_MemUsage += s_NexHeap.GetSizeOf(pBuffer);
#elif NN_PIA_CAFE
    s_MemUsage += MEMGetSizeForMBlockExpHeap(pBuffer);
#elif NN_PIA_NINTENDOSDK
    s_MemUsage += s_NexHeap.GetSizeOf(pBuffer);
#endif
    if (s_MemUsage > s_MemMaxUsage)
    {
        s_MemMaxUsage = s_MemUsage;
    }
    return pBuffer;
}

//-----------------------------------------------------------------------------------------
// FreeFunction から呼ばれる内部メモリ関数
//-----------------------------------------------------------------------------------------
void NexService::InnerFree(void* ptr)
{
#if NN_PIA_CTR
    s_MemUsage -= s_NexHeap.GetSizeOf(ptr);
    s_NexHeap.Free(ptr);
#elif NN_PIA_CAFE
    s_MemUsage -= MEMGetSizeForMBlockExpHeap(ptr);
    MEMFreeToExpHeap(s_NexHeap, ptr);
#elif NN_PIA_NINTENDOSDK
    s_MemUsage -= s_NexHeap.GetSizeOf(ptr);
    s_NexHeap.Free(ptr);
#endif
}

//-----------------------------------------------------------------------------------------
//緊急用のメモリを解放します
//-----------------------------------------------------------------------------------------
void NexService::ReleaseReservedMemory()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    if (s_pEmergencyMemory)
    {
        uint32_t nowUsage = s_MemUsage;
        uint32_t nowMaxUsage = s_MemMaxUsage;

        FreeFunction(s_pEmergencyMemory);

        s_pEmergencyMemory = NULL;

        s_MemUsage = nowUsage;
        s_MemMaxUsage = nowMaxUsage;
    }
}

//-----------------------------------------------------------------------------------------
// 緊急用のメモリを確保します
//-----------------------------------------------------------------------------------------
void NexService::AcquireReservedMemory()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    if (s_pEmergencyMemory == NULL)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "s_pEmergencyMemory == NULL");
        uint32_t nowUsage = s_MemUsage;
        uint32_t nowMaxUsage = s_MemMaxUsage;
        PIA_TRACE_EX(common::TraceFlagFramework, "->nowUsage:%d, nowMaxUsage:%d, s_EmergencyMemorySize:%d", s_MemUsage, nowMaxUsage, s_EmergencyMemorySize);
        s_pEmergencyMemory = AllocFunction(s_EmergencyMemorySize);

        s_MemUsage = nowUsage;
        s_MemMaxUsage = nowMaxUsage;
    }
}

void NexService::FinalizeRuntimeMemory()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    if (s_EmergencyMemorySize != 0)
    {
        // 緊急用メモリが残っていれば解放する
        ReleaseReservedMemory();
    }

    PIA_TRACE_EX(common::TraceFlagFramework, "NEX memory usage: Now memory usage:%d, Max memory usage:%d / %d", s_MemUsage, s_MemMaxUsage, s_MemorySize + s_EmergencyMemorySize);
#if NN_PIA_CTR
    // メモリリークしている
    if (s_MemUsage > 0)
    {
        // リークしているメモリを表示する
        s_NexHeap.Dump();
        // リークしているメモリを無効化する
        s_NexHeap.Invalidate();
    }
    // ヒープを破棄する
    s_NexHeap.Finalize();
#elif NN_PIA_NINTENDOSDK
    s_NexHeap.Finalize();
#elif NN_PIA_CAFE
    // メモリリークしている
    if (s_MemUsage > 0)
    {
        // ヒープの情報をダンプします
        MEMDumpHeap(s_NexHeap);
    }
    // ヒープを破棄します
    MEMDestroyExpHeap(s_NexHeap);
#endif

    s_MemorySize = 0;
    s_EmergencyMemorySize = 0;
    s_pMemory = NULL;

    s_MemUsage = 0;
    s_MemMaxUsage = 0;
}

Result NexService::Dispatch()
{
    if (nn::nex::Scheduler::GetInstance() != NULL)
    {
        nn::nex::Scheduler::GetInstance()->DispatchAll();
    }
    return ResultSuccess();
}

Result NexService::CheckConnection()
{
    if (IsInLogoutProcess())
    {
        return ResultSuccess();

    }

    if (m_pNgsFacade == NULL)
    {
        PIA_ASSERT(false);
        PIA_RETURN_RESULT(ResultInvalidState);
    }

    nn::nex::qResult qResult = m_pNgsFacade->IsConnected();
    if (qResult)
    {
        return ResultSuccess();
    }

    if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnected))
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnected)");
        PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
    }
#if NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 2, 0)
    if (qResult.GetReturnCode() == QERROR(Core, OperationAborted))
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Core, OperationAborted)");
        PIA_RETURN_RESULT(ResultGameServerProcessAborted);
    }

    else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin))
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin)");
        PIA_RETURN_RESULT(ResultNetworkConnectionIsLostByDuplicateLogin);
    }
    else if (qResult.GetReturnCode() == QERROR(Transport, Unknown))
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Transport, Unknown)");
        PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
    }
#endif
    else
    {
        // 未知の Result が返ってきた
        PIA_TRACE_EX(common::TraceFlagFramework, "Unhandle qResult detect. ErrorName: %ls, ReturnCode: 0x%x", qResult.GetErrorName(), qResult.GetReturnCode());

#if NN_PIA_NINTENDOSDK
        nn::err::ErrorCode nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToErrorCodeStruct(qResult);
#else
        uint32_t nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(qResult);
#endif
        Result::SetNexNetworkErrorCode(nexErrorCode);
        PIA_RETURN_RESULT(ResultNexInternalError);
    }
}

transport::NetworkFactory* NexService::GetNetworkFactory()
{
    return &m_NexNetworkFactory;
}

// フレンドサーバへのログインを行います
Result NexService::LoginFriendServer()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

#if NN_PIA_CTR
    nn::os::Event event(true);

    // フレンドライブラリを初期化する
    nn::Result sdkResult = nn::friends::Initialize();
    if (sdkResult.IsFailure())
    {
        // デーモンの内部状態などにより、失敗することがある
        uint32_t networkErrorCode = nn::friends::ResultToErrorCode(sdkResult);
        PIA_TRACE_EX(common::TraceFlagFramework, "friends::Initialize failed. errCode:%d", networkErrorCode);
        PIA_TRACE_RESULT_ALWAYS(sdkResult);
        PIA_ASSERT(sdkResult.GetLevel() != nn::Result::LEVEL_USAGE);

        if (networkErrorCode == 0)
        {
            PIA_RETURN_RESULT(ResultSdkError);
        }
        else
        {
            Result::SetSdkErrorCode(networkErrorCode);
            PIA_RETURN_RESULT(ResultSdkViewerErrorCodeError);
        }
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "friends::Initialize success");

    // フレンドサーバにログインする
    sdkResult = nn::friends::Login(&event);
    if (sdkResult.IsFailure())
    {
        // APと接続できていない場合などに、失敗することがある
        uint32_t networkErrorCode = nn::friends::ResultToErrorCode(sdkResult);
        PIA_TRACE_EX(common::TraceFlagFramework, "friends::Initialize failed. errCode:%d", networkErrorCode);
        PIA_TRACE_RESULT_ALWAYS(sdkResult);
        PIA_ASSERT(sdkResult.GetLevel() != nn::Result::LEVEL_USAGE);

        if (networkErrorCode == 0)
        {
            PIA_RETURN_RESULT(ResultSdkError);
        }
        else
        {
            Result::SetSdkErrorCode(networkErrorCode);
            PIA_RETURN_RESULT(ResultSdkViewerErrorCodeError);
        }
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "friends::Login begin success");

    // 非同期ログイン処理の終了待ち
    // 処理の可読性を重視してここで行います。
    event.Wait();

    // 非同期ログイン処理の結果取得
    sdkResult = nn::friends::GetLastResponseResult();
    if (sdkResult.IsFailure())
    {
        // デーモンの内部状態などにより、失敗することがある
        uint32_t networkErrorCode = nn::friends::ResultToErrorCode(sdkResult);
        PIA_TRACE_EX(common::TraceFlagFramework, "friends::Initialize failed. errCode:%d", networkErrorCode);
        PIA_TRACE_RESULT_ALWAYS(sdkResult);
        PIA_ASSERT(sdkResult.GetLevel() != nn::Result::LEVEL_USAGE);

        if (networkErrorCode == 0)
        {
            PIA_RETURN_RESULT(ResultSdkError);
        }
        else
        {
            Result::SetSdkErrorCode(networkErrorCode);
            PIA_RETURN_RESULT(ResultSdkViewerErrorCodeError);
        }
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "friends::Login process success. MyPrincipalId:%d", nn::friends::GetMyPrincipalId());

#elif NN_PIA_CAFE
    // フレンドサーバへのログインは必須ではないが、フレンドプレゼンスを使用する場合はログインする
    nn::Result asyncResult;

    // 自動モードでイベント初期化
    OSInitEvent(&s_FriendPresenceLoginEvent, FALSE, OS_EVENT_AUTO);
    PIA_TRACE_EX(common::TraceFlagFramework, "fp::LoginAsync called");
    nn::Result sdkResult = nn::fp::LoginAsync(FriendPresenceLoginCompletionCallback, &asyncResult);
    if (sdkResult.IsFailure())
    {
        uint32_t networkErrorCode = nn::fp::ResultToErrorCode(sdkResult);
        PIA_TRACE_EX(common::TraceFlagFramework, "fp::LoginAsync start failed. NetworkErrorCode:%d", networkErrorCode);
        PIA_TRACE_RESULT_ALWAYS(sdkResult);
        PIA_RETURN_RESULT(ResultTemporaryUnavailable);  //一時的に HandlingType_Retry を返す
    }
    // nn::fp::LoginAsync() 完了のイベントシグナルを待ちます
    OSWaitEvent(&s_FriendPresenceLoginEvent);
    if (asyncResult.IsFailure())
    {
        uint32_t networkErrorCode = nn::fp::ResultToErrorCode(asyncResult);
        PIA_TRACE_EX(common::TraceFlagFramework, "fp::LoginAsync async failed. NetworkErrorCode:%d", networkErrorCode);
        PIA_TRACE_RESULT_ALWAYS(asyncResult);
        PIA_RETURN_RESULT(ResultTemporaryUnavailable);  //一時的に HandlingType_Retry を返す
    }

    PIA_TRACE_EX(common::TraceFlagFramework, "fp::LoginAsync succeeded:MyPrincipalId:%d", nn::fp::GetMyPrincipalId());
#elif NN_PIA_NINTENDOSDK
    // friends API 呼び出し時には必要ならライブラリの初期化が行われるので、ここでは nn::friends::Initialize() は呼ばない。
#endif

    return ResultSuccess();
}

// ゲームサーバからログアウトします
Result NexService::LogoutFriendServer()
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    bool isErrorOccurred = false;
    Result errorResult = ResultSuccess();
    uint32_t networkErrorCode = 0;

#if NN_PIA_CTR
    nn::Result sdkResult;

    // フレンドサーバからログアウト
    if (nn::friends::IsInitialized())  // n1769 未初期化状態で Logout() を呼び出すと、SDK 内部のアサートに引っかかるようなので…。
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "friends::Logout begin");
        sdkResult = nn::friends::Logout();
        if (sdkResult.IsFailure())
        {
            uint32_t networkErrorCodeBuf = nn::friends::ResultToErrorCode(sdkResult);
            PIA_TRACE_EX(common::TraceFlagFramework, "friends::Logout failed. errCode:%d", networkErrorCodeBuf);
            PIA_TRACE_RESULT_ALWAYS(sdkResult);
            PIA_ASSERT(sdkResult.GetLevel() != nn::Result::LEVEL_USAGE);

            if (isErrorOccurred == false)
            {
                isErrorOccurred = true;
                if (networkErrorCodeBuf == 0)
                {
                    errorResult = ResultSdkError();
                    networkErrorCode = 0;
                }
                else
                {
                    errorResult = ResultSdkViewerErrorCodeError();
                    networkErrorCode = networkErrorCodeBuf;
                }
            }
        }
        else
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "friends::Logout success");
        }
    }
    else
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "friends::Logout skipped");
    }

    // フレンドライブラリを終了させる
    if (nn::friends::IsInitialized())  // n1769 未初期化状態で Finalize() を呼び出すと、SDK 内部のアサートに引っかかるようなので…。
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "friends::Finalize begin");
        sdkResult = nn::friends::Finalize();
        if (sdkResult.IsFailure())
        {
            uint32_t networkErrorCodeBuf = nn::friends::ResultToErrorCode(sdkResult);
            PIA_TRACE_EX(common::TraceFlagFramework, "friends::Finalize failed. errCode:%d", networkErrorCodeBuf);
            PIA_TRACE_RESULT_ALWAYS(sdkResult);
            PIA_ASSERT(sdkResult.GetLevel() != nn::Result::LEVEL_USAGE);

            if (isErrorOccurred == false)
            {
                isErrorOccurred = true;
                if (networkErrorCodeBuf == 0)
                {
                    errorResult = ResultSdkError();
                    networkErrorCode = 0;
                }
                else
                {
                    errorResult = ResultSdkViewerErrorCodeError();
                    networkErrorCode = networkErrorCodeBuf;
                }
            }
        }
        else
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "friends::Logout success");
        }
    }
    else
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "friends::Logout skipped");
    }

#elif NN_PIA_CAFE
    // フレンドサーバからログアウト
    PIA_TRACE_EX(common::TraceFlagFramework, "fp::Logout");
    if (nn::fp::IsInitialized())
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "fp::Logout called");
        nn::Result sdkResult = nn::fp::Logout();
        if (sdkResult.IsFailure())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "fp::Logout failed");
            PIA_TRACE_RESULT_ALWAYS(sdkResult);
            PIA_RETURN_RESULT(ResultTemporaryUnavailable);  //一時的に HandlingType_Retry を返す
        }
    }
#elif NN_PIA_NINTENDOSDK
    // 何もしなくてよい
#endif

    if (isErrorOccurred)
    {
        if (errorResult == ResultSdkViewerErrorCodeError())
        {
            Result::SetSdkErrorCode(networkErrorCode);
        }
        return errorResult;
    }

    PIA_TRACE_EX(common::TraceFlagFramework, "Logout success");

    return ResultSuccess();
}

// ゲームサーバーへのログインの前処理
Result NexService::PreLoginProcess(const InitializeInetSetting* pSetting)
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

#if NN_PIA_NINTENDOSDK_NX && (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 1, 1))
    Result result;

    if (!PIA_IS_VALID_POINTER(pSetting))
    {
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }

    while (true)
    {
        nn::Result accountResult = nn::account::EnsureNetworkServiceAccountAvailable(pSetting->userHandle);
        if (nn::account::ResultCancelledByUser::Includes(accountResult))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "EnsureNetworkServiceAccountAvailable() has been cancelled by user.");
            PIA_RETURN_RESULT(ResultCancelledByUser);
        }
        else if (accountResult.IsFailure())
        {
            // その他の理由では失敗しない。
            PIA_ASSERT(false);
            PIA_RETURN_RESULT(ResultInvalidState);
        }

        // ネットワークサービスアカウントの ID トークンキャッシュを確定
        {
            // ネットワークサービスアカウントへのログイン + ID トークンのキャッシュ API
            // すでにキャッシュが存在する場合は即時完了する。
            accountResult = nn::account::EnsureNetworkServiceAccountIdTokenCacheAsync(&m_AccountAsyncContext, pSetting->userHandle);
            if (accountResult.IsFailure())
            {
                PIA_TRACE_EX(common::TraceFlagFramework, "EnsureNetworkServiceAccountIdTokenCacheAsync() failed.");
                PIA_TRACE_RESULT_ALWAYS(accountResult);
                PIA_RETURN_RESULT(ResultSdkError);
            }
        }

        accountResult = m_AccountAsyncContext.GetSystemEvent(&m_AccountSystemEvent);
        if (accountResult.IsFailure())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "nn::account::AsyncContext::GetSystemEvent() failed.");
            PIA_TRACE_RESULT_ALWAYS(accountResult);
            PIA_RETURN_RESULT(ResultSdkError);
        }

        break;
    }

    PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::PreLoginProcess success");
#else
    NN_PIA_UNUSED(pSetting);
#endif
    return ResultSuccess();
}

bool NexService::IsInPreLoginProcess()
{
#if NN_PIA_NINTENDOSDK_NX && (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 1, 1))

    return !m_AccountSystemEvent.TryWait();
#else
    return false;
#endif
}

// ゲームサーバーへのログイン
Result NexService::LoginGameServer(const InitializeInetSetting* pSetting, uint32_t timeout)
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called");

    Result result;

    const uint32_t gameId = pSetting->gameId;

    // ゲームサーバにログインする
    PIA_TRACE_EX(common::TraceFlagFramework, "Logging in to Game Server...");
    nn::nex::qResult qResult;
    PIA_ASSERT(m_pNgsFacade != NULL);

#if NN_PIA_NINTENDOSDK
    if (pSetting->hostName.GetEncodingType() == common::String::EncodingType_Utf16)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "The character encoding of LoginSetting::hostName should be UTF-8");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (pSetting->userName.GetEncodingType() == common::String::EncodingType_Utf16)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "The character encoding of LoginSetting::userName should be UTF-8");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (pSetting->password.GetEncodingType() == common::String::EncodingType_Utf16)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "The character encoding of LoginSetting::password should be UTF-8");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
    if (pSetting->accessKey.GetEncodingType() == common::String::EncodingType_Utf16)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "The character encoding of LoginSetting::accessKey should be UTF-8");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }
#if NN_PIA_NINTENDOSDK_WIN
    // デバッグログイン
    const nn::nex::qChar* pHostName = pSetting->hostName.GetCharArray();
    uint16_t port = pSetting->port;
    const nn::nex::qChar* pHostUri = pSetting->hostUri.GetCharArray();
    const nn::nex::qChar* pUserName = pSetting->userName.GetCharArray();
    const char* pPassword = pSetting->password.GetCharArray();
    const nn::nex::qChar* pAccessKey = pSetting->accessKey.GetCharArray();

    PIA_TRACE_EX(common::TraceFlagFramework, "hostName:%s, port:%d, pHostUri:%s, userName:%s, password:%s, accessKey:%s", pHostName, port, pHostUri, pUserName, pPassword, pAccessKey);
    if (!strncmp(NEX_VERSION_PATCH, "beta", 4))
    {
        // デバッグログイン
        qResult = m_pNgsFacade->DebugLoginDirect(m_pCallContext, pHostName, port, pUserName, pPassword, pAccessKey, nn::nex::TimeInterval(timeout));
    }
    else
    {
        // デバッグログイン
        qResult = m_pNgsFacade->DebugLoginDirect(m_pCallContext, pHostUri, pUserName, pPassword, pAccessKey, nn::nex::TimeInterval(timeout));
    }
#else
    const nn::nex::qChar* pAccessKey = pSetting->accessKey.GetCharArray();

    while (true)
    {
        nn::Result accountResult = nn::account::EnsureNetworkServiceAccountAvailable(pSetting->userHandle);
        if (nn::account::ResultCancelledByUser::Includes(accountResult))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "EnsureNetworkServiceAccountAvailable() has been cancelled by user.");
            PIA_RETURN_RESULT(ResultCancelledByUser);
        }
        else if (accountResult.IsFailure())
        {
            // その他の理由では失敗しない。
            PIA_ASSERT(false);
            PIA_RETURN_RESULT(ResultInvalidState);
        }

#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 1, 1))
        accountResult = m_AccountAsyncContext.GetResult();
        if (!accountResult.IsSuccess())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "nn::account::AsyncContext::GetResult() failed.");
            if (nn::account::ResultNetworkServiceAccountUnavailable::Includes(accountResult))
            {
                // ログイン時にネットワークサービスアカウントが利用できないことが発覚
                continue;
            }
            else if (nn::account::ResultNetworkCommunicationError::Includes(accountResult))
            {
                // ログイン時に通信関係のエラーが発生
                PIA_TRACE_RESULT_ALWAYS(accountResult);
                Framework::GetInstance()->SetLastSdkResult(accountResult);
                PIA_RETURN_RESULT(ResultSdkViewerResultError);
            }
            else
            {
                // n3492: SDK の API リファレンスには無いリザルトが返ることがあるため、ResultSdkError を返しておく。
                PIA_TRACE_RESULT_ALWAYS(accountResult);
                PIA_RETURN_RESULT(ResultSdkError);
            }
        }

        accountResult = nn::account::LoadNetworkServiceAccountIdTokenCache(&m_NsaIdTokenSize, m_NsaIdToken, nn::account::NetworkServiceAccountIdTokenLengthMax, pSetting->userHandle);
        if (!accountResult.IsSuccess())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "LoadNetworkServiceAccountIdTokenCache() failed.");
            if (nn::account::ResultNetworkServiceAccountUnavailable::Includes(accountResult))
            {
                // ネットワークサービスアカウントが利用可能ではありません。
                continue;
            }
            else if (nn::account::ResultTokenCacheUnavailable::Includes(accountResult))
            {
                // IDトークンキャッシュを利用できません。
                continue;
            }
            else
            {
                // その他の理由では失敗しない。
                PIA_ASSERT(false);
                PIA_RETURN_RESULT(ResultSdkError);
            }
        }

        accountResult = nn::account::GetNetworkServiceAccountId(&m_NsaId, pSetting->userHandle);
        if (!accountResult.IsSuccess())
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "GetNetworkServiceAccountId() failed.");
            if (nn::account::ResultNetworkServiceAccountUnavailable::Includes(accountResult))
            {
                // ネットワークサービスアカウントが利用可能ではありません。
                continue;
            }
            else
            {
                // その他の理由では失敗しない。
                PIA_ASSERT(false);
                PIA_RETURN_RESULT(ResultSdkError);
            }
        }

        PIA_TRACE_EX(common::TraceFlagFramework, "nsaId: %llu, nsaIdTokenSize: %d, nsaIdToken: %s", m_NsaId, m_NsaIdTokenSize, m_NsaIdToken);
#endif

        break;
    }

    // ログインを呼ぶ前に CallContext をリセット
    m_pCallContext->Reset();

#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 1, 1))
    qResult = m_pNgsFacade->Login(m_pCallContext, gameId, pAccessKey, m_NsaId, m_NsaIdToken, nn::nex::TimeInterval(timeout));
#else
    qResult = m_pNgsFacade->Login(m_pCallContext, gameId, pAccessKey, pSetting->userHandle, nn::nex::TimeInterval(timeout));
#endif

    if (!qResult)
    {
        // ログイン処理の起動に失敗
        PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Login Begin failed (%s:%d)", qResult.GetReturnCodeString(), qResult.GetReturnCode());

        // 処理中のCallContextを渡した
        if (m_pCallContext->GetState() == nn::nex::CallContext::CallInProgress)
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "m_LoginCallContext.GetState() == nn::nex::CallContext::CallInProgress");
            PIA_ASSERT(false);
            PIA_RETURN_RESULT(ResultInvalidArgument);
        }

        if (qResult.GetReturnCode() == QERROR(Core, InvalidArgument))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Core, InvalidArgument)");
            PIA_RETURN_RESULT(ResultInvalidArgument);
        }
        else if (qResult.GetReturnCode() == QERROR(Core, OperationAborted))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Core, OperationAborted)");
            PIA_RETURN_RESULT(ResultGameServerProcessAborted);
        }
#if NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 2, 0)
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnected))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnected)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLostByDuplicateLogin);
        }
        else if (qResult.GetReturnCode() == QERROR(Transport, Unknown))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Transport, Unknown)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
#endif
        else
        {
            // 未知の Result が返ってきた
            PIA_TRACE_EX(common::TraceFlagFramework, "Unhandle qResult detect. ErrorName: %ls, ReturnCode: 0x%x", qResult.GetErrorName(), qResult.GetReturnCode());

            nn::err::ErrorCode nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToErrorCodeStruct(qResult);
            Result::SetNexNetworkErrorCode(nexErrorCode);
            PIA_RETURN_RESULT(ResultNexInternalError);
        }
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Login Begin success");
#endif // NN_PIA_NINTENDOSDK_WIN

#else
    if (pSetting->accessKey.GetEncodingType() == common::String::EncodingType_Utf8)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "The character encoding of NexSetting::accessKey should be UTF-16");
        PIA_RETURN_RESULT(ResultInvalidArgument);
    }

    const nn::nex::qChar* pAccessKey = static_cast<const nn::nex::qChar*>(pSetting->accessKey.GetBuffer());

    // ログインを呼ぶ前に CallContext をリセット
    m_pCallContext->Reset();

    PIA_TRACE_EX(common::TraceFlagFramework, "accessKey:%s", pAccessKey);
    qResult = m_pNgsFacade->Login(m_pCallContext, gameId, pAccessKey, nn::nex::TimeInterval(timeout));

    if (!qResult)
    {
        // ログイン処理の起動に失敗
        PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Login Begin failed (%s:%d)", qResult.GetReturnCodeString(), qResult.GetReturnCode());

        // 処理中のCallContextを渡した
        if (m_pCallContext->GetState() == nn::nex::CallContext::CallInProgress)
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "m_LoginCallContext.GetState() == nn::nex::CallContext::CallInProgress");
            PIA_ASSERT(false);
            PIA_RETURN_RESULT(ResultInvalidArgument);
        }

        // Bind が実行されていない
        if (m_pCallContext->GetCredentials() == NULL)
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "m_LoginCallContext.GetCredentials() == NULL");
            PIA_ASSERT(false);
            PIA_RETURN_RESULT(ResultInvalidState);
        }

        if (qResult.GetReturnCode() == QERROR(Core, InvalidArgument))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Core, InvalidArgument)");
            PIA_RETURN_RESULT(ResultInvalidArgument);
        }
        else if (qResult.GetReturnCode() == QERROR(Core, OperationAborted))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Core, OperationAborted)");
            PIA_RETURN_RESULT(ResultGameServerProcessAborted);
        }
#if NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 2, 0)
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnected))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnected)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLostByDuplicateLogin);
        }
        else if (qResult.GetReturnCode() == QERROR(Transport, Unknown))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Transport, Unknown)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
#endif
        else
        {
            // 未知の Result が返ってきた
            PIA_TRACE_EX(common::TraceFlagFramework, "Unhandle qResult detect. ErrorName: %ls, ReturnCode: 0x%x", qResult.GetErrorName(), qResult.GetReturnCode());
            // n3492: CTR なので ConvertToNetworkErrorCode を使用しても問題なし
            uint32_t nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(qResult);
            Result::SetNexNetworkErrorCode(nexErrorCode);
            PIA_RETURN_RESULT(ResultNexInternalError);
        }
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Login Begin success");

#endif // NN_PIA_NINTENDOSDK

    common::String accessKey(pAccessKey);

    m_LoginInfo.gameServerId = gameId;
    m_LoginInfo.pAccessKey = &accessKey;
    m_LoginInfo.pNgsFacade = m_pNgsFacade;

    return ResultSuccess();
}

Result NexService::PostLoginProcess()
{
#if NN_PIA_NINTENDOSDK
#if NN_PIA_NINTENDOSDK_WIN
    // 何もしない
#else

    // ログイン失敗
    if (m_pCallContext->GetState() == nn::nex::CallContext::CallFailure)
    {
        nn::Result errorResult = m_pNgsFacade->GetLastLoginNnResult();
        if (nn::account::ResultNetworkServiceAccountUnavailable::Includes(errorResult))
        {
            // 「ガイドライン 0121: ネットワークサービスアカウントを利用できない場合の処理の実装」
            // に従い、nn::nex::NgsFacade::GetLastLoginNnResult() が 
            // nn::account::ResultNetworkServiceAccountUnavailable を返した場合、
            // nn::account::EnsureNetworkServiceAccountAvailable() を呼び出して、
            // 本体システムによるネットワークサービスアカウント有効化シーケンスが表示されるようにする。
            // それに成功後、再度 nn::nex::NgsFacade::Login() を呼ぶ。
            PIA_TRACE_EX(common::TraceFlagFramework, "ResultNetworkServiceAccountUnavailable has been returned.");

            // n3487 : リトライの必要な失敗、必要ない失敗、成功の3パターンを区別するため、
            //         ここでは HandlingType_Retry な適当な Result を返しています。
            return ResultTemporaryUnavailable();
        }
        else
        {
            nn::err::ErrorCode networkErrorCode = m_pNgsFacade->GetLastLoginErrorCodeStruct();
            PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Login Process failed errorCode:%d", networkErrorCode.category * 10000 + networkErrorCode.number);
            PIA_ASSERT(networkErrorCode.IsValid());
            Result::SetNexNetworkErrorCode(networkErrorCode);
            PIA_RETURN_RESULT(ResultNexInternalError);
        }
    }
#endif // NN_PIA_NINTENDOSDK_WIN
#else // NN_PIA_NINTENDOSDK
    if (m_pCallContext->GetState() == nn::nex::CallContext::CallFailure)
    {
        uint32_t networkErrorCode = static_cast<uint32_t>(m_pNgsFacade->GetLastLoginErrorCode());
        PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Login Process failed (%d) errorCode:%d", networkErrorCode);
        PIA_ASSERT(networkErrorCode != 0);
        Result::SetNexNetworkErrorCode(networkErrorCode);
        PIA_RETURN_RESULT(ResultNexInternalError);
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Login Process success");
#endif // NN_PIA_NINTENDOSDK

    return ResultSuccess();
}

// ゲームサーバーからログアウト
Result NexService::LogoutGameServer(bool isBlock, bool isInChangeStateAsync)
{
    PIA_TRACE_EX(common::TraceFlagFramework, "called isBlock:%d isInChangeStateAsync:%d", isBlock, isInChangeStateAsync);
    NN_PIA_UNUSED(isInChangeStateAsync);

    if (m_pNgsFacade == NULL)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "m_pNgsFacade == NULL.");
        PIA_RETURN_RESULT(ResultInvalidState);
    }

    // ログアウト前に CallContext をリセット
    m_pCallContext->Reset();

    // ゲームサーバからログアウト
    nn::nex::qResult qResult = m_pNgsFacade->Logout(m_pCallContext);
    if (!qResult)
    {
        // ログアウト処理の起動に失敗
        PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Logout Begin failed (%s:%d)", qResult.GetReturnCodeString(), qResult.GetReturnCode());

        // 処理中のCallContextを渡した
        if (m_pCallContext->GetState() == nn::nex::CallContext::CallInProgress)
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "ccLogout.GetState() == nn::nex::CallContext::CallInProgress");
            PIA_ASSERT(false);
            PIA_RETURN_RESULT(ResultInvalidArgument);
        }

        // Bind が実行されていない
        if (m_pCallContext->GetCredentials() == NULL)
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "ccLogout.GetCredentials() == NULL");
            PIA_ASSERT(false);
            PIA_RETURN_RESULT(ResultInvalidState);
        }

        if (qResult.GetReturnCode() == QERROR(Core, OperationAborted))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Core, OperationAborted)");
            PIA_RETURN_RESULT(ResultGameServerProcessAborted);
        }
#if NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 2, 0)
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnected))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnected)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLostByDuplicateLogin);
        }
        else if (qResult.GetReturnCode() == QERROR(Transport, Unknown))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Transport, Unknown)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
#endif
        else
        {
            // 未知の Result が返ってきた
            PIA_TRACE_EX(common::TraceFlagFramework, "Unhandle qResult detect. ErrorName: %ls, ReturnCode: 0x%x", qResult.GetErrorName(), qResult.GetReturnCode());

#if NN_PIA_NINTENDOSDK
            nn::err::ErrorCode nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToErrorCodeStruct(qResult);
#else
            uint32_t nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(qResult);
#endif
            Result::SetNexNetworkErrorCode(nexErrorCode);
            PIA_RETURN_RESULT(ResultNexInternalError);
        }
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Logout Begin success");

    if (isBlock)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "Wait for logout process");
        m_pCallContext->Wait();

        return GetLogoutOutcome();
    }


    return ResultSuccess();
}

Result NexService::GetLogoutOutcome()
{
    nex::qResult qResult = m_pCallContext->GetOutcome();
    if (!qResult)
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Logout Process failed (%s:%d)", qResult.GetReturnCodeString(), qResult.GetReturnCode());

        if (qResult.GetReturnCode() == QERROR(Core, OperationAborted))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Core, OperationAborted)");
            PIA_RETURN_RESULT(ResultGameServerProcessAborted);
        }
#if NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 2, 0)
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnected))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnected)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
        else if (qResult.GetReturnCode() == QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(RendezVous, ConnectionDisconnectedForConcurrentLogin)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLostByDuplicateLogin);
        }
        else if (qResult.GetReturnCode() == QERROR(Transport, Unknown))
        {
            PIA_TRACE_EX(common::TraceFlagFramework, "QERROR(Transport, Unknown)");
            PIA_RETURN_RESULT(ResultNetworkConnectionIsLost);
        }
#endif
        else
        {
            // 未知の Result が返ってきた
            PIA_TRACE_EX(common::TraceFlagFramework, "Unhandle qResult detect. ErrorName: %ls, ReturnCode: 0x%x", qResult.GetErrorName(), qResult.GetReturnCode());

#if NN_PIA_NINTENDOSDK
            nn::err::ErrorCode nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToErrorCodeStruct(qResult);
#else
            uint32_t nexErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(qResult);
#endif
            Result::SetNexNetworkErrorCode(nexErrorCode);
            PIA_RETURN_RESULT(ResultNexInternalError);
        }
    }
    PIA_TRACE_EX(common::TraceFlagFramework, "NgsFacade::Logout Process success");

    return ResultSuccess();
}

nn::nex::NgsFacade* NexService::GetNgsFacade()
{
    return m_pNgsFacade;
}

nn::nex::Credentials* NexService::GetCredential()
{
    if (m_pNgsFacade == NULL)
    {
        return NULL;
    }

    return m_pNgsFacade->GetCredentials();
}

}
}
}  // end of namespace nn::pia::framework

#endif // NN_PIA_FRAMEWORK_ENABLE_NEX
