﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/reckoning/reckoning_Simple3dReckoningStrategy.h>

#include <nn/pia/common/common_Trace.h>

namespace nn
{
namespace pia
{
namespace reckoning
{

/*!
@brief  デフォルトの閾値を設定します。
*/
Simple3dReckoningStrategy::Simple3dReckoningStrategy(float threshold)
    : m_Threshold(threshold)
{
}

void Simple3dReckoningStrategy::SetThreshold(float threshold)
{
    // 閾値が0より小さい値にならないように範囲制限する
    if (threshold < 0.f)
    {
        m_Threshold = 0.f;
    }
    else
    {
        m_Threshold = threshold;
    }
}


static void calcParameter(float* b, const float& v, const float& t)
{
    *b = v / t;
}


static void calcEstimation(float* out, const float& a, const float& b, const float& t)
{
    *out = (a) * t + b;
}


/*
void Simple3dReckoningStrategy::calcEstimationDelay( Value* pValue ,  const Accessor& accessor , float clock ) const {


    int32_t beginIndex = 0;
    while( beginIndex < accessor.GetNum() ){
        if( clock > ( float)( accessor.GetClock( beginIndex ) ) ){

            break;
        }else{

            beginIndex ++;
        }
    }

    int32_t index0,index1;

    float tt;

    Value baseValue;
    if( beginIndex == 0 ){
        index0 = 0 ;
        index1 = 1;

        tt = clock - (float)( accessor.GetClock( index0 ) );

        baseValue = accessor.GetSample( index0 ).m_Value;
    }else if( beginIndex == accessor.GetNum() ){
        index0 = beginIndex -2;
        index1 = beginIndex - 1;

        tt = clock - (float)( accessor.GetClock( index1 ) );
        baseValue = accessor.GetSample( index1 ).m_Value;

    }else{

        index0 = beginIndex - 1;
        index1 = beginIndex;

        tt = clock - (float)( accessor.GetClock( index1 ) );
        baseValue = accessor.GetSample( index1 ).m_Value;

    }


    Value value1 = accessor.GetSample( index1 ).m_Value;
    nn::pia::reckoning::Vector3f::Mul( &value1 , -1.0f );

    Value diff = accessor.GetSample( index0 ).m_Value;
    nn::pia::reckoning::Vector3f::Add( &diff , value1 );


    nn::pia::reckoning::Vector3f::Mul( &diff , tt / (float)( accessor.GetClock( index0 ) - accessor.GetClock( index1 ) ) );


    nn::pia::reckoning::Vector3f::Add( &baseValue , diff );
    
    *pValue = baseValue;

}
*/


bool Simple3dReckoningStrategy::Estimate(Value* pValue, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{

    //    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" Estimate \n");
    // 標本値がひとつも存在しない場合、推測値を用いない
    if (accessor.GetNum() <= 0)
    {
        return false;
    }

    // 現在の時刻が、最も古い標本値の時刻よりも古い場合、
    // 標本値を推測値として採用する
    if (clock < accessor.GetClock(accessor.GetNum() - 1))
    {
        *pValue = accessor.GetSample(accessor.GetNum() - 1).m_Value;
        return true;
    }

    float tClock = (float)(int)(clock - cpWork->m_BaseClock);

    // SetValue() 時に、停止フラグが渡されていた場合、
    // 最新の標本値を推測値として採用する


    if (accessor.GetSample(0).m_IsStop)
    {
        *pValue = accessor.GetSample(0).m_Value;
        return true;
    }


    // 事前に計算した数値を用いて、値を推測する

    calcEstimation(&pValue->x, cpWork->m_Param.x, cpWork->m_ParamBase.x, tClock);
    calcEstimation(&pValue->y, cpWork->m_Param.y, cpWork->m_ParamBase.y, tClock);
    calcEstimation(&pValue->z, cpWork->m_Param.z, cpWork->m_ParamBase.z, tClock);
    /*
    if( accessor.GetNum() >= 3 ){


        float calcClock;

        if( m_IsGetTiming == true ){
            calcClock = (float)( clock ) - m_DelayClock;
        }else{

            calcClock = (float)( clock);
       }

        calcEstimationDelay( pValue , accessor , calcClock );

    }else{

    calcEstimation(&pValue->x, cpWork->m_Param.x, cpWork->m_ParamBase.x,  tClock );
    calcEstimation(&pValue->y, cpWork->m_Param.y, cpWork->m_ParamBase.y,  tClock );
    calcEstimation(&pValue->z, cpWork->m_Param.z, cpWork->m_ParamBase.z,  tClock );

    }
*/
    return true;
}


bool Simple3dReckoningStrategy::CheckSample(bool* pIsReliable, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{

    // 標本値がひとつも存在しない場合、到達保証をしてSetValue() された値を送信
    if (accessor.GetNum() <= 0)
    {
        *pIsReliable = true;
        return true;
    }

    // 1つ前の標本値に停止フラグが立っておらず、今回の標本値には停止フラグが立っていた場合、
    // 到達保証をしてSetValue() された値を送信
    if (accessor.GetSample(0).m_IsStop != true && *cpSetValueArg == true)
    {
        *pIsReliable = true;
        return true;
    }

    // 標本値が足りない等、推測値が計算できなかった場合、SetValue() された値を送信
    Value estimatedValue;
    bool r = Estimate(&estimatedValue, accessor, clock, cpWork);
    if (r == false)
    {
        return true;
    }

    // 推測値とSetValue() された値の間に設定した閾値以上の差があった場合、SetValue() された値を送信

    float dis = value.SquaredDistance(estimatedValue);
    bool isCheck = dis >= m_Threshold * m_Threshold;

    return isCheck;
}


void Simple3dReckoningStrategy::MakeSample(Sample* pSample, const Value& value, const SetValueArg* cpSetValueArg, const Accessor&, nn::pia::clone::ClockValue, const Work*) const
{

    // 標本値としてSetValue()された値をそのまま用いる
    (*pSample).m_Value = value;
    //
    (*pSample).m_IsStop = *cpSetValueArg;
}

/*!
@brief  2次の評価関数の係数を決定し、Workに保存します。
*/
void Simple3dReckoningStrategy::OnUpdateSample(const Accessor& accessor, int, Work* pWork) const
{

    // 標本値がひとつも存在しない場合
    if (accessor.GetNum() <= 0)
    {
        pWork->m_Param.MakeZero();
        pWork->m_ParamBase.MakeZero();
        pWork->m_BaseClock = 0;
        return;
    }

    Value v0 = accessor.GetSample(0).m_Value;

    //  PIA_SAMPLE_DIRECT_PRINT_ALWAYS("OnUpdate Sample %f \n", v0.x );
    nn::pia::clone::ClockValue c0 = accessor.GetClock(0);

    pWork->m_Param.MakeZero();
    pWork->m_ParamBase = v0;
    pWork->m_BaseClock = c0;


    // 線形に補間するための係数を決定する
    if (accessor.GetNum() >= 2)
    {
        nn::pia::clone::ClockValue c1 = accessor.GetClock(1);

        PIA_ASSERT(c0 != c1);
        {
            float t = (float)(int)(c0 - c1);
            Value paramBase = pWork->m_ParamBase;
            nn::pia::reckoning::Vector3f value1 = accessor.GetSample(1).m_Value;
            nn::pia::reckoning::Vector3f::Mul(&value1, -1.f);
            nn::pia::reckoning::Vector3f::Add(&paramBase, value1);

            calcParameter(&pWork->m_Param.x, paramBase.x, t);
            calcParameter(&pWork->m_Param.y, paramBase.y, t);
            calcParameter(&pWork->m_Param.z, paramBase.z, t);

            return;
        }
    }
}

// デバッグに有用な情報のプリント。
void Simple3dReckoningStrategy::Trace(uint64_t flag) const
{
#if NN_PIA_ENABLE_TRACE
    PIA_TRACE(flag, "Simple3dReckoningStrategy::Trace() called.");
#else   // else of NN_PIA_ENABLE_TRACE
    NN_PIA_UNUSED(flag);
#endif  // end of NN_PIA_ENABLE_TRACE
}
}
}
}  // end of namespace nn::pia::reckoning
