﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>
#include <nn/pia/session/session_Mesh.h>

#include <nn/pia/common/common_AsyncContext.h>
#include <nn/pia/common/common_CryptoSetting.h>
#include <nn/pia/common/common_ObjList.h>
#include <nn/pia/common/common_String.h>

#include <nn/pia/transport/transport_NetworkFactory.h>
#include <nn/pia/transport/transport_Transport.h>
#include <nn/pia/transport/transport_Station.h>
#include <nn/pia/transport/transport_StationConnectionInfo.h>

namespace nn
{
namespace pia
{
namespace session
{

// 先行宣言
class CreateSessionSetting;
class JoinRandomSessionSetting;
class JoinSessionSetting;
class SessionStatusCheckJob;
class CreateSessionJob;
class AutoMatchmakeJob;
class BrowseMatchmakeJob;
class JoinSessionJob;
class DestroySessionJob;
class LeaveSessionJob;
class OpenParticipationJob;
class CloseParticipationJob;
class GenerateMatchmakeSystemPasswordJob;
class ClearMatchmakeSystemPasswordJob;
class JointSessionJob;
class UpdateSessionSettingJob;
class UpdateApplicationDataJob;
class CommunityManagementJob;
class CreateCommunitySetting;
class JoinCommunitySetting;
class CommunitySearchCriteria;
class ICommunityInfoList;
class UpdateCommunitySetting;

class StationIdStatusTable;
class SessionProtocol;
class IMatchmakeSession;
class ISessionInfo;
class ISessionInfoList;
class MeshLayerController;
class MatchNetworkFactory;
class SessionSearchCriteria;
class ISessionInfo;
class ISessionInfoList;
class MeshEventListener;
class UpdateSessionSetting;
class ProcessHostMigrationJob;

#if NN_PIA_NINTENDOSDK && NN_PIA_ENABLE_NSA
class AddPlayHistoryJob;
#endif
/*!
  @class    nn::pia::session::Session
  @brief    マッチメイク処理と P2P 通信用セッションの構築／参加処理をまとめるクラスです。

  @details  このクラスはシングルトンパターンです。単一のインスタンスしか生成できません。
  <br>
            Session 内部の処理の一部は @ref common::Scheduler にジョブとして登録されますので、
            @ref common::Scheduler::Dispatch を定期的に呼び出す必要があります。
 */
class Session : public common::RootObject
{
    friend class SessionStatusCheckJob;
    friend class CreateSessionJob;
    friend class AutoMatchmakeJob;
    friend class JoinSessionJob;
    friend class JointSessionJob;
    friend class DestroySessionJob;
    friend class LeaveSessionJob;
    friend class OpenParticipationJob;
    friend class CloseParticipationJob;
    friend class SessionProtocol;
    friend class StationIdStatusTable;
    friend class MeshEventListener;
    friend class MeshLayerController;
    friend class UpdateApplicationDataJob;
    friend class UpdateSessionSettingJob;
    friend class GenerateMatchmakeSystemPasswordJob;
    friend class ClearMatchmakeSystemPasswordJob;
    friend class CommunityManagementJob;
    friend class ProcessHostMigrationJob;
#if NN_PIA_NINTENDOSDK && NN_PIA_ENABLE_NSA
    friend class AddPlayHistoryJob;
#endif

public:
    /*!
        @brief  @ref session::Session::CreateInstance() に各種パラメータを渡すための構造体です。
        @see    CreateInstance
    */
    struct Setting
    {
        Setting(void)
            : pNetworkFactory(NULL),
              networkTopology(NetworkTopology_FullMesh),
              browsedSessionInfoListNum(0),
              browsedCommunityInfoListNum(0),
              bitRateCheckMode(BitRateCheckMode_Disable)
        {
        }

        transport::NetworkFactory* pNetworkFactory;  //!< ネットワーク毎に指定する NetworkFactory のポインタです。
        NetworkTopology networkTopology;             //!< セッションのネットワークトポロジー設定です。@if CTR_DOC ローカル通信、Lan 通信時には NetworkTopology_FullMesh 以外の値は指定できません。@endif @if CAFE_DOC Lan 通信時には NetworkTopology_FullMesh 以外の値は指定できません。@endif @if NIN_DOC ローカル通信、Lan 通信時には NetworkTopology_FullMesh 以外の値は指定できません。@endif
        uint16_t browsedSessionInfoListNum;          //!< セッションの検索結果を格納するリストのサイズです。
        uint8_t browsedCommunityInfoListNum;         //!< コミュニティの検索結果を格納するリストのサイズです。インターネット通信時以外は 0 （デフォルト値）を設定してください。
        BitRateCheckMode bitRateCheckMode;           //!< ビットレート判定機能の設定です。
    };

    /*!
    @brief  （デバッグ用） @ref session::Session::SetDebugSetting に渡すデバッグ機能設定用構造体です。
    */
    struct DebugSetting
    {
        DebugSetting(bool argIsRelayEmulationEnabled = false, bool argIsMatchmakeAnalysisEnabled = false, bool argIsCheckMaxSilenceTimeEnabled = true)
            : isRelayEmulationEnabled(argIsRelayEmulationEnabled),
              isMatchmakeAnalysisEnabled(argIsMatchmakeAnalysisEnabled),
              isCheckMaxSilenceTimeEnabled(argIsCheckMaxSilenceTimeEnabled)
        {
        }
        bool isRelayEmulationEnabled;       //!< デバッグ用接続経路エミュレーション機能設定です。
        bool isMatchmakeAnalysisEnabled;    //!< マッチメイク分析機能用設定です。
        bool isCheckMaxSilenceTimeEnabled;  //!< 最大無通信時間の設定値の範囲チェックを行います。
    };

    /*!
    @brief  @ref session::Session::Startup に渡す設定用構造体です。
    @details アプリケーションが本構造体のインスタンスを作成することはありません。 本構造体を継承した構造体を使用する必要があります。

    @if CTR_DOC
    @see local::LocalSessionStartupSetting
    @endif
    @if NIN_DOC
    @see local::LocalSessionStartupSetting
    @endif
    @see inet::NexSessionStartupSetting
    @see lan::LanSessionStartupSetting
     */
    struct StartupSetting
    {
    protected:
        StartupSetting()
            : maxSilenceTime(SessionMaxSilenceTimeDefault),
              keepAliveSendingInterval(SessionKeepAliveIntervalDefault),
              pToken(NULL)
        {
        }

    public:
        uint32_t maxSilenceTime;                                //!< 許容される最大無通信時間 [msec] を指定します。デフォルトは session::SessionMaxSilenceTimeDefault です。設定可能な最大値は session::SessionMaxSilenceTimeMax, 最小値は session::SessionMaxSilenceTimeMin です。
        uint32_t keepAliveSendingInterval;                      //!< キープアライブの送信間隔 [msec] を指定します。デフォルトは session::SessionKeepAliveIntervalDefault です。
        const transport::Station::IdentificationToken* pToken;  //!< 自分自身を示す識別トークンのポインタを指定します。NULL を指定した場合、データが全て NULL の識別トークンが使用されます。デフォルトは NULL です。
    };

    /*
        @cond PRIVATE
        @brief  Session の状態を表す列挙型です。
    */
    enum ModuleState
    {
        ModuleState_Initialized = 0,  //!< 初期化完了
        ModuleState_StartupComplete   //!< マッチメイク処理実行可能
    };
    //! @endcond

    /*!
        @cond PRIVATE
        @brief  マッチメイクの状態を表す列挙型です。
    */
    enum MatchmakeStatus
    {
        MatchmakeStatus_NotInSession = 0,        //!< 非接続状態
        MatchmakeStatus_JoinedMatchmake,         //!< マッチメイクセッションへ参加済み
        MatchmakeStatus_EstablishedSession,      //!< セッションへ参加済み
        MatchmakeStatus_ChangingSession,         //!< ジョイントセッション機能によるセッション移行処理中（送受信不可期間有り）
        MatchmakeStatus_EstablishedJointSession  //!< ジョイントセッションへ参加済み
    };
    //! @endcond

    /*!
        @cond PRIVATE
        @brief  接続状態を表す列挙型です。
    */
    enum ConnectionStatus
    {
        ConnectionStatus_NotConnected,                          //!< 通信していません。
        ConnectionStatus_Connected,                             //!< 通信しています。
        ConnectionStatus_DisconnectedSession,                   //!< セッションから切断されました。@ref LeaveSessionAsync でセッションから離脱した後、@ref Cleanup を呼び出す必要があります。
        ConnectionStatus_DisconnectedNetwork,                   //!< ネットワークから切断されました。再度セッションに参加するためには、一度ログアウトしてから再度ログインする必要があります。
        ConnectionStatus_DisconnectedNetworkByDuplicateLogin,   //!< 他の端末からの同一アカウントでのログインによりゲームサーバーから切断されました。
        ConnectionStatus_DisconnectedNetworkByWifiOff,          //!< @if NIN_DOC ローカル通信中に機内モードが有効になり、ネットワークから切断されました。@endif
        ConnectionStatus_DisconnectedNetworkBySleep             //!< @if NIN_DOC ローカル通信中にスリープされ、ネットワークから切断されました。@endif
    };
    //! @endcond

    /*!
        @cond PRIVATE
        @brief  合流処理を表す列挙型です。
    */
    enum ProcessType
    {
        ProcessType_None = 0,                //!< 処理を行っていない場合。（初期化用の値）
        ProcessType_MergeSession,            //!< 現セッション参加者全員を引き連れて別セッションへ合流する処理。合流先は指定する。
        ProcessType_MergeRandomSession,      //!< 現セッション参加者全員を引き連れて別セッションへ合流する処理。合流先はランダムマッチメイクで決定。
        ProcessType_DemergeSession,          //!< セッションを分割して新セッションへ独立する処理。
        ProcessType_EmigrateSession,         //!< 別セッションへ移住する処理。移住先は指定する。
        ProcessType_EmigrateRandomSession,   //!< 別セッションへ移住する処理。移住先はランダムマッチメイクで決定。
        ProcessType_CreateJointSession,      //!< ジョイントセッションを作成し、現セッション参加者全員を引き連れていく処理。
        ProcessType_JoinJointSession,        //!< ジョイントセッションを指定し、現セッション参加者全員を引き連れていく処理。
        ProcessType_JoinRandomJointSession,  //!< ランダムマッチメイクでジョイントセッションを指定し、現セッション参加者全員を引き連れていく処理。
        ProcessType_DestroyJointSession,     //!< ジョイントセッションを解散する処理。
        ProcessType_LeaveJointSession,       //!< ジョイントセッションから離脱する処理。
        ProcessType_Max
    };
    /* @endcond */

    /*!
        @cond PRIVATE
        @brief  メッシュ離脱系非同期処理の種類を区別するための列挙型。
     */
    enum LeaveMeshProcessType
    {
        LeaveMeshProcessType_Unknown = 0,
        LeaveMeshProcessType_Leave,
        LeaveMeshProcessType_LeaveWithHm,
        LeaveMeshProcessType_Destroy
    };
    /* @endcond */

    /*!
    @brief  Sesssion が通信不可能な状態の場合に、通信不可能になった理由を表す列挙型です。

    @details  @ref GetSessionDisconnectReason で取得可能です。
     */
    enum DisconnectReason
    {
        DisconnectReason_UnknownReason = 0,   //!< 通信不可能になった理由が不明です。通信中に（破棄、離脱などを行わずに） Session::Cleanup を行った場合などにこの値となります。
        DisconnectReason_NotYetCommunicated,  //!< まだ通信したことがない場合です。 Session::Startup 後に一度もセッション構築、参加をしていない場合はこの値となります。
        DisconnectReason_OperationOfOwn,      //!< 自分自身の操作により通信不可能になった場合です。 Session::LeaveSessionAsync などを呼び出してセッション離脱した場合はこの値となります。
        DisconnectReason_OperationOfOther,    //!< セッション内の自分以外のステーションの操作により通信不可能になった場合です。自身がクライアントでホストがセッション破棄した場合などはこの値となります。
        DisconnectReason_KickoutByUser,       //!< Session::KickoutStation 呼出によりセッションから追放された場合です。
        DisconnectReason_KickoutBySystem,     //!< ライブラリ内部処理によってセッションから追放された場合です。リレー通信機能有効時のリレー依頼数制限などのライブラリ内部処理によってセッションから追放された場合はこの値となります。
        DisconnectReason_InconsistentInfo,    //!< セッション情報が不整合な状態になったので通信不可能になった場合です。ステーション数不一致などが原因の場合はこの値となります。
        DisconnectReason_MigrationFail,       //!< ホストマイグレーション処理に失敗したことにより通信不可能になった場合です。
        DisconnectReason_ExternalFactor,      //!< セッション以外の要因で通信不可能になった場合です。通信障害などが原因の場合はこの値となります。
        DisconnectReason_MigrationFatalError  //!< ホストマイグレーション関連で致命的なエラーが発生した場合です。セッションが分裂状態になったことをライブラリ内部処理によって検知した場合などに発生します。
    };

    /*!
        @brief  セッションの状態を表す列挙型です。

        @details  Session::GetStatus で取得可能です。
    */
    enum Status
    {
        Status_NotConnected,                        //!< 通信していません。
        Status_ConnectedSession,                    //!< セッションへ参加済みです。
        Status_MigratingSession,                    //!< ジョイントセッション機能によるセッション移行処理中です。（送受信不可期間有り）
        Status_ConnectedJointSession,               //!< ジョイントセッションへ参加済みです。
        Status_DisconnectedSession,                 //!< セッションから切断されました。@ref LeaveSessionAsync を 呼んだ後 @ref Cleanup を呼び出す必要があります。
        Status_DisconnectedNetwork,                 //!< ネットワークから切断されました。セッションに再度参加するためには、一度ログアウトしてから再度ログインする必要があります。
        Status_DisconnectedNetworkByDuplicateLogin, //!< 他の端末からの同一アカウントでのログインによりゲームサーバーから切断されました。
        Status_DisconnectedNetworkByWifiOff,        //!< @if NIN_DOC ローカル通信中に機内モードが有効になり、ネットワークから切断されました。@endif
        Status_DisconnectedNetworkBySleep           //!< @if NIN_DOC ローカル通信中にスリープされ、ネットワークから切断されました。@endif
    };

    /*!
    @brief  セッションの開閉状態を表す列挙型です。

    @details  Session::GetSessionOpenStatus, Session::GetJointSessionOpenStatus で取得可能です。
    */
    enum SessionOpenStatus
    {
        SessionOpenStatus_Unknown,  //!< 参加募集状態は不明（クライアントのみ）
        SessionOpenStatus_Open,     //!< 参加募集状態
        SessionOpenStatus_Close     //!< 参加締切状態
    };

    /*!
        @brief  セッション状態変化通知コールバックで引数として渡される列挙型です。

        @details セッションに発生した状態変化の内容を表現します。
    */
    enum EventType
    {
        EventType_EventJoin = 0,                 //!< ステーションが参加したことを示します。
        EventType_EventLeave,                    //!< ステーションが離脱したことを示します。
        EventType_SessionHostChanged,            //!< セッションのホスト権限が移譲されたことを示します。【ホストマイグレーション有効時のみ発生】
        EventType_HostMigrationFailed,           //!< ホストマイグレーションに失敗したことを示します。【ホストマイグレーション有効時のみ発生】
        EventType_JointSessionHostChanged,       //!< ジョイントセッションのホスト権限が移譲されたことを示します。【ジョイントセッション機能使用時、かつホストマイグレーション有効時のみ発生】
        EventType_StartSessionCreateJoint = 11,  //!< ジョイントセッションの構築処理を開始したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_EndSessionCreateJoint,         //!< ジョイントセッションの構築処理が完了したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_StartSessionJoinJoint,         //!< ジョイントセッションの参加処理を開始したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_EndSessionJoinJoint,           //!< ジョイントセッションの参加処理が完了したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_StartSessionJointRandom,       //!< ランダムマッチメイクによるジョイントセッション処理を開始したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_EndSessionJointRandom,         //!< ランダムマッチメイクによるジョイントセッション処理が完了したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_StartSessionDestroyJoint,      //!< ジョイントセッションの解散処理を開始したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_EndSessionDestroyJoint,        //!< ジョイントセッションの解散処理が完了したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_StartSessionLeaveJoint,        //!< ジョイントセッションの離脱処理を開始したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_EndSessionLeaveJoint,          //!< ジョイントセッションの離脱処理が完了したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_SessionMigrationFailed,        //!< ジョイントセッション機能によるセッション移行処理が失敗したことを示します。【ジョイントセッション機能使用時のみ発生】
        EventType_SetSessionSystemPassword,      //!< セッションのシステムパスワードが設定されました。
        EventType_ClearSessionSystemPassword     //!< セッションのシステムパスワードが解除されました。
    };

    /*!
    @brief セッションの状態が変化した場合に呼び出されるコールバックの関数型です。
    
    @details 【引数仕様】
           <ul>
           <li> eventType ： 状態変化の内容を示す列挙型（@ref EventType 列挙型参照）</li>
           <li> id ： 状態変化が発生した対象の StationId</li>
           <li> pApplicationParam : アプリケーションが登録したパラメータを示すポインタ </li>
           </ul>
   
     */
    typedef void (*SessionEventCallback)(EventType eventType, StationId id, void* pApplicationParam);

    /*!
    @brief  セッションホストとして参加要求を受け取った際に呼び出される承認／拒否判定コールバックの関数型です。

    @details 自分がセッションホストで、クライアントから参加要求を受け取った場合に呼び出されるコールバックです。
             返り値により参加要求を承認するか、拒否するか決定できます。

    @param[in] pRequesterToken 参加要求の送信元ステーションの識別トークンを示すポインタです。
 
    @return 参加要求を承認する場合は true、参加要求を拒否する場合は false を返します。
    */
    typedef bool (*JoiningApprovalCallback)(const transport::Station::IdentificationToken* pRequesterToken);


    typedef common::ObjList<StationId> TStationIdList;

    /*!
     @brief セッション内で有効な StationId を参照するためのイテレータです（非 const 版）。
     */
    class StationIdIterator
    {
    public:
        StationIdIterator(TStationIdList::Iterator it)
            : m_Iterator(it)
        {
        }
        StationId& operator*(void)
        {
            return *m_Iterator;
        }
        StationId* operator->(void)
        {
            return &(*m_Iterator);
        }
        bool operator==(const StationIdIterator& rhs) const
        {
            return m_Iterator == rhs.m_Iterator;
        }
        bool operator!=(const StationIdIterator& rhs) const
        {
            return !(*this == rhs);
        }
        StationIdIterator& operator++(void)
        {
            ++m_Iterator;
            return *this;
        }

    private:
        TStationIdList::Iterator m_Iterator;
    };

    /*!
     @brief セッションに参加している StationId を参照するためのイテレータです（ const 版）。
     */
    class StationIdConstIterator
    {
    public:
        StationIdConstIterator(TStationIdList::ConstIterator it)
            : m_ConstIterator(it)
        {
        }
        StationId operator*(void)
        {
            return *m_ConstIterator;
        }
        const StationId* operator->(void)
        {
            return &(*m_ConstIterator);
        }
        bool operator==(const StationIdConstIterator& rhs) const
        {
            return m_ConstIterator == rhs.m_ConstIterator;
        }
        bool operator!=(const StationIdConstIterator& rhs) const
        {
            return !(*this == rhs);
        }
        StationIdConstIterator& operator++(void)
        {
            ++m_ConstIterator;
            return *this;
        }

    private:
        TStationIdList::ConstIterator m_ConstIterator;
    };


    /*!
     @name シングルトンインスタンスの操作
     @{
     */

    /*!
    @brief    Session クラスのインスタンスを作成します（シングルトンパターン）。

    @details  デバッグ機能設定用関数 @ref SetDebugSetting でデバッグ用接続経路エミュレーション機能を使用する設定とした場合、
              ネットワークトポロジー設定が NetworkTopology_RelayMesh でなければこの関数呼出が失敗します。

    @attention  製品版 ROM では、@ref SetDebugSetting を呼び出してはいけません。
    @attention  ローカル通信、Lan 通信時には Setting.networkTopology メンバに NetworkTopology_FullMesh 以外の値は指定できません。
    @attention  ローカル通信、Lan 通信時には Setting.browsedCommunityInfoListNum メンバには 0 を指定してください。 

    @param[in]  setting インスタンス作成時に設定するパラメータ

    @return   インスタンスの作成に成功すれば、成功の Result が返されます。

    @retval   nn::pia::ResultNotInitialized     session モジュールが未初期化です。[:progErr]
    @retval   nn::pia::ResultInvalidState       インスタンスを作成するタイミングが誤っています。[:progErr]
    @retval   nn::pia::ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    @retval   nn::pia::ResultInvalidArgument    引数で設定したパラメータの内容や組み合わせに問題があります。[:progErr]
     */
    static Result CreateInstance(const Setting& setting);


    /*!
    @brief Session クラスのインスタンスを破棄します（シングルトンパターン）。
     */
    static void DestroyInstance();

    /*!
    @brief    Session クラスのインスタンスを得ます（シングルトンパターン）。

    @details 事前に @ref CreateInstance によるインスタンス生成と初期化を済ませていない場合は、
    NULLポインタが返されます。

    @return   Session インスタンスへのポインタ。
     */
    static Session* GetInstance()
    {
        return s_pInstance;
    }


    /*!
    @brief      （デバッグ用） Session のデバッグ用機能について設定します。

    @details    本関数は Session のインスタンスが存在しない状態で呼出可能です。 @ref CreateInstance で Session のインスタンスを作成する前に呼び出します。

    DebugSetting 構造体の isRelayEmulationEnabled を true に設定した場合、デバッグ用接続経路エミュレーション機能を使用する設定となります。
    接続経路エミュレーション機能を使用する設定の場合、ネットワークトポロジーを NetworkTopology_RelayMesh に設定しない限り @ref CreateInstance が成功しません。

    DebugSetting 構造体の isMatchmakeAnalysisEnabled を true に設定した場合、マッチメイク分析機能を使用する設定となります。

    本関数を呼び出さなかった場合、本関数が関連するデバッグ用機能は全て無効（使用しない）設定となります。

    @attention  製品版 ROM では、本関数を呼び出してはいけません。

    @return 関数の実行結果を返します。処理が成功した場合、IsSuccess() が真となる Result を返します。この関数がエラーを返さないようにアプリケーションを実装する必要があります。<br>
            処理が失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidState     関数が実行可能な状態ではありません。既に Session のインスタンスが作成済みの場合に返ります。[:progErr]
     */
    static Result SetDebugSetting(const DebugSetting& debugSetting);

    /*!
     end of name シングルトンインスタンスの操作
     @}
     */


    /*!
     @name 初期化終了処理
     @{
     */

    /*!
    @brief  スタートアップ処理を実行します。

    @details    session::EndSetup() を呼び出して Session の初期化処理を終えた後で呼び出します。<br>
@if NIN_DOC
    ローカル通信ではスタートアップ処理ごとに事前に nn::ldn::Initialize() も呼び出す必要があります。<br>
@endif
    <br>
    引数に指定する StartupSetting 構造体はマッチメイク種別毎にこの構造体を継承する以下の構造体を指定します。
    <ul>
    <li>インターネットマッチメイク時は inet::NexSessionStartupSetting 構造体を指定します。</li>
@if CTR_DOC
    <li>ローカルマッチメイク時は local::LocalSessionStartupSetting 構造体を指定します。</li>
@endif
@if NIN_DOC
    <li>ローカルマッチメイク時は local::LocalSessionStartupSetting 構造体を指定します。</li>
@endif
    <li>LAN マッチメイク時は lan::LanSessionStartupSetting 構造体を指定します。</li>
    </ul>
    マッチメイク種別に依らず、共通して設定できる項目は以下になります。
    <ul>
    <li>maxSilenceTime で、許容される最大無通信時間を設定できます。ステーションとの無通信時間が設定時間を超えると、そのステーションと切断したと判断されます。デフォルト値はネットワーク種別毎に異なります。</li>
    <li>keepAliveSendingInterval で、キープアライブの送信間隔を設定できます。キープアライブとは、アプリ側で送信処理を行っていない相手にシステム側で送信を行い無通信状態と判断されないようにする機能のことです。デフォルト値はネットワーク種別毎に異なります。</li>
    <li>pToken で、自分自身を示す識別トークンを設定することができます。デフォルト値は NULL です。</li>
    </ul>
    インターネットマッチメイク時は以下の項目を設定できます。
    <ul>
    <li>mtuSize で、パケットの MTU を設定できます。@ref inet::MtuSizeMin から @ref inet::MtuSizeMax までの値を指定します。デフォルト値は inet::DefaultMtuSize です。</li>
    <li>isHostMigrationEnabled で、ホストマイグレーション機能の設定を行います。true にすると有効になります。デフォルト値は true です。</li>
    <li>isBitRateCheckSkipped で、ビットレート判定機能を利用する際に、判定に関わる処理をスキップするかどうかを設定します。スキップする場合には true を設定します。デフォルトは false です。</li>
    <li>uplinkBitRateLowerLimit で、ビットレート判定機能を使用する際に、アプリケーションが要求する水準を指定します。単位は bps です。デフォルトは session::SessionInvalidUplinkBitRateLowerLimit ですので、判定機能を使用する際は必ず設定する必要があります。</li>
    <li>bitRateMeasuringSpan で、ビットレート判定機能を利用する際に、計測にかける時間を設定します。単位は msec です。デフォルトは 1000 msec です。デフォルト値からの変更はお勧めしません。</li>
    <li>bitRateCheckPacketSize で、ビットレート判定機能を利用する際に、判定に用いる IP パケットサイズを指定します。単位はバイトです。アプリケーションが送受信する IP パケットの平均的なサイズを指定することを想定しています。4 の倍数でなくてはなりません。デフォルトは 0 ですので、判定機能を使用する際は必ず設定する必要があります。</li>
    <li>cryptoMode で、暗号化設定モードを指定します。デフォルトは common::CryptoSetting::Mode_Aes128 です。</li>
    <li>isAddRecentPlayRecordEnabled で、セッション参加者を「いっしょにあそんだ人の記録」に追加するかどうかを指定します。デフォルトは true（有効）になっています。本体機能が「いっしょにあそんだ人の記録」に対応していない場合は無視されます。</li>
    <li>playerInfo で、自分自身のステーションと関連付けるプレイヤー情報を設定します。MaxPlayerInfoSize まで指定可能です。</li>
    <li>isPlayersCountedAsParticipants で、自分自身のステーションに複数のプレイヤーを関連付ける際、プレイヤー数をセッション参加者数としてカウントするか指定します。デフォルトは true です。</li>
    </ul>
@if CTR_DOC
    ローカルマッチメイク時は以下の項目を設定できます。
    <ul>
    <li>playerInfo で、自分自身のステーションと関連付けるプレイヤー情報を設定します。</li>
    <li>signatureKeyData で、パケットに署名を付与する際の署名鍵を設定します。</li>
    <li>signatureKeyDataSize で、パケットに署名を付与する際に使用する署名鍵のサイズを設定します。署名を無効にする場合は 0 を指定します。@ref common::SignatureSetting::SignatureKeyDataSizeMax 以下である必要があります。</li>
    <li>isAddRecentPlayRecordEnabled で、セッション参加者を「いっしょにあそんだ人の記録」に追加するかどうかを指定します。デフォルトは true（有効）になっています。本体機能が「いっしょにあそんだ人の記録」に対応していない場合は無視されます。</li>
    </ul>
@endif
@if NIN_DOC
    ローカルマッチメイク時は以下の項目を設定できます。
    <ul>
    <li>playerInfo で、自分自身のステーションと関連付けるプレイヤー情報を設定します。</li>
    <li>signatureKeyData で、パケットに署名を付与する際の署名鍵を設定します。</li>
    <li>signatureKeyDataSize で、パケットに署名を付与する際に使用する署名鍵のサイズを設定します。署名を無効にする場合は 0 を指定します。@ref common::SignatureSetting::SignatureKeyDataSizeMax 以下である必要があります。</li>
    <li>isAddRecentPlayRecordEnabled で、セッション参加者を「いっしょにあそんだ人の記録」に追加するかどうかを指定します。デフォルトは true（有効）になっています。本体機能が「いっしょにあそんだ人の記録」に対応していない場合は無視されます。</li>
    </ul>
@endif
    LAN マッチメイク時は以下の項目を設定できます。
    <ul>
    <li>cryptoMode で、暗号化設定モードを指定します。デフォルトは common::CryptoSetting::Mode_Aes128 です。</li>
    <li>playerInfo で、自分自身のステーションと関連付けるプレイヤー情報を設定します。MaxPlayerInfoSize まで指定可能です。</li>
    <li>isPlayersCountedAsParticipants で、自分自身のステーションに複数のプレイヤーを関連付ける際、プレイヤー数をセッション参加者数としてカウントするか指定します。デフォルトは true です。</li>
    </ul>
    <br>
    maxSilenceTime, keepAliveSendingInterval, mtuSize に関して独自の値を設定する場合、実際の動作を確認しながら調整する必要があります。<br>
    識別トークンは接続処理時に接続相手と共有され、@ref transport::Station と結び付けられて保管されます。
    識別トークンを使用する必要が無い場合は、特に設定を行う必要はありません。
    @ref Startup 以外で、自分自身を示す識別トークンを指定、変更する方法はありません。セッション使用中に識別トークンを変更できません。<br>
@if CAFE_DOC
    プレイヤーの名前の長さは20文字（NULL終端を含めない）以内である必要があります。文字コードは UTF-16BE である必要があります。
    言語コードはアプリケーションが使用している言語を、 nn::fp::Language で定義される値で指定します。
    設定したプレイヤー情報はセッション参加者間で共有されます。<br>
    isAddRecentPlayRecordEnabled に true を指定した場合、自分がセッションに参加したときにはセッションの参加者全員を、他のクライアントが
    セッションに参加したときにはその新規参加者のプレイヤー情報を参照し、ライブラリ内で自動的に nn::fp::AddRecentPlayRecord() を使用して
    「いっしょにあそんだ人の記録」に追加します。 <br>
    「いっしょにあそんだ人の記録」についての詳細はフレンドプレゼンスライブラリのプログラミングマニュアルおよび nn::fp::AddRecentPlayRecord() の
    リファレンスを参照してください。 
@endif
@if CTR_DOC
    プレイヤーの名前の長さは20文字（NULL終端を含めない）以内である必要があります。文字コードは UTF-16LE である必要があります。
    言語コードはアプリケーションで別リージョンの言語を判定する場合に指定します。本体の言語設定を使用したい場合には、nn::cfg::CTR::GetLanguage() の値を使用可能です。
    設定したプレイヤー情報はセッション参加者間で共有されます。<br>
    ローカルマッチメイク時は SDK の UDS 通信層で暗号化されるため、暗号化を設定することはできません。
@endif
@if NIN_DOC
    プレイヤーの名前の長さは20文字（NULL終端を含めない）以内である必要があります。文字コードは UTF-8 である必要があります。
    言語コードはアプリケーションが使用している言語を、nn::settings::Language で定義される値で指定します。
    プレイヤー ID にはアカウントシステムに登録されたユーザーの識別子を指定します。ここでの指定を省略したプレイヤーはゲストプレイヤーとして扱われます。
    設定したプレイヤー情報はセッション参加者間で共有されます。<br>
    同じステーションから複数のプレイヤーを参加させる場合は人数分のプレイヤー情報を設定します。
    その際は isPlayersCountedAsParticipant で指定した人数がマッチメイクセッションの参加者としてカウントさせるかも指定します。<br>
    isAddRecentPlayRecordEnabled に true を指定した場合、自分がセッションに参加したときにはセッションの参加者全員を、他のクライアントが
    セッションに参加したときにはその新規参加者のプレイヤー情報を参照し、ライブラリ内で自動的に「いっしょにあそんだ人の記録」に追加します。
    ただし、ゲストプレイヤーの情報は「いっしょにあそんだ人の記録」には追加されません。<br>
    「いっしょにあそんだ人の記録」についての詳細は NintendoSDK ドキュメントの FRIENDS ライブラリの機能、アカウントガイド、本体機能ガイドを参照してください。<br>
    ローカルマッチメイク時は SDK の LDN 通信層で暗号化されるため、暗号化を設定することはできません。<br>
@endif
@if CTR_DOC
    @attention  ローカル通信時には、本関数呼び出し中はスリープモードに遷移してはいけません。
@endif
    @param[in]  startupSetting     スタートアップ時の各種設定項目（マッチメイク種別毎に継承している構造体を指定します）

    @return     処理に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval     nn::pia::ResultNetworkConnectionIsLost ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval     nn::pia::ResultInvalidState 呼び出し時に適切な状態ではありませんでした。ローカル通信時にのみ返ります。[:progErr]
@if NIN_DOC
    @retval     nn::pia::ResultWifiOff 機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval     nn::pia::ResultSleep   スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
@endif
    @see        Cleanup
     */
    Result Startup(const StartupSetting& startupSetting);


    /*!
    @brief      クリーンアップ処理を実行します。
    @details    セッション終了後やエラー発生後に呼び出し、再度 Session の処理を行える状態にします。
@if NIN_DOC
                <br/>ローカル通信ではクリーンアップ処理ごとに、この関数呼び出し後、 nn::ldn::Finalize() も呼び出す必要があります。<br>
@endif
                @ref CheckConnectionError が @ref ResultSessionConnectionIsLost を返した場合は、
                本関数を呼び出す前に @ref LeaveSessionAsync を呼び出します。

    @ref Cleanup 後に再度 Session を使用する場合には、 Startup を呼び出す必要があります。<br>
        
    @see    Startup
     */
    void Cleanup();

    /*!
     end of name 初期化終了処理
     @}
     */

    /*!
     @name セッションの操作
     @{
     */

    /*!
     @brief     セッション API の非同期処理が行われているかを取得します。
     @return    非同期処理が行われている場合は true, 非同期処理が行われていない場合は false が返ります。
     */
     bool IsAsyncProcessInProgress() const;

    /*!
    @brief    ランダムマッチメイクを行いセッションの構築／参加を行います。
    @details <br/>
           [:disable-local]
           ランダムマッチメイクを行った後、セッションの構築／参加を行う非同期処理を開始します。開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsJoinRandomSessionCompleted でチェックし、結果は @ref GetJoinRandomSessionResult で確認できます。
           非同期処理が成功していた場合は、@ref GetSessionInfo でセッション情報を取得できますが、非同期処理中にセッションホストが @ref UpdateSessionSettingAsync でセッション情報を更新した場合、
           取得する情報が更新前の情報となる場合があります。

           JoinRandomSessionSetting に設定する検索条件はインターネット通信時のみ複数指定でき、最大で @ref inet::SearchCriteriaListSizeMax まで指定できます。
           複数の検索条件を指定した場合は、リストに最初に指定された条件から順に検索を行い、参加できた段階で打ち切ります。
           検索条件として指定できる数より多くの条件を設定すると nn::pia::ResultInvalidArgument が返ります。
           参加が締め切られているセッション、既に満員のセッションは自動的に検索対象外となります。
           また、過去 3 回にわたってクライアントとして参加したセッションは、サーバー側で記録され、参加対象から外れるようになっています。この参加記録は約 1 時間で無効になります。

    @param[in]  pJoinRandomSessionSetting   ランダムマッチメイクの設定
    
    @return   非同期処理の開始に成功すれば、成功の Result が返されます。

    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。ローカル通信では対応していません。[:progErr]
                                                インターネット通信時は他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsJoinRandomSessionCompleted, GetJoinRandomSessionResult
     */
    Result JoinRandomSessionAsync(const JoinRandomSessionSetting* pJoinRandomSessionSetting);

    /*!
    @brief    JoinRandomSessionAsync の非同期処理の完了をチェックします。
    @details  [:disable-local]
              @ref JoinRandomSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    JoinRandomSessionAsync, GetJoinRandomSessionResult
     */
    bool IsJoinRandomSessionCompleted() const;

    /*!
    @brief    JoinRandomSessionAsync の非同期処理の結果を取得します。
    @details  [:disable-local]
              @ref JoinRandomSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument                                      非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState                                         非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost                              ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerMaintenance                                ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultNatCheckFailed                                       NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultDnsFailed                                            DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultSessionIsNotFound                                    非同期処理途中で対象のセッションが消失もしくは締め切られました。[:handling-cleanup]
    @retval nn::pia::ResultCancelled                                            非同期処理がキャンセルされました。[:handling-cleanup]
    @retval nn::pia::ResultStationConnectionFailed                              接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultStationConnectionNatTraversalFailedUnknown           NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEim                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdm                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEim                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdm                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEimSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdmSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalRequestTimeout                           NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultJoinRequestDenied                                    セッションホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultMeshIsFull                                           参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedNoCandidate                               リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRttLimit                                  リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRelayNumLimit                             リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedUnknown                                   リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultInvalidSystemMessage                                 セッションホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval nn::pia::ResultIncompatibleFormat                                   相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval nn::pia::ResultNexInternalError                                     [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted                             ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval nn::pia::ResultSessionWrongState                                    参加したセッションの状態が異常でした。[:handling-cleanup]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                                ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin              他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    @retval nn::pia::ResultLanLowerVersion                                      アプリケーションの通信バージョンが参加を行おうとしたセッションのものよりも低いです。LAN マッチメイク時にのみ返ります。[:handling-cleanup]
    @retval nn::pia::ResultLanHigherVersion                                     アプリケーションの通信バージョンが参加を行おうとしたセッションのものよりも高いです。LAN マッチメイク時にのみ返ります。[:handling-cleanup]

    @see    JoinRandomSessionAsync, IsJoinRandomSessionCompleted
    */
    Result GetJoinRandomSessionResult() const;

    /*!
    @brief    JoinRandomSessionAsync の非同期処理でセッション参加時のスコアを取得します。
    @details  インターネット通信時のみ有効です。
    スコアベースマッチメイク指定で @ref JoinRandomSessionAsync の非同期処理が完了した後、本関数でスコアを取得できます。
    ただし、自分がセッションホストの場合はスコアは取得できません。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument                                      引数で指定されたポインタが有効ではない場合に返ります。[:progErr]
    @retval nn::pia::ResultInvalidState                                         非同期処理が完了していない場合、開始されていない場合、スコアベースマッチメイクを指定していない、自分がセッションホストであるなど、スコアが取得できない状態の場合に返ります。[:progErr]
    */
    Result GetJoinRandomSessionScore(uint32_t* pScore) const;

    /*!
    @brief    JoinRandomSessionAsync の非同期処理をキャンセルします。

    @details    JoinRandomSessionAsync で開始された非同期処理は、本関数でキャンセルできます。
                キャンセル処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                本関数を呼び出した後、ライブラリは速やかに非同期処理のキャンセルを行いますが、
                キャンセルの完了は @ref IsJoinRandomSessionCompleted() でチェックします。

    @return   JoinRandomSessionAsync の非同期処理のキャンセル結果を返します。処理が成功していた場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   ResultInvalidState    JoinRandomSessionAsync の非同期処理が行われていないタイミングで呼び出しています。[:progErr]

    @see    JoinRandomSessionAsync, IsJoinRandomSessionCompleted, GetJoinRandomSessionResult
    */
    Result CancelJoinRandomSessionAsync();

    /*!
    @brief    セッションの検索を行います。
    @details
    <br>
           セッション検索を行う非同期処理を開始します。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsBrowseSessionCompleted でチェックし、結果は @ref GetBrowseSessionResult で確認できます。
           検索処理が成功した場合、検索できたセッションのリストが更新され、@ref GetBrowsedSessionInfoList で取得できます。

    @param[in]  pSearchCriteria         セッション検索時の条件

    @return   非同期処理の開始に成功すれば、成功のResultが返されます。

    @retval   nn::pia::ResultInvalidArgument     非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState        呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsBrowseSessionCompleted, GetBrowseSessionResult, GetBrowsedSessionInfoList
     */
    Result BrowseSessionAsync(const SessionSearchCriteria* pSearchCriteria);

    /*!
    @brief    BrowseSessionAsync の非同期処理の完了をチェックします。
    @details @ref BrowseSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    BrowseSessionAsync, GetBrowseSessionResult, GetBrowsedSessionInfoList
     */
    bool IsBrowseSessionCompleted() const;

    /*!
    @brief    BrowseSessionAsync の非同期処理の結果を取得します。
    @details @ref BrowseSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument           非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState              非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval nn::pia::ResultCancelled                 実行中の非同期処理をキャンセルしました。[:handling-cleanup]
    @retval nn::pia::ResultGameServerMaintenance     ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLost   ネットワークが使用できない状態になっています。インターネットマッチメイク時や LAN マッチメイク時には、無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultWifiOff                   機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultSleep                     スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @else
    @retval nn::pia::ResultNetworkConnectionIsLost   ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @ifnot CAFE_DOC
    @retval nn::pia::ResultErrorOccurred      内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @endif

    @see    BrowseSessionAsync, IsBrowseSessionCompleted, GetBrowsedSessionInfoList
    */
    Result GetBrowseSessionResult() const;

    /*!
    @brief    BrowseSessionAsync の非同期処理で検索できたセッションのリストを取得します。
    @details @ref BrowseSessionAsync で開始された非同期処理が完了し、@ref GetBrowseSessionResult が成功した後、本関数でセッションリストを取得できます。

    @return  検索できたセッションのリストのポインタを返します。

    @see    BrowseSessionAsync, IsBrowseSessionCompleted, GetBrowseSessionResult
    */
    ISessionInfoList* GetBrowsedSessionInfoList();

    /*!
    @brief    セッションの構築を行います。
    @details
    <br>
           セッション構築を行う非同期処理を開始します。
           構築したセッションのホストになります。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsCreateSessionCompleted でチェックし、結果は @ref GetCreateSessionResult で確認できます。
           また、非同期処理が成功していた場合は、@ref GetSessionInfo でセッションの情報を取得できます。

    @param[in]  pCreateSessionSetting   セッションを作成する際の条件

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsCreateSessionCompleted, GetCreateSessionResult
     */
    Result CreateSessionAsync(const CreateSessionSetting* pCreateSessionSetting);

    /*!
    @brief    CreateSessionAsync の非同期処理の完了をチェックします。
    @details @ref CreateSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    CreateSessionAsync, GetCreateSessionResult
     */
    bool IsCreateSessionCompleted() const;

    /*!
    @brief    CreateSessionAsync の非同期処理の結果を取得します。
    @details @ref CreateSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval nn::pia::ResultInvalidArgument                          非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState                             非同期処理を進行できない内部状態になりました。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLost                  ネットワークが使用できない状態になっています。インターネットマッチメイク時や LAN マッチメイク時には、無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultWifiOff                                  機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultSleep                                    スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @else
    @retval nn::pia::ResultNetworkConnectionIsLost                  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval nn::pia::ResultGameServerMaintenance                    ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultDnsFailed                                DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatCheckFailed                           NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultCancelled                                非同期処理がキャンセルされました。[:handling-cleanup]
    @retval nn::pia::ResultTimeOut                                  タイムアウトで終了しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval nn::pia::ResultErrorOccurred                            内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @if NIN_DOC
    @retval nn::pia::ResultWirelessControllerCountLimitation        無線コントローラの接続台数が制限を超えています。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @endif
    @retval nn::pia::ResultNexInternalError                         [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted                 ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval nn::pia::ResultSessionWrongState                        参加したセッションの状態が異常でした。[:handling-cleanup]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                    ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin  他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    CreateSessionAsync, IsCreateSessionCompleted
    */
    Result GetCreateSessionResult() const;

    // セッション参加
    /*!
    @brief    セッションへの参加を行います。
    @details
    <br>
           セッション参加を行う非同期処理を開始します。
                      
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsJoinSessionCompleted でチェックし、結果は @ref GetJoinSessionResult で確認できます。
           非同期処理が成功していた場合は、@ref GetSessionInfo でセッション情報を取得できますが、非同期処理中にセッションホストが @ref UpdateSessionSettingAsync でセッション情報を更新した場合、
           取得する情報が更新前の情報となる場合があります。

    @param  pJoinSessionSetting 参加するセッションの条件

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsJoinSessionCompleted, GetJoinSessionResult
     */
    Result JoinSessionAsync(const JoinSessionSetting* pJoinSessionSetting);

    /*!
    @brief    JoinSessionAsync の非同期処理の完了をチェックします。
    @details @ref JoinSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    JoinSessionAsync, GetJoinSessionResult
     */
    bool IsJoinSessionCompleted() const;

    /*!
    @brief   JoinSessionAsync の非同期処理の結果を取得します。
    @details @ref JoinSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval nn::pia::ResultInvalidArgument                                      非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState                                         非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLost                              ネットワークが使用できない状態になっています。インターネットマッチメイク時や LAN マッチメイク時には、無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultWifiOff                                              機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultSleep                                                スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @else
    @retval nn::pia::ResultNetworkConnectionIsLost                              ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval nn::pia::ResultSessionIsNotFound                                    非同期処理途中で対象のセッションが消失もしくは締め切られました。[:handling-cleanup]
    @retval nn::pia::ResultMatchmakeSessionIsFull                               マッチメイクセッションが満員でした。[:handling-cleanup]
    @retval nn::pia::ResultDeniedByParticipants                                 セッション参加者が自分をブラックリストに登録しています。[:handling-cleanup]
    @retval nn::pia::ResultParticipantInBlockList                               セッションに自分がブロックリストに登録している参加者がいました。[:handling-cleanup]
    @retval nn::pia::ResultGameServerMaintenance                                ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultNatCheckFailed                                       NAT チェックに失敗しました。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultDnsFailed                                            DNS 解決に失敗しています。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultCancelled                                            非同期処理がキャンセルされました。[:handling-cleanup]
    @retval nn::pia::ResultInvalidState                                         接続可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultStationConnectionFailed                              接続処理に失敗しました。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultStationConnectionNatTraversalFailedUnknown           NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEim                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdm                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEim                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdm                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEimSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdmSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalRequestTimeout                           NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultJoinRequestDenied                                    セッションホストに接続を拒否されました。 @cond NIN_DOC ローカルマッチメイク時には、参加しようとしたネットワークが満員だった場合にも返ります。 @endcond [:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultMeshIsFull                                           参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedNoCandidate                               リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRttLimit                                  リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRelayNumLimit                             リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedUnknown                                   リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultInvalidSystemMessage                                 セッションホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval nn::pia::ResultSessionUserPasswordUnmatch                           参加時に指定したユーザーパスワードがセッションに設定されたユーザーパスワードと一致しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultSessionSystemPasswordUnmatch                         参加時に指定したシステムパスワードがセッションに設定されたシステムパスワードと一致しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultSessionIsClosed                                      参加を行おうとしたセッションが締め切られていました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultHostIsNotFriend                                      参加を行おうとしたセッションのホストがフレンドではありませんでした。[:handling-cleanup]
    @retval nn::pia::ResultCompanionStationIsOffline                            同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval nn::pia::ResultIncompatibleFormat                                   相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval nn::pia::ResultNexInternalError                                     [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted                             ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval nn::pia::ResultSessionWrongState                                    参加したセッションの状態が異常でした。[:handling-cleanup]
    @if CTR_DOC
    @retval nn::pia::ResultNetworkIsFull                                        ネットワークが満員でした。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval nn::pia::ResultTimeOut                                              タイムアウトで終了しました。 local::UdsJoinSessionSetting で設定した暗号鍵が誤っている場合にも返ります。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval nn::pia::ResultLocalLowerVersion                                    自分の local::LocalNetworkVersion 、または、アプリケーションバージョンが参加を行おうとしたセッションのものよりも低いです。ローカル通信時にのみ返ります。エラー・EULA アプレットでエラーコード付きのエラー表示は行なわず、アプリケーションで独自のメッセージを表示してください。[:handling-cleanup]
    @retval nn::pia::ResultLocalHigherVersion                                   自分の local::LocalNetworkVersion 、または、アプリケーションバージョンが参加を行おうとしたセッションのものよりも高いです。ローカル通信時にのみ返ります。エラー・EULA アプレットでエラーコード付きのエラー表示は行なわず、アプリケーションで独自のメッセージを表示してください。[:handling-cleanup]
    @elseif CAFE_DOC
    @retval nn::pia::ResultTimeOut                                              タイムアウトで終了しました。 [:handling-cleanup]
    @elseif NIN_DOC
    @retval nn::pia::ResultTimeOut                                              タイムアウトで終了しました。ローカルマッチメイク時には、パスワードが一致しなかった場合にも返ります。[:handling-cleanup]
    @retval nn::pia::ResultLocalLowerVersion                                         自分の local::LocalNetworkVersion 、または、ローカル通信バージョンが参加を行おうとしたセッションのものよりも低いです。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultLocalHigherVersion                                        自分の local::LocalNetworkVersion 、または、ローカル通信バージョンが参加を行おうとしたセッションのものよりも高いです。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultWirelessControllerCountLimitation                    無線コントローラの接続台数が制限を超えています。ローカル通信時にのみ返ります。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultUserAccountNotExisted                                ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin              他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    @retval nn::pia::ResultErrorOccurred                                        内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @retval nn::pia::ResultLanLowerVersion                                      アプリケーションの通信バージョンが参加を行おうとしたセッションのものよりも低いです。LAN マッチメイク時にのみ返ります。[:handling-cleanup]
    @retval nn::pia::ResultLanHigherVersion                                     アプリケーションの通信バージョンが参加を行おうとしたセッションのものよりも高いです。LAN マッチメイク時にのみ返ります。[:handling-cleanup]

    @see    JoinSessionAsync, IsJoinSessionCompleted
    */
    Result GetJoinSessionResult() const;

    /*!
    @brief    JoinSessionAsync の非同期処理をキャンセルします。

    @details    JoinSessionAsync で開始された非同期処理は、本関数でキャンセルできます。
                キャンセル処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                本関数を呼び出した後、ライブラリは速やかに非同期処理のキャンセルを行いますが、
                キャンセルの完了は @ref IsJoinSessionCompleted() でチェックします。

    @return   JoinSessionAsync の非同期処理のキャンセル結果を返します。処理が成功していた場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval   ResultInvalidState    JoinSessionAsync の非同期処理が行われていないタイミングで呼び出しています。[:progErr]

    @see    JoinSessionAsync, IsJoinSessionCompleted, GetJoinSessionResult
    */
    Result CancelJoinSessionAsync();

    /*!
    @brief    セッションからの離脱を行います。
    @details
    <br>
           セッション離脱を行う非同期処理を開始します。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsLeaveSessionCompleted でチェックし、結果は @ref GetLeaveSessionResult で確認できます。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていることを確認してください。[:progErr]
    
    @see      IsLeaveSessionCompleted, GetLeaveSessionResult
     */
    Result LeaveSessionAsync();

    /*!
    @brief   LeaveSessionAsync の非同期処理の完了をチェックします。
    @details @ref LeaveSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    LeaveSessionAsync, GetLeaveSessionResult
     */
    bool IsLeaveSessionCompleted() const;

    /*!
    @brief   LeaveSessionAsync の非同期処理の結果を取得します。
    @details @ref LeaveSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。
    
    @retval nn::pia::ResultInvalidState         処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]

    @see    LeaveSessionAsync, IsLeaveSessionCompleted
     */
    Result GetLeaveSessionResult() const;

    // セッション参加受付開始
    /*!
    @brief    セッションの参加受付を開始します。
    @details
    <br>
           セッション参加受付を開始する非同期処理を開始します。
           セッションホストであるときに実行できます。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsOpenSessionCompleted でチェックし、結果は @ref GetOpenSessionResult で確認できます。<br>
           UpdateSessionSetting の派生クラスのインスタンスを引数に指定する場合、参加受付の開始と同時にセッションの設定を更新します。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、セッション参加状態であること、セッションホストであることを確認してください。[:progErr]

    @see    IsOpenSessionCompleted, GetOpenSessionResult, IsHost
     */
    Result OpenSessionAsync(const UpdateSessionSetting* pUpdateSessionSetting = NULL);

    /*!
    @brief   OpenSessionAsync の非同期処理の完了をチェックします。
    @details @ref OpenSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    OpenSessionAsync, GetOpenSessionResult
     */
    bool IsOpenSessionCompleted() const;

    /*!
    @brief   OpenSessionAsync の非同期処理の結果を取得します。
    @details @ref OpenSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval   nn::pia::ResultInvalidState                               呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLost                    ネットワークが使用できない状態になっています。インターネットマッチメイク時や LAN マッチメイク時には、無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultWifiOff                                    機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSleep                                      スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @else
    @retval   nn::pia::ResultNetworkConnectionIsLost                    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval   nn::pia::ResultSessionConnectionIsLost                    メッシュから切断されました。[:handling-leave]
    @retval   nn::pia::ResultSessionIsNotFound                          非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval   nn::pia::ResultNexInternalError                           [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultGameServerProcessAborted                   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultErrorOccurred                              内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]

    @see    OpenSessionAsync, IsOpenSessionCompleted
     */
    Result GetOpenSessionResult() const;

    // セッション参加締め切り
    /*!
    @brief    セッションの参加締切を行います。
    @details
    <br>
           セッション参加を締め切る非同期処理を開始します。
           セッションホストであるときに実行できます。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsCloseSessionCompleted でチェックし、結果は @ref GetCloseSessionResult で確認できます。
    <br>
           本関数の呼び出し直前にセッションに新規参加しようとしているステーションが非同期処理完了後に参加してくる可能性があるため、
           セッション参加締切後、マッチメイクセッションとメッシュの参加者数が一致するまで待ってから非同期処理を完了します。
           また、一定期間（15秒）経過しても一致しない場合も非同期処理を完了します。この場合、@ref GetCloseSessionResult は
           ResultNegligibleFault を返します。非同期処理完了後に新規参加してくる可能性がありますので、アプリケーションで適切にハンドリング
           する必要があります。
           

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、セッション参加状態であること、セッションホストであることを確認してください。[:progErr]

    @see    IsCloseSessionCompleted, GetCloseSessionResult, IsHost
     */
    Result CloseSessionAsync();

    /*!
    @brief   CloseSessionAsync の非同期処理の完了をチェックします。
    @details @ref CloseSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    CloseSessionAsync, GetCloseSessionResult
     */
    bool IsCloseSessionCompleted() const;

    /*!
    @brief   CloseSessionAsync の非同期処理の結果を取得します。
    @details @ref CloseSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval   nn::pia::ResultInvalidState                               呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLost                    ネットワークが使用できない状態になっています。インターネットマッチメイク時や LAN マッチメイク時には、無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultWifiOff                                    機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSleep                                      スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @else
    @retval   nn::pia::ResultNetworkConnectionIsLost                    ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @endif
    @retval   nn::pia::ResultSessionConnectionIsLost                    メッシュから切断されました。[:handling-leave]
    @retval   nn::pia::ResultSessionIsNotFound                          非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval   nn::pia::ResultNexInternalError                           [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultGameServerProcessAborted                   ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultNegligibleFault                            マッチメイクセッションとメッシュの参加人数が一致しない状態で一定時間経過しました。[:handling]
    @retval   nn::pia::ResultErrorOccurred                              内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]

    @see    CloseSessionAsync, IsCloseSessionCompleted
     */
    Result GetCloseSessionResult() const;

    // セッション用システムパスワード生成
    /*!
    @brief    サーバー側でシステムパスワードを生成し、セッションにそのパスワードを設定します。
    @details <br/>
           [:disable-local]
           非同期にサーバー側でシステムパスワードを生成し、セッションにそのパスワードを設定します。
           非同期処理完了時に引数で渡した pSystemPassword に設定されたパスワードが保存されます。
           クライアントでは Session::EventType_SetSessionSystemPassword で通知され、設定されたパスワードは @ref GetSessionSystemPassword で取得できます。
           セッションホストであるときに実行できます。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsGenerateSessionSystemPasswordCompleted でチェックし、結果は @ref GetGenerateSessionSystemPasswordResult で確認できます。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。ローカル通信では対応していません。[:progErr]
                                       インターネット通信時は他の非同期処理が行われていないこと、Startup が呼び出されていること、ホストであることを確認してください。[:progErr]

    @see    IsGenerateSessionSystemPasswordCompleted, GetGenerateSessionSystemPasswordResult, GetSessionSystemPassword, IsHost
     */
    Result GenerateSessionSystemPasswordAsync();

    /*!
    @brief   GenerateSessionSystemPasswordAsync の非同期処理の完了をチェックします。
    @details [:disable-local]
             @ref GenerateSessionSystemPasswordAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    GenerateSessionSystemPasswordAsync, GetGenerateSessionSystemPasswordResult
     */
    bool IsGenerateSessionSystemPasswordCompleted() const;

    /*!
    @brief   GenerateSessionSystemPasswordAsync の非同期処理の結果を取得します。
    @details <br/>
                [:disable-local]
                @ref GenerateSessionSystemPasswordAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval   nn::pia::ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSessionConnectionIsLost  メッシュから切断されました。[:handling-leave]
    @retval   nn::pia::ResultSessionIsNotFound        非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval   nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    GenerateSessionSystemPasswordAsync, IsGenerateSessionSystemPasswordCompleted
     */
    Result GetGenerateSessionSystemPasswordResult() const;

    //! @cond PRIVATE
    Result SetSessionSystemPassword(const common::String& pSystemPassword);
    void ResetSessionSystemPassword();
    //! @endcond
    /*!
    @brief    GenerateSessionSystemPasswordAsync でセッションに設定されたシステムパスワードを取得します。
    @details
    <br>
              [:disable-local]
              GenerateSessionSystemPasswordAsync でセッションに設定されたシステムパスワードを取得します。
              登録されたイベントコールバックに通知された Session::EventType_SetSessionSystemPassword の後、
              セッションに設定されたシステムパスワードを取得できます。

    @return   設定されたシステムパスワード

    @see      GenerateSessionSystemPasswordAsync
     */
    const common::String& GetSessionSystemPassword();

    /*!
    @brief    セッションに設定されたシステムパスワードを解除します。
    @details [:disable-local]
    <br>
           非同期にセッションに設定されたシステムパスワードを解除します。
           クライアントでは Session::EventType_ClearSessionSystemPassword で通知されます。
           セッションホストであるときに実行できます。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsClearSystemPasswordCompleted でチェックし、結果は @ref GetClearSystemPasswordResult で確認できます。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。ローカル通信では対応していません。[:progErr]
                                       インターネット通信時は他の非同期処理が行われていないこと、Startup が呼び出されていること、ホストであることを確認してください。[:progErr]

    @see    IsClearSystemPasswordCompleted, GetClearSystemPasswordResult, IsHost
     */
    Result ClearSessionSystemPasswordAsync();

    /*!
    @brief   ClearSessionSystemPasswordAsync の非同期処理の完了をチェックします。
    @details [:disable-local]
             @ref ClearSessionSystemPasswordAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    ClearSessionSystemPasswordAsync, GetClearSystemPasswordResult
     */
    bool IsClearSystemPasswordCompleted() const;

    /*!
    @brief   ClearSessionSystemPasswordAsync の非同期処理の結果を取得します。
    @details [:disable-local]
                @ref ClearSessionSystemPasswordAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval   nn::pia::ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSessionConnectionIsLost  メッシュから切断されました。[:handling-leave]
    @retval   nn::pia::ResultSessionIsNotFound 非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval   nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    ClearSessionSystemPasswordAsync, IsClearSystemPasswordCompleted
     */
    Result GetClearSystemPasswordResult() const;

    /*!
    @brief    指定されたセッションの設定を更新します。
    @details  非同期に指定したセッションの設定を更新します。
              セッションホストであるときに実行できます。
              開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
              処理の完了は @ref IsUpdateSessionSettingCompleted でチェックし、結果は @ref GetUpdateSessionSettingResult で確認できます。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState   呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、ホストであることを確認してください。[:progErr]

    @param[in] pUpdateSessionSetting 変更する設定をまとめる UpdateSessionSetting 派生クラスのポインタ

    @see    UpdateSessionSetting, IsUpdateSessionSettingCompleted, GetUpdateSessionSettingResult, IsHost
     */
    Result UpdateSessionSettingAsync(const UpdateSessionSetting* pUpdateSessionSetting);

    /*!
    @brief   UpdateSessionSettingAsync の非同期処理の完了をチェックします。
    @details @ref UpdateSessionSettingAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return  非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see     UpdateSessionSettingAsync, GetUpdateSessionSettingResult
     */
    bool IsUpdateSessionSettingCompleted() const;

    /*!
    @brief   UpdateSessionSettingAsync の非同期処理の結果を取得します。
    @details @ref UpdateSessionSettingAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  nn::pia::ResultInvalidState                            呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval  nn::pia::ResultSessionIsNotFound                       非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval  nn::pia::ResultNexInternalError                        [:nexInternalError][:viewer][:handling]
    @retval  nn::pia::ResultGameServerProcessAborted                ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval  nn::pia::ResultErrorOccurred                           内部でエラーが発生しました。ローカル通信時にのみ返ります。[:handling-cleanup]
    @if NIN_DOC
    @retval  nn::pia::ResultNetworkConnectionIsLost                 ネットワークが使用できない状態になっています。インターネットマッチメイク時や LAN マッチメイク時には、無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval  nn::pia::ResultWifiOff                                 機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval  nn::pia::ResultSleep                                   スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval  nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin 他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @else
    @retval  nn::pia::ResultNetworkConnectionIsLost                 ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @endif

    @see     UpdateSessionSettingAsync, IsUpdateSessionSettingCompleted
     */
    Result GetUpdateSessionSettingResult() const;

    /*!
    @brief   同じ条件のセッションが見つかった際のマッチメイクの優先度を更新します。
    @details [:disable-local]
                セッション中に 0, inet::SessionSelectionPriorityMax 以外の値で複数回更新する場合は、
                前回の呼び出しから 30 秒間待つ必要があります。
                この確認には session::Session::IsUpdatableSelectionPriority を使用します。
                inet::SelectionMethod_BroadenRangeWithSelectionPriority で検索を行った場合、マッチメイクの優先度がより大きいセッションが優先されるようになります。
                inet::SelectionMethod_ScoreBased で検索を行った場合、設定したスコアの計算にマッチメイクの優先度が使用されます。
                NMAS のマッチメイクセッションリストにある進行度の値を確認することで設定の確認が行えます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval   nn::pia::ResultInvalidArgument 設定した引数が設定可能な最大値より大きい値です。[:progErr]
    @retval   nn::pia::ResultInvalidState LAN マッチ機能を使用している、ホストではない、のいずれかの理由で実行できません。[:progErr]
    @retval   nn::pia::ResultTemporaryUnavailable 0, inet::SessionSelectionPriorityMax 以外の値で前回の呼び出しから 30 秒以内に呼び出されました。session::Session::IsUpdatableSelectionPriority で呼び出し可能になっていることを確認して呼ぶ必要があります。 
    @retval   nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @param[in] priority 設定するマッチメイクの優先度

    @see    IsUpdatableSelectionPriority
     */
    Result UpdateSelectionPriority(uint8_t priority) const;

    /*!
    @brief   更新する予定の値で UpdateSelectionPriority が使用可能かを返します。
    @param[in] priority 更新予定のマッチメイクの優先度
    @return UpdateSelectionPriority が使用可能であれば、true を返します。
     */
    bool IsUpdatableSelectionPriority(uint8_t priority) const;

    /// @cond PRIVATE
    MeshLayerController* GetMeshLayerControllerPtr()
    {
        return m_pMeshLayerController;
    }
    /// @endcond

    /*!
     end of name セッションの操作
     @}
     */

    /*!
     @name コミュニティの操作
     @{
     */

    /*!
    @brief      コミュニティIDをコミュニティコードに変換します。
    @param[in] communityId    変換したいコミュニティID
    @return     コミュニティIDを元に変換されたコミュニティコードが返ります。
     */
    uint64_t ConvertCommunityIdToCommunityCode(uint32_t communityId);

    /*!
    @brief      コミュニティコードをコミュニティIDに変換します。
    @param[in] communityCode    変換したいコミュニティコード
    @return     コミュニティコードを元に変換されたコミュニティIDが返ります。
     */
    uint32_t ConvertCommunityCodeToCommunityId(uint64_t communityCode);

    /*!
    @brief    コミュニティの構築を行います。
    @details
    <br>
           コミュニティ構築を行う非同期処理を開始します。
           構築したコミュニティのオーナーになります。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsCreateCommunityCompleted でチェックし、結果は @ref GetCreateCommunityResult で確認できます。

    @param[in]  pCreateCommunitySetting   コミュニティを作成する際の条件

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsCreateCommunityCompleted, GetCreateCommunityResult
     */
    Result CreateCommunityAsync(const CreateCommunitySetting* pCreateCommunitySetting);

    /*!
    @brief    CreateCommunityAsync の非同期処理の完了をチェックします。
    @details @ref CreateCommunityAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    CreateCommunityAsync, GetCreateCommunityResult
     */
    bool IsCreateCommunityCompleted() const;

    /*!
    @brief    CreateCommunityAsync の非同期処理の結果を取得します。
    @details @ref CreateCommunityAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval nn::pia::ResultInvalidArgument                  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState                     非同期処理を進行できない内部状態になりました。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost          ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerMaintenance            ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultCreateCommunityFailedUpperLimit  コミュニティの作成上限を超えました。[:handling-cleanup]
    @retval nn::pia::ResultJoinCommunityFailedUpperLimit    コミュニティの参加上限を超えました。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError                 [:nexInternalError][:viewer][:handling]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin  他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    CreateCommunityAsync, IsCreateCommunitynCompleted
    */
    Result GetCreateCommunityResult() const;

    /*!
    @brief    コミュニティへの参加を行います。
    @details
    <br>
           コミュニティ参加を行う非同期処理を開始します。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsJoinCommunityCompleted でチェックし、結果は @ref GetJoinCommunityResult で確認できます。

    @param[in]  pJoinCommunitySetting   参加するコミュニティの条件

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsJoinCommunityCompleted, GetJoinCommunityResult
     */
    Result JoinCommunityAsync(const JoinCommunitySetting* pJoinCommunitySetting);

    /*!
    @brief    JoinCommunityAsync の非同期処理の完了をチェックします。
    @details @ref JoinCommunityAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    JoinCommunityAsync, GetJoinCommunityResult
     */
    bool IsJoinCommunityCompleted() const;

    /*!
    @brief    JoinCommunityAsync の非同期処理の結果を取得します。
    @details @ref JoinCommunityAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval nn::pia::ResultInvalidArgument                  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState                     非同期処理を進行できない内部状態になりました。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost          ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerMaintenance            ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultAlreadyJoinedCommunity           参加しようとしたコミュニティに既に参加していた。[:handling]
    @retval nn::pia::ResultCommunityUserPasswordUnmatch     参加時に指定したユーザーパスワードがコミュニティに設定されたユーザーパスワードと一致しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultCreateCommunityFailedUpperLimit  コミュニティの作成上限を超えました。[:handling-cleanup]
    @retval nn::pia::ResultJoinCommunityFailedUpperLimit    コミュニティの参加上限を超えました。[:handling-cleanup]
    @retval nn::pia::ResultCommunityIsFull                  参加しようとしたコミュニティが満員でした。[:handling-cleanup]
    @retval nn::pia::ResultCommunityIsNotFound              存在しないコミュニティでした。[:handling-cleanup]
    @retval nn::pia::ResultCommunityIsClosed                参加しようとしたコミュニティが参加可能期間外でした。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError                 [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    JoinCommunityAsync, IsJoinCommunitynCompleted
    */
    Result GetJoinCommunityResult() const;

    /*!
    @brief    コミュニティの破棄を行います。
    @details
    <br>
           コミュニティ破棄を行う非同期処理を開始します。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsDestroyCommunityCompleted でチェックし、結果は @ref GetDestroyCommunityResult で確認できます。

    @param[in]  communityId   破棄するコミュニティ ID

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsDestroyCommunityCompleted, GetDestroyCommunityResult
     */
    Result DestroyCommunityAsync(uint32_t communityId);

    /*!
    @brief   DestroyCommunityAsync の非同期処理の完了をチェックします。
    @details @ref DestroyCommunityAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    DestroyCommunityAsync, GetDestroyCommunityResult
     */
    bool IsDestroyCommunityCompleted() const;

    /*!
    @brief   DestroyCommunityAsync の非同期処理の結果を取得します。
    @details @ref DestroyCommunityAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultInvalidState         対象のコミュニティのオーナーではありません、もしくは対象のコミュニティが公式コミュニティです。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultSessionIsNotFound    対象のコミュニティが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    DestroyCommunityAsync, IsDestroyCommunityCompleted
     */
    Result GetDestroyCommunityResult() const;

    /*!
    @brief    コミュニティからの離脱を行います。
    @details
    <br>
           コミュニティ離脱を行う非同期処理を開始します。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsLeaveCommunityCompleted でチェックし、結果は @ref GetLeaveCommunityResult で確認できます。

    @param[in]  communityId   離脱するコミュニティ ID

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsLeaveCommunityCompleted, GetLeaveCommunityResult
     */
    Result LeaveCommunityAsync(uint32_t communityId);

    /*!
    @brief   LeaveCommunityAsync の非同期処理の完了をチェックします。
    @details @ref LeaveCommunityAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    LeaveCommunityAsync, GetLeaveCommunityResult
     */
    bool IsLeaveCommunityCompleted() const;

    /*!
    @brief   LeaveCommunityAsync の非同期処理の結果を取得します。
    @details @ref LeaveCommunityAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultInvalidState         対象のコミュニティに参加していませんでした。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultSessionIsNotFound    対象のコミュニティが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    LeaveCommunityAsync, IsLeaveCommunityCompleted
     */
    Result GetLeaveCommunityResult() const;

    /*!
    @brief    コミュニティの検索を行います。
    @details
    <br>
           コミュニティ検索を行う非同期処理を開始します。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsBrowseCommunityCompleted でチェックし、結果は @ref GetBrowseCommunityResult で確認できます。
           検索処理が成功した場合、検索できたコミュニティのリストが更新され、@ref GetBrowsedCommunityInfoList で取得できます。

    @param[in]  pSearchCriteria         コミュニティ検索時の条件

    @return   非同期処理の開始に成功すれば、成功のResultが返されます。

    @retval   nn::pia::ResultInvalidArgument     非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState        呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、既に通信中でないか確認してください。[:progErr]

    @see      IsBrowseCommunityCompleted, GetBrowseCommunityResult, GetBrowsedCommunityInfoList
     */
    Result BrowseCommunityAsync(const CommunitySearchCriteria* pSearchCriteria);

    /*!
    @brief    BrowseCommunityAsync の非同期処理の完了をチェックします。
    @details @ref BrowseCommunityAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    BrowseCommunityAsync, GetBrowseCommunityResult, GetBrowsedCommunityInfoList
     */
    bool IsBrowseCommunityCompleted() const;

    /*!
    @brief    BrowseCommunityAsync の非同期処理の結果を取得します。
    @details @ref BrowseCommunityAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument           非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState              非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval nn::pia::ResultCancelled                 実行中の非同期処理をキャンセルしました。[:handling-cleanup]
    @retval nn::pia::ResultGameServerMaintenance     ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultNetworkConnectionIsLost   ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    BrowseCommunityAsync, IsBrowseCommunityCompleted, GetBrowsedCommunityInfoList
    */
    Result GetBrowseCommunityResult() const;

    /*!
    @brief    BrowseCommunityAsync の非同期処理で検索できたコミュニティのリストを取得します。
    @details @ref BrowseCommunityAsync で開始された非同期処理が完了し、@ref GetBrowseCommunityResult が成功した後、本関数でコミュニティリストを取得できます。

    @return  検索できたコミュニティのリストのポインタを返します。

    @see    BrowseCommunityAsync, IsBrowseCommunityCompleted, GetBrowseCommunityResult
    */
    ICommunityInfoList* GetBrowsedCommunityInfoList();

    /*!
    @brief    指定されたコミュニティの設定を更新します。
    @details  非同期に指定したコミュニティの設定を更新します。
    コミュニティホストであるときに実行できます。
    開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
    処理の完了は @ref IsUpdateCommunityCompleted でチェックし、結果は @ref GetUpdateCommunityResult で確認できます。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval   nn::pia::ResultInvalidState   呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、ホストであることを確認してください。[:progErr]

    @param[in] pUpdateCommunitySetting 変更する設定をまとめる UpdateCommunitySetting 派生クラスのポインタ

    @see    UpdateCommunitySetting, IsUpdateCommunityCompleted, GetUpdateCommunityResult
    */
    Result UpdateCommunityAsync(const UpdateCommunitySetting* pUpdateCommunitySetting);

    /*!
    @brief   UpdateCommunityAsync の非同期処理の完了をチェックします。
    @details @ref UpdateCommunityAsync で開始された非同期処理は、本関数で完了をチェックできます。
    @return  非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see     UpdateCommunityAsync, GetUpdateCommunityResult
    */
    bool IsUpdateCommunityCompleted() const;

    /*!
    @brief   UpdateCommunityAsync の非同期処理の結果を取得します。
    @details @ref UpdateCommunityAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  nn::pia::ResultInvalidState             呼び出し時に適切な状態ではありませんでした。コミュニティのオーナーではないか、公式コミュニティを更新しようとしていないか、ユーザーパスワード無しのコミュニティにパスワードを設定しようとしていないか、または他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval  nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval  nn::pia::ResultSessionIsNotFound 非同期処理途中で対象のコミュニティが消失しました。[:handling-cleanup]
    @retval  nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval  nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see     UpdateCommunityAsync, IsUpdateCommunityCompleted
    */
    Result GetUpdateCommunityResult() const;


    /*!
     end of name コミュニティの操作
     @}
     */

    /*!
     @name セッション情報の解析
     @{
     */

    /*!
    @brief      自分自身がホストであるかどうかを取得します。
    @details    この関数は通常セッションのホストであるかどうかを返します。

    @return     自分自身がホストの場合は true、そうでない場合は false が返ります。
     */
    bool IsHost() const;

    /*!
    @cond PRIVATE
    @brief      自分自身がメッシュのホストであるかどうかを取得します。
    @details    この関数はメッシュのホストであるかどうかを返します。

    @return     自分自身がホストの場合は true、そうでない場合は false が返ります。
     */
    bool IsMeshHost() const;
    /// @endcond

    /*!
    @cond PRIVATE
    @brief      自分自身がセッションのオーナーであるかどうかを取得します。
    @return     自分自身がセッションのオーナーの場合は true、そうでない場合は false が返ります。
     */
    bool IsOwner() const;
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief   自分自身を示す StationIndex を取得します
    @details 自分自身の StationIndex が確定していない場合は StationIndex_Invalid が返ります。

    @return  自分自身を示す StationIndex。
     */
    StationIndex GetLocalStationIndex() const;

    /*!
    @brief   ホストを示す StationIndex を取得します
    @details セッション参加中でない場合、この関数は StationIndex_Invalid を返します。
                セッション参加中、ジョイントセッション参加中の場合、この関数はセッションのホストを示す StationIndex を返します。

    @return 自身が参加しているセッションのホストを示す StationIndex を返します。
     */
    StationIndex GetHostStationIndex() const;
    /// @endcond

    /*!
    @brief 任意の StationId がセッション内で現在有効かどうかを確認します。
    @param[in] id   確認したい StationId を指定します。

    @return 指定された StationId が有効な場合は true、そうでなければ false が返ります。
     */
    bool IsValidStationId(StationId id) const;

    /// @cond PRIVATE
    bool IsValidStationIndex(StationIndex index) const;
    /// @endcond


    /*!
    @brief  セッション内での自身を示す StationId を取得します。

    @details セッション参加中でない場合、この関数は StationIdInvalid を返します。

    @return 自身を示す StationId を返します。
     */
    StationId GetLocalStationId() const
    {
        return m_LocalStationId;
    }

    /*!
    @brief  自身が参加しているセッションのホストを示す StationId を取得します。

    @details セッション参加中でない場合、この関数は StationIdInvalid を返します。
             セッション参加中、ジョイントセッション参加中の場合、この関数はセッションのホストを示す StationId を返します。

    @return 自身が参加しているセッションのホストを示す StationId を返します。
     */
    StationId GetHostStationId() const
    {
        return m_HostStationId;
    }

    /*!
    @brief  自分自身を示す StationConnectionInfo を取得します。

    @details セッション参加中、ジョイントセッション参加中の場合、この関数は自分自身を示す StationConnectionInfo を返します。
    <br>
    @param[out] pInfoOut 取得したセッションホストを示すアドレスの格納先を示すポインタ
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval     ResultInvalidArgument       引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState          セッションに参加していません。[:progErr]
    @retval     ResultTemporaryUnavailable  セッション移行中です。[:handling]

    @return 自分自身を示す StationConnectionInfo を返します。
     */
    Result GetLocalStationConnectionInfo(transport::StationConnectionInfo* pInfoOut);

    /*!
    @brief  自身が参加しているセッションのホストを示す StationConnectionInfo を取得します。

    @details セッション参加中、ジョイントセッション参加中の場合、この関数はセッションのホストを示す StationConnectionInfo を返します。
    <br>
    @param[out] pInfoOut 取得したセッションホストを示すアドレスの格納先を示すポインタ
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval     ResultInvalidArgument       引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState          セッションに参加していません。[:progErr]
    @retval     ResultTemporaryUnavailable  セッション移行中です。[:handling]

    @return 自身が参加しているセッションのホストを示す StationConnectionInfo を返します。
     */
    Result GetHostStationConnectionInfo(transport::StationConnectionInfo* pInfoOut);

    /*!
    @brief      現在参加しているセッションで接続している有効なステーション数を取得します。
    @details    現在参加しているセッションで接続している有効なステーション数を取得します。セッション参加者が増減する際に更新されます。<br>
                セッションの状態に応じて、以下の値を返します。<br>
                @ref GetStatus が Status_ConnectedSession を返すときは、参加中のセッションで接続しているステーション数を返します。
                @ref GetStatus が Status_MigratingSession を返すときは、不定値を返します。
                @ref GetStatus が Status_ConnectedJointSession を返すときは、参加中のジョイントセッションのセッションで接続しているステーション数を返します。
    @return 接続中のステーション数。
     */
    uint16_t GetStationNum() const;

    /*!
    @brief      現在参加しているセッションで接続しているプレイヤー数を取得します。
    @details    現在参加しているセッションで接続しているプレイヤー数を取得します。セッション参加者が増減する際に更新されます。<br>
                セッションの状態に応じて、以下の値を返します。<br>
                @ref GetStatus が Status_ConnectedSession を返すときは、参加中のセッションで接続しているプレイヤー数を返します。
                @ref GetStatus が Status_MigratingSession を返すときは、不定値を返します。
                @ref GetStatus が Status_ConnectedJointSession を返すときは、参加中のジョイントセッションのセッションで接続しているプレイヤー数を返します。
    @return 接続中のプレイヤー数。
     */
    uint16_t GetPlayerNum() const;

    /*!
    @brief      現在参加しているセッションで接続しているマッチメイクセッション参加者数を取得します。
    @details    現在参加しているセッションで接続しているマッチメイクセッション参加者数を取得します。マッチメイクセッションの参加者が増減する際に更新されます。<br>
                セッションの状態に応じて、以下の値を返します。<br>
                @ref GetStatus が Status_ConnectedSession を返すときは、参加中のマッチメイクセッションに参加しているマッチメイクセッション参加者数を返します。
                @ref GetStatus が Status_MigratingSession を返すときは、不定値を返します。
                @ref GetStatus が Status_ConnectedJointSession を返すときは、参加中のジョイントセッションのマッチメイクセッションに参加しているマッチメイクセッション参加者数を返します。
    @return 接続中のマッチメイクセッション参加者数。
     */
    uint16_t GetMatchmakeSessionParticipantNum() const;

    /*!
    @brief      マッチメイクセッションに参加しているステーション数を取得します。
    @details    マッチメイクセッションに参加しているステーション数を取得します。マッチメイクセッションの参加者が増減する際に更新されます。
                あるステーションがマッチメイクセッションに参加してからセッションに参加するまでの間に、@ref GetStationNum が返す値と一致しない期間があります。<br>
                セッションの状態に応じて、以下の値を返します。<br>
                @ref GetStatus が Status_ConnectedSession を返すときは、参加中のマッチメイクセッションに参加しているステーション数を返します。
                @ref GetStatus が Status_MigratingSession を返すときは、不定値を返します。
                @ref GetStatus が Status_ConnectedJointSession を返すときは、参加中のジョイントセッションのマッチメイクセッションに参加しているステーション数を返します。

    @return マッチメイクセッションに参加中のステーション数。
     */
    uint16_t GetMatchmakeSessionStationNum() const;

    /*!
    @brief  セッション内の有効な StationId の数を取得します。
    @details セッション参加中でない場合、この関数は 0 を返します。
    @return セッション内の有効な StationId の数を返します。
     */
    uint16_t GetValidStationIdNum() const;


    /*!
    @brief  セッションが通信可能な状態か否かを確認します。

    @details    セッションや接続状況に異常が発生した場合、この関数で取得できる Result が変化します。
    @details    セッション参加中に定期的に呼び出して状態を確認する必要があります。
    また、Startup() 呼び出し後から JoinRandomSessionAsync(), CreateSessionAsync(), JoinSessionAsync() などによりセッションに参加するまでの間、
    定期的に呼び出してネットワークの状態を確認することが可能です。
    非同期処理中は、非同期処理内で確認し、切断時には非同期処理結果としてエラーを返すため、呼び出す必要はありません。（非同期処理中は適切な値を返しません）
    通信可能な状態の場合、 @ref GetStatus で詳細な状態を取得できます。
    セッションへ参加した後にメッシュが通信不可能な状態になった場合、 @ref GetSessionDisconnectReason で通信不可能になった詳細な理由を確認可能です。

    @return 通信可能な状態の場合は IsSuccess() が真となる Result を返します。それ以外では以下の Result を返します。
    @retval nn::pia::ResultSessionConnectionIsLost                  メッシュから切断されました。[:handling-leave]
    @retval nn::pia::ResultNetworkConnectionIsLost                  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin  他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @retval nn::pia::ResultWifiOff                                  機内モードが有効になったため無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultSleep                                    スリープによって無線機能が無効になりました。ローカル通信時にのみ返ります。[:viewer][:handling-shutdownNetwork]
    @endif
    */
    Result CheckConnectionError() const;


    /*!
    @brief  セッションの状態を示す @ref Status を取得します。
    @details    セッションや接続状況に異常が発生した場合、この関数で取得できる @ref Status が変化します。定期的に呼び出して状態を確認する必要があります。
    @return 現在の @ref Status を返します。
     */
    Status GetStatus() const;

    /*!
    @brief  ビットレート判定機能でのビットレート値が取得できるかどうかを取得します。

    @return ビットレート値が取得できる状態になっていれば true を返します。タイムアウトやキャンセル等の理由で測定が失敗に終わった場合も true が返されます。
     */
    bool IsBitRateCheckCompleted() const;

    /*!
    @brief   ビットレート判定機能によって計測された、アップリンク方向の回線のビットレートを取得します。

    @details この API 呼び出しで得られる値は、測定処理において単位時間あたりに受信できたデータ量にすぎず、
             必ずしも回線の真の実力を表すものではないことに注意してください。
             実際には高ビットレートが期待できる回線であったとしても、アプリケーション側の要求水準が
             控えめなものであれば、測定処理におけるデータ送受信量も控えめなものとなるので、
             取得できる bps 値は小さめなものとなります。

    @param[out] pBps 測定されたビットレート値がセットされます。単位は bps です。測定値が得られなかった場合は負の値がセットされます。

    @return  呼び出しが成功すれば、IsSuccess() が true を返す Result が返されます。

    @retval  ResultNotInitialized     インスタンスが初期化されていません。[:progErr]
    @retval  ResultInvalidArgument    引数が誤っています。[:progErr]
    @retval  ResultInvalidState       この関数を呼び出すタイミングが誤っています。ビットレート判定機能を利用しない設定であるにもかかわらず、本 API を呼び出している可能性があります。[:progErr]
     */
    Result GetUplinkBitRate(int32_t* pBps);

    /*!
    @brief  ビットレート判定機能が有効かどうかを取得します。

    @return ビットレート判定機能が有効であれば true を返します。
    */
    bool IsBitRateCheckEnabled() const;

    /// @cond PRIVATE
    /*!
    @brief  セッションへの参加状況を示す MatchmakeStatus を取得します。

    @return 現在の MatchmakeStatus を返します。
     */
    MatchmakeStatus GetMatchmakeStatus() const
    {
        return m_MatchmakeStatus;
    }
    /// @endcond

    /// @cond PRIVATE
    /*!
    @brief      接続状態を示す ConnectionStatus を取得します。

    @return     現在の ConnectionStatus を返します。
     */
    ConnectionStatus GetConnectionStatus() const
    {
        return m_ConnectionStatus;
    }
    /// @endcond

    /// @cond PRIVATE
    StationIdStatusTable* GetStationIdStatusTablePtr()
    {
        return m_pStationIdStatusTable;
    }
    SessionProtocol* GetSessionProtocolPtr()
    {
        return m_pSessionProtocol;
    }
    /// @endcond

    /*!
    @brief  セッション ID を取得します。
    @if CTR_DOC
    @details ローカル通信時には同じセッション ID を持つ異なるセッションが存在する場合があります。
    @endif
    @if NIN_DOC
    @details ローカル通信時には同じセッション ID を持つ異なるセッションが存在する場合があります。
    @endif

    @return 現在参加しているセッションのセッション ID を返します。セッションに参加していない状態では InvalidSessionId を返します。
     */
    uint32_t GetSessionId() const
    {
        return GetCurrentSessionId();
    }

    /*!
    @brief      指定したステーション ID のステーションが参加しているセッション ID を取得します。
    @details    指定したステーション ID のステーションが参加しているセッション ID を取得します。

    @param[in]  id  ステーション ID。

    @return 指定したステーションIDのステーションが参加しているセッションのセッション ID を返します。そのステーションがセッションに参加していない状態では InvalidSessionId を返します。
     */
    uint32_t GetSessionId(StationId id) const;

    /// @cond PRIVATE
    void SetSessionId(uint32_t sessionId)
    {
        SetCurrentSessionId(sessionId);
    }
    /// @endcond


    /*!
    @brief  セッション内で有効な StationId のイテレータを返します（非 const 版）。

    @return 呼出時点でセッション内で有効な StationId リストの先頭を示すイテレータを返します。
    @see    End
     */
    StationIdIterator Begin()
    {
        return StationIdIterator(m_ValidStationIdList.Begin());
    }

    /*!
    @brief  セッション内で有効な StationId のイテレータを返します（非 const 版）。

    @return 呼出時点でセッション内で有効な StationId リストの末尾の要素の次を示すイテレータを返します。
    @see    Begin
     */
    StationIdIterator End()
    {
        return StationIdIterator(m_ValidStationIdList.End());
    }

    /*!
    @brief  セッション内で有効な StationId のイテレータを返します（非 const 版）。

    @return 呼出時点でセッション内で有効な StationId リストの先頭を示すイテレータを返します。
    @see    End
     */
    StationIdConstIterator Begin() const
    {
        return StationIdConstIterator(m_ValidStationIdList.ConstBegin());
    }

    /*!
    @brief  セッション内で有効な StationId のイテレータを返します（非 const 版）。

    @return 呼出時点でセッション内で有効な StationId リストの末尾の要素の次を示すイテレータを返します。
    @see    Begin
     */
    StationIdConstIterator End() const
    {
        return StationIdConstIterator(m_ValidStationIdList.ConstEnd());
    }

    /*!
    @brief      セッションの参加募集状態を取得します。
    @details    セッションホストのみ使用可能です。
    @return     セッションの参加募集状態を返します。
    @see        IsHost
     */
    SessionOpenStatus GetSessionOpenStatus() const;

    /*!
    @brief      参加中のセッション情報を取得します。
    @details    セッション情報は自動的には更新されません。セッション情報の更新は @ref RequestSessionInfoAsync で行う必要があります。
                
    @return     参加中のセッション情報を返します。
                @ref CreateSessionAsync 、@ref JoinSessionAsync 、@ref JoinRandomSessionAsync 、@ref RequestSessionInfoAsync の非同期処理完了後から他の非同期処理開始前まで取得可能です。
                他の非同期処理開始後は NULL を返します。
    */
    const ISessionInfo* GetSessionInfo() const;

    /*!
    @brief      参加中のセッション情報の取得要求を行います。
    @details    参加中のセッション情報の取得要求を行う非同期処理を開始します。
                開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
                処理の完了は @ref IsRequestSessionInfoCompleted でチェックし、結果は @ref GetRequestSessionInfoResult で確認できます。
                非同期処理が成功していた場合は、@ref GetSessionInfo でセッション情報を取得できますが、非同期処理中にセッションホストが UpdateSessionSettingAsync でセッション情報を更新した場合、
                取得する情報が更新前の情報となる場合があります。
                <br>
                インターネット通信の場合、ゲームサーバーアクセスを行います。1 分間に 10 回以上の呼び出しを行ってはいけません。
    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。セッションに参加していないか、他の非同期処理が行われていないか確認してください。[:progErr]

    @see    IsRequestSessionInfoCompleted, GetRequestSessionInfoResult

    */
    Result RequestSessionInfoAsync();

    /*!
    @brief      RequestSessionInfoAsync の非同期処理の完了をチェックします。
    @details    @ref RequestSessionInfoAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see        RequestSessionInfoAsync, GetRequestSessionInfoResult
    */
    bool IsRequestSessionInfoCompleted() const;

    /*!
    @brief      RequestSessionInfoAsync の非同期処理の結果を取得します。
    @details    @ref RequestSessionInfoAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidState              非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval nn::pia::ResultSessionIsNotFound         非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLost   ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultSessionConnectionIsLost   メッシュから切断されました。[:handling-leave]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    */
    Result GetRequestSessionInfoResult() const;

    /*!
     end of name セッション情報の解析
     @}
     */

    /*!
     @name ジョイントセッション機能
     @{
     */

    /*!
    @brief      ランダムマッチメイクでジョイントセッションに参加します。
    @details    [:disable-local]
            ホストのみ使用可能です。
            ランダムマッチメイクでジョイントセッションに参加を行う非同期処理を開始します。
            開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            処理の完了は @ref IsJoinRandomJointSessionCompleted でチェックし、結果は @ref GetJoinRandomJointSessionResult で確認できます。

            ホストが本関数で非同期処理を開始した場合、クライアント側でもランダムマッチメイクでジョイントセッションに参加を行う内部処理が自動的に開始されます。
            自動的に開始された処理の進捗は、状態変化イベント通知をハンドリングするか @ref GetStatus で取得できる @ref Status をチェックすることで確認できます。

            非同期処理中は、送受信処理関連など一部の API は一時的に使用できません。

    @param[in]  pJoinRandomSessionSetting   ランダムマッチメイクの設定を指定します。

    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないか、ホストかどうか確認してください。[:progErr]

    @see    IsJoinRandomJointSessionCompleted, GetJoinRandomJointSessionResult
     */
    Result JoinRandomJointSessionAsync(const JoinRandomSessionSetting* pJoinRandomSessionSetting);

    /*!
    @brief      JoinRandomJointSessionAsync の非同期処理の完了をチェックします。
    @details    [:disable-local]
                @ref JoinRandomJointSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    JoinRandomJointSessionAsync, GetJoinRandomJointSessionResult
     */
    bool IsJoinRandomJointSessionCompleted() const;

    /*!
    @brief      JoinRandomJointSessionAsync の非同期処理の結果を取得します。
    @details    [:disable-local]
                @ref JoinRandomJointSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidState                                         自身がジョイントセッション処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultInvalidArgument                                      非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost                              ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultSessionIsNotFound                                    非同期処理途中で対象のセッションが消失もしくは締め切られました。[:handling-cleanup]
    @retval nn::pia::ResultMatchmakeSessionIsFull                               対象のマッチメイクセッションが満員でした。[:handling-cleanup]
    @retval nn::pia::ResultDeniedByParticipants                                 セッション参加者が自分をブラックリストに登録しています。[:handling-cleanup]
    @retval nn::pia::ResultParticipantInBlockList                               セッションに自分がブロックリストに登録している参加者がいました。[:handling-cleanup]
    @retval nn::pia::ResultSessionUserPasswordUnmatch                           参加時に指定したユーザーパスワードがセッションに設定されたユーザーパスワードと一致しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultSessionSystemPasswordUnmatch                         参加時に指定したシステムパスワードがセッションに設定されたシステムパスワードと一致しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultSessionIsClosed                                      参加を行おうとしたセッションが締め切られていました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultHostIsNotFriend                                      参加を行おうとしたセッションのホストがフレンドではありませんでした。[:handling-cleanup]
    @retval nn::pia::ResultGameServerMaintenance                                ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultCompanionStationIsOffline                            同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval nn::pia::ResultCompanionStationIsLeft                               同行するクライアントとの接続が切断しました。[:handling-cleanup]
    @retval nn::pia::ResultSessionMigrationFailed                               セッション移行処理に失敗しました。[:handling-cleanup]
    @retval nn::pia::ResultStationConnectionFailed                              接続処理に失敗しました。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultStationConnectionNatTraversalFailedUnknown           NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEim                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdm                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEim                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdm                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEimSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdmSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalRequestTimeout                           NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:handling-cleanup]
    @retval nn::pia::ResultJoinRequestDenied                                    メッシュホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultMeshIsFull                                           参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedNoCandidate                               リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRttLimit                                  リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRelayNumLimit                             リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedUnknown                                   リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultInvalidSystemMessage                                 メッシュホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval nn::pia::ResultIncompatibleFormat                                   相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval nn::pia::ResultNexInternalError                                     [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted                             ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval nn::pia::ResultSessionWrongState                                    参加したセッションの状態が異常でした。[:handling-cleanup]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                                ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin              他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    JoinRandomJointSessionAsync, IsJoinRandomJointSessionCompleted
     */
    Result GetJoinRandomJointSessionResult() const;

    /*!
    @brief    JoinRandomJointSessionAsync の非同期処理でセッション参加時のスコアを取得します。
    @details  インターネット通信時のみ有効です。
    スコアベースマッチメイク指定で @ref JoinRandomJointSessionAsync の非同期処理が完了した後、本関数でスコアを取得できます。
    ただし、自分がジョイントセッションホストの場合はスコアは取得できません。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument                                      引数で指定されたポインタが有効ではない場合に返ります。[:progErr]
    @retval nn::pia::ResultInvalidState                                         非同期処理が完了していない場合、開始されていない場合、スコアベースマッチメイクを指定していない、自分が @ref JoinRandomJointSessionAsync を呼び出していない、ジョイントセッションホストであるなど、スコアが取得できない状態の場合に返ります。[:progErr]
    */
    Result GetJoinRandomJointSessionScore(uint32_t* pScore) const;

    //! @cond PRIVATE
    /*!
    @brief      ジョイントセッションの構築を行います。
    @details    [:disable-local]
            ホストのみ使用可能です。
            ジョイントセッション構築を行う非同期処理を開始します。
            開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            処理の完了は @ref IsCreateJointSessionCompleted でチェックし、結果は @ref GetCreateJointSessionResult で確認できます。

            ホストが本関数で非同期処理を開始した場合、クライアント側でもジョイントセッション構築を行う内部処理が自動的に開始されます。
            自動的に開始された処理の進捗は、状態変化イベント通知をハンドリングするか @ref GetStatus で取得できる @ref Status をチェックすることで確認できます。

            非同期処理中は、送受信処理関連など一部の API は一時的に使用できません。

    @param[in]  pCreateSessionSetting   新しいセッションを作成する場合の設定を示す CreateSessionSetting のポインタを指定します。

    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないか、ホストかどうか確認してください。[:progErr]

    @see    IsCreateJointSessionCompleted, GetCreateJointSessionResult
     */
    Result CreateJointSessionAsync(const CreateSessionSetting* pCreateSessionSetting);

    /*!
    @brief      CreateJointSessionAsync の非同期処理の完了をチェックします。
    @details    [:disable-local]
                @ref CreateJointSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    CreateJointSessionAsync, GetCreateJointSessionResult
     */
    bool IsCreateJointSessionCompleted() const;

    /*!
        @brief      CreateJointSessionAsync の非同期処理の結果を取得します。
        @details    [:disable-local]
                    @ref CreateJointSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidState                 自身がジョイントセッション処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultInvalidArgument              非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost      ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerMaintenance        ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultCompanionStationIsOffline    同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval nn::pia::ResultCompanionStationIsLeft       同行するクライアントとの接続が切断しました。[:handling-cleanup]
    @retval nn::pia::ResultSessionMigrationFailed       セッション移行処理に失敗しました。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError             [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval nn::pia::ResultSessionWrongState            構築したセッションの状態が異常でした。[:handling-cleanup]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                    ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin  他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    CreateJointSessionAsync, IsCreateJointSessionCompleted
     */
    Result GetCreateJointSessionResult() const;

    /*!
    @brief      指定したジョイントセッションに参加します。
    @details    [:disable-local]
            ホストのみ使用可能です。
            指定したジョイントセッションに参加を行う非同期処理を開始します。
            開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            処理の完了は @ref IsJoinJointSessionCompleted でチェックし、結果は @ref GetJoinJointSessionResult で確認できます。

            ホストが本関数で非同期処理を開始した場合、クライアント側でもランダムマッチメイクでジョイントセッションに参加を行う内部処理が自動的に開始されます。
            自動的に開始された処理の進捗は、状態変化イベント通知をハンドリングするか @ref GetStatus で取得できる @ref Status をチェックすることで確認できます。

            非同期処理中は、送受信処理関連など一部の API は一時的に使用できません。

    @param  pJoinSessionSetting 参加するセッションの条件

    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないか、ホストかどうか確認してください。[:progErr]

    @see    IsJoinJointSessionCompleted, GetJoinJointSessionResult
     */
    Result JoinJointSessionAsync(const JoinSessionSetting* pJoinSessionSetting);

    /*!
    @brief      JoinJointSessionAsync の非同期処理の完了をチェックします。
    @details    [:disable-local]
                @ref JoinJointSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    JoinJointSessionAsync, GetJoinJointSessionResult
     */
    bool IsJoinJointSessionCompleted() const;

    /*!
        @brief      JoinJointSessionAsync の非同期処理の結果を取得します。
        @details    [:disable-local]
                    @ref JoinJointSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidState                                         自身がジョイントセッション処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultInvalidArgument                                      非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost                              ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultSessionIsNotFound                                    非同期処理途中で対象のセッションが消失もしくは締め切られました。[:handling-cleanup]
    @retval nn::pia::ResultMatchmakeSessionIsFull                               対象のマッチメイクセッションが満員でした。[:handling-cleanup]
    @retval nn::pia::ResultDeniedByParticipants                                 セッション参加者が自分をブラックリストに登録しています。[:handling-cleanup]
    @retval nn::pia::ResultParticipantInBlockList                               セッションに自分がブロックリストに登録している参加者がいました。[:handling-cleanup]
    @retval nn::pia::ResultSessionUserPasswordUnmatch                           参加時に指定したユーザーパスワードがセッションに設定されたユーザーパスワードと一致しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultSessionSystemPasswordUnmatch                         参加時に指定したシステムパスワードがセッションに設定されたシステムパスワードと一致しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultSessionIsClosed                                      参加を行おうとしたセッションが締め切られていました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultHostIsNotFriend                                      参加を行おうとしたセッションのホストがフレンドではありませんでした。[:handling-cleanup]
    @retval nn::pia::ResultGameServerMaintenance                                ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultCompanionStationIsOffline                            同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval nn::pia::ResultCompanionStationIsLeft                               同行するクライアントとの接続が切断しました。[:handling-cleanup]
    @retval nn::pia::ResultSessionMigrationFailed                               セッション移行処理に失敗しました。[:handling-cleanup]
    @retval nn::pia::ResultStationConnectionFailed                              接続処理に失敗しました。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultStationConnectionNatTraversalFailedUnknown           NAT トラバーサル接続処理に失敗しました。NAT タイプは不明でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEim                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdm                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEim                  NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdm                            NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。[:viewer][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEimSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEimRemoteEdmSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EIM, 接続失敗した相手の NAT タイプは EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedLocalEdmRemoteEimSamePublicAddress NAT トラバーサル接続処理に失敗しました。自身の NAT タイプは EDM, 接続失敗した相手の NAT タイプは EIM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalFailedBothEdmSamePublicAddress           NAT トラバーサル接続処理に失敗しました。自身および接続失敗した相手の NAT タイプは共に EDM でした。また、自身および接続失敗した相手は同じグローバルIPアドレスでした。[:viewer][:viewer-local][:handling-cleanup]
    @retval nn::pia::ResultNatTraversalRequestTimeout                           NAT トラバーサル接続処理に失敗しました。NAT トラバーサル要求がタイムアウトしました。 [:viewer][:handling-cleanup]
    @retval nn::pia::ResultJoinRequestDenied                                    メッシュホストに接続を拒否されました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultMeshIsFull                                           参加を希望したメッシュが満員で、参加できませんでした。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedNoCandidate                               リレー経路の候補がいなかったため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRttLimit                                  リレー経路の RTT 制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedRelayNumLimit                             リレー経路のリレー依頼数制限を超えたため、リレー接続に失敗しました。[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultRelayFailedUnknown                                   リレー接続に失敗しました。（詳細不明）[:handling-cleanup][:viewer-handling]
    @retval nn::pia::ResultInvalidSystemMessage                                 メッシュホストから不正な応答が届いたため、処理を中断しました。[:progErr]
    @retval nn::pia::ResultIncompatibleFormat                                   相手の通信形式が非互換です。異なる ROM （通信互換の無い Ver.の Pia を使用した ROM）と通信しようとした場合に返されます。[:progErr]
    @retval nn::pia::ResultNexInternalError                                     [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted                             ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval nn::pia::ResultSessionWrongState                                    参加したセッションの状態が異常でした。[:handling-cleanup]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                                ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin              他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    JoinJointSessionAsync, IsJoinJointSessionCompleted
     */
    Result GetJoinJointSessionResult() const;

    /*!
    @brief      ジョイントセッションから離脱します。
    @details    [:disable-local]
            ジョイントセッションホストのみ使用可能です。
            ジョイントセッションから離脱する非同期処理を開始します。

            開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            処理の完了は @ref IsLeaveJointSessionCompleted でチェックし、結果は @ref GetLeaveJointSessionResult で確認できます。

            ホストが本関数でジョイントセッションの離脱処理を開始した場合、同行者側でも離脱の内部処理が自動的に開始されます。
            自動的に開始された処理の進捗は、状態変化イベント通知をハンドリングするか @ref GetStatus で取得できる @ref Status をチェックすることで確認できます。

            非同期処理中は、送受信処理関連など一部の API は一時的に使用できません。

    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないか確認してください。[:progErr]

    @see    IsLeaveJointSessionCompleted, GetLeaveJointSessionResult
     */
    Result LeaveJointSessionAsync();

    /*!
    @brief      LeaveJointSessionAsync の非同期処理の完了をチェックします。
    @details    [:disable-local]
                @ref LeaveJointSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    LeaveJointSessionAsync, GetLeaveJointSessionResult
     */
    bool IsLeaveJointSessionCompleted() const;

    /*!
        @brief      LeaveJointSessionAsync の非同期処理の結果を取得します。
        @details    [:disable-local]
                    @ref LeaveJointSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidState                 処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultInvalidArgument              非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost      ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerMaintenance        ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultCompanionStationIsOffline    同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval nn::pia::ResultCompanionStationIsLeft       同行するクライアントとの接続が切断しました。[:handling-cleanup]
    @retval nn::pia::ResultSessionMigrationFailed       セッション移行処理に失敗しました。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError             [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                    ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin  他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    LeaveJointSessionAsync, IsLeaveJointSessionCompleted
     */
    Result GetLeaveJointSessionResult() const;
    //! @endcond

    /*!
    @brief      ジョイントセッションの解散を行います。
    @details    [:disable-local]
            ジョイントセッションホストのみ使用可能です。
            ジョイントセッションの解散を行う非同期処理を開始します。

            開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
            処理の完了は @ref IsDestroyJointSessionCompleted でチェックし、結果は @ref GetDestroyJointSessionResult で確認できます。

            ホストが本関数でジョイントセッションの解散処理を開始した場合、同行者側でも解散の内部処理が自動的に開始されます。
            自動的に開始された処理の進捗は、状態変化イベント通知をハンドリングするか @ref GetStatus で取得できる @ref Status をチェックすることで確認できます。

            非同期処理中は、送受信処理関連など一部の API は一時的に使用できません。

    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidArgument  非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないか確認してください。[:progErr]

    @see    IsDestroyJointSessionCompleted, GetDestroyJointSessionResult
     */
    Result DestroyJointSessionAsync();

    /*!
    @brief      DestroyJointSessionAsync の非同期処理の完了をチェックします。
    @details    [:disable-local]
                @ref DestroyJointSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    DestroyJointSessionAsync, GetDestroyJointSessionResult
     */
    bool IsDestroyJointSessionCompleted() const;

    /*!
    @brief      DestroyJointSessionAsync の非同期処理の結果を取得します。
    @details    [:disable-local]
                @ref DestroyJointSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidState                    処理可能な状態では無かった場合に返ります。また、非同期処理が完了していない場合、もしくは開始されていない場合にも返ります。[:progErr]
    @retval nn::pia::ResultInvalidArgument                 非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval nn::pia::ResultNetworkConnectionIsLost         ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultGameServerMaintenance           ゲームサーバーのメンテナンスが行われています。[:viewer][:handling-logout]
    @retval nn::pia::ResultCompanionStationIsOffline       同行するクライアントがゲームサーバにログインしていない状態でした。[:handling-cleanup]
    @retval nn::pia::ResultCompanionStationIsLeft          同行するクライアントとの接続が切断しました。[:handling-cleanup]
    @retval nn::pia::ResultSessionMigrationFailed          セッション移行処理に失敗しました。[:handling-cleanup]
    @retval nn::pia::ResultNexInternalError                [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted        ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultUserAccountNotExisted                    ユーザーアカウントが存在しませんでした。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin  他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    DestroyJointSessionAsync, IsDestroyJointSessionCompleted
     */
    Result GetDestroyJointSessionResult() const;

    // ジョイントセッション参加受付開始
    /*!
    @brief    ジョイントセッションの参加受付を開始します。
    @details  [:disable-local]
              ジョイントセッションホストのみ使用可能です。
              ジョイントセッション参加受付を開始する非同期処理を開始します。
              開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
              処理の完了は @ref IsOpenJointSessionCompleted でチェックし、結果は @ref GetOpenJointSessionResult で確認できます。<br>
              UpdateSessionSetting の派生クラスのインスタンスを引数に指定する場合、参加受付の開始と同時にジョイントセッションの設定を更新します。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidArgument    非同期処理開始時に渡した引数の設定内容が想定外です。[:progErr]
    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、ジョイントセッション参加状態であること、ジョイントセッションホストであることを確認してください。[:progErr]

    @see    IsOpenJointSessionCompleted, GetOpenJointSessionResult, IsJointSessionHost
     */
    Result OpenJointSessionAsync(const UpdateSessionSetting* pUpdateSessionSetting = NULL);

    /*!
    @brief   OpenJointSessionAsync の非同期処理の完了をチェックします。
    @details [:disable-local]
             @ref OpenJointSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    OpenJointSessionAsync, GetOpenJointSessionResult
     */
    bool IsOpenJointSessionCompleted() const;

    /*!
    @brief   OpenJointSessionAsync の非同期処理の結果を取得します。
    @details [:disable-local]
                @ref OpenJointSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        
    @retval   nn::pia::ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSessionConnectionIsLost  メッシュから切断されました。[:handling-leave]
    @retval   nn::pia::ResultSessionIsNotFound        非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval   nn::pia::ResultNexInternalError           [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    OpenJointSessionAsync, IsOpenJointSessionCompleted
     */
    Result GetOpenJointSessionResult() const;

    // ジョイントセッション参加締め切り
    /*!
    @brief    ジョイントセッションの参加締切を行います。
    @details  [:disable-local]
              ジョイントセッションホストのみ使用可能です。
              ジョイントセッション参加を締め切る非同期処理を開始します。
              開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
              処理の完了は @ref IsCloseJointSessionCompleted でチェックし、結果は @ref GetCloseJointSessionResult で確認できます。
              <br>
              本関数の呼び出し直前にジョイントセッションに新規参加しようとしているステーションが非同期処理完了後に参加してくる可能性があるため、
              セッション参加締切後、マッチメイクセッションとメッシュの参加者数が一致するまで待ってから非同期処理を完了します。
              また、一定期間（15秒）経過しても一致しない場合も非同期処理を完了します。この場合、@ref GetCloseJointSessionResult は
              ResultNegligibleFault を返します。非同期処理完了後に新規参加してくる可能性がありますので、アプリケーションで適切にハンドリング
              する必要があります。


    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないこと、Startup が呼び出されていること、ジョイントセッション参加状態であること、ジョイントセッションホストであることを確認してください。[:progErr]

    @see    IsCloseJointSessionCompleted, GetCloseJointSessionResult, IsJointSessionHost
     */
    Result CloseJointSessionAsync();

    /*!
    @brief   CloseJointSessionAsync の非同期処理の完了をチェックします。
    @details [:disable-local]
             @ref CloseJointSessionAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    CloseJointSessionAsync, GetCloseJointSessionResult
     */
    bool IsCloseJointSessionCompleted() const;

    /*!
    @brief   CloseJointSessionAsync の非同期処理の結果を取得します。
    @details [:disable-local]
                @ref CloseJointSessionAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSessionConnectionIsLost  メッシュから切断されました。[:handling-leave]
    @retval   nn::pia::ResultSessionIsNotFound        非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval   nn::pia::ResultNexInternalError           [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @retval   nn::pia::ResultNegligibleFault          マッチメイクセッションとメッシュの参加人数が一致しない状態で一定時間経過しました。[:handling]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    CloseJointSessionAsync, IsCloseJointSessionCompleted
     */
    Result GetCloseJointSessionResult() const;

    /*!
    @brief    ジョイントセッションの設定を更新します。
    @details [:disable-local]
           非同期にジョイントセッションの設定を更新します。
           ジョイントセッションホストであるときに実行できます。
           開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
           処理の完了は @ref IsUpdateJointSessionSettingCompleted でチェックし、結果は @ref GetUpdateJointSessionSettingResult で確認できます。

    @return   非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState       呼び出し時に適切な状態ではありませんでした。ローカル通信では対応していません。[:progErr]
                                                インターネット通信時は他の非同期処理が行われていないこと、Startup が呼び出されていること、ジョイントセッションホストであることを確認してください。[:progErr]

    @param[in] pUpdateSessionSetting 変更する設定をまとめる UpdateSessionSetting 派生クラスのポインタ

    @see    UpdateSessionSetting, IsUpdateJointSessionSettingCompleted, GetUpdateJointSessionSettingResult, IsJointSessionHost
     */
    Result UpdateJointSessionSettingAsync(const UpdateSessionSetting* pUpdateSessionSetting);

    /*!
    @brief   UpdateJointSessionSettingAsync の非同期処理の完了をチェックします。
    @details [:disable-local]
             @ref UpdateJointSessionSettingAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return 非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。

    @see    UpdateJointSessionSettingAsync, GetUpdateJointSessionSettingResult
     */
    bool IsUpdateJointSessionSettingCompleted() const;

    /*!
    @brief   UpdateJointSessionSettingAsync の非同期処理の結果を取得します。
    @details [:disable-local]
                @ref UpdateJointSessionSettingAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return  IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval   nn::pia::ResultInvalidState             呼び出し時に適切な状態ではありませんでした。他の非同期処理が行われていないことを確認してください。[:progErr]
    @retval   nn::pia::ResultNetworkConnectionIsLost  ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval   nn::pia::ResultSessionIsNotFound 非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval   nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval   nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval   nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif

    @see    UpdateJointSessionSettingAsync, IsUpdateJointSessionSettingCompleted
     */
    Result GetUpdateJointSessionSettingResult() const;

    /*!
    @brief  ジョイントセッションのセッション ID を取得します。

    @ifnot CAFE_DOC
    @details ローカル通信の時は、常に InvalidSessionId を返します。
    @endif

    @return 現在参加しているジョイントセッションのセッション ID を返します。ジョイントセッションに参加していない状態では InvalidSessionId を返します。
     */
    uint32_t GetJointSessionId() const;

    /*!
    @brief      自分自身がジョイントセッションのホストであるかどうかを取得します。
    @details    この関数はジョイントセッションのホストであるかどうかを返します。

    @return     自分自身がホストの場合は true、そうでない場合は false が返ります。
     */
    bool IsJointSessionHost() const;

    /*!
    @cond PRIVATE
    @brief   ジョイントセッションホストを示す StationIndex を取得します

    @details ジョイントセッション参加中でない場合、この関数は StationIndex_Invalid を返します。
             ジョイントセッション参加中の場合、この関数はジョイントセッションのホストを示す StationIndex を返します。

    @return 自身が参加しているジョイントセッションのホストを示す StationIndex を返します。
    */
    StationIndex GetJointSessionHostStationIndex() const;
    //! @endcond

    /*!
    @brief  ジョイントセッションのホストを示す StationId を取得します。

    @details ジョイントセッション参加中でない場合、この関数は StationIdInvalid を返します。
             ジョイントセッション参加中の場合、この関数はジョイントセッションのホストを示す StationId を返します。

    @return  ジョイントセッションのホストを示す StationId を返します。
     */
    StationId GetJointSessionHostStationId() const;

    /*!
    @brief  自身が参加しているジョイントセッションのホストを示す StationConnectionInfo を取得します。

    @details ジョイントセッション参加中の場合、この関数はジョイントセッションのホストを示す StationConnectionInfo を返します。
    <br>
    @param[out] pInfoOut 取得したジョイントセッションホストを示すアドレスの格納先を示すポインタ
    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval     ResultInvalidArgument       引数に誤りがあります。[:progErr]
    @retval     ResultInvalidState          ジョイントセッションに参加していません。[:progErr]

    @return 自身が参加しているジョイントセッションのホストを示す StationConnectionInfo を返します。
     */
    Result GetJointSessionHostStationConnectionInfo(transport::StationConnectionInfo* pInfoOut);

    /*!
    @brief      ジョイントセッションの参加募集状態を取得します。
    @details    ジョイントセッションホストのみ使用可能です。
    @return     ジョイントセッションの参加募集状態を返します。
    @see        IsJointSessionHost
     */
    SessionOpenStatus GetJointSessionOpenStatus() const;

    /*!
    @brief      参加中のジョイントセッションのセッション情報を取得します。
    @details    セッション情報は自動的には更新されません。セッション情報の更新は @ref RequestJointSessionInfoAsync で行う必要があります。
    @return     参加中のジョイントセッションのセッション情報を返します。
    */
    const ISessionInfo* GetJointSessionInfo() const;

    /*!
    @brief      参加中のジョイントセッションのセッション情報の取得要求を行います。
    @details    参加中のジョイントセッションのセッション情報の取得要求を行う非同期処理を開始します。
    開始した処理は @ref common::Scheduler::Dispatch を定期的に呼び出すことで進行します。
    処理の完了は @ref IsRequestJointSessionInfoCompleted でチェックし、結果は @ref GetRequestJointSessionInfoResult で確認できます。
    非同期処理が成功していた場合は、@ref GetJointSessionInfo でセッション情報を取得できますが、非同期処理中にジョイントセッションホストが UpdateJointSessionSettingAsync でセッション情報を更新した場合、
    取得する情報が更新前の情報となる場合があります。
    <br>
    インターネット通信の場合、ゲームサーバーアクセスを行います。1 分間に 10 回以上の呼び出しを行ってはいけません。
    @return 非同期処理の開始に成功すれば、成功の Result が返されます。失敗した場合、以下の Result を返します。
    @retval nn::pia::ResultInvalidState     呼び出し時に適切な状態ではありませんでした。セッションに参加していないか、他の非同期処理が行われていないか確認してください。[:progErr]

    @see    IsRequestJointSessionInfoCompleted, GetRequestJointSessionInfoResult

    */
    Result RequestJointSessionInfoAsync();

    /*!
    @brief      RequestJointSessionInfoAsync の非同期処理の完了をチェックします。
    @details    @ref RequestJointSessionInfoAsync で開始された非同期処理は、本関数で完了をチェックできます。

    @return     非同期処理が完了していた場合は true, 処理中の場合は false が返ります。非同期処理が行われていない場合も false が返ります。
    @see        RequestJointSessionInfoAsync, GetRequestJointSessionInfoResult
    */
    bool IsRequestJointSessionInfoCompleted() const;

    /*!
    @brief      RequestJointSessionInfoAsync の非同期処理の結果を取得します。
    @details    @ref RequestJointSessionInfoAsync で開始された非同期処理が完了した後、本関数で結果を取得できます。

    @return 非同期処理が成功した場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。

    @retval nn::pia::ResultInvalidState              非同期処理が完了していない場合、もしくは開始されていない場合に返ります。[:progErr]
    @retval nn::pia::ResultSessionIsNotFound         非同期処理途中で対象のセッションが消失しました。[:handling-cleanup]
    @retval nn::pia::ResultNetworkConnectionIsLost   ネットワークが使用できない状態になっています。無線スイッチOffや、アクセスポイントの障害、ゲームサーバーとの接続の切断などの原因が考えられます。[:viewer][:handling-shutdownNetwork]
    @retval nn::pia::ResultNexInternalError [:nexInternalError][:viewer][:handling]
    @retval nn::pia::ResultGameServerProcessAborted     ゲームサーバーに関する処理が異常終了しました。[:handling-logout]
    @if NIN_DOC
    @retval nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin    他の端末からの同一アカウントでのログインによりゲームサーバーから切断され、非同期処理が中断されました。[:viewer][:handling-logout]
    @endif
    */
    Result GetRequestJointSessionInfoResult() const;

    /*!
        @cond PRIVATE
    */
    int GetJointSequence() const;
    //! @endcond

    /*!
        @cond PRIVATE
    */
    int GetJoinSequence() const;
    int GetJoinAutoMatchmakeSequence() const;
    int GetJoinMeshSequence() const;
    //! @endcond

    /*!
     end of name ジョイントセッション機能
     @}
     */


    /*!
     @name コールバック関数の登録/登録解除
     @{
     */

    /*!
        @brief セッションの状態が変化した場合に呼び出されるコールバック関数を登録します

        @details セッションの状態変化（ステーション参加／離脱）発生時に呼び出されるコールバック関数を登録します。
        登録する関数内には長時間ブロックするような処理を記述してはいけません。

        @param[in]  callback            登録するコールバック関数へのポインタを指定します。
        @param[in]  pApplicationParam   コールバック関数の引数として渡されるパラメータへのポインタを指定します。指定しない場合はパラメータ無し（NULL）となります。

        @return 正常に処理できた場合、 IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
        @retval nn::pia::ResultInvalidState  既にイベント通知コールバックを登録済みの場合に返されます。[:progErr]

        @see    UnregisterSessionEventCallback
     */
    Result RegisterSessionEventCallback(SessionEventCallback callback, void* pApplicationParam = NULL);

    /*!
        @brief  登録されているセッション状態変化コールバック関数を登録解除します。

        @see    RegisterSessionEventCallback
     */
    void UnregisterSessionEventCallback();


    /// @cond PRIVATE
    static void NoticeEventCallback(Session::EventType eventType, StationIndex index);

    void CheckStatusAtChangeMeshState();
    /// @endcond

    /*!
        @brief セッションホストで参加要求が来た際に呼び出される承認／拒否判定コールバック関数を登録します

        @details 自分がセッションホストで、クライアントから参加要求を受け取った場合に呼び出されるコールバックです。
                 コールバック関数は、@ref common::Scheduler::Dispatch の内部で呼び出されます。
                 登録する関数内には長時間ブロックするような処理を記述してはいけません。

        @param[in]  callback    登録するコールバック関数へのポインタを指定します。

        @see    UnregisterJoiningApprovalCallback
     */
    void RegisterJoiningApprovalCallback(JoiningApprovalCallback callback);

    /*!
        @brief  登録されている参加要求が来た際に呼び出される承認／拒否判定コールバック関数の登録を解除します。

        @see    RegisterJoiningApprovalCallback
     */
    void UnregisterJoiningApprovalCallback();

    //! @cond PRIVATE
    static bool SessionJoiningApprovalCallback(const transport::Station::IdentificationToken* pRequesterToken);
    MeshEventListener* GetMeshEventListenerPtr();
    transport::Transport::DisconnectionEventListener GetDisconnectionEventListener();
    static uint8_t CheckHostCandidateRankingCallback(StationIndex targetStationIndex, bool isHost);
    static uint32_t GetSessionIdCallback();
    //! @endcond

    /*!
     end of name コールバック関数の登録/登録解除
     @}
     */

    /// @cond PRIVATE
    // private
    void JoinNotice(StationId targetStationId);
    /// @endcond

    /*!
     @name 同期クロック
     @{
     */

    /*!
    @brief      セッション作成時刻からの経過時間（同期クロック）をミリ秒で返します。

    @details 同期クロックは、セッションに参加している Station 間で同期される時計です。
                セッションに参加したクライアントは、定期的に時刻同期パルスをセッションホストに送信します。
                ホストがこのパルスに応答することで、ホストと各クライアントの時刻同期が実現されます。
                この同期パルス送信間隔はデフォルトで 2 秒ですが、アプリケーションがこの値を
                変更することもできます。詳しくは SetSyncClockPulseInterval(), GetSyncClockPulseInterval()
                を参照してください。

    @return     セッション作成時刻からの経過時間をミリ秒で返します。
                セッション参加直後など、ホストからの応答がまだ返されておらず、
                セッションホストとの同期が取れていないタイミングでこの API を
                呼び出したときは、負の値が返されます。

    @see        SetSyncClockPulseInterval, GetSyncClockPulseInterval
     */
    int64_t GetTime(void) const;


    /*!
    @brief      同期クロックの同期パルス送信間隔を指定します。

    @param[in]  interval 同期パルス送信の時間間隔です。単位はミリ秒です。
    @see        GetTime, GetSyncClockPulseInterval
     */
    void SetSyncClockPulseInterval(int32_t interval);


    /*!
    @brief      同期クロックの同期パルス送信間隔を取得します。

    @return     同期パルス送信間隔を取得します。単位はミリ秒です。
    @see        GetTime, SetSyncClockPulseInterval
     */
    int32_t GetSyncClockPulseInterval(void) const;

    /*!
     end of name 同期クロック
     @}
     */

    /*!
     @name リレー通信機能
     @{
     */

    /*!
    @brief      リレー接続有効時に、ステーション1台に割り当てられる転送依頼数の最大制限値を設定します。

    @details    リレー接続経路探索＆決定時の指標として本関数で設定された制限値を利用します。<br>
                設定変更しない場合のデフォルト値は "[セッション最大参加数] * [セッション最大参加数]" です。<br>

                同じセッションへの参加者は全員同じ値を設定することを推奨します。（異なる値を設定した場合は動作保証外とします）

                リレー接続が有効なネットワークトポロジーの場合かつ @ref Startup 前にのみ使用可能です。

    @param[in]  num  設定したい最大制限値を指定します。

    @return     正常に設定できた場合、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidState    リレー接続が有効ではない、もしくは既に Startup 呼出後の場合に返されます。[:progErr]
     */
    Result SetMaxNumOfRelayRoutesAssignedToStation(uint16_t num);


    /*!
    @brief      リレー接続有効時に、1つのリレー接続経路の合計 RTT の最大制限値を設定します。

    @details    リレー接続経路探索＆決定時の指標として本関数で設定された制限値を利用します。<br>
                設定変更しない場合のデフォルト値は session::SessionMaxRttDefault です。<br>

                同じセッションへの参加者は全員同じ値を設定することを推奨します。（異なる値を設定した場合は動作保証外とします）

                リレー接続が有効なネットワークトポロジーの場合かつ @ref Startup 前にのみ使用可能です。

    @param[in]  rtt  設定したい最大制限値を指定します。単位は msec です。

    @return     正常に設定できた場合、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidState    リレー接続が有効ではない、もしくは既に Startup 呼出後の場合に返されます。[:progErr]
     */
    Result SetMaxRtt(uint16_t rtt);

    /*!
    @brief      （デバッグ用）リレー接続有効時に、任意の StationIndex と自分自身との接続経路情報を取得します。

    @details    本関数はデバッグ用です。製品には組み込まないようお願いします。<br>
                自分自身と targetId のステーションとの接続経路情報について、以下の規則で pRelayId へ格納します。
                （接続経路情報が取得できなかった場合は pRelayId の値を変更することはありません）
                <ul>
                <li>直通接続の場合 : targetId</li>
                <li>リレー接続の場合 : リレー依頼先ステーションの StationId</li>
                <li>自分自身の場合 : StationIdInvalid</li>
                </ul>

    @details    内部処理の状態により接続経路情報の取得に失敗する場合があります。
                この状態になるのは一時的であり、内部処理が正常に進行した後は正しく取得できるようになります。

    @param[in]  targetId 接続経路を取得したい対象の StationId を指定します。
    @param[out] pRelayId 接続経路情報を示す StationId が格納されます。

    @return     正常に取得できた場合、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidState         リレー接続が有効ではない場合に返されます。[:progErr]
    @retval     nn::pia::ResultInvalidArgument      対象の StationId が有効な値ではない場合、もしくは引数で指定されたポインタが有効ではない場合に返ります。[:progErr]
    @retval     nn::pia::ResultNotFound             接続経路情報を取得できませんでした。対象のステーションはセッションに参加していません。[:handling]
    @retval     nn::pia::ResultTemporaryUnavailable 接続経路情報を取得できませんでした。ジョイントセッション処理が進行中のため、一時的に API を利用できない状態です。[:handling]
     */
    Result GetConnectionRoute(StationId targetId, StationId* pRelayId);

    /*!
    @brief      （デバッグ用）リレー接続有効時に、任意の StationIndex 間の接続経路エミュレーション設定を指定します。

    @details    本関数はデバッグ用です。製品には組み込まないようお願いします。

    @details    本関数を使用する場合は @ref Session::SetDebugSetting で接続経路エミュレーション機能が有効となるように設定します。

    @details    対象の StationIndex 間の接続方法は（たとえ実際には直通接続ができる場合でも）必ずリレー接続となります。

    @details    本関数はセッションに接続していないタイミングで呼び出します。
                セッション参加済みの状態で接続経路エミュレーション設定を変更することは動作保証外とします。

    @param[in]  index1  対象の StationIndex を指定します。
    @param[in]  index2  対象の StationIndex を指定します。

    @return     正常に設定できた場合、成功の Result が返されます。
    @retval     nn::pia::ResultInvalidState      リレー接続が有効ではない場合に返されます。[:progErr]
    @retval     nn::pia::ResultInvalidArgument   対象の StationIndex が有効な値ではない場合に返ります。[:progErr]

    @see    SetDebugSetting, ResetRelayEmulation
     */
    Result RelayEmulation(StationIndex index1, StationIndex index2);

    /*!
    @brief      （デバッグ用）リレー接続有効時に、接続経路エミュレーション設定を初期化します。

    @details    本関数はデバッグ用です。製品には組み込まないようお願いします。

    @details    @ref RelayEmulation を使用して設定された接続経路エミュレーションについて全て初期化します。
                （全ての StationIndex の組み合わせに対して EmulationType_None を設定した状態と同等です）

    @details    本関数はセッションに接続していないタイミングで呼び出します。
                セッション参加済みの状態で接続経路エミュレーション設定を変更することは動作保証外とします。

    @see    SetDebugSetting, RelayEmulation
    */
    void ResetRelayEmulation();

    /*!
     end of name リレー通信機能
     @}
     */

    /*!
     @name その他
     @{
     */

    /*!
        @brief  ホストマイグレーションが有効になっているかを取得します。

        @details    session::Session::Startup の引数に設定した値により返る値が決定します。
        @return     ホストマイグレーションが有効な設定で @ref session::Session::Startup を呼び出していた場合は true が返ります。

     */
    bool IsHostMigrationEnabled() const;

    /// @cond PRIVATE
    /*!
    @brief セッションから任意のステーションを追放します。（ホスト専用）

    @details    ホストのみ使用可能です。

    @details    引数で指定した StationIndex が割り当てられたステーションへ追放通知を伝達し、
                追放通知を受けた対象のステーションではセッション離脱処理が行われます。

    @details    追放通知の通信遅延などが原因で、この関数呼出に成功した後実際にセッションから離脱するまでにある程度の時間がかかる場合があります。

    @param[in]  index   追放したいステーションに割り当てられている StationIndex を指定します。

    @return  正常にセッションからのステーション追放処理を実行できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  nn::pia::ResultInvalidState    ホスト以外が呼び出した場合に返されます。[:progErr]
    @retval  nn::pia::ResultInvalidArgument 引数で指定された StationIndex が、ホスト自身を示す Id やセッションに参加していない Id だった場合に返されます。[:progErr]
    @retval  nn::pia::ResultBufferIsFull    追放処理管理用バッファもしくは追放通知送信用バッファに空きが無かった場合に返されます。時間をあけてから再度実行すれば、成功する可能性があります。[:handling]
     */
    Result KickoutStation(StationIndex index);
    /// @endcond

    /*!
    @brief セッションから任意のステーションを追放します。（ホスト専用）

    @details    ホストのみ使用可能です。

    @details    引数で指定した StationId が割り当てられたステーションへ追放通知を伝達し、
                追放通知を受けた対象のステーションではセッション離脱処理が行われます。

    @details    追放通知の通信遅延などが原因で、この関数呼出に成功した後実際にセッションから離脱するまでにある程度の時間がかかる場合があります。

    @param[in]  id  追放したいステーションに割り当てられている StationId を指定します。

    @return  正常にセッションからのステーション追放処理を実行できた場合、IsSuccess() が真となる Result を返します。失敗した場合、以下の Result を返します。
    @retval  nn::pia::ResultInvalidState    ホスト以外が呼び出した場合に返されます。[:progErr]
    @retval  nn::pia::ResultInvalidArgument 引数で指定された StationId が、ホスト自身を示す Id やセッションに参加していない Id だった場合に返されます。[:progErr]
    @retval  nn::pia::ResultBufferIsFull    追放処理管理用バッファもしくは追放通知送信用バッファに空きが無かった場合に返されます。時間をあけてから再度実行すれば、成功する可能性があります。[:handling]
     */
    Result KickoutStation(StationId id);

    /*!
    @brief メッシュが通信不可能となった理由を取得します。

    @details メッシュが通信可能な状態だった場合は、常に UNKNOWN_REASON が返されます。

    @return  メッシュが通信不可能となった理由を表す DisconnectReason

    @see    CheckConnectionError
     */
    DisconnectReason GetSessionDisconnectReason() const;

    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

    /*!
    @brief マッチメイク分析機能のセッション参加承認コールバックの不整合検知判定で使用する transport::Station::IdentificationToken を設定します。

    @param[in]  pToken  セッション参加承認コールバックの不整合検知判定で使用するトークン
     */
    void SetMatchmakeAnalysisToken(const transport::Station::IdentificationToken* pToken);

    /*!
     end of name その他
     @}
     */


    /// @cond PRIVATE
    bool SaveWaitingStationIndex();
    uint32_t UpdateWaitingStationIndexNum();
    void SetWaitingOtherClientsFlag(bool flag);
    /// @endcond

    /// @cond PRIVATE
    IMatchmakeSession* GetCurrentMatchmakeSession()
    {
        return m_pMatchmakeSessionPtrArray[CurrentMatchmakeSessionIndex()];
    }
    IMatchmakeSession* GetBufferMatchmakeSession()
    {
        return m_pMatchmakeSessionPtrArray[BufferMatchmakeSessionIndex()];
    }
    const IMatchmakeSession* GetCurrentMatchmakeSession() const
    {
        return m_pMatchmakeSessionPtrArray[CurrentMatchmakeSessionIndex()];
    }
    const IMatchmakeSession* GetBufferMatchmakeSession() const
    {
        return m_pMatchmakeSessionPtrArray[BufferMatchmakeSessionIndex()];
    }
    ISessionInfoList* GetSessionInfoListPtr()
    {
        return m_pSessionInfoList;
    }
    ICommunityInfoList* GetCommunityInfoListPtr()
    {
        return m_pCommunityInfoList;
    }
    uint32_t GetCurrentSessionId() const
    {
        return m_SessionId[CurrentMatchmakeSessionIndex()];
    }
    uint32_t GetBufferSessionId() const
    {
        return m_SessionId[BufferMatchmakeSessionIndex()];
    }
    uint16_t GetCurrentMaxParticipantNum() const
    {
        return m_MaxParticipantNum[CurrentMatchmakeSessionIndex()];
    }
    uint16_t GetBufferMaxParticipantNum() const
    {
        return m_MaxParticipantNum[BufferMatchmakeSessionIndex()];
    }
    SessionOpenStatus GetCurrentSessionOpenStatus() const
    {
        return m_SessionOpenStatus[CurrentMatchmakeSessionIndex()];
    }
    SessionOpenStatus GetBufferSessionOpenStatus() const
    {
        return m_SessionOpenStatus[BufferMatchmakeSessionIndex()];
    }

    // n2908: ジョイントセッションを使用できるネットワーク種別かどうかを確認する
    bool IsJointSessionEnabled() const
    {
        return (m_pJointSessionJob != NULL);
    }

    bool IsLeaveIgnorable() const
    {
        return m_LeaveIgnorableFlag;
    }
    bool IsPostponeJoinEvent() const
    {
        return m_PostponeJoinEventFlag;
    }
    ModuleState GetModuleState() const
    {
        return m_ModuleState;
    }
    ProcessType GetJointSessionProcessType() const;
    void SetHostMigrationOccurred(bool b);
    bool IsHostMigrationOccurred() const;
    transport::Station::IdentificationToken* GetMatchmakeAnalysisToken();
    void NotifyOwnershipChange(uint32_t sessionId, PrincipalId ownerId);
    void NotifyLeave(uint32_t sessionId, PrincipalId principalId);
    void NotifyUnregister(uint32_t sessionId);
    bool IsPlayersCountedAsParticipants() const;
    uint8_t GetLocalPlayerCount() const;
    uint16_t GetParticipantNum() const;
    void UpdateSessionStationInfo();
    common::AsyncContext* GetInternalAsyncContext();
    bool IsPreparedForCleanup() const;
    /// @endcond

#if NN_PIA_NINTENDOSDK && NN_PIA_ENABLE_NSA
    //! @cond PRIVATE
    struct PlayerHistory
    {
        char screenNameBuffer[MaxPlayerNameLength * common::MultiByteCharMaxSize + 1];
        uint8_t screenNameSize;
        uint8_t screenNameEncodingType;
        uint8_t nameStringLanguage;
        transport::Station::PlayerRegistrationKey registrationKey;
    };
    //! @endcond
    typedef common::ObjList<PlayerHistory> PlayerHistoryList;
#endif

protected:
    bool IsProcessingJoin() const;
    bool IsWaitingOtherClients() const;
    bool IsPostponeNotice() const;
    bool IsProcessingLeave() const;

private:
    //! @cond PRIVATE
    /*
    @brief コンストラクタです。シングルトンパターンのため、コンストラクタは封印します。
     */
    Session(const Setting& setting);


    /*!
    @brief デストラクタです。シングルトンパターンのため、デストラクタは封印します
     */
    virtual ~Session();

    /*!
    @brief シングルトンパターンのため、コピーコンストラクタは封印します。
     */
    Session(const Session&);


    /*!
    @brief シングルトンパターンのため、代入演算子は封印します。
     */
    Session& operator=(const Session&);
    //! @endcond

    bool UpdateStationIdToBecomeHost();
    bool UpdateStationIdToBecomeClient();

    bool AddNewStationId(StationId stationId, StationIndex stationIndex, PrincipalId uniqueId, uint32_t sessionId);
    bool RemoveStationId(StationId stationId, StationIndex stationIndex, PrincipalId uniqueId, uint32_t sessionId);

    void CleanupStatus(bool isPostponeCleanupLocalStationId = false);

    void ChangeStatusToDisconnectedSession();
    void ChangeStatusToDisconnectedNetwork();

    void CancelJoinProcessByFatalError();

    // シングルトンインスタンスへのポインタです。
    static Session* s_pInstance;

    static DebugSetting s_DebugSetting;

    bool m_IsNecessaryServer;

    MeshLayerController* m_pMeshLayerController;

    SessionStatusCheckJob* m_pSessionStatusCheckJob;
    CreateSessionJob* m_pCreateSessionJob;
    AutoMatchmakeJob* m_pAutoMatchmakeJob;
    BrowseMatchmakeJob* m_pBrowseMatchmakeJob;
    JoinSessionJob* m_pJoinSessionJob;
    LeaveSessionJob* m_pLeaveSessionJob;
    DestroySessionJob* m_pDestroySessionJob;
    OpenParticipationJob* m_pOpenParticipationJob;
    CloseParticipationJob* m_pCloseParticipationJob;

    GenerateMatchmakeSystemPasswordJob* m_pGenerateMatchmakeSystemPasswordJob;
    ClearMatchmakeSystemPasswordJob* m_pClearMatchmakeSystemPasswordJob;

    common::String* m_pSessionPassword;

    JointSessionJob* m_pJointSessionJob;

    UpdateSessionSettingJob* m_pUpdateSessionSettingJob;
    UpdateApplicationDataJob* m_pUpdateApplicationDataJob;

    CommunityManagementJob* m_pCommunityManagementJob;

    SessionSearchCriteria* m_pPrimarySessionSearchCriteria;
    SessionSearchCriteria* m_pSecondarySessionSearchCriteria;

    SessionEventCallback m_SessionEventCallback;
    void* m_pSessionEventParam;
    JoiningApprovalCallback m_JoiningApprovalCallback;

    StationIdStatusTable* m_pStationIdStatusTable;

    uint32_t m_SessionProtocolHandle;
    SessionProtocol* m_pSessionProtocol;

    ModuleState m_ModuleState;
    MatchmakeStatus m_MatchmakeStatus;

    ConnectionStatus m_ConnectionStatus;

    StationId m_LocalStationId;
    StationId m_HostStationId;
    StationId m_JointSessionHostStationId;

    PrincipalId m_PrincipalId;

    enum AsyncProcessingName
    {
        AsyncProcessingName_Nothing = 0,
        AsyncProcessingName_JoinRandomSession,
        AsyncProcessingName_BrowseSession,
        AsyncProcessingName_CreateSession,
        AsyncProcessingName_JoinSession,
        AsyncProcessingName_LeaveSession,
        AsyncProcessingName_OpenSession,
        AsyncProcessingName_CloseSession,
        AsyncProcessingName_GenerateMatchmakeSystemPassword,
        AsyncProcessingName_ClearMatchmakeSystemPassword,
        AsyncProcessingName_CreateJointSession,
        AsyncProcessingName_JoinJointSession,
        AsyncProcessingName_JoinRandomJointSession,
        AsyncProcessingName_DestroyJointSession,
        AsyncProcessingName_LeaveJointSession,
        AsyncProcessingName_OpenJointSession,
        AsyncProcessingName_CloseJointSession,
        AsyncProcessingName_UpdateSessionSetting,
        AsyncProcessingName_UpdateJointSessionSetting,
        AsyncProcessingName_CreateCommunity,
        AsyncProcessingName_JoinCommunity,
        AsyncProcessingName_BrowseCommunity,
        AsyncProcessingName_DestroyCommunity,
        AsyncProcessingName_LeaveCommunity,
        AsyncProcessingName_RequestSessionInfo,
        AsyncProcessingName_RequestJointSessionInfo,
        AsyncProcessingName_UpdateCommunity
    };

    void ResetInternalAsyncState();
    bool IsAsyncProcessCompleted(AsyncProcessingName processingName) const;
    Result GetAsyncProcessResult(AsyncProcessingName processingName) const;
    Result CancelAsyncProcess(AsyncProcessingName processingName);

    Result OpenSessionCore(uint32_t sessionId, IMatchmakeSession* pMatchmakeSession, const UpdateSessionSetting* pUpdateSessionSetting);
    Result CloseSessionCore(uint32_t sessionId, IMatchmakeSession* pMatchmakeSession);
    Result UpdateSessionSettingAsyncCore(uint32_t targetSessionId, IMatchmakeSession* pMatchmakeSession, const UpdateSessionSetting* pUpdateSessionSetting, bool isOpenSession);

    common::AsyncContext m_InternalContext;
    AsyncProcessingName m_CurrentAsyncProcessingName;

    MeshEventListener* m_pMeshEventListener;

    void CallSessionEventCallback(EventType eventType, StationId stationId);

    uint32_t m_WaitingStationIndexBitmap;
    bool m_WaitingOtherClientsStationIdFlag;
    bool LeaveNoticeToWaitingStationIndex(StationIndex leaveStationIndex);
    void ClearWaitingStationIndex();

    DisconnectReason m_DisconnectReason;
    bool m_LeaveIgnorableFlag;
    bool m_PostponeJoinEventFlag;
    void SetLeaveIgnorableFlagEnabled(bool isLeaveIgnorable)
    {
        m_LeaveIgnorableFlag = isLeaveIgnorable;
    }
    void SetPostponeJoinEventEnabled(bool isPostponeJoinEvent)
    {
        m_PostponeJoinEventFlag = isPostponeJoinEvent;
    }

    uint8_t CurrentMatchmakeSessionIndex() const
    {
        return m_CurrentMatchmakeSessionIndex;
    }
    uint8_t BufferMatchmakeSessionIndex() const
    {
        return (m_CurrentMatchmakeSessionIndex == 0 ? 1 : 0);
    }
    void SetCurrentSessionId(uint32_t sessionId)
    {
        m_SessionId[CurrentMatchmakeSessionIndex()] = sessionId;
    }
    void SetBufferSessionId(uint32_t sessionId)
    {
        m_SessionId[BufferMatchmakeSessionIndex()] = sessionId;
    }
    void SetCurrentMaxParticipantNum(uint16_t num)
    {
        m_MaxParticipantNum[CurrentMatchmakeSessionIndex()] = num;
    }
    void SetBufferMaxParticipantNum(uint16_t num)
    {
        m_MaxParticipantNum[BufferMatchmakeSessionIndex()] = num;
    }
    void SetJoinRandomScore(uint32_t score)
    {
        m_JoinRandomScore = score;
        m_IsJoinRandomScoreEnabled = true;
    }
    void SetJoinRandomJointScore(uint32_t score)
    {
        m_JoinRandomJointScore = score;
        m_IsJoinRandomJointScoreEnabled = true;
    }

    uint16_t m_MaxParticipantNum[2];
    uint8_t m_CurrentMatchmakeSessionIndex;
    IMatchmakeSession* m_pMatchmakeSessionPtrArray[2];
    ISessionInfoList* m_pSessionInfoList;
    uint32_t m_SessionId[2];
    ICommunityInfoList* m_pCommunityInfoList;

    SessionOpenStatus m_SessionOpenStatus[2];
    void SetCurrentSessionOpenStatus(SessionOpenStatus openStatus);
    void SetBufferSessionOpenStatus(SessionOpenStatus openStatus);
    void ChangeStationIdConvertEnableFlag(bool flag);

    TStationIdList m_ValidStationIdList;
    void* m_pValidStationIdListBuffer;
    void validateStationId(StationId targetStationId);
    void invalidateStationId(StationId targetStationId);

    uint32_t m_SessionIdOfJointSessionHost;

    // マッチメイク分析機能用フラグ
    // ホストマイグレートが起こったイベントを検知するフラグ
    bool m_IsHostMigrationOccurred;
    transport::Station::IdentificationToken m_MatchmakeAnalysisToken;

    uint16_t m_PlayerNum;
    uint16_t m_ParticipantNum;

    uint32_t m_JoinRandomScore;
    bool m_IsJoinRandomScoreEnabled;
    uint32_t m_JoinRandomJointScore;
    bool m_IsJoinRandomJointScoreEnabled;

#if NN_PIA_NINTENDOSDK && NN_PIA_ENABLE_NSA
    void* m_pPlayerHistoryListBuffer;
    PlayerHistoryList m_PlayerHistoryList;
    AddPlayHistoryJob* m_pAddPlayHistoryJob;
#endif
};
}
}
}  // end of namespace nn::pia::session
